from __future__ import (absolute_import, division, print_function)
# Copyright (c) 2022 Fortinet
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

__metaclass__ = type

import json


SCHEMA_DATA = '''
{
    "adom_options": {
        "stated": false,
        "options": {
            "assign_excluded": {"choices": ["disable", "enable"], "type": "str"},
            "specify_assign_pkg_list": {"choices": ["disable", "enable"], "type": "str"},
            "assign_name": {"v_range": [["7.0.5", "7.0.12"], ["7.2.2", ""]], "type": "str"}
        }
    },
    "antivirus_mmschecksum": {
        "stated": true,
        "options": {
            "comment": {"type": "str"},
            "entries": {
                "type": "list",
                "options": {"checksum": {"type": "str"}, "name": {"type": "str"}, "status": {"choices": ["disable", "enable"], "type": "str"}},
                "elements": "dict"
            },
            "id": {"required": true, "type": "int"},
            "name": {"type": "str"}
        }
    },
    "antivirus_mmschecksum_entries": {
        "stated": true,
        "options": {"checksum": {"type": "str"}, "name": {"required": true, "type": "str"}, "status": {"choices": ["disable", "enable"], "type": "str"}}
    },
    "antivirus_notification": {
        "stated": true,
        "options": {
            "comment": {"type": "str"},
            "entries": {
                "type": "list",
                "options": {
                    "name": {"type": "str"},
                    "prefix": {"choices": ["disable", "enable"], "type": "str"},
                    "status": {"choices": ["disable", "enable"], "type": "str"}
                },
                "elements": "dict"
            },
            "id": {"required": true, "type": "int"},
            "name": {"type": "str"}
        }
    },
    "antivirus_notification_entries": {
        "stated": true,
        "options": {
            "name": {"required": true, "type": "str"},
            "prefix": {"choices": ["disable", "enable"], "type": "str"},
            "status": {"choices": ["disable", "enable"], "type": "str"}
        }
    },
    "antivirus_profile": {
        "stated": true,
        "options": {
            "analytics-bl-filetype": {"type": "str"},
            "analytics-db": {"choices": ["disable", "enable"], "type": "str"},
            "analytics-max-upload": {"type": "int"},
            "analytics-wl-filetype": {"type": "str"},
            "av-block-log": {"choices": ["disable", "enable"], "type": "str"},
            "av-virus-log": {"choices": ["disable", "enable"], "type": "str"},
            "comment": {"type": "str"},
            "extended-log": {"choices": ["disable", "enable"], "type": "str"},
            "ftgd-analytics": {"choices": ["disable", "suspicious", "everything"], "type": "str"},
            "inspection-mode": {"v_range": [["6.0.0", "7.2.1"]], "choices": ["proxy", "flow-based"], "type": "str"},
            "mobile-malware-db": {"choices": ["disable", "enable"], "type": "str"},
            "name": {"required": true, "type": "str"},
            "replacemsg-group": {"type": "str"},
            "scan-mode": {"choices": ["quick", "full", "legacy", "default"], "type": "str"},
            "feature-set": {"v_range": [["6.4.0", ""]], "choices": ["proxy", "flow"], "type": "str"},
            "cifs": {
                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                "type": "dict",
                "options": {
                    "archive-block": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "type": "list",
                        "choices": [
                            "encrypted", "corrupted", "multipart", "nested", "mailbomb", "unhandled", "partiallycorrupted", "fileslimit", "timeout"
                        ],
                        "elements": "str"
                    },
                    "archive-log": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "type": "list",
                        "choices": [
                            "encrypted", "corrupted", "multipart", "nested", "mailbomb", "unhandled", "partiallycorrupted", "fileslimit", "timeout"
                        ],
                        "elements": "str"
                    },
                    "emulator": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "options": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "type": "list",
                        "choices": ["scan", "quarantine", "avmonitor"],
                        "elements": "str"
                    },
                    "outbreak-prevention": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "choices": ["disabled", "files", "full-archive", "disable", "block", "monitor"],
                        "type": "str"
                    },
                    "av-scan": {"v_range": [["7.0.0", ""]], "choices": ["disable", "monitor", "block"], "type": "str"},
                    "external-blocklist": {"v_range": [["7.0.0", ""]], "choices": ["disable", "monitor", "block"], "type": "str"},
                    "quarantine": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "fortindr": {"v_range": [["7.0.5", ""]], "choices": ["disable", "block", "monitor"], "type": "str"},
                    "fortisandbox": {"v_range": [["7.2.0", ""]], "choices": ["disable", "block", "monitor"], "type": "str"},
                    "fortiai": {"v_range": [["7.0.1", ""]], "choices": ["disable", "monitor", "block"], "type": "str"}
                }
            },
            "content-disarm": {
                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                "type": "dict",
                "options": {
                    "cover-page": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "detect-only": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "error-action": {"v_range": [["6.4.5", ""]], "choices": ["block", "log-only", "ignore"], "type": "str"},
                    "office-action": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "office-dde": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "office-embed": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "office-hylink": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "office-linked": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "office-macro": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "original-file-destination": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "choices": ["fortisandbox", "quarantine", "discard"],
                        "type": "str"
                    },
                    "pdf-act-form": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "pdf-act-gotor": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "pdf-act-java": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "pdf-act-launch": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "pdf-act-movie": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "pdf-act-sound": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "pdf-embedfile": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "pdf-hyperlink": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "pdf-javacode": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "analytics-suspicious": {"v_range": [["7.6.0", ""]], "choices": ["disable", "enable"], "type": "str"}
                }
            },
            "ftp": {
                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                "type": "dict",
                "options": {
                    "archive-block": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "type": "list",
                        "choices": [
                            "encrypted", "corrupted", "multipart", "nested", "mailbomb", "unhandled", "partiallycorrupted", "fileslimit", "timeout"
                        ],
                        "elements": "str"
                    },
                    "archive-log": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "type": "list",
                        "choices": [
                            "encrypted", "corrupted", "multipart", "nested", "mailbomb", "unhandled", "partiallycorrupted", "fileslimit", "timeout"
                        ],
                        "elements": "str"
                    },
                    "emulator": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "options": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "type": "list",
                        "choices": ["scan", "file-filter", "quarantine", "avquery", "avmonitor"],
                        "elements": "str"
                    },
                    "outbreak-prevention": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "choices": ["disabled", "files", "full-archive", "disable", "block", "monitor"],
                        "type": "str"
                    },
                    "av-scan": {"v_range": [["7.0.0", ""]], "choices": ["disable", "monitor", "block"], "type": "str"},
                    "external-blocklist": {"v_range": [["7.0.0", ""]], "choices": ["disable", "monitor", "block"], "type": "str"},
                    "quarantine": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "fortindr": {"v_range": [["7.0.5", ""]], "choices": ["disable", "block", "monitor"], "type": "str"},
                    "fortisandbox": {"v_range": [["7.2.0", ""]], "choices": ["disable", "block", "monitor"], "type": "str"},
                    "fortiai": {"v_range": [["7.0.1", ""]], "choices": ["disable", "monitor", "block"], "type": "str"}
                }
            },
            "http": {
                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                "type": "dict",
                "options": {
                    "archive-block": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "type": "list",
                        "choices": [
                            "encrypted", "corrupted", "multipart", "nested", "mailbomb", "unhandled", "partiallycorrupted", "fileslimit", "timeout"
                        ],
                        "elements": "str"
                    },
                    "archive-log": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "type": "list",
                        "choices": [
                            "encrypted", "corrupted", "multipart", "nested", "mailbomb", "unhandled", "partiallycorrupted", "fileslimit", "timeout"
                        ],
                        "elements": "str"
                    },
                    "content-disarm": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "emulator": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "options": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "type": "list",
                        "choices": ["scan", "file-filter", "quarantine", "avquery", "avmonitor", "strict-file"],
                        "elements": "str"
                    },
                    "outbreak-prevention": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "choices": ["disabled", "files", "full-archive", "disable", "block", "monitor"],
                        "type": "str"
                    },
                    "av-optimize": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "av-scan": {"v_range": [["7.0.0", ""]], "choices": ["disable", "monitor", "block"], "type": "str"},
                    "external-blocklist": {"v_range": [["7.0.0", ""]], "choices": ["disable", "monitor", "block"], "type": "str"},
                    "quarantine": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "fortindr": {"v_range": [["7.0.5", ""]], "choices": ["disable", "block", "monitor"], "type": "str"},
                    "fortisandbox": {"v_range": [["7.2.0", ""]], "choices": ["disable", "block", "monitor"], "type": "str"},
                    "fortiai": {"v_range": [["7.0.1", ""]], "choices": ["disable", "monitor", "block"], "type": "str"},
                    "unknown-content-encoding": {
                        "v_range": [["7.0.5", "7.0.12"], ["7.2.1", ""]],
                        "choices": ["block", "inspect", "bypass"],
                        "type": "str"
                    }
                }
            },
            "imap": {
                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                "type": "dict",
                "options": {
                    "archive-block": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "type": "list",
                        "choices": [
                            "encrypted", "corrupted", "multipart", "nested", "mailbomb", "unhandled", "partiallycorrupted", "fileslimit", "timeout"
                        ],
                        "elements": "str"
                    },
                    "archive-log": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "type": "list",
                        "choices": [
                            "encrypted", "corrupted", "multipart", "nested", "mailbomb", "unhandled", "partiallycorrupted", "fileslimit", "timeout"
                        ],
                        "elements": "str"
                    },
                    "content-disarm": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "emulator": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "executables": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["default", "virus"], "type": "str"},
                    "options": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "type": "list",
                        "choices": ["scan", "file-filter", "quarantine", "avquery", "avmonitor"],
                        "elements": "str"
                    },
                    "outbreak-prevention": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "choices": ["disabled", "files", "full-archive", "disable", "block", "monitor"],
                        "type": "str"
                    },
                    "av-scan": {"v_range": [["7.0.0", ""]], "choices": ["disable", "monitor", "block"], "type": "str"},
                    "external-blocklist": {"v_range": [["7.0.0", ""]], "choices": ["disable", "monitor", "block"], "type": "str"},
                    "quarantine": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "fortindr": {"v_range": [["7.0.5", ""]], "choices": ["disable", "block", "monitor"], "type": "str"},
                    "fortisandbox": {"v_range": [["7.2.0", ""]], "choices": ["disable", "block", "monitor"], "type": "str"},
                    "fortiai": {"v_range": [["7.0.1", ""]], "choices": ["disable", "monitor", "block"], "type": "str"}
                }
            },
            "mapi": {
                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                "type": "dict",
                "options": {
                    "archive-block": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "type": "list",
                        "choices": [
                            "encrypted", "corrupted", "multipart", "nested", "mailbomb", "unhandled", "partiallycorrupted", "fileslimit", "timeout"
                        ],
                        "elements": "str"
                    },
                    "archive-log": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "type": "list",
                        "choices": [
                            "encrypted", "corrupted", "multipart", "nested", "mailbomb", "unhandled", "partiallycorrupted", "fileslimit", "timeout"
                        ],
                        "elements": "str"
                    },
                    "emulator": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "executables": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["default", "virus"], "type": "str"},
                    "options": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "type": "list",
                        "choices": ["scan", "quarantine", "avquery", "avmonitor"],
                        "elements": "str"
                    },
                    "outbreak-prevention": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "choices": ["disabled", "files", "full-archive", "disable", "block", "monitor"],
                        "type": "str"
                    },
                    "av-scan": {"v_range": [["7.0.0", ""]], "choices": ["disable", "monitor", "block"], "type": "str"},
                    "external-blocklist": {"v_range": [["7.0.0", ""]], "choices": ["disable", "monitor", "block"], "type": "str"},
                    "quarantine": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "fortindr": {"v_range": [["7.2.0", ""]], "choices": ["disable", "block", "monitor"], "type": "str"},
                    "fortisandbox": {"v_range": [["7.2.0", ""]], "choices": ["disable", "block", "monitor"], "type": "str"},
                    "fortiai": {"v_range": [["7.0.1", ""]], "choices": ["disable", "monitor", "block"], "type": "str"}
                }
            },
            "nac-quar": {
                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                "type": "dict",
                "options": {
                    "expiry": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "infected": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["none", "quar-src-ip", "quar-interface"], "type": "str"},
                    "log": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"}
                }
            },
            "nntp": {
                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                "type": "dict",
                "options": {
                    "archive-block": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "type": "list",
                        "choices": [
                            "encrypted", "corrupted", "multipart", "nested", "mailbomb", "unhandled", "partiallycorrupted", "fileslimit", "timeout"
                        ],
                        "elements": "str"
                    },
                    "archive-log": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "type": "list",
                        "choices": [
                            "encrypted", "corrupted", "multipart", "nested", "mailbomb", "unhandled", "partiallycorrupted", "fileslimit", "timeout"
                        ],
                        "elements": "str"
                    },
                    "emulator": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "options": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "type": "list",
                        "choices": ["scan", "file-filter", "quarantine", "avquery", "avmonitor"],
                        "elements": "str"
                    },
                    "outbreak-prevention": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "choices": ["disabled", "files", "full-archive", "disable", "block", "monitor"],
                        "type": "str"
                    },
                    "av-scan": {"v_range": [["7.0.0", ""]], "choices": ["disable", "monitor", "block"], "type": "str"},
                    "external-blocklist": {"v_range": [["7.0.0", ""]], "choices": ["disable", "monitor", "block"], "type": "str"},
                    "quarantine": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "fortindr": {"v_range": [["7.0.5", ""]], "choices": ["disable", "block", "monitor"], "type": "str"},
                    "fortisandbox": {"v_range": [["7.2.0", ""]], "choices": ["disable", "block", "monitor"], "type": "str"},
                    "fortiai": {"v_range": [["7.0.1", ""]], "choices": ["disable", "monitor", "block"], "type": "str"}
                }
            },
            "outbreak-prevention": {
                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                "type": "dict",
                "options": {
                    "external-blocklist": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "ftgd-service": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"}
                }
            },
            "pop3": {
                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                "type": "dict",
                "options": {
                    "archive-block": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "type": "list",
                        "choices": [
                            "encrypted", "corrupted", "multipart", "nested", "mailbomb", "unhandled", "partiallycorrupted", "fileslimit", "timeout"
                        ],
                        "elements": "str"
                    },
                    "archive-log": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "type": "list",
                        "choices": [
                            "encrypted", "corrupted", "multipart", "nested", "mailbomb", "unhandled", "partiallycorrupted", "fileslimit", "timeout"
                        ],
                        "elements": "str"
                    },
                    "content-disarm": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "emulator": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "executables": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["default", "virus"], "type": "str"},
                    "options": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "type": "list",
                        "choices": ["scan", "file-filter", "quarantine", "avquery", "avmonitor"],
                        "elements": "str"
                    },
                    "outbreak-prevention": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "choices": ["disabled", "files", "full-archive", "disable", "block", "monitor"],
                        "type": "str"
                    },
                    "av-scan": {"v_range": [["7.0.0", ""]], "choices": ["disable", "monitor", "block"], "type": "str"},
                    "external-blocklist": {"v_range": [["7.0.0", ""]], "choices": ["disable", "monitor", "block"], "type": "str"},
                    "quarantine": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "fortindr": {"v_range": [["7.0.5", ""]], "choices": ["disable", "block", "monitor"], "type": "str"},
                    "fortisandbox": {"v_range": [["7.2.0", ""]], "choices": ["disable", "block", "monitor"], "type": "str"},
                    "fortiai": {"v_range": [["7.0.1", ""]], "choices": ["disable", "monitor", "block"], "type": "str"}
                }
            },
            "smtp": {
                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                "type": "dict",
                "options": {
                    "archive-block": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "type": "list",
                        "choices": [
                            "encrypted", "corrupted", "multipart", "nested", "mailbomb", "unhandled", "partiallycorrupted", "fileslimit", "timeout"
                        ],
                        "elements": "str"
                    },
                    "archive-log": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "type": "list",
                        "choices": [
                            "encrypted", "corrupted", "multipart", "nested", "mailbomb", "unhandled", "partiallycorrupted", "fileslimit", "timeout"
                        ],
                        "elements": "str"
                    },
                    "content-disarm": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "emulator": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "executables": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["default", "virus"], "type": "str"},
                    "options": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "type": "list",
                        "choices": ["scan", "file-filter", "quarantine", "avquery", "avmonitor"],
                        "elements": "str"
                    },
                    "outbreak-prevention": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "choices": ["disabled", "files", "full-archive", "disable", "block", "monitor"],
                        "type": "str"
                    },
                    "av-scan": {"v_range": [["7.0.0", ""]], "choices": ["disable", "monitor", "block"], "type": "str"},
                    "external-blocklist": {"v_range": [["7.0.0", ""]], "choices": ["disable", "monitor", "block"], "type": "str"},
                    "quarantine": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "fortindr": {"v_range": [["7.0.5", ""]], "choices": ["disable", "block", "monitor"], "type": "str"},
                    "fortisandbox": {"v_range": [["7.2.0", ""]], "choices": ["disable", "block", "monitor"], "type": "str"},
                    "fortiai": {"v_range": [["7.0.1", ""]], "choices": ["disable", "monitor", "block"], "type": "str"}
                }
            },
            "ssh": {
                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                "type": "dict",
                "options": {
                    "archive-block": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "type": "list",
                        "choices": [
                            "encrypted", "corrupted", "multipart", "nested", "mailbomb", "unhandled", "partiallycorrupted", "fileslimit", "timeout"
                        ],
                        "elements": "str"
                    },
                    "archive-log": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "type": "list",
                        "choices": [
                            "encrypted", "corrupted", "multipart", "nested", "mailbomb", "unhandled", "partiallycorrupted", "fileslimit", "timeout"
                        ],
                        "elements": "str"
                    },
                    "emulator": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "options": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "type": "list",
                        "choices": ["avmonitor", "quarantine", "scan"],
                        "elements": "str"
                    },
                    "outbreak-prevention": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "choices": ["disabled", "files", "full-archive", "disable", "block", "monitor"],
                        "type": "str"
                    },
                    "av-scan": {"v_range": [["7.0.0", ""]], "choices": ["disable", "monitor", "block"], "type": "str"},
                    "external-blocklist": {"v_range": [["7.0.0", ""]], "choices": ["disable", "monitor", "block"], "type": "str"},
                    "quarantine": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "fortindr": {"v_range": [["7.0.5", ""]], "choices": ["disable", "block", "monitor"], "type": "str"},
                    "fortisandbox": {"v_range": [["7.2.0", ""]], "choices": ["disable", "block", "monitor"], "type": "str"},
                    "fortiai": {"v_range": [["7.0.1", ""]], "choices": ["disable", "monitor", "block"], "type": "str"}
                }
            },
            "smb": {
                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", "7.2.1"]],
                "type": "dict",
                "options": {
                    "archive-block": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", "7.2.1"]],
                        "type": "list",
                        "choices": [
                            "encrypted", "corrupted", "multipart", "nested", "mailbomb", "unhandled", "partiallycorrupted", "fileslimit", "timeout"
                        ],
                        "elements": "str"
                    },
                    "archive-log": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", "7.2.1"]],
                        "type": "list",
                        "choices": [
                            "encrypted", "corrupted", "multipart", "nested", "mailbomb", "unhandled", "partiallycorrupted", "fileslimit", "timeout"
                        ],
                        "elements": "str"
                    },
                    "emulator": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", "7.2.1"]], "choices": ["disable", "enable"], "type": "str"},
                    "options": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", "7.2.1"]],
                        "type": "list",
                        "choices": ["scan", "quarantine", "avquery", "avmonitor"],
                        "elements": "str"
                    },
                    "outbreak-prevention": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", "7.2.1"]],
                        "choices": ["disabled", "files", "full-archive"],
                        "type": "str"
                    }
                }
            },
            "analytics-accept-filetype": {"v_range": [["7.0.0", ""]], "type": "str"},
            "analytics-ignore-filetype": {"v_range": [["7.0.0", ""]], "type": "str"},
            "ems-threat-feed": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "external-blocklist": {"v_range": [["7.0.0", ""]], "type": "raw"},
            "external-blocklist-archive-scan": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "external-blocklist-enable-all": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "outbreak-prevention-archive-scan": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "fortindr-error-action": {"v_range": [["7.0.5", ""]], "choices": ["log-only", "block", "ignore"], "type": "str"},
            "fortindr-timeout-action": {"v_range": [["7.0.5", ""]], "choices": ["log-only", "block", "ignore"], "type": "str"},
            "fortisandbox-error-action": {"v_range": [["7.2.0", ""]], "choices": ["log-only", "block", "ignore"], "type": "str"},
            "fortisandbox-max-upload": {"v_range": [["7.2.0", ""]], "type": "int"},
            "fortisandbox-mode": {"v_range": [["7.2.0", ""]], "choices": ["inline", "analytics-suspicious", "analytics-everything"], "type": "str"},
            "fortisandbox-timeout-action": {"v_range": [["7.2.0", ""]], "choices": ["log-only", "block", "ignore"], "type": "str"},
            "fortiai-error-action": {"v_range": [["7.0.1", ""]], "choices": ["block", "log-only", "ignore"], "type": "str"},
            "fortiai-timeout-action": {"v_range": [["7.0.2", ""]], "choices": ["block", "log-only", "ignore"], "type": "str"}
        }
    },
    "antivirus_profile_cifs": {
        "stated": false,
        "options": {
            "archive-block": {
                "v_range": [["6.2.0", ""]],
                "type": "list",
                "choices": ["encrypted", "corrupted", "multipart", "nested", "mailbomb", "unhandled", "partiallycorrupted", "fileslimit", "timeout"],
                "elements": "str"
            },
            "archive-log": {
                "v_range": [["6.2.0", ""]],
                "type": "list",
                "choices": ["encrypted", "corrupted", "multipart", "nested", "mailbomb", "unhandled", "partiallycorrupted", "fileslimit", "timeout"],
                "elements": "str"
            },
            "emulator": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "options": {"v_range": [["6.2.0", ""]], "type": "list", "choices": ["scan", "quarantine", "avmonitor"], "elements": "str"},
            "outbreak-prevention": {
                "v_range": [["6.2.0", ""]],
                "choices": ["disabled", "files", "full-archive", "disable", "block", "monitor"],
                "type": "str"
            },
            "av-scan": {"v_range": [["7.0.0", ""]], "choices": ["disable", "monitor", "block"], "type": "str"},
            "external-blocklist": {"v_range": [["7.0.0", ""]], "choices": ["disable", "monitor", "block"], "type": "str"},
            "quarantine": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "fortindr": {"v_range": [["7.0.5", ""]], "choices": ["disable", "block", "monitor"], "type": "str"},
            "fortisandbox": {"v_range": [["7.2.0", ""]], "choices": ["disable", "block", "monitor"], "type": "str"},
            "fortiai": {"v_range": [["7.0.1", ""]], "choices": ["disable", "monitor", "block"], "type": "str"}
        }
    },
    "antivirus_profile_contentdisarm": {
        "stated": false,
        "options": {
            "cover-page": {"choices": ["disable", "enable"], "type": "str"},
            "detect-only": {"choices": ["disable", "enable"], "type": "str"},
            "office-embed": {"choices": ["disable", "enable"], "type": "str"},
            "office-hylink": {"choices": ["disable", "enable"], "type": "str"},
            "office-linked": {"choices": ["disable", "enable"], "type": "str"},
            "office-macro": {"choices": ["disable", "enable"], "type": "str"},
            "original-file-destination": {"choices": ["fortisandbox", "quarantine", "discard"], "type": "str"},
            "pdf-act-form": {"choices": ["disable", "enable"], "type": "str"},
            "pdf-act-gotor": {"choices": ["disable", "enable"], "type": "str"},
            "pdf-act-java": {"choices": ["disable", "enable"], "type": "str"},
            "pdf-act-launch": {"choices": ["disable", "enable"], "type": "str"},
            "pdf-act-movie": {"choices": ["disable", "enable"], "type": "str"},
            "pdf-act-sound": {"choices": ["disable", "enable"], "type": "str"},
            "pdf-embedfile": {"choices": ["disable", "enable"], "type": "str"},
            "pdf-hyperlink": {"choices": ["disable", "enable"], "type": "str"},
            "pdf-javacode": {"choices": ["disable", "enable"], "type": "str"},
            "office-action": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "office-dde": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "error-action": {"v_range": [["6.4.2", ""]], "choices": ["block", "log-only", "ignore"], "type": "str"},
            "analytics-suspicious": {"v_range": [["7.6.0", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "antivirus_profile_ftp": {
        "stated": false,
        "options": {
            "archive-block": {
                "type": "list",
                "choices": ["encrypted", "corrupted", "multipart", "nested", "mailbomb", "unhandled", "partiallycorrupted", "fileslimit", "timeout"],
                "elements": "str"
            },
            "archive-log": {
                "type": "list",
                "choices": ["encrypted", "corrupted", "multipart", "nested", "mailbomb", "unhandled", "partiallycorrupted", "fileslimit", "timeout"],
                "elements": "str"
            },
            "emulator": {"choices": ["disable", "enable"], "type": "str"},
            "options": {"type": "list", "choices": ["scan", "file-filter", "quarantine", "avquery", "avmonitor"], "elements": "str"},
            "outbreak-prevention": {"choices": ["disabled", "files", "full-archive", "disable", "block", "monitor"], "type": "str"},
            "av-scan": {"v_range": [["7.0.0", ""]], "choices": ["disable", "monitor", "block"], "type": "str"},
            "external-blocklist": {"v_range": [["7.0.0", ""]], "choices": ["disable", "monitor", "block"], "type": "str"},
            "quarantine": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "fortindr": {"v_range": [["7.0.5", ""]], "choices": ["disable", "block", "monitor"], "type": "str"},
            "fortisandbox": {"v_range": [["7.2.0", ""]], "choices": ["disable", "block", "monitor"], "type": "str"},
            "fortiai": {"v_range": [["7.0.1", ""]], "choices": ["disable", "monitor", "block"], "type": "str"}
        }
    },
    "antivirus_profile_http": {
        "stated": false,
        "options": {
            "archive-block": {
                "type": "list",
                "choices": ["encrypted", "corrupted", "multipart", "nested", "mailbomb", "unhandled", "partiallycorrupted", "fileslimit", "timeout"],
                "elements": "str"
            },
            "archive-log": {
                "type": "list",
                "choices": ["encrypted", "corrupted", "multipart", "nested", "mailbomb", "unhandled", "partiallycorrupted", "fileslimit", "timeout"],
                "elements": "str"
            },
            "content-disarm": {"choices": ["disable", "enable"], "type": "str"},
            "emulator": {"choices": ["disable", "enable"], "type": "str"},
            "options": {"type": "list", "choices": ["scan", "file-filter", "quarantine", "avquery", "avmonitor", "strict-file"], "elements": "str"},
            "outbreak-prevention": {"choices": ["disabled", "files", "full-archive", "disable", "block", "monitor"], "type": "str"},
            "av-optimize": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "av-scan": {"v_range": [["7.0.0", ""]], "choices": ["disable", "monitor", "block"], "type": "str"},
            "external-blocklist": {"v_range": [["7.0.0", ""]], "choices": ["disable", "monitor", "block"], "type": "str"},
            "quarantine": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "fortindr": {"v_range": [["7.0.5", ""]], "choices": ["disable", "block", "monitor"], "type": "str"},
            "fortisandbox": {"v_range": [["7.2.0", ""]], "choices": ["disable", "block", "monitor"], "type": "str"},
            "fortiai": {"v_range": [["7.0.1", ""]], "choices": ["disable", "monitor", "block"], "type": "str"},
            "unknown-content-encoding": {"v_range": [["7.0.5", "7.0.12"], ["7.2.1", ""]], "choices": ["block", "inspect", "bypass"], "type": "str"}
        }
    },
    "antivirus_profile_imap": {
        "stated": false,
        "options": {
            "archive-block": {
                "type": "list",
                "choices": ["encrypted", "corrupted", "multipart", "nested", "mailbomb", "unhandled", "partiallycorrupted", "fileslimit", "timeout"],
                "elements": "str"
            },
            "archive-log": {
                "type": "list",
                "choices": ["encrypted", "corrupted", "multipart", "nested", "mailbomb", "unhandled", "partiallycorrupted", "fileslimit", "timeout"],
                "elements": "str"
            },
            "content-disarm": {"choices": ["disable", "enable"], "type": "str"},
            "emulator": {"choices": ["disable", "enable"], "type": "str"},
            "executables": {"choices": ["default", "virus"], "type": "str"},
            "options": {"type": "list", "choices": ["scan", "file-filter", "quarantine", "avquery", "avmonitor"], "elements": "str"},
            "outbreak-prevention": {"choices": ["disabled", "files", "full-archive", "disable", "block", "monitor"], "type": "str"},
            "av-scan": {"v_range": [["7.0.0", ""]], "choices": ["disable", "monitor", "block"], "type": "str"},
            "external-blocklist": {"v_range": [["7.0.0", ""]], "choices": ["disable", "monitor", "block"], "type": "str"},
            "quarantine": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "fortindr": {"v_range": [["7.0.5", ""]], "choices": ["disable", "block", "monitor"], "type": "str"},
            "fortisandbox": {"v_range": [["7.2.0", ""]], "choices": ["disable", "block", "monitor"], "type": "str"},
            "fortiai": {"v_range": [["7.0.1", ""]], "choices": ["disable", "monitor", "block"], "type": "str"}
        }
    },
    "antivirus_profile_mapi": {
        "stated": false,
        "options": {
            "archive-block": {
                "type": "list",
                "choices": ["encrypted", "corrupted", "multipart", "nested", "mailbomb", "unhandled", "partiallycorrupted", "fileslimit", "timeout"],
                "elements": "str"
            },
            "archive-log": {
                "type": "list",
                "choices": ["encrypted", "corrupted", "multipart", "nested", "mailbomb", "unhandled", "partiallycorrupted", "fileslimit", "timeout"],
                "elements": "str"
            },
            "emulator": {"choices": ["disable", "enable"], "type": "str"},
            "executables": {"choices": ["default", "virus"], "type": "str"},
            "options": {"type": "list", "choices": ["scan", "quarantine", "avquery", "avmonitor"], "elements": "str"},
            "outbreak-prevention": {"choices": ["disabled", "files", "full-archive", "disable", "block", "monitor"], "type": "str"},
            "av-scan": {"v_range": [["7.0.0", ""]], "choices": ["disable", "monitor", "block"], "type": "str"},
            "external-blocklist": {"v_range": [["7.0.0", ""]], "choices": ["disable", "monitor", "block"], "type": "str"},
            "quarantine": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "fortindr": {"v_range": [["7.2.0", ""]], "choices": ["disable", "block", "monitor"], "type": "str"},
            "fortisandbox": {"v_range": [["7.2.0", ""]], "choices": ["disable", "block", "monitor"], "type": "str"},
            "fortiai": {"v_range": [["7.0.1", ""]], "choices": ["disable", "monitor", "block"], "type": "str"}
        }
    },
    "antivirus_profile_nacquar": {
        "stated": false,
        "options": {
            "expiry": {"type": "str"},
            "infected": {"choices": ["none", "quar-src-ip", "quar-interface"], "type": "str"},
            "log": {"choices": ["disable", "enable"], "type": "str"}
        }
    },
    "antivirus_profile_nntp": {
        "stated": false,
        "options": {
            "archive-block": {
                "type": "list",
                "choices": ["encrypted", "corrupted", "multipart", "nested", "mailbomb", "unhandled", "partiallycorrupted", "fileslimit", "timeout"],
                "elements": "str"
            },
            "archive-log": {
                "type": "list",
                "choices": ["encrypted", "corrupted", "multipart", "nested", "mailbomb", "unhandled", "partiallycorrupted", "fileslimit", "timeout"],
                "elements": "str"
            },
            "emulator": {"choices": ["disable", "enable"], "type": "str"},
            "options": {"type": "list", "choices": ["scan", "file-filter", "quarantine", "avquery", "avmonitor"], "elements": "str"},
            "outbreak-prevention": {"choices": ["disabled", "files", "full-archive", "disable", "block", "monitor"], "type": "str"},
            "av-scan": {"v_range": [["7.0.0", ""]], "choices": ["disable", "monitor", "block"], "type": "str"},
            "external-blocklist": {"v_range": [["7.0.0", ""]], "choices": ["disable", "monitor", "block"], "type": "str"},
            "quarantine": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "fortindr": {"v_range": [["7.0.5", ""]], "choices": ["disable", "block", "monitor"], "type": "str"},
            "fortisandbox": {"v_range": [["7.2.0", ""]], "choices": ["disable", "block", "monitor"], "type": "str"},
            "fortiai": {"v_range": [["7.0.1", ""]], "choices": ["disable", "monitor", "block"], "type": "str"}
        }
    },
    "antivirus_profile_outbreakprevention": {
        "stated": false,
        "options": {
            "external-blocklist": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ftgd-service": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "antivirus_profile_pop3": {
        "stated": false,
        "options": {
            "archive-block": {
                "type": "list",
                "choices": ["encrypted", "corrupted", "multipart", "nested", "mailbomb", "unhandled", "partiallycorrupted", "fileslimit", "timeout"],
                "elements": "str"
            },
            "archive-log": {
                "type": "list",
                "choices": ["encrypted", "corrupted", "multipart", "nested", "mailbomb", "unhandled", "partiallycorrupted", "fileslimit", "timeout"],
                "elements": "str"
            },
            "content-disarm": {"choices": ["disable", "enable"], "type": "str"},
            "emulator": {"choices": ["disable", "enable"], "type": "str"},
            "executables": {"choices": ["default", "virus"], "type": "str"},
            "options": {"type": "list", "choices": ["scan", "file-filter", "quarantine", "avquery", "avmonitor"], "elements": "str"},
            "outbreak-prevention": {"choices": ["disabled", "files", "full-archive", "disable", "block", "monitor"], "type": "str"},
            "av-scan": {"v_range": [["7.0.0", ""]], "choices": ["disable", "monitor", "block"], "type": "str"},
            "external-blocklist": {"v_range": [["7.0.0", ""]], "choices": ["disable", "monitor", "block"], "type": "str"},
            "quarantine": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "fortindr": {"v_range": [["7.0.5", ""]], "choices": ["disable", "block", "monitor"], "type": "str"},
            "fortisandbox": {"v_range": [["7.2.0", ""]], "choices": ["disable", "block", "monitor"], "type": "str"},
            "fortiai": {"v_range": [["7.0.1", ""]], "choices": ["disable", "monitor", "block"], "type": "str"}
        }
    },
    "antivirus_profile_smb": {
        "stated": false,
        "options": {
            "archive-block": {
                "v_range": [["6.0.0", "7.2.1"]],
                "type": "list",
                "choices": ["encrypted", "corrupted", "multipart", "nested", "mailbomb", "unhandled", "partiallycorrupted", "fileslimit", "timeout"],
                "elements": "str"
            },
            "archive-log": {
                "v_range": [["6.0.0", "7.2.1"]],
                "type": "list",
                "choices": ["encrypted", "corrupted", "multipart", "nested", "mailbomb", "unhandled", "partiallycorrupted", "fileslimit", "timeout"],
                "elements": "str"
            },
            "emulator": {"v_range": [["6.0.0", "7.2.1"]], "choices": ["disable", "enable"], "type": "str"},
            "options": {"v_range": [["6.0.0", "7.2.1"]], "type": "list", "choices": ["scan", "quarantine", "avquery", "avmonitor"], "elements": "str"},
            "outbreak-prevention": {"v_range": [["6.0.0", "7.2.1"]], "choices": ["disabled", "files", "full-archive"], "type": "str"}
        }
    },
    "antivirus_profile_smtp": {
        "stated": false,
        "options": {
            "archive-block": {
                "type": "list",
                "choices": ["encrypted", "corrupted", "multipart", "nested", "mailbomb", "unhandled", "partiallycorrupted", "fileslimit", "timeout"],
                "elements": "str"
            },
            "archive-log": {
                "type": "list",
                "choices": ["encrypted", "corrupted", "multipart", "nested", "mailbomb", "unhandled", "partiallycorrupted", "fileslimit", "timeout"],
                "elements": "str"
            },
            "content-disarm": {"choices": ["disable", "enable"], "type": "str"},
            "emulator": {"choices": ["disable", "enable"], "type": "str"},
            "executables": {"choices": ["default", "virus"], "type": "str"},
            "options": {"type": "list", "choices": ["scan", "file-filter", "quarantine", "avquery", "avmonitor"], "elements": "str"},
            "outbreak-prevention": {"choices": ["disabled", "files", "full-archive", "disable", "block", "monitor"], "type": "str"},
            "av-scan": {"v_range": [["7.0.0", ""]], "choices": ["disable", "monitor", "block"], "type": "str"},
            "external-blocklist": {"v_range": [["7.0.0", ""]], "choices": ["disable", "monitor", "block"], "type": "str"},
            "quarantine": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "fortindr": {"v_range": [["7.0.5", ""]], "choices": ["disable", "block", "monitor"], "type": "str"},
            "fortisandbox": {"v_range": [["7.2.0", ""]], "choices": ["disable", "block", "monitor"], "type": "str"},
            "fortiai": {"v_range": [["7.0.1", ""]], "choices": ["disable", "monitor", "block"], "type": "str"}
        }
    },
    "antivirus_profile_ssh": {
        "stated": false,
        "options": {
            "archive-block": {
                "v_range": [["6.2.2", ""]],
                "type": "list",
                "choices": ["encrypted", "corrupted", "multipart", "nested", "mailbomb", "unhandled", "partiallycorrupted", "fileslimit", "timeout"],
                "elements": "str"
            },
            "archive-log": {
                "v_range": [["6.2.2", ""]],
                "type": "list",
                "choices": ["encrypted", "corrupted", "multipart", "nested", "mailbomb", "unhandled", "partiallycorrupted", "fileslimit", "timeout"],
                "elements": "str"
            },
            "emulator": {"v_range": [["6.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "options": {"v_range": [["6.2.2", ""]], "type": "list", "choices": ["avmonitor", "quarantine", "scan"], "elements": "str"},
            "outbreak-prevention": {
                "v_range": [["6.2.2", ""]],
                "choices": ["disabled", "files", "full-archive", "disable", "block", "monitor"],
                "type": "str"
            },
            "av-scan": {"v_range": [["7.0.0", ""]], "choices": ["disable", "monitor", "block"], "type": "str"},
            "external-blocklist": {"v_range": [["7.0.0", ""]], "choices": ["disable", "monitor", "block"], "type": "str"},
            "quarantine": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "fortindr": {"v_range": [["7.0.5", ""]], "choices": ["disable", "block", "monitor"], "type": "str"},
            "fortisandbox": {"v_range": [["7.2.0", ""]], "choices": ["disable", "block", "monitor"], "type": "str"},
            "fortiai": {"v_range": [["7.0.1", ""]], "choices": ["disable", "monitor", "block"], "type": "str"}
        }
    },
    "apcfgprofile": {
        "stated": true,
        "options": {
            "ac-ip": {"v_range": [["6.4.6", ""]], "type": "str"},
            "ac-port": {"v_range": [["6.4.6", ""]], "type": "int"},
            "ac-timer": {"v_range": [["6.4.6", ""]], "type": "int"},
            "ac-type": {"v_range": [["6.4.6", ""]], "choices": ["default", "specify", "apcfg"], "type": "str"},
            "command-list": {
                "v_range": [["6.4.6", ""]],
                "type": "list",
                "options": {
                    "id": {"v_range": [["6.4.6", ""]], "type": "int"},
                    "name": {"v_range": [["6.4.6", ""]], "type": "str"},
                    "passwd-value": {"v_range": [["6.4.6", ""]], "no_log": true, "type": "raw"},
                    "type": {"v_range": [["6.4.6", ""]], "choices": ["non-password", "password"], "type": "str"},
                    "value": {"v_range": [["6.4.6", ""]], "type": "str"}
                },
                "elements": "dict"
            },
            "comment": {"v_range": [["6.4.6", ""]], "type": "str"},
            "name": {"v_range": [["6.4.6", ""]], "required": true, "type": "str"},
            "ap-family": {"v_range": [["6.4.8", "6.4.14"], ["7.0.2", ""]], "choices": ["fap", "fap-u", "fap-c"], "type": "str"}
        }
    },
    "apcfgprofile_commandlist": {
        "stated": true,
        "options": {
            "id": {"v_range": [["6.4.6", ""]], "required": true, "type": "int"},
            "name": {"v_range": [["6.4.6", ""]], "type": "str"},
            "passwd-value": {"v_range": [["6.4.6", ""]], "no_log": true, "type": "raw"},
            "type": {"v_range": [["6.4.6", ""]], "choices": ["non-password", "password"], "type": "str"},
            "value": {"v_range": [["6.4.6", ""]], "type": "str"}
        }
    },
    "application_casi_profile": {
        "stated": true,
        "options": {
            "app-replacemsg": {"v_range": [["6.2.0", "6.2.12"]], "choices": ["disable", "enable"], "type": "str"},
            "comment": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"},
            "entries": {
                "v_range": [["6.2.0", "6.2.12"]],
                "type": "list",
                "options": {
                    "action": {"v_range": [["6.2.0", "6.2.12"]], "choices": ["pass", "block", "reset"], "type": "str"},
                    "application": {"v_range": [["6.2.0", "6.2.12"]], "type": "raw"},
                    "id": {"v_range": [["6.2.0", "6.2.12"]], "type": "int"},
                    "log": {"v_range": [["6.2.0", "6.2.12"]], "choices": ["disable", "enable"], "type": "str"}
                },
                "elements": "dict"
            },
            "name": {"v_range": [["6.2.0", "6.2.12"]], "required": true, "type": "str"},
            "replacemsg-group": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"}
        }
    },
    "application_casi_profile_entries": {
        "stated": true,
        "options": {
            "action": {"v_range": [["6.2.0", "6.2.12"]], "choices": ["pass", "block", "reset"], "type": "str"},
            "application": {"v_range": [["6.2.0", "6.2.12"]], "type": "raw"},
            "id": {"v_range": [["6.2.0", "6.2.12"]], "required": true, "type": "int"},
            "log": {"v_range": [["6.2.0", "6.2.12"]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "application_categories": {"stated": true, "options": {"id": {"required": true, "type": "int"}}},
    "application_custom": {
        "stated": true,
        "options": {
            "behavior": {"type": "str"},
            "category": {"type": "str"},
            "comment": {"type": "str"},
            "id": {"type": "int"},
            "name": {"type": "str"},
            "protocol": {"type": "str"},
            "signature": {"type": "str"},
            "tag": {"required": true, "type": "str"},
            "technology": {"type": "str"},
            "vendor": {"type": "str"}
        }
    },
    "application_group": {
        "stated": true,
        "options": {
            "application": {"type": "raw"},
            "category": {"type": "raw"},
            "comment": {"type": "str"},
            "name": {"required": true, "type": "str"},
            "type": {"choices": ["application", "category", "filter"], "type": "str"},
            "behavior": {"v_range": [["7.0.0", ""]], "type": "raw"},
            "popularity": {"v_range": [["7.0.0", ""]], "type": "list", "choices": ["1", "2", "3", "4", "5"], "elements": "str"},
            "protocols": {"v_range": [["7.0.0", ""]], "type": "raw"},
            "risk": {"v_range": [["7.0.0", ""]], "type": "raw"},
            "technology": {"v_range": [["7.0.0", ""]], "type": "raw"},
            "vendor": {"v_range": [["7.0.0", ""]], "type": "raw"}
        }
    },
    "application_internetservice": {
        "stated": false,
        "options": {
            "entry": {
                "v_range": [["6.2.0", "6.2.12"]],
                "type": "list",
                "options": {
                    "id": {"v_range": [["6.2.0", "6.2.12"]], "type": "int"},
                    "ip-number": {"v_range": [["6.2.0", "6.2.12"]], "type": "int"},
                    "ip-range-number": {"v_range": [["6.2.0", "6.2.12"]], "type": "int"},
                    "port": {"v_range": [["6.2.0", "6.2.12"]], "type": "raw"},
                    "protocol": {"v_range": [["6.2.0", "6.2.12"]], "type": "int"}
                },
                "elements": "dict"
            },
            "id": {"v_range": [["6.2.0", "6.2.12"]], "type": "int"},
            "name": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"},
            "offset": {"v_range": [["6.2.0", "6.2.12"]], "type": "int"},
            "reputation": {"v_range": [["6.2.0", "6.2.12"]], "type": "int"}
        }
    },
    "application_internetservice_entry": {
        "stated": true,
        "options": {
            "id": {"v_range": [["6.2.0", "6.2.12"]], "required": true, "type": "int"},
            "ip-number": {"v_range": [["6.2.0", "6.2.12"]], "type": "int"},
            "ip-range-number": {"v_range": [["6.2.0", "6.2.12"]], "type": "int"},
            "port": {"v_range": [["6.2.0", "6.2.12"]], "type": "raw"},
            "protocol": {"v_range": [["6.2.0", "6.2.12"]], "type": "int"}
        }
    },
    "application_internetservicecustom": {
        "stated": true,
        "options": {
            "comment": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"},
            "disable-entry": {
                "v_range": [["6.2.0", "6.2.12"]],
                "type": "list",
                "options": {
                    "id": {"v_range": [["6.2.0", "6.2.12"]], "type": "int"},
                    "ip-range": {
                        "v_range": [["6.2.0", "6.2.12"]],
                        "type": "list",
                        "options": {
                            "end-ip": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"},
                            "id": {"v_range": [["6.2.0", "6.2.12"]], "type": "int"},
                            "start-ip": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"}
                        },
                        "elements": "dict"
                    },
                    "port": {"v_range": [["6.2.0", "6.2.12"]], "type": "raw"},
                    "protocol": {"v_range": [["6.2.0", "6.2.12"]], "type": "int"}
                },
                "elements": "dict"
            },
            "entry": {
                "v_range": [["6.2.0", "6.2.12"]],
                "type": "list",
                "options": {
                    "dst": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"},
                    "id": {"v_range": [["6.2.0", "6.2.12"]], "type": "int"},
                    "port-range": {
                        "v_range": [["6.2.0", "6.2.12"]],
                        "type": "list",
                        "options": {
                            "end-port": {"v_range": [["6.2.0", "6.2.12"]], "type": "int"},
                            "id": {"v_range": [["6.2.0", "6.2.12"]], "type": "int"},
                            "start-port": {"v_range": [["6.2.0", "6.2.12"]], "type": "int"}
                        },
                        "elements": "dict"
                    },
                    "protocol": {"v_range": [["6.2.0", "6.2.12"]], "type": "int"}
                },
                "elements": "dict"
            },
            "master-service-id": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"},
            "name": {"v_range": [["6.2.0", "6.2.12"]], "required": true, "type": "str"}
        }
    },
    "application_internetservicecustom_disableentry": {
        "stated": true,
        "options": {
            "id": {"v_range": [["6.2.0", "6.2.12"]], "required": true, "type": "int"},
            "ip-range": {
                "v_range": [["6.2.0", "6.2.12"]],
                "type": "list",
                "options": {
                    "end-ip": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"},
                    "id": {"v_range": [["6.2.0", "6.2.12"]], "type": "int"},
                    "start-ip": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"}
                },
                "elements": "dict"
            },
            "port": {"v_range": [["6.2.0", "6.2.12"]], "type": "raw"},
            "protocol": {"v_range": [["6.2.0", "6.2.12"]], "type": "int"}
        }
    },
    "application_internetservicecustom_disableentry_iprange": {
        "stated": true,
        "options": {
            "end-ip": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"},
            "id": {"v_range": [["6.2.0", "6.2.12"]], "required": true, "type": "int"},
            "start-ip": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"}
        }
    },
    "application_internetservicecustom_entry": {
        "stated": true,
        "options": {
            "dst": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"},
            "id": {"v_range": [["6.2.0", "6.2.12"]], "required": true, "type": "int"},
            "port-range": {
                "v_range": [["6.2.0", "6.2.12"]],
                "type": "list",
                "options": {
                    "end-port": {"v_range": [["6.2.0", "6.2.12"]], "type": "int"},
                    "id": {"v_range": [["6.2.0", "6.2.12"]], "type": "int"},
                    "start-port": {"v_range": [["6.2.0", "6.2.12"]], "type": "int"}
                },
                "elements": "dict"
            },
            "protocol": {"v_range": [["6.2.0", "6.2.12"]], "type": "int"}
        }
    },
    "application_internetservicecustom_entry_portrange": {
        "stated": true,
        "options": {
            "end-port": {"v_range": [["6.2.0", "6.2.12"]], "type": "int"},
            "id": {"v_range": [["6.2.0", "6.2.12"]], "required": true, "type": "int"},
            "start-port": {"v_range": [["6.2.0", "6.2.12"]], "type": "int"}
        }
    },
    "application_list": {
        "stated": true,
        "options": {
            "app-replacemsg": {"choices": ["disable", "enable"], "type": "str"},
            "comment": {"type": "str"},
            "deep-app-inspection": {"choices": ["disable", "enable"], "type": "str"},
            "entries": {
                "type": "list",
                "options": {
                    "action": {"choices": ["pass", "block", "reset"], "type": "str"},
                    "application": {"type": "raw"},
                    "behavior": {"type": "raw"},
                    "category": {"type": "raw"},
                    "id": {"type": "int"},
                    "log": {"choices": ["disable", "enable"], "type": "str"},
                    "log-packet": {"choices": ["disable", "enable"], "type": "str"},
                    "parameters": {
                        "type": "list",
                        "options": {
                            "id": {"type": "int"},
                            "value": {"type": "str"},
                            "members": {
                                "v_range": [["6.4.0", ""]],
                                "type": "list",
                                "options": {
                                    "id": {"v_range": [["6.4.0", ""]], "type": "int"},
                                    "name": {"v_range": [["6.4.0", ""]], "type": "str"},
                                    "value": {"v_range": [["6.4.0", ""]], "type": "str"}
                                },
                                "elements": "dict"
                            }
                        },
                        "elements": "dict"
                    },
                    "per-ip-shaper": {"type": "str"},
                    "popularity": {"type": "list", "choices": ["1", "2", "3", "4", "5"], "elements": "str"},
                    "protocols": {"type": "raw"},
                    "quarantine": {"choices": ["none", "attacker"], "type": "str"},
                    "quarantine-expiry": {"type": "str"},
                    "quarantine-log": {"choices": ["disable", "enable"], "type": "str"},
                    "rate-count": {"type": "int"},
                    "rate-duration": {"type": "int"},
                    "rate-mode": {"choices": ["periodical", "continuous"], "type": "str"},
                    "rate-track": {"choices": ["none", "src-ip", "dest-ip", "dhcp-client-mac", "dns-domain"], "type": "str"},
                    "risk": {"type": "raw"},
                    "session-ttl": {"type": "int"},
                    "shaper": {"type": "str"},
                    "shaper-reverse": {"type": "str"},
                    "sub-category": {"type": "raw"},
                    "technology": {"type": "raw"},
                    "vendor": {"type": "raw"},
                    "tags": {"v_range": [["6.2.0", "6.4.14"]], "type": "str"},
                    "exclusion": {"v_range": [["6.2.7", "6.2.12"], ["6.4.3", ""]], "type": "raw"}
                },
                "elements": "dict"
            },
            "extended-log": {"choices": ["disable", "enable"], "type": "str"},
            "name": {"required": true, "type": "str"},
            "options": {"type": "list", "choices": ["allow-dns", "allow-icmp", "allow-http", "allow-ssl", "allow-quic"], "elements": "str"},
            "other-application-action": {"choices": ["pass", "block"], "type": "str"},
            "other-application-log": {"choices": ["disable", "enable"], "type": "str"},
            "p2p-black-list": {"type": "list", "choices": ["skype", "edonkey", "bittorrent"], "elements": "str"},
            "replacemsg-group": {"type": "str"},
            "unknown-application-action": {"choices": ["pass", "block"], "type": "str"},
            "unknown-application-log": {"choices": ["disable", "enable"], "type": "str"},
            "control-default-network-services": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "default-network-services": {
                "v_range": [["6.2.0", ""]],
                "type": "list",
                "options": {
                    "id": {"v_range": [["6.2.0", ""]], "type": "int"},
                    "port": {"v_range": [["6.2.0", ""]], "type": "int"},
                    "services": {
                        "v_range": [["6.2.0", ""]],
                        "type": "list",
                        "choices": ["http", "ssh", "telnet", "ftp", "dns", "smtp", "pop3", "imap", "snmp", "nntp", "https"],
                        "elements": "str"
                    },
                    "violation-action": {"v_range": [["6.2.0", ""]], "choices": ["block", "monitor", "pass"], "type": "str"}
                },
                "elements": "dict"
            },
            "enforce-default-app-port": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "force-inclusion-ssl-di-sigs": {"v_range": [["6.2.5", ""]], "choices": ["disable", "enable"], "type": "str"},
            "p2p-block-list": {"v_range": [["7.0.0", ""]], "type": "list", "choices": ["skype", "edonkey", "bittorrent"], "elements": "str"}
        }
    },
    "application_list_defaultnetworkservices": {
        "stated": true,
        "options": {
            "id": {"v_range": [["6.2.0", ""]], "required": true, "type": "int"},
            "port": {"v_range": [["6.2.0", ""]], "type": "int"},
            "services": {
                "v_range": [["6.2.0", ""]],
                "type": "list",
                "choices": ["http", "ssh", "telnet", "ftp", "dns", "smtp", "pop3", "imap", "snmp", "nntp", "https"],
                "elements": "str"
            },
            "violation-action": {"v_range": [["6.2.0", ""]], "choices": ["block", "monitor", "pass"], "type": "str"}
        }
    },
    "application_list_entries": {
        "stated": true,
        "options": {
            "action": {"choices": ["pass", "block", "reset"], "type": "str"},
            "application": {"type": "raw"},
            "behavior": {"type": "raw"},
            "category": {"type": "raw"},
            "id": {"required": true, "type": "int"},
            "log": {"choices": ["disable", "enable"], "type": "str"},
            "log-packet": {"choices": ["disable", "enable"], "type": "str"},
            "parameters": {
                "type": "list",
                "options": {
                    "id": {"type": "int"},
                    "value": {"type": "str"},
                    "members": {
                        "v_range": [["6.4.0", ""]],
                        "type": "list",
                        "options": {
                            "id": {"v_range": [["6.4.0", ""]], "type": "int"},
                            "name": {"v_range": [["6.4.0", ""]], "type": "str"},
                            "value": {"v_range": [["6.4.0", ""]], "type": "str"}
                        },
                        "elements": "dict"
                    }
                },
                "elements": "dict"
            },
            "per-ip-shaper": {"type": "str"},
            "popularity": {"type": "list", "choices": ["1", "2", "3", "4", "5"], "elements": "str"},
            "protocols": {"type": "raw"},
            "quarantine": {"choices": ["none", "attacker"], "type": "str"},
            "quarantine-expiry": {"type": "str"},
            "quarantine-log": {"choices": ["disable", "enable"], "type": "str"},
            "rate-count": {"type": "int"},
            "rate-duration": {"type": "int"},
            "rate-mode": {"choices": ["periodical", "continuous"], "type": "str"},
            "rate-track": {"choices": ["none", "src-ip", "dest-ip", "dhcp-client-mac", "dns-domain"], "type": "str"},
            "risk": {"type": "raw"},
            "session-ttl": {"type": "int"},
            "shaper": {"type": "str"},
            "shaper-reverse": {"type": "str"},
            "sub-category": {"type": "raw"},
            "technology": {"type": "raw"},
            "vendor": {"type": "raw"},
            "tags": {"v_range": [["6.2.0", "6.4.14"]], "type": "str"},
            "exclusion": {"v_range": [["6.2.7", "6.2.12"], ["6.4.3", ""]], "type": "raw"}
        }
    },
    "application_list_entries_parameters": {
        "stated": true,
        "options": {
            "id": {"required": true, "type": "int"},
            "value": {"type": "str"},
            "members": {
                "v_range": [["6.4.0", ""]],
                "type": "list",
                "options": {
                    "id": {"v_range": [["6.4.0", ""]], "type": "int"},
                    "name": {"v_range": [["6.4.0", ""]], "type": "str"},
                    "value": {"v_range": [["6.4.0", ""]], "type": "str"}
                },
                "elements": "dict"
            }
        }
    },
    "application_list_entries_parameters_members": {
        "stated": true,
        "options": {
            "id": {"v_range": [["6.4.0", ""]], "required": true, "type": "int"},
            "name": {"v_range": [["6.4.0", ""]], "type": "str"},
            "value": {"v_range": [["6.4.0", ""]], "type": "str"}
        }
    },
    "arrpprofile": {
        "stated": true,
        "options": {
            "comment": {"v_range": [["7.0.3", ""]], "type": "str"},
            "darrp-optimize": {"v_range": [["7.0.3", ""]], "type": "int"},
            "darrp-optimize-schedules": {"v_range": [["7.0.3", ""]], "type": "raw"},
            "include-dfs-channel": {"v_range": [["7.0.3", ""]], "choices": ["no", "disable", "yes", "enable"], "type": "str"},
            "include-weather-channel": {"v_range": [["7.0.3", ""]], "choices": ["no", "disable", "yes", "enable"], "type": "str"},
            "monitor-period": {"v_range": [["7.0.3", ""]], "type": "int"},
            "name": {"v_range": [["7.0.3", ""]], "required": true, "type": "str"},
            "override-darrp-optimize": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "selection-period": {"v_range": [["7.0.3", ""]], "type": "int"},
            "threshold-ap": {"v_range": [["7.0.3", ""]], "type": "int"},
            "threshold-channel-load": {"v_range": [["7.0.3", ""]], "type": "int"},
            "threshold-noise-floor": {"v_range": [["7.0.3", ""]], "type": "str"},
            "threshold-rx-errors": {"v_range": [["7.0.3", ""]], "type": "int"},
            "threshold-spectral-rssi": {"v_range": [["7.0.3", ""]], "type": "str"},
            "threshold-tx-retries": {"v_range": [["7.0.3", ""]], "type": "int"},
            "weight-channel-load": {"v_range": [["7.0.3", ""]], "type": "int"},
            "weight-dfs-channel": {"v_range": [["7.0.3", ""]], "type": "int"},
            "weight-managed-ap": {"v_range": [["7.0.3", ""]], "type": "int"},
            "weight-noise-floor": {"v_range": [["7.0.3", ""]], "type": "int"},
            "weight-rogue-ap": {"v_range": [["7.0.3", ""]], "type": "int"},
            "weight-spectral-rssi": {"v_range": [["7.0.3", ""]], "type": "int"},
            "weight-weather-channel": {"v_range": [["7.0.3", ""]], "type": "int"}
        }
    },
    "authentication_scheme": {
        "stated": true,
        "options": {
            "domain-controller": {"v_range": [["6.2.1", ""]], "type": "str"},
            "fsso-agent-for-ntlm": {"v_range": [["6.2.1", ""]], "type": "str"},
            "fsso-guest": {"v_range": [["6.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "kerberos-keytab": {"v_range": [["6.2.1", ""]], "no_log": true, "type": "str"},
            "method": {
                "v_range": [["6.2.1", ""]],
                "type": "list",
                "choices": ["ntlm", "basic", "digest", "form", "negotiate", "fsso", "rsso", "ssh-publickey", "saml", "cert", "x-auth-user", "saml-sp"],
                "elements": "str"
            },
            "name": {"v_range": [["6.2.1", ""]], "required": true, "type": "str"},
            "negotiate-ntlm": {"v_range": [["6.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "require-tfa": {"v_range": [["6.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ssh-ca": {"v_range": [["6.2.1", ""]], "type": "str"},
            "user-database": {"v_range": [["6.2.1", ""]], "type": "raw"},
            "ems-device-owner": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "saml-server": {"v_range": [["7.0.0", ""]], "type": "str"},
            "saml-timeout": {"v_range": [["7.0.0", ""]], "type": "int"},
            "user-cert": {"v_range": [["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "bleprofile": {
        "stated": true,
        "options": {
            "advertising": {"type": "list", "choices": ["ibeacon", "eddystone-uid", "eddystone-url"], "elements": "str"},
            "beacon-interval": {"type": "int"},
            "ble-scanning": {"choices": ["disable", "enable"], "type": "str"},
            "comment": {"type": "str"},
            "eddystone-instance": {"type": "str"},
            "eddystone-namespace": {"type": "str"},
            "eddystone-url": {"type": "str"},
            "eddystone-url-encode-hex": {"type": "str"},
            "ibeacon-uuid": {"type": "str"},
            "major-id": {"type": "int"},
            "minor-id": {"type": "int"},
            "name": {"required": true, "type": "str"},
            "txpower": {"choices": ["0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "10", "11", "12"], "type": "str"},
            "scan-interval": {"v_range": [["7.4.1", ""]], "type": "int"},
            "scan-period": {"v_range": [["7.4.1", ""]], "type": "int"},
            "scan-threshold": {"v_range": [["7.4.1", ""]], "type": "str"},
            "scan-time": {"v_range": [["7.4.1", ""]], "type": "int"},
            "scan-type": {"v_range": [["7.4.1", ""]], "choices": ["active", "passive"], "type": "str"},
            "scan-window": {"v_range": [["7.4.1", ""]], "type": "int"}
        }
    },
    "bonjourprofile": {
        "stated": true,
        "options": {
            "comment": {"type": "str"},
            "name": {"required": true, "type": "str"},
            "policy-list": {
                "type": "list",
                "options": {
                    "description": {"type": "str"},
                    "from-vlan": {"type": "str"},
                    "policy-id": {"type": "int"},
                    "services": {
                        "type": "list",
                        "choices": [
                            "airplay", "afp", "bit-torrent", "ftp", "ichat", "itunes", "printers", "samba", "scanners", "ssh", "chromecast", "all",
                            "miracast"
                        ],
                        "elements": "str"
                    },
                    "to-vlan": {"type": "str"}
                },
                "elements": "dict"
            },
            "micro-location": {"v_range": [["7.6.0", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "bonjourprofile_policylist": {
        "stated": true,
        "options": {
            "description": {"type": "str"},
            "from-vlan": {"type": "str"},
            "policy-id": {"required": true, "type": "int"},
            "services": {
                "type": "list",
                "choices": [
                    "airplay", "afp", "bit-torrent", "ftp", "ichat", "itunes", "printers", "samba", "scanners", "ssh", "chromecast", "all", "miracast"
                ],
                "elements": "str"
            },
            "to-vlan": {"type": "str"}
        }
    },
    "casb_profile": {
        "stated": true,
        "options": {
            "name": {"v_range": [["7.4.1", ""]], "required": true, "type": "str"},
            "saas-application": {
                "v_range": [["7.4.1", ""]],
                "type": "list",
                "options": {
                    "access-rule": {
                        "v_range": [["7.4.1", ""]],
                        "type": "list",
                        "options": {
                            "action": {"v_range": [["7.4.1", ""]], "choices": ["block", "bypass", "monitor"], "type": "str"},
                            "bypass": {
                                "v_range": [["7.4.1", ""]],
                                "type": "list",
                                "choices": ["av", "dlp", "web-filter", "file-filter", "video-filter"],
                                "elements": "str"
                            },
                            "name": {"v_range": [["7.4.1", ""]], "type": "str"}
                        },
                        "elements": "dict"
                    },
                    "custom-control": {
                        "v_range": [["7.4.1", ""]],
                        "type": "list",
                        "options": {
                            "name": {"v_range": [["7.4.1", ""]], "type": "str"},
                            "option": {
                                "v_range": [["7.4.1", ""]],
                                "type": "list",
                                "options": {
                                    "name": {"v_range": [["7.4.1", ""]], "type": "str"},
                                    "user-input": {"v_range": [["7.4.1", ""]], "type": "list", "elements": "str"}
                                },
                                "elements": "dict"
                            }
                        },
                        "elements": "dict"
                    },
                    "domain-control": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "domain-control-domains": {"v_range": [["7.4.1", ""]], "type": "list", "elements": "str"},
                    "log": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "name": {"v_range": [["7.4.1", ""]], "type": "str"},
                    "safe-search": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "safe-search-control": {"v_range": [["7.4.1", ""]], "type": "list", "elements": "str"},
                    "tenant-control": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "tenant-control-tenants": {"v_range": [["7.4.1", ""]], "type": "list", "elements": "str"},
                    "status": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"}
                },
                "elements": "dict"
            },
            "comment": {"v_range": [["7.4.3", ""]], "type": "str"}
        }
    },
    "casb_profile_saasapplication": {
        "stated": true,
        "options": {
            "access-rule": {
                "v_range": [["7.4.1", ""]],
                "type": "list",
                "options": {
                    "action": {"v_range": [["7.4.1", ""]], "choices": ["block", "bypass", "monitor"], "type": "str"},
                    "bypass": {
                        "v_range": [["7.4.1", ""]],
                        "type": "list",
                        "choices": ["av", "dlp", "web-filter", "file-filter", "video-filter"],
                        "elements": "str"
                    },
                    "name": {"v_range": [["7.4.1", ""]], "type": "str"}
                },
                "elements": "dict"
            },
            "custom-control": {
                "v_range": [["7.4.1", ""]],
                "type": "list",
                "options": {
                    "name": {"v_range": [["7.4.1", ""]], "type": "str"},
                    "option": {
                        "v_range": [["7.4.1", ""]],
                        "type": "list",
                        "options": {
                            "name": {"v_range": [["7.4.1", ""]], "type": "str"},
                            "user-input": {"v_range": [["7.4.1", ""]], "type": "list", "elements": "str"}
                        },
                        "elements": "dict"
                    }
                },
                "elements": "dict"
            },
            "domain-control": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "domain-control-domains": {"v_range": [["7.4.1", ""]], "type": "list", "elements": "str"},
            "log": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "name": {"v_range": [["7.4.1", ""]], "required": true, "type": "str"},
            "safe-search": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "safe-search-control": {"v_range": [["7.4.1", ""]], "type": "list", "elements": "str"},
            "tenant-control": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "tenant-control-tenants": {"v_range": [["7.4.1", ""]], "type": "list", "elements": "str"},
            "status": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "casb_profile_saasapplication_accessrule": {
        "stated": true,
        "options": {
            "action": {"v_range": [["7.4.1", ""]], "choices": ["block", "bypass", "monitor"], "type": "str"},
            "bypass": {
                "v_range": [["7.4.1", ""]],
                "type": "list",
                "choices": ["av", "dlp", "web-filter", "file-filter", "video-filter"],
                "elements": "str"
            },
            "name": {"v_range": [["7.4.1", ""]], "required": true, "type": "str"}
        }
    },
    "casb_profile_saasapplication_customcontrol": {
        "stated": true,
        "options": {
            "name": {"v_range": [["7.4.1", ""]], "required": true, "type": "str"},
            "option": {
                "v_range": [["7.4.1", ""]],
                "type": "list",
                "options": {
                    "name": {"v_range": [["7.4.1", ""]], "type": "str"},
                    "user-input": {"v_range": [["7.4.1", ""]], "type": "list", "elements": "str"}
                },
                "elements": "dict"
            }
        }
    },
    "casb_profile_saasapplication_customcontrol_option": {
        "stated": true,
        "options": {
            "name": {"v_range": [["7.4.1", ""]], "required": true, "type": "str"},
            "user-input": {"v_range": [["7.4.1", ""]], "type": "list", "elements": "str"}
        }
    },
    "casb_saasapplication": {
        "stated": true,
        "options": {
            "casb-name": {"v_range": [["7.4.1", ""]], "type": "str"},
            "description": {"v_range": [["7.4.1", ""]], "type": "str"},
            "domains": {"v_range": [["7.4.1", ""]], "type": "list", "elements": "str"},
            "name": {"v_range": [["7.4.1", ""]], "required": true, "type": "str"},
            "type": {"v_range": [["7.4.1", ""]], "choices": ["built-in", "customized"], "type": "str"},
            "uuid": {"v_range": [["7.4.1", ""]], "type": "str"},
            "status": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "casb_useractivity": {
        "stated": true,
        "options": {
            "application": {"v_range": [["7.4.1", ""]], "type": "str"},
            "casb-name": {"v_range": [["7.4.1", ""]], "type": "str"},
            "category": {
                "v_range": [["7.4.1", ""]],
                "choices": ["activity-control", "tenant-control", "domain-control", "safe-search-control", "other"],
                "type": "str"
            },
            "control-options": {
                "v_range": [["7.4.1", ""]],
                "type": "list",
                "options": {
                    "name": {"v_range": [["7.4.1", ""]], "type": "str"},
                    "operations": {
                        "v_range": [["7.4.1", ""]],
                        "type": "list",
                        "options": {
                            "action": {
                                "v_range": [["7.4.1", ""]],
                                "choices": ["append", "prepend", "replace", "new", "new-on-not-found", "delete"],
                                "type": "str"
                            },
                            "case-sensitive": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "direction": {"v_range": [["7.4.1", ""]], "choices": ["request"], "type": "str"},
                            "header-name": {"v_range": [["7.4.1", ""]], "type": "str"},
                            "name": {"v_range": [["7.4.1", ""]], "type": "str"},
                            "search-key": {"v_range": [["7.4.1", ""]], "no_log": true, "type": "str"},
                            "search-pattern": {"v_range": [["7.4.1", ""]], "choices": ["simple", "substr", "regexp"], "type": "str"},
                            "target": {"v_range": [["7.4.1", ""]], "choices": ["header", "path"], "type": "str"},
                            "value-from-input": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "values": {"v_range": [["7.4.1", ""]], "type": "list", "elements": "str"}
                        },
                        "elements": "dict"
                    },
                    "status": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"}
                },
                "elements": "dict"
            },
            "description": {"v_range": [["7.4.1", ""]], "type": "str"},
            "match": {
                "v_range": [["7.4.1", ""]],
                "type": "list",
                "options": {
                    "id": {"v_range": [["7.4.1", ""]], "type": "int"},
                    "rules": {
                        "v_range": [["7.4.1", ""]],
                        "type": "list",
                        "options": {
                            "case-sensitive": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "domains": {"v_range": [["7.4.1", ""]], "type": "list", "elements": "str"},
                            "header-name": {"v_range": [["7.4.1", ""]], "type": "str"},
                            "id": {"v_range": [["7.4.1", ""]], "type": "int"},
                            "match-pattern": {"v_range": [["7.4.1", ""]], "choices": ["simple", "substr", "regexp"], "type": "str"},
                            "match-value": {"v_range": [["7.4.1", ""]], "type": "str"},
                            "methods": {"v_range": [["7.4.1", ""]], "type": "list", "elements": "str"},
                            "negate": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "type": {
                                "v_range": [["7.4.1", ""]],
                                "choices": ["domains", "host", "path", "header", "header-value", "method"],
                                "type": "str"
                            }
                        },
                        "elements": "dict"
                    },
                    "strategy": {"v_range": [["7.4.1", ""]], "choices": ["or", "and"], "type": "str"}
                },
                "elements": "dict"
            },
            "match-strategy": {"v_range": [["7.4.1", ""]], "choices": ["or", "and"], "type": "str"},
            "name": {"v_range": [["7.4.1", ""]], "required": true, "type": "str"},
            "type": {"v_range": [["7.4.1", ""]], "choices": ["built-in", "customized"], "type": "str"},
            "uuid": {"v_range": [["7.4.1", ""]], "type": "str"},
            "status": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "casb_useractivity_controloptions": {
        "stated": true,
        "options": {
            "name": {"v_range": [["7.4.1", ""]], "required": true, "type": "str"},
            "operations": {
                "v_range": [["7.4.1", ""]],
                "type": "list",
                "options": {
                    "action": {
                        "v_range": [["7.4.1", ""]],
                        "choices": ["append", "prepend", "replace", "new", "new-on-not-found", "delete"],
                        "type": "str"
                    },
                    "case-sensitive": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "direction": {"v_range": [["7.4.1", ""]], "choices": ["request"], "type": "str"},
                    "header-name": {"v_range": [["7.4.1", ""]], "type": "str"},
                    "name": {"v_range": [["7.4.1", ""]], "type": "str"},
                    "search-key": {"v_range": [["7.4.1", ""]], "no_log": true, "type": "str"},
                    "search-pattern": {"v_range": [["7.4.1", ""]], "choices": ["simple", "substr", "regexp"], "type": "str"},
                    "target": {"v_range": [["7.4.1", ""]], "choices": ["header", "path"], "type": "str"},
                    "value-from-input": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "values": {"v_range": [["7.4.1", ""]], "type": "list", "elements": "str"}
                },
                "elements": "dict"
            },
            "status": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "casb_useractivity_controloptions_operations": {
        "stated": true,
        "options": {
            "action": {"v_range": [["7.4.1", ""]], "choices": ["append", "prepend", "replace", "new", "new-on-not-found", "delete"], "type": "str"},
            "case-sensitive": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "direction": {"v_range": [["7.4.1", ""]], "choices": ["request"], "type": "str"},
            "header-name": {"v_range": [["7.4.1", ""]], "type": "str"},
            "name": {"v_range": [["7.4.1", ""]], "required": true, "type": "str"},
            "search-key": {"v_range": [["7.4.1", ""]], "no_log": true, "type": "str"},
            "search-pattern": {"v_range": [["7.4.1", ""]], "choices": ["simple", "substr", "regexp"], "type": "str"},
            "target": {"v_range": [["7.4.1", ""]], "choices": ["header", "path"], "type": "str"},
            "value-from-input": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "values": {"v_range": [["7.4.1", ""]], "type": "list", "elements": "str"}
        }
    },
    "casb_useractivity_match": {
        "stated": true,
        "options": {
            "id": {"v_range": [["7.4.1", ""]], "required": true, "type": "int"},
            "rules": {
                "v_range": [["7.4.1", ""]],
                "type": "list",
                "options": {
                    "case-sensitive": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "domains": {"v_range": [["7.4.1", ""]], "type": "list", "elements": "str"},
                    "header-name": {"v_range": [["7.4.1", ""]], "type": "str"},
                    "id": {"v_range": [["7.4.1", ""]], "type": "int"},
                    "match-pattern": {"v_range": [["7.4.1", ""]], "choices": ["simple", "substr", "regexp"], "type": "str"},
                    "match-value": {"v_range": [["7.4.1", ""]], "type": "str"},
                    "methods": {"v_range": [["7.4.1", ""]], "type": "list", "elements": "str"},
                    "negate": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "type": {"v_range": [["7.4.1", ""]], "choices": ["domains", "host", "path", "header", "header-value", "method"], "type": "str"}
                },
                "elements": "dict"
            },
            "strategy": {"v_range": [["7.4.1", ""]], "choices": ["or", "and"], "type": "str"}
        }
    },
    "casb_useractivity_match_rules": {
        "stated": true,
        "options": {
            "case-sensitive": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "domains": {"v_range": [["7.4.1", ""]], "type": "list", "elements": "str"},
            "header-name": {"v_range": [["7.4.1", ""]], "type": "str"},
            "id": {"v_range": [["7.4.1", ""]], "required": true, "type": "int"},
            "match-pattern": {"v_range": [["7.4.1", ""]], "choices": ["simple", "substr", "regexp"], "type": "str"},
            "match-value": {"v_range": [["7.4.1", ""]], "type": "str"},
            "methods": {"v_range": [["7.4.1", ""]], "type": "list", "elements": "str"},
            "negate": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "type": {"v_range": [["7.4.1", ""]], "choices": ["domains", "host", "path", "header", "header-value", "method"], "type": "str"}
        }
    },
    "certificate_template": {
        "stated": true,
        "options": {
            "city": {"type": "str"},
            "country": {"type": "str"},
            "digest-type": {"choices": ["sha1", "sha256"], "type": "str"},
            "email": {"type": "str"},
            "id-type": {"choices": ["host-ip", "domain-name", "email"], "type": "str"},
            "key-size": {"choices": ["512", "1024", "1536", "2048", "4096"], "type": "str"},
            "key-type": {"choices": ["rsa", "ec"], "type": "str"},
            "name": {"required": true, "type": "str"},
            "organization": {"type": "str"},
            "organization-unit": {"type": "raw"},
            "scep-password": {"no_log": true, "type": "raw"},
            "scep-server": {"type": "str"},
            "state": {"type": "str"},
            "subject-name": {"type": "str"},
            "type": {"choices": ["external", "local"], "type": "str"},
            "curve-name": {"v_range": [["6.2.1", ""]], "choices": ["secp256r1", "secp384r1", "secp521r1"], "type": "str"},
            "scep-ca-identifier": {"v_range": [["7.0.4", ""]], "type": "str"}
        }
    },
    "cifs_domaincontroller": {
        "stated": true,
        "options": {
            "domain-name": {"v_range": [["6.2.0", ""]], "type": "str"},
            "ip": {"v_range": [["6.2.0", ""]], "type": "str"},
            "ip6": {"v_range": [["6.2.0", ""]], "type": "str"},
            "password": {"v_range": [["6.2.0", ""]], "no_log": true, "type": "raw"},
            "port": {"v_range": [["6.2.0", ""]], "type": "int"},
            "server-name": {"v_range": [["6.2.0", ""]], "type": "str"},
            "username": {"v_range": [["6.2.0", ""]], "type": "str"}
        }
    },
    "cifs_profile": {
        "stated": true,
        "options": {
            "domain-controller": {"v_range": [["6.2.0", ""]], "type": "str"},
            "name": {"v_range": [["6.2.0", ""]], "required": true, "type": "str"},
            "server-credential-type": {"v_range": [["6.2.0", ""]], "choices": ["none", "credential-replication", "credential-keytab"], "type": "str"},
            "server-keytab": {
                "v_range": [["6.2.0", ""]],
                "no_log": true,
                "type": "list",
                "options": {
                    "keytab": {"v_range": [["6.2.0", ""]], "no_log": true, "type": "str"},
                    "principal": {"v_range": [["6.2.0", ""]], "type": "str"},
                    "password": {"v_range": [["6.2.2", ""]], "no_log": true, "type": "raw"}
                },
                "elements": "dict"
            },
            "file-filter": {
                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                "type": "dict",
                "options": {
                    "entries": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "type": "list",
                        "options": {
                            "action": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["block", "log"], "type": "str"},
                            "comment": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                            "direction": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["incoming", "outgoing", "any"], "type": "str"},
                            "file-type": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "raw"},
                            "filter": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                            "protocol": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "list", "choices": ["cifs"], "elements": "str"}
                        },
                        "elements": "dict"
                    },
                    "log": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "status": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"}
                }
            }
        }
    },
    "cifs_profile_filefilter": {
        "stated": false,
        "options": {
            "entries": {
                "v_range": [["6.2.0", ""]],
                "type": "list",
                "options": {
                    "action": {"v_range": [["6.2.0", ""]], "choices": ["block", "log"], "type": "str"},
                    "comment": {"v_range": [["6.2.0", ""]], "type": "str"},
                    "direction": {"v_range": [["6.2.0", ""]], "choices": ["incoming", "outgoing", "any"], "type": "str"},
                    "file-type": {"v_range": [["6.2.0", ""]], "type": "raw"},
                    "filter": {"v_range": [["6.2.0", ""]], "type": "str"},
                    "protocol": {"v_range": [["6.2.0", ""]], "type": "list", "choices": ["cifs"], "elements": "str"}
                },
                "elements": "dict"
            },
            "log": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "status": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "cifs_profile_filefilter_entries": {
        "stated": true,
        "options": {
            "action": {"v_range": [["6.2.0", ""]], "choices": ["block", "log"], "type": "str"},
            "comment": {"v_range": [["6.2.0", ""]], "type": "str"},
            "direction": {"v_range": [["6.2.0", ""]], "choices": ["incoming", "outgoing", "any"], "type": "str"},
            "file-type": {"v_range": [["6.2.0", ""]], "type": "raw"},
            "filter": {"v_range": [["6.2.0", ""]], "type": "str"},
            "protocol": {"v_range": [["6.2.0", ""]], "type": "list", "choices": ["cifs"], "elements": "str"}
        }
    },
    "cifs_profile_serverkeytab": {
        "stated": true,
        "options": {
            "keytab": {"v_range": [["6.2.0", ""]], "no_log": true, "type": "str"},
            "principal": {"v_range": [["6.2.0", ""]], "type": "str"},
            "password": {"v_range": [["6.2.2", ""]], "no_log": true, "type": "raw"}
        }
    },
    "cloud_orchestaws": {
        "stated": true,
        "options": {
            "connector": {"v_range": [["7.4.0", ""]], "type": "str"},
            "name": {"v_range": [["7.4.0", ""]], "required": true, "type": "str"},
            "region-name": {
                "v_range": [["7.4.0", ""]],
                "choices": [
                    "us-east-1", "us-east-2", "us-west-1", "us-west-2", "eu-west-1", "eu-west-2", "eu-west-3", "eu-north-1", "eu-south-1", "eu-south-2",
                    "eu-central-1", "eu-central-2", "ca-central-1", "ap-southeast-1", "ap-southeast-2", "ap-southeast-3", "ap-southeast-4", "ap-south-1",
                    "ap-south-2", "ap-northeast-1", "ap-northeast-2", "ap-northeast-3", "af-south-1", "me-central-1", "me-south-1", "sa-east-1",
                    "ap-east-1", "us-gov-east-1", "us-gov-west-1"
                ],
                "type": "str"
            },
            "template-configuration": {"v_range": [["7.4.0", ""]], "type": "str"}
        }
    },
    "cloud_orchestawsconnector": {
        "stated": true,
        "options": {
            "access-key-id": {"v_range": [["7.4.0", ""]], "no_log": true, "type": "str"},
            "access-key-secret": {"v_range": [["7.4.0", ""]], "no_log": true, "type": "raw"},
            "name": {"v_range": [["7.4.0", ""]], "required": true, "type": "str"},
            "use-metadata-iam": {"v_range": [["7.4.0", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "cloud_orchestawstemplate_autoscaleexistingvpc": {
        "stated": true,
        "options": {
            "custom-asset-container": {"v_range": [["7.4.0", ""]], "type": "str"},
            "custom-asset-directory": {"v_range": [["7.4.0", ""]], "type": "str"},
            "custom-identifier": {"v_range": [["7.4.0", ""]], "type": "str"},
            "faz-autoscale-admin-password": {"v_range": [["7.4.0", ""]], "no_log": true, "type": "raw"},
            "faz-autoscale-admin-username": {"v_range": [["7.4.0", ""]], "type": "str"},
            "faz-custom-private-ipaddress": {"v_range": [["7.4.0", ""]], "type": "str"},
            "faz-instance-type": {
                "v_range": [["7.4.0", ""]],
                "choices": [
                    "h1.2xlarge", "h1.4xlarge", "h1.8xlarge", "m5.large", "m5.xlarge", "m5.2xlarge", "m5.4xlarge", "m5.12xlarge", "t2.medium",
                    "t2.large", "t2.xlarge"
                ],
                "type": "str"
            },
            "faz-integration-options": {"v_range": [["7.4.0", ""]], "choices": ["no", "yes"], "type": "str"},
            "faz-version": {"v_range": [["7.4.0", ""]], "type": "str"},
            "fgt-admin-cidr": {"v_range": [["7.4.0", ""]], "type": "str"},
            "fgt-admin-port": {"v_range": [["7.4.0", ""]], "type": "int"},
            "fgt-instance-type": {
                "v_range": [["7.4.0", ""]],
                "choices": ["t2.small", "c5.large", "c5.xlarge", "c5.2xlarge", "c5.4xlarge", "c5.9xlarge"],
                "type": "str"
            },
            "fgt-psk-secret": {"v_range": [["7.4.0", ""]], "no_log": true, "type": "str"},
            "fgtasg-cool-down": {"v_range": [["7.4.0", ""]], "type": "int"},
            "fgtasg-desired-capacity-byol": {"v_range": [["7.4.0", ""]], "type": "int"},
            "fgtasg-desired-capacity-payg": {"v_range": [["7.4.0", ""]], "type": "int"},
            "fgtasg-health-check-grace-period": {"v_range": [["7.4.0", ""]], "type": "int"},
            "fgtasg-max-size-byol": {"v_range": [["7.4.0", ""]], "type": "int"},
            "fgtasg-max-size-payg": {"v_range": [["7.4.0", ""]], "type": "int"},
            "fgtasg-min-size-byol": {"v_range": [["7.4.0", ""]], "type": "int"},
            "fgtasg-min-size-payg": {"v_range": [["7.4.0", ""]], "type": "int"},
            "fgtasg-scale-in-threshold": {"v_range": [["7.4.0", ""]], "type": "int"},
            "fgtasg-scale-out-threshold": {"v_range": [["7.4.0", ""]], "type": "int"},
            "fos-version": {"v_range": [["7.4.0", ""]], "type": "str"},
            "get-license-grace-period": {"v_range": [["7.4.0", ""]], "type": "int"},
            "heartbeat-delay-allowance": {"v_range": [["7.4.0", ""]], "type": "int"},
            "heartbeat-interval": {"v_range": [["7.4.0", ""]], "type": "int"},
            "heartbeat-loss-count": {"v_range": [["7.4.0", ""]], "type": "int"},
            "internal-balancer-dns-name": {"v_range": [["7.4.0", ""]], "type": "str"},
            "internal-balancing-options": {
                "v_range": [["7.4.0", ""]],
                "choices": ["add a new internal load balancer", "use a load balancer specified below", "do not need one"],
                "type": "str"
            },
            "internal-target-group-health-check-path": {"v_range": [["7.4.0", ""]], "type": "str"},
            "key-pair-name": {"v_range": [["7.4.0", ""]], "no_log": true, "type": "str"},
            "lifecycle-hook-timeout": {"v_range": [["7.4.0", ""]], "type": "int"},
            "loadbalancing-health-check-threshold": {"v_range": [["7.4.0", ""]], "type": "int"},
            "loadbalancing-traffic-port": {"v_range": [["7.4.0", ""]], "type": "int"},
            "loadbalancing-traffic-protocol": {"v_range": [["7.4.0", ""]], "choices": ["HTTPS", "HTTP", "TCP"], "type": "str"},
            "name": {"v_range": [["7.4.0", ""]], "required": true, "type": "str"},
            "notification-email": {"v_range": [["7.4.0", ""]], "type": "str"},
            "primary-election-timeout": {"v_range": [["7.4.0", ""]], "type": "int"},
            "private-subnet-route-table": {"v_range": [["7.4.0", ""]], "type": "str"},
            "private-subnet1": {"v_range": [["7.4.0", ""]], "type": "str"},
            "private-subnet2": {"v_range": [["7.4.0", ""]], "type": "str"},
            "public-subnet1": {"v_range": [["7.4.0", ""]], "type": "str"},
            "public-subnet2": {"v_range": [["7.4.0", ""]], "type": "str"},
            "resource-tag-prefix": {"v_range": [["7.4.0", ""]], "type": "str"},
            "s3-bucket-name": {"v_range": [["7.4.0", ""]], "type": "str"},
            "s3-key-prefix": {"v_range": [["7.4.0", ""]], "no_log": true, "type": "str"},
            "sync-recovery-count": {"v_range": [["7.4.0", ""]], "type": "int"},
            "terminate-unhealthy-vm": {"v_range": [["7.4.0", ""]], "choices": ["no", "yes"], "type": "str"},
            "use-custom-asset-location": {"v_range": [["7.4.0", ""]], "choices": ["no", "yes"], "type": "str"},
            "vpc-cidr": {"v_range": [["7.4.0", ""]], "type": "str"},
            "vpc-endpoint-id": {"v_range": [["7.4.0", ""]], "type": "str"},
            "vpc-id": {"v_range": [["7.4.0", ""]], "type": "str"}
        }
    },
    "cloud_orchestawstemplate_autoscalenewvpc": {
        "stated": true,
        "options": {
            "availability-zones": {"v_range": [["7.4.0", ""]], "type": "str"},
            "custom-asset-container": {"v_range": [["7.4.0", ""]], "type": "str"},
            "custom-asset-directory": {"v_range": [["7.4.0", ""]], "type": "str"},
            "custom-identifier": {"v_range": [["7.4.0", ""]], "type": "str"},
            "faz-autoscale-admin-password": {"v_range": [["7.4.0", ""]], "no_log": true, "type": "raw"},
            "faz-autoscale-admin-username": {"v_range": [["7.4.0", ""]], "type": "str"},
            "faz-custom-private-ipaddress": {"v_range": [["7.4.0", ""]], "type": "str"},
            "faz-instance-type": {
                "v_range": [["7.4.0", ""]],
                "choices": [
                    "h1.2xlarge", "h1.4xlarge", "h1.8xlarge", "m5.large", "m5.xlarge", "m5.2xlarge", "m5.4xlarge", "m5.12xlarge", "t2.medium",
                    "t2.large", "t2.xlarge"
                ],
                "type": "str"
            },
            "faz-integration-options": {"v_range": [["7.4.0", ""]], "choices": ["no", "yes"], "type": "str"},
            "faz-version": {"v_range": [["7.4.0", ""]], "type": "str"},
            "fgt-admin-cidr": {"v_range": [["7.4.0", ""]], "type": "str"},
            "fgt-admin-port": {"v_range": [["7.4.0", ""]], "type": "int"},
            "fgt-instance-type": {
                "v_range": [["7.4.0", ""]],
                "choices": ["t2.small", "c5.large", "c5.xlarge", "c5.2xlarge", "c5.4xlarge", "c5.9xlarge"],
                "type": "str"
            },
            "fgt-psk-secret": {"v_range": [["7.4.0", ""]], "no_log": true, "type": "str"},
            "fgtasg-cool-down": {"v_range": [["7.4.0", ""]], "type": "int"},
            "fgtasg-desired-capacity-byol": {"v_range": [["7.4.0", ""]], "type": "int"},
            "fgtasg-desired-capacity-payg": {"v_range": [["7.4.0", ""]], "type": "int"},
            "fgtasg-health-check-grace-period": {"v_range": [["7.4.0", ""]], "type": "int"},
            "fgtasg-max-size-byol": {"v_range": [["7.4.0", ""]], "type": "int"},
            "fgtasg-max-size-payg": {"v_range": [["7.4.0", ""]], "type": "int"},
            "fgtasg-min-size-byol": {"v_range": [["7.4.0", ""]], "type": "int"},
            "fgtasg-min-size-payg": {"v_range": [["7.4.0", ""]], "type": "int"},
            "fgtasg-scale-in-threshold": {"v_range": [["7.4.0", ""]], "type": "int"},
            "fgtasg-scale-out-threshold": {"v_range": [["7.4.0", ""]], "type": "int"},
            "fos-version": {"v_range": [["7.4.0", ""]], "type": "str"},
            "get-license-grace-period": {"v_range": [["7.4.0", ""]], "type": "int"},
            "heartbeat-delay-allowance": {"v_range": [["7.4.0", ""]], "type": "int"},
            "heartbeat-interval": {"v_range": [["7.4.0", ""]], "type": "int"},
            "heartbeat-loss-count": {"v_range": [["7.4.0", ""]], "type": "int"},
            "internal-balancer-dns-name": {"v_range": [["7.4.0", ""]], "type": "str"},
            "internal-balancing-options": {
                "v_range": [["7.4.0", ""]],
                "choices": ["add a new internal load balancer", "use a load balancer specified below", "do not need one"],
                "type": "str"
            },
            "internal-target-group-health-check-path": {"v_range": [["7.4.0", ""]], "type": "str"},
            "key-pair-name": {"v_range": [["7.4.0", ""]], "no_log": true, "type": "str"},
            "lifecycle-hook-timeout": {"v_range": [["7.4.0", ""]], "type": "int"},
            "loadbalancing-health-check-threshold": {"v_range": [["7.4.0", ""]], "type": "int"},
            "loadbalancing-traffic-port": {"v_range": [["7.4.0", ""]], "type": "int"},
            "loadbalancing-traffic-protocol": {"v_range": [["7.4.0", ""]], "choices": ["HTTPS", "HTTP", "TCP"], "type": "str"},
            "name": {"v_range": [["7.4.0", ""]], "required": true, "type": "str"},
            "notification-email": {"v_range": [["7.4.0", ""]], "type": "str"},
            "primary-election-timeout": {"v_range": [["7.4.0", ""]], "type": "int"},
            "private-subnet1-cidr": {"v_range": [["7.4.0", ""]], "type": "str"},
            "private-subnet2-cidr": {"v_range": [["7.4.0", ""]], "type": "str"},
            "public-subnet1-cidr": {"v_range": [["7.4.0", ""]], "type": "str"},
            "public-subnet2-cidr": {"v_range": [["7.4.0", ""]], "type": "str"},
            "resource-tag-prefix": {"v_range": [["7.4.0", ""]], "type": "str"},
            "s3-bucket-name": {"v_range": [["7.4.0", ""]], "type": "str"},
            "s3-key-prefix": {"v_range": [["7.4.0", ""]], "no_log": true, "type": "str"},
            "sync-recovery-count": {"v_range": [["7.4.0", ""]], "type": "int"},
            "terminate-unhealthy-vm": {"v_range": [["7.4.0", ""]], "choices": ["no", "yes"], "type": "str"},
            "use-custom-asset-location": {"v_range": [["7.4.0", ""]], "choices": ["no", "yes"], "type": "str"},
            "vpc-cidr": {"v_range": [["7.4.0", ""]], "type": "str"}
        }
    },
    "cloud_orchestawstemplate_autoscaletgwnewvpc": {
        "stated": true,
        "options": {
            "availability-zones": {"v_range": [["7.4.0", ""]], "type": "str"},
            "bgp-asn": {"v_range": [["7.4.0", ""]], "type": "int"},
            "custom-asset-container": {"v_range": [["7.4.0", ""]], "type": "str"},
            "custom-asset-directory": {"v_range": [["7.4.0", ""]], "type": "str"},
            "custom-identifier": {"v_range": [["7.4.0", ""]], "type": "str"},
            "faz-autoscale-admin-password": {"v_range": [["7.4.0", ""]], "no_log": true, "type": "raw"},
            "faz-autoscale-admin-username": {"v_range": [["7.4.0", ""]], "type": "str"},
            "faz-custom-private-ipaddress": {"v_range": [["7.4.0", ""]], "type": "str"},
            "faz-instance-type": {
                "v_range": [["7.4.0", ""]],
                "choices": [
                    "h1.2xlarge", "h1.4xlarge", "h1.8xlarge", "m5.large", "m5.xlarge", "m5.2xlarge", "m5.4xlarge", "m5.12xlarge", "t2.medium",
                    "t2.large", "t2.xlarge"
                ],
                "type": "str"
            },
            "faz-integration-options": {"v_range": [["7.4.0", ""]], "choices": ["no", "yes"], "type": "str"},
            "faz-version": {"v_range": [["7.4.0", ""]], "type": "str"},
            "fgt-admin-cidr": {"v_range": [["7.4.0", ""]], "type": "str"},
            "fgt-admin-port": {"v_range": [["7.4.0", ""]], "type": "int"},
            "fgt-instance-type": {
                "v_range": [["7.4.0", ""]],
                "choices": ["t2.small", "c5.large", "c5.xlarge", "c5.2xlarge", "c5.4xlarge", "c5.9xlarge"],
                "type": "str"
            },
            "fgt-psk-secret": {"v_range": [["7.4.0", ""]], "no_log": true, "type": "str"},
            "fgtasg-cool-down": {"v_range": [["7.4.0", ""]], "type": "int"},
            "fgtasg-desired-capacity-byol": {"v_range": [["7.4.0", ""]], "type": "int"},
            "fgtasg-desired-capacity-payg": {"v_range": [["7.4.0", ""]], "type": "int"},
            "fgtasg-health-check-grace-period": {"v_range": [["7.4.0", ""]], "type": "int"},
            "fgtasg-max-size-byol": {"v_range": [["7.4.0", ""]], "type": "int"},
            "fgtasg-max-size-payg": {"v_range": [["7.4.0", ""]], "type": "int"},
            "fgtasg-min-size-byol": {"v_range": [["7.4.0", ""]], "type": "int"},
            "fgtasg-min-size-payg": {"v_range": [["7.4.0", ""]], "type": "int"},
            "fgtasg-scale-in-threshold": {"v_range": [["7.4.0", ""]], "type": "int"},
            "fgtasg-scale-out-threshold": {"v_range": [["7.4.0", ""]], "type": "int"},
            "fos-version": {"v_range": [["7.4.0", ""]], "type": "str"},
            "get-license-grace-period": {"v_range": [["7.4.0", ""]], "type": "int"},
            "heartbeat-delay-allowance": {"v_range": [["7.4.0", ""]], "type": "int"},
            "heartbeat-interval": {"v_range": [["7.4.0", ""]], "type": "int"},
            "heartbeat-loss-count": {"v_range": [["7.4.0", ""]], "type": "int"},
            "key-pair-name": {"v_range": [["7.4.0", ""]], "no_log": true, "type": "str"},
            "lifecycle-hook-timeout": {"v_range": [["7.4.0", ""]], "type": "int"},
            "name": {"v_range": [["7.4.0", ""]], "required": true, "type": "str"},
            "notification-email": {"v_range": [["7.4.0", ""]], "type": "str"},
            "primary-election-timeout": {"v_range": [["7.4.0", ""]], "type": "int"},
            "public-subnet1-cidr": {"v_range": [["7.4.0", ""]], "type": "str"},
            "public-subnet2-cidr": {"v_range": [["7.4.0", ""]], "type": "str"},
            "resource-tag-prefix": {"v_range": [["7.4.0", ""]], "type": "str"},
            "s3-bucket-name": {"v_range": [["7.4.0", ""]], "type": "str"},
            "s3-key-prefix": {"v_range": [["7.4.0", ""]], "no_log": true, "type": "str"},
            "sync-recovery-count": {"v_range": [["7.4.0", ""]], "type": "int"},
            "terminate-unhealthy-vm": {"v_range": [["7.4.0", ""]], "choices": ["no", "yes"], "type": "str"},
            "transit-gateway-id": {"v_range": [["7.4.0", ""]], "type": "str"},
            "transit-gateway-support-options": {"v_range": [["7.4.0", ""]], "choices": ["create one", "use an existing one"], "type": "str"},
            "use-custom-asset-location": {"v_range": [["7.4.0", ""]], "choices": ["no", "yes"], "type": "str"},
            "vpc-cidr": {"v_range": [["7.4.0", ""]], "type": "str"}
        }
    },
    "cloud_orchestration": {
        "stated": true,
        "options": {
            "configuration": {"v_range": [["7.4.0", ""]], "type": "str"},
            "description": {"v_range": [["7.4.0", ""]], "type": "str"},
            "name": {"v_range": [["7.4.0", ""]], "required": true, "type": "str"},
            "type": {"v_range": [["7.4.0", ""]], "choices": ["aws"], "type": "str"}
        }
    },
    "credentialstore_domaincontroller": {
        "stated": true,
        "options": {
            "domain-name": {"v_range": [["6.4.0", ""]], "type": "str"},
            "ip": {"v_range": [["6.4.0", ""]], "type": "str"},
            "ip6": {"v_range": [["6.4.0", ""]], "type": "str"},
            "password": {"v_range": [["6.4.0", ""]], "no_log": true, "type": "raw"},
            "port": {"v_range": [["6.4.0", ""]], "type": "int"},
            "server-name": {"v_range": [["6.4.0", ""]], "type": "str"},
            "username": {"v_range": [["6.4.0", ""]], "type": "str"},
            "hostname": {"v_range": [["6.4.2", ""]], "type": "str"}
        }
    },
    "devprof_device_profile_fortianalyzer": {
        "stated": false,
        "options": {
            "managed-sn": {"type": "str"},
            "target": {"choices": ["none", "this-fmg", "managed", "others"], "type": "str"},
            "target-ip": {"type": "str"},
            "target-sn": {"type": "raw"}
        }
    },
    "devprof_device_profile_fortiguard": {
        "stated": false,
        "options": {
            "target": {"choices": ["none", "direct", "this-fmg"], "type": "str"},
            "target-ip": {"type": "str"},
            "auto-firmware-upgrade": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "auto-firmware-upgrade-day": {
                "v_range": [["7.4.1", ""]],
                "type": "list",
                "choices": ["sunday", "monday", "tuesday", "wednesday", "thursday", "friday", "saturday"],
                "elements": "str"
            },
            "auto-firmware-upgrade-delay": {"v_range": [["7.4.1", ""]], "type": "int"},
            "auto-firmware-upgrade-end-hour": {"v_range": [["7.4.1", ""]], "type": "int"},
            "auto-firmware-upgrade-start-hour": {"v_range": [["7.4.1", ""]], "type": "int"}
        }
    },
    "devprof_import": {
        "stated": true,
        "options": {
            "device": {"v_range": [["6.2.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "str"},
            "devprof": {"v_range": [["6.2.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "str"}
        }
    },
    "devprof_log_fortianalyzer_setting": {
        "stated": false,
        "options": {
            "certificate": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "str"},
            "conn-timeout": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "int"},
            "enc-algorithm": {
                "v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]],
                "choices": ["default", "high", "low", "disable", "high-medium", "low-medium"],
                "type": "str"
            },
            "hmac-algorithm": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "choices": ["sha256", "sha1"], "type": "str"},
            "ips-archive": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "monitor-failure-retry-period": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "int"},
            "monitor-keepalive-period": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "int"},
            "reliable": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ssl-min-proto-version": {
                "v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]],
                "choices": ["default", "TLSv1", "TLSv1-1", "TLSv1-2", "SSLv3", "TLSv1-3"],
                "type": "str"
            },
            "upload-day": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "str"},
            "upload-interval": {
                "v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]],
                "choices": ["daily", "weekly", "monthly"],
                "type": "str"
            },
            "upload-option": {
                "v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]],
                "choices": ["store-and-upload", "realtime", "1-minute", "5-minute"],
                "type": "str"
            },
            "upload-time": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "str"},
            "access-config": {"v_range": [["6.2.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "certificate-verification": {
                "v_range": [["6.2.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]],
                "choices": ["disable", "enable"],
                "type": "str"
            },
            "max-log-rate": {"v_range": [["6.2.2", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "int"},
            "priority": {"v_range": [["6.2.2", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "choices": ["low", "default"], "type": "str"},
            "interface": {"v_range": [["6.2.7", "6.2.12"], ["6.4.3", ""]], "type": "str"},
            "interface-select-method": {"v_range": [["6.2.7", "6.2.12"], ["6.4.3", ""]], "choices": ["auto", "sdwan", "specify"], "type": "str"},
            "preshared-key": {"v_range": [["7.0.0", ""]], "no_log": true, "type": "str"},
            "alt-server": {"v_range": [["7.2.2", ""]], "type": "str"},
            "fallback-to-primary": {"v_range": [["7.2.5", "7.2.5"], ["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "server-cert-ca": {"v_range": [["7.2.5", "7.2.5"], ["7.4.2", ""]], "type": "str"},
            "serial": {"v_range": [["7.4.3", ""]], "type": "raw"},
            "source-ip": {"v_range": [["7.4.3", ""]], "type": "str"},
            "status": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "__change_ip": {"v_range": [["7.4.3", ""]], "type": "int"},
            "server": {"v_range": [["7.4.3", ""]], "type": "str"}
        }
    },
    "devprof_log_fortianalyzercloud_setting": {
        "stated": false,
        "options": {
            "__change_ip": {"v_range": [["6.2.1", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "int"},
            "access-config": {"v_range": [["6.2.1", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "certificate": {"v_range": [["6.2.1", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "str"},
            "conn-timeout": {"v_range": [["6.2.1", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "int"},
            "enc-algorithm": {
                "v_range": [["6.2.1", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]],
                "choices": ["high", "low", "high-medium", "low-medium"],
                "type": "str"
            },
            "hmac-algorithm": {"v_range": [["6.2.1", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "choices": ["sha256", "sha1"], "type": "str"},
            "ips-archive": {"v_range": [["6.2.1", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "monitor-failure-retry-period": {"v_range": [["6.2.1", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "int"},
            "monitor-keepalive-period": {"v_range": [["6.2.1", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "int"},
            "source-ip": {"v_range": [["6.2.1", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "str"},
            "ssl-min-proto-version": {
                "v_range": [["6.2.1", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]],
                "choices": ["default", "TLSv1-1", "TLSv1-2", "SSLv3", "TLSv1", "TLSv1-3"],
                "type": "str"
            },
            "status": {"v_range": [["6.2.1", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "upload-day": {"v_range": [["6.2.1", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "str"},
            "upload-interval": {
                "v_range": [["6.2.1", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]],
                "choices": ["daily", "weekly", "monthly"],
                "type": "str"
            },
            "upload-option": {
                "v_range": [["6.2.1", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]],
                "choices": ["store-and-upload", "realtime", "1-minute", "5-minute"],
                "type": "str"
            },
            "upload-time": {"v_range": [["6.2.1", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "str"},
            "max-log-rate": {"v_range": [["6.2.2", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "int"},
            "priority": {"v_range": [["6.2.2", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "choices": ["low", "default"], "type": "str"},
            "interface": {"v_range": [["6.2.7", "6.2.12"], ["6.4.3", ""]], "type": "str"},
            "interface-select-method": {"v_range": [["6.2.7", "6.2.12"], ["6.4.3", ""]], "choices": ["auto", "sdwan", "specify"], "type": "str"},
            "preshared-key": {"v_range": [["7.0.0", ""]], "no_log": true, "type": "str"},
            "certificate-verification": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "serial": {"v_range": [["7.0.3", ""]], "type": "raw"}
        }
    },
    "devprof_log_syslogd_filter": {
        "stated": false,
        "options": {
            "severity": {
                "v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]],
                "choices": ["emergency", "alert", "critical", "error", "warning", "notification", "information", "debug"],
                "type": "str"
            },
            "anomaly": {"v_range": [["7.0.4", "7.0.12"], ["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "exclude-list": {
                "v_range": [["7.0.4", "7.0.12"]],
                "type": "list",
                "options": {
                    "category": {
                        "v_range": [["7.0.4", "7.0.12"]],
                        "choices": ["app-ctrl", "attack", "dlp", "event", "traffic", "virus", "voip", "webfilter", "netscan", "spam", "anomaly", "waf"],
                        "type": "str"
                    },
                    "fields": {
                        "v_range": [["7.0.4", "7.0.12"]],
                        "type": "list",
                        "options": {
                            "args": {"v_range": [["7.0.4", "7.0.12"]], "type": "raw"},
                            "field": {"v_range": [["7.0.4", "7.0.12"]], "type": "str"},
                            "negate": {"v_range": [["7.0.4", "7.0.12"]], "choices": ["disable", "enable"], "type": "str"}
                        },
                        "elements": "dict"
                    },
                    "id": {"v_range": [["7.0.4", "7.0.12"]], "type": "int"}
                },
                "elements": "dict"
            },
            "forward-traffic": {"v_range": [["7.0.4", "7.0.12"], ["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "free-style": {
                "v_range": [["7.0.4", "7.0.12"], ["7.2.1", ""]],
                "type": "list",
                "options": {
                    "category": {
                        "v_range": [["7.0.4", "7.0.12"], ["7.2.1", ""]],
                        "choices": [
                            "traffic", "event", "virus", "webfilter", "attack", "spam", "voip", "dlp", "app-ctrl", "anomaly", "waf", "gtp", "dns", "ssh",
                            "ssl", "file-filter", "icap", "ztna", "virtual-patch"
                        ],
                        "type": "str"
                    },
                    "filter": {"v_range": [["7.0.4", "7.0.12"], ["7.2.1", ""]], "type": "str"},
                    "filter-type": {"v_range": [["7.0.4", "7.0.12"], ["7.2.1", ""]], "choices": ["include", "exclude"], "type": "str"},
                    "id": {"v_range": [["7.0.4", "7.0.12"], ["7.2.1", ""]], "type": "int"}
                },
                "elements": "dict"
            },
            "gtp": {"v_range": [["7.0.4", "7.0.12"], ["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "local-traffic": {"v_range": [["7.0.4", "7.0.12"], ["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "multicast-traffic": {"v_range": [["7.0.4", "7.0.12"], ["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "sniffer-traffic": {"v_range": [["7.0.4", "7.0.12"], ["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "voip": {"v_range": [["7.0.4", "7.0.12"], ["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ztna-traffic": {"v_range": [["7.0.4", "7.0.12"], ["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "filter-type": {"v_range": [["7.0.4", "7.0.12"], ["7.2.1", ""]], "choices": ["include", "exclude"], "type": "str"},
            "filter": {"v_range": [["7.0.4", "7.0.12"], ["7.2.1", ""]], "type": "str"},
            "cifs": {"v_range": [["7.0.4", "7.0.12"]], "choices": ["disable", "enable"], "type": "str"},
            "ssl": {"v_range": [["7.0.4", "7.0.12"]], "choices": ["disable", "enable"], "type": "str"},
            "dns": {"v_range": [["7.0.4", "7.0.12"], ["7.2.1", "7.2.1"]], "choices": ["disable", "enable"], "type": "str"},
            "ssh": {"v_range": [["7.0.4", "7.0.12"], ["7.2.1", "7.2.1"]], "choices": ["disable", "enable"], "type": "str"},
            "netscan-discovery": {"v_range": [["7.0.4", "7.0.12"]], "choices": ["disable", "enable"], "type": "str"},
            "netscan-vulnerability": {"v_range": [["7.0.4", "7.0.12"]], "choices": ["disable", "enable"], "type": "str"},
            "forti-switch": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "http-transaction": {"v_range": [["7.6.0", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "devprof_log_syslogd_filter_excludelist": {
        "stated": true,
        "options": {
            "category": {
                "v_range": [["7.0.4", "7.0.12"]],
                "choices": ["app-ctrl", "attack", "dlp", "event", "traffic", "virus", "voip", "webfilter", "netscan", "spam", "anomaly", "waf"],
                "type": "str"
            },
            "fields": {
                "v_range": [["7.0.4", "7.0.12"]],
                "type": "list",
                "options": {
                    "args": {"v_range": [["7.0.4", "7.0.12"]], "type": "raw"},
                    "field": {"v_range": [["7.0.4", "7.0.12"]], "type": "str"},
                    "negate": {"v_range": [["7.0.4", "7.0.12"]], "choices": ["disable", "enable"], "type": "str"}
                },
                "elements": "dict"
            },
            "id": {"v_range": [["7.0.4", "7.0.12"]], "required": true, "type": "int"}
        }
    },
    "devprof_log_syslogd_filter_excludelist_fields": {
        "stated": true,
        "options": {
            "args": {"v_range": [["7.0.4", "7.0.12"]], "type": "raw"},
            "field": {"v_range": [["7.0.4", "7.0.12"]], "type": "str"},
            "negate": {"v_range": [["7.0.4", "7.0.12"]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "devprof_log_syslogd_filter_freestyle": {
        "stated": true,
        "options": {
            "category": {
                "v_range": [["7.0.4", "7.0.12"], ["7.2.1", ""]],
                "choices": [
                    "traffic", "event", "virus", "webfilter", "attack", "spam", "voip", "dlp", "app-ctrl", "anomaly", "waf", "gtp", "dns", "ssh", "ssl",
                    "file-filter", "icap", "ztna", "virtual-patch"
                ],
                "type": "str"
            },
            "filter": {"v_range": [["7.0.4", "7.0.12"], ["7.2.1", ""]], "type": "str"},
            "filter-type": {"v_range": [["7.0.4", "7.0.12"], ["7.2.1", ""]], "choices": ["include", "exclude"], "type": "str"},
            "id": {"v_range": [["7.0.4", "7.0.12"], ["7.2.1", ""]], "required": true, "type": "int"}
        }
    },
    "devprof_log_syslogd_setting": {
        "stated": false,
        "options": {
            "certificate": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "str"},
            "enc-algorithm": {
                "v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]],
                "choices": ["high", "low", "disable", "high-medium"],
                "type": "str"
            },
            "facility": {
                "v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]],
                "choices": [
                    "kernel", "user", "mail", "daemon", "auth", "syslog", "lpr", "news", "uucp", "cron", "authpriv", "ftp", "ntp", "audit", "alert",
                    "clock", "local0", "local1", "local2", "local3", "local4", "local5", "local6", "local7"
                ],
                "type": "str"
            },
            "mode": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "choices": ["udp", "legacy-reliable", "reliable"], "type": "str"},
            "port": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "int"},
            "server": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "str"},
            "ssl-min-proto-version": {
                "v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]],
                "choices": ["default", "TLSv1-1", "TLSv1-2", "SSLv3", "TLSv1", "TLSv1-3"],
                "type": "str"
            },
            "status": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "reliable": {"v_range": [["6.2.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", "6.4.14"]], "choices": ["disable", "enable"], "type": "str"},
            "csv": {"v_range": [["6.2.0", "6.2.5"], ["6.2.7", "6.2.12"]], "choices": ["disable", "enable"], "type": "str"},
            "max-log-rate": {"v_range": [["6.2.2", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "int"},
            "priority": {"v_range": [["6.2.2", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "choices": ["low", "default"], "type": "str"},
            "interface": {"v_range": [["6.2.7", "6.2.12"], ["6.4.3", ""]], "type": "str"},
            "interface-select-method": {"v_range": [["6.2.7", "6.2.12"], ["6.4.3", ""]], "choices": ["auto", "sdwan", "specify"], "type": "str"},
            "format": {"v_range": [["6.4.6", "6.4.14"], ["7.0.1", ""]], "choices": ["default", "csv", "cef", "rfc5424", "json"], "type": "str"},
            "syslog-type": {"v_range": [["6.2.0", "6.2.0"]], "type": "int"},
            "custom-field-name": {
                "v_range": [["7.0.4", "7.0.12"], ["7.2.1", ""]],
                "type": "list",
                "options": {
                    "custom": {"v_range": [["7.0.4", "7.0.12"], ["7.2.1", ""]], "type": "str"},
                    "id": {"v_range": [["7.0.4", "7.0.12"], ["7.2.1", ""]], "type": "int"},
                    "name": {"v_range": [["7.0.4", "7.0.12"], ["7.2.1", ""]], "type": "str"}
                },
                "elements": "dict"
            },
            "source-ip": {"v_range": [["7.4.3", ""]], "type": "str"},
            "source-ip-interface": {"v_range": [["7.6.0", ""]], "type": "raw"}
        }
    },
    "devprof_log_syslogd_setting_customfieldname": {
        "stated": true,
        "options": {
            "custom": {"v_range": [["7.0.4", "7.0.12"], ["7.2.1", ""]], "type": "str"},
            "id": {"v_range": [["7.0.4", "7.0.12"], ["7.2.1", ""]], "required": true, "type": "int"},
            "name": {"v_range": [["7.0.4", "7.0.12"], ["7.2.1", ""]], "type": "str"}
        }
    },
    "devprof_system_centralmanagement": {
        "stated": false,
        "options": {
            "include-default-servers": {
                "v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]],
                "choices": ["disable", "enable"],
                "type": "str"
            },
            "server-list": {
                "v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]],
                "type": "list",
                "options": {
                    "addr-type": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "choices": ["fqdn", "ipv4", "ipv6"], "type": "str"},
                    "fqdn": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "str"},
                    "id": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "int"},
                    "server-address": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "str"},
                    "server-address6": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "str"},
                    "server-type": {
                        "v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]],
                        "type": "list",
                        "choices": ["update", "rating", "iot-query", "iot-collect", "vpatch-query"],
                        "elements": "str"
                    }
                },
                "elements": "dict"
            },
            "ltefw-upgrade-time": {"v_range": [["7.4.3", ""]], "type": "str"},
            "vdom": {"v_range": [["7.4.3", ""]], "type": "raw"},
            "allow-remote-firmware-upgrade": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "local-cert": {"v_range": [["7.4.3", ""]], "type": "str"},
            "allow-push-firmware": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ltefw-upgrade-frequency": {"v_range": [["7.4.3", ""]], "choices": ["everyHour", "every12hour", "everyDay", "everyWeek"], "type": "str"},
            "mode": {"v_range": [["7.4.3", ""]], "choices": ["normal", "backup"], "type": "str"},
            "serial-number": {"v_range": [["7.4.3", ""]], "type": "raw"},
            "fmg-source-ip6": {"v_range": [["7.4.3", ""]], "type": "str"},
            "allow-monitor": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "allow-push-configuration": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ca-cert": {"v_range": [["7.4.3", ""]], "type": "str"},
            "fmg-update-port": {"v_range": [["7.4.3", ""]], "choices": ["443", "8890"], "type": "str"},
            "use-elbc-vdom": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "allow-remote-lte-firmware-upgrade": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "interface": {"v_range": [["7.4.3", ""]], "type": "raw"},
            "schedule-script-restore": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "schedule-config-restore": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "interface-select-method": {"v_range": [["7.4.3", ""]], "choices": ["auto", "sdwan", "specify"], "type": "str"},
            "type": {"v_range": [["7.4.3", ""]], "choices": ["fortimanager", "fortiguard", "none"], "type": "str"},
            "fmg-source-ip": {"v_range": [["7.4.3", ""]], "type": "str"},
            "fortigate-cloud-sso-default-profile": {"v_range": [["7.4.3", ""]], "type": "raw"},
            "fmg": {"v_range": [["7.4.3", ""]], "type": "raw"},
            "enc-algorithm": {"v_range": [["7.4.3", ""]], "choices": ["default", "high", "low"], "type": "str"},
            "allow-remote-modem-firmware-upgrade": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "modem-upgrade-frequency": {"v_range": [["7.4.3", ""]], "choices": ["everyHour", "every12hour", "everyDay", "everyWeek"], "type": "str"},
            "modem-upgrade-time": {"v_range": [["7.4.3", ""]], "type": "str"}
        }
    },
    "devprof_system_centralmanagement_serverlist": {
        "stated": true,
        "options": {
            "addr-type": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "choices": ["fqdn", "ipv4", "ipv6"], "type": "str"},
            "fqdn": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "str"},
            "id": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "required": true, "type": "int"},
            "server-address": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "str"},
            "server-address6": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "str"},
            "server-type": {
                "v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]],
                "type": "list",
                "choices": ["update", "rating", "iot-query", "iot-collect", "vpatch-query"],
                "elements": "str"
            }
        }
    },
    "devprof_system_dns": {
        "stated": false,
        "options": {
            "cache-notfound-responses": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"]], "choices": ["disable", "enable"], "type": "str"},
            "dns-cache-limit": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"]], "type": "int"},
            "dns-cache-ttl": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"]], "type": "int"},
            "domain": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"]], "type": "raw"},
            "ip6-primary": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"]], "type": "str"},
            "ip6-secondary": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"]], "type": "str"},
            "primary": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"]], "type": "str"},
            "secondary": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"]], "type": "str"},
            "dns-over-tls": {"v_range": [["6.2.0", "6.2.5"], ["6.2.7", "6.4.1"]], "choices": ["disable", "enable", "enforce"], "type": "str"},
            "retry": {"v_range": [["6.2.0", "6.2.5"], ["6.2.7", "6.4.1"]], "type": "int"},
            "server-hostname": {"v_range": [["6.2.1", "6.2.5"], ["6.2.7", "6.4.1"]], "type": "raw"},
            "ssl-certificate": {"v_range": [["6.2.0", "6.2.5"], ["6.2.7", "6.4.1"]], "type": "str"},
            "timeout": {"v_range": [["6.2.0", "6.2.5"], ["6.2.7", "6.4.1"]], "type": "int"},
            "interface": {"v_range": [["6.2.5", "6.2.5"], ["6.2.7", "6.2.12"], ["6.4.1", "6.4.1"]], "type": "str"},
            "interface-select-method": {
                "v_range": [["6.2.5", "6.2.5"], ["6.2.7", "6.2.12"], ["6.4.1", "6.4.1"]],
                "choices": ["auto", "sdwan", "specify"],
                "type": "str"
            }
        }
    },
    "devprof_system_emailserver": {
        "stated": false,
        "options": {
            "authenticate": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "password": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "no_log": true, "type": "raw"},
            "port": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "int"},
            "reply-to": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "str"},
            "security": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "choices": ["none", "starttls", "smtps"], "type": "str"},
            "server": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "str"},
            "source-ip": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "str"},
            "source-ip6": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "str"},
            "ssl-min-proto-version": {
                "v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]],
                "choices": ["default", "TLSv1", "TLSv1-1", "TLSv1-2", "SSLv3", "TLSv1-3"],
                "type": "str"
            },
            "type": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "choices": ["custom"], "type": "str"},
            "username": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "str"},
            "validate-server": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "interface": {"v_range": [["7.0.0", ""]], "type": "str"},
            "interface-select-method": {"v_range": [["7.0.0", ""]], "choices": ["auto", "sdwan", "specify"], "type": "str"}
        }
    },
    "devprof_system_global": {
        "stated": false,
        "options": {
            "admin-https-redirect": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "admin-port": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "int"},
            "admin-scp": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "admin-sport": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "int"},
            "admin-ssh-port": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "int"},
            "admin-ssh-v1": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "admin-telnet-port": {
                "v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", "7.2.4"], ["7.4.0", "7.4.1"], ["7.4.3", ""]],
                "type": "int"
            },
            "admintimeout": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "int"},
            "gui-ipv6": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "gui-lines-per-page": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "int"},
            "gui-theme": {
                "v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]],
                "choices": [
                    "blue", "green", "melongene", "red", "mariner", "neutrino", "jade", "graphite", "dark-matter", "onyx", "eclipse", "retro", "fpx",
                    "jet-stream", "security-fabric"
                ],
                "type": "str"
            },
            "language": {
                "v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]],
                "choices": ["english", "simch", "japanese", "korean", "spanish", "trach", "french", "portuguese"],
                "type": "str"
            },
            "switch-controller": {
                "v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", "7.2.0"], ["7.4.3", ""]],
                "choices": ["disable", "enable"],
                "type": "str"
            },
            "gui-device-latitude": {"v_range": [["7.4.3", ""]], "type": "str"},
            "gui-device-longitude": {"v_range": [["7.4.3", ""]], "type": "str"},
            "hostname": {"v_range": [["7.4.3", ""]], "type": "str"},
            "timezone": {"v_range": [["7.4.3", ""]], "type": "raw"},
            "check-reset-range": {"v_range": [["7.4.3", ""]], "choices": ["disable", "strict"], "type": "str"},
            "pmtu-discovery": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "gui-allow-incompatible-fabric-fgt": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "admin-restrict-local": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable", "all", "non-console-only"], "type": "str"},
            "gui-workflow-management": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "send-pmtu-icmp": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "tcp-halfclose-timer": {"v_range": [["7.4.3", ""]], "type": "int"},
            "admin-server-cert": {"v_range": [["7.4.3", ""]], "type": "raw"},
            "dnsproxy-worker-count": {"v_range": [["7.4.3", ""]], "type": "int"},
            "show-backplane-intf": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "gui-custom-language": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ldapconntimeout": {"v_range": [["7.4.3", ""]], "type": "int"},
            "auth-https-port": {"v_range": [["7.4.3", ""]], "type": "int"},
            "revision-backup-on-logout": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "arp-max-entry": {"v_range": [["7.4.3", ""]], "type": "int"},
            "long-vdom-name": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "pre-login-banner": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "qsfpdd-split8-port": {"v_range": [["7.4.3", ""]], "type": "raw"},
            "max-route-cache-size": {"v_range": [["7.4.3", ""]], "type": "int"},
            "fortitoken-cloud-push-status": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ssh-hostkey-override": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "proxy-hardware-acceleration": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "switch-controller-reserved-network": {"v_range": [["7.4.3", ""]], "type": "raw"},
            "ssd-trim-date": {"v_range": [["7.4.3", ""]], "type": "int"},
            "wad-worker-count": {"v_range": [["7.4.3", ""]], "type": "int"},
            "ssh-hostkey": {"v_range": [["7.4.3", ""]], "no_log": true, "type": "str"},
            "wireless-controller-port": {"v_range": [["7.4.3", ""]], "type": "int"},
            "fgd-alert-subscription": {
                "v_range": [["7.4.3", ""]],
                "type": "list",
                "choices": ["advisory", "latest-threat", "latest-virus", "latest-attack", "new-antivirus-db", "new-attack-db"],
                "elements": "str"
            },
            "forticontroller-proxy-port": {"v_range": [["7.4.3", ""]], "type": "int"},
            "dh-params": {"v_range": [["7.4.3", ""]], "choices": ["1024", "1536", "2048", "3072", "4096", "6144", "8192"], "type": "str"},
            "memory-use-threshold-green": {"v_range": [["7.4.3", ""]], "type": "int"},
            "proxy-cert-use-mgmt-vdom": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "proxy-auth-lifetime-timeout": {"v_range": [["7.4.3", ""]], "type": "int"},
            "gui-auto-upgrade-setup-warning": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "gui-cdn-usage": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "two-factor-email-expiry": {"v_range": [["7.4.3", ""]], "type": "int"},
            "udp-idle-timer": {"v_range": [["7.4.3", ""]], "type": "int"},
            "interface-subnet-usage": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "forticontroller-proxy": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ssh-enc-algo": {
                "v_range": [["7.4.3", ""]],
                "type": "list",
                "choices": [
                    "chacha20-poly1305@openssh.com", "aes128-ctr", "aes192-ctr", "aes256-ctr", "arcfour256", "arcfour128", "aes128-cbc", "3des-cbc",
                    "blowfish-cbc", "cast128-cbc", "aes192-cbc", "aes256-cbc", "arcfour", "rijndael-cbc@lysator.liu.se", "aes128-gcm@openssh.com",
                    "aes256-gcm@openssh.com"
                ],
                "elements": "str"
            },
            "block-session-timer": {"v_range": [["7.4.3", ""]], "type": "int"},
            "quic-pmtud": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "admin-https-ssl-ciphersuites": {
                "v_range": [["7.4.3", ""]],
                "type": "list",
                "choices": [
                    "TLS-AES-128-GCM-SHA256", "TLS-AES-256-GCM-SHA384", "TLS-CHACHA20-POLY1305-SHA256", "TLS-AES-128-CCM-SHA256",
                    "TLS-AES-128-CCM-8-SHA256"
                ],
                "elements": "str"
            },
            "security-rating-result-submission": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "user-device-store-max-unified-mem": {"v_range": [["7.4.3", ""]], "type": "int"},
            "management-port": {"v_range": [["7.4.3", ""]], "type": "int"},
            "fortigslb-integration": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "admin-https-ssl-versions": {
                "v_range": [["7.4.3", ""]],
                "type": "list",
                "choices": ["tlsv1-0", "tlsv1-1", "tlsv1-2", "sslv3", "tlsv1-3"],
                "elements": "str"
            },
            "cert-chain-max": {"v_range": [["7.4.3", ""]], "type": "int"},
            "qsfp28-40g-port": {"v_range": [["7.4.3", ""]], "type": "raw"},
            "strong-crypto": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "multi-factor-authentication": {"v_range": [["7.4.3", ""]], "choices": ["optional", "mandatory"], "type": "str"},
            "fds-statistics": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "gui-display-hostname": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "two-factor-ftk-expiry": {"v_range": [["7.4.3", ""]], "type": "int"},
            "wad-source-affinity": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ssl-static-key-ciphers": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "daily-restart": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "snat-route-change": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "tcp-rst-timer": {"v_range": [["7.4.3", ""]], "type": "int"},
            "anti-replay": {"v_range": [["7.4.3", ""]], "choices": ["disable", "loose", "strict"], "type": "str"},
            "ssl-min-proto-version": {"v_range": [["7.4.3", ""]], "choices": ["TLSv1", "TLSv1-1", "TLSv1-2", "SSLv3", "TLSv1-3"], "type": "str"},
            "speedtestd-server-port": {"v_range": [["7.4.3", ""]], "type": "int"},
            "cpu-use-threshold": {"v_range": [["7.4.3", ""]], "type": "int"},
            "admin-host": {"v_range": [["7.4.3", ""]], "type": "str"},
            "csr-ca-attribute": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "fortiservice-port": {"v_range": [["7.4.3", ""]], "type": "int"},
            "ssd-trim-hour": {"v_range": [["7.4.3", ""]], "type": "int"},
            "purdue-level": {"v_range": [["7.4.3", ""]], "choices": ["1", "2", "3", "4", "5", "1.5", "2.5", "3.5", "5.5"], "type": "str"},
            "management-vdom": {"v_range": [["7.4.3", ""]], "type": "raw"},
            "quic-ack-thresold": {"v_range": [["7.4.3", ""]], "type": "int"},
            "qsfpdd-100g-port": {"v_range": [["7.4.3", ""]], "type": "raw"},
            "ips-affinity": {"v_range": [["7.4.3", ""]], "type": "str"},
            "vip-arp-range": {"v_range": [["7.4.3", ""]], "choices": ["restricted", "unlimited"], "type": "str"},
            "internet-service-database": {"v_range": [["7.4.3", ""]], "choices": ["mini", "standard", "full", "on-demand"], "type": "str"},
            "revision-image-auto-backup": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "sflowd-max-children-num": {"v_range": [["7.4.3", ""]], "type": "int"},
            "admin-https-pki-required": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "special-file-23-support": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "npu-neighbor-update": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "log-single-cpu-high": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "management-ip": {"v_range": [["7.4.3", ""]], "type": "str"},
            "proxy-resource-mode": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "admin-ble-button": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "gui-firmware-upgrade-warning": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "dp-tcp-normal-timer": {"v_range": [["7.4.3", ""]], "type": "int"},
            "ipv6-allow-traffic-redirect": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "cli-audit-log": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "memory-use-threshold-extreme": {"v_range": [["7.4.3", ""]], "type": "int"},
            "ha-affinity": {"v_range": [["7.4.3", ""]], "type": "str"},
            "restart-time": {"v_range": [["7.4.3", ""]], "type": "str"},
            "speedtestd-ctrl-port": {"v_range": [["7.4.3", ""]], "type": "int"},
            "gui-wireless-opensecurity": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "memory-use-threshold-red": {"v_range": [["7.4.3", ""]], "type": "int"},
            "dp-fragment-timer": {"v_range": [["7.4.3", ""]], "type": "int"},
            "wad-restart-start-time": {"v_range": [["7.4.3", ""]], "type": "str"},
            "proxy-re-authentication-time": {"v_range": [["7.4.3", ""]], "type": "int"},
            "gui-app-detection-sdwan": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "scanunit-count": {"v_range": [["7.4.3", ""]], "type": "int"},
            "tftp": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "xstools-update-frequency": {"v_range": [["7.4.3", ""]], "type": "int"},
            "clt-cert-req": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "fortiextender-vlan-mode": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "auth-http-port": {"v_range": [["7.4.3", ""]], "type": "int"},
            "per-user-bal": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "gui-date-format": {
                "v_range": [["7.4.3", ""]],
                "choices": ["yyyy/MM/dd", "dd/MM/yyyy", "MM/dd/yyyy", "yyyy-MM-dd", "dd-MM-yyyy", "MM-dd-yyyy"],
                "type": "str"
            },
            "log-uuid-address": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "cloud-communication": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "lldp-reception": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "two-factor-ftm-expiry": {"v_range": [["7.4.3", ""]], "type": "int"},
            "quic-udp-payload-size-shaping-per-cid": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "autorun-log-fsck": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "vpn-ems-sn-check": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "admin-ssh-password": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "airplane-mode": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "batch-cmdb": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ip-src-port-range": {"v_range": [["7.4.3", ""]], "type": "raw"},
            "strict-dirty-session-check": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "user-device-store-max-devices": {"v_range": [["7.4.3", ""]], "type": "int"},
            "dp-udp-idle-timer": {"v_range": [["7.4.3", ""]], "type": "int"},
            "internal-switch-speed": {
                "v_range": [["7.4.3", ""]],
                "type": "list",
                "choices": ["auto", "10full", "10half", "100full", "100half", "1000full", "1000auto"],
                "elements": "str"
            },
            "forticonverter-config-upload": {"v_range": [["7.4.3", ""]], "choices": ["disable", "once"], "type": "str"},
            "ipsec-round-robin": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "wad-affinity": {"v_range": [["7.4.3", ""]], "type": "str"},
            "wifi-ca-certificate": {"v_range": [["7.4.3", ""]], "type": "raw"},
            "wimax-4g-usb": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "miglog-affinity": {"v_range": [["7.4.3", ""]], "type": "str"},
            "faz-disk-buffer-size": {"v_range": [["7.4.3", ""]], "type": "int"},
            "ssh-kex-algo": {
                "v_range": [["7.4.3", ""]],
                "type": "list",
                "choices": [
                    "diffie-hellman-group1-sha1", "diffie-hellman-group14-sha1", "diffie-hellman-group-exchange-sha1",
                    "diffie-hellman-group-exchange-sha256", "curve25519-sha256@libssh.org", "ecdh-sha2-nistp256", "ecdh-sha2-nistp384",
                    "ecdh-sha2-nistp521", "diffie-hellman-group14-sha256", "diffie-hellman-group16-sha512", "diffie-hellman-group18-sha512"
                ],
                "elements": "str"
            },
            "auto-auth-extension-device": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "forticarrier-bypass": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "reset-sessionless-tcp": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "early-tcp-npu-session": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "http-unauthenticated-request-limit": {"v_range": [["7.4.3", ""]], "type": "int"},
            "gui-local-out": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "tcp-option": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "proxy-auth-timeout": {"v_range": [["7.4.3", ""]], "type": "int"},
            "fortiextender-discovery-lockdown": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "lldp-transmission": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "split-port": {"v_range": [["7.4.3", ""]], "type": "raw"},
            "gui-certificates": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "cfg-save": {"v_range": [["7.4.3", ""]], "choices": ["automatic", "manual", "revert"], "type": "str"},
            "auth-keepalive": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "split-port-mode": {
                "v_range": [["7.4.3", ""]],
                "type": "list",
                "options": {
                    "interface": {"v_range": [["7.4.3", ""]], "type": "str"},
                    "split-mode": {
                        "v_range": [["7.4.3", ""]],
                        "choices": ["disable", "4x10G", "4x25G", "4x50G", "8x50G", "4x100G", "2x200G", "8x25G"],
                        "type": "str"
                    }
                },
                "elements": "dict"
            },
            "admin-forticloud-sso-login": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "post-login-banner": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "br-fdb-max-entry": {"v_range": [["7.4.3", ""]], "type": "int"},
            "ip-fragment-mem-thresholds": {"v_range": [["7.4.3", ""]], "type": "int"},
            "fortiextender-provision-on-authorization": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "reboot-upon-config-restore": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "syslog-affinity": {"v_range": [["7.4.3", ""]], "type": "str"},
            "fortiextender-data-port": {"v_range": [["7.4.3", ""]], "type": "int"},
            "quic-tls-handshake-timeout": {"v_range": [["7.4.3", ""]], "type": "int"},
            "forticonverter-integration": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "proxy-keep-alive-mode": {"v_range": [["7.4.3", ""]], "choices": ["session", "traffic", "re-authentication"], "type": "str"},
            "cmdbsvr-affinity": {"v_range": [["7.4.3", ""]], "type": "str"},
            "wad-memory-change-granularity": {"v_range": [["7.4.3", ""]], "type": "int"},
            "dhcp-lease-backup-interval": {"v_range": [["7.4.3", ""]], "type": "int"},
            "check-protocol-header": {"v_range": [["7.4.3", ""]], "choices": ["loose", "strict"], "type": "str"},
            "av-failopen-session": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ipsec-ha-seqjump-rate": {"v_range": [["7.4.3", ""]], "type": "int"},
            "admin-hsts-max-age": {"v_range": [["7.4.3", ""]], "type": "int"},
            "igmp-state-limit": {"v_range": [["7.4.3", ""]], "type": "int"},
            "admin-login-max": {"v_range": [["7.4.3", ""]], "type": "int"},
            "ipv6-allow-multicast-probe": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "virtual-switch-vlan": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "admin-lockout-threshold": {"v_range": [["7.4.3", ""]], "type": "int"},
            "dp-pinhole-timer": {"v_range": [["7.4.3", ""]], "type": "int"},
            "wireless-controller": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "bfd-affinity": {"v_range": [["7.4.3", ""]], "type": "str"},
            "ssd-trim-freq": {"v_range": [["7.4.3", ""]], "choices": ["daily", "weekly", "monthly", "hourly", "never"], "type": "str"},
            "two-factor-sms-expiry": {"v_range": [["7.4.3", ""]], "type": "int"},
            "traffic-priority": {"v_range": [["7.4.3", ""]], "choices": ["tos", "dscp"], "type": "str"},
            "proxy-and-explicit-proxy": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "sslvpn-web-mode": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ssh-hostkey-password": {"v_range": [["7.4.3", ""]], "no_log": true, "type": "raw"},
            "wad-csvc-db-count": {"v_range": [["7.4.3", ""]], "type": "int"},
            "ipv6-allow-anycast-probe": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "honor-df": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "hyper-scale-vdom-num": {"v_range": [["7.4.3", ""]], "type": "int"},
            "wad-csvc-cs-count": {"v_range": [["7.4.3", ""]], "type": "int"},
            "internal-switch-mode": {"v_range": [["7.4.3", ""]], "choices": ["switch", "interface", "hub"], "type": "str"},
            "cfg-revert-timeout": {"v_range": [["7.4.3", ""]], "type": "int"},
            "admin-concurrent": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ipv6-allow-local-in-silent-drop": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "tcp-halfopen-timer": {"v_range": [["7.4.3", ""]], "type": "int"},
            "dp-rsync-timer": {"v_range": [["7.4.3", ""]], "type": "int"},
            "management-port-use-admin-sport": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "gui-forticare-registration-setup-warning": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "gui-replacement-message-groups": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "security-rating-run-on-schedule": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "admin-lockout-duration": {"v_range": [["7.4.3", ""]], "type": "int"},
            "optimize-flow-mode": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "private-data-encryption": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "wireless-mode": {"v_range": [["7.4.3", ""]], "choices": ["ac", "client", "wtp", "fwfap"], "type": "str"},
            "alias": {"v_range": [["7.4.3", ""]], "type": "str"},
            "ssh-hostkey-algo": {
                "v_range": [["7.4.3", ""]],
                "type": "list",
                "choices": ["ssh-rsa", "ecdsa-sha2-nistp521", "rsa-sha2-256", "rsa-sha2-512", "ssh-ed25519", "ecdsa-sha2-nistp384", "ecdsa-sha2-nistp256"],
                "elements": "str"
            },
            "fortitoken-cloud": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "av-affinity": {"v_range": [["7.4.3", ""]], "type": "str"},
            "proxy-worker-count": {"v_range": [["7.4.3", ""]], "type": "int"},
            "ipsec-asic-offload": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "miglogd-children": {"v_range": [["7.4.3", ""]], "type": "int"},
            "sslvpn-max-worker-count": {"v_range": [["7.4.3", ""]], "type": "int"},
            "ssh-mac-algo": {
                "v_range": [["7.4.3", ""]],
                "type": "list",
                "choices": [
                    "hmac-md5", "hmac-md5-etm@openssh.com", "hmac-md5-96", "hmac-md5-96-etm@openssh.com", "hmac-sha1", "hmac-sha1-etm@openssh.com",
                    "hmac-sha2-256", "hmac-sha2-256-etm@openssh.com", "hmac-sha2-512", "hmac-sha2-512-etm@openssh.com", "hmac-ripemd160",
                    "hmac-ripemd160@openssh.com", "hmac-ripemd160-etm@openssh.com", "umac-64@openssh.com", "umac-128@openssh.com",
                    "umac-64-etm@openssh.com", "umac-128-etm@openssh.com"
                ],
                "elements": "str"
            },
            "url-filter-count": {"v_range": [["7.4.3", ""]], "type": "int"},
            "wifi-certificate": {"v_range": [["7.4.3", ""]], "type": "raw"},
            "radius-port": {"v_range": [["7.4.3", ""]], "type": "int"},
            "sys-perf-log-interval": {"v_range": [["7.4.3", ""]], "type": "int"},
            "gui-fortigate-cloud-sandbox": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "auth-cert": {"v_range": [["7.4.3", ""]], "type": "raw"},
            "fortiextender": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "admin-reset-button": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "av-failopen": {"v_range": [["7.4.3", ""]], "choices": ["off", "pass", "one-shot", "idledrop"], "type": "str"},
            "user-device-store-max-users": {"v_range": [["7.4.3", ""]], "type": "int"},
            "auth-session-limit": {"v_range": [["7.4.3", ""]], "choices": ["block-new", "logout-inactive"], "type": "str"},
            "ipv6-allow-local-in-slient-drop": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "quic-congestion-control-algo": {"v_range": [["7.4.3", ""]], "choices": ["cubic", "bbr", "bbr2", "reno"], "type": "str"},
            "auth-ike-saml-port": {"v_range": [["7.4.3", ""]], "type": "int"},
            "wad-restart-end-time": {"v_range": [["7.4.3", ""]], "type": "str"},
            "http-request-limit": {"v_range": [["7.4.3", ""]], "type": "int"},
            "irq-time-accounting": {"v_range": [["7.4.3", ""]], "choices": ["auto", "force"], "type": "str"},
            "remoteauthtimeout": {"v_range": [["7.4.3", ""]], "type": "int"},
            "admin-https-ssl-banned-ciphers": {
                "v_range": [["7.4.3", ""]],
                "type": "list",
                "choices": [
                    "RSA", "DHE", "ECDHE", "DSS", "ECDSA", "AES", "AESGCM", "CAMELLIA", "3DES", "SHA1", "SHA256", "SHA384", "STATIC", "CHACHA20", "ARIA",
                    "AESCCM"
                ],
                "elements": "str"
            },
            "allow-traffic-redirect": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "legacy-poe-device-support": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "wad-restart-mode": {"v_range": [["7.4.3", ""]], "choices": ["none", "time", "memory"], "type": "str"},
            "fds-statistics-period": {"v_range": [["7.4.3", ""]], "type": "int"},
            "admin-telnet": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ipv6-accept-dad": {"v_range": [["7.4.3", ""]], "type": "int"},
            "tcp-timewait-timer": {"v_range": [["7.4.3", ""]], "type": "int"},
            "admin-console-timeout": {"v_range": [["7.4.3", ""]], "type": "int"},
            "default-service-source-port": {"v_range": [["7.4.3", ""]], "type": "str"},
            "quic-max-datagram-size": {"v_range": [["7.4.3", ""]], "type": "int"},
            "refresh": {"v_range": [["7.4.3", ""]], "type": "int"},
            "extender-controller-reserved-network": {"v_range": [["7.4.3", ""]], "type": "raw"},
            "url-filter-affinity": {"v_range": [["7.4.3", ""]], "type": "str"},
            "policy-auth-concurrent": {"v_range": [["7.4.3", ""]], "type": "int"},
            "ipsec-hmac-offload": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "traffic-priority-level": {"v_range": [["7.4.3", ""]], "choices": ["high", "medium", "low"], "type": "str"},
            "ipsec-qat-offload": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ssd-trim-min": {"v_range": [["7.4.3", ""]], "type": "int"},
            "gui-date-time-source": {"v_range": [["7.4.3", ""]], "choices": ["system", "browser"], "type": "str"},
            "log-ssl-connection": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ndp-max-entry": {"v_range": [["7.4.3", ""]], "type": "int"},
            "vdom-mode": {"v_range": [["7.4.3", ""]], "choices": ["no-vdom", "multi-vdom", "split-vdom"], "type": "str"},
            "internet-service-download-list": {"v_range": [["7.4.3", ""]], "type": "raw"},
            "fortitoken-cloud-sync-interval": {"v_range": [["7.4.3", ""]], "no_log": true, "type": "int"},
            "ssd-trim-weekday": {
                "v_range": [["7.4.3", ""]],
                "choices": ["sunday", "monday", "tuesday", "wednesday", "thursday", "friday", "saturday"],
                "type": "str"
            },
            "two-factor-fac-expiry": {"v_range": [["7.4.3", ""]], "type": "int"},
            "gui-rest-api-cache": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "admin-forticloud-sso-default-profile": {"v_range": [["7.4.3", ""]], "type": "raw"},
            "proxy-auth-lifetime": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "device-idle-timeout": {"v_range": [["7.4.3", ""]], "type": "int"},
            "login-timestamp": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "speedtest-server": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "edit-vdom-prompt": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "gui-cdn-domain-override": {"v_range": [["7.4.3", ""]], "type": "str"},
            "admin-ssh-grace-time": {"v_range": [["7.4.3", ""]], "type": "int"},
            "sslvpn-ems-sn-check": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "user-server-cert": {"v_range": [["7.4.3", ""]], "type": "raw"},
            "gui-allow-default-hostname": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "proxy-re-authentication-mode": {"v_range": [["7.4.3", ""]], "choices": ["session", "traffic", "absolute"], "type": "str"},
            "ipsec-soft-dec-async": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "admin-maintainer": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "dst": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "fec-port": {"v_range": [["7.4.3", ""]], "type": "int"},
            "ssh-kex-sha1": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ssh-mac-weak": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "sslvpn-cipher-hardware-acceleration": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "sys-file-check-interval": {"v_range": [["7.4.3", ""]], "type": "int"},
            "ssh-hmac-md5": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ssh-cbc-cipher": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "gui-fortiguard-resource-fetch": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "sslvpn-kxp-hardware-acceleration": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "sslvpn-plugin-version-check": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "fortiipam-integration": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "gui-firmware-upgrade-setup-warning": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "log-uuid-policy": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "per-user-bwl": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "gui-fortisandbox-cloud": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "fortitoken-cloud-service": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "hw-switch-ether-filter": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "virtual-server-count": {"v_range": [["7.4.3", ""]], "type": "int"},
            "endpoint-control-fds-access": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "proxy-cipher-hardware-acceleration": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "proxy-kxp-hardware-acceleration": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "virtual-server-hardware-acceleration": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "user-history-password-threshold": {"v_range": [["7.6.0", ""]], "no_log": true, "type": "int"},
            "delay-tcp-npu-session": {"v_range": [["7.6.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "auth-session-auto-backup-interval": {"v_range": [["7.6.0", ""]], "choices": ["1min", "5min", "15min", "30min", "1hr"], "type": "str"},
            "ip-conflict-detection": {"v_range": [["7.6.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "gtpu-dynamic-source-port": {"v_range": [["7.6.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ip-fragment-timeout": {"v_range": [["7.6.0", ""]], "type": "int"},
            "ipv6-fragment-timeout": {"v_range": [["7.6.0", ""]], "type": "int"},
            "scim-server-cert": {"v_range": [["7.6.0", ""]], "type": "raw"},
            "scim-http-port": {"v_range": [["7.6.0", ""]], "type": "int"},
            "auth-session-auto-backup": {"v_range": [["7.6.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "scim-https-port": {"v_range": [["7.6.0", ""]], "type": "int"},
            "httpd-max-worker-count": {"v_range": [["7.6.0", ""]], "type": "int"}
        }
    },
    "devprof_system_ntp": {
        "stated": false,
        "options": {
            "ntpserver": {
                "v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]],
                "type": "list",
                "options": {
                    "authentication": {
                        "v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]],
                        "choices": ["disable", "enable"],
                        "type": "str"
                    },
                    "id": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "int"},
                    "key": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "no_log": true, "type": "raw"},
                    "key-id": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "no_log": true, "type": "int"},
                    "ntpv3": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "server": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "str"},
                    "interface": {"v_range": [["6.2.7", "6.2.12"], ["6.4.3", "7.0.2"], ["7.4.3", ""]], "type": "str"},
                    "interface-select-method": {
                        "v_range": [["6.2.7", "6.2.12"], ["6.4.3", "7.0.2"], ["7.4.3", ""]],
                        "choices": ["auto", "sdwan", "specify"],
                        "type": "str"
                    },
                    "ip-type": {"v_range": [["7.4.2", ""]], "choices": ["IPv6", "IPv4", "Both"], "type": "str"},
                    "key-type": {"v_range": [["7.4.3", ""]], "choices": ["SHA1", "SHA256", "MD5"], "type": "str"}
                },
                "elements": "dict"
            },
            "ntpsync": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "source-ip6": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "str"},
            "syncinterval": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "int"},
            "type": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "choices": ["fortiguard", "custom"], "type": "str"},
            "authentication": {"v_range": [["6.2.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "key": {"v_range": [["6.2.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "no_log": true, "type": "raw"},
            "key-id": {"v_range": [["6.2.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "no_log": true, "type": "int"},
            "key-type": {"v_range": [["6.2.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "choices": ["MD5", "SHA1", "SHA256"], "type": "str"},
            "interface": {"v_range": [["7.4.3", ""]], "type": "raw"},
            "server-mode": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "source-ip": {"v_range": [["7.4.3", ""]], "type": "str"}
        }
    },
    "devprof_system_ntp_ntpserver": {
        "stated": true,
        "options": {
            "authentication": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "id": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "required": true, "type": "int"},
            "key": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "no_log": true, "type": "raw"},
            "key-id": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "no_log": true, "type": "int"},
            "ntpv3": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "server": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "str"},
            "interface": {"v_range": [["6.2.7", "6.2.12"], ["6.4.3", "7.0.2"], ["7.4.3", ""]], "type": "str"},
            "interface-select-method": {
                "v_range": [["6.2.7", "6.2.12"], ["6.4.3", "7.0.2"], ["7.4.3", ""]],
                "choices": ["auto", "sdwan", "specify"],
                "type": "str"
            },
            "ip-type": {"v_range": [["7.4.2", ""]], "choices": ["IPv6", "IPv4", "Both"], "type": "str"},
            "key-type": {"v_range": [["7.4.3", ""]], "choices": ["SHA1", "SHA256", "MD5"], "type": "str"}
        }
    },
    "devprof_system_replacemsg_admin": {
        "stated": false,
        "options": {
            "buffer": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "str"},
            "format": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "choices": ["none", "text", "html", "wml"], "type": "str"},
            "header": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "choices": ["none", "http", "8bit"], "type": "str"},
            "msg-type": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "str"}
        }
    },
    "devprof_system_replacemsg_alertmail": {
        "stated": false,
        "options": {
            "buffer": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "str"},
            "format": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "choices": ["none", "text", "html", "wml"], "type": "str"},
            "header": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "choices": ["none", "http", "8bit"], "type": "str"},
            "msg-type": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "str"}
        }
    },
    "devprof_system_replacemsg_auth": {
        "stated": false,
        "options": {
            "buffer": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "str"},
            "format": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "choices": ["none", "text", "html", "wml"], "type": "str"},
            "header": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "choices": ["none", "http", "8bit"], "type": "str"},
            "msg-type": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "str"}
        }
    },
    "devprof_system_replacemsg_devicedetectionportal": {
        "stated": false,
        "options": {
            "buffer": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "str"},
            "format": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "choices": ["none", "text", "html", "wml"], "type": "str"},
            "header": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "choices": ["none", "http", "8bit"], "type": "str"},
            "msg-type": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "str"}
        }
    },
    "devprof_system_replacemsg_ec": {
        "stated": false,
        "options": {
            "buffer": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", "7.2.1"]], "type": "str"},
            "format": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", "7.2.1"]], "choices": ["none", "text", "html", "wml"], "type": "str"},
            "header": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", "7.2.1"]], "choices": ["none", "http", "8bit"], "type": "str"},
            "msg-type": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", "7.2.1"]], "type": "str"}
        }
    },
    "devprof_system_replacemsg_fortiguardwf": {
        "stated": false,
        "options": {
            "buffer": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "str"},
            "format": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "choices": ["none", "text", "html", "wml"], "type": "str"},
            "header": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "choices": ["none", "http", "8bit"], "type": "str"},
            "msg-type": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "str"}
        }
    },
    "devprof_system_replacemsg_ftp": {
        "stated": false,
        "options": {
            "buffer": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "str"},
            "format": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "choices": ["none", "text", "html", "wml"], "type": "str"},
            "header": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "choices": ["none", "http", "8bit"], "type": "str"},
            "msg-type": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "str"}
        }
    },
    "devprof_system_replacemsg_http": {
        "stated": false,
        "options": {
            "buffer": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "str"},
            "format": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "choices": ["none", "text", "html", "wml"], "type": "str"},
            "header": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "choices": ["none", "http", "8bit"], "type": "str"},
            "msg-type": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "str"}
        }
    },
    "devprof_system_replacemsg_mail": {
        "stated": false,
        "options": {
            "buffer": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "str"},
            "format": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "choices": ["none", "text", "html", "wml"], "type": "str"},
            "header": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "choices": ["none", "http", "8bit"], "type": "str"},
            "msg-type": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "str"}
        }
    },
    "devprof_system_replacemsg_mms": {
        "stated": false,
        "options": {
            "buffer": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "str"},
            "charset": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "choices": ["us-ascii", "utf-8"], "type": "str"},
            "format": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "choices": ["none", "text", "html", "wml"], "type": "str"},
            "header": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "choices": ["none", "http", "8bit"], "type": "str"},
            "image": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "str"},
            "msg-type": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "str"}
        }
    },
    "devprof_system_replacemsg_nacquar": {
        "stated": false,
        "options": {
            "buffer": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "str"},
            "format": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "choices": ["none", "text", "html", "wml"], "type": "str"},
            "header": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "choices": ["none", "http", "8bit"], "type": "str"},
            "msg-type": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "str"}
        }
    },
    "devprof_system_replacemsg_nntp": {
        "stated": false,
        "options": {
            "buffer": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "str"},
            "format": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "choices": ["none", "text", "html", "wml"], "type": "str"},
            "header": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "choices": ["none", "http", "8bit"], "type": "str"},
            "msg-type": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "str"}
        }
    },
    "devprof_system_replacemsg_spam": {
        "stated": false,
        "options": {
            "buffer": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "str"},
            "format": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "choices": ["none", "text", "html", "wml"], "type": "str"},
            "header": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "choices": ["none", "http", "8bit"], "type": "str"},
            "msg-type": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "str"}
        }
    },
    "devprof_system_replacemsg_sslvpn": {
        "stated": false,
        "options": {
            "buffer": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "str"},
            "format": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "choices": ["none", "text", "html", "wml"], "type": "str"},
            "header": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "choices": ["none", "http", "8bit"], "type": "str"},
            "msg-type": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "str"}
        }
    },
    "devprof_system_replacemsg_trafficquota": {
        "stated": false,
        "options": {
            "buffer": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "str"},
            "format": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "choices": ["none", "text", "html", "wml"], "type": "str"},
            "header": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "choices": ["none", "http", "8bit"], "type": "str"},
            "msg-type": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "str"}
        }
    },
    "devprof_system_replacemsg_utm": {
        "stated": false,
        "options": {
            "buffer": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "str"},
            "format": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "choices": ["none", "text", "html", "wml"], "type": "str"},
            "header": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "choices": ["none", "http", "8bit"], "type": "str"},
            "msg-type": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "str"}
        }
    },
    "devprof_system_replacemsg_webproxy": {
        "stated": false,
        "options": {
            "buffer": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "str"},
            "format": {
                "v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]],
                "choices": ["none", "text", "html", "wml", "opt:none"],
                "type": "str"
            },
            "header": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "choices": ["none", "http", "8bit", "opt:none"], "type": "str"},
            "msg-type": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "str"}
        }
    },
    "devprof_system_snmp_community": {
        "stated": true,
        "options": {
            "events": {
                "v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]],
                "type": "list",
                "choices": [
                    "cpu-high", "mem-low", "log-full", "intf-ip", "vpn-tun-up", "vpn-tun-down", "ha-switch", "ha-hb-failure", "ips-signature",
                    "ips-anomaly", "av-virus", "av-oversize", "av-pattern", "av-fragmented", "fm-if-change", "fm-conf-change", "temperature-high",
                    "voltage-alert", "ha-member-up", "ha-member-down", "ent-conf-change", "av-conserve", "av-bypass", "av-oversize-passed",
                    "av-oversize-blocked", "ips-pkg-update", "power-supply-failure", "amc-bypass", "faz-disconnect", "fan-failure", "bgp-established",
                    "bgp-backward-transition", "wc-ap-up", "wc-ap-down", "fswctl-session-up", "fswctl-session-down", "ips-fail-open",
                    "load-balance-real-server-down", "device-new", "enter-intf-bypass", "exit-intf-bypass", "per-cpu-high", "power-blade-down",
                    "confsync_failure", "dhcp", "pool-usage", "power-redundancy-degrade", "power-redundancy-failure", "ospf-nbr-state-change",
                    "ospf-virtnbr-state-change", "disk-failure", "disk-overload", "faz-main-failover", "faz-alt-failover", "slbc", "faz", "power-supply",
                    "ippool", "interface"
                ],
                "elements": "str"
            },
            "hosts": {
                "v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]],
                "type": "list",
                "options": {
                    "ha-direct": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "host-type": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "choices": ["any", "query", "trap"], "type": "str"},
                    "id": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "int"},
                    "ip": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "str"},
                    "source-ip": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "str"},
                    "interface-select-method": {"v_range": [["7.6.0", ""]], "choices": ["auto", "sdwan", "specify"], "type": "str"},
                    "interface": {"v_range": [["7.6.0", ""]], "type": "raw"}
                },
                "elements": "dict"
            },
            "hosts6": {
                "v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]],
                "type": "list",
                "options": {
                    "ha-direct": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "host-type": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "choices": ["any", "query", "trap"], "type": "str"},
                    "id": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "int"},
                    "ipv6": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "str"},
                    "source-ipv6": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "str"},
                    "interface": {"v_range": [["7.6.0", ""]], "type": "raw"},
                    "interface-select-method": {"v_range": [["7.6.0", ""]], "choices": ["auto", "sdwan", "specify"], "type": "str"}
                },
                "elements": "dict"
            },
            "id": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "required": true, "type": "int"},
            "name": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "str"},
            "query-v1-port": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "int"},
            "query-v1-status": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "query-v2c-port": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "int"},
            "query-v2c-status": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "status": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "trap-v1-lport": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "int"},
            "trap-v1-rport": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "int"},
            "trap-v1-status": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "trap-v2c-lport": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "int"},
            "trap-v2c-rport": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "int"},
            "trap-v2c-status": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "mib-view": {"v_range": [["7.2.0", ""]], "type": "str"},
            "vdoms": {"v_range": [["7.2.0", ""]], "type": "raw"}
        }
    },
    "devprof_system_snmp_community_hosts": {
        "stated": true,
        "options": {
            "ha-direct": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "host-type": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "choices": ["any", "query", "trap"], "type": "str"},
            "id": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "required": true, "type": "int"},
            "ip": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "str"},
            "source-ip": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "str"},
            "interface-select-method": {"v_range": [["7.6.0", ""]], "choices": ["auto", "sdwan", "specify"], "type": "str"},
            "interface": {"v_range": [["7.6.0", ""]], "type": "raw"}
        }
    },
    "devprof_system_snmp_community_hosts6": {
        "stated": true,
        "options": {
            "ha-direct": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "host-type": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "choices": ["any", "query", "trap"], "type": "str"},
            "id": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "required": true, "type": "int"},
            "ipv6": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "str"},
            "source-ipv6": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "str"},
            "interface": {"v_range": [["7.6.0", ""]], "type": "raw"},
            "interface-select-method": {"v_range": [["7.6.0", ""]], "choices": ["auto", "sdwan", "specify"], "type": "str"}
        }
    },
    "devprof_system_snmp_sysinfo": {
        "stated": false,
        "options": {
            "status": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "append-index": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "trap-high-cpu-threshold": {"v_range": [["7.4.3", ""]], "type": "int"},
            "trap-log-full-threshold": {"v_range": [["7.4.3", ""]], "type": "int"},
            "engine-id": {"v_range": [["7.4.3", ""]], "type": "str"},
            "trap-freeable-memory-threshold": {"v_range": [["7.4.3", ""]], "type": "int"},
            "contact-info": {"v_range": [["7.4.3", ""]], "type": "str"},
            "engine-id-type": {"v_range": [["7.4.3", ""]], "choices": ["text", "hex", "mac"], "type": "str"},
            "description": {"v_range": [["7.4.3", ""]], "type": "str"},
            "trap-free-memory-threshold": {"v_range": [["7.4.3", ""]], "type": "int"},
            "trap-low-memory-threshold": {"v_range": [["7.4.3", ""]], "type": "int"},
            "location": {"v_range": [["7.4.3", ""]], "type": "str"},
            "non-mgmt-vdom-query": {"v_range": [["7.6.0", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "devprof_system_snmp_user": {
        "stated": true,
        "options": {
            "auth-proto": {
                "v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]],
                "choices": ["md5", "sha", "sha224", "sha256", "sha384", "sha512"],
                "type": "str"
            },
            "auth-pwd": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "raw"},
            "events": {
                "v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]],
                "type": "list",
                "choices": [
                    "cpu-high", "mem-low", "log-full", "intf-ip", "vpn-tun-up", "vpn-tun-down", "ha-switch", "fm-conf-change", "ips-signature",
                    "ips-anomaly", "temperature-high", "voltage-alert", "av-virus", "av-oversize", "av-pattern", "av-fragmented", "ha-hb-failure",
                    "fan-failure", "ha-member-up", "ha-member-down", "ent-conf-change", "av-conserve", "av-bypass", "av-oversize-passed",
                    "av-oversize-blocked", "ips-pkg-update", "fm-if-change", "power-supply-failure", "amc-bypass", "faz-disconnect", "bgp-established",
                    "bgp-backward-transition", "wc-ap-up", "wc-ap-down", "fswctl-session-up", "fswctl-session-down", "ips-fail-open",
                    "load-balance-real-server-down", "device-new", "enter-intf-bypass", "exit-intf-bypass", "per-cpu-high", "power-blade-down",
                    "confsync_failure", "dhcp", "pool-usage", "power-redundancy-degrade", "power-redundancy-failure", "ospf-nbr-state-change",
                    "ospf-virtnbr-state-change", "disk-failure", "disk-overload", "faz-main-failover", "faz-alt-failover", "slbc", "faz", "power-supply",
                    "ippool", "interface"
                ],
                "elements": "str"
            },
            "ha-direct": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "name": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "required": true, "type": "str"},
            "notify-hosts": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "raw"},
            "notify-hosts6": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "str"},
            "priv-proto": {
                "v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]],
                "choices": ["aes", "des", "aes256", "aes256cisco"],
                "type": "str"
            },
            "priv-pwd": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "raw"},
            "queries": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "query-port": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "int"},
            "security-level": {
                "v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]],
                "choices": ["no-auth-no-priv", "auth-no-priv", "auth-priv"],
                "type": "str"
            },
            "source-ip": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "str"},
            "source-ipv6": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "str"},
            "status": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "trap-lport": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "int"},
            "trap-rport": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "int"},
            "trap-status": {"v_range": [["6.0.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "mib-view": {"v_range": [["7.2.0", ""]], "type": "str"},
            "vdoms": {"v_range": [["7.2.0", ""]], "type": "raw"},
            "interface": {"v_range": [["7.6.0", ""]], "type": "raw"},
            "interface-select-method": {"v_range": [["7.6.0", ""]], "choices": ["auto", "sdwan", "specify"], "type": "str"}
        }
    },
    "diameterfilter_profile": {
        "stated": true,
        "options": {
            "cmd-flags-reserve-set": {"v_range": [["7.4.2", ""]], "choices": ["block", "reset", "monitor", "allow"], "type": "str"},
            "command-code-invalid": {"v_range": [["7.4.2", ""]], "choices": ["block", "reset", "monitor", "allow"], "type": "str"},
            "command-code-range": {"v_range": [["7.4.2", ""]], "type": "str"},
            "comment": {"v_range": [["7.4.2", ""]], "type": "str"},
            "log-packet": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "message-length-invalid": {"v_range": [["7.4.2", ""]], "choices": ["block", "reset", "monitor", "allow"], "type": "str"},
            "missing-request-action": {"v_range": [["7.4.2", ""]], "choices": ["block", "reset", "monitor", "allow"], "type": "str"},
            "monitor-all-messages": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "name": {"v_range": [["7.4.2", ""]], "required": true, "type": "str"},
            "protocol-version-invalid": {"v_range": [["7.4.2", ""]], "choices": ["block", "reset", "monitor", "allow"], "type": "str"},
            "request-error-flag-set": {"v_range": [["7.4.2", ""]], "choices": ["block", "reset", "monitor", "allow"], "type": "str"},
            "track-requests-answers": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "dlp_datatype": {
        "stated": true,
        "options": {
            "comment": {"v_range": [["7.2.0", ""]], "type": "str"},
            "look-ahead": {"v_range": [["7.2.0", ""]], "type": "int"},
            "look-back": {"v_range": [["7.2.0", ""]], "type": "int"},
            "name": {"v_range": [["7.2.0", ""]], "required": true, "type": "str"},
            "pattern": {"v_range": [["7.2.0", ""]], "type": "str"},
            "transform": {"v_range": [["7.2.0", ""]], "type": "str"},
            "verify": {"v_range": [["7.2.0", ""]], "type": "str"},
            "verify-transformed-pattern": {"v_range": [["7.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "match-around": {"v_range": [["7.4.0", ""]], "type": "str"},
            "match-ahead": {"v_range": [["7.4.2", ""]], "type": "int"},
            "match-back": {"v_range": [["7.4.2", ""]], "type": "int"},
            "verify2": {"v_range": [["7.4.2", ""]], "type": "str"},
            "fgd-id": {"v_range": [["7.6.0", ""]], "type": "int"}
        }
    },
    "dlp_dictionary": {
        "stated": true,
        "options": {
            "comment": {"v_range": [["7.2.0", ""]], "type": "str"},
            "entries": {
                "v_range": [["7.2.0", ""]],
                "type": "list",
                "options": {
                    "comment": {"v_range": [["7.2.0", ""]], "type": "str"},
                    "id": {"v_range": [["7.2.0", ""]], "type": "int"},
                    "ignore-case": {"v_range": [["7.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "pattern": {"v_range": [["7.2.0", ""]], "type": "str"},
                    "repeat": {"v_range": [["7.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "status": {"v_range": [["7.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "type": {"v_range": [["7.2.0", ""]], "type": "str"}
                },
                "elements": "dict"
            },
            "match-type": {"v_range": [["7.2.0", ""]], "choices": ["match-all", "match-any"], "type": "str"},
            "name": {"v_range": [["7.2.0", ""]], "required": true, "type": "str"},
            "uuid": {"v_range": [["7.2.0", ""]], "type": "str"},
            "match-around": {"v_range": [["7.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "fgd-id": {"v_range": [["7.6.0", ""]], "type": "int"}
        }
    },
    "dlp_dictionary_entries": {
        "stated": true,
        "options": {
            "comment": {"v_range": [["7.2.0", ""]], "type": "str"},
            "id": {"v_range": [["7.2.0", ""]], "required": true, "type": "int"},
            "ignore-case": {"v_range": [["7.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "pattern": {"v_range": [["7.2.0", ""]], "type": "str"},
            "repeat": {"v_range": [["7.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "status": {"v_range": [["7.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "type": {"v_range": [["7.2.0", ""]], "type": "str"}
        }
    },
    "dlp_filepattern": {
        "stated": true,
        "options": {
            "comment": {"type": "str"},
            "entries": {
                "type": "list",
                "options": {
                    "file-type": {
                        "choices": [
                            "unknown", "ignored", "exe", "elf", "bat", "javascript", "html", "hta", "msoffice", "gzip", "rar", "tar", "lzh", "upx",
                            "zip", "cab", "bzip2", "bzip", "activemime", "mime", "hlp", "arj", "base64", "binhex", "uue", "fsg", "aspack", "msc",
                            "petite", "jpeg", "gif", "tiff", "png", "bmp", "msi", "mpeg", "mov", "mp3", "wma", "wav", "pdf", "avi", "rm", "torrent",
                            "hibun", "7z", "xz", "msofficex", "mach-o", "dmg", ".net", "xar", "chm", "iso", "crx", "sis", "prc", "class", "jad", "cod",
                            "flac", "registry", "hwp", "rpm", "c/cpp", "pfile", "lzip", "wasm", "sylk"
                        ],
                        "type": "str"
                    },
                    "filter-type": {"choices": ["pattern", "type"], "type": "str"},
                    "pattern": {"type": "str"}
                },
                "elements": "dict"
            },
            "id": {"required": true, "type": "int"},
            "name": {"type": "str"}
        }
    },
    "dlp_filepattern_entries": {
        "stated": true,
        "options": {
            "file-type": {
                "choices": [
                    "unknown", "ignored", "exe", "elf", "bat", "javascript", "html", "hta", "msoffice", "gzip", "rar", "tar", "lzh", "upx", "zip", "cab",
                    "bzip2", "bzip", "activemime", "mime", "hlp", "arj", "base64", "binhex", "uue", "fsg", "aspack", "msc", "petite", "jpeg", "gif",
                    "tiff", "png", "bmp", "msi", "mpeg", "mov", "mp3", "wma", "wav", "pdf", "avi", "rm", "torrent", "hibun", "7z", "xz", "msofficex",
                    "mach-o", "dmg", ".net", "xar", "chm", "iso", "crx", "sis", "prc", "class", "jad", "cod", "flac", "registry", "hwp", "rpm", "c/cpp",
                    "pfile", "lzip", "wasm", "sylk"
                ],
                "type": "str"
            },
            "filter-type": {"choices": ["pattern", "type"], "type": "str"},
            "pattern": {"type": "str"}
        }
    },
    "dlp_fpsensitivity": {"stated": true, "options": {"name": {"v_range": [["6.0.0", "7.2.1"]], "required": true, "type": "str"}}},
    "dlp_profile": {
        "stated": true,
        "options": {
            "comment": {"v_range": [["7.2.0", ""]], "type": "str"},
            "dlp-log": {"v_range": [["7.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "extended-log": {"v_range": [["7.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "feature-set": {"v_range": [["7.2.0", ""]], "choices": ["flow", "proxy"], "type": "str"},
            "full-archive-proto": {
                "v_range": [["7.2.0", ""]],
                "type": "list",
                "choices": ["smtp", "pop3", "imap", "http-post", "http-get", "ftp", "nntp", "mapi", "ssh", "cifs"],
                "elements": "str"
            },
            "nac-quar-log": {"v_range": [["7.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "name": {"v_range": [["7.2.0", ""]], "required": true, "type": "str"},
            "replacemsg-group": {"v_range": [["7.2.0", ""]], "type": "str"},
            "rule": {
                "v_range": [["7.2.0", ""]],
                "type": "list",
                "options": {
                    "action": {"v_range": [["7.2.0", ""]], "choices": ["log-only", "block", "quarantine-ip", "allow"], "type": "str"},
                    "archive": {"v_range": [["7.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "expiry": {"v_range": [["7.2.0", ""]], "type": "str"},
                    "file-size": {"v_range": [["7.2.0", ""]], "type": "int"},
                    "file-type": {"v_range": [["7.2.0", ""]], "type": "str"},
                    "filter-by": {"v_range": [["7.2.0", ""]], "choices": ["fingerprint", "sensor", "encrypted", "none", "mip"], "type": "str"},
                    "id": {"v_range": [["7.2.0", ""]], "type": "int"},
                    "label": {"v_range": [["7.2.0", ""]], "type": "str"},
                    "match-percentage": {"v_range": [["7.2.0", ""]], "type": "int"},
                    "name": {"v_range": [["7.2.0", ""]], "type": "str"},
                    "proto": {
                        "v_range": [["7.2.0", ""]],
                        "type": "list",
                        "choices": ["smtp", "pop3", "imap", "http-post", "http-get", "ftp", "nntp", "mapi", "ssh", "cifs"],
                        "elements": "str"
                    },
                    "sensitivity": {"v_range": [["7.2.0", ""]], "type": "raw"},
                    "sensor": {"v_range": [["7.2.0", ""]], "type": "raw"},
                    "severity": {"v_range": [["7.2.0", ""]], "choices": ["info", "low", "medium", "high", "critical"], "type": "str"},
                    "type": {"v_range": [["7.2.0", ""]], "choices": ["file", "message"], "type": "str"}
                },
                "elements": "dict"
            },
            "summary-proto": {
                "v_range": [["7.2.0", ""]],
                "type": "list",
                "choices": ["smtp", "pop3", "imap", "http-post", "http-get", "ftp", "nntp", "mapi", "ssh", "cifs"],
                "elements": "str"
            }
        }
    },
    "dlp_profile_rule": {
        "stated": true,
        "options": {
            "action": {"v_range": [["7.2.0", ""]], "choices": ["log-only", "block", "quarantine-ip", "allow"], "type": "str"},
            "archive": {"v_range": [["7.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "expiry": {"v_range": [["7.2.0", ""]], "type": "str"},
            "file-size": {"v_range": [["7.2.0", ""]], "type": "int"},
            "file-type": {"v_range": [["7.2.0", ""]], "type": "str"},
            "filter-by": {"v_range": [["7.2.0", ""]], "choices": ["fingerprint", "sensor", "encrypted", "none", "mip"], "type": "str"},
            "id": {"v_range": [["7.2.0", ""]], "required": true, "type": "int"},
            "label": {"v_range": [["7.2.0", ""]], "type": "str"},
            "match-percentage": {"v_range": [["7.2.0", ""]], "type": "int"},
            "name": {"v_range": [["7.2.0", ""]], "type": "str"},
            "proto": {
                "v_range": [["7.2.0", ""]],
                "type": "list",
                "choices": ["smtp", "pop3", "imap", "http-post", "http-get", "ftp", "nntp", "mapi", "ssh", "cifs"],
                "elements": "str"
            },
            "sensitivity": {"v_range": [["7.2.0", ""]], "type": "raw"},
            "sensor": {"v_range": [["7.2.0", ""]], "type": "raw"},
            "severity": {"v_range": [["7.2.0", ""]], "choices": ["info", "low", "medium", "high", "critical"], "type": "str"},
            "type": {"v_range": [["7.2.0", ""]], "choices": ["file", "message"], "type": "str"}
        }
    },
    "dlp_sensitivity": {"stated": true, "options": {"name": {"v_range": [["6.2.0", ""]], "required": true, "type": "str"}}},
    "dlp_sensor": {
        "stated": true,
        "options": {
            "comment": {"type": "str"},
            "dlp-log": {"choices": ["disable", "enable"], "type": "str"},
            "extended-log": {"choices": ["disable", "enable"], "type": "str"},
            "filter": {
                "type": "list",
                "options": {
                    "action": {
                        "choices": ["log-only", "block", "exempt", "ban", "ban-sender", "quarantine-ip", "quarantine-port", "none", "allow"],
                        "type": "str"
                    },
                    "archive": {"choices": ["disable", "enable", "summary-only"], "type": "str"},
                    "company-identifier": {"type": "str"},
                    "expiry": {"type": "str"},
                    "file-size": {"type": "int"},
                    "file-type": {"type": "str"},
                    "filter-by": {
                        "choices": [
                            "credit-card", "ssn", "regexp", "file-type", "file-size", "fingerprint", "watermark", "encrypted", "file-type-and-size"
                        ],
                        "type": "str"
                    },
                    "fp-sensitivity": {"v_range": [["6.0.0", "7.2.1"]], "type": "raw"},
                    "id": {"type": "int"},
                    "match-percentage": {"type": "int"},
                    "name": {"type": "str"},
                    "proto": {
                        "type": "list",
                        "choices": [
                            "imap", "smtp", "pop3", "ftp", "nntp", "mm1", "mm3", "mm4", "mm7", "mapi", "aim", "icq", "msn", "yahoo", "http-get",
                            "http-post", "ssh", "cifs"
                        ],
                        "elements": "str"
                    },
                    "regexp": {"type": "str"},
                    "severity": {"choices": ["info", "low", "medium", "high", "critical"], "type": "str"},
                    "type": {"choices": ["file", "message"], "type": "str"},
                    "sensitivity": {"v_range": [["6.2.0", ""]], "type": "raw"}
                },
                "elements": "dict"
            },
            "flow-based": {"v_range": [["6.0.0", "7.2.1"]], "choices": ["disable", "enable"], "type": "str"},
            "full-archive-proto": {
                "type": "list",
                "choices": [
                    "imap", "smtp", "pop3", "ftp", "nntp", "mm1", "mm3", "mm4", "mm7", "mapi", "aim", "icq", "msn", "yahoo", "http-get", "http-post",
                    "ssh", "cifs"
                ],
                "elements": "str"
            },
            "nac-quar-log": {"choices": ["disable", "enable"], "type": "str"},
            "name": {"required": true, "type": "str"},
            "options": {"v_range": [["6.0.0", "7.2.0"]], "choices": ["strict-file"], "type": "str"},
            "replacemsg-group": {"type": "str"},
            "summary-proto": {
                "type": "list",
                "choices": [
                    "imap", "smtp", "pop3", "ftp", "nntp", "mm1", "mm3", "mm4", "mm7", "mapi", "aim", "icq", "msn", "yahoo", "http-get", "http-post",
                    "ssh", "cifs"
                ],
                "elements": "str"
            },
            "feature-set": {"v_range": [["6.4.0", ""]], "choices": ["proxy", "flow"], "type": "str"},
            "entries": {
                "v_range": [["7.2.0", ""]],
                "type": "list",
                "options": {
                    "count": {"v_range": [["7.2.0", ""]], "type": "int"},
                    "dictionary": {"v_range": [["7.2.0", ""]], "type": "str"},
                    "id": {"v_range": [["7.2.0", ""]], "type": "int"},
                    "status": {"v_range": [["7.2.0", ""]], "choices": ["disable", "enable"], "type": "str"}
                },
                "elements": "dict"
            },
            "eval": {"v_range": [["7.2.0", ""]], "type": "str"},
            "match-type": {"v_range": [["7.2.0", ""]], "choices": ["match-all", "match-any", "match-eval"], "type": "str"},
            "fgd-id": {"v_range": [["7.6.0", ""]], "type": "int"}
        }
    },
    "dlp_sensor_entries": {
        "stated": true,
        "options": {
            "count": {"v_range": [["7.2.0", ""]], "type": "int"},
            "dictionary": {"v_range": [["7.2.0", ""]], "type": "str"},
            "id": {"v_range": [["7.2.0", ""]], "required": true, "type": "int"},
            "status": {"v_range": [["7.2.0", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "dlp_sensor_filter": {
        "stated": true,
        "options": {
            "action": {
                "choices": ["log-only", "block", "exempt", "ban", "ban-sender", "quarantine-ip", "quarantine-port", "none", "allow"],
                "type": "str"
            },
            "archive": {"choices": ["disable", "enable", "summary-only"], "type": "str"},
            "company-identifier": {"type": "str"},
            "expiry": {"type": "str"},
            "file-size": {"type": "int"},
            "file-type": {"type": "str"},
            "filter-by": {
                "choices": ["credit-card", "ssn", "regexp", "file-type", "file-size", "fingerprint", "watermark", "encrypted", "file-type-and-size"],
                "type": "str"
            },
            "fp-sensitivity": {"v_range": [["6.0.0", "7.2.1"]], "type": "raw"},
            "id": {"required": true, "type": "int"},
            "match-percentage": {"type": "int"},
            "name": {"type": "str"},
            "proto": {
                "type": "list",
                "choices": [
                    "imap", "smtp", "pop3", "ftp", "nntp", "mm1", "mm3", "mm4", "mm7", "mapi", "aim", "icq", "msn", "yahoo", "http-get", "http-post",
                    "ssh", "cifs"
                ],
                "elements": "str"
            },
            "regexp": {"type": "str"},
            "severity": {"choices": ["info", "low", "medium", "high", "critical"], "type": "str"},
            "type": {"choices": ["file", "message"], "type": "str"},
            "sensitivity": {"v_range": [["6.2.0", ""]], "type": "raw"}
        }
    },
    "dnsfilter_domainfilter": {
        "stated": true,
        "options": {
            "comment": {"type": "str"},
            "entries": {
                "type": "list",
                "options": {
                    "action": {"choices": ["block", "allow", "monitor"], "type": "str"},
                    "domain": {"type": "str"},
                    "id": {"type": "int"},
                    "status": {"choices": ["disable", "enable"], "type": "str"},
                    "type": {"choices": ["simple", "regex", "wildcard"], "type": "str"}
                },
                "elements": "dict"
            },
            "id": {"required": true, "type": "int"},
            "name": {"type": "str"}
        }
    },
    "dnsfilter_domainfilter_entries": {
        "stated": true,
        "options": {
            "action": {"choices": ["block", "allow", "monitor"], "type": "str"},
            "domain": {"type": "str"},
            "id": {"required": true, "type": "int"},
            "status": {"choices": ["disable", "enable"], "type": "str"},
            "type": {"choices": ["simple", "regex", "wildcard"], "type": "str"}
        }
    },
    "dnsfilter_profile": {
        "stated": true,
        "options": {
            "block-action": {"choices": ["block", "redirect", "block-sevrfail"], "type": "str"},
            "block-botnet": {"choices": ["disable", "enable"], "type": "str"},
            "comment": {"type": "str"},
            "external-ip-blocklist": {"type": "raw"},
            "log-all-domain": {"choices": ["disable", "enable"], "type": "str"},
            "name": {"required": true, "type": "str"},
            "redirect-portal": {"type": "str"},
            "safe-search": {"choices": ["disable", "enable"], "type": "str"},
            "sdns-domain-log": {"choices": ["disable", "enable"], "type": "str"},
            "sdns-ftgd-err-log": {"choices": ["disable", "enable"], "type": "str"},
            "youtube-restrict": {"choices": ["strict", "moderate", "none"], "type": "str"},
            "dns-translation": {
                "v_range": [["6.2.0", ""]],
                "type": "list",
                "options": {
                    "dst": {"v_range": [["6.2.0", ""]], "type": "str"},
                    "id": {"v_range": [["6.2.0", ""]], "type": "int"},
                    "netmask": {"v_range": [["6.2.0", ""]], "type": "str"},
                    "src": {"v_range": [["6.2.0", ""]], "type": "str"},
                    "status": {"v_range": [["6.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "addr-type": {"v_range": [["6.2.2", ""]], "choices": ["ipv4", "ipv6"], "type": "str"},
                    "dst6": {"v_range": [["6.2.2", ""]], "type": "str"},
                    "prefix": {"v_range": [["6.2.2", ""]], "type": "int"},
                    "src6": {"v_range": [["6.2.2", ""]], "type": "str"}
                },
                "elements": "dict"
            },
            "redirect-portal6": {"v_range": [["6.2.0", ""]], "type": "str"},
            "log-all-url": {"v_range": [["6.2.0", "6.2.12"]], "choices": ["disable", "enable"], "type": "str"},
            "sdns-url-log": {"v_range": [["6.2.0", "6.2.12"]], "choices": ["disable", "enable"], "type": "str"},
            "domain-filter": {
                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                "type": "dict",
                "options": {"domain-filter-table": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"}}
            },
            "ftgd-dns": {
                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                "type": "dict",
                "options": {
                    "filters": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "type": "list",
                        "options": {
                            "action": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["monitor", "block"], "type": "str"},
                            "category": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                            "id": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                            "log": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"}
                        },
                        "elements": "dict"
                    },
                    "options": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "type": "list",
                        "choices": ["error-allow", "ftgd-disable"],
                        "elements": "str"
                    }
                }
            },
            "urlfilter": {
                "v_range": [["6.2.8", "6.2.12"]],
                "type": "dict",
                "options": {"urlfilter-table": {"v_range": [["6.2.8", "6.2.12"]], "type": "int"}}
            },
            "transparent-dns-database": {"v_range": [["7.4.1", ""]], "type": "raw"},
            "strip-ech": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "dnsfilter_profile_dnstranslation": {
        "stated": true,
        "options": {
            "dst": {"v_range": [["6.2.0", ""]], "type": "str"},
            "id": {"v_range": [["6.2.0", ""]], "required": true, "type": "int"},
            "netmask": {"v_range": [["6.2.0", ""]], "type": "str"},
            "src": {"v_range": [["6.2.0", ""]], "type": "str"},
            "status": {"v_range": [["6.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "addr-type": {"v_range": [["6.2.2", ""]], "choices": ["ipv4", "ipv6"], "type": "str"},
            "dst6": {"v_range": [["6.2.2", ""]], "type": "str"},
            "prefix": {"v_range": [["6.2.2", ""]], "type": "int"},
            "src6": {"v_range": [["6.2.2", ""]], "type": "str"}
        }
    },
    "dnsfilter_profile_domainfilter": {"stated": false, "options": {"domain-filter-table": {"type": "int"}}},
    "dnsfilter_profile_ftgddns": {
        "stated": false,
        "options": {
            "filters": {
                "type": "list",
                "options": {
                    "action": {"choices": ["monitor", "block"], "type": "str"},
                    "category": {"type": "str"},
                    "id": {"type": "int"},
                    "log": {"choices": ["disable", "enable"], "type": "str"}
                },
                "elements": "dict"
            },
            "options": {"type": "list", "choices": ["error-allow", "ftgd-disable"], "elements": "str"}
        }
    },
    "dnsfilter_profile_ftgddns_filters": {
        "stated": true,
        "options": {
            "action": {"choices": ["monitor", "block"], "type": "str"},
            "category": {"type": "str"},
            "id": {"required": true, "type": "int"},
            "log": {"choices": ["disable", "enable"], "type": "str"}
        }
    },
    "dnsfilter_profile_urlfilter": {"stated": false, "options": {"urlfilter-table": {"v_range": [["6.2.0", "6.2.12"]], "type": "int"}}},
    "dnsfilter_urlfilter": {
        "stated": true,
        "options": {
            "comment": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"},
            "entries": {
                "v_range": [["6.2.0", "6.2.12"]],
                "type": "list",
                "options": {
                    "action": {"v_range": [["6.2.0", "6.2.12"]], "choices": ["allow", "monitor", "block"], "type": "str"},
                    "id": {"v_range": [["6.2.0", "6.2.12"]], "type": "int"},
                    "status": {"v_range": [["6.2.0", "6.2.12"]], "choices": ["disable", "enable"], "type": "str"},
                    "type": {"v_range": [["6.2.0", "6.2.12"]], "choices": ["wildcard", "regex", "simple"], "type": "str"},
                    "url": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"}
                },
                "elements": "dict"
            },
            "id": {"v_range": [["6.2.0", "6.2.12"]], "required": true, "type": "int"},
            "name": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"}
        }
    },
    "dnsfilter_urlfilter_entries": {
        "stated": true,
        "options": {
            "action": {"v_range": [["6.2.0", "6.2.12"]], "choices": ["allow", "monitor", "block"], "type": "str"},
            "id": {"v_range": [["6.2.0", "6.2.12"]], "required": true, "type": "int"},
            "status": {"v_range": [["6.2.0", "6.2.12"]], "choices": ["disable", "enable"], "type": "str"},
            "type": {"v_range": [["6.2.0", "6.2.12"]], "choices": ["wildcard", "regex", "simple"], "type": "str"},
            "url": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"}
        }
    },
    "dvm_cmd_add_device": {
        "stated": true,
        "options": {
            "adom": {"type": "str"},
            "device": {
                "type": "dict",
                "options": {
                    "adm_pass": {"no_log": true, "type": "raw"},
                    "adm_usr": {"type": "str"},
                    "desc": {"type": "str"},
                    "device action": {"type": "str"},
                    "faz.quota": {"type": "int"},
                    "ip": {"type": "str"},
                    "meta fields": {"type": "raw"},
                    "mgmt_mode": {"choices": ["unreg", "fmg", "faz", "fmgfaz"], "type": "str"},
                    "mr": {"type": "int"},
                    "name": {"type": "str"},
                    "os_type": {
                        "choices": ["unknown", "fos", "fsw", "foc", "fml", "faz", "fwb", "fch", "fct", "log", "fmg", "fsa", "fdd", "fac", "fpx", "fna"],
                        "type": "str"
                    },
                    "os_ver": {"choices": ["unknown", "0.0", "1.0", "2.0", "3.0", "4.0", "5.0", "6.0", "7.0", "8.0"], "type": "str"},
                    "patch": {"type": "int"},
                    "platform_str": {"type": "str"},
                    "sn": {"type": "str"},
                    "device blueprint": {"v_range": [["7.2.0", ""]], "type": "str"},
                    "authorization template": {"v_range": [["7.4.1", ""]], "type": "str"}
                }
            },
            "flags": {"type": "list", "choices": ["none", "create_task", "nonblocking", "log_dev"], "elements": "str"},
            "groups": {"type": "list", "options": {"name": {"type": "str"}, "vdom": {"type": "str"}}, "elements": "dict"}
        }
    },
    "dvm_cmd_add_devlist": {
        "stated": true,
        "options": {
            "add-dev-list": {
                "type": "list",
                "options": {
                    "adm_pass": {"no_log": true, "type": "raw"},
                    "adm_usr": {"type": "str"},
                    "desc": {"type": "str"},
                    "device action": {"type": "str"},
                    "faz.quota": {"type": "int"},
                    "ip": {"type": "str"},
                    "meta fields": {"type": "raw"},
                    "mgmt_mode": {"choices": ["unreg", "fmg", "faz", "fmgfaz"], "type": "str"},
                    "mr": {"type": "int"},
                    "name": {"type": "str"},
                    "os_type": {
                        "choices": ["unknown", "fos", "fsw", "foc", "fml", "faz", "fwb", "fch", "fct", "log", "fmg", "fsa", "fdd", "fac", "fpx", "fna"],
                        "type": "str"
                    },
                    "os_ver": {"choices": ["unknown", "0.0", "1.0", "2.0", "3.0", "4.0", "5.0", "6.0", "7.0", "8.0"], "type": "str"},
                    "patch": {"type": "int"},
                    "platform_str": {"type": "str"},
                    "sn": {"type": "str"},
                    "device blueprint": {"v_range": [["7.2.0", ""]], "type": "str"},
                    "authorization template": {"v_range": [["7.4.1", ""]], "type": "str"}
                },
                "elements": "dict"
            },
            "adom": {"type": "str"},
            "flags": {"type": "list", "choices": ["none", "create_task", "nonblocking", "log_dev"], "elements": "str"}
        }
    },
    "dvm_cmd_changehaseq": {
        "stated": true,
        "options": {
            "adom": {"type": "str"},
            "device": {"type": "str"},
            "flags": {"type": "list", "choices": ["none", "create_task", "nonblocking", "log_dev"], "elements": "str"},
            "new_master": {"type": "str"}
        }
    },
    "dvm_cmd_del_device": {
        "stated": true,
        "options": {
            "adom": {"type": "str"},
            "device": {"type": "str"},
            "flags": {"type": "list", "choices": ["none", "create_task", "nonblocking", "log_dev"], "elements": "str"}
        }
    },
    "dvm_cmd_del_devlist": {
        "stated": true,
        "options": {
            "adom": {"type": "str"},
            "del-dev-member-list": {"type": "list", "options": {"name": {"type": "str"}, "vdom": {"type": "str"}}, "elements": "dict"},
            "flags": {"type": "list", "choices": ["none", "create_task", "nonblocking", "log_dev"], "elements": "str"}
        }
    },
    "dvm_cmd_discover_device": {
        "stated": true,
        "options": {
            "device": {"type": "dict", "options": {"adm_pass": {"no_log": true, "type": "str"}, "adm_usr": {"type": "str"}, "ip": {"type": "str"}}}
        }
    },
    "dvm_cmd_import_devlist": {
        "stated": true,
        "options": {
            "adom": {"type": "str"},
            "flags": {"type": "list", "choices": ["none", "create_task", "nonblocking", "log_dev"], "elements": "str"},
            "import-adom-members": {
                "type": "list",
                "options": {"adom": {"type": "str"}, "dev": {"type": "str"}, "vdom": {"type": "str"}},
                "elements": "dict"
            },
            "import-adoms": {
                "type": "list",
                "options": {
                    "desc": {"type": "str"},
                    "flags": {
                        "type": "list",
                        "choices": [
                            "migration", "db_export", "no_vpn_console", "backup", "other_devices", "central_sdwan", "is_autosync", "per_device_wtp",
                            "policy_check_on_install", "install_on_policy_check_fail", "auto_push_cfg", "per_device_fsw", "install_deselect_all"
                        ],
                        "elements": "str"
                    },
                    "log_db_retention_hours": {"type": "int"},
                    "log_disk_quota": {"type": "int"},
                    "log_disk_quota_alert_thres": {"type": "int"},
                    "log_disk_quota_split_ratio": {"type": "int"},
                    "log_file_retention_hours": {"type": "int"},
                    "meta fields": {"type": "dict"},
                    "mig_mr": {"type": "int"},
                    "mig_os_ver": {"choices": ["unknown", "0.0", "1.0", "2.0", "3.0", "4.0", "5.0", "6.0", "7.0", "8.0", "9.0"], "type": "str"},
                    "mode": {"choices": ["ems", "gms", "provider"], "type": "str"},
                    "mr": {"type": "int"},
                    "name": {"type": "str"},
                    "os_ver": {"choices": ["unknown", "0.0", "1.0", "2.0", "3.0", "4.0", "5.0", "6.0", "7.0", "8.0", "9.0"], "type": "str"},
                    "restricted_prds": {
                        "type": "raw",
                        "choices": [
                            "fos", "foc", "fml", "fch", "fwb", "log", "fct", "faz", "fsa", "fsw", "fmg", "fdd", "fac", "fpx", "fna", "fdc", "ffw", "fsr",
                            "fad", "fap", "fxt", "fts", "fai", "fwc", "fis", "fed", "fabric", "fpa", "fca", "ftc", "fss"
                        ]
                    },
                    "state": {"type": "int"},
                    "uuid": {"type": "str"},
                    "create_time": {"v_range": [["6.4.1", ""]], "type": "int"},
                    "workspace_mode": {"v_range": [["6.4.3", ""]], "type": "int"},
                    "tz": {"v_range": [["7.4.0", ""]], "type": "int"},
                    "lock_override": {"v_range": [["7.4.1", ""]], "type": "int"},
                    "primary_dns_ip4": {"v_range": [["7.4.3", ""]], "type": "str"},
                    "primary_dns_ip6_1": {"v_range": [["7.4.3", ""]], "type": "int"},
                    "primary_dns_ip6_2": {"v_range": [["7.4.3", ""]], "type": "int"},
                    "primary_dns_ip6_3": {"v_range": [["7.4.3", ""]], "type": "int"},
                    "primary_dns_ip6_4": {"v_range": [["7.4.3", ""]], "type": "int"},
                    "secondary_dns_ip4": {"v_range": [["7.4.3", ""]], "type": "str"},
                    "secondary_dns_ip6_1": {"v_range": [["7.4.3", ""]], "type": "int"},
                    "secondary_dns_ip6_2": {"v_range": [["7.4.3", ""]], "type": "int"},
                    "secondary_dns_ip6_3": {"v_range": [["7.4.3", ""]], "type": "int"},
                    "secondary_dns_ip6_4": {"v_range": [["7.4.3", ""]], "type": "int"}
                },
                "elements": "dict"
            },
            "import-devices": {
                "type": "list",
                "options": {
                    "adm_pass": {"no_log": true, "type": "raw"},
                    "adm_usr": {"type": "str"},
                    "app_ver": {"type": "str"},
                    "av_ver": {"type": "str"},
                    "beta": {"type": "int"},
                    "branch_pt": {"type": "int"},
                    "build": {"type": "int"},
                    "checksum": {"type": "str"},
                    "conf_status": {"choices": ["unknown", "insync", "outofsync"], "type": "str"},
                    "conn_mode": {"choices": ["active", "passive"], "type": "str"},
                    "conn_status": {"choices": ["UNKNOWN", "up", "down"], "type": "str"},
                    "db_status": {"choices": ["unknown", "nomod", "mod"], "type": "str"},
                    "desc": {"type": "str"},
                    "dev_status": {
                        "choices": [
                            "none", "unknown", "checkedin", "inprogress", "installed", "aborted", "sched", "retry", "canceled", "pending", "retrieved",
                            "changed_conf", "sync_fail", "timeout", "rev_revert", "auto_updated"
                        ],
                        "type": "str"
                    },
                    "fap_cnt": {"type": "int"},
                    "faz.full_act": {"type": "int"},
                    "faz.perm": {"type": "int"},
                    "faz.quota": {"type": "int"},
                    "faz.used": {"type": "int"},
                    "fex_cnt": {"type": "int"},
                    "flags": {
                        "type": "list",
                        "choices": [
                            "has_hdd", "vdom_enabled", "discover", "reload", "interim_build", "offline_mode", "is_model", "fips_mode", "linked_to_model",
                            "ip-conflict", "faz-autosync", "need_reset", "backup_mode", "azure_vwan_nva", "fgsp_configured", "cnf_mode", "sase_managed",
                            "override_management_intf", "sdwan_management", "deny_api_access"
                        ],
                        "elements": "str"
                    },
                    "foslic_cpu": {"type": "int"},
                    "foslic_dr_site": {"choices": ["disable", "enable"], "type": "str"},
                    "foslic_inst_time": {"type": "int"},
                    "foslic_last_sync": {"type": "int"},
                    "foslic_ram": {"type": "int"},
                    "foslic_type": {"choices": ["temporary", "trial", "regular", "trial_expired"], "type": "str"},
                    "foslic_utm": {"type": "list", "choices": ["fw", "av", "ips", "app", "url", "utm", "fwb"], "elements": "str"},
                    "fsw_cnt": {"type": "int"},
                    "ha_group_id": {"type": "int"},
                    "ha_group_name": {"type": "str"},
                    "ha_mode": {"choices": ["standalone", "AP", "AA", "ELBC", "DUAL", "enabled", "unknown", "fmg-enabled", "autoscale"], "type": "str"},
                    "ha_slave": {
                        "type": "list",
                        "options": {
                            "idx": {"type": "int"},
                            "name": {"type": "str"},
                            "prio": {"type": "int"},
                            "role": {"choices": ["slave", "master"], "type": "str"},
                            "sn": {"type": "str"},
                            "status": {"type": "int"},
                            "conf_status": {"v_range": [["7.0.10", "7.0.12"], ["7.2.1", ""]], "type": "int"}
                        },
                        "elements": "dict"
                    },
                    "hdisk_size": {"type": "int"},
                    "hostname": {"type": "str"},
                    "hw_rev_major": {"type": "int"},
                    "hw_rev_minor": {"type": "int"},
                    "ip": {"type": "str"},
                    "ips_ext": {"type": "int"},
                    "ips_ver": {"type": "str"},
                    "last_checked": {"type": "int"},
                    "last_resync": {"type": "int"},
                    "latitude": {"type": "str"},
                    "lic_flags": {"type": "int"},
                    "lic_region": {"type": "str"},
                    "location_from": {"type": "str"},
                    "logdisk_size": {"type": "int"},
                    "longitude": {"type": "str"},
                    "maxvdom": {"type": "int"},
                    "meta fields": {"type": "dict"},
                    "mgmt_id": {"v_range": [["6.0.0", "7.2.0"]], "type": "int"},
                    "mgmt_if": {"type": "str"},
                    "mgmt_mode": {"choices": ["unreg", "fmg", "faz", "fmgfaz"], "type": "str"},
                    "mgt_vdom": {"type": "str"},
                    "mr": {"type": "int"},
                    "name": {"type": "str"},
                    "os_type": {
                        "choices": [
                            "unknown", "fos", "fsw", "foc", "fml", "faz", "fwb", "fch", "fct", "log", "fmg", "fsa", "fdd", "fac", "fpx", "fna", "fdc",
                            "ffw", "fsr", "fad", "fap", "fxt", "fts", "fai", "fwc", "fis", "fed", "fpa", "fca", "ftc", "fss"
                        ],
                        "type": "str"
                    },
                    "os_ver": {"choices": ["unknown", "0.0", "1.0", "2.0", "3.0", "4.0", "5.0", "6.0", "7.0", "8.0", "9.0"], "type": "str"},
                    "patch": {"type": "int"},
                    "platform_str": {"type": "str"},
                    "psk": {"type": "str"},
                    "sn": {"type": "str"},
                    "vdom": {
                        "type": "list",
                        "options": {
                            "comments": {"type": "str"},
                            "name": {"type": "str"},
                            "opmode": {"choices": ["nat", "transparent"], "type": "str"},
                            "rtm_prof_id": {"type": "int"},
                            "status": {"type": "str"},
                            "vpn_id": {"v_range": [["6.2.2", ""]], "type": "int"},
                            "meta fields": {"v_range": [["6.4.3", ""]], "type": "dict"},
                            "vdom_type": {"v_range": [["7.2.0", ""]], "choices": ["traffic", "admin"], "type": "str"}
                        },
                        "elements": "dict"
                    },
                    "version": {"type": "int"},
                    "vm_cpu": {"type": "int"},
                    "vm_cpu_limit": {"type": "int"},
                    "vm_lic_expire": {"type": "int"},
                    "vm_mem": {"type": "int"},
                    "vm_mem_limit": {"type": "int"},
                    "vm_status": {"type": "raw"},
                    "module_sn": {"v_range": [["6.2.1", ""]], "type": "str"},
                    "prefer_img_ver": {"v_range": [["6.2.1", ""]], "type": "str"},
                    "prio": {"v_range": [["6.4.1", ""]], "type": "int"},
                    "role": {"v_range": [["6.4.1", ""]], "choices": ["master", "ha-slave", "autoscale-slave"], "type": "str"},
                    "hyperscale": {"v_range": [["6.2.7", "6.2.12"], ["6.4.3", ""]], "type": "int"},
                    "nsxt_service_name": {"v_range": [["6.4.4", ""]], "type": "str"},
                    "private_key": {"v_range": [["6.2.7", "6.2.12"], ["6.4.4", ""]], "no_log": true, "type": "str"},
                    "private_key_status": {"v_range": [["6.2.7", "6.2.12"], ["6.4.4", ""]], "no_log": true, "type": "int"},
                    "vm_lic_overdue_since": {"v_range": [["6.4.12", "6.4.14"], ["7.0.8", "7.0.12"], ["7.2.3", ""]], "type": "int"},
                    "first_tunnel_up": {"v_range": [["7.0.4", "7.0.12"], ["7.2.1", ""]], "type": "int"},
                    "eip": {"v_range": [["7.2.1", ""]], "type": "str"},
                    "mgmt_uuid": {"v_range": [["7.2.1", ""]], "type": "str"},
                    "hw_generation": {"v_range": [["7.2.4", "7.2.5"], ["7.4.1", ""]], "type": "int"},
                    "relver_info": {"v_range": [["7.4.3", ""]], "type": "str"},
                    "cluster_worker": {"v_range": [["7.6.0", ""]], "type": "str"},
                    "ha.vsn": {"v_range": [["7.6.0", ""]], "type": "str"},
                    "ha_upgrade_mode": {"v_range": [["7.6.0", ""]], "type": "int"}
                },
                "elements": "dict"
            },
            "import-group-members": {
                "type": "list",
                "options": {"adom": {"type": "str"}, "dev": {"type": "str"}, "grp": {"type": "str"}, "vdom": {"type": "str"}},
                "elements": "dict"
            }
        }
    },
    "dvm_cmd_reload_devlist": {
        "stated": true,
        "options": {
            "adom": {"type": "str"},
            "flags": {"type": "list", "choices": ["none", "create_task", "nonblocking", "log_dev"], "elements": "str"},
            "from": {"choices": ["um", "fgfm", "apache", "dvm", "fwm", "xml", "json"], "type": "str"},
            "reload-dev-member-list": {"type": "list", "options": {"name": {"type": "str"}, "vdom": {"type": "str"}}, "elements": "dict"},
            "tag": {"type": "str"}
        }
    },
    "dvm_cmd_update_device": {
        "stated": true,
        "options": {
            "adom": {"type": "str"},
            "device": {"type": "str"},
            "flags": {"type": "list", "choices": ["none", "create_task", "nonblocking", "log_dev"], "elements": "str"}
        }
    },
    "dvm_cmd_update_devlist": {
        "stated": true,
        "options": {
            "adom": {"type": "str"},
            "flags": {"type": "list", "choices": ["none", "create_task", "nonblocking", "log_dev"], "elements": "str"},
            "update-dev-member-list": {"type": "list", "options": {"name": {"type": "str"}, "vdom": {"type": "str"}}, "elements": "dict"}
        }
    },
    "dvmdb_adom": {
        "stated": true,
        "options": {
            "desc": {"type": "str"},
            "flags": {
                "type": "list",
                "choices": [
                    "migration", "db_export", "no_vpn_console", "backup", "other_devices", "central_sdwan", "is_autosync", "per_device_wtp",
                    "policy_check_on_install", "install_on_policy_check_fail", "auto_push_cfg", "per_device_fsw", "install_deselect_all"
                ],
                "elements": "str"
            },
            "log_db_retention_hours": {"type": "int"},
            "log_disk_quota": {"type": "int"},
            "log_disk_quota_alert_thres": {"type": "int"},
            "log_disk_quota_split_ratio": {"type": "int"},
            "log_file_retention_hours": {"type": "int"},
            "meta fields": {"type": "dict"},
            "mig_mr": {"type": "int"},
            "mig_os_ver": {"choices": ["unknown", "0.0", "1.0", "2.0", "3.0", "4.0", "5.0", "6.0", "7.0", "8.0", "9.0"], "type": "str"},
            "mode": {"choices": ["ems", "gms", "provider"], "type": "str"},
            "mr": {"type": "int"},
            "name": {"required": true, "type": "str"},
            "os_ver": {"choices": ["unknown", "0.0", "1.0", "2.0", "3.0", "4.0", "5.0", "6.0", "7.0", "8.0", "9.0"], "type": "str"},
            "restricted_prds": {
                "type": "raw",
                "choices": [
                    "fos", "foc", "fml", "fch", "fwb", "log", "fct", "faz", "fsa", "fsw", "fmg", "fdd", "fac", "fpx", "fna", "fdc", "ffw", "fsr", "fad",
                    "fap", "fxt", "fts", "fai", "fwc", "fis", "fed", "fabric", "fpa", "fca", "ftc", "fss"
                ]
            },
            "state": {"type": "int"},
            "uuid": {"type": "str"},
            "create_time": {"v_range": [["6.4.1", ""]], "type": "int"},
            "workspace_mode": {"v_range": [["6.4.3", ""]], "type": "int"},
            "tz": {"v_range": [["7.4.0", ""]], "type": "int"},
            "lock_override": {"v_range": [["7.4.1", ""]], "type": "int"},
            "primary_dns_ip4": {"v_range": [["7.4.3", ""]], "type": "str"},
            "primary_dns_ip6_1": {"v_range": [["7.4.3", ""]], "type": "int"},
            "primary_dns_ip6_2": {"v_range": [["7.4.3", ""]], "type": "int"},
            "primary_dns_ip6_3": {"v_range": [["7.4.3", ""]], "type": "int"},
            "primary_dns_ip6_4": {"v_range": [["7.4.3", ""]], "type": "int"},
            "secondary_dns_ip4": {"v_range": [["7.4.3", ""]], "type": "str"},
            "secondary_dns_ip6_1": {"v_range": [["7.4.3", ""]], "type": "int"},
            "secondary_dns_ip6_2": {"v_range": [["7.4.3", ""]], "type": "int"},
            "secondary_dns_ip6_3": {"v_range": [["7.4.3", ""]], "type": "int"},
            "secondary_dns_ip6_4": {"v_range": [["7.4.3", ""]], "type": "int"}
        }
    },
    "dvmdb_adom_objectmember": {"stated": true, "options": {"name": {"type": "str"}, "vdom": {"type": "str"}}},
    "dvmdb_device": {
        "stated": false,
        "options": {
            "adm_pass": {"no_log": true, "type": "raw"},
            "adm_usr": {"type": "str"},
            "app_ver": {"type": "str"},
            "av_ver": {"type": "str"},
            "beta": {"type": "int"},
            "branch_pt": {"type": "int"},
            "build": {"type": "int"},
            "checksum": {"type": "str"},
            "conf_status": {"choices": ["unknown", "insync", "outofsync"], "type": "str"},
            "conn_mode": {"choices": ["active", "passive"], "type": "str"},
            "conn_status": {"choices": ["UNKNOWN", "up", "down"], "type": "str"},
            "db_status": {"choices": ["unknown", "nomod", "mod"], "type": "str"},
            "desc": {"type": "str"},
            "dev_status": {
                "choices": [
                    "none", "unknown", "checkedin", "inprogress", "installed", "aborted", "sched", "retry", "canceled", "pending", "retrieved",
                    "changed_conf", "sync_fail", "timeout", "rev_revert", "auto_updated"
                ],
                "type": "str"
            },
            "fap_cnt": {"type": "int"},
            "faz.full_act": {"type": "int"},
            "faz.perm": {"type": "int"},
            "faz.quota": {"type": "int"},
            "faz.used": {"type": "int"},
            "fex_cnt": {"type": "int"},
            "flags": {
                "type": "list",
                "choices": [
                    "has_hdd", "vdom_enabled", "discover", "reload", "interim_build", "offline_mode", "is_model", "fips_mode", "linked_to_model",
                    "ip-conflict", "faz-autosync", "need_reset", "backup_mode", "azure_vwan_nva", "fgsp_configured", "cnf_mode", "sase_managed",
                    "override_management_intf", "sdwan_management", "deny_api_access"
                ],
                "elements": "str"
            },
            "foslic_cpu": {"type": "int"},
            "foslic_dr_site": {"choices": ["disable", "enable"], "type": "str"},
            "foslic_inst_time": {"type": "int"},
            "foslic_last_sync": {"type": "int"},
            "foslic_ram": {"type": "int"},
            "foslic_type": {"choices": ["temporary", "trial", "regular", "trial_expired"], "type": "str"},
            "foslic_utm": {"type": "list", "choices": ["fw", "av", "ips", "app", "url", "utm", "fwb"], "elements": "str"},
            "fsw_cnt": {"type": "int"},
            "ha_group_id": {"type": "int"},
            "ha_group_name": {"type": "str"},
            "ha_mode": {"choices": ["standalone", "AP", "AA", "ELBC", "DUAL", "enabled", "unknown", "fmg-enabled", "autoscale"], "type": "str"},
            "ha_slave": {
                "type": "list",
                "options": {
                    "idx": {"type": "int"},
                    "name": {"type": "str"},
                    "prio": {"type": "int"},
                    "role": {"choices": ["slave", "master"], "type": "str"},
                    "sn": {"type": "str"},
                    "status": {"type": "int"},
                    "conf_status": {"v_range": [["7.0.10", "7.0.12"], ["7.2.1", ""]], "type": "int"}
                },
                "elements": "dict"
            },
            "hdisk_size": {"type": "int"},
            "hostname": {"type": "str"},
            "hw_rev_major": {"type": "int"},
            "hw_rev_minor": {"type": "int"},
            "ip": {"type": "str"},
            "ips_ext": {"type": "int"},
            "ips_ver": {"type": "str"},
            "last_checked": {"type": "int"},
            "last_resync": {"type": "int"},
            "latitude": {"type": "str"},
            "lic_flags": {"type": "int"},
            "lic_region": {"type": "str"},
            "location_from": {"type": "str"},
            "logdisk_size": {"type": "int"},
            "longitude": {"type": "str"},
            "maxvdom": {"type": "int"},
            "meta fields": {"type": "dict"},
            "mgmt_id": {"v_range": [["6.0.0", "7.2.0"]], "type": "int"},
            "mgmt_if": {"type": "str"},
            "mgmt_mode": {"choices": ["unreg", "fmg", "faz", "fmgfaz"], "type": "str"},
            "mgt_vdom": {"type": "str"},
            "mr": {"type": "int"},
            "name": {"type": "str"},
            "os_type": {
                "choices": [
                    "unknown", "fos", "fsw", "foc", "fml", "faz", "fwb", "fch", "fct", "log", "fmg", "fsa", "fdd", "fac", "fpx", "fna", "fdc", "ffw",
                    "fsr", "fad", "fap", "fxt", "fts", "fai", "fwc", "fis", "fed", "fpa", "fca", "ftc", "fss"
                ],
                "type": "str"
            },
            "os_ver": {"choices": ["unknown", "0.0", "1.0", "2.0", "3.0", "4.0", "5.0", "6.0", "7.0", "8.0", "9.0"], "type": "str"},
            "patch": {"type": "int"},
            "platform_str": {"type": "str"},
            "psk": {"type": "str"},
            "sn": {"type": "str"},
            "vdom": {
                "type": "list",
                "options": {
                    "comments": {"type": "str"},
                    "name": {"type": "str"},
                    "opmode": {"choices": ["nat", "transparent"], "type": "str"},
                    "rtm_prof_id": {"type": "int"},
                    "status": {"type": "str"},
                    "vpn_id": {"v_range": [["6.2.2", ""]], "type": "int"},
                    "meta fields": {"v_range": [["6.4.3", ""]], "type": "dict"},
                    "vdom_type": {"v_range": [["7.2.0", ""]], "choices": ["traffic", "admin"], "type": "str"}
                },
                "elements": "dict"
            },
            "version": {"type": "int"},
            "vm_cpu": {"type": "int"},
            "vm_cpu_limit": {"type": "int"},
            "vm_lic_expire": {"type": "int"},
            "vm_mem": {"type": "int"},
            "vm_mem_limit": {"type": "int"},
            "vm_status": {"type": "raw"},
            "module_sn": {"v_range": [["6.2.1", ""]], "type": "str"},
            "prefer_img_ver": {"v_range": [["6.2.1", ""]], "type": "str"},
            "prio": {"v_range": [["6.4.1", ""]], "type": "int"},
            "role": {"v_range": [["6.4.1", ""]], "choices": ["master", "ha-slave", "autoscale-slave"], "type": "str"},
            "hyperscale": {"v_range": [["6.2.7", "6.2.12"], ["6.4.3", ""]], "type": "int"},
            "nsxt_service_name": {"v_range": [["6.4.4", ""]], "type": "str"},
            "private_key": {"v_range": [["6.2.7", "6.2.12"], ["6.4.4", ""]], "no_log": true, "type": "str"},
            "private_key_status": {"v_range": [["6.2.7", "6.2.12"], ["6.4.4", ""]], "no_log": true, "type": "int"},
            "vm_lic_overdue_since": {"v_range": [["6.4.12", "6.4.14"], ["7.0.8", "7.0.12"], ["7.2.3", ""]], "type": "int"},
            "first_tunnel_up": {"v_range": [["7.0.4", "7.0.12"], ["7.2.1", ""]], "type": "int"},
            "eip": {"v_range": [["7.2.1", ""]], "type": "str"},
            "mgmt_uuid": {"v_range": [["7.2.1", ""]], "type": "str"},
            "hw_generation": {"v_range": [["7.2.4", "7.2.5"], ["7.4.1", ""]], "type": "int"},
            "relver_info": {"v_range": [["7.4.3", ""]], "type": "str"},
            "cluster_worker": {"v_range": [["7.6.0", ""]], "type": "str"},
            "ha.vsn": {"v_range": [["7.6.0", ""]], "type": "str"},
            "ha_upgrade_mode": {"v_range": [["7.6.0", ""]], "type": "int"}
        }
    },
    "dvmdb_device_replace_sn": {"stated": true, "options": {"sn": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "str"}}},
    "dvmdb_device_vdom": {
        "stated": true,
        "options": {
            "comments": {"type": "str"},
            "name": {"required": true, "type": "str"},
            "opmode": {"choices": ["nat", "transparent"], "type": "str"},
            "rtm_prof_id": {"type": "int"},
            "status": {"type": "str"},
            "vpn_id": {"v_range": [["6.2.2", ""]], "type": "int"},
            "meta fields": {"v_range": [["6.4.3", ""]], "type": "dict"},
            "vdom_type": {"v_range": [["7.2.0", ""]], "choices": ["traffic", "admin"], "type": "str"}
        }
    },
    "dvmdb_folder": {
        "stated": true,
        "options": {
            "desc": {"v_range": [["6.4.2", ""]], "type": "str"},
            "name": {"v_range": [["6.4.2", ""]], "required": true, "type": "str"},
            "parent": {"v_range": [["6.4.2", ""]], "type": "int"}
        }
    },
    "dvmdb_group": {
        "stated": true,
        "options": {
            "desc": {"type": "str"},
            "meta fields": {"type": "dict"},
            "name": {"required": true, "type": "str"},
            "os_type": {
                "choices": [
                    "unknown", "fos", "fsw", "foc", "fml", "faz", "fwb", "fch", "fct", "log", "fmg", "fsa", "fdd", "fac", "fpx", "fna", "fdc", "ffw",
                    "fsr", "fad", "fap", "fxt", "fts", "fai", "fwc", "fis", "fed", "fpa", "fca", "ftc", "fss"
                ],
                "type": "str"
            },
            "type": {"choices": ["normal", "default", "auto", "cluster", "fabric"], "type": "str"},
            "id": {"v_range": [["7.2.1", ""]], "type": "str"},
            "cluster_type": {"v_range": [["7.2.2", ""]], "choices": ["unknown", "vwan", "sase"], "type": "str"}
        }
    },
    "dvmdb_group_objectmember": {"stated": true, "options": {"name": {"type": "str"}, "vdom": {"type": "str"}}},
    "dvmdb_metafields_adom": {
        "stated": false,
        "options": {
            "importance": {"choices": ["optional", "required"], "type": "str"},
            "length": {"type": "int"},
            "name": {"type": "str"},
            "status": {"choices": ["disable", "enable"], "type": "str"}
        }
    },
    "dvmdb_metafields_device": {
        "stated": false,
        "options": {
            "importance": {"choices": ["optional", "required"], "type": "str"},
            "length": {"type": "int"},
            "name": {"type": "str"},
            "status": {"choices": ["disable", "enable"], "type": "str"}
        }
    },
    "dvmdb_metafields_group": {
        "stated": false,
        "options": {
            "importance": {"choices": ["optional", "required"], "type": "str"},
            "length": {"type": "int"},
            "name": {"type": "str"},
            "status": {"choices": ["disable", "enable"], "type": "str"}
        }
    },
    "dvmdb_revision": {
        "stated": true,
        "options": {
            "created_by": {"type": "str"},
            "created_time": {"type": "int"},
            "desc": {"type": "str"},
            "locked": {"type": "int"},
            "name": {"required": true, "type": "str"},
            "version": {"type": "int"}
        }
    },
    "dvmdb_script": {
        "stated": true,
        "options": {
            "content": {"type": "str"},
            "desc": {"type": "str"},
            "filter_build": {"type": "int"},
            "filter_device": {"type": "int"},
            "filter_hostname": {"type": "str"},
            "filter_ostype": {"choices": ["unknown", "fos"], "type": "str"},
            "filter_osver": {"choices": ["unknown", "4.00", "5.00", "6.00"], "type": "str"},
            "filter_platform": {"type": "str"},
            "filter_serial": {"type": "str"},
            "modification_time": {"type": "str"},
            "name": {"required": true, "type": "str"},
            "script_schedule": {
                "type": "list",
                "options": {
                    "datetime": {"type": "str"},
                    "day_of_week": {"choices": ["unknown", "sun", "mon", "tue", "wed", "thu", "fri", "sat"], "type": "str"},
                    "device": {"type": "int"},
                    "name": {"type": "str"},
                    "run_on_db": {"choices": ["disable", "enable"], "type": "str"},
                    "type": {"choices": ["auto", "onetime", "daily", "weekly", "monthly"], "type": "str"}
                },
                "elements": "dict"
            },
            "target": {"choices": ["device_database", "remote_device", "adom_database"], "type": "str"},
            "type": {"choices": ["cli", "tcl", "cligrp"], "type": "str"}
        }
    },
    "dvmdb_script_execute": {
        "stated": true,
        "options": {
            "adom": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "str"},
            "package": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "str"},
            "scope": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "type": "list",
                "options": {
                    "name": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "str"},
                    "vdom": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "str"}
                },
                "elements": "dict"
            },
            "script": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "str"},
            "pblock": {"v_range": [["7.0.10", "7.0.12"], ["7.2.5", "7.2.5"], ["7.4.2", ""]], "type": "str"}
        }
    },
    "dvmdb_script_objectmember": {"stated": true, "options": {"name": {"type": "str"}, "vdom": {"type": "str"}}},
    "dvmdb_script_scriptschedule": {
        "stated": true,
        "options": {
            "datetime": {"type": "str"},
            "day_of_week": {"choices": ["unknown", "sun", "mon", "tue", "wed", "thu", "fri", "sat"], "type": "str"},
            "device": {"type": "int"},
            "name": {"required": true, "type": "str"},
            "run_on_db": {"choices": ["disable", "enable"], "type": "str"},
            "type": {"choices": ["auto", "onetime", "daily", "weekly", "monthly"], "type": "str"}
        }
    },
    "dvmdb_upgrade": {
        "stated": true,
        "options": {
            "scope member": {
                "v_range": [["7.4.1", ""]],
                "type": "list",
                "options": {"name": {"v_range": [["7.4.1", ""]], "type": "str"}, "vdom": {"v_range": [["7.4.1", ""]], "type": "str"}},
                "elements": "dict"
            }
        }
    },
    "dvmdb_workflow_approve": {"stated": true, "options": {"desc": {"type": "str"}}},
    "dvmdb_workflow_discard": {"stated": true, "options": {}},
    "dvmdb_workflow_drop": {"stated": true, "options": {}},
    "dvmdb_workflow_reject": {"stated": true, "options": {"desc": {"type": "str"}}},
    "dvmdb_workflow_repair": {"stated": true, "options": {"desc": {"type": "str"}}},
    "dvmdb_workflow_revert": {
        "stated": true,
        "options": {
            "desc": {"type": "str"},
            "fmgip": {"type": "str"},
            "mail_user": {"type": "raw"},
            "no_diff": {"type": "int"},
            "no_mail": {"type": "int"}
        }
    },
    "dvmdb_workflow_review": {"stated": true, "options": {}},
    "dvmdb_workflow_save": {"stated": true, "options": {}},
    "dvmdb_workflow_start": {"stated": true, "options": {"desc": {"type": "str"}, "name": {"type": "str"}}},
    "dvmdb_workflow_submit": {
        "stated": true,
        "options": {
            "desc": {"type": "str"},
            "fmgip": {"type": "str"},
            "mail_user": {"type": "raw"},
            "no_diff": {"type": "int"},
            "no_mail": {"type": "int"}
        }
    },
    "dvmdb_workspace_commit": {"stated": true, "options": {}},
    "dvmdb_workspace_commit_dev": {"stated": true, "options": {}},
    "dvmdb_workspace_commit_obj": {"stated": true, "options": {}},
    "dvmdb_workspace_commit_pkg": {"stated": true, "options": {}},
    "dvmdb_workspace_lock": {"stated": true, "options": {}},
    "dvmdb_workspace_lock_dev": {"stated": true, "options": {}},
    "dvmdb_workspace_lock_obj": {"stated": true, "options": {}},
    "dvmdb_workspace_lock_pkg": {"stated": true, "options": {}},
    "dvmdb_workspace_unlock": {"stated": true, "options": {}},
    "dvmdb_workspace_unlock_dev": {"stated": true, "options": {}},
    "dvmdb_workspace_unlock_obj": {"stated": true, "options": {}},
    "dvmdb_workspace_unlock_pkg": {"stated": true, "options": {}},
    "dynamic_address": {
        "stated": true,
        "options": {
            "default": {"type": "raw"},
            "description": {"type": "str"},
            "dynamic_addr_mapping": {"type": "list", "options": {"addr": {"type": "str"}, "id": {"type": "int"}}, "elements": "dict"},
            "name": {"required": true, "type": "str"}
        }
    },
    "dynamic_address_dynamicaddrmapping": {"stated": true, "options": {"addr": {"type": "str"}, "id": {"required": true, "type": "int"}}},
    "dynamic_certificate_local": {
        "stated": true,
        "options": {
            "description": {"type": "str"},
            "dynamic_mapping": {
                "type": "list",
                "options": {
                    "_scope": {"type": "list", "options": {"name": {"type": "str"}, "vdom": {"type": "str"}}, "elements": "dict"},
                    "local-cert": {"type": "str"}
                },
                "elements": "dict"
            },
            "name": {"required": true, "type": "str"}
        }
    },
    "dynamic_certificate_local_dynamicmapping": {
        "stated": true,
        "options": {
            "_scope": {"type": "list", "options": {"name": {"type": "str"}, "vdom": {"type": "str"}}, "elements": "dict"},
            "local-cert": {"type": "str"}
        }
    },
    "dynamic_input_interface": {
        "stated": true,
        "options": {
            "default-mapping": {"v_range": [["6.2.2", "6.4.0"]], "choices": ["disable", "enable"], "type": "str"},
            "defmap-intf": {"v_range": [["6.2.2", "6.4.0"]], "type": "str"},
            "description": {"v_range": [["6.2.2", "6.4.0"]], "type": "str"},
            "dynamic_mapping": {
                "v_range": [["6.2.2", "6.4.0"]],
                "type": "list",
                "options": {
                    "_scope": {
                        "v_range": [["6.2.2", "6.4.0"]],
                        "type": "list",
                        "options": {"name": {"v_range": [["6.2.2", "6.4.0"]], "type": "str"}, "vdom": {"v_range": [["6.2.2", "6.4.0"]], "type": "str"}},
                        "elements": "dict"
                    },
                    "local-intf": {"v_range": [["6.2.2", "6.4.0"]], "type": "str"}
                },
                "elements": "dict"
            },
            "name": {"v_range": [["6.2.2", "6.4.0"]], "required": true, "type": "str"},
            "skip-unmapped": {"v_range": [["6.2.2", "6.4.0"]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "dynamic_input_interface_dynamicmapping": {
        "stated": true,
        "options": {
            "_scope": {
                "v_range": [["6.2.2", "6.4.0"]],
                "type": "list",
                "options": {"name": {"v_range": [["6.2.2", "6.4.0"]], "type": "str"}, "vdom": {"v_range": [["6.2.2", "6.4.0"]], "type": "str"}},
                "elements": "dict"
            },
            "local-intf": {"v_range": [["6.2.2", "6.4.0"]], "type": "str"}
        }
    },
    "dynamic_interface": {
        "stated": true,
        "options": {
            "color": {"type": "int"},
            "default-mapping": {"choices": ["disable", "enable"], "type": "str"},
            "defmap-intf": {"type": "str"},
            "defmap-intrazone-deny": {"choices": ["disable", "enable"], "type": "str"},
            "defmap-zonemember": {"type": "raw"},
            "description": {"type": "str"},
            "dynamic_mapping": {
                "type": "list",
                "options": {
                    "_scope": {"type": "list", "options": {"name": {"type": "str"}, "vdom": {"type": "str"}}, "elements": "dict"},
                    "egress-shaping-profile": {"type": "raw"},
                    "intrazone-deny": {"choices": ["disable", "enable"], "type": "str"},
                    "local-intf": {"type": "raw"},
                    "ingress-shaping-profile": {"v_range": [["6.2.2", ""]], "type": "raw"}
                },
                "elements": "dict"
            },
            "egress-shaping-profile": {"type": "raw"},
            "name": {"required": true, "type": "str"},
            "single-intf": {"choices": ["disable", "enable"], "type": "str"},
            "ingress-shaping-profile": {"v_range": [["6.2.2", ""]], "type": "raw"},
            "platform_mapping": {
                "v_range": [["6.4.1", ""]],
                "type": "list",
                "options": {
                    "egress-shaping-profile": {"v_range": [["6.4.1", ""]], "type": "raw"},
                    "ingress-shaping-profile": {"v_range": [["6.4.1", ""]], "type": "raw"},
                    "intf-zone": {"v_range": [["6.4.1", ""]], "type": "str"},
                    "intrazone-deny": {"v_range": [["6.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "name": {"v_range": [["6.4.1", ""]], "type": "str"}
                },
                "elements": "dict"
            },
            "wildcard": {"v_range": [["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "wildcard-intf": {"v_range": [["7.0.1", ""]], "type": "str"},
            "zone-only": {"v_range": [["6.4.7", "6.4.14"], ["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "dynamic_interface_dynamicmapping": {
        "stated": true,
        "options": {
            "_scope": {"type": "list", "options": {"name": {"type": "str"}, "vdom": {"type": "str"}}, "elements": "dict"},
            "egress-shaping-profile": {"type": "raw"},
            "intrazone-deny": {"choices": ["disable", "enable"], "type": "str"},
            "local-intf": {"type": "raw"},
            "ingress-shaping-profile": {"v_range": [["6.2.2", ""]], "type": "raw"}
        }
    },
    "dynamic_interface_platformmapping": {
        "stated": true,
        "options": {
            "egress-shaping-profile": {"v_range": [["6.4.1", ""]], "type": "raw"},
            "ingress-shaping-profile": {"v_range": [["6.4.1", ""]], "type": "raw"},
            "intf-zone": {"v_range": [["6.4.1", ""]], "type": "str"},
            "intrazone-deny": {"v_range": [["6.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "name": {"v_range": [["6.4.1", ""]], "required": true, "type": "str"}
        }
    },
    "dynamic_ippool": {"stated": true, "options": {"description": {"type": "str"}, "name": {"required": true, "type": "str"}}},
    "dynamic_multicast_interface": {
        "stated": true,
        "options": {
            "default-mapping": {"choices": ["disable", "enable"], "type": "str"},
            "defmap-intf": {"type": "str"},
            "description": {"type": "str"},
            "dynamic_mapping": {
                "type": "list",
                "options": {
                    "_scope": {"type": "list", "options": {"name": {"type": "str"}, "vdom": {"type": "str"}}, "elements": "dict"},
                    "local-intf": {"type": "str"}
                },
                "elements": "dict"
            },
            "name": {"required": true, "type": "str"},
            "zone-only": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "dynamic_multicast_interface_dynamicmapping": {
        "stated": true,
        "options": {
            "_scope": {"type": "list", "options": {"name": {"type": "str"}, "vdom": {"type": "str"}}, "elements": "dict"},
            "local-intf": {"type": "str"}
        }
    },
    "dynamic_vip": {"stated": true, "options": {"description": {"type": "str"}, "name": {"required": true, "type": "str"}}},
    "dynamic_virtualwanlink_members": {
        "stated": true,
        "options": {
            "comment": {"v_range": [["6.0.0", "6.4.14"]], "type": "str"},
            "cost": {"v_range": [["6.0.0", "6.4.14"]], "type": "int"},
            "detect-failtime": {"v_range": [["6.0.0", "6.4.14"]], "type": "int"},
            "detect-http-get": {"v_range": [["6.0.0", "6.4.14"]], "type": "str"},
            "detect-http-match": {"v_range": [["6.0.0", "6.4.14"]], "type": "str"},
            "detect-http-port": {"v_range": [["6.0.0", "6.4.14"]], "type": "int"},
            "detect-interval": {"v_range": [["6.0.0", "6.4.14"]], "type": "int"},
            "detect-protocol": {"v_range": [["6.0.0", "6.4.14"]], "choices": ["ping", "tcp-echo", "udp-echo", "http"], "type": "str"},
            "detect-recoverytime": {"v_range": [["6.0.0", "6.4.14"]], "type": "int"},
            "detect-server": {"v_range": [["6.0.0", "6.4.14"]], "type": "str"},
            "detect-timeout": {"v_range": [["6.0.0", "6.4.14"]], "type": "int"},
            "dynamic_mapping": {
                "v_range": [["6.0.0", "6.4.14"]],
                "type": "list",
                "options": {
                    "_scope": {
                        "v_range": [["6.0.0", "6.4.14"]],
                        "type": "list",
                        "options": {"name": {"v_range": [["6.0.0", "6.4.14"]], "type": "str"}, "vdom": {"v_range": [["6.0.0", "6.4.14"]], "type": "str"}},
                        "elements": "dict"
                    },
                    "comment": {"v_range": [["6.0.0", "6.4.14"]], "type": "str"},
                    "cost": {"v_range": [["6.0.0", "6.4.14"]], "type": "int"},
                    "detect-failtime": {"v_range": [["6.0.0", "6.4.14"]], "type": "int"},
                    "detect-http-get": {"v_range": [["6.0.0", "6.4.14"]], "type": "str"},
                    "detect-http-match": {"v_range": [["6.0.0", "6.4.14"]], "type": "str"},
                    "detect-http-port": {"v_range": [["6.0.0", "6.4.14"]], "type": "int"},
                    "detect-interval": {"v_range": [["6.0.0", "6.4.14"]], "type": "int"},
                    "detect-protocol": {"v_range": [["6.0.0", "6.4.14"]], "choices": ["ping", "tcp-echo", "udp-echo", "http"], "type": "str"},
                    "detect-recoverytime": {"v_range": [["6.0.0", "6.4.14"]], "type": "int"},
                    "detect-server": {"v_range": [["6.0.0", "6.4.14"]], "type": "str"},
                    "detect-timeout": {"v_range": [["6.0.0", "6.4.14"]], "type": "int"},
                    "gateway": {"v_range": [["6.0.0", "6.4.14"]], "type": "str"},
                    "gateway6": {"v_range": [["6.0.0", "6.4.14"]], "type": "str"},
                    "ingress-spillover-threshold": {"v_range": [["6.0.0", "6.4.14"]], "type": "int"},
                    "interface": {"v_range": [["6.0.0", "6.4.0"]], "type": "str"},
                    "priority": {"v_range": [["6.0.0", "6.4.14"]], "type": "int"},
                    "source": {"v_range": [["6.0.0", "6.4.14"]], "type": "str"},
                    "source6": {"v_range": [["6.0.0", "6.4.14"]], "type": "str"},
                    "spillover-threshold": {"v_range": [["6.0.0", "6.4.14"]], "type": "int"},
                    "status": {"v_range": [["6.0.0", "6.4.14"]], "choices": ["disable", "enable"], "type": "str"},
                    "volume-ratio": {"v_range": [["6.0.0", "6.4.14"]], "type": "int"},
                    "weight": {"v_range": [["6.0.0", "6.4.14"]], "type": "int"}
                },
                "elements": "dict"
            },
            "gateway": {"v_range": [["6.0.0", "6.4.14"]], "type": "str"},
            "gateway6": {"v_range": [["6.0.0", "6.4.14"]], "type": "str"},
            "ingress-spillover-threshold": {"v_range": [["6.0.0", "6.4.14"]], "type": "int"},
            "interface": {"v_range": [["6.0.0", "6.4.14"]], "type": "str"},
            "name": {"v_range": [["6.0.0", "6.4.14"]], "required": true, "type": "str"},
            "priority": {"v_range": [["6.0.0", "6.4.14"]], "type": "int"},
            "source": {"v_range": [["6.0.0", "6.4.14"]], "type": "str"},
            "source6": {"v_range": [["6.0.0", "6.4.14"]], "type": "str"},
            "spillover-threshold": {"v_range": [["6.0.0", "6.4.14"]], "type": "int"},
            "status": {"v_range": [["6.0.0", "6.4.14"]], "choices": ["disable", "enable"], "type": "str"},
            "volume-ratio": {"v_range": [["6.0.0", "6.4.14"]], "type": "int"},
            "weight": {"v_range": [["6.0.0", "6.4.14"]], "type": "int"}
        }
    },
    "dynamic_virtualwanlink_members_dynamicmapping": {
        "stated": true,
        "options": {
            "_scope": {
                "v_range": [["6.0.0", "6.4.14"]],
                "type": "list",
                "options": {"name": {"v_range": [["6.0.0", "6.4.14"]], "type": "str"}, "vdom": {"v_range": [["6.0.0", "6.4.14"]], "type": "str"}},
                "elements": "dict"
            },
            "comment": {"v_range": [["6.0.0", "6.4.14"]], "type": "str"},
            "cost": {"v_range": [["6.0.0", "6.4.14"]], "type": "int"},
            "detect-failtime": {"v_range": [["6.0.0", "6.4.14"]], "type": "int"},
            "detect-http-get": {"v_range": [["6.0.0", "6.4.14"]], "type": "str"},
            "detect-http-match": {"v_range": [["6.0.0", "6.4.14"]], "type": "str"},
            "detect-http-port": {"v_range": [["6.0.0", "6.4.14"]], "type": "int"},
            "detect-interval": {"v_range": [["6.0.0", "6.4.14"]], "type": "int"},
            "detect-protocol": {"v_range": [["6.0.0", "6.4.14"]], "choices": ["ping", "tcp-echo", "udp-echo", "http"], "type": "str"},
            "detect-recoverytime": {"v_range": [["6.0.0", "6.4.14"]], "type": "int"},
            "detect-server": {"v_range": [["6.0.0", "6.4.14"]], "type": "str"},
            "detect-timeout": {"v_range": [["6.0.0", "6.4.14"]], "type": "int"},
            "gateway": {"v_range": [["6.0.0", "6.4.14"]], "type": "str"},
            "gateway6": {"v_range": [["6.0.0", "6.4.14"]], "type": "str"},
            "ingress-spillover-threshold": {"v_range": [["6.0.0", "6.4.14"]], "type": "int"},
            "interface": {"v_range": [["6.0.0", "6.4.0"]], "type": "str"},
            "priority": {"v_range": [["6.0.0", "6.4.14"]], "type": "int"},
            "source": {"v_range": [["6.0.0", "6.4.14"]], "type": "str"},
            "source6": {"v_range": [["6.0.0", "6.4.14"]], "type": "str"},
            "spillover-threshold": {"v_range": [["6.0.0", "6.4.14"]], "type": "int"},
            "status": {"v_range": [["6.0.0", "6.4.14"]], "choices": ["disable", "enable"], "type": "str"},
            "volume-ratio": {"v_range": [["6.0.0", "6.4.14"]], "type": "int"},
            "weight": {"v_range": [["6.0.0", "6.4.14"]], "type": "int"}
        }
    },
    "dynamic_virtualwanlink_neighbor": {
        "stated": true,
        "options": {
            "description": {"v_range": [["6.2.2", "6.4.14"]], "type": "str"},
            "dynamic_mapping": {
                "v_range": [["6.2.2", "6.4.14"]],
                "type": "list",
                "options": {
                    "_scope": {
                        "v_range": [["6.2.2", "6.4.14"]],
                        "type": "list",
                        "options": {"name": {"v_range": [["6.2.2", "6.4.14"]], "type": "str"}, "vdom": {"v_range": [["6.2.2", "6.4.14"]], "type": "str"}},
                        "elements": "dict"
                    },
                    "description": {"v_range": [["6.2.2", "6.4.14"]], "type": "str"},
                    "ip": {"v_range": [["6.2.2", "6.4.14"]], "type": "str"},
                    "role": {"v_range": [["6.2.2", "6.4.14"]], "choices": ["primary", "secondary", "standalone"], "type": "str"}
                },
                "elements": "dict"
            },
            "ip": {"v_range": [["6.2.2", "6.4.14"]], "type": "str"},
            "name": {"v_range": [["6.2.2", "6.4.14"]], "required": true, "type": "str"},
            "role": {"v_range": [["6.2.2", "6.4.14"]], "choices": ["primary", "secondary", "standalone"], "type": "str"}
        }
    },
    "dynamic_virtualwanlink_neighbor_dynamicmapping": {
        "stated": true,
        "options": {
            "_scope": {
                "v_range": [["6.2.2", "6.4.14"]],
                "type": "list",
                "options": {"name": {"v_range": [["6.2.2", "6.4.14"]], "type": "str"}, "vdom": {"v_range": [["6.2.2", "6.4.14"]], "type": "str"}},
                "elements": "dict"
            },
            "description": {"v_range": [["6.2.2", "6.4.14"]], "type": "str"},
            "ip": {"v_range": [["6.2.2", "6.4.14"]], "type": "str"},
            "role": {"v_range": [["6.2.2", "6.4.14"]], "choices": ["primary", "secondary", "standalone"], "type": "str"}
        }
    },
    "dynamic_virtualwanlink_server": {
        "stated": true,
        "options": {
            "description": {"v_range": [["6.0.0", "6.4.14"]], "type": "str"},
            "dynamic_mapping": {
                "v_range": [["6.0.0", "6.4.14"]],
                "type": "list",
                "options": {
                    "_scope": {
                        "v_range": [["6.0.0", "6.4.14"]],
                        "type": "list",
                        "options": {"name": {"v_range": [["6.0.0", "6.4.14"]], "type": "str"}, "vdom": {"v_range": [["6.0.0", "6.4.14"]], "type": "str"}},
                        "elements": "dict"
                    },
                    "server": {"v_range": [["6.0.0", "6.4.14"]], "type": "raw"}
                },
                "elements": "dict"
            },
            "name": {"v_range": [["6.0.0", "6.4.14"]], "required": true, "type": "str"},
            "server": {"v_range": [["6.0.0", "6.4.14"]], "type": "raw"}
        }
    },
    "dynamic_virtualwanlink_server_dynamicmapping": {
        "stated": true,
        "options": {
            "_scope": {
                "v_range": [["6.0.0", "6.4.14"]],
                "type": "list",
                "options": {"name": {"v_range": [["6.0.0", "6.4.14"]], "type": "str"}, "vdom": {"v_range": [["6.0.0", "6.4.14"]], "type": "str"}},
                "elements": "dict"
            },
            "server": {"v_range": [["6.0.0", "6.4.14"]], "type": "raw"}
        }
    },
    "dynamic_vpntunnel": {
        "stated": true,
        "options": {
            "description": {"type": "str"},
            "dynamic_mapping": {
                "type": "list",
                "options": {
                    "_scope": {"type": "list", "options": {"name": {"type": "str"}, "vdom": {"type": "str"}}, "elements": "dict"},
                    "local-ipsec": {"type": "str"}
                },
                "elements": "dict"
            },
            "name": {"required": true, "type": "str"}
        }
    },
    "dynamic_vpntunnel_dynamicmapping": {
        "stated": true,
        "options": {
            "_scope": {"type": "list", "options": {"name": {"type": "str"}, "vdom": {"type": "str"}}, "elements": "dict"},
            "local-ipsec": {"type": "str"}
        }
    },
    "emailfilter_blockallowlist": {
        "stated": true,
        "options": {
            "comment": {"v_range": [["7.0.0", ""]], "type": "str"},
            "entries": {
                "v_range": [["7.0.0", ""]],
                "type": "list",
                "options": {
                    "action": {"v_range": [["7.0.0", ""]], "choices": ["spam", "clear", "reject"], "type": "str"},
                    "addr-type": {"v_range": [["7.0.0", ""]], "choices": ["ipv4", "ipv6"], "type": "str"},
                    "email-pattern": {"v_range": [["7.0.0", ""]], "type": "str"},
                    "id": {"v_range": [["7.0.0", ""]], "type": "int"},
                    "ip4-subnet": {"v_range": [["7.0.0", ""]], "type": "str"},
                    "ip6-subnet": {"v_range": [["7.0.0", ""]], "type": "str"},
                    "pattern-type": {"v_range": [["7.0.0", ""]], "choices": ["wildcard", "regexp"], "type": "str"},
                    "status": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "type": {"v_range": [["7.0.0", ""]], "choices": ["ip", "email", "email-to", "email-from", "subject"], "type": "str"},
                    "pattern": {"v_range": [["7.2.0", ""]], "type": "str"}
                },
                "elements": "dict"
            },
            "id": {"v_range": [["7.0.0", ""]], "required": true, "type": "int"},
            "name": {"v_range": [["7.0.0", ""]], "type": "str"}
        }
    },
    "emailfilter_blockallowlist_entries": {
        "stated": true,
        "options": {
            "action": {"v_range": [["7.0.0", ""]], "choices": ["spam", "clear", "reject"], "type": "str"},
            "addr-type": {"v_range": [["7.0.0", ""]], "choices": ["ipv4", "ipv6"], "type": "str"},
            "email-pattern": {"v_range": [["7.0.0", ""]], "type": "str"},
            "id": {"v_range": [["7.0.0", ""]], "required": true, "type": "int"},
            "ip4-subnet": {"v_range": [["7.0.0", ""]], "type": "str"},
            "ip6-subnet": {"v_range": [["7.0.0", ""]], "type": "str"},
            "pattern-type": {"v_range": [["7.0.0", ""]], "choices": ["wildcard", "regexp"], "type": "str"},
            "status": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "type": {"v_range": [["7.0.0", ""]], "choices": ["ip", "email", "email-to", "email-from", "subject"], "type": "str"},
            "pattern": {"v_range": [["7.2.0", ""]], "type": "str"}
        }
    },
    "emailfilter_bwl": {
        "stated": true,
        "options": {
            "comment": {"v_range": [["6.2.0", ""]], "type": "str"},
            "entries": {
                "v_range": [["6.2.0", ""]],
                "type": "list",
                "options": {
                    "action": {"v_range": [["6.2.0", ""]], "choices": ["spam", "clear", "reject"], "type": "str"},
                    "addr-type": {"v_range": [["6.2.0", ""]], "choices": ["ipv4", "ipv6"], "type": "str"},
                    "email-pattern": {"v_range": [["6.2.0", ""]], "type": "str"},
                    "id": {"v_range": [["6.2.0", ""]], "type": "int"},
                    "ip4-subnet": {"v_range": [["6.2.0", ""]], "type": "str"},
                    "ip6-subnet": {"v_range": [["6.2.0", ""]], "type": "str"},
                    "pattern-type": {"v_range": [["6.2.0", ""]], "choices": ["wildcard", "regexp"], "type": "str"},
                    "status": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "type": {"v_range": [["6.2.0", ""]], "choices": ["ip", "email"], "type": "str"}
                },
                "elements": "dict"
            },
            "id": {"v_range": [["6.2.0", ""]], "required": true, "type": "int"},
            "name": {"v_range": [["6.2.0", ""]], "type": "str"}
        }
    },
    "emailfilter_bwl_entries": {
        "stated": true,
        "options": {
            "action": {"v_range": [["6.2.0", ""]], "choices": ["spam", "clear", "reject"], "type": "str"},
            "addr-type": {"v_range": [["6.2.0", ""]], "choices": ["ipv4", "ipv6"], "type": "str"},
            "email-pattern": {"v_range": [["6.2.0", ""]], "type": "str"},
            "id": {"v_range": [["6.2.0", ""]], "required": true, "type": "int"},
            "ip4-subnet": {"v_range": [["6.2.0", ""]], "type": "str"},
            "ip6-subnet": {"v_range": [["6.2.0", ""]], "type": "str"},
            "pattern-type": {"v_range": [["6.2.0", ""]], "choices": ["wildcard", "regexp"], "type": "str"},
            "status": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "type": {"v_range": [["6.2.0", ""]], "choices": ["ip", "email"], "type": "str"}
        }
    },
    "emailfilter_bword": {
        "stated": true,
        "options": {
            "comment": {"v_range": [["6.2.0", ""]], "type": "str"},
            "entries": {
                "v_range": [["6.2.0", ""]],
                "type": "list",
                "options": {
                    "action": {"v_range": [["6.2.0", ""]], "choices": ["spam", "clear"], "type": "str"},
                    "id": {"v_range": [["6.2.0", ""]], "type": "int"},
                    "language": {
                        "v_range": [["6.2.0", ""]],
                        "choices": ["western", "simch", "trach", "japanese", "korean", "french", "thai", "spanish"],
                        "type": "str"
                    },
                    "pattern": {"v_range": [["6.2.0", ""]], "type": "str"},
                    "pattern-type": {"v_range": [["6.2.0", ""]], "choices": ["wildcard", "regexp"], "type": "str"},
                    "score": {"v_range": [["6.2.0", ""]], "type": "int"},
                    "status": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "where": {"v_range": [["6.2.0", ""]], "choices": ["all", "subject", "body"], "type": "str"}
                },
                "elements": "dict"
            },
            "id": {"v_range": [["6.2.0", ""]], "required": true, "type": "int"},
            "name": {"v_range": [["6.2.0", ""]], "type": "str"}
        }
    },
    "emailfilter_bword_entries": {
        "stated": true,
        "options": {
            "action": {"v_range": [["6.2.0", ""]], "choices": ["spam", "clear"], "type": "str"},
            "id": {"v_range": [["6.2.0", ""]], "required": true, "type": "int"},
            "language": {
                "v_range": [["6.2.0", ""]],
                "choices": ["western", "simch", "trach", "japanese", "korean", "french", "thai", "spanish"],
                "type": "str"
            },
            "pattern": {"v_range": [["6.2.0", ""]], "type": "str"},
            "pattern-type": {"v_range": [["6.2.0", ""]], "choices": ["wildcard", "regexp"], "type": "str"},
            "score": {"v_range": [["6.2.0", ""]], "type": "int"},
            "status": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "where": {"v_range": [["6.2.0", ""]], "choices": ["all", "subject", "body"], "type": "str"}
        }
    },
    "emailfilter_dnsbl": {
        "stated": true,
        "options": {
            "comment": {"v_range": [["6.2.0", ""]], "type": "str"},
            "entries": {
                "v_range": [["6.2.0", ""]],
                "type": "list",
                "options": {
                    "action": {"v_range": [["6.2.0", ""]], "choices": ["spam", "reject"], "type": "str"},
                    "id": {"v_range": [["6.2.0", ""]], "type": "int"},
                    "server": {"v_range": [["6.2.0", ""]], "type": "str"},
                    "status": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable"], "type": "str"}
                },
                "elements": "dict"
            },
            "id": {"v_range": [["6.2.0", ""]], "required": true, "type": "int"},
            "name": {"v_range": [["6.2.0", ""]], "type": "str"}
        }
    },
    "emailfilter_dnsbl_entries": {
        "stated": true,
        "options": {
            "action": {"v_range": [["6.2.0", ""]], "choices": ["spam", "reject"], "type": "str"},
            "id": {"v_range": [["6.2.0", ""]], "required": true, "type": "int"},
            "server": {"v_range": [["6.2.0", ""]], "type": "str"},
            "status": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "emailfilter_fortishield": {
        "stated": false,
        "options": {
            "spam-submit-force": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "spam-submit-srv": {"v_range": [["6.2.0", ""]], "type": "str"},
            "spam-submit-txt2htm": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "emailfilter_iptrust": {
        "stated": true,
        "options": {
            "comment": {"v_range": [["6.2.0", ""]], "type": "str"},
            "entries": {
                "v_range": [["6.2.0", ""]],
                "type": "list",
                "options": {
                    "addr-type": {"v_range": [["6.2.0", ""]], "choices": ["ipv4", "ipv6"], "type": "str"},
                    "id": {"v_range": [["6.2.0", ""]], "type": "int"},
                    "ip4-subnet": {"v_range": [["6.2.0", ""]], "type": "str"},
                    "ip6-subnet": {"v_range": [["6.2.0", ""]], "type": "str"},
                    "status": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable"], "type": "str"}
                },
                "elements": "dict"
            },
            "id": {"v_range": [["6.2.0", ""]], "required": true, "type": "int"},
            "name": {"v_range": [["6.2.0", ""]], "type": "str"}
        }
    },
    "emailfilter_iptrust_entries": {
        "stated": true,
        "options": {
            "addr-type": {"v_range": [["6.2.0", ""]], "choices": ["ipv4", "ipv6"], "type": "str"},
            "id": {"v_range": [["6.2.0", ""]], "required": true, "type": "int"},
            "ip4-subnet": {"v_range": [["6.2.0", ""]], "type": "str"},
            "ip6-subnet": {"v_range": [["6.2.0", ""]], "type": "str"},
            "status": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "emailfilter_mheader": {
        "stated": true,
        "options": {
            "comment": {"v_range": [["6.2.0", ""]], "type": "str"},
            "entries": {
                "v_range": [["6.2.0", ""]],
                "type": "list",
                "options": {
                    "action": {"v_range": [["6.2.0", ""]], "choices": ["spam", "clear"], "type": "str"},
                    "fieldbody": {"v_range": [["6.2.0", ""]], "type": "str"},
                    "fieldname": {"v_range": [["6.2.0", ""]], "type": "str"},
                    "id": {"v_range": [["6.2.0", ""]], "type": "int"},
                    "pattern-type": {"v_range": [["6.2.0", ""]], "choices": ["wildcard", "regexp"], "type": "str"},
                    "status": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable"], "type": "str"}
                },
                "elements": "dict"
            },
            "id": {"v_range": [["6.2.0", ""]], "required": true, "type": "int"},
            "name": {"v_range": [["6.2.0", ""]], "type": "str"}
        }
    },
    "emailfilter_mheader_entries": {
        "stated": true,
        "options": {
            "action": {"v_range": [["6.2.0", ""]], "choices": ["spam", "clear"], "type": "str"},
            "fieldbody": {"v_range": [["6.2.0", ""]], "type": "str"},
            "fieldname": {"v_range": [["6.2.0", ""]], "type": "str"},
            "id": {"v_range": [["6.2.0", ""]], "required": true, "type": "int"},
            "pattern-type": {"v_range": [["6.2.0", ""]], "choices": ["wildcard", "regexp"], "type": "str"},
            "status": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "emailfilter_options": {"stated": false, "options": {"dns-timeout": {"v_range": [["6.2.0", ""]], "type": "int"}}},
    "emailfilter_profile": {
        "stated": true,
        "options": {
            "comment": {"v_range": [["6.2.0", ""]], "type": "str"},
            "external": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "name": {"v_range": [["6.2.0", ""]], "required": true, "type": "str"},
            "options": {
                "v_range": [["6.2.0", ""]],
                "type": "list",
                "choices": [
                    "bannedword", "spambwl", "spamfsip", "spamfssubmit", "spamfschksum", "spamfsurl", "spamhelodns", "spamraddrdns", "spamrbl",
                    "spamhdrcheck", "spamfsphish", "spambal", "spamfgip", "spamfgsubmit", "spamfgchksum", "spamfgurl", "spamfgphish"
                ],
                "elements": "str"
            },
            "replacemsg-group": {"v_range": [["6.2.0", ""]], "type": "str"},
            "spam-bwl-table": {"v_range": [["6.2.0", ""]], "type": "str"},
            "spam-bword-table": {"v_range": [["6.2.0", ""]], "type": "str"},
            "spam-bword-threshold": {"v_range": [["6.2.0", ""]], "type": "int"},
            "spam-filtering": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "spam-iptrust-table": {"v_range": [["6.2.0", ""]], "type": "str"},
            "spam-log": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "spam-log-fortiguard-response": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "spam-mheader-table": {"v_range": [["6.2.0", ""]], "type": "str"},
            "spam-rbl-table": {"v_range": [["6.2.0", ""]], "type": "str"},
            "feature-set": {"v_range": [["6.4.0", ""]], "choices": ["proxy", "flow"], "type": "str"},
            "gmail": {
                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                "type": "dict",
                "options": {
                    "log": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "log-all": {"v_range": [["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"}
                }
            },
            "imap": {
                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                "type": "dict",
                "options": {
                    "action": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["pass", "tag"], "type": "str"},
                    "log": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "log-all": {"v_range": [["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "tag-msg": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "tag-type": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "type": "list",
                        "choices": ["subject", "header", "spaminfo"],
                        "elements": "str"
                    }
                }
            },
            "mapi": {
                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                "type": "dict",
                "options": {
                    "action": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["pass", "discard"], "type": "str"},
                    "log": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "log-all": {"v_range": [["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"}
                }
            },
            "msn-hotmail": {
                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                "type": "dict",
                "options": {
                    "log": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "log-all": {"v_range": [["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"}
                }
            },
            "other-webmails": {
                "v_range": [["6.4.5", ""]],
                "type": "dict",
                "options": {"log-all": {"v_range": [["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"}}
            },
            "pop3": {
                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                "type": "dict",
                "options": {
                    "action": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["pass", "tag"], "type": "str"},
                    "log": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "log-all": {"v_range": [["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "tag-msg": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "tag-type": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "type": "list",
                        "choices": ["subject", "header", "spaminfo"],
                        "elements": "str"
                    }
                }
            },
            "smtp": {
                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                "type": "dict",
                "options": {
                    "action": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["pass", "tag", "discard"], "type": "str"},
                    "hdrip": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "local-override": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "log": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "log-all": {"v_range": [["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "tag-msg": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "tag-type": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "type": "list",
                        "choices": ["subject", "header", "spaminfo"],
                        "elements": "str"
                    }
                }
            },
            "file-filter": {
                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                "type": "dict",
                "options": {
                    "entries": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "type": "list",
                        "options": {
                            "action": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["log", "block"], "type": "str"},
                            "comment": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                            "encryption": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", "7.2.0"]], "choices": ["any", "yes"], "type": "str"},
                            "file-type": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "raw"},
                            "filter": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                            "password-protected": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["any", "yes"], "type": "str"},
                            "protocol": {
                                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                                "type": "list",
                                "choices": ["smtp", "imap", "pop3"],
                                "elements": "str"
                            }
                        },
                        "elements": "dict"
                    },
                    "log": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "scan-archive-contents": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "status": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"}
                }
            },
            "spam-bal-table": {"v_range": [["7.0.0", ""]], "type": "str"}
        }
    },
    "emailfilter_profile_filefilter": {
        "stated": false,
        "options": {
            "entries": {
                "v_range": [["6.2.0", ""]],
                "type": "list",
                "options": {
                    "action": {"v_range": [["6.2.0", ""]], "choices": ["log", "block"], "type": "str"},
                    "comment": {"v_range": [["6.2.0", ""]], "type": "str"},
                    "encryption": {"v_range": [["6.2.0", "7.2.0"]], "choices": ["any", "yes"], "type": "str"},
                    "file-type": {"v_range": [["6.2.0", ""]], "type": "raw"},
                    "filter": {"v_range": [["6.2.0", ""]], "type": "str"},
                    "password-protected": {"v_range": [["6.2.1", ""]], "choices": ["any", "yes"], "type": "str"},
                    "protocol": {"v_range": [["6.2.0", ""]], "type": "list", "choices": ["smtp", "imap", "pop3"], "elements": "str"}
                },
                "elements": "dict"
            },
            "log": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "scan-archive-contents": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "status": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "emailfilter_profile_filefilter_entries": {
        "stated": true,
        "options": {
            "action": {"v_range": [["6.2.0", ""]], "choices": ["log", "block"], "type": "str"},
            "comment": {"v_range": [["6.2.0", ""]], "type": "str"},
            "encryption": {"v_range": [["6.2.0", "7.2.0"]], "choices": ["any", "yes"], "type": "str"},
            "file-type": {"v_range": [["6.2.0", ""]], "type": "raw"},
            "filter": {"v_range": [["6.2.0", ""]], "type": "str"},
            "password-protected": {"v_range": [["6.2.1", ""]], "choices": ["any", "yes"], "type": "str"},
            "protocol": {"v_range": [["6.2.0", ""]], "type": "list", "choices": ["smtp", "imap", "pop3"], "elements": "str"}
        }
    },
    "emailfilter_profile_gmail": {
        "stated": false,
        "options": {
            "log": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "log-all": {"v_range": [["6.4.2", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "emailfilter_profile_imap": {
        "stated": false,
        "options": {
            "action": {"v_range": [["6.2.0", ""]], "choices": ["pass", "tag"], "type": "str"},
            "log": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "tag-msg": {"v_range": [["6.2.0", ""]], "type": "str"},
            "tag-type": {"v_range": [["6.2.0", ""]], "type": "list", "choices": ["subject", "header", "spaminfo"], "elements": "str"},
            "log-all": {"v_range": [["6.4.2", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "emailfilter_profile_mapi": {
        "stated": false,
        "options": {
            "action": {"v_range": [["6.2.0", ""]], "choices": ["pass", "discard"], "type": "str"},
            "log": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "log-all": {"v_range": [["6.4.2", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "emailfilter_profile_msnhotmail": {
        "stated": false,
        "options": {
            "log": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "log-all": {"v_range": [["6.4.2", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "emailfilter_profile_otherwebmails": {
        "stated": false,
        "options": {"log-all": {"v_range": [["6.4.2", ""]], "choices": ["disable", "enable"], "type": "str"}}
    },
    "emailfilter_profile_pop3": {
        "stated": false,
        "options": {
            "action": {"v_range": [["6.2.0", ""]], "choices": ["pass", "tag"], "type": "str"},
            "log": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "tag-msg": {"v_range": [["6.2.0", ""]], "type": "str"},
            "tag-type": {"v_range": [["6.2.0", ""]], "type": "list", "choices": ["subject", "header", "spaminfo"], "elements": "str"},
            "log-all": {"v_range": [["6.4.2", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "emailfilter_profile_smtp": {
        "stated": false,
        "options": {
            "action": {"v_range": [["6.2.0", ""]], "choices": ["pass", "tag", "discard"], "type": "str"},
            "hdrip": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "local-override": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "log": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "tag-msg": {"v_range": [["6.2.0", ""]], "type": "str"},
            "tag-type": {"v_range": [["6.2.0", ""]], "type": "list", "choices": ["subject", "header", "spaminfo"], "elements": "str"},
            "log-all": {"v_range": [["6.4.2", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "emailfilter_profile_yahoomail": {
        "stated": false,
        "options": {"log": {"v_range": [["6.2.0", "6.2.0"]], "choices": ["disable", "enable"], "type": "str"}}
    },
    "endpointcontrol_fctems": {
        "stated": true,
        "options": {
            "call-timeout": {"v_range": [["7.0.2", ""]], "type": "int"},
            "capabilities": {
                "v_range": [["7.0.2", ""]],
                "type": "list",
                "choices": [
                    "fabric-auth", "silent-approval", "websocket", "websocket-malware", "push-ca-certs", "common-tags-api", "tenant-id",
                    "single-vdom-connector", "client-avatars", "fgt-sysinfo-api", "ztna-server-info"
                ],
                "elements": "str"
            },
            "certificate-fingerprint": {"v_range": [["7.0.2", ""]], "type": "str"},
            "cloud-server-type": {"v_range": [["7.0.2", ""]], "choices": ["production", "alpha", "beta"], "type": "str"},
            "fortinetone-cloud-authentication": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "https-port": {"v_range": [["7.0.2", ""]], "type": "int"},
            "name": {"v_range": [["7.0.2", ""]], "required": true, "type": "str"},
            "out-of-sync-threshold": {"v_range": [["7.0.5", ""]], "type": "int"},
            "preserve-ssl-session": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "pull-avatars": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "pull-malware-hash": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "pull-sysinfo": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "pull-tags": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "pull-vulnerabilities": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "server": {"v_range": [["7.0.2", ""]], "type": "str"},
            "source-ip": {"v_range": [["7.0.2", ""]], "type": "str"},
            "websocket-override": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "status-check-interval": {"v_range": [["7.0.2", ""]], "type": "int"},
            "certificate": {"v_range": [["7.0.2", ""]], "type": "str"},
            "admin-username": {"v_range": [["7.0.2", ""]], "type": "str"},
            "serial-number": {"v_range": [["7.0.2", ""]], "type": "str"},
            "admin-password": {"v_range": [["7.0.2", ""]], "no_log": true, "type": "raw"},
            "interface": {"v_range": [["7.0.4", "7.0.12"], ["7.2.1", ""]], "type": "str"},
            "interface-select-method": {"v_range": [["7.0.4", "7.0.12"], ["7.2.1", ""]], "choices": ["auto", "sdwan", "specify"], "type": "str"},
            "dirty-reason": {"v_range": [["7.0.5", "7.0.12"], ["7.2.1", ""]], "choices": ["none", "mismatched-ems-sn"], "type": "str"},
            "ems-id": {"v_range": [["7.0.5", "7.0.12"], ["7.2.1", ""]], "type": "int"},
            "status": {"v_range": [["7.0.5", "7.0.12"], ["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ca-cn-info": {"v_range": [["7.0.6", "7.0.12"], ["7.2.2", ""]], "type": "str"},
            "trust-ca-cn": {"v_range": [["7.0.6", "7.0.12"], ["7.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "tenant-id": {"v_range": [["7.2.1", ""]], "type": "str"},
            "send-tags-to-all-vdoms": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "verified-cn": {"v_range": [["7.4.2", ""]], "type": "str"},
            "verifying-ca": {"v_range": [["7.4.2", ""]], "type": "str"},
            "cloud-authentication-access-key": {"v_range": [["7.4.3", ""]], "no_log": true, "type": "str"}
        }
    },
    "exec_fgfm_reclaimdevtunnel": {"stated": true, "options": {"flags": {"type": "list", "choices": ["force", "get_info"], "elements": "str"}}},
    "extendercontroller_dataplan": {
        "stated": true,
        "options": {
            "apn": {"v_range": [["6.4.4", ""]], "type": "str"},
            "auth-type": {"v_range": [["6.4.4", ""]], "choices": ["none", "pap", "chap"], "type": "str"},
            "billing-date": {"v_range": [["6.4.4", ""]], "type": "int"},
            "capacity": {"v_range": [["6.4.4", ""]], "type": "int"},
            "carrier": {"v_range": [["6.4.4", ""]], "type": "str"},
            "iccid": {"v_range": [["6.4.4", ""]], "type": "str"},
            "modem-id": {"v_range": [["6.4.4", ""]], "choices": ["all", "modem1", "modem2"], "type": "str"},
            "monthly-fee": {"v_range": [["6.4.4", ""]], "type": "int"},
            "name": {"v_range": [["6.4.4", ""]], "required": true, "type": "str"},
            "overage": {"v_range": [["6.4.4", ""]], "choices": ["disable", "enable"], "type": "str"},
            "password": {"v_range": [["6.4.4", ""]], "no_log": true, "type": "raw"},
            "pdn": {"v_range": [["6.4.4", ""]], "choices": ["ipv4-only", "ipv6-only", "ipv4-ipv6"], "type": "str"},
            "preferred-subnet": {"v_range": [["6.4.4", ""]], "type": "int"},
            "private-network": {"v_range": [["6.4.4", ""]], "choices": ["disable", "enable"], "type": "str"},
            "signal-period": {"v_range": [["6.4.4", ""]], "type": "int"},
            "signal-threshold": {"v_range": [["6.4.4", ""]], "type": "int"},
            "slot": {"v_range": [["6.4.4", ""]], "choices": ["sim1", "sim2"], "type": "str"},
            "status": {"v_range": [["6.4.4", ""]], "choices": ["disable", "enable"], "type": "str"},
            "type": {"v_range": [["6.4.4", ""]], "choices": ["carrier", "slot", "iccid", "generic"], "type": "str"},
            "username": {"v_range": [["6.4.4", ""]], "type": "str"}
        }
    },
    "extendercontroller_extenderprofile": {
        "stated": true,
        "options": {
            "allowaccess": {"v_range": [["7.0.2", ""]], "type": "list", "choices": ["https", "ping", "ssh", "snmp", "http", "telnet"], "elements": "str"},
            "bandwidth-limit": {"v_range": [["7.0.2", ""]], "type": "int"},
            "cellular": {
                "v_range": [["7.0.2", ""]],
                "type": "dict",
                "options": {
                    "controller-report": {
                        "v_range": [["7.0.2", ""]],
                        "type": "dict",
                        "options": {
                            "interval": {"v_range": [["7.0.2", ""]], "type": "int"},
                            "signal-threshold": {"v_range": [["7.0.2", ""]], "type": "int"},
                            "status": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"}
                        }
                    },
                    "dataplan": {"v_range": [["7.0.2", ""]], "type": "raw"},
                    "modem1": {
                        "v_range": [["7.0.2", ""]],
                        "type": "dict",
                        "options": {
                            "auto-switch": {
                                "v_range": [["7.0.2", ""]],
                                "type": "dict",
                                "options": {
                                    "dataplan": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                                    "disconnect": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                                    "disconnect-period": {"v_range": [["7.0.2", ""]], "type": "int"},
                                    "disconnect-threshold": {"v_range": [["7.0.2", ""]], "type": "int"},
                                    "signal": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                                    "switch-back": {"v_range": [["7.0.2", ""]], "type": "list", "choices": ["time", "timer"], "elements": "str"},
                                    "switch-back-time": {"v_range": [["7.0.2", ""]], "type": "str"},
                                    "switch-back-timer": {"v_range": [["7.0.2", ""]], "type": "int"}
                                }
                            },
                            "conn-status": {"v_range": [["7.0.2", ""]], "type": "int"},
                            "default-sim": {"v_range": [["7.0.2", ""]], "choices": ["sim1", "sim2", "carrier", "cost"], "type": "str"},
                            "gps": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "modem-id": {"v_range": [["7.0.2", ""]], "type": "int"},
                            "preferred-carrier": {"v_range": [["7.0.2", ""]], "type": "str"},
                            "redundant-intf": {"v_range": [["7.0.2", ""]], "type": "str"},
                            "redundant-mode": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "sim1-pin": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "sim1-pin-code": {"v_range": [["7.0.2", ""]], "type": "raw"},
                            "sim2-pin": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "sim2-pin-code": {"v_range": [["7.0.2", ""]], "type": "raw"}
                        }
                    },
                    "modem2": {
                        "v_range": [["7.0.2", ""]],
                        "type": "dict",
                        "options": {
                            "auto-switch": {
                                "v_range": [["7.0.2", ""]],
                                "type": "dict",
                                "options": {
                                    "dataplan": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                                    "disconnect": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                                    "disconnect-period": {"v_range": [["7.0.2", ""]], "type": "int"},
                                    "disconnect-threshold": {"v_range": [["7.0.2", ""]], "type": "int"},
                                    "signal": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                                    "switch-back": {"v_range": [["7.0.2", ""]], "type": "list", "choices": ["time", "timer"], "elements": "str"},
                                    "switch-back-time": {"v_range": [["7.0.2", ""]], "type": "str"},
                                    "switch-back-timer": {"v_range": [["7.0.2", ""]], "type": "int"}
                                }
                            },
                            "conn-status": {"v_range": [["7.0.2", ""]], "type": "int"},
                            "default-sim": {"v_range": [["7.0.2", ""]], "choices": ["sim1", "sim2", "carrier", "cost"], "type": "str"},
                            "gps": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "modem-id": {"v_range": [["7.0.2", ""]], "type": "int"},
                            "preferred-carrier": {"v_range": [["7.0.2", ""]], "type": "str"},
                            "redundant-intf": {"v_range": [["7.0.2", ""]], "type": "str"},
                            "redundant-mode": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "sim1-pin": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "sim1-pin-code": {"v_range": [["7.0.2", ""]], "type": "raw"},
                            "sim2-pin": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "sim2-pin-code": {"v_range": [["7.0.2", ""]], "type": "raw"}
                        }
                    },
                    "sms-notification": {
                        "v_range": [["7.0.2", ""]],
                        "type": "dict",
                        "options": {
                            "alert": {
                                "v_range": [["7.0.2", ""]],
                                "type": "dict",
                                "options": {
                                    "data-exhausted": {"v_range": [["7.0.2", ""]], "type": "str"},
                                    "fgt-backup-mode-switch": {"v_range": [["7.0.2", ""]], "type": "str"},
                                    "low-signal-strength": {"v_range": [["7.0.2", ""]], "type": "str"},
                                    "mode-switch": {"v_range": [["7.0.2", ""]], "type": "str"},
                                    "os-image-fallback": {"v_range": [["7.0.2", ""]], "type": "str"},
                                    "session-disconnect": {"v_range": [["7.0.2", ""]], "type": "str"},
                                    "system-reboot": {"v_range": [["7.0.2", ""]], "type": "str"}
                                }
                            },
                            "receiver": {
                                "v_range": [["7.0.2", ""]],
                                "type": "list",
                                "options": {
                                    "alert": {
                                        "v_range": [["7.0.2", ""]],
                                        "type": "list",
                                        "choices": [
                                            "system-reboot", "data-exhausted", "session-disconnect", "low-signal-strength", "mode-switch",
                                            "os-image-fallback", "fgt-backup-mode-switch"
                                        ],
                                        "elements": "str"
                                    },
                                    "name": {"v_range": [["7.0.2", ""]], "type": "str"},
                                    "phone-number": {"v_range": [["7.0.2", ""]], "type": "str"},
                                    "status": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"}
                                },
                                "elements": "dict"
                            },
                            "status": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"}
                        }
                    }
                }
            },
            "enforce-bandwidth": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "extension": {"v_range": [["7.0.2", ""]], "choices": ["wan-extension", "lan-extension"], "type": "str"},
            "id": {"v_range": [["7.0.2", ""]], "required": true, "type": "int"},
            "lan-extension": {
                "v_range": [["7.0.2", ""]],
                "type": "dict",
                "options": {
                    "backhaul": {
                        "v_range": [["7.0.2", ""]],
                        "type": "list",
                        "options": {
                            "name": {"v_range": [["7.0.2", ""]], "type": "str"},
                            "port": {
                                "v_range": [["7.0.2", ""]],
                                "choices": ["wan", "lte1", "lte2", "port1", "port2", "port3", "port4", "port5", "sfp"],
                                "type": "str"
                            },
                            "role": {"v_range": [["7.0.2", ""]], "choices": ["primary", "secondary"], "type": "str"},
                            "weight": {"v_range": [["7.0.2", ""]], "type": "int"}
                        },
                        "elements": "dict"
                    },
                    "backhaul-interface": {"v_range": [["7.0.2", ""]], "type": "str"},
                    "backhaul-ip": {"v_range": [["7.0.2", ""]], "type": "str"},
                    "ipsec-tunnel": {"v_range": [["7.0.2", ""]], "type": "str"},
                    "link-loadbalance": {"v_range": [["7.0.2", ""]], "choices": ["activebackup", "loadbalance"], "type": "str"}
                }
            },
            "login-password": {"v_range": [["7.0.2", ""]], "no_log": true, "type": "raw"},
            "login-password-change": {"v_range": [["7.0.2", ""]], "choices": ["no", "yes", "default"], "type": "str"},
            "model": {
                "v_range": [["7.0.2", ""]],
                "choices": [
                    "FX201E", "FX211E", "FX200F", "FXA11F", "FXE11F", "FXA21F", "FXE21F", "FXA22F", "FXE22F", "FX212F", "FX311F", "FX312F", "FX511F",
                    "FVG21F", "FVA21F", "FVG22F", "FVA22F", "FX04DA", "FX04DN", "FX04DI"
                ],
                "type": "str"
            },
            "name": {"v_range": [["7.0.2", ""]], "type": "str"}
        }
    },
    "extendercontroller_extenderprofile_cellular": {
        "stated": false,
        "options": {
            "controller-report": {
                "v_range": [["7.0.2", ""]],
                "type": "dict",
                "options": {
                    "interval": {"v_range": [["7.0.2", ""]], "type": "int"},
                    "signal-threshold": {"v_range": [["7.0.2", ""]], "type": "int"},
                    "status": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"}
                }
            },
            "dataplan": {"v_range": [["7.0.2", ""]], "type": "raw"},
            "modem1": {
                "v_range": [["7.0.2", ""]],
                "type": "dict",
                "options": {
                    "auto-switch": {
                        "v_range": [["7.0.2", ""]],
                        "type": "dict",
                        "options": {
                            "dataplan": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "disconnect": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "disconnect-period": {"v_range": [["7.0.2", ""]], "type": "int"},
                            "disconnect-threshold": {"v_range": [["7.0.2", ""]], "type": "int"},
                            "signal": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "switch-back": {"v_range": [["7.0.2", ""]], "type": "list", "choices": ["time", "timer"], "elements": "str"},
                            "switch-back-time": {"v_range": [["7.0.2", ""]], "type": "str"},
                            "switch-back-timer": {"v_range": [["7.0.2", ""]], "type": "int"}
                        }
                    },
                    "conn-status": {"v_range": [["7.0.2", ""]], "type": "int"},
                    "default-sim": {"v_range": [["7.0.2", ""]], "choices": ["sim1", "sim2", "carrier", "cost"], "type": "str"},
                    "gps": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "modem-id": {"v_range": [["7.0.2", ""]], "type": "int"},
                    "preferred-carrier": {"v_range": [["7.0.2", ""]], "type": "str"},
                    "redundant-intf": {"v_range": [["7.0.2", ""]], "type": "str"},
                    "redundant-mode": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "sim1-pin": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "sim1-pin-code": {"v_range": [["7.0.2", ""]], "type": "raw"},
                    "sim2-pin": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "sim2-pin-code": {"v_range": [["7.0.2", ""]], "type": "raw"}
                }
            },
            "modem2": {
                "v_range": [["7.0.2", ""]],
                "type": "dict",
                "options": {
                    "auto-switch": {
                        "v_range": [["7.0.2", ""]],
                        "type": "dict",
                        "options": {
                            "dataplan": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "disconnect": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "disconnect-period": {"v_range": [["7.0.2", ""]], "type": "int"},
                            "disconnect-threshold": {"v_range": [["7.0.2", ""]], "type": "int"},
                            "signal": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "switch-back": {"v_range": [["7.0.2", ""]], "type": "list", "choices": ["time", "timer"], "elements": "str"},
                            "switch-back-time": {"v_range": [["7.0.2", ""]], "type": "str"},
                            "switch-back-timer": {"v_range": [["7.0.2", ""]], "type": "int"}
                        }
                    },
                    "conn-status": {"v_range": [["7.0.2", ""]], "type": "int"},
                    "default-sim": {"v_range": [["7.0.2", ""]], "choices": ["sim1", "sim2", "carrier", "cost"], "type": "str"},
                    "gps": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "modem-id": {"v_range": [["7.0.2", ""]], "type": "int"},
                    "preferred-carrier": {"v_range": [["7.0.2", ""]], "type": "str"},
                    "redundant-intf": {"v_range": [["7.0.2", ""]], "type": "str"},
                    "redundant-mode": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "sim1-pin": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "sim1-pin-code": {"v_range": [["7.0.2", ""]], "type": "raw"},
                    "sim2-pin": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "sim2-pin-code": {"v_range": [["7.0.2", ""]], "type": "raw"}
                }
            },
            "sms-notification": {
                "v_range": [["7.0.2", ""]],
                "type": "dict",
                "options": {
                    "alert": {
                        "v_range": [["7.0.2", ""]],
                        "type": "dict",
                        "options": {
                            "data-exhausted": {"v_range": [["7.0.2", ""]], "type": "str"},
                            "fgt-backup-mode-switch": {"v_range": [["7.0.2", ""]], "type": "str"},
                            "low-signal-strength": {"v_range": [["7.0.2", ""]], "type": "str"},
                            "mode-switch": {"v_range": [["7.0.2", ""]], "type": "str"},
                            "os-image-fallback": {"v_range": [["7.0.2", ""]], "type": "str"},
                            "session-disconnect": {"v_range": [["7.0.2", ""]], "type": "str"},
                            "system-reboot": {"v_range": [["7.0.2", ""]], "type": "str"}
                        }
                    },
                    "receiver": {
                        "v_range": [["7.0.2", ""]],
                        "type": "list",
                        "options": {
                            "alert": {
                                "v_range": [["7.0.2", ""]],
                                "type": "list",
                                "choices": [
                                    "system-reboot", "data-exhausted", "session-disconnect", "low-signal-strength", "mode-switch", "os-image-fallback",
                                    "fgt-backup-mode-switch"
                                ],
                                "elements": "str"
                            },
                            "name": {"v_range": [["7.0.2", ""]], "type": "str"},
                            "phone-number": {"v_range": [["7.0.2", ""]], "type": "str"},
                            "status": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"}
                        },
                        "elements": "dict"
                    },
                    "status": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"}
                }
            }
        }
    },
    "extendercontroller_extenderprofile_cellular_controllerreport": {
        "stated": false,
        "options": {
            "interval": {"v_range": [["7.0.2", ""]], "type": "int"},
            "signal-threshold": {"v_range": [["7.0.2", ""]], "type": "int"},
            "status": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "extendercontroller_extenderprofile_cellular_modem1": {
        "stated": false,
        "options": {
            "auto-switch": {
                "v_range": [["7.0.2", ""]],
                "type": "dict",
                "options": {
                    "dataplan": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "disconnect": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "disconnect-period": {"v_range": [["7.0.2", ""]], "type": "int"},
                    "disconnect-threshold": {"v_range": [["7.0.2", ""]], "type": "int"},
                    "signal": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "switch-back": {"v_range": [["7.0.2", ""]], "type": "list", "choices": ["time", "timer"], "elements": "str"},
                    "switch-back-time": {"v_range": [["7.0.2", ""]], "type": "str"},
                    "switch-back-timer": {"v_range": [["7.0.2", ""]], "type": "int"}
                }
            },
            "conn-status": {"v_range": [["7.0.2", ""]], "type": "int"},
            "default-sim": {"v_range": [["7.0.2", ""]], "choices": ["sim1", "sim2", "carrier", "cost"], "type": "str"},
            "gps": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "modem-id": {"v_range": [["7.0.2", ""]], "type": "int"},
            "preferred-carrier": {"v_range": [["7.0.2", ""]], "type": "str"},
            "redundant-intf": {"v_range": [["7.0.2", ""]], "type": "str"},
            "redundant-mode": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "sim1-pin": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "sim1-pin-code": {"v_range": [["7.0.2", ""]], "type": "raw"},
            "sim2-pin": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "sim2-pin-code": {"v_range": [["7.0.2", ""]], "type": "raw"}
        }
    },
    "extendercontroller_extenderprofile_cellular_modem1_autoswitch": {
        "stated": false,
        "options": {
            "dataplan": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "disconnect": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "disconnect-period": {"v_range": [["7.0.2", ""]], "type": "int"},
            "disconnect-threshold": {"v_range": [["7.0.2", ""]], "type": "int"},
            "signal": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "switch-back": {"v_range": [["7.0.2", ""]], "type": "list", "choices": ["time", "timer"], "elements": "str"},
            "switch-back-time": {"v_range": [["7.0.2", ""]], "type": "str"},
            "switch-back-timer": {"v_range": [["7.0.2", ""]], "type": "int"}
        }
    },
    "extendercontroller_extenderprofile_cellular_modem2": {
        "stated": false,
        "options": {
            "auto-switch": {
                "v_range": [["7.0.2", ""]],
                "type": "dict",
                "options": {
                    "dataplan": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "disconnect": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "disconnect-period": {"v_range": [["7.0.2", ""]], "type": "int"},
                    "disconnect-threshold": {"v_range": [["7.0.2", ""]], "type": "int"},
                    "signal": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "switch-back": {"v_range": [["7.0.2", ""]], "type": "list", "choices": ["time", "timer"], "elements": "str"},
                    "switch-back-time": {"v_range": [["7.0.2", ""]], "type": "str"},
                    "switch-back-timer": {"v_range": [["7.0.2", ""]], "type": "int"}
                }
            },
            "conn-status": {"v_range": [["7.0.2", ""]], "type": "int"},
            "default-sim": {"v_range": [["7.0.2", ""]], "choices": ["sim1", "sim2", "carrier", "cost"], "type": "str"},
            "gps": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "modem-id": {"v_range": [["7.0.2", ""]], "type": "int"},
            "preferred-carrier": {"v_range": [["7.0.2", ""]], "type": "str"},
            "redundant-intf": {"v_range": [["7.0.2", ""]], "type": "str"},
            "redundant-mode": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "sim1-pin": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "sim1-pin-code": {"v_range": [["7.0.2", ""]], "type": "raw"},
            "sim2-pin": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "sim2-pin-code": {"v_range": [["7.0.2", ""]], "type": "raw"}
        }
    },
    "extendercontroller_extenderprofile_cellular_modem2_autoswitch": {
        "stated": false,
        "options": {
            "dataplan": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "disconnect": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "disconnect-period": {"v_range": [["7.0.2", ""]], "type": "int"},
            "disconnect-threshold": {"v_range": [["7.0.2", ""]], "type": "int"},
            "signal": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "switch-back": {"v_range": [["7.0.2", ""]], "type": "list", "choices": ["time", "timer"], "elements": "str"},
            "switch-back-time": {"v_range": [["7.0.2", ""]], "type": "str"},
            "switch-back-timer": {"v_range": [["7.0.2", ""]], "type": "int"}
        }
    },
    "extendercontroller_extenderprofile_cellular_smsnotification": {
        "stated": false,
        "options": {
            "alert": {
                "v_range": [["7.0.2", ""]],
                "type": "dict",
                "options": {
                    "data-exhausted": {"v_range": [["7.0.2", ""]], "type": "str"},
                    "fgt-backup-mode-switch": {"v_range": [["7.0.2", ""]], "type": "str"},
                    "low-signal-strength": {"v_range": [["7.0.2", ""]], "type": "str"},
                    "mode-switch": {"v_range": [["7.0.2", ""]], "type": "str"},
                    "os-image-fallback": {"v_range": [["7.0.2", ""]], "type": "str"},
                    "session-disconnect": {"v_range": [["7.0.2", ""]], "type": "str"},
                    "system-reboot": {"v_range": [["7.0.2", ""]], "type": "str"}
                }
            },
            "receiver": {
                "v_range": [["7.0.2", ""]],
                "type": "list",
                "options": {
                    "alert": {
                        "v_range": [["7.0.2", ""]],
                        "type": "list",
                        "choices": [
                            "system-reboot", "data-exhausted", "session-disconnect", "low-signal-strength", "mode-switch", "os-image-fallback",
                            "fgt-backup-mode-switch"
                        ],
                        "elements": "str"
                    },
                    "name": {"v_range": [["7.0.2", ""]], "type": "str"},
                    "phone-number": {"v_range": [["7.0.2", ""]], "type": "str"},
                    "status": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"}
                },
                "elements": "dict"
            },
            "status": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "extendercontroller_extenderprofile_cellular_smsnotification_alert": {
        "stated": false,
        "options": {
            "data-exhausted": {"v_range": [["7.0.2", ""]], "type": "str"},
            "fgt-backup-mode-switch": {"v_range": [["7.0.2", ""]], "type": "str"},
            "low-signal-strength": {"v_range": [["7.0.2", ""]], "type": "str"},
            "mode-switch": {"v_range": [["7.0.2", ""]], "type": "str"},
            "os-image-fallback": {"v_range": [["7.0.2", ""]], "type": "str"},
            "session-disconnect": {"v_range": [["7.0.2", ""]], "type": "str"},
            "system-reboot": {"v_range": [["7.0.2", ""]], "type": "str"}
        }
    },
    "extendercontroller_extenderprofile_cellular_smsnotification_receiver": {
        "stated": true,
        "options": {
            "alert": {
                "v_range": [["7.0.2", ""]],
                "type": "list",
                "choices": [
                    "system-reboot", "data-exhausted", "session-disconnect", "low-signal-strength", "mode-switch", "os-image-fallback",
                    "fgt-backup-mode-switch"
                ],
                "elements": "str"
            },
            "name": {"v_range": [["7.0.2", ""]], "required": true, "type": "str"},
            "phone-number": {"v_range": [["7.0.2", ""]], "type": "str"},
            "status": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "extendercontroller_extenderprofile_lanextension": {
        "stated": false,
        "options": {
            "backhaul": {
                "v_range": [["7.0.2", ""]],
                "type": "list",
                "options": {
                    "name": {"v_range": [["7.0.2", ""]], "type": "str"},
                    "port": {
                        "v_range": [["7.0.2", ""]],
                        "choices": ["wan", "lte1", "lte2", "port1", "port2", "port3", "port4", "port5", "sfp"],
                        "type": "str"
                    },
                    "role": {"v_range": [["7.0.2", ""]], "choices": ["primary", "secondary"], "type": "str"},
                    "weight": {"v_range": [["7.0.2", ""]], "type": "int"}
                },
                "elements": "dict"
            },
            "backhaul-interface": {"v_range": [["7.0.2", ""]], "type": "str"},
            "backhaul-ip": {"v_range": [["7.0.2", ""]], "type": "str"},
            "ipsec-tunnel": {"v_range": [["7.0.2", ""]], "type": "str"},
            "link-loadbalance": {"v_range": [["7.0.2", ""]], "choices": ["activebackup", "loadbalance"], "type": "str"}
        }
    },
    "extendercontroller_extenderprofile_lanextension_backhaul": {
        "stated": true,
        "options": {
            "name": {"v_range": [["7.0.2", ""]], "required": true, "type": "str"},
            "port": {"v_range": [["7.0.2", ""]], "choices": ["wan", "lte1", "lte2", "port1", "port2", "port3", "port4", "port5", "sfp"], "type": "str"},
            "role": {"v_range": [["7.0.2", ""]], "choices": ["primary", "secondary"], "type": "str"},
            "weight": {"v_range": [["7.0.2", ""]], "type": "int"}
        }
    },
    "extendercontroller_simprofile": {
        "stated": true,
        "options": {
            "auto-switch_profile": {
                "v_range": [["6.4.5", ""]],
                "type": "dict",
                "options": {
                    "dataplan": {"v_range": [["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "disconnect": {"v_range": [["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "disconnect-period": {"v_range": [["6.4.5", ""]], "type": "int"},
                    "disconnect-threshold": {"v_range": [["6.4.5", ""]], "type": "int"},
                    "signal": {"v_range": [["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "status": {"v_range": [["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "switch-back": {"v_range": [["6.4.5", ""]], "type": "list", "choices": ["time", "timer"], "elements": "str"},
                    "switch-back-time": {"v_range": [["6.4.5", ""]], "type": "str"},
                    "switch-back-timer": {"v_range": [["6.4.5", ""]], "type": "int"}
                }
            },
            "conn-status": {"v_range": [["6.4.4", ""]], "type": "int"},
            "default-sim": {"v_range": [["6.4.4", ""]], "choices": ["sim1", "sim2", "carrier", "cost"], "type": "str"},
            "description": {"v_range": [["6.4.4", ""]], "type": "str"},
            "gps": {"v_range": [["6.4.4", ""]], "choices": ["disable", "enable"], "type": "str"},
            "modem-id": {"v_range": [["6.4.4", ""]], "type": "int"},
            "name": {"v_range": [["6.4.4", ""]], "required": true, "type": "str"},
            "preferred-carrier": {"v_range": [["6.4.4", ""]], "type": "str"},
            "redundant-intf": {"v_range": [["6.4.4", ""]], "type": "str"},
            "redundant-mode": {"v_range": [["6.4.4", ""]], "choices": ["disable", "enable"], "type": "str"},
            "sim1-pin": {"v_range": [["6.4.4", ""]], "choices": ["disable", "enable"], "type": "str"},
            "sim1-pin-code": {"v_range": [["6.4.4", ""]], "type": "raw"},
            "sim2-pin": {"v_range": [["6.4.4", ""]], "choices": ["disable", "enable"], "type": "str"},
            "sim2-pin-code": {"v_range": [["6.4.4", ""]], "type": "raw"},
            "status": {"v_range": [["6.4.4", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "extendercontroller_simprofile_autoswitchprofile": {
        "stated": false,
        "options": {
            "dataplan": {"v_range": [["6.4.4", ""]], "choices": ["disable", "enable"], "type": "str"},
            "disconnect": {"v_range": [["6.4.4", ""]], "choices": ["disable", "enable"], "type": "str"},
            "disconnect-period": {"v_range": [["6.4.4", ""]], "type": "int"},
            "disconnect-threshold": {"v_range": [["6.4.4", ""]], "type": "int"},
            "signal": {"v_range": [["6.4.4", ""]], "choices": ["disable", "enable"], "type": "str"},
            "status": {"v_range": [["6.4.4", ""]], "choices": ["disable", "enable"], "type": "str"},
            "switch-back": {"v_range": [["6.4.4", ""]], "type": "list", "choices": ["time", "timer"], "elements": "str"},
            "switch-back-time": {"v_range": [["6.4.4", ""]], "type": "str"},
            "switch-back-timer": {"v_range": [["6.4.4", ""]], "type": "int"}
        }
    },
    "extendercontroller_template": {
        "stated": true,
        "options": {
            "dataplan": {"v_range": [["7.0.0", ""]], "type": "raw"},
            "description": {"v_range": [["7.0.0", ""]], "type": "str"},
            "modem1_ifname": {"v_range": [["7.0.0", ""]], "type": "raw"},
            "modem1_sim_profile": {"v_range": [["7.0.0", ""]], "type": "raw"},
            "modem2_ifname": {"v_range": [["7.0.0", ""]], "type": "raw"},
            "modem2_sim_profile": {"v_range": [["7.0.0", ""]], "type": "raw"},
            "name": {"v_range": [["7.0.0", ""]], "required": true, "type": "str"}
        }
    },
    "extensioncontroller_dataplan": {
        "stated": true,
        "options": {
            "apn": {"v_range": [["7.2.1", ""]], "type": "str"},
            "auth-type": {"v_range": [["7.2.1", ""]], "choices": ["none", "pap", "chap"], "type": "str"},
            "billing-date": {"v_range": [["7.2.1", ""]], "type": "int"},
            "capacity": {"v_range": [["7.2.1", ""]], "type": "int"},
            "carrier": {"v_range": [["7.2.1", ""]], "type": "str"},
            "iccid": {"v_range": [["7.2.1", ""]], "type": "str"},
            "modem-id": {"v_range": [["7.2.1", ""]], "choices": ["all", "modem1", "modem2"], "type": "str"},
            "monthly-fee": {"v_range": [["7.2.1", ""]], "type": "int"},
            "name": {"v_range": [["7.2.1", ""]], "required": true, "type": "str"},
            "overage": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "password": {"v_range": [["7.2.1", ""]], "no_log": true, "type": "raw"},
            "pdn": {"v_range": [["7.2.1", ""]], "choices": ["ipv4-only", "ipv6-only", "ipv4-ipv6"], "type": "str"},
            "preferred-subnet": {"v_range": [["7.2.1", ""]], "type": "int"},
            "private-network": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "signal-period": {"v_range": [["7.2.1", ""]], "type": "int"},
            "signal-threshold": {"v_range": [["7.2.1", ""]], "type": "int"},
            "slot": {"v_range": [["7.2.1", ""]], "choices": ["sim1", "sim2"], "type": "str"},
            "type": {"v_range": [["7.2.1", ""]], "choices": ["carrier", "slot", "iccid", "generic"], "type": "str"},
            "username": {"v_range": [["7.2.1", ""]], "type": "str"}
        }
    },
    "extensioncontroller_extenderprofile": {
        "stated": true,
        "options": {
            "allowaccess": {"v_range": [["7.2.1", ""]], "type": "list", "choices": ["https", "ping", "ssh", "snmp", "http", "telnet"], "elements": "str"},
            "bandwidth-limit": {"v_range": [["7.2.1", ""]], "type": "int"},
            "cellular": {
                "v_range": [["7.2.1", ""]],
                "type": "dict",
                "options": {
                    "controller-report": {
                        "v_range": [["7.2.1", ""]],
                        "type": "dict",
                        "options": {
                            "interval": {"v_range": [["7.2.1", ""]], "type": "int"},
                            "signal-threshold": {"v_range": [["7.2.1", ""]], "type": "int"},
                            "status": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"}
                        }
                    },
                    "dataplan": {"v_range": [["7.2.1", ""]], "type": "raw"},
                    "modem1": {
                        "v_range": [["7.2.1", ""]],
                        "type": "dict",
                        "options": {
                            "auto-switch": {
                                "v_range": [["7.2.1", ""]],
                                "type": "dict",
                                "options": {
                                    "dataplan": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                                    "disconnect": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                                    "disconnect-period": {"v_range": [["7.2.1", ""]], "type": "int"},
                                    "disconnect-threshold": {"v_range": [["7.2.1", ""]], "type": "int"},
                                    "signal": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                                    "switch-back": {"v_range": [["7.2.1", ""]], "type": "list", "choices": ["time", "timer"], "elements": "str"},
                                    "switch-back-time": {"v_range": [["7.2.1", ""]], "type": "str"},
                                    "switch-back-timer": {"v_range": [["7.2.1", ""]], "type": "int"}
                                }
                            },
                            "conn-status": {"v_range": [["7.2.1", ""]], "type": "int"},
                            "default-sim": {"v_range": [["7.2.1", ""]], "choices": ["sim1", "sim2", "carrier", "cost"], "type": "str"},
                            "gps": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "modem-id": {"v_range": [["7.2.1", ""]], "type": "int"},
                            "preferred-carrier": {"v_range": [["7.2.1", ""]], "type": "str"},
                            "redundant-intf": {"v_range": [["7.2.1", ""]], "type": "str"},
                            "redundant-mode": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "sim1-pin": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "sim1-pin-code": {"v_range": [["7.2.1", ""]], "type": "raw"},
                            "sim2-pin": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "sim2-pin-code": {"v_range": [["7.2.1", ""]], "type": "raw"}
                        }
                    },
                    "modem2": {
                        "v_range": [["7.2.1", ""]],
                        "type": "dict",
                        "options": {
                            "auto-switch": {
                                "v_range": [["7.2.1", ""]],
                                "type": "dict",
                                "options": {
                                    "dataplan": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                                    "disconnect": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                                    "disconnect-period": {"v_range": [["7.2.1", ""]], "type": "int"},
                                    "disconnect-threshold": {"v_range": [["7.2.1", ""]], "type": "int"},
                                    "signal": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                                    "switch-back": {"v_range": [["7.2.1", ""]], "type": "list", "choices": ["time", "timer"], "elements": "str"},
                                    "switch-back-time": {"v_range": [["7.2.1", ""]], "type": "str"},
                                    "switch-back-timer": {"v_range": [["7.2.1", ""]], "type": "int"}
                                }
                            },
                            "conn-status": {"v_range": [["7.2.1", ""]], "type": "int"},
                            "default-sim": {"v_range": [["7.2.1", ""]], "choices": ["sim1", "sim2", "carrier", "cost"], "type": "str"},
                            "gps": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "modem-id": {"v_range": [["7.2.1", ""]], "type": "int"},
                            "preferred-carrier": {"v_range": [["7.2.1", ""]], "type": "str"},
                            "redundant-intf": {"v_range": [["7.2.1", ""]], "type": "str"},
                            "redundant-mode": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "sim1-pin": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "sim1-pin-code": {"v_range": [["7.2.1", ""]], "type": "raw"},
                            "sim2-pin": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "sim2-pin-code": {"v_range": [["7.2.1", ""]], "type": "raw"}
                        }
                    },
                    "sms-notification": {
                        "v_range": [["7.2.1", ""]],
                        "type": "dict",
                        "options": {
                            "alert": {
                                "v_range": [["7.2.1", ""]],
                                "type": "dict",
                                "options": {
                                    "data-exhausted": {"v_range": [["7.2.1", ""]], "type": "str"},
                                    "fgt-backup-mode-switch": {"v_range": [["7.2.1", ""]], "type": "str"},
                                    "low-signal-strength": {"v_range": [["7.2.1", ""]], "type": "str"},
                                    "mode-switch": {"v_range": [["7.2.1", ""]], "type": "str"},
                                    "os-image-fallback": {"v_range": [["7.2.1", ""]], "type": "str"},
                                    "session-disconnect": {"v_range": [["7.2.1", ""]], "type": "str"},
                                    "system-reboot": {"v_range": [["7.2.1", ""]], "type": "str"}
                                }
                            },
                            "receiver": {
                                "v_range": [["7.2.1", ""]],
                                "type": "list",
                                "options": {
                                    "alert": {
                                        "v_range": [["7.2.1", ""]],
                                        "type": "list",
                                        "choices": [
                                            "system-reboot", "data-exhausted", "session-disconnect", "low-signal-strength", "mode-switch",
                                            "os-image-fallback", "fgt-backup-mode-switch"
                                        ],
                                        "elements": "str"
                                    },
                                    "name": {"v_range": [["7.2.1", ""]], "type": "str"},
                                    "phone-number": {"v_range": [["7.2.1", ""]], "type": "str"},
                                    "status": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"}
                                },
                                "elements": "dict"
                            },
                            "status": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"}
                        }
                    }
                }
            },
            "enforce-bandwidth": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "extension": {"v_range": [["7.2.1", ""]], "choices": ["wan-extension", "lan-extension"], "type": "str"},
            "id": {"v_range": [["7.2.1", ""]], "required": true, "type": "int"},
            "lan-extension": {
                "v_range": [["7.2.1", ""]],
                "type": "dict",
                "options": {
                    "backhaul": {
                        "v_range": [["7.2.1", ""]],
                        "type": "list",
                        "options": {
                            "name": {"v_range": [["7.2.1", ""]], "type": "str"},
                            "port": {
                                "v_range": [["7.2.1", ""]],
                                "choices": ["wan", "lte1", "lte2", "port1", "port2", "port3", "port4", "port5", "sfp"],
                                "type": "str"
                            },
                            "role": {"v_range": [["7.2.1", ""]], "choices": ["primary", "secondary"], "type": "str"},
                            "weight": {"v_range": [["7.2.1", ""]], "type": "int"}
                        },
                        "elements": "dict"
                    },
                    "backhaul-interface": {"v_range": [["7.2.1", ""]], "type": "str"},
                    "backhaul-ip": {"v_range": [["7.2.1", ""]], "type": "str"},
                    "ipsec-tunnel": {"v_range": [["7.2.1", ""]], "type": "str"},
                    "link-loadbalance": {"v_range": [["7.2.1", ""]], "choices": ["activebackup", "loadbalance"], "type": "str"},
                    "downlinks": {
                        "v_range": [["7.6.0", ""]],
                        "type": "list",
                        "options": {
                            "name": {"v_range": [["7.6.0", ""]], "type": "str"},
                            "port": {"v_range": [["7.6.0", ""]], "choices": ["port1", "port2", "port3", "port4", "port5", "lan1", "lan2"], "type": "str"},
                            "pvid": {"v_range": [["7.6.0", ""]], "type": "int"},
                            "type": {"v_range": [["7.6.0", ""]], "choices": ["port", "vap"], "type": "str"},
                            "vap": {"v_range": [["7.6.0", ""]], "type": "raw"}
                        },
                        "elements": "dict"
                    }
                }
            },
            "login-password": {"v_range": [["7.2.1", ""]], "no_log": true, "type": "raw"},
            "login-password-change": {"v_range": [["7.2.1", ""]], "choices": ["no", "yes", "default"], "type": "str"},
            "model": {
                "v_range": [["7.2.1", ""]],
                "choices": [
                    "FX201E", "FX211E", "FX200F", "FXA11F", "FXE11F", "FXA21F", "FXE21F", "FXA22F", "FXE22F", "FX212F", "FX311F", "FX312F", "FX511F",
                    "FVG21F", "FVA21F", "FVG22F", "FVA22F", "FX04DA", "FX04DN", "FX04DI", "FXR51G", "FG", "BS10FW", "BS20GW", "BS20GN", "FXN51G",
                    "FXW51G"
                ],
                "type": "str"
            },
            "name": {"v_range": [["7.2.1", ""]], "type": "str"},
            "_is_factory_setting": {"v_range": [["7.4.0", ""]], "choices": ["disable", "enable", "ext"], "type": "str"},
            "wifi": {
                "v_range": [["7.4.3", ""]],
                "type": "dict",
                "options": {
                    "DFS": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "country": {
                        "v_range": [["7.4.3", ""]],
                        "choices": [
                            "AL", "DZ", "AR", "AM", "AU", "AT", "AZ", "BH", "BD", "BY", "BE", "BZ", "BO", "BA", "BR", "BN", "BG", "CA", "CL", "CN", "CO",
                            "CR", "HR", "CY", "CZ", "DK", "DO", "EC", "EG", "SV", "EE", "FI", "FR", "GE", "DE", "GR", "GT", "HN", "HK", "HU", "IS", "IN",
                            "ID", "IE", "IL", "IT", "JM", "JP", "JO", "KZ", "KE", "KR", "KW", "LV", "LB", "LI", "LT", "LU", "MO", "MK", "MY", "MT", "MX",
                            "MC", "MA", "NP", "NL", "AN", "NZ", "NO", "OM", "PK", "PA", "PG", "PE", "PH", "PL", "PT", "PR", "QA", "RO", "RU", "SA", "SG",
                            "SK", "SI", "ZA", "ES", "LK", "SE", "CH", "TW", "TH", "TT", "TN", "TR", "AE", "UA", "GB", "US", "PS", "UY", "UZ", "VE", "VN",
                            "YE", "ZW", "NA", "BS", "VC", "KH", "MV", "AF", "NG", "TZ", "ZM", "SN", "CI", "GH", "CM", "MW", "AO", "GA", "ML", "BJ", "MG",
                            "TD", "BW", "LY", "RW", "MZ", "GM", "LS", "MU", "CG", "UG", "BF", "SL", "SO", "CD", "NE", "CF", "SZ", "TG", "LR", "MR", "DJ",
                            "RE", "RS", "ME", "IQ", "MD", "KY", "BB", "BM", "TC", "VI", "PM", "MF", "GD", "IM", "FO", "GI", "GL", "TM", "MN", "VU", "FJ",
                            "LA", "GU", "WF", "MH", "BT", "FM", "PF", "NI", "PY", "HT", "GY", "AW", "KN", "GF", "AS", "MP", "PW", "MM", "LC", "GP", "ET",
                            "SR", "CX", "DM", "MQ", "YT", "BL", "--"
                        ],
                        "type": "str"
                    },
                    "radio-1": {
                        "v_range": [["7.4.3", ""]],
                        "type": "dict",
                        "options": {
                            "80211d": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "band": {"v_range": [["7.4.3", ""]], "choices": ["2.4GHz"], "type": "str"},
                            "bandwidth": {"v_range": [["7.4.3", ""]], "choices": ["auto", "20MHz", "40MHz", "80MHz"], "type": "str"},
                            "beacon-interval": {"v_range": [["7.4.3", ""]], "type": "int"},
                            "bss-color": {"v_range": [["7.4.3", ""]], "type": "int"},
                            "bss-color-mode": {"v_range": [["7.4.3", ""]], "choices": ["auto", "static"], "type": "str"},
                            "channel": {
                                "v_range": [["7.4.3", ""]],
                                "type": "list",
                                "choices": ["CH1", "CH2", "CH3", "CH4", "CH5", "CH6", "CH7", "CH8", "CH9", "CH10", "CH11"],
                                "elements": "str"
                            },
                            "extension-channel": {"v_range": [["7.4.3", ""]], "choices": ["auto", "higher", "lower"], "type": "str"},
                            "guard-interval": {"v_range": [["7.4.3", ""]], "choices": ["auto", "400ns", "800ns"], "type": "str"},
                            "lan-ext-vap": {"v_range": [["7.4.3", ""]], "type": "raw"},
                            "local-vaps": {"v_range": [["7.4.3", ""]], "type": "raw"},
                            "max-clients": {"v_range": [["7.4.3", ""]], "type": "int"},
                            "mode": {"v_range": [["7.4.3", ""]], "choices": ["AP", "Client"], "type": "str"},
                            "operating-standard": {
                                "v_range": [["7.4.3", ""]],
                                "choices": [
                                    "auto", "11A-N-AC-AX", "11A-N-AC", "11A-N", "11A", "11N-AC-AX", "11AC-AX", "11AC", "11N-AC", "11B-G-N-AX", "11B-G-N",
                                    "11B-G", "11B", "11G-N-AX", "11N-AX", "11AX", "11G-N", "11N", "11G"
                                ],
                                "type": "str"
                            },
                            "power-level": {"v_range": [["7.4.3", ""]], "type": "int"},
                            "radio-id": {"v_range": [["7.4.3", ""]], "type": "int"},
                            "status": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"}
                        }
                    },
                    "radio-2": {
                        "v_range": [["7.4.3", ""]],
                        "type": "dict",
                        "options": {
                            "80211d": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "band": {"v_range": [["7.4.3", ""]], "choices": ["5GHz"], "type": "str"},
                            "bandwidth": {"v_range": [["7.4.3", ""]], "choices": ["auto", "20MHz", "40MHz", "80MHz"], "type": "str"},
                            "beacon-interval": {"v_range": [["7.4.3", ""]], "type": "int"},
                            "bss-color": {"v_range": [["7.4.3", ""]], "type": "int"},
                            "bss-color-mode": {"v_range": [["7.4.3", ""]], "choices": ["auto", "static"], "type": "str"},
                            "channel": {
                                "v_range": [["7.4.3", ""]],
                                "type": "list",
                                "choices": [
                                    "CH36", "CH40", "CH44", "CH48", "CH52", "CH56", "CH60", "CH64", "CH100", "CH104", "CH108", "CH112", "CH116", "CH120",
                                    "CH124", "CH128", "CH132", "CH136", "CH140", "CH144", "CH149", "CH153", "CH157", "CH161", "CH165"
                                ],
                                "elements": "str"
                            },
                            "extension-channel": {"v_range": [["7.4.3", ""]], "choices": ["auto", "higher", "lower"], "type": "str"},
                            "guard-interval": {"v_range": [["7.4.3", ""]], "choices": ["auto", "400ns", "800ns"], "type": "str"},
                            "lan-ext-vap": {"v_range": [["7.4.3", ""]], "type": "raw"},
                            "local-vaps": {"v_range": [["7.4.3", ""]], "type": "raw"},
                            "max-clients": {"v_range": [["7.4.3", ""]], "type": "int"},
                            "mode": {"v_range": [["7.4.3", ""]], "choices": ["AP", "Client"], "type": "str"},
                            "operating-standard": {
                                "v_range": [["7.4.3", ""]],
                                "choices": [
                                    "auto", "11A-N-AC-AX", "11A-N-AC", "11A-N", "11A", "11N-AC-AX", "11AC-AX", "11AC", "11N-AC", "11B-G-N-AX", "11B-G-N",
                                    "11B-G", "11B", "11G-N-AX", "11N-AX", "11AX", "11G-N", "11N", "11G"
                                ],
                                "type": "str"
                            },
                            "power-level": {"v_range": [["7.4.3", ""]], "type": "int"},
                            "radio-id": {"v_range": [["7.4.3", ""]], "type": "int"},
                            "status": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"}
                        }
                    }
                }
            }
        }
    },
    "extensioncontroller_extenderprofile_cellular": {
        "stated": false,
        "options": {
            "controller-report": {
                "v_range": [["7.2.1", ""]],
                "type": "dict",
                "options": {
                    "interval": {"v_range": [["7.2.1", ""]], "type": "int"},
                    "signal-threshold": {"v_range": [["7.2.1", ""]], "type": "int"},
                    "status": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"}
                }
            },
            "dataplan": {"v_range": [["7.2.1", ""]], "type": "raw"},
            "modem1": {
                "v_range": [["7.2.1", ""]],
                "type": "dict",
                "options": {
                    "auto-switch": {
                        "v_range": [["7.2.1", ""]],
                        "type": "dict",
                        "options": {
                            "dataplan": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "disconnect": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "disconnect-period": {"v_range": [["7.2.1", ""]], "type": "int"},
                            "disconnect-threshold": {"v_range": [["7.2.1", ""]], "type": "int"},
                            "signal": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "switch-back": {"v_range": [["7.2.1", ""]], "type": "list", "choices": ["time", "timer"], "elements": "str"},
                            "switch-back-time": {"v_range": [["7.2.1", ""]], "type": "str"},
                            "switch-back-timer": {"v_range": [["7.2.1", ""]], "type": "int"}
                        }
                    },
                    "conn-status": {"v_range": [["7.2.1", ""]], "type": "int"},
                    "default-sim": {"v_range": [["7.2.1", ""]], "choices": ["sim1", "sim2", "carrier", "cost"], "type": "str"},
                    "gps": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "modem-id": {"v_range": [["7.2.1", ""]], "type": "int"},
                    "preferred-carrier": {"v_range": [["7.2.1", ""]], "type": "str"},
                    "redundant-intf": {"v_range": [["7.2.1", ""]], "type": "str"},
                    "redundant-mode": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "sim1-pin": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "sim1-pin-code": {"v_range": [["7.2.1", ""]], "type": "raw"},
                    "sim2-pin": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "sim2-pin-code": {"v_range": [["7.2.1", ""]], "type": "raw"}
                }
            },
            "modem2": {
                "v_range": [["7.2.1", ""]],
                "type": "dict",
                "options": {
                    "auto-switch": {
                        "v_range": [["7.2.1", ""]],
                        "type": "dict",
                        "options": {
                            "dataplan": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "disconnect": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "disconnect-period": {"v_range": [["7.2.1", ""]], "type": "int"},
                            "disconnect-threshold": {"v_range": [["7.2.1", ""]], "type": "int"},
                            "signal": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "switch-back": {"v_range": [["7.2.1", ""]], "type": "list", "choices": ["time", "timer"], "elements": "str"},
                            "switch-back-time": {"v_range": [["7.2.1", ""]], "type": "str"},
                            "switch-back-timer": {"v_range": [["7.2.1", ""]], "type": "int"}
                        }
                    },
                    "conn-status": {"v_range": [["7.2.1", ""]], "type": "int"},
                    "default-sim": {"v_range": [["7.2.1", ""]], "choices": ["sim1", "sim2", "carrier", "cost"], "type": "str"},
                    "gps": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "modem-id": {"v_range": [["7.2.1", ""]], "type": "int"},
                    "preferred-carrier": {"v_range": [["7.2.1", ""]], "type": "str"},
                    "redundant-intf": {"v_range": [["7.2.1", ""]], "type": "str"},
                    "redundant-mode": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "sim1-pin": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "sim1-pin-code": {"v_range": [["7.2.1", ""]], "type": "raw"},
                    "sim2-pin": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "sim2-pin-code": {"v_range": [["7.2.1", ""]], "type": "raw"}
                }
            },
            "sms-notification": {
                "v_range": [["7.2.1", ""]],
                "type": "dict",
                "options": {
                    "alert": {
                        "v_range": [["7.2.1", ""]],
                        "type": "dict",
                        "options": {
                            "data-exhausted": {"v_range": [["7.2.1", ""]], "type": "str"},
                            "fgt-backup-mode-switch": {"v_range": [["7.2.1", ""]], "type": "str"},
                            "low-signal-strength": {"v_range": [["7.2.1", ""]], "type": "str"},
                            "mode-switch": {"v_range": [["7.2.1", ""]], "type": "str"},
                            "os-image-fallback": {"v_range": [["7.2.1", ""]], "type": "str"},
                            "session-disconnect": {"v_range": [["7.2.1", ""]], "type": "str"},
                            "system-reboot": {"v_range": [["7.2.1", ""]], "type": "str"}
                        }
                    },
                    "receiver": {
                        "v_range": [["7.2.1", ""]],
                        "type": "list",
                        "options": {
                            "alert": {
                                "v_range": [["7.2.1", ""]],
                                "type": "list",
                                "choices": [
                                    "system-reboot", "data-exhausted", "session-disconnect", "low-signal-strength", "mode-switch", "os-image-fallback",
                                    "fgt-backup-mode-switch"
                                ],
                                "elements": "str"
                            },
                            "name": {"v_range": [["7.2.1", ""]], "type": "str"},
                            "phone-number": {"v_range": [["7.2.1", ""]], "type": "str"},
                            "status": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"}
                        },
                        "elements": "dict"
                    },
                    "status": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"}
                }
            }
        }
    },
    "extensioncontroller_extenderprofile_cellular_controllerreport": {
        "stated": false,
        "options": {
            "interval": {"v_range": [["7.2.1", ""]], "type": "int"},
            "signal-threshold": {"v_range": [["7.2.1", ""]], "type": "int"},
            "status": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "extensioncontroller_extenderprofile_cellular_modem1": {
        "stated": false,
        "options": {
            "auto-switch": {
                "v_range": [["7.2.1", ""]],
                "type": "dict",
                "options": {
                    "dataplan": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "disconnect": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "disconnect-period": {"v_range": [["7.2.1", ""]], "type": "int"},
                    "disconnect-threshold": {"v_range": [["7.2.1", ""]], "type": "int"},
                    "signal": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "switch-back": {"v_range": [["7.2.1", ""]], "type": "list", "choices": ["time", "timer"], "elements": "str"},
                    "switch-back-time": {"v_range": [["7.2.1", ""]], "type": "str"},
                    "switch-back-timer": {"v_range": [["7.2.1", ""]], "type": "int"}
                }
            },
            "conn-status": {"v_range": [["7.2.1", ""]], "type": "int"},
            "default-sim": {"v_range": [["7.2.1", ""]], "choices": ["sim1", "sim2", "carrier", "cost"], "type": "str"},
            "gps": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "modem-id": {"v_range": [["7.2.1", ""]], "type": "int"},
            "preferred-carrier": {"v_range": [["7.2.1", ""]], "type": "str"},
            "redundant-intf": {"v_range": [["7.2.1", ""]], "type": "str"},
            "redundant-mode": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "sim1-pin": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "sim1-pin-code": {"v_range": [["7.2.1", ""]], "type": "raw"},
            "sim2-pin": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "sim2-pin-code": {"v_range": [["7.2.1", ""]], "type": "raw"}
        }
    },
    "extensioncontroller_extenderprofile_cellular_modem1_autoswitch": {
        "stated": false,
        "options": {
            "dataplan": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "disconnect": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "disconnect-period": {"v_range": [["7.2.1", ""]], "type": "int"},
            "disconnect-threshold": {"v_range": [["7.2.1", ""]], "type": "int"},
            "signal": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "switch-back": {"v_range": [["7.2.1", ""]], "type": "list", "choices": ["time", "timer"], "elements": "str"},
            "switch-back-time": {"v_range": [["7.2.1", ""]], "type": "str"},
            "switch-back-timer": {"v_range": [["7.2.1", ""]], "type": "int"}
        }
    },
    "extensioncontroller_extenderprofile_cellular_modem2": {
        "stated": false,
        "options": {
            "auto-switch": {
                "v_range": [["7.2.1", ""]],
                "type": "dict",
                "options": {
                    "dataplan": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "disconnect": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "disconnect-period": {"v_range": [["7.2.1", ""]], "type": "int"},
                    "disconnect-threshold": {"v_range": [["7.2.1", ""]], "type": "int"},
                    "signal": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "switch-back": {"v_range": [["7.2.1", ""]], "type": "list", "choices": ["time", "timer"], "elements": "str"},
                    "switch-back-time": {"v_range": [["7.2.1", ""]], "type": "str"},
                    "switch-back-timer": {"v_range": [["7.2.1", ""]], "type": "int"}
                }
            },
            "conn-status": {"v_range": [["7.2.1", ""]], "type": "int"},
            "default-sim": {"v_range": [["7.2.1", ""]], "choices": ["sim1", "sim2", "carrier", "cost"], "type": "str"},
            "gps": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "modem-id": {"v_range": [["7.2.1", ""]], "type": "int"},
            "preferred-carrier": {"v_range": [["7.2.1", ""]], "type": "str"},
            "redundant-intf": {"v_range": [["7.2.1", ""]], "type": "str"},
            "redundant-mode": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "sim1-pin": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "sim1-pin-code": {"v_range": [["7.2.1", ""]], "type": "raw"},
            "sim2-pin": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "sim2-pin-code": {"v_range": [["7.2.1", ""]], "type": "raw"}
        }
    },
    "extensioncontroller_extenderprofile_cellular_modem2_autoswitch": {
        "stated": false,
        "options": {
            "dataplan": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "disconnect": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "disconnect-period": {"v_range": [["7.2.1", ""]], "type": "int"},
            "disconnect-threshold": {"v_range": [["7.2.1", ""]], "type": "int"},
            "signal": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "switch-back": {"v_range": [["7.2.1", ""]], "type": "list", "choices": ["time", "timer"], "elements": "str"},
            "switch-back-time": {"v_range": [["7.2.1", ""]], "type": "str"},
            "switch-back-timer": {"v_range": [["7.2.1", ""]], "type": "int"}
        }
    },
    "extensioncontroller_extenderprofile_cellular_smsnotification": {
        "stated": false,
        "options": {
            "alert": {
                "v_range": [["7.2.1", ""]],
                "type": "dict",
                "options": {
                    "data-exhausted": {"v_range": [["7.2.1", ""]], "type": "str"},
                    "fgt-backup-mode-switch": {"v_range": [["7.2.1", ""]], "type": "str"},
                    "low-signal-strength": {"v_range": [["7.2.1", ""]], "type": "str"},
                    "mode-switch": {"v_range": [["7.2.1", ""]], "type": "str"},
                    "os-image-fallback": {"v_range": [["7.2.1", ""]], "type": "str"},
                    "session-disconnect": {"v_range": [["7.2.1", ""]], "type": "str"},
                    "system-reboot": {"v_range": [["7.2.1", ""]], "type": "str"}
                }
            },
            "receiver": {
                "v_range": [["7.2.1", ""]],
                "type": "list",
                "options": {
                    "alert": {
                        "v_range": [["7.2.1", ""]],
                        "type": "list",
                        "choices": [
                            "system-reboot", "data-exhausted", "session-disconnect", "low-signal-strength", "mode-switch", "os-image-fallback",
                            "fgt-backup-mode-switch"
                        ],
                        "elements": "str"
                    },
                    "name": {"v_range": [["7.2.1", ""]], "type": "str"},
                    "phone-number": {"v_range": [["7.2.1", ""]], "type": "str"},
                    "status": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"}
                },
                "elements": "dict"
            },
            "status": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "extensioncontroller_extenderprofile_cellular_smsnotification_alert": {
        "stated": false,
        "options": {
            "data-exhausted": {"v_range": [["7.2.1", ""]], "type": "str"},
            "fgt-backup-mode-switch": {"v_range": [["7.2.1", ""]], "type": "str"},
            "low-signal-strength": {"v_range": [["7.2.1", ""]], "type": "str"},
            "mode-switch": {"v_range": [["7.2.1", ""]], "type": "str"},
            "os-image-fallback": {"v_range": [["7.2.1", ""]], "type": "str"},
            "session-disconnect": {"v_range": [["7.2.1", ""]], "type": "str"},
            "system-reboot": {"v_range": [["7.2.1", ""]], "type": "str"}
        }
    },
    "extensioncontroller_extenderprofile_cellular_smsnotification_receiver": {
        "stated": true,
        "options": {
            "alert": {
                "v_range": [["7.2.1", ""]],
                "type": "list",
                "choices": [
                    "system-reboot", "data-exhausted", "session-disconnect", "low-signal-strength", "mode-switch", "os-image-fallback",
                    "fgt-backup-mode-switch"
                ],
                "elements": "str"
            },
            "name": {"v_range": [["7.2.1", ""]], "required": true, "type": "str"},
            "phone-number": {"v_range": [["7.2.1", ""]], "type": "str"},
            "status": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "extensioncontroller_extenderprofile_lanextension": {
        "stated": false,
        "options": {
            "backhaul": {
                "v_range": [["7.2.1", ""]],
                "type": "list",
                "options": {
                    "name": {"v_range": [["7.2.1", ""]], "type": "str"},
                    "port": {
                        "v_range": [["7.2.1", ""]],
                        "choices": ["wan", "lte1", "lte2", "port1", "port2", "port3", "port4", "port5", "sfp"],
                        "type": "str"
                    },
                    "role": {"v_range": [["7.2.1", ""]], "choices": ["primary", "secondary"], "type": "str"},
                    "weight": {"v_range": [["7.2.1", ""]], "type": "int"}
                },
                "elements": "dict"
            },
            "backhaul-interface": {"v_range": [["7.2.1", ""]], "type": "str"},
            "backhaul-ip": {"v_range": [["7.2.1", ""]], "type": "str"},
            "ipsec-tunnel": {"v_range": [["7.2.1", ""]], "type": "str"},
            "link-loadbalance": {"v_range": [["7.2.1", ""]], "choices": ["activebackup", "loadbalance"], "type": "str"},
            "downlinks": {
                "v_range": [["7.6.0", ""]],
                "type": "list",
                "options": {
                    "name": {"v_range": [["7.6.0", ""]], "type": "str"},
                    "port": {"v_range": [["7.6.0", ""]], "choices": ["port1", "port2", "port3", "port4", "port5", "lan1", "lan2"], "type": "str"},
                    "pvid": {"v_range": [["7.6.0", ""]], "type": "int"},
                    "type": {"v_range": [["7.6.0", ""]], "choices": ["port", "vap"], "type": "str"},
                    "vap": {"v_range": [["7.6.0", ""]], "type": "raw"}
                },
                "elements": "dict"
            }
        }
    },
    "extensioncontroller_extenderprofile_lanextension_backhaul": {
        "stated": true,
        "options": {
            "name": {"v_range": [["7.2.1", ""]], "required": true, "type": "str"},
            "port": {"v_range": [["7.2.1", ""]], "choices": ["wan", "lte1", "lte2", "port1", "port2", "port3", "port4", "port5", "sfp"], "type": "str"},
            "role": {"v_range": [["7.2.1", ""]], "choices": ["primary", "secondary"], "type": "str"},
            "weight": {"v_range": [["7.2.1", ""]], "type": "int"}
        }
    },
    "extensioncontroller_extenderprofile_wifi": {
        "stated": false,
        "options": {
            "DFS": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "country": {
                "v_range": [["7.4.3", ""]],
                "choices": [
                    "AL", "DZ", "AR", "AM", "AU", "AT", "AZ", "BH", "BD", "BY", "BE", "BZ", "BO", "BA", "BR", "BN", "BG", "CA", "CL", "CN", "CO", "CR",
                    "HR", "CY", "CZ", "DK", "DO", "EC", "EG", "SV", "EE", "FI", "FR", "GE", "DE", "GR", "GT", "HN", "HK", "HU", "IS", "IN", "ID", "IE",
                    "IL", "IT", "JM", "JP", "JO", "KZ", "KE", "KR", "KW", "LV", "LB", "LI", "LT", "LU", "MO", "MK", "MY", "MT", "MX", "MC", "MA", "NP",
                    "NL", "AN", "NZ", "NO", "OM", "PK", "PA", "PG", "PE", "PH", "PL", "PT", "PR", "QA", "RO", "RU", "SA", "SG", "SK", "SI", "ZA", "ES",
                    "LK", "SE", "CH", "TW", "TH", "TT", "TN", "TR", "AE", "UA", "GB", "US", "PS", "UY", "UZ", "VE", "VN", "YE", "ZW", "NA", "BS", "VC",
                    "KH", "MV", "AF", "NG", "TZ", "ZM", "SN", "CI", "GH", "CM", "MW", "AO", "GA", "ML", "BJ", "MG", "TD", "BW", "LY", "RW", "MZ", "GM",
                    "LS", "MU", "CG", "UG", "BF", "SL", "SO", "CD", "NE", "CF", "SZ", "TG", "LR", "MR", "DJ", "RE", "RS", "ME", "IQ", "MD", "KY", "BB",
                    "BM", "TC", "VI", "PM", "MF", "GD", "IM", "FO", "GI", "GL", "TM", "MN", "VU", "FJ", "LA", "GU", "WF", "MH", "BT", "FM", "PF", "NI",
                    "PY", "HT", "GY", "AW", "KN", "GF", "AS", "MP", "PW", "MM", "LC", "GP", "ET", "SR", "CX", "DM", "MQ", "YT", "BL", "--"
                ],
                "type": "str"
            },
            "radio-1": {
                "v_range": [["7.4.3", ""]],
                "type": "dict",
                "options": {
                    "80211d": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "band": {"v_range": [["7.4.3", ""]], "choices": ["2.4GHz"], "type": "str"},
                    "bandwidth": {"v_range": [["7.4.3", ""]], "choices": ["auto", "20MHz", "40MHz", "80MHz"], "type": "str"},
                    "beacon-interval": {"v_range": [["7.4.3", ""]], "type": "int"},
                    "bss-color": {"v_range": [["7.4.3", ""]], "type": "int"},
                    "bss-color-mode": {"v_range": [["7.4.3", ""]], "choices": ["auto", "static"], "type": "str"},
                    "channel": {
                        "v_range": [["7.4.3", ""]],
                        "type": "list",
                        "choices": ["CH1", "CH2", "CH3", "CH4", "CH5", "CH6", "CH7", "CH8", "CH9", "CH10", "CH11"],
                        "elements": "str"
                    },
                    "extension-channel": {"v_range": [["7.4.3", ""]], "choices": ["auto", "higher", "lower"], "type": "str"},
                    "guard-interval": {"v_range": [["7.4.3", ""]], "choices": ["auto", "400ns", "800ns"], "type": "str"},
                    "lan-ext-vap": {"v_range": [["7.4.3", ""]], "type": "list", "elements": "str"},
                    "local-vaps": {"v_range": [["7.4.3", ""]], "type": "list", "elements": "str"},
                    "max-clients": {"v_range": [["7.4.3", ""]], "type": "int"},
                    "mode": {"v_range": [["7.4.3", ""]], "choices": ["AP", "Client"], "type": "str"},
                    "operating-standard": {
                        "v_range": [["7.4.3", ""]],
                        "choices": [
                            "auto", "11A-N-AC-AX", "11A-N-AC", "11A-N", "11A", "11N-AC-AX", "11AC-AX", "11AC", "11N-AC", "11B-G-N-AX", "11B-G-N",
                            "11B-G", "11B", "11G-N-AX", "11N-AX", "11AX", "11G-N", "11N", "11G"
                        ],
                        "type": "str"
                    },
                    "power-level": {"v_range": [["7.4.3", ""]], "type": "int"},
                    "radio-id": {"v_range": [["7.4.3", ""]], "type": "int"},
                    "status": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"}
                }
            },
            "radio-2": {
                "v_range": [["7.4.3", ""]],
                "type": "dict",
                "options": {
                    "80211d": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "band": {"v_range": [["7.4.3", ""]], "choices": ["5GHz"], "type": "str"},
                    "bandwidth": {"v_range": [["7.4.3", ""]], "choices": ["auto", "20MHz", "40MHz", "80MHz"], "type": "str"},
                    "beacon-interval": {"v_range": [["7.4.3", ""]], "type": "int"},
                    "bss-color": {"v_range": [["7.4.3", ""]], "type": "int"},
                    "bss-color-mode": {"v_range": [["7.4.3", ""]], "choices": ["auto", "static"], "type": "str"},
                    "channel": {
                        "v_range": [["7.4.3", ""]],
                        "type": "list",
                        "choices": [
                            "CH36", "CH40", "CH44", "CH48", "CH52", "CH56", "CH60", "CH64", "CH100", "CH104", "CH108", "CH112", "CH116", "CH120",
                            "CH124", "CH128", "CH132", "CH136", "CH140", "CH144", "CH149", "CH153", "CH157", "CH161", "CH165"
                        ],
                        "elements": "str"
                    },
                    "extension-channel": {"v_range": [["7.4.3", ""]], "choices": ["auto", "higher", "lower"], "type": "str"},
                    "guard-interval": {"v_range": [["7.4.3", ""]], "choices": ["auto", "400ns", "800ns"], "type": "str"},
                    "lan-ext-vap": {"v_range": [["7.4.3", ""]], "type": "list", "elements": "str"},
                    "local-vaps": {"v_range": [["7.4.3", ""]], "type": "list", "elements": "str"},
                    "max-clients": {"v_range": [["7.4.3", ""]], "type": "int"},
                    "mode": {"v_range": [["7.4.3", ""]], "choices": ["AP", "Client"], "type": "str"},
                    "operating-standard": {
                        "v_range": [["7.4.3", ""]],
                        "choices": [
                            "auto", "11A-N-AC-AX", "11A-N-AC", "11A-N", "11A", "11N-AC-AX", "11AC-AX", "11AC", "11N-AC", "11B-G-N-AX", "11B-G-N",
                            "11B-G", "11B", "11G-N-AX", "11N-AX", "11AX", "11G-N", "11N", "11G"
                        ],
                        "type": "str"
                    },
                    "power-level": {"v_range": [["7.4.3", ""]], "type": "int"},
                    "radio-id": {"v_range": [["7.4.3", ""]], "type": "int"},
                    "status": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"}
                }
            }
        }
    },
    "extensioncontroller_extenderprofile_wifi_radio1": {
        "stated": false,
        "options": {
            "80211d": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "band": {"v_range": [["7.4.3", ""]], "choices": ["2.4GHz"], "type": "str"},
            "bandwidth": {"v_range": [["7.4.3", ""]], "choices": ["auto", "20MHz", "40MHz", "80MHz"], "type": "str"},
            "beacon-interval": {"v_range": [["7.4.3", ""]], "type": "int"},
            "bss-color": {"v_range": [["7.4.3", ""]], "type": "int"},
            "bss-color-mode": {"v_range": [["7.4.3", ""]], "choices": ["auto", "static"], "type": "str"},
            "channel": {
                "v_range": [["7.4.3", ""]],
                "type": "list",
                "choices": ["CH1", "CH2", "CH3", "CH4", "CH5", "CH6", "CH7", "CH8", "CH9", "CH10", "CH11"],
                "elements": "str"
            },
            "extension-channel": {"v_range": [["7.4.3", ""]], "choices": ["auto", "higher", "lower"], "type": "str"},
            "guard-interval": {"v_range": [["7.4.3", ""]], "choices": ["auto", "400ns", "800ns"], "type": "str"},
            "lan-ext-vap": {"v_range": [["7.4.3", ""]], "type": "list", "elements": "str"},
            "local-vaps": {"v_range": [["7.4.3", ""]], "type": "list", "elements": "str"},
            "max-clients": {"v_range": [["7.4.3", ""]], "type": "int"},
            "mode": {"v_range": [["7.4.3", ""]], "choices": ["AP", "Client"], "type": "str"},
            "operating-standard": {
                "v_range": [["7.4.3", ""]],
                "choices": [
                    "auto", "11A-N-AC-AX", "11A-N-AC", "11A-N", "11A", "11N-AC-AX", "11AC-AX", "11AC", "11N-AC", "11B-G-N-AX", "11B-G-N", "11B-G", "11B",
                    "11G-N-AX", "11N-AX", "11AX", "11G-N", "11N", "11G"
                ],
                "type": "str"
            },
            "power-level": {"v_range": [["7.4.3", ""]], "type": "int"},
            "radio-id": {"v_range": [["7.4.3", ""]], "type": "int"},
            "status": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "extensioncontroller_extenderprofile_wifi_radio2": {
        "stated": false,
        "options": {
            "80211d": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "band": {"v_range": [["7.4.3", ""]], "choices": ["5GHz"], "type": "str"},
            "bandwidth": {"v_range": [["7.4.3", ""]], "choices": ["auto", "20MHz", "40MHz", "80MHz"], "type": "str"},
            "beacon-interval": {"v_range": [["7.4.3", ""]], "type": "int"},
            "bss-color": {"v_range": [["7.4.3", ""]], "type": "int"},
            "bss-color-mode": {"v_range": [["7.4.3", ""]], "choices": ["auto", "static"], "type": "str"},
            "channel": {
                "v_range": [["7.4.3", ""]],
                "type": "list",
                "choices": [
                    "CH36", "CH40", "CH44", "CH48", "CH52", "CH56", "CH60", "CH64", "CH100", "CH104", "CH108", "CH112", "CH116", "CH120", "CH124",
                    "CH128", "CH132", "CH136", "CH140", "CH144", "CH149", "CH153", "CH157", "CH161", "CH165"
                ],
                "elements": "str"
            },
            "extension-channel": {"v_range": [["7.4.3", ""]], "choices": ["auto", "higher", "lower"], "type": "str"},
            "guard-interval": {"v_range": [["7.4.3", ""]], "choices": ["auto", "400ns", "800ns"], "type": "str"},
            "lan-ext-vap": {"v_range": [["7.4.3", ""]], "type": "list", "elements": "str"},
            "local-vaps": {"v_range": [["7.4.3", ""]], "type": "list", "elements": "str"},
            "max-clients": {"v_range": [["7.4.3", ""]], "type": "int"},
            "mode": {"v_range": [["7.4.3", ""]], "choices": ["AP", "Client"], "type": "str"},
            "operating-standard": {
                "v_range": [["7.4.3", ""]],
                "choices": [
                    "auto", "11A-N-AC-AX", "11A-N-AC", "11A-N", "11A", "11N-AC-AX", "11AC-AX", "11AC", "11N-AC", "11B-G-N-AX", "11B-G-N", "11B-G", "11B",
                    "11G-N-AX", "11N-AX", "11AX", "11G-N", "11N", "11G"
                ],
                "type": "str"
            },
            "power-level": {"v_range": [["7.4.3", ""]], "type": "int"},
            "radio-id": {"v_range": [["7.4.3", ""]], "type": "int"},
            "status": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "filefilter_profile": {
        "stated": true,
        "options": {
            "comment": {"v_range": [["6.4.1", ""]], "type": "str"},
            "extended-log": {"v_range": [["6.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "feature-set": {"v_range": [["6.4.1", ""]], "choices": ["proxy", "flow"], "type": "str"},
            "log": {"v_range": [["6.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "name": {"v_range": [["6.4.1", ""]], "required": true, "type": "str"},
            "replacemsg-group": {"v_range": [["6.4.1", ""]], "type": "str"},
            "rules": {
                "v_range": [["6.4.1", ""]],
                "type": "list",
                "options": {
                    "action": {"v_range": [["6.4.1", ""]], "choices": ["log-only", "block"], "type": "str"},
                    "comment": {"v_range": [["6.4.1", ""]], "type": "str"},
                    "direction": {"v_range": [["6.4.1", ""]], "choices": ["any", "incoming", "outgoing"], "type": "str"},
                    "file-type": {"v_range": [["6.4.1", ""]], "type": "raw"},
                    "name": {"v_range": [["6.4.1", ""]], "type": "str"},
                    "password-protected": {"v_range": [["6.4.1", ""]], "choices": ["any", "yes"], "type": "str"},
                    "protocol": {
                        "v_range": [["6.4.1", ""]],
                        "type": "list",
                        "choices": ["imap", "smtp", "pop3", "http", "ftp", "mapi", "cifs", "ssh"],
                        "elements": "str"
                    }
                },
                "elements": "dict"
            },
            "scan-archive-contents": {"v_range": [["6.4.1", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "filefilter_profile_rules": {
        "stated": true,
        "options": {
            "action": {"v_range": [["6.4.1", ""]], "choices": ["log-only", "block"], "type": "str"},
            "comment": {"v_range": [["6.4.1", ""]], "type": "str"},
            "direction": {"v_range": [["6.4.1", ""]], "choices": ["any", "incoming", "outgoing"], "type": "str"},
            "file-type": {"v_range": [["6.4.1", ""]], "type": "raw"},
            "name": {"v_range": [["6.4.1", ""]], "required": true, "type": "str"},
            "password-protected": {"v_range": [["6.4.1", ""]], "choices": ["any", "yes"], "type": "str"},
            "protocol": {
                "v_range": [["6.4.1", ""]],
                "type": "list",
                "choices": ["imap", "smtp", "pop3", "http", "ftp", "mapi", "cifs", "ssh"],
                "elements": "str"
            }
        }
    },
    "firewall_accessproxy": {
        "stated": true,
        "options": {
            "api-gateway": {
                "v_range": [["7.0.0", ""]],
                "type": "list",
                "options": {
                    "http-cookie-age": {"v_range": [["7.0.0", ""]], "type": "int"},
                    "http-cookie-domain": {"v_range": [["7.0.0", ""]], "type": "str"},
                    "http-cookie-domain-from-host": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "http-cookie-generation": {"v_range": [["7.0.0", ""]], "type": "int"},
                    "http-cookie-path": {"v_range": [["7.0.0", ""]], "type": "str"},
                    "http-cookie-share": {"v_range": [["7.0.0", ""]], "choices": ["disable", "same-ip"], "type": "str"},
                    "https-cookie-secure": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "id": {"v_range": [["7.0.0", ""]], "type": "int"},
                    "ldb-method": {
                        "v_range": [["7.0.0", ""]],
                        "choices": ["static", "round-robin", "weighted", "least-session", "least-rtt", "first-alive", "http-host"],
                        "type": "str"
                    },
                    "persistence": {"v_range": [["7.0.0", ""]], "choices": ["none", "http-cookie"], "type": "str"},
                    "realservers": {
                        "v_range": [["7.0.0", ""]],
                        "type": "list",
                        "options": {
                            "address": {"v_range": [["7.0.0", ""]], "type": "str"},
                            "health-check": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "health-check-proto": {"v_range": [["7.0.0", ""]], "choices": ["ping", "http", "tcp-connect"], "type": "str"},
                            "http-host": {"v_range": [["7.0.0", ""]], "type": "str"},
                            "id": {"v_range": [["7.0.0", ""]], "type": "int"},
                            "ip": {"v_range": [["7.0.0", ""]], "type": "str"},
                            "mappedport": {"v_range": [["7.0.0", ""]], "type": "raw"},
                            "port": {"v_range": [["7.0.0", ""]], "type": "int"},
                            "status": {"v_range": [["7.0.0", ""]], "choices": ["active", "standby", "disable"], "type": "str"},
                            "weight": {"v_range": [["7.0.0", ""]], "type": "int"},
                            "addr-type": {"v_range": [["7.0.2", ""]], "choices": ["fqdn", "ip"], "type": "str"},
                            "domain": {"v_range": [["7.0.3", ""]], "type": "str"},
                            "holddown-interval": {"v_range": [["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "ssh-client-cert": {"v_range": [["7.0.1", ""]], "type": "str"},
                            "ssh-host-key": {"v_range": [["7.0.1", ""]], "no_log": true, "type": "raw"},
                            "ssh-host-key-validation": {"v_range": [["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "type": {"v_range": [["7.0.1", ""]], "choices": ["tcp-forwarding", "ssh"], "type": "str"},
                            "translate-host": {"v_range": [["7.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "external-auth": {"v_range": [["7.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "tunnel-encryption": {"v_range": [["7.4.0", ""]], "choices": ["disable", "enable"], "type": "str"}
                        },
                        "elements": "dict"
                    },
                    "saml-server": {"v_range": [["7.0.0", ""]], "type": "str"},
                    "service": {"v_range": [["7.0.0", ""]], "choices": ["http", "https", "tcp-forwarding", "samlsp", "web-portal", "saas"], "type": "str"},
                    "ssl-algorithm": {"v_range": [["7.0.0", ""]], "choices": ["high", "medium", "low", "custom"], "type": "str"},
                    "ssl-cipher-suites": {
                        "v_range": [["7.0.0", ""]],
                        "type": "list",
                        "options": {
                            "cipher": {
                                "v_range": [["7.0.0", ""]],
                                "choices": [
                                    "TLS-RSA-WITH-RC4-128-MD5", "TLS-RSA-WITH-RC4-128-SHA", "TLS-RSA-WITH-DES-CBC-SHA", "TLS-RSA-WITH-3DES-EDE-CBC-SHA",
                                    "TLS-RSA-WITH-AES-128-CBC-SHA", "TLS-RSA-WITH-AES-256-CBC-SHA", "TLS-RSA-WITH-AES-128-CBC-SHA256",
                                    "TLS-RSA-WITH-AES-256-CBC-SHA256", "TLS-RSA-WITH-CAMELLIA-128-CBC-SHA", "TLS-RSA-WITH-CAMELLIA-256-CBC-SHA",
                                    "TLS-RSA-WITH-CAMELLIA-128-CBC-SHA256", "TLS-RSA-WITH-CAMELLIA-256-CBC-SHA256", "TLS-RSA-WITH-SEED-CBC-SHA",
                                    "TLS-RSA-WITH-ARIA-128-CBC-SHA256", "TLS-RSA-WITH-ARIA-256-CBC-SHA384", "TLS-DHE-RSA-WITH-DES-CBC-SHA",
                                    "TLS-DHE-RSA-WITH-3DES-EDE-CBC-SHA", "TLS-DHE-RSA-WITH-AES-128-CBC-SHA", "TLS-DHE-RSA-WITH-AES-256-CBC-SHA",
                                    "TLS-DHE-RSA-WITH-AES-128-CBC-SHA256", "TLS-DHE-RSA-WITH-AES-256-CBC-SHA256",
                                    "TLS-DHE-RSA-WITH-CAMELLIA-128-CBC-SHA", "TLS-DHE-RSA-WITH-CAMELLIA-256-CBC-SHA",
                                    "TLS-DHE-RSA-WITH-CAMELLIA-128-CBC-SHA256", "TLS-DHE-RSA-WITH-CAMELLIA-256-CBC-SHA256",
                                    "TLS-DHE-RSA-WITH-SEED-CBC-SHA", "TLS-DHE-RSA-WITH-ARIA-128-CBC-SHA256", "TLS-DHE-RSA-WITH-ARIA-256-CBC-SHA384",
                                    "TLS-ECDHE-RSA-WITH-RC4-128-SHA", "TLS-ECDHE-RSA-WITH-3DES-EDE-CBC-SHA", "TLS-ECDHE-RSA-WITH-AES-128-CBC-SHA",
                                    "TLS-ECDHE-RSA-WITH-AES-256-CBC-SHA", "TLS-ECDHE-RSA-WITH-CHACHA20-POLY1305-SHA256",
                                    "TLS-ECDHE-ECDSA-WITH-CHACHA20-POLY1305-SHA256", "TLS-DHE-RSA-WITH-CHACHA20-POLY1305-SHA256",
                                    "TLS-DHE-RSA-WITH-AES-128-GCM-SHA256", "TLS-DHE-RSA-WITH-AES-256-GCM-SHA384", "TLS-DHE-DSS-WITH-AES-128-CBC-SHA",
                                    "TLS-DHE-DSS-WITH-AES-256-CBC-SHA", "TLS-DHE-DSS-WITH-AES-128-CBC-SHA256", "TLS-DHE-DSS-WITH-AES-128-GCM-SHA256",
                                    "TLS-DHE-DSS-WITH-AES-256-CBC-SHA256", "TLS-DHE-DSS-WITH-AES-256-GCM-SHA384",
                                    "TLS-ECDHE-RSA-WITH-AES-128-CBC-SHA256", "TLS-ECDHE-RSA-WITH-AES-128-GCM-SHA256",
                                    "TLS-ECDHE-RSA-WITH-AES-256-CBC-SHA384", "TLS-ECDHE-RSA-WITH-AES-256-GCM-SHA384",
                                    "TLS-ECDHE-ECDSA-WITH-AES-128-CBC-SHA", "TLS-ECDHE-ECDSA-WITH-AES-128-CBC-SHA256",
                                    "TLS-ECDHE-ECDSA-WITH-AES-128-GCM-SHA256", "TLS-ECDHE-ECDSA-WITH-AES-256-CBC-SHA384",
                                    "TLS-ECDHE-ECDSA-WITH-AES-256-GCM-SHA384", "TLS-RSA-WITH-AES-128-GCM-SHA256", "TLS-RSA-WITH-AES-256-GCM-SHA384",
                                    "TLS-DHE-DSS-WITH-CAMELLIA-128-CBC-SHA", "TLS-DHE-DSS-WITH-CAMELLIA-256-CBC-SHA",
                                    "TLS-DHE-DSS-WITH-CAMELLIA-128-CBC-SHA256", "TLS-DHE-DSS-WITH-CAMELLIA-256-CBC-SHA256",
                                    "TLS-DHE-DSS-WITH-SEED-CBC-SHA", "TLS-DHE-DSS-WITH-ARIA-128-CBC-SHA256", "TLS-DHE-DSS-WITH-ARIA-256-CBC-SHA384",
                                    "TLS-ECDHE-RSA-WITH-ARIA-128-CBC-SHA256", "TLS-ECDHE-RSA-WITH-ARIA-256-CBC-SHA384",
                                    "TLS-ECDHE-ECDSA-WITH-ARIA-128-CBC-SHA256", "TLS-ECDHE-ECDSA-WITH-ARIA-256-CBC-SHA384",
                                    "TLS-DHE-DSS-WITH-3DES-EDE-CBC-SHA", "TLS-DHE-DSS-WITH-DES-CBC-SHA", "TLS-AES-128-GCM-SHA256",
                                    "TLS-AES-256-GCM-SHA384", "TLS-CHACHA20-POLY1305-SHA256", "TLS-ECDHE-ECDSA-WITH-AES-256-CBC-SHA"
                                ],
                                "type": "str"
                            },
                            "priority": {"v_range": [["7.0.0", ""]], "type": "int"},
                            "versions": {
                                "v_range": [["7.0.0", ""]],
                                "type": "list",
                                "choices": ["tls-1.0", "tls-1.1", "tls-1.2", "tls-1.3"],
                                "elements": "str"
                            }
                        },
                        "elements": "dict"
                    },
                    "ssl-dh-bits": {"v_range": [["7.0.0", ""]], "choices": ["768", "1024", "1536", "2048", "3072", "4096"], "type": "str"},
                    "ssl-max-version": {"v_range": [["7.0.0", ""]], "choices": ["tls-1.0", "tls-1.1", "tls-1.2", "tls-1.3"], "type": "str"},
                    "ssl-min-version": {"v_range": [["7.0.0", ""]], "choices": ["tls-1.0", "tls-1.1", "tls-1.2", "tls-1.3"], "type": "str"},
                    "url-map": {"v_range": [["7.0.0", ""]], "type": "str"},
                    "url-map-type": {"v_range": [["7.0.0", ""]], "choices": ["sub-string", "wildcard", "regex"], "type": "str"},
                    "virtual-host": {"v_range": [["7.0.0", ""]], "type": "str"},
                    "saml-redirect": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "ssl-vpn-web-portal": {"v_range": [["7.0.3", ""]], "type": "str"},
                    "application": {"v_range": [["7.2.1", ""]], "type": "raw"},
                    "ssl-renegotiation": {"v_range": [["7.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "h2-support": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "h3-support": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "quic": {
                        "v_range": [["7.4.1", ""]],
                        "type": "dict",
                        "options": {
                            "ack-delay-exponent": {"v_range": [["7.4.1", ""]], "type": "int"},
                            "active-connection-id-limit": {"v_range": [["7.4.1", ""]], "type": "int"},
                            "active-migration": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "grease-quic-bit": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "max-ack-delay": {"v_range": [["7.4.1", ""]], "type": "int"},
                            "max-datagram-frame-size": {"v_range": [["7.4.1", ""]], "type": "int"},
                            "max-idle-timeout": {"v_range": [["7.4.1", ""]], "type": "int"},
                            "max-udp-payload-size": {"v_range": [["7.4.1", ""]], "type": "int"}
                        }
                    }
                },
                "elements": "dict"
            },
            "client-cert": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "empty-cert-action": {"v_range": [["7.0.0", ""]], "choices": ["block", "accept", "accept-unmanageable"], "type": "str"},
            "ldb-method": {
                "v_range": [["7.0.0", ""]],
                "choices": ["static", "round-robin", "weighted", "least-session", "least-rtt", "first-alive"],
                "type": "str"
            },
            "name": {"v_range": [["7.0.0", ""]], "required": true, "type": "str"},
            "realservers": {
                "v_range": [["7.0.0", ""]],
                "type": "list",
                "options": {
                    "id": {"v_range": [["7.0.0", ""]], "type": "int"},
                    "ip": {"v_range": [["7.0.0", ""]], "type": "str"},
                    "port": {"v_range": [["7.0.0", ""]], "type": "int"},
                    "status": {"v_range": [["7.0.0", ""]], "choices": ["active", "standby", "disable"], "type": "str"},
                    "weight": {"v_range": [["7.0.0", ""]], "type": "int"}
                },
                "elements": "dict"
            },
            "server-pubkey-auth": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "server-pubkey-auth-settings": {
                "v_range": [["7.0.0", ""]],
                "type": "dict",
                "options": {
                    "auth-ca": {"v_range": [["7.0.0", ""]], "type": "str"},
                    "cert-extension": {
                        "v_range": [["7.0.0", ""]],
                        "type": "list",
                        "options": {
                            "critical": {"v_range": [["7.0.0", ""]], "choices": ["no", "yes"], "type": "str"},
                            "data": {"v_range": [["7.0.0", ""]], "type": "str"},
                            "name": {"v_range": [["7.0.0", ""]], "type": "str"},
                            "type": {"v_range": [["7.0.0", ""]], "choices": ["fixed", "user"], "type": "str"}
                        },
                        "elements": "dict"
                    },
                    "permit-agent-forwarding": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "permit-port-forwarding": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "permit-pty": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "permit-user-rc": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "permit-x11-forwarding": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "source-address": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"}
                }
            },
            "vip": {"v_range": [["7.0.0", ""]], "type": "str"},
            "api-gateway6": {
                "v_range": [["7.0.1", ""]],
                "type": "list",
                "options": {
                    "http-cookie-age": {"v_range": [["7.0.1", ""]], "type": "int"},
                    "http-cookie-domain": {"v_range": [["7.0.1", ""]], "type": "str"},
                    "http-cookie-domain-from-host": {"v_range": [["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "http-cookie-generation": {"v_range": [["7.0.1", ""]], "type": "int"},
                    "http-cookie-path": {"v_range": [["7.0.1", ""]], "type": "str"},
                    "http-cookie-share": {"v_range": [["7.0.1", ""]], "choices": ["disable", "same-ip"], "type": "str"},
                    "https-cookie-secure": {"v_range": [["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "id": {"v_range": [["7.0.1", ""]], "type": "int"},
                    "ldb-method": {
                        "v_range": [["7.0.1", ""]],
                        "choices": ["static", "round-robin", "weighted", "first-alive", "http-host"],
                        "type": "str"
                    },
                    "persistence": {"v_range": [["7.0.1", ""]], "choices": ["none", "http-cookie"], "type": "str"},
                    "realservers": {
                        "v_range": [["7.0.1", ""]],
                        "type": "list",
                        "options": {
                            "addr-type": {"v_range": [["7.0.2", ""]], "choices": ["fqdn", "ip"], "type": "str"},
                            "address": {"v_range": [["7.0.1", ""]], "type": "str"},
                            "domain": {"v_range": [["7.0.3", ""]], "type": "str"},
                            "health-check": {"v_range": [["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "health-check-proto": {"v_range": [["7.0.1", ""]], "choices": ["ping", "http", "tcp-connect"], "type": "str"},
                            "holddown-interval": {"v_range": [["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "http-host": {"v_range": [["7.0.1", ""]], "type": "str"},
                            "id": {"v_range": [["7.0.1", ""]], "type": "int"},
                            "ip": {"v_range": [["7.0.1", ""]], "type": "str"},
                            "mappedport": {"v_range": [["7.0.1", ""]], "type": "raw"},
                            "port": {"v_range": [["7.0.1", ""]], "type": "int"},
                            "ssh-client-cert": {"v_range": [["7.0.1", ""]], "type": "str"},
                            "ssh-host-key": {"v_range": [["7.0.1", ""]], "no_log": true, "type": "raw"},
                            "ssh-host-key-validation": {"v_range": [["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "status": {"v_range": [["7.0.1", ""]], "choices": ["active", "standby", "disable"], "type": "str"},
                            "type": {"v_range": [["7.0.1", ""]], "choices": ["tcp-forwarding", "ssh"], "type": "str"},
                            "weight": {"v_range": [["7.0.1", ""]], "type": "int"},
                            "translate-host": {"v_range": [["7.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "external-auth": {"v_range": [["7.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "tunnel-encryption": {"v_range": [["7.4.0", ""]], "choices": ["disable", "enable"], "type": "str"}
                        },
                        "elements": "dict"
                    },
                    "saml-redirect": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "saml-server": {"v_range": [["7.0.1", ""]], "type": "str"},
                    "service": {"v_range": [["7.0.1", ""]], "choices": ["http", "https", "tcp-forwarding", "samlsp", "web-portal", "saas"], "type": "str"},
                    "ssl-algorithm": {"v_range": [["7.0.1", ""]], "choices": ["high", "medium", "low"], "type": "str"},
                    "ssl-cipher-suites": {
                        "v_range": [["7.0.1", ""]],
                        "type": "list",
                        "options": {
                            "cipher": {
                                "v_range": [["7.0.1", ""]],
                                "choices": [
                                    "TLS-RSA-WITH-RC4-128-MD5", "TLS-RSA-WITH-RC4-128-SHA", "TLS-RSA-WITH-DES-CBC-SHA", "TLS-RSA-WITH-3DES-EDE-CBC-SHA",
                                    "TLS-RSA-WITH-AES-128-CBC-SHA", "TLS-RSA-WITH-AES-256-CBC-SHA", "TLS-RSA-WITH-AES-128-CBC-SHA256",
                                    "TLS-RSA-WITH-AES-256-CBC-SHA256", "TLS-RSA-WITH-CAMELLIA-128-CBC-SHA", "TLS-RSA-WITH-CAMELLIA-256-CBC-SHA",
                                    "TLS-RSA-WITH-CAMELLIA-128-CBC-SHA256", "TLS-RSA-WITH-CAMELLIA-256-CBC-SHA256", "TLS-RSA-WITH-SEED-CBC-SHA",
                                    "TLS-RSA-WITH-ARIA-128-CBC-SHA256", "TLS-RSA-WITH-ARIA-256-CBC-SHA384", "TLS-DHE-RSA-WITH-DES-CBC-SHA",
                                    "TLS-DHE-RSA-WITH-3DES-EDE-CBC-SHA", "TLS-DHE-RSA-WITH-AES-128-CBC-SHA", "TLS-DHE-RSA-WITH-AES-256-CBC-SHA",
                                    "TLS-DHE-RSA-WITH-AES-128-CBC-SHA256", "TLS-DHE-RSA-WITH-AES-256-CBC-SHA256",
                                    "TLS-DHE-RSA-WITH-CAMELLIA-128-CBC-SHA", "TLS-DHE-RSA-WITH-CAMELLIA-256-CBC-SHA",
                                    "TLS-DHE-RSA-WITH-CAMELLIA-128-CBC-SHA256", "TLS-DHE-RSA-WITH-CAMELLIA-256-CBC-SHA256",
                                    "TLS-DHE-RSA-WITH-SEED-CBC-SHA", "TLS-DHE-RSA-WITH-ARIA-128-CBC-SHA256", "TLS-DHE-RSA-WITH-ARIA-256-CBC-SHA384",
                                    "TLS-ECDHE-RSA-WITH-RC4-128-SHA", "TLS-ECDHE-RSA-WITH-3DES-EDE-CBC-SHA", "TLS-ECDHE-RSA-WITH-AES-128-CBC-SHA",
                                    "TLS-ECDHE-RSA-WITH-AES-256-CBC-SHA", "TLS-ECDHE-RSA-WITH-CHACHA20-POLY1305-SHA256",
                                    "TLS-ECDHE-ECDSA-WITH-CHACHA20-POLY1305-SHA256", "TLS-DHE-RSA-WITH-CHACHA20-POLY1305-SHA256",
                                    "TLS-DHE-RSA-WITH-AES-128-GCM-SHA256", "TLS-DHE-RSA-WITH-AES-256-GCM-SHA384", "TLS-DHE-DSS-WITH-AES-128-CBC-SHA",
                                    "TLS-DHE-DSS-WITH-AES-256-CBC-SHA", "TLS-DHE-DSS-WITH-AES-128-CBC-SHA256", "TLS-DHE-DSS-WITH-AES-128-GCM-SHA256",
                                    "TLS-DHE-DSS-WITH-AES-256-CBC-SHA256", "TLS-DHE-DSS-WITH-AES-256-GCM-SHA384",
                                    "TLS-ECDHE-RSA-WITH-AES-128-CBC-SHA256", "TLS-ECDHE-RSA-WITH-AES-128-GCM-SHA256",
                                    "TLS-ECDHE-RSA-WITH-AES-256-CBC-SHA384", "TLS-ECDHE-RSA-WITH-AES-256-GCM-SHA384",
                                    "TLS-ECDHE-ECDSA-WITH-AES-128-CBC-SHA", "TLS-ECDHE-ECDSA-WITH-AES-128-CBC-SHA256",
                                    "TLS-ECDHE-ECDSA-WITH-AES-128-GCM-SHA256", "TLS-ECDHE-ECDSA-WITH-AES-256-CBC-SHA384",
                                    "TLS-ECDHE-ECDSA-WITH-AES-256-GCM-SHA384", "TLS-RSA-WITH-AES-128-GCM-SHA256", "TLS-RSA-WITH-AES-256-GCM-SHA384",
                                    "TLS-DHE-DSS-WITH-CAMELLIA-128-CBC-SHA", "TLS-DHE-DSS-WITH-CAMELLIA-256-CBC-SHA",
                                    "TLS-DHE-DSS-WITH-CAMELLIA-128-CBC-SHA256", "TLS-DHE-DSS-WITH-CAMELLIA-256-CBC-SHA256",
                                    "TLS-DHE-DSS-WITH-SEED-CBC-SHA", "TLS-DHE-DSS-WITH-ARIA-128-CBC-SHA256", "TLS-DHE-DSS-WITH-ARIA-256-CBC-SHA384",
                                    "TLS-ECDHE-RSA-WITH-ARIA-128-CBC-SHA256", "TLS-ECDHE-RSA-WITH-ARIA-256-CBC-SHA384",
                                    "TLS-ECDHE-ECDSA-WITH-ARIA-128-CBC-SHA256", "TLS-ECDHE-ECDSA-WITH-ARIA-256-CBC-SHA384",
                                    "TLS-DHE-DSS-WITH-3DES-EDE-CBC-SHA", "TLS-DHE-DSS-WITH-DES-CBC-SHA", "TLS-AES-128-GCM-SHA256",
                                    "TLS-AES-256-GCM-SHA384", "TLS-CHACHA20-POLY1305-SHA256", "TLS-ECDHE-ECDSA-WITH-AES-256-CBC-SHA"
                                ],
                                "type": "str"
                            },
                            "priority": {"v_range": [["7.0.1", ""]], "type": "int"},
                            "versions": {
                                "v_range": [["7.0.1", ""]],
                                "type": "list",
                                "choices": ["tls-1.0", "tls-1.1", "tls-1.2", "tls-1.3"],
                                "elements": "str"
                            }
                        },
                        "elements": "dict"
                    },
                    "ssl-dh-bits": {"v_range": [["7.0.1", ""]], "choices": ["768", "1024", "1536", "2048", "3072", "4096"], "type": "str"},
                    "ssl-max-version": {"v_range": [["7.0.1", ""]], "choices": ["tls-1.0", "tls-1.1", "tls-1.2", "tls-1.3"], "type": "str"},
                    "ssl-min-version": {"v_range": [["7.0.1", ""]], "choices": ["tls-1.0", "tls-1.1", "tls-1.2", "tls-1.3"], "type": "str"},
                    "ssl-vpn-web-portal": {"v_range": [["7.0.3", ""]], "type": "str"},
                    "url-map": {"v_range": [["7.0.1", ""]], "type": "str"},
                    "url-map-type": {"v_range": [["7.0.1", ""]], "choices": ["sub-string", "wildcard", "regex"], "type": "str"},
                    "virtual-host": {"v_range": [["7.0.1", ""]], "type": "str"},
                    "application": {"v_range": [["7.2.1", ""]], "type": "raw"},
                    "ssl-renegotiation": {"v_range": [["7.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "h2-support": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "h3-support": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "quic": {
                        "v_range": [["7.4.1", ""]],
                        "type": "dict",
                        "options": {
                            "ack-delay-exponent": {"v_range": [["7.4.1", ""]], "type": "int"},
                            "active-connection-id-limit": {"v_range": [["7.4.1", ""]], "type": "int"},
                            "active-migration": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "grease-quic-bit": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "max-ack-delay": {"v_range": [["7.4.1", ""]], "type": "int"},
                            "max-datagram-frame-size": {"v_range": [["7.4.1", ""]], "type": "int"},
                            "max-idle-timeout": {"v_range": [["7.4.1", ""]], "type": "int"},
                            "max-udp-payload-size": {"v_range": [["7.4.1", ""]], "type": "int"}
                        }
                    }
                },
                "elements": "dict"
            },
            "auth-portal": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "auth-virtual-host": {"v_range": [["7.0.3", ""]], "type": "str"},
            "decrypted-traffic-mirror": {"v_range": [["7.0.2", ""]], "type": "str"},
            "log-blocked-traffic": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "add-vhost-domain-to-dnsdb": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "user-agent-detect": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "http-supported-max-version": {"v_range": [["7.2.2", ""]], "choices": ["http1", "http2"], "type": "str"},
            "svr-pool-multiplex": {"v_range": [["7.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "svr-pool-server-max-request": {"v_range": [["7.2.2", ""]], "type": "int"},
            "svr-pool-ttl": {"v_range": [["7.2.2", ""]], "type": "int"},
            "svr-pool-server-max-concurrent-request": {"v_range": [["7.4.1", ""]], "type": "int"}
        }
    },
    "firewall_accessproxy6": {
        "stated": true,
        "options": {
            "add-vhost-domain-to-dnsdb": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "api-gateway": {
                "v_range": [["7.2.1", ""]],
                "type": "list",
                "options": {
                    "application": {"v_range": [["7.2.1", ""]], "type": "raw"},
                    "http-cookie-age": {"v_range": [["7.2.1", ""]], "type": "int"},
                    "http-cookie-domain": {"v_range": [["7.2.1", ""]], "type": "str"},
                    "http-cookie-domain-from-host": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "http-cookie-generation": {"v_range": [["7.2.1", ""]], "type": "int"},
                    "http-cookie-path": {"v_range": [["7.2.1", ""]], "type": "str"},
                    "http-cookie-share": {"v_range": [["7.2.1", ""]], "choices": ["disable", "same-ip"], "type": "str"},
                    "https-cookie-secure": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "id": {"v_range": [["7.2.1", ""]], "type": "int"},
                    "ldb-method": {
                        "v_range": [["7.2.1", ""]],
                        "choices": ["static", "round-robin", "weighted", "first-alive", "http-host"],
                        "type": "str"
                    },
                    "persistence": {"v_range": [["7.2.1", ""]], "choices": ["none", "http-cookie"], "type": "str"},
                    "realservers": {
                        "v_range": [["7.2.1", ""]],
                        "type": "list",
                        "options": {
                            "addr-type": {"v_range": [["7.2.1", ""]], "choices": ["fqdn", "ip"], "type": "str"},
                            "address": {"v_range": [["7.2.1", ""]], "type": "str"},
                            "domain": {"v_range": [["7.2.1", ""]], "type": "str"},
                            "health-check": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "health-check-proto": {"v_range": [["7.2.1", ""]], "choices": ["ping", "http", "tcp-connect"], "type": "str"},
                            "holddown-interval": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "http-host": {"v_range": [["7.2.1", ""]], "type": "str"},
                            "id": {"v_range": [["7.2.1", ""]], "type": "int"},
                            "ip": {"v_range": [["7.2.1", ""]], "type": "str"},
                            "mappedport": {"v_range": [["7.2.1", ""]], "type": "raw"},
                            "port": {"v_range": [["7.2.1", ""]], "type": "int"},
                            "ssh-client-cert": {"v_range": [["7.2.1", ""]], "type": "str"},
                            "ssh-host-key": {"v_range": [["7.2.1", ""]], "no_log": true, "type": "raw"},
                            "ssh-host-key-validation": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "status": {"v_range": [["7.2.1", ""]], "choices": ["active", "standby", "disable"], "type": "str"},
                            "type": {"v_range": [["7.2.1", ""]], "choices": ["tcp-forwarding", "ssh"], "type": "str"},
                            "weight": {"v_range": [["7.2.1", ""]], "type": "int"},
                            "translate-host": {"v_range": [["7.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "external-auth": {"v_range": [["7.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "tunnel-encryption": {"v_range": [["7.4.0", ""]], "choices": ["disable", "enable"], "type": "str"}
                        },
                        "elements": "dict"
                    },
                    "saml-redirect": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "saml-server": {"v_range": [["7.2.1", ""]], "type": "str"},
                    "service": {"v_range": [["7.2.1", ""]], "choices": ["http", "https", "tcp-forwarding", "samlsp", "web-portal", "saas"], "type": "str"},
                    "ssl-algorithm": {"v_range": [["7.2.1", ""]], "choices": ["high", "medium", "low"], "type": "str"},
                    "ssl-cipher-suites": {
                        "v_range": [["7.2.1", ""]],
                        "type": "list",
                        "options": {
                            "cipher": {
                                "v_range": [["7.2.1", ""]],
                                "choices": [
                                    "TLS-RSA-WITH-RC4-128-MD5", "TLS-RSA-WITH-RC4-128-SHA", "TLS-RSA-WITH-DES-CBC-SHA", "TLS-RSA-WITH-3DES-EDE-CBC-SHA",
                                    "TLS-RSA-WITH-AES-128-CBC-SHA", "TLS-RSA-WITH-AES-256-CBC-SHA", "TLS-RSA-WITH-AES-128-CBC-SHA256",
                                    "TLS-RSA-WITH-AES-256-CBC-SHA256", "TLS-RSA-WITH-CAMELLIA-128-CBC-SHA", "TLS-RSA-WITH-CAMELLIA-256-CBC-SHA",
                                    "TLS-RSA-WITH-CAMELLIA-128-CBC-SHA256", "TLS-RSA-WITH-CAMELLIA-256-CBC-SHA256", "TLS-RSA-WITH-SEED-CBC-SHA",
                                    "TLS-RSA-WITH-ARIA-128-CBC-SHA256", "TLS-RSA-WITH-ARIA-256-CBC-SHA384", "TLS-DHE-RSA-WITH-DES-CBC-SHA",
                                    "TLS-DHE-RSA-WITH-3DES-EDE-CBC-SHA", "TLS-DHE-RSA-WITH-AES-128-CBC-SHA", "TLS-DHE-RSA-WITH-AES-256-CBC-SHA",
                                    "TLS-DHE-RSA-WITH-AES-128-CBC-SHA256", "TLS-DHE-RSA-WITH-AES-256-CBC-SHA256",
                                    "TLS-DHE-RSA-WITH-CAMELLIA-128-CBC-SHA", "TLS-DHE-RSA-WITH-CAMELLIA-256-CBC-SHA",
                                    "TLS-DHE-RSA-WITH-CAMELLIA-128-CBC-SHA256", "TLS-DHE-RSA-WITH-CAMELLIA-256-CBC-SHA256",
                                    "TLS-DHE-RSA-WITH-SEED-CBC-SHA", "TLS-DHE-RSA-WITH-ARIA-128-CBC-SHA256", "TLS-DHE-RSA-WITH-ARIA-256-CBC-SHA384",
                                    "TLS-ECDHE-RSA-WITH-RC4-128-SHA", "TLS-ECDHE-RSA-WITH-3DES-EDE-CBC-SHA", "TLS-ECDHE-RSA-WITH-AES-128-CBC-SHA",
                                    "TLS-ECDHE-RSA-WITH-AES-256-CBC-SHA", "TLS-ECDHE-RSA-WITH-CHACHA20-POLY1305-SHA256",
                                    "TLS-ECDHE-ECDSA-WITH-CHACHA20-POLY1305-SHA256", "TLS-DHE-RSA-WITH-CHACHA20-POLY1305-SHA256",
                                    "TLS-DHE-RSA-WITH-AES-128-GCM-SHA256", "TLS-DHE-RSA-WITH-AES-256-GCM-SHA384", "TLS-DHE-DSS-WITH-AES-128-CBC-SHA",
                                    "TLS-DHE-DSS-WITH-AES-256-CBC-SHA", "TLS-DHE-DSS-WITH-AES-128-CBC-SHA256", "TLS-DHE-DSS-WITH-AES-128-GCM-SHA256",
                                    "TLS-DHE-DSS-WITH-AES-256-CBC-SHA256", "TLS-DHE-DSS-WITH-AES-256-GCM-SHA384",
                                    "TLS-ECDHE-RSA-WITH-AES-128-CBC-SHA256", "TLS-ECDHE-RSA-WITH-AES-128-GCM-SHA256",
                                    "TLS-ECDHE-RSA-WITH-AES-256-CBC-SHA384", "TLS-ECDHE-RSA-WITH-AES-256-GCM-SHA384",
                                    "TLS-ECDHE-ECDSA-WITH-AES-128-CBC-SHA", "TLS-ECDHE-ECDSA-WITH-AES-128-CBC-SHA256",
                                    "TLS-ECDHE-ECDSA-WITH-AES-128-GCM-SHA256", "TLS-ECDHE-ECDSA-WITH-AES-256-CBC-SHA384",
                                    "TLS-ECDHE-ECDSA-WITH-AES-256-GCM-SHA384", "TLS-RSA-WITH-AES-128-GCM-SHA256", "TLS-RSA-WITH-AES-256-GCM-SHA384",
                                    "TLS-DHE-DSS-WITH-CAMELLIA-128-CBC-SHA", "TLS-DHE-DSS-WITH-CAMELLIA-256-CBC-SHA",
                                    "TLS-DHE-DSS-WITH-CAMELLIA-128-CBC-SHA256", "TLS-DHE-DSS-WITH-CAMELLIA-256-CBC-SHA256",
                                    "TLS-DHE-DSS-WITH-SEED-CBC-SHA", "TLS-DHE-DSS-WITH-ARIA-128-CBC-SHA256", "TLS-DHE-DSS-WITH-ARIA-256-CBC-SHA384",
                                    "TLS-ECDHE-RSA-WITH-ARIA-128-CBC-SHA256", "TLS-ECDHE-RSA-WITH-ARIA-256-CBC-SHA384",
                                    "TLS-ECDHE-ECDSA-WITH-ARIA-128-CBC-SHA256", "TLS-ECDHE-ECDSA-WITH-ARIA-256-CBC-SHA384",
                                    "TLS-DHE-DSS-WITH-3DES-EDE-CBC-SHA", "TLS-DHE-DSS-WITH-DES-CBC-SHA", "TLS-AES-128-GCM-SHA256",
                                    "TLS-AES-256-GCM-SHA384", "TLS-CHACHA20-POLY1305-SHA256", "TLS-ECDHE-ECDSA-WITH-AES-256-CBC-SHA"
                                ],
                                "type": "str"
                            },
                            "priority": {"v_range": [["7.2.1", ""]], "type": "int"},
                            "versions": {
                                "v_range": [["7.2.1", ""]],
                                "type": "list",
                                "choices": ["tls-1.0", "tls-1.1", "tls-1.2", "tls-1.3"],
                                "elements": "str"
                            }
                        },
                        "elements": "dict"
                    },
                    "ssl-dh-bits": {"v_range": [["7.2.1", ""]], "choices": ["768", "1024", "1536", "2048", "3072", "4096"], "type": "str"},
                    "ssl-max-version": {"v_range": [["7.2.1", ""]], "choices": ["tls-1.0", "tls-1.1", "tls-1.2", "tls-1.3"], "type": "str"},
                    "ssl-min-version": {"v_range": [["7.2.1", ""]], "choices": ["tls-1.0", "tls-1.1", "tls-1.2", "tls-1.3"], "type": "str"},
                    "ssl-vpn-web-portal": {"v_range": [["7.2.1", ""]], "type": "str"},
                    "url-map": {"v_range": [["7.2.1", ""]], "type": "str"},
                    "url-map-type": {"v_range": [["7.2.1", ""]], "choices": ["sub-string", "wildcard", "regex"], "type": "str"},
                    "virtual-host": {"v_range": [["7.2.1", ""]], "type": "str"},
                    "ssl-renegotiation": {"v_range": [["7.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "h2-support": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "h3-support": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "quic": {
                        "v_range": [["7.4.1", ""]],
                        "type": "dict",
                        "options": {
                            "ack-delay-exponent": {"v_range": [["7.4.1", ""]], "type": "int"},
                            "active-connection-id-limit": {"v_range": [["7.4.1", ""]], "type": "int"},
                            "active-migration": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "grease-quic-bit": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "max-ack-delay": {"v_range": [["7.4.1", ""]], "type": "int"},
                            "max-datagram-frame-size": {"v_range": [["7.4.1", ""]], "type": "int"},
                            "max-idle-timeout": {"v_range": [["7.4.1", ""]], "type": "int"},
                            "max-udp-payload-size": {"v_range": [["7.4.1", ""]], "type": "int"}
                        }
                    }
                },
                "elements": "dict"
            },
            "api-gateway6": {
                "v_range": [["7.2.1", ""]],
                "type": "list",
                "options": {
                    "application": {"v_range": [["7.2.1", ""]], "type": "raw"},
                    "http-cookie-age": {"v_range": [["7.2.1", ""]], "type": "int"},
                    "http-cookie-domain": {"v_range": [["7.2.1", ""]], "type": "str"},
                    "http-cookie-domain-from-host": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "http-cookie-generation": {"v_range": [["7.2.1", ""]], "type": "int"},
                    "http-cookie-path": {"v_range": [["7.2.1", ""]], "type": "str"},
                    "http-cookie-share": {"v_range": [["7.2.1", ""]], "choices": ["disable", "same-ip"], "type": "str"},
                    "https-cookie-secure": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "id": {"v_range": [["7.2.1", ""]], "type": "int"},
                    "ldb-method": {
                        "v_range": [["7.2.1", ""]],
                        "choices": ["static", "round-robin", "weighted", "first-alive", "http-host"],
                        "type": "str"
                    },
                    "persistence": {"v_range": [["7.2.1", ""]], "choices": ["none", "http-cookie"], "type": "str"},
                    "realservers": {
                        "v_range": [["7.2.1", ""]],
                        "type": "list",
                        "options": {
                            "addr-type": {"v_range": [["7.2.1", ""]], "choices": ["fqdn", "ip"], "type": "str"},
                            "address": {"v_range": [["7.2.1", ""]], "type": "str"},
                            "domain": {"v_range": [["7.2.1", ""]], "type": "str"},
                            "health-check": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "health-check-proto": {"v_range": [["7.2.1", ""]], "choices": ["ping", "http", "tcp-connect"], "type": "str"},
                            "holddown-interval": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "http-host": {"v_range": [["7.2.1", ""]], "type": "str"},
                            "id": {"v_range": [["7.2.1", ""]], "type": "int"},
                            "ip": {"v_range": [["7.2.1", ""]], "type": "str"},
                            "mappedport": {"v_range": [["7.2.1", ""]], "type": "raw"},
                            "port": {"v_range": [["7.2.1", ""]], "type": "int"},
                            "ssh-client-cert": {"v_range": [["7.2.1", ""]], "type": "str"},
                            "ssh-host-key": {"v_range": [["7.2.1", ""]], "no_log": true, "type": "raw"},
                            "ssh-host-key-validation": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "status": {"v_range": [["7.2.1", ""]], "choices": ["active", "standby", "disable"], "type": "str"},
                            "type": {"v_range": [["7.2.1", ""]], "choices": ["tcp-forwarding", "ssh"], "type": "str"},
                            "weight": {"v_range": [["7.2.1", ""]], "type": "int"},
                            "translate-host": {"v_range": [["7.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "external-auth": {"v_range": [["7.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "tunnel-encryption": {"v_range": [["7.4.0", ""]], "choices": ["disable", "enable"], "type": "str"}
                        },
                        "elements": "dict"
                    },
                    "saml-redirect": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "saml-server": {"v_range": [["7.2.1", ""]], "type": "str"},
                    "service": {"v_range": [["7.2.1", ""]], "choices": ["http", "https", "tcp-forwarding", "samlsp", "web-portal", "saas"], "type": "str"},
                    "ssl-algorithm": {"v_range": [["7.2.1", ""]], "choices": ["high", "medium", "low"], "type": "str"},
                    "ssl-cipher-suites": {
                        "v_range": [["7.2.1", ""]],
                        "type": "list",
                        "options": {
                            "cipher": {
                                "v_range": [["7.2.1", ""]],
                                "choices": [
                                    "TLS-RSA-WITH-RC4-128-MD5", "TLS-RSA-WITH-RC4-128-SHA", "TLS-RSA-WITH-DES-CBC-SHA", "TLS-RSA-WITH-3DES-EDE-CBC-SHA",
                                    "TLS-RSA-WITH-AES-128-CBC-SHA", "TLS-RSA-WITH-AES-256-CBC-SHA", "TLS-RSA-WITH-AES-128-CBC-SHA256",
                                    "TLS-RSA-WITH-AES-256-CBC-SHA256", "TLS-RSA-WITH-CAMELLIA-128-CBC-SHA", "TLS-RSA-WITH-CAMELLIA-256-CBC-SHA",
                                    "TLS-RSA-WITH-CAMELLIA-128-CBC-SHA256", "TLS-RSA-WITH-CAMELLIA-256-CBC-SHA256", "TLS-RSA-WITH-SEED-CBC-SHA",
                                    "TLS-RSA-WITH-ARIA-128-CBC-SHA256", "TLS-RSA-WITH-ARIA-256-CBC-SHA384", "TLS-DHE-RSA-WITH-DES-CBC-SHA",
                                    "TLS-DHE-RSA-WITH-3DES-EDE-CBC-SHA", "TLS-DHE-RSA-WITH-AES-128-CBC-SHA", "TLS-DHE-RSA-WITH-AES-256-CBC-SHA",
                                    "TLS-DHE-RSA-WITH-AES-128-CBC-SHA256", "TLS-DHE-RSA-WITH-AES-256-CBC-SHA256",
                                    "TLS-DHE-RSA-WITH-CAMELLIA-128-CBC-SHA", "TLS-DHE-RSA-WITH-CAMELLIA-256-CBC-SHA",
                                    "TLS-DHE-RSA-WITH-CAMELLIA-128-CBC-SHA256", "TLS-DHE-RSA-WITH-CAMELLIA-256-CBC-SHA256",
                                    "TLS-DHE-RSA-WITH-SEED-CBC-SHA", "TLS-DHE-RSA-WITH-ARIA-128-CBC-SHA256", "TLS-DHE-RSA-WITH-ARIA-256-CBC-SHA384",
                                    "TLS-ECDHE-RSA-WITH-RC4-128-SHA", "TLS-ECDHE-RSA-WITH-3DES-EDE-CBC-SHA", "TLS-ECDHE-RSA-WITH-AES-128-CBC-SHA",
                                    "TLS-ECDHE-RSA-WITH-AES-256-CBC-SHA", "TLS-ECDHE-RSA-WITH-CHACHA20-POLY1305-SHA256",
                                    "TLS-ECDHE-ECDSA-WITH-CHACHA20-POLY1305-SHA256", "TLS-DHE-RSA-WITH-CHACHA20-POLY1305-SHA256",
                                    "TLS-DHE-RSA-WITH-AES-128-GCM-SHA256", "TLS-DHE-RSA-WITH-AES-256-GCM-SHA384", "TLS-DHE-DSS-WITH-AES-128-CBC-SHA",
                                    "TLS-DHE-DSS-WITH-AES-256-CBC-SHA", "TLS-DHE-DSS-WITH-AES-128-CBC-SHA256", "TLS-DHE-DSS-WITH-AES-128-GCM-SHA256",
                                    "TLS-DHE-DSS-WITH-AES-256-CBC-SHA256", "TLS-DHE-DSS-WITH-AES-256-GCM-SHA384",
                                    "TLS-ECDHE-RSA-WITH-AES-128-CBC-SHA256", "TLS-ECDHE-RSA-WITH-AES-128-GCM-SHA256",
                                    "TLS-ECDHE-RSA-WITH-AES-256-CBC-SHA384", "TLS-ECDHE-RSA-WITH-AES-256-GCM-SHA384",
                                    "TLS-ECDHE-ECDSA-WITH-AES-128-CBC-SHA", "TLS-ECDHE-ECDSA-WITH-AES-128-CBC-SHA256",
                                    "TLS-ECDHE-ECDSA-WITH-AES-128-GCM-SHA256", "TLS-ECDHE-ECDSA-WITH-AES-256-CBC-SHA384",
                                    "TLS-ECDHE-ECDSA-WITH-AES-256-GCM-SHA384", "TLS-RSA-WITH-AES-128-GCM-SHA256", "TLS-RSA-WITH-AES-256-GCM-SHA384",
                                    "TLS-DHE-DSS-WITH-CAMELLIA-128-CBC-SHA", "TLS-DHE-DSS-WITH-CAMELLIA-256-CBC-SHA",
                                    "TLS-DHE-DSS-WITH-CAMELLIA-128-CBC-SHA256", "TLS-DHE-DSS-WITH-CAMELLIA-256-CBC-SHA256",
                                    "TLS-DHE-DSS-WITH-SEED-CBC-SHA", "TLS-DHE-DSS-WITH-ARIA-128-CBC-SHA256", "TLS-DHE-DSS-WITH-ARIA-256-CBC-SHA384",
                                    "TLS-ECDHE-RSA-WITH-ARIA-128-CBC-SHA256", "TLS-ECDHE-RSA-WITH-ARIA-256-CBC-SHA384",
                                    "TLS-ECDHE-ECDSA-WITH-ARIA-128-CBC-SHA256", "TLS-ECDHE-ECDSA-WITH-ARIA-256-CBC-SHA384",
                                    "TLS-DHE-DSS-WITH-3DES-EDE-CBC-SHA", "TLS-DHE-DSS-WITH-DES-CBC-SHA", "TLS-AES-128-GCM-SHA256",
                                    "TLS-AES-256-GCM-SHA384", "TLS-CHACHA20-POLY1305-SHA256", "TLS-ECDHE-ECDSA-WITH-AES-256-CBC-SHA"
                                ],
                                "type": "str"
                            },
                            "priority": {"v_range": [["7.2.1", ""]], "type": "int"},
                            "versions": {
                                "v_range": [["7.2.1", ""]],
                                "type": "list",
                                "choices": ["tls-1.0", "tls-1.1", "tls-1.2", "tls-1.3"],
                                "elements": "str"
                            }
                        },
                        "elements": "dict"
                    },
                    "ssl-dh-bits": {"v_range": [["7.2.1", ""]], "choices": ["768", "1024", "1536", "2048", "3072", "4096"], "type": "str"},
                    "ssl-max-version": {"v_range": [["7.2.1", ""]], "choices": ["tls-1.0", "tls-1.1", "tls-1.2", "tls-1.3"], "type": "str"},
                    "ssl-min-version": {"v_range": [["7.2.1", ""]], "choices": ["tls-1.0", "tls-1.1", "tls-1.2", "tls-1.3"], "type": "str"},
                    "ssl-vpn-web-portal": {"v_range": [["7.2.1", ""]], "type": "str"},
                    "url-map": {"v_range": [["7.2.1", ""]], "type": "str"},
                    "url-map-type": {"v_range": [["7.2.1", ""]], "choices": ["sub-string", "wildcard", "regex"], "type": "str"},
                    "virtual-host": {"v_range": [["7.2.1", ""]], "type": "str"},
                    "ssl-renegotiation": {"v_range": [["7.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "h2-support": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "h3-support": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "quic": {
                        "v_range": [["7.4.1", ""]],
                        "type": "dict",
                        "options": {
                            "ack-delay-exponent": {"v_range": [["7.4.1", ""]], "type": "int"},
                            "active-connection-id-limit": {"v_range": [["7.4.1", ""]], "type": "int"},
                            "active-migration": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "grease-quic-bit": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "max-ack-delay": {"v_range": [["7.4.1", ""]], "type": "int"},
                            "max-datagram-frame-size": {"v_range": [["7.4.1", ""]], "type": "int"},
                            "max-idle-timeout": {"v_range": [["7.4.1", ""]], "type": "int"},
                            "max-udp-payload-size": {"v_range": [["7.4.1", ""]], "type": "int"}
                        }
                    }
                },
                "elements": "dict"
            },
            "auth-portal": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "auth-virtual-host": {"v_range": [["7.2.1", ""]], "type": "str"},
            "client-cert": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "decrypted-traffic-mirror": {"v_range": [["7.2.1", ""]], "type": "str"},
            "empty-cert-action": {"v_range": [["7.2.1", ""]], "choices": ["block", "accept", "accept-unmanageable"], "type": "str"},
            "log-blocked-traffic": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "name": {"v_range": [["7.2.1", ""]], "required": true, "type": "str"},
            "user-agent-detect": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "vip": {"v_range": [["7.2.1", ""]], "type": "str"},
            "http-supported-max-version": {"v_range": [["7.2.2", ""]], "choices": ["http1", "http2"], "type": "str"},
            "svr-pool-multiplex": {"v_range": [["7.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "svr-pool-server-max-request": {"v_range": [["7.2.2", ""]], "type": "int"},
            "svr-pool-ttl": {"v_range": [["7.2.2", ""]], "type": "int"},
            "svr-pool-server-max-concurrent-request": {"v_range": [["7.4.1", ""]], "type": "int"}
        }
    },
    "firewall_accessproxy6_apigateway": {
        "stated": true,
        "options": {
            "application": {"v_range": [["7.2.1", ""]], "type": "raw"},
            "http-cookie-age": {"v_range": [["7.2.1", ""]], "type": "int"},
            "http-cookie-domain": {"v_range": [["7.2.1", ""]], "type": "str"},
            "http-cookie-domain-from-host": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "http-cookie-generation": {"v_range": [["7.2.1", ""]], "type": "int"},
            "http-cookie-path": {"v_range": [["7.2.1", ""]], "type": "str"},
            "http-cookie-share": {"v_range": [["7.2.1", ""]], "choices": ["disable", "same-ip"], "type": "str"},
            "https-cookie-secure": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "id": {"v_range": [["7.2.1", ""]], "required": true, "type": "int"},
            "ldb-method": {"v_range": [["7.2.1", ""]], "choices": ["static", "round-robin", "weighted", "first-alive", "http-host"], "type": "str"},
            "persistence": {"v_range": [["7.2.1", ""]], "choices": ["none", "http-cookie"], "type": "str"},
            "realservers": {
                "v_range": [["7.2.1", ""]],
                "type": "list",
                "options": {
                    "addr-type": {"v_range": [["7.2.1", ""]], "choices": ["fqdn", "ip"], "type": "str"},
                    "address": {"v_range": [["7.2.1", ""]], "type": "str"},
                    "domain": {"v_range": [["7.2.1", ""]], "type": "str"},
                    "health-check": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "health-check-proto": {"v_range": [["7.2.1", ""]], "choices": ["ping", "http", "tcp-connect"], "type": "str"},
                    "holddown-interval": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "http-host": {"v_range": [["7.2.1", ""]], "type": "str"},
                    "id": {"v_range": [["7.2.1", ""]], "type": "int"},
                    "ip": {"v_range": [["7.2.1", ""]], "type": "str"},
                    "mappedport": {"v_range": [["7.2.1", ""]], "type": "raw"},
                    "port": {"v_range": [["7.2.1", ""]], "type": "int"},
                    "ssh-client-cert": {"v_range": [["7.2.1", ""]], "type": "str"},
                    "ssh-host-key": {"v_range": [["7.2.1", ""]], "no_log": true, "type": "raw"},
                    "ssh-host-key-validation": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "status": {"v_range": [["7.2.1", ""]], "choices": ["active", "standby", "disable"], "type": "str"},
                    "type": {"v_range": [["7.2.1", ""]], "choices": ["tcp-forwarding", "ssh"], "type": "str"},
                    "weight": {"v_range": [["7.2.1", ""]], "type": "int"},
                    "translate-host": {"v_range": [["7.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "external-auth": {"v_range": [["7.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "tunnel-encryption": {"v_range": [["7.4.0", ""]], "choices": ["disable", "enable"], "type": "str"}
                },
                "elements": "dict"
            },
            "saml-redirect": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "saml-server": {"v_range": [["7.2.1", ""]], "type": "str"},
            "service": {"v_range": [["7.2.1", ""]], "choices": ["http", "https", "tcp-forwarding", "samlsp", "web-portal", "saas"], "type": "str"},
            "ssl-algorithm": {"v_range": [["7.2.1", ""]], "choices": ["high", "medium", "low"], "type": "str"},
            "ssl-cipher-suites": {
                "v_range": [["7.2.1", ""]],
                "type": "list",
                "options": {
                    "cipher": {
                        "v_range": [["7.2.1", ""]],
                        "choices": [
                            "TLS-RSA-WITH-RC4-128-MD5", "TLS-RSA-WITH-RC4-128-SHA", "TLS-RSA-WITH-DES-CBC-SHA", "TLS-RSA-WITH-3DES-EDE-CBC-SHA",
                            "TLS-RSA-WITH-AES-128-CBC-SHA", "TLS-RSA-WITH-AES-256-CBC-SHA", "TLS-RSA-WITH-AES-128-CBC-SHA256",
                            "TLS-RSA-WITH-AES-256-CBC-SHA256", "TLS-RSA-WITH-CAMELLIA-128-CBC-SHA", "TLS-RSA-WITH-CAMELLIA-256-CBC-SHA",
                            "TLS-RSA-WITH-CAMELLIA-128-CBC-SHA256", "TLS-RSA-WITH-CAMELLIA-256-CBC-SHA256", "TLS-RSA-WITH-SEED-CBC-SHA",
                            "TLS-RSA-WITH-ARIA-128-CBC-SHA256", "TLS-RSA-WITH-ARIA-256-CBC-SHA384", "TLS-DHE-RSA-WITH-DES-CBC-SHA",
                            "TLS-DHE-RSA-WITH-3DES-EDE-CBC-SHA", "TLS-DHE-RSA-WITH-AES-128-CBC-SHA", "TLS-DHE-RSA-WITH-AES-256-CBC-SHA",
                            "TLS-DHE-RSA-WITH-AES-128-CBC-SHA256", "TLS-DHE-RSA-WITH-AES-256-CBC-SHA256", "TLS-DHE-RSA-WITH-CAMELLIA-128-CBC-SHA",
                            "TLS-DHE-RSA-WITH-CAMELLIA-256-CBC-SHA", "TLS-DHE-RSA-WITH-CAMELLIA-128-CBC-SHA256",
                            "TLS-DHE-RSA-WITH-CAMELLIA-256-CBC-SHA256", "TLS-DHE-RSA-WITH-SEED-CBC-SHA", "TLS-DHE-RSA-WITH-ARIA-128-CBC-SHA256",
                            "TLS-DHE-RSA-WITH-ARIA-256-CBC-SHA384", "TLS-ECDHE-RSA-WITH-RC4-128-SHA", "TLS-ECDHE-RSA-WITH-3DES-EDE-CBC-SHA",
                            "TLS-ECDHE-RSA-WITH-AES-128-CBC-SHA", "TLS-ECDHE-RSA-WITH-AES-256-CBC-SHA", "TLS-ECDHE-RSA-WITH-CHACHA20-POLY1305-SHA256",
                            "TLS-ECDHE-ECDSA-WITH-CHACHA20-POLY1305-SHA256", "TLS-DHE-RSA-WITH-CHACHA20-POLY1305-SHA256",
                            "TLS-DHE-RSA-WITH-AES-128-GCM-SHA256", "TLS-DHE-RSA-WITH-AES-256-GCM-SHA384", "TLS-DHE-DSS-WITH-AES-128-CBC-SHA",
                            "TLS-DHE-DSS-WITH-AES-256-CBC-SHA", "TLS-DHE-DSS-WITH-AES-128-CBC-SHA256", "TLS-DHE-DSS-WITH-AES-128-GCM-SHA256",
                            "TLS-DHE-DSS-WITH-AES-256-CBC-SHA256", "TLS-DHE-DSS-WITH-AES-256-GCM-SHA384", "TLS-ECDHE-RSA-WITH-AES-128-CBC-SHA256",
                            "TLS-ECDHE-RSA-WITH-AES-128-GCM-SHA256", "TLS-ECDHE-RSA-WITH-AES-256-CBC-SHA384", "TLS-ECDHE-RSA-WITH-AES-256-GCM-SHA384",
                            "TLS-ECDHE-ECDSA-WITH-AES-128-CBC-SHA", "TLS-ECDHE-ECDSA-WITH-AES-128-CBC-SHA256", "TLS-ECDHE-ECDSA-WITH-AES-128-GCM-SHA256",
                            "TLS-ECDHE-ECDSA-WITH-AES-256-CBC-SHA384", "TLS-ECDHE-ECDSA-WITH-AES-256-GCM-SHA384", "TLS-RSA-WITH-AES-128-GCM-SHA256",
                            "TLS-RSA-WITH-AES-256-GCM-SHA384", "TLS-DHE-DSS-WITH-CAMELLIA-128-CBC-SHA", "TLS-DHE-DSS-WITH-CAMELLIA-256-CBC-SHA",
                            "TLS-DHE-DSS-WITH-CAMELLIA-128-CBC-SHA256", "TLS-DHE-DSS-WITH-CAMELLIA-256-CBC-SHA256", "TLS-DHE-DSS-WITH-SEED-CBC-SHA",
                            "TLS-DHE-DSS-WITH-ARIA-128-CBC-SHA256", "TLS-DHE-DSS-WITH-ARIA-256-CBC-SHA384", "TLS-ECDHE-RSA-WITH-ARIA-128-CBC-SHA256",
                            "TLS-ECDHE-RSA-WITH-ARIA-256-CBC-SHA384", "TLS-ECDHE-ECDSA-WITH-ARIA-128-CBC-SHA256",
                            "TLS-ECDHE-ECDSA-WITH-ARIA-256-CBC-SHA384", "TLS-DHE-DSS-WITH-3DES-EDE-CBC-SHA", "TLS-DHE-DSS-WITH-DES-CBC-SHA",
                            "TLS-AES-128-GCM-SHA256", "TLS-AES-256-GCM-SHA384", "TLS-CHACHA20-POLY1305-SHA256", "TLS-ECDHE-ECDSA-WITH-AES-256-CBC-SHA"
                        ],
                        "type": "str"
                    },
                    "priority": {"v_range": [["7.2.1", ""]], "type": "int"},
                    "versions": {"v_range": [["7.2.1", ""]], "type": "list", "choices": ["tls-1.0", "tls-1.1", "tls-1.2", "tls-1.3"], "elements": "str"}
                },
                "elements": "dict"
            },
            "ssl-dh-bits": {"v_range": [["7.2.1", ""]], "choices": ["768", "1024", "1536", "2048", "3072", "4096"], "type": "str"},
            "ssl-max-version": {"v_range": [["7.2.1", ""]], "choices": ["tls-1.0", "tls-1.1", "tls-1.2", "tls-1.3"], "type": "str"},
            "ssl-min-version": {"v_range": [["7.2.1", ""]], "choices": ["tls-1.0", "tls-1.1", "tls-1.2", "tls-1.3"], "type": "str"},
            "ssl-vpn-web-portal": {"v_range": [["7.2.1", ""]], "type": "str"},
            "url-map": {"v_range": [["7.2.1", ""]], "type": "str"},
            "url-map-type": {"v_range": [["7.2.1", ""]], "choices": ["sub-string", "wildcard", "regex"], "type": "str"},
            "virtual-host": {"v_range": [["7.2.1", ""]], "type": "str"},
            "ssl-renegotiation": {"v_range": [["7.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "h2-support": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "h3-support": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "quic": {
                "v_range": [["7.4.1", ""]],
                "type": "dict",
                "options": {
                    "ack-delay-exponent": {"v_range": [["7.4.1", ""]], "type": "int"},
                    "active-connection-id-limit": {"v_range": [["7.4.1", ""]], "type": "int"},
                    "active-migration": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "grease-quic-bit": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "max-ack-delay": {"v_range": [["7.4.1", ""]], "type": "int"},
                    "max-datagram-frame-size": {"v_range": [["7.4.1", ""]], "type": "int"},
                    "max-idle-timeout": {"v_range": [["7.4.1", ""]], "type": "int"},
                    "max-udp-payload-size": {"v_range": [["7.4.1", ""]], "type": "int"}
                }
            }
        }
    },
    "firewall_accessproxy6_apigateway6": {
        "stated": true,
        "options": {
            "application": {"v_range": [["7.2.1", ""]], "type": "raw"},
            "http-cookie-age": {"v_range": [["7.2.1", ""]], "type": "int"},
            "http-cookie-domain": {"v_range": [["7.2.1", ""]], "type": "str"},
            "http-cookie-domain-from-host": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "http-cookie-generation": {"v_range": [["7.2.1", ""]], "type": "int"},
            "http-cookie-path": {"v_range": [["7.2.1", ""]], "type": "str"},
            "http-cookie-share": {"v_range": [["7.2.1", ""]], "choices": ["disable", "same-ip"], "type": "str"},
            "https-cookie-secure": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "id": {"v_range": [["7.2.1", ""]], "required": true, "type": "int"},
            "ldb-method": {"v_range": [["7.2.1", ""]], "choices": ["static", "round-robin", "weighted", "first-alive", "http-host"], "type": "str"},
            "persistence": {"v_range": [["7.2.1", ""]], "choices": ["none", "http-cookie"], "type": "str"},
            "realservers": {
                "v_range": [["7.2.1", ""]],
                "type": "list",
                "options": {
                    "addr-type": {"v_range": [["7.2.1", ""]], "choices": ["fqdn", "ip"], "type": "str"},
                    "address": {"v_range": [["7.2.1", ""]], "type": "str"},
                    "domain": {"v_range": [["7.2.1", ""]], "type": "str"},
                    "health-check": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "health-check-proto": {"v_range": [["7.2.1", ""]], "choices": ["ping", "http", "tcp-connect"], "type": "str"},
                    "holddown-interval": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "http-host": {"v_range": [["7.2.1", ""]], "type": "str"},
                    "id": {"v_range": [["7.2.1", ""]], "type": "int"},
                    "ip": {"v_range": [["7.2.1", ""]], "type": "str"},
                    "mappedport": {"v_range": [["7.2.1", ""]], "type": "raw"},
                    "port": {"v_range": [["7.2.1", ""]], "type": "int"},
                    "ssh-client-cert": {"v_range": [["7.2.1", ""]], "type": "str"},
                    "ssh-host-key": {"v_range": [["7.2.1", ""]], "no_log": true, "type": "raw"},
                    "ssh-host-key-validation": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "status": {"v_range": [["7.2.1", ""]], "choices": ["active", "standby", "disable"], "type": "str"},
                    "type": {"v_range": [["7.2.1", ""]], "choices": ["tcp-forwarding", "ssh"], "type": "str"},
                    "weight": {"v_range": [["7.2.1", ""]], "type": "int"},
                    "translate-host": {"v_range": [["7.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "external-auth": {"v_range": [["7.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "tunnel-encryption": {"v_range": [["7.4.0", ""]], "choices": ["disable", "enable"], "type": "str"}
                },
                "elements": "dict"
            },
            "saml-redirect": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "saml-server": {"v_range": [["7.2.1", ""]], "type": "str"},
            "service": {"v_range": [["7.2.1", ""]], "choices": ["http", "https", "tcp-forwarding", "samlsp", "web-portal", "saas"], "type": "str"},
            "ssl-algorithm": {"v_range": [["7.2.1", ""]], "choices": ["high", "medium", "low"], "type": "str"},
            "ssl-cipher-suites": {
                "v_range": [["7.2.1", ""]],
                "type": "list",
                "options": {
                    "cipher": {
                        "v_range": [["7.2.1", ""]],
                        "choices": [
                            "TLS-RSA-WITH-RC4-128-MD5", "TLS-RSA-WITH-RC4-128-SHA", "TLS-RSA-WITH-DES-CBC-SHA", "TLS-RSA-WITH-3DES-EDE-CBC-SHA",
                            "TLS-RSA-WITH-AES-128-CBC-SHA", "TLS-RSA-WITH-AES-256-CBC-SHA", "TLS-RSA-WITH-AES-128-CBC-SHA256",
                            "TLS-RSA-WITH-AES-256-CBC-SHA256", "TLS-RSA-WITH-CAMELLIA-128-CBC-SHA", "TLS-RSA-WITH-CAMELLIA-256-CBC-SHA",
                            "TLS-RSA-WITH-CAMELLIA-128-CBC-SHA256", "TLS-RSA-WITH-CAMELLIA-256-CBC-SHA256", "TLS-RSA-WITH-SEED-CBC-SHA",
                            "TLS-RSA-WITH-ARIA-128-CBC-SHA256", "TLS-RSA-WITH-ARIA-256-CBC-SHA384", "TLS-DHE-RSA-WITH-DES-CBC-SHA",
                            "TLS-DHE-RSA-WITH-3DES-EDE-CBC-SHA", "TLS-DHE-RSA-WITH-AES-128-CBC-SHA", "TLS-DHE-RSA-WITH-AES-256-CBC-SHA",
                            "TLS-DHE-RSA-WITH-AES-128-CBC-SHA256", "TLS-DHE-RSA-WITH-AES-256-CBC-SHA256", "TLS-DHE-RSA-WITH-CAMELLIA-128-CBC-SHA",
                            "TLS-DHE-RSA-WITH-CAMELLIA-256-CBC-SHA", "TLS-DHE-RSA-WITH-CAMELLIA-128-CBC-SHA256",
                            "TLS-DHE-RSA-WITH-CAMELLIA-256-CBC-SHA256", "TLS-DHE-RSA-WITH-SEED-CBC-SHA", "TLS-DHE-RSA-WITH-ARIA-128-CBC-SHA256",
                            "TLS-DHE-RSA-WITH-ARIA-256-CBC-SHA384", "TLS-ECDHE-RSA-WITH-RC4-128-SHA", "TLS-ECDHE-RSA-WITH-3DES-EDE-CBC-SHA",
                            "TLS-ECDHE-RSA-WITH-AES-128-CBC-SHA", "TLS-ECDHE-RSA-WITH-AES-256-CBC-SHA", "TLS-ECDHE-RSA-WITH-CHACHA20-POLY1305-SHA256",
                            "TLS-ECDHE-ECDSA-WITH-CHACHA20-POLY1305-SHA256", "TLS-DHE-RSA-WITH-CHACHA20-POLY1305-SHA256",
                            "TLS-DHE-RSA-WITH-AES-128-GCM-SHA256", "TLS-DHE-RSA-WITH-AES-256-GCM-SHA384", "TLS-DHE-DSS-WITH-AES-128-CBC-SHA",
                            "TLS-DHE-DSS-WITH-AES-256-CBC-SHA", "TLS-DHE-DSS-WITH-AES-128-CBC-SHA256", "TLS-DHE-DSS-WITH-AES-128-GCM-SHA256",
                            "TLS-DHE-DSS-WITH-AES-256-CBC-SHA256", "TLS-DHE-DSS-WITH-AES-256-GCM-SHA384", "TLS-ECDHE-RSA-WITH-AES-128-CBC-SHA256",
                            "TLS-ECDHE-RSA-WITH-AES-128-GCM-SHA256", "TLS-ECDHE-RSA-WITH-AES-256-CBC-SHA384", "TLS-ECDHE-RSA-WITH-AES-256-GCM-SHA384",
                            "TLS-ECDHE-ECDSA-WITH-AES-128-CBC-SHA", "TLS-ECDHE-ECDSA-WITH-AES-128-CBC-SHA256", "TLS-ECDHE-ECDSA-WITH-AES-128-GCM-SHA256",
                            "TLS-ECDHE-ECDSA-WITH-AES-256-CBC-SHA384", "TLS-ECDHE-ECDSA-WITH-AES-256-GCM-SHA384", "TLS-RSA-WITH-AES-128-GCM-SHA256",
                            "TLS-RSA-WITH-AES-256-GCM-SHA384", "TLS-DHE-DSS-WITH-CAMELLIA-128-CBC-SHA", "TLS-DHE-DSS-WITH-CAMELLIA-256-CBC-SHA",
                            "TLS-DHE-DSS-WITH-CAMELLIA-128-CBC-SHA256", "TLS-DHE-DSS-WITH-CAMELLIA-256-CBC-SHA256", "TLS-DHE-DSS-WITH-SEED-CBC-SHA",
                            "TLS-DHE-DSS-WITH-ARIA-128-CBC-SHA256", "TLS-DHE-DSS-WITH-ARIA-256-CBC-SHA384", "TLS-ECDHE-RSA-WITH-ARIA-128-CBC-SHA256",
                            "TLS-ECDHE-RSA-WITH-ARIA-256-CBC-SHA384", "TLS-ECDHE-ECDSA-WITH-ARIA-128-CBC-SHA256",
                            "TLS-ECDHE-ECDSA-WITH-ARIA-256-CBC-SHA384", "TLS-DHE-DSS-WITH-3DES-EDE-CBC-SHA", "TLS-DHE-DSS-WITH-DES-CBC-SHA",
                            "TLS-AES-128-GCM-SHA256", "TLS-AES-256-GCM-SHA384", "TLS-CHACHA20-POLY1305-SHA256", "TLS-ECDHE-ECDSA-WITH-AES-256-CBC-SHA"
                        ],
                        "type": "str"
                    },
                    "priority": {"v_range": [["7.2.1", ""]], "type": "int"},
                    "versions": {"v_range": [["7.2.1", ""]], "type": "list", "choices": ["tls-1.0", "tls-1.1", "tls-1.2", "tls-1.3"], "elements": "str"}
                },
                "elements": "dict"
            },
            "ssl-dh-bits": {"v_range": [["7.2.1", ""]], "choices": ["768", "1024", "1536", "2048", "3072", "4096"], "type": "str"},
            "ssl-max-version": {"v_range": [["7.2.1", ""]], "choices": ["tls-1.0", "tls-1.1", "tls-1.2", "tls-1.3"], "type": "str"},
            "ssl-min-version": {"v_range": [["7.2.1", ""]], "choices": ["tls-1.0", "tls-1.1", "tls-1.2", "tls-1.3"], "type": "str"},
            "ssl-vpn-web-portal": {"v_range": [["7.2.1", ""]], "type": "str"},
            "url-map": {"v_range": [["7.2.1", ""]], "type": "str"},
            "url-map-type": {"v_range": [["7.2.1", ""]], "choices": ["sub-string", "wildcard", "regex"], "type": "str"},
            "virtual-host": {"v_range": [["7.2.1", ""]], "type": "str"},
            "ssl-renegotiation": {"v_range": [["7.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "h2-support": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "h3-support": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "quic": {
                "v_range": [["7.4.1", ""]],
                "type": "dict",
                "options": {
                    "ack-delay-exponent": {"v_range": [["7.4.1", ""]], "type": "int"},
                    "active-connection-id-limit": {"v_range": [["7.4.1", ""]], "type": "int"},
                    "active-migration": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "grease-quic-bit": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "max-ack-delay": {"v_range": [["7.4.1", ""]], "type": "int"},
                    "max-datagram-frame-size": {"v_range": [["7.4.1", ""]], "type": "int"},
                    "max-idle-timeout": {"v_range": [["7.4.1", ""]], "type": "int"},
                    "max-udp-payload-size": {"v_range": [["7.4.1", ""]], "type": "int"}
                }
            }
        }
    },
    "firewall_accessproxy6_apigateway6_quic": {
        "stated": false,
        "options": {
            "ack-delay-exponent": {"v_range": [["7.4.1", ""]], "type": "int"},
            "active-connection-id-limit": {"v_range": [["7.4.1", ""]], "type": "int"},
            "active-migration": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "grease-quic-bit": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "max-ack-delay": {"v_range": [["7.4.1", ""]], "type": "int"},
            "max-datagram-frame-size": {"v_range": [["7.4.1", ""]], "type": "int"},
            "max-idle-timeout": {"v_range": [["7.4.1", ""]], "type": "int"},
            "max-udp-payload-size": {"v_range": [["7.4.1", ""]], "type": "int"}
        }
    },
    "firewall_accessproxy6_apigateway6_realservers": {
        "stated": true,
        "options": {
            "addr-type": {"v_range": [["7.2.1", ""]], "choices": ["fqdn", "ip"], "type": "str"},
            "address": {"v_range": [["7.2.1", ""]], "type": "str"},
            "domain": {"v_range": [["7.2.1", ""]], "type": "str"},
            "health-check": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "health-check-proto": {"v_range": [["7.2.1", ""]], "choices": ["ping", "http", "tcp-connect"], "type": "str"},
            "holddown-interval": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "http-host": {"v_range": [["7.2.1", ""]], "type": "str"},
            "id": {"v_range": [["7.2.1", ""]], "required": true, "type": "int"},
            "ip": {"v_range": [["7.2.1", ""]], "type": "str"},
            "mappedport": {"v_range": [["7.2.1", ""]], "type": "raw"},
            "port": {"v_range": [["7.2.1", ""]], "type": "int"},
            "ssh-client-cert": {"v_range": [["7.2.1", ""]], "type": "str"},
            "ssh-host-key": {"v_range": [["7.2.1", ""]], "no_log": true, "type": "raw"},
            "ssh-host-key-validation": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "status": {"v_range": [["7.2.1", ""]], "choices": ["active", "standby", "disable"], "type": "str"},
            "type": {"v_range": [["7.2.1", ""]], "choices": ["tcp-forwarding", "ssh"], "type": "str"},
            "weight": {"v_range": [["7.2.1", ""]], "type": "int"},
            "translate-host": {"v_range": [["7.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "external-auth": {"v_range": [["7.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "tunnel-encryption": {"v_range": [["7.4.0", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "firewall_accessproxy6_apigateway6_sslciphersuites": {
        "stated": true,
        "options": {
            "cipher": {
                "v_range": [["7.2.1", ""]],
                "choices": [
                    "TLS-RSA-WITH-RC4-128-MD5", "TLS-RSA-WITH-RC4-128-SHA", "TLS-RSA-WITH-DES-CBC-SHA", "TLS-RSA-WITH-3DES-EDE-CBC-SHA",
                    "TLS-RSA-WITH-AES-128-CBC-SHA", "TLS-RSA-WITH-AES-256-CBC-SHA", "TLS-RSA-WITH-AES-128-CBC-SHA256", "TLS-RSA-WITH-AES-256-CBC-SHA256",
                    "TLS-RSA-WITH-CAMELLIA-128-CBC-SHA", "TLS-RSA-WITH-CAMELLIA-256-CBC-SHA", "TLS-RSA-WITH-CAMELLIA-128-CBC-SHA256",
                    "TLS-RSA-WITH-CAMELLIA-256-CBC-SHA256", "TLS-RSA-WITH-SEED-CBC-SHA", "TLS-RSA-WITH-ARIA-128-CBC-SHA256",
                    "TLS-RSA-WITH-ARIA-256-CBC-SHA384", "TLS-DHE-RSA-WITH-DES-CBC-SHA", "TLS-DHE-RSA-WITH-3DES-EDE-CBC-SHA",
                    "TLS-DHE-RSA-WITH-AES-128-CBC-SHA", "TLS-DHE-RSA-WITH-AES-256-CBC-SHA", "TLS-DHE-RSA-WITH-AES-128-CBC-SHA256",
                    "TLS-DHE-RSA-WITH-AES-256-CBC-SHA256", "TLS-DHE-RSA-WITH-CAMELLIA-128-CBC-SHA", "TLS-DHE-RSA-WITH-CAMELLIA-256-CBC-SHA",
                    "TLS-DHE-RSA-WITH-CAMELLIA-128-CBC-SHA256", "TLS-DHE-RSA-WITH-CAMELLIA-256-CBC-SHA256", "TLS-DHE-RSA-WITH-SEED-CBC-SHA",
                    "TLS-DHE-RSA-WITH-ARIA-128-CBC-SHA256", "TLS-DHE-RSA-WITH-ARIA-256-CBC-SHA384", "TLS-ECDHE-RSA-WITH-RC4-128-SHA",
                    "TLS-ECDHE-RSA-WITH-3DES-EDE-CBC-SHA", "TLS-ECDHE-RSA-WITH-AES-128-CBC-SHA", "TLS-ECDHE-RSA-WITH-AES-256-CBC-SHA",
                    "TLS-ECDHE-RSA-WITH-CHACHA20-POLY1305-SHA256", "TLS-ECDHE-ECDSA-WITH-CHACHA20-POLY1305-SHA256",
                    "TLS-DHE-RSA-WITH-CHACHA20-POLY1305-SHA256", "TLS-DHE-RSA-WITH-AES-128-GCM-SHA256", "TLS-DHE-RSA-WITH-AES-256-GCM-SHA384",
                    "TLS-DHE-DSS-WITH-AES-128-CBC-SHA", "TLS-DHE-DSS-WITH-AES-256-CBC-SHA", "TLS-DHE-DSS-WITH-AES-128-CBC-SHA256",
                    "TLS-DHE-DSS-WITH-AES-128-GCM-SHA256", "TLS-DHE-DSS-WITH-AES-256-CBC-SHA256", "TLS-DHE-DSS-WITH-AES-256-GCM-SHA384",
                    "TLS-ECDHE-RSA-WITH-AES-128-CBC-SHA256", "TLS-ECDHE-RSA-WITH-AES-128-GCM-SHA256", "TLS-ECDHE-RSA-WITH-AES-256-CBC-SHA384",
                    "TLS-ECDHE-RSA-WITH-AES-256-GCM-SHA384", "TLS-ECDHE-ECDSA-WITH-AES-128-CBC-SHA", "TLS-ECDHE-ECDSA-WITH-AES-128-CBC-SHA256",
                    "TLS-ECDHE-ECDSA-WITH-AES-128-GCM-SHA256", "TLS-ECDHE-ECDSA-WITH-AES-256-CBC-SHA384", "TLS-ECDHE-ECDSA-WITH-AES-256-GCM-SHA384",
                    "TLS-RSA-WITH-AES-128-GCM-SHA256", "TLS-RSA-WITH-AES-256-GCM-SHA384", "TLS-DHE-DSS-WITH-CAMELLIA-128-CBC-SHA",
                    "TLS-DHE-DSS-WITH-CAMELLIA-256-CBC-SHA", "TLS-DHE-DSS-WITH-CAMELLIA-128-CBC-SHA256", "TLS-DHE-DSS-WITH-CAMELLIA-256-CBC-SHA256",
                    "TLS-DHE-DSS-WITH-SEED-CBC-SHA", "TLS-DHE-DSS-WITH-ARIA-128-CBC-SHA256", "TLS-DHE-DSS-WITH-ARIA-256-CBC-SHA384",
                    "TLS-ECDHE-RSA-WITH-ARIA-128-CBC-SHA256", "TLS-ECDHE-RSA-WITH-ARIA-256-CBC-SHA384", "TLS-ECDHE-ECDSA-WITH-ARIA-128-CBC-SHA256",
                    "TLS-ECDHE-ECDSA-WITH-ARIA-256-CBC-SHA384", "TLS-DHE-DSS-WITH-3DES-EDE-CBC-SHA", "TLS-DHE-DSS-WITH-DES-CBC-SHA",
                    "TLS-AES-128-GCM-SHA256", "TLS-AES-256-GCM-SHA384", "TLS-CHACHA20-POLY1305-SHA256", "TLS-ECDHE-ECDSA-WITH-AES-256-CBC-SHA"
                ],
                "type": "str"
            },
            "priority": {"v_range": [["7.2.1", ""]], "type": "int"},
            "versions": {"v_range": [["7.2.1", ""]], "type": "list", "choices": ["tls-1.0", "tls-1.1", "tls-1.2", "tls-1.3"], "elements": "str"}
        }
    },
    "firewall_accessproxy6_apigateway_quic": {
        "stated": false,
        "options": {
            "ack-delay-exponent": {"v_range": [["7.4.1", ""]], "type": "int"},
            "active-connection-id-limit": {"v_range": [["7.4.1", ""]], "type": "int"},
            "active-migration": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "grease-quic-bit": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "max-ack-delay": {"v_range": [["7.4.1", ""]], "type": "int"},
            "max-datagram-frame-size": {"v_range": [["7.4.1", ""]], "type": "int"},
            "max-idle-timeout": {"v_range": [["7.4.1", ""]], "type": "int"},
            "max-udp-payload-size": {"v_range": [["7.4.1", ""]], "type": "int"}
        }
    },
    "firewall_accessproxy6_apigateway_realservers": {
        "stated": true,
        "options": {
            "addr-type": {"v_range": [["7.2.1", ""]], "choices": ["fqdn", "ip"], "type": "str"},
            "address": {"v_range": [["7.2.1", ""]], "type": "str"},
            "domain": {"v_range": [["7.2.1", ""]], "type": "str"},
            "health-check": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "health-check-proto": {"v_range": [["7.2.1", ""]], "choices": ["ping", "http", "tcp-connect"], "type": "str"},
            "holddown-interval": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "http-host": {"v_range": [["7.2.1", ""]], "type": "str"},
            "id": {"v_range": [["7.2.1", ""]], "required": true, "type": "int"},
            "ip": {"v_range": [["7.2.1", ""]], "type": "str"},
            "mappedport": {"v_range": [["7.2.1", ""]], "type": "raw"},
            "port": {"v_range": [["7.2.1", ""]], "type": "int"},
            "ssh-client-cert": {"v_range": [["7.2.1", ""]], "type": "str"},
            "ssh-host-key": {"v_range": [["7.2.1", ""]], "no_log": true, "type": "raw"},
            "ssh-host-key-validation": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "status": {"v_range": [["7.2.1", ""]], "choices": ["active", "standby", "disable"], "type": "str"},
            "type": {"v_range": [["7.2.1", ""]], "choices": ["tcp-forwarding", "ssh"], "type": "str"},
            "weight": {"v_range": [["7.2.1", ""]], "type": "int"},
            "translate-host": {"v_range": [["7.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "external-auth": {"v_range": [["7.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "tunnel-encryption": {"v_range": [["7.4.0", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "firewall_accessproxy6_apigateway_sslciphersuites": {
        "stated": true,
        "options": {
            "cipher": {
                "v_range": [["7.2.1", ""]],
                "choices": [
                    "TLS-RSA-WITH-RC4-128-MD5", "TLS-RSA-WITH-RC4-128-SHA", "TLS-RSA-WITH-DES-CBC-SHA", "TLS-RSA-WITH-3DES-EDE-CBC-SHA",
                    "TLS-RSA-WITH-AES-128-CBC-SHA", "TLS-RSA-WITH-AES-256-CBC-SHA", "TLS-RSA-WITH-AES-128-CBC-SHA256", "TLS-RSA-WITH-AES-256-CBC-SHA256",
                    "TLS-RSA-WITH-CAMELLIA-128-CBC-SHA", "TLS-RSA-WITH-CAMELLIA-256-CBC-SHA", "TLS-RSA-WITH-CAMELLIA-128-CBC-SHA256",
                    "TLS-RSA-WITH-CAMELLIA-256-CBC-SHA256", "TLS-RSA-WITH-SEED-CBC-SHA", "TLS-RSA-WITH-ARIA-128-CBC-SHA256",
                    "TLS-RSA-WITH-ARIA-256-CBC-SHA384", "TLS-DHE-RSA-WITH-DES-CBC-SHA", "TLS-DHE-RSA-WITH-3DES-EDE-CBC-SHA",
                    "TLS-DHE-RSA-WITH-AES-128-CBC-SHA", "TLS-DHE-RSA-WITH-AES-256-CBC-SHA", "TLS-DHE-RSA-WITH-AES-128-CBC-SHA256",
                    "TLS-DHE-RSA-WITH-AES-256-CBC-SHA256", "TLS-DHE-RSA-WITH-CAMELLIA-128-CBC-SHA", "TLS-DHE-RSA-WITH-CAMELLIA-256-CBC-SHA",
                    "TLS-DHE-RSA-WITH-CAMELLIA-128-CBC-SHA256", "TLS-DHE-RSA-WITH-CAMELLIA-256-CBC-SHA256", "TLS-DHE-RSA-WITH-SEED-CBC-SHA",
                    "TLS-DHE-RSA-WITH-ARIA-128-CBC-SHA256", "TLS-DHE-RSA-WITH-ARIA-256-CBC-SHA384", "TLS-ECDHE-RSA-WITH-RC4-128-SHA",
                    "TLS-ECDHE-RSA-WITH-3DES-EDE-CBC-SHA", "TLS-ECDHE-RSA-WITH-AES-128-CBC-SHA", "TLS-ECDHE-RSA-WITH-AES-256-CBC-SHA",
                    "TLS-ECDHE-RSA-WITH-CHACHA20-POLY1305-SHA256", "TLS-ECDHE-ECDSA-WITH-CHACHA20-POLY1305-SHA256",
                    "TLS-DHE-RSA-WITH-CHACHA20-POLY1305-SHA256", "TLS-DHE-RSA-WITH-AES-128-GCM-SHA256", "TLS-DHE-RSA-WITH-AES-256-GCM-SHA384",
                    "TLS-DHE-DSS-WITH-AES-128-CBC-SHA", "TLS-DHE-DSS-WITH-AES-256-CBC-SHA", "TLS-DHE-DSS-WITH-AES-128-CBC-SHA256",
                    "TLS-DHE-DSS-WITH-AES-128-GCM-SHA256", "TLS-DHE-DSS-WITH-AES-256-CBC-SHA256", "TLS-DHE-DSS-WITH-AES-256-GCM-SHA384",
                    "TLS-ECDHE-RSA-WITH-AES-128-CBC-SHA256", "TLS-ECDHE-RSA-WITH-AES-128-GCM-SHA256", "TLS-ECDHE-RSA-WITH-AES-256-CBC-SHA384",
                    "TLS-ECDHE-RSA-WITH-AES-256-GCM-SHA384", "TLS-ECDHE-ECDSA-WITH-AES-128-CBC-SHA", "TLS-ECDHE-ECDSA-WITH-AES-128-CBC-SHA256",
                    "TLS-ECDHE-ECDSA-WITH-AES-128-GCM-SHA256", "TLS-ECDHE-ECDSA-WITH-AES-256-CBC-SHA384", "TLS-ECDHE-ECDSA-WITH-AES-256-GCM-SHA384",
                    "TLS-RSA-WITH-AES-128-GCM-SHA256", "TLS-RSA-WITH-AES-256-GCM-SHA384", "TLS-DHE-DSS-WITH-CAMELLIA-128-CBC-SHA",
                    "TLS-DHE-DSS-WITH-CAMELLIA-256-CBC-SHA", "TLS-DHE-DSS-WITH-CAMELLIA-128-CBC-SHA256", "TLS-DHE-DSS-WITH-CAMELLIA-256-CBC-SHA256",
                    "TLS-DHE-DSS-WITH-SEED-CBC-SHA", "TLS-DHE-DSS-WITH-ARIA-128-CBC-SHA256", "TLS-DHE-DSS-WITH-ARIA-256-CBC-SHA384",
                    "TLS-ECDHE-RSA-WITH-ARIA-128-CBC-SHA256", "TLS-ECDHE-RSA-WITH-ARIA-256-CBC-SHA384", "TLS-ECDHE-ECDSA-WITH-ARIA-128-CBC-SHA256",
                    "TLS-ECDHE-ECDSA-WITH-ARIA-256-CBC-SHA384", "TLS-DHE-DSS-WITH-3DES-EDE-CBC-SHA", "TLS-DHE-DSS-WITH-DES-CBC-SHA",
                    "TLS-AES-128-GCM-SHA256", "TLS-AES-256-GCM-SHA384", "TLS-CHACHA20-POLY1305-SHA256", "TLS-ECDHE-ECDSA-WITH-AES-256-CBC-SHA"
                ],
                "type": "str"
            },
            "priority": {"v_range": [["7.2.1", ""]], "type": "int"},
            "versions": {"v_range": [["7.2.1", ""]], "type": "list", "choices": ["tls-1.0", "tls-1.1", "tls-1.2", "tls-1.3"], "elements": "str"}
        }
    },
    "firewall_accessproxy_apigateway": {
        "stated": true,
        "options": {
            "http-cookie-age": {"v_range": [["7.0.0", ""]], "type": "int"},
            "http-cookie-domain": {"v_range": [["7.0.0", ""]], "type": "str"},
            "http-cookie-domain-from-host": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "http-cookie-generation": {"v_range": [["7.0.0", ""]], "type": "int"},
            "http-cookie-path": {"v_range": [["7.0.0", ""]], "type": "str"},
            "http-cookie-share": {"v_range": [["7.0.0", ""]], "choices": ["disable", "same-ip"], "type": "str"},
            "https-cookie-secure": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "id": {"v_range": [["7.0.0", ""]], "required": true, "type": "int"},
            "ldb-method": {
                "v_range": [["7.0.0", ""]],
                "choices": ["static", "round-robin", "weighted", "least-session", "least-rtt", "first-alive", "http-host"],
                "type": "str"
            },
            "persistence": {"v_range": [["7.0.0", ""]], "choices": ["none", "http-cookie"], "type": "str"},
            "realservers": {
                "v_range": [["7.0.0", ""]],
                "type": "list",
                "options": {
                    "address": {"v_range": [["7.0.0", ""]], "type": "str"},
                    "health-check": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "health-check-proto": {"v_range": [["7.0.0", ""]], "choices": ["ping", "http", "tcp-connect"], "type": "str"},
                    "http-host": {"v_range": [["7.0.0", ""]], "type": "str"},
                    "id": {"v_range": [["7.0.0", ""]], "type": "int"},
                    "ip": {"v_range": [["7.0.0", ""]], "type": "str"},
                    "mappedport": {"v_range": [["7.0.0", ""]], "type": "raw"},
                    "port": {"v_range": [["7.0.0", ""]], "type": "int"},
                    "status": {"v_range": [["7.0.0", ""]], "choices": ["active", "standby", "disable"], "type": "str"},
                    "weight": {"v_range": [["7.0.0", ""]], "type": "int"},
                    "addr-type": {"v_range": [["7.0.2", ""]], "choices": ["fqdn", "ip"], "type": "str"},
                    "domain": {"v_range": [["7.0.3", ""]], "type": "str"},
                    "holddown-interval": {"v_range": [["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "ssh-client-cert": {"v_range": [["7.0.1", ""]], "type": "str"},
                    "ssh-host-key": {"v_range": [["7.0.1", ""]], "no_log": true, "type": "raw"},
                    "ssh-host-key-validation": {"v_range": [["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "type": {"v_range": [["7.0.1", ""]], "choices": ["tcp-forwarding", "ssh"], "type": "str"},
                    "translate-host": {"v_range": [["7.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "external-auth": {"v_range": [["7.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "tunnel-encryption": {"v_range": [["7.4.0", ""]], "choices": ["disable", "enable"], "type": "str"}
                },
                "elements": "dict"
            },
            "saml-server": {"v_range": [["7.0.0", ""]], "type": "str"},
            "service": {"v_range": [["7.0.0", ""]], "choices": ["http", "https", "tcp-forwarding", "samlsp", "web-portal", "saas"], "type": "str"},
            "ssl-algorithm": {"v_range": [["7.0.0", ""]], "choices": ["high", "medium", "low", "custom"], "type": "str"},
            "ssl-cipher-suites": {
                "v_range": [["7.0.0", ""]],
                "type": "list",
                "options": {
                    "cipher": {
                        "v_range": [["7.0.0", ""]],
                        "choices": [
                            "TLS-RSA-WITH-RC4-128-MD5", "TLS-RSA-WITH-RC4-128-SHA", "TLS-RSA-WITH-DES-CBC-SHA", "TLS-RSA-WITH-3DES-EDE-CBC-SHA",
                            "TLS-RSA-WITH-AES-128-CBC-SHA", "TLS-RSA-WITH-AES-256-CBC-SHA", "TLS-RSA-WITH-AES-128-CBC-SHA256",
                            "TLS-RSA-WITH-AES-256-CBC-SHA256", "TLS-RSA-WITH-CAMELLIA-128-CBC-SHA", "TLS-RSA-WITH-CAMELLIA-256-CBC-SHA",
                            "TLS-RSA-WITH-CAMELLIA-128-CBC-SHA256", "TLS-RSA-WITH-CAMELLIA-256-CBC-SHA256", "TLS-RSA-WITH-SEED-CBC-SHA",
                            "TLS-RSA-WITH-ARIA-128-CBC-SHA256", "TLS-RSA-WITH-ARIA-256-CBC-SHA384", "TLS-DHE-RSA-WITH-DES-CBC-SHA",
                            "TLS-DHE-RSA-WITH-3DES-EDE-CBC-SHA", "TLS-DHE-RSA-WITH-AES-128-CBC-SHA", "TLS-DHE-RSA-WITH-AES-256-CBC-SHA",
                            "TLS-DHE-RSA-WITH-AES-128-CBC-SHA256", "TLS-DHE-RSA-WITH-AES-256-CBC-SHA256", "TLS-DHE-RSA-WITH-CAMELLIA-128-CBC-SHA",
                            "TLS-DHE-RSA-WITH-CAMELLIA-256-CBC-SHA", "TLS-DHE-RSA-WITH-CAMELLIA-128-CBC-SHA256",
                            "TLS-DHE-RSA-WITH-CAMELLIA-256-CBC-SHA256", "TLS-DHE-RSA-WITH-SEED-CBC-SHA", "TLS-DHE-RSA-WITH-ARIA-128-CBC-SHA256",
                            "TLS-DHE-RSA-WITH-ARIA-256-CBC-SHA384", "TLS-ECDHE-RSA-WITH-RC4-128-SHA", "TLS-ECDHE-RSA-WITH-3DES-EDE-CBC-SHA",
                            "TLS-ECDHE-RSA-WITH-AES-128-CBC-SHA", "TLS-ECDHE-RSA-WITH-AES-256-CBC-SHA", "TLS-ECDHE-RSA-WITH-CHACHA20-POLY1305-SHA256",
                            "TLS-ECDHE-ECDSA-WITH-CHACHA20-POLY1305-SHA256", "TLS-DHE-RSA-WITH-CHACHA20-POLY1305-SHA256",
                            "TLS-DHE-RSA-WITH-AES-128-GCM-SHA256", "TLS-DHE-RSA-WITH-AES-256-GCM-SHA384", "TLS-DHE-DSS-WITH-AES-128-CBC-SHA",
                            "TLS-DHE-DSS-WITH-AES-256-CBC-SHA", "TLS-DHE-DSS-WITH-AES-128-CBC-SHA256", "TLS-DHE-DSS-WITH-AES-128-GCM-SHA256",
                            "TLS-DHE-DSS-WITH-AES-256-CBC-SHA256", "TLS-DHE-DSS-WITH-AES-256-GCM-SHA384", "TLS-ECDHE-RSA-WITH-AES-128-CBC-SHA256",
                            "TLS-ECDHE-RSA-WITH-AES-128-GCM-SHA256", "TLS-ECDHE-RSA-WITH-AES-256-CBC-SHA384", "TLS-ECDHE-RSA-WITH-AES-256-GCM-SHA384",
                            "TLS-ECDHE-ECDSA-WITH-AES-128-CBC-SHA", "TLS-ECDHE-ECDSA-WITH-AES-128-CBC-SHA256", "TLS-ECDHE-ECDSA-WITH-AES-128-GCM-SHA256",
                            "TLS-ECDHE-ECDSA-WITH-AES-256-CBC-SHA384", "TLS-ECDHE-ECDSA-WITH-AES-256-GCM-SHA384", "TLS-RSA-WITH-AES-128-GCM-SHA256",
                            "TLS-RSA-WITH-AES-256-GCM-SHA384", "TLS-DHE-DSS-WITH-CAMELLIA-128-CBC-SHA", "TLS-DHE-DSS-WITH-CAMELLIA-256-CBC-SHA",
                            "TLS-DHE-DSS-WITH-CAMELLIA-128-CBC-SHA256", "TLS-DHE-DSS-WITH-CAMELLIA-256-CBC-SHA256", "TLS-DHE-DSS-WITH-SEED-CBC-SHA",
                            "TLS-DHE-DSS-WITH-ARIA-128-CBC-SHA256", "TLS-DHE-DSS-WITH-ARIA-256-CBC-SHA384", "TLS-ECDHE-RSA-WITH-ARIA-128-CBC-SHA256",
                            "TLS-ECDHE-RSA-WITH-ARIA-256-CBC-SHA384", "TLS-ECDHE-ECDSA-WITH-ARIA-128-CBC-SHA256",
                            "TLS-ECDHE-ECDSA-WITH-ARIA-256-CBC-SHA384", "TLS-DHE-DSS-WITH-3DES-EDE-CBC-SHA", "TLS-DHE-DSS-WITH-DES-CBC-SHA",
                            "TLS-AES-128-GCM-SHA256", "TLS-AES-256-GCM-SHA384", "TLS-CHACHA20-POLY1305-SHA256", "TLS-ECDHE-ECDSA-WITH-AES-256-CBC-SHA"
                        ],
                        "type": "str"
                    },
                    "priority": {"v_range": [["7.0.0", ""]], "type": "int"},
                    "versions": {"v_range": [["7.0.0", ""]], "type": "list", "choices": ["tls-1.0", "tls-1.1", "tls-1.2", "tls-1.3"], "elements": "str"}
                },
                "elements": "dict"
            },
            "ssl-dh-bits": {"v_range": [["7.0.0", ""]], "choices": ["768", "1024", "1536", "2048", "3072", "4096"], "type": "str"},
            "ssl-max-version": {"v_range": [["7.0.0", ""]], "choices": ["tls-1.0", "tls-1.1", "tls-1.2", "tls-1.3"], "type": "str"},
            "ssl-min-version": {"v_range": [["7.0.0", ""]], "choices": ["tls-1.0", "tls-1.1", "tls-1.2", "tls-1.3"], "type": "str"},
            "url-map": {"v_range": [["7.0.0", ""]], "type": "str"},
            "url-map-type": {"v_range": [["7.0.0", ""]], "choices": ["sub-string", "wildcard", "regex"], "type": "str"},
            "virtual-host": {"v_range": [["7.0.0", ""]], "type": "str"},
            "saml-redirect": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ssl-vpn-web-portal": {"v_range": [["7.0.3", ""]], "type": "str"},
            "application": {"v_range": [["7.2.1", ""]], "type": "raw"},
            "ssl-renegotiation": {"v_range": [["7.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "h2-support": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "h3-support": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "quic": {
                "v_range": [["7.4.1", ""]],
                "type": "dict",
                "options": {
                    "ack-delay-exponent": {"v_range": [["7.4.1", ""]], "type": "int"},
                    "active-connection-id-limit": {"v_range": [["7.4.1", ""]], "type": "int"},
                    "active-migration": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "grease-quic-bit": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "max-ack-delay": {"v_range": [["7.4.1", ""]], "type": "int"},
                    "max-datagram-frame-size": {"v_range": [["7.4.1", ""]], "type": "int"},
                    "max-idle-timeout": {"v_range": [["7.4.1", ""]], "type": "int"},
                    "max-udp-payload-size": {"v_range": [["7.4.1", ""]], "type": "int"}
                }
            }
        }
    },
    "firewall_accessproxy_apigateway6": {
        "stated": true,
        "options": {
            "http-cookie-age": {"v_range": [["7.0.1", ""]], "type": "int"},
            "http-cookie-domain": {"v_range": [["7.0.1", ""]], "type": "str"},
            "http-cookie-domain-from-host": {"v_range": [["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "http-cookie-generation": {"v_range": [["7.0.1", ""]], "type": "int"},
            "http-cookie-path": {"v_range": [["7.0.1", ""]], "type": "str"},
            "http-cookie-share": {"v_range": [["7.0.1", ""]], "choices": ["disable", "same-ip"], "type": "str"},
            "https-cookie-secure": {"v_range": [["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "id": {"v_range": [["7.0.1", ""]], "required": true, "type": "int"},
            "ldb-method": {"v_range": [["7.0.1", ""]], "choices": ["static", "round-robin", "weighted", "first-alive", "http-host"], "type": "str"},
            "persistence": {"v_range": [["7.0.1", ""]], "choices": ["none", "http-cookie"], "type": "str"},
            "realservers": {
                "v_range": [["7.0.1", ""]],
                "type": "list",
                "options": {
                    "addr-type": {"v_range": [["7.0.2", ""]], "choices": ["fqdn", "ip"], "type": "str"},
                    "address": {"v_range": [["7.0.1", ""]], "type": "str"},
                    "domain": {"v_range": [["7.0.3", ""]], "type": "str"},
                    "health-check": {"v_range": [["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "health-check-proto": {"v_range": [["7.0.1", ""]], "choices": ["ping", "http", "tcp-connect"], "type": "str"},
                    "holddown-interval": {"v_range": [["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "http-host": {"v_range": [["7.0.1", ""]], "type": "str"},
                    "id": {"v_range": [["7.0.1", ""]], "type": "int"},
                    "ip": {"v_range": [["7.0.1", ""]], "type": "str"},
                    "mappedport": {"v_range": [["7.0.1", ""]], "type": "raw"},
                    "port": {"v_range": [["7.0.1", ""]], "type": "int"},
                    "ssh-client-cert": {"v_range": [["7.0.1", ""]], "type": "str"},
                    "ssh-host-key": {"v_range": [["7.0.1", ""]], "no_log": true, "type": "raw"},
                    "ssh-host-key-validation": {"v_range": [["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "status": {"v_range": [["7.0.1", ""]], "choices": ["active", "standby", "disable"], "type": "str"},
                    "type": {"v_range": [["7.0.1", ""]], "choices": ["tcp-forwarding", "ssh"], "type": "str"},
                    "weight": {"v_range": [["7.0.1", ""]], "type": "int"},
                    "translate-host": {"v_range": [["7.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "external-auth": {"v_range": [["7.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "tunnel-encryption": {"v_range": [["7.4.0", ""]], "choices": ["disable", "enable"], "type": "str"}
                },
                "elements": "dict"
            },
            "saml-redirect": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "saml-server": {"v_range": [["7.0.1", ""]], "type": "str"},
            "service": {"v_range": [["7.0.1", ""]], "choices": ["http", "https", "tcp-forwarding", "samlsp", "web-portal", "saas"], "type": "str"},
            "ssl-algorithm": {"v_range": [["7.0.1", ""]], "choices": ["high", "medium", "low"], "type": "str"},
            "ssl-cipher-suites": {
                "v_range": [["7.0.1", ""]],
                "type": "list",
                "options": {
                    "cipher": {
                        "v_range": [["7.0.1", ""]],
                        "choices": [
                            "TLS-RSA-WITH-RC4-128-MD5", "TLS-RSA-WITH-RC4-128-SHA", "TLS-RSA-WITH-DES-CBC-SHA", "TLS-RSA-WITH-3DES-EDE-CBC-SHA",
                            "TLS-RSA-WITH-AES-128-CBC-SHA", "TLS-RSA-WITH-AES-256-CBC-SHA", "TLS-RSA-WITH-AES-128-CBC-SHA256",
                            "TLS-RSA-WITH-AES-256-CBC-SHA256", "TLS-RSA-WITH-CAMELLIA-128-CBC-SHA", "TLS-RSA-WITH-CAMELLIA-256-CBC-SHA",
                            "TLS-RSA-WITH-CAMELLIA-128-CBC-SHA256", "TLS-RSA-WITH-CAMELLIA-256-CBC-SHA256", "TLS-RSA-WITH-SEED-CBC-SHA",
                            "TLS-RSA-WITH-ARIA-128-CBC-SHA256", "TLS-RSA-WITH-ARIA-256-CBC-SHA384", "TLS-DHE-RSA-WITH-DES-CBC-SHA",
                            "TLS-DHE-RSA-WITH-3DES-EDE-CBC-SHA", "TLS-DHE-RSA-WITH-AES-128-CBC-SHA", "TLS-DHE-RSA-WITH-AES-256-CBC-SHA",
                            "TLS-DHE-RSA-WITH-AES-128-CBC-SHA256", "TLS-DHE-RSA-WITH-AES-256-CBC-SHA256", "TLS-DHE-RSA-WITH-CAMELLIA-128-CBC-SHA",
                            "TLS-DHE-RSA-WITH-CAMELLIA-256-CBC-SHA", "TLS-DHE-RSA-WITH-CAMELLIA-128-CBC-SHA256",
                            "TLS-DHE-RSA-WITH-CAMELLIA-256-CBC-SHA256", "TLS-DHE-RSA-WITH-SEED-CBC-SHA", "TLS-DHE-RSA-WITH-ARIA-128-CBC-SHA256",
                            "TLS-DHE-RSA-WITH-ARIA-256-CBC-SHA384", "TLS-ECDHE-RSA-WITH-RC4-128-SHA", "TLS-ECDHE-RSA-WITH-3DES-EDE-CBC-SHA",
                            "TLS-ECDHE-RSA-WITH-AES-128-CBC-SHA", "TLS-ECDHE-RSA-WITH-AES-256-CBC-SHA", "TLS-ECDHE-RSA-WITH-CHACHA20-POLY1305-SHA256",
                            "TLS-ECDHE-ECDSA-WITH-CHACHA20-POLY1305-SHA256", "TLS-DHE-RSA-WITH-CHACHA20-POLY1305-SHA256",
                            "TLS-DHE-RSA-WITH-AES-128-GCM-SHA256", "TLS-DHE-RSA-WITH-AES-256-GCM-SHA384", "TLS-DHE-DSS-WITH-AES-128-CBC-SHA",
                            "TLS-DHE-DSS-WITH-AES-256-CBC-SHA", "TLS-DHE-DSS-WITH-AES-128-CBC-SHA256", "TLS-DHE-DSS-WITH-AES-128-GCM-SHA256",
                            "TLS-DHE-DSS-WITH-AES-256-CBC-SHA256", "TLS-DHE-DSS-WITH-AES-256-GCM-SHA384", "TLS-ECDHE-RSA-WITH-AES-128-CBC-SHA256",
                            "TLS-ECDHE-RSA-WITH-AES-128-GCM-SHA256", "TLS-ECDHE-RSA-WITH-AES-256-CBC-SHA384", "TLS-ECDHE-RSA-WITH-AES-256-GCM-SHA384",
                            "TLS-ECDHE-ECDSA-WITH-AES-128-CBC-SHA", "TLS-ECDHE-ECDSA-WITH-AES-128-CBC-SHA256", "TLS-ECDHE-ECDSA-WITH-AES-128-GCM-SHA256",
                            "TLS-ECDHE-ECDSA-WITH-AES-256-CBC-SHA384", "TLS-ECDHE-ECDSA-WITH-AES-256-GCM-SHA384", "TLS-RSA-WITH-AES-128-GCM-SHA256",
                            "TLS-RSA-WITH-AES-256-GCM-SHA384", "TLS-DHE-DSS-WITH-CAMELLIA-128-CBC-SHA", "TLS-DHE-DSS-WITH-CAMELLIA-256-CBC-SHA",
                            "TLS-DHE-DSS-WITH-CAMELLIA-128-CBC-SHA256", "TLS-DHE-DSS-WITH-CAMELLIA-256-CBC-SHA256", "TLS-DHE-DSS-WITH-SEED-CBC-SHA",
                            "TLS-DHE-DSS-WITH-ARIA-128-CBC-SHA256", "TLS-DHE-DSS-WITH-ARIA-256-CBC-SHA384", "TLS-ECDHE-RSA-WITH-ARIA-128-CBC-SHA256",
                            "TLS-ECDHE-RSA-WITH-ARIA-256-CBC-SHA384", "TLS-ECDHE-ECDSA-WITH-ARIA-128-CBC-SHA256",
                            "TLS-ECDHE-ECDSA-WITH-ARIA-256-CBC-SHA384", "TLS-DHE-DSS-WITH-3DES-EDE-CBC-SHA", "TLS-DHE-DSS-WITH-DES-CBC-SHA",
                            "TLS-AES-128-GCM-SHA256", "TLS-AES-256-GCM-SHA384", "TLS-CHACHA20-POLY1305-SHA256", "TLS-ECDHE-ECDSA-WITH-AES-256-CBC-SHA"
                        ],
                        "type": "str"
                    },
                    "priority": {"v_range": [["7.0.1", ""]], "type": "int"},
                    "versions": {"v_range": [["7.0.1", ""]], "type": "list", "choices": ["tls-1.0", "tls-1.1", "tls-1.2", "tls-1.3"], "elements": "str"}
                },
                "elements": "dict"
            },
            "ssl-dh-bits": {"v_range": [["7.0.1", ""]], "choices": ["768", "1024", "1536", "2048", "3072", "4096"], "type": "str"},
            "ssl-max-version": {"v_range": [["7.0.1", ""]], "choices": ["tls-1.0", "tls-1.1", "tls-1.2", "tls-1.3"], "type": "str"},
            "ssl-min-version": {"v_range": [["7.0.1", ""]], "choices": ["tls-1.0", "tls-1.1", "tls-1.2", "tls-1.3"], "type": "str"},
            "ssl-vpn-web-portal": {"v_range": [["7.0.3", ""]], "type": "str"},
            "url-map": {"v_range": [["7.0.1", ""]], "type": "str"},
            "url-map-type": {"v_range": [["7.0.1", ""]], "choices": ["sub-string", "wildcard", "regex"], "type": "str"},
            "virtual-host": {"v_range": [["7.0.1", ""]], "type": "str"},
            "application": {"v_range": [["7.2.1", ""]], "type": "raw"},
            "ssl-renegotiation": {"v_range": [["7.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "h2-support": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "h3-support": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "quic": {
                "v_range": [["7.4.1", ""]],
                "type": "dict",
                "options": {
                    "ack-delay-exponent": {"v_range": [["7.4.1", ""]], "type": "int"},
                    "active-connection-id-limit": {"v_range": [["7.4.1", ""]], "type": "int"},
                    "active-migration": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "grease-quic-bit": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "max-ack-delay": {"v_range": [["7.4.1", ""]], "type": "int"},
                    "max-datagram-frame-size": {"v_range": [["7.4.1", ""]], "type": "int"},
                    "max-idle-timeout": {"v_range": [["7.4.1", ""]], "type": "int"},
                    "max-udp-payload-size": {"v_range": [["7.4.1", ""]], "type": "int"}
                }
            }
        }
    },
    "firewall_accessproxy_apigateway6_quic": {
        "stated": false,
        "options": {
            "ack-delay-exponent": {"v_range": [["7.4.1", ""]], "type": "int"},
            "active-connection-id-limit": {"v_range": [["7.4.1", ""]], "type": "int"},
            "active-migration": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "grease-quic-bit": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "max-ack-delay": {"v_range": [["7.4.1", ""]], "type": "int"},
            "max-datagram-frame-size": {"v_range": [["7.4.1", ""]], "type": "int"},
            "max-idle-timeout": {"v_range": [["7.4.1", ""]], "type": "int"},
            "max-udp-payload-size": {"v_range": [["7.4.1", ""]], "type": "int"}
        }
    },
    "firewall_accessproxy_apigateway6_realservers": {
        "stated": true,
        "options": {
            "addr-type": {"v_range": [["7.0.2", ""]], "choices": ["fqdn", "ip"], "type": "str"},
            "address": {"v_range": [["7.0.1", ""]], "type": "str"},
            "domain": {"v_range": [["7.0.3", ""]], "type": "str"},
            "health-check": {"v_range": [["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "health-check-proto": {"v_range": [["7.0.1", ""]], "choices": ["ping", "http", "tcp-connect"], "type": "str"},
            "holddown-interval": {"v_range": [["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "http-host": {"v_range": [["7.0.1", ""]], "type": "str"},
            "id": {"v_range": [["7.0.1", ""]], "required": true, "type": "int"},
            "ip": {"v_range": [["7.0.1", ""]], "type": "str"},
            "mappedport": {"v_range": [["7.0.1", ""]], "type": "raw"},
            "port": {"v_range": [["7.0.1", ""]], "type": "int"},
            "ssh-client-cert": {"v_range": [["7.0.1", ""]], "type": "str"},
            "ssh-host-key": {"v_range": [["7.0.1", ""]], "no_log": true, "type": "raw"},
            "ssh-host-key-validation": {"v_range": [["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "status": {"v_range": [["7.0.1", ""]], "choices": ["active", "standby", "disable"], "type": "str"},
            "type": {"v_range": [["7.0.1", ""]], "choices": ["tcp-forwarding", "ssh"], "type": "str"},
            "weight": {"v_range": [["7.0.1", ""]], "type": "int"},
            "translate-host": {"v_range": [["7.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "external-auth": {"v_range": [["7.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "tunnel-encryption": {"v_range": [["7.4.0", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "firewall_accessproxy_apigateway6_sslciphersuites": {
        "stated": true,
        "options": {
            "cipher": {
                "v_range": [["7.0.1", ""]],
                "choices": [
                    "TLS-RSA-WITH-RC4-128-MD5", "TLS-RSA-WITH-RC4-128-SHA", "TLS-RSA-WITH-DES-CBC-SHA", "TLS-RSA-WITH-3DES-EDE-CBC-SHA",
                    "TLS-RSA-WITH-AES-128-CBC-SHA", "TLS-RSA-WITH-AES-256-CBC-SHA", "TLS-RSA-WITH-AES-128-CBC-SHA256", "TLS-RSA-WITH-AES-256-CBC-SHA256",
                    "TLS-RSA-WITH-CAMELLIA-128-CBC-SHA", "TLS-RSA-WITH-CAMELLIA-256-CBC-SHA", "TLS-RSA-WITH-CAMELLIA-128-CBC-SHA256",
                    "TLS-RSA-WITH-CAMELLIA-256-CBC-SHA256", "TLS-RSA-WITH-SEED-CBC-SHA", "TLS-RSA-WITH-ARIA-128-CBC-SHA256",
                    "TLS-RSA-WITH-ARIA-256-CBC-SHA384", "TLS-DHE-RSA-WITH-DES-CBC-SHA", "TLS-DHE-RSA-WITH-3DES-EDE-CBC-SHA",
                    "TLS-DHE-RSA-WITH-AES-128-CBC-SHA", "TLS-DHE-RSA-WITH-AES-256-CBC-SHA", "TLS-DHE-RSA-WITH-AES-128-CBC-SHA256",
                    "TLS-DHE-RSA-WITH-AES-256-CBC-SHA256", "TLS-DHE-RSA-WITH-CAMELLIA-128-CBC-SHA", "TLS-DHE-RSA-WITH-CAMELLIA-256-CBC-SHA",
                    "TLS-DHE-RSA-WITH-CAMELLIA-128-CBC-SHA256", "TLS-DHE-RSA-WITH-CAMELLIA-256-CBC-SHA256", "TLS-DHE-RSA-WITH-SEED-CBC-SHA",
                    "TLS-DHE-RSA-WITH-ARIA-128-CBC-SHA256", "TLS-DHE-RSA-WITH-ARIA-256-CBC-SHA384", "TLS-ECDHE-RSA-WITH-RC4-128-SHA",
                    "TLS-ECDHE-RSA-WITH-3DES-EDE-CBC-SHA", "TLS-ECDHE-RSA-WITH-AES-128-CBC-SHA", "TLS-ECDHE-RSA-WITH-AES-256-CBC-SHA",
                    "TLS-ECDHE-RSA-WITH-CHACHA20-POLY1305-SHA256", "TLS-ECDHE-ECDSA-WITH-CHACHA20-POLY1305-SHA256",
                    "TLS-DHE-RSA-WITH-CHACHA20-POLY1305-SHA256", "TLS-DHE-RSA-WITH-AES-128-GCM-SHA256", "TLS-DHE-RSA-WITH-AES-256-GCM-SHA384",
                    "TLS-DHE-DSS-WITH-AES-128-CBC-SHA", "TLS-DHE-DSS-WITH-AES-256-CBC-SHA", "TLS-DHE-DSS-WITH-AES-128-CBC-SHA256",
                    "TLS-DHE-DSS-WITH-AES-128-GCM-SHA256", "TLS-DHE-DSS-WITH-AES-256-CBC-SHA256", "TLS-DHE-DSS-WITH-AES-256-GCM-SHA384",
                    "TLS-ECDHE-RSA-WITH-AES-128-CBC-SHA256", "TLS-ECDHE-RSA-WITH-AES-128-GCM-SHA256", "TLS-ECDHE-RSA-WITH-AES-256-CBC-SHA384",
                    "TLS-ECDHE-RSA-WITH-AES-256-GCM-SHA384", "TLS-ECDHE-ECDSA-WITH-AES-128-CBC-SHA", "TLS-ECDHE-ECDSA-WITH-AES-128-CBC-SHA256",
                    "TLS-ECDHE-ECDSA-WITH-AES-128-GCM-SHA256", "TLS-ECDHE-ECDSA-WITH-AES-256-CBC-SHA384", "TLS-ECDHE-ECDSA-WITH-AES-256-GCM-SHA384",
                    "TLS-RSA-WITH-AES-128-GCM-SHA256", "TLS-RSA-WITH-AES-256-GCM-SHA384", "TLS-DHE-DSS-WITH-CAMELLIA-128-CBC-SHA",
                    "TLS-DHE-DSS-WITH-CAMELLIA-256-CBC-SHA", "TLS-DHE-DSS-WITH-CAMELLIA-128-CBC-SHA256", "TLS-DHE-DSS-WITH-CAMELLIA-256-CBC-SHA256",
                    "TLS-DHE-DSS-WITH-SEED-CBC-SHA", "TLS-DHE-DSS-WITH-ARIA-128-CBC-SHA256", "TLS-DHE-DSS-WITH-ARIA-256-CBC-SHA384",
                    "TLS-ECDHE-RSA-WITH-ARIA-128-CBC-SHA256", "TLS-ECDHE-RSA-WITH-ARIA-256-CBC-SHA384", "TLS-ECDHE-ECDSA-WITH-ARIA-128-CBC-SHA256",
                    "TLS-ECDHE-ECDSA-WITH-ARIA-256-CBC-SHA384", "TLS-DHE-DSS-WITH-3DES-EDE-CBC-SHA", "TLS-DHE-DSS-WITH-DES-CBC-SHA",
                    "TLS-AES-128-GCM-SHA256", "TLS-AES-256-GCM-SHA384", "TLS-CHACHA20-POLY1305-SHA256", "TLS-ECDHE-ECDSA-WITH-AES-256-CBC-SHA"
                ],
                "type": "str"
            },
            "priority": {"v_range": [["7.0.1", ""]], "type": "int"},
            "versions": {"v_range": [["7.0.1", ""]], "type": "list", "choices": ["tls-1.0", "tls-1.1", "tls-1.2", "tls-1.3"], "elements": "str"}
        }
    },
    "firewall_accessproxy_apigateway_quic": {
        "stated": false,
        "options": {
            "ack-delay-exponent": {"v_range": [["7.4.1", ""]], "type": "int"},
            "active-connection-id-limit": {"v_range": [["7.4.1", ""]], "type": "int"},
            "active-migration": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "grease-quic-bit": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "max-ack-delay": {"v_range": [["7.4.1", ""]], "type": "int"},
            "max-datagram-frame-size": {"v_range": [["7.4.1", ""]], "type": "int"},
            "max-idle-timeout": {"v_range": [["7.4.1", ""]], "type": "int"},
            "max-udp-payload-size": {"v_range": [["7.4.1", ""]], "type": "int"}
        }
    },
    "firewall_accessproxy_apigateway_realservers": {
        "stated": true,
        "options": {
            "address": {"v_range": [["7.0.0", ""]], "type": "str"},
            "health-check": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "health-check-proto": {"v_range": [["7.0.0", ""]], "choices": ["ping", "http", "tcp-connect"], "type": "str"},
            "http-host": {"v_range": [["7.0.0", ""]], "type": "str"},
            "id": {"v_range": [["7.0.0", ""]], "required": true, "type": "int"},
            "ip": {"v_range": [["7.0.0", ""]], "type": "str"},
            "mappedport": {"v_range": [["7.0.0", ""]], "type": "raw"},
            "port": {"v_range": [["7.0.0", ""]], "type": "int"},
            "status": {"v_range": [["7.0.0", ""]], "choices": ["active", "standby", "disable"], "type": "str"},
            "weight": {"v_range": [["7.0.0", ""]], "type": "int"},
            "addr-type": {"v_range": [["7.0.2", ""]], "choices": ["fqdn", "ip"], "type": "str"},
            "domain": {"v_range": [["7.0.3", ""]], "type": "str"},
            "holddown-interval": {"v_range": [["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ssh-client-cert": {"v_range": [["7.0.1", ""]], "type": "str"},
            "ssh-host-key": {"v_range": [["7.0.1", ""]], "no_log": true, "type": "raw"},
            "ssh-host-key-validation": {"v_range": [["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "type": {"v_range": [["7.0.1", ""]], "choices": ["tcp-forwarding", "ssh"], "type": "str"},
            "translate-host": {"v_range": [["7.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "external-auth": {"v_range": [["7.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "tunnel-encryption": {"v_range": [["7.4.0", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "firewall_accessproxy_apigateway_sslciphersuites": {
        "stated": true,
        "options": {
            "cipher": {
                "v_range": [["7.0.0", ""]],
                "choices": [
                    "TLS-RSA-WITH-RC4-128-MD5", "TLS-RSA-WITH-RC4-128-SHA", "TLS-RSA-WITH-DES-CBC-SHA", "TLS-RSA-WITH-3DES-EDE-CBC-SHA",
                    "TLS-RSA-WITH-AES-128-CBC-SHA", "TLS-RSA-WITH-AES-256-CBC-SHA", "TLS-RSA-WITH-AES-128-CBC-SHA256", "TLS-RSA-WITH-AES-256-CBC-SHA256",
                    "TLS-RSA-WITH-CAMELLIA-128-CBC-SHA", "TLS-RSA-WITH-CAMELLIA-256-CBC-SHA", "TLS-RSA-WITH-CAMELLIA-128-CBC-SHA256",
                    "TLS-RSA-WITH-CAMELLIA-256-CBC-SHA256", "TLS-RSA-WITH-SEED-CBC-SHA", "TLS-RSA-WITH-ARIA-128-CBC-SHA256",
                    "TLS-RSA-WITH-ARIA-256-CBC-SHA384", "TLS-DHE-RSA-WITH-DES-CBC-SHA", "TLS-DHE-RSA-WITH-3DES-EDE-CBC-SHA",
                    "TLS-DHE-RSA-WITH-AES-128-CBC-SHA", "TLS-DHE-RSA-WITH-AES-256-CBC-SHA", "TLS-DHE-RSA-WITH-AES-128-CBC-SHA256",
                    "TLS-DHE-RSA-WITH-AES-256-CBC-SHA256", "TLS-DHE-RSA-WITH-CAMELLIA-128-CBC-SHA", "TLS-DHE-RSA-WITH-CAMELLIA-256-CBC-SHA",
                    "TLS-DHE-RSA-WITH-CAMELLIA-128-CBC-SHA256", "TLS-DHE-RSA-WITH-CAMELLIA-256-CBC-SHA256", "TLS-DHE-RSA-WITH-SEED-CBC-SHA",
                    "TLS-DHE-RSA-WITH-ARIA-128-CBC-SHA256", "TLS-DHE-RSA-WITH-ARIA-256-CBC-SHA384", "TLS-ECDHE-RSA-WITH-RC4-128-SHA",
                    "TLS-ECDHE-RSA-WITH-3DES-EDE-CBC-SHA", "TLS-ECDHE-RSA-WITH-AES-128-CBC-SHA", "TLS-ECDHE-RSA-WITH-AES-256-CBC-SHA",
                    "TLS-ECDHE-RSA-WITH-CHACHA20-POLY1305-SHA256", "TLS-ECDHE-ECDSA-WITH-CHACHA20-POLY1305-SHA256",
                    "TLS-DHE-RSA-WITH-CHACHA20-POLY1305-SHA256", "TLS-DHE-RSA-WITH-AES-128-GCM-SHA256", "TLS-DHE-RSA-WITH-AES-256-GCM-SHA384",
                    "TLS-DHE-DSS-WITH-AES-128-CBC-SHA", "TLS-DHE-DSS-WITH-AES-256-CBC-SHA", "TLS-DHE-DSS-WITH-AES-128-CBC-SHA256",
                    "TLS-DHE-DSS-WITH-AES-128-GCM-SHA256", "TLS-DHE-DSS-WITH-AES-256-CBC-SHA256", "TLS-DHE-DSS-WITH-AES-256-GCM-SHA384",
                    "TLS-ECDHE-RSA-WITH-AES-128-CBC-SHA256", "TLS-ECDHE-RSA-WITH-AES-128-GCM-SHA256", "TLS-ECDHE-RSA-WITH-AES-256-CBC-SHA384",
                    "TLS-ECDHE-RSA-WITH-AES-256-GCM-SHA384", "TLS-ECDHE-ECDSA-WITH-AES-128-CBC-SHA", "TLS-ECDHE-ECDSA-WITH-AES-128-CBC-SHA256",
                    "TLS-ECDHE-ECDSA-WITH-AES-128-GCM-SHA256", "TLS-ECDHE-ECDSA-WITH-AES-256-CBC-SHA384", "TLS-ECDHE-ECDSA-WITH-AES-256-GCM-SHA384",
                    "TLS-RSA-WITH-AES-128-GCM-SHA256", "TLS-RSA-WITH-AES-256-GCM-SHA384", "TLS-DHE-DSS-WITH-CAMELLIA-128-CBC-SHA",
                    "TLS-DHE-DSS-WITH-CAMELLIA-256-CBC-SHA", "TLS-DHE-DSS-WITH-CAMELLIA-128-CBC-SHA256", "TLS-DHE-DSS-WITH-CAMELLIA-256-CBC-SHA256",
                    "TLS-DHE-DSS-WITH-SEED-CBC-SHA", "TLS-DHE-DSS-WITH-ARIA-128-CBC-SHA256", "TLS-DHE-DSS-WITH-ARIA-256-CBC-SHA384",
                    "TLS-ECDHE-RSA-WITH-ARIA-128-CBC-SHA256", "TLS-ECDHE-RSA-WITH-ARIA-256-CBC-SHA384", "TLS-ECDHE-ECDSA-WITH-ARIA-128-CBC-SHA256",
                    "TLS-ECDHE-ECDSA-WITH-ARIA-256-CBC-SHA384", "TLS-DHE-DSS-WITH-3DES-EDE-CBC-SHA", "TLS-DHE-DSS-WITH-DES-CBC-SHA",
                    "TLS-AES-128-GCM-SHA256", "TLS-AES-256-GCM-SHA384", "TLS-CHACHA20-POLY1305-SHA256", "TLS-ECDHE-ECDSA-WITH-AES-256-CBC-SHA"
                ],
                "type": "str"
            },
            "priority": {"v_range": [["7.0.0", ""]], "type": "int"},
            "versions": {"v_range": [["7.0.0", ""]], "type": "list", "choices": ["tls-1.0", "tls-1.1", "tls-1.2", "tls-1.3"], "elements": "str"}
        }
    },
    "firewall_accessproxy_realservers": {
        "stated": true,
        "options": {
            "id": {"v_range": [["7.0.0", ""]], "required": true, "type": "int"},
            "ip": {"v_range": [["7.0.0", ""]], "type": "str"},
            "port": {"v_range": [["7.0.0", ""]], "type": "int"},
            "status": {"v_range": [["7.0.0", ""]], "choices": ["active", "standby", "disable"], "type": "str"},
            "weight": {"v_range": [["7.0.0", ""]], "type": "int"}
        }
    },
    "firewall_accessproxy_serverpubkeyauthsettings": {
        "stated": false,
        "options": {
            "auth-ca": {"v_range": [["7.0.0", ""]], "type": "str"},
            "cert-extension": {
                "v_range": [["7.0.0", ""]],
                "type": "list",
                "options": {
                    "critical": {"v_range": [["7.0.0", ""]], "choices": ["no", "yes"], "type": "str"},
                    "data": {"v_range": [["7.0.0", ""]], "type": "str"},
                    "name": {"v_range": [["7.0.0", ""]], "type": "str"},
                    "type": {"v_range": [["7.0.0", ""]], "choices": ["fixed", "user"], "type": "str"}
                },
                "elements": "dict"
            },
            "permit-agent-forwarding": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "permit-port-forwarding": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "permit-pty": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "permit-user-rc": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "permit-x11-forwarding": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "source-address": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "firewall_accessproxy_serverpubkeyauthsettings_certextension": {
        "stated": true,
        "options": {
            "critical": {"v_range": [["7.0.0", ""]], "choices": ["no", "yes"], "type": "str"},
            "data": {"v_range": [["7.0.0", ""]], "type": "str"},
            "name": {"v_range": [["7.0.0", ""]], "required": true, "type": "str"},
            "type": {"v_range": [["7.0.0", ""]], "choices": ["fixed", "user"], "type": "str"}
        }
    },
    "firewall_accessproxysshclientcert": {
        "stated": true,
        "options": {
            "auth-ca": {"v_range": [["7.4.2", ""]], "type": "str"},
            "cert-extension": {
                "v_range": [["7.4.2", ""]],
                "type": "list",
                "options": {
                    "critical": {"v_range": [["7.4.2", ""]], "choices": ["no", "yes"], "type": "str"},
                    "data": {"v_range": [["7.4.2", ""]], "type": "str"},
                    "name": {"v_range": [["7.4.2", ""]], "type": "str"},
                    "type": {"v_range": [["7.4.2", ""]], "choices": ["fixed", "user"], "type": "str"}
                },
                "elements": "dict"
            },
            "name": {"v_range": [["7.4.2", ""]], "required": true, "type": "str"},
            "permit-agent-forwarding": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "permit-port-forwarding": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "permit-pty": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "permit-user-rc": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "permit-x11-forwarding": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "source-address": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "firewall_accessproxysshclientcert_certextension": {
        "stated": true,
        "options": {
            "critical": {"v_range": [["7.4.2", ""]], "choices": ["no", "yes"], "type": "str"},
            "data": {"v_range": [["7.4.2", ""]], "type": "str"},
            "name": {"v_range": [["7.4.2", ""]], "required": true, "type": "str"},
            "type": {"v_range": [["7.4.2", ""]], "choices": ["fixed", "user"], "type": "str"}
        }
    },
    "firewall_accessproxyvirtualhost": {
        "stated": true,
        "options": {
            "host": {"v_range": [["7.0.1", ""]], "type": "str"},
            "host-type": {"v_range": [["7.0.1", ""]], "choices": ["sub-string", "wildcard"], "type": "str"},
            "name": {"v_range": [["7.0.1", ""]], "required": true, "type": "str"},
            "ssl-certificate": {"v_range": [["7.0.1", ""]], "type": "str"},
            "replacemsg-group": {"v_range": [["7.0.5", "7.0.12"], ["7.2.1", ""]], "type": "str"}
        }
    },
    "firewall_address": {
        "stated": true,
        "options": {
            "allow-routing": {"choices": ["disable", "enable"], "type": "str"},
            "associated-interface": {"type": "str"},
            "cache-ttl": {"type": "int"},
            "color": {"type": "int"},
            "comment": {"type": "raw"},
            "country": {"type": "str"},
            "dynamic_mapping": {
                "type": "list",
                "options": {
                    "_scope": {"type": "list", "options": {"name": {"type": "str"}, "vdom": {"type": "str"}}, "elements": "dict"},
                    "allow-routing": {"choices": ["disable", "enable"], "type": "str"},
                    "associated-interface": {"type": "str"},
                    "cache-ttl": {"type": "int"},
                    "color": {"type": "int"},
                    "comment": {"type": "raw"},
                    "country": {"type": "str"},
                    "end-ip": {"type": "str"},
                    "end-mac": {"type": "str"},
                    "epg-name": {"type": "str"},
                    "filter": {"type": "str"},
                    "fqdn": {"type": "str"},
                    "interface": {"type": "str"},
                    "obj-id": {"type": "str"},
                    "organization": {"type": "str"},
                    "policy-group": {"type": "str"},
                    "sdn": {"choices": ["aci", "aws", "nsx", "nuage", "azure", "gcp", "oci", "openstack"], "type": "str"},
                    "sdn-addr-type": {"choices": ["private", "public", "all"], "type": "str"},
                    "sdn-tag": {"type": "str"},
                    "start-ip": {"type": "str"},
                    "start-mac": {"type": "str"},
                    "subnet": {"type": "str"},
                    "subnet-name": {"type": "str"},
                    "tags": {"type": "raw"},
                    "tenant": {"type": "str"},
                    "type": {
                        "choices": [
                            "ipmask", "iprange", "fqdn", "wildcard", "geography", "url", "wildcard-fqdn", "nsx", "aws", "dynamic", "interface-subnet",
                            "mac", "fqdn-group", "route-tag"
                        ],
                        "type": "str"
                    },
                    "url": {"type": "str"},
                    "uuid": {"type": "str"},
                    "visibility": {"choices": ["disable", "enable"], "type": "str"},
                    "wildcard": {"type": "str"},
                    "wildcard-fqdn": {"type": "str"},
                    "_image-base64": {"v_range": [["6.2.2", ""]], "type": "str"},
                    "clearpass-spt": {
                        "v_range": [["6.2.2", ""]],
                        "choices": ["unknown", "healthy", "quarantine", "checkup", "transition", "infected", "transient"],
                        "type": "str"
                    },
                    "fsso-group": {"v_range": [["6.2.2", ""]], "type": "raw"},
                    "sub-type": {
                        "v_range": [["6.2.2", ""]],
                        "choices": [
                            "sdn", "clearpass-spt", "fsso", "ems-tag", "swc-tag", "fortivoice-tag", "fortinac-tag", "fortipolicy-tag",
                            "device-identification"
                        ],
                        "type": "str"
                    },
                    "global-object": {"v_range": [["6.4.0", ""]], "type": "int"},
                    "obj-tag": {"v_range": [["6.4.2", ""]], "type": "str"},
                    "obj-type": {"v_range": [["6.4.2", ""]], "choices": ["ip", "mac"], "type": "str"},
                    "fabric-object": {"v_range": [["6.4.4", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "macaddr": {"v_range": [["7.0.0", ""]], "type": "raw"},
                    "node-ip-only": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "dirty": {"v_range": [["7.0.3", ""]], "choices": ["dirty", "clean"], "type": "str"},
                    "pattern-end": {"v_range": [["7.0.3", ""]], "type": "int"},
                    "pattern-start": {"v_range": [["7.0.3", ""]], "type": "int"},
                    "tag-detection-level": {"v_range": [["7.0.3", ""]], "type": "str"},
                    "tag-type": {"v_range": [["7.0.3", ""]], "type": "str"},
                    "hw-model": {"v_range": [["7.4.0", ""]], "type": "str"},
                    "hw-vendor": {"v_range": [["7.4.0", ""]], "type": "str"},
                    "os": {"v_range": [["7.4.0", ""]], "type": "str"},
                    "route-tag": {"v_range": [["7.4.0", ""]], "type": "int"},
                    "sw-version": {"v_range": [["7.4.0", ""]], "type": "str"}
                },
                "elements": "dict"
            },
            "end-ip": {"type": "str"},
            "epg-name": {"type": "str"},
            "filter": {"type": "str"},
            "fqdn": {"type": "str"},
            "list": {
                "type": "list",
                "options": {
                    "ip": {"type": "str"},
                    "net-id": {"v_range": [["6.2.0", ""]], "type": "str"},
                    "obj-id": {"v_range": [["6.2.0", ""]], "type": "str"}
                },
                "elements": "dict"
            },
            "name": {"required": true, "type": "str"},
            "obj-id": {"type": "str"},
            "organization": {"type": "str"},
            "policy-group": {"type": "str"},
            "sdn": {"choices": ["aci", "aws", "nsx", "nuage", "azure", "gcp", "oci", "openstack"], "type": "str"},
            "sdn-tag": {"type": "str"},
            "start-ip": {"type": "str"},
            "subnet": {"type": "str"},
            "subnet-name": {"type": "str"},
            "tagging": {"type": "list", "options": {"category": {"type": "str"}, "name": {"type": "str"}, "tags": {"type": "raw"}}, "elements": "dict"},
            "tenant": {"type": "str"},
            "type": {
                "choices": [
                    "ipmask", "iprange", "fqdn", "wildcard", "geography", "url", "wildcard-fqdn", "nsx", "aws", "dynamic", "interface-subnet", "mac",
                    "fqdn-group", "route-tag"
                ],
                "type": "str"
            },
            "uuid": {"type": "str"},
            "visibility": {"choices": ["disable", "enable"], "type": "str"},
            "wildcard": {"type": "str"},
            "wildcard-fqdn": {"type": "str"},
            "end-mac": {"v_range": [["6.2.0", ""]], "type": "str"},
            "interface": {"v_range": [["6.2.0", ""]], "type": "str"},
            "sdn-addr-type": {"v_range": [["6.2.0", ""]], "choices": ["private", "public", "all"], "type": "str"},
            "start-mac": {"v_range": [["6.2.0", ""]], "type": "str"},
            "tags": {"v_range": [["6.2.0", "6.4.14"]], "type": "str"},
            "profile-list": {
                "v_range": [["6.2.0", "6.2.12"]],
                "type": "list",
                "options": {"profile-id": {"v_range": [["6.2.0", "6.2.12"]], "type": "int"}},
                "elements": "dict"
            },
            "_image-base64": {"v_range": [["6.2.2", ""]], "type": "str"},
            "clearpass-spt": {
                "v_range": [["6.2.2", ""]],
                "choices": ["unknown", "healthy", "quarantine", "checkup", "transition", "infected", "transient"],
                "type": "str"
            },
            "fsso-group": {"v_range": [["6.2.2", ""]], "type": "raw"},
            "sub-type": {
                "v_range": [["6.2.2", ""]],
                "choices": [
                    "sdn", "clearpass-spt", "fsso", "ems-tag", "swc-tag", "fortivoice-tag", "fortinac-tag", "fortipolicy-tag", "device-identification"
                ],
                "type": "str"
            },
            "global-object": {"v_range": [["6.4.0", ""]], "type": "int"},
            "obj-tag": {"v_range": [["6.4.2", ""]], "type": "str"},
            "obj-type": {"v_range": [["6.4.2", ""]], "choices": ["ip", "mac"], "type": "str"},
            "fabric-object": {"v_range": [["6.4.4", ""]], "choices": ["disable", "enable"], "type": "str"},
            "macaddr": {"v_range": [["7.0.0", ""]], "type": "raw"},
            "node-ip-only": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "dirty": {"v_range": [["7.0.3", ""]], "choices": ["dirty", "clean"], "type": "str"},
            "tag-detection-level": {"v_range": [["7.0.3", ""]], "type": "str"},
            "tag-type": {"v_range": [["7.0.3", ""]], "type": "str"},
            "hw-model": {"v_range": [["7.4.0", ""]], "type": "str"},
            "hw-vendor": {"v_range": [["7.4.0", ""]], "type": "str"},
            "os": {"v_range": [["7.4.0", ""]], "type": "str"},
            "route-tag": {"v_range": [["7.4.0", ""]], "type": "int"},
            "sw-version": {"v_range": [["7.4.0", ""]], "type": "str"}
        }
    },
    "firewall_address6": {
        "stated": true,
        "options": {
            "cache-ttl": {"type": "int"},
            "color": {"type": "int"},
            "comment": {"type": "str"},
            "dynamic_mapping": {
                "type": "list",
                "options": {
                    "_scope": {"type": "list", "options": {"name": {"type": "str"}, "vdom": {"type": "str"}}, "elements": "dict"},
                    "cache-ttl": {"type": "int"},
                    "color": {"type": "int"},
                    "comment": {"type": "str"},
                    "end-ip": {"type": "str"},
                    "fqdn": {"type": "str"},
                    "host": {"type": "str"},
                    "host-type": {"choices": ["any", "specific"], "type": "str"},
                    "ip6": {"type": "str"},
                    "obj-id": {"type": "str"},
                    "sdn": {"choices": ["nsx"], "type": "str"},
                    "start-ip": {"type": "str"},
                    "tags": {"type": "raw"},
                    "template": {"type": "str"},
                    "type": {"choices": ["ipprefix", "iprange", "nsx", "dynamic", "fqdn", "template", "mac", "geography", "route-tag"], "type": "str"},
                    "uuid": {"type": "str"},
                    "visibility": {"choices": ["disable", "enable"], "type": "str"},
                    "subnet-segment": {
                        "v_range": [["6.2.1", ""]],
                        "type": "list",
                        "options": {
                            "name": {"v_range": [["6.2.1", ""]], "type": "str"},
                            "type": {"v_range": [["6.2.1", ""]], "choices": ["any", "specific"], "type": "str"},
                            "value": {"v_range": [["6.2.1", ""]], "type": "str"}
                        },
                        "elements": "dict"
                    },
                    "_image-base64": {"v_range": [["6.2.2", ""]], "type": "str"},
                    "end-mac": {"v_range": [["6.2.5", "6.2.12"], ["6.4.1", ""]], "type": "str"},
                    "start-mac": {"v_range": [["6.2.5", "6.2.12"], ["6.4.1", ""]], "type": "str"},
                    "country": {"v_range": [["6.4.0", ""]], "type": "str"},
                    "global-object": {"v_range": [["6.4.0", ""]], "type": "int"},
                    "fabric-object": {"v_range": [["6.4.4", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "macaddr": {"v_range": [["7.0.0", ""]], "type": "raw"},
                    "epg-name": {"v_range": [["7.2.1", ""]], "type": "str"},
                    "sdn-tag": {"v_range": [["7.2.1", ""]], "type": "str"},
                    "tenant": {"v_range": [["7.2.1", ""]], "type": "str"},
                    "route-tag": {"v_range": [["7.4.0", ""]], "type": "int"}
                },
                "elements": "dict"
            },
            "end-ip": {"type": "str"},
            "fqdn": {"type": "str"},
            "host": {"type": "str"},
            "host-type": {"choices": ["any", "specific"], "type": "str"},
            "ip6": {"type": "str"},
            "list": {
                "type": "list",
                "options": {
                    "ip": {"type": "str"},
                    "net-id": {"v_range": [["6.2.1", ""]], "type": "str"},
                    "obj-id": {"v_range": [["6.2.1", ""]], "type": "str"}
                },
                "elements": "dict"
            },
            "name": {"required": true, "type": "str"},
            "obj-id": {"type": "str"},
            "sdn": {"choices": ["nsx"], "type": "str"},
            "start-ip": {"type": "str"},
            "subnet-segment": {
                "type": "list",
                "options": {"name": {"type": "str"}, "type": {"choices": ["any", "specific"], "type": "str"}, "value": {"type": "str"}},
                "elements": "dict"
            },
            "tagging": {"type": "list", "options": {"category": {"type": "str"}, "name": {"type": "str"}, "tags": {"type": "raw"}}, "elements": "dict"},
            "template": {"type": "str"},
            "type": {"choices": ["ipprefix", "iprange", "nsx", "dynamic", "fqdn", "template", "mac", "geography", "route-tag"], "type": "str"},
            "uuid": {"type": "str"},
            "visibility": {"choices": ["disable", "enable"], "type": "str"},
            "tags": {"v_range": [["6.2.0", "6.4.14"]], "type": "str"},
            "profile-list": {
                "v_range": [["6.2.0", "6.2.12"]],
                "type": "list",
                "options": {"profile-id": {"v_range": [["6.2.0", "6.2.12"]], "type": "int"}},
                "elements": "dict"
            },
            "_image-base64": {"v_range": [["6.2.2", ""]], "type": "str"},
            "end-mac": {"v_range": [["6.2.5", "6.2.12"], ["6.4.1", ""]], "type": "str"},
            "start-mac": {"v_range": [["6.2.5", "6.2.12"], ["6.4.1", ""]], "type": "str"},
            "country": {"v_range": [["6.4.0", ""]], "type": "str"},
            "global-object": {"v_range": [["6.4.0", ""]], "type": "int"},
            "fabric-object": {"v_range": [["6.4.4", ""]], "choices": ["disable", "enable"], "type": "str"},
            "macaddr": {"v_range": [["7.0.0", ""]], "type": "raw"},
            "epg-name": {"v_range": [["7.2.1", ""]], "type": "str"},
            "sdn-tag": {"v_range": [["7.2.1", ""]], "type": "str"},
            "tenant": {"v_range": [["7.2.1", ""]], "type": "str"},
            "route-tag": {"v_range": [["7.4.0", ""]], "type": "int"}
        }
    },
    "firewall_address6_dynamicmapping": {
        "stated": true,
        "options": {
            "_scope": {"type": "list", "options": {"name": {"type": "str"}, "vdom": {"type": "str"}}, "elements": "dict"},
            "cache-ttl": {"type": "int"},
            "color": {"type": "int"},
            "comment": {"type": "str"},
            "end-ip": {"type": "str"},
            "fqdn": {"type": "str"},
            "host": {"type": "str"},
            "host-type": {"choices": ["any", "specific"], "type": "str"},
            "ip6": {"type": "str"},
            "obj-id": {"type": "str"},
            "sdn": {"choices": ["nsx"], "type": "str"},
            "start-ip": {"type": "str"},
            "tags": {"type": "raw"},
            "template": {"type": "str"},
            "type": {"choices": ["ipprefix", "iprange", "nsx", "dynamic", "fqdn", "template", "mac", "geography", "route-tag"], "type": "str"},
            "uuid": {"type": "str"},
            "visibility": {"choices": ["disable", "enable"], "type": "str"},
            "subnet-segment": {
                "v_range": [["6.2.1", ""]],
                "type": "list",
                "options": {
                    "name": {"v_range": [["6.2.1", ""]], "type": "str"},
                    "type": {"v_range": [["6.2.1", ""]], "choices": ["any", "specific"], "type": "str"},
                    "value": {"v_range": [["6.2.1", ""]], "type": "str"}
                },
                "elements": "dict"
            },
            "_image-base64": {"v_range": [["6.2.2", ""]], "type": "str"},
            "end-mac": {"v_range": [["6.2.5", "6.2.12"], ["6.4.1", ""]], "type": "str"},
            "start-mac": {"v_range": [["6.2.5", "6.2.12"], ["6.4.1", ""]], "type": "str"},
            "country": {"v_range": [["6.4.0", ""]], "type": "str"},
            "global-object": {"v_range": [["6.4.0", ""]], "type": "int"},
            "fabric-object": {"v_range": [["6.4.4", ""]], "choices": ["disable", "enable"], "type": "str"},
            "macaddr": {"v_range": [["7.0.0", ""]], "type": "raw"},
            "epg-name": {"v_range": [["7.2.1", ""]], "type": "str"},
            "sdn-tag": {"v_range": [["7.2.1", ""]], "type": "str"},
            "tenant": {"v_range": [["7.2.1", ""]], "type": "str"},
            "route-tag": {"v_range": [["7.4.0", ""]], "type": "int"}
        }
    },
    "firewall_address6_dynamicmapping_subnetsegment": {
        "stated": true,
        "options": {
            "name": {"v_range": [["6.2.1", "7.4.0"]], "required": true, "type": "str"},
            "type": {"v_range": [["6.2.1", "7.4.0"]], "choices": ["any", "specific"], "type": "str"},
            "value": {"v_range": [["6.2.1", "7.4.0"]], "type": "str"}
        }
    },
    "firewall_address6_list": {
        "stated": true,
        "options": {
            "ip": {"required": true, "type": "str"},
            "net-id": {"v_range": [["6.2.1", ""]], "type": "str"},
            "obj-id": {"v_range": [["6.2.1", ""]], "type": "str"}
        }
    },
    "firewall_address6_profilelist": {"stated": true, "options": {"profile-id": {"v_range": [["6.2.0", "6.2.12"]], "type": "int"}}},
    "firewall_address6_subnetsegment": {
        "stated": true,
        "options": {"name": {"required": true, "type": "str"}, "type": {"choices": ["any", "specific"], "type": "str"}, "value": {"type": "str"}}
    },
    "firewall_address6_tagging": {
        "stated": true,
        "options": {"category": {"type": "str"}, "name": {"required": true, "type": "str"}, "tags": {"type": "raw"}}
    },
    "firewall_address6template": {
        "stated": true,
        "options": {
            "ip6": {"type": "str"},
            "name": {"required": true, "type": "str"},
            "subnet-segment": {
                "type": "list",
                "options": {
                    "bits": {"type": "int"},
                    "exclusive": {"choices": ["disable", "enable"], "type": "str"},
                    "id": {"type": "int"},
                    "name": {"type": "str"},
                    "values": {"type": "list", "options": {"name": {"type": "str"}, "value": {"type": "str"}}, "elements": "dict"}
                },
                "elements": "dict"
            },
            "subnet-segment-count": {"type": "int"},
            "_image-base64": {"v_range": [["6.2.2", ""]], "type": "str"},
            "fabric-object": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "uuid": {"v_range": [["7.6.0", ""]], "type": "str"}
        }
    },
    "firewall_address6template_subnetsegment": {
        "stated": true,
        "options": {
            "bits": {"type": "int"},
            "exclusive": {"choices": ["disable", "enable"], "type": "str"},
            "id": {"required": true, "type": "int"},
            "name": {"type": "str"},
            "values": {"type": "list", "options": {"name": {"type": "str"}, "value": {"type": "str"}}, "elements": "dict"}
        }
    },
    "firewall_address6template_subnetsegment_values": {"stated": true, "options": {"name": {"required": true, "type": "str"}, "value": {"type": "str"}}},
    "firewall_address_dynamicmapping": {
        "stated": true,
        "options": {
            "_scope": {"type": "list", "options": {"name": {"type": "str"}, "vdom": {"type": "str"}}, "elements": "dict"},
            "allow-routing": {"choices": ["disable", "enable"], "type": "str"},
            "associated-interface": {"type": "str"},
            "cache-ttl": {"type": "int"},
            "color": {"type": "int"},
            "comment": {"type": "raw"},
            "country": {"type": "str"},
            "end-ip": {"type": "str"},
            "end-mac": {"type": "str"},
            "epg-name": {"type": "str"},
            "filter": {"type": "str"},
            "fqdn": {"type": "str"},
            "interface": {"type": "str"},
            "obj-id": {"type": "str"},
            "organization": {"type": "str"},
            "policy-group": {"type": "str"},
            "sdn": {"choices": ["aci", "aws", "nsx", "nuage", "azure", "gcp", "oci", "openstack"], "type": "str"},
            "sdn-addr-type": {"choices": ["private", "public", "all"], "type": "str"},
            "sdn-tag": {"type": "str"},
            "start-ip": {"type": "str"},
            "start-mac": {"type": "str"},
            "subnet": {"type": "str"},
            "subnet-name": {"type": "str"},
            "tags": {"type": "raw"},
            "tenant": {"type": "str"},
            "type": {
                "choices": [
                    "ipmask", "iprange", "fqdn", "wildcard", "geography", "url", "wildcard-fqdn", "nsx", "aws", "dynamic", "interface-subnet", "mac",
                    "fqdn-group", "route-tag"
                ],
                "type": "str"
            },
            "url": {"type": "str"},
            "uuid": {"type": "str"},
            "visibility": {"choices": ["disable", "enable"], "type": "str"},
            "wildcard": {"type": "str"},
            "wildcard-fqdn": {"type": "str"},
            "_image-base64": {"v_range": [["6.2.2", ""]], "type": "str"},
            "clearpass-spt": {
                "v_range": [["6.2.2", ""]],
                "choices": ["unknown", "healthy", "quarantine", "checkup", "transition", "infected", "transient"],
                "type": "str"
            },
            "fsso-group": {"v_range": [["6.2.2", ""]], "type": "raw"},
            "sub-type": {
                "v_range": [["6.2.2", ""]],
                "choices": [
                    "sdn", "clearpass-spt", "fsso", "ems-tag", "swc-tag", "fortivoice-tag", "fortinac-tag", "fortipolicy-tag", "device-identification"
                ],
                "type": "str"
            },
            "global-object": {"v_range": [["6.4.0", ""]], "type": "int"},
            "obj-tag": {"v_range": [["6.4.2", ""]], "type": "str"},
            "obj-type": {"v_range": [["6.4.2", ""]], "choices": ["ip", "mac"], "type": "str"},
            "fabric-object": {"v_range": [["6.4.4", ""]], "choices": ["disable", "enable"], "type": "str"},
            "macaddr": {"v_range": [["7.0.0", ""]], "type": "raw"},
            "node-ip-only": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "dirty": {"v_range": [["7.0.3", ""]], "choices": ["dirty", "clean"], "type": "str"},
            "pattern-end": {"v_range": [["7.0.3", ""]], "type": "int"},
            "pattern-start": {"v_range": [["7.0.3", ""]], "type": "int"},
            "tag-detection-level": {"v_range": [["7.0.3", ""]], "type": "str"},
            "tag-type": {"v_range": [["7.0.3", ""]], "type": "str"},
            "hw-model": {"v_range": [["7.4.0", ""]], "type": "str"},
            "hw-vendor": {"v_range": [["7.4.0", ""]], "type": "str"},
            "os": {"v_range": [["7.4.0", ""]], "type": "str"},
            "route-tag": {"v_range": [["7.4.0", ""]], "type": "int"},
            "sw-version": {"v_range": [["7.4.0", ""]], "type": "str"}
        }
    },
    "firewall_address_list": {
        "stated": true,
        "options": {
            "ip": {"required": true, "type": "str"},
            "net-id": {"v_range": [["6.2.0", ""]], "type": "str"},
            "obj-id": {"v_range": [["6.2.0", ""]], "type": "str"}
        }
    },
    "firewall_address_profilelist": {"stated": true, "options": {"profile-id": {"v_range": [["6.2.0", "6.2.12"]], "type": "int"}}},
    "firewall_address_tagging": {
        "stated": true,
        "options": {"category": {"type": "str"}, "name": {"required": true, "type": "str"}, "tags": {"type": "raw"}}
    },
    "firewall_addrgrp": {
        "stated": true,
        "options": {
            "allow-routing": {"choices": ["disable", "enable"], "type": "str"},
            "color": {"type": "int"},
            "comment": {"type": "raw"},
            "dynamic_mapping": {
                "type": "list",
                "options": {
                    "_scope": {"type": "list", "options": {"name": {"type": "str"}, "vdom": {"type": "str"}}, "elements": "dict"},
                    "allow-routing": {"choices": ["disable", "enable"], "type": "str"},
                    "color": {"type": "int"},
                    "comment": {"type": "raw"},
                    "exclude": {"choices": ["disable", "enable"], "type": "str"},
                    "exclude-member": {"type": "raw"},
                    "member": {"type": "list", "elements": "str"},
                    "tags": {"type": "raw"},
                    "uuid": {"type": "str"},
                    "visibility": {"choices": ["disable", "enable"], "type": "str"},
                    "_image-base64": {"v_range": [["6.2.2", ""]], "type": "str"},
                    "global-object": {"v_range": [["6.4.0", ""]], "type": "int"},
                    "type": {"v_range": [["6.4.0", ""]], "choices": ["default", "array", "folder"], "type": "str"},
                    "fabric-object": {"v_range": [["6.4.4", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "category": {"v_range": [["7.0.0", ""]], "choices": ["default", "ztna-ems-tag", "ztna-geo-tag"], "type": "str"}
                },
                "elements": "dict"
            },
            "member": {"type": "list", "elements": "str"},
            "name": {"required": true, "type": "str"},
            "tagging": {"type": "list", "options": {"category": {"type": "str"}, "name": {"type": "str"}, "tags": {"type": "raw"}}, "elements": "dict"},
            "uuid": {"type": "str"},
            "visibility": {"choices": ["disable", "enable"], "type": "str"},
            "exclude": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "exclude-member": {"v_range": [["6.2.0", ""]], "type": "raw"},
            "tags": {"v_range": [["6.2.0", "6.4.14"]], "type": "str"},
            "_image-base64": {"v_range": [["6.2.2", ""]], "type": "str"},
            "global-object": {"v_range": [["6.4.0", ""]], "type": "int"},
            "type": {"v_range": [["6.4.0", ""]], "choices": ["default", "array", "folder"], "type": "str"},
            "fabric-object": {"v_range": [["6.4.4", ""]], "choices": ["disable", "enable"], "type": "str"},
            "category": {"v_range": [["7.0.0", ""]], "choices": ["default", "ztna-ems-tag", "ztna-geo-tag"], "type": "str"}
        }
    },
    "firewall_addrgrp6": {
        "stated": true,
        "options": {
            "color": {"type": "int"},
            "comment": {"type": "str"},
            "dynamic_mapping": {
                "type": "list",
                "options": {
                    "_scope": {"type": "list", "options": {"name": {"type": "str"}, "vdom": {"type": "str"}}, "elements": "dict"},
                    "color": {"type": "int"},
                    "comment": {"type": "str"},
                    "member": {"type": "raw"},
                    "tags": {"type": "raw"},
                    "uuid": {"type": "str"},
                    "visibility": {"choices": ["disable", "enable"], "type": "str"},
                    "_image-base64": {"v_range": [["6.2.2", ""]], "type": "str"},
                    "global-object": {"v_range": [["6.4.0", ""]], "type": "int"},
                    "fabric-object": {"v_range": [["6.4.4", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "exclude": {"v_range": [["7.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "exclude-member": {"v_range": [["7.4.0", ""]], "type": "raw"}
                },
                "elements": "dict"
            },
            "member": {"type": "raw"},
            "name": {"required": true, "type": "str"},
            "tagging": {"type": "list", "options": {"category": {"type": "str"}, "name": {"type": "str"}, "tags": {"type": "raw"}}, "elements": "dict"},
            "uuid": {"type": "str"},
            "visibility": {"choices": ["disable", "enable"], "type": "str"},
            "tags": {"v_range": [["6.2.0", "6.4.14"]], "type": "str"},
            "_image-base64": {"v_range": [["6.2.2", ""]], "type": "str"},
            "global-object": {"v_range": [["6.4.0", ""]], "type": "int"},
            "fabric-object": {"v_range": [["6.4.4", ""]], "choices": ["disable", "enable"], "type": "str"},
            "exclude": {"v_range": [["7.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "exclude-member": {"v_range": [["7.4.0", ""]], "type": "raw"}
        }
    },
    "firewall_addrgrp6_dynamicmapping": {
        "stated": true,
        "options": {
            "_scope": {"type": "list", "options": {"name": {"type": "str"}, "vdom": {"type": "str"}}, "elements": "dict"},
            "color": {"type": "int"},
            "comment": {"type": "str"},
            "member": {"type": "raw"},
            "tags": {"type": "raw"},
            "uuid": {"type": "str"},
            "visibility": {"choices": ["disable", "enable"], "type": "str"},
            "_image-base64": {"v_range": [["6.2.2", ""]], "type": "str"},
            "global-object": {"v_range": [["6.4.0", ""]], "type": "int"},
            "fabric-object": {"v_range": [["6.4.4", ""]], "choices": ["disable", "enable"], "type": "str"},
            "exclude": {"v_range": [["7.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "exclude-member": {"v_range": [["7.4.0", ""]], "type": "raw"}
        }
    },
    "firewall_addrgrp6_tagging": {
        "stated": true,
        "options": {"category": {"type": "str"}, "name": {"required": true, "type": "str"}, "tags": {"type": "raw"}}
    },
    "firewall_addrgrp_dynamicmapping": {
        "stated": true,
        "options": {
            "_scope": {"type": "list", "options": {"name": {"type": "str"}, "vdom": {"type": "str"}}, "elements": "dict"},
            "allow-routing": {"choices": ["disable", "enable"], "type": "str"},
            "color": {"type": "int"},
            "comment": {"type": "raw"},
            "exclude": {"choices": ["disable", "enable"], "type": "str"},
            "exclude-member": {"type": "raw"},
            "member": {"type": "raw"},
            "tags": {"type": "raw"},
            "uuid": {"type": "str"},
            "visibility": {"choices": ["disable", "enable"], "type": "str"},
            "_image-base64": {"v_range": [["6.2.2", ""]], "type": "str"},
            "global-object": {"v_range": [["6.4.0", ""]], "type": "int"},
            "type": {"v_range": [["6.4.0", ""]], "choices": ["default", "array", "folder"], "type": "str"},
            "fabric-object": {"v_range": [["6.4.4", ""]], "choices": ["disable", "enable"], "type": "str"},
            "category": {"v_range": [["7.0.0", ""]], "choices": ["default", "ztna-ems-tag", "ztna-geo-tag"], "type": "str"}
        }
    },
    "firewall_addrgrp_tagging": {
        "stated": true,
        "options": {"category": {"type": "str"}, "name": {"required": true, "type": "str"}, "tags": {"type": "raw"}}
    },
    "firewall_carrierendpointbwl": {
        "stated": true,
        "options": {
            "comment": {"type": "str"},
            "entries": {
                "type": "list",
                "options": {
                    "action": {"type": "list", "choices": ["block", "exempt", "exempt-mass-mms"], "elements": "str"},
                    "carrier-endpoint": {"type": "str"},
                    "log-action": {"type": "list", "choices": ["archive", "intercept"], "elements": "str"},
                    "pattern-type": {"choices": ["wildcard", "regexp", "simple"], "type": "str"},
                    "status": {"choices": ["disable", "enable"], "type": "str"}
                },
                "elements": "dict"
            },
            "id": {"required": true, "type": "int"},
            "name": {"type": "str"}
        }
    },
    "firewall_carrierendpointbwl_entries": {
        "stated": true,
        "options": {
            "action": {"type": "list", "choices": ["block", "exempt", "exempt-mass-mms"], "elements": "str"},
            "carrier-endpoint": {"required": true, "type": "str"},
            "log-action": {"type": "list", "choices": ["archive", "intercept"], "elements": "str"},
            "pattern-type": {"choices": ["wildcard", "regexp", "simple"], "type": "str"},
            "status": {"choices": ["disable", "enable"], "type": "str"}
        }
    },
    "firewall_casbprofile": {
        "stated": true,
        "options": {
            "name": {"v_range": [["7.4.1", "7.4.1"]], "required": true, "type": "str"},
            "saas-application": {
                "v_range": [["7.4.1", "7.4.1"]],
                "type": "list",
                "options": {
                    "access-rule": {
                        "v_range": [["7.4.1", "7.4.1"]],
                        "type": "list",
                        "options": {
                            "action": {"v_range": [["7.4.1", "7.4.1"]], "choices": ["block", "monitor", "bypass"], "type": "str"},
                            "bypass": {
                                "v_range": [["7.4.1", "7.4.1"]],
                                "type": "list",
                                "choices": ["av", "dlp", "web-filter", "file-filter", "video-filter"],
                                "elements": "str"
                            },
                            "name": {"v_range": [["7.4.1", "7.4.1"]], "type": "str"}
                        },
                        "elements": "dict"
                    },
                    "custom-control": {
                        "v_range": [["7.4.1", "7.4.1"]],
                        "type": "list",
                        "options": {
                            "name": {"v_range": [["7.4.1", "7.4.1"]], "type": "str"},
                            "option": {
                                "v_range": [["7.4.1", "7.4.1"]],
                                "type": "list",
                                "options": {
                                    "name": {"v_range": [["7.4.1", "7.4.1"]], "type": "str"},
                                    "user-input": {"v_range": [["7.4.1", "7.4.1"]], "type": "list", "elements": "str"}
                                },
                                "elements": "dict"
                            }
                        },
                        "elements": "dict"
                    },
                    "domain-control": {"v_range": [["7.4.1", "7.4.1"]], "choices": ["disable", "enable"], "type": "str"},
                    "domain-control-domains": {"v_range": [["7.4.1", "7.4.1"]], "type": "list", "elements": "str"},
                    "log": {"v_range": [["7.4.1", "7.4.1"]], "choices": ["disable", "enable"], "type": "str"},
                    "name": {"v_range": [["7.4.1", "7.4.1"]], "type": "str"},
                    "safe-search": {"v_range": [["7.4.1", "7.4.1"]], "choices": ["disable", "enable"], "type": "str"},
                    "safe-search-control": {"v_range": [["7.4.1", "7.4.1"]], "type": "list", "elements": "str"},
                    "tenant-control": {"v_range": [["7.4.1", "7.4.1"]], "choices": ["disable", "enable"], "type": "str"},
                    "tenant-control-tenants": {"v_range": [["7.4.1", "7.4.1"]], "type": "list", "elements": "str"}
                },
                "elements": "dict"
            }
        }
    },
    "firewall_casbprofile_saasapplication": {
        "stated": true,
        "options": {
            "access-rule": {
                "v_range": [["7.4.1", "7.4.1"]],
                "type": "list",
                "options": {
                    "action": {"v_range": [["7.4.1", "7.4.1"]], "choices": ["block", "monitor", "bypass"], "type": "str"},
                    "bypass": {
                        "v_range": [["7.4.1", "7.4.1"]],
                        "type": "list",
                        "choices": ["av", "dlp", "web-filter", "file-filter", "video-filter"],
                        "elements": "str"
                    },
                    "name": {"v_range": [["7.4.1", "7.4.1"]], "type": "str"}
                },
                "elements": "dict"
            },
            "custom-control": {
                "v_range": [["7.4.1", "7.4.1"]],
                "type": "list",
                "options": {
                    "name": {"v_range": [["7.4.1", "7.4.1"]], "type": "str"},
                    "option": {
                        "v_range": [["7.4.1", "7.4.1"]],
                        "type": "list",
                        "options": {
                            "name": {"v_range": [["7.4.1", "7.4.1"]], "type": "str"},
                            "user-input": {"v_range": [["7.4.1", "7.4.1"]], "type": "list", "elements": "str"}
                        },
                        "elements": "dict"
                    }
                },
                "elements": "dict"
            },
            "domain-control": {"v_range": [["7.4.1", "7.4.1"]], "choices": ["disable", "enable"], "type": "str"},
            "domain-control-domains": {"v_range": [["7.4.1", "7.4.1"]], "type": "list", "elements": "str"},
            "log": {"v_range": [["7.4.1", "7.4.1"]], "choices": ["disable", "enable"], "type": "str"},
            "name": {"v_range": [["7.4.1", "7.4.1"]], "required": true, "type": "str"},
            "safe-search": {"v_range": [["7.4.1", "7.4.1"]], "choices": ["disable", "enable"], "type": "str"},
            "safe-search-control": {"v_range": [["7.4.1", "7.4.1"]], "type": "list", "elements": "str"},
            "tenant-control": {"v_range": [["7.4.1", "7.4.1"]], "choices": ["disable", "enable"], "type": "str"},
            "tenant-control-tenants": {"v_range": [["7.4.1", "7.4.1"]], "type": "list", "elements": "str"}
        }
    },
    "firewall_casbprofile_saasapplication_accessrule": {
        "stated": true,
        "options": {
            "action": {"v_range": [["7.4.1", "7.4.1"]], "choices": ["block", "monitor", "bypass"], "type": "str"},
            "bypass": {
                "v_range": [["7.4.1", "7.4.1"]],
                "type": "list",
                "choices": ["av", "dlp", "web-filter", "file-filter", "video-filter"],
                "elements": "str"
            },
            "name": {"v_range": [["7.4.1", "7.4.1"]], "required": true, "type": "str"}
        }
    },
    "firewall_casbprofile_saasapplication_customcontrol": {
        "stated": true,
        "options": {
            "name": {"v_range": [["7.4.1", "7.4.1"]], "required": true, "type": "str"},
            "option": {
                "v_range": [["7.4.1", "7.4.1"]],
                "type": "list",
                "options": {
                    "name": {"v_range": [["7.4.1", "7.4.1"]], "type": "str"},
                    "user-input": {"v_range": [["7.4.1", "7.4.1"]], "type": "list", "elements": "str"}
                },
                "elements": "dict"
            }
        }
    },
    "firewall_casbprofile_saasapplication_customcontrol_option": {
        "stated": true,
        "options": {
            "name": {"v_range": [["7.4.1", "7.4.1"]], "required": true, "type": "str"},
            "user-input": {"v_range": [["7.4.1", "7.4.1"]], "type": "list", "elements": "str"}
        }
    },
    "firewall_decryptedtrafficmirror": {
        "stated": true,
        "options": {
            "dstmac": {"v_range": [["6.4.1", ""]], "type": "str"},
            "interface": {"v_range": [["6.4.1", ""]], "type": "raw"},
            "name": {"v_range": [["6.4.1", ""]], "required": true, "type": "str"},
            "traffic-source": {"v_range": [["6.4.1", ""]], "choices": ["client", "server", "both"], "type": "str"},
            "traffic-type": {"v_range": [["6.4.1", ""]], "type": "list", "choices": ["ssl", "ssh"], "elements": "str"}
        }
    },
    "firewall_explicitproxyaddress": {
        "stated": true,
        "options": {
            "case-sensitivity": {"v_range": [["6.2.0", "6.2.12"]], "choices": ["disable", "enable"], "type": "str"},
            "category": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"},
            "color": {"v_range": [["6.2.0", "6.2.12"]], "type": "int"},
            "comment": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"},
            "header": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"},
            "header-group": {
                "v_range": [["6.2.0", "6.2.12"]],
                "type": "list",
                "options": {
                    "case-sensitivity": {"v_range": [["6.2.0", "6.2.12"]], "choices": ["disable", "enable"], "type": "str"},
                    "header": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"},
                    "header-name": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"},
                    "id": {"v_range": [["6.2.0", "6.2.12"]], "type": "int"}
                },
                "elements": "dict"
            },
            "header-name": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"},
            "host": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"},
            "host-regex": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"},
            "method": {
                "v_range": [["6.2.0", "6.2.12"]],
                "type": "list",
                "choices": ["delete", "get", "head", "options", "post", "put", "trace", "connect"],
                "elements": "str"
            },
            "name": {"v_range": [["6.2.0", "6.2.12"]], "required": true, "type": "str"},
            "path": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"},
            "tags": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"},
            "type": {
                "v_range": [["6.2.0", "6.2.12"]],
                "choices": ["host-regex", "url", "category", "method", "ua", "header", "src-advanced", "dst-advanced"],
                "type": "str"
            },
            "ua": {"v_range": [["6.2.0", "6.2.12"]], "type": "list", "choices": ["chrome", "ms", "firefox", "safari", "other"], "elements": "str"},
            "uuid": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"},
            "visibility": {"v_range": [["6.2.0", "6.2.12"]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "firewall_explicitproxyaddress_headergroup": {
        "stated": true,
        "options": {
            "case-sensitivity": {"v_range": [["6.2.0", "6.2.12"]], "choices": ["disable", "enable"], "type": "str"},
            "header": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"},
            "header-name": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"},
            "id": {"v_range": [["6.2.0", "6.2.12"]], "required": true, "type": "int"}
        }
    },
    "firewall_explicitproxyaddrgrp": {
        "stated": true,
        "options": {
            "color": {"v_range": [["6.2.0", "6.2.12"]], "type": "int"},
            "comment": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"},
            "member": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"},
            "name": {"v_range": [["6.2.0", "6.2.12"]], "required": true, "type": "str"},
            "tags": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"},
            "type": {"v_range": [["6.2.0", "6.2.12"]], "choices": ["src", "dst"], "type": "str"},
            "uuid": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"},
            "visibility": {"v_range": [["6.2.0", "6.2.12"]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "firewall_gtp": {
        "stated": true,
        "options": {
            "addr-notify": {"type": "str"},
            "apn": {
                "type": "list",
                "options": {
                    "action": {"choices": ["allow", "deny"], "type": "str"},
                    "apnmember": {"type": "raw"},
                    "id": {"type": "int"},
                    "selection-mode": {"type": "list", "choices": ["ms", "net", "vrf"], "elements": "str"}
                },
                "elements": "dict"
            },
            "apn-filter": {"choices": ["disable", "enable"], "type": "str"},
            "authorized-ggsns": {"type": "str"},
            "authorized-sgsns": {"type": "str"},
            "comment": {"type": "str"},
            "context-id": {"type": "int"},
            "control-plane-message-rate-limit": {"type": "int"},
            "default-apn-action": {"choices": ["allow", "deny"], "type": "str"},
            "default-imsi-action": {"choices": ["allow", "deny"], "type": "str"},
            "default-ip-action": {"choices": ["allow", "deny"], "type": "str"},
            "default-noip-action": {"choices": ["allow", "deny"], "type": "str"},
            "default-policy-action": {"choices": ["allow", "deny"], "type": "str"},
            "denied-log": {"choices": ["disable", "enable"], "type": "str"},
            "echo-request-interval": {"type": "int"},
            "extension-log": {"choices": ["disable", "enable"], "type": "str"},
            "forwarded-log": {"choices": ["disable", "enable"], "type": "str"},
            "global-tunnel-limit": {"type": "str"},
            "gtp-in-gtp": {"choices": ["allow", "deny"], "type": "str"},
            "gtpu-denied-log": {"choices": ["disable", "enable"], "type": "str"},
            "gtpu-forwarded-log": {"choices": ["disable", "enable"], "type": "str"},
            "gtpu-log-freq": {"type": "int"},
            "half-close-timeout": {"type": "int"},
            "half-open-timeout": {"type": "int"},
            "handover-group": {"type": "str"},
            "ie-remove-policy": {
                "type": "list",
                "options": {
                    "id": {"type": "int"},
                    "remove-ies": {"type": "list", "choices": ["apn-restriction", "rat-type", "rai", "uli", "imei"], "elements": "str"},
                    "sgsn-addr": {"type": "str"},
                    "sgsn-addr6": {"v_range": [["6.4.2", ""]], "type": "str"}
                },
                "elements": "dict"
            },
            "ie-remover": {"choices": ["disable", "enable"], "type": "str"},
            "ie-white-list-v0v1": {"type": "str"},
            "ie-white-list-v2": {"type": "str"},
            "imsi": {
                "type": "list",
                "options": {
                    "action": {"choices": ["allow", "deny"], "type": "str"},
                    "apnmember": {"type": "raw"},
                    "id": {"type": "int"},
                    "mcc-mnc": {"type": "str"},
                    "msisdn-prefix": {"type": "str"},
                    "selection-mode": {"type": "list", "choices": ["ms", "net", "vrf"], "elements": "str"}
                },
                "elements": "dict"
            },
            "imsi-filter": {"choices": ["disable", "enable"], "type": "str"},
            "interface-notify": {"type": "str"},
            "invalid-reserved-field": {"choices": ["allow", "deny"], "type": "str"},
            "invalid-sgsns-to-log": {"type": "str"},
            "ip-filter": {"choices": ["disable", "enable"], "type": "str"},
            "ip-policy": {
                "type": "list",
                "options": {
                    "action": {"choices": ["allow", "deny"], "type": "str"},
                    "dstaddr": {"type": "str"},
                    "id": {"type": "int"},
                    "srcaddr": {"type": "str"},
                    "dstaddr6": {"v_range": [["6.4.2", ""]], "type": "str"},
                    "srcaddr6": {"v_range": [["6.4.2", ""]], "type": "str"}
                },
                "elements": "dict"
            },
            "log-freq": {"type": "int"},
            "log-gtpu-limit": {"type": "int"},
            "log-imsi-prefix": {"type": "str"},
            "log-msisdn-prefix": {"type": "str"},
            "max-message-length": {"type": "int"},
            "message-filter-v0v1": {"type": "str"},
            "message-filter-v2": {"type": "str"},
            "min-message-length": {"type": "int"},
            "miss-must-ie": {"choices": ["allow", "deny"], "type": "str"},
            "monitor-mode": {"choices": ["disable", "enable", "vdom"], "type": "str"},
            "name": {"required": true, "type": "str"},
            "noip-filter": {"choices": ["disable", "enable"], "type": "str"},
            "noip-policy": {
                "type": "list",
                "options": {
                    "action": {"choices": ["allow", "deny"], "type": "str"},
                    "end": {"type": "int"},
                    "id": {"type": "int"},
                    "start": {"type": "int"},
                    "type": {"choices": ["etsi", "ietf"], "type": "str"}
                },
                "elements": "dict"
            },
            "out-of-state-ie": {"choices": ["allow", "deny"], "type": "str"},
            "out-of-state-message": {"choices": ["allow", "deny"], "type": "str"},
            "per-apn-shaper": {
                "type": "list",
                "options": {"apn": {"type": "str"}, "id": {"type": "int"}, "rate-limit": {"type": "int"}, "version": {"type": "int"}},
                "elements": "dict"
            },
            "policy": {
                "type": "list",
                "options": {
                    "action": {"choices": ["allow", "deny"], "type": "str"},
                    "apn-sel-mode": {"type": "list", "choices": ["ms", "net", "vrf"], "elements": "str"},
                    "apnmember": {"type": "raw"},
                    "id": {"type": "int"},
                    "imei": {"type": "str"},
                    "imsi": {"v_range": [["6.0.0", "7.2.1"]], "type": "str"},
                    "max-apn-restriction": {"choices": ["all", "public-1", "public-2", "private-1", "private-2"], "type": "str"},
                    "messages": {"type": "list", "choices": ["create-req", "create-res", "update-req", "update-res"], "elements": "str"},
                    "msisdn": {"v_range": [["6.0.0", "7.2.1"]], "type": "str"},
                    "rai": {"type": "str"},
                    "rat-type": {
                        "type": "list",
                        "choices": ["any", "utran", "geran", "wlan", "gan", "hspa", "eutran", "virtual", "nbiot"],
                        "elements": "str"
                    },
                    "uli": {"type": "str"},
                    "imsi-prefix": {"v_range": [["6.2.0", ""]], "type": "str"},
                    "msisdn-prefix": {"v_range": [["6.2.0", ""]], "type": "str"},
                    "apn": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"}
                },
                "elements": "dict"
            },
            "policy-filter": {"choices": ["disable", "enable"], "type": "str"},
            "port-notify": {"type": "int"},
            "rate-limit-mode": {"choices": ["per-profile", "per-stream", "per-apn"], "type": "str"},
            "rate-limited-log": {"choices": ["disable", "enable"], "type": "str"},
            "rate-sampling-interval": {"type": "int"},
            "remove-if-echo-expires": {"choices": ["disable", "enable"], "type": "str"},
            "remove-if-recovery-differ": {"choices": ["disable", "enable"], "type": "str"},
            "reserved-ie": {"choices": ["allow", "deny"], "type": "str"},
            "send-delete-when-timeout": {"choices": ["disable", "enable"], "type": "str"},
            "send-delete-when-timeout-v2": {"choices": ["disable", "enable"], "type": "str"},
            "spoof-src-addr": {"choices": ["allow", "deny"], "type": "str"},
            "state-invalid-log": {"choices": ["disable", "enable"], "type": "str"},
            "traffic-count-log": {"choices": ["disable", "enable"], "type": "str"},
            "tunnel-limit": {"type": "int"},
            "tunnel-limit-log": {"choices": ["disable", "enable"], "type": "str"},
            "tunnel-timeout": {"type": "int"},
            "unknown-version-action": {"choices": ["allow", "deny"], "type": "str"},
            "user-plane-message-rate-limit": {"type": "int"},
            "warning-threshold": {"type": "int"},
            "policy-v2": {
                "v_range": [["6.2.1", ""]],
                "type": "list",
                "options": {
                    "action": {"v_range": [["6.2.1", ""]], "choices": ["deny", "allow"], "type": "str"},
                    "apn-sel-mode": {"v_range": [["6.2.1", ""]], "type": "list", "choices": ["ms", "net", "vrf"], "elements": "str"},
                    "apnmember": {"v_range": [["6.2.1", ""]], "type": "raw"},
                    "id": {"v_range": [["6.2.1", ""]], "type": "int"},
                    "imsi-prefix": {"v_range": [["6.2.1", ""]], "type": "str"},
                    "max-apn-restriction": {
                        "v_range": [["6.2.1", ""]],
                        "choices": ["all", "public-1", "public-2", "private-1", "private-2"],
                        "type": "str"
                    },
                    "mei": {"v_range": [["6.2.1", ""]], "type": "str"},
                    "messages": {
                        "v_range": [["6.2.1", ""]],
                        "type": "list",
                        "choices": ["create-ses-req", "create-ses-res", "modify-bearer-req", "modify-bearer-res"],
                        "elements": "str"
                    },
                    "msisdn-prefix": {"v_range": [["6.2.1", ""]], "type": "str"},
                    "rat-type": {
                        "v_range": [["6.2.1", ""]],
                        "type": "list",
                        "choices": ["any", "utran", "geran", "wlan", "gan", "hspa", "eutran", "virtual", "nbiot", "ltem", "nr"],
                        "elements": "str"
                    },
                    "uli": {"v_range": [["6.2.1", ""]], "type": "raw"}
                },
                "elements": "dict"
            },
            "sub-second-interval": {"v_range": [["6.2.2", ""]], "choices": ["0.1", "0.25", "0.5"], "type": "str"},
            "sub-second-sampling": {"v_range": [["6.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "authorized-ggsns6": {"v_range": [["6.4.2", ""]], "type": "str"},
            "authorized-sgsns6": {"v_range": [["6.4.2", ""]], "type": "str"},
            "handover-group6": {"v_range": [["6.4.2", ""]], "type": "str"},
            "invalid-sgsns6-to-log": {"v_range": [["6.4.2", ""]], "type": "str"},
            "ie-validation": {
                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                "type": "dict",
                "options": {
                    "apn-restriction": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "charging-ID": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "charging-gateway-addr": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "end-user-addr": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "gsn-addr": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "imei": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "imsi": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "mm-context": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "ms-tzone": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "ms-validated": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "msisdn": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "nsapi": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "pdp-context": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "qos-profile": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "rai": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "rat-type": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "reordering-required": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "selection-mode": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "uli": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"}
                }
            },
            "message-rate-limit": {
                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                "type": "dict",
                "options": {
                    "create-aa-pdp-request": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "create-aa-pdp-response": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "create-mbms-request": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "create-mbms-response": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "create-pdp-request": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "create-pdp-response": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "delete-aa-pdp-request": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "delete-aa-pdp-response": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "delete-mbms-request": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "delete-mbms-response": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "delete-pdp-request": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "delete-pdp-response": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "echo-reponse": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "echo-request": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "error-indication": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "failure-report-request": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "failure-report-response": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "fwd-reloc-complete-ack": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "fwd-relocation-complete": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "fwd-relocation-request": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "fwd-relocation-response": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "fwd-srns-context": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "fwd-srns-context-ack": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "g-pdu": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "identification-request": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "identification-response": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "mbms-de-reg-request": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "mbms-de-reg-response": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "mbms-notify-rej-request": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "mbms-notify-rej-response": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "mbms-notify-request": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "mbms-notify-response": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "mbms-reg-request": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "mbms-reg-response": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "mbms-ses-start-request": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "mbms-ses-start-response": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "mbms-ses-stop-request": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "mbms-ses-stop-response": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "note-ms-request": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "note-ms-response": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "pdu-notify-rej-request": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "pdu-notify-rej-response": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "pdu-notify-request": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "pdu-notify-response": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "ran-info": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "relocation-cancel-request": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "relocation-cancel-response": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "send-route-request": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "send-route-response": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "sgsn-context-ack": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "sgsn-context-request": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "sgsn-context-response": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "support-ext-hdr-notify": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "update-mbms-request": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "update-mbms-response": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "update-pdp-request": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "update-pdp-response": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "version-not-support": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "echo-response": {"v_range": [["7.4.3", ""]], "type": "int"}
                }
            },
            "message-rate-limit-v0": {
                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                "type": "dict",
                "options": {
                    "create-pdp-request": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "delete-pdp-request": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "echo-request": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"}
                }
            },
            "message-rate-limit-v1": {
                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                "type": "dict",
                "options": {
                    "create-pdp-request": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "delete-pdp-request": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "echo-request": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"}
                }
            },
            "message-rate-limit-v2": {
                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                "type": "dict",
                "options": {
                    "create-session-request": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "delete-session-request": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "echo-request": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"}
                }
            },
            "ie-allow-list-v0v1": {"v_range": [["7.0.0", ""]], "type": "str"},
            "ie-allow-list-v2": {"v_range": [["7.0.0", ""]], "type": "str"},
            "rat-timeout-profile": {"v_range": [["7.0.1", ""]], "type": "str"},
            "message-filter": {
                "v_range": [["6.2.8", "6.2.12"]],
                "type": "dict",
                "options": {
                    "create-aa-pdp": {"v_range": [["6.2.8", "6.2.12"]], "choices": ["allow", "deny"], "type": "str"},
                    "create-mbms": {"v_range": [["6.2.8", "6.2.12"]], "choices": ["allow", "deny"], "type": "str"},
                    "create-pdp": {"v_range": [["6.2.8", "6.2.12"]], "choices": ["allow", "deny"], "type": "str"},
                    "data-record": {"v_range": [["6.2.8", "6.2.12"]], "choices": ["allow", "deny"], "type": "str"},
                    "delete-aa-pdp": {"v_range": [["6.2.8", "6.2.12"]], "choices": ["allow", "deny"], "type": "str"},
                    "delete-mbms": {"v_range": [["6.2.8", "6.2.12"]], "choices": ["allow", "deny"], "type": "str"},
                    "delete-pdp": {"v_range": [["6.2.8", "6.2.12"]], "choices": ["allow", "deny"], "type": "str"},
                    "echo": {"v_range": [["6.2.8", "6.2.12"]], "choices": ["allow", "deny"], "type": "str"},
                    "error-indication": {"v_range": [["6.2.8", "6.2.12"]], "choices": ["allow", "deny"], "type": "str"},
                    "failure-report": {"v_range": [["6.2.8", "6.2.12"]], "choices": ["allow", "deny"], "type": "str"},
                    "fwd-relocation": {"v_range": [["6.2.8", "6.2.12"]], "choices": ["allow", "deny"], "type": "str"},
                    "fwd-srns-context": {"v_range": [["6.2.8", "6.2.12"]], "choices": ["allow", "deny"], "type": "str"},
                    "gtp-pdu": {"v_range": [["6.2.8", "6.2.12"]], "choices": ["allow", "deny"], "type": "str"},
                    "identification": {"v_range": [["6.2.8", "6.2.12"]], "choices": ["allow", "deny"], "type": "str"},
                    "mbms-notification": {"v_range": [["6.2.8", "6.2.12"]], "choices": ["allow", "deny"], "type": "str"},
                    "node-alive": {"v_range": [["6.2.8", "6.2.12"]], "choices": ["allow", "deny"], "type": "str"},
                    "note-ms-present": {"v_range": [["6.2.8", "6.2.12"]], "choices": ["allow", "deny"], "type": "str"},
                    "pdu-notification": {"v_range": [["6.2.8", "6.2.12"]], "choices": ["allow", "deny"], "type": "str"},
                    "ran-info": {"v_range": [["6.2.8", "6.2.12"]], "choices": ["allow", "deny"], "type": "str"},
                    "redirection": {"v_range": [["6.2.8", "6.2.12"]], "choices": ["allow", "deny"], "type": "str"},
                    "relocation-cancel": {"v_range": [["6.2.8", "6.2.12"]], "choices": ["allow", "deny"], "type": "str"},
                    "send-route": {"v_range": [["6.2.8", "6.2.12"]], "choices": ["allow", "deny"], "type": "str"},
                    "sgsn-context": {"v_range": [["6.2.8", "6.2.12"]], "choices": ["allow", "deny"], "type": "str"},
                    "support-extension": {"v_range": [["6.2.8", "6.2.12"]], "choices": ["allow", "deny"], "type": "str"},
                    "unknown-message-action": {"v_range": [["6.2.8", "6.2.12"]], "choices": ["allow", "deny"], "type": "str"},
                    "update-mbms": {"v_range": [["6.2.8", "6.2.12"]], "choices": ["allow", "deny"], "type": "str"},
                    "update-pdp": {"v_range": [["6.2.8", "6.2.12"]], "choices": ["allow", "deny"], "type": "str"},
                    "version-not-support": {"v_range": [["6.2.8", "6.2.12"]], "choices": ["allow", "deny"], "type": "str"}
                }
            },
            "gtpv0": {"v_range": [["7.6.0", ""]], "choices": ["allow", "deny"], "type": "str"}
        }
    },
    "firewall_gtp_apn": {
        "stated": true,
        "options": {
            "action": {"choices": ["allow", "deny"], "type": "str"},
            "apnmember": {"type": "raw"},
            "id": {"required": true, "type": "int"},
            "selection-mode": {"type": "list", "choices": ["ms", "net", "vrf"], "elements": "str"}
        }
    },
    "firewall_gtp_ieremovepolicy": {
        "stated": true,
        "options": {
            "id": {"required": true, "type": "int"},
            "remove-ies": {"type": "list", "choices": ["apn-restriction", "rat-type", "rai", "uli", "imei"], "elements": "str"},
            "sgsn-addr": {"type": "str"},
            "sgsn-addr6": {"v_range": [["6.4.2", ""]], "type": "str"}
        }
    },
    "firewall_gtp_ievalidation": {
        "stated": false,
        "options": {
            "apn-restriction": {"choices": ["disable", "enable"], "type": "str"},
            "charging-ID": {"choices": ["disable", "enable"], "type": "str"},
            "charging-gateway-addr": {"choices": ["disable", "enable"], "type": "str"},
            "end-user-addr": {"choices": ["disable", "enable"], "type": "str"},
            "gsn-addr": {"choices": ["disable", "enable"], "type": "str"},
            "imei": {"choices": ["disable", "enable"], "type": "str"},
            "imsi": {"choices": ["disable", "enable"], "type": "str"},
            "mm-context": {"choices": ["disable", "enable"], "type": "str"},
            "ms-tzone": {"choices": ["disable", "enable"], "type": "str"},
            "ms-validated": {"choices": ["disable", "enable"], "type": "str"},
            "msisdn": {"choices": ["disable", "enable"], "type": "str"},
            "nsapi": {"choices": ["disable", "enable"], "type": "str"},
            "pdp-context": {"choices": ["disable", "enable"], "type": "str"},
            "qos-profile": {"choices": ["disable", "enable"], "type": "str"},
            "rai": {"choices": ["disable", "enable"], "type": "str"},
            "rat-type": {"choices": ["disable", "enable"], "type": "str"},
            "reordering-required": {"choices": ["disable", "enable"], "type": "str"},
            "selection-mode": {"choices": ["disable", "enable"], "type": "str"},
            "uli": {"choices": ["disable", "enable"], "type": "str"}
        }
    },
    "firewall_gtp_imsi": {
        "stated": true,
        "options": {
            "action": {"choices": ["allow", "deny"], "type": "str"},
            "apnmember": {"type": "raw"},
            "id": {"required": true, "type": "int"},
            "mcc-mnc": {"type": "str"},
            "msisdn-prefix": {"type": "str"},
            "selection-mode": {"type": "list", "choices": ["ms", "net", "vrf"], "elements": "str"}
        }
    },
    "firewall_gtp_ippolicy": {
        "stated": true,
        "options": {
            "action": {"choices": ["allow", "deny"], "type": "str"},
            "dstaddr": {"type": "str"},
            "id": {"required": true, "type": "int"},
            "srcaddr": {"type": "str"},
            "dstaddr6": {"v_range": [["6.4.2", ""]], "type": "str"},
            "srcaddr6": {"v_range": [["6.4.2", ""]], "type": "str"}
        }
    },
    "firewall_gtp_messagefilter": {
        "stated": false,
        "options": {
            "create-aa-pdp": {"v_range": [["6.2.0", "6.2.12"]], "choices": ["allow", "deny"], "type": "str"},
            "create-mbms": {"v_range": [["6.2.0", "6.2.12"]], "choices": ["allow", "deny"], "type": "str"},
            "create-pdp": {"v_range": [["6.2.0", "6.2.12"]], "choices": ["allow", "deny"], "type": "str"},
            "data-record": {"v_range": [["6.2.0", "6.2.12"]], "choices": ["allow", "deny"], "type": "str"},
            "delete-aa-pdp": {"v_range": [["6.2.0", "6.2.12"]], "choices": ["allow", "deny"], "type": "str"},
            "delete-mbms": {"v_range": [["6.2.0", "6.2.12"]], "choices": ["allow", "deny"], "type": "str"},
            "delete-pdp": {"v_range": [["6.2.0", "6.2.12"]], "choices": ["allow", "deny"], "type": "str"},
            "echo": {"v_range": [["6.2.0", "6.2.12"]], "choices": ["allow", "deny"], "type": "str"},
            "error-indication": {"v_range": [["6.2.0", "6.2.12"]], "choices": ["allow", "deny"], "type": "str"},
            "failure-report": {"v_range": [["6.2.0", "6.2.12"]], "choices": ["allow", "deny"], "type": "str"},
            "fwd-relocation": {"v_range": [["6.2.0", "6.2.12"]], "choices": ["allow", "deny"], "type": "str"},
            "fwd-srns-context": {"v_range": [["6.2.0", "6.2.12"]], "choices": ["allow", "deny"], "type": "str"},
            "gtp-pdu": {"v_range": [["6.2.0", "6.2.12"]], "choices": ["allow", "deny"], "type": "str"},
            "identification": {"v_range": [["6.2.0", "6.2.12"]], "choices": ["allow", "deny"], "type": "str"},
            "mbms-notification": {"v_range": [["6.2.0", "6.2.12"]], "choices": ["allow", "deny"], "type": "str"},
            "node-alive": {"v_range": [["6.2.0", "6.2.12"]], "choices": ["allow", "deny"], "type": "str"},
            "note-ms-present": {"v_range": [["6.2.0", "6.2.12"]], "choices": ["allow", "deny"], "type": "str"},
            "pdu-notification": {"v_range": [["6.2.0", "6.2.12"]], "choices": ["allow", "deny"], "type": "str"},
            "ran-info": {"v_range": [["6.2.0", "6.2.12"]], "choices": ["allow", "deny"], "type": "str"},
            "redirection": {"v_range": [["6.2.0", "6.2.12"]], "choices": ["allow", "deny"], "type": "str"},
            "relocation-cancel": {"v_range": [["6.2.0", "6.2.12"]], "choices": ["allow", "deny"], "type": "str"},
            "send-route": {"v_range": [["6.2.0", "6.2.12"]], "choices": ["allow", "deny"], "type": "str"},
            "sgsn-context": {"v_range": [["6.2.0", "6.2.12"]], "choices": ["allow", "deny"], "type": "str"},
            "support-extension": {"v_range": [["6.2.0", "6.2.12"]], "choices": ["allow", "deny"], "type": "str"},
            "unknown-message-action": {"v_range": [["6.2.0", "6.2.12"]], "choices": ["allow", "deny"], "type": "str"},
            "update-mbms": {"v_range": [["6.2.0", "6.2.12"]], "choices": ["allow", "deny"], "type": "str"},
            "update-pdp": {"v_range": [["6.2.0", "6.2.12"]], "choices": ["allow", "deny"], "type": "str"},
            "version-not-support": {"v_range": [["6.2.0", "6.2.12"]], "choices": ["allow", "deny"], "type": "str"}
        }
    },
    "firewall_gtp_messageratelimit": {
        "stated": false,
        "options": {
            "create-aa-pdp-request": {"type": "int"},
            "create-aa-pdp-response": {"type": "int"},
            "create-mbms-request": {"type": "int"},
            "create-mbms-response": {"type": "int"},
            "create-pdp-request": {"type": "int"},
            "create-pdp-response": {"type": "int"},
            "delete-aa-pdp-request": {"type": "int"},
            "delete-aa-pdp-response": {"type": "int"},
            "delete-mbms-request": {"type": "int"},
            "delete-mbms-response": {"type": "int"},
            "delete-pdp-request": {"type": "int"},
            "delete-pdp-response": {"type": "int"},
            "echo-reponse": {"type": "int"},
            "echo-request": {"type": "int"},
            "error-indication": {"type": "int"},
            "failure-report-request": {"type": "int"},
            "failure-report-response": {"type": "int"},
            "fwd-reloc-complete-ack": {"type": "int"},
            "fwd-relocation-complete": {"type": "int"},
            "fwd-relocation-request": {"type": "int"},
            "fwd-relocation-response": {"type": "int"},
            "fwd-srns-context": {"type": "int"},
            "fwd-srns-context-ack": {"type": "int"},
            "g-pdu": {"type": "int"},
            "identification-request": {"type": "int"},
            "identification-response": {"type": "int"},
            "mbms-de-reg-request": {"type": "int"},
            "mbms-de-reg-response": {"type": "int"},
            "mbms-notify-rej-request": {"type": "int"},
            "mbms-notify-rej-response": {"type": "int"},
            "mbms-notify-request": {"type": "int"},
            "mbms-notify-response": {"type": "int"},
            "mbms-reg-request": {"type": "int"},
            "mbms-reg-response": {"type": "int"},
            "mbms-ses-start-request": {"type": "int"},
            "mbms-ses-start-response": {"type": "int"},
            "mbms-ses-stop-request": {"type": "int"},
            "mbms-ses-stop-response": {"type": "int"},
            "note-ms-request": {"type": "int"},
            "note-ms-response": {"type": "int"},
            "pdu-notify-rej-request": {"type": "int"},
            "pdu-notify-rej-response": {"type": "int"},
            "pdu-notify-request": {"type": "int"},
            "pdu-notify-response": {"type": "int"},
            "ran-info": {"type": "int"},
            "relocation-cancel-request": {"type": "int"},
            "relocation-cancel-response": {"type": "int"},
            "send-route-request": {"type": "int"},
            "send-route-response": {"type": "int"},
            "sgsn-context-ack": {"type": "int"},
            "sgsn-context-request": {"type": "int"},
            "sgsn-context-response": {"type": "int"},
            "support-ext-hdr-notify": {"type": "int"},
            "update-mbms-request": {"type": "int"},
            "update-mbms-response": {"type": "int"},
            "update-pdp-request": {"type": "int"},
            "update-pdp-response": {"type": "int"},
            "version-not-support": {"type": "int"},
            "echo-response": {"v_range": [["7.4.3", ""]], "type": "int"}
        }
    },
    "firewall_gtp_messageratelimitv0": {
        "stated": false,
        "options": {"create-pdp-request": {"type": "int"}, "delete-pdp-request": {"type": "int"}, "echo-request": {"type": "int"}}
    },
    "firewall_gtp_messageratelimitv1": {
        "stated": false,
        "options": {"create-pdp-request": {"type": "int"}, "delete-pdp-request": {"type": "int"}, "echo-request": {"type": "int"}}
    },
    "firewall_gtp_messageratelimitv2": {
        "stated": false,
        "options": {"create-session-request": {"type": "int"}, "delete-session-request": {"type": "int"}, "echo-request": {"type": "int"}}
    },
    "firewall_gtp_noippolicy": {
        "stated": true,
        "options": {
            "action": {"choices": ["allow", "deny"], "type": "str"},
            "end": {"type": "int"},
            "id": {"required": true, "type": "int"},
            "start": {"type": "int"},
            "type": {"choices": ["etsi", "ietf"], "type": "str"}
        }
    },
    "firewall_gtp_perapnshaper": {
        "stated": true,
        "options": {"apn": {"type": "str"}, "id": {"required": true, "type": "int"}, "rate-limit": {"type": "int"}, "version": {"type": "int"}}
    },
    "firewall_gtp_policy": {
        "stated": true,
        "options": {
            "action": {"choices": ["allow", "deny"], "type": "str"},
            "apn-sel-mode": {"type": "list", "choices": ["ms", "net", "vrf"], "elements": "str"},
            "apnmember": {"type": "raw"},
            "id": {"required": true, "type": "int"},
            "imei": {"type": "str"},
            "imsi": {"v_range": [["6.0.0", "7.2.1"]], "type": "str"},
            "max-apn-restriction": {"choices": ["all", "public-1", "public-2", "private-1", "private-2"], "type": "str"},
            "messages": {"type": "list", "choices": ["create-req", "create-res", "update-req", "update-res"], "elements": "str"},
            "msisdn": {"v_range": [["6.0.0", "7.2.1"]], "type": "str"},
            "rai": {"type": "str"},
            "rat-type": {"type": "list", "choices": ["any", "utran", "geran", "wlan", "gan", "hspa", "eutran", "virtual", "nbiot"], "elements": "str"},
            "uli": {"type": "str"},
            "imsi-prefix": {"v_range": [["6.2.0", ""]], "type": "str"},
            "msisdn-prefix": {"v_range": [["6.2.0", ""]], "type": "str"},
            "apn": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"}
        }
    },
    "firewall_gtp_policyv2": {
        "stated": true,
        "options": {
            "action": {"v_range": [["6.2.1", ""]], "choices": ["deny", "allow"], "type": "str"},
            "apn-sel-mode": {"v_range": [["6.2.1", ""]], "type": "list", "choices": ["ms", "net", "vrf"], "elements": "str"},
            "apnmember": {"v_range": [["6.2.1", ""]], "type": "raw"},
            "id": {"v_range": [["6.2.1", ""]], "required": true, "type": "int"},
            "imsi-prefix": {"v_range": [["6.2.1", ""]], "type": "str"},
            "max-apn-restriction": {"v_range": [["6.2.1", ""]], "choices": ["all", "public-1", "public-2", "private-1", "private-2"], "type": "str"},
            "mei": {"v_range": [["6.2.1", ""]], "type": "str"},
            "messages": {
                "v_range": [["6.2.1", ""]],
                "type": "list",
                "choices": ["create-ses-req", "create-ses-res", "modify-bearer-req", "modify-bearer-res"],
                "elements": "str"
            },
            "msisdn-prefix": {"v_range": [["6.2.1", ""]], "type": "str"},
            "rat-type": {
                "v_range": [["6.2.1", ""]],
                "type": "list",
                "choices": ["any", "utran", "geran", "wlan", "gan", "hspa", "eutran", "virtual", "nbiot", "ltem", "nr"],
                "elements": "str"
            },
            "uli": {"v_range": [["6.2.1", ""]], "type": "raw"}
        }
    },
    "firewall_identitybasedroute": {
        "stated": true,
        "options": {
            "comments": {"type": "str"},
            "name": {"required": true, "type": "str"},
            "rule": {
                "type": "list",
                "options": {"device": {"type": "str"}, "gateway": {"type": "str"}, "groups": {"type": "raw"}, "id": {"type": "int"}},
                "elements": "dict"
            }
        }
    },
    "firewall_identitybasedroute_rule": {
        "stated": true,
        "options": {"device": {"type": "str"}, "gateway": {"type": "str"}, "groups": {"type": "raw"}, "id": {"required": true, "type": "int"}}
    },
    "firewall_internetservice": {
        "stated": false,
        "options": {
            "database": {"choices": ["isdb", "irdb"], "type": "str"},
            "direction": {"choices": ["src", "dst", "both"], "type": "str"},
            "entry": {
                "v_range": [["6.0.0", "7.2.1"]],
                "type": "list",
                "options": {
                    "id": {"v_range": [["6.0.0", "7.2.1"]], "type": "int"},
                    "ip-number": {"v_range": [["6.0.0", "7.2.1"]], "type": "int"},
                    "ip-range-number": {"v_range": [["6.0.0", "7.2.1"]], "type": "int"},
                    "port": {"v_range": [["6.0.0", "7.2.1"]], "type": "raw"},
                    "protocol": {"v_range": [["6.0.0", "7.2.1"]], "type": "int"}
                },
                "elements": "dict"
            },
            "icon-id": {"type": "int"},
            "id": {"type": "int"},
            "name": {"type": "str"},
            "offset": {"v_range": [["6.0.0", "7.2.1"]], "type": "int"},
            "reputation": {"type": "int"},
            "sld-id": {"type": "int"},
            "extra-ip-range-number": {"v_range": [["6.2.0", ""]], "type": "int"},
            "ip-number": {"v_range": [["6.2.0", ""]], "type": "int"},
            "ip-range-number": {"v_range": [["6.2.0", ""]], "type": "int"},
            "jitter-threshold": {"v_range": [["6.2.0", "7.2.0"]], "type": "int"},
            "latency-threshold": {"v_range": [["6.2.0", "7.2.0"]], "type": "int"},
            "obsolete": {"v_range": [["6.2.0", ""]], "type": "int"},
            "packetloss-threshold": {"v_range": [["6.2.0", "7.2.0"]], "type": "int"},
            "singularity": {"v_range": [["6.2.0", ""]], "type": "int"},
            "city": {"v_range": [["6.4.0", "7.4.2"]], "type": "raw"},
            "country": {"v_range": [["6.4.0", "7.4.2"]], "type": "raw"},
            "region": {"v_range": [["6.4.0", "7.4.2"]], "type": "raw"},
            "city6": {"v_range": [["7.2.1", "7.4.2"]], "type": "raw"},
            "country6": {"v_range": [["7.2.1", "7.4.2"]], "type": "raw"},
            "extra-ip6-range-number": {"v_range": [["7.2.1", ""]], "type": "int"},
            "ip6-range-number": {"v_range": [["7.2.1", ""]], "type": "int"},
            "region6": {"v_range": [["7.2.1", "7.4.2"]], "type": "raw"}
        }
    },
    "firewall_internetservice_entry": {
        "stated": true,
        "options": {
            "id": {"v_range": [["6.0.0", "7.2.1"]], "required": true, "type": "int"},
            "ip-number": {"v_range": [["6.0.0", "7.2.1"]], "type": "int"},
            "ip-range-number": {"v_range": [["6.0.0", "7.2.1"]], "type": "int"},
            "port": {"v_range": [["6.0.0", "7.2.1"]], "type": "raw"},
            "protocol": {"v_range": [["6.0.0", "7.2.1"]], "type": "int"}
        }
    },
    "firewall_internetserviceaddition": {
        "stated": true,
        "options": {
            "comment": {"v_range": [["6.2.2", ""]], "type": "str"},
            "entry": {
                "v_range": [["6.2.2", ""]],
                "type": "list",
                "options": {
                    "id": {"v_range": [["6.2.2", ""]], "type": "int"},
                    "port-range": {
                        "v_range": [["6.2.2", ""]],
                        "type": "list",
                        "options": {
                            "end-port": {"v_range": [["6.2.2", ""]], "type": "int"},
                            "id": {"v_range": [["6.2.2", ""]], "type": "int"},
                            "start-port": {"v_range": [["6.2.2", ""]], "type": "int"}
                        },
                        "elements": "dict"
                    },
                    "protocol": {"v_range": [["6.2.2", ""]], "type": "int"},
                    "addr-mode": {"v_range": [["7.2.1", ""]], "choices": ["ipv4", "ipv6"], "type": "str"}
                },
                "elements": "dict"
            },
            "id": {"v_range": [["6.2.2", ""]], "required": true, "type": "str"}
        }
    },
    "firewall_internetserviceaddition_entry": {
        "stated": true,
        "options": {
            "id": {"v_range": [["6.2.2", ""]], "required": true, "type": "int"},
            "port-range": {
                "v_range": [["6.2.2", ""]],
                "type": "list",
                "options": {
                    "end-port": {"v_range": [["6.2.2", ""]], "type": "int"},
                    "id": {"v_range": [["6.2.2", ""]], "type": "int"},
                    "start-port": {"v_range": [["6.2.2", ""]], "type": "int"}
                },
                "elements": "dict"
            },
            "protocol": {"v_range": [["6.2.2", ""]], "type": "int"},
            "addr-mode": {"v_range": [["7.2.1", ""]], "choices": ["ipv4", "ipv6"], "type": "str"}
        }
    },
    "firewall_internetserviceaddition_entry_portrange": {
        "stated": true,
        "options": {
            "end-port": {"v_range": [["6.2.2", ""]], "type": "int"},
            "id": {"v_range": [["6.2.2", ""]], "required": true, "type": "int"},
            "start-port": {"v_range": [["6.2.2", ""]], "type": "int"}
        }
    },
    "firewall_internetservicecustom": {
        "stated": true,
        "options": {
            "comment": {"type": "str"},
            "disable-entry": {
                "v_range": [["6.0.0", "7.2.1"]],
                "type": "list",
                "options": {
                    "id": {"v_range": [["6.0.0", "7.2.1"]], "type": "int"},
                    "ip-range": {
                        "v_range": [["6.0.0", "7.2.1"]],
                        "type": "list",
                        "options": {
                            "end-ip": {"v_range": [["6.0.0", "7.2.1"]], "type": "str"},
                            "id": {"v_range": [["6.0.0", "7.2.1"]], "type": "int"},
                            "start-ip": {"v_range": [["6.0.0", "7.2.1"]], "type": "str"}
                        },
                        "elements": "dict"
                    },
                    "port": {"v_range": [["6.0.0", "7.2.1"]], "type": "raw"},
                    "protocol": {"v_range": [["6.0.0", "7.2.1"]], "type": "int"}
                },
                "elements": "dict"
            },
            "entry": {
                "type": "list",
                "options": {
                    "dst": {"type": "raw"},
                    "id": {"type": "int"},
                    "port-range": {
                        "type": "list",
                        "options": {"end-port": {"type": "int"}, "id": {"type": "int"}, "start-port": {"type": "int"}},
                        "elements": "dict"
                    },
                    "protocol": {"type": "int"},
                    "addr-mode": {"v_range": [["7.2.1", ""]], "choices": ["ipv4", "ipv6"], "type": "str"},
                    "dst6": {"v_range": [["7.2.1", ""]], "type": "raw"}
                },
                "elements": "dict"
            },
            "master-service-id": {"v_range": [["6.0.0", "7.2.1"]], "type": "str"},
            "name": {"required": true, "type": "str"},
            "reputation": {"v_range": [["6.2.0", ""]], "type": "int"},
            "id": {"v_range": [["6.4.2", ""]], "type": "int"}
        }
    },
    "firewall_internetservicecustom_disableentry": {
        "stated": true,
        "options": {
            "id": {"v_range": [["6.0.0", "7.2.1"]], "required": true, "type": "int"},
            "ip-range": {
                "v_range": [["6.0.0", "7.2.1"]],
                "type": "list",
                "options": {
                    "end-ip": {"v_range": [["6.0.0", "7.2.1"]], "type": "str"},
                    "id": {"v_range": [["6.0.0", "7.2.1"]], "type": "int"},
                    "start-ip": {"v_range": [["6.0.0", "7.2.1"]], "type": "str"}
                },
                "elements": "dict"
            },
            "port": {"v_range": [["6.0.0", "7.2.1"]], "type": "raw"},
            "protocol": {"v_range": [["6.0.0", "7.2.1"]], "type": "int"}
        }
    },
    "firewall_internetservicecustom_disableentry_iprange": {
        "stated": true,
        "options": {
            "end-ip": {"v_range": [["6.0.0", "7.2.1"]], "type": "str"},
            "id": {"v_range": [["6.0.0", "7.2.1"]], "required": true, "type": "int"},
            "start-ip": {"v_range": [["6.0.0", "7.2.1"]], "type": "str"}
        }
    },
    "firewall_internetservicecustom_entry": {
        "stated": true,
        "options": {
            "dst": {"type": "raw"},
            "id": {"required": true, "type": "int"},
            "port-range": {
                "type": "list",
                "options": {"end-port": {"type": "int"}, "id": {"type": "int"}, "start-port": {"type": "int"}},
                "elements": "dict"
            },
            "protocol": {"type": "int"},
            "addr-mode": {"v_range": [["7.2.1", ""]], "choices": ["ipv4", "ipv6"], "type": "str"},
            "dst6": {"v_range": [["7.2.1", ""]], "type": "raw"}
        }
    },
    "firewall_internetservicecustom_entry_portrange": {
        "stated": true,
        "options": {"end-port": {"type": "int"}, "id": {"required": true, "type": "int"}, "start-port": {"type": "int"}}
    },
    "firewall_internetservicecustomgroup": {
        "stated": true,
        "options": {"comment": {"type": "str"}, "member": {"type": "raw"}, "name": {"required": true, "type": "str"}}
    },
    "firewall_internetservicegroup": {
        "stated": true,
        "options": {
            "comment": {"type": "str"},
            "member": {"type": "raw"},
            "name": {"required": true, "type": "str"},
            "direction": {"v_range": [["6.2.0", ""]], "choices": ["both", "source", "destination"], "type": "str"}
        }
    },
    "firewall_internetservicename": {
        "stated": true,
        "options": {
            "city-id": {"v_range": [["6.4.0", ""]], "type": "int"},
            "country-id": {"v_range": [["6.4.0", ""]], "type": "int"},
            "internet-service-id": {"v_range": [["6.4.0", ""]], "type": "str"},
            "name": {"v_range": [["6.4.0", ""]], "required": true, "type": "str"},
            "region-id": {"v_range": [["6.4.0", ""]], "type": "int"},
            "type": {"v_range": [["6.4.0", ""]], "choices": ["default", "location"], "type": "str"}
        }
    },
    "firewall_ippool": {
        "stated": true,
        "options": {
            "arp-intf": {"type": "str"},
            "arp-reply": {"choices": ["disable", "enable"], "type": "str"},
            "associated-interface": {"type": "str"},
            "block-size": {"type": "int"},
            "comments": {"type": "str"},
            "dynamic_mapping": {
                "type": "list",
                "options": {
                    "_scope": {"type": "list", "options": {"name": {"type": "str"}, "vdom": {"type": "str"}}, "elements": "dict"},
                    "arp-intf": {"type": "str"},
                    "arp-reply": {"choices": ["disable", "enable"], "type": "str"},
                    "associated-interface": {"type": "str"},
                    "block-size": {"type": "int"},
                    "comments": {"type": "str"},
                    "endip": {"type": "str"},
                    "num-blocks-per-user": {"type": "int"},
                    "pba-timeout": {"type": "int"},
                    "permit-any-host": {"choices": ["disable", "enable"], "type": "str"},
                    "source-endip": {"type": "str"},
                    "source-startip": {"type": "str"},
                    "startip": {"type": "str"},
                    "type": {"choices": ["overload", "one-to-one", "fixed-port-range", "port-block-allocation", "cgn-resource-allocation"], "type": "str"},
                    "cgn-block-size": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "type": "int"},
                    "cgn-client-endip": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "type": "str"},
                    "cgn-client-startip": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "type": "str"},
                    "cgn-fixedalloc": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "cgn-overload": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "cgn-port-end": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "type": "int"},
                    "cgn-port-start": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "type": "int"},
                    "cgn-spa": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "utilization-alarm-clear": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "type": "int"},
                    "utilization-alarm-raise": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "type": "int"},
                    "endport": {"v_range": [["7.0.0", ""]], "type": "int"},
                    "port-per-user": {"v_range": [["7.0.0", ""]], "type": "int"},
                    "startport": {"v_range": [["7.0.0", ""]], "type": "int"},
                    "add-nat64-route": {"v_range": [["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "cgn-client-ipv6shift": {"v_range": [["6.2.9", "6.2.12"], ["6.4.7", "6.4.14"], ["7.0.2", ""]], "type": "int"},
                    "nat64": {"v_range": [["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "subnet-broadcast-in-ippool": {"v_range": [["7.0.5", "7.0.12"], ["7.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "exclude-ip": {"v_range": [["7.2.2", ""]], "type": "raw"},
                    "pba-interim-log": {"v_range": [["7.4.3", ""]], "type": "int"},
                    "client-prefix-length": {"v_range": [["7.6.0", ""]], "type": "int"},
                    "icmp-session-quota": {"v_range": [["7.6.0", ""]], "type": "int"},
                    "privileged-port-use-pba": {"v_range": [["7.6.0", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "source-prefix6": {"v_range": [["7.6.0", ""]], "type": "str"},
                    "tcp-session-quota": {"v_range": [["7.6.0", ""]], "type": "int"},
                    "udp-session-quota": {"v_range": [["7.6.0", ""]], "type": "int"}
                },
                "elements": "dict"
            },
            "endip": {"type": "str"},
            "name": {"required": true, "type": "str"},
            "num-blocks-per-user": {"type": "int"},
            "pba-timeout": {"type": "int"},
            "permit-any-host": {"choices": ["disable", "enable"], "type": "str"},
            "source-endip": {"type": "str"},
            "source-startip": {"type": "str"},
            "startip": {"type": "str"},
            "type": {"choices": ["overload", "one-to-one", "fixed-port-range", "port-block-allocation", "cgn-resource-allocation"], "type": "str"},
            "utilization-alarm-clear": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "type": "int"},
            "cgn-fixedalloc": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "cgn-client-startip": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "type": "str"},
            "cgn-client-endip": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "type": "str"},
            "cgn-overload": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "cgn-block-size": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "type": "int"},
            "utilization-alarm-raise": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "type": "int"},
            "cgn-port-start": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "type": "int"},
            "cgn-spa": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "cgn-port-end": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "type": "int"},
            "endport": {"v_range": [["7.0.0", ""]], "type": "int"},
            "port-per-user": {"v_range": [["7.0.0", ""]], "type": "int"},
            "startport": {"v_range": [["7.0.0", ""]], "type": "int"},
            "add-nat64-route": {"v_range": [["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "nat64": {"v_range": [["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "cgn-client-ipv6shift": {"v_range": [["6.2.9", "6.2.12"], ["6.4.7", "6.4.14"], ["7.0.2", ""]], "type": "int"},
            "subnet-broadcast-in-ippool": {"v_range": [["7.0.5", "7.0.12"], ["7.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "exclude-ip": {"v_range": [["7.2.2", ""]], "type": "raw"},
            "pba-interim-log": {"v_range": [["7.4.3", ""]], "type": "int"},
            "client-prefix-length": {"v_range": [["7.6.0", ""]], "type": "int"},
            "icmp-session-quota": {"v_range": [["7.6.0", ""]], "type": "int"},
            "privileged-port-use-pba": {"v_range": [["7.6.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "source-prefix6": {"v_range": [["7.6.0", ""]], "type": "str"},
            "tcp-session-quota": {"v_range": [["7.6.0", ""]], "type": "int"},
            "udp-session-quota": {"v_range": [["7.6.0", ""]], "type": "int"}
        }
    },
    "firewall_ippool6": {
        "stated": true,
        "options": {
            "comments": {"type": "str"},
            "dynamic_mapping": {
                "type": "list",
                "options": {
                    "_scope": {"type": "list", "options": {"name": {"type": "str"}, "vdom": {"type": "str"}}, "elements": "dict"},
                    "comments": {"type": "str"},
                    "endip": {"type": "str"},
                    "startip": {"type": "str"},
                    "add-nat46-route": {"v_range": [["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "nat46": {"v_range": [["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "external-prefix": {"v_range": [["7.6.0", ""]], "type": "str"},
                    "internal-prefix": {"v_range": [["7.6.0", ""]], "type": "str"},
                    "type": {"v_range": [["7.6.0", ""]], "choices": ["overload", "nptv6"], "type": "str"}
                },
                "elements": "dict"
            },
            "endip": {"type": "str"},
            "name": {"required": true, "type": "str"},
            "startip": {"type": "str"},
            "add-nat46-route": {"v_range": [["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "nat46": {"v_range": [["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "external-prefix": {"v_range": [["7.6.0", ""]], "type": "str"},
            "internal-prefix": {"v_range": [["7.6.0", ""]], "type": "str"},
            "type": {"v_range": [["7.6.0", ""]], "choices": ["overload", "nptv6"], "type": "str"}
        }
    },
    "firewall_ippool6_dynamicmapping": {
        "stated": true,
        "options": {
            "_scope": {"type": "list", "options": {"name": {"type": "str"}, "vdom": {"type": "str"}}, "elements": "dict"},
            "comments": {"type": "str"},
            "endip": {"type": "str"},
            "startip": {"type": "str"},
            "add-nat46-route": {"v_range": [["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "nat46": {"v_range": [["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "external-prefix": {"v_range": [["7.6.0", ""]], "type": "str"},
            "internal-prefix": {"v_range": [["7.6.0", ""]], "type": "str"},
            "type": {"v_range": [["7.6.0", ""]], "choices": ["overload", "nptv6"], "type": "str"}
        }
    },
    "firewall_ippool_dynamicmapping": {
        "stated": true,
        "options": {
            "_scope": {"type": "list", "options": {"name": {"type": "str"}, "vdom": {"type": "str"}}, "elements": "dict"},
            "arp-intf": {"type": "str"},
            "arp-reply": {"choices": ["disable", "enable"], "type": "str"},
            "associated-interface": {"type": "str"},
            "block-size": {"type": "int"},
            "comments": {"type": "str"},
            "endip": {"type": "str"},
            "num-blocks-per-user": {"type": "int"},
            "pba-timeout": {"type": "int"},
            "permit-any-host": {"choices": ["disable", "enable"], "type": "str"},
            "source-endip": {"type": "str"},
            "source-startip": {"type": "str"},
            "startip": {"type": "str"},
            "type": {"choices": ["overload", "one-to-one", "fixed-port-range", "port-block-allocation", "cgn-resource-allocation"], "type": "str"},
            "cgn-block-size": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "type": "int"},
            "cgn-client-endip": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "type": "str"},
            "cgn-client-startip": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "type": "str"},
            "cgn-fixedalloc": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "cgn-overload": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "cgn-port-end": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "type": "int"},
            "cgn-port-start": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "type": "int"},
            "cgn-spa": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "utilization-alarm-clear": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "type": "int"},
            "utilization-alarm-raise": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "type": "int"},
            "endport": {"v_range": [["7.0.0", ""]], "type": "int"},
            "port-per-user": {"v_range": [["7.0.0", ""]], "type": "int"},
            "startport": {"v_range": [["7.0.0", ""]], "type": "int"},
            "add-nat64-route": {"v_range": [["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "cgn-client-ipv6shift": {"v_range": [["6.2.9", "6.2.12"], ["6.4.7", "6.4.14"], ["7.0.2", ""]], "type": "int"},
            "nat64": {"v_range": [["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "subnet-broadcast-in-ippool": {"v_range": [["7.0.5", "7.0.12"], ["7.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "exclude-ip": {"v_range": [["7.2.2", ""]], "type": "raw"},
            "pba-interim-log": {"v_range": [["7.4.3", ""]], "type": "int"},
            "client-prefix-length": {"v_range": [["7.6.0", ""]], "type": "int"},
            "icmp-session-quota": {"v_range": [["7.6.0", ""]], "type": "int"},
            "privileged-port-use-pba": {"v_range": [["7.6.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "source-prefix6": {"v_range": [["7.6.0", ""]], "type": "str"},
            "tcp-session-quota": {"v_range": [["7.6.0", ""]], "type": "int"},
            "udp-session-quota": {"v_range": [["7.6.0", ""]], "type": "int"}
        }
    },
    "firewall_ippoolgrp": {
        "stated": true,
        "options": {
            "comments": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "str"},
            "member": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "raw"},
            "name": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "required": true, "type": "str"},
            "uuid": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "str"}
        }
    },
    "firewall_ldbmonitor": {
        "stated": true,
        "options": {
            "http-get": {"type": "str"},
            "http-match": {"type": "str"},
            "http-max-redirects": {"type": "int"},
            "interval": {"type": "int"},
            "name": {"required": true, "type": "str"},
            "port": {"type": "int"},
            "retry": {"type": "int"},
            "timeout": {"type": "int"},
            "type": {"choices": ["ping", "tcp", "http", "passive-sip", "https", "dns"], "type": "str"},
            "src-ip": {"v_range": [["6.4.0", ""]], "type": "str"},
            "dns-match-ip": {"v_range": [["7.0.0", ""]], "type": "str"},
            "dns-protocol": {"v_range": [["7.0.0", ""]], "choices": ["udp", "tcp"], "type": "str"},
            "dns-request-domain": {"v_range": [["7.0.0", ""]], "type": "str"}
        }
    },
    "firewall_mmsprofile": {
        "stated": true,
        "options": {
            "avnotificationtable": {"type": "str"},
            "bwordtable": {"type": "str"},
            "carrier-endpoint-prefix": {"choices": ["disable", "enable"], "type": "str"},
            "carrier-endpoint-prefix-range-max": {"type": "int"},
            "carrier-endpoint-prefix-range-min": {"type": "int"},
            "carrier-endpoint-prefix-string": {"type": "str"},
            "carrierendpointbwltable": {"type": "str"},
            "comment": {"type": "str"},
            "mm1": {
                "type": "list",
                "choices": [
                    "avmonitor", "block", "oversize", "quarantine", "scan", "avquery", "bannedword", "no-content-summary", "archive-summary",
                    "archive-full", "carrier-endpoint-bwl", "remove-blocked", "chunkedbypass", "clientcomfort", "servercomfort", "strict-file",
                    "mms-checksum"
                ],
                "elements": "str"
            },
            "mm1-addr-hdr": {"type": "str"},
            "mm1-addr-source": {"choices": ["http-header", "cookie"], "type": "str"},
            "mm1-convert-hex": {"choices": ["disable", "enable"], "type": "str"},
            "mm1-outbreak-prevention": {"choices": ["disabled", "files", "full-archive"], "type": "str"},
            "mm1-retr-dupe": {"choices": ["disable", "enable"], "type": "str"},
            "mm1-retrieve-scan": {"choices": ["disable", "enable"], "type": "str"},
            "mm1comfortamount": {"type": "int"},
            "mm1comfortinterval": {"type": "int"},
            "mm1oversizelimit": {"type": "int"},
            "mm3": {
                "type": "list",
                "choices": [
                    "avmonitor", "block", "oversize", "quarantine", "scan", "avquery", "bannedword", "no-content-summary", "archive-summary",
                    "archive-full", "carrier-endpoint-bwl", "remove-blocked", "fragmail", "splice", "mms-checksum"
                ],
                "elements": "str"
            },
            "mm3-outbreak-prevention": {"choices": ["disabled", "files", "full-archive"], "type": "str"},
            "mm3oversizelimit": {"type": "int"},
            "mm4": {
                "type": "list",
                "choices": [
                    "avmonitor", "block", "oversize", "quarantine", "scan", "avquery", "bannedword", "no-content-summary", "archive-summary",
                    "archive-full", "carrier-endpoint-bwl", "remove-blocked", "fragmail", "splice", "mms-checksum"
                ],
                "elements": "str"
            },
            "mm4-outbreak-prevention": {"choices": ["disabled", "files", "full-archive"], "type": "str"},
            "mm4oversizelimit": {"type": "int"},
            "mm7": {
                "type": "list",
                "choices": [
                    "avmonitor", "block", "oversize", "quarantine", "scan", "avquery", "bannedword", "no-content-summary", "archive-summary",
                    "archive-full", "carrier-endpoint-bwl", "remove-blocked", "chunkedbypass", "clientcomfort", "servercomfort", "strict-file",
                    "mms-checksum"
                ],
                "elements": "str"
            },
            "mm7-addr-hdr": {"type": "str"},
            "mm7-addr-source": {"choices": ["http-header", "cookie"], "type": "str"},
            "mm7-convert-hex": {"choices": ["disable", "enable"], "type": "str"},
            "mm7-outbreak-prevention": {"choices": ["disabled", "files", "full-archive"], "type": "str"},
            "mm7comfortamount": {"type": "int"},
            "mm7comfortinterval": {"type": "int"},
            "mm7oversizelimit": {"type": "int"},
            "mms-antispam-mass-log": {"choices": ["disable", "enable"], "type": "str"},
            "mms-av-block-log": {"choices": ["disable", "enable"], "type": "str"},
            "mms-av-oversize-log": {"choices": ["disable", "enable"], "type": "str"},
            "mms-av-virus-log": {"choices": ["disable", "enable"], "type": "str"},
            "mms-carrier-endpoint-filter-log": {"choices": ["disable", "enable"], "type": "str"},
            "mms-checksum-log": {"choices": ["disable", "enable"], "type": "str"},
            "mms-checksum-table": {"type": "str"},
            "mms-notification-log": {"choices": ["disable", "enable"], "type": "str"},
            "mms-web-content-log": {"choices": ["disable", "enable"], "type": "str"},
            "mmsbwordthreshold": {"type": "int"},
            "name": {"required": true, "type": "str"},
            "notif-msisdn": {
                "type": "list",
                "options": {
                    "msisdn": {"type": "str"},
                    "threshold": {
                        "type": "list",
                        "choices": ["flood-thresh-1", "flood-thresh-2", "flood-thresh-3", "dupe-thresh-1", "dupe-thresh-2", "dupe-thresh-3"],
                        "elements": "str"
                    }
                },
                "elements": "dict"
            },
            "remove-blocked-const-length": {"choices": ["disable", "enable"], "type": "str"},
            "replacemsg-group": {"type": "str"},
            "dupe": {
                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                "type": "dict",
                "options": {
                    "action1": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "type": "list",
                        "choices": ["log", "archive", "intercept", "block", "archive-first", "alert-notif"],
                        "elements": "str"
                    },
                    "action2": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "type": "list",
                        "choices": ["log", "archive", "intercept", "block", "archive-first", "alert-notif"],
                        "elements": "str"
                    },
                    "action3": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "type": "list",
                        "choices": ["log", "archive", "intercept", "block", "archive-first", "alert-notif"],
                        "elements": "str"
                    },
                    "block-time1": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "block-time2": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "block-time3": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "limit1": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "limit2": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "limit3": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "protocol": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "status1": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "status2": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "status3": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "window1": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "window2": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "window3": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"}
                }
            },
            "flood": {
                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                "type": "dict",
                "options": {
                    "action1": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "type": "list",
                        "choices": ["log", "archive", "intercept", "block", "archive-first", "alert-notif"],
                        "elements": "str"
                    },
                    "action2": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "type": "list",
                        "choices": ["log", "archive", "intercept", "block", "archive-first", "alert-notif"],
                        "elements": "str"
                    },
                    "action3": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "type": "list",
                        "choices": ["log", "archive", "intercept", "block", "archive-first", "alert-notif"],
                        "elements": "str"
                    },
                    "block-time1": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "block-time2": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "block-time3": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "limit1": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "limit2": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "limit3": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "protocol": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "status1": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "status2": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "status3": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "window1": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "window2": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "window3": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"}
                }
            },
            "notification": {
                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                "type": "dict",
                "options": {
                    "alert-int": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "alert-int-mode": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["hours", "minutes"], "type": "str"},
                    "alert-src-msisdn": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "alert-status": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "bword-int": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "bword-int-mode": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["hours", "minutes"], "type": "str"},
                    "bword-status": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "carrier-endpoint-bwl-int": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "carrier-endpoint-bwl-int-mode": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["hours", "minutes"], "type": "str"},
                    "carrier-endpoint-bwl-status": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "days-allowed": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "type": "list",
                        "choices": ["sunday", "monday", "tuesday", "wednesday", "thursday", "friday", "saturday"],
                        "elements": "str"
                    },
                    "detect-server": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "dupe-int": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "dupe-int-mode": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["hours", "minutes"], "type": "str"},
                    "dupe-status": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "file-block-int": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "file-block-int-mode": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["hours", "minutes"], "type": "str"},
                    "file-block-status": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "flood-int": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "flood-int-mode": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["hours", "minutes"], "type": "str"},
                    "flood-status": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "from-in-header": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "mms-checksum-int": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "mms-checksum-int-mode": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["hours", "minutes"], "type": "str"},
                    "mms-checksum-status": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "mmsc-hostname": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "mmsc-password": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "no_log": true, "type": "raw"},
                    "mmsc-port": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "mmsc-url": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "mmsc-username": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "msg-protocol": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["mm1", "mm3", "mm4", "mm7"], "type": "str"},
                    "msg-type": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["submit-req", "deliver-req"], "type": "str"},
                    "protocol": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "rate-limit": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "tod-window-duration": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "tod-window-end": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", "7.2.0"]], "type": "str"},
                    "tod-window-start": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "user-domain": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "vas-id": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "vasp-id": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "virus-int": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "virus-int-mode": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["hours", "minutes"], "type": "str"},
                    "virus-status": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"}
                }
            },
            "outbreak-prevention": {
                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                "type": "dict",
                "options": {
                    "external-blocklist": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "ftgd-service": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"}
                }
            }
        }
    },
    "firewall_mmsprofile_dupe": {
        "stated": false,
        "options": {
            "action1": {"type": "list", "choices": ["log", "archive", "intercept", "block", "archive-first", "alert-notif"], "elements": "str"},
            "action2": {"type": "list", "choices": ["log", "archive", "intercept", "block", "archive-first", "alert-notif"], "elements": "str"},
            "action3": {"type": "list", "choices": ["log", "archive", "intercept", "block", "archive-first", "alert-notif"], "elements": "str"},
            "block-time1": {"type": "int"},
            "block-time2": {"type": "int"},
            "block-time3": {"type": "int"},
            "limit1": {"type": "int"},
            "limit2": {"type": "int"},
            "limit3": {"type": "int"},
            "protocol": {"type": "str"},
            "status1": {"choices": ["disable", "enable"], "type": "str"},
            "status2": {"choices": ["disable", "enable"], "type": "str"},
            "status3": {"choices": ["disable", "enable"], "type": "str"},
            "window1": {"type": "int"},
            "window2": {"type": "int"},
            "window3": {"type": "int"}
        }
    },
    "firewall_mmsprofile_flood": {
        "stated": false,
        "options": {
            "action1": {"type": "list", "choices": ["log", "archive", "intercept", "block", "archive-first", "alert-notif"], "elements": "str"},
            "action2": {"type": "list", "choices": ["log", "archive", "intercept", "block", "archive-first", "alert-notif"], "elements": "str"},
            "action3": {"type": "list", "choices": ["log", "archive", "intercept", "block", "archive-first", "alert-notif"], "elements": "str"},
            "block-time1": {"type": "int"},
            "block-time2": {"type": "int"},
            "block-time3": {"type": "int"},
            "limit1": {"type": "int"},
            "limit2": {"type": "int"},
            "limit3": {"type": "int"},
            "protocol": {"type": "str"},
            "status1": {"choices": ["disable", "enable"], "type": "str"},
            "status2": {"choices": ["disable", "enable"], "type": "str"},
            "status3": {"choices": ["disable", "enable"], "type": "str"},
            "window1": {"type": "int"},
            "window2": {"type": "int"},
            "window3": {"type": "int"}
        }
    },
    "firewall_mmsprofile_notification": {
        "stated": false,
        "options": {
            "alert-int": {"type": "int"},
            "alert-int-mode": {"choices": ["hours", "minutes"], "type": "str"},
            "alert-src-msisdn": {"type": "str"},
            "alert-status": {"choices": ["disable", "enable"], "type": "str"},
            "bword-int": {"type": "int"},
            "bword-int-mode": {"choices": ["hours", "minutes"], "type": "str"},
            "bword-status": {"choices": ["disable", "enable"], "type": "str"},
            "carrier-endpoint-bwl-int": {"type": "int"},
            "carrier-endpoint-bwl-int-mode": {"choices": ["hours", "minutes"], "type": "str"},
            "carrier-endpoint-bwl-status": {"choices": ["disable", "enable"], "type": "str"},
            "days-allowed": {"type": "list", "choices": ["sunday", "monday", "tuesday", "wednesday", "thursday", "friday", "saturday"], "elements": "str"},
            "detect-server": {"choices": ["disable", "enable"], "type": "str"},
            "dupe-int": {"type": "int"},
            "dupe-int-mode": {"choices": ["hours", "minutes"], "type": "str"},
            "dupe-status": {"choices": ["disable", "enable"], "type": "str"},
            "file-block-int": {"type": "int"},
            "file-block-int-mode": {"choices": ["hours", "minutes"], "type": "str"},
            "file-block-status": {"choices": ["disable", "enable"], "type": "str"},
            "flood-int": {"type": "int"},
            "flood-int-mode": {"choices": ["hours", "minutes"], "type": "str"},
            "flood-status": {"choices": ["disable", "enable"], "type": "str"},
            "from-in-header": {"choices": ["disable", "enable"], "type": "str"},
            "mms-checksum-int": {"type": "int"},
            "mms-checksum-int-mode": {"choices": ["hours", "minutes"], "type": "str"},
            "mms-checksum-status": {"choices": ["disable", "enable"], "type": "str"},
            "mmsc-hostname": {"type": "str"},
            "mmsc-password": {"no_log": true, "type": "raw"},
            "mmsc-port": {"type": "int"},
            "mmsc-url": {"type": "str"},
            "mmsc-username": {"type": "str"},
            "msg-protocol": {"choices": ["mm1", "mm3", "mm4", "mm7"], "type": "str"},
            "msg-type": {"choices": ["submit-req", "deliver-req"], "type": "str"},
            "protocol": {"type": "str"},
            "rate-limit": {"type": "int"},
            "tod-window-duration": {"type": "str"},
            "tod-window-end": {"v_range": [["6.0.0", "7.2.0"]], "type": "str"},
            "tod-window-start": {"type": "str"},
            "user-domain": {"type": "str"},
            "vas-id": {"type": "str"},
            "vasp-id": {"type": "str"},
            "virus-int": {"type": "int"},
            "virus-int-mode": {"choices": ["hours", "minutes"], "type": "str"},
            "virus-status": {"choices": ["disable", "enable"], "type": "str"}
        }
    },
    "firewall_mmsprofile_notifmsisdn": {
        "stated": true,
        "options": {
            "msisdn": {"required": true, "type": "str"},
            "threshold": {
                "type": "list",
                "choices": ["flood-thresh-1", "flood-thresh-2", "flood-thresh-3", "dupe-thresh-1", "dupe-thresh-2", "dupe-thresh-3"],
                "elements": "str"
            }
        }
    },
    "firewall_mmsprofile_outbreakprevention": {
        "stated": false,
        "options": {
            "external-blocklist": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ftgd-service": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "firewall_multicastaddress": {
        "stated": true,
        "options": {
            "associated-interface": {"type": "str"},
            "color": {"type": "int"},
            "comment": {"type": "str"},
            "end-ip": {"type": "str"},
            "name": {"required": true, "type": "str"},
            "start-ip": {"type": "str"},
            "subnet": {"type": "str"},
            "tagging": {"type": "list", "options": {"category": {"type": "str"}, "name": {"type": "str"}, "tags": {"type": "raw"}}, "elements": "dict"},
            "type": {"choices": ["multicastrange", "broadcastmask"], "type": "str"},
            "visibility": {"choices": ["disable", "enable"], "type": "str"},
            "tags": {"v_range": [["6.2.0", "6.4.14"]], "type": "str"}
        }
    },
    "firewall_multicastaddress6": {
        "stated": true,
        "options": {
            "color": {"type": "int"},
            "comment": {"type": "str"},
            "ip6": {"type": "str"},
            "name": {"required": true, "type": "str"},
            "tagging": {"type": "list", "options": {"category": {"type": "str"}, "name": {"type": "str"}, "tags": {"type": "raw"}}, "elements": "dict"},
            "visibility": {"choices": ["disable", "enable"], "type": "str"},
            "tags": {"v_range": [["6.2.0", "6.4.14"]], "type": "str"}
        }
    },
    "firewall_multicastaddress6_tagging": {
        "stated": true,
        "options": {"category": {"type": "str"}, "name": {"required": true, "type": "str"}, "tags": {"type": "raw"}}
    },
    "firewall_multicastaddress_tagging": {
        "stated": true,
        "options": {"category": {"type": "str"}, "name": {"required": true, "type": "str"}, "tags": {"type": "raw"}}
    },
    "firewall_networkservicedynamic": {
        "stated": true,
        "options": {
            "comment": {"v_range": [["7.2.2", ""]], "type": "str"},
            "filter": {"v_range": [["7.2.2", ""]], "type": "str"},
            "id": {"v_range": [["7.2.2", ""]], "required": true, "type": "int"},
            "name": {"v_range": [["7.2.2", ""]], "type": "str"},
            "sdn": {"v_range": [["7.2.2", ""]], "type": "str"}
        }
    },
    "firewall_profilegroup": {
        "stated": true,
        "options": {
            "application-list": {"type": "str"},
            "av-profile": {"type": "str"},
            "dlp-sensor": {"type": "str"},
            "dnsfilter-profile": {"type": "str"},
            "icap-profile": {"type": "str"},
            "ips-sensor": {"type": "str"},
            "mms-profile": {"type": "str"},
            "name": {"required": true, "type": "str"},
            "profile-protocol-options": {"type": "str"},
            "spamfilter-profile": {"v_range": [["6.0.0", "7.2.1"]], "type": "str"},
            "ssh-filter-profile": {"type": "str"},
            "ssl-ssh-profile": {"type": "str"},
            "voip-profile": {"type": "str"},
            "waf-profile": {"type": "str"},
            "webfilter-profile": {"type": "str"},
            "cifs-profile": {"v_range": [["6.2.0", ""]], "type": "str"},
            "emailfilter-profile": {"v_range": [["6.2.0", ""]], "type": "str"},
            "casi-profile": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"},
            "file-filter-profile": {"v_range": [["6.4.1", ""]], "type": "str"},
            "videofilter-profile": {"v_range": [["7.0.0", ""]], "type": "str"},
            "dlp-profile": {"v_range": [["7.2.0", ""]], "type": "str"},
            "sctp-filter-profile": {"v_range": [["7.0.1", ""]], "type": "str"},
            "ips-voip-filter": {"v_range": [["7.2.3", ""]], "type": "str"},
            "casb-profile": {"v_range": [["7.4.1", ""]], "type": "str"},
            "virtual-patch-profile": {"v_range": [["7.4.1", ""]], "type": "str"},
            "diameter-filter-profile": {"v_range": [["7.4.2", ""]], "type": "str"}
        }
    },
    "firewall_profileprotocoloptions": {
        "stated": true,
        "options": {
            "comment": {"type": "str"},
            "name": {"required": true, "type": "str"},
            "oversize-log": {"choices": ["disable", "enable"], "type": "str"},
            "replacemsg-group": {"type": "str"},
            "rpc-over-http": {"choices": ["disable", "enable"], "type": "str"},
            "switching-protocols-log": {"choices": ["disable", "enable"], "type": "str"},
            "feature-set": {"v_range": [["6.4.0", ""]], "choices": ["proxy", "flow"], "type": "str"},
            "cifs": {
                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                "type": "dict",
                "options": {
                    "domain-controller": {"v_range": [["6.4.5", ""]], "type": "str"},
                    "file-filter": {
                        "v_range": [["6.4.5", ""]],
                        "type": "dict",
                        "options": {
                            "entries": {
                                "v_range": [["6.4.5", ""]],
                                "type": "list",
                                "options": {
                                    "action": {"v_range": [["6.4.5", ""]], "choices": ["log", "block"], "type": "str"},
                                    "comment": {"v_range": [["6.4.5", ""]], "type": "str"},
                                    "direction": {"v_range": [["6.4.5", ""]], "choices": ["any", "incoming", "outgoing"], "type": "str"},
                                    "file-type": {"v_range": [["6.4.5", ""]], "type": "raw"},
                                    "filter": {"v_range": [["6.4.5", ""]], "type": "str"},
                                    "protocol": {"v_range": [["6.4.5", ""]], "type": "list", "choices": ["cifs"], "elements": "str"}
                                },
                                "elements": "dict"
                            },
                            "log": {"v_range": [["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "status": {"v_range": [["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"}
                        }
                    },
                    "options": {"v_range": [["6.4.5", ""]], "type": "list", "choices": ["oversize"], "elements": "str"},
                    "oversize-limit": {"v_range": [["6.4.5", ""]], "type": "int"},
                    "ports": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "raw"},
                    "scan-bzip2": {"v_range": [["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "server-credential-type": {
                        "v_range": [["6.4.5", ""]],
                        "choices": ["none", "credential-replication", "credential-keytab"],
                        "type": "str"
                    },
                    "server-keytab": {
                        "v_range": [["6.4.5", ""]],
                        "no_log": true,
                        "type": "list",
                        "options": {
                            "keytab": {"v_range": [["6.4.5", ""]], "no_log": true, "type": "str"},
                            "password": {"v_range": [["6.4.5", ""]], "no_log": true, "type": "raw"},
                            "principal": {"v_range": [["6.4.5", ""]], "type": "str"}
                        },
                        "elements": "dict"
                    },
                    "status": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "tcp-window-maximum": {"v_range": [["6.4.5", ""]], "type": "int"},
                    "tcp-window-minimum": {"v_range": [["6.4.5", ""]], "type": "int"},
                    "tcp-window-size": {"v_range": [["6.4.5", ""]], "type": "int"},
                    "tcp-window-type": {"v_range": [["6.4.5", ""]], "choices": ["system", "static", "dynamic", "auto-tuning"], "type": "str"},
                    "uncompressed-nest-limit": {"v_range": [["6.4.5", ""]], "type": "int"},
                    "uncompressed-oversize-limit": {"v_range": [["6.4.5", ""]], "type": "int"}
                }
            },
            "dns": {
                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                "type": "dict",
                "options": {
                    "ports": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "raw"},
                    "status": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"}
                }
            },
            "ftp": {
                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                "type": "dict",
                "options": {
                    "comfort-amount": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "comfort-interval": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "inspect-all": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "options": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "type": "list",
                        "choices": ["clientcomfort", "no-content-summary", "oversize", "splice", "bypass-rest-command", "bypass-mode-command"],
                        "elements": "str"
                    },
                    "oversize-limit": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "ports": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "raw"},
                    "scan-bzip2": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "ssl-offloaded": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["no", "yes"], "type": "str"},
                    "status": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "uncompressed-nest-limit": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "uncompressed-oversize-limit": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "stream-based-uncompressed-limit": {"v_range": [["7.0.0", ""]], "type": "int"},
                    "tcp-window-maximum": {"v_range": [["7.0.0", ""]], "type": "int"},
                    "tcp-window-minimum": {"v_range": [["7.0.0", ""]], "type": "int"},
                    "tcp-window-size": {"v_range": [["7.0.0", ""]], "type": "int"},
                    "tcp-window-type": {"v_range": [["7.0.0", ""]], "choices": ["system", "static", "dynamic", "auto-tuning"], "type": "str"},
                    "explicit-ftp-tls": {"v_range": [["7.0.5", "7.0.12"], ["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"}
                }
            },
            "http": {
                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                "type": "dict",
                "options": {
                    "block-page-status-code": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "comfort-amount": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "comfort-interval": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "fortinet-bar": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "fortinet-bar-port": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "inspect-all": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "options": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "type": "list",
                        "choices": ["oversize", "chunkedbypass", "clientcomfort", "no-content-summary", "servercomfort"],
                        "elements": "str"
                    },
                    "oversize-limit": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "ports": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "raw"},
                    "post-lang": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "type": "list",
                        "choices": [
                            "jisx0201", "jisx0208", "jisx0212", "gb2312", "ksc5601-ex", "euc-jp", "sjis", "iso2022-jp", "iso2022-jp-1", "iso2022-jp-2",
                            "euc-cn", "ces-gbk", "hz", "ces-big5", "euc-kr", "iso2022-jp-3", "iso8859-1", "tis620", "cp874", "cp1252", "cp1251"
                        ],
                        "elements": "str"
                    },
                    "proxy-after-tcp-handshake": {"v_range": [["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "range-block": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "retry-count": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "scan-bzip2": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "ssl-offloaded": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["no", "yes"], "type": "str"},
                    "status": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "stream-based-uncompressed-limit": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "streaming-content-bypass": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "strip-x-forwarded-for": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "switching-protocols": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["bypass", "block"], "type": "str"},
                    "tcp-window-maximum": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "tcp-window-minimum": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "tcp-window-size": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "tcp-window-type": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "choices": ["system", "static", "dynamic", "auto-tuning"],
                        "type": "str"
                    },
                    "tunnel-non-http": {"v_range": [["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "uncompressed-nest-limit": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "uncompressed-oversize-limit": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "unknown-http-version": {"v_range": [["6.4.5", ""]], "choices": ["best-effort", "reject", "tunnel"], "type": "str"},
                    "http-policy": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", "7.2.1"]], "choices": ["disable", "enable"], "type": "str"},
                    "address-ip-rating": {"v_range": [["7.0.4", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "h2c": {"v_range": [["7.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "verify-dns-for-policy-matching": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "unknown-content-encoding": {"v_range": [["7.2.2", ""]], "choices": ["block", "inspect", "bypass"], "type": "str"},
                    "domain-fronting": {"v_range": [["7.6.0", ""]], "choices": ["block", "monitor", "allow"], "type": "str"}
                }
            },
            "imap": {
                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                "type": "dict",
                "options": {
                    "inspect-all": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "options": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "type": "list",
                        "choices": ["oversize", "fragmail", "no-content-summary"],
                        "elements": "str"
                    },
                    "oversize-limit": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "ports": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "raw"},
                    "proxy-after-tcp-handshake": {"v_range": [["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "scan-bzip2": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "ssl-offloaded": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["no", "yes"], "type": "str"},
                    "status": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "uncompressed-nest-limit": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "uncompressed-oversize-limit": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"}
                }
            },
            "mail-signature": {
                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                "type": "dict",
                "options": {
                    "signature": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "status": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"}
                }
            },
            "mapi": {
                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                "type": "dict",
                "options": {
                    "options": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "type": "list",
                        "choices": ["fragmail", "oversize", "no-content-summary"],
                        "elements": "str"
                    },
                    "oversize-limit": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "ports": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "raw"},
                    "scan-bzip2": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "status": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "uncompressed-nest-limit": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "uncompressed-oversize-limit": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"}
                }
            },
            "nntp": {
                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                "type": "dict",
                "options": {
                    "inspect-all": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "options": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "type": "list",
                        "choices": ["oversize", "no-content-summary", "splice"],
                        "elements": "str"
                    },
                    "oversize-limit": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "ports": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "raw"},
                    "proxy-after-tcp-handshake": {"v_range": [["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "scan-bzip2": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "status": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "uncompressed-nest-limit": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "uncompressed-oversize-limit": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"}
                }
            },
            "pop3": {
                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                "type": "dict",
                "options": {
                    "inspect-all": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "options": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "type": "list",
                        "choices": ["oversize", "fragmail", "no-content-summary"],
                        "elements": "str"
                    },
                    "oversize-limit": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "ports": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "raw"},
                    "proxy-after-tcp-handshake": {"v_range": [["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "scan-bzip2": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "ssl-offloaded": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["no", "yes"], "type": "str"},
                    "status": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "uncompressed-nest-limit": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "uncompressed-oversize-limit": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"}
                }
            },
            "smtp": {
                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                "type": "dict",
                "options": {
                    "inspect-all": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "options": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "type": "list",
                        "choices": ["oversize", "fragmail", "no-content-summary", "splice"],
                        "elements": "str"
                    },
                    "oversize-limit": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "ports": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "raw"},
                    "proxy-after-tcp-handshake": {"v_range": [["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "scan-bzip2": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "server-busy": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "ssl-offloaded": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["no", "yes"], "type": "str"},
                    "status": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "uncompressed-nest-limit": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "uncompressed-oversize-limit": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"}
                }
            },
            "ssh": {
                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                "type": "dict",
                "options": {
                    "comfort-amount": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "comfort-interval": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "options": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "type": "list",
                        "choices": ["oversize", "clientcomfort", "servercomfort"],
                        "elements": "str"
                    },
                    "oversize-limit": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "scan-bzip2": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "uncompressed-nest-limit": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "uncompressed-oversize-limit": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "ssl-offloaded": {"v_range": [["7.0.0", ""]], "choices": ["no", "yes"], "type": "str"},
                    "stream-based-uncompressed-limit": {"v_range": [["7.0.0", ""]], "type": "int"},
                    "tcp-window-maximum": {"v_range": [["7.0.0", ""]], "type": "int"},
                    "tcp-window-minimum": {"v_range": [["7.0.0", ""]], "type": "int"},
                    "tcp-window-size": {"v_range": [["7.0.0", ""]], "type": "int"},
                    "tcp-window-type": {"v_range": [["7.0.0", ""]], "choices": ["system", "static", "dynamic", "auto-tuning"], "type": "str"}
                }
            }
        }
    },
    "firewall_profileprotocoloptions_cifs": {
        "stated": false,
        "options": {
            "ports": {"v_range": [["6.2.0", ""]], "type": "raw"},
            "status": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "options": {"v_range": [["6.4.0", ""]], "type": "list", "choices": ["oversize"], "elements": "str"},
            "oversize-limit": {"v_range": [["6.4.0", ""]], "type": "int"},
            "scan-bzip2": {"v_range": [["6.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "tcp-window-maximum": {"v_range": [["6.4.0", ""]], "type": "int"},
            "tcp-window-minimum": {"v_range": [["6.4.0", ""]], "type": "int"},
            "tcp-window-size": {"v_range": [["6.4.0", ""]], "type": "int"},
            "tcp-window-type": {"v_range": [["6.4.0", ""]], "choices": ["system", "static", "dynamic", "auto-tuning"], "type": "str"},
            "uncompressed-nest-limit": {"v_range": [["6.4.0", ""]], "type": "int"},
            "uncompressed-oversize-limit": {"v_range": [["6.4.0", ""]], "type": "int"},
            "domain-controller": {"v_range": [["6.4.2", ""]], "type": "str"},
            "file-filter": {
                "v_range": [["6.4.2", ""]],
                "type": "dict",
                "options": {
                    "entries": {
                        "v_range": [["6.4.2", ""]],
                        "type": "list",
                        "options": {
                            "action": {"v_range": [["6.4.2", ""]], "choices": ["log", "block"], "type": "str"},
                            "comment": {"v_range": [["6.4.2", ""]], "type": "str"},
                            "direction": {"v_range": [["6.4.2", ""]], "choices": ["any", "incoming", "outgoing"], "type": "str"},
                            "file-type": {"v_range": [["6.4.2", ""]], "type": "raw"},
                            "filter": {"v_range": [["6.4.2", ""]], "type": "str"},
                            "protocol": {"v_range": [["6.4.2", ""]], "type": "list", "choices": ["cifs"], "elements": "str"}
                        },
                        "elements": "dict"
                    },
                    "log": {"v_range": [["6.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "status": {"v_range": [["6.4.2", ""]], "choices": ["disable", "enable"], "type": "str"}
                }
            },
            "server-credential-type": {"v_range": [["6.4.2", ""]], "choices": ["none", "credential-replication", "credential-keytab"], "type": "str"},
            "server-keytab": {
                "v_range": [["6.4.2", ""]],
                "no_log": true,
                "type": "list",
                "options": {
                    "keytab": {"v_range": [["6.4.2", ""]], "no_log": true, "type": "str"},
                    "password": {"v_range": [["6.4.2", ""]], "no_log": true, "type": "raw"},
                    "principal": {"v_range": [["6.4.2", ""]], "type": "str"}
                },
                "elements": "dict"
            }
        }
    },
    "firewall_profileprotocoloptions_cifs_filefilter": {
        "stated": false,
        "options": {
            "entries": {
                "v_range": [["6.4.2", ""]],
                "type": "list",
                "options": {
                    "action": {"v_range": [["6.4.2", ""]], "choices": ["log", "block"], "type": "str"},
                    "comment": {"v_range": [["6.4.2", ""]], "type": "str"},
                    "direction": {"v_range": [["6.4.2", ""]], "choices": ["any", "incoming", "outgoing"], "type": "str"},
                    "file-type": {"v_range": [["6.4.2", ""]], "type": "raw"},
                    "filter": {"v_range": [["6.4.2", ""]], "type": "str"},
                    "protocol": {"v_range": [["6.4.2", ""]], "type": "list", "choices": ["cifs"], "elements": "str"}
                },
                "elements": "dict"
            },
            "log": {"v_range": [["6.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "status": {"v_range": [["6.4.2", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "firewall_profileprotocoloptions_cifs_filefilter_entries": {
        "stated": true,
        "options": {
            "action": {"v_range": [["6.4.2", ""]], "choices": ["log", "block"], "type": "str"},
            "comment": {"v_range": [["6.4.2", ""]], "type": "str"},
            "direction": {"v_range": [["6.4.2", ""]], "choices": ["any", "incoming", "outgoing"], "type": "str"},
            "file-type": {"v_range": [["6.4.2", ""]], "type": "raw"},
            "filter": {"v_range": [["6.4.2", ""]], "type": "str"},
            "protocol": {"v_range": [["6.4.2", ""]], "type": "list", "choices": ["cifs"], "elements": "str"}
        }
    },
    "firewall_profileprotocoloptions_cifs_serverkeytab": {
        "stated": true,
        "options": {
            "keytab": {"v_range": [["6.4.2", ""]], "no_log": true, "type": "str"},
            "password": {"v_range": [["6.4.2", ""]], "no_log": true, "type": "raw"},
            "principal": {"v_range": [["6.4.2", ""]], "type": "str"}
        }
    },
    "firewall_profileprotocoloptions_dns": {
        "stated": false,
        "options": {"ports": {"type": "raw"}, "status": {"choices": ["disable", "enable"], "type": "str"}}
    },
    "firewall_profileprotocoloptions_ftp": {
        "stated": false,
        "options": {
            "comfort-amount": {"type": "int"},
            "comfort-interval": {"type": "int"},
            "inspect-all": {"choices": ["disable", "enable"], "type": "str"},
            "options": {
                "type": "list",
                "choices": ["clientcomfort", "no-content-summary", "oversize", "splice", "bypass-rest-command", "bypass-mode-command"],
                "elements": "str"
            },
            "oversize-limit": {"type": "int"},
            "ports": {"type": "raw"},
            "scan-bzip2": {"choices": ["disable", "enable"], "type": "str"},
            "status": {"choices": ["disable", "enable"], "type": "str"},
            "uncompressed-nest-limit": {"type": "int"},
            "uncompressed-oversize-limit": {"type": "int"},
            "ssl-offloaded": {"v_range": [["6.2.2", ""]], "choices": ["no", "yes"], "type": "str"},
            "stream-based-uncompressed-limit": {"v_range": [["7.0.0", ""]], "type": "int"},
            "tcp-window-maximum": {"v_range": [["7.0.0", ""]], "type": "int"},
            "tcp-window-minimum": {"v_range": [["7.0.0", ""]], "type": "int"},
            "tcp-window-size": {"v_range": [["7.0.0", ""]], "type": "int"},
            "tcp-window-type": {"v_range": [["7.0.0", ""]], "choices": ["system", "static", "dynamic", "auto-tuning"], "type": "str"},
            "explicit-ftp-tls": {"v_range": [["7.0.5", "7.0.12"], ["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "firewall_profileprotocoloptions_http": {
        "stated": false,
        "options": {
            "block-page-status-code": {"type": "int"},
            "comfort-amount": {"type": "int"},
            "comfort-interval": {"type": "int"},
            "fortinet-bar": {"choices": ["disable", "enable"], "type": "str"},
            "fortinet-bar-port": {"type": "int"},
            "http-policy": {"v_range": [["6.0.0", "7.2.1"]], "choices": ["disable", "enable"], "type": "str"},
            "inspect-all": {"choices": ["disable", "enable"], "type": "str"},
            "options": {
                "type": "list",
                "choices": ["oversize", "chunkedbypass", "clientcomfort", "no-content-summary", "servercomfort"],
                "elements": "str"
            },
            "oversize-limit": {"type": "int"},
            "ports": {"type": "raw"},
            "post-lang": {
                "type": "list",
                "choices": [
                    "jisx0201", "jisx0208", "jisx0212", "gb2312", "ksc5601-ex", "euc-jp", "sjis", "iso2022-jp", "iso2022-jp-1", "iso2022-jp-2", "euc-cn",
                    "ces-gbk", "hz", "ces-big5", "euc-kr", "iso2022-jp-3", "iso8859-1", "tis620", "cp874", "cp1252", "cp1251"
                ],
                "elements": "str"
            },
            "range-block": {"choices": ["disable", "enable"], "type": "str"},
            "retry-count": {"type": "int"},
            "scan-bzip2": {"choices": ["disable", "enable"], "type": "str"},
            "status": {"choices": ["disable", "enable"], "type": "str"},
            "streaming-content-bypass": {"choices": ["disable", "enable"], "type": "str"},
            "strip-x-forwarded-for": {"choices": ["disable", "enable"], "type": "str"},
            "switching-protocols": {"choices": ["bypass", "block"], "type": "str"},
            "uncompressed-nest-limit": {"type": "int"},
            "uncompressed-oversize-limit": {"type": "int"},
            "tcp-window-maximum": {"v_range": [["6.2.0", ""]], "type": "int"},
            "tcp-window-minimum": {"v_range": [["6.2.0", ""]], "type": "int"},
            "tcp-window-size": {"v_range": [["6.2.0", ""]], "type": "int"},
            "tcp-window-type": {"v_range": [["6.2.0", ""]], "choices": ["system", "static", "dynamic", "auto-tuning"], "type": "str"},
            "ssl-offloaded": {"v_range": [["6.2.2", ""]], "choices": ["no", "yes"], "type": "str"},
            "stream-based-uncompressed-limit": {"v_range": [["6.2.2", ""]], "type": "int"},
            "proxy-after-tcp-handshake": {"v_range": [["6.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "tunnel-non-http": {"v_range": [["6.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "unknown-http-version": {"v_range": [["6.4.0", ""]], "choices": ["best-effort", "reject", "tunnel"], "type": "str"},
            "address-ip-rating": {"v_range": [["7.0.4", ""]], "choices": ["disable", "enable"], "type": "str"},
            "h2c": {"v_range": [["7.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "verify-dns-for-policy-matching": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "unknown-content-encoding": {"v_range": [["7.2.2", ""]], "choices": ["block", "inspect", "bypass"], "type": "str"},
            "domain-fronting": {"v_range": [["7.6.0", ""]], "choices": ["block", "monitor", "allow"], "type": "str"}
        }
    },
    "firewall_profileprotocoloptions_imap": {
        "stated": false,
        "options": {
            "inspect-all": {"choices": ["disable", "enable"], "type": "str"},
            "options": {"type": "list", "choices": ["oversize", "fragmail", "no-content-summary"], "elements": "str"},
            "oversize-limit": {"type": "int"},
            "ports": {"type": "raw"},
            "scan-bzip2": {"choices": ["disable", "enable"], "type": "str"},
            "status": {"choices": ["disable", "enable"], "type": "str"},
            "uncompressed-nest-limit": {"type": "int"},
            "uncompressed-oversize-limit": {"type": "int"},
            "ssl-offloaded": {"v_range": [["6.2.2", ""]], "choices": ["no", "yes"], "type": "str"},
            "proxy-after-tcp-handshake": {"v_range": [["6.4.0", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "firewall_profileprotocoloptions_mailsignature": {
        "stated": false,
        "options": {"signature": {"type": "str"}, "status": {"choices": ["disable", "enable"], "type": "str"}}
    },
    "firewall_profileprotocoloptions_mapi": {
        "stated": false,
        "options": {
            "options": {"type": "list", "choices": ["fragmail", "oversize", "no-content-summary"], "elements": "str"},
            "oversize-limit": {"type": "int"},
            "ports": {"type": "raw"},
            "scan-bzip2": {"choices": ["disable", "enable"], "type": "str"},
            "status": {"choices": ["disable", "enable"], "type": "str"},
            "uncompressed-nest-limit": {"type": "int"},
            "uncompressed-oversize-limit": {"type": "int"}
        }
    },
    "firewall_profileprotocoloptions_nntp": {
        "stated": false,
        "options": {
            "inspect-all": {"choices": ["disable", "enable"], "type": "str"},
            "options": {"type": "list", "choices": ["oversize", "no-content-summary", "splice"], "elements": "str"},
            "oversize-limit": {"type": "int"},
            "ports": {"type": "raw"},
            "scan-bzip2": {"choices": ["disable", "enable"], "type": "str"},
            "status": {"choices": ["disable", "enable"], "type": "str"},
            "uncompressed-nest-limit": {"type": "int"},
            "uncompressed-oversize-limit": {"type": "int"},
            "proxy-after-tcp-handshake": {"v_range": [["6.4.0", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "firewall_profileprotocoloptions_pop3": {
        "stated": false,
        "options": {
            "inspect-all": {"choices": ["disable", "enable"], "type": "str"},
            "options": {"type": "list", "choices": ["oversize", "fragmail", "no-content-summary"], "elements": "str"},
            "oversize-limit": {"type": "int"},
            "ports": {"type": "raw"},
            "scan-bzip2": {"choices": ["disable", "enable"], "type": "str"},
            "status": {"choices": ["disable", "enable"], "type": "str"},
            "uncompressed-nest-limit": {"type": "int"},
            "uncompressed-oversize-limit": {"type": "int"},
            "ssl-offloaded": {"v_range": [["6.2.2", ""]], "choices": ["no", "yes"], "type": "str"},
            "proxy-after-tcp-handshake": {"v_range": [["6.4.0", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "firewall_profileprotocoloptions_smtp": {
        "stated": false,
        "options": {
            "inspect-all": {"choices": ["disable", "enable"], "type": "str"},
            "options": {"type": "list", "choices": ["oversize", "fragmail", "no-content-summary", "splice"], "elements": "str"},
            "oversize-limit": {"type": "int"},
            "ports": {"type": "raw"},
            "scan-bzip2": {"choices": ["disable", "enable"], "type": "str"},
            "server-busy": {"choices": ["disable", "enable"], "type": "str"},
            "status": {"choices": ["disable", "enable"], "type": "str"},
            "uncompressed-nest-limit": {"type": "int"},
            "uncompressed-oversize-limit": {"type": "int"},
            "ssl-offloaded": {"v_range": [["6.2.2", ""]], "choices": ["no", "yes"], "type": "str"},
            "proxy-after-tcp-handshake": {"v_range": [["6.4.0", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "firewall_profileprotocoloptions_ssh": {
        "stated": false,
        "options": {
            "comfort-amount": {"v_range": [["6.2.2", ""]], "type": "int"},
            "comfort-interval": {"v_range": [["6.2.2", ""]], "type": "int"},
            "options": {"v_range": [["6.2.2", ""]], "type": "list", "choices": ["oversize", "clientcomfort", "servercomfort"], "elements": "str"},
            "oversize-limit": {"v_range": [["6.2.2", ""]], "type": "int"},
            "scan-bzip2": {"v_range": [["6.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "uncompressed-nest-limit": {"v_range": [["6.2.2", ""]], "type": "int"},
            "uncompressed-oversize-limit": {"v_range": [["6.2.2", ""]], "type": "int"},
            "ssl-offloaded": {"v_range": [["7.0.0", ""]], "choices": ["no", "yes"], "type": "str"},
            "stream-based-uncompressed-limit": {"v_range": [["7.0.0", ""]], "type": "int"},
            "tcp-window-maximum": {"v_range": [["7.0.0", ""]], "type": "int"},
            "tcp-window-minimum": {"v_range": [["7.0.0", ""]], "type": "int"},
            "tcp-window-size": {"v_range": [["7.0.0", ""]], "type": "int"},
            "tcp-window-type": {"v_range": [["7.0.0", ""]], "choices": ["system", "static", "dynamic", "auto-tuning"], "type": "str"}
        }
    },
    "firewall_proxyaddress": {
        "stated": true,
        "options": {
            "case-sensitivity": {"choices": ["disable", "enable"], "type": "str"},
            "category": {"type": "raw"},
            "color": {"type": "int"},
            "comment": {"type": "str"},
            "header": {"type": "str"},
            "header-group": {
                "type": "list",
                "options": {
                    "case-sensitivity": {"choices": ["disable", "enable"], "type": "str"},
                    "header": {"type": "str"},
                    "header-name": {"type": "str"},
                    "id": {"type": "int"}
                },
                "elements": "dict"
            },
            "header-name": {"type": "str"},
            "host": {"type": "str"},
            "host-regex": {"type": "str"},
            "method": {"type": "list", "choices": ["delete", "get", "head", "options", "post", "put", "trace", "connect"], "elements": "str"},
            "name": {"required": true, "type": "str"},
            "path": {"type": "str"},
            "query": {"type": "str"},
            "referrer": {"choices": ["disable", "enable"], "type": "str"},
            "tagging": {"type": "list", "options": {"category": {"type": "str"}, "name": {"type": "str"}, "tags": {"type": "raw"}}, "elements": "dict"},
            "type": {
                "choices": ["host-regex", "url", "category", "method", "ua", "header", "src-advanced", "dst-advanced", "url-list", "saas"],
                "type": "str"
            },
            "ua": {"type": "list", "choices": ["chrome", "ms", "firefox", "safari", "other", "ie", "edge"], "elements": "str"},
            "uuid": {"type": "str"},
            "visibility": {"choices": ["disable", "enable"], "type": "str"},
            "tags": {"v_range": [["6.2.0", "6.4.14"]], "type": "str"},
            "_image-base64": {"v_range": [["6.2.2", ""]], "type": "str"},
            "application": {"v_range": [["7.2.1", ""]], "type": "raw"},
            "ua-max-ver": {"v_range": [["7.2.2", ""]], "type": "str"},
            "ua-min-ver": {"v_range": [["7.2.2", ""]], "type": "str"}
        }
    },
    "firewall_proxyaddress_headergroup": {
        "stated": true,
        "options": {
            "case-sensitivity": {"choices": ["disable", "enable"], "type": "str"},
            "header": {"type": "str"},
            "header-name": {"type": "str"},
            "id": {"required": true, "type": "int"}
        }
    },
    "firewall_proxyaddress_tagging": {
        "stated": true,
        "options": {"category": {"type": "str"}, "name": {"required": true, "type": "str"}, "tags": {"type": "raw"}}
    },
    "firewall_proxyaddrgrp": {
        "stated": true,
        "options": {
            "color": {"type": "int"},
            "comment": {"type": "str"},
            "member": {"type": "raw"},
            "name": {"required": true, "type": "str"},
            "tagging": {"type": "list", "options": {"category": {"type": "str"}, "name": {"type": "str"}, "tags": {"type": "raw"}}, "elements": "dict"},
            "type": {"choices": ["src", "dst"], "type": "str"},
            "uuid": {"type": "str"},
            "visibility": {"choices": ["disable", "enable"], "type": "str"},
            "tags": {"v_range": [["6.2.0", "6.4.14"]], "type": "str"},
            "_image-base64": {"v_range": [["6.2.2", ""]], "type": "str"}
        }
    },
    "firewall_proxyaddrgrp_tagging": {
        "stated": true,
        "options": {"category": {"type": "str"}, "name": {"required": true, "type": "str"}, "tags": {"type": "raw"}}
    },
    "firewall_schedule_group": {
        "stated": true,
        "options": {
            "color": {"type": "int"},
            "member": {"type": "raw"},
            "name": {"required": true, "type": "str"},
            "global-object": {"v_range": [["6.4.0", ""]], "type": "int"},
            "fabric-object": {"v_range": [["6.4.4", ""]], "choices": ["disable", "enable"], "type": "str"},
            "uuid": {"v_range": [["7.6.0", ""]], "type": "str"}
        }
    },
    "firewall_schedule_onetime": {
        "stated": true,
        "options": {
            "color": {"type": "int"},
            "end": {"type": "str"},
            "expiration-days": {"type": "int"},
            "name": {"required": true, "type": "str"},
            "start": {"type": "str"},
            "global-object": {"v_range": [["6.4.0", ""]], "type": "int"},
            "fabric-object": {"v_range": [["6.4.4", ""]], "choices": ["disable", "enable"], "type": "str"},
            "end-utc": {"v_range": [["7.2.2", ""]], "type": "str"},
            "start-utc": {"v_range": [["7.2.2", ""]], "type": "str"},
            "uuid": {"v_range": [["7.6.0", ""]], "type": "str"}
        }
    },
    "firewall_schedule_recurring": {
        "stated": true,
        "options": {
            "color": {"type": "int"},
            "day": {"type": "list", "choices": ["sunday", "monday", "tuesday", "wednesday", "thursday", "friday", "saturday", "none"], "elements": "str"},
            "end": {"type": "str"},
            "name": {"required": true, "type": "str"},
            "start": {"type": "str"},
            "global-object": {"v_range": [["6.4.0", ""]], "type": "int"},
            "fabric-object": {"v_range": [["6.4.4", ""]], "choices": ["disable", "enable"], "type": "str"},
            "uuid": {"v_range": [["7.6.0", ""]], "type": "str"}
        }
    },
    "firewall_service_category": {
        "stated": true,
        "options": {
            "comment": {"type": "str"},
            "name": {"required": true, "type": "str"},
            "global-object": {"v_range": [["6.4.0", ""]], "type": "int"},
            "fabric-object": {"v_range": [["6.4.4", ""]], "choices": ["disable", "enable"], "type": "str"},
            "uuid": {"v_range": [["7.6.0", ""]], "type": "str"}
        }
    },
    "firewall_service_custom": {
        "stated": true,
        "options": {
            "app-category": {"type": "raw"},
            "app-service-type": {"choices": ["disable", "app-id", "app-category"], "type": "str"},
            "application": {"type": "raw"},
            "category": {"type": "str"},
            "check-reset-range": {"choices": ["disable", "default", "strict"], "type": "str"},
            "color": {"type": "int"},
            "comment": {"type": "raw"},
            "fqdn": {"type": "str"},
            "helper": {
                "choices": [
                    "disable", "auto", "ftp", "tftp", "ras", "h323", "tns", "mms", "sip", "pptp", "rtsp", "dns-udp", "dns-tcp", "pmap", "rsh", "dcerpc",
                    "mgcp", "gtp-c", "gtp-u", "gtp-b", "pfcp"
                ],
                "type": "str"
            },
            "icmpcode": {"type": "int"},
            "icmptype": {"type": "int"},
            "iprange": {"type": "str"},
            "name": {"required": true, "type": "str"},
            "protocol": {
                "choices": [
                    "ICMP", "IP", "TCP/UDP/SCTP", "ICMP6", "HTTP", "FTP", "CONNECT", "SOCKS", "ALL", "SOCKS-TCP", "SOCKS-UDP", "TCP/UDP/UDP-Lite/SCTP"
                ],
                "type": "str"
            },
            "protocol-number": {"type": "int"},
            "proxy": {"choices": ["disable", "enable"], "type": "str"},
            "sctp-portrange": {"type": "str"},
            "session-ttl": {"type": "raw"},
            "tcp-halfclose-timer": {"type": "int"},
            "tcp-halfopen-timer": {"type": "int"},
            "tcp-portrange": {"type": "str"},
            "tcp-timewait-timer": {"type": "int"},
            "udp-idle-timer": {"type": "int"},
            "udp-portrange": {"type": "str"},
            "visibility": {"choices": ["disable", "enable"], "type": "str"},
            "explicit-proxy": {"v_range": [["6.2.0", "6.2.12"]], "choices": ["disable", "enable"], "type": "str"},
            "global-object": {"v_range": [["6.4.0", ""]], "type": "int"},
            "fabric-object": {"v_range": [["6.4.4", ""]], "choices": ["disable", "enable"], "type": "str"},
            "tcp-rst-timer": {"v_range": [["7.0.0", ""]], "type": "int"},
            "uuid": {"v_range": [["7.4.2", ""]], "type": "str"},
            "udplite-portrange": {"v_range": [["7.6.0", ""]], "type": "str"}
        }
    },
    "firewall_service_group": {
        "stated": true,
        "options": {
            "color": {"type": "int"},
            "comment": {"type": "raw"},
            "member": {"type": "raw"},
            "name": {"required": true, "type": "str"},
            "proxy": {"choices": ["disable", "enable"], "type": "str"},
            "explicit-proxy": {"v_range": [["6.2.0", "6.2.12"]], "choices": ["disable", "enable"], "type": "str"},
            "global-object": {"v_range": [["6.4.0", ""]], "type": "int"},
            "fabric-object": {"v_range": [["6.4.4", ""]], "choices": ["disable", "enable"], "type": "str"},
            "uuid": {"v_range": [["7.4.2", ""]], "type": "str"}
        }
    },
    "firewall_shaper_peripshaper": {
        "stated": true,
        "options": {
            "bandwidth-unit": {"choices": ["kbps", "mbps", "gbps"], "type": "str"},
            "diffserv-forward": {"choices": ["disable", "enable"], "type": "str"},
            "diffserv-reverse": {"choices": ["disable", "enable"], "type": "str"},
            "diffservcode-forward": {"type": "str"},
            "diffservcode-rev": {"type": "str"},
            "max-bandwidth": {"type": "int"},
            "max-concurrent-session": {"type": "int"},
            "name": {"required": true, "type": "str"},
            "max-concurrent-tcp-session": {"v_range": [["7.0.0", ""]], "type": "int"},
            "max-concurrent-udp-session": {"v_range": [["7.0.0", ""]], "type": "int"}
        }
    },
    "firewall_shaper_trafficshaper": {
        "stated": true,
        "options": {
            "bandwidth-unit": {"choices": ["kbps", "mbps", "gbps"], "type": "str"},
            "diffserv": {"choices": ["disable", "enable"], "type": "str"},
            "diffservcode": {"type": "str"},
            "guaranteed-bandwidth": {"type": "int"},
            "maximum-bandwidth": {"type": "int"},
            "name": {"required": true, "type": "str"},
            "per-policy": {"choices": ["disable", "enable"], "type": "str"},
            "priority": {"choices": ["high", "medium", "low"], "type": "str"},
            "dscp-marking-method": {"v_range": [["6.2.1", ""]], "choices": ["multi-stage", "static"], "type": "str"},
            "exceed-bandwidth": {"v_range": [["6.2.1", ""]], "type": "int"},
            "exceed-class-id": {"v_range": [["6.2.1", ""]], "type": "int"},
            "exceed-dscp": {"v_range": [["6.2.1", ""]], "type": "str"},
            "maximum-dscp": {"v_range": [["6.2.1", ""]], "type": "str"},
            "overhead": {"v_range": [["6.2.1", ""]], "type": "int"},
            "cos": {"v_range": [["7.4.0", ""]], "type": "str"},
            "cos-marking": {"v_range": [["7.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "cos-marking-method": {"v_range": [["7.4.0", ""]], "choices": ["multi-stage", "static"], "type": "str"},
            "exceed-cos": {"v_range": [["7.4.0", ""]], "type": "str"},
            "maximum-cos": {"v_range": [["7.4.0", ""]], "type": "str"}
        }
    },
    "firewall_shapingprofile": {
        "stated": true,
        "options": {
            "comment": {"type": "str"},
            "default-class-id": {"type": "raw"},
            "profile-name": {"required": true, "type": "str"},
            "shaping-entries": {
                "type": "list",
                "options": {
                    "class-id": {"type": "raw"},
                    "guaranteed-bandwidth-percentage": {"type": "int"},
                    "id": {"type": "int"},
                    "maximum-bandwidth-percentage": {"type": "int"},
                    "priority": {"choices": ["low", "medium", "high", "critical", "top"], "type": "str"},
                    "burst-in-msec": {"v_range": [["6.2.1", ""]], "type": "int"},
                    "cburst-in-msec": {"v_range": [["6.2.1", ""]], "type": "int"},
                    "limit": {"v_range": [["6.2.1", ""]], "type": "int"},
                    "max": {"v_range": [["6.2.1", ""]], "type": "int"},
                    "min": {"v_range": [["6.2.1", ""]], "type": "int"},
                    "red-probability": {"v_range": [["6.2.1", ""]], "type": "int"}
                },
                "elements": "dict"
            },
            "type": {"v_range": [["6.2.1", ""]], "choices": ["policing", "queuing"], "type": "str"},
            "npu-offloading": {"v_range": [["7.6.0", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "firewall_shapingprofile_shapingentries": {
        "stated": true,
        "options": {
            "class-id": {"type": "raw"},
            "guaranteed-bandwidth-percentage": {"type": "int"},
            "id": {"required": true, "type": "int"},
            "maximum-bandwidth-percentage": {"type": "int"},
            "priority": {"choices": ["low", "medium", "high", "critical", "top"], "type": "str"},
            "burst-in-msec": {"v_range": [["6.2.1", ""]], "type": "int"},
            "cburst-in-msec": {"v_range": [["6.2.1", ""]], "type": "int"},
            "limit": {"v_range": [["6.2.1", ""]], "type": "int"},
            "max": {"v_range": [["6.2.1", ""]], "type": "int"},
            "min": {"v_range": [["6.2.1", ""]], "type": "int"},
            "red-probability": {"v_range": [["6.2.1", ""]], "type": "int"}
        }
    },
    "firewall_ssh_localca": {
        "stated": true,
        "options": {
            "name": {"v_range": [["6.2.1", ""]], "required": true, "type": "str"},
            "password": {"v_range": [["6.2.1", ""]], "no_log": true, "type": "raw"},
            "private-key": {"v_range": [["6.2.1", ""]], "no_log": true, "type": "str"},
            "public-key": {"v_range": [["6.2.1", ""]], "no_log": true, "type": "str"},
            "source": {"v_range": [["6.2.1", ""]], "choices": ["built-in", "user"], "type": "str"}
        }
    },
    "firewall_sslsshprofile": {
        "stated": true,
        "options": {
            "caname": {"type": "str"},
            "comment": {"type": "str"},
            "mapi-over-https": {"choices": ["disable", "enable"], "type": "str"},
            "name": {"required": true, "type": "str"},
            "rpc-over-https": {"choices": ["disable", "enable"], "type": "str"},
            "server-cert": {"type": "raw"},
            "server-cert-mode": {"choices": ["re-sign", "replace"], "type": "str"},
            "ssl-anomalies-log": {"choices": ["disable", "enable"], "type": "str"},
            "ssl-exempt": {
                "type": "list",
                "options": {
                    "address": {"type": "str"},
                    "address6": {"type": "str"},
                    "fortiguard-category": {"type": "str"},
                    "id": {"type": "int"},
                    "regex": {"type": "str"},
                    "type": {"choices": ["fortiguard-category", "address", "address6", "wildcard-fqdn", "regex", "finger-print"], "type": "str"},
                    "wildcard-fqdn": {"type": "str"}
                },
                "elements": "dict"
            },
            "ssl-exemptions-log": {"choices": ["disable", "enable"], "type": "str"},
            "ssl-server": {
                "type": "list",
                "options": {
                    "ftps-client-cert-request": {"choices": ["bypass", "inspect", "block"], "type": "str"},
                    "https-client-cert-request": {"choices": ["bypass", "inspect", "block"], "type": "str"},
                    "id": {"type": "int"},
                    "imaps-client-cert-request": {"choices": ["bypass", "inspect", "block"], "type": "str"},
                    "ip": {"type": "str"},
                    "pop3s-client-cert-request": {"choices": ["bypass", "inspect", "block"], "type": "str"},
                    "smtps-client-cert-request": {"choices": ["bypass", "inspect", "block"], "type": "str"},
                    "ssl-other-client-cert-request": {"choices": ["bypass", "inspect", "block"], "type": "str"},
                    "ftps-client-certificate": {"v_range": [["6.4.0", ""]], "choices": ["bypass", "inspect", "block"], "type": "str"},
                    "https-client-certificate": {"v_range": [["6.4.0", ""]], "choices": ["bypass", "inspect", "block"], "type": "str"},
                    "imaps-client-certificate": {"v_range": [["6.4.0", ""]], "choices": ["bypass", "inspect", "block"], "type": "str"},
                    "pop3s-client-certificate": {"v_range": [["6.4.0", ""]], "choices": ["bypass", "inspect", "block"], "type": "str"},
                    "smtps-client-certificate": {"v_range": [["6.4.0", ""]], "choices": ["bypass", "inspect", "block"], "type": "str"},
                    "ssl-other-client-certificate": {"v_range": [["6.4.0", ""]], "choices": ["bypass", "inspect", "block"], "type": "str"}
                },
                "elements": "dict"
            },
            "untrusted-caname": {"type": "str"},
            "use-ssl-server": {"choices": ["disable", "enable"], "type": "str"},
            "whitelist": {"choices": ["disable", "enable"], "type": "str"},
            "block-blacklisted-certificates": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "certname": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"},
            "ssl-invalid-server-cert-log": {"v_range": [["6.2.0", "6.2.12"]], "choices": ["disable", "enable"], "type": "str"},
            "ssl-negotiation-log": {"v_range": [["6.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ftps": {
                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                "type": "dict",
                "options": {
                    "cert-validation-failure": {"v_range": [["6.4.5", ""]], "choices": ["allow", "block", "ignore"], "type": "str"},
                    "cert-validation-timeout": {"v_range": [["6.4.5", ""]], "choices": ["allow", "block", "ignore"], "type": "str"},
                    "client-certificate": {"v_range": [["6.4.5", ""]], "choices": ["bypass", "inspect", "block"], "type": "str"},
                    "expired-server-cert": {"v_range": [["6.4.5", ""]], "choices": ["allow", "block", "ignore"], "type": "str"},
                    "ports": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "raw"},
                    "revoked-server-cert": {"v_range": [["6.4.5", ""]], "choices": ["allow", "block", "ignore"], "type": "str"},
                    "sni-server-cert-check": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable", "strict"], "type": "str"},
                    "status": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "deep-inspection"], "type": "str"},
                    "unsupported-ssl-cipher": {"v_range": [["6.4.5", ""]], "choices": ["allow", "block"], "type": "str"},
                    "unsupported-ssl-negotiation": {"v_range": [["6.4.5", ""]], "choices": ["allow", "block"], "type": "str"},
                    "untrusted-server-cert": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["allow", "block", "ignore"], "type": "str"},
                    "unsupported-ssl": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["bypass", "inspect", "block"], "type": "str"},
                    "client-cert-request": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["bypass", "inspect", "block"], "type": "str"},
                    "invalid-server-cert": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["allow", "block"], "type": "str"},
                    "allow-invalid-server-cert": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", "7.2.1"]], "choices": ["disable", "enable"], "type": "str"},
                    "untrusted-cert": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", "7.2.1"]], "choices": ["allow", "block", "ignore"], "type": "str"},
                    "min-allowed-ssl-version": {
                        "v_range": [["7.0.3", ""]],
                        "choices": ["ssl-3.0", "tls-1.0", "tls-1.1", "tls-1.2", "tls-1.3"],
                        "type": "str"
                    },
                    "unsupported-ssl-version": {"v_range": [["7.0.1", ""]], "choices": ["block", "allow", "inspect"], "type": "str"}
                }
            },
            "https": {
                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                "type": "dict",
                "options": {
                    "cert-validation-failure": {"v_range": [["6.4.5", ""]], "choices": ["allow", "block", "ignore"], "type": "str"},
                    "cert-validation-timeout": {"v_range": [["6.4.5", ""]], "choices": ["allow", "block", "ignore"], "type": "str"},
                    "client-certificate": {"v_range": [["6.4.5", ""]], "choices": ["bypass", "inspect", "block"], "type": "str"},
                    "expired-server-cert": {"v_range": [["6.4.5", ""]], "choices": ["allow", "block", "ignore"], "type": "str"},
                    "ports": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "raw"},
                    "proxy-after-tcp-handshake": {"v_range": [["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "revoked-server-cert": {"v_range": [["6.4.5", ""]], "choices": ["allow", "block", "ignore"], "type": "str"},
                    "sni-server-cert-check": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable", "strict"], "type": "str"},
                    "status": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "choices": ["disable", "certificate-inspection", "deep-inspection"],
                        "type": "str"
                    },
                    "unsupported-ssl-cipher": {"v_range": [["6.4.5", ""]], "choices": ["allow", "block"], "type": "str"},
                    "unsupported-ssl-negotiation": {"v_range": [["6.4.5", ""]], "choices": ["allow", "block"], "type": "str"},
                    "untrusted-server-cert": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["allow", "block", "ignore"], "type": "str"},
                    "unsupported-ssl": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["bypass", "inspect", "block"], "type": "str"},
                    "client-cert-request": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["bypass", "inspect", "block"], "type": "str"},
                    "invalid-server-cert": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["allow", "block"], "type": "str"},
                    "allow-invalid-server-cert": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", "7.2.1"]], "choices": ["disable", "enable"], "type": "str"},
                    "untrusted-cert": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", "7.2.1"]], "choices": ["allow", "block", "ignore"], "type": "str"},
                    "cert-probe-failure": {"v_range": [["7.0.0", ""]], "choices": ["block", "allow"], "type": "str"},
                    "min-allowed-ssl-version": {
                        "v_range": [["7.0.3", ""]],
                        "choices": ["ssl-3.0", "tls-1.0", "tls-1.1", "tls-1.2", "tls-1.3"],
                        "type": "str"
                    },
                    "unsupported-ssl-version": {"v_range": [["7.0.1", ""]], "choices": ["block", "allow", "inspect"], "type": "str"},
                    "quic": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable", "bypass", "block", "inspect"], "type": "str"},
                    "encrypted-client-hello": {"v_range": [["7.4.3", ""]], "choices": ["block", "allow"], "type": "str"}
                }
            },
            "imaps": {
                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                "type": "dict",
                "options": {
                    "cert-validation-failure": {"v_range": [["6.4.5", ""]], "choices": ["allow", "block", "ignore"], "type": "str"},
                    "cert-validation-timeout": {"v_range": [["6.4.5", ""]], "choices": ["allow", "block", "ignore"], "type": "str"},
                    "client-certificate": {"v_range": [["6.4.5", ""]], "choices": ["bypass", "inspect", "block"], "type": "str"},
                    "expired-server-cert": {"v_range": [["6.4.5", ""]], "choices": ["allow", "block", "ignore"], "type": "str"},
                    "ports": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "raw"},
                    "proxy-after-tcp-handshake": {"v_range": [["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "revoked-server-cert": {"v_range": [["6.4.5", ""]], "choices": ["allow", "block", "ignore"], "type": "str"},
                    "sni-server-cert-check": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable", "strict"], "type": "str"},
                    "status": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "deep-inspection"], "type": "str"},
                    "unsupported-ssl-cipher": {"v_range": [["6.4.5", ""]], "choices": ["allow", "block"], "type": "str"},
                    "unsupported-ssl-negotiation": {"v_range": [["6.4.5", ""]], "choices": ["allow", "block"], "type": "str"},
                    "untrusted-server-cert": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["allow", "block", "ignore"], "type": "str"},
                    "unsupported-ssl": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["bypass", "inspect", "block"], "type": "str"},
                    "client-cert-request": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["bypass", "inspect", "block"], "type": "str"},
                    "invalid-server-cert": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["allow", "block"], "type": "str"},
                    "allow-invalid-server-cert": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", "7.2.1"]], "choices": ["disable", "enable"], "type": "str"},
                    "untrusted-cert": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", "7.2.1"]], "choices": ["allow", "block", "ignore"], "type": "str"},
                    "unsupported-ssl-version": {"v_range": [["7.0.1", ""]], "choices": ["block", "allow", "inspect"], "type": "str"},
                    "min-allowed-ssl-version": {
                        "v_range": [["7.0.3", ""]],
                        "choices": ["ssl-3.0", "tls-1.0", "tls-1.1", "tls-1.2", "tls-1.3"],
                        "type": "str"
                    }
                }
            },
            "pop3s": {
                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                "type": "dict",
                "options": {
                    "cert-validation-failure": {"v_range": [["6.4.5", ""]], "choices": ["allow", "block", "ignore"], "type": "str"},
                    "cert-validation-timeout": {"v_range": [["6.4.5", ""]], "choices": ["allow", "block", "ignore"], "type": "str"},
                    "client-certificate": {"v_range": [["6.4.5", ""]], "choices": ["bypass", "inspect", "block"], "type": "str"},
                    "expired-server-cert": {"v_range": [["6.4.5", ""]], "choices": ["allow", "block", "ignore"], "type": "str"},
                    "ports": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "raw"},
                    "proxy-after-tcp-handshake": {"v_range": [["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "revoked-server-cert": {"v_range": [["6.4.5", ""]], "choices": ["allow", "block", "ignore"], "type": "str"},
                    "sni-server-cert-check": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable", "strict"], "type": "str"},
                    "status": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "deep-inspection"], "type": "str"},
                    "unsupported-ssl-cipher": {"v_range": [["6.4.5", ""]], "choices": ["allow", "block"], "type": "str"},
                    "unsupported-ssl-negotiation": {"v_range": [["6.4.5", ""]], "choices": ["allow", "block"], "type": "str"},
                    "untrusted-server-cert": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["allow", "block", "ignore"], "type": "str"},
                    "unsupported-ssl": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["bypass", "inspect", "block"], "type": "str"},
                    "client-cert-request": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["bypass", "inspect", "block"], "type": "str"},
                    "invalid-server-cert": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["allow", "block"], "type": "str"},
                    "allow-invalid-server-cert": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", "7.2.1"]], "choices": ["disable", "enable"], "type": "str"},
                    "untrusted-cert": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", "7.2.1"]], "choices": ["allow", "block", "ignore"], "type": "str"},
                    "unsupported-ssl-version": {"v_range": [["7.0.1", ""]], "choices": ["block", "allow", "inspect"], "type": "str"},
                    "min-allowed-ssl-version": {
                        "v_range": [["7.0.3", ""]],
                        "choices": ["ssl-3.0", "tls-1.0", "tls-1.1", "tls-1.2", "tls-1.3"],
                        "type": "str"
                    }
                }
            },
            "smtps": {
                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                "type": "dict",
                "options": {
                    "cert-validation-failure": {"v_range": [["6.4.5", ""]], "choices": ["allow", "block", "ignore"], "type": "str"},
                    "cert-validation-timeout": {"v_range": [["6.4.5", ""]], "choices": ["allow", "block", "ignore"], "type": "str"},
                    "client-certificate": {"v_range": [["6.4.5", ""]], "choices": ["bypass", "inspect", "block"], "type": "str"},
                    "expired-server-cert": {"v_range": [["6.4.5", ""]], "choices": ["allow", "block", "ignore"], "type": "str"},
                    "ports": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "raw"},
                    "proxy-after-tcp-handshake": {"v_range": [["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "revoked-server-cert": {"v_range": [["6.4.5", ""]], "choices": ["allow", "block", "ignore"], "type": "str"},
                    "sni-server-cert-check": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable", "strict"], "type": "str"},
                    "status": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "deep-inspection"], "type": "str"},
                    "unsupported-ssl-cipher": {"v_range": [["6.4.5", ""]], "choices": ["allow", "block"], "type": "str"},
                    "unsupported-ssl-negotiation": {"v_range": [["6.4.5", ""]], "choices": ["allow", "block"], "type": "str"},
                    "untrusted-server-cert": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["allow", "block", "ignore"], "type": "str"},
                    "unsupported-ssl": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["bypass", "inspect", "block"], "type": "str"},
                    "client-cert-request": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["bypass", "inspect", "block"], "type": "str"},
                    "invalid-server-cert": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["allow", "block"], "type": "str"},
                    "allow-invalid-server-cert": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", "7.2.1"]], "choices": ["disable", "enable"], "type": "str"},
                    "untrusted-cert": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", "7.2.1"]], "choices": ["allow", "block", "ignore"], "type": "str"},
                    "unsupported-ssl-version": {"v_range": [["7.0.1", ""]], "choices": ["block", "allow", "inspect"], "type": "str"},
                    "min-allowed-ssl-version": {
                        "v_range": [["7.0.3", ""]],
                        "choices": ["ssl-3.0", "tls-1.0", "tls-1.1", "tls-1.2", "tls-1.3"],
                        "type": "str"
                    }
                }
            },
            "ssh": {
                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                "type": "dict",
                "options": {
                    "inspect-all": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "deep-inspection"], "type": "str"},
                    "ports": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "raw"},
                    "proxy-after-tcp-handshake": {"v_range": [["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "ssh-algorithm": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["compatible", "high-encryption"], "type": "str"},
                    "ssh-tun-policy-check": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "status": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "deep-inspection"], "type": "str"},
                    "unsupported-version": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["block", "bypass"], "type": "str"},
                    "ssh-policy-check": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", "7.2.1"]], "choices": ["disable", "enable"], "type": "str"},
                    "block": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", "6.4.14"]],
                        "type": "list",
                        "choices": ["x11-filter", "ssh-shell", "exec", "port-forward"],
                        "elements": "str"
                    },
                    "log": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", "6.4.14"]],
                        "type": "list",
                        "choices": ["x11-filter", "ssh-shell", "exec", "port-forward"],
                        "elements": "str"
                    }
                }
            },
            "ssl": {
                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                "type": "dict",
                "options": {
                    "cert-validation-failure": {"v_range": [["6.4.5", ""]], "choices": ["allow", "block", "ignore"], "type": "str"},
                    "cert-validation-timeout": {"v_range": [["6.4.5", ""]], "choices": ["allow", "block", "ignore"], "type": "str"},
                    "client-certificate": {"v_range": [["6.4.5", ""]], "choices": ["bypass", "inspect", "block"], "type": "str"},
                    "expired-server-cert": {"v_range": [["6.4.5", ""]], "choices": ["allow", "block", "ignore"], "type": "str"},
                    "inspect-all": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "choices": ["disable", "certificate-inspection", "deep-inspection"],
                        "type": "str"
                    },
                    "revoked-server-cert": {"v_range": [["6.4.5", ""]], "choices": ["allow", "block", "ignore"], "type": "str"},
                    "sni-server-cert-check": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable", "strict"], "type": "str"},
                    "unsupported-ssl-cipher": {"v_range": [["6.4.5", ""]], "choices": ["allow", "block"], "type": "str"},
                    "unsupported-ssl-negotiation": {"v_range": [["6.4.5", ""]], "choices": ["allow", "block"], "type": "str"},
                    "untrusted-server-cert": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["allow", "block", "ignore"], "type": "str"},
                    "unsupported-ssl": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["bypass", "inspect", "block"], "type": "str"},
                    "client-cert-request": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["bypass", "inspect", "block"], "type": "str"},
                    "invalid-server-cert": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["allow", "block"], "type": "str"},
                    "allow-invalid-server-cert": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", "7.2.1"]], "choices": ["disable", "enable"], "type": "str"},
                    "untrusted-cert": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", "7.2.1"]], "choices": ["allow", "block", "ignore"], "type": "str"},
                    "cert-probe-failure": {"v_range": [["7.0.1", ""]], "choices": ["block", "allow"], "type": "str"},
                    "min-allowed-ssl-version": {
                        "v_range": [["7.0.3", ""]],
                        "choices": ["ssl-3.0", "tls-1.0", "tls-1.1", "tls-1.2", "tls-1.3"],
                        "type": "str"
                    },
                    "unsupported-ssl-version": {"v_range": [["7.0.1", ""]], "choices": ["block", "allow", "inspect"], "type": "str"},
                    "encrypted-client-hello": {"v_range": [["7.4.3", ""]], "choices": ["block", "allow"], "type": "str"}
                }
            },
            "allowlist": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "block-blocklisted-certificates": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "dot": {
                "v_range": [["7.0.0", ""]],
                "type": "dict",
                "options": {
                    "cert-validation-failure": {"v_range": [["7.0.0", ""]], "choices": ["allow", "block", "ignore"], "type": "str"},
                    "cert-validation-timeout": {"v_range": [["7.0.0", ""]], "choices": ["allow", "block", "ignore"], "type": "str"},
                    "client-certificate": {"v_range": [["7.0.0", ""]], "choices": ["bypass", "inspect", "block"], "type": "str"},
                    "expired-server-cert": {"v_range": [["7.0.0", ""]], "choices": ["allow", "block", "ignore"], "type": "str"},
                    "proxy-after-tcp-handshake": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "revoked-server-cert": {"v_range": [["7.0.0", ""]], "choices": ["allow", "block", "ignore"], "type": "str"},
                    "sni-server-cert-check": {"v_range": [["7.0.0", ""]], "choices": ["enable", "strict", "disable"], "type": "str"},
                    "status": {"v_range": [["7.0.0", ""]], "choices": ["disable", "deep-inspection"], "type": "str"},
                    "unsupported-ssl-cipher": {"v_range": [["7.0.0", ""]], "choices": ["block", "allow"], "type": "str"},
                    "unsupported-ssl-negotiation": {"v_range": [["7.0.0", ""]], "choices": ["block", "allow"], "type": "str"},
                    "untrusted-server-cert": {"v_range": [["7.0.0", ""]], "choices": ["allow", "block", "ignore"], "type": "str"},
                    "unsupported-ssl-version": {"v_range": [["7.0.1", ""]], "choices": ["block", "allow", "inspect"], "type": "str"},
                    "min-allowed-ssl-version": {
                        "v_range": [["7.0.3", ""]],
                        "choices": ["ssl-3.0", "tls-1.0", "tls-1.1", "tls-1.2", "tls-1.3"],
                        "type": "str"
                    },
                    "quic": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable", "bypass", "block", "inspect"], "type": "str"}
                }
            },
            "supported-alpn": {"v_range": [["7.0.0", ""]], "choices": ["none", "http1-1", "http2", "all"], "type": "str"},
            "ssl-anomaly-log": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ssl-exemption-ip-rating": {"v_range": [["7.0.4", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ssl-exemption-log": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ssl-handshake-log": {"v_range": [["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ssl-server-cert-log": {"v_range": [["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ech-outer-sni": {
                "v_range": [["7.4.3", ""]],
                "type": "list",
                "options": {"name": {"v_range": [["7.4.3", ""]], "type": "str"}, "sni": {"v_range": [["7.4.3", ""]], "type": "str"}},
                "elements": "dict"
            }
        }
    },
    "firewall_sslsshprofile_dot": {
        "stated": false,
        "options": {
            "cert-validation-failure": {"v_range": [["7.0.0", ""]], "choices": ["allow", "block", "ignore"], "type": "str"},
            "cert-validation-timeout": {"v_range": [["7.0.0", ""]], "choices": ["allow", "block", "ignore"], "type": "str"},
            "client-certificate": {"v_range": [["7.0.0", ""]], "choices": ["bypass", "inspect", "block"], "type": "str"},
            "expired-server-cert": {"v_range": [["7.0.0", ""]], "choices": ["allow", "block", "ignore"], "type": "str"},
            "proxy-after-tcp-handshake": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "revoked-server-cert": {"v_range": [["7.0.0", ""]], "choices": ["allow", "block", "ignore"], "type": "str"},
            "sni-server-cert-check": {"v_range": [["7.0.0", ""]], "choices": ["enable", "strict", "disable"], "type": "str"},
            "status": {"v_range": [["7.0.0", ""]], "choices": ["disable", "deep-inspection"], "type": "str"},
            "unsupported-ssl-cipher": {"v_range": [["7.0.0", ""]], "choices": ["block", "allow"], "type": "str"},
            "unsupported-ssl-negotiation": {"v_range": [["7.0.0", ""]], "choices": ["block", "allow"], "type": "str"},
            "untrusted-server-cert": {"v_range": [["7.0.0", ""]], "choices": ["allow", "block", "ignore"], "type": "str"},
            "unsupported-ssl-version": {"v_range": [["7.0.1", ""]], "choices": ["block", "allow", "inspect"], "type": "str"},
            "min-allowed-ssl-version": {"v_range": [["7.0.3", ""]], "choices": ["ssl-3.0", "tls-1.0", "tls-1.1", "tls-1.2", "tls-1.3"], "type": "str"},
            "quic": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable", "bypass", "block", "inspect"], "type": "str"}
        }
    },
    "firewall_sslsshprofile_echoutersni": {
        "stated": true,
        "options": {"name": {"v_range": [["7.4.3", ""]], "required": true, "type": "str"}, "sni": {"v_range": [["7.4.3", ""]], "type": "str"}}
    },
    "firewall_sslsshprofile_ftps": {
        "stated": false,
        "options": {
            "allow-invalid-server-cert": {"v_range": [["6.0.0", "7.2.1"]], "choices": ["disable", "enable"], "type": "str"},
            "client-cert-request": {"choices": ["bypass", "inspect", "block"], "type": "str"},
            "ports": {"type": "raw"},
            "status": {"choices": ["disable", "deep-inspection"], "type": "str"},
            "unsupported-ssl": {"choices": ["bypass", "inspect", "block"], "type": "str"},
            "untrusted-cert": {"v_range": [["6.0.0", "7.2.1"]], "choices": ["allow", "block", "ignore"], "type": "str"},
            "invalid-server-cert": {"v_range": [["6.2.0", ""]], "choices": ["allow", "block"], "type": "str"},
            "sni-server-cert-check": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable", "strict"], "type": "str"},
            "untrusted-server-cert": {"v_range": [["6.2.0", ""]], "choices": ["allow", "block", "ignore"], "type": "str"},
            "cert-validation-failure": {"v_range": [["6.4.0", ""]], "choices": ["allow", "block", "ignore"], "type": "str"},
            "cert-validation-timeout": {"v_range": [["6.4.0", ""]], "choices": ["allow", "block", "ignore"], "type": "str"},
            "client-certificate": {"v_range": [["6.4.0", ""]], "choices": ["bypass", "inspect", "block"], "type": "str"},
            "expired-server-cert": {"v_range": [["6.4.0", ""]], "choices": ["allow", "block", "ignore"], "type": "str"},
            "revoked-server-cert": {"v_range": [["6.4.0", ""]], "choices": ["allow", "block", "ignore"], "type": "str"},
            "unsupported-ssl-cipher": {"v_range": [["6.4.0", ""]], "choices": ["allow", "block"], "type": "str"},
            "unsupported-ssl-negotiation": {"v_range": [["6.4.0", ""]], "choices": ["allow", "block"], "type": "str"},
            "min-allowed-ssl-version": {"v_range": [["7.0.3", ""]], "choices": ["ssl-3.0", "tls-1.0", "tls-1.1", "tls-1.2", "tls-1.3"], "type": "str"},
            "unsupported-ssl-version": {"v_range": [["7.0.1", ""]], "choices": ["block", "allow", "inspect"], "type": "str"}
        }
    },
    "firewall_sslsshprofile_https": {
        "stated": false,
        "options": {
            "allow-invalid-server-cert": {"v_range": [["6.0.0", "7.2.1"]], "choices": ["disable", "enable"], "type": "str"},
            "client-cert-request": {"choices": ["bypass", "inspect", "block"], "type": "str"},
            "ports": {"type": "raw"},
            "status": {"choices": ["disable", "certificate-inspection", "deep-inspection"], "type": "str"},
            "unsupported-ssl": {"choices": ["bypass", "inspect", "block"], "type": "str"},
            "untrusted-cert": {"v_range": [["6.0.0", "7.2.1"]], "choices": ["allow", "block", "ignore"], "type": "str"},
            "invalid-server-cert": {"v_range": [["6.2.0", ""]], "choices": ["allow", "block"], "type": "str"},
            "sni-server-cert-check": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable", "strict"], "type": "str"},
            "untrusted-server-cert": {"v_range": [["6.2.0", ""]], "choices": ["allow", "block", "ignore"], "type": "str"},
            "cert-validation-failure": {"v_range": [["6.4.0", ""]], "choices": ["allow", "block", "ignore"], "type": "str"},
            "cert-validation-timeout": {"v_range": [["6.4.0", ""]], "choices": ["allow", "block", "ignore"], "type": "str"},
            "client-certificate": {"v_range": [["6.4.0", ""]], "choices": ["bypass", "inspect", "block"], "type": "str"},
            "expired-server-cert": {"v_range": [["6.4.0", ""]], "choices": ["allow", "block", "ignore"], "type": "str"},
            "proxy-after-tcp-handshake": {"v_range": [["6.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "revoked-server-cert": {"v_range": [["6.4.0", ""]], "choices": ["allow", "block", "ignore"], "type": "str"},
            "unsupported-ssl-cipher": {"v_range": [["6.4.0", ""]], "choices": ["allow", "block"], "type": "str"},
            "unsupported-ssl-negotiation": {"v_range": [["6.4.0", ""]], "choices": ["allow", "block"], "type": "str"},
            "cert-probe-failure": {"v_range": [["7.0.0", ""]], "choices": ["block", "allow"], "type": "str"},
            "min-allowed-ssl-version": {"v_range": [["7.0.3", ""]], "choices": ["ssl-3.0", "tls-1.0", "tls-1.1", "tls-1.2", "tls-1.3"], "type": "str"},
            "unsupported-ssl-version": {"v_range": [["7.0.1", ""]], "choices": ["block", "allow", "inspect"], "type": "str"},
            "quic": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable", "bypass", "block", "inspect"], "type": "str"},
            "encrypted-client-hello": {"v_range": [["7.4.3", ""]], "choices": ["block", "allow"], "type": "str"}
        }
    },
    "firewall_sslsshprofile_imaps": {
        "stated": false,
        "options": {
            "allow-invalid-server-cert": {"v_range": [["6.0.0", "7.2.1"]], "choices": ["disable", "enable"], "type": "str"},
            "client-cert-request": {"choices": ["bypass", "inspect", "block"], "type": "str"},
            "ports": {"type": "raw"},
            "status": {"choices": ["disable", "deep-inspection"], "type": "str"},
            "unsupported-ssl": {"choices": ["bypass", "inspect", "block"], "type": "str"},
            "untrusted-cert": {"v_range": [["6.0.0", "7.2.1"]], "choices": ["allow", "block", "ignore"], "type": "str"},
            "invalid-server-cert": {"v_range": [["6.2.0", ""]], "choices": ["allow", "block"], "type": "str"},
            "sni-server-cert-check": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable", "strict"], "type": "str"},
            "untrusted-server-cert": {"v_range": [["6.2.0", ""]], "choices": ["allow", "block", "ignore"], "type": "str"},
            "cert-validation-failure": {"v_range": [["6.4.0", ""]], "choices": ["allow", "block", "ignore"], "type": "str"},
            "cert-validation-timeout": {"v_range": [["6.4.0", ""]], "choices": ["allow", "block", "ignore"], "type": "str"},
            "client-certificate": {"v_range": [["6.4.0", ""]], "choices": ["bypass", "inspect", "block"], "type": "str"},
            "expired-server-cert": {"v_range": [["6.4.0", ""]], "choices": ["allow", "block", "ignore"], "type": "str"},
            "proxy-after-tcp-handshake": {"v_range": [["6.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "revoked-server-cert": {"v_range": [["6.4.0", ""]], "choices": ["allow", "block", "ignore"], "type": "str"},
            "unsupported-ssl-cipher": {"v_range": [["6.4.0", ""]], "choices": ["allow", "block"], "type": "str"},
            "unsupported-ssl-negotiation": {"v_range": [["6.4.0", ""]], "choices": ["allow", "block"], "type": "str"},
            "unsupported-ssl-version": {"v_range": [["7.0.1", ""]], "choices": ["block", "allow", "inspect"], "type": "str"},
            "min-allowed-ssl-version": {"v_range": [["7.0.3", ""]], "choices": ["ssl-3.0", "tls-1.0", "tls-1.1", "tls-1.2", "tls-1.3"], "type": "str"}
        }
    },
    "firewall_sslsshprofile_pop3s": {
        "stated": false,
        "options": {
            "allow-invalid-server-cert": {"v_range": [["6.0.0", "7.2.1"]], "choices": ["disable", "enable"], "type": "str"},
            "client-cert-request": {"choices": ["bypass", "inspect", "block"], "type": "str"},
            "ports": {"type": "raw"},
            "status": {"choices": ["disable", "deep-inspection"], "type": "str"},
            "unsupported-ssl": {"choices": ["bypass", "inspect", "block"], "type": "str"},
            "untrusted-cert": {"v_range": [["6.0.0", "7.2.1"]], "choices": ["allow", "block", "ignore"], "type": "str"},
            "invalid-server-cert": {"v_range": [["6.2.0", ""]], "choices": ["allow", "block"], "type": "str"},
            "sni-server-cert-check": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable", "strict"], "type": "str"},
            "untrusted-server-cert": {"v_range": [["6.2.0", ""]], "choices": ["allow", "block", "ignore"], "type": "str"},
            "cert-validation-failure": {"v_range": [["6.4.0", ""]], "choices": ["allow", "block", "ignore"], "type": "str"},
            "cert-validation-timeout": {"v_range": [["6.4.0", ""]], "choices": ["allow", "block", "ignore"], "type": "str"},
            "client-certificate": {"v_range": [["6.4.0", ""]], "choices": ["bypass", "inspect", "block"], "type": "str"},
            "expired-server-cert": {"v_range": [["6.4.0", ""]], "choices": ["allow", "block", "ignore"], "type": "str"},
            "proxy-after-tcp-handshake": {"v_range": [["6.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "revoked-server-cert": {"v_range": [["6.4.0", ""]], "choices": ["allow", "block", "ignore"], "type": "str"},
            "unsupported-ssl-cipher": {"v_range": [["6.4.0", ""]], "choices": ["allow", "block"], "type": "str"},
            "unsupported-ssl-negotiation": {"v_range": [["6.4.0", ""]], "choices": ["allow", "block"], "type": "str"},
            "unsupported-ssl-version": {"v_range": [["7.0.1", ""]], "choices": ["block", "allow", "inspect"], "type": "str"},
            "min-allowed-ssl-version": {"v_range": [["7.0.3", ""]], "choices": ["ssl-3.0", "tls-1.0", "tls-1.1", "tls-1.2", "tls-1.3"], "type": "str"}
        }
    },
    "firewall_sslsshprofile_smtps": {
        "stated": false,
        "options": {
            "allow-invalid-server-cert": {"v_range": [["6.0.0", "7.2.1"]], "choices": ["disable", "enable"], "type": "str"},
            "client-cert-request": {"choices": ["bypass", "inspect", "block"], "type": "str"},
            "ports": {"type": "raw"},
            "status": {"choices": ["disable", "deep-inspection"], "type": "str"},
            "unsupported-ssl": {"choices": ["bypass", "inspect", "block"], "type": "str"},
            "untrusted-cert": {"v_range": [["6.0.0", "7.2.1"]], "choices": ["allow", "block", "ignore"], "type": "str"},
            "invalid-server-cert": {"v_range": [["6.2.0", ""]], "choices": ["allow", "block"], "type": "str"},
            "sni-server-cert-check": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable", "strict"], "type": "str"},
            "untrusted-server-cert": {"v_range": [["6.2.0", ""]], "choices": ["allow", "block", "ignore"], "type": "str"},
            "cert-validation-failure": {"v_range": [["6.4.0", ""]], "choices": ["allow", "block", "ignore"], "type": "str"},
            "cert-validation-timeout": {"v_range": [["6.4.0", ""]], "choices": ["allow", "block", "ignore"], "type": "str"},
            "client-certificate": {"v_range": [["6.4.0", ""]], "choices": ["bypass", "inspect", "block"], "type": "str"},
            "expired-server-cert": {"v_range": [["6.4.0", ""]], "choices": ["allow", "block", "ignore"], "type": "str"},
            "proxy-after-tcp-handshake": {"v_range": [["6.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "revoked-server-cert": {"v_range": [["6.4.0", ""]], "choices": ["allow", "block", "ignore"], "type": "str"},
            "unsupported-ssl-cipher": {"v_range": [["6.4.0", ""]], "choices": ["allow", "block"], "type": "str"},
            "unsupported-ssl-negotiation": {"v_range": [["6.4.0", ""]], "choices": ["allow", "block"], "type": "str"},
            "unsupported-ssl-version": {"v_range": [["7.0.1", ""]], "choices": ["block", "allow", "inspect"], "type": "str"},
            "min-allowed-ssl-version": {"v_range": [["7.0.3", ""]], "choices": ["ssl-3.0", "tls-1.0", "tls-1.1", "tls-1.2", "tls-1.3"], "type": "str"}
        }
    },
    "firewall_sslsshprofile_ssh": {
        "stated": false,
        "options": {
            "inspect-all": {"choices": ["disable", "deep-inspection"], "type": "str"},
            "ports": {"type": "raw"},
            "ssh-algorithm": {"choices": ["compatible", "high-encryption"], "type": "str"},
            "ssh-policy-check": {"v_range": [["6.0.0", "7.2.1"]], "choices": ["disable", "enable"], "type": "str"},
            "ssh-tun-policy-check": {"choices": ["disable", "enable"], "type": "str"},
            "status": {"choices": ["disable", "deep-inspection"], "type": "str"},
            "unsupported-version": {"choices": ["block", "bypass"], "type": "str"},
            "block": {"v_range": [["6.2.0", "6.4.14"]], "type": "list", "choices": ["x11-filter", "ssh-shell", "exec", "port-forward"], "elements": "str"},
            "log": {"v_range": [["6.2.0", "6.4.14"]], "type": "list", "choices": ["x11-filter", "ssh-shell", "exec", "port-forward"], "elements": "str"},
            "proxy-after-tcp-handshake": {"v_range": [["6.4.0", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "firewall_sslsshprofile_ssl": {
        "stated": false,
        "options": {
            "allow-invalid-server-cert": {"v_range": [["6.0.0", "7.2.1"]], "choices": ["disable", "enable"], "type": "str"},
            "client-cert-request": {"choices": ["bypass", "inspect", "block"], "type": "str"},
            "inspect-all": {"choices": ["disable", "certificate-inspection", "deep-inspection"], "type": "str"},
            "unsupported-ssl": {"choices": ["bypass", "inspect", "block"], "type": "str"},
            "untrusted-cert": {"v_range": [["6.0.0", "7.2.1"]], "choices": ["allow", "block", "ignore"], "type": "str"},
            "invalid-server-cert": {"v_range": [["6.2.0", ""]], "choices": ["allow", "block"], "type": "str"},
            "sni-server-cert-check": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable", "strict"], "type": "str"},
            "untrusted-server-cert": {"v_range": [["6.2.0", ""]], "choices": ["allow", "block", "ignore"], "type": "str"},
            "cert-validation-failure": {"v_range": [["6.4.0", ""]], "choices": ["allow", "block", "ignore"], "type": "str"},
            "cert-validation-timeout": {"v_range": [["6.4.0", ""]], "choices": ["allow", "block", "ignore"], "type": "str"},
            "client-certificate": {"v_range": [["6.4.0", ""]], "choices": ["bypass", "inspect", "block"], "type": "str"},
            "expired-server-cert": {"v_range": [["6.4.0", ""]], "choices": ["allow", "block", "ignore"], "type": "str"},
            "revoked-server-cert": {"v_range": [["6.4.0", ""]], "choices": ["allow", "block", "ignore"], "type": "str"},
            "unsupported-ssl-cipher": {"v_range": [["6.4.0", ""]], "choices": ["allow", "block"], "type": "str"},
            "unsupported-ssl-negotiation": {"v_range": [["6.4.0", ""]], "choices": ["allow", "block"], "type": "str"},
            "cert-probe-failure": {"v_range": [["7.0.1", ""]], "choices": ["block", "allow"], "type": "str"},
            "min-allowed-ssl-version": {"v_range": [["7.0.3", ""]], "choices": ["ssl-3.0", "tls-1.0", "tls-1.1", "tls-1.2", "tls-1.3"], "type": "str"},
            "unsupported-ssl-version": {"v_range": [["7.0.1", ""]], "choices": ["block", "allow", "inspect"], "type": "str"},
            "encrypted-client-hello": {"v_range": [["7.4.3", ""]], "choices": ["block", "allow"], "type": "str"}
        }
    },
    "firewall_sslsshprofile_sslexempt": {
        "stated": true,
        "options": {
            "address": {"type": "str"},
            "address6": {"type": "str"},
            "fortiguard-category": {"type": "str"},
            "id": {"required": true, "type": "int"},
            "regex": {"type": "str"},
            "type": {"choices": ["fortiguard-category", "address", "address6", "wildcard-fqdn", "regex", "finger-print"], "type": "str"},
            "wildcard-fqdn": {"type": "str"}
        }
    },
    "firewall_sslsshprofile_sslserver": {
        "stated": true,
        "options": {
            "ftps-client-cert-request": {"choices": ["bypass", "inspect", "block"], "type": "str"},
            "https-client-cert-request": {"choices": ["bypass", "inspect", "block"], "type": "str"},
            "id": {"required": true, "type": "int"},
            "imaps-client-cert-request": {"choices": ["bypass", "inspect", "block"], "type": "str"},
            "ip": {"type": "str"},
            "pop3s-client-cert-request": {"choices": ["bypass", "inspect", "block"], "type": "str"},
            "smtps-client-cert-request": {"choices": ["bypass", "inspect", "block"], "type": "str"},
            "ssl-other-client-cert-request": {"choices": ["bypass", "inspect", "block"], "type": "str"},
            "ftps-client-certificate": {"v_range": [["6.4.0", ""]], "choices": ["bypass", "inspect", "block"], "type": "str"},
            "https-client-certificate": {"v_range": [["6.4.0", ""]], "choices": ["bypass", "inspect", "block"], "type": "str"},
            "imaps-client-certificate": {"v_range": [["6.4.0", ""]], "choices": ["bypass", "inspect", "block"], "type": "str"},
            "pop3s-client-certificate": {"v_range": [["6.4.0", ""]], "choices": ["bypass", "inspect", "block"], "type": "str"},
            "smtps-client-certificate": {"v_range": [["6.4.0", ""]], "choices": ["bypass", "inspect", "block"], "type": "str"},
            "ssl-other-client-certificate": {"v_range": [["6.4.0", ""]], "choices": ["bypass", "inspect", "block"], "type": "str"}
        }
    },
    "firewall_trafficclass": {
        "stated": true,
        "options": {"class-id": {"v_range": [["6.2.2", ""]], "type": "int"}, "class-name": {"v_range": [["6.2.2", ""]], "type": "str"}}
    },
    "firewall_vendormac": {
        "stated": false,
        "options": {
            "id": {"v_range": [["7.2.4", "7.2.5"], ["7.4.1", ""]], "type": "int"},
            "mac-number": {"v_range": [["7.2.4", "7.2.5"], ["7.4.1", ""]], "type": "int"},
            "name": {"v_range": [["7.2.4", "7.2.5"], ["7.4.1", ""]], "type": "str"},
            "obsolete": {"v_range": [["7.2.4", "7.2.5"], ["7.4.1", ""]], "type": "int"}
        }
    },
    "firewall_vip": {
        "stated": true,
        "options": {
            "arp-reply": {"choices": ["disable", "enable"], "type": "str"},
            "color": {"type": "int"},
            "comment": {"type": "str"},
            "dns-mapping-ttl": {"type": "int"},
            "dynamic_mapping": {
                "type": "list",
                "options": {
                    "_scope": {"type": "list", "options": {"name": {"type": "str"}, "vdom": {"type": "str"}}, "elements": "dict"},
                    "arp-reply": {"choices": ["disable", "enable"], "type": "str"},
                    "color": {"type": "int"},
                    "comment": {"type": "str"},
                    "dns-mapping-ttl": {"type": "int"},
                    "extaddr": {"type": "raw"},
                    "extintf": {"type": "str"},
                    "extip": {"type": "str"},
                    "extport": {"type": "str"},
                    "gratuitous-arp-interval": {"type": "int"},
                    "http-cookie-age": {"type": "int"},
                    "http-cookie-domain": {"type": "str"},
                    "http-cookie-domain-from-host": {"choices": ["disable", "enable"], "type": "str"},
                    "http-cookie-generation": {"type": "int"},
                    "http-cookie-path": {"type": "str"},
                    "http-cookie-share": {"choices": ["disable", "same-ip"], "type": "str"},
                    "http-ip-header": {"choices": ["disable", "enable"], "type": "str"},
                    "http-ip-header-name": {"type": "str"},
                    "http-multiplex": {"choices": ["disable", "enable"], "type": "str"},
                    "https-cookie-secure": {"choices": ["disable", "enable"], "type": "str"},
                    "id": {"type": "int"},
                    "ldb-method": {
                        "choices": ["static", "round-robin", "weighted", "least-session", "least-rtt", "first-alive", "http-host"],
                        "type": "str"
                    },
                    "mapped-addr": {"type": "str"},
                    "mappedip": {"type": "raw"},
                    "mappedport": {"type": "str"},
                    "max-embryonic-connections": {"type": "int"},
                    "monitor": {"type": "raw"},
                    "nat-source-vip": {"choices": ["disable", "enable"], "type": "str"},
                    "outlook-web-access": {"choices": ["disable", "enable"], "type": "str"},
                    "persistence": {"choices": ["none", "http-cookie", "ssl-session-id"], "type": "str"},
                    "portforward": {"choices": ["disable", "enable"], "type": "str"},
                    "portmapping-type": {"choices": ["1-to-1", "m-to-n"], "type": "str"},
                    "protocol": {"choices": ["tcp", "udp", "sctp", "icmp"], "type": "str"},
                    "realservers": {
                        "type": "list",
                        "options": {
                            "client-ip": {"type": "raw"},
                            "healthcheck": {"choices": ["disable", "enable", "vip"], "type": "str"},
                            "holddown-interval": {"type": "int"},
                            "http-host": {"type": "str"},
                            "ip": {"type": "str"},
                            "max-connections": {"type": "int"},
                            "monitor": {"type": "raw"},
                            "port": {"type": "int"},
                            "seq": {"type": "int"},
                            "status": {"choices": ["active", "standby", "disable"], "type": "str"},
                            "weight": {"type": "int"},
                            "address": {"v_range": [["6.4.0", ""]], "type": "str"},
                            "id": {"v_range": [["6.4.0", ""]], "type": "int"},
                            "type": {"v_range": [["6.4.0", ""]], "choices": ["ip", "address"], "type": "str"},
                            "translate-host": {"v_range": [["7.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "health-check-proto": {"v_range": [["7.2.3", ""]], "choices": ["ping", "http"], "type": "str"}
                        },
                        "elements": "dict"
                    },
                    "server-type": {"choices": ["http", "https", "ssl", "tcp", "udp", "ip", "imaps", "pop3s", "smtps", "ssh"], "type": "str"},
                    "service": {"type": "raw"},
                    "src-filter": {"type": "raw"},
                    "srcintf-filter": {"type": "raw"},
                    "ssl-algorithm": {"choices": ["high", "medium", "low", "custom"], "type": "str"},
                    "ssl-certificate": {"type": "str"},
                    "ssl-cipher-suites": {
                        "type": "list",
                        "options": {
                            "cipher": {
                                "choices": [
                                    "TLS-RSA-WITH-RC4-128-MD5", "TLS-RSA-WITH-RC4-128-SHA", "TLS-RSA-WITH-DES-CBC-SHA", "TLS-RSA-WITH-3DES-EDE-CBC-SHA",
                                    "TLS-RSA-WITH-AES-128-CBC-SHA", "TLS-RSA-WITH-AES-256-CBC-SHA", "TLS-RSA-WITH-AES-128-CBC-SHA256",
                                    "TLS-RSA-WITH-AES-256-CBC-SHA256", "TLS-RSA-WITH-CAMELLIA-128-CBC-SHA", "TLS-RSA-WITH-CAMELLIA-256-CBC-SHA",
                                    "TLS-RSA-WITH-CAMELLIA-128-CBC-SHA256", "TLS-RSA-WITH-CAMELLIA-256-CBC-SHA256", "TLS-RSA-WITH-SEED-CBC-SHA",
                                    "TLS-RSA-WITH-ARIA-128-CBC-SHA256", "TLS-RSA-WITH-ARIA-256-CBC-SHA384", "TLS-DHE-RSA-WITH-DES-CBC-SHA",
                                    "TLS-DHE-RSA-WITH-3DES-EDE-CBC-SHA", "TLS-DHE-RSA-WITH-AES-128-CBC-SHA", "TLS-DHE-RSA-WITH-AES-256-CBC-SHA",
                                    "TLS-DHE-RSA-WITH-AES-128-CBC-SHA256", "TLS-DHE-RSA-WITH-AES-256-CBC-SHA256",
                                    "TLS-DHE-RSA-WITH-CAMELLIA-128-CBC-SHA", "TLS-DHE-RSA-WITH-CAMELLIA-256-CBC-SHA",
                                    "TLS-DHE-RSA-WITH-CAMELLIA-128-CBC-SHA256", "TLS-DHE-RSA-WITH-CAMELLIA-256-CBC-SHA256",
                                    "TLS-DHE-RSA-WITH-SEED-CBC-SHA", "TLS-DHE-RSA-WITH-ARIA-128-CBC-SHA256", "TLS-DHE-RSA-WITH-ARIA-256-CBC-SHA384",
                                    "TLS-ECDHE-RSA-WITH-RC4-128-SHA", "TLS-ECDHE-RSA-WITH-3DES-EDE-CBC-SHA", "TLS-ECDHE-RSA-WITH-AES-128-CBC-SHA",
                                    "TLS-ECDHE-RSA-WITH-AES-256-CBC-SHA", "TLS-ECDHE-RSA-WITH-CHACHA20-POLY1305-SHA256",
                                    "TLS-ECDHE-ECDSA-WITH-CHACHA20-POLY1305-SHA256", "TLS-DHE-RSA-WITH-CHACHA20-POLY1305-SHA256",
                                    "TLS-DHE-RSA-WITH-AES-128-GCM-SHA256", "TLS-DHE-RSA-WITH-AES-256-GCM-SHA384", "TLS-DHE-DSS-WITH-AES-128-CBC-SHA",
                                    "TLS-DHE-DSS-WITH-AES-256-CBC-SHA", "TLS-DHE-DSS-WITH-AES-128-CBC-SHA256", "TLS-DHE-DSS-WITH-AES-128-GCM-SHA256",
                                    "TLS-DHE-DSS-WITH-AES-256-CBC-SHA256", "TLS-DHE-DSS-WITH-AES-256-GCM-SHA384",
                                    "TLS-ECDHE-RSA-WITH-AES-128-CBC-SHA256", "TLS-ECDHE-RSA-WITH-AES-128-GCM-SHA256",
                                    "TLS-ECDHE-RSA-WITH-AES-256-CBC-SHA384", "TLS-ECDHE-RSA-WITH-AES-256-GCM-SHA384",
                                    "TLS-ECDHE-ECDSA-WITH-AES-128-CBC-SHA", "TLS-ECDHE-ECDSA-WITH-AES-128-CBC-SHA256",
                                    "TLS-ECDHE-ECDSA-WITH-AES-128-GCM-SHA256", "TLS-ECDHE-ECDSA-WITH-AES-256-CBC-SHA384",
                                    "TLS-ECDHE-ECDSA-WITH-AES-256-GCM-SHA384", "TLS-RSA-WITH-AES-128-GCM-SHA256", "TLS-RSA-WITH-AES-256-GCM-SHA384",
                                    "TLS-DHE-DSS-WITH-CAMELLIA-128-CBC-SHA", "TLS-DHE-DSS-WITH-CAMELLIA-256-CBC-SHA",
                                    "TLS-DHE-DSS-WITH-CAMELLIA-128-CBC-SHA256", "TLS-DHE-DSS-WITH-CAMELLIA-256-CBC-SHA256",
                                    "TLS-DHE-DSS-WITH-SEED-CBC-SHA", "TLS-DHE-DSS-WITH-ARIA-128-CBC-SHA256", "TLS-DHE-DSS-WITH-ARIA-256-CBC-SHA384",
                                    "TLS-ECDHE-RSA-WITH-ARIA-128-CBC-SHA256", "TLS-ECDHE-RSA-WITH-ARIA-256-CBC-SHA384",
                                    "TLS-ECDHE-ECDSA-WITH-ARIA-128-CBC-SHA256", "TLS-ECDHE-ECDSA-WITH-ARIA-256-CBC-SHA384",
                                    "TLS-DHE-DSS-WITH-3DES-EDE-CBC-SHA", "TLS-DHE-DSS-WITH-DES-CBC-SHA", "TLS-AES-128-GCM-SHA256",
                                    "TLS-AES-256-GCM-SHA384", "TLS-CHACHA20-POLY1305-SHA256", "TLS-ECDHE-ECDSA-WITH-AES-256-CBC-SHA"
                                ],
                                "type": "str"
                            },
                            "id": {"type": "int"},
                            "versions": {"type": "list", "choices": ["ssl-3.0", "tls-1.0", "tls-1.1", "tls-1.2", "tls-1.3"], "elements": "str"},
                            "priority": {"v_range": [["6.4.0", ""]], "type": "int"}
                        },
                        "elements": "dict"
                    },
                    "ssl-client-fallback": {"choices": ["disable", "enable"], "type": "str"},
                    "ssl-client-renegotiation": {"choices": ["deny", "allow", "secure"], "type": "str"},
                    "ssl-client-session-state-max": {"type": "int"},
                    "ssl-client-session-state-timeout": {"type": "int"},
                    "ssl-client-session-state-type": {"choices": ["disable", "time", "count", "both"], "type": "str"},
                    "ssl-dh-bits": {"choices": ["768", "1024", "1536", "2048", "3072", "4096"], "type": "str"},
                    "ssl-hpkp": {"choices": ["disable", "enable", "report-only"], "type": "str"},
                    "ssl-hpkp-age": {"type": "int"},
                    "ssl-hpkp-backup": {"type": "str"},
                    "ssl-hpkp-include-subdomains": {"choices": ["disable", "enable"], "type": "str"},
                    "ssl-hpkp-primary": {"type": "str"},
                    "ssl-hpkp-report-uri": {"type": "str"},
                    "ssl-hsts": {"choices": ["disable", "enable"], "type": "str"},
                    "ssl-hsts-age": {"type": "int"},
                    "ssl-hsts-include-subdomains": {"choices": ["disable", "enable"], "type": "str"},
                    "ssl-http-location-conversion": {"choices": ["disable", "enable"], "type": "str"},
                    "ssl-http-match-host": {"choices": ["disable", "enable"], "type": "str"},
                    "ssl-max-version": {"choices": ["ssl-3.0", "tls-1.0", "tls-1.1", "tls-1.2", "tls-1.3"], "type": "str"},
                    "ssl-min-version": {"choices": ["ssl-3.0", "tls-1.0", "tls-1.1", "tls-1.2", "tls-1.3"], "type": "str"},
                    "ssl-mode": {"choices": ["half", "full"], "type": "str"},
                    "ssl-pfs": {"choices": ["require", "deny", "allow"], "type": "str"},
                    "ssl-send-empty-frags": {"choices": ["disable", "enable"], "type": "str"},
                    "ssl-server-algorithm": {"choices": ["high", "low", "medium", "custom", "client"], "type": "str"},
                    "ssl-server-max-version": {"choices": ["ssl-3.0", "tls-1.0", "tls-1.1", "tls-1.2", "client", "tls-1.3"], "type": "str"},
                    "ssl-server-min-version": {"choices": ["ssl-3.0", "tls-1.0", "tls-1.1", "tls-1.2", "client", "tls-1.3"], "type": "str"},
                    "ssl-server-session-state-max": {"type": "int"},
                    "ssl-server-session-state-timeout": {"type": "int"},
                    "ssl-server-session-state-type": {"choices": ["disable", "time", "count", "both"], "type": "str"},
                    "type": {"choices": ["static-nat", "load-balance", "server-load-balance", "dns-translation", "fqdn", "access-proxy"], "type": "str"},
                    "uuid": {"type": "str"},
                    "weblogic-server": {"choices": ["disable", "enable"], "type": "str"},
                    "websphere-server": {"choices": ["disable", "enable"], "type": "str"},
                    "http-redirect": {"v_range": [["6.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "ssl-client-rekey-count": {"v_range": [["6.2.1", ""]], "no_log": true, "type": "int"},
                    "status": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "add-nat46-route": {"v_range": [["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "ipv6-mappedip": {"v_range": [["7.0.1", ""]], "type": "str"},
                    "ipv6-mappedport": {"v_range": [["7.0.1", ""]], "type": "str"},
                    "nat44": {"v_range": [["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "nat46": {"v_range": [["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "ssl-accept-ffdhe-groups": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "http-multiplex-max-request": {"v_range": [["7.2.2", ""]], "type": "int"},
                    "http-multiplex-ttl": {"v_range": [["7.2.2", ""]], "type": "int"},
                    "http-supported-max-version": {"v_range": [["7.2.2", ""]], "choices": ["http1", "http2"], "type": "str"},
                    "ssl-server-renegotiation": {"v_range": [["7.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "h2-support": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "h3-support": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "http-multiplex-max-concurrent-request": {"v_range": [["7.4.1", ""]], "type": "int"},
                    "gslb-domain-name": {"v_range": [["7.4.2", ""]], "type": "str"},
                    "gslb-hostname": {"v_range": [["7.4.2", ""]], "type": "str"},
                    "one-click-gslb-server": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "src-vip-filter": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"}
                },
                "elements": "dict"
            },
            "extaddr": {"type": "raw"},
            "extintf": {"type": "str"},
            "extip": {"type": "str"},
            "extport": {"type": "str"},
            "gratuitous-arp-interval": {"type": "int"},
            "http-cookie-age": {"type": "int"},
            "http-cookie-domain": {"type": "str"},
            "http-cookie-domain-from-host": {"choices": ["disable", "enable"], "type": "str"},
            "http-cookie-generation": {"type": "int"},
            "http-cookie-path": {"type": "str"},
            "http-cookie-share": {"choices": ["disable", "same-ip"], "type": "str"},
            "http-ip-header": {"choices": ["disable", "enable"], "type": "str"},
            "http-ip-header-name": {"type": "str"},
            "http-multiplex": {"choices": ["disable", "enable"], "type": "str"},
            "https-cookie-secure": {"choices": ["disable", "enable"], "type": "str"},
            "id": {"type": "int"},
            "ldb-method": {"choices": ["static", "round-robin", "weighted", "least-session", "least-rtt", "first-alive", "http-host"], "type": "str"},
            "mapped-addr": {"type": "str"},
            "mappedip": {"type": "raw"},
            "mappedport": {"type": "str"},
            "max-embryonic-connections": {"type": "int"},
            "monitor": {"type": "raw"},
            "name": {"required": true, "type": "str"},
            "nat-source-vip": {"choices": ["disable", "enable"], "type": "str"},
            "outlook-web-access": {"choices": ["disable", "enable"], "type": "str"},
            "persistence": {"choices": ["none", "http-cookie", "ssl-session-id"], "type": "str"},
            "portforward": {"choices": ["disable", "enable"], "type": "str"},
            "portmapping-type": {"choices": ["1-to-1", "m-to-n"], "type": "str"},
            "protocol": {"choices": ["tcp", "udp", "sctp", "icmp"], "type": "str"},
            "realservers": {
                "type": "list",
                "options": {
                    "client-ip": {"type": "raw"},
                    "healthcheck": {"choices": ["disable", "enable", "vip"], "type": "str"},
                    "holddown-interval": {"type": "int"},
                    "http-host": {"type": "str"},
                    "ip": {"type": "str"},
                    "max-connections": {"type": "int"},
                    "monitor": {"type": "raw"},
                    "port": {"type": "int"},
                    "seq": {"type": "int"},
                    "status": {"choices": ["active", "standby", "disable"], "type": "str"},
                    "weight": {"type": "int"},
                    "address": {"v_range": [["6.4.0", ""]], "type": "str"},
                    "id": {"v_range": [["6.4.0", ""]], "type": "int"},
                    "type": {"v_range": [["6.4.0", ""]], "choices": ["ip", "address"], "type": "str"},
                    "translate-host": {"v_range": [["7.2.2", ""]], "choices": ["disable", "enable"], "type": "str"}
                },
                "elements": "dict"
            },
            "server-type": {"choices": ["http", "https", "ssl", "tcp", "udp", "ip", "imaps", "pop3s", "smtps", "ssh"], "type": "str"},
            "service": {"type": "raw"},
            "src-filter": {"type": "raw"},
            "srcintf-filter": {"type": "raw"},
            "ssl-algorithm": {"choices": ["high", "medium", "low", "custom"], "type": "str"},
            "ssl-certificate": {"type": "str"},
            "ssl-cipher-suites": {
                "type": "list",
                "options": {
                    "cipher": {
                        "choices": [
                            "TLS-RSA-WITH-RC4-128-MD5", "TLS-RSA-WITH-RC4-128-SHA", "TLS-RSA-WITH-DES-CBC-SHA", "TLS-RSA-WITH-3DES-EDE-CBC-SHA",
                            "TLS-RSA-WITH-AES-128-CBC-SHA", "TLS-RSA-WITH-AES-256-CBC-SHA", "TLS-RSA-WITH-AES-128-CBC-SHA256",
                            "TLS-RSA-WITH-AES-256-CBC-SHA256", "TLS-RSA-WITH-CAMELLIA-128-CBC-SHA", "TLS-RSA-WITH-CAMELLIA-256-CBC-SHA",
                            "TLS-RSA-WITH-CAMELLIA-128-CBC-SHA256", "TLS-RSA-WITH-CAMELLIA-256-CBC-SHA256", "TLS-RSA-WITH-SEED-CBC-SHA",
                            "TLS-RSA-WITH-ARIA-128-CBC-SHA256", "TLS-RSA-WITH-ARIA-256-CBC-SHA384", "TLS-DHE-RSA-WITH-DES-CBC-SHA",
                            "TLS-DHE-RSA-WITH-3DES-EDE-CBC-SHA", "TLS-DHE-RSA-WITH-AES-128-CBC-SHA", "TLS-DHE-RSA-WITH-AES-256-CBC-SHA",
                            "TLS-DHE-RSA-WITH-AES-128-CBC-SHA256", "TLS-DHE-RSA-WITH-AES-256-CBC-SHA256", "TLS-DHE-RSA-WITH-CAMELLIA-128-CBC-SHA",
                            "TLS-DHE-RSA-WITH-CAMELLIA-256-CBC-SHA", "TLS-DHE-RSA-WITH-CAMELLIA-128-CBC-SHA256",
                            "TLS-DHE-RSA-WITH-CAMELLIA-256-CBC-SHA256", "TLS-DHE-RSA-WITH-SEED-CBC-SHA", "TLS-DHE-RSA-WITH-ARIA-128-CBC-SHA256",
                            "TLS-DHE-RSA-WITH-ARIA-256-CBC-SHA384", "TLS-ECDHE-RSA-WITH-RC4-128-SHA", "TLS-ECDHE-RSA-WITH-3DES-EDE-CBC-SHA",
                            "TLS-ECDHE-RSA-WITH-AES-128-CBC-SHA", "TLS-ECDHE-RSA-WITH-AES-256-CBC-SHA", "TLS-ECDHE-RSA-WITH-CHACHA20-POLY1305-SHA256",
                            "TLS-ECDHE-ECDSA-WITH-CHACHA20-POLY1305-SHA256", "TLS-DHE-RSA-WITH-CHACHA20-POLY1305-SHA256",
                            "TLS-DHE-RSA-WITH-AES-128-GCM-SHA256", "TLS-DHE-RSA-WITH-AES-256-GCM-SHA384", "TLS-DHE-DSS-WITH-AES-128-CBC-SHA",
                            "TLS-DHE-DSS-WITH-AES-256-CBC-SHA", "TLS-DHE-DSS-WITH-AES-128-CBC-SHA256", "TLS-DHE-DSS-WITH-AES-128-GCM-SHA256",
                            "TLS-DHE-DSS-WITH-AES-256-CBC-SHA256", "TLS-DHE-DSS-WITH-AES-256-GCM-SHA384", "TLS-ECDHE-RSA-WITH-AES-128-CBC-SHA256",
                            "TLS-ECDHE-RSA-WITH-AES-128-GCM-SHA256", "TLS-ECDHE-RSA-WITH-AES-256-CBC-SHA384", "TLS-ECDHE-RSA-WITH-AES-256-GCM-SHA384",
                            "TLS-ECDHE-ECDSA-WITH-AES-128-CBC-SHA", "TLS-ECDHE-ECDSA-WITH-AES-128-CBC-SHA256", "TLS-ECDHE-ECDSA-WITH-AES-128-GCM-SHA256",
                            "TLS-ECDHE-ECDSA-WITH-AES-256-CBC-SHA384", "TLS-ECDHE-ECDSA-WITH-AES-256-GCM-SHA384", "TLS-RSA-WITH-AES-128-GCM-SHA256",
                            "TLS-RSA-WITH-AES-256-GCM-SHA384", "TLS-DHE-DSS-WITH-CAMELLIA-128-CBC-SHA", "TLS-DHE-DSS-WITH-CAMELLIA-256-CBC-SHA",
                            "TLS-DHE-DSS-WITH-CAMELLIA-128-CBC-SHA256", "TLS-DHE-DSS-WITH-CAMELLIA-256-CBC-SHA256", "TLS-DHE-DSS-WITH-SEED-CBC-SHA",
                            "TLS-DHE-DSS-WITH-ARIA-128-CBC-SHA256", "TLS-DHE-DSS-WITH-ARIA-256-CBC-SHA384", "TLS-ECDHE-RSA-WITH-ARIA-128-CBC-SHA256",
                            "TLS-ECDHE-RSA-WITH-ARIA-256-CBC-SHA384", "TLS-ECDHE-ECDSA-WITH-ARIA-128-CBC-SHA256",
                            "TLS-ECDHE-ECDSA-WITH-ARIA-256-CBC-SHA384", "TLS-DHE-DSS-WITH-3DES-EDE-CBC-SHA", "TLS-DHE-DSS-WITH-DES-CBC-SHA",
                            "TLS-AES-128-GCM-SHA256", "TLS-AES-256-GCM-SHA384", "TLS-CHACHA20-POLY1305-SHA256", "TLS-ECDHE-ECDSA-WITH-AES-256-CBC-SHA"
                        ],
                        "type": "str"
                    },
                    "id": {"type": "int"},
                    "versions": {"type": "list", "choices": ["ssl-3.0", "tls-1.0", "tls-1.1", "tls-1.2", "tls-1.3"], "elements": "str"},
                    "priority": {"v_range": [["6.4.0", ""]], "type": "int"}
                },
                "elements": "dict"
            },
            "ssl-client-fallback": {"choices": ["disable", "enable"], "type": "str"},
            "ssl-client-renegotiation": {"choices": ["deny", "allow", "secure"], "type": "str"},
            "ssl-client-session-state-max": {"type": "int"},
            "ssl-client-session-state-timeout": {"type": "int"},
            "ssl-client-session-state-type": {"choices": ["disable", "time", "count", "both"], "type": "str"},
            "ssl-dh-bits": {"choices": ["768", "1024", "1536", "2048", "3072", "4096"], "type": "str"},
            "ssl-hpkp": {"choices": ["disable", "enable", "report-only"], "type": "str"},
            "ssl-hpkp-age": {"type": "int"},
            "ssl-hpkp-backup": {"type": "str"},
            "ssl-hpkp-include-subdomains": {"choices": ["disable", "enable"], "type": "str"},
            "ssl-hpkp-primary": {"type": "str"},
            "ssl-hpkp-report-uri": {"type": "str"},
            "ssl-hsts": {"choices": ["disable", "enable"], "type": "str"},
            "ssl-hsts-age": {"type": "int"},
            "ssl-hsts-include-subdomains": {"choices": ["disable", "enable"], "type": "str"},
            "ssl-http-location-conversion": {"choices": ["disable", "enable"], "type": "str"},
            "ssl-http-match-host": {"choices": ["disable", "enable"], "type": "str"},
            "ssl-max-version": {"choices": ["ssl-3.0", "tls-1.0", "tls-1.1", "tls-1.2", "tls-1.3"], "type": "str"},
            "ssl-min-version": {"choices": ["ssl-3.0", "tls-1.0", "tls-1.1", "tls-1.2", "tls-1.3"], "type": "str"},
            "ssl-mode": {"choices": ["half", "full"], "type": "str"},
            "ssl-pfs": {"choices": ["require", "deny", "allow"], "type": "str"},
            "ssl-send-empty-frags": {"choices": ["disable", "enable"], "type": "str"},
            "ssl-server-algorithm": {"choices": ["high", "low", "medium", "custom", "client"], "type": "str"},
            "ssl-server-cipher-suites": {
                "type": "list",
                "options": {
                    "cipher": {
                        "choices": [
                            "TLS-RSA-WITH-RC4-128-MD5", "TLS-RSA-WITH-RC4-128-SHA", "TLS-RSA-WITH-DES-CBC-SHA", "TLS-RSA-WITH-3DES-EDE-CBC-SHA",
                            "TLS-RSA-WITH-AES-128-CBC-SHA", "TLS-RSA-WITH-AES-256-CBC-SHA", "TLS-RSA-WITH-AES-128-CBC-SHA256",
                            "TLS-RSA-WITH-AES-256-CBC-SHA256", "TLS-RSA-WITH-CAMELLIA-128-CBC-SHA", "TLS-RSA-WITH-CAMELLIA-256-CBC-SHA",
                            "TLS-RSA-WITH-CAMELLIA-128-CBC-SHA256", "TLS-RSA-WITH-CAMELLIA-256-CBC-SHA256", "TLS-RSA-WITH-SEED-CBC-SHA",
                            "TLS-RSA-WITH-ARIA-128-CBC-SHA256", "TLS-RSA-WITH-ARIA-256-CBC-SHA384", "TLS-DHE-RSA-WITH-DES-CBC-SHA",
                            "TLS-DHE-RSA-WITH-3DES-EDE-CBC-SHA", "TLS-DHE-RSA-WITH-AES-128-CBC-SHA", "TLS-DHE-RSA-WITH-AES-256-CBC-SHA",
                            "TLS-DHE-RSA-WITH-AES-128-CBC-SHA256", "TLS-DHE-RSA-WITH-AES-256-CBC-SHA256", "TLS-DHE-RSA-WITH-CAMELLIA-128-CBC-SHA",
                            "TLS-DHE-RSA-WITH-CAMELLIA-256-CBC-SHA", "TLS-DHE-RSA-WITH-CAMELLIA-128-CBC-SHA256",
                            "TLS-DHE-RSA-WITH-CAMELLIA-256-CBC-SHA256", "TLS-DHE-RSA-WITH-SEED-CBC-SHA", "TLS-DHE-RSA-WITH-ARIA-128-CBC-SHA256",
                            "TLS-DHE-RSA-WITH-ARIA-256-CBC-SHA384", "TLS-ECDHE-RSA-WITH-RC4-128-SHA", "TLS-ECDHE-RSA-WITH-3DES-EDE-CBC-SHA",
                            "TLS-ECDHE-RSA-WITH-AES-128-CBC-SHA", "TLS-ECDHE-RSA-WITH-AES-256-CBC-SHA", "TLS-ECDHE-RSA-WITH-CHACHA20-POLY1305-SHA256",
                            "TLS-ECDHE-ECDSA-WITH-CHACHA20-POLY1305-SHA256", "TLS-DHE-RSA-WITH-CHACHA20-POLY1305-SHA256",
                            "TLS-DHE-RSA-WITH-AES-128-GCM-SHA256", "TLS-DHE-RSA-WITH-AES-256-GCM-SHA384", "TLS-DHE-DSS-WITH-AES-128-CBC-SHA",
                            "TLS-DHE-DSS-WITH-AES-256-CBC-SHA", "TLS-DHE-DSS-WITH-AES-128-CBC-SHA256", "TLS-DHE-DSS-WITH-AES-128-GCM-SHA256",
                            "TLS-DHE-DSS-WITH-AES-256-CBC-SHA256", "TLS-DHE-DSS-WITH-AES-256-GCM-SHA384", "TLS-ECDHE-RSA-WITH-AES-128-CBC-SHA256",
                            "TLS-ECDHE-RSA-WITH-AES-128-GCM-SHA256", "TLS-ECDHE-RSA-WITH-AES-256-CBC-SHA384", "TLS-ECDHE-RSA-WITH-AES-256-GCM-SHA384",
                            "TLS-ECDHE-ECDSA-WITH-AES-128-CBC-SHA", "TLS-ECDHE-ECDSA-WITH-AES-128-CBC-SHA256", "TLS-ECDHE-ECDSA-WITH-AES-128-GCM-SHA256",
                            "TLS-ECDHE-ECDSA-WITH-AES-256-CBC-SHA384", "TLS-ECDHE-ECDSA-WITH-AES-256-GCM-SHA384", "TLS-RSA-WITH-AES-128-GCM-SHA256",
                            "TLS-RSA-WITH-AES-256-GCM-SHA384", "TLS-DHE-DSS-WITH-CAMELLIA-128-CBC-SHA", "TLS-DHE-DSS-WITH-CAMELLIA-256-CBC-SHA",
                            "TLS-DHE-DSS-WITH-CAMELLIA-128-CBC-SHA256", "TLS-DHE-DSS-WITH-CAMELLIA-256-CBC-SHA256", "TLS-DHE-DSS-WITH-SEED-CBC-SHA",
                            "TLS-DHE-DSS-WITH-ARIA-128-CBC-SHA256", "TLS-DHE-DSS-WITH-ARIA-256-CBC-SHA384", "TLS-ECDHE-RSA-WITH-ARIA-128-CBC-SHA256",
                            "TLS-ECDHE-RSA-WITH-ARIA-256-CBC-SHA384", "TLS-ECDHE-ECDSA-WITH-ARIA-128-CBC-SHA256",
                            "TLS-ECDHE-ECDSA-WITH-ARIA-256-CBC-SHA384", "TLS-DHE-DSS-WITH-3DES-EDE-CBC-SHA", "TLS-DHE-DSS-WITH-DES-CBC-SHA",
                            "TLS-AES-128-GCM-SHA256", "TLS-AES-256-GCM-SHA384", "TLS-CHACHA20-POLY1305-SHA256", "TLS-ECDHE-ECDSA-WITH-AES-256-CBC-SHA"
                        ],
                        "type": "str"
                    },
                    "priority": {"type": "int"},
                    "versions": {"type": "list", "choices": ["ssl-3.0", "tls-1.0", "tls-1.1", "tls-1.2", "tls-1.3"], "elements": "str"}
                },
                "elements": "dict"
            },
            "ssl-server-max-version": {"choices": ["ssl-3.0", "tls-1.0", "tls-1.1", "tls-1.2", "client", "tls-1.3"], "type": "str"},
            "ssl-server-min-version": {"choices": ["ssl-3.0", "tls-1.0", "tls-1.1", "tls-1.2", "client", "tls-1.3"], "type": "str"},
            "ssl-server-session-state-max": {"type": "int"},
            "ssl-server-session-state-timeout": {"type": "int"},
            "ssl-server-session-state-type": {"choices": ["disable", "time", "count", "both"], "type": "str"},
            "type": {"choices": ["static-nat", "load-balance", "server-load-balance", "dns-translation", "fqdn", "access-proxy"], "type": "str"},
            "uuid": {"type": "str"},
            "weblogic-server": {"choices": ["disable", "enable"], "type": "str"},
            "websphere-server": {"choices": ["disable", "enable"], "type": "str"},
            "http-redirect": {"v_range": [["6.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ssl-client-rekey-count": {"v_range": [["6.2.1", ""]], "no_log": true, "type": "int"},
            "status": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "add-nat46-route": {"v_range": [["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ipv6-mappedip": {"v_range": [["7.0.1", ""]], "type": "str"},
            "ipv6-mappedport": {"v_range": [["7.0.1", ""]], "type": "str"},
            "nat44": {"v_range": [["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "nat46": {"v_range": [["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ssl-accept-ffdhe-groups": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "http-multiplex-max-request": {"v_range": [["7.2.2", ""]], "type": "int"},
            "http-multiplex-ttl": {"v_range": [["7.2.2", ""]], "type": "int"},
            "http-supported-max-version": {"v_range": [["7.2.2", ""]], "choices": ["http1", "http2"], "type": "str"},
            "ssl-server-renegotiation": {"v_range": [["7.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "h2-support": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "h3-support": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "http-multiplex-max-concurrent-request": {"v_range": [["7.4.1", ""]], "type": "int"},
            "quic": {
                "v_range": [["7.4.1", ""]],
                "type": "dict",
                "options": {
                    "ack-delay-exponent": {"v_range": [["7.4.1", ""]], "type": "int"},
                    "active-connection-id-limit": {"v_range": [["7.4.1", ""]], "type": "int"},
                    "active-migration": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "grease-quic-bit": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "max-ack-delay": {"v_range": [["7.4.1", ""]], "type": "int"},
                    "max-datagram-frame-size": {"v_range": [["7.4.1", ""]], "type": "int"},
                    "max-idle-timeout": {"v_range": [["7.4.1", ""]], "type": "int"},
                    "max-udp-payload-size": {"v_range": [["7.4.1", ""]], "type": "int"}
                }
            },
            "gslb-domain-name": {"v_range": [["7.4.2", ""]], "type": "str"},
            "gslb-hostname": {"v_range": [["7.4.2", ""]], "type": "str"},
            "gslb-public-ips": {
                "v_range": [["7.4.2", ""]],
                "type": "list",
                "options": {"index": {"v_range": [["7.4.2", ""]], "type": "int"}, "ip": {"v_range": [["7.4.2", ""]], "type": "str"}},
                "elements": "dict"
            },
            "one-click-gslb-server": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "src-vip-filter": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "firewall_vip46": {
        "stated": true,
        "options": {
            "arp-reply": {"choices": ["disable", "enable"], "type": "str"},
            "color": {"type": "int"},
            "comment": {"type": "str"},
            "dynamic_mapping": {
                "type": "list",
                "options": {
                    "_scope": {"type": "list", "options": {"name": {"type": "str"}, "vdom": {"type": "str"}}, "elements": "dict"},
                    "arp-reply": {"choices": ["disable", "enable"], "type": "str"},
                    "color": {"type": "int"},
                    "comment": {"type": "str"},
                    "extip": {"type": "str"},
                    "extport": {"type": "str"},
                    "id": {"type": "int"},
                    "ldb-method": {"choices": ["static", "round-robin", "weighted", "least-session", "least-rtt", "first-alive"], "type": "str"},
                    "mappedip": {"type": "str"},
                    "mappedport": {"type": "str"},
                    "monitor": {"type": "raw"},
                    "portforward": {"choices": ["disable", "enable"], "type": "str"},
                    "protocol": {"choices": ["tcp", "udp"], "type": "str"},
                    "server-type": {"choices": ["http", "tcp", "udp", "ip"], "type": "str"},
                    "src-filter": {"type": "raw"},
                    "type": {"choices": ["static-nat", "server-load-balance"], "type": "str"},
                    "uuid": {"type": "str"},
                    "srcintf-filter": {"v_range": [["6.2.7", "6.2.12"], ["6.4.4", ""]], "type": "raw"}
                },
                "elements": "dict"
            },
            "extip": {"type": "str"},
            "extport": {"type": "str"},
            "id": {"type": "int"},
            "ldb-method": {"choices": ["static", "round-robin", "weighted", "least-session", "least-rtt", "first-alive"], "type": "str"},
            "mappedip": {"type": "str"},
            "mappedport": {"type": "str"},
            "monitor": {"type": "raw"},
            "name": {"required": true, "type": "str"},
            "portforward": {"choices": ["disable", "enable"], "type": "str"},
            "protocol": {"choices": ["tcp", "udp"], "type": "str"},
            "realservers": {
                "type": "list",
                "options": {
                    "client-ip": {"type": "str"},
                    "healthcheck": {"choices": ["disable", "enable", "vip"], "type": "str"},
                    "holddown-interval": {"type": "int"},
                    "id": {"type": "int"},
                    "ip": {"type": "str"},
                    "max-connections": {"type": "int"},
                    "monitor": {"type": "raw"},
                    "port": {"type": "int"},
                    "status": {"choices": ["active", "standby", "disable"], "type": "str"},
                    "weight": {"type": "int"}
                },
                "elements": "dict"
            },
            "server-type": {"choices": ["http", "tcp", "udp", "ip"], "type": "str"},
            "src-filter": {"type": "raw"},
            "type": {"choices": ["static-nat", "server-load-balance"], "type": "str"},
            "uuid": {"type": "str"},
            "srcintf-filter": {"v_range": [["6.2.7", "6.2.12"], ["6.4.4", ""]], "type": "raw"}
        }
    },
    "firewall_vip46_dynamicmapping": {
        "stated": true,
        "options": {
            "_scope": {"type": "list", "options": {"name": {"type": "str"}, "vdom": {"type": "str"}}, "elements": "dict"},
            "arp-reply": {"choices": ["disable", "enable"], "type": "str"},
            "color": {"type": "int"},
            "comment": {"type": "str"},
            "extip": {"type": "str"},
            "extport": {"type": "str"},
            "id": {"type": "int"},
            "ldb-method": {"choices": ["static", "round-robin", "weighted", "least-session", "least-rtt", "first-alive"], "type": "str"},
            "mappedip": {"type": "str"},
            "mappedport": {"type": "str"},
            "monitor": {"type": "raw"},
            "portforward": {"choices": ["disable", "enable"], "type": "str"},
            "protocol": {"choices": ["tcp", "udp"], "type": "str"},
            "server-type": {"choices": ["http", "tcp", "udp", "ip"], "type": "str"},
            "src-filter": {"type": "raw"},
            "type": {"choices": ["static-nat", "server-load-balance"], "type": "str"},
            "uuid": {"type": "str"},
            "srcintf-filter": {"v_range": [["6.2.7", "6.2.12"], ["6.4.4", ""]], "type": "raw"}
        }
    },
    "firewall_vip46_realservers": {
        "stated": true,
        "options": {
            "client-ip": {"type": "str"},
            "healthcheck": {"choices": ["disable", "enable", "vip"], "type": "str"},
            "holddown-interval": {"type": "int"},
            "id": {"required": true, "type": "int"},
            "ip": {"type": "str"},
            "max-connections": {"type": "int"},
            "monitor": {"type": "raw"},
            "port": {"type": "int"},
            "status": {"choices": ["active", "standby", "disable"], "type": "str"},
            "weight": {"type": "int"}
        }
    },
    "firewall_vip6": {
        "stated": true,
        "options": {
            "arp-reply": {"choices": ["disable", "enable"], "type": "str"},
            "color": {"type": "int"},
            "comment": {"type": "str"},
            "dynamic_mapping": {
                "type": "list",
                "options": {
                    "_scope": {"type": "list", "options": {"name": {"type": "str"}, "vdom": {"type": "str"}}, "elements": "dict"},
                    "arp-reply": {"choices": ["disable", "enable"], "type": "str"},
                    "color": {"type": "int"},
                    "comment": {"type": "str"},
                    "extip": {"type": "str"},
                    "extport": {"type": "str"},
                    "http-cookie-age": {"type": "int"},
                    "http-cookie-domain": {"type": "str"},
                    "http-cookie-domain-from-host": {"choices": ["disable", "enable"], "type": "str"},
                    "http-cookie-generation": {"type": "int"},
                    "http-cookie-path": {"type": "str"},
                    "http-cookie-share": {"choices": ["disable", "same-ip"], "type": "str"},
                    "http-ip-header": {"choices": ["disable", "enable"], "type": "str"},
                    "http-ip-header-name": {"type": "str"},
                    "http-multiplex": {"choices": ["disable", "enable"], "type": "str"},
                    "https-cookie-secure": {"choices": ["disable", "enable"], "type": "str"},
                    "id": {"type": "int"},
                    "ldb-method": {
                        "choices": ["static", "round-robin", "weighted", "least-session", "least-rtt", "first-alive", "http-host"],
                        "type": "str"
                    },
                    "mappedip": {"type": "str"},
                    "mappedport": {"type": "str"},
                    "max-embryonic-connections": {"type": "int"},
                    "monitor": {"type": "raw"},
                    "outlook-web-access": {"choices": ["disable", "enable"], "type": "str"},
                    "persistence": {"choices": ["none", "http-cookie", "ssl-session-id"], "type": "str"},
                    "portforward": {"choices": ["disable", "enable"], "type": "str"},
                    "protocol": {"choices": ["tcp", "udp", "sctp"], "type": "str"},
                    "server-type": {"choices": ["http", "https", "ssl", "tcp", "udp", "ip", "imaps", "pop3s", "smtps"], "type": "str"},
                    "src-filter": {"type": "raw"},
                    "ssl-algorithm": {"choices": ["high", "low", "medium", "custom"], "type": "str"},
                    "ssl-certificate": {"type": "str"},
                    "ssl-client-fallback": {"choices": ["disable", "enable"], "type": "str"},
                    "ssl-client-renegotiation": {"choices": ["deny", "allow", "secure"], "type": "str"},
                    "ssl-client-session-state-max": {"type": "int"},
                    "ssl-client-session-state-timeout": {"type": "int"},
                    "ssl-client-session-state-type": {"choices": ["disable", "time", "count", "both"], "type": "str"},
                    "ssl-dh-bits": {"choices": ["768", "1024", "1536", "2048", "3072", "4096"], "type": "str"},
                    "ssl-hpkp": {"choices": ["disable", "enable", "report-only"], "type": "str"},
                    "ssl-hpkp-age": {"type": "int"},
                    "ssl-hpkp-backup": {"type": "str"},
                    "ssl-hpkp-include-subdomains": {"choices": ["disable", "enable"], "type": "str"},
                    "ssl-hpkp-primary": {"type": "str"},
                    "ssl-hpkp-report-uri": {"type": "str"},
                    "ssl-hsts": {"choices": ["disable", "enable"], "type": "str"},
                    "ssl-hsts-age": {"type": "int"},
                    "ssl-hsts-include-subdomains": {"choices": ["disable", "enable"], "type": "str"},
                    "ssl-http-location-conversion": {"choices": ["disable", "enable"], "type": "str"},
                    "ssl-http-match-host": {"choices": ["disable", "enable"], "type": "str"},
                    "ssl-max-version": {"choices": ["ssl-3.0", "tls-1.0", "tls-1.1", "tls-1.2", "tls-1.3"], "type": "str"},
                    "ssl-min-version": {"choices": ["ssl-3.0", "tls-1.0", "tls-1.1", "tls-1.2", "tls-1.3"], "type": "str"},
                    "ssl-mode": {"choices": ["half", "full"], "type": "str"},
                    "ssl-pfs": {"choices": ["require", "deny", "allow"], "type": "str"},
                    "ssl-send-empty-frags": {"choices": ["disable", "enable"], "type": "str"},
                    "ssl-server-algorithm": {"choices": ["high", "low", "medium", "custom", "client"], "type": "str"},
                    "ssl-server-max-version": {"choices": ["ssl-3.0", "tls-1.0", "tls-1.1", "tls-1.2", "client", "tls-1.3"], "type": "str"},
                    "ssl-server-min-version": {"choices": ["ssl-3.0", "tls-1.0", "tls-1.1", "tls-1.2", "client", "tls-1.3"], "type": "str"},
                    "ssl-server-session-state-max": {"type": "int"},
                    "ssl-server-session-state-timeout": {"type": "int"},
                    "ssl-server-session-state-type": {"choices": ["disable", "time", "count", "both"], "type": "str"},
                    "type": {"choices": ["static-nat", "server-load-balance", "access-proxy"], "type": "str"},
                    "uuid": {"type": "str"},
                    "weblogic-server": {"choices": ["disable", "enable"], "type": "str"},
                    "websphere-server": {"choices": ["disable", "enable"], "type": "str"},
                    "http-redirect": {"v_range": [["6.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "ssl-client-rekey-count": {"v_range": [["6.2.1", ""]], "no_log": true, "type": "int"},
                    "nat-source-vip": {"v_range": [["6.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "add-nat64-route": {"v_range": [["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "embedded-ipv4-address": {"v_range": [["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "ipv4-mappedip": {"v_range": [["7.0.1", ""]], "type": "str"},
                    "ipv4-mappedport": {"v_range": [["7.0.1", ""]], "type": "str"},
                    "nat64": {"v_range": [["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "nat66": {"v_range": [["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "realservers": {
                        "v_range": [["7.0.2", ""]],
                        "type": "list",
                        "options": {
                            "client-ip": {"v_range": [["7.0.2", ""]], "type": "str"},
                            "healthcheck": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable", "vip"], "type": "str"},
                            "holddown-interval": {"v_range": [["7.0.2", ""]], "type": "int"},
                            "http-host": {"v_range": [["7.0.2", ""]], "type": "str"},
                            "id": {"v_range": [["7.0.2", ""]], "type": "int"},
                            "ip": {"v_range": [["7.0.2", ""]], "type": "str"},
                            "max-connections": {"v_range": [["7.0.2", ""]], "type": "int"},
                            "monitor": {"v_range": [["7.0.2", ""]], "type": "raw"},
                            "port": {"v_range": [["7.0.2", ""]], "type": "int"},
                            "status": {"v_range": [["7.0.2", ""]], "choices": ["active", "standby", "disable"], "type": "str"},
                            "weight": {"v_range": [["7.0.2", ""]], "type": "int"},
                            "translate-host": {"v_range": [["7.2.2", ""]], "choices": ["disable", "enable"], "type": "str"}
                        },
                        "elements": "dict"
                    },
                    "ssl-accept-ffdhe-groups": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "ssl-cipher-suites": {
                        "v_range": [["7.0.2", ""]],
                        "type": "list",
                        "options": {
                            "cipher": {
                                "v_range": [["7.0.2", ""]],
                                "choices": [
                                    "TLS-RSA-WITH-RC4-128-MD5", "TLS-RSA-WITH-RC4-128-SHA", "TLS-RSA-WITH-DES-CBC-SHA", "TLS-RSA-WITH-3DES-EDE-CBC-SHA",
                                    "TLS-RSA-WITH-AES-128-CBC-SHA", "TLS-RSA-WITH-AES-256-CBC-SHA", "TLS-RSA-WITH-AES-128-CBC-SHA256",
                                    "TLS-RSA-WITH-AES-256-CBC-SHA256", "TLS-RSA-WITH-CAMELLIA-128-CBC-SHA", "TLS-RSA-WITH-CAMELLIA-256-CBC-SHA",
                                    "TLS-RSA-WITH-CAMELLIA-128-CBC-SHA256", "TLS-RSA-WITH-CAMELLIA-256-CBC-SHA256", "TLS-RSA-WITH-SEED-CBC-SHA",
                                    "TLS-RSA-WITH-ARIA-128-CBC-SHA256", "TLS-RSA-WITH-ARIA-256-CBC-SHA384", "TLS-DHE-RSA-WITH-DES-CBC-SHA",
                                    "TLS-DHE-RSA-WITH-3DES-EDE-CBC-SHA", "TLS-DHE-RSA-WITH-AES-128-CBC-SHA", "TLS-DHE-RSA-WITH-AES-256-CBC-SHA",
                                    "TLS-DHE-RSA-WITH-AES-128-CBC-SHA256", "TLS-DHE-RSA-WITH-AES-256-CBC-SHA256",
                                    "TLS-DHE-RSA-WITH-CAMELLIA-128-CBC-SHA", "TLS-DHE-RSA-WITH-CAMELLIA-256-CBC-SHA",
                                    "TLS-DHE-RSA-WITH-CAMELLIA-128-CBC-SHA256", "TLS-DHE-RSA-WITH-CAMELLIA-256-CBC-SHA256",
                                    "TLS-DHE-RSA-WITH-SEED-CBC-SHA", "TLS-DHE-RSA-WITH-ARIA-128-CBC-SHA256", "TLS-DHE-RSA-WITH-ARIA-256-CBC-SHA384",
                                    "TLS-ECDHE-RSA-WITH-RC4-128-SHA", "TLS-ECDHE-RSA-WITH-3DES-EDE-CBC-SHA", "TLS-ECDHE-RSA-WITH-AES-128-CBC-SHA",
                                    "TLS-ECDHE-RSA-WITH-AES-256-CBC-SHA", "TLS-ECDHE-RSA-WITH-CHACHA20-POLY1305-SHA256",
                                    "TLS-ECDHE-ECDSA-WITH-CHACHA20-POLY1305-SHA256", "TLS-DHE-RSA-WITH-CHACHA20-POLY1305-SHA256",
                                    "TLS-DHE-RSA-WITH-AES-128-GCM-SHA256", "TLS-DHE-RSA-WITH-AES-256-GCM-SHA384", "TLS-DHE-DSS-WITH-AES-128-CBC-SHA",
                                    "TLS-DHE-DSS-WITH-AES-256-CBC-SHA", "TLS-DHE-DSS-WITH-AES-128-CBC-SHA256", "TLS-DHE-DSS-WITH-AES-128-GCM-SHA256",
                                    "TLS-DHE-DSS-WITH-AES-256-CBC-SHA256", "TLS-DHE-DSS-WITH-AES-256-GCM-SHA384",
                                    "TLS-ECDHE-RSA-WITH-AES-128-CBC-SHA256", "TLS-ECDHE-RSA-WITH-AES-128-GCM-SHA256",
                                    "TLS-ECDHE-RSA-WITH-AES-256-CBC-SHA384", "TLS-ECDHE-RSA-WITH-AES-256-GCM-SHA384",
                                    "TLS-ECDHE-ECDSA-WITH-AES-128-CBC-SHA", "TLS-ECDHE-ECDSA-WITH-AES-128-CBC-SHA256",
                                    "TLS-ECDHE-ECDSA-WITH-AES-128-GCM-SHA256", "TLS-ECDHE-ECDSA-WITH-AES-256-CBC-SHA384",
                                    "TLS-ECDHE-ECDSA-WITH-AES-256-GCM-SHA384", "TLS-RSA-WITH-AES-128-GCM-SHA256", "TLS-RSA-WITH-AES-256-GCM-SHA384",
                                    "TLS-DHE-DSS-WITH-CAMELLIA-128-CBC-SHA", "TLS-DHE-DSS-WITH-CAMELLIA-256-CBC-SHA",
                                    "TLS-DHE-DSS-WITH-CAMELLIA-128-CBC-SHA256", "TLS-DHE-DSS-WITH-CAMELLIA-256-CBC-SHA256",
                                    "TLS-DHE-DSS-WITH-SEED-CBC-SHA", "TLS-DHE-DSS-WITH-ARIA-128-CBC-SHA256", "TLS-DHE-DSS-WITH-ARIA-256-CBC-SHA384",
                                    "TLS-ECDHE-RSA-WITH-ARIA-128-CBC-SHA256", "TLS-ECDHE-RSA-WITH-ARIA-256-CBC-SHA384",
                                    "TLS-ECDHE-ECDSA-WITH-ARIA-128-CBC-SHA256", "TLS-ECDHE-ECDSA-WITH-ARIA-256-CBC-SHA384",
                                    "TLS-DHE-DSS-WITH-3DES-EDE-CBC-SHA", "TLS-DHE-DSS-WITH-DES-CBC-SHA", "TLS-AES-128-GCM-SHA256",
                                    "TLS-AES-256-GCM-SHA384", "TLS-CHACHA20-POLY1305-SHA256", "TLS-ECDHE-ECDSA-WITH-AES-256-CBC-SHA"
                                ],
                                "type": "str"
                            },
                            "priority": {"v_range": [["7.0.2", ""]], "type": "int"},
                            "versions": {
                                "v_range": [["7.0.2", ""]],
                                "type": "list",
                                "choices": ["ssl-3.0", "tls-1.0", "tls-1.1", "tls-1.2", "tls-1.3"],
                                "elements": "str"
                            }
                        },
                        "elements": "dict"
                    },
                    "ndp-reply": {"v_range": [["7.0.5", "7.0.12"], ["7.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "ssl-server-renegotiation": {"v_range": [["7.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "h2-support": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "h3-support": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "src-vip-filter": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"}
                },
                "elements": "dict"
            },
            "extip": {"type": "str"},
            "extport": {"type": "str"},
            "http-cookie-age": {"type": "int"},
            "http-cookie-domain": {"type": "str"},
            "http-cookie-domain-from-host": {"choices": ["disable", "enable"], "type": "str"},
            "http-cookie-generation": {"type": "int"},
            "http-cookie-path": {"type": "str"},
            "http-cookie-share": {"choices": ["disable", "same-ip"], "type": "str"},
            "http-ip-header": {"choices": ["disable", "enable"], "type": "str"},
            "http-ip-header-name": {"type": "str"},
            "http-multiplex": {"choices": ["disable", "enable"], "type": "str"},
            "https-cookie-secure": {"choices": ["disable", "enable"], "type": "str"},
            "id": {"type": "int"},
            "ldb-method": {"choices": ["static", "round-robin", "weighted", "least-session", "least-rtt", "first-alive", "http-host"], "type": "str"},
            "mappedip": {"type": "str"},
            "mappedport": {"type": "str"},
            "max-embryonic-connections": {"type": "int"},
            "monitor": {"type": "raw"},
            "name": {"required": true, "type": "str"},
            "outlook-web-access": {"choices": ["disable", "enable"], "type": "str"},
            "persistence": {"choices": ["none", "http-cookie", "ssl-session-id"], "type": "str"},
            "portforward": {"choices": ["disable", "enable"], "type": "str"},
            "protocol": {"choices": ["tcp", "udp", "sctp"], "type": "str"},
            "realservers": {
                "type": "list",
                "options": {
                    "client-ip": {"type": "str"},
                    "healthcheck": {"choices": ["disable", "enable", "vip"], "type": "str"},
                    "holddown-interval": {"type": "int"},
                    "http-host": {"type": "str"},
                    "id": {"type": "int"},
                    "ip": {"type": "str"},
                    "max-connections": {"type": "int"},
                    "monitor": {"type": "raw"},
                    "port": {"type": "int"},
                    "status": {"choices": ["active", "standby", "disable"], "type": "str"},
                    "weight": {"type": "int"},
                    "translate-host": {"v_range": [["7.2.2", ""]], "choices": ["disable", "enable"], "type": "str"}
                },
                "elements": "dict"
            },
            "server-type": {"choices": ["http", "https", "ssl", "tcp", "udp", "ip", "imaps", "pop3s", "smtps"], "type": "str"},
            "src-filter": {"type": "raw"},
            "ssl-algorithm": {"choices": ["high", "low", "medium", "custom"], "type": "str"},
            "ssl-certificate": {"type": "str"},
            "ssl-cipher-suites": {
                "type": "list",
                "options": {
                    "cipher": {
                        "choices": [
                            "TLS-RSA-WITH-RC4-128-MD5", "TLS-RSA-WITH-RC4-128-SHA", "TLS-RSA-WITH-DES-CBC-SHA", "TLS-RSA-WITH-3DES-EDE-CBC-SHA",
                            "TLS-RSA-WITH-AES-128-CBC-SHA", "TLS-RSA-WITH-AES-256-CBC-SHA", "TLS-RSA-WITH-AES-128-CBC-SHA256",
                            "TLS-RSA-WITH-AES-256-CBC-SHA256", "TLS-RSA-WITH-CAMELLIA-128-CBC-SHA", "TLS-RSA-WITH-CAMELLIA-256-CBC-SHA",
                            "TLS-RSA-WITH-CAMELLIA-128-CBC-SHA256", "TLS-RSA-WITH-CAMELLIA-256-CBC-SHA256", "TLS-RSA-WITH-SEED-CBC-SHA",
                            "TLS-RSA-WITH-ARIA-128-CBC-SHA256", "TLS-RSA-WITH-ARIA-256-CBC-SHA384", "TLS-DHE-RSA-WITH-DES-CBC-SHA",
                            "TLS-DHE-RSA-WITH-3DES-EDE-CBC-SHA", "TLS-DHE-RSA-WITH-AES-128-CBC-SHA", "TLS-DHE-RSA-WITH-AES-256-CBC-SHA",
                            "TLS-DHE-RSA-WITH-AES-128-CBC-SHA256", "TLS-DHE-RSA-WITH-AES-256-CBC-SHA256", "TLS-DHE-RSA-WITH-CAMELLIA-128-CBC-SHA",
                            "TLS-DHE-RSA-WITH-CAMELLIA-256-CBC-SHA", "TLS-DHE-RSA-WITH-CAMELLIA-128-CBC-SHA256",
                            "TLS-DHE-RSA-WITH-CAMELLIA-256-CBC-SHA256", "TLS-DHE-RSA-WITH-SEED-CBC-SHA", "TLS-DHE-RSA-WITH-ARIA-128-CBC-SHA256",
                            "TLS-DHE-RSA-WITH-ARIA-256-CBC-SHA384", "TLS-ECDHE-RSA-WITH-RC4-128-SHA", "TLS-ECDHE-RSA-WITH-3DES-EDE-CBC-SHA",
                            "TLS-ECDHE-RSA-WITH-AES-128-CBC-SHA", "TLS-ECDHE-RSA-WITH-AES-256-CBC-SHA", "TLS-ECDHE-RSA-WITH-CHACHA20-POLY1305-SHA256",
                            "TLS-ECDHE-ECDSA-WITH-CHACHA20-POLY1305-SHA256", "TLS-DHE-RSA-WITH-CHACHA20-POLY1305-SHA256",
                            "TLS-DHE-RSA-WITH-AES-128-GCM-SHA256", "TLS-DHE-RSA-WITH-AES-256-GCM-SHA384", "TLS-DHE-DSS-WITH-AES-128-CBC-SHA",
                            "TLS-DHE-DSS-WITH-AES-256-CBC-SHA", "TLS-DHE-DSS-WITH-AES-128-CBC-SHA256", "TLS-DHE-DSS-WITH-AES-128-GCM-SHA256",
                            "TLS-DHE-DSS-WITH-AES-256-CBC-SHA256", "TLS-DHE-DSS-WITH-AES-256-GCM-SHA384", "TLS-ECDHE-RSA-WITH-AES-128-CBC-SHA256",
                            "TLS-ECDHE-RSA-WITH-AES-128-GCM-SHA256", "TLS-ECDHE-RSA-WITH-AES-256-CBC-SHA384", "TLS-ECDHE-RSA-WITH-AES-256-GCM-SHA384",
                            "TLS-ECDHE-ECDSA-WITH-AES-128-CBC-SHA", "TLS-ECDHE-ECDSA-WITH-AES-128-CBC-SHA256", "TLS-ECDHE-ECDSA-WITH-AES-128-GCM-SHA256",
                            "TLS-ECDHE-ECDSA-WITH-AES-256-CBC-SHA384", "TLS-ECDHE-ECDSA-WITH-AES-256-GCM-SHA384", "TLS-RSA-WITH-AES-128-GCM-SHA256",
                            "TLS-RSA-WITH-AES-256-GCM-SHA384", "TLS-DHE-DSS-WITH-CAMELLIA-128-CBC-SHA", "TLS-DHE-DSS-WITH-CAMELLIA-256-CBC-SHA",
                            "TLS-DHE-DSS-WITH-CAMELLIA-128-CBC-SHA256", "TLS-DHE-DSS-WITH-CAMELLIA-256-CBC-SHA256", "TLS-DHE-DSS-WITH-SEED-CBC-SHA",
                            "TLS-DHE-DSS-WITH-ARIA-128-CBC-SHA256", "TLS-DHE-DSS-WITH-ARIA-256-CBC-SHA384", "TLS-ECDHE-RSA-WITH-ARIA-128-CBC-SHA256",
                            "TLS-ECDHE-RSA-WITH-ARIA-256-CBC-SHA384", "TLS-ECDHE-ECDSA-WITH-ARIA-128-CBC-SHA256",
                            "TLS-ECDHE-ECDSA-WITH-ARIA-256-CBC-SHA384", "TLS-DHE-DSS-WITH-3DES-EDE-CBC-SHA", "TLS-DHE-DSS-WITH-DES-CBC-SHA",
                            "TLS-AES-128-GCM-SHA256", "TLS-AES-256-GCM-SHA384", "TLS-CHACHA20-POLY1305-SHA256", "TLS-ECDHE-ECDSA-WITH-AES-256-CBC-SHA"
                        ],
                        "type": "str"
                    },
                    "priority": {"type": "int"},
                    "versions": {"type": "list", "choices": ["ssl-3.0", "tls-1.0", "tls-1.1", "tls-1.2", "tls-1.3"], "elements": "str"}
                },
                "elements": "dict"
            },
            "ssl-client-fallback": {"choices": ["disable", "enable"], "type": "str"},
            "ssl-client-renegotiation": {"choices": ["deny", "allow", "secure"], "type": "str"},
            "ssl-client-session-state-max": {"type": "int"},
            "ssl-client-session-state-timeout": {"type": "int"},
            "ssl-client-session-state-type": {"choices": ["disable", "time", "count", "both"], "type": "str"},
            "ssl-dh-bits": {"choices": ["768", "1024", "1536", "2048", "3072", "4096"], "type": "str"},
            "ssl-hpkp": {"choices": ["disable", "enable", "report-only"], "type": "str"},
            "ssl-hpkp-age": {"type": "int"},
            "ssl-hpkp-backup": {"type": "str"},
            "ssl-hpkp-include-subdomains": {"choices": ["disable", "enable"], "type": "str"},
            "ssl-hpkp-primary": {"type": "str"},
            "ssl-hpkp-report-uri": {"type": "str"},
            "ssl-hsts": {"choices": ["disable", "enable"], "type": "str"},
            "ssl-hsts-age": {"type": "int"},
            "ssl-hsts-include-subdomains": {"choices": ["disable", "enable"], "type": "str"},
            "ssl-http-location-conversion": {"choices": ["disable", "enable"], "type": "str"},
            "ssl-http-match-host": {"choices": ["disable", "enable"], "type": "str"},
            "ssl-max-version": {"choices": ["ssl-3.0", "tls-1.0", "tls-1.1", "tls-1.2", "tls-1.3"], "type": "str"},
            "ssl-min-version": {"choices": ["ssl-3.0", "tls-1.0", "tls-1.1", "tls-1.2", "tls-1.3"], "type": "str"},
            "ssl-mode": {"choices": ["half", "full"], "type": "str"},
            "ssl-pfs": {"choices": ["require", "deny", "allow"], "type": "str"},
            "ssl-send-empty-frags": {"choices": ["disable", "enable"], "type": "str"},
            "ssl-server-algorithm": {"choices": ["high", "low", "medium", "custom", "client"], "type": "str"},
            "ssl-server-cipher-suites": {
                "type": "list",
                "options": {
                    "cipher": {
                        "choices": [
                            "TLS-RSA-WITH-RC4-128-MD5", "TLS-RSA-WITH-RC4-128-SHA", "TLS-RSA-WITH-DES-CBC-SHA", "TLS-RSA-WITH-3DES-EDE-CBC-SHA",
                            "TLS-RSA-WITH-AES-128-CBC-SHA", "TLS-RSA-WITH-AES-256-CBC-SHA", "TLS-RSA-WITH-AES-128-CBC-SHA256",
                            "TLS-RSA-WITH-AES-256-CBC-SHA256", "TLS-RSA-WITH-CAMELLIA-128-CBC-SHA", "TLS-RSA-WITH-CAMELLIA-256-CBC-SHA",
                            "TLS-RSA-WITH-CAMELLIA-128-CBC-SHA256", "TLS-RSA-WITH-CAMELLIA-256-CBC-SHA256", "TLS-RSA-WITH-SEED-CBC-SHA",
                            "TLS-RSA-WITH-ARIA-128-CBC-SHA256", "TLS-RSA-WITH-ARIA-256-CBC-SHA384", "TLS-DHE-RSA-WITH-DES-CBC-SHA",
                            "TLS-DHE-RSA-WITH-3DES-EDE-CBC-SHA", "TLS-DHE-RSA-WITH-AES-128-CBC-SHA", "TLS-DHE-RSA-WITH-AES-256-CBC-SHA",
                            "TLS-DHE-RSA-WITH-AES-128-CBC-SHA256", "TLS-DHE-RSA-WITH-AES-256-CBC-SHA256", "TLS-DHE-RSA-WITH-CAMELLIA-128-CBC-SHA",
                            "TLS-DHE-RSA-WITH-CAMELLIA-256-CBC-SHA", "TLS-DHE-RSA-WITH-CAMELLIA-128-CBC-SHA256",
                            "TLS-DHE-RSA-WITH-CAMELLIA-256-CBC-SHA256", "TLS-DHE-RSA-WITH-SEED-CBC-SHA", "TLS-DHE-RSA-WITH-ARIA-128-CBC-SHA256",
                            "TLS-DHE-RSA-WITH-ARIA-256-CBC-SHA384", "TLS-ECDHE-RSA-WITH-RC4-128-SHA", "TLS-ECDHE-RSA-WITH-3DES-EDE-CBC-SHA",
                            "TLS-ECDHE-RSA-WITH-AES-128-CBC-SHA", "TLS-ECDHE-RSA-WITH-AES-256-CBC-SHA", "TLS-ECDHE-RSA-WITH-CHACHA20-POLY1305-SHA256",
                            "TLS-ECDHE-ECDSA-WITH-CHACHA20-POLY1305-SHA256", "TLS-DHE-RSA-WITH-CHACHA20-POLY1305-SHA256",
                            "TLS-DHE-RSA-WITH-AES-128-GCM-SHA256", "TLS-DHE-RSA-WITH-AES-256-GCM-SHA384", "TLS-DHE-DSS-WITH-AES-128-CBC-SHA",
                            "TLS-DHE-DSS-WITH-AES-256-CBC-SHA", "TLS-DHE-DSS-WITH-AES-128-CBC-SHA256", "TLS-DHE-DSS-WITH-AES-128-GCM-SHA256",
                            "TLS-DHE-DSS-WITH-AES-256-CBC-SHA256", "TLS-DHE-DSS-WITH-AES-256-GCM-SHA384", "TLS-ECDHE-RSA-WITH-AES-128-CBC-SHA256",
                            "TLS-ECDHE-RSA-WITH-AES-128-GCM-SHA256", "TLS-ECDHE-RSA-WITH-AES-256-CBC-SHA384", "TLS-ECDHE-RSA-WITH-AES-256-GCM-SHA384",
                            "TLS-ECDHE-ECDSA-WITH-AES-128-CBC-SHA", "TLS-ECDHE-ECDSA-WITH-AES-128-CBC-SHA256", "TLS-ECDHE-ECDSA-WITH-AES-128-GCM-SHA256",
                            "TLS-ECDHE-ECDSA-WITH-AES-256-CBC-SHA384", "TLS-ECDHE-ECDSA-WITH-AES-256-GCM-SHA384", "TLS-RSA-WITH-AES-128-GCM-SHA256",
                            "TLS-RSA-WITH-AES-256-GCM-SHA384", "TLS-DHE-DSS-WITH-CAMELLIA-128-CBC-SHA", "TLS-DHE-DSS-WITH-CAMELLIA-256-CBC-SHA",
                            "TLS-DHE-DSS-WITH-CAMELLIA-128-CBC-SHA256", "TLS-DHE-DSS-WITH-CAMELLIA-256-CBC-SHA256", "TLS-DHE-DSS-WITH-SEED-CBC-SHA",
                            "TLS-DHE-DSS-WITH-ARIA-128-CBC-SHA256", "TLS-DHE-DSS-WITH-ARIA-256-CBC-SHA384", "TLS-ECDHE-RSA-WITH-ARIA-128-CBC-SHA256",
                            "TLS-ECDHE-RSA-WITH-ARIA-256-CBC-SHA384", "TLS-ECDHE-ECDSA-WITH-ARIA-128-CBC-SHA256",
                            "TLS-ECDHE-ECDSA-WITH-ARIA-256-CBC-SHA384", "TLS-DHE-DSS-WITH-3DES-EDE-CBC-SHA", "TLS-DHE-DSS-WITH-DES-CBC-SHA",
                            "TLS-AES-128-GCM-SHA256", "TLS-AES-256-GCM-SHA384", "TLS-CHACHA20-POLY1305-SHA256", "TLS-ECDHE-ECDSA-WITH-AES-256-CBC-SHA"
                        ],
                        "type": "str"
                    },
                    "priority": {"type": "int"},
                    "versions": {"type": "list", "choices": ["ssl-3.0", "tls-1.0", "tls-1.1", "tls-1.2", "tls-1.3"], "elements": "str"}
                },
                "elements": "dict"
            },
            "ssl-server-max-version": {"choices": ["ssl-3.0", "tls-1.0", "tls-1.1", "tls-1.2", "client", "tls-1.3"], "type": "str"},
            "ssl-server-min-version": {"choices": ["ssl-3.0", "tls-1.0", "tls-1.1", "tls-1.2", "client", "tls-1.3"], "type": "str"},
            "ssl-server-session-state-max": {"type": "int"},
            "ssl-server-session-state-timeout": {"type": "int"},
            "ssl-server-session-state-type": {"choices": ["disable", "time", "count", "both"], "type": "str"},
            "type": {"choices": ["static-nat", "server-load-balance", "access-proxy"], "type": "str"},
            "uuid": {"type": "str"},
            "weblogic-server": {"choices": ["disable", "enable"], "type": "str"},
            "websphere-server": {"choices": ["disable", "enable"], "type": "str"},
            "http-redirect": {"v_range": [["6.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ssl-client-rekey-count": {"v_range": [["6.2.1", ""]], "no_log": true, "type": "int"},
            "nat-source-vip": {"v_range": [["6.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "add-nat64-route": {"v_range": [["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "embedded-ipv4-address": {"v_range": [["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ipv4-mappedip": {"v_range": [["7.0.1", ""]], "type": "str"},
            "ipv4-mappedport": {"v_range": [["7.0.1", ""]], "type": "str"},
            "nat64": {"v_range": [["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "nat66": {"v_range": [["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ssl-accept-ffdhe-groups": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ndp-reply": {"v_range": [["7.0.5", "7.0.12"], ["7.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ssl-server-renegotiation": {"v_range": [["7.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "h2-support": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "h3-support": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "quic": {
                "v_range": [["7.4.2", ""]],
                "type": "dict",
                "options": {
                    "ack-delay-exponent": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "active-connection-id-limit": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "active-migration": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "grease-quic-bit": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "max-ack-delay": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "max-datagram-frame-size": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "max-idle-timeout": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "max-udp-payload-size": {"v_range": [["7.4.2", ""]], "type": "int"}
                }
            },
            "src-vip-filter": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "firewall_vip64": {
        "stated": true,
        "options": {
            "arp-reply": {"choices": ["disable", "enable"], "type": "str"},
            "color": {"type": "int"},
            "comment": {"type": "str"},
            "dynamic_mapping": {
                "type": "list",
                "options": {
                    "_scope": {"type": "list", "options": {"name": {"type": "str"}, "vdom": {"type": "str"}}, "elements": "dict"},
                    "arp-reply": {"choices": ["disable", "enable"], "type": "str"},
                    "color": {"type": "int"},
                    "comment": {"type": "str"},
                    "extip": {"type": "str"},
                    "extport": {"type": "str"},
                    "id": {"type": "int"},
                    "ldb-method": {"choices": ["static", "round-robin", "weighted", "least-session", "least-rtt", "first-alive"], "type": "str"},
                    "mappedip": {"type": "str"},
                    "mappedport": {"type": "str"},
                    "monitor": {"type": "raw"},
                    "portforward": {"choices": ["disable", "enable"], "type": "str"},
                    "protocol": {"choices": ["tcp", "udp"], "type": "str"},
                    "server-type": {"choices": ["http", "tcp", "udp", "ip"], "type": "str"},
                    "src-filter": {"type": "raw"},
                    "type": {"choices": ["static-nat", "server-load-balance"], "type": "str"},
                    "uuid": {"type": "str"}
                },
                "elements": "dict"
            },
            "extip": {"type": "str"},
            "extport": {"type": "str"},
            "id": {"type": "int"},
            "ldb-method": {"choices": ["static", "round-robin", "weighted", "least-session", "least-rtt", "first-alive"], "type": "str"},
            "mappedip": {"type": "str"},
            "mappedport": {"type": "str"},
            "monitor": {"type": "raw"},
            "name": {"required": true, "type": "str"},
            "portforward": {"choices": ["disable", "enable"], "type": "str"},
            "protocol": {"choices": ["tcp", "udp"], "type": "str"},
            "realservers": {
                "type": "list",
                "options": {
                    "client-ip": {"type": "str"},
                    "healthcheck": {"choices": ["disable", "enable", "vip"], "type": "str"},
                    "holddown-interval": {"type": "int"},
                    "id": {"type": "int"},
                    "ip": {"type": "str"},
                    "max-connections": {"type": "int"},
                    "monitor": {"type": "raw"},
                    "port": {"type": "int"},
                    "status": {"choices": ["active", "standby", "disable"], "type": "str"},
                    "weight": {"type": "int"}
                },
                "elements": "dict"
            },
            "server-type": {"choices": ["http", "tcp", "udp", "ip"], "type": "str"},
            "src-filter": {"type": "raw"},
            "type": {"choices": ["static-nat", "server-load-balance"], "type": "str"},
            "uuid": {"type": "str"}
        }
    },
    "firewall_vip64_dynamicmapping": {
        "stated": true,
        "options": {
            "_scope": {"type": "list", "options": {"name": {"type": "str"}, "vdom": {"type": "str"}}, "elements": "dict"},
            "arp-reply": {"choices": ["disable", "enable"], "type": "str"},
            "color": {"type": "int"},
            "comment": {"type": "str"},
            "extip": {"type": "str"},
            "extport": {"type": "str"},
            "id": {"type": "int"},
            "ldb-method": {"choices": ["static", "round-robin", "weighted", "least-session", "least-rtt", "first-alive"], "type": "str"},
            "mappedip": {"type": "str"},
            "mappedport": {"type": "str"},
            "monitor": {"type": "raw"},
            "portforward": {"choices": ["disable", "enable"], "type": "str"},
            "protocol": {"choices": ["tcp", "udp"], "type": "str"},
            "server-type": {"choices": ["http", "tcp", "udp", "ip"], "type": "str"},
            "src-filter": {"type": "raw"},
            "type": {"choices": ["static-nat", "server-load-balance"], "type": "str"},
            "uuid": {"type": "str"}
        }
    },
    "firewall_vip64_realservers": {
        "stated": true,
        "options": {
            "client-ip": {"type": "str"},
            "healthcheck": {"choices": ["disable", "enable", "vip"], "type": "str"},
            "holddown-interval": {"type": "int"},
            "id": {"required": true, "type": "int"},
            "ip": {"type": "str"},
            "max-connections": {"type": "int"},
            "monitor": {"type": "raw"},
            "port": {"type": "int"},
            "status": {"choices": ["active", "standby", "disable"], "type": "str"},
            "weight": {"type": "int"}
        }
    },
    "firewall_vip6_dynamicmapping": {
        "stated": true,
        "options": {
            "_scope": {"type": "list", "options": {"name": {"type": "str"}, "vdom": {"type": "str"}}, "elements": "dict"},
            "arp-reply": {"choices": ["disable", "enable"], "type": "str"},
            "color": {"type": "int"},
            "comment": {"type": "str"},
            "extip": {"type": "str"},
            "extport": {"type": "str"},
            "http-cookie-age": {"type": "int"},
            "http-cookie-domain": {"type": "str"},
            "http-cookie-domain-from-host": {"choices": ["disable", "enable"], "type": "str"},
            "http-cookie-generation": {"type": "int"},
            "http-cookie-path": {"type": "str"},
            "http-cookie-share": {"choices": ["disable", "same-ip"], "type": "str"},
            "http-ip-header": {"choices": ["disable", "enable"], "type": "str"},
            "http-ip-header-name": {"type": "str"},
            "http-multiplex": {"choices": ["disable", "enable"], "type": "str"},
            "https-cookie-secure": {"choices": ["disable", "enable"], "type": "str"},
            "id": {"type": "int"},
            "ldb-method": {"choices": ["static", "round-robin", "weighted", "least-session", "least-rtt", "first-alive", "http-host"], "type": "str"},
            "mappedip": {"type": "str"},
            "mappedport": {"type": "str"},
            "max-embryonic-connections": {"type": "int"},
            "monitor": {"type": "raw"},
            "outlook-web-access": {"choices": ["disable", "enable"], "type": "str"},
            "persistence": {"choices": ["none", "http-cookie", "ssl-session-id"], "type": "str"},
            "portforward": {"choices": ["disable", "enable"], "type": "str"},
            "protocol": {"choices": ["tcp", "udp", "sctp"], "type": "str"},
            "server-type": {"choices": ["http", "https", "ssl", "tcp", "udp", "ip", "imaps", "pop3s", "smtps"], "type": "str"},
            "src-filter": {"type": "raw"},
            "ssl-algorithm": {"choices": ["high", "low", "medium", "custom"], "type": "str"},
            "ssl-certificate": {"type": "str"},
            "ssl-client-fallback": {"choices": ["disable", "enable"], "type": "str"},
            "ssl-client-renegotiation": {"choices": ["deny", "allow", "secure"], "type": "str"},
            "ssl-client-session-state-max": {"type": "int"},
            "ssl-client-session-state-timeout": {"type": "int"},
            "ssl-client-session-state-type": {"choices": ["disable", "time", "count", "both"], "type": "str"},
            "ssl-dh-bits": {"choices": ["768", "1024", "1536", "2048", "3072", "4096"], "type": "str"},
            "ssl-hpkp": {"choices": ["disable", "enable", "report-only"], "type": "str"},
            "ssl-hpkp-age": {"type": "int"},
            "ssl-hpkp-backup": {"type": "str"},
            "ssl-hpkp-include-subdomains": {"choices": ["disable", "enable"], "type": "str"},
            "ssl-hpkp-primary": {"type": "str"},
            "ssl-hpkp-report-uri": {"type": "str"},
            "ssl-hsts": {"choices": ["disable", "enable"], "type": "str"},
            "ssl-hsts-age": {"type": "int"},
            "ssl-hsts-include-subdomains": {"choices": ["disable", "enable"], "type": "str"},
            "ssl-http-location-conversion": {"choices": ["disable", "enable"], "type": "str"},
            "ssl-http-match-host": {"choices": ["disable", "enable"], "type": "str"},
            "ssl-max-version": {"choices": ["ssl-3.0", "tls-1.0", "tls-1.1", "tls-1.2", "tls-1.3"], "type": "str"},
            "ssl-min-version": {"choices": ["ssl-3.0", "tls-1.0", "tls-1.1", "tls-1.2", "tls-1.3"], "type": "str"},
            "ssl-mode": {"choices": ["half", "full"], "type": "str"},
            "ssl-pfs": {"choices": ["require", "deny", "allow"], "type": "str"},
            "ssl-send-empty-frags": {"choices": ["disable", "enable"], "type": "str"},
            "ssl-server-algorithm": {"choices": ["high", "low", "medium", "custom", "client"], "type": "str"},
            "ssl-server-max-version": {"choices": ["ssl-3.0", "tls-1.0", "tls-1.1", "tls-1.2", "client", "tls-1.3"], "type": "str"},
            "ssl-server-min-version": {"choices": ["ssl-3.0", "tls-1.0", "tls-1.1", "tls-1.2", "client", "tls-1.3"], "type": "str"},
            "ssl-server-session-state-max": {"type": "int"},
            "ssl-server-session-state-timeout": {"type": "int"},
            "ssl-server-session-state-type": {"choices": ["disable", "time", "count", "both"], "type": "str"},
            "type": {"choices": ["static-nat", "server-load-balance", "access-proxy"], "type": "str"},
            "uuid": {"type": "str"},
            "weblogic-server": {"choices": ["disable", "enable"], "type": "str"},
            "websphere-server": {"choices": ["disable", "enable"], "type": "str"},
            "http-redirect": {"v_range": [["6.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ssl-client-rekey-count": {"v_range": [["6.2.1", ""]], "no_log": true, "type": "int"},
            "nat-source-vip": {"v_range": [["6.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "add-nat64-route": {"v_range": [["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "embedded-ipv4-address": {"v_range": [["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ipv4-mappedip": {"v_range": [["7.0.1", ""]], "type": "str"},
            "ipv4-mappedport": {"v_range": [["7.0.1", ""]], "type": "str"},
            "nat64": {"v_range": [["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "nat66": {"v_range": [["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "realservers": {
                "v_range": [["7.0.2", ""]],
                "type": "list",
                "options": {
                    "client-ip": {"v_range": [["7.0.2", ""]], "type": "str"},
                    "healthcheck": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable", "vip"], "type": "str"},
                    "holddown-interval": {"v_range": [["7.0.2", ""]], "type": "int"},
                    "http-host": {"v_range": [["7.0.2", ""]], "type": "str"},
                    "id": {"v_range": [["7.0.2", ""]], "type": "int"},
                    "ip": {"v_range": [["7.0.2", ""]], "type": "str"},
                    "max-connections": {"v_range": [["7.0.2", ""]], "type": "int"},
                    "monitor": {"v_range": [["7.0.2", ""]], "type": "raw"},
                    "port": {"v_range": [["7.0.2", ""]], "type": "int"},
                    "status": {"v_range": [["7.0.2", ""]], "choices": ["active", "standby", "disable"], "type": "str"},
                    "weight": {"v_range": [["7.0.2", ""]], "type": "int"},
                    "translate-host": {"v_range": [["7.2.2", ""]], "choices": ["disable", "enable"], "type": "str"}
                },
                "elements": "dict"
            },
            "ssl-accept-ffdhe-groups": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ssl-cipher-suites": {
                "v_range": [["7.0.2", ""]],
                "type": "list",
                "options": {
                    "cipher": {
                        "v_range": [["7.0.2", ""]],
                        "choices": [
                            "TLS-RSA-WITH-RC4-128-MD5", "TLS-RSA-WITH-RC4-128-SHA", "TLS-RSA-WITH-DES-CBC-SHA", "TLS-RSA-WITH-3DES-EDE-CBC-SHA",
                            "TLS-RSA-WITH-AES-128-CBC-SHA", "TLS-RSA-WITH-AES-256-CBC-SHA", "TLS-RSA-WITH-AES-128-CBC-SHA256",
                            "TLS-RSA-WITH-AES-256-CBC-SHA256", "TLS-RSA-WITH-CAMELLIA-128-CBC-SHA", "TLS-RSA-WITH-CAMELLIA-256-CBC-SHA",
                            "TLS-RSA-WITH-CAMELLIA-128-CBC-SHA256", "TLS-RSA-WITH-CAMELLIA-256-CBC-SHA256", "TLS-RSA-WITH-SEED-CBC-SHA",
                            "TLS-RSA-WITH-ARIA-128-CBC-SHA256", "TLS-RSA-WITH-ARIA-256-CBC-SHA384", "TLS-DHE-RSA-WITH-DES-CBC-SHA",
                            "TLS-DHE-RSA-WITH-3DES-EDE-CBC-SHA", "TLS-DHE-RSA-WITH-AES-128-CBC-SHA", "TLS-DHE-RSA-WITH-AES-256-CBC-SHA",
                            "TLS-DHE-RSA-WITH-AES-128-CBC-SHA256", "TLS-DHE-RSA-WITH-AES-256-CBC-SHA256", "TLS-DHE-RSA-WITH-CAMELLIA-128-CBC-SHA",
                            "TLS-DHE-RSA-WITH-CAMELLIA-256-CBC-SHA", "TLS-DHE-RSA-WITH-CAMELLIA-128-CBC-SHA256",
                            "TLS-DHE-RSA-WITH-CAMELLIA-256-CBC-SHA256", "TLS-DHE-RSA-WITH-SEED-CBC-SHA", "TLS-DHE-RSA-WITH-ARIA-128-CBC-SHA256",
                            "TLS-DHE-RSA-WITH-ARIA-256-CBC-SHA384", "TLS-ECDHE-RSA-WITH-RC4-128-SHA", "TLS-ECDHE-RSA-WITH-3DES-EDE-CBC-SHA",
                            "TLS-ECDHE-RSA-WITH-AES-128-CBC-SHA", "TLS-ECDHE-RSA-WITH-AES-256-CBC-SHA", "TLS-ECDHE-RSA-WITH-CHACHA20-POLY1305-SHA256",
                            "TLS-ECDHE-ECDSA-WITH-CHACHA20-POLY1305-SHA256", "TLS-DHE-RSA-WITH-CHACHA20-POLY1305-SHA256",
                            "TLS-DHE-RSA-WITH-AES-128-GCM-SHA256", "TLS-DHE-RSA-WITH-AES-256-GCM-SHA384", "TLS-DHE-DSS-WITH-AES-128-CBC-SHA",
                            "TLS-DHE-DSS-WITH-AES-256-CBC-SHA", "TLS-DHE-DSS-WITH-AES-128-CBC-SHA256", "TLS-DHE-DSS-WITH-AES-128-GCM-SHA256",
                            "TLS-DHE-DSS-WITH-AES-256-CBC-SHA256", "TLS-DHE-DSS-WITH-AES-256-GCM-SHA384", "TLS-ECDHE-RSA-WITH-AES-128-CBC-SHA256",
                            "TLS-ECDHE-RSA-WITH-AES-128-GCM-SHA256", "TLS-ECDHE-RSA-WITH-AES-256-CBC-SHA384", "TLS-ECDHE-RSA-WITH-AES-256-GCM-SHA384",
                            "TLS-ECDHE-ECDSA-WITH-AES-128-CBC-SHA", "TLS-ECDHE-ECDSA-WITH-AES-128-CBC-SHA256", "TLS-ECDHE-ECDSA-WITH-AES-128-GCM-SHA256",
                            "TLS-ECDHE-ECDSA-WITH-AES-256-CBC-SHA384", "TLS-ECDHE-ECDSA-WITH-AES-256-GCM-SHA384", "TLS-RSA-WITH-AES-128-GCM-SHA256",
                            "TLS-RSA-WITH-AES-256-GCM-SHA384", "TLS-DHE-DSS-WITH-CAMELLIA-128-CBC-SHA", "TLS-DHE-DSS-WITH-CAMELLIA-256-CBC-SHA",
                            "TLS-DHE-DSS-WITH-CAMELLIA-128-CBC-SHA256", "TLS-DHE-DSS-WITH-CAMELLIA-256-CBC-SHA256", "TLS-DHE-DSS-WITH-SEED-CBC-SHA",
                            "TLS-DHE-DSS-WITH-ARIA-128-CBC-SHA256", "TLS-DHE-DSS-WITH-ARIA-256-CBC-SHA384", "TLS-ECDHE-RSA-WITH-ARIA-128-CBC-SHA256",
                            "TLS-ECDHE-RSA-WITH-ARIA-256-CBC-SHA384", "TLS-ECDHE-ECDSA-WITH-ARIA-128-CBC-SHA256",
                            "TLS-ECDHE-ECDSA-WITH-ARIA-256-CBC-SHA384", "TLS-DHE-DSS-WITH-3DES-EDE-CBC-SHA", "TLS-DHE-DSS-WITH-DES-CBC-SHA",
                            "TLS-AES-128-GCM-SHA256", "TLS-AES-256-GCM-SHA384", "TLS-CHACHA20-POLY1305-SHA256", "TLS-ECDHE-ECDSA-WITH-AES-256-CBC-SHA"
                        ],
                        "type": "str"
                    },
                    "priority": {"v_range": [["7.0.2", ""]], "type": "int"},
                    "versions": {
                        "v_range": [["7.0.2", ""]],
                        "type": "list",
                        "choices": ["ssl-3.0", "tls-1.0", "tls-1.1", "tls-1.2", "tls-1.3"],
                        "elements": "str"
                    }
                },
                "elements": "dict"
            },
            "ndp-reply": {"v_range": [["7.0.5", "7.0.12"], ["7.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ssl-server-renegotiation": {"v_range": [["7.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "h2-support": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "h3-support": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "src-vip-filter": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "firewall_vip6_dynamicmapping_realservers": {
        "stated": true,
        "options": {
            "client-ip": {"v_range": [["7.0.2", "7.4.0"]], "type": "str"},
            "healthcheck": {"v_range": [["7.0.2", "7.4.0"]], "choices": ["disable", "enable", "vip"], "type": "str"},
            "holddown-interval": {"v_range": [["7.0.2", "7.4.0"]], "type": "int"},
            "http-host": {"v_range": [["7.0.2", "7.4.0"]], "type": "str"},
            "id": {"v_range": [["7.0.2", "7.4.0"]], "required": true, "type": "int"},
            "ip": {"v_range": [["7.0.2", "7.4.0"]], "type": "str"},
            "max-connections": {"v_range": [["7.0.2", "7.4.0"]], "type": "int"},
            "monitor": {"v_range": [["7.0.2", "7.4.0"]], "type": "raw"},
            "port": {"v_range": [["7.0.2", "7.4.0"]], "type": "int"},
            "status": {"v_range": [["7.0.2", "7.4.0"]], "choices": ["active", "standby", "disable"], "type": "str"},
            "weight": {"v_range": [["7.0.2", "7.4.0"]], "type": "int"},
            "translate-host": {"v_range": [["7.2.2", "7.4.0"]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "firewall_vip6_dynamicmapping_sslciphersuites": {
        "stated": true,
        "options": {
            "cipher": {
                "v_range": [["7.0.2", "7.4.0"]],
                "choices": [
                    "TLS-RSA-WITH-RC4-128-MD5", "TLS-RSA-WITH-RC4-128-SHA", "TLS-RSA-WITH-DES-CBC-SHA", "TLS-RSA-WITH-3DES-EDE-CBC-SHA",
                    "TLS-RSA-WITH-AES-128-CBC-SHA", "TLS-RSA-WITH-AES-256-CBC-SHA", "TLS-RSA-WITH-AES-128-CBC-SHA256", "TLS-RSA-WITH-AES-256-CBC-SHA256",
                    "TLS-RSA-WITH-CAMELLIA-128-CBC-SHA", "TLS-RSA-WITH-CAMELLIA-256-CBC-SHA", "TLS-RSA-WITH-CAMELLIA-128-CBC-SHA256",
                    "TLS-RSA-WITH-CAMELLIA-256-CBC-SHA256", "TLS-RSA-WITH-SEED-CBC-SHA", "TLS-RSA-WITH-ARIA-128-CBC-SHA256",
                    "TLS-RSA-WITH-ARIA-256-CBC-SHA384", "TLS-DHE-RSA-WITH-DES-CBC-SHA", "TLS-DHE-RSA-WITH-3DES-EDE-CBC-SHA",
                    "TLS-DHE-RSA-WITH-AES-128-CBC-SHA", "TLS-DHE-RSA-WITH-AES-256-CBC-SHA", "TLS-DHE-RSA-WITH-AES-128-CBC-SHA256",
                    "TLS-DHE-RSA-WITH-AES-256-CBC-SHA256", "TLS-DHE-RSA-WITH-CAMELLIA-128-CBC-SHA", "TLS-DHE-RSA-WITH-CAMELLIA-256-CBC-SHA",
                    "TLS-DHE-RSA-WITH-CAMELLIA-128-CBC-SHA256", "TLS-DHE-RSA-WITH-CAMELLIA-256-CBC-SHA256", "TLS-DHE-RSA-WITH-SEED-CBC-SHA",
                    "TLS-DHE-RSA-WITH-ARIA-128-CBC-SHA256", "TLS-DHE-RSA-WITH-ARIA-256-CBC-SHA384", "TLS-ECDHE-RSA-WITH-RC4-128-SHA",
                    "TLS-ECDHE-RSA-WITH-3DES-EDE-CBC-SHA", "TLS-ECDHE-RSA-WITH-AES-128-CBC-SHA", "TLS-ECDHE-RSA-WITH-AES-256-CBC-SHA",
                    "TLS-ECDHE-RSA-WITH-CHACHA20-POLY1305-SHA256", "TLS-ECDHE-ECDSA-WITH-CHACHA20-POLY1305-SHA256",
                    "TLS-DHE-RSA-WITH-CHACHA20-POLY1305-SHA256", "TLS-DHE-RSA-WITH-AES-128-GCM-SHA256", "TLS-DHE-RSA-WITH-AES-256-GCM-SHA384",
                    "TLS-DHE-DSS-WITH-AES-128-CBC-SHA", "TLS-DHE-DSS-WITH-AES-256-CBC-SHA", "TLS-DHE-DSS-WITH-AES-128-CBC-SHA256",
                    "TLS-DHE-DSS-WITH-AES-128-GCM-SHA256", "TLS-DHE-DSS-WITH-AES-256-CBC-SHA256", "TLS-DHE-DSS-WITH-AES-256-GCM-SHA384",
                    "TLS-ECDHE-RSA-WITH-AES-128-CBC-SHA256", "TLS-ECDHE-RSA-WITH-AES-128-GCM-SHA256", "TLS-ECDHE-RSA-WITH-AES-256-CBC-SHA384",
                    "TLS-ECDHE-RSA-WITH-AES-256-GCM-SHA384", "TLS-ECDHE-ECDSA-WITH-AES-128-CBC-SHA", "TLS-ECDHE-ECDSA-WITH-AES-128-CBC-SHA256",
                    "TLS-ECDHE-ECDSA-WITH-AES-128-GCM-SHA256", "TLS-ECDHE-ECDSA-WITH-AES-256-CBC-SHA384", "TLS-ECDHE-ECDSA-WITH-AES-256-GCM-SHA384",
                    "TLS-RSA-WITH-AES-128-GCM-SHA256", "TLS-RSA-WITH-AES-256-GCM-SHA384", "TLS-DHE-DSS-WITH-CAMELLIA-128-CBC-SHA",
                    "TLS-DHE-DSS-WITH-CAMELLIA-256-CBC-SHA", "TLS-DHE-DSS-WITH-CAMELLIA-128-CBC-SHA256", "TLS-DHE-DSS-WITH-CAMELLIA-256-CBC-SHA256",
                    "TLS-DHE-DSS-WITH-SEED-CBC-SHA", "TLS-DHE-DSS-WITH-ARIA-128-CBC-SHA256", "TLS-DHE-DSS-WITH-ARIA-256-CBC-SHA384",
                    "TLS-ECDHE-RSA-WITH-ARIA-128-CBC-SHA256", "TLS-ECDHE-RSA-WITH-ARIA-256-CBC-SHA384", "TLS-ECDHE-ECDSA-WITH-ARIA-128-CBC-SHA256",
                    "TLS-ECDHE-ECDSA-WITH-ARIA-256-CBC-SHA384", "TLS-DHE-DSS-WITH-3DES-EDE-CBC-SHA", "TLS-DHE-DSS-WITH-DES-CBC-SHA",
                    "TLS-AES-128-GCM-SHA256", "TLS-AES-256-GCM-SHA384", "TLS-CHACHA20-POLY1305-SHA256", "TLS-ECDHE-ECDSA-WITH-AES-256-CBC-SHA"
                ],
                "type": "str"
            },
            "priority": {"v_range": [["7.0.2", "7.4.0"]], "type": "int"},
            "versions": {
                "v_range": [["7.0.2", "7.4.0"]],
                "type": "list",
                "choices": ["ssl-3.0", "tls-1.0", "tls-1.1", "tls-1.2", "tls-1.3"],
                "elements": "str"
            }
        }
    },
    "firewall_vip6_quic": {
        "stated": false,
        "options": {
            "ack-delay-exponent": {"v_range": [["7.4.2", ""]], "type": "int"},
            "active-connection-id-limit": {"v_range": [["7.4.2", ""]], "type": "int"},
            "active-migration": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "grease-quic-bit": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "max-ack-delay": {"v_range": [["7.4.2", ""]], "type": "int"},
            "max-datagram-frame-size": {"v_range": [["7.4.2", ""]], "type": "int"},
            "max-idle-timeout": {"v_range": [["7.4.2", ""]], "type": "int"},
            "max-udp-payload-size": {"v_range": [["7.4.2", ""]], "type": "int"}
        }
    },
    "firewall_vip6_realservers": {
        "stated": true,
        "options": {
            "client-ip": {"type": "str"},
            "healthcheck": {"choices": ["disable", "enable", "vip"], "type": "str"},
            "holddown-interval": {"type": "int"},
            "http-host": {"type": "str"},
            "id": {"required": true, "type": "int"},
            "ip": {"type": "str"},
            "max-connections": {"type": "int"},
            "monitor": {"type": "raw"},
            "port": {"type": "int"},
            "status": {"choices": ["active", "standby", "disable"], "type": "str"},
            "weight": {"type": "int"},
            "translate-host": {"v_range": [["7.2.2", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "firewall_vip6_sslciphersuites": {
        "stated": true,
        "options": {
            "cipher": {
                "choices": [
                    "TLS-RSA-WITH-RC4-128-MD5", "TLS-RSA-WITH-RC4-128-SHA", "TLS-RSA-WITH-DES-CBC-SHA", "TLS-RSA-WITH-3DES-EDE-CBC-SHA",
                    "TLS-RSA-WITH-AES-128-CBC-SHA", "TLS-RSA-WITH-AES-256-CBC-SHA", "TLS-RSA-WITH-AES-128-CBC-SHA256", "TLS-RSA-WITH-AES-256-CBC-SHA256",
                    "TLS-RSA-WITH-CAMELLIA-128-CBC-SHA", "TLS-RSA-WITH-CAMELLIA-256-CBC-SHA", "TLS-RSA-WITH-CAMELLIA-128-CBC-SHA256",
                    "TLS-RSA-WITH-CAMELLIA-256-CBC-SHA256", "TLS-RSA-WITH-SEED-CBC-SHA", "TLS-RSA-WITH-ARIA-128-CBC-SHA256",
                    "TLS-RSA-WITH-ARIA-256-CBC-SHA384", "TLS-DHE-RSA-WITH-DES-CBC-SHA", "TLS-DHE-RSA-WITH-3DES-EDE-CBC-SHA",
                    "TLS-DHE-RSA-WITH-AES-128-CBC-SHA", "TLS-DHE-RSA-WITH-AES-256-CBC-SHA", "TLS-DHE-RSA-WITH-AES-128-CBC-SHA256",
                    "TLS-DHE-RSA-WITH-AES-256-CBC-SHA256", "TLS-DHE-RSA-WITH-CAMELLIA-128-CBC-SHA", "TLS-DHE-RSA-WITH-CAMELLIA-256-CBC-SHA",
                    "TLS-DHE-RSA-WITH-CAMELLIA-128-CBC-SHA256", "TLS-DHE-RSA-WITH-CAMELLIA-256-CBC-SHA256", "TLS-DHE-RSA-WITH-SEED-CBC-SHA",
                    "TLS-DHE-RSA-WITH-ARIA-128-CBC-SHA256", "TLS-DHE-RSA-WITH-ARIA-256-CBC-SHA384", "TLS-ECDHE-RSA-WITH-RC4-128-SHA",
                    "TLS-ECDHE-RSA-WITH-3DES-EDE-CBC-SHA", "TLS-ECDHE-RSA-WITH-AES-128-CBC-SHA", "TLS-ECDHE-RSA-WITH-AES-256-CBC-SHA",
                    "TLS-ECDHE-RSA-WITH-CHACHA20-POLY1305-SHA256", "TLS-ECDHE-ECDSA-WITH-CHACHA20-POLY1305-SHA256",
                    "TLS-DHE-RSA-WITH-CHACHA20-POLY1305-SHA256", "TLS-DHE-RSA-WITH-AES-128-GCM-SHA256", "TLS-DHE-RSA-WITH-AES-256-GCM-SHA384",
                    "TLS-DHE-DSS-WITH-AES-128-CBC-SHA", "TLS-DHE-DSS-WITH-AES-256-CBC-SHA", "TLS-DHE-DSS-WITH-AES-128-CBC-SHA256",
                    "TLS-DHE-DSS-WITH-AES-128-GCM-SHA256", "TLS-DHE-DSS-WITH-AES-256-CBC-SHA256", "TLS-DHE-DSS-WITH-AES-256-GCM-SHA384",
                    "TLS-ECDHE-RSA-WITH-AES-128-CBC-SHA256", "TLS-ECDHE-RSA-WITH-AES-128-GCM-SHA256", "TLS-ECDHE-RSA-WITH-AES-256-CBC-SHA384",
                    "TLS-ECDHE-RSA-WITH-AES-256-GCM-SHA384", "TLS-ECDHE-ECDSA-WITH-AES-128-CBC-SHA", "TLS-ECDHE-ECDSA-WITH-AES-128-CBC-SHA256",
                    "TLS-ECDHE-ECDSA-WITH-AES-128-GCM-SHA256", "TLS-ECDHE-ECDSA-WITH-AES-256-CBC-SHA384", "TLS-ECDHE-ECDSA-WITH-AES-256-GCM-SHA384",
                    "TLS-RSA-WITH-AES-128-GCM-SHA256", "TLS-RSA-WITH-AES-256-GCM-SHA384", "TLS-DHE-DSS-WITH-CAMELLIA-128-CBC-SHA",
                    "TLS-DHE-DSS-WITH-CAMELLIA-256-CBC-SHA", "TLS-DHE-DSS-WITH-CAMELLIA-128-CBC-SHA256", "TLS-DHE-DSS-WITH-CAMELLIA-256-CBC-SHA256",
                    "TLS-DHE-DSS-WITH-SEED-CBC-SHA", "TLS-DHE-DSS-WITH-ARIA-128-CBC-SHA256", "TLS-DHE-DSS-WITH-ARIA-256-CBC-SHA384",
                    "TLS-ECDHE-RSA-WITH-ARIA-128-CBC-SHA256", "TLS-ECDHE-RSA-WITH-ARIA-256-CBC-SHA384", "TLS-ECDHE-ECDSA-WITH-ARIA-128-CBC-SHA256",
                    "TLS-ECDHE-ECDSA-WITH-ARIA-256-CBC-SHA384", "TLS-DHE-DSS-WITH-3DES-EDE-CBC-SHA", "TLS-DHE-DSS-WITH-DES-CBC-SHA",
                    "TLS-AES-128-GCM-SHA256", "TLS-AES-256-GCM-SHA384", "TLS-CHACHA20-POLY1305-SHA256", "TLS-ECDHE-ECDSA-WITH-AES-256-CBC-SHA"
                ],
                "type": "str"
            },
            "priority": {"required": true, "type": "int"},
            "versions": {"type": "list", "choices": ["ssl-3.0", "tls-1.0", "tls-1.1", "tls-1.2", "tls-1.3"], "elements": "str"}
        }
    },
    "firewall_vip6_sslserverciphersuites": {
        "stated": true,
        "options": {
            "cipher": {
                "choices": [
                    "TLS-RSA-WITH-RC4-128-MD5", "TLS-RSA-WITH-RC4-128-SHA", "TLS-RSA-WITH-DES-CBC-SHA", "TLS-RSA-WITH-3DES-EDE-CBC-SHA",
                    "TLS-RSA-WITH-AES-128-CBC-SHA", "TLS-RSA-WITH-AES-256-CBC-SHA", "TLS-RSA-WITH-AES-128-CBC-SHA256", "TLS-RSA-WITH-AES-256-CBC-SHA256",
                    "TLS-RSA-WITH-CAMELLIA-128-CBC-SHA", "TLS-RSA-WITH-CAMELLIA-256-CBC-SHA", "TLS-RSA-WITH-CAMELLIA-128-CBC-SHA256",
                    "TLS-RSA-WITH-CAMELLIA-256-CBC-SHA256", "TLS-RSA-WITH-SEED-CBC-SHA", "TLS-RSA-WITH-ARIA-128-CBC-SHA256",
                    "TLS-RSA-WITH-ARIA-256-CBC-SHA384", "TLS-DHE-RSA-WITH-DES-CBC-SHA", "TLS-DHE-RSA-WITH-3DES-EDE-CBC-SHA",
                    "TLS-DHE-RSA-WITH-AES-128-CBC-SHA", "TLS-DHE-RSA-WITH-AES-256-CBC-SHA", "TLS-DHE-RSA-WITH-AES-128-CBC-SHA256",
                    "TLS-DHE-RSA-WITH-AES-256-CBC-SHA256", "TLS-DHE-RSA-WITH-CAMELLIA-128-CBC-SHA", "TLS-DHE-RSA-WITH-CAMELLIA-256-CBC-SHA",
                    "TLS-DHE-RSA-WITH-CAMELLIA-128-CBC-SHA256", "TLS-DHE-RSA-WITH-CAMELLIA-256-CBC-SHA256", "TLS-DHE-RSA-WITH-SEED-CBC-SHA",
                    "TLS-DHE-RSA-WITH-ARIA-128-CBC-SHA256", "TLS-DHE-RSA-WITH-ARIA-256-CBC-SHA384", "TLS-ECDHE-RSA-WITH-RC4-128-SHA",
                    "TLS-ECDHE-RSA-WITH-3DES-EDE-CBC-SHA", "TLS-ECDHE-RSA-WITH-AES-128-CBC-SHA", "TLS-ECDHE-RSA-WITH-AES-256-CBC-SHA",
                    "TLS-ECDHE-RSA-WITH-CHACHA20-POLY1305-SHA256", "TLS-ECDHE-ECDSA-WITH-CHACHA20-POLY1305-SHA256",
                    "TLS-DHE-RSA-WITH-CHACHA20-POLY1305-SHA256", "TLS-DHE-RSA-WITH-AES-128-GCM-SHA256", "TLS-DHE-RSA-WITH-AES-256-GCM-SHA384",
                    "TLS-DHE-DSS-WITH-AES-128-CBC-SHA", "TLS-DHE-DSS-WITH-AES-256-CBC-SHA", "TLS-DHE-DSS-WITH-AES-128-CBC-SHA256",
                    "TLS-DHE-DSS-WITH-AES-128-GCM-SHA256", "TLS-DHE-DSS-WITH-AES-256-CBC-SHA256", "TLS-DHE-DSS-WITH-AES-256-GCM-SHA384",
                    "TLS-ECDHE-RSA-WITH-AES-128-CBC-SHA256", "TLS-ECDHE-RSA-WITH-AES-128-GCM-SHA256", "TLS-ECDHE-RSA-WITH-AES-256-CBC-SHA384",
                    "TLS-ECDHE-RSA-WITH-AES-256-GCM-SHA384", "TLS-ECDHE-ECDSA-WITH-AES-128-CBC-SHA", "TLS-ECDHE-ECDSA-WITH-AES-128-CBC-SHA256",
                    "TLS-ECDHE-ECDSA-WITH-AES-128-GCM-SHA256", "TLS-ECDHE-ECDSA-WITH-AES-256-CBC-SHA384", "TLS-ECDHE-ECDSA-WITH-AES-256-GCM-SHA384",
                    "TLS-RSA-WITH-AES-128-GCM-SHA256", "TLS-RSA-WITH-AES-256-GCM-SHA384", "TLS-DHE-DSS-WITH-CAMELLIA-128-CBC-SHA",
                    "TLS-DHE-DSS-WITH-CAMELLIA-256-CBC-SHA", "TLS-DHE-DSS-WITH-CAMELLIA-128-CBC-SHA256", "TLS-DHE-DSS-WITH-CAMELLIA-256-CBC-SHA256",
                    "TLS-DHE-DSS-WITH-SEED-CBC-SHA", "TLS-DHE-DSS-WITH-ARIA-128-CBC-SHA256", "TLS-DHE-DSS-WITH-ARIA-256-CBC-SHA384",
                    "TLS-ECDHE-RSA-WITH-ARIA-128-CBC-SHA256", "TLS-ECDHE-RSA-WITH-ARIA-256-CBC-SHA384", "TLS-ECDHE-ECDSA-WITH-ARIA-128-CBC-SHA256",
                    "TLS-ECDHE-ECDSA-WITH-ARIA-256-CBC-SHA384", "TLS-DHE-DSS-WITH-3DES-EDE-CBC-SHA", "TLS-DHE-DSS-WITH-DES-CBC-SHA",
                    "TLS-AES-128-GCM-SHA256", "TLS-AES-256-GCM-SHA384", "TLS-CHACHA20-POLY1305-SHA256", "TLS-ECDHE-ECDSA-WITH-AES-256-CBC-SHA"
                ],
                "type": "str"
            },
            "priority": {"required": true, "type": "int"},
            "versions": {"type": "list", "choices": ["ssl-3.0", "tls-1.0", "tls-1.1", "tls-1.2", "tls-1.3"], "elements": "str"}
        }
    },
    "firewall_vip_dynamicmapping": {
        "stated": true,
        "options": {
            "_scope": {"type": "list", "options": {"name": {"type": "str"}, "vdom": {"type": "str"}}, "elements": "dict"},
            "arp-reply": {"choices": ["disable", "enable"], "type": "str"},
            "color": {"type": "int"},
            "comment": {"type": "str"},
            "dns-mapping-ttl": {"type": "int"},
            "extaddr": {"type": "raw"},
            "extintf": {"type": "str"},
            "extip": {"type": "str"},
            "extport": {"type": "str"},
            "gratuitous-arp-interval": {"type": "int"},
            "http-cookie-age": {"type": "int"},
            "http-cookie-domain": {"type": "str"},
            "http-cookie-domain-from-host": {"choices": ["disable", "enable"], "type": "str"},
            "http-cookie-generation": {"type": "int"},
            "http-cookie-path": {"type": "str"},
            "http-cookie-share": {"choices": ["disable", "same-ip"], "type": "str"},
            "http-ip-header": {"choices": ["disable", "enable"], "type": "str"},
            "http-ip-header-name": {"type": "str"},
            "http-multiplex": {"choices": ["disable", "enable"], "type": "str"},
            "https-cookie-secure": {"choices": ["disable", "enable"], "type": "str"},
            "id": {"type": "int"},
            "ldb-method": {"choices": ["static", "round-robin", "weighted", "least-session", "least-rtt", "first-alive", "http-host"], "type": "str"},
            "mapped-addr": {"type": "str"},
            "mappedip": {"type": "raw"},
            "mappedport": {"type": "str"},
            "max-embryonic-connections": {"type": "int"},
            "monitor": {"type": "raw"},
            "nat-source-vip": {"choices": ["disable", "enable"], "type": "str"},
            "outlook-web-access": {"choices": ["disable", "enable"], "type": "str"},
            "persistence": {"choices": ["none", "http-cookie", "ssl-session-id"], "type": "str"},
            "portforward": {"choices": ["disable", "enable"], "type": "str"},
            "portmapping-type": {"choices": ["1-to-1", "m-to-n"], "type": "str"},
            "protocol": {"choices": ["tcp", "udp", "sctp", "icmp"], "type": "str"},
            "realservers": {
                "type": "list",
                "options": {
                    "client-ip": {"type": "raw"},
                    "healthcheck": {"choices": ["disable", "enable", "vip"], "type": "str"},
                    "holddown-interval": {"type": "int"},
                    "http-host": {"type": "str"},
                    "ip": {"type": "str"},
                    "max-connections": {"type": "int"},
                    "monitor": {"type": "raw"},
                    "port": {"type": "int"},
                    "seq": {"type": "int"},
                    "status": {"choices": ["active", "standby", "disable"], "type": "str"},
                    "weight": {"type": "int"},
                    "address": {"v_range": [["6.4.0", ""]], "type": "str"},
                    "id": {"v_range": [["6.4.0", ""]], "type": "int"},
                    "type": {"v_range": [["6.4.0", ""]], "choices": ["ip", "address"], "type": "str"},
                    "translate-host": {"v_range": [["7.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "health-check-proto": {"v_range": [["7.2.3", ""]], "choices": ["ping", "http"], "type": "str"}
                },
                "elements": "dict"
            },
            "server-type": {"choices": ["http", "https", "ssl", "tcp", "udp", "ip", "imaps", "pop3s", "smtps", "ssh"], "type": "str"},
            "service": {"type": "raw"},
            "src-filter": {"type": "raw"},
            "srcintf-filter": {"type": "raw"},
            "ssl-algorithm": {"choices": ["high", "medium", "low", "custom"], "type": "str"},
            "ssl-certificate": {"type": "str"},
            "ssl-cipher-suites": {
                "type": "list",
                "options": {
                    "cipher": {
                        "choices": [
                            "TLS-RSA-WITH-RC4-128-MD5", "TLS-RSA-WITH-RC4-128-SHA", "TLS-RSA-WITH-DES-CBC-SHA", "TLS-RSA-WITH-3DES-EDE-CBC-SHA",
                            "TLS-RSA-WITH-AES-128-CBC-SHA", "TLS-RSA-WITH-AES-256-CBC-SHA", "TLS-RSA-WITH-AES-128-CBC-SHA256",
                            "TLS-RSA-WITH-AES-256-CBC-SHA256", "TLS-RSA-WITH-CAMELLIA-128-CBC-SHA", "TLS-RSA-WITH-CAMELLIA-256-CBC-SHA",
                            "TLS-RSA-WITH-CAMELLIA-128-CBC-SHA256", "TLS-RSA-WITH-CAMELLIA-256-CBC-SHA256", "TLS-RSA-WITH-SEED-CBC-SHA",
                            "TLS-RSA-WITH-ARIA-128-CBC-SHA256", "TLS-RSA-WITH-ARIA-256-CBC-SHA384", "TLS-DHE-RSA-WITH-DES-CBC-SHA",
                            "TLS-DHE-RSA-WITH-3DES-EDE-CBC-SHA", "TLS-DHE-RSA-WITH-AES-128-CBC-SHA", "TLS-DHE-RSA-WITH-AES-256-CBC-SHA",
                            "TLS-DHE-RSA-WITH-AES-128-CBC-SHA256", "TLS-DHE-RSA-WITH-AES-256-CBC-SHA256", "TLS-DHE-RSA-WITH-CAMELLIA-128-CBC-SHA",
                            "TLS-DHE-RSA-WITH-CAMELLIA-256-CBC-SHA", "TLS-DHE-RSA-WITH-CAMELLIA-128-CBC-SHA256",
                            "TLS-DHE-RSA-WITH-CAMELLIA-256-CBC-SHA256", "TLS-DHE-RSA-WITH-SEED-CBC-SHA", "TLS-DHE-RSA-WITH-ARIA-128-CBC-SHA256",
                            "TLS-DHE-RSA-WITH-ARIA-256-CBC-SHA384", "TLS-ECDHE-RSA-WITH-RC4-128-SHA", "TLS-ECDHE-RSA-WITH-3DES-EDE-CBC-SHA",
                            "TLS-ECDHE-RSA-WITH-AES-128-CBC-SHA", "TLS-ECDHE-RSA-WITH-AES-256-CBC-SHA", "TLS-ECDHE-RSA-WITH-CHACHA20-POLY1305-SHA256",
                            "TLS-ECDHE-ECDSA-WITH-CHACHA20-POLY1305-SHA256", "TLS-DHE-RSA-WITH-CHACHA20-POLY1305-SHA256",
                            "TLS-DHE-RSA-WITH-AES-128-GCM-SHA256", "TLS-DHE-RSA-WITH-AES-256-GCM-SHA384", "TLS-DHE-DSS-WITH-AES-128-CBC-SHA",
                            "TLS-DHE-DSS-WITH-AES-256-CBC-SHA", "TLS-DHE-DSS-WITH-AES-128-CBC-SHA256", "TLS-DHE-DSS-WITH-AES-128-GCM-SHA256",
                            "TLS-DHE-DSS-WITH-AES-256-CBC-SHA256", "TLS-DHE-DSS-WITH-AES-256-GCM-SHA384", "TLS-ECDHE-RSA-WITH-AES-128-CBC-SHA256",
                            "TLS-ECDHE-RSA-WITH-AES-128-GCM-SHA256", "TLS-ECDHE-RSA-WITH-AES-256-CBC-SHA384", "TLS-ECDHE-RSA-WITH-AES-256-GCM-SHA384",
                            "TLS-ECDHE-ECDSA-WITH-AES-128-CBC-SHA", "TLS-ECDHE-ECDSA-WITH-AES-128-CBC-SHA256", "TLS-ECDHE-ECDSA-WITH-AES-128-GCM-SHA256",
                            "TLS-ECDHE-ECDSA-WITH-AES-256-CBC-SHA384", "TLS-ECDHE-ECDSA-WITH-AES-256-GCM-SHA384", "TLS-RSA-WITH-AES-128-GCM-SHA256",
                            "TLS-RSA-WITH-AES-256-GCM-SHA384", "TLS-DHE-DSS-WITH-CAMELLIA-128-CBC-SHA", "TLS-DHE-DSS-WITH-CAMELLIA-256-CBC-SHA",
                            "TLS-DHE-DSS-WITH-CAMELLIA-128-CBC-SHA256", "TLS-DHE-DSS-WITH-CAMELLIA-256-CBC-SHA256", "TLS-DHE-DSS-WITH-SEED-CBC-SHA",
                            "TLS-DHE-DSS-WITH-ARIA-128-CBC-SHA256", "TLS-DHE-DSS-WITH-ARIA-256-CBC-SHA384", "TLS-ECDHE-RSA-WITH-ARIA-128-CBC-SHA256",
                            "TLS-ECDHE-RSA-WITH-ARIA-256-CBC-SHA384", "TLS-ECDHE-ECDSA-WITH-ARIA-128-CBC-SHA256",
                            "TLS-ECDHE-ECDSA-WITH-ARIA-256-CBC-SHA384", "TLS-DHE-DSS-WITH-3DES-EDE-CBC-SHA", "TLS-DHE-DSS-WITH-DES-CBC-SHA",
                            "TLS-AES-128-GCM-SHA256", "TLS-AES-256-GCM-SHA384", "TLS-CHACHA20-POLY1305-SHA256", "TLS-ECDHE-ECDSA-WITH-AES-256-CBC-SHA"
                        ],
                        "type": "str"
                    },
                    "id": {"type": "int"},
                    "versions": {"type": "list", "choices": ["ssl-3.0", "tls-1.0", "tls-1.1", "tls-1.2", "tls-1.3"], "elements": "str"},
                    "priority": {"v_range": [["6.4.0", ""]], "type": "int"}
                },
                "elements": "dict"
            },
            "ssl-client-fallback": {"choices": ["disable", "enable"], "type": "str"},
            "ssl-client-renegotiation": {"choices": ["deny", "allow", "secure"], "type": "str"},
            "ssl-client-session-state-max": {"type": "int"},
            "ssl-client-session-state-timeout": {"type": "int"},
            "ssl-client-session-state-type": {"choices": ["disable", "time", "count", "both"], "type": "str"},
            "ssl-dh-bits": {"choices": ["768", "1024", "1536", "2048", "3072", "4096"], "type": "str"},
            "ssl-hpkp": {"choices": ["disable", "enable", "report-only"], "type": "str"},
            "ssl-hpkp-age": {"type": "int"},
            "ssl-hpkp-backup": {"type": "str"},
            "ssl-hpkp-include-subdomains": {"choices": ["disable", "enable"], "type": "str"},
            "ssl-hpkp-primary": {"type": "str"},
            "ssl-hpkp-report-uri": {"type": "str"},
            "ssl-hsts": {"choices": ["disable", "enable"], "type": "str"},
            "ssl-hsts-age": {"type": "int"},
            "ssl-hsts-include-subdomains": {"choices": ["disable", "enable"], "type": "str"},
            "ssl-http-location-conversion": {"choices": ["disable", "enable"], "type": "str"},
            "ssl-http-match-host": {"choices": ["disable", "enable"], "type": "str"},
            "ssl-max-version": {"choices": ["ssl-3.0", "tls-1.0", "tls-1.1", "tls-1.2", "tls-1.3"], "type": "str"},
            "ssl-min-version": {"choices": ["ssl-3.0", "tls-1.0", "tls-1.1", "tls-1.2", "tls-1.3"], "type": "str"},
            "ssl-mode": {"choices": ["half", "full"], "type": "str"},
            "ssl-pfs": {"choices": ["require", "deny", "allow"], "type": "str"},
            "ssl-send-empty-frags": {"choices": ["disable", "enable"], "type": "str"},
            "ssl-server-algorithm": {"choices": ["high", "low", "medium", "custom", "client"], "type": "str"},
            "ssl-server-max-version": {"choices": ["ssl-3.0", "tls-1.0", "tls-1.1", "tls-1.2", "client", "tls-1.3"], "type": "str"},
            "ssl-server-min-version": {"choices": ["ssl-3.0", "tls-1.0", "tls-1.1", "tls-1.2", "client", "tls-1.3"], "type": "str"},
            "ssl-server-session-state-max": {"type": "int"},
            "ssl-server-session-state-timeout": {"type": "int"},
            "ssl-server-session-state-type": {"choices": ["disable", "time", "count", "both"], "type": "str"},
            "type": {"choices": ["static-nat", "load-balance", "server-load-balance", "dns-translation", "fqdn", "access-proxy"], "type": "str"},
            "uuid": {"type": "str"},
            "weblogic-server": {"choices": ["disable", "enable"], "type": "str"},
            "websphere-server": {"choices": ["disable", "enable"], "type": "str"},
            "http-redirect": {"v_range": [["6.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ssl-client-rekey-count": {"v_range": [["6.2.1", ""]], "no_log": true, "type": "int"},
            "status": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "add-nat46-route": {"v_range": [["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ipv6-mappedip": {"v_range": [["7.0.1", ""]], "type": "str"},
            "ipv6-mappedport": {"v_range": [["7.0.1", ""]], "type": "str"},
            "nat44": {"v_range": [["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "nat46": {"v_range": [["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ssl-accept-ffdhe-groups": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "http-multiplex-max-request": {"v_range": [["7.2.2", ""]], "type": "int"},
            "http-multiplex-ttl": {"v_range": [["7.2.2", ""]], "type": "int"},
            "http-supported-max-version": {"v_range": [["7.2.2", ""]], "choices": ["http1", "http2"], "type": "str"},
            "ssl-server-renegotiation": {"v_range": [["7.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "h2-support": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "h3-support": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "http-multiplex-max-concurrent-request": {"v_range": [["7.4.1", ""]], "type": "int"},
            "gslb-domain-name": {"v_range": [["7.4.2", ""]], "type": "str"},
            "gslb-hostname": {"v_range": [["7.4.2", ""]], "type": "str"},
            "one-click-gslb-server": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "src-vip-filter": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "firewall_vip_dynamicmapping_realservers": {
        "stated": true,
        "options": {
            "client-ip": {"v_range": [["6.0.0", "7.4.0"]], "type": "raw"},
            "healthcheck": {"v_range": [["6.0.0", "7.4.0"]], "choices": ["disable", "enable", "vip"], "type": "str"},
            "holddown-interval": {"v_range": [["6.0.0", "7.4.0"]], "type": "int"},
            "http-host": {"v_range": [["6.0.0", "7.4.0"]], "type": "str"},
            "ip": {"v_range": [["6.0.0", "7.4.0"]], "type": "str"},
            "max-connections": {"v_range": [["6.0.0", "7.4.0"]], "type": "int"},
            "monitor": {"v_range": [["6.0.0", "7.4.0"]], "type": "raw"},
            "port": {"v_range": [["6.0.0", "7.4.0"]], "type": "int"},
            "seq": {"v_range": [["6.0.0", "7.4.0"]], "required": true, "type": "int"},
            "status": {"v_range": [["6.0.0", "7.4.0"]], "choices": ["active", "standby", "disable"], "type": "str"},
            "weight": {"v_range": [["6.0.0", "7.4.0"]], "type": "int"},
            "address": {"v_range": [["6.4.0", "7.4.0"]], "type": "str"},
            "id": {"v_range": [["6.4.0", "7.4.0"]], "type": "int"},
            "type": {"v_range": [["6.4.0", "7.4.0"]], "choices": ["ip", "address"], "type": "str"},
            "translate-host": {"v_range": [["7.2.2", "7.4.0"]], "choices": ["disable", "enable"], "type": "str"},
            "health-check-proto": {"v_range": [["7.2.3", "7.4.0"]], "choices": ["ping", "http"], "type": "str"}
        }
    },
    "firewall_vip_dynamicmapping_sslciphersuites": {
        "stated": true,
        "options": {
            "cipher": {
                "v_range": [["6.0.0", "7.4.0"]],
                "choices": [
                    "TLS-RSA-WITH-RC4-128-MD5", "TLS-RSA-WITH-RC4-128-SHA", "TLS-RSA-WITH-DES-CBC-SHA", "TLS-RSA-WITH-3DES-EDE-CBC-SHA",
                    "TLS-RSA-WITH-AES-128-CBC-SHA", "TLS-RSA-WITH-AES-256-CBC-SHA", "TLS-RSA-WITH-AES-128-CBC-SHA256", "TLS-RSA-WITH-AES-256-CBC-SHA256",
                    "TLS-RSA-WITH-CAMELLIA-128-CBC-SHA", "TLS-RSA-WITH-CAMELLIA-256-CBC-SHA", "TLS-RSA-WITH-CAMELLIA-128-CBC-SHA256",
                    "TLS-RSA-WITH-CAMELLIA-256-CBC-SHA256", "TLS-RSA-WITH-SEED-CBC-SHA", "TLS-RSA-WITH-ARIA-128-CBC-SHA256",
                    "TLS-RSA-WITH-ARIA-256-CBC-SHA384", "TLS-DHE-RSA-WITH-DES-CBC-SHA", "TLS-DHE-RSA-WITH-3DES-EDE-CBC-SHA",
                    "TLS-DHE-RSA-WITH-AES-128-CBC-SHA", "TLS-DHE-RSA-WITH-AES-256-CBC-SHA", "TLS-DHE-RSA-WITH-AES-128-CBC-SHA256",
                    "TLS-DHE-RSA-WITH-AES-256-CBC-SHA256", "TLS-DHE-RSA-WITH-CAMELLIA-128-CBC-SHA", "TLS-DHE-RSA-WITH-CAMELLIA-256-CBC-SHA",
                    "TLS-DHE-RSA-WITH-CAMELLIA-128-CBC-SHA256", "TLS-DHE-RSA-WITH-CAMELLIA-256-CBC-SHA256", "TLS-DHE-RSA-WITH-SEED-CBC-SHA",
                    "TLS-DHE-RSA-WITH-ARIA-128-CBC-SHA256", "TLS-DHE-RSA-WITH-ARIA-256-CBC-SHA384", "TLS-ECDHE-RSA-WITH-RC4-128-SHA",
                    "TLS-ECDHE-RSA-WITH-3DES-EDE-CBC-SHA", "TLS-ECDHE-RSA-WITH-AES-128-CBC-SHA", "TLS-ECDHE-RSA-WITH-AES-256-CBC-SHA",
                    "TLS-ECDHE-RSA-WITH-CHACHA20-POLY1305-SHA256", "TLS-ECDHE-ECDSA-WITH-CHACHA20-POLY1305-SHA256",
                    "TLS-DHE-RSA-WITH-CHACHA20-POLY1305-SHA256", "TLS-DHE-RSA-WITH-AES-128-GCM-SHA256", "TLS-DHE-RSA-WITH-AES-256-GCM-SHA384",
                    "TLS-DHE-DSS-WITH-AES-128-CBC-SHA", "TLS-DHE-DSS-WITH-AES-256-CBC-SHA", "TLS-DHE-DSS-WITH-AES-128-CBC-SHA256",
                    "TLS-DHE-DSS-WITH-AES-128-GCM-SHA256", "TLS-DHE-DSS-WITH-AES-256-CBC-SHA256", "TLS-DHE-DSS-WITH-AES-256-GCM-SHA384",
                    "TLS-ECDHE-RSA-WITH-AES-128-CBC-SHA256", "TLS-ECDHE-RSA-WITH-AES-128-GCM-SHA256", "TLS-ECDHE-RSA-WITH-AES-256-CBC-SHA384",
                    "TLS-ECDHE-RSA-WITH-AES-256-GCM-SHA384", "TLS-ECDHE-ECDSA-WITH-AES-128-CBC-SHA", "TLS-ECDHE-ECDSA-WITH-AES-128-CBC-SHA256",
                    "TLS-ECDHE-ECDSA-WITH-AES-128-GCM-SHA256", "TLS-ECDHE-ECDSA-WITH-AES-256-CBC-SHA384", "TLS-ECDHE-ECDSA-WITH-AES-256-GCM-SHA384",
                    "TLS-RSA-WITH-AES-128-GCM-SHA256", "TLS-RSA-WITH-AES-256-GCM-SHA384", "TLS-DHE-DSS-WITH-CAMELLIA-128-CBC-SHA",
                    "TLS-DHE-DSS-WITH-CAMELLIA-256-CBC-SHA", "TLS-DHE-DSS-WITH-CAMELLIA-128-CBC-SHA256", "TLS-DHE-DSS-WITH-CAMELLIA-256-CBC-SHA256",
                    "TLS-DHE-DSS-WITH-SEED-CBC-SHA", "TLS-DHE-DSS-WITH-ARIA-128-CBC-SHA256", "TLS-DHE-DSS-WITH-ARIA-256-CBC-SHA384",
                    "TLS-ECDHE-RSA-WITH-ARIA-128-CBC-SHA256", "TLS-ECDHE-RSA-WITH-ARIA-256-CBC-SHA384", "TLS-ECDHE-ECDSA-WITH-ARIA-128-CBC-SHA256",
                    "TLS-ECDHE-ECDSA-WITH-ARIA-256-CBC-SHA384", "TLS-DHE-DSS-WITH-3DES-EDE-CBC-SHA", "TLS-DHE-DSS-WITH-DES-CBC-SHA",
                    "TLS-AES-128-GCM-SHA256", "TLS-AES-256-GCM-SHA384", "TLS-CHACHA20-POLY1305-SHA256", "TLS-ECDHE-ECDSA-WITH-AES-256-CBC-SHA"
                ],
                "type": "str"
            },
            "id": {"v_range": [["6.0.0", "7.4.0"]], "required": true, "type": "int"},
            "versions": {
                "v_range": [["6.0.0", "7.4.0"]],
                "type": "list",
                "choices": ["ssl-3.0", "tls-1.0", "tls-1.1", "tls-1.2", "tls-1.3"],
                "elements": "str"
            },
            "priority": {"v_range": [["6.4.0", "7.4.0"]], "type": "int"}
        }
    },
    "firewall_vip_gslbpublicips": {
        "stated": true,
        "options": {"index": {"v_range": [["7.4.2", ""]], "type": "int"}, "ip": {"v_range": [["7.4.2", ""]], "type": "str"}}
    },
    "firewall_vip_quic": {
        "stated": false,
        "options": {
            "ack-delay-exponent": {"v_range": [["7.4.1", ""]], "type": "int"},
            "active-connection-id-limit": {"v_range": [["7.4.1", ""]], "type": "int"},
            "active-migration": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "grease-quic-bit": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "max-ack-delay": {"v_range": [["7.4.1", ""]], "type": "int"},
            "max-datagram-frame-size": {"v_range": [["7.4.1", ""]], "type": "int"},
            "max-idle-timeout": {"v_range": [["7.4.1", ""]], "type": "int"},
            "max-udp-payload-size": {"v_range": [["7.4.1", ""]], "type": "int"}
        }
    },
    "firewall_vip_realservers": {
        "stated": true,
        "options": {
            "client-ip": {"type": "raw"},
            "healthcheck": {"choices": ["disable", "enable", "vip"], "type": "str"},
            "holddown-interval": {"type": "int"},
            "http-host": {"type": "str"},
            "ip": {"type": "str"},
            "max-connections": {"type": "int"},
            "monitor": {"type": "raw"},
            "port": {"type": "int"},
            "seq": {"required": true, "type": "int"},
            "status": {"choices": ["active", "standby", "disable"], "type": "str"},
            "weight": {"type": "int"},
            "address": {"v_range": [["6.4.0", ""]], "type": "str"},
            "id": {"v_range": [["6.4.0", ""]], "type": "int"},
            "type": {"v_range": [["6.4.0", ""]], "choices": ["ip", "address"], "type": "str"},
            "translate-host": {"v_range": [["7.2.2", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "firewall_vip_sslciphersuites": {
        "stated": true,
        "options": {
            "cipher": {
                "choices": [
                    "TLS-RSA-WITH-RC4-128-MD5", "TLS-RSA-WITH-RC4-128-SHA", "TLS-RSA-WITH-DES-CBC-SHA", "TLS-RSA-WITH-3DES-EDE-CBC-SHA",
                    "TLS-RSA-WITH-AES-128-CBC-SHA", "TLS-RSA-WITH-AES-256-CBC-SHA", "TLS-RSA-WITH-AES-128-CBC-SHA256", "TLS-RSA-WITH-AES-256-CBC-SHA256",
                    "TLS-RSA-WITH-CAMELLIA-128-CBC-SHA", "TLS-RSA-WITH-CAMELLIA-256-CBC-SHA", "TLS-RSA-WITH-CAMELLIA-128-CBC-SHA256",
                    "TLS-RSA-WITH-CAMELLIA-256-CBC-SHA256", "TLS-RSA-WITH-SEED-CBC-SHA", "TLS-RSA-WITH-ARIA-128-CBC-SHA256",
                    "TLS-RSA-WITH-ARIA-256-CBC-SHA384", "TLS-DHE-RSA-WITH-DES-CBC-SHA", "TLS-DHE-RSA-WITH-3DES-EDE-CBC-SHA",
                    "TLS-DHE-RSA-WITH-AES-128-CBC-SHA", "TLS-DHE-RSA-WITH-AES-256-CBC-SHA", "TLS-DHE-RSA-WITH-AES-128-CBC-SHA256",
                    "TLS-DHE-RSA-WITH-AES-256-CBC-SHA256", "TLS-DHE-RSA-WITH-CAMELLIA-128-CBC-SHA", "TLS-DHE-RSA-WITH-CAMELLIA-256-CBC-SHA",
                    "TLS-DHE-RSA-WITH-CAMELLIA-128-CBC-SHA256", "TLS-DHE-RSA-WITH-CAMELLIA-256-CBC-SHA256", "TLS-DHE-RSA-WITH-SEED-CBC-SHA",
                    "TLS-DHE-RSA-WITH-ARIA-128-CBC-SHA256", "TLS-DHE-RSA-WITH-ARIA-256-CBC-SHA384", "TLS-ECDHE-RSA-WITH-RC4-128-SHA",
                    "TLS-ECDHE-RSA-WITH-3DES-EDE-CBC-SHA", "TLS-ECDHE-RSA-WITH-AES-128-CBC-SHA", "TLS-ECDHE-RSA-WITH-AES-256-CBC-SHA",
                    "TLS-ECDHE-RSA-WITH-CHACHA20-POLY1305-SHA256", "TLS-ECDHE-ECDSA-WITH-CHACHA20-POLY1305-SHA256",
                    "TLS-DHE-RSA-WITH-CHACHA20-POLY1305-SHA256", "TLS-DHE-RSA-WITH-AES-128-GCM-SHA256", "TLS-DHE-RSA-WITH-AES-256-GCM-SHA384",
                    "TLS-DHE-DSS-WITH-AES-128-CBC-SHA", "TLS-DHE-DSS-WITH-AES-256-CBC-SHA", "TLS-DHE-DSS-WITH-AES-128-CBC-SHA256",
                    "TLS-DHE-DSS-WITH-AES-128-GCM-SHA256", "TLS-DHE-DSS-WITH-AES-256-CBC-SHA256", "TLS-DHE-DSS-WITH-AES-256-GCM-SHA384",
                    "TLS-ECDHE-RSA-WITH-AES-128-CBC-SHA256", "TLS-ECDHE-RSA-WITH-AES-128-GCM-SHA256", "TLS-ECDHE-RSA-WITH-AES-256-CBC-SHA384",
                    "TLS-ECDHE-RSA-WITH-AES-256-GCM-SHA384", "TLS-ECDHE-ECDSA-WITH-AES-128-CBC-SHA", "TLS-ECDHE-ECDSA-WITH-AES-128-CBC-SHA256",
                    "TLS-ECDHE-ECDSA-WITH-AES-128-GCM-SHA256", "TLS-ECDHE-ECDSA-WITH-AES-256-CBC-SHA384", "TLS-ECDHE-ECDSA-WITH-AES-256-GCM-SHA384",
                    "TLS-RSA-WITH-AES-128-GCM-SHA256", "TLS-RSA-WITH-AES-256-GCM-SHA384", "TLS-DHE-DSS-WITH-CAMELLIA-128-CBC-SHA",
                    "TLS-DHE-DSS-WITH-CAMELLIA-256-CBC-SHA", "TLS-DHE-DSS-WITH-CAMELLIA-128-CBC-SHA256", "TLS-DHE-DSS-WITH-CAMELLIA-256-CBC-SHA256",
                    "TLS-DHE-DSS-WITH-SEED-CBC-SHA", "TLS-DHE-DSS-WITH-ARIA-128-CBC-SHA256", "TLS-DHE-DSS-WITH-ARIA-256-CBC-SHA384",
                    "TLS-ECDHE-RSA-WITH-ARIA-128-CBC-SHA256", "TLS-ECDHE-RSA-WITH-ARIA-256-CBC-SHA384", "TLS-ECDHE-ECDSA-WITH-ARIA-128-CBC-SHA256",
                    "TLS-ECDHE-ECDSA-WITH-ARIA-256-CBC-SHA384", "TLS-DHE-DSS-WITH-3DES-EDE-CBC-SHA", "TLS-DHE-DSS-WITH-DES-CBC-SHA",
                    "TLS-AES-128-GCM-SHA256", "TLS-AES-256-GCM-SHA384", "TLS-CHACHA20-POLY1305-SHA256", "TLS-ECDHE-ECDSA-WITH-AES-256-CBC-SHA"
                ],
                "type": "str"
            },
            "id": {"required": true, "type": "int"},
            "versions": {"type": "list", "choices": ["ssl-3.0", "tls-1.0", "tls-1.1", "tls-1.2", "tls-1.3"], "elements": "str"},
            "priority": {"v_range": [["6.4.0", ""]], "type": "int"}
        }
    },
    "firewall_vip_sslserverciphersuites": {
        "stated": true,
        "options": {
            "cipher": {
                "choices": [
                    "TLS-RSA-WITH-RC4-128-MD5", "TLS-RSA-WITH-RC4-128-SHA", "TLS-RSA-WITH-DES-CBC-SHA", "TLS-RSA-WITH-3DES-EDE-CBC-SHA",
                    "TLS-RSA-WITH-AES-128-CBC-SHA", "TLS-RSA-WITH-AES-256-CBC-SHA", "TLS-RSA-WITH-AES-128-CBC-SHA256", "TLS-RSA-WITH-AES-256-CBC-SHA256",
                    "TLS-RSA-WITH-CAMELLIA-128-CBC-SHA", "TLS-RSA-WITH-CAMELLIA-256-CBC-SHA", "TLS-RSA-WITH-CAMELLIA-128-CBC-SHA256",
                    "TLS-RSA-WITH-CAMELLIA-256-CBC-SHA256", "TLS-RSA-WITH-SEED-CBC-SHA", "TLS-RSA-WITH-ARIA-128-CBC-SHA256",
                    "TLS-RSA-WITH-ARIA-256-CBC-SHA384", "TLS-DHE-RSA-WITH-DES-CBC-SHA", "TLS-DHE-RSA-WITH-3DES-EDE-CBC-SHA",
                    "TLS-DHE-RSA-WITH-AES-128-CBC-SHA", "TLS-DHE-RSA-WITH-AES-256-CBC-SHA", "TLS-DHE-RSA-WITH-AES-128-CBC-SHA256",
                    "TLS-DHE-RSA-WITH-AES-256-CBC-SHA256", "TLS-DHE-RSA-WITH-CAMELLIA-128-CBC-SHA", "TLS-DHE-RSA-WITH-CAMELLIA-256-CBC-SHA",
                    "TLS-DHE-RSA-WITH-CAMELLIA-128-CBC-SHA256", "TLS-DHE-RSA-WITH-CAMELLIA-256-CBC-SHA256", "TLS-DHE-RSA-WITH-SEED-CBC-SHA",
                    "TLS-DHE-RSA-WITH-ARIA-128-CBC-SHA256", "TLS-DHE-RSA-WITH-ARIA-256-CBC-SHA384", "TLS-ECDHE-RSA-WITH-RC4-128-SHA",
                    "TLS-ECDHE-RSA-WITH-3DES-EDE-CBC-SHA", "TLS-ECDHE-RSA-WITH-AES-128-CBC-SHA", "TLS-ECDHE-RSA-WITH-AES-256-CBC-SHA",
                    "TLS-ECDHE-RSA-WITH-CHACHA20-POLY1305-SHA256", "TLS-ECDHE-ECDSA-WITH-CHACHA20-POLY1305-SHA256",
                    "TLS-DHE-RSA-WITH-CHACHA20-POLY1305-SHA256", "TLS-DHE-RSA-WITH-AES-128-GCM-SHA256", "TLS-DHE-RSA-WITH-AES-256-GCM-SHA384",
                    "TLS-DHE-DSS-WITH-AES-128-CBC-SHA", "TLS-DHE-DSS-WITH-AES-256-CBC-SHA", "TLS-DHE-DSS-WITH-AES-128-CBC-SHA256",
                    "TLS-DHE-DSS-WITH-AES-128-GCM-SHA256", "TLS-DHE-DSS-WITH-AES-256-CBC-SHA256", "TLS-DHE-DSS-WITH-AES-256-GCM-SHA384",
                    "TLS-ECDHE-RSA-WITH-AES-128-CBC-SHA256", "TLS-ECDHE-RSA-WITH-AES-128-GCM-SHA256", "TLS-ECDHE-RSA-WITH-AES-256-CBC-SHA384",
                    "TLS-ECDHE-RSA-WITH-AES-256-GCM-SHA384", "TLS-ECDHE-ECDSA-WITH-AES-128-CBC-SHA", "TLS-ECDHE-ECDSA-WITH-AES-128-CBC-SHA256",
                    "TLS-ECDHE-ECDSA-WITH-AES-128-GCM-SHA256", "TLS-ECDHE-ECDSA-WITH-AES-256-CBC-SHA384", "TLS-ECDHE-ECDSA-WITH-AES-256-GCM-SHA384",
                    "TLS-RSA-WITH-AES-128-GCM-SHA256", "TLS-RSA-WITH-AES-256-GCM-SHA384", "TLS-DHE-DSS-WITH-CAMELLIA-128-CBC-SHA",
                    "TLS-DHE-DSS-WITH-CAMELLIA-256-CBC-SHA", "TLS-DHE-DSS-WITH-CAMELLIA-128-CBC-SHA256", "TLS-DHE-DSS-WITH-CAMELLIA-256-CBC-SHA256",
                    "TLS-DHE-DSS-WITH-SEED-CBC-SHA", "TLS-DHE-DSS-WITH-ARIA-128-CBC-SHA256", "TLS-DHE-DSS-WITH-ARIA-256-CBC-SHA384",
                    "TLS-ECDHE-RSA-WITH-ARIA-128-CBC-SHA256", "TLS-ECDHE-RSA-WITH-ARIA-256-CBC-SHA384", "TLS-ECDHE-ECDSA-WITH-ARIA-128-CBC-SHA256",
                    "TLS-ECDHE-ECDSA-WITH-ARIA-256-CBC-SHA384", "TLS-DHE-DSS-WITH-3DES-EDE-CBC-SHA", "TLS-DHE-DSS-WITH-DES-CBC-SHA",
                    "TLS-AES-128-GCM-SHA256", "TLS-AES-256-GCM-SHA384", "TLS-CHACHA20-POLY1305-SHA256", "TLS-ECDHE-ECDSA-WITH-AES-256-CBC-SHA"
                ],
                "type": "str"
            },
            "priority": {"required": true, "type": "int"},
            "versions": {"type": "list", "choices": ["ssl-3.0", "tls-1.0", "tls-1.1", "tls-1.2", "tls-1.3"], "elements": "str"}
        }
    },
    "firewall_vipgrp": {
        "stated": true,
        "options": {
            "color": {"type": "int"},
            "comments": {"type": "str"},
            "dynamic_mapping": {
                "type": "list",
                "options": {
                    "_scope": {"type": "list", "options": {"name": {"type": "str"}, "vdom": {"type": "str"}}, "elements": "dict"},
                    "color": {"type": "int"},
                    "comments": {"type": "str"},
                    "interface": {"type": "str"},
                    "member": {"type": "raw"},
                    "uuid": {"type": "str"}
                },
                "elements": "dict"
            },
            "interface": {"type": "str"},
            "member": {"type": "raw"},
            "name": {"required": true, "type": "str"},
            "uuid": {"type": "str"}
        }
    },
    "firewall_vipgrp46": {
        "stated": true,
        "options": {
            "color": {"type": "int"},
            "comments": {"type": "str"},
            "member": {"type": "raw"},
            "name": {"required": true, "type": "str"},
            "uuid": {"type": "str"}
        }
    },
    "firewall_vipgrp6": {
        "stated": true,
        "options": {
            "color": {"type": "int"},
            "comments": {"type": "str"},
            "member": {"type": "raw"},
            "name": {"required": true, "type": "str"},
            "uuid": {"type": "str"}
        }
    },
    "firewall_vipgrp64": {
        "stated": true,
        "options": {
            "color": {"type": "int"},
            "comments": {"type": "str"},
            "member": {"type": "raw"},
            "name": {"required": true, "type": "str"},
            "uuid": {"type": "str"}
        }
    },
    "firewall_vipgrp_dynamicmapping": {
        "stated": true,
        "options": {
            "_scope": {"type": "list", "options": {"name": {"type": "str"}, "vdom": {"type": "str"}}, "elements": "dict"},
            "color": {"type": "int"},
            "comments": {"type": "str"},
            "interface": {"type": "str"},
            "member": {"type": "raw"},
            "uuid": {"type": "str"}
        }
    },
    "firewall_wildcardfqdn_custom": {
        "stated": true,
        "options": {
            "color": {"type": "int"},
            "comment": {"type": "str"},
            "name": {"required": true, "type": "str"},
            "uuid": {"type": "str"},
            "visibility": {"choices": ["disable", "enable"], "type": "str"},
            "wildcard-fqdn": {"type": "str"}
        }
    },
    "firewall_wildcardfqdn_group": {
        "stated": true,
        "options": {
            "color": {"type": "int"},
            "comment": {"type": "str"},
            "member": {"type": "raw"},
            "name": {"required": true, "type": "str"},
            "uuid": {"type": "str"},
            "visibility": {"choices": ["disable", "enable"], "type": "str"}
        }
    },
    "fmg_device_blueprint": {
        "stated": true,
        "options": {
            "cliprofs": {"v_range": [["7.2.0", ""]], "type": "raw"},
            "description": {"v_range": [["7.2.0", ""]], "type": "str"},
            "dev-group": {"v_range": [["7.2.0", ""]], "type": "raw"},
            "folder": {"v_range": [["7.2.0", ""]], "type": "str"},
            "name": {"v_range": [["7.2.0", ""]], "required": true, "type": "str"},
            "pkg": {"v_range": [["7.2.0", ""]], "type": "str"},
            "platform": {"v_range": [["7.2.0", ""]], "type": "str"},
            "prefer-img-ver": {"v_range": [["7.2.0", ""]], "type": "str"},
            "prerun-cliprof": {"v_range": [["7.2.0", ""]], "type": "raw"},
            "prov-type": {"v_range": [["7.2.0", ""]], "choices": ["none", "templates", "template-group"], "type": "str"},
            "template-group": {"v_range": [["7.2.0", ""]], "type": "str"},
            "templates": {"v_range": [["7.2.0", ""]], "type": "raw"},
            "enforce-device-config": {"v_range": [["7.2.5", "7.2.5"], ["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "auth-template": {"v_range": [["7.4.1", ""]], "type": "raw"},
            "ha-config": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ha-hbdev": {"v_range": [["7.4.1", ""]], "type": "str"},
            "ha-monitor": {"v_range": [["7.4.1", ""]], "type": "raw"},
            "ha-password": {"v_range": [["7.4.1", ""]], "no_log": true, "type": "raw"},
            "linked-to-model": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "port-provisioning": {"v_range": [["7.6.0", ""]], "type": "int"},
            "sdwan-management": {"v_range": [["7.6.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "split-switch-port": {"v_range": [["7.6.0", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "fmg_fabric_authorization_template": {
        "stated": true,
        "options": {
            "description": {"v_range": [["7.2.1", ""]], "type": "str"},
            "extender-controller": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "name": {"v_range": [["7.2.1", ""]], "required": true, "type": "str"},
            "platforms": {
                "v_range": [["7.2.1", ""]],
                "type": "list",
                "options": {
                    "count": {"v_range": [["7.2.1", ""]], "type": "int"},
                    "extension-type": {"v_range": [["7.2.1", ""]], "choices": ["wan-extension", "lan-extension"], "type": "str"},
                    "fortilink": {"v_range": [["7.2.1", ""]], "type": "str"},
                    "prefix": {"v_range": [["7.2.1", ""]], "type": "str"},
                    "type": {"v_range": [["7.2.1", ""]], "choices": ["ap", "extender", "switch"], "type": "str"}
                },
                "elements": "dict"
            },
            "switch-controller": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "wireless-controller": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "fmg_fabric_authorization_template_platforms": {
        "stated": true,
        "options": {
            "count": {"v_range": [["7.2.1", ""]], "type": "int"},
            "extension-type": {"v_range": [["7.2.1", ""]], "choices": ["wan-extension", "lan-extension"], "type": "str"},
            "fortilink": {"v_range": [["7.2.1", ""]], "type": "str"},
            "prefix": {"v_range": [["7.2.1", ""]], "type": "str"},
            "type": {"v_range": [["7.2.1", ""]], "choices": ["ap", "extender", "switch"], "type": "str"}
        }
    },
    "fmg_sasemanager_settings": {
        "stated": false,
        "options": {
            "address": {"v_range": [["7.6.0", ""]], "type": "list", "elements": "str"},
            "profile-group": {"v_range": [["7.6.0", ""]], "type": "list", "elements": "str"},
            "sync-address": {"v_range": [["7.6.0", ""]], "choices": ["disable", "specify", "all"], "type": "str"},
            "sync-profile-group": {"v_range": [["7.6.0", ""]], "choices": ["disable", "specify", "all"], "type": "str"},
            "sync-user": {"v_range": [["7.6.0", ""]], "choices": ["disable", "specify", "all"], "type": "str"},
            "user": {"v_range": [["7.6.0", ""]], "type": "list", "elements": "str"}
        }
    },
    "fmg_sasemanager_status": {
        "stated": false,
        "options": {
            "forticlient-ver": {"v_range": [["7.6.0", ""]], "type": "str"},
            "forticloud-id": {"v_range": [["7.6.0", ""]], "type": "int"},
            "license-type": {"v_range": [["7.6.0", ""]], "choices": ["standard_license", "advanced_license", "comprehensive_license"], "type": "str"},
            "spa-hubs": {"v_range": [["7.6.0", ""]], "type": "int"}
        }
    },
    "fmg_variable": {
        "stated": true,
        "options": {
            "description": {"v_range": [["7.2.0", ""]], "type": "str"},
            "dynamic_mapping": {
                "v_range": [["7.2.0", ""]],
                "type": "list",
                "options": {
                    "_scope": {
                        "v_range": [["7.2.0", ""]],
                        "type": "list",
                        "options": {"name": {"v_range": [["7.2.0", ""]], "type": "str"}, "vdom": {"v_range": [["7.2.0", ""]], "type": "str"}},
                        "elements": "dict"
                    },
                    "value": {"v_range": [["7.2.0", ""]], "type": "str"}
                },
                "elements": "dict"
            },
            "name": {"v_range": [["7.2.0", ""]], "required": true, "type": "str"},
            "value": {"v_range": [["7.2.0", ""]], "type": "str"}
        }
    },
    "fmg_variable_dynamicmapping": {
        "stated": true,
        "options": {
            "_scope": {
                "v_range": [["7.2.0", ""]],
                "type": "list",
                "options": {"name": {"v_range": [["7.2.0", ""]], "type": "str"}, "vdom": {"v_range": [["7.2.0", ""]], "type": "str"}},
                "elements": "dict"
            },
            "value": {"v_range": [["7.2.0", ""]], "type": "str"}
        }
    },
    "fmupdate_analyzer_virusreport": {"stated": false, "options": {"status": {"choices": ["disable", "enable"], "type": "str"}}},
    "fmupdate_avips_advancedlog": {
        "stated": false,
        "options": {"log-fortigate": {"choices": ["disable", "enable"], "type": "str"}, "log-server": {"choices": ["disable", "enable"], "type": "str"}}
    },
    "fmupdate_avips_webproxy": {
        "stated": false,
        "options": {
            "ip": {"v_range": [["6.0.0", "6.2.1"]], "type": "str"},
            "ip6": {"v_range": [["6.0.0", "6.2.1"]], "type": "str"},
            "mode": {"v_range": [["6.0.0", "7.4.0"]], "choices": ["proxy", "tunnel"], "type": "str"},
            "password": {"v_range": [["6.0.0", "7.4.0"]], "no_log": true, "type": "raw"},
            "port": {"v_range": [["6.0.0", "7.4.0"]], "type": "int"},
            "status": {"v_range": [["6.0.0", "7.4.0"]], "choices": ["disable", "enable"], "type": "str"},
            "username": {"v_range": [["6.0.0", "7.4.0"]], "type": "str"},
            "address": {"v_range": [["6.2.2", "7.4.0"]], "type": "str"}
        }
    },
    "fmupdate_customurllist": {
        "stated": false,
        "options": {"db_selection": {"type": "list", "choices": ["both", "custom-url", "fortiguard-db"], "elements": "str"}}
    },
    "fmupdate_diskquota": {"stated": false, "options": {"value": {"type": "int"}}},
    "fmupdate_fctservices": {"stated": false, "options": {"port": {"type": "int"}, "status": {"choices": ["disable", "enable"], "type": "str"}}},
    "fmupdate_fdssetting": {
        "stated": false,
        "options": {
            "User-Agent": {"type": "str"},
            "fds-clt-ssl-protocol": {"choices": ["sslv3", "tlsv1.0", "tlsv1.1", "tlsv1.2", "tlsv1.3"], "type": "str"},
            "fds-ssl-protocol": {"choices": ["sslv3", "tlsv1.0", "tlsv1.1", "tlsv1.2", "tlsv1.3"], "type": "str"},
            "fmtr-log": {"choices": ["emergency", "alert", "critical", "error", "warn", "notice", "info", "debug", "disable"], "type": "str"},
            "linkd-log": {"choices": ["emergency", "alert", "critical", "error", "warn", "notice", "info", "debug", "disable"], "type": "str"},
            "max-av-ips-version": {"type": "int"},
            "max-work": {"type": "int"},
            "push-override": {
                "type": "dict",
                "options": {"ip": {"type": "str"}, "port": {"type": "int"}, "status": {"choices": ["disable", "enable"], "type": "str"}}
            },
            "push-override-to-client": {
                "type": "dict",
                "options": {
                    "announce-ip": {
                        "type": "list",
                        "options": {"id": {"type": "int"}, "ip": {"type": "str"}, "port": {"type": "int"}},
                        "elements": "dict"
                    },
                    "status": {"choices": ["disable", "enable"], "type": "str"}
                }
            },
            "send_report": {"choices": ["disable", "enable"], "type": "str"},
            "send_setup": {"choices": ["disable", "enable"], "type": "str"},
            "server-override": {
                "type": "dict",
                "options": {
                    "servlist": {
                        "type": "list",
                        "options": {
                            "id": {"type": "int"},
                            "ip": {"type": "str"},
                            "ip6": {"type": "str"},
                            "port": {"type": "int"},
                            "service-type": {"type": "raw", "choices": ["fds", "fct", "fai"]}
                        },
                        "elements": "dict"
                    },
                    "status": {"choices": ["disable", "enable"], "type": "str"}
                }
            },
            "system-support-fct": {"type": "list", "choices": ["4.x", "5.0", "5.2", "5.4", "5.6", "6.0", "6.2", "6.4", "7.0", "7.2"], "elements": "str"},
            "system-support-fgt": {"type": "list", "choices": ["5.4", "5.6", "6.0", "6.2", "6.4", "7.0", "7.2", "7.4", "7.6"], "elements": "str"},
            "system-support-fml": {"type": "list", "choices": ["4.x", "5.x", "6.x", "6.0", "6.2", "6.4", "7.0", "7.2", "7.x"], "elements": "str"},
            "system-support-fsa": {"type": "list", "choices": ["1.x", "2.x", "3.x", "4.x", "3.0", "3.1", "3.2"], "elements": "str"},
            "system-support-fsw": {
                "v_range": [["6.0.0", "6.4.5"], ["7.0.0", "7.0.0"]],
                "type": "list",
                "choices": ["5.4", "5.6", "6.0", "6.2", "4.x", "5.0", "5.2", "6.4"],
                "elements": "str"
            },
            "umsvc-log": {"choices": ["emergency", "alert", "critical", "error", "warn", "notice", "info", "debug", "disable"], "type": "str"},
            "unreg-dev-option": {"choices": ["ignore", "svc-only", "add-service"], "type": "str"},
            "update-schedule": {
                "type": "dict",
                "options": {
                    "day": {"choices": ["Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday"], "type": "str"},
                    "frequency": {"choices": ["every", "daily", "weekly"], "type": "str"},
                    "status": {"choices": ["disable", "enable"], "type": "str"},
                    "time": {"type": "raw"}
                }
            },
            "wanip-query-mode": {"choices": ["disable", "ipify"], "type": "str"},
            "fortiguard-anycast": {"v_range": [["6.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "fortiguard-anycast-source": {"v_range": [["6.4.0", ""]], "choices": ["fortinet", "aws"], "type": "str"},
            "system-support-fdc": {"v_range": [["6.4.6", "6.4.14"], ["7.0.1", ""]], "type": "list", "choices": ["3.x", "4.x"], "elements": "str"},
            "system-support-fts": {"v_range": [["6.4.6", "6.4.14"], ["7.0.1", ""]], "type": "list", "choices": ["3.x", "4.x", "7.x"], "elements": "str"},
            "system-support-faz": {"v_range": [["7.0.7", "7.0.12"], ["7.2.2", ""]], "type": "list", "choices": ["6.x", "7.x"], "elements": "str"},
            "system-support-fis": {"v_range": [["7.4.0", ""]], "type": "list", "choices": ["1.x", "2.x"], "elements": "str"},
            "system-support-fai": {"v_range": [["7.6.0", ""]], "type": "list", "choices": ["7.x"], "elements": "str"}
        }
    },
    "fmupdate_fdssetting_pushoverride": {
        "stated": false,
        "options": {"ip": {"type": "str"}, "port": {"type": "int"}, "status": {"choices": ["disable", "enable"], "type": "str"}}
    },
    "fmupdate_fdssetting_pushoverridetoclient": {
        "stated": false,
        "options": {
            "announce-ip": {"type": "list", "options": {"id": {"type": "int"}, "ip": {"type": "str"}, "port": {"type": "int"}}, "elements": "dict"},
            "status": {"choices": ["disable", "enable"], "type": "str"}
        }
    },
    "fmupdate_fdssetting_pushoverridetoclient_announceip": {
        "stated": true,
        "options": {"id": {"required": true, "type": "int"}, "ip": {"type": "str"}, "port": {"type": "int"}}
    },
    "fmupdate_fdssetting_serveroverride": {
        "stated": false,
        "options": {
            "servlist": {
                "type": "list",
                "options": {
                    "id": {"type": "int"},
                    "ip": {"type": "str"},
                    "ip6": {"type": "str"},
                    "port": {"type": "int"},
                    "service-type": {"type": "raw", "choices": ["fds", "fct", "fai"]}
                },
                "elements": "dict"
            },
            "status": {"choices": ["disable", "enable"], "type": "str"}
        }
    },
    "fmupdate_fdssetting_serveroverride_servlist": {
        "stated": true,
        "options": {
            "id": {"required": true, "type": "int"},
            "ip": {"type": "str"},
            "ip6": {"type": "str"},
            "port": {"type": "int"},
            "service-type": {"type": "raw", "choices": ["fds", "fct", "fai"]}
        }
    },
    "fmupdate_fdssetting_updateschedule": {
        "stated": false,
        "options": {
            "day": {"choices": ["Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday"], "type": "str"},
            "frequency": {"choices": ["every", "daily", "weekly"], "type": "str"},
            "status": {"choices": ["disable", "enable"], "type": "str"},
            "time": {"type": "raw"}
        }
    },
    "fmupdate_fwmsetting": {
        "stated": false,
        "options": {
            "fds-image-timeout": {"v_range": [["6.2.2", ""]], "type": "int"},
            "max-fds-retry": {"v_range": [["6.2.2", "6.2.3"]], "type": "int"},
            "multiple-steps-interval": {"v_range": [["6.2.2", ""]], "type": "int"},
            "skip-disk-check": {"v_range": [["6.2.2", "6.2.3"]], "choices": ["disable", "enable"], "type": "str"},
            "auto-scan-fgt-disk": {"v_range": [["6.2.5", ""]], "choices": ["disable", "enable"], "type": "str"},
            "check-fgt-disk": {"v_range": [["6.2.5", ""]], "choices": ["disable", "enable"], "type": "str"},
            "fds-failover-fmg": {"v_range": [["6.2.5", ""]], "choices": ["disable", "enable"], "type": "str"},
            "immx-source": {"v_range": [["6.4.2", ""]], "choices": ["fmg", "fgt", "cloud"], "type": "str"},
            "log": {"v_range": [["6.4.8", "6.4.14"], ["7.0.1", ""]], "choices": ["fwm", "fwm_dm", "fwm_dm_json"], "type": "str"},
            "upgrade-timeout": {
                "v_range": [["7.0.5", "7.0.12"], ["7.2.2", ""]],
                "type": "dict",
                "options": {
                    "check-status-timeout": {"v_range": [["7.0.5", "7.0.12"], ["7.2.2", ""]], "type": "int"},
                    "ctrl-check-status-timeout": {"v_range": [["7.0.5", "7.0.12"], ["7.2.2", ""]], "type": "int"},
                    "ctrl-put-image-by-fds-timeout": {"v_range": [["7.0.5", "7.0.12"], ["7.2.2", ""]], "type": "int"},
                    "ha-sync-timeout": {"v_range": [["7.0.5", "7.0.12"], ["7.2.2", ""]], "type": "int"},
                    "license-check-timeout": {"v_range": [["7.0.5", "7.0.12"], ["7.2.2", ""]], "type": "int"},
                    "prepare-image-timeout": {"v_range": [["7.0.5", "7.0.12"], ["7.2.2", ""]], "type": "int"},
                    "put-image-by-fds-timeout": {"v_range": [["7.0.5", "7.0.12"], ["7.2.2", ""]], "type": "int"},
                    "put-image-timeout": {"v_range": [["7.0.5", "7.0.12"], ["7.2.2", ""]], "type": "int"},
                    "reboot-of-fsck-timeout": {"v_range": [["7.0.5", "7.0.12"], ["7.2.2", ""]], "type": "int"},
                    "reboot-of-upgrade-timeout": {"v_range": [["7.0.5", "7.0.12"], ["7.2.2", ""]], "type": "int"},
                    "retrieve-timeout": {"v_range": [["7.0.5", "7.0.12"], ["7.2.2", ""]], "type": "int"},
                    "rpc-timeout": {"v_range": [["7.0.5", "7.0.12"], ["7.2.2", ""]], "type": "int"},
                    "total-timeout": {"v_range": [["7.0.5", "7.0.12"], ["7.2.2", ""]], "type": "int"},
                    "health-check-timeout": {"v_range": [["7.4.2", ""]], "type": "int"}
                }
            },
            "retry-interval": {"v_range": [["7.0.10", "7.0.12"], ["7.2.5", "7.2.5"], ["7.4.2", ""]], "type": "int"},
            "retry-max": {"v_range": [["7.0.10", "7.0.12"], ["7.2.5", "7.2.5"], ["7.4.2", ""]], "type": "int"},
            "health-check": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "max-device-history": {"v_range": [["7.4.2", ""]], "type": "int"},
            "max-profile-history": {"v_range": [["7.4.2", ""]], "type": "int"},
            "retrieve": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "revision-diff": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "fmupdate_fwmsetting_upgradetimeout": {
        "stated": false,
        "options": {
            "check-status-timeout": {"v_range": [["7.0.5", "7.0.12"], ["7.2.2", ""]], "type": "int"},
            "ctrl-check-status-timeout": {"v_range": [["7.0.5", "7.0.12"], ["7.2.2", ""]], "type": "int"},
            "ctrl-put-image-by-fds-timeout": {"v_range": [["7.0.5", "7.0.12"], ["7.2.2", ""]], "type": "int"},
            "ha-sync-timeout": {"v_range": [["7.0.5", "7.0.12"], ["7.2.2", ""]], "type": "int"},
            "license-check-timeout": {"v_range": [["7.0.5", "7.0.12"], ["7.2.2", ""]], "type": "int"},
            "prepare-image-timeout": {"v_range": [["7.0.5", "7.0.12"], ["7.2.2", ""]], "type": "int"},
            "put-image-by-fds-timeout": {"v_range": [["7.0.5", "7.0.12"], ["7.2.2", ""]], "type": "int"},
            "put-image-timeout": {"v_range": [["7.0.5", "7.0.12"], ["7.2.2", ""]], "type": "int"},
            "reboot-of-fsck-timeout": {"v_range": [["7.0.5", "7.0.12"], ["7.2.2", ""]], "type": "int"},
            "reboot-of-upgrade-timeout": {"v_range": [["7.0.5", "7.0.12"], ["7.2.2", ""]], "type": "int"},
            "retrieve-timeout": {"v_range": [["7.0.5", "7.0.12"], ["7.2.2", ""]], "type": "int"},
            "rpc-timeout": {"v_range": [["7.0.5", "7.0.12"], ["7.2.2", ""]], "type": "int"},
            "total-timeout": {"v_range": [["7.0.5", "7.0.12"], ["7.2.2", ""]], "type": "int"},
            "health-check-timeout": {"v_range": [["7.4.2", ""]], "type": "int"}
        }
    },
    "fmupdate_multilayer": {"stated": false, "options": {"webspam-rating": {"choices": ["disable", "enable"], "type": "str"}}},
    "fmupdate_publicnetwork": {"stated": false, "options": {"status": {"choices": ["disable", "enable"], "type": "str"}}},
    "fmupdate_serveraccesspriorities": {
        "stated": false,
        "options": {
            "access-public": {"choices": ["disable", "enable"], "type": "str"},
            "av-ips": {"choices": ["disable", "enable"], "type": "str"},
            "private-server": {
                "type": "list",
                "options": {"id": {"type": "int"}, "ip": {"type": "str"}, "ip6": {"type": "str"}, "time_zone": {"type": "int"}},
                "elements": "dict"
            },
            "web-spam": {"choices": ["disable", "enable"], "type": "str"}
        }
    },
    "fmupdate_serveraccesspriorities_privateserver": {
        "stated": true,
        "options": {"id": {"required": true, "type": "int"}, "ip": {"type": "str"}, "ip6": {"type": "str"}, "time_zone": {"type": "int"}}
    },
    "fmupdate_serveroverridestatus": {"stated": false, "options": {"mode": {"choices": ["strict", "loose"], "type": "str"}}},
    "fmupdate_service": {
        "stated": false,
        "options": {
            "avips": {"choices": ["disable", "enable"], "type": "str"},
            "query-antispam": {"choices": ["disable", "enable"], "type": "str"},
            "query-antivirus": {"choices": ["disable", "enable"], "type": "str"},
            "query-filequery": {"choices": ["disable", "enable"], "type": "str"},
            "query-geoip": {"v_range": [["6.0.0", "6.4.1"]], "choices": ["disable", "enable"], "type": "str"},
            "query-outbreak-prevention": {"choices": ["disable", "enable"], "type": "str"},
            "query-webfilter": {"choices": ["disable", "enable"], "type": "str"},
            "webfilter-https-traversal": {"choices": ["disable", "enable"], "type": "str"},
            "query-iot": {"v_range": [["6.4.6", "6.4.14"], ["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "query-iot-collection": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "query-iot-vulnerability": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "query-ioc": {"v_range": [["7.6.0", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "fmupdate_webspam_fgdsetting": {
        "stated": false,
        "options": {
            "as-cache": {"type": "int"},
            "as-log": {"choices": ["disable", "nospam", "all"], "type": "str"},
            "as-preload": {"choices": ["disable", "enable"], "type": "str"},
            "av-cache": {"type": "int"},
            "av-log": {"choices": ["disable", "novirus", "all"], "type": "str"},
            "av-preload": {"choices": ["disable", "enable"], "type": "str"},
            "av2-cache": {"type": "int"},
            "av2-log": {"choices": ["disable", "noav2", "all"], "type": "str"},
            "av2-preload": {"choices": ["disable", "enable"], "type": "str"},
            "eventlog-query": {"choices": ["disable", "enable"], "type": "str"},
            "fgd-pull-interval": {"type": "int"},
            "fq-cache": {"type": "int"},
            "fq-log": {"choices": ["disable", "nofilequery", "all"], "type": "str"},
            "fq-preload": {"choices": ["disable", "enable"], "type": "str"},
            "linkd-log": {"choices": ["emergency", "alert", "critical", "error", "warn", "notice", "info", "debug", "disable"], "type": "str"},
            "max-client-worker": {"type": "int"},
            "max-log-quota": {"type": "int"},
            "max-unrated-site": {"type": "int"},
            "restrict-as1-dbver": {"type": "str"},
            "restrict-as2-dbver": {"type": "str"},
            "restrict-as4-dbver": {"type": "str"},
            "restrict-av-dbver": {"type": "str"},
            "restrict-av2-dbver": {"type": "str"},
            "restrict-fq-dbver": {"type": "str"},
            "restrict-wf-dbver": {"type": "str"},
            "server-override": {
                "type": "dict",
                "options": {
                    "servlist": {
                        "type": "list",
                        "options": {
                            "id": {"type": "int"},
                            "ip": {"type": "str"},
                            "ip6": {"type": "str"},
                            "port": {"type": "int"},
                            "service-type": {"type": "raw", "choices": ["fgd", "fgc", "fsa", "fgfq", "geoip", "iot-collect"]}
                        },
                        "elements": "dict"
                    },
                    "status": {"choices": ["disable", "enable"], "type": "str"}
                }
            },
            "stat-log-interval": {"type": "int"},
            "stat-sync-interval": {"type": "int"},
            "update-interval": {"type": "int"},
            "update-log": {"choices": ["disable", "enable"], "type": "str"},
            "wf-cache": {"type": "int"},
            "wf-dn-cache-expire-time": {"type": "int"},
            "wf-dn-cache-max-number": {"type": "int"},
            "wf-log": {"choices": ["disable", "nourl", "all"], "type": "str"},
            "wf-preload": {"choices": ["disable", "enable"], "type": "str"},
            "iot-cache": {"v_range": [["6.4.6", "6.4.14"], ["7.0.1", ""]], "type": "int"},
            "iot-log": {"v_range": [["6.4.6", "6.4.14"], ["7.0.1", ""]], "choices": ["disable", "nofilequery", "all", "noiot"], "type": "str"},
            "iot-preload": {"v_range": [["6.4.6", "6.4.14"], ["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "restrict-iots-dbver": {"v_range": [["6.4.6", "6.4.14"], ["7.0.1", ""]], "type": "str"},
            "stat-log": {
                "v_range": [["7.0.10", "7.0.12"], ["7.2.5", "7.2.5"], ["7.4.2", ""]],
                "choices": ["emergency", "alert", "critical", "error", "warn", "notice", "info", "debug", "disable"],
                "type": "str"
            },
            "iotv-preload": {"v_range": [["7.2.2", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "fmupdate_webspam_fgdsetting_serveroverride": {
        "stated": false,
        "options": {
            "servlist": {
                "type": "list",
                "options": {
                    "id": {"type": "int"},
                    "ip": {"type": "str"},
                    "ip6": {"type": "str"},
                    "port": {"type": "int"},
                    "service-type": {"type": "raw", "choices": ["fgd", "fgc", "fsa", "fgfq", "geoip", "iot-collect"]}
                },
                "elements": "dict"
            },
            "status": {"choices": ["disable", "enable"], "type": "str"}
        }
    },
    "fmupdate_webspam_fgdsetting_serveroverride_servlist": {
        "stated": true,
        "options": {
            "id": {"required": true, "type": "int"},
            "ip": {"type": "str"},
            "ip6": {"type": "str"},
            "port": {"type": "int"},
            "service-type": {"type": "raw", "choices": ["fgd", "fgc", "fsa", "fgfq", "geoip", "iot-collect"]}
        }
    },
    "fmupdate_webspam_webproxy": {
        "stated": false,
        "options": {
            "ip": {"v_range": [["6.0.0", "6.2.1"]], "type": "str"},
            "ip6": {"v_range": [["6.0.0", "6.2.1"]], "type": "str"},
            "mode": {"v_range": [["6.0.0", "7.4.0"]], "choices": ["proxy", "tunnel"], "type": "str"},
            "password": {"v_range": [["6.0.0", "7.4.0"]], "no_log": true, "type": "raw"},
            "port": {"v_range": [["6.0.0", "7.4.0"]], "type": "int"},
            "status": {"v_range": [["6.0.0", "7.4.0"]], "choices": ["disable", "enable"], "type": "str"},
            "username": {"v_range": [["6.0.0", "7.4.0"]], "type": "str"},
            "address": {"v_range": [["6.2.2", "7.4.0"]], "type": "str"}
        }
    },
    "fsp_vlan": {
        "stated": true,
        "options": {
            "_dhcp-status": {"choices": ["disable", "enable"], "type": "str"},
            "auth": {"v_range": [["6.0.0", "6.2.1"]], "choices": ["radius", "usergroup"], "type": "str"},
            "color": {"type": "int"},
            "comments": {"v_range": [["6.0.0", "6.2.1"]], "type": "str"},
            "dynamic_mapping": {
                "type": "list",
                "options": {
                    "_dhcp-status": {"choices": ["disable", "enable"], "type": "str"},
                    "_scope": {"type": "list", "options": {"name": {"type": "str"}, "vdom": {"type": "str"}}, "elements": "dict"},
                    "dhcp-server": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "type": "dict",
                        "options": {
                            "auto-configuration": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "auto-managed-status": {"v_range": [["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "conflicted-ip-timeout": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                            "ddns-auth": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "tsig"], "type": "str"},
                            "ddns-key": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "no_log": true, "type": "raw"},
                            "ddns-keyname": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "no_log": true, "type": "str"},
                            "ddns-server-ip": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                            "ddns-ttl": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                            "ddns-update": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "ddns-update-override": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "ddns-zone": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                            "default-gateway": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                            "dhcp-settings-from-fortiipam": {"v_range": [["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "dns-server1": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                            "dns-server2": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                            "dns-server3": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                            "dns-server4": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                            "dns-service": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["default", "specify", "local"], "type": "str"},
                            "domain": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                            "enable": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "exclude-range": {
                                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                                "type": "list",
                                "options": {
                                    "end-ip": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                                    "id": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                                    "start-ip": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                                    "vci-match": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                                    "vci-string": {"v_range": [["7.2.1", ""]], "type": "raw"},
                                    "lease-time": {"v_range": [["7.2.2", ""]], "type": "int"},
                                    "uci-match": {"v_range": [["7.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                                    "uci-string": {"v_range": [["7.2.2", ""]], "type": "raw"}
                                },
                                "elements": "dict"
                            },
                            "filename": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                            "forticlient-on-net-status": {
                                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                                "choices": ["disable", "enable"],
                                "type": "str"
                            },
                            "id": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                            "ip-mode": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["range", "usrgrp"], "type": "str"},
                            "ip-range": {
                                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                                "type": "list",
                                "options": {
                                    "end-ip": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                                    "id": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                                    "start-ip": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                                    "vci-match": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                                    "vci-string": {"v_range": [["7.2.1", ""]], "type": "raw"},
                                    "lease-time": {"v_range": [["7.2.2", ""]], "type": "int"},
                                    "uci-match": {"v_range": [["7.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                                    "uci-string": {"v_range": [["7.2.2", ""]], "type": "raw"}
                                },
                                "elements": "dict"
                            },
                            "ipsec-lease-hold": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                            "lease-time": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                            "mac-acl-default-action": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["assign", "block"], "type": "str"},
                            "netmask": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                            "next-server": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                            "ntp-server1": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                            "ntp-server2": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                            "ntp-server3": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                            "ntp-service": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["default", "specify", "local"], "type": "str"},
                            "option1": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "raw"},
                            "option2": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "raw"},
                            "option3": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "raw"},
                            "option4": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                            "option5": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                            "option6": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                            "options": {
                                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                                "type": "list",
                                "options": {
                                    "code": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                                    "id": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                                    "ip": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "raw"},
                                    "type": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["hex", "string", "ip", "fqdn"], "type": "str"},
                                    "value": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                                    "vci-match": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                                    "vci-string": {"v_range": [["7.2.1", ""]], "type": "raw"},
                                    "uci-match": {"v_range": [["7.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                                    "uci-string": {"v_range": [["7.2.2", ""]], "type": "raw"}
                                },
                                "elements": "dict"
                            },
                            "reserved-address": {
                                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                                "type": "list",
                                "options": {
                                    "action": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["assign", "block", "reserved"], "type": "str"},
                                    "circuit-id": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                                    "circuit-id-type": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["hex", "string"], "type": "str"},
                                    "description": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                                    "id": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                                    "ip": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                                    "mac": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                                    "remote-id": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                                    "remote-id-type": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["hex", "string"], "type": "str"},
                                    "type": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["mac", "option82"], "type": "str"}
                                },
                                "elements": "dict"
                            },
                            "server-type": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["regular", "ipsec"], "type": "str"},
                            "status": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "tftp-server": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "raw"},
                            "timezone": {
                                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                                "choices": [
                                    "00", "01", "02", "03", "04", "05", "06", "07", "08", "09", "10", "11", "12", "13", "14", "15", "16", "17", "18",
                                    "19", "20", "21", "22", "23", "24", "25", "26", "27", "28", "29", "30", "31", "32", "33", "34", "35", "36", "37",
                                    "38", "39", "40", "41", "42", "43", "44", "45", "46", "47", "48", "49", "50", "51", "52", "53", "54", "55", "56",
                                    "57", "58", "59", "60", "61", "62", "63", "64", "65", "66", "67", "68", "69", "70", "71", "72", "73", "74", "75",
                                    "76", "77", "78", "79", "80", "81", "82", "83", "84", "85", "86", "87"
                                ],
                                "type": "str"
                            },
                            "timezone-option": {
                                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                                "choices": ["disable", "default", "specify"],
                                "type": "str"
                            },
                            "vci-match": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "vci-string": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "raw"},
                            "wifi-ac-service": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["specify", "local"], "type": "str"},
                            "wifi-ac1": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                            "wifi-ac2": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                            "wifi-ac3": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                            "wins-server1": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                            "wins-server2": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                            "relay-agent": {"v_range": [["7.4.0", ""]], "type": "str"},
                            "shared-subnet": {"v_range": [["7.4.0", ""]], "choices": ["disable", "enable"], "type": "str"}
                        }
                    },
                    "interface": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "type": "dict",
                        "options": {
                            "dhcp-relay-agent-option": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "dhcp-relay-ip": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "raw"},
                            "dhcp-relay-service": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "dhcp-relay-type": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["regular", "ipsec"], "type": "str"},
                            "ip": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                            "ipv6": {
                                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                                "type": "dict",
                                "options": {
                                    "autoconf": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                                    "dhcp6-client-options": {
                                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                                        "type": "list",
                                        "choices": ["rapid", "iapd", "iana", "dns", "dnsname"],
                                        "elements": "str"
                                    },
                                    "dhcp6-information-request": {
                                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                                        "choices": ["disable", "enable"],
                                        "type": "str"
                                    },
                                    "dhcp6-prefix-delegation": {
                                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                                        "choices": ["disable", "enable"],
                                        "type": "str"
                                    },
                                    "dhcp6-prefix-hint": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                                    "dhcp6-prefix-hint-plt": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                                    "dhcp6-prefix-hint-vlt": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                                    "dhcp6-relay-ip": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                                    "dhcp6-relay-service": {
                                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                                        "choices": ["disable", "enable"],
                                        "type": "str"
                                    },
                                    "dhcp6-relay-type": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["regular"], "type": "str"},
                                    "icmp6-send-redirect": {"v_range": [["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                                    "interface-identifier": {"v_range": [["6.4.5", ""]], "type": "str"},
                                    "ip6-address": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                                    "ip6-allowaccess": {
                                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                                        "type": "list",
                                        "choices": ["https", "ping", "ssh", "snmp", "http", "telnet", "fgfm", "capwap", "fabric"],
                                        "elements": "str"
                                    },
                                    "ip6-default-life": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                                    "ip6-delegated-prefix-list": {
                                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                                        "type": "list",
                                        "options": {
                                            "autonomous-flag": {
                                                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                                                "choices": ["disable", "enable"],
                                                "type": "str"
                                            },
                                            "onlink-flag": {
                                                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                                                "choices": ["disable", "enable"],
                                                "type": "str"
                                            },
                                            "prefix-id": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                                            "rdnss": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "raw"},
                                            "rdnss-service": {
                                                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                                                "choices": ["delegated", "default", "specify"],
                                                "type": "str"
                                            },
                                            "subnet": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                                            "upstream-interface": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                                            "delegated-prefix-iaid": {"v_range": [["7.0.2", ""]], "type": "int"}
                                        },
                                        "elements": "dict"
                                    },
                                    "ip6-dns-server-override": {
                                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                                        "choices": ["disable", "enable"],
                                        "type": "str"
                                    },
                                    "ip6-extra-addr": {
                                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                                        "type": "list",
                                        "options": {"prefix": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"}},
                                        "elements": "dict"
                                    },
                                    "ip6-hop-limit": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                                    "ip6-link-mtu": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                                    "ip6-manage-flag": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                                    "ip6-max-interval": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                                    "ip6-min-interval": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                                    "ip6-mode": {
                                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                                        "choices": ["static", "dhcp", "pppoe", "delegated"],
                                        "type": "str"
                                    },
                                    "ip6-other-flag": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                                    "ip6-prefix-list": {
                                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                                        "type": "list",
                                        "options": {
                                            "autonomous-flag": {
                                                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                                                "choices": ["disable", "enable"],
                                                "type": "str"
                                            },
                                            "dnssl": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "raw"},
                                            "onlink-flag": {
                                                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                                                "choices": ["disable", "enable"],
                                                "type": "str"
                                            },
                                            "preferred-life-time": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                                            "prefix": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                                            "rdnss": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "raw"},
                                            "valid-life-time": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"}
                                        },
                                        "elements": "dict"
                                    },
                                    "ip6-reachable-time": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                                    "ip6-retrans-time": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                                    "ip6-send-adv": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                                    "ip6-subnet": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                                    "ip6-upstream-interface": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                                    "nd-cert": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                                    "nd-cga-modifier": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                                    "nd-mode": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["basic", "SEND-compatible"], "type": "str"},
                                    "nd-security-level": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                                    "nd-timestamp-delta": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                                    "nd-timestamp-fuzz": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                                    "unique-autoconf-addr": {"v_range": [["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                                    "vrip6_link_local": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                                    "vrrp-virtual-mac6": {
                                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                                        "choices": ["disable", "enable"],
                                        "type": "str"
                                    },
                                    "vrrp6": {
                                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                                        "type": "list",
                                        "options": {
                                            "accept-mode": {
                                                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                                                "choices": ["disable", "enable"],
                                                "type": "str"
                                            },
                                            "adv-interval": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                                            "preempt": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                                            "priority": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                                            "start-time": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                                            "status": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                                            "vrdst6": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                                            "vrgrp": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                                            "vrid": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                                            "vrip6": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                                            "ignore-default-route": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                                            "vrdst-priority": {"v_range": [["7.6.0", ""]], "type": "int"}
                                        },
                                        "elements": "dict"
                                    },
                                    "cli-conn6-status": {"v_range": [["7.0.0", ""]], "type": "int"},
                                    "ip6-prefix-mode": {"v_range": [["7.0.0", ""]], "choices": ["dhcp6", "ra"], "type": "str"},
                                    "ra-send-mtu": {"v_range": [["6.4.6", ""]], "choices": ["disable", "enable"], "type": "str"},
                                    "ip6-delegated-prefix-iaid": {"v_range": [["7.0.2", ""]], "type": "int"},
                                    "dhcp6-relay-source-interface": {"v_range": [["7.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                                    "dhcp6-relay-interface-id": {"v_range": [["7.4.1", ""]], "type": "str"},
                                    "dhcp6-relay-source-ip": {"v_range": [["7.4.1", ""]], "type": "str"}
                                }
                            },
                            "secondary-IP": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "secondaryip": {
                                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                                "type": "list",
                                "options": {
                                    "allowaccess": {
                                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                                        "type": "list",
                                        "choices": [
                                            "https", "ping", "ssh", "snmp", "http", "telnet", "fgfm", "auto-ipsec", "radius-acct", "probe-response",
                                            "capwap", "dnp", "ftm", "fabric", "speed-test", "icond", "scim"
                                        ],
                                        "elements": "str"
                                    },
                                    "detectprotocol": {
                                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", "7.2.0"]],
                                        "type": "list",
                                        "choices": ["ping", "tcp-echo", "udp-echo"],
                                        "elements": "str"
                                    },
                                    "detectserver": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", "7.2.0"]], "type": "str"},
                                    "gwdetect": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", "7.2.0"]], "choices": ["disable", "enable"], "type": "str"},
                                    "ha-priority": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", "7.2.0"]], "type": "int"},
                                    "id": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                                    "ip": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                                    "ping-serv-status": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", "7.2.0"]], "type": "int"},
                                    "seq": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                                    "secip-relay-ip": {"v_range": [["7.4.0", ""]], "type": "str"}
                                },
                                "elements": "dict"
                            },
                            "vlanid": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                            "dhcp-relay-interface-select-method": {
                                "v_range": [["6.4.8", "6.4.14"], ["7.0.4", ""]],
                                "choices": ["auto", "sdwan", "specify"],
                                "type": "str"
                            },
                            "vrrp": {
                                "v_range": [["7.4.0", ""]],
                                "type": "list",
                                "options": {
                                    "accept-mode": {"v_range": [["7.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
                                    "adv-interval": {"v_range": [["7.4.0", ""]], "type": "int"},
                                    "ignore-default-route": {"v_range": [["7.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
                                    "preempt": {"v_range": [["7.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
                                    "priority": {"v_range": [["7.4.0", ""]], "type": "int"},
                                    "proxy-arp": {
                                        "v_range": [["7.4.0", ""]],
                                        "type": "list",
                                        "options": {"id": {"v_range": [["7.4.0", ""]], "type": "int"}, "ip": {"v_range": [["7.4.0", ""]], "type": "str"}},
                                        "elements": "dict"
                                    },
                                    "start-time": {"v_range": [["7.4.0", ""]], "type": "int"},
                                    "status": {"v_range": [["7.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
                                    "version": {"v_range": [["7.4.0", ""]], "choices": ["2", "3"], "type": "str"},
                                    "vrdst": {"v_range": [["7.4.0", ""]], "type": "raw"},
                                    "vrdst-priority": {"v_range": [["7.4.0", ""]], "type": "int"},
                                    "vrgrp": {"v_range": [["7.4.0", ""]], "type": "int"},
                                    "vrid": {"v_range": [["7.4.0", ""]], "type": "int"},
                                    "vrip": {"v_range": [["7.4.0", ""]], "type": "str"}
                                },
                                "elements": "dict"
                            }
                        }
                    }
                },
                "elements": "dict"
            },
            "name": {"required": true, "type": "str"},
            "portal-message-override-group": {"v_range": [["6.0.0", "6.2.1"]], "type": "str"},
            "radius-server": {"v_range": [["6.0.0", "6.2.1"]], "type": "str"},
            "security": {"v_range": [["6.0.0", "6.2.1"]], "choices": ["open", "captive-portal", "8021x"], "type": "str"},
            "selected-usergroups": {"v_range": [["6.0.0", "6.2.1"]], "type": "str"},
            "usergroup": {"v_range": [["6.0.0", "6.2.1"]], "type": "str"},
            "vdom": {"type": "str"},
            "vlanid": {"type": "int"},
            "dhcp-server": {
                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                "type": "dict",
                "options": {
                    "auto-configuration": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "auto-managed-status": {"v_range": [["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "conflicted-ip-timeout": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "ddns-auth": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "tsig"], "type": "str"},
                    "ddns-key": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "no_log": true, "type": "raw"},
                    "ddns-keyname": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "no_log": true, "type": "str"},
                    "ddns-server-ip": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "ddns-ttl": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "ddns-update": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "ddns-update-override": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "ddns-zone": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "default-gateway": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "dhcp-settings-from-fortiipam": {"v_range": [["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "dns-server1": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "dns-server2": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "dns-server3": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "dns-server4": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "dns-service": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["default", "specify", "local"], "type": "str"},
                    "domain": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "enable": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "exclude-range": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "type": "list",
                        "options": {
                            "end-ip": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                            "id": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                            "start-ip": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                            "vci-match": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "vci-string": {"v_range": [["7.2.1", ""]], "type": "raw"},
                            "lease-time": {"v_range": [["7.2.2", ""]], "type": "int"},
                            "uci-match": {"v_range": [["7.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "uci-string": {"v_range": [["7.2.2", ""]], "type": "raw"}
                        },
                        "elements": "dict"
                    },
                    "filename": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "forticlient-on-net-status": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "id": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "ip-mode": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["range", "usrgrp"], "type": "str"},
                    "ip-range": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "type": "list",
                        "options": {
                            "end-ip": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                            "id": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                            "start-ip": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                            "vci-match": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "vci-string": {"v_range": [["7.2.1", ""]], "type": "raw"},
                            "lease-time": {"v_range": [["7.2.2", ""]], "type": "int"},
                            "uci-match": {"v_range": [["7.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "uci-string": {"v_range": [["7.2.2", ""]], "type": "raw"}
                        },
                        "elements": "dict"
                    },
                    "ipsec-lease-hold": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "lease-time": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "mac-acl-default-action": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["assign", "block"], "type": "str"},
                    "netmask": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "next-server": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "ntp-server1": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "ntp-server2": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "ntp-server3": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "ntp-service": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["default", "specify", "local"], "type": "str"},
                    "option1": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "raw"},
                    "option2": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "raw"},
                    "option3": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "raw"},
                    "option4": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "option5": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "option6": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "options": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "type": "list",
                        "options": {
                            "code": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                            "id": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                            "ip": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "raw"},
                            "type": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["hex", "string", "ip", "fqdn"], "type": "str"},
                            "value": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                            "vci-match": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "vci-string": {"v_range": [["7.2.1", ""]], "type": "raw"},
                            "uci-match": {"v_range": [["7.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "uci-string": {"v_range": [["7.2.2", ""]], "type": "raw"}
                        },
                        "elements": "dict"
                    },
                    "reserved-address": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "type": "list",
                        "options": {
                            "action": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["assign", "block", "reserved"], "type": "str"},
                            "circuit-id": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                            "circuit-id-type": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["hex", "string"], "type": "str"},
                            "description": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                            "id": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                            "ip": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                            "mac": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                            "remote-id": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                            "remote-id-type": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["hex", "string"], "type": "str"},
                            "type": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["mac", "option82"], "type": "str"}
                        },
                        "elements": "dict"
                    },
                    "server-type": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["regular", "ipsec"], "type": "str"},
                    "status": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "tftp-server": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "raw"},
                    "timezone": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "choices": [
                            "00", "01", "02", "03", "04", "05", "06", "07", "08", "09", "10", "11", "12", "13", "14", "15", "16", "17", "18", "19", "20",
                            "21", "22", "23", "24", "25", "26", "27", "28", "29", "30", "31", "32", "33", "34", "35", "36", "37", "38", "39", "40", "41",
                            "42", "43", "44", "45", "46", "47", "48", "49", "50", "51", "52", "53", "54", "55", "56", "57", "58", "59", "60", "61", "62",
                            "63", "64", "65", "66", "67", "68", "69", "70", "71", "72", "73", "74", "75", "76", "77", "78", "79", "80", "81", "82", "83",
                            "84", "85", "86", "87"
                        ],
                        "type": "str"
                    },
                    "timezone-option": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "default", "specify"], "type": "str"},
                    "vci-match": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "vci-string": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "raw"},
                    "wifi-ac-service": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["specify", "local"], "type": "str"},
                    "wifi-ac1": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "wifi-ac2": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "wifi-ac3": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "wins-server1": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "wins-server2": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "relay-agent": {"v_range": [["7.4.0", ""]], "type": "str"},
                    "shared-subnet": {"v_range": [["7.4.0", ""]], "choices": ["disable", "enable"], "type": "str"}
                }
            },
            "interface": {
                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                "type": "dict",
                "options": {
                    "ac-name": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "aggregate": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "algorithm": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["L2", "L3", "L4", "LB", "Source-MAC"], "type": "str"},
                    "alias": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "allowaccess": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "type": "list",
                        "choices": [
                            "https", "ping", "ssh", "snmp", "http", "telnet", "fgfm", "auto-ipsec", "radius-acct", "probe-response", "capwap", "dnp",
                            "ftm", "fabric", "speed-test"
                        ],
                        "elements": "str"
                    },
                    "ap-discover": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "arpforward": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "atm-protocol": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["none", "ipoa"], "type": "str"},
                    "auth-type": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "choices": ["auto", "pap", "chap", "mschapv1", "mschapv2"],
                        "type": "str"
                    },
                    "auto-auth-extension-device": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "bandwidth-measure-time": {"v_range": [["6.4.5", ""]], "type": "int"},
                    "bfd": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["global", "enable", "disable"], "type": "str"},
                    "bfd-desired-min-tx": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "bfd-detect-mult": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "bfd-required-min-rx": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "broadcast-forticlient-discovery": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "broadcast-forward": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "captive-portal": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "cli-conn-status": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "color": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "ddns": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "ddns-auth": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "tsig"], "type": "str"},
                    "ddns-domain": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "ddns-key": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "no_log": true, "type": "raw"},
                    "ddns-keyname": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "no_log": true, "type": "str"},
                    "ddns-password": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "no_log": true, "type": "raw"},
                    "ddns-server": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "choices": [
                            "dhs.org", "dyndns.org", "dyns.net", "tzo.com", "ods.org", "vavic.com", "now.net.cn", "dipdns.net", "easydns.com",
                            "genericDDNS"
                        ],
                        "type": "str"
                    },
                    "ddns-server-ip": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "ddns-sn": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "ddns-ttl": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "ddns-username": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "ddns-zone": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "dedicated-to": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["none", "management"], "type": "str"},
                    "defaultgw": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "description": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "detected-peer-mtu": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "detectprotocol": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "type": "list",
                        "choices": ["ping", "tcp-echo", "udp-echo"],
                        "elements": "str"
                    },
                    "detectserver": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "device-access-list": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "raw"},
                    "device-identification": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "device-identification-active-scan": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "choices": ["disable", "enable"],
                        "type": "str"
                    },
                    "device-netscan": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "device-user-identification": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "devindex": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "dhcp-client-identifier": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "dhcp-relay-agent-option": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "dhcp-relay-interface": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "dhcp-relay-interface-select-method": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "choices": ["auto", "sdwan", "specify"],
                        "type": "str"
                    },
                    "dhcp-relay-ip": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "raw"},
                    "dhcp-relay-service": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "dhcp-relay-type": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["regular", "ipsec"], "type": "str"},
                    "dhcp-renew-time": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "disc-retry-timeout": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "disconnect-threshold": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "distance": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "dns-query": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "recursive", "non-recursive"], "type": "str"},
                    "dns-server-override": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "drop-fragment": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "drop-overlapped-fragment": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "egress-cos": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "choices": ["disable", "cos0", "cos1", "cos2", "cos3", "cos4", "cos5", "cos6", "cos7"],
                        "type": "str"
                    },
                    "egress-shaping-profile": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "eip": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "endpoint-compliance": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "estimated-downstream-bandwidth": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "estimated-upstream-bandwidth": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "explicit-ftp-proxy": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "explicit-web-proxy": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "external": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "fail-action-on-extender": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "choices": ["soft-restart", "hard-restart", "reboot"],
                        "type": "str"
                    },
                    "fail-alert-interfaces": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "raw"},
                    "fail-alert-method": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["link-failed-signal", "link-down"], "type": "str"},
                    "fail-detect": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "fail-detect-option": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "type": "list",
                        "choices": ["detectserver", "link-down"],
                        "elements": "str"
                    },
                    "fdp": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "fortiheartbeat": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "fortilink": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "fortilink-backup-link": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "fortilink-neighbor-detect": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["lldp", "fortilink"], "type": "str"},
                    "fortilink-split-interface": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "fortilink-stacking": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "forward-domain": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "forward-error-correction": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "choices": [
                            "disable", "enable", "rs-fec", "base-r-fec", "fec-cl91", "fec-cl74", "rs-544", "none", "cl91-rs-fec", "cl74-fc-fec", "auto"
                        ],
                        "type": "str"
                    },
                    "fp-anomaly": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "type": "list",
                        "choices": [
                            "drop_tcp_fin_noack", "pass_winnuke", "pass_tcpland", "pass_udpland", "pass_icmpland", "pass_ipland", "pass_iprr",
                            "pass_ipssrr", "pass_iplsrr", "pass_ipstream", "pass_ipsecurity", "pass_iptimestamp", "pass_ipunknown_option",
                            "pass_ipunknown_prot", "pass_icmp_frag", "pass_tcp_no_flag", "pass_tcp_fin_noack", "drop_winnuke", "drop_tcpland",
                            "drop_udpland", "drop_icmpland", "drop_ipland", "drop_iprr", "drop_ipssrr", "drop_iplsrr", "drop_ipstream",
                            "drop_ipsecurity", "drop_iptimestamp", "drop_ipunknown_option", "drop_ipunknown_prot", "drop_icmp_frag", "drop_tcp_no_flag"
                        ],
                        "elements": "str"
                    },
                    "fp-disable": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "type": "list",
                        "choices": ["all", "ipsec", "none"],
                        "elements": "str"
                    },
                    "gateway-address": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "gi-gk": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "gwaddr": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", "7.2.0"]], "type": "str"},
                    "gwdetect": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "ha-priority": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "icmp-accept-redirect": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "icmp-redirect": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "icmp-send-redirect": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "ident-accept": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "idle-timeout": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "if-mdix": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["auto", "normal", "crossover"], "type": "str"},
                    "if-media": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["auto", "copper", "fiber"], "type": "str"},
                    "in-force-vlan-cos": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "inbandwidth": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "ingress-cos": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "choices": ["disable", "cos0", "cos1", "cos2", "cos3", "cos4", "cos5", "cos6", "cos7"],
                        "type": "str"
                    },
                    "ingress-shaping-profile": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "ingress-spillover-threshold": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "internal": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "ip": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "ip-managed-by-fortiipam": {"v_range": [["6.4.5", ""]], "choices": ["disable", "enable", "inherit-global"], "type": "str"},
                    "ipmac": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "ips-sniffer-mode": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "ipunnumbered": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "ipv6": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "type": "dict",
                        "options": {
                            "autoconf": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "dhcp6-client-options": {
                                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                                "type": "list",
                                "choices": ["rapid", "iapd", "iana", "dns", "dnsname"],
                                "elements": "str"
                            },
                            "dhcp6-information-request": {
                                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                                "choices": ["disable", "enable"],
                                "type": "str"
                            },
                            "dhcp6-prefix-delegation": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "dhcp6-prefix-hint": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                            "dhcp6-prefix-hint-plt": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                            "dhcp6-prefix-hint-vlt": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                            "dhcp6-relay-ip": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                            "dhcp6-relay-service": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "dhcp6-relay-type": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["regular"], "type": "str"},
                            "icmp6-send-redirect": {"v_range": [["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "interface-identifier": {"v_range": [["6.4.5", ""]], "type": "str"},
                            "ip6-address": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                            "ip6-allowaccess": {
                                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                                "type": "list",
                                "choices": ["https", "ping", "ssh", "snmp", "http", "telnet", "fgfm", "capwap", "fabric"],
                                "elements": "str"
                            },
                            "ip6-default-life": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                            "ip6-delegated-prefix-list": {
                                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                                "type": "list",
                                "options": {
                                    "autonomous-flag": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                                    "onlink-flag": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                                    "prefix-id": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                                    "rdnss": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "raw"},
                                    "rdnss-service": {
                                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                                        "choices": ["delegated", "default", "specify"],
                                        "type": "str"
                                    },
                                    "subnet": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                                    "upstream-interface": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                                    "delegated-prefix-iaid": {"v_range": [["7.0.2", ""]], "type": "int"}
                                },
                                "elements": "dict"
                            },
                            "ip6-dns-server-override": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "ip6-extra-addr": {
                                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                                "type": "list",
                                "options": {"prefix": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"}},
                                "elements": "dict"
                            },
                            "ip6-hop-limit": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                            "ip6-link-mtu": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                            "ip6-manage-flag": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "ip6-max-interval": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                            "ip6-min-interval": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                            "ip6-mode": {
                                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                                "choices": ["static", "dhcp", "pppoe", "delegated"],
                                "type": "str"
                            },
                            "ip6-other-flag": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "ip6-prefix-list": {
                                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                                "type": "list",
                                "options": {
                                    "autonomous-flag": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                                    "dnssl": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "raw"},
                                    "onlink-flag": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                                    "preferred-life-time": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                                    "prefix": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                                    "rdnss": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "raw"},
                                    "valid-life-time": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"}
                                },
                                "elements": "dict"
                            },
                            "ip6-reachable-time": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                            "ip6-retrans-time": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                            "ip6-send-adv": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "ip6-subnet": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                            "ip6-upstream-interface": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                            "nd-cert": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                            "nd-cga-modifier": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                            "nd-mode": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["basic", "SEND-compatible"], "type": "str"},
                            "nd-security-level": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                            "nd-timestamp-delta": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                            "nd-timestamp-fuzz": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                            "unique-autoconf-addr": {"v_range": [["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "vrip6_link_local": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                            "vrrp-virtual-mac6": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "vrrp6": {
                                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                                "type": "list",
                                "options": {
                                    "accept-mode": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                                    "adv-interval": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                                    "preempt": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                                    "priority": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                                    "start-time": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                                    "status": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                                    "vrdst6": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                                    "vrgrp": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                                    "vrid": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                                    "vrip6": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                                    "ignore-default-route": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                                    "vrdst-priority": {"v_range": [["7.6.0", ""]], "type": "int"}
                                },
                                "elements": "dict"
                            },
                            "cli-conn6-status": {"v_range": [["7.0.0", ""]], "type": "int"},
                            "ip6-prefix-mode": {"v_range": [["7.0.0", ""]], "choices": ["dhcp6", "ra"], "type": "str"},
                            "ra-send-mtu": {"v_range": [["6.4.6", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "ip6-delegated-prefix-iaid": {"v_range": [["7.0.2", ""]], "type": "int"},
                            "dhcp6-relay-source-interface": {"v_range": [["7.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "dhcp6-relay-interface-id": {"v_range": [["7.4.1", ""]], "type": "str"},
                            "dhcp6-relay-source-ip": {"v_range": [["7.4.1", ""]], "type": "str"}
                        }
                    },
                    "l2forward": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "l2tp-client": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "lacp-ha-slave": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "lacp-mode": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["static", "passive", "active"], "type": "str"},
                    "lacp-speed": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["slow", "fast"], "type": "str"},
                    "lcp-echo-interval": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "lcp-max-echo-fails": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "link-up-delay": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "listen-forticlient-connection": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "lldp-network-policy": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "lldp-reception": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable", "vdom"], "type": "str"},
                    "lldp-transmission": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["enable", "disable", "vdom"], "type": "str"},
                    "log": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "macaddr": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "managed-subnetwork-size": {
                        "v_range": [["6.4.5", ""]],
                        "choices": ["256", "512", "1024", "2048", "4096", "8192", "16384", "32768", "65536", "32", "64", "128"],
                        "type": "str"
                    },
                    "management-ip": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "max-egress-burst-rate": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "max-egress-rate": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "measured-downstream-bandwidth": {"v_range": [["6.4.5", ""]], "type": "int"},
                    "measured-upstream-bandwidth": {"v_range": [["6.4.5", ""]], "type": "int"},
                    "mediatype": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "choices": [
                            "serdes-sfp", "sgmii-sfp", "cfp2-sr10", "cfp2-lr4", "serdes-copper-sfp", "sr", "cr", "lr", "qsfp28-sr4", "qsfp28-lr4",
                            "qsfp28-cr4", "sr4", "cr4", "lr4", "none", "gmii", "sgmii", "sr2", "lr2", "cr2", "sr8", "lr8", "cr8"
                        ],
                        "type": "str"
                    },
                    "member": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "raw"},
                    "min-links": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "min-links-down": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["operational", "administrative"], "type": "str"},
                    "mode": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "choices": ["static", "dhcp", "pppoe", "pppoa", "ipoa", "eoa"],
                        "type": "str"
                    },
                    "monitor-bandwidth": {"v_range": [["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "mtu": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "mtu-override": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "mux-type": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["llc-encaps", "vc-encaps"], "type": "str"},
                    "name": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "ndiscforward": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "netbios-forward": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "netflow-sampler": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "tx", "rx", "both"], "type": "str"},
                    "np-qos-profile": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "npu-fastpath": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "nst": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "out-force-vlan-cos": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "outbandwidth": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "padt-retry-timeout": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "password": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "no_log": true, "type": "raw"},
                    "peer-interface": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "raw"},
                    "phy-mode": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "choices": [
                            "auto", "adsl", "vdsl", "adsl-auto", "vdsl2", "adsl2+", "adsl2", "g.dmt", "t1.413", "g.lite", "g-dmt", "t1-413", "g-lite"
                        ],
                        "type": "str"
                    },
                    "ping-serv-status": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", "7.2.0"]], "type": "int"},
                    "poe": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "polling-interval": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "pppoe-unnumbered-negotiate": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "pptp-auth-type": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "choices": ["auto", "pap", "chap", "mschapv1", "mschapv2"],
                        "type": "str"
                    },
                    "pptp-client": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "pptp-password": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "no_log": true, "type": "raw"},
                    "pptp-server-ip": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "pptp-timeout": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "pptp-user": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "preserve-session-route": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "priority": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "priority-override": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "proxy-captive-portal": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "redundant-interface": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "remote-ip": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "replacemsg-override-group": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "retransmission": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "ring-rx": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "ring-tx": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "role": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["lan", "wan", "dmz", "undefined"], "type": "str"},
                    "sample-direction": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["rx", "tx", "both"], "type": "str"},
                    "sample-rate": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "scan-botnet-connections": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "choices": ["disable", "block", "monitor"],
                        "type": "str"
                    },
                    "secondary-IP": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "secondaryip": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "type": "list",
                        "options": {
                            "allowaccess": {
                                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                                "type": "list",
                                "choices": [
                                    "https", "ping", "ssh", "snmp", "http", "telnet", "fgfm", "auto-ipsec", "radius-acct", "probe-response", "capwap",
                                    "dnp", "ftm", "fabric", "speed-test", "icond", "scim"
                                ],
                                "elements": "str"
                            },
                            "detectprotocol": {
                                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", "7.2.0"]],
                                "type": "list",
                                "choices": ["ping", "tcp-echo", "udp-echo"],
                                "elements": "str"
                            },
                            "detectserver": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", "7.2.0"]], "type": "str"},
                            "gwdetect": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", "7.2.0"]], "choices": ["disable", "enable"], "type": "str"},
                            "ha-priority": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", "7.2.0"]], "type": "int"},
                            "id": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                            "ip": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                            "ping-serv-status": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", "7.2.0"]], "type": "int"},
                            "seq": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                            "secip-relay-ip": {"v_range": [["7.4.0", ""]], "type": "str"}
                        },
                        "elements": "dict"
                    },
                    "security-8021x-dynamic-vlan-id": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "security-8021x-master": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "security-8021x-mode": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "choices": ["default", "dynamic-vlan", "fallback", "slave"],
                        "type": "str"
                    },
                    "security-exempt-list": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "security-external-logout": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "security-external-web": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "security-groups": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "raw"},
                    "security-mac-auth-bypass": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "choices": ["disable", "enable", "mac-auth-only"],
                        "type": "str"
                    },
                    "security-mode": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["none", "captive-portal", "802.1X"], "type": "str"},
                    "security-redirect-url": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "service-name": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "sflow-sampler": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "speed": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "choices": [
                            "auto", "10full", "10half", "100full", "100half", "1000full", "1000half", "10000full", "1000auto", "10000auto", "40000full",
                            "100Gfull", "25000full", "40000auto", "25000auto", "100Gauto", "400Gfull", "400Gauto", "50000full", "2500auto", "5000auto",
                            "50000auto", "200Gfull", "200Gauto", "100auto"
                        ],
                        "type": "str"
                    },
                    "spillover-threshold": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "src-check": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "status": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["down", "up"], "type": "str"},
                    "stp": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "stp-ha-slave": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable", "priority-adjust"], "type": "str"},
                    "stpforward": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "stpforward-mode": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "choices": ["rpl-all-ext-id", "rpl-bridge-ext-id", "rpl-nothing"],
                        "type": "str"
                    },
                    "strip-priority-vlan-tag": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "subst": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "substitute-dst-mac": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "swc-first-create": {"v_range": [["6.4.5", ""]], "type": "int"},
                    "swc-vlan": {"v_range": [["6.4.5", ""]], "type": "int"},
                    "switch": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "switch-controller-access-vlan": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "switch-controller-arp-inspection": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "choices": ["disable", "enable", "monitor"],
                        "type": "str"
                    },
                    "switch-controller-auth": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["radius", "usergroup"], "type": "str"},
                    "switch-controller-dhcp-snooping": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "switch-controller-dhcp-snooping-option82": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "choices": ["disable", "enable"],
                        "type": "str"
                    },
                    "switch-controller-dhcp-snooping-verify-mac": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "choices": ["disable", "enable"],
                        "type": "str"
                    },
                    "switch-controller-feature": {
                        "v_range": [["6.4.5", ""]],
                        "choices": ["none", "default-vlan", "quarantine", "sniffer", "voice", "camera", "rspan", "video", "nac", "nac-segment"],
                        "type": "str"
                    },
                    "switch-controller-igmp-snooping": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "switch-controller-igmp-snooping-fast-leave": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "choices": ["disable", "enable"],
                        "type": "str"
                    },
                    "switch-controller-igmp-snooping-proxy": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "choices": ["disable", "enable"],
                        "type": "str"
                    },
                    "switch-controller-iot-scanning": {"v_range": [["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "switch-controller-learning-limit": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "switch-controller-mgmt-vlan": {"v_range": [["6.4.5", ""]], "type": "int"},
                    "switch-controller-nac": {"v_range": [["6.4.5", ""]], "type": "str"},
                    "switch-controller-radius-server": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "switch-controller-rspan-mode": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "switch-controller-source-ip": {"v_range": [["6.4.5", ""]], "choices": ["outbound", "fixed"], "type": "str"},
                    "switch-controller-traffic-policy": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "tc-mode": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["ptm", "atm"], "type": "str"},
                    "tcp-mss": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "trunk": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "trust-ip-1": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "trust-ip-2": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "trust-ip-3": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "trust-ip6-1": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "trust-ip6-2": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "trust-ip6-3": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "type": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "choices": [
                            "physical", "vlan", "aggregate", "redundant", "tunnel", "wireless", "vdom-link", "loopback", "switch", "hard-switch", "hdlc",
                            "vap-switch", "wl-mesh", "fortilink", "switch-vlan", "fctrl-trunk", "tdm", "fext-wan", "vxlan", "emac-vlan", "geneve", "ssl",
                            "lan-extension"
                        ],
                        "type": "str"
                    },
                    "username": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "vci": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "vectoring": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "vindex": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "vlan-protocol": {"v_range": [["6.4.5", ""]], "choices": ["8021q", "8021ad"], "type": "str"},
                    "vlanforward": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "vlanid": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "vpi": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "vrf": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "vrrp": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "type": "list",
                        "options": {
                            "accept-mode": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "adv-interval": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                            "ignore-default-route": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "preempt": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "priority": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                            "start-time": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                            "status": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "version": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["2", "3"], "type": "str"},
                            "vrdst": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "raw"},
                            "vrdst-priority": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                            "vrgrp": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                            "vrid": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                            "vrip": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                            "proxy-arp": {
                                "v_range": [["7.4.0", ""]],
                                "type": "list",
                                "options": {"id": {"v_range": [["7.4.0", ""]], "type": "int"}, "ip": {"v_range": [["7.4.0", ""]], "type": "str"}},
                                "elements": "dict"
                            }
                        },
                        "elements": "dict"
                    },
                    "vrrp-virtual-mac": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "wccp": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "weight": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "wifi-5g-threshold": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "wifi-acl": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["deny", "allow"], "type": "str"},
                    "wifi-ap-band": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["any", "5g-preferred", "5g-only"], "type": "str"},
                    "wifi-auth": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["PSK", "RADIUS", "radius", "usergroup"], "type": "str"},
                    "wifi-auto-connect": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "wifi-auto-save": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "wifi-broadcast-ssid": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "wifi-encrypt": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["TKIP", "AES"], "type": "str"},
                    "wifi-fragment-threshold": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "wifi-key": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "no_log": true, "type": "raw"},
                    "wifi-keyindex": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "no_log": true, "type": "int"},
                    "wifi-mac-filter": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "wifi-passphrase": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "no_log": true, "type": "raw"},
                    "wifi-radius-server": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "wifi-rts-threshold": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "wifi-security": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "choices": [
                            "None", "WEP64", "wep64", "WEP128", "wep128", "WPA_PSK", "WPA_RADIUS", "WPA", "WPA2", "WPA2_AUTO", "open", "wpa-personal",
                            "wpa-enterprise", "wpa-only-personal", "wpa-only-enterprise", "wpa2-only-personal", "wpa2-only-enterprise"
                        ],
                        "type": "str"
                    },
                    "wifi-ssid": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "wifi-usergroup": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "wins-ip": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "dhcp-relay-request-all-server": {"v_range": [["6.2.8", "6.2.12"], ["6.4.6", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "stp-ha-secondary": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable", "priority-adjust"], "type": "str"},
                    "switch-controller-dynamic": {"v_range": [["7.0.0", ""]], "type": "str"},
                    "auth-cert": {"v_range": [["7.0.3", ""]], "type": "str"},
                    "auth-portal-addr": {"v_range": [["7.0.3", ""]], "type": "str"},
                    "dhcp-classless-route-addition": {"v_range": [["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "dhcp-relay-link-selection": {"v_range": [["7.0.3", ""]], "type": "str"},
                    "dns-server-protocol": {"v_range": [["7.0.3", ""]], "type": "list", "choices": ["cleartext", "dot", "doh"], "elements": "str"},
                    "eap-ca-cert": {"v_range": [["7.2.0", ""]], "type": "str"},
                    "eap-identity": {"v_range": [["7.2.0", ""]], "type": "str"},
                    "eap-method": {"v_range": [["7.2.0", ""]], "choices": ["tls", "peap"], "type": "str"},
                    "eap-password": {"v_range": [["7.2.0", ""]], "no_log": true, "type": "raw"},
                    "eap-supplicant": {"v_range": [["7.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "eap-user-cert": {"v_range": [["7.2.0", ""]], "type": "str"},
                    "ike-saml-server": {"v_range": [["7.2.0", ""]], "type": "str"},
                    "lacp-ha-secondary": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "pvc-atm-qos": {"v_range": [["6.4.8", "6.4.14"], ["7.0.2", ""]], "choices": ["cbr", "rt-vbr", "nrt-vbr", "ubr"], "type": "str"},
                    "pvc-chan": {"v_range": [["6.4.8", "6.4.14"], ["7.0.2", ""]], "type": "int"},
                    "pvc-crc": {"v_range": [["6.4.8", "6.4.14"], ["7.0.2", ""]], "type": "int"},
                    "pvc-pcr": {"v_range": [["6.4.8", "6.4.14"], ["7.0.2", ""]], "type": "int"},
                    "pvc-scr": {"v_range": [["6.4.8", "6.4.14"], ["7.0.2", ""]], "type": "int"},
                    "pvc-vlan-id": {"v_range": [["6.4.8", "6.4.14"], ["7.0.2", ""]], "type": "int"},
                    "pvc-vlan-rx-id": {"v_range": [["6.4.8", "6.4.14"], ["7.0.2", ""]], "type": "int"},
                    "pvc-vlan-rx-op": {"v_range": [["6.4.8", "6.4.14"], ["7.0.2", ""]], "choices": ["pass-through", "replace", "remove"], "type": "str"},
                    "pvc-vlan-tx-id": {"v_range": [["6.4.8", "6.4.14"], ["7.0.2", ""]], "type": "int"},
                    "pvc-vlan-tx-op": {"v_range": [["6.4.8", "6.4.14"], ["7.0.2", ""]], "choices": ["pass-through", "replace", "remove"], "type": "str"},
                    "reachable-time": {"v_range": [["7.0.3", ""]], "type": "int"},
                    "select-profile-30a-35b": {
                        "v_range": [["6.2.9", "6.2.12"], ["6.4.8", "6.4.14"], ["7.0.3", ""]],
                        "choices": ["30A", "35B"],
                        "type": "str"
                    },
                    "sfp-dsl": {"v_range": [["6.4.8", "6.4.14"], ["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "sfp-dsl-adsl-fallback": {"v_range": [["6.4.8", "6.4.14"], ["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "sfp-dsl-autodetect": {"v_range": [["6.4.8", "6.4.14"], ["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "sfp-dsl-mac": {"v_range": [["6.4.8", "6.4.14"], ["7.0.2", ""]], "type": "str"},
                    "sw-algorithm": {"v_range": [["7.0.1", ""]], "choices": ["l2", "l3", "eh", "default"], "type": "str"},
                    "system-id": {"v_range": [["6.4.7", "6.4.14"], ["7.0.2", ""]], "type": "str"},
                    "system-id-type": {"v_range": [["6.4.7", "6.4.14"], ["7.0.2", ""]], "choices": ["auto", "user"], "type": "str"},
                    "vlan-id": {"v_range": [["6.2.9", "6.2.12"], ["6.4.8", "6.4.14"], ["7.0.3", ""]], "type": "int"},
                    "vlan-op-mode": {
                        "v_range": [["6.2.9", "6.2.12"], ["6.4.8", "6.4.14"], ["7.0.3", ""]],
                        "choices": ["tag", "untag", "passthrough"],
                        "type": "str"
                    },
                    "generic-receive-offload": {"v_range": [["7.0.5", "7.0.12"], ["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "interconnect-profile": {
                        "v_range": [["7.0.5", "7.0.12"], ["7.2.1", ""]],
                        "choices": ["default", "profile1", "profile2"],
                        "type": "str"
                    },
                    "large-receive-offload": {"v_range": [["7.0.5", "7.0.12"], ["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "annex": {
                        "v_range": [["7.0.10", "7.0.12"], ["7.2.5", "7.2.5"], ["7.4.2", ""]],
                        "choices": ["a", "b", "j", "bjm", "i", "al", "m", "aijlm", "bj"],
                        "type": "str"
                    },
                    "aggregate-type": {"v_range": [["7.2.1", ""]], "choices": ["physical", "vxlan"], "type": "str"},
                    "switch-controller-netflow-collect": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "wifi-dns-server1": {"v_range": [["7.2.2", ""]], "type": "str"},
                    "wifi-dns-server2": {"v_range": [["7.2.2", ""]], "type": "str"},
                    "wifi-gateway": {"v_range": [["7.2.2", ""]], "type": "str"},
                    "default-purdue-level": {"v_range": [["7.4.0", ""]], "choices": ["1", "2", "3", "4", "5", "1.5", "2.5", "3.5", "5.5"], "type": "str"},
                    "dhcp-broadcast-flag": {"v_range": [["7.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "dhcp-smart-relay": {"v_range": [["7.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "switch-controller-offloading": {"v_range": [["7.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "switch-controller-offloading-gw": {"v_range": [["7.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "switch-controller-offloading-ip": {"v_range": [["7.4.0", ""]], "type": "str"},
                    "dhcp-relay-circuit-id": {"v_range": [["7.4.1", ""]], "type": "str"},
                    "dhcp-relay-source-ip": {"v_range": [["7.4.1", ""]], "type": "str"},
                    "switch-controller-offload": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "switch-controller-offload-gw": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "switch-controller-offload-ip": {"v_range": [["7.4.1", ""]], "type": "str"},
                    "mirroring-direction": {"v_range": [["7.4.2", ""]], "choices": ["rx", "tx", "both"], "type": "str"},
                    "mirroring-port": {"v_range": [["7.4.2", ""]], "type": "str"},
                    "port-mirroring": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "security-8021x-member-mode": {"v_range": [["7.4.2", ""]], "choices": ["disable", "switch"], "type": "str"},
                    "stp-edge": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "dhcp-relay-allow-no-end-option": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "netflow-sample-rate": {"v_range": [["7.6.0", ""]], "type": "int"},
                    "netflow-sampler-id": {"v_range": [["7.6.0", ""]], "type": "int"},
                    "pppoe-egress-cos": {
                        "v_range": [["7.6.0", ""]],
                        "choices": ["cos0", "cos1", "cos2", "cos3", "cos4", "cos5", "cos6", "cos7"],
                        "type": "str"
                    },
                    "security-ip-auth-bypass": {"v_range": [["7.6.0", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "virtual-mac": {"v_range": [["7.6.0", ""]], "type": "str"}
                }
            }
        }
    },
    "fsp_vlan_dhcpserver": {
        "stated": false,
        "options": {
            "auto-configuration": {"choices": ["disable", "enable"], "type": "str"},
            "conflicted-ip-timeout": {"type": "int"},
            "ddns-auth": {"choices": ["disable", "tsig"], "type": "str"},
            "ddns-key": {"no_log": true, "type": "raw"},
            "ddns-keyname": {"no_log": true, "type": "str"},
            "ddns-server-ip": {"type": "str"},
            "ddns-ttl": {"type": "int"},
            "ddns-update": {"choices": ["disable", "enable"], "type": "str"},
            "ddns-update-override": {"choices": ["disable", "enable"], "type": "str"},
            "ddns-zone": {"type": "str"},
            "default-gateway": {"type": "str"},
            "dns-server1": {"type": "str"},
            "dns-server2": {"type": "str"},
            "dns-server3": {"type": "str"},
            "dns-service": {"choices": ["default", "specify", "local"], "type": "str"},
            "domain": {"type": "str"},
            "enable": {"choices": ["disable", "enable"], "type": "str"},
            "exclude-range": {
                "type": "list",
                "options": {
                    "end-ip": {"type": "str"},
                    "id": {"type": "int"},
                    "start-ip": {"type": "str"},
                    "vci-match": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "vci-string": {"v_range": [["7.2.1", ""]], "type": "raw"},
                    "lease-time": {"v_range": [["7.2.2", ""]], "type": "int"},
                    "uci-match": {"v_range": [["7.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "uci-string": {"v_range": [["7.2.2", ""]], "type": "raw"}
                },
                "elements": "dict"
            },
            "filename": {"type": "str"},
            "forticlient-on-net-status": {"choices": ["disable", "enable"], "type": "str"},
            "id": {"type": "int"},
            "interface": {"v_range": [["6.0.0", "6.2.1"]], "type": "str"},
            "ip-mode": {"choices": ["range", "usrgrp"], "type": "str"},
            "ip-range": {
                "type": "list",
                "options": {
                    "end-ip": {"type": "str"},
                    "id": {"type": "int"},
                    "start-ip": {"type": "str"},
                    "vci-match": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "vci-string": {"v_range": [["7.2.1", ""]], "type": "raw"},
                    "lease-time": {"v_range": [["7.2.2", ""]], "type": "int"},
                    "uci-match": {"v_range": [["7.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "uci-string": {"v_range": [["7.2.2", ""]], "type": "raw"}
                },
                "elements": "dict"
            },
            "ipsec-lease-hold": {"type": "int"},
            "lease-time": {"type": "int"},
            "mac-acl-default-action": {"choices": ["assign", "block"], "type": "str"},
            "netmask": {"type": "str"},
            "next-server": {"type": "str"},
            "ntp-server1": {"type": "str"},
            "ntp-server2": {"type": "str"},
            "ntp-server3": {"type": "str"},
            "ntp-service": {"choices": ["default", "specify", "local"], "type": "str"},
            "option1": {"type": "raw"},
            "option2": {"type": "raw"},
            "option3": {"type": "raw"},
            "option4": {"type": "str"},
            "option5": {"type": "str"},
            "option6": {"type": "str"},
            "options": {
                "type": "list",
                "options": {
                    "code": {"type": "int"},
                    "id": {"type": "int"},
                    "ip": {"type": "raw"},
                    "type": {"choices": ["hex", "string", "ip", "fqdn"], "type": "str"},
                    "value": {"type": "str"},
                    "vci-match": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "vci-string": {"v_range": [["7.2.1", ""]], "type": "raw"},
                    "uci-match": {"v_range": [["7.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "uci-string": {"v_range": [["7.2.2", ""]], "type": "raw"}
                },
                "elements": "dict"
            },
            "reserved-address": {
                "type": "list",
                "options": {
                    "action": {"choices": ["assign", "block", "reserved"], "type": "str"},
                    "circuit-id": {"type": "str"},
                    "circuit-id-type": {"choices": ["hex", "string"], "type": "str"},
                    "description": {"type": "str"},
                    "id": {"type": "int"},
                    "ip": {"type": "str"},
                    "mac": {"type": "str"},
                    "remote-id": {"type": "str"},
                    "remote-id-type": {"choices": ["hex", "string"], "type": "str"},
                    "type": {"choices": ["mac", "option82"], "type": "str"}
                },
                "elements": "dict"
            },
            "server-type": {"choices": ["regular", "ipsec"], "type": "str"},
            "status": {"choices": ["disable", "enable"], "type": "str"},
            "tftp-server": {"type": "raw"},
            "timezone": {
                "choices": [
                    "00", "01", "02", "03", "04", "05", "06", "07", "08", "09", "10", "11", "12", "13", "14", "15", "16", "17", "18", "19", "20", "21",
                    "22", "23", "24", "25", "26", "27", "28", "29", "30", "31", "32", "33", "34", "35", "36", "37", "38", "39", "40", "41", "42", "43",
                    "44", "45", "46", "47", "48", "49", "50", "51", "52", "53", "54", "55", "56", "57", "58", "59", "60", "61", "62", "63", "64", "65",
                    "66", "67", "68", "69", "70", "71", "72", "73", "74", "75", "76", "77", "78", "79", "80", "81", "82", "83", "84", "85", "86", "87"
                ],
                "type": "str"
            },
            "timezone-option": {"choices": ["disable", "default", "specify"], "type": "str"},
            "vci-match": {"choices": ["disable", "enable"], "type": "str"},
            "vci-string": {"type": "raw"},
            "wifi-ac1": {"type": "str"},
            "wifi-ac2": {"type": "str"},
            "wifi-ac3": {"type": "str"},
            "wins-server1": {"type": "str"},
            "wins-server2": {"type": "str"},
            "dns-server4": {"v_range": [["6.2.2", ""]], "type": "str"},
            "wifi-ac-service": {"v_range": [["6.2.2", ""]], "choices": ["specify", "local"], "type": "str"},
            "auto-managed-status": {"v_range": [["6.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "dhcp-settings-from-fortiipam": {"v_range": [["6.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "relay-agent": {"v_range": [["7.4.0", ""]], "type": "str"},
            "shared-subnet": {"v_range": [["7.4.0", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "fsp_vlan_dhcpserver_excluderange": {
        "stated": true,
        "options": {
            "end-ip": {"type": "str"},
            "id": {"required": true, "type": "int"},
            "start-ip": {"type": "str"},
            "vci-match": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "vci-string": {"v_range": [["7.2.1", ""]], "type": "raw"},
            "lease-time": {"v_range": [["7.2.2", ""]], "type": "int"},
            "uci-match": {"v_range": [["7.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "uci-string": {"v_range": [["7.2.2", ""]], "type": "raw"}
        }
    },
    "fsp_vlan_dhcpserver_iprange": {
        "stated": true,
        "options": {
            "end-ip": {"type": "str"},
            "id": {"required": true, "type": "int"},
            "start-ip": {"type": "str"},
            "vci-match": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "vci-string": {"v_range": [["7.2.1", ""]], "type": "raw"},
            "lease-time": {"v_range": [["7.2.2", ""]], "type": "int"},
            "uci-match": {"v_range": [["7.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "uci-string": {"v_range": [["7.2.2", ""]], "type": "raw"}
        }
    },
    "fsp_vlan_dhcpserver_options": {
        "stated": true,
        "options": {
            "code": {"type": "int"},
            "id": {"required": true, "type": "int"},
            "ip": {"type": "raw"},
            "type": {"choices": ["hex", "string", "ip", "fqdn"], "type": "str"},
            "value": {"type": "str"},
            "vci-match": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "vci-string": {"v_range": [["7.2.1", ""]], "type": "raw"},
            "uci-match": {"v_range": [["7.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "uci-string": {"v_range": [["7.2.2", ""]], "type": "raw"}
        }
    },
    "fsp_vlan_dhcpserver_reservedaddress": {
        "stated": true,
        "options": {
            "action": {"choices": ["assign", "block", "reserved"], "type": "str"},
            "circuit-id": {"type": "str"},
            "circuit-id-type": {"choices": ["hex", "string"], "type": "str"},
            "description": {"type": "str"},
            "id": {"required": true, "type": "int"},
            "ip": {"type": "str"},
            "mac": {"type": "str"},
            "remote-id": {"type": "str"},
            "remote-id-type": {"choices": ["hex", "string"], "type": "str"},
            "type": {"choices": ["mac", "option82"], "type": "str"}
        }
    },
    "fsp_vlan_dynamicmapping": {
        "stated": true,
        "options": {
            "_dhcp-status": {"choices": ["disable", "enable"], "type": "str"},
            "_scope": {"type": "list", "options": {"name": {"type": "str"}, "vdom": {"type": "str"}}, "elements": "dict"},
            "dhcp-server": {
                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                "type": "dict",
                "options": {
                    "auto-configuration": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "auto-managed-status": {"v_range": [["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "conflicted-ip-timeout": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "ddns-auth": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "tsig"], "type": "str"},
                    "ddns-key": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "no_log": true, "type": "raw"},
                    "ddns-keyname": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "no_log": true, "type": "str"},
                    "ddns-server-ip": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "ddns-ttl": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "ddns-update": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "ddns-update-override": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "ddns-zone": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "default-gateway": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "dhcp-settings-from-fortiipam": {"v_range": [["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "dns-server1": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "dns-server2": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "dns-server3": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "dns-server4": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "dns-service": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["default", "specify", "local"], "type": "str"},
                    "domain": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "enable": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "exclude-range": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "type": "list",
                        "options": {
                            "end-ip": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                            "id": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                            "start-ip": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                            "vci-match": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "vci-string": {"v_range": [["7.2.1", ""]], "type": "raw"},
                            "lease-time": {"v_range": [["7.2.2", ""]], "type": "int"},
                            "uci-match": {"v_range": [["7.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "uci-string": {"v_range": [["7.2.2", ""]], "type": "raw"}
                        },
                        "elements": "dict"
                    },
                    "filename": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "forticlient-on-net-status": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "id": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "ip-mode": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["range", "usrgrp"], "type": "str"},
                    "ip-range": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "type": "list",
                        "options": {
                            "end-ip": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                            "id": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                            "start-ip": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                            "vci-match": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "vci-string": {"v_range": [["7.2.1", ""]], "type": "raw"},
                            "lease-time": {"v_range": [["7.2.2", ""]], "type": "int"},
                            "uci-match": {"v_range": [["7.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "uci-string": {"v_range": [["7.2.2", ""]], "type": "raw"}
                        },
                        "elements": "dict"
                    },
                    "ipsec-lease-hold": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "lease-time": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "mac-acl-default-action": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["assign", "block"], "type": "str"},
                    "netmask": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "next-server": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "ntp-server1": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "ntp-server2": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "ntp-server3": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "ntp-service": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["default", "specify", "local"], "type": "str"},
                    "option1": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "raw"},
                    "option2": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "raw"},
                    "option3": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "raw"},
                    "option4": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "option5": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "option6": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "options": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "type": "list",
                        "options": {
                            "code": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                            "id": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                            "ip": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "raw"},
                            "type": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["hex", "string", "ip", "fqdn"], "type": "str"},
                            "value": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                            "vci-match": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "vci-string": {"v_range": [["7.2.1", ""]], "type": "raw"},
                            "uci-match": {"v_range": [["7.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "uci-string": {"v_range": [["7.2.2", ""]], "type": "raw"}
                        },
                        "elements": "dict"
                    },
                    "reserved-address": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "type": "list",
                        "options": {
                            "action": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["assign", "block", "reserved"], "type": "str"},
                            "circuit-id": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                            "circuit-id-type": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["hex", "string"], "type": "str"},
                            "description": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                            "id": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                            "ip": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                            "mac": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                            "remote-id": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                            "remote-id-type": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["hex", "string"], "type": "str"},
                            "type": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["mac", "option82"], "type": "str"}
                        },
                        "elements": "dict"
                    },
                    "server-type": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["regular", "ipsec"], "type": "str"},
                    "status": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "tftp-server": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "raw"},
                    "timezone": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "choices": [
                            "00", "01", "02", "03", "04", "05", "06", "07", "08", "09", "10", "11", "12", "13", "14", "15", "16", "17", "18", "19", "20",
                            "21", "22", "23", "24", "25", "26", "27", "28", "29", "30", "31", "32", "33", "34", "35", "36", "37", "38", "39", "40", "41",
                            "42", "43", "44", "45", "46", "47", "48", "49", "50", "51", "52", "53", "54", "55", "56", "57", "58", "59", "60", "61", "62",
                            "63", "64", "65", "66", "67", "68", "69", "70", "71", "72", "73", "74", "75", "76", "77", "78", "79", "80", "81", "82", "83",
                            "84", "85", "86", "87"
                        ],
                        "type": "str"
                    },
                    "timezone-option": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "default", "specify"], "type": "str"},
                    "vci-match": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "vci-string": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "raw"},
                    "wifi-ac-service": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["specify", "local"], "type": "str"},
                    "wifi-ac1": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "wifi-ac2": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "wifi-ac3": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "wins-server1": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "wins-server2": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "relay-agent": {"v_range": [["7.4.0", ""]], "type": "str"},
                    "shared-subnet": {"v_range": [["7.4.0", ""]], "choices": ["disable", "enable"], "type": "str"}
                }
            },
            "interface": {
                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                "type": "dict",
                "options": {
                    "dhcp-relay-agent-option": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "dhcp-relay-ip": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "raw"},
                    "dhcp-relay-service": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "dhcp-relay-type": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["regular", "ipsec"], "type": "str"},
                    "ip": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "ipv6": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "type": "dict",
                        "options": {
                            "autoconf": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "dhcp6-client-options": {
                                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                                "type": "list",
                                "choices": ["rapid", "iapd", "iana", "dns", "dnsname"],
                                "elements": "str"
                            },
                            "dhcp6-information-request": {
                                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                                "choices": ["disable", "enable"],
                                "type": "str"
                            },
                            "dhcp6-prefix-delegation": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "dhcp6-prefix-hint": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                            "dhcp6-prefix-hint-plt": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                            "dhcp6-prefix-hint-vlt": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                            "dhcp6-relay-ip": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                            "dhcp6-relay-service": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "dhcp6-relay-type": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["regular"], "type": "str"},
                            "icmp6-send-redirect": {"v_range": [["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "interface-identifier": {"v_range": [["6.4.5", ""]], "type": "str"},
                            "ip6-address": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                            "ip6-allowaccess": {
                                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                                "type": "list",
                                "choices": ["https", "ping", "ssh", "snmp", "http", "telnet", "fgfm", "capwap", "fabric"],
                                "elements": "str"
                            },
                            "ip6-default-life": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                            "ip6-delegated-prefix-list": {
                                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                                "type": "list",
                                "options": {
                                    "autonomous-flag": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                                    "onlink-flag": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                                    "prefix-id": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                                    "rdnss": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "raw"},
                                    "rdnss-service": {
                                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                                        "choices": ["delegated", "default", "specify"],
                                        "type": "str"
                                    },
                                    "subnet": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                                    "upstream-interface": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                                    "delegated-prefix-iaid": {"v_range": [["7.0.2", ""]], "type": "int"}
                                },
                                "elements": "dict"
                            },
                            "ip6-dns-server-override": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "ip6-extra-addr": {
                                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                                "type": "list",
                                "options": {"prefix": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"}},
                                "elements": "dict"
                            },
                            "ip6-hop-limit": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                            "ip6-link-mtu": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                            "ip6-manage-flag": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "ip6-max-interval": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                            "ip6-min-interval": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                            "ip6-mode": {
                                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                                "choices": ["static", "dhcp", "pppoe", "delegated"],
                                "type": "str"
                            },
                            "ip6-other-flag": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "ip6-prefix-list": {
                                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                                "type": "list",
                                "options": {
                                    "autonomous-flag": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                                    "dnssl": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "raw"},
                                    "onlink-flag": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                                    "preferred-life-time": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                                    "prefix": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                                    "rdnss": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "raw"},
                                    "valid-life-time": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"}
                                },
                                "elements": "dict"
                            },
                            "ip6-reachable-time": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                            "ip6-retrans-time": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                            "ip6-send-adv": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "ip6-subnet": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                            "ip6-upstream-interface": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                            "nd-cert": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                            "nd-cga-modifier": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                            "nd-mode": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["basic", "SEND-compatible"], "type": "str"},
                            "nd-security-level": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                            "nd-timestamp-delta": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                            "nd-timestamp-fuzz": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                            "unique-autoconf-addr": {"v_range": [["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "vrip6_link_local": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                            "vrrp-virtual-mac6": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "vrrp6": {
                                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                                "type": "list",
                                "options": {
                                    "accept-mode": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                                    "adv-interval": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                                    "preempt": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                                    "priority": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                                    "start-time": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                                    "status": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                                    "vrdst6": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                                    "vrgrp": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                                    "vrid": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                                    "vrip6": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                                    "ignore-default-route": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                                    "vrdst-priority": {"v_range": [["7.6.0", ""]], "type": "int"}
                                },
                                "elements": "dict"
                            },
                            "cli-conn6-status": {"v_range": [["7.0.0", ""]], "type": "int"},
                            "ip6-prefix-mode": {"v_range": [["7.0.0", ""]], "choices": ["dhcp6", "ra"], "type": "str"},
                            "ra-send-mtu": {"v_range": [["6.4.6", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "ip6-delegated-prefix-iaid": {"v_range": [["7.0.2", ""]], "type": "int"},
                            "dhcp6-relay-source-interface": {"v_range": [["7.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "dhcp6-relay-interface-id": {"v_range": [["7.4.1", ""]], "type": "str"},
                            "dhcp6-relay-source-ip": {"v_range": [["7.4.1", ""]], "type": "str"}
                        }
                    },
                    "secondary-IP": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "secondaryip": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "type": "list",
                        "options": {
                            "allowaccess": {
                                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                                "type": "list",
                                "choices": [
                                    "https", "ping", "ssh", "snmp", "http", "telnet", "fgfm", "auto-ipsec", "radius-acct", "probe-response", "capwap",
                                    "dnp", "ftm", "fabric", "speed-test", "icond", "scim"
                                ],
                                "elements": "str"
                            },
                            "detectprotocol": {
                                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", "7.2.0"]],
                                "type": "list",
                                "choices": ["ping", "tcp-echo", "udp-echo"],
                                "elements": "str"
                            },
                            "detectserver": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", "7.2.0"]], "type": "str"},
                            "gwdetect": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", "7.2.0"]], "choices": ["disable", "enable"], "type": "str"},
                            "ha-priority": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", "7.2.0"]], "type": "int"},
                            "id": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                            "ip": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                            "ping-serv-status": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", "7.2.0"]], "type": "int"},
                            "seq": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                            "secip-relay-ip": {"v_range": [["7.4.0", ""]], "type": "str"}
                        },
                        "elements": "dict"
                    },
                    "vlanid": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "dhcp-relay-interface-select-method": {
                        "v_range": [["6.4.8", "6.4.14"], ["7.0.4", ""]],
                        "choices": ["auto", "sdwan", "specify"],
                        "type": "str"
                    },
                    "vrrp": {
                        "v_range": [["7.4.0", ""]],
                        "type": "list",
                        "options": {
                            "accept-mode": {"v_range": [["7.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "adv-interval": {"v_range": [["7.4.0", ""]], "type": "int"},
                            "ignore-default-route": {"v_range": [["7.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "preempt": {"v_range": [["7.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "priority": {"v_range": [["7.4.0", ""]], "type": "int"},
                            "proxy-arp": {
                                "v_range": [["7.4.0", ""]],
                                "type": "list",
                                "options": {"id": {"v_range": [["7.4.0", ""]], "type": "int"}, "ip": {"v_range": [["7.4.0", ""]], "type": "str"}},
                                "elements": "dict"
                            },
                            "start-time": {"v_range": [["7.4.0", ""]], "type": "int"},
                            "status": {"v_range": [["7.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "version": {"v_range": [["7.4.0", ""]], "choices": ["2", "3"], "type": "str"},
                            "vrdst": {"v_range": [["7.4.0", ""]], "type": "raw"},
                            "vrdst-priority": {"v_range": [["7.4.0", ""]], "type": "int"},
                            "vrgrp": {"v_range": [["7.4.0", ""]], "type": "int"},
                            "vrid": {"v_range": [["7.4.0", ""]], "type": "int"},
                            "vrip": {"v_range": [["7.4.0", ""]], "type": "str"}
                        },
                        "elements": "dict"
                    }
                }
            }
        }
    },
    "fsp_vlan_dynamicmapping_dhcpserver": {
        "stated": false,
        "options": {
            "auto-configuration": {"v_range": [["6.0.0", "7.4.0"]], "choices": ["disable", "enable"], "type": "str"},
            "conflicted-ip-timeout": {"v_range": [["6.0.0", "7.4.0"]], "type": "int"},
            "ddns-auth": {"v_range": [["6.0.0", "7.4.0"]], "choices": ["disable", "tsig"], "type": "str"},
            "ddns-key": {"v_range": [["6.0.0", "7.4.0"]], "no_log": true, "type": "raw"},
            "ddns-keyname": {"v_range": [["6.0.0", "7.4.0"]], "no_log": true, "type": "str"},
            "ddns-server-ip": {"v_range": [["6.0.0", "7.4.0"]], "type": "str"},
            "ddns-ttl": {"v_range": [["6.0.0", "7.4.0"]], "type": "int"},
            "ddns-update": {"v_range": [["6.0.0", "7.4.0"]], "choices": ["disable", "enable"], "type": "str"},
            "ddns-update-override": {"v_range": [["6.0.0", "7.4.0"]], "choices": ["disable", "enable"], "type": "str"},
            "ddns-zone": {"v_range": [["6.0.0", "7.4.0"]], "type": "str"},
            "default-gateway": {"v_range": [["6.0.0", "7.4.0"]], "type": "str"},
            "dns-server1": {"v_range": [["6.0.0", "7.4.0"]], "type": "str"},
            "dns-server2": {"v_range": [["6.0.0", "7.4.0"]], "type": "str"},
            "dns-server3": {"v_range": [["6.0.0", "7.4.0"]], "type": "str"},
            "dns-service": {"v_range": [["6.0.0", "7.4.0"]], "choices": ["default", "specify", "local"], "type": "str"},
            "domain": {"v_range": [["6.0.0", "7.4.0"]], "type": "str"},
            "enable": {"v_range": [["6.0.0", "7.4.0"]], "choices": ["disable", "enable"], "type": "str"},
            "exclude-range": {
                "v_range": [["6.0.0", "7.4.0"]],
                "type": "list",
                "options": {
                    "end-ip": {"v_range": [["6.0.0", "7.4.0"]], "type": "str"},
                    "id": {"v_range": [["6.0.0", "7.4.0"]], "type": "int"},
                    "start-ip": {"v_range": [["6.0.0", "7.4.0"]], "type": "str"},
                    "vci-match": {"v_range": [["7.2.1", "7.4.0"]], "choices": ["disable", "enable"], "type": "str"},
                    "vci-string": {"v_range": [["7.2.1", "7.4.0"]], "type": "raw"},
                    "lease-time": {"v_range": [["7.2.2", "7.4.0"]], "type": "int"},
                    "uci-match": {"v_range": [["7.2.2", "7.4.0"]], "choices": ["disable", "enable"], "type": "str"},
                    "uci-string": {"v_range": [["7.2.2", "7.4.0"]], "type": "raw"}
                },
                "elements": "dict"
            },
            "filename": {"v_range": [["6.0.0", "7.4.0"]], "type": "str"},
            "forticlient-on-net-status": {"v_range": [["6.0.0", "7.4.0"]], "choices": ["disable", "enable"], "type": "str"},
            "id": {"v_range": [["6.0.0", "7.4.0"]], "type": "int"},
            "interface": {"v_range": [["6.0.0", "6.2.1"]], "type": "str"},
            "ip-mode": {"v_range": [["6.0.0", "7.4.0"]], "choices": ["range", "usrgrp"], "type": "str"},
            "ip-range": {
                "v_range": [["6.0.0", "7.4.0"]],
                "type": "list",
                "options": {
                    "end-ip": {"v_range": [["6.0.0", "7.4.0"]], "type": "str"},
                    "id": {"v_range": [["6.0.0", "7.4.0"]], "type": "int"},
                    "start-ip": {"v_range": [["6.0.0", "7.4.0"]], "type": "str"},
                    "vci-match": {"v_range": [["7.2.1", "7.4.0"]], "choices": ["disable", "enable"], "type": "str"},
                    "vci-string": {"v_range": [["7.2.1", "7.4.0"]], "type": "raw"},
                    "lease-time": {"v_range": [["7.2.2", "7.4.0"]], "type": "int"},
                    "uci-match": {"v_range": [["7.2.2", "7.4.0"]], "choices": ["disable", "enable"], "type": "str"},
                    "uci-string": {"v_range": [["7.2.2", "7.4.0"]], "type": "raw"}
                },
                "elements": "dict"
            },
            "ipsec-lease-hold": {"v_range": [["6.0.0", "7.4.0"]], "type": "int"},
            "lease-time": {"v_range": [["6.0.0", "7.4.0"]], "type": "int"},
            "mac-acl-default-action": {"v_range": [["6.0.0", "7.4.0"]], "choices": ["assign", "block"], "type": "str"},
            "netmask": {"v_range": [["6.0.0", "7.4.0"]], "type": "str"},
            "next-server": {"v_range": [["6.0.0", "7.4.0"]], "type": "str"},
            "ntp-server1": {"v_range": [["6.0.0", "7.4.0"]], "type": "str"},
            "ntp-server2": {"v_range": [["6.0.0", "7.4.0"]], "type": "str"},
            "ntp-server3": {"v_range": [["6.0.0", "7.4.0"]], "type": "str"},
            "ntp-service": {"v_range": [["6.0.0", "7.4.0"]], "choices": ["default", "specify", "local"], "type": "str"},
            "option1": {"v_range": [["6.0.0", "7.4.0"]], "type": "raw"},
            "option2": {"v_range": [["6.0.0", "7.4.0"]], "type": "raw"},
            "option3": {"v_range": [["6.0.0", "7.4.0"]], "type": "raw"},
            "option4": {"v_range": [["6.0.0", "7.4.0"]], "type": "str"},
            "option5": {"v_range": [["6.0.0", "7.4.0"]], "type": "str"},
            "option6": {"v_range": [["6.0.0", "7.4.0"]], "type": "str"},
            "options": {
                "v_range": [["6.0.0", "7.4.0"]],
                "type": "list",
                "options": {
                    "code": {"v_range": [["6.0.0", "7.4.0"]], "type": "int"},
                    "id": {"v_range": [["6.0.0", "7.4.0"]], "type": "int"},
                    "ip": {"v_range": [["6.0.0", "7.4.0"]], "type": "raw"},
                    "type": {"v_range": [["6.0.0", "7.4.0"]], "choices": ["hex", "string", "ip", "fqdn"], "type": "str"},
                    "value": {"v_range": [["6.0.0", "7.4.0"]], "type": "str"},
                    "vci-match": {"v_range": [["7.2.1", "7.4.0"]], "choices": ["disable", "enable"], "type": "str"},
                    "vci-string": {"v_range": [["7.2.1", "7.4.0"]], "type": "raw"},
                    "uci-match": {"v_range": [["7.2.2", "7.4.0"]], "choices": ["disable", "enable"], "type": "str"},
                    "uci-string": {"v_range": [["7.2.2", "7.4.0"]], "type": "raw"}
                },
                "elements": "dict"
            },
            "reserved-address": {
                "v_range": [["6.0.0", "7.4.0"]],
                "type": "list",
                "options": {
                    "action": {"v_range": [["6.0.0", "7.4.0"]], "choices": ["assign", "block", "reserved"], "type": "str"},
                    "circuit-id": {"v_range": [["6.0.0", "7.4.0"]], "type": "str"},
                    "circuit-id-type": {"v_range": [["6.0.0", "7.4.0"]], "choices": ["hex", "string"], "type": "str"},
                    "description": {"v_range": [["6.0.0", "7.4.0"]], "type": "str"},
                    "id": {"v_range": [["6.0.0", "7.4.0"]], "type": "int"},
                    "ip": {"v_range": [["6.0.0", "7.4.0"]], "type": "str"},
                    "mac": {"v_range": [["6.0.0", "7.4.0"]], "type": "str"},
                    "remote-id": {"v_range": [["6.0.0", "7.4.0"]], "type": "str"},
                    "remote-id-type": {"v_range": [["6.0.0", "7.4.0"]], "choices": ["hex", "string"], "type": "str"},
                    "type": {"v_range": [["6.0.0", "7.4.0"]], "choices": ["mac", "option82"], "type": "str"}
                },
                "elements": "dict"
            },
            "server-type": {"v_range": [["6.0.0", "7.4.0"]], "choices": ["regular", "ipsec"], "type": "str"},
            "status": {"v_range": [["6.0.0", "7.4.0"]], "choices": ["disable", "enable"], "type": "str"},
            "tftp-server": {"v_range": [["6.0.0", "7.4.0"]], "type": "raw"},
            "timezone": {
                "v_range": [["6.0.0", "7.4.0"]],
                "choices": [
                    "00", "01", "02", "03", "04", "05", "06", "07", "08", "09", "10", "11", "12", "13", "14", "15", "16", "17", "18", "19", "20", "21",
                    "22", "23", "24", "25", "26", "27", "28", "29", "30", "31", "32", "33", "34", "35", "36", "37", "38", "39", "40", "41", "42", "43",
                    "44", "45", "46", "47", "48", "49", "50", "51", "52", "53", "54", "55", "56", "57", "58", "59", "60", "61", "62", "63", "64", "65",
                    "66", "67", "68", "69", "70", "71", "72", "73", "74", "75", "76", "77", "78", "79", "80", "81", "82", "83", "84", "85", "86", "87"
                ],
                "type": "str"
            },
            "timezone-option": {"v_range": [["6.0.0", "7.4.0"]], "choices": ["disable", "default", "specify"], "type": "str"},
            "vci-match": {"v_range": [["6.0.0", "7.4.0"]], "choices": ["disable", "enable"], "type": "str"},
            "vci-string": {"v_range": [["6.0.0", "7.4.0"]], "type": "raw"},
            "wifi-ac1": {"v_range": [["6.0.0", "7.4.0"]], "type": "str"},
            "wifi-ac2": {"v_range": [["6.0.0", "7.4.0"]], "type": "str"},
            "wifi-ac3": {"v_range": [["6.0.0", "7.4.0"]], "type": "str"},
            "wins-server1": {"v_range": [["6.0.0", "7.4.0"]], "type": "str"},
            "wins-server2": {"v_range": [["6.0.0", "7.4.0"]], "type": "str"},
            "dns-server4": {"v_range": [["6.2.2", "7.4.0"]], "type": "str"},
            "wifi-ac-service": {"v_range": [["6.2.2", "7.4.0"]], "choices": ["specify", "local"], "type": "str"},
            "auto-managed-status": {"v_range": [["6.4.0", "7.4.0"]], "choices": ["disable", "enable"], "type": "str"},
            "dhcp-settings-from-fortiipam": {"v_range": [["6.4.0", "7.4.0"]], "choices": ["disable", "enable"], "type": "str"},
            "relay-agent": {"v_range": [["7.4.0", "7.4.0"]], "type": "str"},
            "shared-subnet": {"v_range": [["7.4.0", "7.4.0"]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "fsp_vlan_dynamicmapping_dhcpserver_excluderange": {
        "stated": true,
        "options": {
            "end-ip": {"v_range": [["6.0.0", "7.4.0"]], "type": "str"},
            "id": {"v_range": [["6.0.0", "7.4.0"]], "required": true, "type": "int"},
            "start-ip": {"v_range": [["6.0.0", "7.4.0"]], "type": "str"},
            "vci-match": {"v_range": [["7.2.1", "7.4.0"]], "choices": ["disable", "enable"], "type": "str"},
            "vci-string": {"v_range": [["7.2.1", "7.4.0"]], "type": "raw"},
            "lease-time": {"v_range": [["7.2.2", "7.4.0"]], "type": "int"},
            "uci-match": {"v_range": [["7.2.2", "7.4.0"]], "choices": ["disable", "enable"], "type": "str"},
            "uci-string": {"v_range": [["7.2.2", "7.4.0"]], "type": "raw"}
        }
    },
    "fsp_vlan_dynamicmapping_dhcpserver_iprange": {
        "stated": true,
        "options": {
            "end-ip": {"v_range": [["6.0.0", "7.4.0"]], "type": "str"},
            "id": {"v_range": [["6.0.0", "7.4.0"]], "required": true, "type": "int"},
            "start-ip": {"v_range": [["6.0.0", "7.4.0"]], "type": "str"},
            "vci-match": {"v_range": [["7.2.1", "7.4.0"]], "choices": ["disable", "enable"], "type": "str"},
            "vci-string": {"v_range": [["7.2.1", "7.4.0"]], "type": "raw"},
            "lease-time": {"v_range": [["7.2.2", "7.4.0"]], "type": "int"},
            "uci-match": {"v_range": [["7.2.2", "7.4.0"]], "choices": ["disable", "enable"], "type": "str"},
            "uci-string": {"v_range": [["7.2.2", "7.4.0"]], "type": "raw"}
        }
    },
    "fsp_vlan_dynamicmapping_dhcpserver_options": {
        "stated": true,
        "options": {
            "code": {"v_range": [["6.0.0", "7.4.0"]], "type": "int"},
            "id": {"v_range": [["6.0.0", "7.4.0"]], "required": true, "type": "int"},
            "ip": {"v_range": [["6.0.0", "7.4.0"]], "type": "raw"},
            "type": {"v_range": [["6.0.0", "7.4.0"]], "choices": ["hex", "string", "ip", "fqdn"], "type": "str"},
            "value": {"v_range": [["6.0.0", "7.4.0"]], "type": "str"},
            "vci-match": {"v_range": [["7.2.1", "7.4.0"]], "choices": ["disable", "enable"], "type": "str"},
            "vci-string": {"v_range": [["7.2.1", "7.4.0"]], "type": "raw"},
            "uci-match": {"v_range": [["7.2.2", "7.4.0"]], "choices": ["disable", "enable"], "type": "str"},
            "uci-string": {"v_range": [["7.2.2", "7.4.0"]], "type": "raw"}
        }
    },
    "fsp_vlan_dynamicmapping_dhcpserver_reservedaddress": {
        "stated": true,
        "options": {
            "action": {"v_range": [["6.0.0", "7.4.0"]], "choices": ["assign", "block", "reserved"], "type": "str"},
            "circuit-id": {"v_range": [["6.0.0", "7.4.0"]], "type": "str"},
            "circuit-id-type": {"v_range": [["6.0.0", "7.4.0"]], "choices": ["hex", "string"], "type": "str"},
            "description": {"v_range": [["6.0.0", "7.4.0"]], "type": "str"},
            "id": {"v_range": [["6.0.0", "7.4.0"]], "required": true, "type": "int"},
            "ip": {"v_range": [["6.0.0", "7.4.0"]], "type": "str"},
            "mac": {"v_range": [["6.0.0", "7.4.0"]], "type": "str"},
            "remote-id": {"v_range": [["6.0.0", "7.4.0"]], "type": "str"},
            "remote-id-type": {"v_range": [["6.0.0", "7.4.0"]], "choices": ["hex", "string"], "type": "str"},
            "type": {"v_range": [["6.0.0", "7.4.0"]], "choices": ["mac", "option82"], "type": "str"}
        }
    },
    "fsp_vlan_dynamicmapping_interface": {
        "stated": false,
        "options": {
            "ip": {"v_range": [["6.0.0", "7.4.0"]], "type": "str"},
            "vlanid": {"v_range": [["6.0.0", "7.4.0"]], "type": "int"},
            "dhcp-relay-agent-option": {"v_range": [["6.2.2", "7.4.0"]], "choices": ["disable", "enable"], "type": "str"},
            "dhcp-relay-ip": {"v_range": [["6.2.2", "7.4.0"]], "type": "raw"},
            "dhcp-relay-service": {"v_range": [["6.2.2", "7.4.0"]], "choices": ["disable", "enable"], "type": "str"},
            "dhcp-relay-type": {"v_range": [["6.2.2", "7.4.0"]], "choices": ["regular", "ipsec"], "type": "str"},
            "ipv6": {
                "v_range": [["6.2.2", "7.4.0"]],
                "type": "dict",
                "options": {
                    "autoconf": {"v_range": [["6.2.2", "7.4.0"]], "choices": ["disable", "enable"], "type": "str"},
                    "dhcp6-client-options": {
                        "v_range": [["6.2.2", "7.4.0"]],
                        "type": "list",
                        "choices": ["rapid", "iapd", "iana", "dns", "dnsname"],
                        "elements": "str"
                    },
                    "dhcp6-information-request": {"v_range": [["6.2.2", "7.4.0"]], "choices": ["disable", "enable"], "type": "str"},
                    "dhcp6-prefix-delegation": {"v_range": [["6.2.2", "7.4.0"]], "choices": ["disable", "enable"], "type": "str"},
                    "dhcp6-prefix-hint": {"v_range": [["6.2.2", "7.4.0"]], "type": "str"},
                    "dhcp6-prefix-hint-plt": {"v_range": [["6.2.2", "7.4.0"]], "type": "int"},
                    "dhcp6-prefix-hint-vlt": {"v_range": [["6.2.2", "7.4.0"]], "type": "int"},
                    "dhcp6-relay-ip": {"v_range": [["6.2.2", "7.4.0"]], "type": "str"},
                    "dhcp6-relay-service": {"v_range": [["6.2.2", "7.4.0"]], "choices": ["disable", "enable"], "type": "str"},
                    "dhcp6-relay-type": {"v_range": [["6.2.2", "7.4.0"]], "choices": ["regular"], "type": "str"},
                    "ip6-address": {"v_range": [["6.2.2", "7.4.0"]], "type": "str"},
                    "ip6-allowaccess": {
                        "v_range": [["6.2.2", "7.4.0"]],
                        "type": "list",
                        "choices": ["https", "ping", "ssh", "snmp", "http", "telnet", "fgfm", "capwap", "fabric"],
                        "elements": "str"
                    },
                    "ip6-default-life": {"v_range": [["6.2.2", "7.4.0"]], "type": "int"},
                    "ip6-delegated-prefix-list": {
                        "v_range": [["6.2.2", "7.4.0"]],
                        "type": "list",
                        "options": {
                            "autonomous-flag": {"v_range": [["6.2.2", "7.4.0"]], "choices": ["disable", "enable"], "type": "str"},
                            "onlink-flag": {"v_range": [["6.2.2", "7.4.0"]], "choices": ["disable", "enable"], "type": "str"},
                            "prefix-id": {"v_range": [["6.2.2", "7.4.0"]], "type": "int"},
                            "rdnss": {"v_range": [["6.2.2", "7.4.0"]], "type": "raw"},
                            "rdnss-service": {"v_range": [["6.2.2", "7.4.0"]], "choices": ["delegated", "default", "specify"], "type": "str"},
                            "subnet": {"v_range": [["6.2.2", "7.4.0"]], "type": "str"},
                            "upstream-interface": {"v_range": [["6.2.2", "7.4.0"]], "type": "str"},
                            "delegated-prefix-iaid": {"v_range": [["7.0.2", "7.4.0"]], "type": "int"}
                        },
                        "elements": "dict"
                    },
                    "ip6-dns-server-override": {"v_range": [["6.2.2", "7.4.0"]], "choices": ["disable", "enable"], "type": "str"},
                    "ip6-extra-addr": {
                        "v_range": [["6.2.2", "7.4.0"]],
                        "type": "list",
                        "options": {"prefix": {"v_range": [["6.2.2", "7.4.0"]], "type": "str"}},
                        "elements": "dict"
                    },
                    "ip6-hop-limit": {"v_range": [["6.2.2", "7.4.0"]], "type": "int"},
                    "ip6-link-mtu": {"v_range": [["6.2.2", "7.4.0"]], "type": "int"},
                    "ip6-manage-flag": {"v_range": [["6.2.2", "7.4.0"]], "choices": ["disable", "enable"], "type": "str"},
                    "ip6-max-interval": {"v_range": [["6.2.2", "7.4.0"]], "type": "int"},
                    "ip6-min-interval": {"v_range": [["6.2.2", "7.4.0"]], "type": "int"},
                    "ip6-mode": {"v_range": [["6.2.2", "7.4.0"]], "choices": ["static", "dhcp", "pppoe", "delegated"], "type": "str"},
                    "ip6-other-flag": {"v_range": [["6.2.2", "7.4.0"]], "choices": ["disable", "enable"], "type": "str"},
                    "ip6-prefix-list": {
                        "v_range": [["6.2.2", "7.4.0"]],
                        "type": "list",
                        "options": {
                            "autonomous-flag": {"v_range": [["6.2.2", "7.4.0"]], "choices": ["disable", "enable"], "type": "str"},
                            "dnssl": {"v_range": [["6.2.2", "7.4.0"]], "type": "raw"},
                            "onlink-flag": {"v_range": [["6.2.2", "7.4.0"]], "choices": ["disable", "enable"], "type": "str"},
                            "preferred-life-time": {"v_range": [["6.2.2", "7.4.0"]], "type": "int"},
                            "prefix": {"v_range": [["6.2.2", "7.4.0"]], "type": "str"},
                            "rdnss": {"v_range": [["6.2.2", "7.4.0"]], "type": "raw"},
                            "valid-life-time": {"v_range": [["6.2.2", "7.4.0"]], "type": "int"}
                        },
                        "elements": "dict"
                    },
                    "ip6-reachable-time": {"v_range": [["6.2.2", "7.4.0"]], "type": "int"},
                    "ip6-retrans-time": {"v_range": [["6.2.2", "7.4.0"]], "type": "int"},
                    "ip6-send-adv": {"v_range": [["6.2.2", "7.4.0"]], "choices": ["disable", "enable"], "type": "str"},
                    "ip6-subnet": {"v_range": [["6.2.2", "7.4.0"]], "type": "str"},
                    "ip6-upstream-interface": {"v_range": [["6.2.2", "7.4.0"]], "type": "str"},
                    "nd-cert": {"v_range": [["6.2.2", "7.4.0"]], "type": "str"},
                    "nd-cga-modifier": {"v_range": [["6.2.2", "7.4.0"]], "type": "str"},
                    "nd-mode": {"v_range": [["6.2.2", "7.4.0"]], "choices": ["basic", "SEND-compatible"], "type": "str"},
                    "nd-security-level": {"v_range": [["6.2.2", "7.4.0"]], "type": "int"},
                    "nd-timestamp-delta": {"v_range": [["6.2.2", "7.4.0"]], "type": "int"},
                    "nd-timestamp-fuzz": {"v_range": [["6.2.2", "7.4.0"]], "type": "int"},
                    "vrip6_link_local": {"v_range": [["6.2.2", "7.4.0"]], "type": "str"},
                    "vrrp-virtual-mac6": {"v_range": [["6.2.2", "7.4.0"]], "choices": ["disable", "enable"], "type": "str"},
                    "vrrp6": {
                        "v_range": [["6.2.2", "7.4.0"]],
                        "type": "list",
                        "options": {
                            "accept-mode": {"v_range": [["6.2.2", "7.4.0"]], "choices": ["disable", "enable"], "type": "str"},
                            "adv-interval": {"v_range": [["6.2.2", "7.4.0"]], "type": "int"},
                            "preempt": {"v_range": [["6.2.2", "7.4.0"]], "choices": ["disable", "enable"], "type": "str"},
                            "priority": {"v_range": [["6.2.2", "7.4.0"]], "type": "int"},
                            "start-time": {"v_range": [["6.2.2", "7.4.0"]], "type": "int"},
                            "status": {"v_range": [["6.2.2", "7.4.0"]], "choices": ["disable", "enable"], "type": "str"},
                            "vrdst6": {"v_range": [["6.2.2", "7.4.0"]], "type": "str"},
                            "vrgrp": {"v_range": [["6.2.2", "7.4.0"]], "type": "int"},
                            "vrid": {"v_range": [["6.2.2", "7.4.0"]], "type": "int"},
                            "vrip6": {"v_range": [["6.2.2", "7.4.0"]], "type": "str"}
                        },
                        "elements": "dict"
                    },
                    "interface-identifier": {"v_range": [["6.4.1", "7.4.0"]], "type": "str"},
                    "unique-autoconf-addr": {"v_range": [["6.4.1", "7.4.0"]], "choices": ["disable", "enable"], "type": "str"},
                    "icmp6-send-redirect": {"v_range": [["6.4.4", "7.4.0"]], "choices": ["disable", "enable"], "type": "str"},
                    "cli-conn6-status": {"v_range": [["7.0.0", "7.4.0"]], "type": "int"},
                    "ip6-prefix-mode": {"v_range": [["7.0.0", "7.4.0"]], "choices": ["dhcp6", "ra"], "type": "str"},
                    "ra-send-mtu": {"v_range": [["6.4.6", "7.4.0"]], "choices": ["disable", "enable"], "type": "str"},
                    "ip6-delegated-prefix-iaid": {"v_range": [["7.0.2", "7.4.0"]], "type": "int"},
                    "dhcp6-relay-source-interface": {"v_range": [["7.2.2", "7.4.0"]], "choices": ["disable", "enable"], "type": "str"}
                }
            },
            "secondary-IP": {"v_range": [["6.2.3", "7.4.0"]], "choices": ["disable", "enable"], "type": "str"},
            "secondaryip": {
                "v_range": [["6.2.3", "7.4.0"]],
                "type": "list",
                "options": {
                    "allowaccess": {
                        "v_range": [["6.2.3", "7.4.0"]],
                        "type": "list",
                        "choices": [
                            "https", "ping", "ssh", "snmp", "http", "telnet", "fgfm", "auto-ipsec", "radius-acct", "probe-response", "capwap", "dnp",
                            "ftm", "fabric", "speed-test"
                        ],
                        "elements": "str"
                    },
                    "detectprotocol": {"v_range": [["6.2.3", "7.2.0"]], "type": "list", "choices": ["ping", "tcp-echo", "udp-echo"], "elements": "str"},
                    "detectserver": {"v_range": [["6.2.3", "7.2.0"]], "type": "str"},
                    "gwdetect": {"v_range": [["6.2.3", "7.2.0"]], "choices": ["disable", "enable"], "type": "str"},
                    "ha-priority": {"v_range": [["6.2.3", "7.2.0"]], "type": "int"},
                    "id": {"v_range": [["6.2.3", "7.4.0"]], "type": "int"},
                    "ip": {"v_range": [["6.2.3", "7.4.0"]], "type": "str"},
                    "ping-serv-status": {"v_range": [["6.2.3", "7.2.0"]], "type": "int"},
                    "seq": {"v_range": [["6.2.3", "7.4.0"]], "type": "int"},
                    "secip-relay-ip": {"v_range": [["7.4.0", "7.4.0"]], "type": "str"}
                },
                "elements": "dict"
            },
            "dhcp-relay-interface-select-method": {
                "v_range": [["6.4.8", "6.4.14"], ["7.0.4", "7.4.0"]],
                "choices": ["auto", "sdwan", "specify"],
                "type": "str"
            },
            "vrrp": {
                "v_range": [["7.4.0", "7.4.0"]],
                "type": "list",
                "options": {
                    "accept-mode": {"v_range": [["7.4.0", "7.4.0"]], "choices": ["disable", "enable"], "type": "str"},
                    "adv-interval": {"v_range": [["7.4.0", "7.4.0"]], "type": "int"},
                    "ignore-default-route": {"v_range": [["7.4.0", "7.4.0"]], "choices": ["disable", "enable"], "type": "str"},
                    "preempt": {"v_range": [["7.4.0", "7.4.0"]], "choices": ["disable", "enable"], "type": "str"},
                    "priority": {"v_range": [["7.4.0", "7.4.0"]], "type": "int"},
                    "proxy-arp": {
                        "v_range": [["7.4.0", "7.4.0"]],
                        "type": "list",
                        "options": {"id": {"v_range": [["7.4.0", "7.4.0"]], "type": "int"}, "ip": {"v_range": [["7.4.0", "7.4.0"]], "type": "str"}},
                        "elements": "dict"
                    },
                    "start-time": {"v_range": [["7.4.0", "7.4.0"]], "type": "int"},
                    "status": {"v_range": [["7.4.0", "7.4.0"]], "choices": ["disable", "enable"], "type": "str"},
                    "version": {"v_range": [["7.4.0", "7.4.0"]], "choices": ["2", "3"], "type": "str"},
                    "vrdst": {"v_range": [["7.4.0", "7.4.0"]], "type": "raw"},
                    "vrdst-priority": {"v_range": [["7.4.0", "7.4.0"]], "type": "int"},
                    "vrgrp": {"v_range": [["7.4.0", "7.4.0"]], "type": "int"},
                    "vrid": {"v_range": [["7.4.0", "7.4.0"]], "type": "int"},
                    "vrip": {"v_range": [["7.4.0", "7.4.0"]], "type": "str"}
                },
                "elements": "dict"
            }
        }
    },
    "fsp_vlan_dynamicmapping_interface_ipv6": {
        "stated": false,
        "options": {
            "autoconf": {"v_range": [["6.2.2", "7.4.0"]], "choices": ["disable", "enable"], "type": "str"},
            "dhcp6-client-options": {
                "v_range": [["6.2.2", "7.4.0"]],
                "type": "list",
                "choices": ["rapid", "iapd", "iana", "dns", "dnsname"],
                "elements": "str"
            },
            "dhcp6-information-request": {"v_range": [["6.2.2", "7.4.0"]], "choices": ["disable", "enable"], "type": "str"},
            "dhcp6-prefix-delegation": {"v_range": [["6.2.2", "7.4.0"]], "choices": ["disable", "enable"], "type": "str"},
            "dhcp6-prefix-hint": {"v_range": [["6.2.2", "7.4.0"]], "type": "str"},
            "dhcp6-prefix-hint-plt": {"v_range": [["6.2.2", "7.4.0"]], "type": "int"},
            "dhcp6-prefix-hint-vlt": {"v_range": [["6.2.2", "7.4.0"]], "type": "int"},
            "dhcp6-relay-ip": {"v_range": [["6.2.2", "7.4.0"]], "type": "str"},
            "dhcp6-relay-service": {"v_range": [["6.2.2", "7.4.0"]], "choices": ["disable", "enable"], "type": "str"},
            "dhcp6-relay-type": {"v_range": [["6.2.2", "7.4.0"]], "choices": ["regular"], "type": "str"},
            "ip6-address": {"v_range": [["6.2.2", "7.4.0"]], "type": "str"},
            "ip6-allowaccess": {
                "v_range": [["6.2.2", "7.4.0"]],
                "type": "list",
                "choices": ["https", "ping", "ssh", "snmp", "http", "telnet", "fgfm", "capwap", "fabric"],
                "elements": "str"
            },
            "ip6-default-life": {"v_range": [["6.2.2", "7.4.0"]], "type": "int"},
            "ip6-delegated-prefix-list": {
                "v_range": [["6.2.2", "7.4.0"]],
                "type": "list",
                "options": {
                    "autonomous-flag": {"v_range": [["6.2.2", "7.4.0"]], "choices": ["disable", "enable"], "type": "str"},
                    "onlink-flag": {"v_range": [["6.2.2", "7.4.0"]], "choices": ["disable", "enable"], "type": "str"},
                    "prefix-id": {"v_range": [["6.2.2", "7.4.0"]], "type": "int"},
                    "rdnss": {"v_range": [["6.2.2", "7.4.0"]], "type": "raw"},
                    "rdnss-service": {"v_range": [["6.2.2", "7.4.0"]], "choices": ["delegated", "default", "specify"], "type": "str"},
                    "subnet": {"v_range": [["6.2.2", "7.4.0"]], "type": "str"},
                    "upstream-interface": {"v_range": [["6.2.2", "7.4.0"]], "type": "str"},
                    "delegated-prefix-iaid": {"v_range": [["7.0.2", "7.4.0"]], "type": "int"}
                },
                "elements": "dict"
            },
            "ip6-dns-server-override": {"v_range": [["6.2.2", "7.4.0"]], "choices": ["disable", "enable"], "type": "str"},
            "ip6-extra-addr": {
                "v_range": [["6.2.2", "7.4.0"]],
                "type": "list",
                "options": {"prefix": {"v_range": [["6.2.2", "7.4.0"]], "type": "str"}},
                "elements": "dict"
            },
            "ip6-hop-limit": {"v_range": [["6.2.2", "7.4.0"]], "type": "int"},
            "ip6-link-mtu": {"v_range": [["6.2.2", "7.4.0"]], "type": "int"},
            "ip6-manage-flag": {"v_range": [["6.2.2", "7.4.0"]], "choices": ["disable", "enable"], "type": "str"},
            "ip6-max-interval": {"v_range": [["6.2.2", "7.4.0"]], "type": "int"},
            "ip6-min-interval": {"v_range": [["6.2.2", "7.4.0"]], "type": "int"},
            "ip6-mode": {"v_range": [["6.2.2", "7.4.0"]], "choices": ["static", "dhcp", "pppoe", "delegated"], "type": "str"},
            "ip6-other-flag": {"v_range": [["6.2.2", "7.4.0"]], "choices": ["disable", "enable"], "type": "str"},
            "ip6-prefix-list": {
                "v_range": [["6.2.2", "7.4.0"]],
                "type": "list",
                "options": {
                    "autonomous-flag": {"v_range": [["6.2.2", "7.4.0"]], "choices": ["disable", "enable"], "type": "str"},
                    "dnssl": {"v_range": [["6.2.2", "7.4.0"]], "type": "raw"},
                    "onlink-flag": {"v_range": [["6.2.2", "7.4.0"]], "choices": ["disable", "enable"], "type": "str"},
                    "preferred-life-time": {"v_range": [["6.2.2", "7.4.0"]], "type": "int"},
                    "prefix": {"v_range": [["6.2.2", "7.4.0"]], "type": "str"},
                    "rdnss": {"v_range": [["6.2.2", "7.4.0"]], "type": "raw"},
                    "valid-life-time": {"v_range": [["6.2.2", "7.4.0"]], "type": "int"}
                },
                "elements": "dict"
            },
            "ip6-reachable-time": {"v_range": [["6.2.2", "7.4.0"]], "type": "int"},
            "ip6-retrans-time": {"v_range": [["6.2.2", "7.4.0"]], "type": "int"},
            "ip6-send-adv": {"v_range": [["6.2.2", "7.4.0"]], "choices": ["disable", "enable"], "type": "str"},
            "ip6-subnet": {"v_range": [["6.2.2", "7.4.0"]], "type": "str"},
            "ip6-upstream-interface": {"v_range": [["6.2.2", "7.4.0"]], "type": "str"},
            "nd-cert": {"v_range": [["6.2.2", "7.4.0"]], "type": "str"},
            "nd-cga-modifier": {"v_range": [["6.2.2", "7.4.0"]], "type": "str"},
            "nd-mode": {"v_range": [["6.2.2", "7.4.0"]], "choices": ["basic", "SEND-compatible"], "type": "str"},
            "nd-security-level": {"v_range": [["6.2.2", "7.4.0"]], "type": "int"},
            "nd-timestamp-delta": {"v_range": [["6.2.2", "7.4.0"]], "type": "int"},
            "nd-timestamp-fuzz": {"v_range": [["6.2.2", "7.4.0"]], "type": "int"},
            "vrip6_link_local": {"v_range": [["6.2.2", "7.4.0"]], "type": "str"},
            "vrrp-virtual-mac6": {"v_range": [["6.2.2", "7.4.0"]], "choices": ["disable", "enable"], "type": "str"},
            "vrrp6": {
                "v_range": [["6.2.2", "7.4.0"]],
                "type": "list",
                "options": {
                    "accept-mode": {"v_range": [["6.2.2", "7.4.0"]], "choices": ["disable", "enable"], "type": "str"},
                    "adv-interval": {"v_range": [["6.2.2", "7.4.0"]], "type": "int"},
                    "preempt": {"v_range": [["6.2.2", "7.4.0"]], "choices": ["disable", "enable"], "type": "str"},
                    "priority": {"v_range": [["6.2.2", "7.4.0"]], "type": "int"},
                    "start-time": {"v_range": [["6.2.2", "7.4.0"]], "type": "int"},
                    "status": {"v_range": [["6.2.2", "7.4.0"]], "choices": ["disable", "enable"], "type": "str"},
                    "vrdst6": {"v_range": [["6.2.2", "7.4.0"]], "type": "str"},
                    "vrgrp": {"v_range": [["6.2.2", "7.4.0"]], "type": "int"},
                    "vrid": {"v_range": [["6.2.2", "7.4.0"]], "type": "int"},
                    "vrip6": {"v_range": [["6.2.2", "7.4.0"]], "type": "str"}
                },
                "elements": "dict"
            },
            "interface-identifier": {"v_range": [["6.4.1", "7.4.0"]], "type": "str"},
            "unique-autoconf-addr": {"v_range": [["6.4.1", "7.4.0"]], "choices": ["disable", "enable"], "type": "str"},
            "icmp6-send-redirect": {"v_range": [["6.4.4", "7.4.0"]], "choices": ["disable", "enable"], "type": "str"},
            "cli-conn6-status": {"v_range": [["7.0.0", "7.4.0"]], "type": "int"},
            "ip6-prefix-mode": {"v_range": [["7.0.0", "7.4.0"]], "choices": ["dhcp6", "ra"], "type": "str"},
            "ra-send-mtu": {"v_range": [["6.4.6", "7.4.0"]], "choices": ["disable", "enable"], "type": "str"},
            "ip6-delegated-prefix-iaid": {"v_range": [["7.0.2", "7.4.0"]], "type": "int"},
            "dhcp6-relay-source-interface": {"v_range": [["7.2.2", "7.4.0"]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "fsp_vlan_dynamicmapping_interface_ipv6_ip6delegatedprefixlist": {
        "stated": true,
        "options": {
            "autonomous-flag": {"v_range": [["6.2.2", "7.4.0"]], "choices": ["disable", "enable"], "type": "str"},
            "onlink-flag": {"v_range": [["6.2.2", "7.4.0"]], "choices": ["disable", "enable"], "type": "str"},
            "prefix-id": {"v_range": [["6.2.2", "7.4.0"]], "type": "int"},
            "rdnss": {"v_range": [["6.2.2", "7.4.0"]], "type": "raw"},
            "rdnss-service": {"v_range": [["6.2.2", "7.4.0"]], "choices": ["delegated", "default", "specify"], "type": "str"},
            "subnet": {"v_range": [["6.2.2", "7.4.0"]], "type": "str"},
            "upstream-interface": {"v_range": [["6.2.2", "7.4.0"]], "type": "str"},
            "delegated-prefix-iaid": {"v_range": [["7.0.2", "7.4.0"]], "type": "int"}
        }
    },
    "fsp_vlan_dynamicmapping_interface_ipv6_ip6extraaddr": {"stated": true, "options": {"prefix": {"v_range": [["6.2.2", "7.4.0"]], "type": "str"}}},
    "fsp_vlan_dynamicmapping_interface_ipv6_ip6prefixlist": {
        "stated": true,
        "options": {
            "autonomous-flag": {"v_range": [["6.2.2", "7.4.0"]], "choices": ["disable", "enable"], "type": "str"},
            "dnssl": {"v_range": [["6.2.2", "7.4.0"]], "type": "raw"},
            "onlink-flag": {"v_range": [["6.2.2", "7.4.0"]], "choices": ["disable", "enable"], "type": "str"},
            "preferred-life-time": {"v_range": [["6.2.2", "7.4.0"]], "type": "int"},
            "prefix": {"v_range": [["6.2.2", "7.4.0"]], "type": "str"},
            "rdnss": {"v_range": [["6.2.2", "7.4.0"]], "type": "raw"},
            "valid-life-time": {"v_range": [["6.2.2", "7.4.0"]], "type": "int"}
        }
    },
    "fsp_vlan_dynamicmapping_interface_ipv6_vrrp6": {
        "stated": true,
        "options": {
            "accept-mode": {"v_range": [["6.2.2", "7.4.0"]], "choices": ["disable", "enable"], "type": "str"},
            "adv-interval": {"v_range": [["6.2.2", "7.4.0"]], "type": "int"},
            "preempt": {"v_range": [["6.2.2", "7.4.0"]], "choices": ["disable", "enable"], "type": "str"},
            "priority": {"v_range": [["6.2.2", "7.4.0"]], "type": "int"},
            "start-time": {"v_range": [["6.2.2", "7.4.0"]], "type": "int"},
            "status": {"v_range": [["6.2.2", "7.4.0"]], "choices": ["disable", "enable"], "type": "str"},
            "vrdst6": {"v_range": [["6.2.2", "7.4.0"]], "type": "str"},
            "vrgrp": {"v_range": [["6.2.2", "7.4.0"]], "type": "int"},
            "vrid": {"v_range": [["6.2.2", "7.4.0"]], "type": "int"},
            "vrip6": {"v_range": [["6.2.2", "7.4.0"]], "type": "str"}
        }
    },
    "fsp_vlan_dynamicmapping_interface_secondaryip": {
        "stated": true,
        "options": {
            "allowaccess": {
                "v_range": [["6.2.3", "7.4.0"]],
                "type": "list",
                "choices": [
                    "https", "ping", "ssh", "snmp", "http", "telnet", "fgfm", "auto-ipsec", "radius-acct", "probe-response", "capwap", "dnp", "ftm",
                    "fabric", "speed-test"
                ],
                "elements": "str"
            },
            "detectprotocol": {"v_range": [["6.2.3", "7.2.0"]], "type": "list", "choices": ["ping", "tcp-echo", "udp-echo"], "elements": "str"},
            "detectserver": {"v_range": [["6.2.3", "7.2.0"]], "type": "str"},
            "gwdetect": {"v_range": [["6.2.3", "7.2.0"]], "choices": ["disable", "enable"], "type": "str"},
            "ha-priority": {"v_range": [["6.2.3", "7.2.0"]], "type": "int"},
            "id": {"v_range": [["6.2.3", "7.4.0"]], "required": true, "type": "int"},
            "ip": {"v_range": [["6.2.3", "7.4.0"]], "type": "str"},
            "ping-serv-status": {"v_range": [["6.2.3", "7.2.0"]], "type": "int"},
            "seq": {"v_range": [["6.2.3", "7.4.0"]], "type": "int"},
            "secip-relay-ip": {"v_range": [["7.4.0", "7.4.0"]], "type": "str"}
        }
    },
    "fsp_vlan_dynamicmapping_interface_vrrp": {
        "stated": true,
        "options": {
            "accept-mode": {"v_range": [["7.4.0", "7.4.0"]], "choices": ["disable", "enable"], "type": "str"},
            "adv-interval": {"v_range": [["7.4.0", "7.4.0"]], "type": "int"},
            "ignore-default-route": {"v_range": [["7.4.0", "7.4.0"]], "choices": ["disable", "enable"], "type": "str"},
            "preempt": {"v_range": [["7.4.0", "7.4.0"]], "choices": ["disable", "enable"], "type": "str"},
            "priority": {"v_range": [["7.4.0", "7.4.0"]], "type": "int"},
            "proxy-arp": {
                "v_range": [["7.4.0", "7.4.0"]],
                "type": "list",
                "options": {"id": {"v_range": [["7.4.0", "7.4.0"]], "type": "int"}, "ip": {"v_range": [["7.4.0", "7.4.0"]], "type": "str"}},
                "elements": "dict"
            },
            "start-time": {"v_range": [["7.4.0", "7.4.0"]], "type": "int"},
            "status": {"v_range": [["7.4.0", "7.4.0"]], "choices": ["disable", "enable"], "type": "str"},
            "version": {"v_range": [["7.4.0", "7.4.0"]], "choices": ["2", "3"], "type": "str"},
            "vrdst": {"v_range": [["7.4.0", "7.4.0"]], "type": "raw"},
            "vrdst-priority": {"v_range": [["7.4.0", "7.4.0"]], "type": "int"},
            "vrgrp": {"v_range": [["7.4.0", "7.4.0"]], "type": "int"},
            "vrid": {"v_range": [["7.4.0", "7.4.0"]], "type": "int"},
            "vrip": {"v_range": [["7.4.0", "7.4.0"]], "type": "str"}
        }
    },
    "fsp_vlan_dynamicmapping_interface_vrrp_proxyarp": {
        "stated": true,
        "options": {"id": {"v_range": [["7.4.0", "7.4.0"]], "required": true, "type": "int"}, "ip": {"v_range": [["7.4.0", "7.4.0"]], "type": "str"}}
    },
    "fsp_vlan_interface": {
        "stated": false,
        "options": {
            "ac-name": {"type": "str"},
            "aggregate": {"type": "str"},
            "algorithm": {"choices": ["L2", "L3", "L4", "LB", "Source-MAC"], "type": "str"},
            "alias": {"type": "str"},
            "allowaccess": {
                "type": "list",
                "choices": [
                    "https", "ping", "ssh", "snmp", "http", "telnet", "fgfm", "auto-ipsec", "radius-acct", "probe-response", "capwap", "dnp", "ftm",
                    "fabric", "speed-test"
                ],
                "elements": "str"
            },
            "ap-discover": {"choices": ["disable", "enable"], "type": "str"},
            "arpforward": {"choices": ["disable", "enable"], "type": "str"},
            "atm-protocol": {"choices": ["none", "ipoa"], "type": "str"},
            "auth-type": {"choices": ["auto", "pap", "chap", "mschapv1", "mschapv2"], "type": "str"},
            "auto-auth-extension-device": {"choices": ["disable", "enable"], "type": "str"},
            "bfd": {"choices": ["global", "enable", "disable"], "type": "str"},
            "bfd-desired-min-tx": {"type": "int"},
            "bfd-detect-mult": {"type": "int"},
            "bfd-required-min-rx": {"type": "int"},
            "broadcast-forticlient-discovery": {"choices": ["disable", "enable"], "type": "str"},
            "broadcast-forward": {"choices": ["disable", "enable"], "type": "str"},
            "captive-portal": {"type": "int"},
            "cli-conn-status": {"type": "int"},
            "color": {"type": "int"},
            "ddns": {"choices": ["disable", "enable"], "type": "str"},
            "ddns-auth": {"choices": ["disable", "tsig"], "type": "str"},
            "ddns-domain": {"type": "str"},
            "ddns-key": {"no_log": true, "type": "raw"},
            "ddns-keyname": {"no_log": true, "type": "str"},
            "ddns-password": {"no_log": true, "type": "raw"},
            "ddns-server": {
                "choices": [
                    "dhs.org", "dyndns.org", "dyns.net", "tzo.com", "ods.org", "vavic.com", "now.net.cn", "dipdns.net", "easydns.com", "genericDDNS"
                ],
                "type": "str"
            },
            "ddns-server-ip": {"type": "str"},
            "ddns-sn": {"type": "str"},
            "ddns-ttl": {"type": "int"},
            "ddns-username": {"type": "str"},
            "ddns-zone": {"type": "str"},
            "dedicated-to": {"choices": ["none", "management"], "type": "str"},
            "defaultgw": {"choices": ["disable", "enable"], "type": "str"},
            "description": {"type": "str"},
            "detected-peer-mtu": {"type": "int"},
            "detectprotocol": {"type": "list", "choices": ["ping", "tcp-echo", "udp-echo"], "elements": "str"},
            "detectserver": {"type": "str"},
            "device-access-list": {"type": "raw"},
            "device-identification": {"choices": ["disable", "enable"], "type": "str"},
            "device-identification-active-scan": {"choices": ["disable", "enable"], "type": "str"},
            "device-netscan": {"choices": ["disable", "enable"], "type": "str"},
            "device-user-identification": {"choices": ["disable", "enable"], "type": "str"},
            "devindex": {"type": "int"},
            "dhcp-client-identifier": {"type": "str"},
            "dhcp-relay-agent-option": {"choices": ["disable", "enable"], "type": "str"},
            "dhcp-relay-ip": {"type": "raw"},
            "dhcp-relay-service": {"choices": ["disable", "enable"], "type": "str"},
            "dhcp-relay-type": {"choices": ["regular", "ipsec"], "type": "str"},
            "dhcp-renew-time": {"type": "int"},
            "disc-retry-timeout": {"type": "int"},
            "disconnect-threshold": {"type": "int"},
            "distance": {"type": "int"},
            "dns-query": {"choices": ["disable", "recursive", "non-recursive"], "type": "str"},
            "dns-server-override": {"choices": ["disable", "enable"], "type": "str"},
            "drop-fragment": {"choices": ["disable", "enable"], "type": "str"},
            "drop-overlapped-fragment": {"choices": ["disable", "enable"], "type": "str"},
            "egress-cos": {"choices": ["disable", "cos0", "cos1", "cos2", "cos3", "cos4", "cos5", "cos6", "cos7"], "type": "str"},
            "egress-shaping-profile": {"type": "str"},
            "endpoint-compliance": {"choices": ["disable", "enable"], "type": "str"},
            "estimated-downstream-bandwidth": {"type": "int"},
            "estimated-upstream-bandwidth": {"type": "int"},
            "explicit-ftp-proxy": {"choices": ["disable", "enable"], "type": "str"},
            "explicit-web-proxy": {"choices": ["disable", "enable"], "type": "str"},
            "external": {"choices": ["disable", "enable"], "type": "str"},
            "fail-action-on-extender": {"choices": ["soft-restart", "hard-restart", "reboot"], "type": "str"},
            "fail-alert-interfaces": {"type": "raw"},
            "fail-alert-method": {"choices": ["link-failed-signal", "link-down"], "type": "str"},
            "fail-detect": {"choices": ["disable", "enable"], "type": "str"},
            "fail-detect-option": {"type": "list", "choices": ["detectserver", "link-down"], "elements": "str"},
            "fdp": {"choices": ["disable", "enable"], "type": "str"},
            "fortiheartbeat": {"choices": ["disable", "enable"], "type": "str"},
            "fortilink": {"choices": ["disable", "enable"], "type": "str"},
            "fortilink-backup-link": {"type": "int"},
            "fortilink-split-interface": {"choices": ["disable", "enable"], "type": "str"},
            "fortilink-stacking": {"choices": ["disable", "enable"], "type": "str"},
            "forward-domain": {"type": "int"},
            "forward-error-correction": {
                "choices": ["disable", "enable", "rs-fec", "base-r-fec", "fec-cl91", "fec-cl74", "rs-544", "none", "cl91-rs-fec", "cl74-fc-fec", "auto"],
                "type": "str"
            },
            "fp-anomaly": {
                "type": "list",
                "choices": [
                    "drop_tcp_fin_noack", "pass_winnuke", "pass_tcpland", "pass_udpland", "pass_icmpland", "pass_ipland", "pass_iprr", "pass_ipssrr",
                    "pass_iplsrr", "pass_ipstream", "pass_ipsecurity", "pass_iptimestamp", "pass_ipunknown_option", "pass_ipunknown_prot",
                    "pass_icmp_frag", "pass_tcp_no_flag", "pass_tcp_fin_noack", "drop_winnuke", "drop_tcpland", "drop_udpland", "drop_icmpland",
                    "drop_ipland", "drop_iprr", "drop_ipssrr", "drop_iplsrr", "drop_ipstream", "drop_ipsecurity", "drop_iptimestamp",
                    "drop_ipunknown_option", "drop_ipunknown_prot", "drop_icmp_frag", "drop_tcp_no_flag"
                ],
                "elements": "str"
            },
            "fp-disable": {"type": "list", "choices": ["all", "ipsec", "none"], "elements": "str"},
            "gateway-address": {"type": "str"},
            "gi-gk": {"choices": ["disable", "enable"], "type": "str"},
            "gwaddr": {"v_range": [["6.0.0", "7.2.0"]], "type": "str"},
            "gwdetect": {"choices": ["disable", "enable"], "type": "str"},
            "ha-priority": {"type": "int"},
            "icmp-accept-redirect": {"choices": ["disable", "enable"], "type": "str"},
            "icmp-redirect": {"choices": ["disable", "enable"], "type": "str"},
            "icmp-send-redirect": {"choices": ["disable", "enable"], "type": "str"},
            "ident-accept": {"choices": ["disable", "enable"], "type": "str"},
            "idle-timeout": {"type": "int"},
            "if-mdix": {"choices": ["auto", "normal", "crossover"], "type": "str"},
            "if-media": {"choices": ["auto", "copper", "fiber"], "type": "str"},
            "in-force-vlan-cos": {"type": "int"},
            "inbandwidth": {"type": "int"},
            "ingress-cos": {"choices": ["disable", "cos0", "cos1", "cos2", "cos3", "cos4", "cos5", "cos6", "cos7"], "type": "str"},
            "ingress-spillover-threshold": {"type": "int"},
            "internal": {"type": "int"},
            "ip": {"type": "str"},
            "ipmac": {"choices": ["disable", "enable"], "type": "str"},
            "ips-sniffer-mode": {"choices": ["disable", "enable"], "type": "str"},
            "ipunnumbered": {"type": "str"},
            "ipv6": {
                "type": "dict",
                "options": {
                    "autoconf": {"choices": ["disable", "enable"], "type": "str"},
                    "dhcp6-client-options": {"type": "list", "choices": ["rapid", "iapd", "iana", "dns", "dnsname"], "elements": "str"},
                    "dhcp6-information-request": {"choices": ["disable", "enable"], "type": "str"},
                    "dhcp6-prefix-delegation": {"choices": ["disable", "enable"], "type": "str"},
                    "dhcp6-prefix-hint": {"type": "str"},
                    "dhcp6-prefix-hint-plt": {"type": "int"},
                    "dhcp6-prefix-hint-vlt": {"type": "int"},
                    "dhcp6-relay-ip": {"type": "str"},
                    "dhcp6-relay-service": {"choices": ["disable", "enable"], "type": "str"},
                    "dhcp6-relay-type": {"choices": ["regular"], "type": "str"},
                    "ip6-address": {"type": "str"},
                    "ip6-allowaccess": {
                        "type": "list",
                        "choices": ["https", "ping", "ssh", "snmp", "http", "telnet", "fgfm", "capwap", "fabric"],
                        "elements": "str"
                    },
                    "ip6-default-life": {"type": "int"},
                    "ip6-dns-server-override": {"choices": ["disable", "enable"], "type": "str"},
                    "ip6-hop-limit": {"type": "int"},
                    "ip6-link-mtu": {"type": "int"},
                    "ip6-manage-flag": {"choices": ["disable", "enable"], "type": "str"},
                    "ip6-max-interval": {"type": "int"},
                    "ip6-min-interval": {"type": "int"},
                    "ip6-mode": {"choices": ["static", "dhcp", "pppoe", "delegated"], "type": "str"},
                    "ip6-other-flag": {"choices": ["disable", "enable"], "type": "str"},
                    "ip6-reachable-time": {"type": "int"},
                    "ip6-retrans-time": {"type": "int"},
                    "ip6-send-adv": {"choices": ["disable", "enable"], "type": "str"},
                    "ip6-subnet": {"type": "str"},
                    "ip6-upstream-interface": {"type": "str"},
                    "nd-cert": {"type": "str"},
                    "nd-cga-modifier": {"type": "str"},
                    "nd-mode": {"choices": ["basic", "SEND-compatible"], "type": "str"},
                    "nd-security-level": {"type": "int"},
                    "nd-timestamp-delta": {"type": "int"},
                    "nd-timestamp-fuzz": {"type": "int"},
                    "vrip6_link_local": {"type": "str"},
                    "vrrp-virtual-mac6": {"choices": ["disable", "enable"], "type": "str"},
                    "ip6-delegated-prefix-list": {
                        "v_range": [["6.2.2", ""]],
                        "type": "list",
                        "options": {
                            "autonomous-flag": {"v_range": [["6.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "onlink-flag": {"v_range": [["6.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "prefix-id": {"v_range": [["6.2.2", ""]], "type": "int"},
                            "rdnss": {"v_range": [["6.2.2", ""]], "type": "raw"},
                            "rdnss-service": {"v_range": [["6.2.2", ""]], "choices": ["delegated", "default", "specify"], "type": "str"},
                            "subnet": {"v_range": [["6.2.2", ""]], "type": "str"},
                            "upstream-interface": {"v_range": [["6.2.2", ""]], "type": "str"},
                            "delegated-prefix-iaid": {"v_range": [["7.0.2", ""]], "type": "int"}
                        },
                        "elements": "dict"
                    },
                    "ip6-extra-addr": {
                        "v_range": [["6.2.2", ""]],
                        "type": "list",
                        "options": {"prefix": {"v_range": [["6.2.2", ""]], "type": "str"}},
                        "elements": "dict"
                    },
                    "ip6-prefix-list": {
                        "v_range": [["6.2.2", ""]],
                        "type": "list",
                        "options": {
                            "autonomous-flag": {"v_range": [["6.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "dnssl": {"v_range": [["6.2.2", ""]], "type": "raw"},
                            "onlink-flag": {"v_range": [["6.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "preferred-life-time": {"v_range": [["6.2.2", ""]], "type": "int"},
                            "prefix": {"v_range": [["6.2.2", ""]], "type": "str"},
                            "rdnss": {"v_range": [["6.2.2", ""]], "type": "raw"},
                            "valid-life-time": {"v_range": [["6.2.2", ""]], "type": "int"}
                        },
                        "elements": "dict"
                    },
                    "vrrp6": {
                        "v_range": [["6.2.2", ""]],
                        "type": "list",
                        "options": {
                            "accept-mode": {"v_range": [["6.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "adv-interval": {"v_range": [["6.2.2", ""]], "type": "int"},
                            "preempt": {"v_range": [["6.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "priority": {"v_range": [["6.2.2", ""]], "type": "int"},
                            "start-time": {"v_range": [["6.2.2", ""]], "type": "int"},
                            "status": {"v_range": [["6.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "vrdst6": {"v_range": [["6.2.2", ""]], "type": "str"},
                            "vrgrp": {"v_range": [["6.2.2", ""]], "type": "int"},
                            "vrid": {"v_range": [["6.2.2", ""]], "type": "int"},
                            "vrip6": {"v_range": [["6.2.2", ""]], "type": "str"},
                            "ignore-default-route": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "vrdst-priority": {"v_range": [["7.6.0", ""]], "type": "int"}
                        },
                        "elements": "dict"
                    },
                    "interface-identifier": {"v_range": [["6.4.1", ""]], "type": "str"},
                    "unique-autoconf-addr": {"v_range": [["6.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "icmp6-send-redirect": {"v_range": [["6.4.4", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "cli-conn6-status": {"v_range": [["7.0.0", ""]], "type": "int"},
                    "ip6-prefix-mode": {"v_range": [["7.0.0", ""]], "choices": ["dhcp6", "ra"], "type": "str"},
                    "ra-send-mtu": {"v_range": [["6.4.6", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "ip6-delegated-prefix-iaid": {"v_range": [["7.0.2", ""]], "type": "int"},
                    "dhcp6-relay-source-interface": {"v_range": [["7.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "dhcp6-relay-interface-id": {"v_range": [["7.4.1", ""]], "type": "str"},
                    "dhcp6-relay-source-ip": {"v_range": [["7.4.1", ""]], "type": "str"}
                }
            },
            "l2forward": {"choices": ["disable", "enable"], "type": "str"},
            "l2tp-client": {"choices": ["disable", "enable"], "type": "str"},
            "lacp-ha-slave": {"choices": ["disable", "enable"], "type": "str"},
            "lacp-mode": {"choices": ["static", "passive", "active"], "type": "str"},
            "lacp-speed": {"choices": ["slow", "fast"], "type": "str"},
            "lcp-echo-interval": {"type": "int"},
            "lcp-max-echo-fails": {"type": "int"},
            "link-up-delay": {"type": "int"},
            "listen-forticlient-connection": {"choices": ["disable", "enable"], "type": "str"},
            "lldp-network-policy": {"type": "str"},
            "lldp-reception": {"choices": ["disable", "enable", "vdom"], "type": "str"},
            "lldp-transmission": {"choices": ["enable", "disable", "vdom"], "type": "str"},
            "log": {"choices": ["disable", "enable"], "type": "str"},
            "macaddr": {"type": "str"},
            "management-ip": {"type": "str"},
            "max-egress-burst-rate": {"type": "int"},
            "max-egress-rate": {"type": "int"},
            "mediatype": {
                "choices": [
                    "serdes-sfp", "sgmii-sfp", "cfp2-sr10", "cfp2-lr4", "serdes-copper-sfp", "sr", "cr", "lr", "qsfp28-sr4", "qsfp28-lr4", "qsfp28-cr4",
                    "sr4", "cr4", "lr4", "none", "gmii", "sgmii", "sr2", "lr2", "cr2", "sr8", "lr8", "cr8"
                ],
                "type": "str"
            },
            "member": {"type": "raw"},
            "min-links": {"type": "int"},
            "min-links-down": {"choices": ["operational", "administrative"], "type": "str"},
            "mode": {"choices": ["static", "dhcp", "pppoe", "pppoa", "ipoa", "eoa"], "type": "str"},
            "mtu": {"type": "int"},
            "mtu-override": {"choices": ["disable", "enable"], "type": "str"},
            "mux-type": {"choices": ["llc-encaps", "vc-encaps"], "type": "str"},
            "name": {"type": "str"},
            "ndiscforward": {"choices": ["disable", "enable"], "type": "str"},
            "netbios-forward": {"choices": ["disable", "enable"], "type": "str"},
            "netflow-sampler": {"choices": ["disable", "tx", "rx", "both"], "type": "str"},
            "npu-fastpath": {"choices": ["disable", "enable"], "type": "str"},
            "nst": {"choices": ["disable", "enable"], "type": "str"},
            "out-force-vlan-cos": {"type": "int"},
            "outbandwidth": {"type": "int"},
            "padt-retry-timeout": {"type": "int"},
            "password": {"no_log": true, "type": "raw"},
            "peer-interface": {"type": "raw"},
            "phy-mode": {
                "choices": ["auto", "adsl", "vdsl", "adsl-auto", "vdsl2", "adsl2+", "adsl2", "g.dmt", "t1.413", "g.lite", "g-dmt", "t1-413", "g-lite"],
                "type": "str"
            },
            "ping-serv-status": {"v_range": [["6.0.0", "7.2.0"]], "type": "int"},
            "poe": {"choices": ["disable", "enable"], "type": "str"},
            "polling-interval": {"type": "int"},
            "pppoe-unnumbered-negotiate": {"choices": ["disable", "enable"], "type": "str"},
            "pptp-auth-type": {"choices": ["auto", "pap", "chap", "mschapv1", "mschapv2"], "type": "str"},
            "pptp-client": {"choices": ["disable", "enable"], "type": "str"},
            "pptp-password": {"no_log": true, "type": "raw"},
            "pptp-server-ip": {"type": "str"},
            "pptp-timeout": {"type": "int"},
            "pptp-user": {"type": "str"},
            "preserve-session-route": {"choices": ["disable", "enable"], "type": "str"},
            "priority": {"type": "int"},
            "priority-override": {"choices": ["disable", "enable"], "type": "str"},
            "proxy-captive-portal": {"choices": ["disable", "enable"], "type": "str"},
            "redundant-interface": {"type": "str"},
            "remote-ip": {"type": "str"},
            "replacemsg-override-group": {"type": "str"},
            "retransmission": {"choices": ["disable", "enable"], "type": "str"},
            "role": {"choices": ["lan", "wan", "dmz", "undefined"], "type": "str"},
            "sample-direction": {"choices": ["rx", "tx", "both"], "type": "str"},
            "sample-rate": {"type": "int"},
            "scan-botnet-connections": {"choices": ["disable", "block", "monitor"], "type": "str"},
            "secondary-IP": {"choices": ["disable", "enable"], "type": "str"},
            "secondaryip": {
                "type": "list",
                "options": {
                    "allowaccess": {
                        "type": "list",
                        "choices": [
                            "https", "ping", "ssh", "snmp", "http", "telnet", "fgfm", "auto-ipsec", "radius-acct", "probe-response", "capwap", "dnp",
                            "ftm", "fabric", "speed-test", "icond", "scim"
                        ],
                        "elements": "str"
                    },
                    "detectprotocol": {"v_range": [["6.0.0", "7.2.0"]], "type": "list", "choices": ["ping", "tcp-echo", "udp-echo"], "elements": "str"},
                    "detectserver": {"v_range": [["6.0.0", "7.2.0"]], "type": "str"},
                    "gwdetect": {"v_range": [["6.0.0", "7.2.0"]], "choices": ["disable", "enable"], "type": "str"},
                    "ha-priority": {"v_range": [["6.0.0", "7.2.0"]], "type": "int"},
                    "id": {"type": "int"},
                    "ip": {"type": "str"},
                    "ping-serv-status": {"v_range": [["6.0.0", "7.2.0"]], "type": "int"},
                    "seq": {"type": "int"},
                    "secip-relay-ip": {"v_range": [["7.4.0", ""]], "type": "str"}
                },
                "elements": "dict"
            },
            "security-8021x-dynamic-vlan-id": {"type": "int"},
            "security-8021x-master": {"type": "str"},
            "security-8021x-mode": {"choices": ["default", "dynamic-vlan", "fallback", "slave"], "type": "str"},
            "security-exempt-list": {"type": "str"},
            "security-external-logout": {"type": "str"},
            "security-external-web": {"type": "str"},
            "security-groups": {"type": "raw"},
            "security-mac-auth-bypass": {"choices": ["disable", "enable", "mac-auth-only"], "type": "str"},
            "security-mode": {"choices": ["none", "captive-portal", "802.1X"], "type": "str"},
            "security-redirect-url": {"type": "str"},
            "service-name": {"type": "str"},
            "sflow-sampler": {"choices": ["disable", "enable"], "type": "str"},
            "speed": {
                "choices": [
                    "auto", "10full", "10half", "100full", "100half", "1000full", "1000half", "10000full", "1000auto", "10000auto", "40000full",
                    "100Gfull", "25000full", "40000auto", "25000auto", "100Gauto", "400Gfull", "400Gauto", "50000full", "2500auto", "5000auto",
                    "50000auto", "200Gfull", "200Gauto", "100auto"
                ],
                "type": "str"
            },
            "spillover-threshold": {"type": "int"},
            "src-check": {"choices": ["disable", "enable"], "type": "str"},
            "status": {"choices": ["down", "up"], "type": "str"},
            "stp": {"choices": ["disable", "enable"], "type": "str"},
            "stp-ha-slave": {"choices": ["disable", "enable", "priority-adjust"], "type": "str"},
            "stpforward": {"choices": ["disable", "enable"], "type": "str"},
            "stpforward-mode": {"choices": ["rpl-all-ext-id", "rpl-bridge-ext-id", "rpl-nothing"], "type": "str"},
            "strip-priority-vlan-tag": {"choices": ["disable", "enable"], "type": "str"},
            "subst": {"choices": ["disable", "enable"], "type": "str"},
            "substitute-dst-mac": {"type": "str"},
            "switch": {"type": "str"},
            "switch-controller-access-vlan": {"choices": ["disable", "enable"], "type": "str"},
            "switch-controller-arp-inspection": {"choices": ["disable", "enable", "monitor"], "type": "str"},
            "switch-controller-auth": {"choices": ["radius", "usergroup"], "type": "str"},
            "switch-controller-dhcp-snooping": {"choices": ["disable", "enable"], "type": "str"},
            "switch-controller-dhcp-snooping-option82": {"choices": ["disable", "enable"], "type": "str"},
            "switch-controller-dhcp-snooping-verify-mac": {"choices": ["disable", "enable"], "type": "str"},
            "switch-controller-igmp-snooping": {"choices": ["disable", "enable"], "type": "str"},
            "switch-controller-learning-limit": {"type": "int"},
            "switch-controller-radius-server": {"type": "str"},
            "switch-controller-traffic-policy": {"type": "str"},
            "tc-mode": {"choices": ["ptm", "atm"], "type": "str"},
            "tcp-mss": {"type": "int"},
            "trunk": {"choices": ["disable", "enable"], "type": "str"},
            "trust-ip-1": {"type": "str"},
            "trust-ip-2": {"type": "str"},
            "trust-ip-3": {"type": "str"},
            "trust-ip6-1": {"type": "str"},
            "trust-ip6-2": {"type": "str"},
            "trust-ip6-3": {"type": "str"},
            "type": {
                "choices": [
                    "physical", "vlan", "aggregate", "redundant", "tunnel", "wireless", "vdom-link", "loopback", "switch", "hard-switch", "hdlc",
                    "vap-switch", "wl-mesh", "fortilink", "switch-vlan", "fctrl-trunk", "tdm", "fext-wan", "vxlan", "emac-vlan", "geneve", "ssl",
                    "lan-extension"
                ],
                "type": "str"
            },
            "username": {"type": "str"},
            "vci": {"type": "int"},
            "vectoring": {"choices": ["disable", "enable"], "type": "str"},
            "vindex": {"type": "int"},
            "vlanforward": {"choices": ["disable", "enable"], "type": "str"},
            "vlanid": {"type": "int"},
            "vpi": {"type": "int"},
            "vrf": {"type": "int"},
            "vrrp": {
                "type": "list",
                "options": {
                    "accept-mode": {"choices": ["disable", "enable"], "type": "str"},
                    "adv-interval": {"type": "int"},
                    "ignore-default-route": {"choices": ["disable", "enable"], "type": "str"},
                    "preempt": {"choices": ["disable", "enable"], "type": "str"},
                    "priority": {"type": "int"},
                    "start-time": {"type": "int"},
                    "status": {"choices": ["disable", "enable"], "type": "str"},
                    "version": {"choices": ["2", "3"], "type": "str"},
                    "vrdst": {"type": "raw"},
                    "vrdst-priority": {"type": "int"},
                    "vrgrp": {"type": "int"},
                    "vrid": {"type": "int"},
                    "vrip": {"type": "str"},
                    "proxy-arp": {
                        "v_range": [["7.4.0", ""]],
                        "type": "list",
                        "options": {"id": {"v_range": [["7.4.0", ""]], "type": "int"}, "ip": {"v_range": [["7.4.0", ""]], "type": "str"}},
                        "elements": "dict"
                    }
                },
                "elements": "dict"
            },
            "vrrp-virtual-mac": {"choices": ["disable", "enable"], "type": "str"},
            "wccp": {"choices": ["disable", "enable"], "type": "str"},
            "weight": {"type": "int"},
            "wifi-5g-threshold": {"type": "str"},
            "wifi-acl": {"choices": ["deny", "allow"], "type": "str"},
            "wifi-ap-band": {"choices": ["any", "5g-preferred", "5g-only"], "type": "str"},
            "wifi-auth": {"choices": ["PSK", "RADIUS", "radius", "usergroup"], "type": "str"},
            "wifi-auto-connect": {"choices": ["disable", "enable"], "type": "str"},
            "wifi-auto-save": {"choices": ["disable", "enable"], "type": "str"},
            "wifi-broadcast-ssid": {"choices": ["disable", "enable"], "type": "str"},
            "wifi-encrypt": {"choices": ["TKIP", "AES"], "type": "str"},
            "wifi-fragment-threshold": {"type": "int"},
            "wifi-key": {"no_log": true, "type": "raw"},
            "wifi-keyindex": {"no_log": true, "type": "int"},
            "wifi-mac-filter": {"choices": ["disable", "enable"], "type": "str"},
            "wifi-passphrase": {"no_log": true, "type": "raw"},
            "wifi-radius-server": {"type": "str"},
            "wifi-rts-threshold": {"type": "int"},
            "wifi-security": {
                "choices": [
                    "None", "WEP64", "wep64", "WEP128", "wep128", "WPA_PSK", "WPA_RADIUS", "WPA", "WPA2", "WPA2_AUTO", "open", "wpa-personal",
                    "wpa-enterprise", "wpa-only-personal", "wpa-only-enterprise", "wpa2-only-personal", "wpa2-only-enterprise"
                ],
                "type": "str"
            },
            "wifi-ssid": {"type": "str"},
            "wifi-usergroup": {"type": "str"},
            "wins-ip": {"type": "str"},
            "eip": {"v_range": [["6.2.1", ""]], "type": "str"},
            "fortilink-neighbor-detect": {"v_range": [["6.2.1", ""]], "choices": ["lldp", "fortilink"], "type": "str"},
            "ingress-shaping-profile": {"v_range": [["6.2.1", ""]], "type": "str"},
            "ring-rx": {"v_range": [["6.2.1", ""]], "type": "int"},
            "ring-tx": {"v_range": [["6.2.1", ""]], "type": "int"},
            "switch-controller-igmp-snooping-fast-leave": {"v_range": [["6.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "switch-controller-igmp-snooping-proxy": {"v_range": [["6.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "switch-controller-rspan-mode": {"v_range": [["6.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "bandwidth-measure-time": {"v_range": [["6.4.0", ""]], "type": "int"},
            "ip-managed-by-fortiipam": {"v_range": [["6.4.0", ""]], "choices": ["disable", "enable", "inherit-global"], "type": "str"},
            "managed-subnetwork-size": {
                "v_range": [["6.4.0", ""]],
                "choices": ["256", "512", "1024", "2048", "4096", "8192", "16384", "32768", "65536", "32", "64", "128"],
                "type": "str"
            },
            "measured-downstream-bandwidth": {"v_range": [["6.4.0", ""]], "type": "int"},
            "measured-upstream-bandwidth": {"v_range": [["6.4.0", ""]], "type": "int"},
            "monitor-bandwidth": {"v_range": [["6.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "swc-vlan": {"v_range": [["6.4.0", ""]], "type": "int"},
            "switch-controller-feature": {
                "v_range": [["6.4.0", ""]],
                "choices": ["none", "default-vlan", "quarantine", "sniffer", "voice", "camera", "rspan", "video", "nac", "nac-segment"],
                "type": "str"
            },
            "switch-controller-mgmt-vlan": {"v_range": [["6.4.0", ""]], "type": "int"},
            "switch-controller-nac": {"v_range": [["6.4.0", ""]], "type": "str"},
            "vlan-protocol": {"v_range": [["6.4.0", ""]], "choices": ["8021q", "8021ad"], "type": "str"},
            "dhcp-relay-interface": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "type": "str"},
            "dhcp-relay-interface-select-method": {
                "v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]],
                "choices": ["auto", "sdwan", "specify"],
                "type": "str"
            },
            "np-qos-profile": {"v_range": [["6.2.7", "6.2.12"], ["6.4.4", ""]], "type": "int"},
            "swc-first-create": {"v_range": [["6.4.3", ""]], "type": "int"},
            "switch-controller-iot-scanning": {"v_range": [["6.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "switch-controller-source-ip": {"v_range": [["6.4.3", ""]], "choices": ["outbound", "fixed"], "type": "str"},
            "dhcp-relay-request-all-server": {"v_range": [["6.2.8", "6.2.12"], ["6.4.6", ""]], "choices": ["disable", "enable"], "type": "str"},
            "stp-ha-secondary": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable", "priority-adjust"], "type": "str"},
            "switch-controller-dynamic": {"v_range": [["7.0.0", ""]], "type": "str"},
            "auth-cert": {"v_range": [["7.0.3", ""]], "type": "str"},
            "auth-portal-addr": {"v_range": [["7.0.3", ""]], "type": "str"},
            "dhcp-classless-route-addition": {"v_range": [["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "dhcp-relay-link-selection": {"v_range": [["7.0.3", ""]], "type": "str"},
            "dns-server-protocol": {"v_range": [["7.0.3", ""]], "type": "list", "choices": ["cleartext", "dot", "doh"], "elements": "str"},
            "eap-ca-cert": {"v_range": [["7.2.0", ""]], "type": "str"},
            "eap-identity": {"v_range": [["7.2.0", ""]], "type": "str"},
            "eap-method": {"v_range": [["7.2.0", ""]], "choices": ["tls", "peap"], "type": "str"},
            "eap-password": {"v_range": [["7.2.0", ""]], "no_log": true, "type": "raw"},
            "eap-supplicant": {"v_range": [["7.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "eap-user-cert": {"v_range": [["7.2.0", ""]], "type": "str"},
            "ike-saml-server": {"v_range": [["7.2.0", ""]], "type": "str"},
            "lacp-ha-secondary": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "pvc-atm-qos": {"v_range": [["6.4.8", "6.4.14"], ["7.0.2", ""]], "choices": ["cbr", "rt-vbr", "nrt-vbr", "ubr"], "type": "str"},
            "pvc-chan": {"v_range": [["6.4.8", "6.4.14"], ["7.0.2", ""]], "type": "int"},
            "pvc-crc": {"v_range": [["6.4.8", "6.4.14"], ["7.0.2", ""]], "type": "int"},
            "pvc-pcr": {"v_range": [["6.4.8", "6.4.14"], ["7.0.2", ""]], "type": "int"},
            "pvc-scr": {"v_range": [["6.4.8", "6.4.14"], ["7.0.2", ""]], "type": "int"},
            "pvc-vlan-id": {"v_range": [["6.4.8", "6.4.14"], ["7.0.2", ""]], "type": "int"},
            "pvc-vlan-rx-id": {"v_range": [["6.4.8", "6.4.14"], ["7.0.2", ""]], "type": "int"},
            "pvc-vlan-rx-op": {"v_range": [["6.4.8", "6.4.14"], ["7.0.2", ""]], "choices": ["pass-through", "replace", "remove"], "type": "str"},
            "pvc-vlan-tx-id": {"v_range": [["6.4.8", "6.4.14"], ["7.0.2", ""]], "type": "int"},
            "pvc-vlan-tx-op": {"v_range": [["6.4.8", "6.4.14"], ["7.0.2", ""]], "choices": ["pass-through", "replace", "remove"], "type": "str"},
            "reachable-time": {"v_range": [["7.0.3", ""]], "type": "int"},
            "select-profile-30a-35b": {"v_range": [["6.2.9", "6.2.12"], ["6.4.8", "6.4.14"], ["7.0.3", ""]], "choices": ["30A", "35B"], "type": "str"},
            "sfp-dsl": {"v_range": [["6.4.8", "6.4.14"], ["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "sfp-dsl-adsl-fallback": {"v_range": [["6.4.8", "6.4.14"], ["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "sfp-dsl-autodetect": {"v_range": [["6.4.8", "6.4.14"], ["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "sfp-dsl-mac": {"v_range": [["6.4.8", "6.4.14"], ["7.0.2", ""]], "type": "str"},
            "sw-algorithm": {"v_range": [["7.0.1", ""]], "choices": ["l2", "l3", "eh", "default"], "type": "str"},
            "system-id": {"v_range": [["6.4.7", "6.4.14"], ["7.0.2", ""]], "type": "str"},
            "system-id-type": {"v_range": [["6.4.7", "6.4.14"], ["7.0.2", ""]], "choices": ["auto", "user"], "type": "str"},
            "vlan-id": {"v_range": [["6.2.9", "6.2.12"], ["6.4.8", "6.4.14"], ["7.0.3", ""]], "type": "int"},
            "vlan-op-mode": {
                "v_range": [["6.2.9", "6.2.12"], ["6.4.8", "6.4.14"], ["7.0.3", ""]],
                "choices": ["tag", "untag", "passthrough"],
                "type": "str"
            },
            "generic-receive-offload": {"v_range": [["7.0.5", "7.0.12"], ["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "interconnect-profile": {"v_range": [["7.0.5", "7.0.12"], ["7.2.1", ""]], "choices": ["default", "profile1", "profile2"], "type": "str"},
            "large-receive-offload": {"v_range": [["7.0.5", "7.0.12"], ["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "annex": {
                "v_range": [["7.0.10", "7.0.12"], ["7.2.5", "7.2.5"], ["7.4.2", ""]],
                "choices": ["a", "b", "j", "bjm", "i", "al", "m", "aijlm", "bj"],
                "type": "str"
            },
            "aggregate-type": {"v_range": [["7.2.1", ""]], "choices": ["physical", "vxlan"], "type": "str"},
            "switch-controller-netflow-collect": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "wifi-dns-server1": {"v_range": [["7.2.2", ""]], "type": "str"},
            "wifi-dns-server2": {"v_range": [["7.2.2", ""]], "type": "str"},
            "wifi-gateway": {"v_range": [["7.2.2", ""]], "type": "str"},
            "default-purdue-level": {"v_range": [["7.4.0", ""]], "choices": ["1", "2", "3", "4", "5", "1.5", "2.5", "3.5", "5.5"], "type": "str"},
            "dhcp-broadcast-flag": {"v_range": [["7.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "dhcp-smart-relay": {"v_range": [["7.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "switch-controller-offloading": {"v_range": [["7.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "switch-controller-offloading-gw": {"v_range": [["7.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "switch-controller-offloading-ip": {"v_range": [["7.4.0", ""]], "type": "str"},
            "dhcp-relay-circuit-id": {"v_range": [["7.4.1", ""]], "type": "str"},
            "dhcp-relay-source-ip": {"v_range": [["7.4.1", ""]], "type": "str"},
            "switch-controller-offload": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "switch-controller-offload-gw": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "switch-controller-offload-ip": {"v_range": [["7.4.1", ""]], "type": "str"},
            "mirroring-direction": {"v_range": [["7.4.2", ""]], "choices": ["rx", "tx", "both"], "type": "str"},
            "mirroring-port": {"v_range": [["7.4.2", ""]], "type": "str"},
            "port-mirroring": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "security-8021x-member-mode": {"v_range": [["7.4.2", ""]], "choices": ["disable", "switch"], "type": "str"},
            "stp-edge": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "dhcp-relay-allow-no-end-option": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "netflow-sample-rate": {"v_range": [["7.6.0", ""]], "type": "int"},
            "netflow-sampler-id": {"v_range": [["7.6.0", ""]], "type": "int"},
            "pppoe-egress-cos": {"v_range": [["7.6.0", ""]], "choices": ["cos0", "cos1", "cos2", "cos3", "cos4", "cos5", "cos6", "cos7"], "type": "str"},
            "security-ip-auth-bypass": {"v_range": [["7.6.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "virtual-mac": {"v_range": [["7.6.0", ""]], "type": "str"}
        }
    },
    "fsp_vlan_interface_ipv6": {
        "stated": false,
        "options": {
            "autoconf": {"choices": ["disable", "enable"], "type": "str"},
            "dhcp6-client-options": {"type": "list", "choices": ["rapid", "iapd", "iana", "dns", "dnsname"], "elements": "str"},
            "dhcp6-information-request": {"choices": ["disable", "enable"], "type": "str"},
            "dhcp6-prefix-delegation": {"choices": ["disable", "enable"], "type": "str"},
            "dhcp6-prefix-hint": {"type": "str"},
            "dhcp6-prefix-hint-plt": {"type": "int"},
            "dhcp6-prefix-hint-vlt": {"type": "int"},
            "dhcp6-relay-ip": {"type": "str"},
            "dhcp6-relay-service": {"choices": ["disable", "enable"], "type": "str"},
            "dhcp6-relay-type": {"choices": ["regular"], "type": "str"},
            "ip6-address": {"type": "str"},
            "ip6-allowaccess": {
                "type": "list",
                "choices": ["https", "ping", "ssh", "snmp", "http", "telnet", "fgfm", "capwap", "fabric"],
                "elements": "str"
            },
            "ip6-default-life": {"type": "int"},
            "ip6-dns-server-override": {"choices": ["disable", "enable"], "type": "str"},
            "ip6-hop-limit": {"type": "int"},
            "ip6-link-mtu": {"type": "int"},
            "ip6-manage-flag": {"choices": ["disable", "enable"], "type": "str"},
            "ip6-max-interval": {"type": "int"},
            "ip6-min-interval": {"type": "int"},
            "ip6-mode": {"choices": ["static", "dhcp", "pppoe", "delegated"], "type": "str"},
            "ip6-other-flag": {"choices": ["disable", "enable"], "type": "str"},
            "ip6-reachable-time": {"type": "int"},
            "ip6-retrans-time": {"type": "int"},
            "ip6-send-adv": {"choices": ["disable", "enable"], "type": "str"},
            "ip6-subnet": {"type": "str"},
            "ip6-upstream-interface": {"type": "str"},
            "nd-cert": {"type": "str"},
            "nd-cga-modifier": {"type": "str"},
            "nd-mode": {"choices": ["basic", "SEND-compatible"], "type": "str"},
            "nd-security-level": {"type": "int"},
            "nd-timestamp-delta": {"type": "int"},
            "nd-timestamp-fuzz": {"type": "int"},
            "vrip6_link_local": {"type": "str"},
            "vrrp-virtual-mac6": {"choices": ["disable", "enable"], "type": "str"},
            "ip6-delegated-prefix-list": {
                "v_range": [["6.2.2", ""]],
                "type": "list",
                "options": {
                    "autonomous-flag": {"v_range": [["6.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "onlink-flag": {"v_range": [["6.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "prefix-id": {"v_range": [["6.2.2", ""]], "type": "int"},
                    "rdnss": {"v_range": [["6.2.2", ""]], "type": "raw"},
                    "rdnss-service": {"v_range": [["6.2.2", ""]], "choices": ["delegated", "default", "specify"], "type": "str"},
                    "subnet": {"v_range": [["6.2.2", ""]], "type": "str"},
                    "upstream-interface": {"v_range": [["6.2.2", ""]], "type": "str"},
                    "delegated-prefix-iaid": {"v_range": [["7.0.2", ""]], "type": "int"}
                },
                "elements": "dict"
            },
            "ip6-extra-addr": {
                "v_range": [["6.2.2", ""]],
                "type": "list",
                "options": {"prefix": {"v_range": [["6.2.2", ""]], "type": "str"}},
                "elements": "dict"
            },
            "ip6-prefix-list": {
                "v_range": [["6.2.2", ""]],
                "type": "list",
                "options": {
                    "autonomous-flag": {"v_range": [["6.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "dnssl": {"v_range": [["6.2.2", ""]], "type": "raw"},
                    "onlink-flag": {"v_range": [["6.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "preferred-life-time": {"v_range": [["6.2.2", ""]], "type": "int"},
                    "prefix": {"v_range": [["6.2.2", ""]], "type": "str"},
                    "rdnss": {"v_range": [["6.2.2", ""]], "type": "raw"},
                    "valid-life-time": {"v_range": [["6.2.2", ""]], "type": "int"}
                },
                "elements": "dict"
            },
            "vrrp6": {
                "v_range": [["6.2.2", ""]],
                "type": "list",
                "options": {
                    "accept-mode": {"v_range": [["6.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "adv-interval": {"v_range": [["6.2.2", ""]], "type": "int"},
                    "preempt": {"v_range": [["6.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "priority": {"v_range": [["6.2.2", ""]], "type": "int"},
                    "start-time": {"v_range": [["6.2.2", ""]], "type": "int"},
                    "status": {"v_range": [["6.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "vrdst6": {"v_range": [["6.2.2", ""]], "type": "str"},
                    "vrgrp": {"v_range": [["6.2.2", ""]], "type": "int"},
                    "vrid": {"v_range": [["6.2.2", ""]], "type": "int"},
                    "vrip6": {"v_range": [["6.2.2", ""]], "type": "str"},
                    "ignore-default-route": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "vrdst-priority": {"v_range": [["7.6.0", ""]], "type": "int"}
                },
                "elements": "dict"
            },
            "interface-identifier": {"v_range": [["6.4.1", ""]], "type": "str"},
            "unique-autoconf-addr": {"v_range": [["6.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "icmp6-send-redirect": {"v_range": [["6.4.4", ""]], "choices": ["disable", "enable"], "type": "str"},
            "cli-conn6-status": {"v_range": [["7.0.0", ""]], "type": "int"},
            "ip6-prefix-mode": {"v_range": [["7.0.0", ""]], "choices": ["dhcp6", "ra"], "type": "str"},
            "ra-send-mtu": {"v_range": [["6.4.6", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ip6-delegated-prefix-iaid": {"v_range": [["7.0.2", ""]], "type": "int"},
            "dhcp6-relay-source-interface": {"v_range": [["7.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "dhcp6-relay-interface-id": {"v_range": [["7.4.1", ""]], "type": "str"},
            "dhcp6-relay-source-ip": {"v_range": [["7.4.1", ""]], "type": "str"}
        }
    },
    "fsp_vlan_interface_ipv6_ip6delegatedprefixlist": {
        "stated": true,
        "options": {
            "autonomous-flag": {"v_range": [["6.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "onlink-flag": {"v_range": [["6.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "prefix-id": {"v_range": [["6.2.2", ""]], "type": "int"},
            "rdnss": {"v_range": [["6.2.2", ""]], "type": "raw"},
            "rdnss-service": {"v_range": [["6.2.2", ""]], "choices": ["delegated", "default", "specify"], "type": "str"},
            "subnet": {"v_range": [["6.2.2", ""]], "type": "str"},
            "upstream-interface": {"v_range": [["6.2.2", ""]], "type": "str"},
            "delegated-prefix-iaid": {"v_range": [["7.0.2", ""]], "type": "int"}
        }
    },
    "fsp_vlan_interface_ipv6_ip6extraaddr": {"stated": true, "options": {"prefix": {"v_range": [["6.2.2", ""]], "type": "str"}}},
    "fsp_vlan_interface_ipv6_ip6prefixlist": {
        "stated": true,
        "options": {
            "autonomous-flag": {"v_range": [["6.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "dnssl": {"v_range": [["6.2.2", ""]], "type": "raw"},
            "onlink-flag": {"v_range": [["6.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "preferred-life-time": {"v_range": [["6.2.2", ""]], "type": "int"},
            "prefix": {"v_range": [["6.2.2", ""]], "type": "str"},
            "rdnss": {"v_range": [["6.2.2", ""]], "type": "raw"},
            "valid-life-time": {"v_range": [["6.2.2", ""]], "type": "int"}
        }
    },
    "fsp_vlan_interface_ipv6_vrrp6": {
        "stated": true,
        "options": {
            "accept-mode": {"v_range": [["6.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "adv-interval": {"v_range": [["6.2.2", ""]], "type": "int"},
            "preempt": {"v_range": [["6.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "priority": {"v_range": [["6.2.2", ""]], "type": "int"},
            "start-time": {"v_range": [["6.2.2", ""]], "type": "int"},
            "status": {"v_range": [["6.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "vrdst6": {"v_range": [["6.2.2", ""]], "type": "str"},
            "vrgrp": {"v_range": [["6.2.2", ""]], "type": "int"},
            "vrid": {"v_range": [["6.2.2", ""]], "type": "int"},
            "vrip6": {"v_range": [["6.2.2", ""]], "type": "str"},
            "ignore-default-route": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "vrdst-priority": {"v_range": [["7.6.0", ""]], "type": "int"}
        }
    },
    "fsp_vlan_interface_secondaryip": {
        "stated": true,
        "options": {
            "allowaccess": {
                "type": "list",
                "choices": [
                    "https", "ping", "ssh", "snmp", "http", "telnet", "fgfm", "auto-ipsec", "radius-acct", "probe-response", "capwap", "dnp", "ftm",
                    "fabric", "speed-test", "icond", "scim"
                ],
                "elements": "str"
            },
            "detectprotocol": {"v_range": [["6.0.0", "7.2.0"]], "type": "list", "choices": ["ping", "tcp-echo", "udp-echo"], "elements": "str"},
            "detectserver": {"v_range": [["6.0.0", "7.2.0"]], "type": "str"},
            "gwdetect": {"v_range": [["6.0.0", "7.2.0"]], "choices": ["disable", "enable"], "type": "str"},
            "ha-priority": {"v_range": [["6.0.0", "7.2.0"]], "type": "int"},
            "id": {"required": true, "type": "int"},
            "ip": {"type": "str"},
            "ping-serv-status": {"v_range": [["6.0.0", "7.2.0"]], "type": "int"},
            "seq": {"type": "int"},
            "secip-relay-ip": {"v_range": [["7.4.0", ""]], "type": "str"}
        }
    },
    "fsp_vlan_interface_vrrp": {
        "stated": true,
        "options": {
            "accept-mode": {"choices": ["disable", "enable"], "type": "str"},
            "adv-interval": {"type": "int"},
            "ignore-default-route": {"choices": ["disable", "enable"], "type": "str"},
            "preempt": {"choices": ["disable", "enable"], "type": "str"},
            "priority": {"type": "int"},
            "start-time": {"type": "int"},
            "status": {"choices": ["disable", "enable"], "type": "str"},
            "version": {"choices": ["2", "3"], "type": "str"},
            "vrdst": {"type": "raw"},
            "vrdst-priority": {"type": "int"},
            "vrgrp": {"type": "int"},
            "vrid": {"required": true, "type": "int"},
            "vrip": {"type": "str"},
            "proxy-arp": {
                "v_range": [["7.4.0", ""]],
                "type": "list",
                "options": {"id": {"v_range": [["7.4.0", ""]], "type": "int"}, "ip": {"v_range": [["7.4.0", ""]], "type": "str"}},
                "elements": "dict"
            }
        }
    },
    "fsp_vlan_interface_vrrp_proxyarp": {
        "stated": true,
        "options": {"id": {"v_range": [["7.4.0", ""]], "required": true, "type": "int"}, "ip": {"v_range": [["7.4.0", ""]], "type": "str"}}
    },
    "gtp_apn": {"stated": true, "options": {"apn": {"type": "str"}, "name": {"required": true, "type": "str"}}},
    "gtp_apngrp": {"stated": true, "options": {"member": {"type": "raw"}, "name": {"required": true, "type": "str"}}},
    "gtp_iewhitelist": {
        "stated": true,
        "options": {
            "entries": {"type": "list", "options": {"id": {"type": "int"}, "ie": {"type": "int"}, "fmgr_message": {"type": "int"}}, "elements": "dict"},
            "name": {"required": true, "type": "str"}
        }
    },
    "gtp_iewhitelist_entries": {
        "stated": true,
        "options": {"id": {"required": true, "type": "int"}, "ie": {"type": "int"}, "fmgr_message": {"type": "int"}}
    },
    "gtp_messagefilterv0v1": {
        "stated": true,
        "options": {
            "create-mbms": {"choices": ["allow", "deny"], "type": "str"},
            "create-pdp": {"choices": ["allow", "deny"], "type": "str"},
            "data-record": {"choices": ["allow", "deny"], "type": "str"},
            "delete-aa-pdp": {"choices": ["allow", "deny"], "type": "str"},
            "delete-mbms": {"choices": ["allow", "deny"], "type": "str"},
            "delete-pdp": {"choices": ["allow", "deny"], "type": "str"},
            "echo": {"choices": ["allow", "deny"], "type": "str"},
            "end-marker": {"choices": ["allow", "deny"], "type": "str"},
            "error-indication": {"choices": ["allow", "deny"], "type": "str"},
            "failure-report": {"choices": ["allow", "deny"], "type": "str"},
            "fwd-relocation": {"choices": ["allow", "deny"], "type": "str"},
            "fwd-srns-context": {"choices": ["allow", "deny"], "type": "str"},
            "gtp-pdu": {"choices": ["allow", "deny"], "type": "str"},
            "identification": {"choices": ["allow", "deny"], "type": "str"},
            "mbms-de-registration": {"choices": ["allow", "deny"], "type": "str"},
            "mbms-notification": {"choices": ["allow", "deny"], "type": "str"},
            "mbms-registration": {"choices": ["allow", "deny"], "type": "str"},
            "mbms-session-start": {"choices": ["allow", "deny"], "type": "str"},
            "mbms-session-stop": {"choices": ["allow", "deny"], "type": "str"},
            "mbms-session-update": {"choices": ["allow", "deny"], "type": "str"},
            "ms-info-change-notif": {"choices": ["allow", "deny"], "type": "str"},
            "name": {"required": true, "type": "str"},
            "node-alive": {"choices": ["allow", "deny"], "type": "str"},
            "note-ms-present": {"choices": ["allow", "deny"], "type": "str"},
            "pdu-notification": {"choices": ["allow", "deny"], "type": "str"},
            "ran-info": {"choices": ["allow", "deny"], "type": "str"},
            "redirection": {"choices": ["allow", "deny"], "type": "str"},
            "relocation-cancel": {"choices": ["allow", "deny"], "type": "str"},
            "send-route": {"choices": ["allow", "deny"], "type": "str"},
            "sgsn-context": {"choices": ["allow", "deny"], "type": "str"},
            "support-extension": {"choices": ["allow", "deny"], "type": "str"},
            "unknown-message": {"choices": ["allow", "deny"], "type": "str"},
            "unknown-message-white-list": {"type": "raw"},
            "update-mbms": {"choices": ["allow", "deny"], "type": "str"},
            "update-pdp": {"choices": ["allow", "deny"], "type": "str"},
            "v0-create-aa-pdp--v1-init-pdp-ctx": {"choices": ["deny", "allow"], "type": "str"},
            "version-not-support": {"choices": ["allow", "deny"], "type": "str"},
            "create-aa-pdp|init-pdp-ctx": {"v_range": [["6.2.0", "6.4.14"]], "choices": ["allow", "deny"], "type": "str"},
            "ue-registration-query": {"v_range": [["7.2.1", ""]], "choices": ["allow", "deny"], "type": "str"}
        }
    },
    "gtp_messagefilterv2": {
        "stated": true,
        "options": {
            "bearer-resource-cmd-fail": {"choices": ["allow", "deny"], "type": "str"},
            "change-notification": {"choices": ["allow", "deny"], "type": "str"},
            "create-bearer": {"choices": ["allow", "deny"], "type": "str"},
            "create-session": {"choices": ["allow", "deny"], "type": "str"},
            "delete-bearer-cmd-fail": {"choices": ["allow", "deny"], "type": "str"},
            "delete-bearer-req-resp": {"choices": ["allow", "deny"], "type": "str"},
            "delete-pdn-connection-set": {"choices": ["allow", "deny"], "type": "str"},
            "delete-session": {"choices": ["allow", "deny"], "type": "str"},
            "echo": {"choices": ["allow", "deny"], "type": "str"},
            "modify-bearer-cmd-fail": {"choices": ["allow", "deny"], "type": "str"},
            "modify-bearer-req-resp": {"choices": ["allow", "deny"], "type": "str"},
            "name": {"required": true, "type": "str"},
            "resume": {"choices": ["allow", "deny"], "type": "str"},
            "suspend": {"choices": ["allow", "deny"], "type": "str"},
            "trace-session": {"choices": ["allow", "deny"], "type": "str"},
            "unknown-message": {"choices": ["allow", "deny"], "type": "str"},
            "unknown-message-white-list": {"type": "raw"},
            "update-bearer": {"choices": ["allow", "deny"], "type": "str"},
            "update-pdn-connection-set": {"choices": ["allow", "deny"], "type": "str"},
            "version-not-support": {"choices": ["allow", "deny"], "type": "str"},
            "context-req-res-ack": {"v_range": [["7.0.2", ""]], "choices": ["allow", "deny"], "type": "str"},
            "forward-relocation-cmp-notif-ack": {"v_range": [["7.0.2", ""]], "choices": ["allow", "deny"], "type": "str"},
            "forward-relocation-req-res": {"v_range": [["7.0.2", ""]], "choices": ["allow", "deny"], "type": "str"},
            "alert-mme-notif-ack": {"v_range": [["7.2.1", ""]], "choices": ["allow", "deny"], "type": "str"},
            "configuration-transfer-tunnel": {"v_range": [["7.2.1", ""]], "choices": ["allow", "deny"], "type": "str"},
            "create-forwarding-tunnel-req-resp": {"v_range": [["7.2.1", ""]], "choices": ["allow", "deny"], "type": "str"},
            "create-indirect-forwarding-tunnel-req-resp": {"v_range": [["7.2.1", ""]], "choices": ["allow", "deny"], "type": "str"},
            "cs-paging": {"v_range": [["7.2.1", ""]], "choices": ["allow", "deny"], "type": "str"},
            "delete-indirect-forwarding-tunnel-req-resp": {"v_range": [["7.2.1", ""]], "choices": ["allow", "deny"], "type": "str"},
            "detach-notif-ack": {"v_range": [["7.2.1", ""]], "choices": ["allow", "deny"], "type": "str"},
            "dlink-data-notif-ack": {"v_range": [["7.2.1", ""]], "choices": ["allow", "deny"], "type": "str"},
            "dlink-notif-failure": {"v_range": [["7.2.1", ""]], "choices": ["allow", "deny"], "type": "str"},
            "forward-access-notif-ack": {"v_range": [["7.2.1", ""]], "choices": ["allow", "deny"], "type": "str"},
            "identification-req-resp": {"v_range": [["7.2.1", ""]], "choices": ["allow", "deny"], "type": "str"},
            "isr-status": {"v_range": [["7.2.1", ""]], "choices": ["allow", "deny"], "type": "str"},
            "mbms-session-start-req-resp": {"v_range": [["7.2.1", ""]], "choices": ["allow", "deny"], "type": "str"},
            "mbms-session-stop-req-resp": {"v_range": [["7.2.1", ""]], "choices": ["allow", "deny"], "type": "str"},
            "mbms-session-update-req-resp": {"v_range": [["7.2.1", ""]], "choices": ["allow", "deny"], "type": "str"},
            "modify-access-req-resp": {"v_range": [["7.2.1", ""]], "choices": ["allow", "deny"], "type": "str"},
            "pgw-dlink-notif-ack": {"v_range": [["7.2.1", ""]], "choices": ["allow", "deny"], "type": "str"},
            "pgw-restart-notif-ack": {"v_range": [["7.2.1", ""]], "choices": ["allow", "deny"], "type": "str"},
            "ran-info-relay": {"v_range": [["7.2.1", ""]], "choices": ["allow", "deny"], "type": "str"},
            "release-access-bearer-req-resp": {"v_range": [["7.2.1", ""]], "choices": ["allow", "deny"], "type": "str"},
            "relocation-cancel-req-resp": {"v_range": [["7.2.1", ""]], "choices": ["allow", "deny"], "type": "str"},
            "remote-ue-report-notif-ack": {"v_range": [["7.2.1", ""]], "choices": ["allow", "deny"], "type": "str"},
            "reserved-for-earlier-version": {"v_range": [["7.2.1", ""]], "choices": ["allow", "deny"], "type": "str"},
            "stop-paging-indication": {"v_range": [["7.2.1", ""]], "choices": ["allow", "deny"], "type": "str"},
            "ue-activity-notif-ack": {"v_range": [["7.2.1", ""]], "choices": ["allow", "deny"], "type": "str"},
            "ue-registration-query-req-resp": {"v_range": [["7.2.1", ""]], "choices": ["allow", "deny"], "type": "str"}
        }
    },
    "gtp_tunnellimit": {"stated": true, "options": {"name": {"required": true, "type": "str"}, "tunnel-limit": {"type": "int"}}},
    "hotspot20_anqp3gppcellular": {
        "stated": true,
        "options": {
            "mcc-mnc-list": {"type": "list", "options": {"id": {"type": "int"}, "mcc": {"type": "str"}, "mnc": {"type": "str"}}, "elements": "dict"},
            "name": {"required": true, "type": "str"}
        }
    },
    "hotspot20_anqp3gppcellular_mccmnclist": {
        "stated": true,
        "options": {"id": {"required": true, "type": "int"}, "mcc": {"type": "str"}, "mnc": {"type": "str"}}
    },
    "hotspot20_anqpipaddresstype": {
        "stated": true,
        "options": {
            "ipv4-address-type": {
                "choices": [
                    "not-available", "not-known", "public", "port-restricted", "single-NATed-private", "double-NATed-private",
                    "port-restricted-and-single-NATed", "port-restricted-and-double-NATed"
                ],
                "type": "str"
            },
            "ipv6-address-type": {"choices": ["not-available", "available", "not-known"], "type": "str"},
            "name": {"required": true, "type": "str"}
        }
    },
    "hotspot20_anqpnairealm": {
        "stated": true,
        "options": {
            "nai-list": {
                "type": "list",
                "options": {
                    "eap-method": {
                        "type": "list",
                        "options": {
                            "auth-param": {
                                "type": "list",
                                "options": {
                                    "id": {"choices": ["non-eap-inner-auth", "inner-auth-eap", "credential", "tunneled-credential"], "type": "str"},
                                    "index": {"type": "int"},
                                    "val": {
                                        "choices": [
                                            "eap-identity", "eap-md5", "eap-tls", "eap-ttls", "eap-peap", "eap-sim", "eap-aka", "eap-aka-prime",
                                            "non-eap-pap", "non-eap-chap", "non-eap-mschap", "non-eap-mschapv2", "cred-sim", "cred-usim", "cred-nfc",
                                            "cred-hardware-token", "cred-softoken", "cred-certificate", "cred-user-pwd", "cred-none",
                                            "cred-vendor-specific", "tun-cred-sim", "tun-cred-usim", "tun-cred-nfc", "tun-cred-hardware-token",
                                            "tun-cred-softoken", "tun-cred-certificate", "tun-cred-user-pwd", "tun-cred-anonymous",
                                            "tun-cred-vendor-specific"
                                        ],
                                        "type": "str"
                                    }
                                },
                                "elements": "dict"
                            },
                            "index": {"type": "int"},
                            "method": {
                                "choices": ["eap-identity", "eap-md5", "eap-tls", "eap-ttls", "eap-peap", "eap-sim", "eap-aka", "eap-aka-prime"],
                                "type": "str"
                            }
                        },
                        "elements": "dict"
                    },
                    "encoding": {"choices": ["disable", "enable"], "type": "str"},
                    "nai-realm": {"type": "str"},
                    "name": {"type": "str"}
                },
                "elements": "dict"
            },
            "name": {"required": true, "type": "str"}
        }
    },
    "hotspot20_anqpnairealm_nailist": {
        "stated": true,
        "options": {
            "eap-method": {
                "type": "list",
                "options": {
                    "auth-param": {
                        "type": "list",
                        "options": {
                            "id": {"choices": ["non-eap-inner-auth", "inner-auth-eap", "credential", "tunneled-credential"], "type": "str"},
                            "index": {"type": "int"},
                            "val": {
                                "choices": [
                                    "eap-identity", "eap-md5", "eap-tls", "eap-ttls", "eap-peap", "eap-sim", "eap-aka", "eap-aka-prime", "non-eap-pap",
                                    "non-eap-chap", "non-eap-mschap", "non-eap-mschapv2", "cred-sim", "cred-usim", "cred-nfc", "cred-hardware-token",
                                    "cred-softoken", "cred-certificate", "cred-user-pwd", "cred-none", "cred-vendor-specific", "tun-cred-sim",
                                    "tun-cred-usim", "tun-cred-nfc", "tun-cred-hardware-token", "tun-cred-softoken", "tun-cred-certificate",
                                    "tun-cred-user-pwd", "tun-cred-anonymous", "tun-cred-vendor-specific"
                                ],
                                "type": "str"
                            }
                        },
                        "elements": "dict"
                    },
                    "index": {"type": "int"},
                    "method": {
                        "choices": ["eap-identity", "eap-md5", "eap-tls", "eap-ttls", "eap-peap", "eap-sim", "eap-aka", "eap-aka-prime"],
                        "type": "str"
                    }
                },
                "elements": "dict"
            },
            "encoding": {"choices": ["disable", "enable"], "type": "str"},
            "nai-realm": {"type": "str"},
            "name": {"required": true, "type": "str"}
        }
    },
    "hotspot20_anqpnairealm_nailist_eapmethod": {
        "stated": true,
        "options": {
            "auth-param": {
                "type": "list",
                "options": {
                    "id": {"choices": ["non-eap-inner-auth", "inner-auth-eap", "credential", "tunneled-credential"], "type": "str"},
                    "index": {"type": "int"},
                    "val": {
                        "choices": [
                            "eap-identity", "eap-md5", "eap-tls", "eap-ttls", "eap-peap", "eap-sim", "eap-aka", "eap-aka-prime", "non-eap-pap",
                            "non-eap-chap", "non-eap-mschap", "non-eap-mschapv2", "cred-sim", "cred-usim", "cred-nfc", "cred-hardware-token",
                            "cred-softoken", "cred-certificate", "cred-user-pwd", "cred-none", "cred-vendor-specific", "tun-cred-sim", "tun-cred-usim",
                            "tun-cred-nfc", "tun-cred-hardware-token", "tun-cred-softoken", "tun-cred-certificate", "tun-cred-user-pwd",
                            "tun-cred-anonymous", "tun-cred-vendor-specific"
                        ],
                        "type": "str"
                    }
                },
                "elements": "dict"
            },
            "index": {"required": true, "type": "int"},
            "method": {"choices": ["eap-identity", "eap-md5", "eap-tls", "eap-ttls", "eap-peap", "eap-sim", "eap-aka", "eap-aka-prime"], "type": "str"}
        }
    },
    "hotspot20_anqpnairealm_nailist_eapmethod_authparam": {
        "stated": true,
        "options": {
            "id": {"required": true, "choices": ["non-eap-inner-auth", "inner-auth-eap", "credential", "tunneled-credential"], "type": "str"},
            "index": {"type": "int"},
            "val": {
                "choices": [
                    "eap-identity", "eap-md5", "eap-tls", "eap-ttls", "eap-peap", "eap-sim", "eap-aka", "eap-aka-prime", "non-eap-pap", "non-eap-chap",
                    "non-eap-mschap", "non-eap-mschapv2", "cred-sim", "cred-usim", "cred-nfc", "cred-hardware-token", "cred-softoken",
                    "cred-certificate", "cred-user-pwd", "cred-none", "cred-vendor-specific", "tun-cred-sim", "tun-cred-usim", "tun-cred-nfc",
                    "tun-cred-hardware-token", "tun-cred-softoken", "tun-cred-certificate", "tun-cred-user-pwd", "tun-cred-anonymous",
                    "tun-cred-vendor-specific"
                ],
                "type": "str"
            }
        }
    },
    "hotspot20_anqpnetworkauthtype": {
        "stated": true,
        "options": {
            "auth-type": {"choices": ["acceptance-of-terms", "online-enrollment", "http-redirection", "dns-redirection"], "type": "str"},
            "name": {"required": true, "type": "str"},
            "url": {"type": "str"}
        }
    },
    "hotspot20_anqproamingconsortium": {
        "stated": true,
        "options": {
            "name": {"required": true, "type": "str"},
            "oi-list": {"type": "list", "options": {"comment": {"type": "str"}, "index": {"type": "int"}, "oi": {"type": "str"}}, "elements": "dict"}
        }
    },
    "hotspot20_anqproamingconsortium_oilist": {
        "stated": true,
        "options": {"comment": {"type": "str"}, "index": {"required": true, "type": "int"}, "oi": {"type": "str"}}
    },
    "hotspot20_anqpvenuename": {
        "stated": true,
        "options": {
            "name": {"required": true, "type": "str"},
            "value-list": {"type": "list", "options": {"index": {"type": "int"}, "lang": {"type": "str"}, "value": {"type": "str"}}, "elements": "dict"}
        }
    },
    "hotspot20_anqpvenuename_valuelist": {
        "stated": true,
        "options": {"index": {"required": true, "type": "int"}, "lang": {"type": "str"}, "value": {"type": "str"}}
    },
    "hotspot20_anqpvenueurl": {
        "stated": true,
        "options": {
            "name": {"v_range": [["7.0.3", ""]], "required": true, "type": "str"},
            "value-list": {
                "v_range": [["7.0.3", ""]],
                "type": "list",
                "options": {
                    "index": {"v_range": [["7.0.3", ""]], "type": "int"},
                    "number": {"v_range": [["7.0.3", ""]], "type": "int"},
                    "value": {"v_range": [["7.0.3", ""]], "type": "str"}
                },
                "elements": "dict"
            }
        }
    },
    "hotspot20_anqpvenueurl_valuelist": {
        "stated": true,
        "options": {
            "index": {"v_range": [["7.0.3", ""]], "type": "int"},
            "number": {"v_range": [["7.0.3", ""]], "type": "int"},
            "value": {"v_range": [["7.0.3", ""]], "type": "str"}
        }
    },
    "hotspot20_h2qpadviceofcharge": {
        "stated": true,
        "options": {
            "aoc-list": {
                "v_range": [["7.0.3", ""]],
                "type": "list",
                "options": {
                    "nai-realm": {"v_range": [["7.0.3", ""]], "type": "str"},
                    "nai-realm-encoding": {"v_range": [["7.0.3", ""]], "type": "str"},
                    "name": {"v_range": [["7.0.3", ""]], "type": "str"},
                    "plan-info": {
                        "v_range": [["7.0.3", ""]],
                        "type": "list",
                        "options": {
                            "currency": {"v_range": [["7.0.3", ""]], "type": "str"},
                            "info-file": {"v_range": [["7.0.3", ""]], "type": "str"},
                            "lang": {"v_range": [["7.0.3", ""]], "type": "str"},
                            "name": {"v_range": [["7.0.3", ""]], "type": "str"}
                        },
                        "elements": "dict"
                    },
                    "type": {"v_range": [["7.0.3", ""]], "choices": ["time-based", "volume-based", "time-and-volume-based", "unlimited"], "type": "str"}
                },
                "elements": "dict"
            },
            "name": {"v_range": [["7.0.3", ""]], "required": true, "type": "str"}
        }
    },
    "hotspot20_h2qpadviceofcharge_aoclist": {
        "stated": true,
        "options": {
            "nai-realm": {"v_range": [["7.0.3", ""]], "type": "str"},
            "nai-realm-encoding": {"v_range": [["7.0.3", ""]], "type": "str"},
            "name": {"v_range": [["7.0.3", ""]], "required": true, "type": "str"},
            "plan-info": {
                "v_range": [["7.0.3", ""]],
                "type": "list",
                "options": {
                    "currency": {"v_range": [["7.0.3", ""]], "type": "str"},
                    "info-file": {"v_range": [["7.0.3", ""]], "type": "str"},
                    "lang": {"v_range": [["7.0.3", ""]], "type": "str"},
                    "name": {"v_range": [["7.0.3", ""]], "type": "str"}
                },
                "elements": "dict"
            },
            "type": {"v_range": [["7.0.3", ""]], "choices": ["time-based", "volume-based", "time-and-volume-based", "unlimited"], "type": "str"}
        }
    },
    "hotspot20_h2qpadviceofcharge_aoclist_planinfo": {
        "stated": true,
        "options": {
            "currency": {"v_range": [["7.0.3", ""]], "type": "str"},
            "info-file": {"v_range": [["7.0.3", ""]], "type": "str"},
            "lang": {"v_range": [["7.0.3", ""]], "type": "str"},
            "name": {"v_range": [["7.0.3", ""]], "required": true, "type": "str"}
        }
    },
    "hotspot20_h2qpconncapability": {
        "stated": true,
        "options": {
            "esp-port": {"choices": ["closed", "open", "unknown"], "type": "str"},
            "ftp-port": {"choices": ["closed", "open", "unknown"], "type": "str"},
            "http-port": {"choices": ["closed", "open", "unknown"], "type": "str"},
            "icmp-port": {"choices": ["closed", "open", "unknown"], "type": "str"},
            "ikev2-port": {"choices": ["closed", "open", "unknown"], "type": "str"},
            "ikev2-xx-port": {"choices": ["closed", "open", "unknown"], "type": "str"},
            "name": {"required": true, "type": "str"},
            "pptp-vpn-port": {"choices": ["closed", "open", "unknown"], "type": "str"},
            "ssh-port": {"choices": ["closed", "open", "unknown"], "type": "str"},
            "tls-port": {"choices": ["closed", "open", "unknown"], "type": "str"},
            "voip-tcp-port": {"choices": ["closed", "open", "unknown"], "type": "str"},
            "voip-udp-port": {"choices": ["closed", "open", "unknown"], "type": "str"}
        }
    },
    "hotspot20_h2qpoperatorname": {
        "stated": true,
        "options": {
            "name": {"required": true, "type": "str"},
            "value-list": {"type": "list", "options": {"index": {"type": "int"}, "lang": {"type": "str"}, "value": {"type": "str"}}, "elements": "dict"}
        }
    },
    "hotspot20_h2qpoperatorname_valuelist": {
        "stated": true,
        "options": {"index": {"required": true, "type": "int"}, "lang": {"type": "str"}, "value": {"type": "str"}}
    },
    "hotspot20_h2qposuprovider": {
        "stated": true,
        "options": {
            "friendly-name": {
                "type": "list",
                "options": {"friendly-name": {"type": "str"}, "index": {"type": "int"}, "lang": {"type": "str"}},
                "elements": "dict"
            },
            "icon": {"type": "str"},
            "name": {"required": true, "type": "str"},
            "osu-method": {"type": "list", "choices": ["oma-dm", "soap-xml-spp", "reserved"], "elements": "str"},
            "osu-nai": {"type": "str"},
            "server-uri": {"type": "str"},
            "service-description": {
                "type": "list",
                "options": {"lang": {"type": "str"}, "service-description": {"type": "str"}, "service-id": {"type": "int"}},
                "elements": "dict"
            }
        }
    },
    "hotspot20_h2qposuprovider_friendlyname": {
        "stated": true,
        "options": {"friendly-name": {"type": "str"}, "index": {"required": true, "type": "int"}, "lang": {"type": "str"}}
    },
    "hotspot20_h2qposuprovider_servicedescription": {
        "stated": true,
        "options": {"lang": {"type": "str"}, "service-description": {"type": "str"}, "service-id": {"required": true, "type": "int"}}
    },
    "hotspot20_h2qposuprovidernai": {
        "stated": true,
        "options": {
            "nai-list": {
                "v_range": [["7.0.3", ""]],
                "type": "list",
                "options": {"name": {"v_range": [["7.0.3", ""]], "type": "str"}, "osu-nai": {"v_range": [["7.0.3", ""]], "type": "str"}},
                "elements": "dict"
            },
            "name": {"v_range": [["7.0.3", ""]], "required": true, "type": "str"}
        }
    },
    "hotspot20_h2qposuprovidernai_nailist": {
        "stated": true,
        "options": {"name": {"v_range": [["7.0.3", ""]], "required": true, "type": "str"}, "osu-nai": {"v_range": [["7.0.3", ""]], "type": "str"}}
    },
    "hotspot20_h2qptermsandconditions": {
        "stated": true,
        "options": {
            "filename": {"v_range": [["7.0.3", ""]], "type": "str"},
            "name": {"v_range": [["7.0.3", ""]], "required": true, "type": "str"},
            "timestamp": {"v_range": [["7.0.3", ""]], "type": "int"},
            "url": {"v_range": [["7.0.3", ""]], "type": "str"}
        }
    },
    "hotspot20_h2qpwanmetric": {
        "stated": true,
        "options": {
            "downlink-load": {"type": "int"},
            "downlink-speed": {"type": "int"},
            "link-at-capacity": {"choices": ["disable", "enable"], "type": "str"},
            "link-status": {"choices": ["down", "up", "in-test"], "type": "str"},
            "load-measurement-duration": {"type": "int"},
            "name": {"required": true, "type": "str"},
            "symmetric-wan-link": {"choices": ["asymmetric", "symmetric"], "type": "str"},
            "uplink-load": {"type": "int"},
            "uplink-speed": {"type": "int"}
        }
    },
    "hotspot20_hsprofile": {
        "stated": true,
        "options": {
            "3gpp-plmn": {"type": "str"},
            "access-network-asra": {"choices": ["disable", "enable"], "type": "str"},
            "access-network-esr": {"choices": ["disable", "enable"], "type": "str"},
            "access-network-internet": {"choices": ["disable", "enable"], "type": "str"},
            "access-network-type": {
                "choices": [
                    "private-network", "private-network-with-guest-access", "chargeable-public-network", "free-public-network",
                    "personal-device-network", "emergency-services-only-network", "test-or-experimental", "wildcard"
                ],
                "type": "str"
            },
            "access-network-uesa": {"choices": ["disable", "enable"], "type": "str"},
            "anqp-domain-id": {"type": "int"},
            "bss-transition": {"choices": ["disable", "enable"], "type": "str"},
            "conn-cap": {"type": "str"},
            "deauth-request-timeout": {"type": "int"},
            "dgaf": {"choices": ["disable", "enable"], "type": "str"},
            "domain-name": {"type": "str"},
            "gas-comeback-delay": {"type": "int"},
            "gas-fragmentation-limit": {"type": "int"},
            "hessid": {"type": "str"},
            "ip-addr-type": {"type": "str"},
            "l2tif": {"choices": ["disable", "enable"], "type": "str"},
            "nai-realm": {"type": "str"},
            "name": {"required": true, "type": "str"},
            "network-auth": {"type": "str"},
            "oper-friendly-name": {"type": "str"},
            "osu-provider": {"type": "raw"},
            "osu-ssid": {"type": "str"},
            "pame-bi": {"choices": ["disable", "enable"], "type": "str"},
            "proxy-arp": {"choices": ["disable", "enable"], "type": "str"},
            "qos-map": {"type": "str"},
            "roaming-consortium": {"type": "str"},
            "venue-group": {
                "choices": [
                    "unspecified", "assembly", "business", "educational", "factory", "institutional", "mercantile", "residential", "storage", "utility",
                    "vehicular", "outdoor"
                ],
                "type": "str"
            },
            "venue-name": {"type": "str"},
            "venue-type": {
                "choices": [
                    "unspecified", "arena", "stadium", "passenger-terminal", "amphitheater", "amusement-park", "place-of-worship", "convention-center",
                    "library", "museum", "restaurant", "theater", "bar", "coffee-shop", "zoo-or-aquarium", "emergency-center", "doctor-office", "bank",
                    "fire-station", "police-station", "post-office", "professional-office", "research-facility", "attorney-office", "primary-school",
                    "secondary-school", "university-or-college", "factory", "hospital", "long-term-care-facility", "rehab-center", "group-home",
                    "prison-or-jail", "retail-store", "grocery-market", "auto-service-station", "shopping-mall", "gas-station", "private",
                    "hotel-or-motel", "dormitory", "boarding-house", "automobile", "airplane", "bus", "ferry", "ship-or-boat", "train", "motor-bike",
                    "muni-mesh-network", "city-park", "rest-area", "traffic-control", "bus-stop", "kiosk"
                ],
                "type": "str"
            },
            "wan-metrics": {"type": "str"},
            "wnm-sleep-mode": {"choices": ["disable", "enable"], "type": "str"},
            "advice-of-charge": {"v_range": [["7.0.2", ""]], "type": "str"},
            "oper-icon": {"v_range": [["7.0.2", ""]], "type": "str"},
            "osu-provider-nai": {"v_range": [["7.0.2", ""]], "type": "str"},
            "release": {"v_range": [["7.0.2", ""]], "type": "int"},
            "terms-and-conditions": {"v_range": [["7.0.2", ""]], "type": "str"},
            "venue-url": {"v_range": [["7.0.2", ""]], "type": "str"},
            "wba-charging-currency": {"v_range": [["7.6.0", ""]], "type": "str"},
            "wba-charging-rate": {"v_range": [["7.6.0", ""]], "type": "int"},
            "wba-data-clearing-provider": {"v_range": [["7.6.0", ""]], "type": "str"},
            "wba-financial-clearing-provider": {"v_range": [["7.6.0", ""]], "type": "str"},
            "wba-open-roaming": {"v_range": [["7.6.0", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "hotspot20_icon": {
        "stated": true,
        "options": {
            "icon-list": {
                "v_range": [["7.0.3", ""]],
                "type": "list",
                "options": {
                    "file": {"v_range": [["7.0.3", ""]], "type": "str"},
                    "height": {"v_range": [["7.0.3", ""]], "type": "int"},
                    "lang": {"v_range": [["7.0.3", ""]], "type": "str"},
                    "name": {"v_range": [["7.0.3", ""]], "type": "str"},
                    "type": {"v_range": [["7.0.3", ""]], "choices": ["bmp", "gif", "jpeg", "png", "tiff"], "type": "str"},
                    "width": {"v_range": [["7.0.3", ""]], "type": "int"}
                },
                "elements": "dict"
            },
            "name": {"v_range": [["7.0.3", ""]], "required": true, "type": "str"}
        }
    },
    "hotspot20_icon_iconlist": {
        "stated": true,
        "options": {
            "file": {"v_range": [["7.0.3", ""]], "type": "str"},
            "height": {"v_range": [["7.0.3", ""]], "type": "int"},
            "lang": {"v_range": [["7.0.3", ""]], "type": "str"},
            "name": {"v_range": [["7.0.3", ""]], "required": true, "type": "str"},
            "type": {"v_range": [["7.0.3", ""]], "choices": ["bmp", "gif", "jpeg", "png", "tiff"], "type": "str"},
            "width": {"v_range": [["7.0.3", ""]], "type": "int"}
        }
    },
    "hotspot20_qosmap": {
        "stated": true,
        "options": {
            "dscp-except": {"type": "list", "options": {"dscp": {"type": "int"}, "index": {"type": "int"}, "up": {"type": "int"}}, "elements": "dict"},
            "dscp-range": {
                "type": "list",
                "options": {"high": {"type": "int"}, "index": {"type": "int"}, "low": {"type": "int"}, "up": {"type": "int"}},
                "elements": "dict"
            },
            "name": {"required": true, "type": "str"}
        }
    },
    "hotspot20_qosmap_dscpexcept": {
        "stated": true,
        "options": {"dscp": {"type": "int"}, "index": {"required": true, "type": "int"}, "up": {"type": "int"}}
    },
    "hotspot20_qosmap_dscprange": {
        "stated": true,
        "options": {"high": {"type": "int"}, "index": {"required": true, "type": "int"}, "low": {"type": "int"}, "up": {"type": "int"}}
    },
    "icap_profile": {
        "stated": true,
        "options": {
            "methods": {"type": "list", "choices": ["delete", "get", "head", "options", "post", "put", "trace", "other", "connect"], "elements": "str"},
            "name": {"required": true, "type": "str"},
            "replacemsg-group": {"type": "str"},
            "request": {"choices": ["disable", "enable"], "type": "str"},
            "request-failure": {"choices": ["error", "bypass"], "type": "str"},
            "request-path": {"type": "str"},
            "request-server": {"type": "str"},
            "response": {"choices": ["disable", "enable"], "type": "str"},
            "response-failure": {"choices": ["error", "bypass"], "type": "str"},
            "response-path": {"type": "str"},
            "response-server": {"type": "str"},
            "streaming-content-bypass": {"choices": ["disable", "enable"], "type": "str"},
            "icap-headers": {
                "v_range": [["6.2.0", ""]],
                "type": "list",
                "options": {
                    "base64-encoding": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "content": {"v_range": [["6.2.0", ""]], "type": "str"},
                    "id": {"v_range": [["6.2.0", ""]], "type": "int"},
                    "name": {"v_range": [["6.2.0", ""]], "type": "str"}
                },
                "elements": "dict"
            },
            "preview": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "preview-data-length": {"v_range": [["6.2.0", ""]], "type": "int"},
            "response-req-hdr": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "respmod-default-action": {"v_range": [["6.4.0", ""]], "choices": ["bypass", "forward"], "type": "str"},
            "respmod-forward-rules": {
                "v_range": [["6.4.0", ""]],
                "type": "list",
                "options": {
                    "action": {"v_range": [["6.4.0", ""]], "choices": ["bypass", "forward"], "type": "str"},
                    "header-group": {
                        "v_range": [["6.4.0", ""]],
                        "type": "list",
                        "options": {
                            "case-sensitivity": {"v_range": [["6.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "header": {"v_range": [["6.4.0", ""]], "type": "str"},
                            "header-name": {"v_range": [["6.4.0", ""]], "type": "str"},
                            "id": {"v_range": [["6.4.0", ""]], "type": "int"}
                        },
                        "elements": "dict"
                    },
                    "host": {"v_range": [["6.4.0", ""]], "type": "str"},
                    "http-resp-status-code": {"v_range": [["6.4.0", ""]], "type": "raw"},
                    "name": {"v_range": [["6.4.0", ""]], "type": "str"}
                },
                "elements": "dict"
            },
            "204-response": {"v_range": [["7.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "204-size-limit": {"v_range": [["7.2.0", ""]], "type": "int"},
            "chunk-encap": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "extension-feature": {"v_range": [["7.0.2", ""]], "type": "list", "choices": ["scan-progress"], "elements": "str"},
            "file-transfer": {"v_range": [["7.2.0", ""]], "type": "list", "choices": ["ssh", "ftp"], "elements": "str"},
            "file-transfer-failure": {"v_range": [["7.2.0", ""]], "choices": ["error", "bypass"], "type": "str"},
            "file-transfer-path": {"v_range": [["7.2.0", ""]], "type": "str"},
            "file-transfer-server": {"v_range": [["7.2.0", ""]], "type": "str"},
            "icap-block-log": {"v_range": [["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "scan-progress-interval": {"v_range": [["7.0.2", ""]], "type": "int"},
            "timeout": {"v_range": [["7.2.0", ""]], "type": "int"},
            "comment": {"v_range": [["7.2.2", ""]], "type": "str"}
        }
    },
    "icap_profile_icapheaders": {
        "stated": true,
        "options": {
            "base64-encoding": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "content": {"v_range": [["6.2.0", ""]], "type": "str"},
            "id": {"v_range": [["6.2.0", ""]], "required": true, "type": "int"},
            "name": {"v_range": [["6.2.0", ""]], "type": "str"}
        }
    },
    "icap_profile_respmodforwardrules": {
        "stated": true,
        "options": {
            "action": {"v_range": [["6.4.0", ""]], "choices": ["bypass", "forward"], "type": "str"},
            "header-group": {
                "v_range": [["6.4.0", ""]],
                "type": "list",
                "options": {
                    "case-sensitivity": {"v_range": [["6.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "header": {"v_range": [["6.4.0", ""]], "type": "str"},
                    "header-name": {"v_range": [["6.4.0", ""]], "type": "str"},
                    "id": {"v_range": [["6.4.0", ""]], "type": "int"}
                },
                "elements": "dict"
            },
            "host": {"v_range": [["6.4.0", ""]], "type": "str"},
            "http-resp-status-code": {"v_range": [["6.4.0", ""]], "type": "raw"},
            "name": {"v_range": [["6.4.0", ""]], "required": true, "type": "str"}
        }
    },
    "icap_profile_respmodforwardrules_headergroup": {
        "stated": true,
        "options": {
            "case-sensitivity": {"v_range": [["6.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "header": {"v_range": [["6.4.0", ""]], "type": "str"},
            "header-name": {"v_range": [["6.4.0", ""]], "type": "str"},
            "id": {"v_range": [["6.4.0", ""]], "required": true, "type": "int"}
        }
    },
    "icap_server": {
        "stated": true,
        "options": {
            "ip-address": {"type": "str"},
            "ip-version": {"choices": ["4", "6"], "type": "str"},
            "ip6-address": {"type": "str"},
            "max-connections": {"type": "int"},
            "name": {"required": true, "type": "str"},
            "port": {"type": "int"},
            "secure": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ssl-cert": {"v_range": [["7.0.0", ""]], "type": "str"},
            "addr-type": {"v_range": [["7.2.0", ""]], "choices": ["fqdn", "ip4", "ip6"], "type": "str"},
            "fqdn": {"v_range": [["7.2.0", ""]], "type": "str"},
            "healthcheck": {"v_range": [["7.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "healthcheck-service": {"v_range": [["7.2.0", ""]], "type": "str"}
        }
    },
    "ips_baseline_sensor": {
        "stated": true,
        "options": {
            "block-malicious-url": {"v_range": [["7.0.1", "7.0.2"]], "choices": ["disable", "enable"], "type": "str"},
            "comment": {"v_range": [["7.0.1", "7.0.2"]], "type": "str"},
            "entries": {
                "v_range": [["7.0.1", "7.0.2"]],
                "type": "list",
                "options": {
                    "action": {"v_range": [["7.0.1", "7.0.2"]], "choices": ["pass", "block", "reset", "default"], "type": "str"},
                    "application": {"v_range": [["7.0.1", "7.0.2"]], "type": "raw"},
                    "cve": {"v_range": [["7.0.1", "7.0.2"]], "type": "raw"},
                    "exempt-ip": {
                        "v_range": [["7.0.1", "7.0.2"]],
                        "type": "list",
                        "options": {
                            "dst-ip": {"v_range": [["7.0.1", "7.0.2"]], "type": "str"},
                            "id": {"v_range": [["7.0.1", "7.0.2"]], "type": "int"},
                            "src-ip": {"v_range": [["7.0.1", "7.0.2"]], "type": "str"}
                        },
                        "elements": "dict"
                    },
                    "id": {"v_range": [["7.0.1", "7.0.2"]], "type": "int"},
                    "location": {"v_range": [["7.0.1", "7.0.2"]], "type": "raw"},
                    "log": {"v_range": [["7.0.1", "7.0.2"]], "choices": ["disable", "enable"], "type": "str"},
                    "log-attack-context": {"v_range": [["7.0.1", "7.0.2"]], "choices": ["disable", "enable"], "type": "str"},
                    "log-packet": {"v_range": [["7.0.1", "7.0.2"]], "choices": ["disable", "enable"], "type": "str"},
                    "os": {"v_range": [["7.0.1", "7.0.2"]], "type": "raw"},
                    "protocol": {"v_range": [["7.0.1", "7.0.2"]], "type": "raw"},
                    "quarantine": {"v_range": [["7.0.1", "7.0.2"]], "choices": ["none", "attacker", "both", "interface"], "type": "str"},
                    "quarantine-expiry": {"v_range": [["7.0.1", "7.0.2"]], "type": "str"},
                    "quarantine-log": {"v_range": [["7.0.1", "7.0.2"]], "choices": ["disable", "enable"], "type": "str"},
                    "rate-count": {"v_range": [["7.0.1", "7.0.2"]], "type": "int"},
                    "rate-duration": {"v_range": [["7.0.1", "7.0.2"]], "type": "int"},
                    "rate-mode": {"v_range": [["7.0.1", "7.0.2"]], "choices": ["periodical", "continuous"], "type": "str"},
                    "rate-track": {
                        "v_range": [["7.0.1", "7.0.2"]],
                        "choices": ["none", "src-ip", "dest-ip", "dhcp-client-mac", "dns-domain"],
                        "type": "str"
                    },
                    "rule": {"v_range": [["7.0.1", "7.0.2"]], "type": "str"},
                    "severity": {"v_range": [["7.0.1", "7.0.2"]], "type": "raw"},
                    "status": {"v_range": [["7.0.1", "7.0.2"]], "choices": ["disable", "enable", "default"], "type": "str"},
                    "tags": {"v_range": [["7.0.1", "7.0.2"]], "type": "str"}
                },
                "elements": "dict"
            },
            "extended-log": {"v_range": [["7.0.1", "7.0.2"]], "choices": ["disable", "enable"], "type": "str"},
            "filter": {
                "v_range": [["7.0.1", "7.0.2"]],
                "type": "list",
                "options": {
                    "action": {"v_range": [["7.0.1", "7.0.2"]], "choices": ["pass", "block", "default", "reset"], "type": "str"},
                    "application": {"v_range": [["7.0.1", "7.0.2"]], "type": "raw"},
                    "application(real)": {"v_range": [["7.0.1", "7.0.2"]], "type": "str"},
                    "location": {"v_range": [["7.0.1", "7.0.2"]], "type": "raw"},
                    "location(real)": {"v_range": [["7.0.1", "7.0.2"]], "type": "str"},
                    "log": {"v_range": [["7.0.1", "7.0.2"]], "choices": ["disable", "enable", "default"], "type": "str"},
                    "log-packet": {"v_range": [["7.0.1", "7.0.2"]], "choices": ["disable", "enable", "default"], "type": "str"},
                    "name": {"v_range": [["7.0.1", "7.0.2"]], "type": "str"},
                    "os": {"v_range": [["7.0.1", "7.0.2"]], "type": "raw"},
                    "os(real)": {"v_range": [["7.0.1", "7.0.2"]], "type": "str"},
                    "protocol": {"v_range": [["7.0.1", "7.0.2"]], "type": "raw"},
                    "protocol(real)": {"v_range": [["7.0.1", "7.0.2"]], "type": "str"},
                    "quarantine": {"v_range": [["7.0.1", "7.0.2"]], "choices": ["none", "attacker", "both", "interface"], "type": "str"},
                    "quarantine-expiry": {"v_range": [["7.0.1", "7.0.2"]], "type": "int"},
                    "quarantine-log": {"v_range": [["7.0.1", "7.0.2"]], "choices": ["disable", "enable"], "type": "str"},
                    "severity": {"v_range": [["7.0.1", "7.0.2"]], "type": "raw"},
                    "severity(real)": {"v_range": [["7.0.1", "7.0.2"]], "type": "str"},
                    "status": {"v_range": [["7.0.1", "7.0.2"]], "choices": ["disable", "enable", "default"], "type": "str"}
                },
                "elements": "dict"
            },
            "log": {"v_range": [["7.0.1", "7.0.2"]], "choices": ["disable", "enable"], "type": "str"},
            "name": {"v_range": [["7.0.1", "7.0.2"]], "required": true, "type": "str"},
            "override": {
                "v_range": [["7.0.1", "7.0.2"]],
                "type": "list",
                "options": {
                    "action": {"v_range": [["7.0.1", "7.0.2"]], "choices": ["pass", "block", "reset"], "type": "str"},
                    "exempt-ip": {
                        "v_range": [["7.0.1", "7.0.2"]],
                        "type": "list",
                        "options": {
                            "dst-ip": {"v_range": [["7.0.1", "7.0.2"]], "type": "str"},
                            "id": {"v_range": [["7.0.1", "7.0.2"]], "type": "int"},
                            "src-ip": {"v_range": [["7.0.1", "7.0.2"]], "type": "str"}
                        },
                        "elements": "dict"
                    },
                    "log": {"v_range": [["7.0.1", "7.0.2"]], "choices": ["disable", "enable"], "type": "str"},
                    "log-packet": {"v_range": [["7.0.1", "7.0.2"]], "choices": ["disable", "enable"], "type": "str"},
                    "quarantine": {"v_range": [["7.0.1", "7.0.2"]], "choices": ["none", "attacker", "both", "interface"], "type": "str"},
                    "quarantine-expiry": {"v_range": [["7.0.1", "7.0.2"]], "type": "int"},
                    "quarantine-log": {"v_range": [["7.0.1", "7.0.2"]], "choices": ["disable", "enable"], "type": "str"},
                    "rule-id": {"v_range": [["7.0.1", "7.0.2"]], "type": "int"},
                    "status": {"v_range": [["7.0.1", "7.0.2"]], "choices": ["disable", "enable"], "type": "str"}
                },
                "elements": "dict"
            },
            "replacemsg-group": {"v_range": [["7.0.1", "7.0.2"]], "type": "str"},
            "scan-botnet-connections": {"v_range": [["7.0.1", "7.0.2"]], "choices": ["disable", "block", "monitor"], "type": "str"}
        }
    },
    "ips_baseline_sensor_entries": {
        "stated": true,
        "options": {
            "action": {"v_range": [["7.0.1", "7.0.2"]], "choices": ["pass", "block", "reset", "default"], "type": "str"},
            "application": {"v_range": [["7.0.1", "7.0.2"]], "type": "raw"},
            "cve": {"v_range": [["7.0.1", "7.0.2"]], "type": "raw"},
            "exempt-ip": {
                "v_range": [["7.0.1", "7.0.2"]],
                "type": "list",
                "options": {
                    "dst-ip": {"v_range": [["7.0.1", "7.0.2"]], "type": "str"},
                    "id": {"v_range": [["7.0.1", "7.0.2"]], "type": "int"},
                    "src-ip": {"v_range": [["7.0.1", "7.0.2"]], "type": "str"}
                },
                "elements": "dict"
            },
            "id": {"v_range": [["7.0.1", "7.0.2"]], "required": true, "type": "int"},
            "location": {"v_range": [["7.0.1", "7.0.2"]], "type": "raw"},
            "log": {"v_range": [["7.0.1", "7.0.2"]], "choices": ["disable", "enable"], "type": "str"},
            "log-attack-context": {"v_range": [["7.0.1", "7.0.2"]], "choices": ["disable", "enable"], "type": "str"},
            "log-packet": {"v_range": [["7.0.1", "7.0.2"]], "choices": ["disable", "enable"], "type": "str"},
            "os": {"v_range": [["7.0.1", "7.0.2"]], "type": "raw"},
            "protocol": {"v_range": [["7.0.1", "7.0.2"]], "type": "raw"},
            "quarantine": {"v_range": [["7.0.1", "7.0.2"]], "choices": ["none", "attacker", "both", "interface"], "type": "str"},
            "quarantine-expiry": {"v_range": [["7.0.1", "7.0.2"]], "type": "str"},
            "quarantine-log": {"v_range": [["7.0.1", "7.0.2"]], "choices": ["disable", "enable"], "type": "str"},
            "rate-count": {"v_range": [["7.0.1", "7.0.2"]], "type": "int"},
            "rate-duration": {"v_range": [["7.0.1", "7.0.2"]], "type": "int"},
            "rate-mode": {"v_range": [["7.0.1", "7.0.2"]], "choices": ["periodical", "continuous"], "type": "str"},
            "rate-track": {"v_range": [["7.0.1", "7.0.2"]], "choices": ["none", "src-ip", "dest-ip", "dhcp-client-mac", "dns-domain"], "type": "str"},
            "rule": {"v_range": [["7.0.1", "7.0.2"]], "type": "str"},
            "severity": {"v_range": [["7.0.1", "7.0.2"]], "type": "raw"},
            "status": {"v_range": [["7.0.1", "7.0.2"]], "choices": ["disable", "enable", "default"], "type": "str"},
            "tags": {"v_range": [["7.0.1", "7.0.2"]], "type": "str"}
        }
    },
    "ips_baseline_sensor_entries_exemptip": {
        "stated": true,
        "options": {
            "dst-ip": {"v_range": [["7.0.1", "7.0.2"]], "type": "str"},
            "id": {"v_range": [["7.0.1", "7.0.2"]], "required": true, "type": "int"},
            "src-ip": {"v_range": [["7.0.1", "7.0.2"]], "type": "str"}
        }
    },
    "ips_baseline_sensor_filter": {
        "stated": true,
        "options": {
            "action": {"v_range": [["7.0.1", "7.0.2"]], "choices": ["pass", "block", "default", "reset"], "type": "str"},
            "application": {"v_range": [["7.0.1", "7.0.2"]], "type": "raw"},
            "application(real)": {"v_range": [["7.0.1", "7.0.2"]], "type": "str"},
            "location": {"v_range": [["7.0.1", "7.0.2"]], "type": "raw"},
            "location(real)": {"v_range": [["7.0.1", "7.0.2"]], "type": "str"},
            "log": {"v_range": [["7.0.1", "7.0.2"]], "choices": ["disable", "enable", "default"], "type": "str"},
            "log-packet": {"v_range": [["7.0.1", "7.0.2"]], "choices": ["disable", "enable", "default"], "type": "str"},
            "name": {"v_range": [["7.0.1", "7.0.2"]], "required": true, "type": "str"},
            "os": {"v_range": [["7.0.1", "7.0.2"]], "type": "raw"},
            "os(real)": {"v_range": [["7.0.1", "7.0.2"]], "type": "str"},
            "protocol": {"v_range": [["7.0.1", "7.0.2"]], "type": "raw"},
            "protocol(real)": {"v_range": [["7.0.1", "7.0.2"]], "type": "str"},
            "quarantine": {"v_range": [["7.0.1", "7.0.2"]], "choices": ["none", "attacker", "both", "interface"], "type": "str"},
            "quarantine-expiry": {"v_range": [["7.0.1", "7.0.2"]], "type": "int"},
            "quarantine-log": {"v_range": [["7.0.1", "7.0.2"]], "choices": ["disable", "enable"], "type": "str"},
            "severity": {"v_range": [["7.0.1", "7.0.2"]], "type": "raw"},
            "severity(real)": {"v_range": [["7.0.1", "7.0.2"]], "type": "str"},
            "status": {"v_range": [["7.0.1", "7.0.2"]], "choices": ["disable", "enable", "default"], "type": "str"}
        }
    },
    "ips_baseline_sensor_override": {
        "stated": true,
        "options": {
            "action": {"v_range": [["7.0.1", "7.0.2"]], "choices": ["pass", "block", "reset"], "type": "str"},
            "exempt-ip": {
                "v_range": [["7.0.1", "7.0.2"]],
                "type": "list",
                "options": {
                    "dst-ip": {"v_range": [["7.0.1", "7.0.2"]], "type": "str"},
                    "id": {"v_range": [["7.0.1", "7.0.2"]], "type": "int"},
                    "src-ip": {"v_range": [["7.0.1", "7.0.2"]], "type": "str"}
                },
                "elements": "dict"
            },
            "log": {"v_range": [["7.0.1", "7.0.2"]], "choices": ["disable", "enable"], "type": "str"},
            "log-packet": {"v_range": [["7.0.1", "7.0.2"]], "choices": ["disable", "enable"], "type": "str"},
            "quarantine": {"v_range": [["7.0.1", "7.0.2"]], "choices": ["none", "attacker", "both", "interface"], "type": "str"},
            "quarantine-expiry": {"v_range": [["7.0.1", "7.0.2"]], "type": "int"},
            "quarantine-log": {"v_range": [["7.0.1", "7.0.2"]], "choices": ["disable", "enable"], "type": "str"},
            "rule-id": {"v_range": [["7.0.1", "7.0.2"]], "type": "int"},
            "status": {"v_range": [["7.0.1", "7.0.2"]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "ips_baseline_sensor_override_exemptip": {
        "stated": true,
        "options": {
            "dst-ip": {"v_range": [["7.0.1", "7.0.2"]], "type": "str"},
            "id": {"v_range": [["7.0.1", "7.0.2"]], "required": true, "type": "int"},
            "src-ip": {"v_range": [["7.0.1", "7.0.2"]], "type": "str"}
        }
    },
    "ips_custom": {
        "stated": true,
        "options": {
            "action": {"choices": ["block", "pass"], "type": "str"},
            "application": {"type": "raw"},
            "comment": {"type": "str"},
            "location": {"type": "raw"},
            "log": {"choices": ["disable", "enable"], "type": "str"},
            "log-packet": {"choices": ["disable", "enable"], "type": "str"},
            "os": {"type": "raw"},
            "protocol": {"type": "str"},
            "rule-id": {"type": "int"},
            "severity": {"type": "str"},
            "sig-name": {"type": "str"},
            "signature": {"type": "str"},
            "status": {"choices": ["disable", "enable"], "type": "str"},
            "tag": {"required": true, "type": "str"}
        }
    },
    "ips_sensor": {
        "stated": true,
        "options": {
            "block-malicious-url": {"choices": ["disable", "enable"], "type": "str"},
            "comment": {"type": "str"},
            "entries": {
                "type": "list",
                "options": {
                    "action": {"choices": ["pass", "block", "reset", "default"], "type": "str"},
                    "application": {"type": "raw"},
                    "exempt-ip": {
                        "type": "list",
                        "options": {"dst-ip": {"type": "str"}, "id": {"type": "int"}, "src-ip": {"type": "str"}},
                        "elements": "dict"
                    },
                    "id": {"type": "int"},
                    "location": {"type": "raw"},
                    "log": {"choices": ["disable", "enable"], "type": "str"},
                    "log-attack-context": {"choices": ["disable", "enable"], "type": "str"},
                    "log-packet": {"choices": ["disable", "enable"], "type": "str"},
                    "os": {"type": "raw"},
                    "protocol": {"type": "raw"},
                    "quarantine": {"choices": ["none", "attacker", "both", "interface"], "type": "str"},
                    "quarantine-expiry": {"type": "str"},
                    "quarantine-log": {"choices": ["disable", "enable"], "type": "str"},
                    "rate-count": {"type": "int"},
                    "rate-duration": {"type": "int"},
                    "rate-mode": {"choices": ["periodical", "continuous"], "type": "str"},
                    "rate-track": {"choices": ["none", "src-ip", "dest-ip", "dhcp-client-mac", "dns-domain"], "type": "str"},
                    "rule": {"type": "raw"},
                    "severity": {"type": "raw"},
                    "status": {"choices": ["disable", "enable", "default"], "type": "str"},
                    "tags": {"v_range": [["6.2.0", "6.4.14"]], "type": "str"},
                    "cve": {"v_range": [["6.4.2", ""]], "type": "raw"},
                    "default-action": {"v_range": [["7.2.0", ""]], "choices": ["block", "pass", "all", "drop"], "type": "str"},
                    "default-status": {"v_range": [["7.2.0", ""]], "choices": ["disable", "enable", "all"], "type": "str"},
                    "last-modified": {"v_range": [["7.2.0", ""]], "type": "raw"},
                    "vuln-type": {"v_range": [["7.2.0", ""]], "type": "raw"}
                },
                "elements": "dict"
            },
            "extended-log": {"choices": ["disable", "enable"], "type": "str"},
            "filter": {
                "v_range": [["6.0.0", "6.2.0"]],
                "type": "list",
                "options": {
                    "action": {"v_range": [["6.0.0", "6.2.0"]], "choices": ["pass", "block", "default", "reset"], "type": "str"},
                    "application": {"v_range": [["6.0.0", "6.2.0"]], "type": "raw"},
                    "location": {"v_range": [["6.0.0", "6.2.0"]], "type": "raw"},
                    "log": {"v_range": [["6.0.0", "6.2.0"]], "choices": ["disable", "enable", "default"], "type": "str"},
                    "log-packet": {"v_range": [["6.0.0", "6.2.0"]], "choices": ["disable", "enable", "default"], "type": "str"},
                    "name": {"v_range": [["6.0.0", "6.2.0"]], "type": "str"},
                    "os": {"v_range": [["6.0.0", "6.2.0"]], "type": "raw"},
                    "protocol": {"v_range": [["6.0.0", "6.2.0"]], "type": "raw"},
                    "quarantine": {"v_range": [["6.0.0", "6.2.0"]], "choices": ["none", "attacker", "both", "interface"], "type": "str"},
                    "quarantine-expiry": {"v_range": [["6.0.0", "6.2.0"]], "type": "int"},
                    "quarantine-log": {"v_range": [["6.0.0", "6.2.0"]], "choices": ["disable", "enable"], "type": "str"},
                    "severity": {"v_range": [["6.0.0", "6.2.0"]], "type": "raw"},
                    "status": {"v_range": [["6.0.0", "6.2.0"]], "choices": ["disable", "enable", "default"], "type": "str"}
                },
                "elements": "dict"
            },
            "name": {"required": true, "type": "str"},
            "override": {
                "v_range": [["6.0.0", "6.2.0"]],
                "type": "list",
                "options": {
                    "action": {"v_range": [["6.0.0", "6.2.0"]], "choices": ["pass", "block", "reset"], "type": "str"},
                    "exempt-ip": {
                        "v_range": [["6.0.0", "6.2.0"]],
                        "type": "list",
                        "options": {
                            "dst-ip": {"v_range": [["6.0.0", "6.2.0"]], "type": "str"},
                            "id": {"v_range": [["6.0.0", "6.2.0"]], "type": "int"},
                            "src-ip": {"v_range": [["6.0.0", "6.2.0"]], "type": "str"}
                        },
                        "elements": "dict"
                    },
                    "log": {"v_range": [["6.0.0", "6.2.0"]], "choices": ["disable", "enable"], "type": "str"},
                    "log-packet": {"v_range": [["6.0.0", "6.2.0"]], "choices": ["disable", "enable"], "type": "str"},
                    "quarantine": {"v_range": [["6.0.0", "6.2.0"]], "choices": ["none", "attacker", "both", "interface"], "type": "str"},
                    "quarantine-expiry": {"v_range": [["6.0.0", "6.2.0"]], "type": "int"},
                    "quarantine-log": {"v_range": [["6.0.0", "6.2.0"]], "choices": ["disable", "enable"], "type": "str"},
                    "rule-id": {"v_range": [["6.0.0", "6.2.0"]], "type": "int"},
                    "status": {"v_range": [["6.0.0", "6.2.0"]], "choices": ["disable", "enable"], "type": "str"}
                },
                "elements": "dict"
            },
            "replacemsg-group": {"type": "str"},
            "scan-botnet-connections": {"v_range": [["6.2.0", ""]], "choices": ["disable", "block", "monitor"], "type": "str"},
            "_baseline": {"v_range": [["7.0.1", "7.0.2"]], "type": "str"}
        }
    },
    "ips_sensor_entries": {
        "stated": true,
        "options": {
            "action": {"choices": ["pass", "block", "reset", "default"], "type": "str"},
            "application": {"type": "raw"},
            "exempt-ip": {"type": "list", "options": {"dst-ip": {"type": "str"}, "id": {"type": "int"}, "src-ip": {"type": "str"}}, "elements": "dict"},
            "id": {"required": true, "type": "int"},
            "location": {"type": "raw"},
            "log": {"choices": ["disable", "enable"], "type": "str"},
            "log-attack-context": {"choices": ["disable", "enable"], "type": "str"},
            "log-packet": {"choices": ["disable", "enable"], "type": "str"},
            "os": {"type": "raw"},
            "protocol": {"type": "raw"},
            "quarantine": {"choices": ["none", "attacker", "both", "interface"], "type": "str"},
            "quarantine-expiry": {"type": "str"},
            "quarantine-log": {"choices": ["disable", "enable"], "type": "str"},
            "rate-count": {"type": "int"},
            "rate-duration": {"type": "int"},
            "rate-mode": {"choices": ["periodical", "continuous"], "type": "str"},
            "rate-track": {"choices": ["none", "src-ip", "dest-ip", "dhcp-client-mac", "dns-domain"], "type": "str"},
            "rule": {"type": "raw"},
            "severity": {"type": "raw"},
            "status": {"choices": ["disable", "enable", "default"], "type": "str"},
            "tags": {"v_range": [["6.2.0", "6.4.14"]], "type": "str"},
            "cve": {"v_range": [["6.4.2", ""]], "type": "raw"},
            "default-action": {"v_range": [["7.2.0", ""]], "choices": ["block", "pass", "all", "drop"], "type": "str"},
            "default-status": {"v_range": [["7.2.0", ""]], "choices": ["disable", "enable", "all"], "type": "str"},
            "last-modified": {"v_range": [["7.2.0", ""]], "type": "raw"},
            "vuln-type": {"v_range": [["7.2.0", ""]], "type": "raw"}
        }
    },
    "ips_sensor_entries_exemptip": {
        "stated": true,
        "options": {"dst-ip": {"type": "str"}, "id": {"required": true, "type": "int"}, "src-ip": {"type": "str"}}
    },
    "log_customfield": {"stated": true, "options": {"id": {"required": true, "type": "str"}, "name": {"type": "str"}, "value": {"type": "str"}}},
    "log_npuserver": {
        "stated": false,
        "options": {
            "log-processing": {"v_range": [["6.4.8", "6.4.14"], ["7.0.3", ""]], "choices": ["may-drop", "no-drop"], "type": "str"},
            "log-processor": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["hardware", "host"], "type": "str"},
            "netflow-ver": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["v9", "v10"], "type": "str"},
            "server-group": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "type": "list",
                "options": {
                    "group-name": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "str"},
                    "log-format": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["syslog", "netflow"], "type": "str"},
                    "log-mode": {
                        "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                        "choices": ["per-session", "per-nat-mapping", "per-session-ending"],
                        "type": "str"
                    },
                    "log-tx-mode": {"v_range": [["6.4.7", "6.4.14"], ["7.0.2", ""]], "choices": ["multicast", "roundrobin"], "type": "str"},
                    "server-number": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"},
                    "server-start-id": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"},
                    "sw-log-flags": {"v_range": [["6.4.8", "6.4.14"], ["7.0.3", ""]], "type": "raw"},
                    "log-gen-event": {"v_range": [["7.0.4", "7.0.12"], ["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "log-user-info": {"v_range": [["7.0.4", "7.0.12"], ["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"}
                },
                "elements": "dict"
            },
            "server-info": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "type": "list",
                "options": {
                    "dest-port": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"},
                    "id": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"},
                    "ip-family": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["v4", "v6"], "type": "str"},
                    "ipv4-server": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "str"},
                    "ipv6-server": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "str"},
                    "source-port": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"},
                    "template-tx-timeout": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"},
                    "vdom": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "str"},
                    "log-transport": {"v_range": [["7.4.2", ""]], "choices": ["udp", "tcp"], "type": "str"}
                },
                "elements": "dict"
            },
            "syslog-facility": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"},
            "syslog-severity": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"},
            "enforce-seq-order": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "log_npuserver_servergroup": {
        "stated": true,
        "options": {
            "group-name": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "str"},
            "log-format": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["syslog", "netflow"], "type": "str"},
            "log-mode": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "choices": ["per-session", "per-nat-mapping", "per-session-ending"],
                "type": "str"
            },
            "log-tx-mode": {"v_range": [["6.4.7", "6.4.14"], ["7.0.2", ""]], "choices": ["multicast", "roundrobin"], "type": "str"},
            "server-number": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"},
            "server-start-id": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"},
            "sw-log-flags": {"v_range": [["6.4.8", "6.4.14"], ["7.0.3", ""]], "type": "raw"},
            "log-gen-event": {"v_range": [["7.0.4", "7.0.12"], ["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "log-user-info": {"v_range": [["7.0.4", "7.0.12"], ["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "log_npuserver_serverinfo": {
        "stated": true,
        "options": {
            "dest-port": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"},
            "id": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "required": true, "type": "int"},
            "ip-family": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["v4", "v6"], "type": "str"},
            "ipv4-server": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "str"},
            "ipv6-server": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "str"},
            "source-port": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"},
            "template-tx-timeout": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"},
            "vdom": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "str"},
            "log-transport": {"v_range": [["7.4.2", ""]], "choices": ["udp", "tcp"], "type": "str"}
        }
    },
    "metafields_system_admin_user": {
        "stated": false,
        "options": {
            "importance": {"choices": ["optional", "required"], "type": "str"},
            "length": {"type": "int"},
            "name": {"type": "str"},
            "status": {"choices": ["disable", "enable"], "type": "str"}
        }
    },
    "mpskprofile": {
        "stated": true,
        "options": {
            "mpsk-concurrent-clients": {"v_range": [["6.4.2", ""]], "type": "int"},
            "mpsk-group": {
                "v_range": [["6.4.2", ""]],
                "type": "list",
                "options": {
                    "mpsk-key": {
                        "v_range": [["6.4.2", ""]],
                        "no_log": true,
                        "type": "list",
                        "options": {
                            "comment": {"v_range": [["6.4.2", ""]], "type": "str"},
                            "concurrent-client-limit-type": {"v_range": [["6.4.2", ""]], "choices": ["default", "unlimited", "specified"], "type": "str"},
                            "concurrent-clients": {"v_range": [["6.4.2", ""]], "type": "int"},
                            "mac": {"v_range": [["6.4.2", ""]], "type": "str"},
                            "mpsk-schedules": {"v_range": [["6.4.2", ""]], "type": "raw"},
                            "name": {"v_range": [["6.4.2", ""]], "type": "str"},
                            "passphrase": {"v_range": [["6.4.2", ""]], "no_log": true, "type": "raw"},
                            "pmk": {"v_range": [["6.4.2", ""]], "type": "raw"},
                            "key-type": {"v_range": [["7.4.3", ""]], "choices": ["wpa2-personal", "wpa3-sae"], "type": "str"},
                            "sae-password": {"v_range": [["7.4.3", ""]], "no_log": true, "type": "raw"},
                            "sae-pk": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "sae-private-key": {"v_range": [["7.4.3", ""]], "no_log": true, "type": "str"}
                        },
                        "elements": "dict"
                    },
                    "name": {"v_range": [["6.4.2", ""]], "type": "str"},
                    "vlan-id": {"v_range": [["6.4.2", ""]], "type": "int"},
                    "vlan-type": {"v_range": [["6.4.2", ""]], "choices": ["no-vlan", "fixed-vlan"], "type": "str"}
                },
                "elements": "dict"
            },
            "name": {"v_range": [["6.4.2", ""]], "required": true, "type": "str"},
            "ssid": {"v_range": [["6.4.2", ""]], "type": "str"},
            "mpsk-external-server": {"v_range": [["7.4.3", ""]], "type": "raw"},
            "mpsk-external-server-auth": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "mpsk-type": {"v_range": [["7.4.3", ""]], "choices": ["wpa2-personal", "wpa3-sae", "wpa3-sae-transition"], "type": "str"}
        }
    },
    "mpskprofile_mpskgroup": {
        "stated": true,
        "options": {
            "mpsk-key": {
                "v_range": [["6.4.2", ""]],
                "no_log": true,
                "type": "list",
                "options": {
                    "comment": {"v_range": [["6.4.2", ""]], "type": "str"},
                    "concurrent-client-limit-type": {"v_range": [["6.4.2", ""]], "choices": ["default", "unlimited", "specified"], "type": "str"},
                    "concurrent-clients": {"v_range": [["6.4.2", ""]], "type": "int"},
                    "mac": {"v_range": [["6.4.2", ""]], "type": "str"},
                    "mpsk-schedules": {"v_range": [["6.4.2", ""]], "type": "raw"},
                    "name": {"v_range": [["6.4.2", ""]], "type": "str"},
                    "passphrase": {"v_range": [["6.4.2", ""]], "no_log": true, "type": "raw"},
                    "pmk": {"v_range": [["6.4.2", ""]], "type": "raw"},
                    "key-type": {"v_range": [["7.4.3", ""]], "choices": ["wpa2-personal", "wpa3-sae"], "type": "str"},
                    "sae-password": {"v_range": [["7.4.3", ""]], "no_log": true, "type": "raw"},
                    "sae-pk": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "sae-private-key": {"v_range": [["7.4.3", ""]], "no_log": true, "type": "str"}
                },
                "elements": "dict"
            },
            "name": {"v_range": [["6.4.2", ""]], "required": true, "type": "str"},
            "vlan-id": {"v_range": [["6.4.2", ""]], "type": "int"},
            "vlan-type": {"v_range": [["6.4.2", ""]], "choices": ["no-vlan", "fixed-vlan"], "type": "str"}
        }
    },
    "mpskprofile_mpskgroup_mpskkey": {
        "stated": true,
        "options": {
            "comment": {"v_range": [["6.4.2", ""]], "type": "str"},
            "concurrent-client-limit-type": {"v_range": [["6.4.2", ""]], "choices": ["default", "unlimited", "specified"], "type": "str"},
            "concurrent-clients": {"v_range": [["6.4.2", ""]], "type": "int"},
            "mac": {"v_range": [["6.4.2", ""]], "type": "str"},
            "mpsk-schedules": {"v_range": [["6.4.2", ""]], "type": "raw"},
            "name": {"v_range": [["6.4.2", ""]], "required": true, "type": "str"},
            "passphrase": {"v_range": [["6.4.2", ""]], "no_log": true, "type": "raw"},
            "pmk": {"v_range": [["6.4.2", ""]], "type": "raw"},
            "key-type": {"v_range": [["7.4.3", ""]], "choices": ["wpa2-personal", "wpa3-sae"], "type": "str"},
            "sae-password": {"v_range": [["7.4.3", ""]], "no_log": true, "type": "raw"},
            "sae-pk": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "sae-private-key": {"v_range": [["7.4.3", ""]], "no_log": true, "type": "str"}
        }
    },
    "nacprofile": {
        "stated": true,
        "options": {
            "comment": {"v_range": [["7.0.3", ""]], "type": "str"},
            "name": {"v_range": [["7.0.3", ""]], "required": true, "type": "str"},
            "onboarding-vlan": {"v_range": [["7.0.3", ""]], "type": "str"}
        }
    },
    "pkg_authentication_rule": {
        "stated": true,
        "options": {
            "active-auth-method": {"v_range": [["6.2.1", ""]], "type": "str"},
            "comments": {"v_range": [["6.2.1", ""]], "type": "str"},
            "ip-based": {"v_range": [["6.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "name": {"v_range": [["6.2.1", ""]], "required": true, "type": "str"},
            "protocol": {"v_range": [["6.2.1", ""]], "choices": ["http", "ftp", "socks", "ssh"], "type": "str"},
            "srcaddr": {"v_range": [["6.2.1", ""]], "type": "raw"},
            "srcaddr6": {"v_range": [["6.2.1", ""]], "type": "raw"},
            "sso-auth-method": {"v_range": [["6.2.1", ""]], "type": "str"},
            "status": {"v_range": [["6.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "transaction-based": {"v_range": [["6.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "web-auth-cookie": {"v_range": [["6.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "web-portal": {"v_range": [["6.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "dstaddr": {"v_range": [["7.0.0", ""]], "type": "raw"},
            "dstaddr6": {"v_range": [["7.0.0", ""]], "type": "raw"},
            "srcintf": {"v_range": [["7.0.0", ""]], "type": "raw"},
            "cors-depth": {"v_range": [["7.4.1", ""]], "type": "int"},
            "cors-stateful": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "cert-auth-cookie": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "pkg_authentication_setting": {
        "stated": false,
        "options": {
            "active-auth-scheme": {"v_range": [["6.2.1", ""]], "type": "str"},
            "auth-https": {"v_range": [["6.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "captive-portal": {"v_range": [["6.2.1", ""]], "type": "str"},
            "captive-portal-ip": {"v_range": [["6.2.1", ""]], "type": "str"},
            "captive-portal-ip6": {"v_range": [["6.2.1", ""]], "type": "str"},
            "captive-portal-port": {"v_range": [["6.2.1", ""]], "type": "int"},
            "captive-portal-ssl-port": {"v_range": [["6.2.1", ""]], "type": "int"},
            "captive-portal-type": {"v_range": [["6.2.1", ""]], "choices": ["fqdn", "ip"], "type": "str"},
            "captive-portal6": {"v_range": [["6.2.1", ""]], "type": "str"},
            "rewrite-https-port": {"v_range": [["6.2.1", ""]], "type": "int"},
            "sso-auth-scheme": {"v_range": [["6.2.1", ""]], "type": "str"},
            "dev-range": {"v_range": [["7.0.0", ""]], "type": "raw"},
            "user-cert-ca": {"v_range": [["7.0.0", ""]], "type": "raw"},
            "cert-auth": {"v_range": [["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "cert-captive-portal": {"v_range": [["7.0.1", ""]], "type": "str"},
            "cert-captive-portal-ip": {"v_range": [["7.0.1", ""]], "type": "str"},
            "cert-captive-portal-port": {"v_range": [["7.0.1", ""]], "type": "int"},
            "cookie-max-age": {"v_range": [["7.2.0", ""]], "type": "int"},
            "cookie-refresh-div": {"v_range": [["7.2.0", ""]], "type": "int"},
            "ip-auth-cookie": {"v_range": [["7.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "persistent-cookie": {"v_range": [["7.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "update-time": {"v_range": [["7.2.0", ""]], "type": "str"}
        }
    },
    "pkg_central_dnat": {"stated": true, "options": {"name": {"required": true, "type": "raw"}}},
    "pkg_central_dnat6": {"stated": true, "options": {"name": {"v_range": [["6.4.2", ""]], "required": true, "type": "raw"}}},
    "pkg_firewall_acl": {
        "stated": true,
        "options": {
            "comments": {"v_range": [["7.2.0", "7.2.0"], ["7.4.3", ""]], "type": "str"},
            "dstaddr": {"v_range": [["7.2.0", "7.2.0"], ["7.4.3", ""]], "type": "raw"},
            "interface": {"v_range": [["7.2.0", "7.2.0"], ["7.4.3", ""]], "type": "str"},
            "name": {"v_range": [["7.2.0", "7.2.0"], ["7.4.3", ""]], "type": "str"},
            "policyid": {"v_range": [["7.2.0", "7.2.0"], ["7.4.3", ""]], "required": true, "type": "int"},
            "service": {"v_range": [["7.2.0", "7.2.0"], ["7.4.3", ""]], "type": "raw"},
            "srcaddr": {"v_range": [["7.2.0", "7.2.0"], ["7.4.3", ""]], "type": "raw"},
            "status": {"v_range": [["7.2.0", "7.2.0"], ["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "uuid": {"v_range": [["7.2.0", "7.2.0"]], "type": "str"},
            "fragment": {"v_range": [["7.4.3", ""]], "choices": ["pass", "drop"], "type": "str"}
        }
    },
    "pkg_firewall_acl6": {
        "stated": true,
        "options": {
            "comments": {"v_range": [["7.2.0", "7.2.0"], ["7.4.3", ""]], "type": "str"},
            "dstaddr": {"v_range": [["7.2.0", "7.2.0"], ["7.4.3", ""]], "type": "raw"},
            "interface": {"v_range": [["7.2.0", "7.2.0"], ["7.4.3", ""]], "type": "str"},
            "name": {"v_range": [["7.2.0", "7.2.0"], ["7.4.3", ""]], "type": "str"},
            "policyid": {"v_range": [["7.2.0", "7.2.0"], ["7.4.3", ""]], "required": true, "type": "int"},
            "service": {"v_range": [["7.2.0", "7.2.0"], ["7.4.3", ""]], "type": "raw"},
            "srcaddr": {"v_range": [["7.2.0", "7.2.0"], ["7.4.3", ""]], "type": "raw"},
            "status": {"v_range": [["7.2.0", "7.2.0"], ["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "uuid": {"v_range": [["7.2.0", "7.2.0"]], "type": "str"},
            "fragment": {"v_range": [["7.4.3", ""]], "choices": ["pass", "drop"], "type": "str"}
        }
    },
    "pkg_firewall_centralsnatmap": {
        "stated": true,
        "options": {
            "dst-addr": {"type": "raw"},
            "dstintf": {"type": "raw"},
            "nat": {"choices": ["disable", "enable"], "type": "str"},
            "nat-ippool": {"type": "raw"},
            "nat-port": {"type": "str"},
            "orig-addr": {"type": "raw"},
            "orig-port": {"type": "raw"},
            "policyid": {"required": true, "type": "int"},
            "protocol": {"type": "int"},
            "srcintf": {"type": "raw"},
            "status": {"choices": ["disable", "enable"], "type": "str"},
            "comments": {"v_range": [["6.2.0", ""]], "type": "str"},
            "dst-addr6": {"v_range": [["6.4.0", ""]], "type": "raw"},
            "nat-ippool6": {"v_range": [["6.4.0", ""]], "type": "raw"},
            "orig-addr6": {"v_range": [["6.4.0", ""]], "type": "raw"},
            "type": {"v_range": [["6.4.0", ""]], "choices": ["ipv4", "ipv6"], "type": "str"},
            "uuid": {"v_range": [["6.4.0", ""]], "type": "str"},
            "nat46": {"v_range": [["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "nat64": {"v_range": [["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "dst-port": {"v_range": [["7.4.0", ""]], "type": "str"},
            "port-preserve": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "pkg_firewall_consolidated_policy": {
        "stated": true,
        "options": {
            "action": {"v_range": [["6.2.0", ""]], "choices": ["deny", "accept", "ipsec"], "type": "str"},
            "app-category": {"v_range": [["6.2.0", "7.4.2"]], "type": "raw"},
            "app-group": {"v_range": [["6.2.0", "7.4.2"]], "type": "raw"},
            "application": {"v_range": [["6.2.0", "7.4.2"]], "type": "raw"},
            "application-list": {"v_range": [["6.2.0", ""]], "type": "str"},
            "auto-asic-offload": {"v_range": [["6.2.0", "7.2.0"], ["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "av-profile": {"v_range": [["6.2.0", ""]], "type": "str"},
            "cifs-profile": {"v_range": [["6.2.0", ""]], "type": "str"},
            "comments": {"v_range": [["6.2.0", ""]], "type": "str"},
            "diffserv-forward": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "diffserv-reverse": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "diffservcode-forward": {"v_range": [["6.2.0", ""]], "type": "str"},
            "diffservcode-rev": {"v_range": [["6.2.0", ""]], "type": "str"},
            "dlp-sensor": {"v_range": [["6.2.0", ""]], "type": "str"},
            "dnsfilter-profile": {"v_range": [["6.2.0", ""]], "type": "str"},
            "dstaddr4": {"v_range": [["6.2.0", ""]], "type": "raw"},
            "dstaddr6": {"v_range": [["6.2.0", ""]], "type": "raw"},
            "dstintf": {"v_range": [["6.2.0", ""]], "type": "raw"},
            "emailfilter-profile": {"v_range": [["6.2.0", ""]], "type": "str"},
            "fixedport": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "groups": {"v_range": [["6.2.0", ""]], "type": "raw"},
            "http-policy-redirect": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "icap-profile": {"v_range": [["6.2.0", ""]], "type": "str"},
            "inbound": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "inspection-mode": {"v_range": [["6.2.0", ""]], "choices": ["proxy", "flow"], "type": "str"},
            "internet-service": {"v_range": [["6.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "internet-service-custom": {"v_range": [["6.2.1", ""]], "type": "raw"},
            "internet-service-custom-group": {"v_range": [["6.2.1", ""]], "type": "raw"},
            "internet-service-group": {"v_range": [["6.2.1", ""]], "type": "raw"},
            "internet-service-id": {"v_range": [["6.2.1", ""]], "type": "raw"},
            "internet-service-src": {"v_range": [["6.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "internet-service-src-custom": {"v_range": [["6.2.1", ""]], "type": "raw"},
            "internet-service-src-custom-group": {"v_range": [["6.2.1", ""]], "type": "raw"},
            "internet-service-src-group": {"v_range": [["6.2.1", ""]], "type": "raw"},
            "internet-service-src-id": {"v_range": [["6.2.1", ""]], "type": "raw"},
            "ippool": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ips-sensor": {"v_range": [["6.2.0", ""]], "type": "str"},
            "logtraffic": {"v_range": [["6.2.0", ""]], "choices": ["disable", "all", "utm"], "type": "str"},
            "logtraffic-start": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "mms-profile": {"v_range": [["6.2.0", "7.2.0"], ["7.4.3", ""]], "type": "str"},
            "name": {"v_range": [["6.2.0", ""]], "type": "str"},
            "nat": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "outbound": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "per-ip-shaper": {"v_range": [["6.2.0", ""]], "type": "str"},
            "policyid": {"v_range": [["6.2.0", ""]], "required": true, "type": "int"},
            "poolname4": {"v_range": [["6.2.0", ""]], "type": "raw"},
            "poolname6": {"v_range": [["6.2.0", ""]], "type": "raw"},
            "profile-group": {"v_range": [["6.2.0", ""]], "type": "str"},
            "profile-protocol-options": {"v_range": [["6.2.0", "7.2.0"], ["7.4.3", ""]], "type": "str"},
            "profile-type": {"v_range": [["6.2.0", ""]], "choices": ["single", "group"], "type": "str"},
            "schedule": {"v_range": [["6.2.0", ""]], "type": "str"},
            "service": {"v_range": [["6.2.0", ""]], "type": "raw"},
            "session-ttl": {"v_range": [["6.2.0", ""]], "type": "int"},
            "srcaddr4": {"v_range": [["6.2.0", ""]], "type": "raw"},
            "srcaddr6": {"v_range": [["6.2.0", ""]], "type": "raw"},
            "srcintf": {"v_range": [["6.2.0", ""]], "type": "raw"},
            "ssh-filter-profile": {"v_range": [["6.2.0", "7.2.0"], ["7.4.3", ""]], "type": "str"},
            "ssh-policy-redirect": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ssl-ssh-profile": {"v_range": [["6.2.0", "7.2.0"], ["7.4.3", ""]], "type": "str"},
            "status": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "tcp-mss-receiver": {"v_range": [["6.2.0", ""]], "type": "int"},
            "tcp-mss-sender": {"v_range": [["6.2.0", ""]], "type": "int"},
            "traffic-shaper": {"v_range": [["6.2.0", ""]], "type": "str"},
            "traffic-shaper-reverse": {"v_range": [["6.2.0", ""]], "type": "str"},
            "url-category": {"v_range": [["6.2.0", "7.4.2"]], "type": "raw"},
            "users": {"v_range": [["6.2.0", ""]], "type": "raw"},
            "utm-status": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "uuid": {"v_range": [["6.2.0", ""]], "type": "str"},
            "voip-profile": {"v_range": [["6.2.0", ""]], "type": "str"},
            "vpntunnel": {"v_range": [["6.2.0", ""]], "type": "str"},
            "waf-profile": {"v_range": [["6.2.0", ""]], "type": "str"},
            "wanopt": {"v_range": [["6.2.1", "7.2.0"], ["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "wanopt-detection": {"v_range": [["6.2.1", "7.2.0"], ["7.4.3", ""]], "choices": ["active", "passive", "off"], "type": "str"},
            "wanopt-passive-opt": {
                "v_range": [["6.2.1", "7.2.0"], ["7.4.3", ""]],
                "choices": ["default", "transparent", "non-transparent"],
                "type": "str"
            },
            "wanopt-peer": {"v_range": [["6.2.1", "7.2.0"], ["7.4.3", ""]], "type": "str"},
            "wanopt-profile": {"v_range": [["6.2.1", "7.2.0"], ["7.4.3", ""]], "type": "str"},
            "webcache": {"v_range": [["6.2.1", "7.2.0"], ["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "webcache-https": {"v_range": [["6.2.1", "7.2.0"], ["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "webfilter-profile": {"v_range": [["6.2.0", ""]], "type": "str"},
            "webproxy-forward-server": {"v_range": [["6.2.1", ""]], "type": "str"},
            "webproxy-profile": {"v_range": [["6.2.1", ""]], "type": "str"},
            "captive-portal-exempt": {"v_range": [["6.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "dstaddr-negate": {"v_range": [["6.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "fsso-groups": {"v_range": [["6.2.3", ""]], "type": "raw"},
            "global-label": {"v_range": [["6.2.3", ""]], "type": "str"},
            "internet-service-negate": {"v_range": [["6.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "internet-service-src-negate": {"v_range": [["6.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "service-negate": {"v_range": [["6.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "srcaddr-negate": {"v_range": [["6.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "_policy_block": {"v_range": [["7.0.3", ""]], "type": "int"}
        }
    },
    "pkg_firewall_consolidated_policy_sectionvalue": {
        "stated": true,
        "options": {
            "attr": {"v_range": [["6.2.5", ""]], "choices": ["label", "global-label"], "type": "str"},
            "name": {"v_range": [["6.2.5", ""]], "type": "str"}
        }
    },
    "pkg_firewall_dospolicy": {
        "stated": true,
        "options": {
            "anomaly": {
                "type": "list",
                "options": {
                    "action": {"choices": ["pass", "block", "proxy"], "type": "str"},
                    "log": {"choices": ["disable", "enable"], "type": "str"},
                    "name": {"type": "str"},
                    "quarantine": {"choices": ["none", "attacker", "both", "interface"], "type": "str"},
                    "quarantine-expiry": {"type": "str"},
                    "quarantine-log": {"choices": ["disable", "enable"], "type": "str"},
                    "status": {"choices": ["disable", "enable"], "type": "str"},
                    "threshold": {"type": "int"},
                    "threshold(default)": {"type": "int"},
                    "synproxy_tos": {
                        "v_range": [["6.2.5", "7.2.0"]],
                        "choices": ["0", "10", "12", "14", "18", "20", "22", "26", "28", "30", "34", "36", "38", "40", "46", "255"],
                        "type": "str"
                    },
                    "synproxy_ttl": {"v_range": [["6.2.5", "7.2.0"]], "choices": ["32", "64", "128", "255"], "type": "str"},
                    "synproxy_tcp_sack": {"v_range": [["6.2.5", "7.2.0"]], "choices": ["disable", "enable"], "type": "str"},
                    "synproxy_tcp_window": {"v_range": [["6.2.5", "7.2.0"]], "choices": ["4096", "8192", "16384", "32768"], "type": "str"},
                    "synproxy_tcp_timestamp": {"v_range": [["6.2.5", "7.2.0"]], "choices": ["disable", "enable"], "type": "str"},
                    "synproxy_tcp_mss": {
                        "v_range": [["6.2.5", "7.2.0"]],
                        "choices": ["0", "256", "512", "1024", "1300", "1360", "1460", "1500"],
                        "type": "str"
                    },
                    "synproxy_tcp_windowscale": {
                        "v_range": [["6.2.5", "7.2.0"]],
                        "choices": ["0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "10", "11", "12", "13", "14"],
                        "type": "str"
                    },
                    "synproxy-tos": {
                        "v_range": [["6.2.6", "6.2.12"], ["6.4.2", "7.2.0"], ["7.4.3", ""]],
                        "choices": ["0", "10", "12", "14", "18", "20", "22", "26", "28", "30", "34", "36", "38", "40", "46", "255"],
                        "type": "str"
                    },
                    "synproxy-tcp-window": {
                        "v_range": [["6.2.6", "6.2.12"], ["6.4.2", "7.2.0"], ["7.4.3", ""]],
                        "choices": ["4096", "8192", "16384", "32768"],
                        "type": "str"
                    },
                    "synproxy-tcp-windowscale": {
                        "v_range": [["6.2.6", "6.2.12"], ["6.4.2", "7.2.0"], ["7.4.3", ""]],
                        "choices": ["0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "10", "11", "12", "13", "14"],
                        "type": "str"
                    },
                    "synproxy-tcp-timestamp": {
                        "v_range": [["6.2.6", "6.2.12"], ["6.4.2", "7.2.0"], ["7.4.3", ""]],
                        "choices": ["disable", "enable"],
                        "type": "str"
                    },
                    "synproxy-ttl": {
                        "v_range": [["6.2.6", "6.2.12"], ["6.4.2", "7.2.0"], ["7.4.3", ""]],
                        "choices": ["32", "64", "128", "255"],
                        "type": "str"
                    },
                    "synproxy-tcp-mss": {
                        "v_range": [["6.2.6", "6.2.12"], ["6.4.2", "7.2.0"], ["7.4.3", ""]],
                        "choices": ["0", "256", "512", "1024", "1300", "1360", "1460", "1500"],
                        "type": "str"
                    },
                    "synproxy-tcp-sack": {
                        "v_range": [["6.2.6", "6.2.12"], ["6.4.2", "7.2.0"], ["7.4.3", ""]],
                        "choices": ["disable", "enable"],
                        "type": "str"
                    }
                },
                "elements": "dict"
            },
            "comments": {"type": "str"},
            "dstaddr": {"type": "raw"},
            "interface": {"type": "str"},
            "policyid": {"required": true, "type": "int"},
            "service": {"type": "raw"},
            "srcaddr": {"type": "raw"},
            "status": {"choices": ["disable", "enable"], "type": "str"},
            "uuid": {"v_range": [["6.2.1", "7.2.0"]], "type": "str"},
            "name": {"v_range": [["6.4.2", ""]], "type": "str"}
        }
    },
    "pkg_firewall_dospolicy6": {
        "stated": true,
        "options": {
            "anomaly": {
                "type": "list",
                "options": {
                    "action": {"choices": ["pass", "block", "proxy"], "type": "str"},
                    "log": {"choices": ["disable", "enable"], "type": "str"},
                    "name": {"type": "str"},
                    "quarantine": {"choices": ["none", "attacker", "both", "interface"], "type": "str"},
                    "quarantine-expiry": {"type": "str"},
                    "quarantine-log": {"choices": ["disable", "enable"], "type": "str"},
                    "status": {"choices": ["disable", "enable"], "type": "str"},
                    "threshold": {"type": "int"},
                    "threshold(default)": {"type": "int"},
                    "synproxy_tos": {
                        "v_range": [["6.2.5", "7.2.0"]],
                        "choices": ["0", "10", "12", "14", "18", "20", "22", "26", "28", "30", "34", "36", "38", "40", "46", "255"],
                        "type": "str"
                    },
                    "synproxy_ttl": {"v_range": [["6.2.5", "7.2.0"]], "choices": ["32", "64", "128", "255"], "type": "str"},
                    "synproxy_tcp_sack": {"v_range": [["6.2.5", "7.2.0"]], "choices": ["disable", "enable"], "type": "str"},
                    "synproxy_tcp_window": {"v_range": [["6.2.5", "7.2.0"]], "choices": ["4096", "8192", "16384", "32768"], "type": "str"},
                    "synproxy_tcp_timestamp": {"v_range": [["6.2.5", "7.2.0"]], "choices": ["disable", "enable"], "type": "str"},
                    "synproxy_tcp_mss": {
                        "v_range": [["6.2.5", "7.2.0"]],
                        "choices": ["0", "256", "512", "1024", "1300", "1360", "1460", "1500"],
                        "type": "str"
                    },
                    "synproxy_tcp_windowscale": {
                        "v_range": [["6.2.5", "7.2.0"]],
                        "choices": ["0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "10", "11", "12", "13", "14"],
                        "type": "str"
                    },
                    "synproxy-tos": {
                        "v_range": [["6.2.6", "6.2.12"], ["6.4.2", "7.2.0"], ["7.4.3", ""]],
                        "choices": ["0", "10", "12", "14", "18", "20", "22", "26", "28", "30", "34", "36", "38", "40", "46", "255"],
                        "type": "str"
                    },
                    "synproxy-tcp-window": {
                        "v_range": [["6.2.6", "6.2.12"], ["6.4.2", "7.2.0"], ["7.4.3", ""]],
                        "choices": ["4096", "8192", "16384", "32768"],
                        "type": "str"
                    },
                    "synproxy-tcp-windowscale": {
                        "v_range": [["6.2.6", "6.2.12"], ["6.4.2", "7.2.0"], ["7.4.3", ""]],
                        "choices": ["0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "10", "11", "12", "13", "14"],
                        "type": "str"
                    },
                    "synproxy-tcp-timestamp": {
                        "v_range": [["6.2.6", "6.2.12"], ["6.4.2", "7.2.0"], ["7.4.3", ""]],
                        "choices": ["disable", "enable"],
                        "type": "str"
                    },
                    "synproxy-ttl": {
                        "v_range": [["6.2.6", "6.2.12"], ["6.4.2", "7.2.0"], ["7.4.3", ""]],
                        "choices": ["32", "64", "128", "255"],
                        "type": "str"
                    },
                    "synproxy-tcp-mss": {
                        "v_range": [["6.2.6", "6.2.12"], ["6.4.2", "7.2.0"], ["7.4.3", ""]],
                        "choices": ["0", "256", "512", "1024", "1300", "1360", "1460", "1500"],
                        "type": "str"
                    },
                    "synproxy-tcp-sack": {
                        "v_range": [["6.2.6", "6.2.12"], ["6.4.2", "7.2.0"], ["7.4.3", ""]],
                        "choices": ["disable", "enable"],
                        "type": "str"
                    }
                },
                "elements": "dict"
            },
            "comments": {"type": "str"},
            "dstaddr": {"type": "raw"},
            "interface": {"type": "str"},
            "policyid": {"required": true, "type": "int"},
            "service": {"type": "raw"},
            "srcaddr": {"type": "raw"},
            "status": {"choices": ["disable", "enable"], "type": "str"},
            "uuid": {"v_range": [["6.2.1", "7.2.0"]], "type": "str"},
            "name": {"v_range": [["6.4.2", ""]], "type": "str"}
        }
    },
    "pkg_firewall_dospolicy6_anomaly": {
        "stated": true,
        "options": {
            "action": {"choices": ["pass", "block", "proxy"], "type": "str"},
            "log": {"choices": ["disable", "enable"], "type": "str"},
            "name": {"required": true, "type": "str"},
            "quarantine": {"choices": ["none", "attacker", "both", "interface"], "type": "str"},
            "quarantine-expiry": {"type": "str"},
            "quarantine-log": {"choices": ["disable", "enable"], "type": "str"},
            "status": {"choices": ["disable", "enable"], "type": "str"},
            "threshold": {"type": "int"},
            "threshold(default)": {"type": "int"},
            "synproxy_tos": {
                "v_range": [["6.2.5", "7.2.0"]],
                "choices": ["0", "10", "12", "14", "18", "20", "22", "26", "28", "30", "34", "36", "38", "40", "46", "255"],
                "type": "str"
            },
            "synproxy_ttl": {"v_range": [["6.2.5", "7.2.0"]], "choices": ["32", "64", "128", "255"], "type": "str"},
            "synproxy_tcp_sack": {"v_range": [["6.2.5", "7.2.0"]], "choices": ["disable", "enable"], "type": "str"},
            "synproxy_tcp_window": {"v_range": [["6.2.5", "7.2.0"]], "choices": ["4096", "8192", "16384", "32768"], "type": "str"},
            "synproxy_tcp_timestamp": {"v_range": [["6.2.5", "7.2.0"]], "choices": ["disable", "enable"], "type": "str"},
            "synproxy_tcp_mss": {"v_range": [["6.2.5", "7.2.0"]], "choices": ["0", "256", "512", "1024", "1300", "1360", "1460", "1500"], "type": "str"},
            "synproxy_tcp_windowscale": {
                "v_range": [["6.2.5", "7.2.0"]],
                "choices": ["0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "10", "11", "12", "13", "14"],
                "type": "str"
            },
            "synproxy-tos": {
                "v_range": [["6.2.6", "6.2.12"], ["6.4.2", "7.2.0"], ["7.4.3", ""]],
                "choices": ["0", "10", "12", "14", "18", "20", "22", "26", "28", "30", "34", "36", "38", "40", "46", "255"],
                "type": "str"
            },
            "synproxy-tcp-window": {
                "v_range": [["6.2.6", "6.2.12"], ["6.4.2", "7.2.0"], ["7.4.3", ""]],
                "choices": ["4096", "8192", "16384", "32768"],
                "type": "str"
            },
            "synproxy-tcp-windowscale": {
                "v_range": [["6.2.6", "6.2.12"], ["6.4.2", "7.2.0"], ["7.4.3", ""]],
                "choices": ["0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "10", "11", "12", "13", "14"],
                "type": "str"
            },
            "synproxy-tcp-timestamp": {
                "v_range": [["6.2.6", "6.2.12"], ["6.4.2", "7.2.0"], ["7.4.3", ""]],
                "choices": ["disable", "enable"],
                "type": "str"
            },
            "synproxy-ttl": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", "7.2.0"], ["7.4.3", ""]], "choices": ["32", "64", "128", "255"], "type": "str"},
            "synproxy-tcp-mss": {
                "v_range": [["6.2.6", "6.2.12"], ["6.4.2", "7.2.0"], ["7.4.3", ""]],
                "choices": ["0", "256", "512", "1024", "1300", "1360", "1460", "1500"],
                "type": "str"
            },
            "synproxy-tcp-sack": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", "7.2.0"], ["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "pkg_firewall_dospolicy_anomaly": {
        "stated": true,
        "options": {
            "action": {"choices": ["pass", "block", "proxy"], "type": "str"},
            "log": {"choices": ["disable", "enable"], "type": "str"},
            "name": {"required": true, "type": "str"},
            "quarantine": {"choices": ["none", "attacker", "both", "interface"], "type": "str"},
            "quarantine-expiry": {"type": "str"},
            "quarantine-log": {"choices": ["disable", "enable"], "type": "str"},
            "status": {"choices": ["disable", "enable"], "type": "str"},
            "threshold": {"type": "int"},
            "threshold(default)": {"type": "int"},
            "synproxy_tos": {
                "v_range": [["6.2.5", "7.2.0"]],
                "choices": ["0", "10", "12", "14", "18", "20", "22", "26", "28", "30", "34", "36", "38", "40", "46", "255"],
                "type": "str"
            },
            "synproxy_ttl": {"v_range": [["6.2.5", "7.2.0"]], "choices": ["32", "64", "128", "255"], "type": "str"},
            "synproxy_tcp_sack": {"v_range": [["6.2.5", "7.2.0"]], "choices": ["disable", "enable"], "type": "str"},
            "synproxy_tcp_window": {"v_range": [["6.2.5", "7.2.0"]], "choices": ["4096", "8192", "16384", "32768"], "type": "str"},
            "synproxy_tcp_timestamp": {"v_range": [["6.2.5", "7.2.0"]], "choices": ["disable", "enable"], "type": "str"},
            "synproxy_tcp_mss": {"v_range": [["6.2.5", "7.2.0"]], "choices": ["0", "256", "512", "1024", "1300", "1360", "1460", "1500"], "type": "str"},
            "synproxy_tcp_windowscale": {
                "v_range": [["6.2.5", "7.2.0"]],
                "choices": ["0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "10", "11", "12", "13", "14"],
                "type": "str"
            },
            "synproxy-tos": {
                "v_range": [["6.2.6", "6.2.12"], ["6.4.2", "7.2.0"], ["7.4.3", ""]],
                "choices": ["0", "10", "12", "14", "18", "20", "22", "26", "28", "30", "34", "36", "38", "40", "46", "255"],
                "type": "str"
            },
            "synproxy-tcp-window": {
                "v_range": [["6.2.6", "6.2.12"], ["6.4.2", "7.2.0"], ["7.4.3", ""]],
                "choices": ["4096", "8192", "16384", "32768"],
                "type": "str"
            },
            "synproxy-tcp-windowscale": {
                "v_range": [["6.2.6", "6.2.12"], ["6.4.2", "7.2.0"], ["7.4.3", ""]],
                "choices": ["0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "10", "11", "12", "13", "14"],
                "type": "str"
            },
            "synproxy-tcp-timestamp": {
                "v_range": [["6.2.6", "6.2.12"], ["6.4.2", "7.2.0"], ["7.4.3", ""]],
                "choices": ["disable", "enable"],
                "type": "str"
            },
            "synproxy-ttl": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", "7.2.0"], ["7.4.3", ""]], "choices": ["32", "64", "128", "255"], "type": "str"},
            "synproxy-tcp-mss": {
                "v_range": [["6.2.6", "6.2.12"], ["6.4.2", "7.2.0"], ["7.4.3", ""]],
                "choices": ["0", "256", "512", "1024", "1300", "1360", "1460", "1500"],
                "type": "str"
            },
            "synproxy-tcp-sack": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", "7.2.0"], ["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "pkg_firewall_explicitproxypolicy": {
        "stated": true,
        "options": {
            "action": {"v_range": [["6.2.0", "6.2.12"]], "choices": ["deny", "accept"], "type": "str"},
            "active-auth-method": {"v_range": [["6.2.0", "6.2.12"]], "choices": ["ntlm", "basic", "digest", "form", "none", "negotiate"], "type": "str"},
            "application-list": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"},
            "av-profile": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"},
            "casi-profile": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"},
            "comments": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"},
            "disclaimer": {"v_range": [["6.2.0", "6.2.12"]], "choices": ["disable", "domain", "policy", "user"], "type": "str"},
            "dlp-sensor": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"},
            "dstaddr": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"},
            "dstaddr-negate": {"v_range": [["6.2.0", "6.2.12"]], "choices": ["disable", "enable"], "type": "str"},
            "dstaddr6": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"},
            "dstintf": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"},
            "global-label": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"},
            "icap-profile": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"},
            "identity-based": {"v_range": [["6.2.0", "6.2.12"]], "choices": ["disable", "enable"], "type": "str"},
            "identity-based-policy": {
                "v_range": [["6.2.0", "6.2.12"]],
                "type": "list",
                "options": {
                    "application-list": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"},
                    "av-profile": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"},
                    "casi-profile": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"},
                    "disclaimer": {"v_range": [["6.2.0", "6.2.12"]], "choices": ["disable", "domain", "policy", "user"], "type": "str"},
                    "dlp-sensor": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"},
                    "groups": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"},
                    "icap-profile": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"},
                    "id": {"v_range": [["6.2.0", "6.2.12"]], "type": "int"},
                    "ips-sensor": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"},
                    "logtraffic": {"v_range": [["6.2.0", "6.2.12"]], "choices": ["disable", "all", "utm"], "type": "str"},
                    "logtraffic-start": {"v_range": [["6.2.0", "6.2.12"]], "choices": ["disable", "enable"], "type": "str"},
                    "mms-profile": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"},
                    "profile-group": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"},
                    "profile-protocol-options": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"},
                    "profile-type": {"v_range": [["6.2.0", "6.2.12"]], "choices": ["single", "group"], "type": "str"},
                    "replacemsg-override-group": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"},
                    "scan-botnet-connections": {"v_range": [["6.2.0", "6.2.12"]], "choices": ["disable", "block", "monitor"], "type": "str"},
                    "schedule": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"},
                    "spamfilter-profile": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"},
                    "ssl-ssh-profile": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"},
                    "users": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"},
                    "utm-status": {"v_range": [["6.2.0", "6.2.12"]], "choices": ["disable", "enable"], "type": "str"},
                    "waf-profile": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"},
                    "webfilter-profile": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"}
                },
                "elements": "dict"
            },
            "ip-based": {"v_range": [["6.2.0", "6.2.12"]], "choices": ["disable", "enable"], "type": "str"},
            "ips-sensor": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"},
            "label": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"},
            "logtraffic": {"v_range": [["6.2.0", "6.2.12"]], "choices": ["disable", "all", "utm"], "type": "str"},
            "logtraffic-start": {"v_range": [["6.2.0", "6.2.12"]], "choices": ["disable", "enable"], "type": "str"},
            "mms-profile": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"},
            "policyid": {"v_range": [["6.2.0", "6.2.12"]], "required": true, "type": "int"},
            "profile-group": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"},
            "profile-protocol-options": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"},
            "profile-type": {"v_range": [["6.2.0", "6.2.12"]], "choices": ["single", "group"], "type": "str"},
            "proxy": {"v_range": [["6.2.0", "6.2.12"]], "choices": ["web", "ftp", "wanopt"], "type": "str"},
            "replacemsg-override-group": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"},
            "require-tfa": {"v_range": [["6.2.0", "6.2.12"]], "choices": ["disable", "enable"], "type": "str"},
            "scan-botnet-connections": {"v_range": [["6.2.0", "6.2.12"]], "choices": ["disable", "block", "monitor"], "type": "str"},
            "schedule": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"},
            "service": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"},
            "service-negate": {"v_range": [["6.2.0", "6.2.12"]], "choices": ["disable", "enable"], "type": "str"},
            "spamfilter-profile": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"},
            "srcaddr": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"},
            "srcaddr-negate": {"v_range": [["6.2.0", "6.2.12"]], "choices": ["disable", "enable"], "type": "str"},
            "srcaddr6": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"},
            "ssl-ssh-profile": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"},
            "sso-auth-method": {"v_range": [["6.2.0", "6.2.12"]], "choices": ["fsso", "rsso", "none"], "type": "str"},
            "status": {"v_range": [["6.2.0", "6.2.12"]], "choices": ["disable", "enable"], "type": "str"},
            "tags": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"},
            "transaction-based": {"v_range": [["6.2.0", "6.2.12"]], "choices": ["disable", "enable"], "type": "str"},
            "transparent": {"v_range": [["6.2.0", "6.2.12"]], "choices": ["disable", "enable"], "type": "str"},
            "utm-status": {"v_range": [["6.2.0", "6.2.12"]], "choices": ["disable", "enable"], "type": "str"},
            "uuid": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"},
            "waf-profile": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"},
            "web-auth-cookie": {"v_range": [["6.2.0", "6.2.12"]], "choices": ["disable", "enable"], "type": "str"},
            "webcache": {"v_range": [["6.2.0", "6.2.12"]], "choices": ["disable", "enable"], "type": "str"},
            "webcache-https": {"v_range": [["6.2.0", "6.2.12"]], "choices": ["disable", "any", "enable"], "type": "str"},
            "webfilter-profile": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"},
            "webproxy-forward-server": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"},
            "webproxy-profile": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"}
        }
    },
    "pkg_firewall_explicitproxypolicy_identitybasedpolicy": {
        "stated": true,
        "options": {
            "application-list": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"},
            "av-profile": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"},
            "casi-profile": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"},
            "disclaimer": {"v_range": [["6.2.0", "6.2.12"]], "choices": ["disable", "domain", "policy", "user"], "type": "str"},
            "dlp-sensor": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"},
            "groups": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"},
            "icap-profile": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"},
            "id": {"v_range": [["6.2.0", "6.2.12"]], "required": true, "type": "int"},
            "ips-sensor": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"},
            "logtraffic": {"v_range": [["6.2.0", "6.2.12"]], "choices": ["disable", "all", "utm"], "type": "str"},
            "logtraffic-start": {"v_range": [["6.2.0", "6.2.12"]], "choices": ["disable", "enable"], "type": "str"},
            "mms-profile": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"},
            "profile-group": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"},
            "profile-protocol-options": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"},
            "profile-type": {"v_range": [["6.2.0", "6.2.12"]], "choices": ["single", "group"], "type": "str"},
            "replacemsg-override-group": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"},
            "scan-botnet-connections": {"v_range": [["6.2.0", "6.2.12"]], "choices": ["disable", "block", "monitor"], "type": "str"},
            "schedule": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"},
            "spamfilter-profile": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"},
            "ssl-ssh-profile": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"},
            "users": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"},
            "utm-status": {"v_range": [["6.2.0", "6.2.12"]], "choices": ["disable", "enable"], "type": "str"},
            "waf-profile": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"},
            "webfilter-profile": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"}
        }
    },
    "pkg_firewall_explicitproxypolicy_sectionvalue": {
        "stated": true,
        "options": {
            "attr": {"v_range": [["6.2.0", "6.2.12"]], "choices": ["label", "global-label"], "type": "str"},
            "name": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"}
        }
    },
    "pkg_firewall_hyperscalepolicy": {
        "stated": true,
        "options": {
            "action": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", "7.2.0"], ["7.4.3", ""]], "choices": ["deny", "accept"], "type": "str"},
            "auto-asic-offload": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", "7.2.0"], ["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "cgn-eif": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", "7.2.0"], ["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "cgn-eim": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", "7.2.0"], ["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "cgn-log-server-grp": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", "7.2.0"], ["7.4.3", ""]], "type": "str"},
            "cgn-resource-quota": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", "7.2.0"], ["7.4.3", ""]], "type": "int"},
            "cgn-session-quota": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", "7.2.0"], ["7.4.3", ""]], "type": "int"},
            "comments": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", "7.2.0"], ["7.4.3", ""]], "type": "str"},
            "delay-tcp-npu-session": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.2", "7.2.0"], ["7.4.3", ""]],
                "choices": ["disable", "enable"],
                "type": "str"
            },
            "dstaddr": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", "7.2.0"], ["7.4.3", ""]], "type": "raw"},
            "dstaddr-negate": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", "7.2.0"], ["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "dstaddr6": {"v_range": [["6.4.7", "6.4.14"], ["7.0.2", "7.2.0"], ["7.4.3", ""]], "type": "raw"},
            "dstintf": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", "7.2.0"], ["7.4.3", ""]], "type": "raw"},
            "firewall-session-dirty": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.2", "7.2.0"], ["7.4.3", ""]],
                "choices": ["check-all", "check-new"],
                "type": "str"
            },
            "global-label": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", "7.2.0"], ["7.4.3", ""]], "type": "str"},
            "ippool": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", "7.2.0"], ["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "label": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", "7.2.0"], ["7.4.3", ""]], "type": "str"},
            "name": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", "7.2.0"], ["7.4.3", ""]], "type": "str"},
            "nat": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", "7.2.0"], ["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "policy-offload": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", "7.2.0"], ["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "policyid": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", "7.2.0"], ["7.4.3", ""]], "required": true, "type": "int"},
            "poolname": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", "7.2.0"], ["7.4.3", ""]], "type": "raw"},
            "poolname6": {"v_range": [["6.4.7", "6.4.14"], ["7.0.2", "7.2.0"], ["7.4.3", ""]], "type": "raw"},
            "send-deny-packet": {"v_range": [["6.4.7", "6.4.14"], ["7.0.2", "7.2.0"], ["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "service": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", "7.2.0"], ["7.4.3", ""]], "type": "raw"},
            "service-negate": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", "7.2.0"], ["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "srcaddr": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", "7.2.0"], ["7.4.3", ""]], "type": "raw"},
            "srcaddr-negate": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", "7.2.0"], ["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "srcaddr6": {"v_range": [["6.4.7", "6.4.14"], ["7.0.2", "7.2.0"], ["7.4.3", ""]], "type": "raw"},
            "srcintf": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", "7.2.0"], ["7.4.3", ""]], "type": "raw"},
            "status": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", "7.2.0"], ["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "tcp-timeout-pid": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", "7.2.0"], ["7.4.3", ""]], "type": "str"},
            "traffic-shaper": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", "7.2.0"], ["7.4.3", ""]], "type": "str"},
            "traffic-shaper-reverse": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", "7.2.0"], ["7.4.3", ""]], "type": "str"},
            "udp-timeout-pid": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", "7.2.0"], ["7.4.3", ""]], "type": "str"},
            "uuid": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", "7.2.0"], ["7.4.3", ""]], "type": "str"}
        }
    },
    "pkg_firewall_hyperscalepolicy46": {
        "stated": true,
        "options": {
            "action": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", "7.2.0"], ["7.4.3", ""]], "choices": ["deny", "accept"], "type": "str"},
            "cgn-log-server-grp": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", "7.2.0"], ["7.4.3", ""]], "type": "str"},
            "comments": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", "7.2.0"], ["7.4.3", ""]], "type": "str"},
            "dstaddr": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", "7.2.0"], ["7.4.3", ""]], "type": "raw"},
            "dstintf": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", "7.2.0"], ["7.4.3", ""]], "type": "str"},
            "name": {"v_range": [["6.4.8", "6.4.14"], ["7.0.3", "7.2.0"], ["7.4.3", ""]], "type": "str"},
            "policy-offload": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", "7.2.0"], ["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "policyid": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", "7.2.0"], ["7.4.3", ""]], "required": true, "type": "int"},
            "service": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", "7.2.0"], ["7.4.3", ""]], "type": "raw"},
            "srcaddr": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", "7.2.0"], ["7.4.3", ""]], "type": "raw"},
            "srcintf": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", "7.2.0"], ["7.4.3", ""]], "type": "str"},
            "status": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", "7.2.0"], ["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "tcp-timeout-pid": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", "7.2.0"], ["7.4.3", ""]], "type": "str"},
            "traffic-shaper": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", "7.2.0"], ["7.4.3", ""]], "type": "str"},
            "traffic-shaper-reverse": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", "7.2.0"], ["7.4.3", ""]], "type": "str"},
            "udp-timeout-pid": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", "7.2.0"], ["7.4.3", ""]], "type": "str"},
            "uuid": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", "7.2.0"], ["7.4.3", ""]], "type": "str"}
        }
    },
    "pkg_firewall_hyperscalepolicy6": {
        "stated": true,
        "options": {
            "action": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", "7.2.0"], ["7.4.3", ""]], "choices": ["deny", "accept", "ipsec"], "type": "str"},
            "auto-asic-offload": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", "7.2.0"], ["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "cgn-log-server-grp": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", "7.2.0"], ["7.4.3", ""]], "type": "str"},
            "comments": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", "7.2.0"], ["7.4.3", ""]], "type": "str"},
            "dstaddr": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", "7.2.0"], ["7.4.3", ""]], "type": "raw"},
            "dstaddr-negate": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", "7.2.0"], ["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "dstintf": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", "7.2.0"], ["7.4.3", ""]], "type": "raw"},
            "name": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", "7.2.0"], ["7.4.3", ""]], "type": "str"},
            "policy-offload": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", "7.2.0"], ["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "policyid": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", "7.2.0"], ["7.4.3", ""]], "required": true, "type": "int"},
            "service": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", "7.2.0"], ["7.4.3", ""]], "type": "raw"},
            "service-negate": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", "7.2.0"], ["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "srcaddr": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", "7.2.0"], ["7.4.3", ""]], "type": "raw"},
            "srcaddr-negate": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", "7.2.0"], ["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "srcintf": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", "7.2.0"], ["7.4.3", ""]], "type": "raw"},
            "status": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", "7.2.0"], ["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "tcp-timeout-pid": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", "7.2.0"], ["7.4.3", ""]], "type": "str"},
            "traffic-shaper": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", "7.2.0"], ["7.4.3", ""]], "type": "str"},
            "traffic-shaper-reverse": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", "7.2.0"], ["7.4.3", ""]], "type": "str"},
            "udp-timeout-pid": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", "7.2.0"], ["7.4.3", ""]], "type": "str"},
            "uuid": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", "7.2.0"], ["7.4.3", ""]], "type": "str"}
        }
    },
    "pkg_firewall_hyperscalepolicy64": {
        "stated": true,
        "options": {
            "action": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", "7.2.0"], ["7.4.3", ""]], "choices": ["deny", "accept"], "type": "str"},
            "cgn-eif": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", "7.2.0"], ["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "cgn-eim": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", "7.2.0"], ["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "cgn-log-server-grp": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", "7.2.0"], ["7.4.3", ""]], "type": "str"},
            "cgn-resource-quota": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", "7.2.0"], ["7.4.3", ""]], "type": "int"},
            "cgn-session-quota": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", "7.2.0"], ["7.4.3", ""]], "type": "int"},
            "comments": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", "7.2.0"], ["7.4.3", ""]], "type": "str"},
            "dstaddr": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", "7.2.0"], ["7.4.3", ""]], "type": "raw"},
            "dstintf": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", "7.2.0"], ["7.4.3", ""]], "type": "str"},
            "ippool": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", "7.2.0"], ["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "name": {"v_range": [["6.4.8", "6.4.14"], ["7.0.3", "7.2.0"], ["7.4.3", ""]], "type": "str"},
            "policy-offload": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", "7.2.0"], ["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "policyid": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", "7.2.0"], ["7.4.3", ""]], "required": true, "type": "int"},
            "poolname": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", "7.2.0"], ["7.4.3", ""]], "type": "raw"},
            "service": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", "7.2.0"], ["7.4.3", ""]], "type": "raw"},
            "srcaddr": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", "7.2.0"], ["7.4.3", ""]], "type": "raw"},
            "srcintf": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", "7.2.0"], ["7.4.3", ""]], "type": "str"},
            "status": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", "7.2.0"], ["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "tcp-timeout-pid": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", "7.2.0"], ["7.4.3", ""]], "type": "str"},
            "traffic-shaper": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", "7.2.0"], ["7.4.3", ""]], "type": "str"},
            "traffic-shaper-reverse": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", "7.2.0"], ["7.4.3", ""]], "type": "str"},
            "udp-timeout-pid": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", "7.2.0"], ["7.4.3", ""]], "type": "str"},
            "uuid": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", "7.2.0"], ["7.4.3", ""]], "type": "str"}
        }
    },
    "pkg_firewall_interfacepolicy": {
        "stated": true,
        "options": {
            "address-type": {"v_range": [["6.0.0", "7.2.2"], ["7.4.3", ""]], "choices": ["ipv4", "ipv6"], "type": "str"},
            "application-list": {"v_range": [["6.0.0", "7.2.2"], ["7.4.3", ""]], "type": "str"},
            "application-list-status": {"v_range": [["6.0.0", "7.2.2"], ["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "av-profile": {"v_range": [["6.0.0", "7.2.2"], ["7.4.3", ""]], "type": "str"},
            "av-profile-status": {"v_range": [["6.0.0", "7.2.2"], ["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "comments": {"v_range": [["6.0.0", "7.2.2"], ["7.4.3", ""]], "type": "str"},
            "dlp-sensor": {"v_range": [["6.0.0", "7.2.2"], ["7.4.3", ""]], "type": "str"},
            "dlp-sensor-status": {"v_range": [["6.0.0", "7.2.2"], ["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "dsri": {"v_range": [["6.0.0", "7.2.2"], ["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "dstaddr": {"v_range": [["6.0.0", "7.2.2"], ["7.4.3", ""]], "type": "raw"},
            "interface": {"v_range": [["6.0.0", "7.2.2"], ["7.4.3", ""]], "type": "str"},
            "ips-sensor": {"v_range": [["6.0.0", "7.2.2"], ["7.4.3", ""]], "type": "str"},
            "ips-sensor-status": {"v_range": [["6.0.0", "7.2.2"], ["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "label": {"v_range": [["6.0.0", "7.2.2"], ["7.4.3", ""]], "type": "str"},
            "logtraffic": {"v_range": [["6.0.0", "7.2.2"], ["7.4.3", ""]], "choices": ["disable", "all", "utm"], "type": "str"},
            "policyid": {"v_range": [["6.0.0", "7.2.2"], ["7.4.3", ""]], "required": true, "type": "int"},
            "scan-botnet-connections": {"v_range": [["6.0.0", "7.2.1"]], "choices": ["disable", "block", "monitor"], "type": "str"},
            "service": {"v_range": [["6.0.0", "7.2.2"], ["7.4.3", ""]], "type": "raw"},
            "spamfilter-profile": {"v_range": [["6.0.0", "7.2.1"]], "type": "str"},
            "spamfilter-profile-status": {"v_range": [["6.0.0", "7.2.1"]], "choices": ["disable", "enable"], "type": "str"},
            "srcaddr": {"v_range": [["6.0.0", "7.2.2"], ["7.4.3", ""]], "type": "raw"},
            "status": {"v_range": [["6.0.0", "7.2.2"], ["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "webfilter-profile": {"v_range": [["6.0.0", "7.2.2"], ["7.4.3", ""]], "type": "str"},
            "webfilter-profile-status": {"v_range": [["6.0.0", "7.2.2"], ["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "emailfilter-profile": {"v_range": [["6.2.0", "7.2.2"], ["7.4.3", ""]], "type": "str"},
            "emailfilter-profile-status": {"v_range": [["6.2.0", "7.2.2"], ["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "uuid": {"v_range": [["6.2.1", "7.2.0"], ["7.4.3", ""]], "type": "str"},
            "casi-profile": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"},
            "casi-profile-status": {"v_range": [["6.2.0", "6.2.12"]], "choices": ["disable", "enable"], "type": "str"},
            "dlp-profile": {"v_range": [["7.2.0", "7.2.1"], ["7.4.3", ""]], "type": "str"},
            "dlp-profile-status": {"v_range": [["7.2.0", "7.2.1"], ["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "casb-profile": {"v_range": [["7.4.3", ""]], "type": "raw"},
            "casb-profile-status": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "pkg_firewall_interfacepolicy6": {
        "stated": true,
        "options": {
            "address-type": {"v_range": [["6.0.0", "7.2.2"], ["7.4.3", ""]], "choices": ["ipv4", "ipv6"], "type": "str"},
            "application-list": {"v_range": [["6.0.0", "7.2.2"], ["7.4.3", ""]], "type": "str"},
            "application-list-status": {"v_range": [["6.0.0", "7.2.2"], ["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "av-profile": {"v_range": [["6.0.0", "7.2.2"], ["7.4.3", ""]], "type": "str"},
            "av-profile-status": {"v_range": [["6.0.0", "7.2.2"], ["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "comments": {"v_range": [["6.0.0", "7.2.2"], ["7.4.3", ""]], "type": "str"},
            "dlp-sensor": {"v_range": [["6.0.0", "7.2.2"], ["7.4.3", ""]], "type": "str"},
            "dlp-sensor-status": {"v_range": [["6.0.0", "7.2.2"], ["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "dsri": {"v_range": [["6.0.0", "7.2.2"], ["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "dstaddr6": {"v_range": [["6.0.0", "7.2.2"], ["7.4.3", ""]], "type": "raw"},
            "interface": {"v_range": [["6.0.0", "7.2.2"], ["7.4.3", ""]], "type": "str"},
            "ips-sensor": {"v_range": [["6.0.0", "7.2.2"], ["7.4.3", ""]], "type": "str"},
            "ips-sensor-status": {"v_range": [["6.0.0", "7.2.2"], ["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "label": {"v_range": [["6.0.0", "7.2.2"], ["7.4.3", ""]], "type": "str"},
            "logtraffic": {"v_range": [["6.0.0", "7.2.2"], ["7.4.3", ""]], "choices": ["disable", "all", "utm"], "type": "str"},
            "policyid": {"v_range": [["6.0.0", "7.2.2"], ["7.4.3", ""]], "required": true, "type": "int"},
            "scan-botnet-connections": {"v_range": [["6.0.0", "7.2.1"]], "choices": ["disable", "block", "monitor"], "type": "str"},
            "service6": {"v_range": [["6.0.0", "7.2.2"], ["7.4.3", ""]], "type": "raw"},
            "spamfilter-profile": {"v_range": [["6.0.0", "7.2.1"]], "type": "str"},
            "spamfilter-profile-status": {"v_range": [["6.0.0", "7.2.1"]], "choices": ["disable", "enable"], "type": "str"},
            "srcaddr6": {"v_range": [["6.0.0", "7.2.2"], ["7.4.3", ""]], "type": "raw"},
            "status": {"v_range": [["6.0.0", "7.2.2"], ["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "webfilter-profile": {"v_range": [["6.0.0", "7.2.2"], ["7.4.3", ""]], "type": "str"},
            "webfilter-profile-status": {"v_range": [["6.0.0", "7.2.2"], ["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "emailfilter-profile": {"v_range": [["6.2.0", "7.2.2"], ["7.4.3", ""]], "type": "str"},
            "emailfilter-profile-status": {"v_range": [["6.2.0", "7.2.2"], ["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "uuid": {"v_range": [["6.2.1", "7.2.0"], ["7.4.3", ""]], "type": "str"},
            "casi-profile": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"},
            "casi-profile-status": {"v_range": [["6.2.0", "6.2.12"]], "choices": ["disable", "enable"], "type": "str"},
            "dlp-profile": {"v_range": [["7.2.0", "7.2.1"], ["7.4.3", ""]], "type": "str"},
            "dlp-profile-status": {"v_range": [["7.2.0", "7.2.1"], ["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "casb-profile": {"v_range": [["7.4.3", ""]], "type": "raw"},
            "casb-profile-status": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "pkg_firewall_interfacepolicy6_sectionvalue": {
        "stated": true,
        "options": {
            "attr": {"v_range": [["6.0.0", "7.2.2"], ["7.4.3", ""]], "choices": ["label", "global-label"], "type": "str"},
            "name": {"v_range": [["6.0.0", "7.2.2"], ["7.4.3", ""]], "type": "str"}
        }
    },
    "pkg_firewall_interfacepolicy_sectionvalue": {
        "stated": true,
        "options": {
            "attr": {"v_range": [["6.0.0", "7.2.2"], ["7.4.3", ""]], "choices": ["label", "global-label"], "type": "str"},
            "name": {"v_range": [["6.0.0", "7.2.2"], ["7.4.3", ""]], "type": "str"}
        }
    },
    "pkg_firewall_localinpolicy": {
        "stated": true,
        "options": {
            "action": {"choices": ["deny", "accept"], "type": "str"},
            "dstaddr": {"type": "raw"},
            "ha-mgmt-intf-only": {"choices": ["disable", "enable"], "type": "str"},
            "intf": {"type": "str"},
            "policyid": {"required": true, "type": "int"},
            "schedule": {"type": "str"},
            "service": {"type": "raw"},
            "srcaddr": {"type": "raw"},
            "status": {"choices": ["disable", "enable"], "type": "str"},
            "comments": {"v_range": [["6.2.0", ""]], "type": "str"},
            "uuid": {"v_range": [["6.2.1", ""]], "type": "str"},
            "dstaddr-negate": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "service-negate": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "srcaddr-negate": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "virtual-patch": {"v_range": [["7.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "internet-service-src": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "internet-service-src-custom": {"v_range": [["7.4.3", ""]], "type": "raw"},
            "internet-service-src-custom-group": {"v_range": [["7.4.3", ""]], "type": "raw"},
            "internet-service-src-group": {"v_range": [["7.4.3", ""]], "type": "raw"},
            "internet-service-src-name": {"v_range": [["7.4.3", ""]], "type": "raw"},
            "internet-service-src-negate": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "logtraffic": {"v_range": [["7.6.0", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "pkg_firewall_localinpolicy6": {
        "stated": true,
        "options": {
            "action": {"choices": ["deny", "accept"], "type": "str"},
            "dstaddr": {"type": "raw"},
            "intf": {"type": "str"},
            "policyid": {"required": true, "type": "int"},
            "schedule": {"type": "str"},
            "service": {"type": "raw"},
            "srcaddr": {"type": "raw"},
            "status": {"choices": ["disable", "enable"], "type": "str"},
            "comments": {"v_range": [["6.2.0", ""]], "type": "str"},
            "uuid": {"v_range": [["6.2.1", ""]], "type": "str"},
            "dstaddr-negate": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "service-negate": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "srcaddr-negate": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "virtual-patch": {"v_range": [["7.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "internet-service6-src": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "internet-service6-src-custom": {"v_range": [["7.4.3", ""]], "type": "raw"},
            "internet-service6-src-custom-group": {"v_range": [["7.4.3", ""]], "type": "raw"},
            "internet-service6-src-group": {"v_range": [["7.4.3", ""]], "type": "raw"},
            "internet-service6-src-name": {"v_range": [["7.4.3", ""]], "type": "raw"},
            "internet-service6-src-negate": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "logtraffic": {"v_range": [["7.6.0", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "pkg_firewall_multicastpolicy": {
        "stated": true,
        "options": {
            "action": {"choices": ["deny", "accept"], "type": "str"},
            "auto-asic-offload": {"v_range": [["6.0.0", "7.2.0"], ["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "dnat": {"type": "str"},
            "dstaddr": {"type": "raw"},
            "dstintf": {"type": "str"},
            "end-port": {"type": "int"},
            "id": {"required": true, "type": "int"},
            "logtraffic": {"choices": ["disable", "enable", "all", "utm"], "type": "str"},
            "protocol": {"type": "int"},
            "snat": {"choices": ["disable", "enable"], "type": "str"},
            "snat-ip": {"type": "str"},
            "srcaddr": {"type": "raw"},
            "srcintf": {"type": "str"},
            "start-port": {"type": "int"},
            "status": {"choices": ["disable", "enable"], "type": "str"},
            "comments": {"v_range": [["6.4.0", ""]], "type": "str"},
            "uuid": {"v_range": [["6.4.0", ""]], "type": "str"},
            "name": {"v_range": [["6.4.2", ""]], "type": "str"},
            "traffic-shaper": {"v_range": [["7.2.0", ""]], "type": "str"},
            "ips-sensor": {"v_range": [["7.4.2", ""]], "type": "str"},
            "utm-status": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "pkg_firewall_multicastpolicy6": {
        "stated": true,
        "options": {
            "action": {"choices": ["deny", "accept"], "type": "str"},
            "auto-asic-offload": {"v_range": [["6.0.0", "7.2.0"], ["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "dstaddr": {"type": "raw"},
            "dstintf": {"type": "str"},
            "end-port": {"type": "int"},
            "id": {"required": true, "type": "int"},
            "logtraffic": {"choices": ["disable", "enable", "all", "utm"], "type": "str"},
            "protocol": {"type": "int"},
            "srcaddr": {"type": "raw"},
            "srcintf": {"type": "str"},
            "start-port": {"type": "int"},
            "status": {"choices": ["disable", "enable"], "type": "str"},
            "name": {"v_range": [["6.4.2", ""]], "type": "str"},
            "comments": {"v_range": [["7.0.0", ""]], "type": "str"},
            "uuid": {"v_range": [["7.0.0", ""]], "type": "str"},
            "ips-sensor": {"v_range": [["7.4.2", ""]], "type": "str"},
            "utm-status": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "pkg_firewall_policy": {
        "stated": true,
        "options": {
            "action": {"choices": ["deny", "accept", "ipsec", "ssl-vpn", "redirect", "isolate"], "type": "str"},
            "app-category": {"type": "raw"},
            "application": {"type": "raw"},
            "application-list": {"type": "str"},
            "auth-cert": {"type": "str"},
            "auth-path": {"choices": ["disable", "enable"], "type": "str"},
            "auth-redirect-addr": {"type": "str"},
            "auto-asic-offload": {"v_range": [["6.0.0", "7.2.0"], ["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "av-profile": {"type": "str"},
            "block-notification": {"choices": ["disable", "enable"], "type": "str"},
            "captive-portal-exempt": {"choices": ["disable", "enable"], "type": "str"},
            "capture-packet": {"v_range": [["6.0.0", "7.2.0"], ["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "comments": {"type": "raw"},
            "custom-log-fields": {"type": "raw"},
            "delay-tcp-npu-session": {"choices": ["disable", "enable"], "type": "str"},
            "devices": {"v_range": [["6.0.0", "7.2.1"]], "type": "raw"},
            "diffserv-forward": {"choices": ["disable", "enable"], "type": "str"},
            "diffserv-reverse": {"choices": ["disable", "enable"], "type": "str"},
            "diffservcode-forward": {"type": "str"},
            "diffservcode-rev": {"type": "str"},
            "disclaimer": {"choices": ["disable", "enable", "user", "domain", "policy"], "type": "str"},
            "dlp-sensor": {"type": "str"},
            "dnsfilter-profile": {"type": "str"},
            "dscp-match": {"v_range": [["6.0.0", "7.2.1"]], "choices": ["disable", "enable"], "type": "str"},
            "dscp-negate": {"v_range": [["6.0.0", "7.2.1"]], "choices": ["disable", "enable"], "type": "str"},
            "dscp-value": {"v_range": [["6.0.0", "7.2.1"]], "type": "str"},
            "dsri": {"choices": ["disable", "enable"], "type": "str"},
            "dstaddr": {"type": "raw"},
            "dstaddr-negate": {"choices": ["disable", "enable"], "type": "str"},
            "dstintf": {"type": "raw"},
            "firewall-session-dirty": {"choices": ["check-all", "check-new"], "type": "str"},
            "fixedport": {"choices": ["disable", "enable"], "type": "str"},
            "fsso": {"choices": ["disable", "enable"], "type": "str"},
            "fsso-agent-for-ntlm": {"type": "str"},
            "global-label": {"type": "str"},
            "groups": {"type": "raw"},
            "gtp-profile": {"v_range": [["6.0.0", "7.2.0"], ["7.4.3", ""]], "type": "str"},
            "icap-profile": {"type": "str"},
            "identity-based-route": {"type": "str"},
            "inbound": {"choices": ["disable", "enable"], "type": "str"},
            "internet-service": {"choices": ["disable", "enable"], "type": "str"},
            "internet-service-custom": {"type": "raw"},
            "internet-service-id": {"type": "raw"},
            "internet-service-negate": {"choices": ["disable", "enable"], "type": "str"},
            "ippool": {"choices": ["disable", "enable"], "type": "str"},
            "ips-sensor": {"type": "str"},
            "label": {"type": "str"},
            "learning-mode": {"v_range": [["6.0.0", "7.2.1"]], "choices": ["disable", "enable"], "type": "str"},
            "logtraffic": {"choices": ["disable", "enable", "all", "utm"], "type": "str"},
            "logtraffic-start": {"choices": ["disable", "enable"], "type": "str"},
            "match-vip": {"choices": ["disable", "enable"], "type": "str"},
            "mms-profile": {"v_range": [["6.0.0", "7.2.0"], ["7.4.3", ""]], "type": "str"},
            "name": {"type": "str"},
            "nat": {"choices": ["disable", "enable"], "type": "str"},
            "natinbound": {"choices": ["disable", "enable"], "type": "str"},
            "natip": {"type": "str"},
            "natoutbound": {"choices": ["disable", "enable"], "type": "str"},
            "ntlm": {"choices": ["disable", "enable"], "type": "str"},
            "ntlm-enabled-browsers": {"type": "raw"},
            "ntlm-guest": {"choices": ["disable", "enable"], "type": "str"},
            "outbound": {"choices": ["disable", "enable"], "type": "str"},
            "per-ip-shaper": {"type": "str"},
            "permit-any-host": {"choices": ["disable", "enable"], "type": "str"},
            "permit-stun-host": {"choices": ["disable", "enable"], "type": "str"},
            "policyid": {"required": true, "type": "int"},
            "poolname": {"type": "raw"},
            "profile-group": {"type": "str"},
            "profile-protocol-options": {"type": "str"},
            "profile-type": {"choices": ["single", "group"], "type": "str"},
            "radius-mac-auth-bypass": {"choices": ["disable", "enable"], "type": "str"},
            "redirect-url": {"type": "str"},
            "replacemsg-override-group": {"type": "str"},
            "rsso": {"choices": ["disable", "enable"], "type": "str"},
            "rtp-addr": {"type": "raw"},
            "rtp-nat": {"choices": ["disable", "enable"], "type": "str"},
            "scan-botnet-connections": {"v_range": [["6.0.0", "7.2.1"]], "choices": ["disable", "block", "monitor"], "type": "str"},
            "schedule": {"type": "str"},
            "schedule-timeout": {"choices": ["disable", "enable"], "type": "str"},
            "send-deny-packet": {"choices": ["disable", "enable"], "type": "str"},
            "service": {"type": "raw"},
            "service-negate": {"choices": ["disable", "enable"], "type": "str"},
            "session-ttl": {"type": "raw"},
            "spamfilter-profile": {"v_range": [["6.0.0", "7.2.1"]], "type": "str"},
            "srcaddr": {"type": "raw"},
            "srcaddr-negate": {"choices": ["disable", "enable"], "type": "str"},
            "srcintf": {"type": "raw"},
            "ssl-mirror": {"choices": ["disable", "enable"], "type": "str"},
            "ssl-mirror-intf": {"type": "raw"},
            "ssl-ssh-profile": {"type": "str"},
            "status": {"choices": ["disable", "enable"], "type": "str"},
            "tags": {"v_range": [["6.0.0", "6.4.14"]], "type": "str"},
            "tcp-mss-receiver": {"type": "int"},
            "tcp-mss-sender": {"type": "int"},
            "tcp-session-without-syn": {"choices": ["all", "data-only", "disable"], "type": "str"},
            "timeout-send-rst": {"choices": ["disable", "enable"], "type": "str"},
            "traffic-shaper": {"type": "str"},
            "traffic-shaper-reverse": {"type": "str"},
            "url-category": {"type": "raw"},
            "users": {"type": "raw"},
            "utm-status": {"choices": ["disable", "enable"], "type": "str"},
            "uuid": {"type": "str"},
            "vlan-cos-fwd": {"type": "int"},
            "vlan-cos-rev": {"type": "int"},
            "voip-profile": {"type": "str"},
            "vpn_dst_node": {
                "v_range": [["6.0.0", "7.0.2"]],
                "type": "list",
                "options": {
                    "host": {"v_range": [["6.0.0", "7.0.2"]], "type": "str"},
                    "seq": {"v_range": [["6.0.0", "7.0.2"]], "type": "int"},
                    "subnet": {"v_range": [["6.0.0", "7.0.2"]], "type": "str"}
                },
                "elements": "dict"
            },
            "vpn_src_node": {
                "v_range": [["6.0.0", "7.0.2"]],
                "type": "list",
                "options": {
                    "host": {"v_range": [["6.0.0", "7.0.2"]], "type": "str"},
                    "seq": {"v_range": [["6.0.0", "7.0.2"]], "type": "int"},
                    "subnet": {"v_range": [["6.0.0", "7.0.2"]], "type": "str"}
                },
                "elements": "dict"
            },
            "vpntunnel": {"type": "str"},
            "waf-profile": {"type": "str"},
            "wanopt": {"v_range": [["6.0.0", "7.2.0"], ["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "wanopt-detection": {"v_range": [["6.0.0", "7.2.0"], ["7.4.3", ""]], "choices": ["active", "passive", "off"], "type": "str"},
            "wanopt-passive-opt": {
                "v_range": [["6.0.0", "7.2.0"], ["7.4.3", ""]],
                "choices": ["default", "transparent", "non-transparent"],
                "type": "str"
            },
            "wanopt-peer": {"v_range": [["6.0.0", "7.2.0"], ["7.4.3", ""]], "type": "str"},
            "wanopt-profile": {"v_range": [["6.0.0", "7.2.0"], ["7.4.3", ""]], "type": "str"},
            "wccp": {"choices": ["disable", "enable"], "type": "str"},
            "webcache": {"v_range": [["6.0.0", "7.2.0"], ["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "webcache-https": {"v_range": [["6.0.0", "7.2.0"], ["7.4.3", ""]], "choices": ["disable", "ssl-server", "any", "enable"], "type": "str"},
            "webfilter-profile": {"type": "str"},
            "wsso": {"choices": ["disable", "enable"], "type": "str"},
            "anti-replay": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "app-group": {"v_range": [["6.2.0", ""]], "type": "raw"},
            "cifs-profile": {"v_range": [["6.2.0", ""]], "type": "str"},
            "email-collect": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "emailfilter-profile": {"v_range": [["6.2.0", ""]], "type": "str"},
            "fsso-groups": {"v_range": [["6.2.1", ""]], "type": "raw"},
            "geoip-anycast": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "http-policy-redirect": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "inspection-mode": {"v_range": [["6.2.0", ""]], "choices": ["proxy", "flow"], "type": "str"},
            "internet-service-custom-group": {"v_range": [["6.2.0", ""]], "type": "raw"},
            "internet-service-group": {"v_range": [["6.2.0", ""]], "type": "raw"},
            "internet-service-src": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "internet-service-src-custom": {"v_range": [["6.2.0", ""]], "type": "raw"},
            "internet-service-src-custom-group": {"v_range": [["6.2.0", ""]], "type": "raw"},
            "internet-service-src-group": {"v_range": [["6.2.0", ""]], "type": "raw"},
            "internet-service-src-id": {"v_range": [["6.2.0", ""]], "type": "raw"},
            "internet-service-src-negate": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "match-vip-only": {"v_range": [["6.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "np-acceleration": {"v_range": [["6.2.0", "7.2.0"], ["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "reputation-direction": {"v_range": [["6.2.0", ""]], "choices": ["source", "destination"], "type": "str"},
            "reputation-minimum": {"v_range": [["6.2.0", ""]], "type": "int"},
            "ssh-filter-profile": {"v_range": [["6.2.0", "7.2.4"], ["7.4.0", ""]], "type": "str"},
            "ssh-policy-redirect": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "tos": {"v_range": [["6.2.0", ""]], "type": "str"},
            "tos-mask": {"v_range": [["6.2.0", ""]], "type": "str"},
            "tos-negate": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "vlan-filter": {"v_range": [["6.2.0", ""]], "type": "str"},
            "webproxy-forward-server": {"v_range": [["6.2.0", ""]], "type": "str"},
            "webproxy-profile": {"v_range": [["6.2.0", ""]], "type": "str"},
            "np-accelation": {"v_range": [["6.2.1", "6.4.14"]], "choices": ["disable", "enable"], "type": "str"},
            "delay-tcp-npu-sessoin": {"v_range": [["6.2.0", "6.2.12"]], "choices": ["disable", "enable"], "type": "str"},
            "casi-profile": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"},
            "best-route": {"v_range": [["6.2.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "decrypted-traffic-mirror": {"v_range": [["6.4.0", ""]], "type": "str"},
            "dstaddr6": {"v_range": [["6.4.0", ""]], "type": "raw"},
            "geoip-match": {"v_range": [["6.4.0", ""]], "choices": ["physical-location", "registered-location"], "type": "str"},
            "internet-service-name": {"v_range": [["6.4.0", ""]], "type": "raw"},
            "internet-service-src-name": {"v_range": [["6.4.0", ""]], "type": "raw"},
            "poolname6": {"v_range": [["6.4.0", ""]], "type": "raw"},
            "src-vendor-mac": {"v_range": [["6.4.0", ""]], "type": "raw"},
            "srcaddr6": {"v_range": [["6.4.0", ""]], "type": "raw"},
            "file-filter-profile": {"v_range": [["6.4.1", "7.2.4"], ["7.4.0", ""]], "type": "str"},
            "policy-offload": {"v_range": [["6.2.7", "6.2.12"], ["6.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "cgn-session-quota": {"v_range": [["6.2.7", "6.2.12"], ["6.4.3", ""]], "type": "int"},
            "cgn-eif": {"v_range": [["6.2.7", "6.2.12"], ["6.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "cgn-log-server-grp": {"v_range": [["6.2.7", "6.2.12"], ["6.4.3", ""]], "type": "str"},
            "cgn-eim": {"v_range": [["6.2.7", "6.2.12"], ["6.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "cgn-resource-quota": {"v_range": [["6.2.7", "6.2.12"], ["6.4.3", ""]], "type": "int"},
            "dynamic-shaping": {"v_range": [["6.4.6", ""]], "choices": ["disable", "enable"], "type": "str"},
            "passive-wan-health-measurement": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "videofilter-profile": {"v_range": [["7.0.0", "7.2.4"], ["7.4.0", ""]], "type": "str"},
            "ztna-ems-tag": {"v_range": [["7.0.0", ""]], "type": "raw"},
            "ztna-geo-tag": {"v_range": [["7.0.0", ""]], "type": "raw"},
            "ztna-status": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "_policy_block": {"v_range": [["7.0.3", ""]], "type": "int"},
            "dlp-profile": {"v_range": [["7.2.0", ""]], "type": "str"},
            "fec": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "nat46": {"v_range": [["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "nat64": {"v_range": [["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "pfcp-profile": {"v_range": [["7.0.1", "7.2.0"], ["7.4.3", ""]], "type": "str"},
            "policy-expiry": {"v_range": [["7.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "policy-expiry-date": {"v_range": [["7.2.0", ""]], "type": "str"},
            "sctp-filter-profile": {"v_range": [["7.0.1", "7.2.4"], ["7.4.0", ""]], "type": "str"},
            "sgt": {"v_range": [["7.0.1", ""]], "type": "raw"},
            "sgt-check": {"v_range": [["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "tcp-timeout-pid": {"v_range": [["7.0.3", "7.2.0"], ["7.4.3", ""]], "type": "str"},
            "udp-timeout-pid": {"v_range": [["7.0.3", "7.2.0"], ["7.4.3", ""]], "type": "str"},
            "diffserv-copy": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "dstaddr6-negate": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "internet-service6": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "internet-service6-custom": {"v_range": [["7.2.1", ""]], "type": "raw"},
            "internet-service6-custom-group": {"v_range": [["7.2.1", ""]], "type": "raw"},
            "internet-service6-group": {"v_range": [["7.2.1", ""]], "type": "raw"},
            "internet-service6-name": {"v_range": [["7.2.1", ""]], "type": "raw"},
            "internet-service6-negate": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "internet-service6-src": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "internet-service6-src-custom": {"v_range": [["7.2.1", ""]], "type": "raw"},
            "internet-service6-src-custom-group": {"v_range": [["7.2.1", ""]], "type": "raw"},
            "internet-service6-src-group": {"v_range": [["7.2.1", ""]], "type": "raw"},
            "internet-service6-src-name": {"v_range": [["7.2.1", ""]], "type": "raw"},
            "internet-service6-src-negate": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "network-service-dynamic": {"v_range": [["7.2.1", ""]], "type": "raw"},
            "network-service-src-dynamic": {"v_range": [["7.2.1", ""]], "type": "raw"},
            "reputation-direction6": {"v_range": [["7.2.1", ""]], "choices": ["source", "destination"], "type": "str"},
            "reputation-minimum6": {"v_range": [["7.2.1", ""]], "type": "int"},
            "srcaddr6-negate": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ip-version-type": {"v_range": [["7.4.0", ""]], "type": "str"},
            "ips-voip-filter": {"v_range": [["7.4.0", ""]], "type": "str"},
            "pcp-inbound": {"v_range": [["7.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "pcp-outbound": {"v_range": [["7.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "pcp-poolname": {"v_range": [["7.4.0", ""]], "type": "raw"},
            "policy-behaviour-type": {"v_range": [["7.4.0", ""]], "type": "str"},
            "policy-expiry-date-utc": {"v_range": [["7.4.0", ""]], "type": "str"},
            "ztna-device-ownership": {"v_range": [["7.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ztna-ems-tag-secondary": {"v_range": [["7.4.0", ""]], "type": "raw"},
            "ztna-policy-redirect": {"v_range": [["7.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ztna-tags-match-logic": {"v_range": [["7.4.0", ""]], "choices": ["or", "and"], "type": "str"},
            "casb-profile": {"v_range": [["7.4.1", ""]], "type": "str"},
            "virtual-patch-profile": {"v_range": [["7.4.1", ""]], "type": "str"},
            "diameter-filter-profile": {"v_range": [["7.4.2", ""]], "type": "str"},
            "port-preserve": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "cgn-sw-eif-ctrl": {"v_range": [["7.6.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "eif-check": {"v_range": [["7.6.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "eif-learn": {"v_range": [["7.6.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "log-http-transaction": {"v_range": [["7.6.0", ""]], "choices": ["disable", "enable", "all", "utm"], "type": "str"},
            "radius-ip-auth-bypass": {"v_range": [["7.6.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "object position": {"type": "list", "elements": "str"}
        }
    },
    "pkg_firewall_policy46": {
        "stated": true,
        "options": {
            "action": {"choices": ["deny", "accept"], "type": "str"},
            "comments": {"type": "str"},
            "dstaddr": {"type": "raw"},
            "dstintf": {"type": "str"},
            "fixedport": {"choices": ["disable", "enable"], "type": "str"},
            "logtraffic": {"choices": ["disable", "enable"], "type": "str"},
            "per-ip-shaper": {"type": "str"},
            "permit-any-host": {"choices": ["disable", "enable"], "type": "str"},
            "policyid": {"required": true, "type": "int"},
            "schedule": {"type": "str"},
            "service": {"type": "raw"},
            "srcaddr": {"type": "raw"},
            "srcintf": {"type": "str"},
            "status": {"choices": ["disable", "enable"], "type": "str"},
            "tags": {"v_range": [["6.0.0", "6.4.14"]], "type": "str"},
            "tcp-mss-receiver": {"type": "int"},
            "tcp-mss-sender": {"type": "int"},
            "traffic-shaper": {"type": "str"},
            "traffic-shaper-reverse": {"type": "str"},
            "uuid": {"type": "str"},
            "ippool": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "logtraffic-start": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "poolname": {"v_range": [["6.2.0", ""]], "type": "raw"},
            "name": {"v_range": [["6.4.2", ""]], "type": "str"},
            "cgn-log-server-grp": {"v_range": [["6.2.7", "6.2.12"], ["6.4.3", ""]], "type": "str"},
            "policy-offload": {"v_range": [["6.2.7", "6.2.12"], ["6.4.3", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "pkg_firewall_policy6": {
        "stated": true,
        "options": {
            "action": {"choices": ["deny", "accept", "ipsec", "ssl-vpn"], "type": "str"},
            "app-category": {"type": "raw"},
            "application": {"type": "raw"},
            "application-list": {"type": "str"},
            "auto-asic-offload": {"v_range": [["6.0.0", "7.2.0"], ["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "av-profile": {"type": "str"},
            "comments": {"type": "str"},
            "custom-log-fields": {"type": "raw"},
            "devices": {"v_range": [["6.0.0", "7.2.1"]], "type": "raw"},
            "diffserv-forward": {"choices": ["disable", "enable"], "type": "str"},
            "diffserv-reverse": {"choices": ["disable", "enable"], "type": "str"},
            "diffservcode-forward": {"type": "str"},
            "diffservcode-rev": {"type": "str"},
            "dlp-sensor": {"type": "str"},
            "dscp-match": {"v_range": [["6.0.0", "7.2.1"]], "choices": ["disable", "enable"], "type": "str"},
            "dscp-negate": {"v_range": [["6.0.0", "7.2.1"]], "choices": ["disable", "enable"], "type": "str"},
            "dscp-value": {"v_range": [["6.0.0", "7.2.1"]], "type": "str"},
            "dsri": {"choices": ["disable", "enable"], "type": "str"},
            "dstaddr": {"type": "raw"},
            "dstaddr-negate": {"choices": ["disable", "enable"], "type": "str"},
            "dstintf": {"type": "raw"},
            "firewall-session-dirty": {"choices": ["check-all", "check-new"], "type": "str"},
            "fixedport": {"choices": ["disable", "enable"], "type": "str"},
            "global-label": {"type": "str"},
            "groups": {"type": "raw"},
            "icap-profile": {"type": "str"},
            "inbound": {"choices": ["disable", "enable"], "type": "str"},
            "ippool": {"choices": ["disable", "enable"], "type": "str"},
            "ips-sensor": {"type": "str"},
            "label": {"type": "str"},
            "logtraffic": {"choices": ["disable", "enable", "all", "utm"], "type": "str"},
            "logtraffic-start": {"choices": ["disable", "enable"], "type": "str"},
            "mms-profile": {"v_range": [["6.0.0", "7.2.0"], ["7.4.3", ""]], "type": "str"},
            "name": {"type": "str"},
            "nat": {"choices": ["disable", "enable"], "type": "str"},
            "natinbound": {"choices": ["disable", "enable"], "type": "str"},
            "natoutbound": {"choices": ["disable", "enable"], "type": "str"},
            "np-accelation": {"v_range": [["6.0.0", "6.4.14"]], "choices": ["disable", "enable"], "type": "str"},
            "outbound": {"choices": ["disable", "enable"], "type": "str"},
            "per-ip-shaper": {"type": "str"},
            "policyid": {"required": true, "type": "int"},
            "poolname": {"type": "raw"},
            "profile-group": {"type": "str"},
            "profile-protocol-options": {"v_range": [["6.0.0", "7.2.1"], ["7.4.3", ""]], "type": "str"},
            "profile-type": {"choices": ["single", "group"], "type": "str"},
            "replacemsg-override-group": {"type": "str"},
            "rsso": {"choices": ["disable", "enable"], "type": "str"},
            "schedule": {"type": "str"},
            "send-deny-packet": {"choices": ["disable", "enable"], "type": "str"},
            "service": {"type": "raw"},
            "service-negate": {"choices": ["disable", "enable"], "type": "str"},
            "session-ttl": {"type": "raw"},
            "spamfilter-profile": {"v_range": [["6.0.0", "7.2.1"]], "type": "str"},
            "srcaddr": {"type": "raw"},
            "srcaddr-negate": {"choices": ["disable", "enable"], "type": "str"},
            "srcintf": {"type": "raw"},
            "ssl-mirror": {"choices": ["disable", "enable"], "type": "str"},
            "ssl-mirror-intf": {"type": "raw"},
            "ssl-ssh-profile": {"v_range": [["6.0.0", "7.2.1"], ["7.4.3", ""]], "type": "str"},
            "status": {"choices": ["disable", "enable"], "type": "str"},
            "tags": {"v_range": [["6.0.0", "6.4.14"]], "type": "str"},
            "tcp-mss-receiver": {"type": "int"},
            "tcp-mss-sender": {"type": "int"},
            "tcp-session-without-syn": {"choices": ["all", "data-only", "disable"], "type": "str"},
            "timeout-send-rst": {"choices": ["disable", "enable"], "type": "str"},
            "traffic-shaper": {"type": "str"},
            "traffic-shaper-reverse": {"type": "str"},
            "url-category": {"type": "raw"},
            "users": {"type": "raw"},
            "utm-status": {"choices": ["disable", "enable"], "type": "str"},
            "uuid": {"type": "str"},
            "vlan-cos-fwd": {"type": "int"},
            "vlan-cos-rev": {"type": "int"},
            "voip-profile": {"type": "str"},
            "vpntunnel": {"type": "str"},
            "webfilter-profile": {"type": "str"},
            "anti-replay": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "app-group": {"v_range": [["6.2.0", ""]], "type": "raw"},
            "cifs-profile": {"v_range": [["6.2.0", ""]], "type": "str"},
            "dnsfilter-profile": {"v_range": [["6.2.0", ""]], "type": "str"},
            "emailfilter-profile": {"v_range": [["6.2.0", ""]], "type": "str"},
            "http-policy-redirect": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "inspection-mode": {"v_range": [["6.2.0", ""]], "choices": ["proxy", "flow"], "type": "str"},
            "np-acceleration": {"v_range": [["6.2.0", "7.2.0"], ["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ssh-filter-profile": {"v_range": [["6.2.0", "7.2.1"], ["7.4.3", ""]], "type": "str"},
            "ssh-policy-redirect": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "tos": {"v_range": [["6.2.0", ""]], "type": "str"},
            "tos-mask": {"v_range": [["6.2.0", ""]], "type": "str"},
            "tos-negate": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "vlan-filter": {"v_range": [["6.2.0", ""]], "type": "str"},
            "waf-profile": {"v_range": [["6.2.1", ""]], "type": "str"},
            "webcache": {"v_range": [["6.2.1", "7.2.0"], ["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "webcache-https": {"v_range": [["6.2.1", "7.2.0"], ["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "webproxy-forward-server": {"v_range": [["6.2.1", ""]], "type": "str"},
            "webproxy-profile": {"v_range": [["6.2.1", ""]], "type": "str"},
            "casi-profile": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"},
            "fsso-groups": {"v_range": [["6.2.3", ""]], "type": "raw"},
            "decrypted-traffic-mirror": {"v_range": [["6.4.0", "6.4.0"]], "type": "str"},
            "cgn-log-server-grp": {"v_range": [["6.2.7", "6.2.12"], ["6.4.3", ""]], "type": "str"},
            "policy-offload": {"v_range": [["6.2.7", "6.2.12"], ["6.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "_policy_block": {"v_range": [["7.0.3", ""]], "type": "int"}
        }
    },
    "pkg_firewall_policy64": {
        "stated": true,
        "options": {
            "action": {"choices": ["deny", "accept"], "type": "str"},
            "comments": {"type": "str"},
            "dstaddr": {"type": "raw"},
            "dstintf": {"type": "str"},
            "fixedport": {"choices": ["disable", "enable"], "type": "str"},
            "ippool": {"choices": ["disable", "enable"], "type": "str"},
            "logtraffic": {"choices": ["disable", "enable"], "type": "str"},
            "per-ip-shaper": {"type": "str"},
            "permit-any-host": {"choices": ["disable", "enable"], "type": "str"},
            "policyid": {"required": true, "type": "int"},
            "poolname": {"type": "raw"},
            "schedule": {"type": "str"},
            "service": {"type": "raw"},
            "srcaddr": {"type": "raw"},
            "srcintf": {"type": "str"},
            "status": {"choices": ["disable", "enable"], "type": "str"},
            "tags": {"v_range": [["6.0.0", "6.4.14"]], "type": "str"},
            "tcp-mss-receiver": {"type": "int"},
            "tcp-mss-sender": {"type": "int"},
            "traffic-shaper": {"type": "str"},
            "traffic-shaper-reverse": {"type": "str"},
            "uuid": {"type": "str"},
            "logtraffic-start": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "name": {"v_range": [["6.4.2", ""]], "type": "str"},
            "policy-offload": {"v_range": [["6.2.7", "6.2.12"], ["6.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "cgn-session-quota": {"v_range": [["6.2.7", "6.2.12"], ["6.4.3", ""]], "type": "int"},
            "cgn-eif": {"v_range": [["6.2.7", "6.2.12"], ["6.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "cgn-log-server-grp": {"v_range": [["6.2.7", "6.2.12"], ["6.4.3", ""]], "type": "str"},
            "cgn-resource-quota": {"v_range": [["6.2.7", "6.2.12"], ["6.4.3", ""]], "type": "int"},
            "cgn-eim": {"v_range": [["6.2.7", "6.2.12"], ["6.4.3", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "pkg_firewall_policy6_sectionvalue": {
        "stated": true,
        "options": {"attr": {"choices": ["label", "global-label"], "type": "str"}, "name": {"type": "str"}}
    },
    "pkg_firewall_policy_sectionvalue": {
        "stated": true,
        "options": {"attr": {"choices": ["label", "global-label"], "type": "str"}, "name": {"type": "str"}}
    },
    "pkg_firewall_policy_vpndstnode": {
        "stated": true,
        "options": {
            "host": {"v_range": [["6.0.0", "7.0.2"]], "type": "str"},
            "seq": {"v_range": [["6.0.0", "7.0.2"]], "required": true, "type": "int"},
            "subnet": {"v_range": [["6.0.0", "7.0.2"]], "type": "str"}
        }
    },
    "pkg_firewall_policy_vpnsrcnode": {
        "stated": true,
        "options": {
            "host": {"v_range": [["6.0.0", "7.0.2"]], "type": "str"},
            "seq": {"v_range": [["6.0.0", "7.0.2"]], "required": true, "type": "int"},
            "subnet": {"v_range": [["6.0.0", "7.0.2"]], "type": "str"}
        }
    },
    "pkg_firewall_proxypolicy": {
        "stated": true,
        "options": {
            "action": {"choices": ["accept", "deny", "redirect"], "type": "str"},
            "application-list": {"type": "str"},
            "av-profile": {"type": "str"},
            "comments": {"type": "str"},
            "disclaimer": {"choices": ["disable", "domain", "policy", "user"], "type": "str"},
            "dlp-sensor": {"type": "str"},
            "dstaddr": {"type": "raw"},
            "dstaddr-negate": {"choices": ["disable", "enable"], "type": "str"},
            "dstaddr6": {"type": "raw"},
            "dstintf": {"type": "raw"},
            "global-label": {"type": "str"},
            "groups": {"type": "raw"},
            "http-tunnel-auth": {"choices": ["disable", "enable"], "type": "str"},
            "icap-profile": {"type": "str"},
            "internet-service": {"choices": ["disable", "enable"], "type": "str"},
            "internet-service-custom": {"type": "raw"},
            "internet-service-id": {"type": "raw"},
            "internet-service-negate": {"choices": ["disable", "enable"], "type": "str"},
            "ips-sensor": {"type": "str"},
            "label": {"type": "str"},
            "logtraffic": {"choices": ["disable", "all", "utm"], "type": "str"},
            "logtraffic-start": {"choices": ["disable", "enable"], "type": "str"},
            "mms-profile": {"v_range": [["6.0.0", "7.2.0"], ["7.4.3", ""]], "type": "str"},
            "policyid": {"required": true, "type": "int"},
            "poolname": {"type": "raw"},
            "profile-group": {"type": "str"},
            "profile-protocol-options": {"v_range": [["6.0.0", "7.2.2"], ["7.2.4", "7.2.4"], ["7.4.2", ""]], "type": "str"},
            "profile-type": {"choices": ["single", "group"], "type": "str"},
            "proxy": {"choices": ["explicit-web", "transparent-web", "ftp", "wanopt", "ssh", "ssh-tunnel", "access-proxy", "ztna-proxy"], "type": "str"},
            "redirect-url": {"type": "str"},
            "replacemsg-override-group": {"type": "str"},
            "scan-botnet-connections": {"v_range": [["6.0.0", "7.2.1"]], "choices": ["disable", "block", "monitor"], "type": "str"},
            "schedule": {"type": "str"},
            "service": {"type": "raw"},
            "service-negate": {"choices": ["disable", "enable"], "type": "str"},
            "spamfilter-profile": {"v_range": [["6.0.0", "7.2.1"]], "type": "str"},
            "srcaddr": {"type": "raw"},
            "srcaddr-negate": {"choices": ["disable", "enable"], "type": "str"},
            "srcaddr6": {"type": "raw"},
            "srcintf": {"type": "raw"},
            "ssl-ssh-profile": {"v_range": [["6.0.0", "7.2.2"], ["7.2.4", "7.2.4"], ["7.4.2", ""]], "type": "str"},
            "status": {"choices": ["disable", "enable"], "type": "str"},
            "tags": {"v_range": [["6.0.0", "6.4.14"]], "type": "str"},
            "transparent": {"choices": ["disable", "enable"], "type": "str"},
            "users": {"type": "raw"},
            "utm-status": {"choices": ["disable", "enable"], "type": "str"},
            "uuid": {"type": "str"},
            "waf-profile": {"type": "str"},
            "webcache": {"v_range": [["6.0.0", "7.2.0"], ["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "webcache-https": {"v_range": [["6.0.0", "7.2.0"], ["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "webfilter-profile": {"type": "str"},
            "webproxy-forward-server": {"type": "str"},
            "webproxy-profile": {"type": "str"},
            "cifs-profile": {"v_range": [["6.2.0", ""]], "type": "str"},
            "emailfilter-profile": {"v_range": [["6.2.0", ""]], "type": "str"},
            "internet-service-custom-group": {"v_range": [["6.2.0", ""]], "type": "raw"},
            "internet-service-group": {"v_range": [["6.2.0", ""]], "type": "raw"},
            "session-ttl": {"v_range": [["6.2.0", ""]], "type": "raw"},
            "ssh-filter-profile": {"v_range": [["6.2.0", "7.2.2"], ["7.2.4", "7.2.4"], ["7.4.2", ""]], "type": "str"},
            "ssh-policy-redirect": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "decrypted-traffic-mirror": {"v_range": [["6.4.0", "7.2.2"], ["7.2.4", "7.2.4"], ["7.4.2", ""]], "type": "str"},
            "internet-service-name": {"v_range": [["6.4.0", "7.2.2"], ["7.2.4", "7.2.4"], ["7.4.2", ""]], "type": "raw"},
            "file-filter-profile": {"v_range": [["6.4.1", "7.2.2"], ["7.2.4", "7.2.4"], ["7.4.2", ""]], "type": "str"},
            "name": {"v_range": [["6.4.2", "7.2.2"], ["7.2.4", "7.2.4"], ["7.4.2", ""]], "type": "str"},
            "access-proxy": {"v_range": [["7.0.0", "7.2.2"], ["7.2.4", "7.2.4"], ["7.4.2", ""]], "type": "raw"},
            "device-ownership": {"v_range": [["7.0.0", "7.2.2"], ["7.2.4", "7.2.4"], ["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "videofilter-profile": {"v_range": [["7.0.0", "7.2.2"], ["7.2.4", "7.2.4"], ["7.4.2", ""]], "type": "str"},
            "voip-profile": {"v_range": [["7.0.0", "7.2.2"], ["7.4.3", ""]], "type": "str"},
            "ztna-ems-tag": {"v_range": [["7.0.0", "7.2.2"], ["7.2.4", "7.2.4"], ["7.4.2", ""]], "type": "raw"},
            "access-proxy6": {"v_range": [["7.0.1", "7.2.2"], ["7.2.4", "7.2.4"], ["7.4.2", ""]], "type": "raw"},
            "block-notification": {"v_range": [["7.0.3", "7.2.1"], ["7.2.4", "7.2.4"], ["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "dlp-profile": {"v_range": [["7.2.0", "7.2.1"], ["7.2.4", "7.2.4"], ["7.4.2", ""]], "type": "str"},
            "sctp-filter-profile": {"v_range": [["7.0.1", "7.2.2"], ["7.2.4", "7.2.4"], ["7.4.2", ""]], "type": "str"},
            "ztna-tags-match-logic": {"v_range": [["7.0.2", "7.2.1"], ["7.2.4", "7.2.4"], ["7.4.2", ""]], "choices": ["or", "and"], "type": "str"},
            "casb-profile": {"v_range": [["7.4.2", ""]], "type": "str"},
            "detect-https-in-http-request": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "diameter-filter-profile": {"v_range": [["7.4.2", ""]], "type": "str"},
            "internet-service6": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "internet-service6-custom": {"v_range": [["7.4.2", ""]], "type": "raw"},
            "internet-service6-custom-group": {"v_range": [["7.4.2", ""]], "type": "raw"},
            "internet-service6-group": {"v_range": [["7.4.2", ""]], "type": "raw"},
            "internet-service6-name": {"v_range": [["7.4.2", ""]], "type": "raw"},
            "internet-service6-negate": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ips-voip-filter": {"v_range": [["7.4.2", ""]], "type": "str"},
            "virtual-patch-profile": {"v_range": [["7.4.2", ""]], "type": "str"},
            "_policy_block": {"v_range": [["7.6.0", ""]], "type": "int"},
            "dnsfilter-profile": {"v_range": [["7.6.0", ""]], "type": "raw"},
            "log-http-transaction": {"v_range": [["7.6.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ztna-proxy": {"v_range": [["7.6.0", ""]], "type": "raw"}
        }
    },
    "pkg_firewall_proxypolicy_sectionvalue": {
        "stated": true,
        "options": {"attr": {"choices": ["label", "global-label"], "type": "str"}, "name": {"type": "str"}}
    },
    "pkg_firewall_securitypolicy": {
        "stated": true,
        "options": {
            "action": {"v_range": [["6.2.1", ""]], "choices": ["deny", "accept"], "type": "str"},
            "app-category": {"v_range": [["6.2.1", ""]], "type": "raw"},
            "app-group": {"v_range": [["6.2.1", ""]], "type": "raw"},
            "application": {"v_range": [["6.2.1", ""]], "type": "raw"},
            "application-list": {"v_range": [["6.2.1", ""]], "type": "str"},
            "av-profile": {"v_range": [["6.2.1", ""]], "type": "str"},
            "cifs-profile": {"v_range": [["6.2.1", ""]], "type": "str"},
            "comments": {"v_range": [["6.2.1", ""]], "type": "str"},
            "dlp-sensor": {"v_range": [["6.2.1", ""]], "type": "str"},
            "dnsfilter-profile": {"v_range": [["6.2.1", ""]], "type": "str"},
            "dstaddr4": {"v_range": [["6.2.1", ""]], "type": "raw"},
            "dstaddr6": {"v_range": [["6.2.1", ""]], "type": "raw"},
            "dstintf": {"v_range": [["6.2.1", ""]], "type": "raw"},
            "emailfilter-profile": {"v_range": [["6.2.1", ""]], "type": "str"},
            "enforce-default-app-port": {"v_range": [["6.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "groups": {"v_range": [["6.2.1", ""]], "type": "raw"},
            "icap-profile": {"v_range": [["6.2.1", ""]], "type": "str"},
            "internet-service": {"v_range": [["6.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "internet-service-custom": {"v_range": [["6.2.1", ""]], "type": "raw"},
            "internet-service-custom-group": {"v_range": [["6.2.1", ""]], "type": "raw"},
            "internet-service-group": {"v_range": [["6.2.1", ""]], "type": "raw"},
            "internet-service-id": {"v_range": [["6.2.1", ""]], "type": "raw"},
            "internet-service-negate": {"v_range": [["6.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "internet-service-src": {"v_range": [["6.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "internet-service-src-custom": {"v_range": [["6.2.1", ""]], "type": "raw"},
            "internet-service-src-custom-group": {"v_range": [["6.2.1", ""]], "type": "raw"},
            "internet-service-src-group": {"v_range": [["6.2.1", ""]], "type": "raw"},
            "internet-service-src-id": {"v_range": [["6.2.1", ""]], "type": "raw"},
            "internet-service-src-negate": {"v_range": [["6.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ips-sensor": {"v_range": [["6.2.1", ""]], "type": "str"},
            "logtraffic": {"v_range": [["6.2.1", ""]], "choices": ["disable", "all", "utm"], "type": "str"},
            "logtraffic-start": {"v_range": [["6.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "mms-profile": {"v_range": [["6.2.1", "7.2.0"], ["7.4.3", ""]], "type": "str"},
            "name": {"v_range": [["6.2.1", ""]], "type": "str"},
            "policyid": {"v_range": [["6.2.1", ""]], "required": true, "type": "int"},
            "profile-group": {"v_range": [["6.2.1", ""]], "type": "str"},
            "profile-protocol-options": {"v_range": [["6.2.1", "7.2.2"], ["7.2.4", "7.2.4"], ["7.4.2", ""]], "type": "str"},
            "profile-type": {"v_range": [["6.2.1", ""]], "choices": ["single", "group"], "type": "str"},
            "schedule": {"v_range": [["6.2.1", ""]], "type": "str"},
            "service": {"v_range": [["6.2.1", ""]], "type": "raw"},
            "service-negate": {"v_range": [["6.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "srcaddr4": {"v_range": [["6.2.1", ""]], "type": "raw"},
            "srcaddr6": {"v_range": [["6.2.1", ""]], "type": "raw"},
            "srcintf": {"v_range": [["6.2.1", ""]], "type": "raw"},
            "ssh-filter-profile": {"v_range": [["6.2.1", ""]], "type": "str"},
            "ssl-ssh-profile": {"v_range": [["6.2.1", "7.2.2"], ["7.2.4", "7.2.4"], ["7.4.2", ""]], "type": "str"},
            "status": {"v_range": [["6.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "url-category": {"v_range": [["6.2.1", ""]], "type": "raw"},
            "users": {"v_range": [["6.2.1", ""]], "type": "raw"},
            "utm-status": {"v_range": [["6.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "uuid": {"v_range": [["6.2.1", ""]], "type": "str"},
            "voip-profile": {"v_range": [["6.2.1", ""]], "type": "str"},
            "webfilter-profile": {"v_range": [["6.2.1", ""]], "type": "str"},
            "fsso-groups": {"v_range": [["6.2.2", ""]], "type": "raw"},
            "global-label": {"v_range": [["6.2.3", ""]], "type": "str"},
            "send-deny-packet": {"v_range": [["6.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "dstaddr": {"v_range": [["6.4.0", "7.2.2"], ["7.2.4", "7.2.4"], ["7.4.2", ""]], "type": "raw"},
            "internet-service-name": {"v_range": [["6.4.0", "7.2.2"], ["7.2.4", "7.2.4"], ["7.4.2", ""]], "type": "raw"},
            "internet-service-src-name": {"v_range": [["6.4.0", "7.2.2"], ["7.2.4", "7.2.4"], ["7.4.2", ""]], "type": "raw"},
            "srcaddr": {"v_range": [["6.4.0", "7.2.2"], ["7.2.4", "7.2.4"], ["7.4.2", ""]], "type": "raw"},
            "dstaddr-negate": {"v_range": [["6.4.2", "7.2.2"], ["7.2.4", "7.2.4"], ["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "file-filter-profile": {"v_range": [["6.4.1", "7.2.2"], ["7.2.4", "7.2.4"], ["7.4.2", ""]], "type": "str"},
            "srcaddr-negate": {"v_range": [["6.4.2", "7.2.2"], ["7.2.4", "7.2.4"], ["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "learning-mode": {"v_range": [["7.0.0", "7.2.2"], ["7.2.4", "7.2.4"], ["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "videofilter-profile": {"v_range": [["7.0.0", "7.2.2"], ["7.2.4", "7.2.4"], ["7.4.2", ""]], "type": "str"},
            "_policy_block": {"v_range": [["7.0.3", ""]], "type": "int"},
            "dlp-profile": {"v_range": [["7.2.0", "7.2.1"], ["7.2.4", "7.2.4"], ["7.4.2", ""]], "type": "str"},
            "nat46": {"v_range": [["7.0.2", "7.2.1"], ["7.2.4", "7.2.4"], ["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "nat64": {"v_range": [["7.0.2", "7.2.1"], ["7.2.4", "7.2.4"], ["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "sctp-filter-profile": {"v_range": [["7.0.1", "7.2.2"], ["7.2.4", "7.2.4"], ["7.4.2", ""]], "type": "str"},
            "internet-service6": {"v_range": [["7.2.1", "7.2.1"], ["7.2.4", "7.2.4"], ["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "internet-service6-custom": {"v_range": [["7.2.1", "7.2.1"], ["7.2.4", "7.2.4"], ["7.4.2", ""]], "type": "raw"},
            "internet-service6-custom-group": {"v_range": [["7.2.1", "7.2.1"], ["7.2.4", "7.2.4"], ["7.4.2", ""]], "type": "raw"},
            "internet-service6-group": {"v_range": [["7.2.1", "7.2.1"], ["7.2.4", "7.2.4"], ["7.4.2", ""]], "type": "raw"},
            "internet-service6-name": {"v_range": [["7.2.1", "7.2.1"], ["7.2.4", "7.2.4"], ["7.4.2", ""]], "type": "raw"},
            "internet-service6-negate": {
                "v_range": [["7.2.1", "7.2.1"], ["7.2.4", "7.2.4"], ["7.4.2", ""]],
                "choices": ["disable", "enable"],
                "type": "str"
            },
            "internet-service6-src": {"v_range": [["7.2.1", "7.2.1"], ["7.2.4", "7.2.4"], ["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "internet-service6-src-custom": {"v_range": [["7.2.1", "7.2.1"], ["7.2.4", "7.2.4"], ["7.4.2", ""]], "type": "raw"},
            "internet-service6-src-custom-group": {"v_range": [["7.2.1", "7.2.1"], ["7.2.4", "7.2.4"], ["7.4.2", ""]], "type": "raw"},
            "internet-service6-src-group": {"v_range": [["7.2.1", "7.2.1"], ["7.2.4", "7.2.4"], ["7.4.2", ""]], "type": "raw"},
            "internet-service6-src-name": {"v_range": [["7.2.1", "7.2.1"], ["7.2.4", "7.2.4"], ["7.4.2", ""]], "type": "raw"},
            "internet-service6-src-negate": {
                "v_range": [["7.2.1", "7.2.1"], ["7.2.4", "7.2.4"], ["7.4.2", ""]],
                "choices": ["disable", "enable"],
                "type": "str"
            },
            "casb-profile": {"v_range": [["7.4.2", ""]], "type": "str"},
            "diameter-filter-profile": {"v_range": [["7.4.2", ""]], "type": "str"},
            "dstaddr6-negate": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ips-voip-filter": {"v_range": [["7.4.2", ""]], "type": "str"},
            "srcaddr6-negate": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "virtual-patch-profile": {"v_range": [["7.4.2", ""]], "type": "str"}
        }
    },
    "pkg_firewall_securitypolicy_sectionvalue": {
        "stated": true,
        "options": {
            "attr": {"v_range": [["6.4.0", "7.2.2"], ["7.2.4", "7.2.4"], ["7.4.2", ""]], "choices": ["label", "global-label"], "type": "str"},
            "name": {"v_range": [["6.4.0", "7.2.2"], ["7.2.4", "7.2.4"], ["7.4.2", ""]], "type": "str"}
        }
    },
    "pkg_firewall_shapingpolicy": {
        "stated": true,
        "options": {
            "app-category": {"type": "raw"},
            "application": {"type": "raw"},
            "dstaddr": {"type": "raw"},
            "dstaddr6": {"type": "raw"},
            "dstintf": {"type": "raw"},
            "groups": {"type": "raw"},
            "id": {"required": true, "type": "int"},
            "ip-version": {"choices": ["4", "6"], "type": "str"},
            "per-ip-shaper": {"type": "str"},
            "schedule": {"type": "str"},
            "service": {"type": "raw"},
            "srcaddr": {"type": "raw"},
            "srcaddr6": {"type": "raw"},
            "status": {"choices": ["disable", "enable"], "type": "str"},
            "traffic-shaper": {"type": "str"},
            "traffic-shaper-reverse": {"type": "str"},
            "url-category": {"type": "raw"},
            "users": {"type": "raw"},
            "app-group": {"v_range": [["6.2.0", ""]], "type": "raw"},
            "class-id": {"v_range": [["6.2.0", ""]], "type": "raw"},
            "comment": {"v_range": [["6.2.0", ""]], "type": "str"},
            "diffserv-forward": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "diffserv-reverse": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "diffservcode-forward": {"v_range": [["6.2.0", ""]], "type": "str"},
            "diffservcode-rev": {"v_range": [["6.2.0", ""]], "type": "str"},
            "internet-service": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "internet-service-custom": {"v_range": [["6.2.0", ""]], "type": "raw"},
            "internet-service-custom-group": {"v_range": [["6.2.0", ""]], "type": "raw"},
            "internet-service-group": {"v_range": [["6.2.0", ""]], "type": "raw"},
            "internet-service-id": {"v_range": [["6.2.0", ""]], "type": "raw"},
            "internet-service-src": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "internet-service-src-custom": {"v_range": [["6.2.0", ""]], "type": "raw"},
            "internet-service-src-custom-group": {"v_range": [["6.2.0", ""]], "type": "raw"},
            "internet-service-src-group": {"v_range": [["6.2.0", ""]], "type": "raw"},
            "internet-service-src-id": {"v_range": [["6.2.0", ""]], "type": "raw"},
            "name": {"v_range": [["6.2.0", ""]], "type": "str"},
            "srcintf": {"v_range": [["6.2.0", ""]], "type": "raw"},
            "tos": {"v_range": [["6.2.0", ""]], "type": "str"},
            "tos-mask": {"v_range": [["6.2.0", ""]], "type": "str"},
            "tos-negate": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "uuid": {"v_range": [["6.2.1", ""]], "type": "str"},
            "internet-service-name": {"v_range": [["6.4.0", ""]], "type": "raw"},
            "internet-service-src-name": {"v_range": [["6.4.0", ""]], "type": "raw"},
            "cos": {"v_range": [["7.4.0", ""]], "type": "str"},
            "cos-mask": {"v_range": [["7.4.0", ""]], "type": "str"},
            "traffic-type": {"v_range": [["7.4.0", ""]], "choices": ["forwarding", "local-in", "local-out"], "type": "str"}
        }
    },
    "pkg_footer_policy": {
        "stated": true,
        "options": {
            "action": {"choices": ["deny", "accept", "ipsec", "ssl-vpn", "redirect", "isolate"], "type": "str"},
            "active-auth-method": {"choices": ["ntlm", "basic", "digest", "form"], "type": "str"},
            "anti-replay": {"choices": ["disable", "enable"], "type": "str"},
            "app-category": {"type": "raw"},
            "app-group": {"type": "raw"},
            "application": {"type": "raw"},
            "application-charts": {"type": "list", "choices": ["top10-app", "top10-p2p-user", "top10-media-user"], "elements": "str"},
            "application-list": {"type": "str"},
            "auth-cert": {"type": "str"},
            "auth-method": {"choices": ["basic", "digest", "ntlm", "fsae", "form", "fsso", "rsso"], "type": "str"},
            "auth-path": {"choices": ["disable", "enable"], "type": "str"},
            "auth-portal": {"choices": ["disable", "enable"], "type": "str"},
            "auth-redirect-addr": {"type": "str"},
            "auto-asic-offload": {"choices": ["disable", "enable"], "type": "str"},
            "av-profile": {"type": "str"},
            "bandwidth": {"choices": ["disable", "enable"], "type": "str"},
            "block-notification": {"choices": ["disable", "enable"], "type": "str"},
            "captive-portal-exempt": {"choices": ["disable", "enable"], "type": "str"},
            "capture-packet": {"choices": ["disable", "enable"], "type": "str"},
            "casi-profile": {"type": "raw"},
            "central-nat": {"choices": ["disable", "enable"], "type": "str"},
            "cifs-profile": {"type": "str"},
            "client-reputation": {"choices": ["disable", "enable"], "type": "str"},
            "client-reputation-mode": {"choices": ["learning", "monitoring"], "type": "str"},
            "comments": {"type": "raw"},
            "custom-log-fields": {"type": "raw"},
            "deep-inspection-options": {"type": "raw"},
            "delay-tcp-npu-session": {"choices": ["disable", "enable"], "type": "str"},
            "delay-tcp-npu-sessoin": {"choices": ["disable", "enable"], "type": "str"},
            "device-detection-portal": {"choices": ["disable", "enable"], "type": "str"},
            "devices": {"type": "raw"},
            "diffserv-forward": {"choices": ["disable", "enable"], "type": "str"},
            "diffserv-reverse": {"choices": ["disable", "enable"], "type": "str"},
            "diffservcode-forward": {"type": "str"},
            "diffservcode-rev": {"type": "str"},
            "disclaimer": {"choices": ["disable", "enable", "user", "domain", "policy"], "type": "str"},
            "dlp-sensor": {"type": "raw"},
            "dnsfilter-profile": {"type": "str"},
            "dponly": {"choices": ["disable", "enable"], "type": "str"},
            "dscp-match": {"choices": ["disable", "enable"], "type": "str"},
            "dscp-negate": {"choices": ["disable", "enable"], "type": "str"},
            "dscp-value": {"type": "str"},
            "dsri": {"choices": ["disable", "enable"], "type": "str"},
            "dstaddr": {"type": "raw"},
            "dstaddr-negate": {"choices": ["disable", "enable"], "type": "str"},
            "dstaddr6": {"type": "raw"},
            "dstintf": {"type": "raw"},
            "dynamic-profile": {"choices": ["disable", "enable"], "type": "str"},
            "dynamic-profile-access": {
                "type": "list",
                "choices": ["imap", "smtp", "pop3", "http", "ftp", "im", "nntp", "imaps", "smtps", "pop3s", "https", "ftps", "ssh"],
                "elements": "str"
            },
            "dynamic-profile-fallthrough": {"choices": ["disable", "enable"], "type": "str"},
            "dynamic-profile-group": {"type": "raw"},
            "email-collect": {"choices": ["disable", "enable"], "type": "str"},
            "email-collection-portal": {"choices": ["disable", "enable"], "type": "str"},
            "emailfilter-profile": {"type": "str"},
            "endpoint-check": {"choices": ["disable", "enable"], "type": "str"},
            "endpoint-compliance": {"choices": ["disable", "enable"], "type": "str"},
            "endpoint-keepalive-interface": {"type": "raw"},
            "endpoint-profile": {"type": "raw"},
            "failed-connection": {"choices": ["disable", "enable"], "type": "str"},
            "fall-through-unauthenticated": {"choices": ["disable", "enable"], "type": "str"},
            "firewall-session-dirty": {"choices": ["check-all", "check-new"], "type": "str"},
            "fixedport": {"choices": ["disable", "enable"], "type": "str"},
            "forticlient-compliance-devices": {"type": "list", "choices": ["windows-pc", "mac", "iphone-ipad", "android"], "elements": "str"},
            "forticlient-compliance-enforcement-portal": {"choices": ["disable", "enable"], "type": "str"},
            "fsae": {"choices": ["disable", "enable"], "type": "str"},
            "fsae-server-for-ntlm": {"type": "raw"},
            "fsso": {"choices": ["disable", "enable"], "type": "str"},
            "fsso-agent-for-ntlm": {"type": "str"},
            "geo-location": {"choices": ["disable", "enable"], "type": "str"},
            "geoip-anycast": {"choices": ["disable", "enable"], "type": "str"},
            "global-label": {"type": "str"},
            "groups": {"type": "raw"},
            "gtp-profile": {"type": "str"},
            "http-policy-redirect": {"choices": ["disable", "enable"], "type": "str"},
            "icap-profile": {"type": "str"},
            "identity-based": {"choices": ["disable", "enable"], "type": "str"},
            "identity-based-policy": {
                "v_range": [["6.0.0", "6.2.0"]],
                "type": "list",
                "options": {
                    "action": {"v_range": [["6.0.0", "6.2.0"]], "choices": ["deny", "accept"], "type": "str"},
                    "application-charts": {
                        "v_range": [["6.0.0", "6.2.0"]],
                        "type": "list",
                        "choices": ["top10-app", "top10-p2p-user", "top10-media-user"],
                        "elements": "str"
                    },
                    "application-list": {"v_range": [["6.0.0", "6.2.0"]], "type": "str"},
                    "av-profile": {"v_range": [["6.0.0", "6.2.0"]], "type": "str"},
                    "capture-packet": {"v_range": [["6.0.0", "6.2.0"]], "choices": ["disable", "enable"], "type": "str"},
                    "deep-inspection-options": {"v_range": [["6.0.0", "6.2.0"]], "type": "str"},
                    "devices": {"v_range": [["6.0.0", "6.2.0"]], "type": "str"},
                    "dlp-sensor": {"v_range": [["6.0.0", "6.2.0"]], "type": "str"},
                    "dstaddr": {"v_range": [["6.0.0", "6.2.0"]], "type": "str"},
                    "dstaddr-negate": {"v_range": [["6.0.0", "6.2.0"]], "choices": ["disable", "enable"], "type": "str"},
                    "endpoint-compliance": {"v_range": [["6.0.0", "6.2.0"]], "choices": ["disable", "enable"], "type": "str"},
                    "groups": {"v_range": [["6.0.0", "6.2.0"]], "type": "str"},
                    "icap-profile": {"v_range": [["6.0.0", "6.2.0"]], "type": "str"},
                    "id": {"v_range": [["6.0.0", "6.2.0"]], "type": "int"},
                    "ips-sensor": {"v_range": [["6.0.0", "6.2.0"]], "type": "str"},
                    "logtraffic": {"v_range": [["6.0.0", "6.2.0"]], "choices": ["disable", "enable", "all", "utm"], "type": "str"},
                    "logtraffic-app": {"v_range": [["6.0.0", "6.2.0"]], "choices": ["disable", "enable"], "type": "str"},
                    "logtraffic-start": {"v_range": [["6.0.0", "6.2.0"]], "choices": ["disable", "enable"], "type": "str"},
                    "mms-profile": {"v_range": [["6.0.0", "6.2.0"]], "type": "str"},
                    "per-ip-shaper": {"v_range": [["6.0.0", "6.2.0"]], "type": "str"},
                    "profile-group": {"v_range": [["6.0.0", "6.2.0"]], "type": "str"},
                    "profile-protocol-options": {"v_range": [["6.0.0", "6.2.0"]], "type": "str"},
                    "profile-type": {"v_range": [["6.0.0", "6.2.0"]], "choices": ["single", "group"], "type": "str"},
                    "replacemsg-group": {"v_range": [["6.0.0", "6.2.0"]], "type": "str"},
                    "schedule": {"v_range": [["6.0.0", "6.2.0"]], "type": "str"},
                    "send-deny-packet": {"v_range": [["6.0.0", "6.2.0"]], "choices": ["disable", "enable"], "type": "str"},
                    "service": {"v_range": [["6.0.0", "6.2.0"]], "type": "str"},
                    "service-negate": {"v_range": [["6.0.0", "6.2.0"]], "choices": ["disable", "enable"], "type": "str"},
                    "spamfilter-profile": {"v_range": [["6.0.0", "6.2.0"]], "type": "str"},
                    "sslvpn-portal": {"v_range": [["6.0.0", "6.2.0"]], "type": "str"},
                    "sslvpn-realm": {"v_range": [["6.0.0", "6.2.0"]], "type": "str"},
                    "traffic-shaper": {"v_range": [["6.0.0", "6.2.0"]], "type": "str"},
                    "traffic-shaper-reverse": {"v_range": [["6.0.0", "6.2.0"]], "type": "str"},
                    "users": {"v_range": [["6.0.0", "6.2.0"]], "type": "str"},
                    "utm-status": {"v_range": [["6.0.0", "6.2.0"]], "choices": ["disable", "enable"], "type": "str"},
                    "voip-profile": {"v_range": [["6.0.0", "6.2.0"]], "type": "str"},
                    "webfilter-profile": {"v_range": [["6.0.0", "6.2.0"]], "type": "str"}
                },
                "elements": "dict"
            },
            "identity-based-route": {"type": "str"},
            "identity-from": {"choices": ["auth", "device"], "type": "str"},
            "inbound": {"choices": ["disable", "enable"], "type": "str"},
            "inspection-mode": {"choices": ["proxy", "flow"], "type": "str"},
            "internet-service": {"choices": ["disable", "enable"], "type": "str"},
            "internet-service-custom": {"type": "raw"},
            "internet-service-custom-group": {"type": "raw"},
            "internet-service-group": {"type": "raw"},
            "internet-service-id": {"type": "raw"},
            "internet-service-negate": {"choices": ["disable", "enable"], "type": "str"},
            "internet-service-src": {"choices": ["disable", "enable"], "type": "str"},
            "internet-service-src-custom": {"type": "raw"},
            "internet-service-src-custom-group": {"type": "raw"},
            "internet-service-src-group": {"type": "raw"},
            "internet-service-src-id": {"type": "raw"},
            "internet-service-src-negate": {"choices": ["disable", "enable"], "type": "str"},
            "ip-based": {"choices": ["disable", "enable"], "type": "str"},
            "ippool": {"choices": ["disable", "enable"], "type": "str"},
            "ips-sensor": {"type": "str"},
            "label": {"type": "str"},
            "learning-mode": {"choices": ["disable", "enable"], "type": "str"},
            "log-unmatched-traffic": {"choices": ["disable", "enable"], "type": "str"},
            "logtraffic": {"choices": ["disable", "enable", "all", "utm"], "type": "str"},
            "logtraffic-app": {"choices": ["disable", "enable"], "type": "str"},
            "logtraffic-start": {"choices": ["disable", "enable"], "type": "str"},
            "match-vip": {"choices": ["disable", "enable"], "type": "str"},
            "mms-profile": {"type": "raw"},
            "name": {"type": "str"},
            "nat": {"choices": ["disable", "enable"], "type": "str"},
            "natinbound": {"choices": ["disable", "enable"], "type": "str"},
            "natip": {"type": "str"},
            "natoutbound": {"choices": ["disable", "enable"], "type": "str"},
            "np-acceleration": {"choices": ["disable", "enable"], "type": "str"},
            "ntlm": {"choices": ["disable", "enable"], "type": "str"},
            "ntlm-enabled-browsers": {"type": "raw"},
            "ntlm-guest": {"choices": ["disable", "enable"], "type": "str"},
            "outbound": {"choices": ["disable", "enable"], "type": "str"},
            "per-ip-shaper": {"type": "str"},
            "permit-any-host": {"choices": ["disable", "enable"], "type": "str"},
            "permit-stun-host": {"choices": ["disable", "enable"], "type": "str"},
            "policyid": {"required": true, "type": "int"},
            "poolname": {"type": "raw"},
            "profile-group": {"type": "str"},
            "profile-protocol-options": {"type": "str"},
            "profile-type": {"choices": ["single", "group"], "type": "str"},
            "radius-mac-auth-bypass": {"choices": ["disable", "enable"], "type": "str"},
            "redirect-url": {"type": "str"},
            "replacemsg-group": {"type": "raw"},
            "replacemsg-override-group": {"type": "str"},
            "reputation-direction": {"choices": ["source", "destination"], "type": "str"},
            "reputation-minimum": {"type": "int"},
            "require-tfa": {"choices": ["disable", "enable"], "type": "str"},
            "rsso": {"choices": ["disable", "enable"], "type": "str"},
            "rtp-addr": {"type": "raw"},
            "rtp-nat": {"choices": ["disable", "enable"], "type": "str"},
            "scan-botnet-connections": {"choices": ["disable", "block", "monitor"], "type": "str"},
            "schedule": {"type": "str"},
            "schedule-timeout": {"choices": ["disable", "enable"], "type": "str"},
            "send-deny-packet": {"choices": ["disable", "enable"], "type": "str"},
            "service": {"type": "raw"},
            "service-negate": {"choices": ["disable", "enable"], "type": "str"},
            "session-ttl": {"type": "raw"},
            "sessions": {"choices": ["disable", "enable"], "type": "str"},
            "spamfilter-profile": {"type": "raw"},
            "srcaddr": {"type": "raw"},
            "srcaddr-negate": {"choices": ["disable", "enable"], "type": "str"},
            "srcaddr6": {"type": "raw"},
            "srcintf": {"type": "raw"},
            "ssh-filter-profile": {"type": "str"},
            "ssh-policy-redirect": {"choices": ["disable", "enable"], "type": "str"},
            "ssl-mirror": {"choices": ["disable", "enable"], "type": "str"},
            "ssl-mirror-intf": {"type": "raw"},
            "ssl-ssh-profile": {"type": "str"},
            "sslvpn-auth": {"choices": ["any", "local", "radius", "ldap", "tacacs+"], "type": "str"},
            "sslvpn-ccert": {"choices": ["disable", "enable"], "type": "str"},
            "sslvpn-cipher": {"choices": ["any", "high", "medium"], "type": "str"},
            "sso-auth-method": {"choices": ["fsso", "rsso"], "type": "str"},
            "status": {"choices": ["disable", "enable"], "type": "str"},
            "tags": {"type": "raw"},
            "tcp-mss-receiver": {"type": "int"},
            "tcp-mss-sender": {"type": "int"},
            "tcp-reset": {"choices": ["disable", "enable"], "type": "str"},
            "tcp-session-without-syn": {"choices": ["all", "data-only", "disable"], "type": "str"},
            "timeout-send-rst": {"choices": ["disable", "enable"], "type": "str"},
            "tos": {"type": "str"},
            "tos-mask": {"type": "str"},
            "tos-negate": {"choices": ["disable", "enable"], "type": "str"},
            "traffic-shaper": {"type": "str"},
            "traffic-shaper-reverse": {"type": "str"},
            "transaction-based": {"choices": ["disable", "enable"], "type": "str"},
            "url-category": {"type": "raw"},
            "users": {"type": "raw"},
            "utm-inspection-mode": {"choices": ["proxy", "flow"], "type": "str"},
            "utm-status": {"choices": ["disable", "enable"], "type": "str"},
            "uuid": {"type": "str"},
            "vlan-cos-fwd": {"type": "int"},
            "vlan-cos-rev": {"type": "int"},
            "vlan-filter": {"type": "str"},
            "voip-profile": {"type": "str"},
            "vpntunnel": {"type": "str"},
            "waf-profile": {"type": "str"},
            "wanopt": {"choices": ["disable", "enable"], "type": "str"},
            "wanopt-detection": {"choices": ["active", "passive", "off"], "type": "str"},
            "wanopt-passive-opt": {"choices": ["default", "transparent", "non-transparent"], "type": "str"},
            "wanopt-peer": {"type": "str"},
            "wanopt-profile": {"type": "str"},
            "wccp": {"choices": ["disable", "enable"], "type": "str"},
            "web-auth-cookie": {"choices": ["disable", "enable"], "type": "str"},
            "webcache": {"choices": ["disable", "enable"], "type": "str"},
            "webcache-https": {"choices": ["disable", "ssl-server", "any", "enable"], "type": "str"},
            "webfilter-profile": {"type": "str"},
            "webproxy-forward-server": {"type": "str"},
            "webproxy-profile": {"type": "str"},
            "wsso": {"choices": ["disable", "enable"], "type": "str"},
            "fsso-groups": {"v_range": [["6.2.1", ""]], "type": "raw"},
            "match-vip-only": {"v_range": [["6.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "np-accelation": {"v_range": [["6.2.1", "6.4.14"]], "choices": ["disable", "enable"], "type": "str"},
            "best-route": {"v_range": [["6.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "decrypted-traffic-mirror": {"v_range": [["6.4.0", ""]], "type": "str"},
            "geoip-match": {"v_range": [["6.4.0", ""]], "choices": ["physical-location", "registered-location"], "type": "str"},
            "internet-service-name": {"v_range": [["6.4.0", ""]], "type": "raw"},
            "internet-service-src-name": {"v_range": [["6.4.0", ""]], "type": "raw"},
            "poolname6": {"v_range": [["6.4.0", ""]], "type": "raw"},
            "src-vendor-mac": {"v_range": [["6.4.0", ""]], "type": "raw"},
            "vendor-mac": {"v_range": [["6.4.0", ""]], "type": "raw"},
            "file-filter-profile": {"v_range": [["6.4.1", ""]], "type": "str"},
            "cgn-eif": {"v_range": [["6.2.7", "6.2.12"], ["6.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "cgn-eim": {"v_range": [["6.2.7", "6.2.12"], ["6.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "cgn-log-server-grp": {"v_range": [["6.2.7", "6.2.12"], ["6.4.3", ""]], "type": "raw"},
            "cgn-resource-quota": {"v_range": [["6.2.7", "6.2.12"], ["6.4.3", ""]], "type": "int"},
            "cgn-session-quota": {"v_range": [["6.2.7", "6.2.12"], ["6.4.3", ""]], "type": "int"},
            "policy-offload": {"v_range": [["6.2.7", "6.2.12"], ["6.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "dynamic-shaping": {"v_range": [["6.4.6", ""]], "choices": ["disable", "enable"], "type": "str"},
            "passive-wan-health-measurement": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "videofilter-profile": {"v_range": [["7.0.0", ""]], "type": "str"},
            "ztna-ems-tag": {"v_range": [["7.0.0", ""]], "type": "raw"},
            "ztna-geo-tag": {"v_range": [["7.0.0", ""]], "type": "raw"},
            "ztna-status": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "access-proxy": {"v_range": [["7.0.3", ""]], "type": "raw"},
            "dlp-profile": {"v_range": [["7.2.0", ""]], "type": "str"},
            "dynamic-bypass": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "fec": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "force-proxy": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "http-tunnel-auth": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ia-profile": {"v_range": [["7.0.3", ""]], "type": "raw"},
            "isolator-server": {"v_range": [["7.0.3", ""]], "type": "raw"},
            "log-http-transaction": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable", "all", "utm"], "type": "str"},
            "max-session-per-user": {"v_range": [["7.0.3", ""]], "type": "int"},
            "nat46": {"v_range": [["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "nat64": {"v_range": [["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "pass-through": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "pfcp-profile": {"v_range": [["7.0.1", ""]], "type": "str"},
            "policy-expiry": {"v_range": [["7.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "policy-expiry-date": {"v_range": [["7.2.0", ""]], "type": "str"},
            "reverse-cache": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "sctp-filter-profile": {"v_range": [["7.0.1", ""]], "type": "str"},
            "sgt": {"v_range": [["7.0.1", ""]], "type": "raw"},
            "sgt-check": {"v_range": [["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "tcp-timeout-pid": {"v_range": [["7.0.3", ""]], "type": "raw"},
            "transparent": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "type": {
                "v_range": [["7.0.3", ""]],
                "choices": ["explicit-web", "transparent", "explicit-ftp", "ssh-tunnel", "ssh", "wanopt", "access-proxy"],
                "type": "str"
            },
            "udp-timeout-pid": {"v_range": [["7.0.3", ""]], "type": "raw"},
            "ztna-tags-match-logic": {"v_range": [["7.0.3", ""]], "choices": ["or", "and"], "type": "str"},
            "uuid-idx": {"v_range": [["7.0.1", ""]], "type": "int"},
            "device-ownership": {"v_range": [["7.0.5", "7.0.12"], ["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ssh-policy-check": {"v_range": [["7.0.5", "7.0.12"], ["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "extended-log": {"v_range": [["7.0.11", "7.0.12"], ["7.2.5", "7.2.5"], ["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "diffserv-copy": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "dstaddr6-negate": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "internet-service6": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "internet-service6-custom": {"v_range": [["7.2.1", ""]], "type": "raw"},
            "internet-service6-custom-group": {"v_range": [["7.2.1", ""]], "type": "raw"},
            "internet-service6-group": {"v_range": [["7.2.1", ""]], "type": "raw"},
            "internet-service6-name": {"v_range": [["7.2.1", ""]], "type": "raw"},
            "internet-service6-negate": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "internet-service6-src": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "internet-service6-src-custom": {"v_range": [["7.2.1", ""]], "type": "raw"},
            "internet-service6-src-custom-group": {"v_range": [["7.2.1", ""]], "type": "raw"},
            "internet-service6-src-group": {"v_range": [["7.2.1", ""]], "type": "raw"},
            "internet-service6-src-name": {"v_range": [["7.2.1", ""]], "type": "raw"},
            "internet-service6-src-negate": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "network-service-dynamic": {"v_range": [["7.2.1", ""]], "type": "raw"},
            "network-service-src-dynamic": {"v_range": [["7.2.1", ""]], "type": "raw"},
            "reputation-direction6": {"v_range": [["7.2.1", ""]], "choices": ["source", "destination"], "type": "str"},
            "reputation-minimum6": {"v_range": [["7.2.1", ""]], "type": "int"},
            "srcaddr6-negate": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "_policy_block": {"v_range": [["7.2.2", ""]], "type": "int"},
            "isolator-profile": {"v_range": [["7.2.2", ""]], "type": "raw"},
            "policy-expiry-date-utc": {"v_range": [["7.2.2", ""]], "type": "str"},
            "ztna-device-ownership": {"v_range": [["7.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ztna-policy-redirect": {"v_range": [["7.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ip-version-type": {"v_range": [["7.2.3", ""]], "type": "str"},
            "ips-voip-filter": {"v_range": [["7.2.3", ""]], "type": "str"},
            "policy-behaviour-type": {"v_range": [["7.2.3", ""]], "type": "str"},
            "pcp-inbound": {"v_range": [["7.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "pcp-outbound": {"v_range": [["7.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "pcp-poolname": {"v_range": [["7.4.0", ""]], "type": "raw"},
            "ztna-ems-tag-secondary": {"v_range": [["7.4.0", ""]], "type": "raw"},
            "casb-profile": {"v_range": [["7.4.1", ""]], "type": "str"},
            "implicit-proxy-detection": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "virtual-patch-profile": {"v_range": [["7.4.1", ""]], "type": "str"},
            "detect-https-in-http-request": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "diameter-filter-profile": {"v_range": [["7.4.2", ""]], "type": "str"},
            "redirect-profile": {"v_range": [["7.4.2", ""]], "type": "raw"},
            "port-preserve": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "cgn-sw-eif-ctrl": {"v_range": [["7.6.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "eif-check": {"v_range": [["7.6.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "eif-learn": {"v_range": [["7.6.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "radius-ip-auth-bypass": {"v_range": [["7.6.0", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "pkg_footer_policy6": {
        "stated": true,
        "options": {
            "action": {"choices": ["deny", "accept", "ipsec", "ssl-vpn"], "type": "str"},
            "anti-replay": {"choices": ["disable", "enable"], "type": "str"},
            "app-category": {"type": "raw"},
            "app-group": {"type": "raw"},
            "application": {"type": "raw"},
            "application-charts": {"type": "list", "choices": ["top10-app", "top10-p2p-user", "top10-media-user"], "elements": "str"},
            "application-list": {"type": "raw"},
            "auto-asic-offload": {"choices": ["disable", "enable"], "type": "str"},
            "av-profile": {"type": "raw"},
            "casi-profile": {"type": "raw"},
            "cifs-profile": {"type": "raw"},
            "comments": {"type": "str"},
            "custom-log-fields": {"type": "raw"},
            "deep-inspection-options": {"type": "raw"},
            "device-detection-portal": {"choices": ["disable", "enable"], "type": "str"},
            "devices": {"type": "raw"},
            "diffserv-forward": {"choices": ["disable", "enable"], "type": "str"},
            "diffserv-reverse": {"choices": ["disable", "enable"], "type": "str"},
            "diffservcode-forward": {"type": "str"},
            "diffservcode-rev": {"type": "str"},
            "dlp-sensor": {"type": "raw"},
            "dnsfilter-profile": {"type": "raw"},
            "dscp-match": {"choices": ["disable", "enable"], "type": "str"},
            "dscp-negate": {"choices": ["disable", "enable"], "type": "str"},
            "dscp-value": {"type": "str"},
            "dsri": {"choices": ["disable", "enable"], "type": "str"},
            "dstaddr": {"type": "raw"},
            "dstaddr-negate": {"choices": ["disable", "enable"], "type": "str"},
            "dstintf": {"type": "raw"},
            "dynamic-profile": {"choices": ["disable", "enable"], "type": "str"},
            "dynamic-profile-access": {
                "type": "list",
                "choices": ["imap", "smtp", "pop3", "http", "ftp", "im", "nntp", "imaps", "smtps", "pop3s", "https", "ftps"],
                "elements": "str"
            },
            "dynamic-profile-group": {"type": "raw"},
            "email-collection-portal": {"choices": ["disable", "enable"], "type": "str"},
            "emailfilter-profile": {"type": "raw"},
            "firewall-session-dirty": {"choices": ["check-all", "check-new"], "type": "str"},
            "fixedport": {"choices": ["disable", "enable"], "type": "str"},
            "fsae": {"choices": ["disable", "enable"], "type": "str"},
            "global-label": {"type": "str"},
            "groups": {"type": "raw"},
            "http-policy-redirect": {"choices": ["disable", "enable"], "type": "str"},
            "icap-profile": {"type": "raw"},
            "identity-based": {"choices": ["disable", "enable"], "type": "str"},
            "identity-based-policy6": {
                "v_range": [["6.0.0", "6.2.0"]],
                "type": "list",
                "options": {
                    "action": {"v_range": [["6.0.0", "6.2.0"]], "choices": ["deny", "accept"], "type": "str"},
                    "application-list": {"v_range": [["6.0.0", "6.2.0"]], "type": "str"},
                    "av-profile": {"v_range": [["6.0.0", "6.2.0"]], "type": "str"},
                    "deep-inspection-options": {"v_range": [["6.0.0", "6.2.0"]], "type": "str"},
                    "devices": {"v_range": [["6.0.0", "6.2.0"]], "type": "str"},
                    "dlp-sensor": {"v_range": [["6.0.0", "6.2.0"]], "type": "str"},
                    "endpoint-compliance": {"v_range": [["6.0.0", "6.2.0"]], "choices": ["disable", "enable"], "type": "str"},
                    "groups": {"v_range": [["6.0.0", "6.2.0"]], "type": "str"},
                    "icap-profile": {"v_range": [["6.0.0", "6.2.0"]], "type": "str"},
                    "id": {"v_range": [["6.0.0", "6.2.0"]], "type": "int"},
                    "ips-sensor": {"v_range": [["6.0.0", "6.2.0"]], "type": "str"},
                    "logtraffic": {"v_range": [["6.0.0", "6.2.0"]], "choices": ["disable", "enable", "all", "utm"], "type": "str"},
                    "mms-profile": {"v_range": [["6.0.0", "6.2.0"]], "type": "str"},
                    "per-ip-shaper": {"v_range": [["6.0.0", "6.2.0"]], "type": "str"},
                    "profile-group": {"v_range": [["6.0.0", "6.2.0"]], "type": "str"},
                    "profile-protocol-options": {"v_range": [["6.0.0", "6.2.0"]], "type": "str"},
                    "profile-type": {"v_range": [["6.0.0", "6.2.0"]], "choices": ["single", "group"], "type": "str"},
                    "replacemsg-group": {"v_range": [["6.0.0", "6.2.0"]], "type": "str"},
                    "schedule": {"v_range": [["6.0.0", "6.2.0"]], "type": "str"},
                    "send-deny-packet": {"v_range": [["6.0.0", "6.2.0"]], "choices": ["disable", "enable"], "type": "str"},
                    "service": {"v_range": [["6.0.0", "6.2.0"]], "type": "str"},
                    "service-negate": {"v_range": [["6.0.0", "6.2.0"]], "choices": ["disable", "enable"], "type": "str"},
                    "spamfilter-profile": {"v_range": [["6.0.0", "6.2.0"]], "type": "str"},
                    "sslvpn-portal": {"v_range": [["6.0.0", "6.2.0"]], "type": "str"},
                    "sslvpn-realm": {"v_range": [["6.0.0", "6.2.0"]], "type": "str"},
                    "traffic-shaper": {"v_range": [["6.0.0", "6.2.0"]], "type": "str"},
                    "traffic-shaper-reverse": {"v_range": [["6.0.0", "6.2.0"]], "type": "str"},
                    "utm-status": {"v_range": [["6.0.0", "6.2.0"]], "choices": ["disable", "enable"], "type": "str"},
                    "voip-profile": {"v_range": [["6.0.0", "6.2.0"]], "type": "str"},
                    "webfilter-profile": {"v_range": [["6.0.0", "6.2.0"]], "type": "str"}
                },
                "elements": "dict"
            },
            "identity-from": {"choices": ["auth", "device"], "type": "str"},
            "inbound": {"choices": ["disable", "enable"], "type": "str"},
            "inspection-mode": {"choices": ["proxy", "flow"], "type": "str"},
            "ippool": {"choices": ["disable", "enable"], "type": "str"},
            "ips-sensor": {"type": "raw"},
            "label": {"type": "str"},
            "logtraffic": {"choices": ["disable", "enable", "all", "utm"], "type": "str"},
            "logtraffic-start": {"choices": ["disable", "enable"], "type": "str"},
            "mms-profile": {"type": "raw"},
            "name": {"type": "str"},
            "nat": {"choices": ["disable", "enable"], "type": "str"},
            "natinbound": {"choices": ["disable", "enable"], "type": "str"},
            "natoutbound": {"choices": ["disable", "enable"], "type": "str"},
            "np-accelation": {"choices": ["disable", "enable"], "type": "str"},
            "np-acceleration": {"choices": ["disable", "enable"], "type": "str"},
            "outbound": {"choices": ["disable", "enable"], "type": "str"},
            "per-ip-shaper": {"type": "raw"},
            "policyid": {"required": true, "type": "int"},
            "poolname": {"type": "raw"},
            "profile-group": {"type": "raw"},
            "profile-protocol-options": {"type": "raw"},
            "profile-type": {"choices": ["single", "group"], "type": "str"},
            "replacemsg-group": {"type": "raw"},
            "replacemsg-override-group": {"type": "raw"},
            "rsso": {"choices": ["disable", "enable"], "type": "str"},
            "schedule": {"type": "raw"},
            "send-deny-packet": {"choices": ["disable", "enable"], "type": "str"},
            "service": {"type": "raw"},
            "service-negate": {"choices": ["disable", "enable"], "type": "str"},
            "session-ttl": {"type": "raw"},
            "spamfilter-profile": {"type": "raw"},
            "srcaddr": {"type": "raw"},
            "srcaddr-negate": {"choices": ["disable", "enable"], "type": "str"},
            "srcintf": {"type": "raw"},
            "ssh-filter-profile": {"type": "raw"},
            "ssh-policy-redirect": {"choices": ["disable", "enable"], "type": "str"},
            "ssl-mirror": {"choices": ["disable", "enable"], "type": "str"},
            "ssl-mirror-intf": {"type": "raw"},
            "ssl-ssh-profile": {"type": "raw"},
            "sslvpn-auth": {"choices": ["any", "local", "radius", "ldap", "tacacs+"], "type": "str"},
            "sslvpn-ccert": {"choices": ["disable", "enable"], "type": "str"},
            "sslvpn-cipher": {"choices": ["any", "high", "medium"], "type": "str"},
            "status": {"choices": ["disable", "enable"], "type": "str"},
            "tags": {"type": "raw"},
            "tcp-mss-receiver": {"type": "int"},
            "tcp-mss-sender": {"type": "int"},
            "tcp-session-without-syn": {"choices": ["all", "data-only", "disable"], "type": "str"},
            "timeout-send-rst": {"choices": ["disable", "enable"], "type": "str"},
            "tos": {"type": "str"},
            "tos-mask": {"type": "str"},
            "tos-negate": {"choices": ["disable", "enable"], "type": "str"},
            "traffic-shaper": {"type": "raw"},
            "traffic-shaper-reverse": {"type": "raw"},
            "url-category": {"type": "raw"},
            "users": {"type": "raw"},
            "utm-inspection-mode": {"choices": ["proxy", "flow"], "type": "str"},
            "utm-status": {"choices": ["disable", "enable"], "type": "str"},
            "uuid": {"type": "str"},
            "vlan-cos-fwd": {"type": "int"},
            "vlan-cos-rev": {"type": "int"},
            "vlan-filter": {"type": "str"},
            "voip-profile": {"type": "raw"},
            "vpntunnel": {"type": "raw"},
            "webfilter-profile": {"type": "raw"},
            "waf-profile": {"v_range": [["6.2.1", ""]], "type": "raw"},
            "webcache": {"v_range": [["6.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "webcache-https": {"v_range": [["6.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "webproxy-forward-server": {"v_range": [["6.2.1", ""]], "type": "raw"},
            "webproxy-profile": {"v_range": [["6.2.1", ""]], "type": "raw"},
            "fsso-groups": {"v_range": [["6.2.3", ""]], "type": "raw"},
            "decrypted-traffic-mirror": {"v_range": [["6.4.0", ""]], "type": "raw"},
            "file-filter-profile": {"v_range": [["6.4.1", ""]], "type": "raw"},
            "cgn-log-server-grp": {"v_range": [["6.2.7", "6.2.12"], ["6.4.3", ""]], "type": "str"},
            "policy-offload": {"v_range": [["6.2.7", "6.2.12"], ["6.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "_policy_block": {"v_range": [["7.2.2", ""]], "type": "int"}
        }
    },
    "pkg_footer_shapingpolicy": {
        "stated": true,
        "options": {
            "app-category": {"type": "raw"},
            "app-group": {"type": "raw"},
            "application": {"type": "raw"},
            "class-id": {"type": "raw"},
            "comment": {"type": "str"},
            "diffserv-forward": {"choices": ["disable", "enable"], "type": "str"},
            "diffserv-reverse": {"choices": ["disable", "enable"], "type": "str"},
            "diffservcode-forward": {"type": "str"},
            "diffservcode-rev": {"type": "str"},
            "dstaddr": {"type": "raw"},
            "dstaddr6": {"type": "raw"},
            "dstintf": {"type": "raw"},
            "groups": {"type": "raw"},
            "id": {"required": true, "type": "int"},
            "internet-service": {"choices": ["disable", "enable"], "type": "str"},
            "internet-service-custom": {"type": "raw"},
            "internet-service-custom-group": {"type": "raw"},
            "internet-service-group": {"type": "raw"},
            "internet-service-id": {"type": "raw"},
            "internet-service-src": {"choices": ["disable", "enable"], "type": "str"},
            "internet-service-src-custom": {"type": "raw"},
            "internet-service-src-custom-group": {"type": "raw"},
            "internet-service-src-group": {"type": "raw"},
            "internet-service-src-id": {"type": "raw"},
            "ip-version": {"choices": ["4", "6"], "type": "str"},
            "per-ip-shaper": {"type": "str"},
            "schedule": {"type": "str"},
            "service": {"type": "raw"},
            "srcaddr": {"type": "raw"},
            "srcaddr6": {"type": "raw"},
            "srcintf": {"type": "raw"},
            "status": {"choices": ["disable", "enable"], "type": "str"},
            "tos": {"type": "str"},
            "tos-mask": {"type": "str"},
            "tos-negate": {"choices": ["disable", "enable"], "type": "str"},
            "traffic-shaper": {"type": "str"},
            "traffic-shaper-reverse": {"type": "str"},
            "url-category": {"type": "raw"},
            "users": {"type": "raw"},
            "uuid": {"v_range": [["6.2.1", ""]], "type": "str"},
            "internet-service-name": {"v_range": [["6.4.0", ""]], "type": "raw"},
            "internet-service-src-name": {"v_range": [["6.4.0", ""]], "type": "raw"},
            "class-id-reverse": {"v_range": [["7.0.3", ""]], "type": "int"},
            "service-type": {"v_range": [["7.0.3", ""]], "choices": ["service", "internet-service"], "type": "str"},
            "uuid-idx": {"v_range": [["7.2.1", ""]], "type": "int"},
            "cos": {"v_range": [["7.4.0", ""]], "type": "str"},
            "cos-mask": {"v_range": [["7.4.0", ""]], "type": "str"},
            "traffic-type": {"v_range": [["7.4.0", ""]], "choices": ["forwarding", "local-in", "local-out"], "type": "str"}
        }
    },
    "pkg_header_policy": {
        "stated": true,
        "options": {
            "action": {"choices": ["deny", "accept", "ipsec", "ssl-vpn", "redirect", "isolate"], "type": "str"},
            "active-auth-method": {"choices": ["ntlm", "basic", "digest", "form"], "type": "str"},
            "anti-replay": {"choices": ["disable", "enable"], "type": "str"},
            "app-category": {"type": "raw"},
            "app-group": {"type": "raw"},
            "application": {"type": "raw"},
            "application-charts": {"type": "list", "choices": ["top10-app", "top10-p2p-user", "top10-media-user"], "elements": "str"},
            "application-list": {"type": "str"},
            "auth-cert": {"type": "str"},
            "auth-method": {"choices": ["basic", "digest", "ntlm", "fsae", "form", "fsso", "rsso"], "type": "str"},
            "auth-path": {"choices": ["disable", "enable"], "type": "str"},
            "auth-portal": {"choices": ["disable", "enable"], "type": "str"},
            "auth-redirect-addr": {"type": "str"},
            "auto-asic-offload": {"choices": ["disable", "enable"], "type": "str"},
            "av-profile": {"type": "str"},
            "bandwidth": {"choices": ["disable", "enable"], "type": "str"},
            "block-notification": {"choices": ["disable", "enable"], "type": "str"},
            "captive-portal-exempt": {"choices": ["disable", "enable"], "type": "str"},
            "capture-packet": {"choices": ["disable", "enable"], "type": "str"},
            "casi-profile": {"type": "raw"},
            "central-nat": {"choices": ["disable", "enable"], "type": "str"},
            "cifs-profile": {"type": "str"},
            "client-reputation": {"choices": ["disable", "enable"], "type": "str"},
            "client-reputation-mode": {"choices": ["learning", "monitoring"], "type": "str"},
            "comments": {"type": "raw"},
            "custom-log-fields": {"type": "raw"},
            "deep-inspection-options": {"type": "raw"},
            "delay-tcp-npu-session": {"choices": ["disable", "enable"], "type": "str"},
            "delay-tcp-npu-sessoin": {"choices": ["disable", "enable"], "type": "str"},
            "device-detection-portal": {"choices": ["disable", "enable"], "type": "str"},
            "devices": {"type": "raw"},
            "diffserv-forward": {"choices": ["disable", "enable"], "type": "str"},
            "diffserv-reverse": {"choices": ["disable", "enable"], "type": "str"},
            "diffservcode-forward": {"type": "str"},
            "diffservcode-rev": {"type": "str"},
            "disclaimer": {"choices": ["disable", "enable", "user", "domain", "policy"], "type": "str"},
            "dlp-sensor": {"type": "raw"},
            "dnsfilter-profile": {"type": "str"},
            "dponly": {"choices": ["disable", "enable"], "type": "str"},
            "dscp-match": {"choices": ["disable", "enable"], "type": "str"},
            "dscp-negate": {"choices": ["disable", "enable"], "type": "str"},
            "dscp-value": {"type": "str"},
            "dsri": {"choices": ["disable", "enable"], "type": "str"},
            "dstaddr": {"type": "raw"},
            "dstaddr-negate": {"choices": ["disable", "enable"], "type": "str"},
            "dstaddr6": {"type": "raw"},
            "dstintf": {"type": "raw"},
            "dynamic-profile": {"choices": ["disable", "enable"], "type": "str"},
            "dynamic-profile-access": {
                "type": "list",
                "choices": ["imap", "smtp", "pop3", "http", "ftp", "im", "nntp", "imaps", "smtps", "pop3s", "https", "ftps", "ssh"],
                "elements": "str"
            },
            "dynamic-profile-fallthrough": {"choices": ["disable", "enable"], "type": "str"},
            "dynamic-profile-group": {"type": "raw"},
            "email-collect": {"choices": ["disable", "enable"], "type": "str"},
            "email-collection-portal": {"choices": ["disable", "enable"], "type": "str"},
            "emailfilter-profile": {"type": "str"},
            "endpoint-check": {"choices": ["disable", "enable"], "type": "str"},
            "endpoint-compliance": {"choices": ["disable", "enable"], "type": "str"},
            "endpoint-keepalive-interface": {"type": "raw"},
            "endpoint-profile": {"type": "raw"},
            "failed-connection": {"choices": ["disable", "enable"], "type": "str"},
            "fall-through-unauthenticated": {"choices": ["disable", "enable"], "type": "str"},
            "firewall-session-dirty": {"choices": ["check-all", "check-new"], "type": "str"},
            "fixedport": {"choices": ["disable", "enable"], "type": "str"},
            "forticlient-compliance-devices": {"type": "list", "choices": ["windows-pc", "mac", "iphone-ipad", "android"], "elements": "str"},
            "forticlient-compliance-enforcement-portal": {"choices": ["disable", "enable"], "type": "str"},
            "fsae": {"choices": ["disable", "enable"], "type": "str"},
            "fsae-server-for-ntlm": {"type": "raw"},
            "fsso": {"choices": ["disable", "enable"], "type": "str"},
            "fsso-agent-for-ntlm": {"type": "str"},
            "geo-location": {"choices": ["disable", "enable"], "type": "str"},
            "geoip-anycast": {"choices": ["disable", "enable"], "type": "str"},
            "global-label": {"type": "str"},
            "groups": {"type": "raw"},
            "gtp-profile": {"type": "str"},
            "http-policy-redirect": {"choices": ["disable", "enable"], "type": "str"},
            "icap-profile": {"type": "str"},
            "identity-based": {"choices": ["disable", "enable"], "type": "str"},
            "identity-based-policy": {
                "v_range": [["6.0.0", "6.2.0"]],
                "type": "list",
                "options": {
                    "action": {"v_range": [["6.0.0", "6.2.0"]], "choices": ["deny", "accept"], "type": "str"},
                    "application-charts": {
                        "v_range": [["6.0.0", "6.2.0"]],
                        "type": "list",
                        "choices": ["top10-app", "top10-p2p-user", "top10-media-user"],
                        "elements": "str"
                    },
                    "application-list": {"v_range": [["6.0.0", "6.2.0"]], "type": "str"},
                    "av-profile": {"v_range": [["6.0.0", "6.2.0"]], "type": "str"},
                    "capture-packet": {"v_range": [["6.0.0", "6.2.0"]], "choices": ["disable", "enable"], "type": "str"},
                    "deep-inspection-options": {"v_range": [["6.0.0", "6.2.0"]], "type": "str"},
                    "devices": {"v_range": [["6.0.0", "6.2.0"]], "type": "str"},
                    "dlp-sensor": {"v_range": [["6.0.0", "6.2.0"]], "type": "str"},
                    "dstaddr": {"v_range": [["6.0.0", "6.2.0"]], "type": "str"},
                    "dstaddr-negate": {"v_range": [["6.0.0", "6.2.0"]], "choices": ["disable", "enable"], "type": "str"},
                    "endpoint-compliance": {"v_range": [["6.0.0", "6.2.0"]], "choices": ["disable", "enable"], "type": "str"},
                    "groups": {"v_range": [["6.0.0", "6.2.0"]], "type": "str"},
                    "icap-profile": {"v_range": [["6.0.0", "6.2.0"]], "type": "str"},
                    "id": {"v_range": [["6.0.0", "6.2.0"]], "type": "int"},
                    "ips-sensor": {"v_range": [["6.0.0", "6.2.0"]], "type": "str"},
                    "logtraffic": {"v_range": [["6.0.0", "6.2.0"]], "choices": ["disable", "enable", "all", "utm"], "type": "str"},
                    "logtraffic-app": {"v_range": [["6.0.0", "6.2.0"]], "choices": ["disable", "enable"], "type": "str"},
                    "logtraffic-start": {"v_range": [["6.0.0", "6.2.0"]], "choices": ["disable", "enable"], "type": "str"},
                    "mms-profile": {"v_range": [["6.0.0", "6.2.0"]], "type": "str"},
                    "per-ip-shaper": {"v_range": [["6.0.0", "6.2.0"]], "type": "str"},
                    "profile-group": {"v_range": [["6.0.0", "6.2.0"]], "type": "str"},
                    "profile-protocol-options": {"v_range": [["6.0.0", "6.2.0"]], "type": "str"},
                    "profile-type": {"v_range": [["6.0.0", "6.2.0"]], "choices": ["single", "group"], "type": "str"},
                    "replacemsg-group": {"v_range": [["6.0.0", "6.2.0"]], "type": "str"},
                    "schedule": {"v_range": [["6.0.0", "6.2.0"]], "type": "str"},
                    "send-deny-packet": {"v_range": [["6.0.0", "6.2.0"]], "choices": ["disable", "enable"], "type": "str"},
                    "service": {"v_range": [["6.0.0", "6.2.0"]], "type": "str"},
                    "service-negate": {"v_range": [["6.0.0", "6.2.0"]], "choices": ["disable", "enable"], "type": "str"},
                    "spamfilter-profile": {"v_range": [["6.0.0", "6.2.0"]], "type": "str"},
                    "sslvpn-portal": {"v_range": [["6.0.0", "6.2.0"]], "type": "str"},
                    "sslvpn-realm": {"v_range": [["6.0.0", "6.2.0"]], "type": "str"},
                    "traffic-shaper": {"v_range": [["6.0.0", "6.2.0"]], "type": "str"},
                    "traffic-shaper-reverse": {"v_range": [["6.0.0", "6.2.0"]], "type": "str"},
                    "users": {"v_range": [["6.0.0", "6.2.0"]], "type": "str"},
                    "utm-status": {"v_range": [["6.0.0", "6.2.0"]], "choices": ["disable", "enable"], "type": "str"},
                    "voip-profile": {"v_range": [["6.0.0", "6.2.0"]], "type": "str"},
                    "webfilter-profile": {"v_range": [["6.0.0", "6.2.0"]], "type": "str"}
                },
                "elements": "dict"
            },
            "identity-based-route": {"type": "str"},
            "identity-from": {"choices": ["auth", "device"], "type": "str"},
            "inbound": {"choices": ["disable", "enable"], "type": "str"},
            "inspection-mode": {"choices": ["proxy", "flow"], "type": "str"},
            "internet-service": {"choices": ["disable", "enable"], "type": "str"},
            "internet-service-custom": {"type": "raw"},
            "internet-service-custom-group": {"type": "raw"},
            "internet-service-group": {"type": "raw"},
            "internet-service-id": {"type": "raw"},
            "internet-service-negate": {"choices": ["disable", "enable"], "type": "str"},
            "internet-service-src": {"choices": ["disable", "enable"], "type": "str"},
            "internet-service-src-custom": {"type": "raw"},
            "internet-service-src-custom-group": {"type": "raw"},
            "internet-service-src-group": {"type": "raw"},
            "internet-service-src-id": {"type": "raw"},
            "internet-service-src-negate": {"choices": ["disable", "enable"], "type": "str"},
            "ip-based": {"choices": ["disable", "enable"], "type": "str"},
            "ippool": {"choices": ["disable", "enable"], "type": "str"},
            "ips-sensor": {"type": "str"},
            "label": {"type": "str"},
            "learning-mode": {"choices": ["disable", "enable"], "type": "str"},
            "log-unmatched-traffic": {"choices": ["disable", "enable"], "type": "str"},
            "logtraffic": {"choices": ["disable", "enable", "all", "utm"], "type": "str"},
            "logtraffic-app": {"choices": ["disable", "enable"], "type": "str"},
            "logtraffic-start": {"choices": ["disable", "enable"], "type": "str"},
            "match-vip": {"choices": ["disable", "enable"], "type": "str"},
            "mms-profile": {"type": "raw"},
            "name": {"type": "str"},
            "nat": {"choices": ["disable", "enable"], "type": "str"},
            "natinbound": {"choices": ["disable", "enable"], "type": "str"},
            "natip": {"type": "str"},
            "natoutbound": {"choices": ["disable", "enable"], "type": "str"},
            "np-acceleration": {"choices": ["disable", "enable"], "type": "str"},
            "ntlm": {"choices": ["disable", "enable"], "type": "str"},
            "ntlm-enabled-browsers": {"type": "raw"},
            "ntlm-guest": {"choices": ["disable", "enable"], "type": "str"},
            "outbound": {"choices": ["disable", "enable"], "type": "str"},
            "per-ip-shaper": {"type": "str"},
            "permit-any-host": {"choices": ["disable", "enable"], "type": "str"},
            "permit-stun-host": {"choices": ["disable", "enable"], "type": "str"},
            "policyid": {"required": true, "type": "int"},
            "poolname": {"type": "raw"},
            "profile-group": {"type": "str"},
            "profile-protocol-options": {"type": "str"},
            "profile-type": {"choices": ["single", "group"], "type": "str"},
            "radius-mac-auth-bypass": {"choices": ["disable", "enable"], "type": "str"},
            "redirect-url": {"type": "str"},
            "replacemsg-group": {"type": "raw"},
            "replacemsg-override-group": {"type": "str"},
            "reputation-direction": {"choices": ["source", "destination"], "type": "str"},
            "reputation-minimum": {"type": "int"},
            "require-tfa": {"choices": ["disable", "enable"], "type": "str"},
            "rsso": {"choices": ["disable", "enable"], "type": "str"},
            "rtp-addr": {"type": "raw"},
            "rtp-nat": {"choices": ["disable", "enable"], "type": "str"},
            "scan-botnet-connections": {"choices": ["disable", "block", "monitor"], "type": "str"},
            "schedule": {"type": "str"},
            "schedule-timeout": {"choices": ["disable", "enable"], "type": "str"},
            "send-deny-packet": {"choices": ["disable", "enable"], "type": "str"},
            "service": {"type": "raw"},
            "service-negate": {"choices": ["disable", "enable"], "type": "str"},
            "session-ttl": {"type": "raw"},
            "sessions": {"choices": ["disable", "enable"], "type": "str"},
            "spamfilter-profile": {"type": "raw"},
            "srcaddr": {"type": "raw"},
            "srcaddr-negate": {"choices": ["disable", "enable"], "type": "str"},
            "srcaddr6": {"type": "raw"},
            "srcintf": {"type": "raw"},
            "ssh-filter-profile": {"type": "str"},
            "ssh-policy-redirect": {"choices": ["disable", "enable"], "type": "str"},
            "ssl-mirror": {"choices": ["disable", "enable"], "type": "str"},
            "ssl-mirror-intf": {"type": "raw"},
            "ssl-ssh-profile": {"type": "str"},
            "sslvpn-auth": {"choices": ["any", "local", "radius", "ldap", "tacacs+"], "type": "str"},
            "sslvpn-ccert": {"choices": ["disable", "enable"], "type": "str"},
            "sslvpn-cipher": {"choices": ["any", "high", "medium"], "type": "str"},
            "sso-auth-method": {"choices": ["fsso", "rsso"], "type": "str"},
            "status": {"choices": ["disable", "enable"], "type": "str"},
            "tags": {"type": "raw"},
            "tcp-mss-receiver": {"type": "int"},
            "tcp-mss-sender": {"type": "int"},
            "tcp-reset": {"choices": ["disable", "enable"], "type": "str"},
            "tcp-session-without-syn": {"choices": ["all", "data-only", "disable"], "type": "str"},
            "timeout-send-rst": {"choices": ["disable", "enable"], "type": "str"},
            "tos": {"type": "str"},
            "tos-mask": {"type": "str"},
            "tos-negate": {"choices": ["disable", "enable"], "type": "str"},
            "traffic-shaper": {"type": "str"},
            "traffic-shaper-reverse": {"type": "str"},
            "transaction-based": {"choices": ["disable", "enable"], "type": "str"},
            "url-category": {"type": "raw"},
            "users": {"type": "raw"},
            "utm-inspection-mode": {"choices": ["proxy", "flow"], "type": "str"},
            "utm-status": {"choices": ["disable", "enable"], "type": "str"},
            "uuid": {"type": "str"},
            "vlan-cos-fwd": {"type": "int"},
            "vlan-cos-rev": {"type": "int"},
            "vlan-filter": {"type": "str"},
            "voip-profile": {"type": "str"},
            "vpntunnel": {"type": "str"},
            "waf-profile": {"type": "str"},
            "wanopt": {"choices": ["disable", "enable"], "type": "str"},
            "wanopt-detection": {"choices": ["active", "passive", "off"], "type": "str"},
            "wanopt-passive-opt": {"choices": ["default", "transparent", "non-transparent"], "type": "str"},
            "wanopt-peer": {"type": "str"},
            "wanopt-profile": {"type": "str"},
            "wccp": {"choices": ["disable", "enable"], "type": "str"},
            "web-auth-cookie": {"choices": ["disable", "enable"], "type": "str"},
            "webcache": {"choices": ["disable", "enable"], "type": "str"},
            "webcache-https": {"choices": ["disable", "ssl-server", "any", "enable"], "type": "str"},
            "webfilter-profile": {"type": "str"},
            "webproxy-forward-server": {"type": "str"},
            "webproxy-profile": {"type": "str"},
            "wsso": {"choices": ["disable", "enable"], "type": "str"},
            "fsso-groups": {"v_range": [["6.2.1", ""]], "type": "raw"},
            "match-vip-only": {"v_range": [["6.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "np-accelation": {"v_range": [["6.2.1", "6.4.14"]], "choices": ["disable", "enable"], "type": "str"},
            "best-route": {"v_range": [["6.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "decrypted-traffic-mirror": {"v_range": [["6.4.0", ""]], "type": "str"},
            "geoip-match": {"v_range": [["6.4.0", ""]], "choices": ["physical-location", "registered-location"], "type": "str"},
            "internet-service-name": {"v_range": [["6.4.0", ""]], "type": "raw"},
            "internet-service-src-name": {"v_range": [["6.4.0", ""]], "type": "raw"},
            "poolname6": {"v_range": [["6.4.0", ""]], "type": "raw"},
            "src-vendor-mac": {"v_range": [["6.4.0", ""]], "type": "raw"},
            "vendor-mac": {"v_range": [["6.4.0", ""]], "type": "raw"},
            "file-filter-profile": {"v_range": [["6.4.1", ""]], "type": "str"},
            "cgn-eif": {"v_range": [["6.2.7", "6.2.12"], ["6.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "cgn-eim": {"v_range": [["6.2.7", "6.2.12"], ["6.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "cgn-log-server-grp": {"v_range": [["6.2.7", "6.2.12"], ["6.4.3", ""]], "type": "raw"},
            "cgn-resource-quota": {"v_range": [["6.2.7", "6.2.12"], ["6.4.3", ""]], "type": "int"},
            "cgn-session-quota": {"v_range": [["6.2.7", "6.2.12"], ["6.4.3", ""]], "type": "int"},
            "policy-offload": {"v_range": [["6.2.7", "6.2.12"], ["6.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "dynamic-shaping": {"v_range": [["6.4.6", ""]], "choices": ["disable", "enable"], "type": "str"},
            "passive-wan-health-measurement": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "videofilter-profile": {"v_range": [["7.0.0", ""]], "type": "str"},
            "ztna-ems-tag": {"v_range": [["7.0.0", ""]], "type": "raw"},
            "ztna-geo-tag": {"v_range": [["7.0.0", ""]], "type": "raw"},
            "ztna-status": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "access-proxy": {"v_range": [["7.0.3", ""]], "type": "raw"},
            "dlp-profile": {"v_range": [["7.2.0", ""]], "type": "str"},
            "dynamic-bypass": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "fec": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "force-proxy": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "http-tunnel-auth": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ia-profile": {"v_range": [["7.0.3", ""]], "type": "raw"},
            "isolator-server": {"v_range": [["7.0.3", ""]], "type": "raw"},
            "log-http-transaction": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable", "all", "utm"], "type": "str"},
            "max-session-per-user": {"v_range": [["7.0.3", ""]], "type": "int"},
            "nat46": {"v_range": [["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "nat64": {"v_range": [["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "pass-through": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "pfcp-profile": {"v_range": [["7.0.1", ""]], "type": "str"},
            "policy-expiry": {"v_range": [["7.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "policy-expiry-date": {"v_range": [["7.2.0", ""]], "type": "str"},
            "reverse-cache": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "sctp-filter-profile": {"v_range": [["7.0.1", ""]], "type": "str"},
            "sgt": {"v_range": [["7.0.1", ""]], "type": "raw"},
            "sgt-check": {"v_range": [["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "tcp-timeout-pid": {"v_range": [["7.0.3", ""]], "type": "raw"},
            "transparent": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "type": {
                "v_range": [["7.0.3", ""]],
                "choices": ["explicit-web", "transparent", "explicit-ftp", "ssh-tunnel", "ssh", "wanopt", "access-proxy"],
                "type": "str"
            },
            "udp-timeout-pid": {"v_range": [["7.0.3", ""]], "type": "raw"},
            "ztna-tags-match-logic": {"v_range": [["7.0.3", ""]], "choices": ["or", "and"], "type": "str"},
            "uuid-idx": {"v_range": [["7.0.1", ""]], "type": "int"},
            "device-ownership": {"v_range": [["7.0.5", "7.0.12"], ["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ssh-policy-check": {"v_range": [["7.0.5", "7.0.12"], ["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "extended-log": {"v_range": [["7.0.11", "7.0.12"], ["7.2.5", "7.2.5"], ["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "diffserv-copy": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "dstaddr6-negate": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "internet-service6": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "internet-service6-custom": {"v_range": [["7.2.1", ""]], "type": "raw"},
            "internet-service6-custom-group": {"v_range": [["7.2.1", ""]], "type": "raw"},
            "internet-service6-group": {"v_range": [["7.2.1", ""]], "type": "raw"},
            "internet-service6-name": {"v_range": [["7.2.1", ""]], "type": "raw"},
            "internet-service6-negate": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "internet-service6-src": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "internet-service6-src-custom": {"v_range": [["7.2.1", ""]], "type": "raw"},
            "internet-service6-src-custom-group": {"v_range": [["7.2.1", ""]], "type": "raw"},
            "internet-service6-src-group": {"v_range": [["7.2.1", ""]], "type": "raw"},
            "internet-service6-src-name": {"v_range": [["7.2.1", ""]], "type": "raw"},
            "internet-service6-src-negate": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "network-service-dynamic": {"v_range": [["7.2.1", ""]], "type": "raw"},
            "network-service-src-dynamic": {"v_range": [["7.2.1", ""]], "type": "raw"},
            "reputation-direction6": {"v_range": [["7.2.1", ""]], "choices": ["source", "destination"], "type": "str"},
            "reputation-minimum6": {"v_range": [["7.2.1", ""]], "type": "int"},
            "srcaddr6-negate": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "_policy_block": {"v_range": [["7.2.2", ""]], "type": "int"},
            "isolator-profile": {"v_range": [["7.2.2", ""]], "type": "raw"},
            "policy-expiry-date-utc": {"v_range": [["7.2.2", ""]], "type": "str"},
            "ztna-device-ownership": {"v_range": [["7.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ztna-policy-redirect": {"v_range": [["7.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ip-version-type": {"v_range": [["7.2.3", ""]], "type": "str"},
            "ips-voip-filter": {"v_range": [["7.2.3", ""]], "type": "str"},
            "policy-behaviour-type": {"v_range": [["7.2.3", ""]], "type": "str"},
            "pcp-inbound": {"v_range": [["7.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "pcp-outbound": {"v_range": [["7.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "pcp-poolname": {"v_range": [["7.4.0", ""]], "type": "raw"},
            "ztna-ems-tag-secondary": {"v_range": [["7.4.0", ""]], "type": "raw"},
            "casb-profile": {"v_range": [["7.4.1", ""]], "type": "str"},
            "implicit-proxy-detection": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "virtual-patch-profile": {"v_range": [["7.4.1", ""]], "type": "str"},
            "detect-https-in-http-request": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "diameter-filter-profile": {"v_range": [["7.4.2", ""]], "type": "str"},
            "redirect-profile": {"v_range": [["7.4.2", ""]], "type": "raw"},
            "port-preserve": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "cgn-sw-eif-ctrl": {"v_range": [["7.6.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "eif-check": {"v_range": [["7.6.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "eif-learn": {"v_range": [["7.6.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "radius-ip-auth-bypass": {"v_range": [["7.6.0", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "pkg_header_policy6": {
        "stated": true,
        "options": {
            "action": {"choices": ["deny", "accept", "ipsec", "ssl-vpn"], "type": "str"},
            "anti-replay": {"choices": ["disable", "enable"], "type": "str"},
            "app-category": {"type": "raw"},
            "app-group": {"type": "raw"},
            "application": {"type": "raw"},
            "application-charts": {"type": "list", "choices": ["top10-app", "top10-p2p-user", "top10-media-user"], "elements": "str"},
            "application-list": {"type": "raw"},
            "auto-asic-offload": {"choices": ["disable", "enable"], "type": "str"},
            "av-profile": {"type": "raw"},
            "casi-profile": {"type": "raw"},
            "cifs-profile": {"type": "raw"},
            "comments": {"type": "str"},
            "custom-log-fields": {"type": "raw"},
            "deep-inspection-options": {"type": "raw"},
            "device-detection-portal": {"choices": ["disable", "enable"], "type": "str"},
            "devices": {"type": "raw"},
            "diffserv-forward": {"choices": ["disable", "enable"], "type": "str"},
            "diffserv-reverse": {"choices": ["disable", "enable"], "type": "str"},
            "diffservcode-forward": {"type": "str"},
            "diffservcode-rev": {"type": "str"},
            "dlp-sensor": {"type": "raw"},
            "dnsfilter-profile": {"type": "raw"},
            "dscp-match": {"choices": ["disable", "enable"], "type": "str"},
            "dscp-negate": {"choices": ["disable", "enable"], "type": "str"},
            "dscp-value": {"type": "str"},
            "dsri": {"choices": ["disable", "enable"], "type": "str"},
            "dstaddr": {"type": "raw"},
            "dstaddr-negate": {"choices": ["disable", "enable"], "type": "str"},
            "dstintf": {"type": "raw"},
            "dynamic-profile": {"choices": ["disable", "enable"], "type": "str"},
            "dynamic-profile-access": {
                "type": "list",
                "choices": ["imap", "smtp", "pop3", "http", "ftp", "im", "nntp", "imaps", "smtps", "pop3s", "https", "ftps"],
                "elements": "str"
            },
            "dynamic-profile-group": {"type": "raw"},
            "email-collection-portal": {"choices": ["disable", "enable"], "type": "str"},
            "emailfilter-profile": {"type": "raw"},
            "firewall-session-dirty": {"choices": ["check-all", "check-new"], "type": "str"},
            "fixedport": {"choices": ["disable", "enable"], "type": "str"},
            "fsae": {"choices": ["disable", "enable"], "type": "str"},
            "global-label": {"type": "str"},
            "groups": {"type": "raw"},
            "http-policy-redirect": {"choices": ["disable", "enable"], "type": "str"},
            "icap-profile": {"type": "raw"},
            "identity-based": {"choices": ["disable", "enable"], "type": "str"},
            "identity-based-policy6": {
                "v_range": [["6.0.0", "6.2.0"]],
                "type": "list",
                "options": {
                    "action": {"v_range": [["6.0.0", "6.2.0"]], "choices": ["deny", "accept"], "type": "str"},
                    "application-list": {"v_range": [["6.0.0", "6.2.0"]], "type": "str"},
                    "av-profile": {"v_range": [["6.0.0", "6.2.0"]], "type": "str"},
                    "deep-inspection-options": {"v_range": [["6.0.0", "6.2.0"]], "type": "str"},
                    "devices": {"v_range": [["6.0.0", "6.2.0"]], "type": "str"},
                    "dlp-sensor": {"v_range": [["6.0.0", "6.2.0"]], "type": "str"},
                    "endpoint-compliance": {"v_range": [["6.0.0", "6.2.0"]], "choices": ["disable", "enable"], "type": "str"},
                    "groups": {"v_range": [["6.0.0", "6.2.0"]], "type": "str"},
                    "icap-profile": {"v_range": [["6.0.0", "6.2.0"]], "type": "str"},
                    "id": {"v_range": [["6.0.0", "6.2.0"]], "type": "int"},
                    "ips-sensor": {"v_range": [["6.0.0", "6.2.0"]], "type": "str"},
                    "logtraffic": {"v_range": [["6.0.0", "6.2.0"]], "choices": ["disable", "enable", "all", "utm"], "type": "str"},
                    "mms-profile": {"v_range": [["6.0.0", "6.2.0"]], "type": "str"},
                    "per-ip-shaper": {"v_range": [["6.0.0", "6.2.0"]], "type": "str"},
                    "profile-group": {"v_range": [["6.0.0", "6.2.0"]], "type": "str"},
                    "profile-protocol-options": {"v_range": [["6.0.0", "6.2.0"]], "type": "str"},
                    "profile-type": {"v_range": [["6.0.0", "6.2.0"]], "choices": ["single", "group"], "type": "str"},
                    "replacemsg-group": {"v_range": [["6.0.0", "6.2.0"]], "type": "str"},
                    "schedule": {"v_range": [["6.0.0", "6.2.0"]], "type": "str"},
                    "send-deny-packet": {"v_range": [["6.0.0", "6.2.0"]], "choices": ["disable", "enable"], "type": "str"},
                    "service": {"v_range": [["6.0.0", "6.2.0"]], "type": "str"},
                    "service-negate": {"v_range": [["6.0.0", "6.2.0"]], "choices": ["disable", "enable"], "type": "str"},
                    "spamfilter-profile": {"v_range": [["6.0.0", "6.2.0"]], "type": "str"},
                    "sslvpn-portal": {"v_range": [["6.0.0", "6.2.0"]], "type": "str"},
                    "sslvpn-realm": {"v_range": [["6.0.0", "6.2.0"]], "type": "str"},
                    "traffic-shaper": {"v_range": [["6.0.0", "6.2.0"]], "type": "str"},
                    "traffic-shaper-reverse": {"v_range": [["6.0.0", "6.2.0"]], "type": "str"},
                    "utm-status": {"v_range": [["6.0.0", "6.2.0"]], "choices": ["disable", "enable"], "type": "str"},
                    "voip-profile": {"v_range": [["6.0.0", "6.2.0"]], "type": "str"},
                    "webfilter-profile": {"v_range": [["6.0.0", "6.2.0"]], "type": "str"}
                },
                "elements": "dict"
            },
            "identity-from": {"choices": ["auth", "device"], "type": "str"},
            "inbound": {"choices": ["disable", "enable"], "type": "str"},
            "inspection-mode": {"choices": ["proxy", "flow"], "type": "str"},
            "ippool": {"choices": ["disable", "enable"], "type": "str"},
            "ips-sensor": {"type": "raw"},
            "label": {"type": "str"},
            "logtraffic": {"choices": ["disable", "enable", "all", "utm"], "type": "str"},
            "logtraffic-start": {"choices": ["disable", "enable"], "type": "str"},
            "mms-profile": {"type": "raw"},
            "name": {"type": "str"},
            "nat": {"choices": ["disable", "enable"], "type": "str"},
            "natinbound": {"choices": ["disable", "enable"], "type": "str"},
            "natoutbound": {"choices": ["disable", "enable"], "type": "str"},
            "np-accelation": {"choices": ["disable", "enable"], "type": "str"},
            "np-acceleration": {"choices": ["disable", "enable"], "type": "str"},
            "outbound": {"choices": ["disable", "enable"], "type": "str"},
            "per-ip-shaper": {"type": "raw"},
            "policyid": {"required": true, "type": "int"},
            "poolname": {"type": "raw"},
            "profile-group": {"type": "raw"},
            "profile-protocol-options": {"type": "raw"},
            "profile-type": {"choices": ["single", "group"], "type": "str"},
            "replacemsg-group": {"type": "raw"},
            "replacemsg-override-group": {"type": "raw"},
            "rsso": {"choices": ["disable", "enable"], "type": "str"},
            "schedule": {"type": "raw"},
            "send-deny-packet": {"choices": ["disable", "enable"], "type": "str"},
            "service": {"type": "raw"},
            "service-negate": {"choices": ["disable", "enable"], "type": "str"},
            "session-ttl": {"type": "raw"},
            "spamfilter-profile": {"type": "raw"},
            "srcaddr": {"type": "raw"},
            "srcaddr-negate": {"choices": ["disable", "enable"], "type": "str"},
            "srcintf": {"type": "raw"},
            "ssh-filter-profile": {"type": "raw"},
            "ssh-policy-redirect": {"choices": ["disable", "enable"], "type": "str"},
            "ssl-mirror": {"choices": ["disable", "enable"], "type": "str"},
            "ssl-mirror-intf": {"type": "raw"},
            "ssl-ssh-profile": {"type": "raw"},
            "sslvpn-auth": {"choices": ["any", "local", "radius", "ldap", "tacacs+"], "type": "str"},
            "sslvpn-ccert": {"choices": ["disable", "enable"], "type": "str"},
            "sslvpn-cipher": {"choices": ["any", "high", "medium"], "type": "str"},
            "status": {"choices": ["disable", "enable"], "type": "str"},
            "tags": {"type": "raw"},
            "tcp-mss-receiver": {"type": "int"},
            "tcp-mss-sender": {"type": "int"},
            "tcp-session-without-syn": {"choices": ["all", "data-only", "disable"], "type": "str"},
            "timeout-send-rst": {"choices": ["disable", "enable"], "type": "str"},
            "tos": {"type": "str"},
            "tos-mask": {"type": "str"},
            "tos-negate": {"choices": ["disable", "enable"], "type": "str"},
            "traffic-shaper": {"type": "raw"},
            "traffic-shaper-reverse": {"type": "raw"},
            "url-category": {"type": "raw"},
            "users": {"type": "raw"},
            "utm-inspection-mode": {"choices": ["proxy", "flow"], "type": "str"},
            "utm-status": {"choices": ["disable", "enable"], "type": "str"},
            "uuid": {"type": "str"},
            "vlan-cos-fwd": {"type": "int"},
            "vlan-cos-rev": {"type": "int"},
            "vlan-filter": {"type": "str"},
            "voip-profile": {"type": "raw"},
            "vpntunnel": {"type": "raw"},
            "webfilter-profile": {"type": "raw"},
            "waf-profile": {"v_range": [["6.2.1", ""]], "type": "raw"},
            "webcache": {"v_range": [["6.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "webcache-https": {"v_range": [["6.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "webproxy-forward-server": {"v_range": [["6.2.1", ""]], "type": "raw"},
            "webproxy-profile": {"v_range": [["6.2.1", ""]], "type": "raw"},
            "fsso-groups": {"v_range": [["6.2.3", ""]], "type": "raw"},
            "decrypted-traffic-mirror": {"v_range": [["6.4.0", ""]], "type": "raw"},
            "file-filter-profile": {"v_range": [["6.4.1", ""]], "type": "raw"},
            "cgn-log-server-grp": {"v_range": [["6.2.7", "6.2.12"], ["6.4.3", ""]], "type": "str"},
            "policy-offload": {"v_range": [["6.2.7", "6.2.12"], ["6.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "_policy_block": {"v_range": [["7.2.2", ""]], "type": "int"}
        }
    },
    "pkg_header_shapingpolicy": {
        "stated": true,
        "options": {
            "app-category": {"type": "raw"},
            "app-group": {"type": "raw"},
            "application": {"type": "raw"},
            "class-id": {"type": "raw"},
            "comment": {"type": "str"},
            "diffserv-forward": {"choices": ["disable", "enable"], "type": "str"},
            "diffserv-reverse": {"choices": ["disable", "enable"], "type": "str"},
            "diffservcode-forward": {"type": "str"},
            "diffservcode-rev": {"type": "str"},
            "dstaddr": {"type": "raw"},
            "dstaddr6": {"type": "raw"},
            "dstintf": {"type": "raw"},
            "groups": {"type": "raw"},
            "id": {"required": true, "type": "int"},
            "internet-service": {"choices": ["disable", "enable"], "type": "str"},
            "internet-service-custom": {"type": "raw"},
            "internet-service-custom-group": {"type": "raw"},
            "internet-service-group": {"type": "raw"},
            "internet-service-id": {"type": "raw"},
            "internet-service-src": {"choices": ["disable", "enable"], "type": "str"},
            "internet-service-src-custom": {"type": "raw"},
            "internet-service-src-custom-group": {"type": "raw"},
            "internet-service-src-group": {"type": "raw"},
            "internet-service-src-id": {"type": "raw"},
            "ip-version": {"choices": ["4", "6"], "type": "str"},
            "per-ip-shaper": {"type": "str"},
            "schedule": {"type": "str"},
            "service": {"type": "raw"},
            "srcaddr": {"type": "raw"},
            "srcaddr6": {"type": "raw"},
            "srcintf": {"type": "raw"},
            "status": {"choices": ["disable", "enable"], "type": "str"},
            "tos": {"type": "str"},
            "tos-mask": {"type": "str"},
            "tos-negate": {"choices": ["disable", "enable"], "type": "str"},
            "traffic-shaper": {"type": "str"},
            "traffic-shaper-reverse": {"type": "str"},
            "url-category": {"type": "raw"},
            "users": {"type": "raw"},
            "uuid": {"v_range": [["6.2.1", ""]], "type": "str"},
            "internet-service-name": {"v_range": [["6.4.0", ""]], "type": "raw"},
            "internet-service-src-name": {"v_range": [["6.4.0", ""]], "type": "raw"},
            "class-id-reverse": {"v_range": [["7.0.3", ""]], "type": "int"},
            "service-type": {"v_range": [["7.0.3", ""]], "choices": ["service", "internet-service"], "type": "str"},
            "uuid-idx": {"v_range": [["7.2.1", ""]], "type": "int"},
            "cos": {"v_range": [["7.4.0", ""]], "type": "str"},
            "cos-mask": {"v_range": [["7.4.0", ""]], "type": "str"},
            "traffic-type": {"v_range": [["7.4.0", ""]], "choices": ["forwarding", "local-in", "local-out"], "type": "str"}
        }
    },
    "pkg_user_nacpolicy": {
        "stated": true,
        "options": {
            "category": {"v_range": [["7.2.1", ""]], "choices": ["device", "firewall-user", "ems-tag", "vulnerability", "fortivoice-tag"], "type": "str"},
            "description": {"v_range": [["7.2.1", ""]], "type": "str"},
            "ems-tag": {"v_range": [["7.2.1", ""]], "type": "str"},
            "family": {"v_range": [["7.2.1", ""]], "type": "str"},
            "host": {"v_range": [["7.2.1", ""]], "type": "str"},
            "hw-vendor": {"v_range": [["7.2.1", ""]], "type": "str"},
            "hw-version": {"v_range": [["7.2.1", ""]], "type": "str"},
            "mac": {"v_range": [["7.2.1", ""]], "type": "str"},
            "name": {"v_range": [["7.2.1", ""]], "required": true, "type": "str"},
            "os": {"v_range": [["7.2.1", ""]], "type": "str"},
            "src": {"v_range": [["7.2.1", ""]], "type": "str"},
            "ssid-policy": {"v_range": [["7.2.1", ""]], "type": "str"},
            "status": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "sw-version": {"v_range": [["7.2.1", ""]], "type": "str"},
            "type": {"v_range": [["7.2.1", ""]], "type": "str"},
            "user": {"v_range": [["7.2.1", ""]], "type": "str"},
            "user-group": {"v_range": [["7.2.1", ""]], "type": "str"},
            "severity": {"v_range": [["7.4.0", ""]], "type": "raw"},
            "firewall-address": {"v_range": [["7.4.3", ""]], "type": "raw"},
            "fortivoice-tag": {"v_range": [["7.4.3", ""]], "type": "raw"},
            "match-period": {"v_range": [["7.4.3", ""]], "type": "int"},
            "match-type": {"v_range": [["7.4.3", ""]], "choices": ["dynamic", "override"], "type": "str"},
            "switch-fortilink": {"v_range": [["7.4.3", ""]], "type": "raw"},
            "switch-group": {"v_range": [["7.4.3", ""]], "type": "raw"},
            "switch-mac-policy": {"v_range": [["7.4.3", ""]], "type": "raw"},
            "switch-scope": {"v_range": [["7.4.3", ""]], "type": "raw"},
            "switch-port-policy": {"v_range": [["7.4.3", ""]], "type": "raw"},
            "switch-auto-auth": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable", "global"], "type": "str"}
        }
    },
    "pm_config_meta_reference": {
        "stated": true,
        "options": {
            "package list": {
                "v_range": [["7.2.4", "7.2.5"], ["7.4.1", ""]],
                "type": "list",
                "options": {
                    "name": {"v_range": [["7.2.4", "7.2.5"], ["7.4.1", ""]], "type": "str"},
                    "vdom": {"v_range": [["7.2.4", "7.2.5"], ["7.4.1", ""]], "type": "str"}
                },
                "elements": "dict"
            }
        }
    },
    "pm_config_metafields_firewall_address": {
        "stated": false,
        "options": {
            "importance": {"v_range": [["6.2.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "choices": ["optional", "required"], "type": "str"},
            "length": {"v_range": [["6.2.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "int"},
            "name": {"v_range": [["6.2.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "str"}
        }
    },
    "pm_config_metafields_firewall_addrgrp": {
        "stated": false,
        "options": {
            "importance": {"v_range": [["6.2.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "choices": ["optional", "required"], "type": "str"},
            "length": {"v_range": [["6.2.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "int"},
            "name": {"v_range": [["6.2.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "str"}
        }
    },
    "pm_config_metafields_firewall_centralsnatmap": {
        "stated": false,
        "options": {
            "importance": {"v_range": [["6.2.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "choices": ["optional", "required"], "type": "str"},
            "length": {"v_range": [["6.2.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "int"},
            "name": {"v_range": [["6.2.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "str"}
        }
    },
    "pm_config_metafields_firewall_policy": {
        "stated": false,
        "options": {
            "importance": {"v_range": [["6.2.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "choices": ["optional", "required"], "type": "str"},
            "length": {"v_range": [["6.2.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "int"},
            "name": {"v_range": [["6.2.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "str"}
        }
    },
    "pm_config_metafields_firewall_service_custom": {
        "stated": false,
        "options": {
            "importance": {"v_range": [["6.2.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "choices": ["optional", "required"], "type": "str"},
            "length": {"v_range": [["6.2.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "int"},
            "name": {"v_range": [["6.2.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "str"}
        }
    },
    "pm_config_metafields_firewall_service_group": {
        "stated": false,
        "options": {
            "importance": {"v_range": [["6.2.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "choices": ["optional", "required"], "type": "str"},
            "length": {"v_range": [["6.2.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "int"},
            "name": {"v_range": [["6.2.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "str"}
        }
    },
    "pm_config_pblock_firewall_consolidated_policy": {
        "stated": true,
        "options": {
            "_policy_block": {"v_range": [["7.0.3", ""]], "type": "int"},
            "action": {"v_range": [["7.0.3", ""]], "choices": ["deny", "accept", "ipsec"], "type": "str"},
            "app-category": {"v_range": [["7.0.3", "7.4.2"]], "type": "raw"},
            "app-group": {"v_range": [["7.0.3", "7.4.2"]], "type": "raw"},
            "application": {"v_range": [["7.0.3", "7.4.2"]], "type": "raw"},
            "application-list": {"v_range": [["7.0.3", ""]], "type": "str"},
            "auto-asic-offload": {"v_range": [["7.0.3", "7.2.0"], ["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "av-profile": {"v_range": [["7.0.3", ""]], "type": "str"},
            "captive-portal-exempt": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "cifs-profile": {"v_range": [["7.0.3", ""]], "type": "str"},
            "comments": {"v_range": [["7.0.3", ""]], "type": "str"},
            "diffserv-forward": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "diffserv-reverse": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "diffservcode-forward": {"v_range": [["7.0.3", ""]], "type": "str"},
            "diffservcode-rev": {"v_range": [["7.0.3", ""]], "type": "str"},
            "dlp-sensor": {"v_range": [["7.0.3", ""]], "type": "str"},
            "dnsfilter-profile": {"v_range": [["7.0.3", ""]], "type": "str"},
            "dstaddr-negate": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "dstaddr4": {"v_range": [["7.0.3", ""]], "type": "raw"},
            "dstaddr6": {"v_range": [["7.0.3", ""]], "type": "raw"},
            "dstintf": {"v_range": [["7.0.3", ""]], "type": "raw"},
            "emailfilter-profile": {"v_range": [["7.0.3", ""]], "type": "str"},
            "fixedport": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "fsso-groups": {"v_range": [["7.0.3", ""]], "type": "raw"},
            "global-label": {"v_range": [["7.0.3", ""]], "type": "str"},
            "groups": {"v_range": [["7.0.3", ""]], "type": "raw"},
            "http-policy-redirect": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "icap-profile": {"v_range": [["7.0.3", ""]], "type": "str"},
            "inbound": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "inspection-mode": {"v_range": [["7.0.3", ""]], "choices": ["proxy", "flow"], "type": "str"},
            "internet-service": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "internet-service-custom": {"v_range": [["7.0.3", ""]], "type": "raw"},
            "internet-service-custom-group": {"v_range": [["7.0.3", ""]], "type": "raw"},
            "internet-service-group": {"v_range": [["7.0.3", ""]], "type": "raw"},
            "internet-service-id": {"v_range": [["7.0.3", ""]], "type": "raw"},
            "internet-service-negate": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "internet-service-src": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "internet-service-src-custom": {"v_range": [["7.0.3", ""]], "type": "raw"},
            "internet-service-src-custom-group": {"v_range": [["7.0.3", ""]], "type": "raw"},
            "internet-service-src-group": {"v_range": [["7.0.3", ""]], "type": "raw"},
            "internet-service-src-id": {"v_range": [["7.0.3", ""]], "type": "raw"},
            "internet-service-src-negate": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ippool": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ips-sensor": {"v_range": [["7.0.3", ""]], "type": "str"},
            "logtraffic": {"v_range": [["7.0.3", ""]], "choices": ["disable", "all", "utm"], "type": "str"},
            "logtraffic-start": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "mms-profile": {"v_range": [["7.0.3", "7.2.0"], ["7.4.3", ""]], "type": "str"},
            "name": {"v_range": [["7.0.3", ""]], "type": "str"},
            "nat": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "outbound": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "per-ip-shaper": {"v_range": [["7.0.3", ""]], "type": "str"},
            "policyid": {"v_range": [["7.0.3", ""]], "required": true, "type": "int"},
            "poolname4": {"v_range": [["7.0.3", ""]], "type": "raw"},
            "poolname6": {"v_range": [["7.0.3", ""]], "type": "raw"},
            "profile-group": {"v_range": [["7.0.3", ""]], "type": "str"},
            "profile-protocol-options": {"v_range": [["7.0.3", "7.2.0"], ["7.4.3", ""]], "type": "str"},
            "profile-type": {"v_range": [["7.0.3", ""]], "choices": ["single", "group"], "type": "str"},
            "schedule": {"v_range": [["7.0.3", ""]], "type": "str"},
            "service": {"v_range": [["7.0.3", ""]], "type": "raw"},
            "service-negate": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "session-ttl": {"v_range": [["7.0.3", ""]], "type": "int"},
            "srcaddr-negate": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "srcaddr4": {"v_range": [["7.0.3", ""]], "type": "raw"},
            "srcaddr6": {"v_range": [["7.0.3", ""]], "type": "raw"},
            "srcintf": {"v_range": [["7.0.3", ""]], "type": "raw"},
            "ssh-filter-profile": {"v_range": [["7.0.3", "7.2.0"], ["7.4.3", ""]], "type": "str"},
            "ssh-policy-redirect": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ssl-ssh-profile": {"v_range": [["7.0.3", "7.2.0"], ["7.4.3", ""]], "type": "str"},
            "status": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "tcp-mss-receiver": {"v_range": [["7.0.3", ""]], "type": "int"},
            "tcp-mss-sender": {"v_range": [["7.0.3", ""]], "type": "int"},
            "traffic-shaper": {"v_range": [["7.0.3", ""]], "type": "str"},
            "traffic-shaper-reverse": {"v_range": [["7.0.3", ""]], "type": "str"},
            "url-category": {"v_range": [["7.0.3", "7.4.2"]], "type": "raw"},
            "users": {"v_range": [["7.0.3", ""]], "type": "raw"},
            "utm-status": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "uuid": {"v_range": [["7.0.3", ""]], "type": "str"},
            "voip-profile": {"v_range": [["7.0.3", ""]], "type": "str"},
            "vpntunnel": {"v_range": [["7.0.3", ""]], "type": "str"},
            "waf-profile": {"v_range": [["7.0.3", ""]], "type": "str"},
            "wanopt": {"v_range": [["7.0.3", "7.2.0"], ["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "wanopt-detection": {"v_range": [["7.0.3", "7.2.0"], ["7.4.3", ""]], "choices": ["active", "passive", "off"], "type": "str"},
            "wanopt-passive-opt": {
                "v_range": [["7.0.3", "7.2.0"], ["7.4.3", ""]],
                "choices": ["default", "transparent", "non-transparent"],
                "type": "str"
            },
            "wanopt-peer": {"v_range": [["7.0.3", "7.2.0"], ["7.4.3", ""]], "type": "str"},
            "wanopt-profile": {"v_range": [["7.0.3", "7.2.0"], ["7.4.3", ""]], "type": "str"},
            "webcache": {"v_range": [["7.0.3", "7.2.0"], ["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "webcache-https": {"v_range": [["7.0.3", "7.2.0"], ["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "webfilter-profile": {"v_range": [["7.0.3", ""]], "type": "str"},
            "webproxy-forward-server": {"v_range": [["7.0.3", ""]], "type": "str"},
            "webproxy-profile": {"v_range": [["7.0.3", ""]], "type": "str"}
        }
    },
    "pm_config_pblock_firewall_consolidated_policy_sectionvalue": {
        "stated": true,
        "options": {
            "attr": {"v_range": [["7.0.3", ""]], "choices": ["label", "global-label"], "type": "str"},
            "name": {"v_range": [["7.0.3", ""]], "type": "str"}
        }
    },
    "pm_config_pblock_firewall_policy": {
        "stated": true,
        "options": {
            "_policy_block": {"v_range": [["7.0.3", ""]], "type": "int"},
            "action": {"v_range": [["7.0.3", ""]], "choices": ["deny", "accept", "ipsec", "ssl-vpn", "redirect", "isolate"], "type": "str"},
            "anti-replay": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "application-list": {"v_range": [["7.0.3", ""]], "type": "str"},
            "auth-cert": {"v_range": [["7.0.3", ""]], "type": "str"},
            "auth-path": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "auth-redirect-addr": {"v_range": [["7.0.3", ""]], "type": "str"},
            "auto-asic-offload": {"v_range": [["7.0.3", "7.2.0"], ["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "av-profile": {"v_range": [["7.0.3", ""]], "type": "str"},
            "block-notification": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "captive-portal-exempt": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "capture-packet": {"v_range": [["7.0.3", "7.2.0"], ["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "cifs-profile": {"v_range": [["7.0.3", ""]], "type": "str"},
            "comments": {"v_range": [["7.0.3", ""]], "type": "str"},
            "custom-log-fields": {"v_range": [["7.0.3", ""]], "type": "raw"},
            "decrypted-traffic-mirror": {"v_range": [["7.0.3", ""]], "type": "str"},
            "delay-tcp-npu-session": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "diffserv-forward": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "diffserv-reverse": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "diffservcode-forward": {"v_range": [["7.0.3", ""]], "type": "str"},
            "diffservcode-rev": {"v_range": [["7.0.3", ""]], "type": "str"},
            "disclaimer": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable", "user", "domain", "policy"], "type": "str"},
            "dlp-profile": {"v_range": [["7.2.0", ""]], "type": "str"},
            "dnsfilter-profile": {"v_range": [["7.0.3", ""]], "type": "str"},
            "dsri": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "dstaddr": {"v_range": [["7.0.3", ""]], "type": "raw"},
            "dstaddr-negate": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "dstaddr6": {"v_range": [["7.0.3", ""]], "type": "raw"},
            "dstintf": {"v_range": [["7.0.3", ""]], "type": "raw"},
            "dynamic-shaping": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "email-collect": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "emailfilter-profile": {"v_range": [["7.0.3", ""]], "type": "str"},
            "fec": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "file-filter-profile": {"v_range": [["7.0.3", "7.2.4"], ["7.4.0", ""]], "type": "str"},
            "firewall-session-dirty": {"v_range": [["7.0.3", ""]], "choices": ["check-all", "check-new"], "type": "str"},
            "fixedport": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "fsso-agent-for-ntlm": {"v_range": [["7.0.3", ""]], "type": "str"},
            "fsso-groups": {"v_range": [["7.0.3", ""]], "type": "raw"},
            "geoip-anycast": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "geoip-match": {"v_range": [["7.0.3", ""]], "choices": ["physical-location", "registered-location"], "type": "str"},
            "global-label": {"v_range": [["7.0.3", ""]], "type": "str"},
            "groups": {"v_range": [["7.0.3", ""]], "type": "raw"},
            "gtp-profile": {"v_range": [["7.0.3", "7.2.0"], ["7.4.3", ""]], "type": "str"},
            "http-policy-redirect": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "icap-profile": {"v_range": [["7.0.3", ""]], "type": "str"},
            "identity-based-route": {"v_range": [["7.0.3", ""]], "type": "str"},
            "inbound": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "inspection-mode": {"v_range": [["7.0.3", ""]], "choices": ["proxy", "flow"], "type": "str"},
            "internet-service": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "internet-service-custom": {"v_range": [["7.0.3", ""]], "type": "raw"},
            "internet-service-custom-group": {"v_range": [["7.0.3", ""]], "type": "raw"},
            "internet-service-group": {"v_range": [["7.0.3", ""]], "type": "raw"},
            "internet-service-name": {"v_range": [["7.0.3", ""]], "type": "raw"},
            "internet-service-negate": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "internet-service-src": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "internet-service-src-custom": {"v_range": [["7.0.3", ""]], "type": "raw"},
            "internet-service-src-custom-group": {"v_range": [["7.0.3", ""]], "type": "raw"},
            "internet-service-src-group": {"v_range": [["7.0.3", ""]], "type": "raw"},
            "internet-service-src-name": {"v_range": [["7.0.3", ""]], "type": "raw"},
            "internet-service-src-negate": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ippool": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ips-sensor": {"v_range": [["7.0.3", ""]], "type": "str"},
            "label": {"v_range": [["7.0.3", ""]], "type": "str"},
            "logtraffic": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable", "all", "utm"], "type": "str"},
            "logtraffic-start": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "match-vip": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "match-vip-only": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "name": {"v_range": [["7.0.3", ""]], "type": "str"},
            "nat": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "nat46": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "nat64": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "natinbound": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "natip": {"v_range": [["7.0.3", ""]], "type": "str"},
            "natoutbound": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "np-acceleration": {"v_range": [["7.0.3", "7.2.0"], ["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ntlm": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ntlm-enabled-browsers": {"v_range": [["7.0.3", ""]], "type": "raw"},
            "ntlm-guest": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "outbound": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "passive-wan-health-measurement": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "per-ip-shaper": {"v_range": [["7.0.3", ""]], "type": "str"},
            "permit-any-host": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "permit-stun-host": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "pfcp-profile": {"v_range": [["7.0.3", "7.2.0"], ["7.4.3", ""]], "type": "str"},
            "policy-expiry": {"v_range": [["7.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "policy-expiry-date": {"v_range": [["7.2.0", ""]], "type": "str"},
            "policyid": {"v_range": [["7.0.3", ""]], "required": true, "type": "int"},
            "poolname": {"v_range": [["7.0.3", ""]], "type": "raw"},
            "poolname6": {"v_range": [["7.0.3", ""]], "type": "raw"},
            "profile-group": {"v_range": [["7.0.3", ""]], "type": "str"},
            "profile-protocol-options": {"v_range": [["7.0.3", ""]], "type": "str"},
            "profile-type": {"v_range": [["7.0.3", ""]], "choices": ["single", "group"], "type": "str"},
            "radius-mac-auth-bypass": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "redirect-url": {"v_range": [["7.0.3", ""]], "type": "str"},
            "replacemsg-override-group": {"v_range": [["7.0.3", ""]], "type": "str"},
            "reputation-direction": {"v_range": [["7.0.3", ""]], "choices": ["source", "destination"], "type": "str"},
            "reputation-minimum": {"v_range": [["7.0.3", ""]], "type": "int"},
            "rtp-addr": {"v_range": [["7.0.3", ""]], "type": "raw"},
            "rtp-nat": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "schedule": {"v_range": [["7.0.3", ""]], "type": "str"},
            "schedule-timeout": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "sctp-filter-profile": {"v_range": [["7.0.3", "7.2.4"], ["7.4.0", ""]], "type": "str"},
            "send-deny-packet": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "service": {"v_range": [["7.0.3", ""]], "type": "raw"},
            "service-negate": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "session-ttl": {"v_range": [["7.0.3", ""]], "type": "raw"},
            "sgt": {"v_range": [["7.0.3", ""]], "type": "raw"},
            "sgt-check": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "src-vendor-mac": {"v_range": [["7.0.3", ""]], "type": "raw"},
            "srcaddr": {"v_range": [["7.0.3", ""]], "type": "raw"},
            "srcaddr-negate": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "srcaddr6": {"v_range": [["7.0.3", ""]], "type": "raw"},
            "srcintf": {"v_range": [["7.0.3", ""]], "type": "raw"},
            "ssh-filter-profile": {"v_range": [["7.0.3", "7.2.4"], ["7.4.0", ""]], "type": "str"},
            "ssh-policy-redirect": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ssl-ssh-profile": {"v_range": [["7.0.3", ""]], "type": "str"},
            "status": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "tcp-mss-receiver": {"v_range": [["7.0.3", ""]], "type": "int"},
            "tcp-mss-sender": {"v_range": [["7.0.3", ""]], "type": "int"},
            "tcp-session-without-syn": {"v_range": [["7.0.3", ""]], "choices": ["all", "data-only", "disable"], "type": "str"},
            "timeout-send-rst": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "tos": {"v_range": [["7.0.3", ""]], "type": "str"},
            "tos-mask": {"v_range": [["7.0.3", ""]], "type": "str"},
            "tos-negate": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "traffic-shaper": {"v_range": [["7.0.3", ""]], "type": "str"},
            "traffic-shaper-reverse": {"v_range": [["7.0.3", ""]], "type": "str"},
            "users": {"v_range": [["7.0.3", ""]], "type": "raw"},
            "utm-status": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "uuid": {"v_range": [["7.0.3", ""]], "type": "str"},
            "videofilter-profile": {"v_range": [["7.0.3", "7.2.4"], ["7.4.0", ""]], "type": "str"},
            "vlan-cos-fwd": {"v_range": [["7.0.3", ""]], "type": "int"},
            "vlan-cos-rev": {"v_range": [["7.0.3", ""]], "type": "int"},
            "vlan-filter": {"v_range": [["7.0.3", ""]], "type": "str"},
            "voip-profile": {"v_range": [["7.0.3", ""]], "type": "str"},
            "vpntunnel": {"v_range": [["7.0.3", ""]], "type": "str"},
            "waf-profile": {"v_range": [["7.0.3", ""]], "type": "str"},
            "wanopt": {"v_range": [["7.0.3", "7.2.0"], ["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "wanopt-detection": {"v_range": [["7.0.3", "7.2.0"], ["7.4.3", ""]], "choices": ["active", "passive", "off"], "type": "str"},
            "wanopt-passive-opt": {
                "v_range": [["7.0.3", "7.2.0"], ["7.4.3", ""]],
                "choices": ["default", "transparent", "non-transparent"],
                "type": "str"
            },
            "wanopt-peer": {"v_range": [["7.0.3", "7.2.0"], ["7.4.3", ""]], "type": "str"},
            "wanopt-profile": {"v_range": [["7.0.3", "7.2.0"], ["7.4.3", ""]], "type": "str"},
            "wccp": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "webcache": {"v_range": [["7.0.3", "7.2.0"], ["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "webcache-https": {"v_range": [["7.0.3", "7.2.0"], ["7.4.3", ""]], "choices": ["disable", "ssl-server", "any", "enable"], "type": "str"},
            "webfilter-profile": {"v_range": [["7.0.3", ""]], "type": "str"},
            "webproxy-forward-server": {"v_range": [["7.0.3", ""]], "type": "str"},
            "webproxy-profile": {"v_range": [["7.0.3", ""]], "type": "str"},
            "ztna-ems-tag": {"v_range": [["7.0.3", ""]], "type": "raw"},
            "ztna-geo-tag": {"v_range": [["7.0.3", ""]], "type": "raw"},
            "ztna-status": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "policy-offload": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "cgn-session-quota": {"v_range": [["7.0.3", ""]], "type": "int"},
            "tcp-timeout-pid": {"v_range": [["7.0.3", "7.2.0"], ["7.4.3", ""]], "type": "str"},
            "udp-timeout-pid": {"v_range": [["7.0.3", "7.2.0"], ["7.4.3", ""]], "type": "str"},
            "dlp-sensor": {"v_range": [["7.0.3", ""]], "type": "str"},
            "cgn-eif": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "cgn-log-server-grp": {"v_range": [["7.0.3", ""]], "type": "str"},
            "cgn-resource-quota": {"v_range": [["7.0.3", ""]], "type": "int"},
            "cgn-eim": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "mms-profile": {"v_range": [["7.0.3", "7.2.0"], ["7.4.3", ""]], "type": "str"},
            "app-category": {"v_range": [["7.0.3", ""]], "type": "raw"},
            "internet-service-src-id": {"v_range": [["7.0.3", ""]], "type": "raw"},
            "rsso": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "internet-service-id": {"v_range": [["7.0.3", ""]], "type": "raw"},
            "best-route": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "fsso": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "url-category": {"v_range": [["7.0.3", ""]], "type": "raw"},
            "app-group": {"v_range": [["7.0.3", ""]], "type": "raw"},
            "ssl-mirror-intf": {"v_range": [["7.0.3", ""]], "type": "raw"},
            "wsso": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ssl-mirror": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "application": {"v_range": [["7.0.3", ""]], "type": "raw"},
            "dscp-negate": {"v_range": [["7.0.3", "7.2.1"]], "choices": ["disable", "enable"], "type": "str"},
            "learning-mode": {"v_range": [["7.0.3", "7.2.1"]], "choices": ["disable", "enable"], "type": "str"},
            "devices": {"v_range": [["7.0.3", "7.2.1"]], "type": "raw"},
            "dscp-value": {"v_range": [["7.0.3", "7.2.1"]], "type": "str"},
            "spamfilter-profile": {"v_range": [["7.0.3", "7.2.1"]], "type": "str"},
            "scan-botnet-connections": {"v_range": [["7.0.3", "7.2.1"]], "choices": ["disable", "block", "monitor"], "type": "str"},
            "dscp-match": {"v_range": [["7.0.3", "7.2.1"]], "choices": ["disable", "enable"], "type": "str"},
            "diffserv-copy": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "dstaddr6-negate": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "internet-service6": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "internet-service6-custom": {"v_range": [["7.2.1", ""]], "type": "raw"},
            "internet-service6-custom-group": {"v_range": [["7.2.1", ""]], "type": "raw"},
            "internet-service6-group": {"v_range": [["7.2.1", ""]], "type": "raw"},
            "internet-service6-name": {"v_range": [["7.2.1", ""]], "type": "raw"},
            "internet-service6-negate": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "internet-service6-src": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "internet-service6-src-custom": {"v_range": [["7.2.1", ""]], "type": "raw"},
            "internet-service6-src-custom-group": {"v_range": [["7.2.1", ""]], "type": "raw"},
            "internet-service6-src-group": {"v_range": [["7.2.1", ""]], "type": "raw"},
            "internet-service6-src-name": {"v_range": [["7.2.1", ""]], "type": "raw"},
            "internet-service6-src-negate": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "network-service-dynamic": {"v_range": [["7.2.1", ""]], "type": "raw"},
            "network-service-src-dynamic": {"v_range": [["7.2.1", ""]], "type": "raw"},
            "reputation-direction6": {"v_range": [["7.2.1", ""]], "choices": ["source", "destination"], "type": "str"},
            "reputation-minimum6": {"v_range": [["7.2.1", ""]], "type": "int"},
            "srcaddr6-negate": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ip-version-type": {"v_range": [["7.4.0", ""]], "type": "str"},
            "ips-voip-filter": {"v_range": [["7.4.0", ""]], "type": "str"},
            "pcp-inbound": {"v_range": [["7.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "pcp-outbound": {"v_range": [["7.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "pcp-poolname": {"v_range": [["7.4.0", ""]], "type": "raw"},
            "policy-behaviour-type": {"v_range": [["7.4.0", ""]], "type": "str"},
            "policy-expiry-date-utc": {"v_range": [["7.4.0", ""]], "type": "str"},
            "ztna-device-ownership": {"v_range": [["7.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ztna-ems-tag-secondary": {"v_range": [["7.4.0", ""]], "type": "raw"},
            "ztna-policy-redirect": {"v_range": [["7.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ztna-tags-match-logic": {"v_range": [["7.4.0", ""]], "choices": ["or", "and"], "type": "str"},
            "casb-profile": {"v_range": [["7.4.1", ""]], "type": "str"},
            "virtual-patch-profile": {"v_range": [["7.4.1", ""]], "type": "str"},
            "diameter-filter-profile": {"v_range": [["7.4.2", ""]], "type": "str"},
            "port-preserve": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "cgn-sw-eif-ctrl": {"v_range": [["7.6.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "eif-check": {"v_range": [["7.6.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "eif-learn": {"v_range": [["7.6.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "log-http-transaction": {"v_range": [["7.6.0", ""]], "choices": ["disable", "enable", "all", "utm"], "type": "str"},
            "radius-ip-auth-bypass": {"v_range": [["7.6.0", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "pm_config_pblock_firewall_policy6": {
        "stated": true,
        "options": {
            "_policy_block": {"v_range": [["7.0.3", ""]], "type": "int"},
            "action": {"v_range": [["7.0.3", ""]], "choices": ["deny", "accept", "ipsec", "ssl-vpn"], "type": "str"},
            "anti-replay": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "app-category": {"v_range": [["7.0.3", ""]], "type": "raw"},
            "app-group": {"v_range": [["7.0.3", ""]], "type": "raw"},
            "application": {"v_range": [["7.0.3", ""]], "type": "raw"},
            "application-list": {"v_range": [["7.0.3", ""]], "type": "str"},
            "auto-asic-offload": {"v_range": [["7.0.3", "7.2.0"], ["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "av-profile": {"v_range": [["7.0.3", ""]], "type": "str"},
            "cgn-log-server-grp": {"v_range": [["7.0.3", ""]], "type": "str"},
            "cifs-profile": {"v_range": [["7.0.3", ""]], "type": "str"},
            "comments": {"v_range": [["7.0.3", ""]], "type": "str"},
            "custom-log-fields": {"v_range": [["7.0.3", ""]], "type": "raw"},
            "diffserv-forward": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "diffserv-reverse": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "diffservcode-forward": {"v_range": [["7.0.3", ""]], "type": "str"},
            "diffservcode-rev": {"v_range": [["7.0.3", ""]], "type": "str"},
            "dlp-sensor": {"v_range": [["7.0.3", ""]], "type": "str"},
            "dnsfilter-profile": {"v_range": [["7.0.3", ""]], "type": "str"},
            "dsri": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "dstaddr": {"v_range": [["7.0.3", ""]], "type": "raw"},
            "dstaddr-negate": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "dstintf": {"v_range": [["7.0.3", ""]], "type": "raw"},
            "emailfilter-profile": {"v_range": [["7.0.3", ""]], "type": "str"},
            "firewall-session-dirty": {"v_range": [["7.0.3", ""]], "choices": ["check-all", "check-new"], "type": "str"},
            "fixedport": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "fsso-groups": {"v_range": [["7.0.3", ""]], "type": "raw"},
            "global-label": {"v_range": [["7.0.3", ""]], "type": "str"},
            "groups": {"v_range": [["7.0.3", ""]], "type": "raw"},
            "http-policy-redirect": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "icap-profile": {"v_range": [["7.0.3", ""]], "type": "str"},
            "inbound": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "inspection-mode": {"v_range": [["7.0.3", ""]], "choices": ["proxy", "flow"], "type": "str"},
            "ippool": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ips-sensor": {"v_range": [["7.0.3", ""]], "type": "str"},
            "label": {"v_range": [["7.0.3", ""]], "type": "str"},
            "logtraffic": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable", "all", "utm"], "type": "str"},
            "logtraffic-start": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "mms-profile": {"v_range": [["7.0.3", "7.2.0"], ["7.4.3", ""]], "type": "str"},
            "name": {"v_range": [["7.0.3", ""]], "type": "str"},
            "nat": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "natinbound": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "natoutbound": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "np-acceleration": {"v_range": [["7.0.3", "7.2.0"], ["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "outbound": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "per-ip-shaper": {"v_range": [["7.0.3", ""]], "type": "str"},
            "policy-offload": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "policyid": {"v_range": [["7.0.3", ""]], "required": true, "type": "int"},
            "poolname": {"v_range": [["7.0.3", ""]], "type": "raw"},
            "profile-group": {"v_range": [["7.0.3", ""]], "type": "str"},
            "profile-protocol-options": {"v_range": [["7.0.3", "7.2.1"], ["7.4.3", ""]], "type": "str"},
            "profile-type": {"v_range": [["7.0.3", ""]], "choices": ["single", "group"], "type": "str"},
            "replacemsg-override-group": {"v_range": [["7.0.3", ""]], "type": "str"},
            "rsso": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "schedule": {"v_range": [["7.0.3", ""]], "type": "str"},
            "send-deny-packet": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "service": {"v_range": [["7.0.3", ""]], "type": "raw"},
            "service-negate": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "session-ttl": {"v_range": [["7.0.3", ""]], "type": "str"},
            "srcaddr": {"v_range": [["7.0.3", ""]], "type": "raw"},
            "srcaddr-negate": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "srcintf": {"v_range": [["7.0.3", ""]], "type": "raw"},
            "ssh-filter-profile": {"v_range": [["7.0.3", "7.2.1"], ["7.4.3", ""]], "type": "str"},
            "ssh-policy-redirect": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ssl-mirror": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ssl-mirror-intf": {"v_range": [["7.0.3", ""]], "type": "raw"},
            "ssl-ssh-profile": {"v_range": [["7.0.3", "7.2.1"], ["7.4.3", ""]], "type": "str"},
            "status": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "tcp-mss-receiver": {"v_range": [["7.0.3", ""]], "type": "int"},
            "tcp-mss-sender": {"v_range": [["7.0.3", ""]], "type": "int"},
            "tcp-session-without-syn": {"v_range": [["7.0.3", ""]], "choices": ["all", "data-only", "disable"], "type": "str"},
            "timeout-send-rst": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "tos": {"v_range": [["7.0.3", ""]], "type": "str"},
            "tos-mask": {"v_range": [["7.0.3", ""]], "type": "str"},
            "tos-negate": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "traffic-shaper": {"v_range": [["7.0.3", ""]], "type": "str"},
            "traffic-shaper-reverse": {"v_range": [["7.0.3", ""]], "type": "str"},
            "url-category": {"v_range": [["7.0.3", ""]], "type": "raw"},
            "users": {"v_range": [["7.0.3", ""]], "type": "raw"},
            "utm-status": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "uuid": {"v_range": [["7.0.3", ""]], "type": "str"},
            "vlan-cos-fwd": {"v_range": [["7.0.3", ""]], "type": "int"},
            "vlan-cos-rev": {"v_range": [["7.0.3", ""]], "type": "int"},
            "vlan-filter": {"v_range": [["7.0.3", ""]], "type": "str"},
            "voip-profile": {"v_range": [["7.0.3", ""]], "type": "str"},
            "vpntunnel": {"v_range": [["7.0.3", ""]], "type": "str"},
            "waf-profile": {"v_range": [["7.0.3", ""]], "type": "str"},
            "webcache": {"v_range": [["7.0.3", "7.2.0"], ["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "webcache-https": {"v_range": [["7.0.3", "7.2.0"], ["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "webfilter-profile": {"v_range": [["7.0.3", ""]], "type": "str"},
            "webproxy-forward-server": {"v_range": [["7.0.3", ""]], "type": "str"},
            "webproxy-profile": {"v_range": [["7.0.3", ""]], "type": "str"},
            "dscp-negate": {"v_range": [["7.0.3", "7.2.1"]], "choices": ["disable", "enable"], "type": "str"},
            "devices": {"v_range": [["7.0.3", "7.2.1"]], "type": "raw"},
            "dscp-value": {"v_range": [["7.0.3", "7.2.1"]], "type": "str"},
            "spamfilter-profile": {"v_range": [["7.0.3", "7.2.1"]], "type": "str"},
            "dscp-match": {"v_range": [["7.0.3", "7.2.1"]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "pm_config_pblock_firewall_policy6_sectionvalue": {
        "stated": true,
        "options": {
            "attr": {"v_range": [["7.0.3", ""]], "choices": ["label", "global-label"], "type": "str"},
            "name": {"v_range": [["7.0.3", ""]], "type": "str"}
        }
    },
    "pm_config_pblock_firewall_policy_sectionvalue": {
        "stated": true,
        "options": {
            "attr": {"v_range": [["7.0.3", ""]], "choices": ["label", "global-label"], "type": "str"},
            "name": {"v_range": [["7.0.3", ""]], "type": "str"}
        }
    },
    "pm_config_pblock_firewall_proxypolicy": {
        "stated": true,
        "options": {
            "_policy_block": {"v_range": [["7.6.0", ""]], "type": "int"},
            "access-proxy": {"v_range": [["7.6.0", ""]], "type": "list", "elements": "str"},
            "access-proxy6": {"v_range": [["7.6.0", ""]], "type": "list", "elements": "str"},
            "action": {"v_range": [["7.6.0", ""]], "choices": ["accept", "deny", "redirect"], "type": "str"},
            "application-list": {"v_range": [["7.6.0", ""]], "type": "list", "elements": "str"},
            "av-profile": {"v_range": [["7.6.0", ""]], "type": "list", "elements": "str"},
            "block-notification": {"v_range": [["7.6.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "casb-profile": {"v_range": [["7.6.0", ""]], "type": "list", "elements": "str"},
            "comments": {"v_range": [["7.6.0", ""]], "type": "str"},
            "decrypted-traffic-mirror": {"v_range": [["7.6.0", ""]], "type": "list", "elements": "str"},
            "detect-https-in-http-request": {"v_range": [["7.6.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "device-ownership": {"v_range": [["7.6.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "disclaimer": {"v_range": [["7.6.0", ""]], "choices": ["disable", "domain", "policy", "user"], "type": "str"},
            "dlp-profile": {"v_range": [["7.6.0", ""]], "type": "list", "elements": "str"},
            "dnsfilter-profile": {"v_range": [["7.6.0", ""]], "type": "list", "elements": "str"},
            "dstaddr": {"v_range": [["7.6.0", ""]], "type": "list", "elements": "str"},
            "dstaddr-negate": {"v_range": [["7.6.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "dstaddr6": {"v_range": [["7.6.0", ""]], "type": "list", "elements": "str"},
            "dstintf": {"v_range": [["7.6.0", ""]], "type": "list", "elements": "str"},
            "emailfilter-profile": {"v_range": [["7.6.0", ""]], "type": "list", "elements": "str"},
            "file-filter-profile": {"v_range": [["7.6.0", ""]], "type": "list", "elements": "str"},
            "global-label": {"v_range": [["7.6.0", ""]], "type": "str"},
            "groups": {"v_range": [["7.6.0", ""]], "type": "list", "elements": "str"},
            "http-tunnel-auth": {"v_range": [["7.6.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "icap-profile": {"v_range": [["7.6.0", ""]], "type": "list", "elements": "str"},
            "internet-service": {"v_range": [["7.6.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "internet-service-custom": {"v_range": [["7.6.0", ""]], "type": "list", "elements": "str"},
            "internet-service-custom-group": {"v_range": [["7.6.0", ""]], "type": "list", "elements": "str"},
            "internet-service-group": {"v_range": [["7.6.0", ""]], "type": "list", "elements": "str"},
            "internet-service-name": {"v_range": [["7.6.0", ""]], "type": "list", "elements": "str"},
            "internet-service-negate": {"v_range": [["7.6.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "internet-service6": {"v_range": [["7.6.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "internet-service6-custom": {"v_range": [["7.6.0", ""]], "type": "list", "elements": "str"},
            "internet-service6-custom-group": {"v_range": [["7.6.0", ""]], "type": "list", "elements": "str"},
            "internet-service6-group": {"v_range": [["7.6.0", ""]], "type": "list", "elements": "str"},
            "internet-service6-name": {"v_range": [["7.6.0", ""]], "type": "list", "elements": "str"},
            "internet-service6-negate": {"v_range": [["7.6.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ips-sensor": {"v_range": [["7.6.0", ""]], "type": "list", "elements": "str"},
            "ips-voip-filter": {"v_range": [["7.6.0", ""]], "type": "list", "elements": "str"},
            "label": {"v_range": [["7.6.0", ""]], "type": "str"},
            "log-http-transaction": {"v_range": [["7.6.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "logtraffic": {"v_range": [["7.6.0", ""]], "choices": ["disable", "all", "utm"], "type": "str"},
            "logtraffic-start": {"v_range": [["7.6.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "name": {"v_range": [["7.6.0", ""]], "type": "str"},
            "policyid": {"v_range": [["7.6.0", ""]], "required": true, "type": "int"},
            "poolname": {"v_range": [["7.6.0", ""]], "type": "list", "elements": "str"},
            "profile-group": {"v_range": [["7.6.0", ""]], "type": "list", "elements": "str"},
            "profile-protocol-options": {"v_range": [["7.6.0", ""]], "type": "list", "elements": "str"},
            "profile-type": {"v_range": [["7.6.0", ""]], "choices": ["single", "group"], "type": "str"},
            "proxy": {
                "v_range": [["7.6.0", ""]],
                "choices": ["explicit-web", "transparent-web", "ftp", "wanopt", "ssh", "ssh-tunnel", "access-proxy", "ztna-proxy"],
                "type": "str"
            },
            "redirect-url": {"v_range": [["7.6.0", ""]], "type": "str"},
            "replacemsg-override-group": {"v_range": [["7.6.0", ""]], "type": "list", "elements": "str"},
            "schedule": {"v_range": [["7.6.0", ""]], "type": "list", "elements": "str"},
            "sctp-filter-profile": {"v_range": [["7.6.0", ""]], "type": "list", "elements": "str"},
            "service": {"v_range": [["7.6.0", ""]], "type": "list", "elements": "str"},
            "service-negate": {"v_range": [["7.6.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "session-ttl": {"v_range": [["7.6.0", ""]], "type": "str"},
            "srcaddr": {"v_range": [["7.6.0", ""]], "type": "list", "elements": "str"},
            "srcaddr-negate": {"v_range": [["7.6.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "srcaddr6": {"v_range": [["7.6.0", ""]], "type": "list", "elements": "str"},
            "srcintf": {"v_range": [["7.6.0", ""]], "type": "list", "elements": "str"},
            "ssh-filter-profile": {"v_range": [["7.6.0", ""]], "type": "list", "elements": "str"},
            "ssh-policy-redirect": {"v_range": [["7.6.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ssl-ssh-profile": {"v_range": [["7.6.0", ""]], "type": "list", "elements": "str"},
            "status": {"v_range": [["7.6.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "transparent": {"v_range": [["7.6.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "users": {"v_range": [["7.6.0", ""]], "type": "list", "elements": "str"},
            "utm-status": {"v_range": [["7.6.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "uuid": {"v_range": [["7.6.0", ""]], "type": "str"},
            "videofilter-profile": {"v_range": [["7.6.0", ""]], "type": "list", "elements": "str"},
            "waf-profile": {"v_range": [["7.6.0", ""]], "type": "list", "elements": "str"},
            "webcache": {"v_range": [["7.6.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "webcache-https": {"v_range": [["7.6.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "webfilter-profile": {"v_range": [["7.6.0", ""]], "type": "list", "elements": "str"},
            "webproxy-forward-server": {"v_range": [["7.6.0", ""]], "type": "list", "elements": "str"},
            "webproxy-profile": {"v_range": [["7.6.0", ""]], "type": "list", "elements": "str"},
            "ztna-ems-tag": {"v_range": [["7.6.0", ""]], "type": "list", "elements": "str"},
            "ztna-proxy": {"v_range": [["7.6.0", ""]], "type": "list", "elements": "str"},
            "ztna-tags-match-logic": {"v_range": [["7.6.0", ""]], "choices": ["or", "and"], "type": "str"},
            "diameter-filter-profile": {"v_range": [["7.6.0", ""]], "type": "list", "elements": "str"},
            "virtual-patch-profile": {"v_range": [["7.6.0", ""]], "type": "list", "elements": "str"},
            "voip-profile": {"v_range": [["7.6.0", ""]], "type": "list", "elements": "str"},
            "dlp-sensor": {"v_range": [["7.6.0", ""]], "type": "list", "elements": "str"},
            "cifs-profile": {"v_range": [["7.6.0", ""]], "type": "list", "elements": "str"},
            "internet-service-id": {"v_range": [["7.6.0", ""]], "type": "list", "elements": "str"},
            "mms-profile": {"v_range": [["7.6.0", ""]], "type": "list", "elements": "str"}
        }
    },
    "pm_config_pblock_firewall_proxypolicy_sectionvalue": {
        "stated": true,
        "options": {
            "attr": {"v_range": [["7.6.0", ""]], "choices": ["label", "global-label"], "type": "str"},
            "name": {"v_range": [["7.6.0", ""]], "type": "str"}
        }
    },
    "pm_config_pblock_firewall_securitypolicy": {
        "stated": true,
        "options": {
            "_policy_block": {"v_range": [["7.0.3", ""]], "type": "int"},
            "action": {"v_range": [["7.0.3", ""]], "choices": ["deny", "accept"], "type": "str"},
            "app-category": {"v_range": [["7.0.3", ""]], "type": "raw"},
            "app-group": {"v_range": [["7.0.3", ""]], "type": "raw"},
            "application": {"v_range": [["7.0.3", ""]], "type": "raw"},
            "application-list": {"v_range": [["7.0.3", ""]], "type": "str"},
            "av-profile": {"v_range": [["7.0.3", ""]], "type": "str"},
            "cifs-profile": {"v_range": [["7.0.3", ""]], "type": "str"},
            "comments": {"v_range": [["7.0.3", ""]], "type": "str"},
            "dlp-profile": {"v_range": [["7.2.0", "7.2.1"], ["7.2.4", "7.2.4"], ["7.4.2", ""]], "type": "str"},
            "dnsfilter-profile": {"v_range": [["7.0.3", ""]], "type": "str"},
            "dstaddr": {"v_range": [["7.0.3", "7.2.2"], ["7.2.4", "7.2.4"], ["7.4.2", ""]], "type": "raw"},
            "dstaddr-negate": {"v_range": [["7.0.3", "7.2.2"], ["7.2.4", "7.2.4"], ["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "dstaddr6": {"v_range": [["7.0.3", ""]], "type": "raw"},
            "dstintf": {"v_range": [["7.0.3", ""]], "type": "raw"},
            "emailfilter-profile": {"v_range": [["7.0.3", ""]], "type": "str"},
            "enforce-default-app-port": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "file-filter-profile": {"v_range": [["7.0.3", "7.2.2"], ["7.2.4", "7.2.4"], ["7.4.2", ""]], "type": "str"},
            "fsso-groups": {"v_range": [["7.0.3", ""]], "type": "raw"},
            "global-label": {"v_range": [["7.0.3", ""]], "type": "str"},
            "groups": {"v_range": [["7.0.3", ""]], "type": "raw"},
            "icap-profile": {"v_range": [["7.0.3", ""]], "type": "str"},
            "internet-service": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "internet-service-custom": {"v_range": [["7.0.3", ""]], "type": "raw"},
            "internet-service-custom-group": {"v_range": [["7.0.3", ""]], "type": "raw"},
            "internet-service-group": {"v_range": [["7.0.3", ""]], "type": "raw"},
            "internet-service-name": {"v_range": [["7.0.3", "7.2.2"], ["7.2.4", "7.2.4"], ["7.4.2", ""]], "type": "raw"},
            "internet-service-negate": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "internet-service-src": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "internet-service-src-custom": {"v_range": [["7.0.3", ""]], "type": "raw"},
            "internet-service-src-custom-group": {"v_range": [["7.0.3", ""]], "type": "raw"},
            "internet-service-src-group": {"v_range": [["7.0.3", ""]], "type": "raw"},
            "internet-service-src-name": {"v_range": [["7.0.3", "7.2.2"], ["7.2.4", "7.2.4"], ["7.4.2", ""]], "type": "raw"},
            "internet-service-src-negate": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ips-sensor": {"v_range": [["7.0.3", ""]], "type": "str"},
            "learning-mode": {"v_range": [["7.0.3", "7.2.2"], ["7.2.4", "7.2.4"], ["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "logtraffic": {"v_range": [["7.0.3", ""]], "choices": ["disable", "all", "utm"], "type": "str"},
            "name": {"v_range": [["7.0.3", ""]], "type": "str"},
            "nat46": {"v_range": [["7.0.3", "7.2.1"], ["7.2.4", "7.2.4"], ["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "nat64": {"v_range": [["7.0.3", "7.2.1"], ["7.2.4", "7.2.4"], ["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "policyid": {"v_range": [["7.0.3", ""]], "required": true, "type": "int"},
            "profile-group": {"v_range": [["7.0.3", ""]], "type": "str"},
            "profile-protocol-options": {"v_range": [["7.0.3", "7.2.2"], ["7.2.4", "7.2.4"], ["7.4.2", ""]], "type": "str"},
            "profile-type": {"v_range": [["7.0.3", ""]], "choices": ["single", "group"], "type": "str"},
            "schedule": {"v_range": [["7.0.3", ""]], "type": "str"},
            "sctp-filter-profile": {"v_range": [["7.0.3", "7.2.2"], ["7.2.4", "7.2.4"], ["7.4.2", ""]], "type": "str"},
            "send-deny-packet": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "service": {"v_range": [["7.0.3", ""]], "type": "raw"},
            "service-negate": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "srcaddr": {"v_range": [["7.0.3", "7.2.2"], ["7.2.4", "7.2.4"], ["7.4.2", ""]], "type": "raw"},
            "srcaddr-negate": {"v_range": [["7.0.3", "7.2.2"], ["7.2.4", "7.2.4"], ["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "srcaddr6": {"v_range": [["7.0.3", ""]], "type": "raw"},
            "srcintf": {"v_range": [["7.0.3", ""]], "type": "raw"},
            "ssh-filter-profile": {"v_range": [["7.0.3", ""]], "type": "str"},
            "ssl-ssh-profile": {"v_range": [["7.0.3", "7.2.2"], ["7.2.4", "7.2.4"], ["7.4.2", ""]], "type": "str"},
            "status": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "url-category": {"v_range": [["7.0.3", ""]], "type": "raw"},
            "users": {"v_range": [["7.0.3", ""]], "type": "raw"},
            "utm-status": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "uuid": {"v_range": [["7.0.3", ""]], "type": "str"},
            "videofilter-profile": {"v_range": [["7.0.3", "7.2.2"], ["7.2.4", "7.2.4"], ["7.4.2", ""]], "type": "str"},
            "voip-profile": {"v_range": [["7.0.3", ""]], "type": "str"},
            "webfilter-profile": {"v_range": [["7.0.3", ""]], "type": "str"},
            "dlp-sensor": {"v_range": [["7.0.3", ""]], "type": "str"},
            "mms-profile": {"v_range": [["7.0.3", "7.2.0"], ["7.4.3", ""]], "type": "str"},
            "internet-service-id": {"v_range": [["7.0.3", ""]], "type": "raw"},
            "logtraffic-start": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "srcaddr4": {"v_range": [["7.0.3", ""]], "type": "raw"},
            "dstaddr4": {"v_range": [["7.0.3", ""]], "type": "raw"},
            "internet-service-src-id": {"v_range": [["7.0.3", ""]], "type": "raw"},
            "internet-service6": {"v_range": [["7.2.1", "7.2.1"], ["7.2.4", "7.2.4"], ["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "internet-service6-custom": {"v_range": [["7.2.1", "7.2.1"], ["7.2.4", "7.2.4"], ["7.4.2", ""]], "type": "raw"},
            "internet-service6-custom-group": {"v_range": [["7.2.1", "7.2.1"], ["7.2.4", "7.2.4"], ["7.4.2", ""]], "type": "raw"},
            "internet-service6-group": {"v_range": [["7.2.1", "7.2.1"], ["7.2.4", "7.2.4"], ["7.4.2", ""]], "type": "raw"},
            "internet-service6-name": {"v_range": [["7.2.1", "7.2.1"], ["7.2.4", "7.2.4"], ["7.4.2", ""]], "type": "raw"},
            "internet-service6-negate": {
                "v_range": [["7.2.1", "7.2.1"], ["7.2.4", "7.2.4"], ["7.4.2", ""]],
                "choices": ["disable", "enable"],
                "type": "str"
            },
            "internet-service6-src": {"v_range": [["7.2.1", "7.2.1"], ["7.2.4", "7.2.4"], ["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "internet-service6-src-custom": {"v_range": [["7.2.1", "7.2.1"], ["7.2.4", "7.2.4"], ["7.4.2", ""]], "type": "raw"},
            "internet-service6-src-custom-group": {"v_range": [["7.2.1", "7.2.1"], ["7.2.4", "7.2.4"], ["7.4.2", ""]], "type": "raw"},
            "internet-service6-src-group": {"v_range": [["7.2.1", "7.2.1"], ["7.2.4", "7.2.4"], ["7.4.2", ""]], "type": "raw"},
            "internet-service6-src-name": {"v_range": [["7.2.1", "7.2.1"], ["7.2.4", "7.2.4"], ["7.4.2", ""]], "type": "raw"},
            "internet-service6-src-negate": {
                "v_range": [["7.2.1", "7.2.1"], ["7.2.4", "7.2.4"], ["7.4.2", ""]],
                "choices": ["disable", "enable"],
                "type": "str"
            },
            "casb-profile": {"v_range": [["7.4.2", ""]], "type": "str"},
            "diameter-filter-profile": {"v_range": [["7.4.2", ""]], "type": "str"},
            "dstaddr6-negate": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ips-voip-filter": {"v_range": [["7.4.2", ""]], "type": "str"},
            "srcaddr6-negate": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "virtual-patch-profile": {"v_range": [["7.4.2", ""]], "type": "str"}
        }
    },
    "pm_config_pblock_firewall_securitypolicy_sectionvalue": {
        "stated": true,
        "options": {
            "attr": {"v_range": [["7.0.3", "7.2.2"], ["7.2.4", "7.2.4"], ["7.4.2", ""]], "choices": ["label", "global-label"], "type": "str"},
            "name": {"v_range": [["7.0.3", "7.2.2"], ["7.2.4", "7.2.4"], ["7.4.2", ""]], "type": "str"}
        }
    },
    "pm_config_reset_database": {
        "stated": true,
        "options": {
            "mr": {"v_range": [["6.2.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "int"},
            "version": {"v_range": [["6.2.0", "6.2.5"], ["6.2.7", "6.4.1"], ["6.4.3", ""]], "type": "int"}
        }
    },
    "pm_config_upgrade": {"stated": true, "options": {}},
    "pm_config_workspace_commit": {"stated": true, "options": {}},
    "pm_config_workspace_lock": {"stated": true, "options": {}},
    "pm_config_workspace_unlock": {"stated": true, "options": {}},
    "pm_devprof_adom": {
        "stated": false,
        "options": {
            "description": {"type": "str"},
            "enabled options": {"type": "list", "choices": ["none", "dns", "ntp", "email", "admin", "snmp", "repmsg", "ftgd", "log"], "elements": "str"},
            "name": {"type": "str"},
            "oid": {"type": "int"},
            "scope member": {"type": "list", "options": {"name": {"type": "str"}, "vdom": {"type": "str"}}, "elements": "dict"},
            "type": {"choices": ["devprof"], "type": "str"}
        }
    },
    "pm_devprof_pkg": {
        "stated": true,
        "options": {
            "description": {"type": "str"},
            "enabled options": {"type": "list", "choices": ["none", "dns", "ntp", "email", "admin", "snmp", "repmsg", "ftgd", "log"], "elements": "str"},
            "name": {"required": true, "type": "str"},
            "oid": {"type": "int"},
            "scope member": {"type": "list", "options": {"name": {"type": "str"}, "vdom": {"type": "str"}}, "elements": "dict"},
            "type": {"choices": ["devprof"], "type": "str"}
        }
    },
    "pm_devprof_scopemember": {
        "stated": true,
        "options": {"name": {"v_range": [["7.2.1", ""]], "type": "str"}, "vdom": {"v_range": [["7.2.1", ""]], "type": "str"}}
    },
    "pm_pblock_adom": {
        "stated": false,
        "options": {
            "name": {"v_range": [["7.0.3", ""]], "type": "str"},
            "oid": {"v_range": [["7.0.3", ""]], "type": "int"},
            "package settings": {
                "v_range": [["7.0.3", ""]],
                "type": "dict",
                "options": {
                    "central-nat": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "consolidated-firewall-mode": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "fwpolicy-implicit-log": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "fwpolicy6-implicit-log": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "inspection-mode": {"v_range": [["7.0.3", ""]], "choices": ["proxy", "flow"], "type": "str"},
                    "ngfw-mode": {"v_range": [["7.0.3", ""]], "choices": ["profile-based", "policy-based"], "type": "str"},
                    "policy-offload-level": {"v_range": [["7.0.3", ""]], "choices": ["disable", "default", "dos-offload", "full-offload"], "type": "str"},
                    "ssl-ssh-profile": {"v_range": [["7.0.3", ""]], "type": "str"}
                }
            },
            "type": {"v_range": [["7.0.3", ""]], "choices": ["pblock"], "type": "str"}
        }
    },
    "pm_pblock_obj": {
        "stated": true,
        "options": {
            "name": {"v_range": [["7.0.3", ""]], "required": true, "type": "str"},
            "oid": {"v_range": [["7.0.3", ""]], "type": "int"},
            "package settings": {
                "v_range": [["7.0.3", ""]],
                "type": "dict",
                "options": {
                    "central-nat": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "consolidated-firewall-mode": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "fwpolicy-implicit-log": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "fwpolicy6-implicit-log": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "inspection-mode": {"v_range": [["7.0.3", ""]], "choices": ["proxy", "flow"], "type": "str"},
                    "ngfw-mode": {"v_range": [["7.0.3", ""]], "choices": ["profile-based", "policy-based"], "type": "str"},
                    "policy-offload-level": {"v_range": [["7.0.3", ""]], "choices": ["disable", "default", "dos-offload", "full-offload"], "type": "str"},
                    "ssl-ssh-profile": {"v_range": [["7.0.3", ""]], "type": "str"}
                }
            },
            "type": {"v_range": [["7.0.3", ""]], "choices": ["pblock"], "type": "str"}
        }
    },
    "pm_pkg": {
        "stated": true,
        "options": {
            "name": {"required": true, "type": "str"},
            "obj ver": {"type": "int"},
            "oid": {"type": "int"},
            "package setting": {
                "v_range": [["6.0.0", "6.4.6"], ["7.0.0", "7.0.0"]],
                "type": "dict",
                "options": {
                    "central-nat": {"v_range": [["6.0.0", "6.4.6"], ["7.0.0", "7.0.0"]], "choices": ["disable", "enable"], "type": "str"},
                    "consolidated-firewall-mode": {"v_range": [["6.0.0", "6.4.6"], ["7.0.0", "7.0.0"]], "choices": ["disable", "enable"], "type": "str"},
                    "fwpolicy-implicit-log": {"v_range": [["6.0.0", "6.4.6"], ["7.0.0", "7.0.0"]], "choices": ["disable", "enable"], "type": "str"},
                    "fwpolicy6-implicit-log": {"v_range": [["6.0.0", "6.4.6"], ["7.0.0", "7.0.0"]], "choices": ["disable", "enable"], "type": "str"},
                    "inspection-mode": {"v_range": [["6.0.0", "6.4.6"], ["7.0.0", "7.0.0"]], "choices": ["proxy", "flow"], "type": "str"},
                    "ngfw-mode": {"v_range": [["6.0.0", "6.4.6"], ["7.0.0", "7.0.0"]], "choices": ["profile-based", "policy-based"], "type": "str"},
                    "ssl-ssh-profile": {"v_range": [["6.0.0", "6.4.6"], ["7.0.0", "7.0.0"]], "type": "str"}
                }
            },
            "scope member": {"type": "list", "options": {"name": {"type": "str"}, "vdom": {"type": "str"}}, "elements": "dict"},
            "type": {"choices": ["pkg", "folder"], "type": "str"},
            "package settings": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "type": "dict",
                "options": {
                    "central-nat": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "consolidated-firewall-mode": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "fwpolicy-implicit-log": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "fwpolicy6-implicit-log": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "inspection-mode": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["proxy", "flow"], "type": "str"},
                    "ngfw-mode": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["profile-based", "policy-based"], "type": "str"},
                    "policy-offload-level": {
                        "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                        "choices": ["disable", "default", "dos-offload", "full-offload"],
                        "type": "str"
                    },
                    "ssl-ssh-profile": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "str"}
                }
            }
        }
    },
    "pm_pkg_adom": {
        "stated": false,
        "options": {
            "name": {"type": "str"},
            "obj ver": {"type": "int"},
            "oid": {"type": "int"},
            "package setting": {
                "v_range": [["6.0.0", "6.4.6"], ["7.0.0", "7.0.0"]],
                "type": "dict",
                "options": {
                    "central-nat": {"v_range": [["6.0.0", "6.4.6"], ["7.0.0", "7.0.0"]], "choices": ["disable", "enable"], "type": "str"},
                    "consolidated-firewall-mode": {"v_range": [["6.0.0", "6.4.6"], ["7.0.0", "7.0.0"]], "choices": ["disable", "enable"], "type": "str"},
                    "fwpolicy-implicit-log": {"v_range": [["6.0.0", "6.4.6"], ["7.0.0", "7.0.0"]], "choices": ["disable", "enable"], "type": "str"},
                    "fwpolicy6-implicit-log": {"v_range": [["6.0.0", "6.4.6"], ["7.0.0", "7.0.0"]], "choices": ["disable", "enable"], "type": "str"},
                    "inspection-mode": {"v_range": [["6.0.0", "6.4.6"], ["7.0.0", "7.0.0"]], "choices": ["proxy", "flow"], "type": "str"},
                    "ngfw-mode": {"v_range": [["6.0.0", "6.4.6"], ["7.0.0", "7.0.0"]], "choices": ["profile-based", "policy-based"], "type": "str"},
                    "ssl-ssh-profile": {"v_range": [["6.0.0", "6.4.6"], ["7.0.0", "7.0.0"]], "type": "str"}
                }
            },
            "scope member": {"type": "list", "options": {"name": {"type": "str"}, "vdom": {"type": "str"}}, "elements": "dict"},
            "type": {"choices": ["pkg", "folder"], "type": "str"},
            "package settings": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "type": "dict",
                "options": {
                    "central-nat": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "consolidated-firewall-mode": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "fwpolicy-implicit-log": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "fwpolicy6-implicit-log": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "inspection-mode": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["proxy", "flow"], "type": "str"},
                    "ngfw-mode": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["profile-based", "policy-based"], "type": "str"},
                    "policy-offload-level": {
                        "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                        "choices": ["disable", "default", "dos-offload", "full-offload"],
                        "type": "str"
                    },
                    "ssl-ssh-profile": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "str"}
                }
            }
        }
    },
    "pm_pkg_global": {
        "stated": false,
        "options": {
            "name": {"type": "str"},
            "obj ver": {"type": "int"},
            "oid": {"type": "int"},
            "package setting": {
                "v_range": [["6.0.0", "6.4.6"], ["7.0.0", "7.0.0"]],
                "type": "dict",
                "options": {
                    "central-nat": {"v_range": [["6.0.0", "6.4.6"], ["7.0.0", "7.0.0"]], "choices": ["disable", "enable"], "type": "str"},
                    "consolidated-firewall-mode": {"v_range": [["6.0.0", "6.4.6"], ["7.0.0", "7.0.0"]], "choices": ["disable", "enable"], "type": "str"},
                    "fwpolicy-implicit-log": {"v_range": [["6.0.0", "6.4.6"], ["7.0.0", "7.0.0"]], "choices": ["disable", "enable"], "type": "str"},
                    "fwpolicy6-implicit-log": {"v_range": [["6.0.0", "6.4.6"], ["7.0.0", "7.0.0"]], "choices": ["disable", "enable"], "type": "str"},
                    "inspection-mode": {"v_range": [["6.0.0", "6.4.6"], ["7.0.0", "7.0.0"]], "choices": ["proxy", "flow"], "type": "str"},
                    "ngfw-mode": {"v_range": [["6.0.0", "6.4.6"], ["7.0.0", "7.0.0"]], "choices": ["profile-based", "policy-based"], "type": "str"},
                    "ssl-ssh-profile": {"v_range": [["6.0.0", "6.4.6"], ["7.0.0", "7.0.0"]], "type": "str"}
                }
            },
            "scope member": {"type": "list", "options": {"name": {"type": "str"}, "vdom": {"type": "str"}}, "elements": "dict"},
            "type": {"choices": ["pkg", "folder"], "type": "str"},
            "package settings": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "type": "dict",
                "options": {
                    "central-nat": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "consolidated-firewall-mode": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "fwpolicy-implicit-log": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "fwpolicy6-implicit-log": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "inspection-mode": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["proxy", "flow"], "type": "str"},
                    "ngfw-mode": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["profile-based", "policy-based"], "type": "str"},
                    "policy-offload-level": {
                        "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                        "choices": ["disable", "default", "dos-offload", "full-offload"],
                        "type": "str"
                    },
                    "ssl-ssh-profile": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "str"}
                }
            }
        }
    },
    "pm_pkg_scopemember": {
        "stated": true,
        "options": {"name": {"v_range": [["7.2.1", ""]], "type": "str"}, "vdom": {"v_range": [["7.2.1", ""]], "type": "str"}}
    },
    "pm_wanprof_adom": {
        "stated": false,
        "options": {
            "description": {"type": "str"},
            "name": {"type": "str"},
            "oid": {"type": "int"},
            "scope member": {"type": "list", "options": {"name": {"type": "str"}, "vdom": {"type": "str"}}, "elements": "dict"},
            "type": {"choices": ["wanprof"], "type": "str"}
        }
    },
    "pm_wanprof_pkg": {
        "stated": true,
        "options": {
            "description": {"type": "str"},
            "name": {"required": true, "type": "str"},
            "oid": {"type": "int"},
            "scope member": {"type": "list", "options": {"name": {"type": "str"}, "vdom": {"type": "str"}}, "elements": "dict"},
            "type": {"choices": ["wanprof"], "type": "str"}
        }
    },
    "pm_wanprof_scopemember": {
        "stated": true,
        "options": {"name": {"v_range": [["7.2.1", ""]], "type": "str"}, "vdom": {"v_range": [["7.2.1", ""]], "type": "str"}}
    },
    "qosprofile": {
        "stated": true,
        "options": {
            "bandwidth-admission-control": {"choices": ["disable", "enable"], "type": "str"},
            "bandwidth-capacity": {"type": "int"},
            "burst": {"choices": ["disable", "enable"], "type": "str"},
            "call-admission-control": {"choices": ["disable", "enable"], "type": "str"},
            "call-capacity": {"type": "int"},
            "comment": {"type": "str"},
            "downlink": {"type": "int"},
            "downlink-sta": {"type": "int"},
            "dscp-wmm-be": {"type": "raw"},
            "dscp-wmm-bk": {"type": "raw"},
            "dscp-wmm-mapping": {"choices": ["disable", "enable"], "type": "str"},
            "dscp-wmm-vi": {"type": "raw"},
            "dscp-wmm-vo": {"type": "raw"},
            "name": {"required": true, "type": "str"},
            "uplink": {"type": "int"},
            "uplink-sta": {"type": "int"},
            "wmm": {"choices": ["disable", "enable"], "type": "str"},
            "wmm-uapsd": {"choices": ["disable", "enable"], "type": "str"},
            "wmm-be-dscp": {"v_range": [["6.2.0", ""]], "type": "int"},
            "wmm-bk-dscp": {"v_range": [["6.2.0", ""]], "type": "int"},
            "wmm-dscp-marking": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "wmm-vi-dscp": {"v_range": [["6.2.0", ""]], "type": "int"},
            "wmm-vo-dscp": {"v_range": [["6.2.0", ""]], "type": "int"}
        }
    },
    "region": {
        "stated": true,
        "options": {
            "comments": {"v_range": [["6.2.8", "6.2.12"], ["6.4.6", ""]], "type": "str"},
            "grayscale": {"v_range": [["6.2.8", "6.2.12"], ["6.4.6", ""]], "choices": ["disable", "enable"], "type": "str"},
            "image-type": {"v_range": [["6.2.8", "6.2.12"], ["6.4.6", ""]], "choices": ["gif", "jpeg", "png"], "type": "str"},
            "name": {"v_range": [["6.2.8", "6.2.12"], ["6.4.6", ""]], "required": true, "type": "str"},
            "opacity": {"v_range": [["6.2.8", "6.2.12"], ["6.4.6", ""]], "type": "int"}
        }
    },
    "router_accesslist": {
        "stated": true,
        "options": {
            "comments": {"v_range": [["7.0.2", ""]], "type": "str"},
            "name": {"v_range": [["7.0.2", ""]], "required": true, "type": "str"},
            "rule": {
                "v_range": [["7.0.2", ""]],
                "type": "list",
                "options": {
                    "action": {"v_range": [["7.0.2", ""]], "choices": ["permit", "deny"], "type": "str"},
                    "exact-match": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "flags": {"v_range": [["7.0.2", ""]], "type": "int"},
                    "id": {"v_range": [["7.0.2", ""]], "type": "int"},
                    "prefix": {"v_range": [["7.0.2", ""]], "type": "str"},
                    "wildcard": {"v_range": [["7.0.2", ""]], "type": "str"}
                },
                "elements": "dict"
            }
        }
    },
    "router_accesslist6": {
        "stated": true,
        "options": {
            "comments": {"v_range": [["7.0.2", ""]], "type": "str"},
            "name": {"v_range": [["7.0.2", ""]], "required": true, "type": "str"},
            "rule": {
                "v_range": [["7.0.2", ""]],
                "type": "list",
                "options": {
                    "action": {"v_range": [["7.0.2", ""]], "choices": ["permit", "deny"], "type": "str"},
                    "exact-match": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "flags": {"v_range": [["7.0.2", ""]], "type": "int"},
                    "id": {"v_range": [["7.0.2", ""]], "type": "int"},
                    "prefix6": {"v_range": [["7.0.2", ""]], "type": "str"}
                },
                "elements": "dict"
            }
        }
    },
    "router_accesslist6_rule": {
        "stated": true,
        "options": {
            "action": {"v_range": [["7.0.2", ""]], "choices": ["permit", "deny"], "type": "str"},
            "exact-match": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "flags": {"v_range": [["7.0.2", ""]], "type": "int"},
            "id": {"v_range": [["7.0.2", ""]], "required": true, "type": "int"},
            "prefix6": {"v_range": [["7.0.2", ""]], "type": "str"}
        }
    },
    "router_accesslist_rule": {
        "stated": true,
        "options": {
            "action": {"v_range": [["7.0.2", ""]], "choices": ["permit", "deny"], "type": "str"},
            "exact-match": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "flags": {"v_range": [["7.0.2", ""]], "type": "int"},
            "id": {"v_range": [["7.0.2", ""]], "required": true, "type": "int"},
            "prefix": {"v_range": [["7.0.2", ""]], "type": "str"},
            "wildcard": {"v_range": [["7.0.2", ""]], "type": "str"}
        }
    },
    "router_aspathlist": {
        "stated": true,
        "options": {
            "name": {"v_range": [["7.0.2", ""]], "required": true, "type": "str"},
            "rule": {
                "v_range": [["7.0.2", ""]],
                "type": "list",
                "options": {
                    "action": {"v_range": [["7.0.2", ""]], "choices": ["deny", "permit"], "type": "str"},
                    "id": {"v_range": [["7.0.2", ""]], "type": "int"},
                    "regexp": {"v_range": [["7.0.2", ""]], "type": "str"}
                },
                "elements": "dict"
            }
        }
    },
    "router_aspathlist_rule": {
        "stated": true,
        "options": {
            "action": {"v_range": [["7.0.2", ""]], "choices": ["deny", "permit"], "type": "str"},
            "id": {"v_range": [["7.0.2", ""]], "required": true, "type": "int"},
            "regexp": {"v_range": [["7.0.2", ""]], "type": "str"}
        }
    },
    "router_communitylist": {
        "stated": true,
        "options": {
            "name": {"v_range": [["7.0.2", ""]], "required": true, "type": "str"},
            "rule": {
                "v_range": [["7.0.2", ""]],
                "type": "list",
                "options": {
                    "action": {"v_range": [["7.0.2", ""]], "choices": ["deny", "permit"], "type": "str"},
                    "id": {"v_range": [["7.0.2", ""]], "type": "int"},
                    "match": {"v_range": [["7.0.2", ""]], "type": "str"},
                    "regexp": {"v_range": [["7.0.2", ""]], "type": "str"}
                },
                "elements": "dict"
            },
            "type": {"v_range": [["7.0.2", ""]], "choices": ["standard", "expanded"], "type": "str"}
        }
    },
    "router_communitylist_rule": {
        "stated": true,
        "options": {
            "action": {"v_range": [["7.0.2", ""]], "choices": ["deny", "permit"], "type": "str"},
            "id": {"v_range": [["7.0.2", ""]], "required": true, "type": "int"},
            "match": {"v_range": [["7.0.2", ""]], "type": "str"},
            "regexp": {"v_range": [["7.0.2", ""]], "type": "str"}
        }
    },
    "router_prefixlist": {
        "stated": true,
        "options": {
            "comments": {"v_range": [["7.0.2", ""]], "type": "str"},
            "name": {"v_range": [["7.0.2", ""]], "required": true, "type": "str"},
            "rule": {
                "v_range": [["7.0.2", ""]],
                "type": "list",
                "options": {
                    "action": {"v_range": [["7.0.2", ""]], "choices": ["permit", "deny"], "type": "str"},
                    "flags": {"v_range": [["7.0.2", ""]], "type": "int"},
                    "ge": {"v_range": [["7.0.2", ""]], "type": "int"},
                    "id": {"v_range": [["7.0.2", ""]], "type": "int"},
                    "le": {"v_range": [["7.0.2", ""]], "type": "int"},
                    "prefix": {"v_range": [["7.0.2", ""]], "type": "str"}
                },
                "elements": "dict"
            }
        }
    },
    "router_prefixlist6": {
        "stated": true,
        "options": {
            "comments": {"v_range": [["7.0.2", ""]], "type": "str"},
            "name": {"v_range": [["7.0.2", ""]], "required": true, "type": "str"},
            "rule": {
                "v_range": [["7.0.2", ""]],
                "type": "list",
                "options": {
                    "action": {"v_range": [["7.0.2", ""]], "choices": ["permit", "deny"], "type": "str"},
                    "flags": {"v_range": [["7.0.2", ""]], "type": "int"},
                    "ge": {"v_range": [["7.0.2", ""]], "type": "int"},
                    "id": {"v_range": [["7.0.2", ""]], "type": "int"},
                    "le": {"v_range": [["7.0.2", ""]], "type": "int"},
                    "prefix6": {"v_range": [["7.0.2", ""]], "type": "str"}
                },
                "elements": "dict"
            }
        }
    },
    "router_prefixlist6_rule": {
        "stated": true,
        "options": {
            "action": {"v_range": [["7.0.2", ""]], "choices": ["permit", "deny"], "type": "str"},
            "flags": {"v_range": [["7.0.2", ""]], "type": "int"},
            "ge": {"v_range": [["7.0.2", ""]], "type": "int"},
            "id": {"v_range": [["7.0.2", ""]], "required": true, "type": "int"},
            "le": {"v_range": [["7.0.2", ""]], "type": "int"},
            "prefix6": {"v_range": [["7.0.2", ""]], "type": "str"}
        }
    },
    "router_prefixlist_rule": {
        "stated": true,
        "options": {
            "action": {"v_range": [["7.0.2", ""]], "choices": ["permit", "deny"], "type": "str"},
            "flags": {"v_range": [["7.0.2", ""]], "type": "int"},
            "ge": {"v_range": [["7.0.2", ""]], "type": "int"},
            "id": {"v_range": [["7.0.2", ""]], "required": true, "type": "int"},
            "le": {"v_range": [["7.0.2", ""]], "type": "int"},
            "prefix": {"v_range": [["7.0.2", ""]], "type": "str"}
        }
    },
    "router_routemap": {
        "stated": true,
        "options": {
            "comments": {"v_range": [["7.0.2", ""]], "type": "str"},
            "name": {"v_range": [["7.0.2", ""]], "required": true, "type": "str"},
            "rule": {
                "v_range": [["7.0.2", ""]],
                "type": "list",
                "options": {
                    "action": {"v_range": [["7.0.2", ""]], "choices": ["permit", "deny"], "type": "str"},
                    "id": {"v_range": [["7.0.2", ""]], "type": "int"},
                    "match-as-path": {"v_range": [["7.0.2", ""]], "type": "str"},
                    "match-community": {"v_range": [["7.0.2", ""]], "type": "str"},
                    "match-community-exact": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "match-flags": {"v_range": [["7.0.2", ""]], "type": "int"},
                    "match-interface": {"v_range": [["7.0.2", ""]], "type": "str"},
                    "match-ip-address": {"v_range": [["7.0.2", ""]], "type": "str"},
                    "match-ip-nexthop": {"v_range": [["7.0.2", ""]], "type": "str"},
                    "match-ip6-address": {"v_range": [["7.0.2", ""]], "type": "str"},
                    "match-ip6-nexthop": {"v_range": [["7.0.2", ""]], "type": "str"},
                    "match-metric": {"v_range": [["7.0.2", ""]], "type": "str"},
                    "match-origin": {"v_range": [["7.0.2", ""]], "choices": ["none", "egp", "igp", "incomplete"], "type": "str"},
                    "match-route-type": {"v_range": [["7.0.2", ""]], "choices": ["1", "2", "none", "external-type1", "external-type2"], "type": "str"},
                    "match-tag": {"v_range": [["7.0.2", ""]], "type": "str"},
                    "match-vrf": {"v_range": [["7.0.2", ""]], "type": "int"},
                    "set-aggregator-as": {"v_range": [["7.0.2", ""]], "type": "int"},
                    "set-aggregator-ip": {"v_range": [["7.0.2", ""]], "type": "str"},
                    "set-aspath": {"v_range": [["7.0.2", ""]], "type": "raw"},
                    "set-aspath-action": {"v_range": [["7.0.2", ""]], "choices": ["prepend", "replace"], "type": "str"},
                    "set-atomic-aggregate": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "set-community": {"v_range": [["7.0.2", ""]], "type": "raw"},
                    "set-community-additive": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "set-community-delete": {"v_range": [["7.0.2", ""]], "type": "str"},
                    "set-dampening-max-suppress": {"v_range": [["7.0.2", ""]], "type": "int"},
                    "set-dampening-reachability-half-life": {"v_range": [["7.0.2", ""]], "type": "int"},
                    "set-dampening-reuse": {"v_range": [["7.0.2", ""]], "type": "int"},
                    "set-dampening-suppress": {"v_range": [["7.0.2", ""]], "type": "int"},
                    "set-dampening-unreachability-half-life": {"v_range": [["7.0.2", ""]], "type": "int"},
                    "set-extcommunity-rt": {"v_range": [["7.0.2", ""]], "type": "raw"},
                    "set-extcommunity-soo": {"v_range": [["7.0.2", ""]], "type": "raw"},
                    "set-flags": {"v_range": [["7.0.2", ""]], "type": "int"},
                    "set-ip-nexthop": {"v_range": [["7.0.2", ""]], "type": "str"},
                    "set-ip6-nexthop": {"v_range": [["7.0.2", ""]], "type": "str"},
                    "set-ip6-nexthop-local": {"v_range": [["7.0.2", ""]], "type": "str"},
                    "set-local-preference": {"v_range": [["7.0.2", ""]], "type": "str"},
                    "set-metric": {"v_range": [["7.0.2", ""]], "type": "str"},
                    "set-metric-type": {"v_range": [["7.0.2", ""]], "choices": ["1", "2", "none", "external-type1", "external-type2"], "type": "str"},
                    "set-origin": {"v_range": [["7.0.2", ""]], "choices": ["none", "egp", "igp", "incomplete"], "type": "str"},
                    "set-originator-id": {"v_range": [["7.0.2", ""]], "type": "str"},
                    "set-priority": {"v_range": [["7.2.0", ""]], "type": "int"},
                    "set-route-tag": {"v_range": [["7.0.2", ""]], "type": "str"},
                    "set-tag": {"v_range": [["7.0.2", ""]], "type": "str"},
                    "set-weight": {"v_range": [["7.0.2", ""]], "type": "str"},
                    "match-extcommunity": {"v_range": [["7.2.2", ""]], "type": "str"},
                    "match-extcommunity-exact": {"v_range": [["7.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "set-ip-prefsrc": {"v_range": [["7.4.0", ""]], "type": "str"},
                    "set-vpnv4-nexthop": {"v_range": [["7.4.1", ""]], "type": "str"},
                    "set-vpnv6-nexthop": {"v_range": [["7.4.2", ""]], "type": "str"},
                    "set-vpnv6-nexthop-local": {"v_range": [["7.4.2", ""]], "type": "str"}
                },
                "elements": "dict"
            }
        }
    },
    "router_routemap_rule": {
        "stated": true,
        "options": {
            "action": {"v_range": [["7.0.2", ""]], "choices": ["permit", "deny"], "type": "str"},
            "id": {"v_range": [["7.0.2", ""]], "required": true, "type": "int"},
            "match-as-path": {"v_range": [["7.0.2", ""]], "type": "str"},
            "match-community": {"v_range": [["7.0.2", ""]], "type": "str"},
            "match-community-exact": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "match-flags": {"v_range": [["7.0.2", ""]], "type": "int"},
            "match-interface": {"v_range": [["7.0.2", ""]], "type": "str"},
            "match-ip-address": {"v_range": [["7.0.2", ""]], "type": "str"},
            "match-ip-nexthop": {"v_range": [["7.0.2", ""]], "type": "str"},
            "match-ip6-address": {"v_range": [["7.0.2", ""]], "type": "str"},
            "match-ip6-nexthop": {"v_range": [["7.0.2", ""]], "type": "str"},
            "match-metric": {"v_range": [["7.0.2", ""]], "type": "str"},
            "match-origin": {"v_range": [["7.0.2", ""]], "choices": ["none", "egp", "igp", "incomplete"], "type": "str"},
            "match-route-type": {"v_range": [["7.0.2", ""]], "choices": ["1", "2", "none", "external-type1", "external-type2"], "type": "str"},
            "match-tag": {"v_range": [["7.0.2", ""]], "type": "str"},
            "match-vrf": {"v_range": [["7.0.2", ""]], "type": "int"},
            "set-aggregator-as": {"v_range": [["7.0.2", ""]], "type": "int"},
            "set-aggregator-ip": {"v_range": [["7.0.2", ""]], "type": "str"},
            "set-aspath": {"v_range": [["7.0.2", ""]], "type": "raw"},
            "set-aspath-action": {"v_range": [["7.0.2", ""]], "choices": ["prepend", "replace"], "type": "str"},
            "set-atomic-aggregate": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "set-community": {"v_range": [["7.0.2", ""]], "type": "raw"},
            "set-community-additive": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "set-community-delete": {"v_range": [["7.0.2", ""]], "type": "str"},
            "set-dampening-max-suppress": {"v_range": [["7.0.2", ""]], "type": "int"},
            "set-dampening-reachability-half-life": {"v_range": [["7.0.2", ""]], "type": "int"},
            "set-dampening-reuse": {"v_range": [["7.0.2", ""]], "type": "int"},
            "set-dampening-suppress": {"v_range": [["7.0.2", ""]], "type": "int"},
            "set-dampening-unreachability-half-life": {"v_range": [["7.0.2", ""]], "type": "int"},
            "set-extcommunity-rt": {"v_range": [["7.0.2", ""]], "type": "raw"},
            "set-extcommunity-soo": {"v_range": [["7.0.2", ""]], "type": "raw"},
            "set-flags": {"v_range": [["7.0.2", ""]], "type": "int"},
            "set-ip-nexthop": {"v_range": [["7.0.2", ""]], "type": "str"},
            "set-ip6-nexthop": {"v_range": [["7.0.2", ""]], "type": "str"},
            "set-ip6-nexthop-local": {"v_range": [["7.0.2", ""]], "type": "str"},
            "set-local-preference": {"v_range": [["7.0.2", ""]], "type": "str"},
            "set-metric": {"v_range": [["7.0.2", ""]], "type": "str"},
            "set-metric-type": {"v_range": [["7.0.2", ""]], "choices": ["1", "2", "none", "external-type1", "external-type2"], "type": "str"},
            "set-origin": {"v_range": [["7.0.2", ""]], "choices": ["none", "egp", "igp", "incomplete"], "type": "str"},
            "set-originator-id": {"v_range": [["7.0.2", ""]], "type": "str"},
            "set-priority": {"v_range": [["7.2.0", ""]], "type": "int"},
            "set-route-tag": {"v_range": [["7.0.2", ""]], "type": "str"},
            "set-tag": {"v_range": [["7.0.2", ""]], "type": "str"},
            "set-weight": {"v_range": [["7.0.2", ""]], "type": "str"},
            "match-extcommunity": {"v_range": [["7.2.2", ""]], "type": "str"},
            "match-extcommunity-exact": {"v_range": [["7.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "set-ip-prefsrc": {"v_range": [["7.4.0", ""]], "type": "str"},
            "set-vpnv4-nexthop": {"v_range": [["7.4.1", ""]], "type": "str"},
            "set-vpnv6-nexthop": {"v_range": [["7.4.2", ""]], "type": "str"},
            "set-vpnv6-nexthop-local": {"v_range": [["7.4.2", ""]], "type": "str"}
        }
    },
    "sctpfilter_profile": {
        "stated": true,
        "options": {
            "comment": {"v_range": [["7.2.5", "7.2.5"], ["7.4.2", ""]], "type": "str"},
            "name": {"v_range": [["7.2.5", "7.2.5"], ["7.4.2", ""]], "required": true, "type": "str"},
            "ppid-filters": {
                "v_range": [["7.2.5", "7.2.5"], ["7.4.2", ""]],
                "type": "list",
                "options": {
                    "action": {"v_range": [["7.2.5", "7.2.5"], ["7.4.2", ""]], "choices": ["pass", "reset", "replace"], "type": "str"},
                    "comment": {"v_range": [["7.2.5", "7.2.5"], ["7.4.2", ""]], "type": "str"},
                    "id": {"v_range": [["7.2.5", "7.2.5"], ["7.4.2", ""]], "type": "int"},
                    "ppid": {"v_range": [["7.2.5", "7.2.5"], ["7.4.2", ""]], "type": "int"}
                },
                "elements": "dict"
            }
        }
    },
    "sctpfilter_profile_ppidfilters": {
        "stated": true,
        "options": {
            "action": {"v_range": [["7.2.5", "7.2.5"], ["7.4.2", ""]], "choices": ["pass", "reset", "replace"], "type": "str"},
            "comment": {"v_range": [["7.2.5", "7.2.5"], ["7.4.2", ""]], "type": "str"},
            "id": {"v_range": [["7.2.5", "7.2.5"], ["7.4.2", ""]], "required": true, "type": "int"},
            "ppid": {"v_range": [["7.2.5", "7.2.5"], ["7.4.2", ""]], "type": "int"}
        }
    },
    "securityconsole_abort": {"stated": true, "options": {"adom": {"type": "str"}}},
    "securityconsole_assign_package": {
        "stated": true,
        "options": {
            "flags": {"type": "list", "choices": ["none", "cp_all_objs", "copy_assigned_pkg", "unassign"], "elements": "str"},
            "pkg": {"type": "str"},
            "target": {
                "type": "list",
                "options": {"adom": {"type": "str"}, "excluded": {"choices": ["disable", "enable"], "type": "str"}, "pkg": {"type": "str"}},
                "elements": "dict"
            }
        }
    },
    "securityconsole_cliprof_check": {
        "stated": true,
        "options": {
            "adom": {"v_range": [["7.0.1", ""]], "type": "str"},
            "cliprof": {"v_range": [["7.0.1", ""]], "type": "str"},
            "scope": {
                "v_range": [["7.2.1", ""]],
                "type": "list",
                "options": {"name": {"v_range": [["7.2.1", ""]], "type": "str"}, "vdom": {"v_range": [["7.2.1", ""]], "type": "str"}},
                "elements": "dict"
            }
        }
    },
    "securityconsole_import_dev_objs": {
        "stated": true,
        "options": {
            "add_mappings": {"choices": ["disable", "enable"], "type": "str"},
            "adom": {"type": "str"},
            "dst_name": {"type": "str"},
            "dst_parent": {"type": "str"},
            "if_all_objs": {"choices": ["none", "all", "filter"], "type": "str"},
            "if_all_policy": {"choices": ["disable", "enable"], "type": "str"},
            "import_action": {"choices": ["do", "policy_search", "obj_search"], "type": "str"},
            "name": {"type": "str"},
            "position": {"choices": ["bottom", "top"], "type": "str"},
            "vdom": {"type": "str"}
        }
    },
    "securityconsole_install_device": {
        "stated": true,
        "options": {
            "adom": {"type": "str"},
            "dev_rev_comments": {"type": "str"},
            "flags": {"type": "list", "choices": ["none", "preview", "auto_lock_ws"], "elements": "str"},
            "scope": {"type": "list", "options": {"name": {"type": "str"}, "vdom": {"type": "str"}}, "elements": "dict"}
        }
    },
    "securityconsole_install_objects_v2": {
        "stated": true,
        "options": {
            "adom": {"v_range": [["7.4.1", ""]], "type": "str"},
            "category": {"v_range": [["7.4.1", ""]], "type": "str"},
            "objects": {"v_range": [["7.4.1", ""]], "type": "list", "elements": "str"},
            "scope": {
                "v_range": [["7.4.1", ""]],
                "type": "list",
                "options": {"name": {"v_range": [["7.4.1", ""]], "type": "str"}, "vdom": {"v_range": [["7.4.1", ""]], "type": "str"}},
                "elements": "dict"
            }
        }
    },
    "securityconsole_install_package": {
        "stated": true,
        "options": {
            "adom": {"type": "str"},
            "adom_rev_comments": {"type": "str"},
            "adom_rev_name": {"type": "str"},
            "dev_rev_comments": {"type": "str"},
            "flags": {
                "type": "list",
                "choices": [
                    "none", "cp_all_objs", "preview", "generate_rev", "copy_assigned_pkg", "unassign", "ifpolicy_only", "no_ifpolicy", "objs_only",
                    "auto_lock_ws", "check_pkg_st", "copy_only"
                ],
                "elements": "str"
            },
            "pkg": {"type": "str"},
            "scope": {"type": "list", "options": {"name": {"type": "str"}, "vdom": {"type": "str"}}, "elements": "dict"}
        }
    },
    "securityconsole_install_preview": {
        "stated": true,
        "options": {
            "adom": {"type": "str"},
            "device": {"v_range": [["6.0.0", "7.4.3"]], "type": "str"},
            "flags": {"type": "list", "choices": ["none", "json", "default_cfg"], "elements": "str"},
            "vdoms": {"v_range": [["6.0.0", "7.4.3"]], "type": "raw"},
            "scope": {
                "v_range": [["7.6.0", ""]],
                "type": "list",
                "options": {"name": {"v_range": [["7.6.0", ""]], "type": "str"}, "vdom": {"v_range": [["7.6.0", ""]], "type": "str"}},
                "elements": "dict"
            }
        }
    },
    "securityconsole_package_cancel_install": {"stated": true, "options": {"adom": {"type": "str"}}},
    "securityconsole_package_clone": {
        "stated": true,
        "options": {
            "adom": {"type": "str"},
            "dst_name": {"type": "str"},
            "dst_parent": {"type": "str"},
            "pkg": {"type": "str"},
            "scope": {"type": "list", "options": {"name": {"type": "str"}, "vdom": {"type": "str"}}, "elements": "dict"}
        }
    },
    "securityconsole_package_commit": {
        "stated": true,
        "options": {"adom": {"type": "str"}, "scope": {"type": "list", "options": {"name": {"type": "str"}, "vdom": {"type": "str"}}, "elements": "dict"}}
    },
    "securityconsole_package_move": {
        "stated": true,
        "options": {"adom": {"type": "str"}, "dst_name": {"type": "str"}, "dst_parent": {"type": "str"}, "pkg": {"type": "str"}}
    },
    "securityconsole_pblock_clone": {"stated": true, "options": {"adom": {"type": "str"}, "dst_name": {"type": "str"}, "pblock": {"type": "str"}}},
    "securityconsole_preview_result": {
        "stated": true,
        "options": {
            "adom": {"type": "str"},
            "device": {"v_range": [["6.0.0", "7.4.3"]], "type": "str"},
            "scope": {
                "v_range": [["7.6.0", ""]],
                "type": "list",
                "options": {"name": {"v_range": [["7.6.0", ""]], "type": "str"}, "vdom": {"v_range": [["7.6.0", ""]], "type": "str"}},
                "elements": "dict"
            }
        }
    },
    "securityconsole_reinstall_package": {
        "stated": true,
        "options": {
            "adom": {"type": "str"},
            "flags": {
                "type": "list",
                "choices": [
                    "none", "cp_all_objs", "preview", "generate_rev", "copy_assigned_pkg", "unassign", "ifpolicy_only", "no_ifpolicy", "objs_only",
                    "auto_lock_ws", "check_pkg_st", "copy_only"
                ],
                "elements": "str"
            },
            "target": {
                "type": "list",
                "options": {
                    "pkg": {"type": "str"},
                    "scope": {"type": "list", "options": {"name": {"type": "str"}, "vdom": {"type": "str"}}, "elements": "dict"}
                },
                "elements": "dict"
            }
        }
    },
    "securityconsole_sign_certificate_template": {
        "stated": true,
        "options": {
            "adom": {"type": "str"},
            "scope": {"type": "list", "options": {"name": {"type": "str"}, "vdom": {"type": "str"}}, "elements": "dict"},
            "template": {"type": "str"}
        }
    },
    "securityconsole_template_cli_preview": {
        "stated": true,
        "options": {
            "adom": {"v_range": [["7.4.0", ""]], "type": "str"},
            "filename": {"v_range": [["7.4.0", ""]], "type": "str"},
            "pkg": {"v_range": [["7.4.0", ""]], "type": "str"},
            "scope": {
                "v_range": [["7.4.0", ""]],
                "type": "list",
                "options": {"name": {"v_range": [["7.4.0", ""]], "type": "str"}, "vdom": {"v_range": [["7.4.0", ""]], "type": "str"}},
                "elements": "dict"
            }
        }
    },
    "spamfilter_bwl": {
        "stated": true,
        "options": {
            "comment": {"v_range": [["6.0.0", "7.2.1"]], "type": "str"},
            "entries": {
                "v_range": [["6.0.0", "7.2.1"]],
                "type": "list",
                "options": {
                    "action": {"v_range": [["6.0.0", "7.2.1"]], "choices": ["spam", "clear", "reject"], "type": "str"},
                    "addr-type": {"v_range": [["6.0.0", "7.2.1"]], "choices": ["ipv4", "ipv6"], "type": "str"},
                    "email-pattern": {"v_range": [["6.0.0", "7.2.1"]], "type": "str"},
                    "id": {"v_range": [["6.0.0", "7.2.1"]], "type": "int"},
                    "ip4-subnet": {"v_range": [["6.0.0", "7.2.1"]], "type": "str"},
                    "ip6-subnet": {"v_range": [["6.0.0", "7.2.1"]], "type": "str"},
                    "pattern-type": {"v_range": [["6.0.0", "7.2.1"]], "choices": ["wildcard", "regexp"], "type": "str"},
                    "status": {"v_range": [["6.0.0", "7.2.1"]], "choices": ["disable", "enable"], "type": "str"},
                    "type": {"v_range": [["6.0.0", "7.2.1"]], "choices": ["ip", "email"], "type": "str"}
                },
                "elements": "dict"
            },
            "id": {"v_range": [["6.0.0", "7.2.1"]], "required": true, "type": "int"},
            "name": {"v_range": [["6.0.0", "7.2.1"]], "type": "str"}
        }
    },
    "spamfilter_bwl_entries": {
        "stated": true,
        "options": {
            "action": {"v_range": [["6.0.0", "7.2.1"]], "choices": ["spam", "clear", "reject"], "type": "str"},
            "addr-type": {"v_range": [["6.0.0", "7.2.1"]], "choices": ["ipv4", "ipv6"], "type": "str"},
            "email-pattern": {"v_range": [["6.0.0", "7.2.1"]], "type": "str"},
            "id": {"v_range": [["6.0.0", "7.2.1"]], "required": true, "type": "int"},
            "ip4-subnet": {"v_range": [["6.0.0", "7.2.1"]], "type": "str"},
            "ip6-subnet": {"v_range": [["6.0.0", "7.2.1"]], "type": "str"},
            "pattern-type": {"v_range": [["6.0.0", "7.2.1"]], "choices": ["wildcard", "regexp"], "type": "str"},
            "status": {"v_range": [["6.0.0", "7.2.1"]], "choices": ["disable", "enable"], "type": "str"},
            "type": {"v_range": [["6.0.0", "7.2.1"]], "choices": ["ip", "email"], "type": "str"}
        }
    },
    "spamfilter_bword": {
        "stated": true,
        "options": {
            "comment": {"v_range": [["6.0.0", "7.2.1"]], "type": "str"},
            "entries": {
                "v_range": [["6.0.0", "7.2.1"]],
                "type": "list",
                "options": {
                    "action": {"v_range": [["6.0.0", "7.2.1"]], "choices": ["spam", "clear"], "type": "str"},
                    "id": {"v_range": [["6.0.0", "7.2.1"]], "type": "int"},
                    "language": {
                        "v_range": [["6.0.0", "7.2.1"]],
                        "choices": ["western", "simch", "trach", "japanese", "korean", "french", "thai", "spanish"],
                        "type": "str"
                    },
                    "pattern": {"v_range": [["6.0.0", "7.2.1"]], "type": "str"},
                    "pattern-type": {"v_range": [["6.0.0", "7.2.1"]], "choices": ["wildcard", "regexp"], "type": "str"},
                    "score": {"v_range": [["6.0.0", "7.2.1"]], "type": "int"},
                    "status": {"v_range": [["6.0.0", "7.2.1"]], "choices": ["disable", "enable"], "type": "str"},
                    "where": {"v_range": [["6.0.0", "7.2.1"]], "choices": ["subject", "body", "all"], "type": "str"}
                },
                "elements": "dict"
            },
            "id": {"v_range": [["6.0.0", "7.2.1"]], "required": true, "type": "int"},
            "name": {"v_range": [["6.0.0", "7.2.1"]], "type": "str"}
        }
    },
    "spamfilter_bword_entries": {
        "stated": true,
        "options": {
            "action": {"v_range": [["6.0.0", "7.2.1"]], "choices": ["spam", "clear"], "type": "str"},
            "id": {"v_range": [["6.0.0", "7.2.1"]], "required": true, "type": "int"},
            "language": {
                "v_range": [["6.0.0", "7.2.1"]],
                "choices": ["western", "simch", "trach", "japanese", "korean", "french", "thai", "spanish"],
                "type": "str"
            },
            "pattern": {"v_range": [["6.0.0", "7.2.1"]], "type": "str"},
            "pattern-type": {"v_range": [["6.0.0", "7.2.1"]], "choices": ["wildcard", "regexp"], "type": "str"},
            "score": {"v_range": [["6.0.0", "7.2.1"]], "type": "int"},
            "status": {"v_range": [["6.0.0", "7.2.1"]], "choices": ["disable", "enable"], "type": "str"},
            "where": {"v_range": [["6.0.0", "7.2.1"]], "choices": ["subject", "body", "all"], "type": "str"}
        }
    },
    "spamfilter_dnsbl": {
        "stated": true,
        "options": {
            "comment": {"v_range": [["6.0.0", "7.2.1"]], "type": "str"},
            "entries": {
                "v_range": [["6.0.0", "7.2.1"]],
                "type": "list",
                "options": {
                    "action": {"v_range": [["6.0.0", "7.2.1"]], "choices": ["spam", "reject"], "type": "str"},
                    "id": {"v_range": [["6.0.0", "7.2.1"]], "type": "int"},
                    "server": {"v_range": [["6.0.0", "7.2.1"]], "type": "str"},
                    "status": {"v_range": [["6.0.0", "7.2.1"]], "choices": ["disable", "enable"], "type": "str"}
                },
                "elements": "dict"
            },
            "id": {"v_range": [["6.0.0", "7.2.1"]], "required": true, "type": "int"},
            "name": {"v_range": [["6.0.0", "7.2.1"]], "type": "str"}
        }
    },
    "spamfilter_dnsbl_entries": {
        "stated": true,
        "options": {
            "action": {"v_range": [["6.0.0", "7.2.1"]], "choices": ["spam", "reject"], "type": "str"},
            "id": {"v_range": [["6.0.0", "7.2.1"]], "required": true, "type": "int"},
            "server": {"v_range": [["6.0.0", "7.2.1"]], "type": "str"},
            "status": {"v_range": [["6.0.0", "7.2.1"]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "spamfilter_iptrust": {
        "stated": true,
        "options": {
            "comment": {"v_range": [["6.0.0", "7.2.1"]], "type": "str"},
            "entries": {
                "v_range": [["6.0.0", "7.2.1"]],
                "type": "list",
                "options": {
                    "addr-type": {"v_range": [["6.0.0", "7.2.1"]], "choices": ["ipv4", "ipv6"], "type": "str"},
                    "id": {"v_range": [["6.0.0", "7.2.1"]], "type": "int"},
                    "ip4-subnet": {"v_range": [["6.0.0", "7.2.1"]], "type": "str"},
                    "ip6-subnet": {"v_range": [["6.0.0", "7.2.1"]], "type": "str"},
                    "status": {"v_range": [["6.0.0", "7.2.1"]], "choices": ["disable", "enable"], "type": "str"}
                },
                "elements": "dict"
            },
            "id": {"v_range": [["6.0.0", "7.2.1"]], "required": true, "type": "int"},
            "name": {"v_range": [["6.0.0", "7.2.1"]], "type": "str"}
        }
    },
    "spamfilter_iptrust_entries": {
        "stated": true,
        "options": {
            "addr-type": {"v_range": [["6.0.0", "7.2.1"]], "choices": ["ipv4", "ipv6"], "type": "str"},
            "id": {"v_range": [["6.0.0", "7.2.1"]], "required": true, "type": "int"},
            "ip4-subnet": {"v_range": [["6.0.0", "7.2.1"]], "type": "str"},
            "ip6-subnet": {"v_range": [["6.0.0", "7.2.1"]], "type": "str"},
            "status": {"v_range": [["6.0.0", "7.2.1"]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "spamfilter_mheader": {
        "stated": true,
        "options": {
            "comment": {"v_range": [["6.0.0", "7.2.1"]], "type": "str"},
            "entries": {
                "v_range": [["6.0.0", "7.2.1"]],
                "type": "list",
                "options": {
                    "action": {"v_range": [["6.0.0", "7.2.1"]], "choices": ["spam", "clear"], "type": "str"},
                    "fieldbody": {"v_range": [["6.0.0", "7.2.1"]], "type": "str"},
                    "fieldname": {"v_range": [["6.0.0", "7.2.1"]], "type": "str"},
                    "id": {"v_range": [["6.0.0", "7.2.1"]], "type": "int"},
                    "pattern-type": {"v_range": [["6.0.0", "7.2.1"]], "choices": ["wildcard", "regexp"], "type": "str"},
                    "status": {"v_range": [["6.0.0", "7.2.1"]], "choices": ["disable", "enable"], "type": "str"}
                },
                "elements": "dict"
            },
            "id": {"v_range": [["6.0.0", "7.2.1"]], "required": true, "type": "int"},
            "name": {"v_range": [["6.0.0", "7.2.1"]], "type": "str"}
        }
    },
    "spamfilter_mheader_entries": {
        "stated": true,
        "options": {
            "action": {"v_range": [["6.0.0", "7.2.1"]], "choices": ["spam", "clear"], "type": "str"},
            "fieldbody": {"v_range": [["6.0.0", "7.2.1"]], "type": "str"},
            "fieldname": {"v_range": [["6.0.0", "7.2.1"]], "type": "str"},
            "id": {"v_range": [["6.0.0", "7.2.1"]], "required": true, "type": "int"},
            "pattern-type": {"v_range": [["6.0.0", "7.2.1"]], "choices": ["wildcard", "regexp"], "type": "str"},
            "status": {"v_range": [["6.0.0", "7.2.1"]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "spamfilter_profile": {
        "stated": true,
        "options": {
            "comment": {"v_range": [["6.0.0", "7.2.1"]], "type": "str"},
            "external": {"v_range": [["6.0.0", "7.2.1"]], "choices": ["disable", "enable"], "type": "str"},
            "flow-based": {"v_range": [["6.0.0", "7.2.1"]], "choices": ["disable", "enable"], "type": "str"},
            "name": {"v_range": [["6.0.0", "7.2.1"]], "required": true, "type": "str"},
            "options": {
                "v_range": [["6.0.0", "7.2.1"]],
                "type": "list",
                "choices": [
                    "bannedword", "spamemailbwl", "spamfsip", "spamfssubmit", "spamfschksum", "spamfsurl", "spamhelodns", "spamipbwl", "spamraddrdns",
                    "spamrbl", "spamhdrcheck", "spamfsphish", "spambwl"
                ],
                "elements": "str"
            },
            "replacemsg-group": {"v_range": [["6.0.0", "7.2.1"]], "type": "str"},
            "spam-bwl-table": {"v_range": [["6.0.0", "7.2.1"]], "type": "str"},
            "spam-bword-table": {"v_range": [["6.0.0", "7.2.1"]], "type": "str"},
            "spam-bword-threshold": {"v_range": [["6.0.0", "7.2.1"]], "type": "int"},
            "spam-filtering": {"v_range": [["6.0.0", "7.2.1"]], "choices": ["disable", "enable"], "type": "str"},
            "spam-iptrust-table": {"v_range": [["6.0.0", "7.2.1"]], "type": "str"},
            "spam-log": {"v_range": [["6.0.0", "7.2.1"]], "choices": ["disable", "enable"], "type": "str"},
            "spam-log-fortiguard-response": {"v_range": [["6.0.0", "7.2.1"]], "choices": ["disable", "enable"], "type": "str"},
            "spam-mheader-table": {"v_range": [["6.0.0", "7.2.1"]], "type": "str"},
            "spam-rbl-table": {"v_range": [["6.0.0", "7.2.1"]], "type": "str"},
            "gmail": {
                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", "7.2.1"]],
                "type": "dict",
                "options": {"log": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", "7.2.1"]], "choices": ["disable", "enable"], "type": "str"}}
            },
            "imap": {
                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", "7.2.1"]],
                "type": "dict",
                "options": {
                    "action": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", "7.2.1"]], "choices": ["pass", "tag"], "type": "str"},
                    "log": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", "7.2.1"]], "choices": ["disable", "enable"], "type": "str"},
                    "tag-msg": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", "7.2.1"]], "type": "str"},
                    "tag-type": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", "7.2.1"]],
                        "type": "list",
                        "choices": ["subject", "header", "spaminfo"],
                        "elements": "str"
                    }
                }
            },
            "mapi": {
                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", "7.2.1"]],
                "type": "dict",
                "options": {
                    "action": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", "7.2.1"]], "choices": ["pass", "discard"], "type": "str"},
                    "log": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", "7.2.1"]], "choices": ["disable", "enable"], "type": "str"}
                }
            },
            "msn-hotmail": {
                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", "7.2.1"]],
                "type": "dict",
                "options": {"log": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", "7.2.1"]], "choices": ["disable", "enable"], "type": "str"}}
            },
            "pop3": {
                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", "7.2.1"]],
                "type": "dict",
                "options": {
                    "action": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", "7.2.1"]], "choices": ["pass", "tag"], "type": "str"},
                    "log": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", "7.2.1"]], "choices": ["disable", "enable"], "type": "str"},
                    "tag-msg": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", "7.2.1"]], "type": "str"},
                    "tag-type": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", "7.2.1"]],
                        "type": "list",
                        "choices": ["subject", "header", "spaminfo"],
                        "elements": "str"
                    }
                }
            },
            "smtp": {
                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", "7.2.1"]],
                "type": "dict",
                "options": {
                    "action": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", "7.2.1"]], "choices": ["pass", "tag", "discard"], "type": "str"},
                    "hdrip": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", "7.2.1"]], "choices": ["disable", "enable"], "type": "str"},
                    "local-override": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", "7.2.1"]], "choices": ["disable", "enable"], "type": "str"},
                    "log": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", "7.2.1"]], "choices": ["disable", "enable"], "type": "str"},
                    "tag-msg": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", "7.2.1"]], "type": "str"},
                    "tag-type": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", "7.2.1"]],
                        "type": "list",
                        "choices": ["subject", "header", "spaminfo"],
                        "elements": "str"
                    }
                }
            },
            "yahoo-mail": {
                "v_range": [["6.2.8", "6.2.12"]],
                "type": "dict",
                "options": {"log": {"v_range": [["6.2.8", "6.2.12"]], "choices": ["disable", "enable"], "type": "str"}}
            }
        }
    },
    "spamfilter_profile_gmail": {"stated": false, "options": {"log": {"v_range": [["6.0.0", "7.2.1"]], "choices": ["disable", "enable"], "type": "str"}}},
    "spamfilter_profile_imap": {
        "stated": false,
        "options": {
            "action": {"v_range": [["6.0.0", "7.2.1"]], "choices": ["pass", "tag"], "type": "str"},
            "log": {"v_range": [["6.0.0", "7.2.1"]], "choices": ["disable", "enable"], "type": "str"},
            "tag-msg": {"v_range": [["6.0.0", "7.2.1"]], "type": "str"},
            "tag-type": {"v_range": [["6.0.0", "7.2.1"]], "type": "list", "choices": ["subject", "header", "spaminfo"], "elements": "str"}
        }
    },
    "spamfilter_profile_mapi": {
        "stated": false,
        "options": {
            "action": {"v_range": [["6.0.0", "7.2.1"]], "choices": ["pass", "discard"], "type": "str"},
            "log": {"v_range": [["6.0.0", "7.2.1"]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "spamfilter_profile_msnhotmail": {
        "stated": false,
        "options": {"log": {"v_range": [["6.0.0", "7.2.1"]], "choices": ["disable", "enable"], "type": "str"}}
    },
    "spamfilter_profile_pop3": {
        "stated": false,
        "options": {
            "action": {"v_range": [["6.0.0", "7.2.1"]], "choices": ["pass", "tag"], "type": "str"},
            "log": {"v_range": [["6.0.0", "7.2.1"]], "choices": ["disable", "enable"], "type": "str"},
            "tag-msg": {"v_range": [["6.0.0", "7.2.1"]], "type": "str"},
            "tag-type": {"v_range": [["6.0.0", "7.2.1"]], "type": "list", "choices": ["subject", "header", "spaminfo"], "elements": "str"}
        }
    },
    "spamfilter_profile_smtp": {
        "stated": false,
        "options": {
            "action": {"v_range": [["6.0.0", "7.2.1"]], "choices": ["pass", "tag", "discard"], "type": "str"},
            "hdrip": {"v_range": [["6.0.0", "7.2.1"]], "choices": ["disable", "enable"], "type": "str"},
            "local-override": {"v_range": [["6.0.0", "7.2.1"]], "choices": ["disable", "enable"], "type": "str"},
            "log": {"v_range": [["6.0.0", "7.2.1"]], "choices": ["disable", "enable"], "type": "str"},
            "tag-msg": {"v_range": [["6.0.0", "7.2.1"]], "type": "str"},
            "tag-type": {"v_range": [["6.0.0", "7.2.1"]], "type": "list", "choices": ["subject", "header", "spaminfo"], "elements": "str"}
        }
    },
    "spamfilter_profile_yahoomail": {
        "stated": false,
        "options": {"log": {"v_range": [["6.0.0", "6.2.12"]], "choices": ["disable", "enable"], "type": "str"}}
    },
    "sshfilter_profile": {
        "stated": true,
        "options": {
            "block": {"type": "list", "choices": ["x11", "shell", "exec", "port-forward", "tun-forward", "sftp", "unknown", "scp"], "elements": "str"},
            "default-command-log": {"choices": ["disable", "enable"], "type": "str"},
            "log": {"type": "list", "choices": ["x11", "shell", "exec", "port-forward", "tun-forward", "sftp", "unknown", "scp"], "elements": "str"},
            "name": {"required": true, "type": "str"},
            "shell-commands": {
                "type": "list",
                "options": {
                    "action": {"choices": ["block", "allow"], "type": "str"},
                    "alert": {"choices": ["disable", "enable"], "type": "str"},
                    "id": {"type": "int"},
                    "log": {"choices": ["disable", "enable"], "type": "str"},
                    "pattern": {"type": "str"},
                    "severity": {"choices": ["low", "medium", "high", "critical"], "type": "str"},
                    "type": {"choices": ["regex", "simple"], "type": "str"}
                },
                "elements": "dict"
            },
            "file-filter": {
                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                "type": "dict",
                "options": {
                    "entries": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "type": "list",
                        "options": {
                            "action": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["log", "block"], "type": "str"},
                            "comment": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                            "direction": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["any", "incoming", "outgoing"], "type": "str"},
                            "file-type": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "raw"},
                            "filter": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                            "password-protected": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["any", "yes"], "type": "str"},
                            "protocol": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "list", "choices": ["ssh"], "elements": "str"}
                        },
                        "elements": "dict"
                    },
                    "log": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "scan-archive-contents": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "status": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"}
                }
            }
        }
    },
    "sshfilter_profile_filefilter": {
        "stated": false,
        "options": {
            "entries": {
                "v_range": [["6.2.2", ""]],
                "type": "list",
                "options": {
                    "action": {"v_range": [["6.2.2", ""]], "choices": ["log", "block"], "type": "str"},
                    "comment": {"v_range": [["6.2.2", ""]], "type": "str"},
                    "direction": {"v_range": [["6.2.2", ""]], "choices": ["any", "incoming", "outgoing"], "type": "str"},
                    "file-type": {"v_range": [["6.2.2", ""]], "type": "raw"},
                    "filter": {"v_range": [["6.2.2", ""]], "type": "str"},
                    "password-protected": {"v_range": [["6.2.2", ""]], "choices": ["any", "yes"], "type": "str"},
                    "protocol": {"v_range": [["6.2.2", ""]], "type": "list", "choices": ["ssh"], "elements": "str"}
                },
                "elements": "dict"
            },
            "log": {"v_range": [["6.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "scan-archive-contents": {"v_range": [["6.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "status": {"v_range": [["6.2.2", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "sshfilter_profile_filefilter_entries": {
        "stated": true,
        "options": {
            "action": {"v_range": [["6.2.2", ""]], "choices": ["log", "block"], "type": "str"},
            "comment": {"v_range": [["6.2.2", ""]], "type": "str"},
            "direction": {"v_range": [["6.2.2", ""]], "choices": ["any", "incoming", "outgoing"], "type": "str"},
            "file-type": {"v_range": [["6.2.2", ""]], "type": "raw"},
            "filter": {"v_range": [["6.2.2", ""]], "type": "str"},
            "password-protected": {"v_range": [["6.2.2", ""]], "choices": ["any", "yes"], "type": "str"},
            "protocol": {"v_range": [["6.2.2", ""]], "type": "list", "choices": ["ssh"], "elements": "str"}
        }
    },
    "sshfilter_profile_shellcommands": {
        "stated": true,
        "options": {
            "action": {"choices": ["block", "allow"], "type": "str"},
            "alert": {"choices": ["disable", "enable"], "type": "str"},
            "id": {"required": true, "type": "int"},
            "log": {"choices": ["disable", "enable"], "type": "str"},
            "pattern": {"type": "str"},
            "severity": {"choices": ["low", "medium", "high", "critical"], "type": "str"},
            "type": {"choices": ["regex", "simple"], "type": "str"}
        }
    },
    "switchcontroller_acl_group": {
        "stated": true,
        "options": {"ingress": {"v_range": [["7.4.0", ""]], "type": "raw"}, "name": {"v_range": [["7.4.0", ""]], "required": true, "type": "str"}}
    },
    "switchcontroller_acl_ingress": {
        "stated": true,
        "options": {
            "action": {
                "v_range": [["7.4.0", ""]],
                "type": "dict",
                "options": {
                    "count": {"v_range": [["7.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "drop": {"v_range": [["7.4.0", ""]], "choices": ["disable", "enable"], "type": "str"}
                }
            },
            "classifier": {
                "v_range": [["7.4.0", ""]],
                "type": "dict",
                "options": {
                    "dst-ip-prefix": {"v_range": [["7.4.0", ""]], "type": "str"},
                    "dst-mac": {"v_range": [["7.4.0", ""]], "type": "str"},
                    "src-ip-prefix": {"v_range": [["7.4.0", ""]], "type": "str"},
                    "src-mac": {"v_range": [["7.4.0", ""]], "type": "str"},
                    "vlan": {"v_range": [["7.4.0", ""]], "type": "int"}
                }
            },
            "description": {"v_range": [["7.4.0", ""]], "type": "str"},
            "id": {"v_range": [["7.4.0", ""]], "required": true, "type": "int"}
        }
    },
    "switchcontroller_acl_ingress_action": {
        "stated": false,
        "options": {
            "count": {"v_range": [["7.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "drop": {"v_range": [["7.4.0", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "switchcontroller_acl_ingress_classifier": {
        "stated": false,
        "options": {
            "dst-ip-prefix": {"v_range": [["7.4.0", ""]], "type": "str"},
            "dst-mac": {"v_range": [["7.4.0", ""]], "type": "str"},
            "src-ip-prefix": {"v_range": [["7.4.0", ""]], "type": "str"},
            "src-mac": {"v_range": [["7.4.0", ""]], "type": "str"},
            "vlan": {"v_range": [["7.4.0", ""]], "type": "int"}
        }
    },
    "switchcontroller_customcommand": {
        "stated": true,
        "options": {
            "command": {"v_range": [["7.0.0", ""]], "type": "str"},
            "command-name": {"v_range": [["7.0.0", ""]], "type": "str"},
            "description": {"v_range": [["7.0.0", ""]], "type": "str"}
        }
    },
    "switchcontroller_dsl_policy": {
        "stated": true,
        "options": {
            "append_padding": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "cpe-aele": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "cpe-aele-mode": {"v_range": [["7.0.3", ""]], "choices": ["ELE_M0", "ELE_DS", "ELE_PB", "ELE_MIN"], "type": "str"},
            "cs": {"v_range": [["7.0.3", ""]], "type": "list", "choices": ["A43", "B43", "A43C", "V43"], "elements": "str"},
            "ds-bitswap": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "name": {"v_range": [["7.0.3", ""]], "required": true, "type": "str"},
            "pause-frame": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "profile": {"v_range": [["7.0.3", ""]], "choices": ["auto-30a", "auto-17a", "auto-12ab"], "type": "str"},
            "type": {"v_range": [["7.0.3", ""]], "choices": ["Procend", "Proscend"], "type": "str"},
            "us-bitswap": {"v_range": [["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "switchcontroller_dynamicportpolicy": {
        "stated": true,
        "options": {
            "description": {"v_range": [["7.2.1", ""]], "type": "str"},
            "name": {"v_range": [["7.2.1", ""]], "required": true, "type": "str"},
            "policy": {
                "v_range": [["7.2.1", ""]],
                "type": "list",
                "options": {
                    "802-1x": {"v_range": [["7.2.1", ""]], "type": "str"},
                    "bounce-port-link": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "category": {"v_range": [["7.2.1", ""]], "choices": ["device", "interface-tag"], "type": "str"},
                    "description": {"v_range": [["7.2.1", ""]], "type": "str"},
                    "family": {"v_range": [["7.2.1", ""]], "type": "str"},
                    "host": {"v_range": [["7.2.1", ""]], "type": "str"},
                    "hw-vendor": {"v_range": [["7.2.1", ""]], "type": "str"},
                    "interface-tags": {"v_range": [["7.2.1", ""]], "type": "raw"},
                    "lldp-profile": {"v_range": [["7.2.1", ""]], "type": "str"},
                    "mac": {"v_range": [["7.2.1", ""]], "type": "str"},
                    "name": {"v_range": [["7.2.1", ""]], "type": "str"},
                    "qos-policy": {"v_range": [["7.2.1", ""]], "type": "str"},
                    "status": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "type": {"v_range": [["7.2.1", ""]], "type": "str"},
                    "vlan-policy": {"v_range": [["7.2.1", ""]], "type": "str"},
                    "match-period": {"v_range": [["7.4.3", ""]], "type": "int"},
                    "match-type": {"v_range": [["7.4.3", ""]], "choices": ["dynamic", "override"], "type": "str"}
                },
                "elements": "dict"
            },
            "fortilink": {"v_range": [["7.4.3", ""]], "type": "raw"}
        }
    },
    "switchcontroller_dynamicportpolicy_policy": {
        "stated": true,
        "options": {
            "802-1x": {"v_range": [["7.2.1", ""]], "type": "str"},
            "bounce-port-link": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "category": {"v_range": [["7.2.1", ""]], "choices": ["device", "interface-tag"], "type": "str"},
            "description": {"v_range": [["7.2.1", ""]], "type": "str"},
            "family": {"v_range": [["7.2.1", ""]], "type": "str"},
            "host": {"v_range": [["7.2.1", ""]], "type": "str"},
            "hw-vendor": {"v_range": [["7.2.1", ""]], "type": "str"},
            "interface-tags": {"v_range": [["7.2.1", ""]], "type": "raw"},
            "lldp-profile": {"v_range": [["7.2.1", ""]], "type": "str"},
            "mac": {"v_range": [["7.2.1", ""]], "type": "str"},
            "name": {"v_range": [["7.2.1", ""]], "required": true, "type": "str"},
            "qos-policy": {"v_range": [["7.2.1", ""]], "type": "str"},
            "status": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "type": {"v_range": [["7.2.1", ""]], "type": "str"},
            "vlan-policy": {"v_range": [["7.2.1", ""]], "type": "str"},
            "match-period": {"v_range": [["7.4.3", ""]], "type": "int"},
            "match-type": {"v_range": [["7.4.3", ""]], "choices": ["dynamic", "override"], "type": "str"}
        }
    },
    "switchcontroller_fortilinksettings": {
        "stated": true,
        "options": {
            "fortilink": {"v_range": [["7.2.1", "7.2.3"], ["7.4.0", "7.4.0"], ["7.4.3", ""]], "type": "str"},
            "inactive-timer": {"v_range": [["7.2.1", ""]], "type": "int"},
            "link-down-flush": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "nac-ports": {
                "v_range": [["7.2.1", ""]],
                "type": "dict",
                "options": {
                    "lan-segment": {"v_range": [["7.2.1", ""]], "choices": ["disabled", "enabled"], "type": "str"},
                    "member-change": {"v_range": [["7.2.1", ""]], "type": "int"},
                    "nac-lan-interface": {"v_range": [["7.2.1", ""]], "type": "str"},
                    "nac-segment-vlans": {"v_range": [["7.2.1", ""]], "type": "raw"},
                    "onboarding-vlan": {"v_range": [["7.2.1", ""]], "type": "str"},
                    "parent-key": {"v_range": [["7.2.1", ""]], "no_log": true, "type": "str"},
                    "bounce-nac-port": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"}
                }
            },
            "name": {"v_range": [["7.2.1", ""]], "required": true, "type": "str"},
            "access-vlan-mode": {"v_range": [["7.4.1", ""]], "choices": ["legacy", "fail-open", "fail-close"], "type": "str"}
        }
    },
    "switchcontroller_fortilinksettings_nacports": {
        "stated": false,
        "options": {
            "lan-segment": {"v_range": [["7.2.1", ""]], "choices": ["disabled", "enabled"], "type": "str"},
            "member-change": {"v_range": [["7.2.1", ""]], "type": "int"},
            "nac-lan-interface": {"v_range": [["7.2.1", ""]], "type": "str"},
            "nac-segment-vlans": {"v_range": [["7.2.1", ""]], "type": "raw"},
            "onboarding-vlan": {"v_range": [["7.2.1", ""]], "type": "str"},
            "parent-key": {"v_range": [["7.2.1", ""]], "no_log": true, "type": "str"},
            "bounce-nac-port": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "switchcontroller_lldpprofile": {
        "stated": true,
        "options": {
            "802.1-tlvs": {"type": "list", "choices": ["port-vlan-id"], "elements": "str"},
            "802.3-tlvs": {"type": "list", "choices": ["max-frame-size", "power-negotiation"], "elements": "str"},
            "auto-isl": {"choices": ["disable", "enable"], "type": "str"},
            "auto-isl-hello-timer": {"type": "int"},
            "auto-isl-port-group": {"type": "int"},
            "auto-isl-receive-timeout": {"type": "int"},
            "custom-tlvs": {
                "type": "list",
                "options": {"information-string": {"type": "str"}, "name": {"type": "str"}, "oui": {"type": "str"}, "subtype": {"type": "int"}},
                "elements": "dict"
            },
            "med-network-policy": {
                "type": "list",
                "options": {
                    "dscp": {"type": "int"},
                    "name": {"type": "str"},
                    "priority": {"type": "int"},
                    "status": {"choices": ["disable", "enable"], "type": "str"},
                    "vlan": {"type": "int"},
                    "vlan-intf": {"v_range": [["6.2.0", ""]], "type": "str"},
                    "assign-vlan": {"v_range": [["6.2.2", ""]], "choices": ["disable", "enable"], "type": "str"}
                },
                "elements": "dict"
            },
            "med-tlvs": {
                "type": "list",
                "choices": ["inventory-management", "network-policy", "power-management", "location-identification"],
                "elements": "str"
            },
            "name": {"required": true, "type": "str"},
            "med-location-service": {
                "v_range": [["6.2.0", ""]],
                "type": "list",
                "options": {
                    "name": {"v_range": [["6.2.0", ""]], "type": "str"},
                    "status": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "sys-location-id": {"v_range": [["6.2.0", ""]], "type": "str"}
                },
                "elements": "dict"
            },
            "auto-mclag-icl": {"v_range": [["6.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "auto-isl-auth": {"v_range": [["7.4.1", ""]], "choices": ["legacy", "strict", "relax"], "type": "str"},
            "auto-isl-auth-encrypt": {"v_range": [["7.4.1", ""]], "choices": ["none", "mixed", "must"], "type": "str"},
            "auto-isl-auth-identity": {"v_range": [["7.4.1", ""]], "type": "str"},
            "auto-isl-auth-macsec-profile": {"v_range": [["7.4.1", ""]], "type": "str"},
            "auto-isl-auth-reauth": {"v_range": [["7.4.1", ""]], "type": "int"},
            "auto-isl-auth-user": {"v_range": [["7.4.1", ""]], "type": "str"}
        }
    },
    "switchcontroller_lldpprofile_customtlvs": {
        "stated": true,
        "options": {"information-string": {"type": "str"}, "name": {"required": true, "type": "str"}, "oui": {"type": "str"}, "subtype": {"type": "int"}}
    },
    "switchcontroller_lldpprofile_medlocationservice": {
        "stated": true,
        "options": {
            "name": {"v_range": [["6.2.0", ""]], "required": true, "type": "str"},
            "status": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "sys-location-id": {"v_range": [["6.2.0", ""]], "type": "str"}
        }
    },
    "switchcontroller_lldpprofile_mednetworkpolicy": {
        "stated": true,
        "options": {
            "dscp": {"type": "int"},
            "name": {"required": true, "type": "str"},
            "priority": {"type": "int"},
            "status": {"choices": ["disable", "enable"], "type": "str"},
            "vlan": {"type": "int"},
            "vlan-intf": {"v_range": [["6.2.0", ""]], "type": "str"},
            "assign-vlan": {"v_range": [["6.2.2", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "switchcontroller_macpolicy": {
        "stated": true,
        "options": {
            "bounce-port-link": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "count": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "description": {"v_range": [["7.2.1", ""]], "type": "str"},
            "name": {"v_range": [["7.2.1", ""]], "required": true, "type": "str"},
            "traffic-policy": {"v_range": [["7.2.1", ""]], "type": "str"},
            "vlan": {"v_range": [["7.2.1", ""]], "type": "str"},
            "drop": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "fortilink": {"v_range": [["7.4.3", ""]], "type": "raw"}
        }
    },
    "switchcontroller_managedswitch": {
        "stated": true,
        "options": {
            "_platform": {"type": "str"},
            "description": {"type": "str"},
            "name": {"type": "str"},
            "ports": {
                "type": "list",
                "options": {
                    "allowed-vlans": {"type": "raw"},
                    "allowed-vlans-all": {"choices": ["disable", "enable"], "type": "str"},
                    "arp-inspection-trust": {"choices": ["untrusted", "trusted"], "type": "str"},
                    "bundle": {"choices": ["disable", "enable"], "type": "str"},
                    "description": {"type": "str"},
                    "dhcp-snoop-option82-trust": {"choices": ["disable", "enable"], "type": "str"},
                    "dhcp-snooping": {"choices": ["trusted", "untrusted"], "type": "str"},
                    "discard-mode": {"choices": ["none", "all-untagged", "all-tagged"], "type": "str"},
                    "edge-port": {"choices": ["disable", "enable"], "type": "str"},
                    "igmp-snooping": {"choices": ["disable", "enable"], "type": "str"},
                    "igmps-flood-reports": {"choices": ["disable", "enable"], "type": "str"},
                    "igmps-flood-traffic": {"choices": ["disable", "enable"], "type": "str"},
                    "lacp-speed": {"choices": ["slow", "fast"], "type": "str"},
                    "learning-limit": {"type": "int"},
                    "lldp-profile": {"type": "str"},
                    "lldp-status": {"choices": ["disable", "rx-only", "tx-only", "tx-rx"], "type": "str"},
                    "loop-guard": {"choices": ["disabled", "enabled"], "type": "str"},
                    "loop-guard-timeout": {"type": "int"},
                    "max-bundle": {"type": "int"},
                    "mclag": {"choices": ["disable", "enable"], "type": "str"},
                    "member-withdrawal-behavior": {"choices": ["forward", "block"], "type": "str"},
                    "members": {"type": "raw"},
                    "min-bundle": {"type": "int"},
                    "mode": {"choices": ["static", "lacp-passive", "lacp-active"], "type": "str"},
                    "poe-pre-standard-detection": {"choices": ["disable", "enable"], "type": "str"},
                    "poe-status": {"choices": ["disable", "enable"], "type": "str"},
                    "port-name": {"type": "str"},
                    "port-owner": {"type": "str"},
                    "port-security-policy": {"type": "str"},
                    "port-selection-criteria": {"choices": ["src-mac", "dst-mac", "src-dst-mac", "src-ip", "dst-ip", "src-dst-ip"], "type": "str"},
                    "qos-policy": {"type": "str"},
                    "sample-direction": {"choices": ["rx", "tx", "both"], "type": "str"},
                    "sflow-counter-interval": {"type": "int"},
                    "sflow-sample-rate": {"v_range": [["6.0.0", "7.2.1"]], "type": "int"},
                    "sflow-sampler": {"v_range": [["6.0.0", "7.2.1"]], "choices": ["disabled", "enabled"], "type": "str"},
                    "stp-bpdu-guard": {"choices": ["disabled", "enabled"], "type": "str"},
                    "stp-bpdu-guard-timeout": {"type": "int"},
                    "stp-root-guard": {"choices": ["disabled", "enabled"], "type": "str"},
                    "stp-state": {"choices": ["disabled", "enabled"], "type": "str"},
                    "type": {"choices": ["physical", "trunk"], "type": "str"},
                    "untagged-vlans": {"type": "raw"},
                    "vlan": {"type": "str"},
                    "export-to-pool-flag": {"v_range": [["6.2.1", "6.2.3"], ["7.4.3", ""]], "type": "int"},
                    "mac-addr": {"v_range": [["6.2.1", "6.2.1"], ["7.4.3", ""]], "type": "str"},
                    "packet-sample-rate": {"v_range": [["6.2.0", ""]], "type": "int"},
                    "packet-sampler": {"v_range": [["6.2.0", ""]], "choices": ["disabled", "enabled"], "type": "str"},
                    "sticky-mac": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "storm-control-policy": {"v_range": [["6.2.0", "6.2.3"], ["7.4.3", ""]], "type": "str"},
                    "dot1x-enable": {"v_range": [["6.2.0", "6.2.12"]], "choices": ["disable", "enable"], "type": "str"},
                    "max-miss-heartbeats": {"v_range": [["6.2.0", "6.2.12"]], "type": "int"},
                    "access-mode": {"v_range": [["6.4.0", ""]], "choices": ["normal", "nac", "dynamic", "static"], "type": "str"},
                    "ip-source-guard": {"v_range": [["6.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "mclag-icl-port": {"v_range": [["6.4.0", ""]], "type": "int"},
                    "p2p-port": {"v_range": [["6.4.0", ""]], "type": "int"},
                    "aggregator-mode": {"v_range": [["6.4.2", ""]], "choices": ["bandwidth", "count"], "type": "str"},
                    "rpvst-port": {"v_range": [["6.4.2", ""]], "choices": ["disabled", "enabled"], "type": "str"},
                    "flow-control": {"v_range": [["6.4.3", ""]], "choices": ["disable", "tx", "rx", "both"], "type": "str"},
                    "media-type": {"v_range": [["6.4.3", ""]], "type": "str"},
                    "pause-meter": {"v_range": [["6.4.3", ""]], "type": "int"},
                    "pause-meter-resume": {"v_range": [["6.4.3", ""]], "choices": ["25%", "50%", "75%"], "type": "str"},
                    "trunk-member": {"v_range": [["6.2.7", "6.2.12"], ["6.4.3", ""]], "type": "int"},
                    "fec-capable": {"v_range": [["7.0.0", ""]], "type": "int"},
                    "fec-state": {"v_range": [["7.0.0", ""]], "choices": ["disabled", "cl74", "cl91", "detect-by-module"], "type": "str"},
                    "matched-dpp-intf-tags": {"v_range": [["7.0.0", ""]], "type": "str"},
                    "matched-dpp-policy": {"v_range": [["7.0.0", ""]], "type": "str"},
                    "port-policy": {"v_range": [["7.0.0", "7.0.4"], ["7.2.0", ""]], "type": "str"},
                    "status": {"v_range": [["6.4.6", ""]], "choices": ["down", "up"], "type": "str"},
                    "dsl-profile": {"v_range": [["7.0.2", ""]], "type": "str"},
                    "flap-duration": {"v_range": [["7.2.0", ""]], "type": "int"},
                    "flap-rate": {"v_range": [["7.2.0", ""]], "type": "int"},
                    "flap-timeout": {"v_range": [["7.2.0", ""]], "type": "int"},
                    "flapguard": {"v_range": [["7.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "interface-tags": {"v_range": [["7.0.2", ""]], "type": "raw"},
                    "poe-max-power": {"v_range": [["7.0.1", ""]], "type": "str"},
                    "poe-standard": {"v_range": [["7.0.1", ""]], "type": "str"},
                    "igmp-snooping-flood-reports": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "mcast-snooping-flood-traffic": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "link-status": {"v_range": [["7.2.2", ""]], "choices": ["down", "up"], "type": "str"},
                    "poe-mode-bt-cabable": {"v_range": [["7.2.2", ""]], "type": "int"},
                    "poe-port-mode": {"v_range": [["7.2.2", ""]], "choices": ["ieee802-3af", "ieee802-3at", "ieee802-3bt"], "type": "str"},
                    "poe-port-power": {"v_range": [["7.2.2", ""]], "choices": ["normal", "perpetual", "perpetual-fast"], "type": "str"},
                    "poe-port-priority": {
                        "v_range": [["7.2.2", ""]],
                        "choices": ["critical-priority", "high-priority", "low-priority", "medium-priority"],
                        "type": "str"
                    },
                    "acl-group": {"v_range": [["7.4.0", ""]], "type": "raw"},
                    "dhcp-snoop-option82-override": {
                        "v_range": [["7.4.0", ""]],
                        "type": "list",
                        "options": {
                            "circuit-id": {"v_range": [["7.4.0", ""]], "type": "str"},
                            "remote-id": {"v_range": [["7.4.0", ""]], "type": "str"},
                            "vlan-name": {"v_range": [["7.4.0", ""]], "type": "str"}
                        },
                        "elements": "dict"
                    },
                    "fortiswitch-acls": {"v_range": [["7.4.0", ""]], "type": "raw"},
                    "isl-peer-device-sn": {"v_range": [["7.4.0", ""]], "type": "str"},
                    "authenticated-port": {"v_range": [["7.4.1", ""]], "type": "int"},
                    "encrypted-port": {"v_range": [["7.4.1", ""]], "type": "int"},
                    "ptp-status": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "restricted-auth-port": {"v_range": [["7.4.1", ""]], "type": "int"},
                    "allow-arp-monitor": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "export-to": {"v_range": [["7.4.3", ""]], "type": "raw"},
                    "export-to-pool": {"v_range": [["7.4.3", ""]], "type": "raw"},
                    "fallback-port": {"v_range": [["7.4.3", ""]], "type": "str"},
                    "fgt-peer-device-name": {"v_range": [["7.4.3", ""]], "type": "str"},
                    "fgt-peer-port-name": {"v_range": [["7.4.3", ""]], "type": "str"},
                    "fiber-port": {"v_range": [["7.4.3", ""]], "type": "int"},
                    "flags": {"v_range": [["7.4.3", ""]], "type": "int"},
                    "fortilink-port": {"v_range": [["7.4.3", ""]], "type": "int"},
                    "isl-local-trunk-name": {"v_range": [["7.4.3", ""]], "type": "str"},
                    "isl-peer-device-name": {"v_range": [["7.4.3", ""]], "type": "str"},
                    "isl-peer-port-name": {"v_range": [["7.4.3", ""]], "type": "str"},
                    "poe-capable": {"v_range": [["7.4.3", ""]], "type": "int"},
                    "port-number": {"v_range": [["7.4.3", ""]], "type": "int"},
                    "port-prefix-type": {"v_range": [["7.4.3", ""]], "type": "int"},
                    "ptp-policy": {"v_range": [["7.4.3", ""]], "type": "raw"},
                    "speed": {
                        "v_range": [["7.4.3", ""]],
                        "choices": [
                            "auto", "10full", "10half", "100full", "100half", "1000full", "10000full", "1000auto", "40000full", "1000fiber", "10000",
                            "40000", "auto-module", "100FX-half", "100FX-full", "100000full", "2500full", "25000full", "50000full", "40000auto",
                            "10000cr", "10000sr", "100000sr4", "100000cr4", "25000cr4", "25000sr4", "5000full", "2500auto", "5000auto", "1000full-fiber",
                            "40000sr4", "40000cr4", "25000cr", "25000sr", "50000cr", "50000sr"
                        ],
                        "type": "str"
                    },
                    "speed-mask": {"v_range": [["7.4.3", ""]], "type": "int"},
                    "stacking-port": {"v_range": [["7.4.3", ""]], "type": "int"},
                    "switch-id": {"v_range": [["7.4.3", ""]], "type": "str"},
                    "virtual-port": {"v_range": [["7.4.3", ""]], "type": "int"},
                    "export-tags": {"v_range": [["7.4.3", ""]], "type": "raw"},
                    "log-mac-event": {"v_range": [["7.6.0", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "pd-capable": {"v_range": [["7.6.0", ""]], "type": "int"},
                    "qnq": {"v_range": [["7.6.0", ""]], "type": "raw"}
                },
                "elements": "dict"
            },
            "switch-id": {"required": true, "type": "str"},
            "override-snmp-community": {"v_range": [["6.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "override-snmp-sysinfo": {"v_range": [["6.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "override-snmp-trap-threshold": {"v_range": [["6.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "override-snmp-user": {"v_range": [["6.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "poe-detection-type": {"v_range": [["6.2.0", ""]], "type": "int"},
            "remote-log": {
                "v_range": [["6.2.1", "6.2.3"], ["7.4.3", ""]],
                "type": "list",
                "options": {
                    "csv": {"v_range": [["6.2.1", "6.2.3"], ["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "facility": {
                        "v_range": [["6.2.1", "6.2.3"], ["7.4.3", ""]],
                        "choices": [
                            "kernel", "user", "mail", "daemon", "auth", "syslog", "lpr", "news", "uucp", "cron", "authpriv", "ftp", "ntp", "audit",
                            "alert", "clock", "local0", "local1", "local2", "local3", "local4", "local5", "local6", "local7"
                        ],
                        "type": "str"
                    },
                    "name": {"v_range": [["6.2.1", "6.2.3"], ["7.4.3", ""]], "type": "str"},
                    "port": {"v_range": [["6.2.1", "6.2.3"], ["7.4.3", ""]], "type": "int"},
                    "server": {"v_range": [["6.2.1", "6.2.3"], ["7.4.3", ""]], "type": "str"},
                    "severity": {
                        "v_range": [["6.2.1", "6.2.3"], ["7.4.3", ""]],
                        "choices": ["emergency", "alert", "critical", "error", "warning", "notification", "information", "debug"],
                        "type": "str"
                    },
                    "status": {"v_range": [["6.2.1", "6.2.3"], ["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"}
                },
                "elements": "dict"
            },
            "snmp-community": {
                "v_range": [["6.2.1", "6.2.3"], ["7.4.3", ""]],
                "type": "list",
                "options": {
                    "events": {
                        "v_range": [["6.2.1", "6.2.3"], ["7.4.3", ""]],
                        "type": "list",
                        "choices": ["cpu-high", "mem-low", "log-full", "intf-ip", "ent-conf-change", "l2mac"],
                        "elements": "str"
                    },
                    "hosts": {
                        "v_range": [["6.2.1", "6.2.3"], ["7.4.3", ""]],
                        "type": "list",
                        "options": {
                            "id": {"v_range": [["6.2.1", "6.2.3"], ["7.4.3", ""]], "type": "int"},
                            "ip": {"v_range": [["6.2.1", "6.2.3"], ["7.4.3", ""]], "type": "str"}
                        },
                        "elements": "dict"
                    },
                    "id": {"v_range": [["6.2.1", "6.2.3"], ["7.4.3", ""]], "type": "int"},
                    "name": {"v_range": [["6.2.1", "6.2.3"], ["7.4.3", ""]], "type": "str"},
                    "query-v1-port": {"v_range": [["6.2.1", "6.2.3"], ["7.4.3", ""]], "type": "int"},
                    "query-v1-status": {"v_range": [["6.2.1", "6.2.3"], ["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "query-v2c-port": {"v_range": [["6.2.1", "6.2.3"], ["7.4.3", ""]], "type": "int"},
                    "query-v2c-status": {"v_range": [["6.2.1", "6.2.3"], ["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "status": {"v_range": [["6.2.1", "6.2.3"], ["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "trap-v1-lport": {"v_range": [["6.2.1", "6.2.3"], ["7.4.3", ""]], "type": "int"},
                    "trap-v1-rport": {"v_range": [["6.2.1", "6.2.3"], ["7.4.3", ""]], "type": "int"},
                    "trap-v1-status": {"v_range": [["6.2.1", "6.2.3"], ["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "trap-v2c-lport": {"v_range": [["6.2.1", "6.2.3"], ["7.4.3", ""]], "type": "int"},
                    "trap-v2c-rport": {"v_range": [["6.2.1", "6.2.3"], ["7.4.3", ""]], "type": "int"},
                    "trap-v2c-status": {"v_range": [["6.2.1", "6.2.3"], ["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"}
                },
                "elements": "dict"
            },
            "snmp-user": {
                "v_range": [["6.2.1", "6.2.3"], ["7.4.3", ""]],
                "type": "list",
                "options": {
                    "auth-proto": {
                        "v_range": [["6.2.1", "6.2.3"], ["7.4.3", ""]],
                        "choices": ["md5", "sha", "sha1", "sha256", "sha384", "sha512", "sha224"],
                        "type": "str"
                    },
                    "auth-pwd": {"v_range": [["6.2.1", "6.2.3"], ["7.4.3", ""]], "type": "raw"},
                    "name": {"v_range": [["6.2.1", "6.2.3"], ["7.4.3", ""]], "type": "str"},
                    "priv-proto": {
                        "v_range": [["6.2.1", "6.2.3"], ["7.4.3", ""]],
                        "choices": ["des", "aes", "aes128", "aes192", "aes256", "aes192c", "aes256c"],
                        "type": "str"
                    },
                    "priv-pwd": {"v_range": [["6.2.1", "6.2.3"], ["7.4.3", ""]], "type": "raw"},
                    "queries": {"v_range": [["6.2.1", "6.2.3"], ["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "query-port": {"v_range": [["6.2.1", "6.2.3"], ["7.4.3", ""]], "type": "int"},
                    "security-level": {
                        "v_range": [["6.2.1", "6.2.3"], ["7.4.3", ""]],
                        "choices": ["no-auth-no-priv", "auth-no-priv", "auth-priv"],
                        "type": "str"
                    }
                },
                "elements": "dict"
            },
            "mclag-igmp-snooping-aware": {"v_range": [["6.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ip-source-guard": {
                "v_range": [["6.4.0", "6.4.1"], ["7.4.3", ""]],
                "type": "list",
                "options": {
                    "binding-entry": {
                        "v_range": [["6.4.0", "6.4.1"], ["7.4.3", ""]],
                        "type": "list",
                        "options": {
                            "entry-name": {"v_range": [["6.4.0", "6.4.1"], ["7.4.3", ""]], "type": "str"},
                            "ip": {"v_range": [["6.4.0", "6.4.1"], ["7.4.3", ""]], "type": "str"},
                            "mac": {"v_range": [["6.4.0", "6.4.1"], ["7.4.3", ""]], "type": "str"}
                        },
                        "elements": "dict"
                    },
                    "description": {"v_range": [["6.4.0", "6.4.1"], ["7.4.3", ""]], "type": "str"},
                    "port": {"v_range": [["6.4.0", "6.4.1"], ["7.4.3", ""]], "type": "str"}
                },
                "elements": "dict"
            },
            "l3-discovered": {"v_range": [["6.4.0", ""]], "type": "int"},
            "qos-drop-policy": {"v_range": [["6.4.0", ""]], "choices": ["taildrop", "random-early-detection"], "type": "str"},
            "qos-red-probability": {"v_range": [["6.4.0", ""]], "type": "int"},
            "switch-dhcp_opt43_key": {"v_range": [["6.4.0", ""]], "no_log": true, "type": "str"},
            "tdr-supported": {"v_range": [["6.4.3", ""]], "type": "str"},
            "custom-command": {
                "v_range": [["7.0.0", ""]],
                "type": "list",
                "options": {"command-entry": {"v_range": [["7.0.0", ""]], "type": "str"}, "command-name": {"v_range": [["7.0.0", ""]], "type": "str"}},
                "elements": "dict"
            },
            "firmware-provision": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "firmware-provision-version": {"v_range": [["7.0.0", ""]], "type": "str"},
            "dhcp-server-access-list": {"v_range": [["7.0.1", ""]], "choices": ["disable", "enable", "global"], "type": "str"},
            "firmware-provision-latest": {"v_range": [["7.0.3", ""]], "choices": ["disable", "once"], "type": "str"},
            "dhcp-snooping-static-client": {
                "v_range": [["7.2.2", ""]],
                "type": "list",
                "options": {
                    "ip": {"v_range": [["7.2.2", ""]], "type": "str"},
                    "mac": {"v_range": [["7.2.2", ""]], "type": "str"},
                    "name": {"v_range": [["7.2.2", ""]], "type": "str"},
                    "port": {"v_range": [["7.2.2", ""]], "type": "str"},
                    "vlan": {"v_range": [["7.2.2", ""]], "type": "str"}
                },
                "elements": "dict"
            },
            "ptp-profile": {"v_range": [["7.4.1", ""]], "type": "str"},
            "ptp-status": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "route-offload": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "route-offload-mclag": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "route-offload-router": {
                "v_range": [["7.4.1", ""]],
                "type": "list",
                "options": {"router-ip": {"v_range": [["7.4.1", ""]], "type": "str"}, "vlan-name": {"v_range": [["7.4.1", ""]], "type": "str"}},
                "elements": "dict"
            },
            "mgmt-mode": {"v_range": [["7.4.2", ""]], "type": "int"},
            "purdue-level": {"v_range": [["7.4.2", ""]], "choices": ["1", "2", "3", "4", "5", "1.5", "2.5", "3.5", "5.5"], "type": "str"},
            "radius-nas-ip": {"v_range": [["7.4.2", ""]], "type": "str"},
            "radius-nas-ip-override": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "tunnel-discovered": {"v_range": [["7.4.2", ""]], "type": "int"},
            "vlan": {
                "v_range": [["7.4.2", ""]],
                "type": "list",
                "options": {"assignment-priority": {"v_range": [["7.4.2", ""]], "type": "int"}, "vlan-name": {"v_range": [["7.4.2", ""]], "type": "str"}},
                "elements": "dict"
            },
            "802-1X-settings": {
                "v_range": [["7.4.3", ""]],
                "type": "dict",
                "options": {
                    "link-down-auth": {"v_range": [["7.4.3", ""]], "choices": ["set-unauth", "no-action"], "type": "str"},
                    "local-override": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "mab-reauth": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "mac-called-station-delimiter": {"v_range": [["7.4.3", ""]], "choices": ["hyphen", "single-hyphen", "colon", "none"], "type": "str"},
                    "mac-calling-station-delimiter": {"v_range": [["7.4.3", ""]], "choices": ["hyphen", "single-hyphen", "colon", "none"], "type": "str"},
                    "mac-case": {"v_range": [["7.4.3", ""]], "choices": ["uppercase", "lowercase"], "type": "str"},
                    "mac-password-delimiter": {"v_range": [["7.4.3", ""]], "choices": ["hyphen", "single-hyphen", "colon", "none"], "type": "str"},
                    "mac-username-delimiter": {"v_range": [["7.4.3", ""]], "choices": ["hyphen", "single-hyphen", "colon", "none"], "type": "str"},
                    "max-reauth-attempt": {"v_range": [["7.4.3", ""]], "type": "int"},
                    "reauth-period": {"v_range": [["7.4.3", ""]], "type": "int"},
                    "tx-period": {"v_range": [["7.4.3", ""]], "type": "int"}
                }
            },
            "access-profile": {"v_range": [["7.4.3", ""]], "type": "raw"},
            "delayed-restart-trigger": {"v_range": [["7.4.3", ""]], "type": "int"},
            "directly-connected": {"v_range": [["7.4.3", ""]], "type": "int"},
            "dynamic-capability": {"v_range": [["7.4.3", ""]], "type": "str"},
            "dynamically-discovered": {"v_range": [["7.4.3", ""]], "type": "int"},
            "flow-identity": {"v_range": [["7.4.3", ""]], "type": "str"},
            "fsw-wan1-admin": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable", "discovered"], "type": "str"},
            "fsw-wan1-peer": {"v_range": [["7.4.3", ""]], "type": "raw"},
            "fsw-wan2-admin": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable", "discovered"], "type": "str"},
            "fsw-wan2-peer": {"v_range": [["7.4.3", ""]], "type": "str"},
            "igmp-snooping": {
                "v_range": [["7.4.3", ""]],
                "type": "dict",
                "options": {
                    "aging-time": {"v_range": [["7.4.3", ""]], "type": "int"},
                    "flood-unknown-multicast": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "local-override": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "vlans": {
                        "v_range": [["7.4.3", ""]],
                        "type": "list",
                        "options": {
                            "proxy": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable", "global"], "type": "str"},
                            "querier": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "querier-addr": {"v_range": [["7.4.3", ""]], "type": "str"},
                            "version": {"v_range": [["7.4.3", ""]], "type": "int"},
                            "vlan-name": {"v_range": [["7.4.3", ""]], "type": "raw"}
                        },
                        "elements": "dict"
                    }
                }
            },
            "max-allowed-trunk-members": {"v_range": [["7.4.3", ""]], "type": "int"},
            "mirror": {
                "v_range": [["7.4.3", ""]],
                "type": "list",
                "options": {
                    "dst": {"v_range": [["7.4.3", ""]], "type": "str"},
                    "name": {"v_range": [["7.4.3", ""]], "type": "str"},
                    "src-egress": {"v_range": [["7.4.3", ""]], "type": "raw"},
                    "src-ingress": {"v_range": [["7.4.3", ""]], "type": "raw"},
                    "status": {"v_range": [["7.4.3", ""]], "choices": ["inactive", "active"], "type": "str"},
                    "switching-packet": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"}
                },
                "elements": "dict"
            },
            "owner-vdom": {"v_range": [["7.4.3", ""]], "type": "str"},
            "poe-pre-standard-detection": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "pre-provisioned": {"v_range": [["7.4.3", ""]], "type": "int"},
            "sn": {"v_range": [["7.4.3", ""]], "type": "str"},
            "snmp-sysinfo": {
                "v_range": [["7.4.3", ""]],
                "type": "dict",
                "options": {
                    "contact-info": {"v_range": [["7.4.3", ""]], "type": "str"},
                    "description": {"v_range": [["7.4.3", ""]], "type": "str"},
                    "engine-id": {"v_range": [["7.4.3", ""]], "type": "str"},
                    "location": {"v_range": [["7.4.3", ""]], "type": "str"},
                    "status": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"}
                }
            },
            "snmp-trap-threshold": {
                "v_range": [["7.4.3", ""]],
                "type": "dict",
                "options": {
                    "trap-high-cpu-threshold": {"v_range": [["7.4.3", ""]], "type": "int"},
                    "trap-log-full-threshold": {"v_range": [["7.4.3", ""]], "type": "int"},
                    "trap-low-memory-threshold": {"v_range": [["7.4.3", ""]], "type": "int"}
                }
            },
            "staged-image-version": {"v_range": [["7.4.3", ""]], "type": "str"},
            "static-mac": {
                "v_range": [["7.4.3", ""]],
                "type": "list",
                "options": {
                    "description": {"v_range": [["7.4.3", ""]], "type": "str"},
                    "id": {"v_range": [["7.4.3", ""]], "type": "int"},
                    "interface": {"v_range": [["7.4.3", ""]], "type": "str"},
                    "mac": {"v_range": [["7.4.3", ""]], "type": "str"},
                    "type": {"v_range": [["7.4.3", ""]], "choices": ["static", "sticky"], "type": "str"},
                    "vlan": {"v_range": [["7.4.3", ""]], "type": "raw"}
                },
                "elements": "dict"
            },
            "storm-control": {
                "v_range": [["7.4.3", ""]],
                "type": "dict",
                "options": {
                    "broadcast": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "local-override": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "rate": {"v_range": [["7.4.3", ""]], "type": "int"},
                    "unknown-multicast": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "unknown-unicast": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"}
                }
            },
            "stp-instance": {
                "v_range": [["7.4.3", ""]],
                "type": "list",
                "options": {
                    "id": {"v_range": [["7.4.3", ""]], "type": "str"},
                    "priority": {
                        "v_range": [["7.4.3", ""]],
                        "choices": [
                            "0", "4096", "8192", "12288", "12328", "16384", "20480", "24576", "28672", "32768", "36864", "40960", "45056", "49152",
                            "53248", "57344", "61440"
                        ],
                        "type": "str"
                    }
                },
                "elements": "dict"
            },
            "stp-settings": {
                "v_range": [["7.4.3", ""]],
                "type": "dict",
                "options": {
                    "forward-time": {"v_range": [["7.4.3", ""]], "type": "int"},
                    "hello-time": {"v_range": [["7.4.3", ""]], "type": "int"},
                    "local-override": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "max-age": {"v_range": [["7.4.3", ""]], "type": "int"},
                    "max-hops": {"v_range": [["7.4.3", ""]], "type": "int"},
                    "name": {"v_range": [["7.4.3", ""]], "type": "str"},
                    "pending-timer": {"v_range": [["7.4.3", ""]], "type": "int"},
                    "revision": {"v_range": [["7.4.3", ""]], "type": "int"},
                    "status": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"}
                }
            },
            "switch-device-tag": {"v_range": [["7.4.3", ""]], "type": "str"},
            "switch-log": {
                "v_range": [["7.4.3", ""]],
                "type": "dict",
                "options": {
                    "local-override": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "severity": {
                        "v_range": [["7.4.3", ""]],
                        "choices": ["emergency", "alert", "critical", "error", "warning", "notification", "information", "debug"],
                        "type": "str"
                    },
                    "status": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"}
                }
            },
            "switch-profile": {"v_range": [["7.4.3", ""]], "type": "raw"},
            "type": {"v_range": [["7.4.3", ""]], "choices": ["physical", "virtual"], "type": "str"},
            "version": {"v_range": [["7.4.3", ""]], "type": "int"},
            "poe-lldp-detection": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "switchcontroller_managedswitch_customcommand": {
        "stated": true,
        "options": {"command-entry": {"v_range": [["7.0.0", ""]], "type": "str"}, "command-name": {"v_range": [["7.0.0", ""]], "type": "str"}}
    },
    "switchcontroller_managedswitch_dhcpsnoopingstaticclient": {
        "stated": true,
        "options": {
            "ip": {"v_range": [["7.2.2", ""]], "type": "str"},
            "mac": {"v_range": [["7.2.2", ""]], "type": "str"},
            "name": {"v_range": [["7.2.2", ""]], "required": true, "type": "str"},
            "port": {"v_range": [["7.2.2", ""]], "type": "str"},
            "vlan": {"v_range": [["7.2.2", ""]], "type": "str"}
        }
    },
    "switchcontroller_managedswitch_ipsourceguard": {
        "stated": true,
        "options": {
            "binding-entry": {
                "v_range": [["6.4.0", "6.4.1"]],
                "type": "list",
                "options": {
                    "entry-name": {"v_range": [["6.4.0", "6.4.1"]], "type": "str"},
                    "ip": {"v_range": [["6.4.0", "6.4.1"]], "type": "str"},
                    "mac": {"v_range": [["6.4.0", "6.4.1"]], "type": "str"}
                },
                "elements": "dict"
            },
            "description": {"v_range": [["6.4.0", "6.4.1"]], "type": "str"},
            "port": {"v_range": [["6.4.0", "6.4.1"]], "type": "str"}
        }
    },
    "switchcontroller_managedswitch_ipsourceguard_bindingentry": {
        "stated": true,
        "options": {
            "entry-name": {"v_range": [["6.4.0", "6.4.1"]], "type": "str"},
            "ip": {"v_range": [["6.4.0", "6.4.1"]], "type": "str"},
            "mac": {"v_range": [["6.4.0", "6.4.1"]], "type": "str"}
        }
    },
    "switchcontroller_managedswitch_ports": {
        "stated": true,
        "options": {
            "allowed-vlans": {"type": "raw"},
            "allowed-vlans-all": {"choices": ["disable", "enable"], "type": "str"},
            "arp-inspection-trust": {"choices": ["untrusted", "trusted"], "type": "str"},
            "bundle": {"choices": ["disable", "enable"], "type": "str"},
            "description": {"type": "str"},
            "dhcp-snoop-option82-trust": {"choices": ["disable", "enable"], "type": "str"},
            "dhcp-snooping": {"choices": ["trusted", "untrusted"], "type": "str"},
            "discard-mode": {"choices": ["none", "all-untagged", "all-tagged"], "type": "str"},
            "edge-port": {"choices": ["disable", "enable"], "type": "str"},
            "igmp-snooping": {"choices": ["disable", "enable"], "type": "str"},
            "igmps-flood-reports": {"choices": ["disable", "enable"], "type": "str"},
            "igmps-flood-traffic": {"choices": ["disable", "enable"], "type": "str"},
            "lacp-speed": {"choices": ["slow", "fast"], "type": "str"},
            "learning-limit": {"type": "int"},
            "lldp-profile": {"type": "str"},
            "lldp-status": {"choices": ["disable", "rx-only", "tx-only", "tx-rx"], "type": "str"},
            "loop-guard": {"choices": ["disabled", "enabled"], "type": "str"},
            "loop-guard-timeout": {"type": "int"},
            "max-bundle": {"type": "int"},
            "mclag": {"choices": ["disable", "enable"], "type": "str"},
            "member-withdrawal-behavior": {"choices": ["forward", "block"], "type": "str"},
            "members": {"type": "raw"},
            "min-bundle": {"type": "int"},
            "mode": {"choices": ["static", "lacp-passive", "lacp-active"], "type": "str"},
            "poe-pre-standard-detection": {"choices": ["disable", "enable"], "type": "str"},
            "poe-status": {"choices": ["disable", "enable"], "type": "str"},
            "port-name": {"required": true, "type": "str"},
            "port-owner": {"type": "str"},
            "port-security-policy": {"type": "str"},
            "port-selection-criteria": {"choices": ["src-mac", "dst-mac", "src-dst-mac", "src-ip", "dst-ip", "src-dst-ip"], "type": "str"},
            "qos-policy": {"type": "str"},
            "sample-direction": {"choices": ["rx", "tx", "both"], "type": "str"},
            "sflow-counter-interval": {"type": "int"},
            "sflow-sample-rate": {"v_range": [["6.0.0", "7.2.1"]], "type": "int"},
            "sflow-sampler": {"v_range": [["6.0.0", "7.2.1"]], "choices": ["disabled", "enabled"], "type": "str"},
            "stp-bpdu-guard": {"choices": ["disabled", "enabled"], "type": "str"},
            "stp-bpdu-guard-timeout": {"type": "int"},
            "stp-root-guard": {"choices": ["disabled", "enabled"], "type": "str"},
            "stp-state": {"choices": ["disabled", "enabled"], "type": "str"},
            "type": {"choices": ["physical", "trunk"], "type": "str"},
            "untagged-vlans": {"type": "raw"},
            "vlan": {"type": "str"},
            "export-to-pool-flag": {"v_range": [["6.2.1", "6.2.3"], ["7.4.3", ""]], "type": "int"},
            "mac-addr": {"v_range": [["6.2.1", "6.2.1"], ["7.4.3", ""]], "type": "str"},
            "packet-sample-rate": {"v_range": [["6.2.0", ""]], "type": "int"},
            "packet-sampler": {"v_range": [["6.2.0", ""]], "choices": ["disabled", "enabled"], "type": "str"},
            "sticky-mac": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "storm-control-policy": {"v_range": [["6.2.0", "6.2.3"], ["7.4.3", ""]], "type": "str"},
            "dot1x-enable": {"v_range": [["6.2.0", "6.2.12"]], "choices": ["disable", "enable"], "type": "str"},
            "max-miss-heartbeats": {"v_range": [["6.2.0", "6.2.12"]], "type": "int"},
            "access-mode": {"v_range": [["6.4.0", ""]], "choices": ["normal", "nac", "dynamic", "static"], "type": "str"},
            "ip-source-guard": {"v_range": [["6.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "mclag-icl-port": {"v_range": [["6.4.0", ""]], "type": "int"},
            "p2p-port": {"v_range": [["6.4.0", ""]], "type": "int"},
            "aggregator-mode": {"v_range": [["6.4.2", ""]], "choices": ["bandwidth", "count"], "type": "str"},
            "rpvst-port": {"v_range": [["6.4.2", ""]], "choices": ["disabled", "enabled"], "type": "str"},
            "flow-control": {"v_range": [["6.4.3", ""]], "choices": ["disable", "tx", "rx", "both"], "type": "str"},
            "media-type": {"v_range": [["6.4.3", ""]], "type": "str"},
            "pause-meter": {"v_range": [["6.4.3", ""]], "type": "int"},
            "pause-meter-resume": {"v_range": [["6.4.3", ""]], "choices": ["25%", "50%", "75%"], "type": "str"},
            "trunk-member": {"v_range": [["6.2.7", "6.2.12"], ["6.4.3", ""]], "type": "int"},
            "fec-capable": {"v_range": [["7.0.0", ""]], "type": "int"},
            "fec-state": {"v_range": [["7.0.0", ""]], "choices": ["disabled", "cl74", "cl91", "detect-by-module"], "type": "str"},
            "matched-dpp-intf-tags": {"v_range": [["7.0.0", ""]], "type": "str"},
            "matched-dpp-policy": {"v_range": [["7.0.0", ""]], "type": "str"},
            "port-policy": {"v_range": [["7.0.0", "7.0.4"], ["7.2.0", ""]], "type": "str"},
            "status": {"v_range": [["6.4.6", ""]], "choices": ["down", "up"], "type": "str"},
            "dsl-profile": {"v_range": [["7.0.2", ""]], "type": "str"},
            "flap-duration": {"v_range": [["7.2.0", ""]], "type": "int"},
            "flap-rate": {"v_range": [["7.2.0", ""]], "type": "int"},
            "flap-timeout": {"v_range": [["7.2.0", ""]], "type": "int"},
            "flapguard": {"v_range": [["7.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "interface-tags": {"v_range": [["7.0.2", ""]], "type": "raw"},
            "poe-max-power": {"v_range": [["7.0.1", ""]], "type": "str"},
            "poe-standard": {"v_range": [["7.0.1", ""]], "type": "str"},
            "igmp-snooping-flood-reports": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "mcast-snooping-flood-traffic": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "link-status": {"v_range": [["7.2.2", ""]], "choices": ["down", "up"], "type": "str"},
            "poe-mode-bt-cabable": {"v_range": [["7.2.2", ""]], "type": "int"},
            "poe-port-mode": {"v_range": [["7.2.2", ""]], "choices": ["ieee802-3af", "ieee802-3at", "ieee802-3bt"], "type": "str"},
            "poe-port-power": {"v_range": [["7.2.2", ""]], "choices": ["normal", "perpetual", "perpetual-fast"], "type": "str"},
            "poe-port-priority": {
                "v_range": [["7.2.2", ""]],
                "choices": ["critical-priority", "high-priority", "low-priority", "medium-priority"],
                "type": "str"
            },
            "acl-group": {"v_range": [["7.4.0", ""]], "type": "raw"},
            "dhcp-snoop-option82-override": {
                "v_range": [["7.4.0", ""]],
                "type": "list",
                "options": {
                    "circuit-id": {"v_range": [["7.4.0", ""]], "type": "str"},
                    "remote-id": {"v_range": [["7.4.0", ""]], "type": "str"},
                    "vlan-name": {"v_range": [["7.4.0", ""]], "type": "str"}
                },
                "elements": "dict"
            },
            "fortiswitch-acls": {"v_range": [["7.4.0", ""]], "type": "raw"},
            "isl-peer-device-sn": {"v_range": [["7.4.0", ""]], "type": "str"},
            "authenticated-port": {"v_range": [["7.4.1", ""]], "type": "int"},
            "encrypted-port": {"v_range": [["7.4.1", ""]], "type": "int"},
            "ptp-status": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "restricted-auth-port": {"v_range": [["7.4.1", ""]], "type": "int"},
            "allow-arp-monitor": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "export-to": {"v_range": [["7.4.3", ""]], "type": "raw"},
            "export-to-pool": {"v_range": [["7.4.3", ""]], "type": "raw"},
            "fallback-port": {"v_range": [["7.4.3", ""]], "type": "str"},
            "fgt-peer-device-name": {"v_range": [["7.4.3", ""]], "type": "str"},
            "fgt-peer-port-name": {"v_range": [["7.4.3", ""]], "type": "str"},
            "fiber-port": {"v_range": [["7.4.3", ""]], "type": "int"},
            "flags": {"v_range": [["7.4.3", ""]], "type": "int"},
            "fortilink-port": {"v_range": [["7.4.3", ""]], "type": "int"},
            "isl-local-trunk-name": {"v_range": [["7.4.3", ""]], "type": "str"},
            "isl-peer-device-name": {"v_range": [["7.4.3", ""]], "type": "str"},
            "isl-peer-port-name": {"v_range": [["7.4.3", ""]], "type": "str"},
            "poe-capable": {"v_range": [["7.4.3", ""]], "type": "int"},
            "port-number": {"v_range": [["7.4.3", ""]], "type": "int"},
            "port-prefix-type": {"v_range": [["7.4.3", ""]], "type": "int"},
            "ptp-policy": {"v_range": [["7.4.3", ""]], "type": "raw"},
            "speed": {
                "v_range": [["7.4.3", ""]],
                "choices": [
                    "auto", "10full", "10half", "100full", "100half", "1000full", "10000full", "1000auto", "40000full", "1000fiber", "10000", "40000",
                    "auto-module", "100FX-half", "100FX-full", "100000full", "2500full", "25000full", "50000full", "40000auto", "10000cr", "10000sr",
                    "100000sr4", "100000cr4", "25000cr4", "25000sr4", "5000full", "2500auto", "5000auto", "1000full-fiber", "40000sr4", "40000cr4",
                    "25000cr", "25000sr", "50000cr", "50000sr"
                ],
                "type": "str"
            },
            "speed-mask": {"v_range": [["7.4.3", ""]], "type": "int"},
            "stacking-port": {"v_range": [["7.4.3", ""]], "type": "int"},
            "switch-id": {"v_range": [["7.4.3", ""]], "type": "str"},
            "virtual-port": {"v_range": [["7.4.3", ""]], "type": "int"},
            "export-tags": {"v_range": [["7.4.3", ""]], "type": "raw"},
            "log-mac-event": {"v_range": [["7.6.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "pd-capable": {"v_range": [["7.6.0", ""]], "type": "int"},
            "qnq": {"v_range": [["7.6.0", ""]], "type": "raw"}
        }
    },
    "switchcontroller_managedswitch_ports_dhcpsnoopoption82override": {
        "stated": true,
        "options": {
            "circuit-id": {"v_range": [["7.4.0", ""]], "type": "str"},
            "remote-id": {"v_range": [["7.4.0", ""]], "type": "str"},
            "vlan-name": {"v_range": [["7.4.0", ""]], "type": "str"}
        }
    },
    "switchcontroller_managedswitch_remotelog": {
        "stated": true,
        "options": {
            "csv": {"v_range": [["6.2.1", "6.2.3"]], "choices": ["disable", "enable"], "type": "str"},
            "facility": {
                "v_range": [["6.2.1", "6.2.3"]],
                "choices": [
                    "kernel", "user", "mail", "daemon", "auth", "syslog", "lpr", "news", "uucp", "cron", "authpriv", "ftp", "ntp", "audit", "alert",
                    "clock", "local0", "local1", "local2", "local3", "local4", "local5", "local6", "local7"
                ],
                "type": "str"
            },
            "name": {"v_range": [["6.2.1", "6.2.3"]], "required": true, "type": "str"},
            "port": {"v_range": [["6.2.1", "6.2.3"]], "type": "int"},
            "server": {"v_range": [["6.2.1", "6.2.3"]], "type": "str"},
            "severity": {
                "v_range": [["6.2.1", "6.2.3"]],
                "choices": ["emergency", "alert", "critical", "error", "warning", "notification", "information", "debug"],
                "type": "str"
            },
            "status": {"v_range": [["6.2.1", "6.2.3"]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "switchcontroller_managedswitch_routeoffloadrouter": {
        "stated": true,
        "options": {"router-ip": {"v_range": [["7.4.1", ""]], "type": "str"}, "vlan-name": {"v_range": [["7.4.1", ""]], "type": "str"}}
    },
    "switchcontroller_managedswitch_snmpcommunity": {
        "stated": true,
        "options": {
            "events": {
                "v_range": [["6.2.1", "6.2.3"]],
                "type": "list",
                "choices": ["cpu-high", "mem-low", "log-full", "intf-ip", "ent-conf-change"],
                "elements": "str"
            },
            "hosts": {
                "v_range": [["6.2.1", "6.2.3"]],
                "type": "list",
                "options": {"id": {"v_range": [["6.2.1", "6.2.3"]], "type": "int"}, "ip": {"v_range": [["6.2.1", "6.2.3"]], "type": "str"}},
                "elements": "dict"
            },
            "id": {"v_range": [["6.2.1", "6.2.3"]], "required": true, "type": "int"},
            "name": {"v_range": [["6.2.1", "6.2.3"]], "type": "str"},
            "query-v1-port": {"v_range": [["6.2.1", "6.2.3"]], "type": "int"},
            "query-v1-status": {"v_range": [["6.2.1", "6.2.3"]], "choices": ["disable", "enable"], "type": "str"},
            "query-v2c-port": {"v_range": [["6.2.1", "6.2.3"]], "type": "int"},
            "query-v2c-status": {"v_range": [["6.2.1", "6.2.3"]], "choices": ["disable", "enable"], "type": "str"},
            "status": {"v_range": [["6.2.1", "6.2.3"]], "choices": ["disable", "enable"], "type": "str"},
            "trap-v1-lport": {"v_range": [["6.2.1", "6.2.3"]], "type": "int"},
            "trap-v1-rport": {"v_range": [["6.2.1", "6.2.3"]], "type": "int"},
            "trap-v1-status": {"v_range": [["6.2.1", "6.2.3"]], "choices": ["disable", "enable"], "type": "str"},
            "trap-v2c-lport": {"v_range": [["6.2.1", "6.2.3"]], "type": "int"},
            "trap-v2c-rport": {"v_range": [["6.2.1", "6.2.3"]], "type": "int"},
            "trap-v2c-status": {"v_range": [["6.2.1", "6.2.3"]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "switchcontroller_managedswitch_snmpcommunity_hosts": {
        "stated": true,
        "options": {"id": {"v_range": [["6.2.1", "6.2.3"]], "required": true, "type": "int"}, "ip": {"v_range": [["6.2.1", "6.2.3"]], "type": "str"}}
    },
    "switchcontroller_managedswitch_snmpsysinfo": {
        "stated": false,
        "options": {
            "contact-info": {"v_range": [["6.2.1", "6.2.3"]], "type": "str"},
            "description": {"v_range": [["6.2.1", "6.2.3"]], "type": "str"},
            "engine-id": {"v_range": [["6.2.1", "6.2.3"]], "type": "str"},
            "location": {"v_range": [["6.2.1", "6.2.3"]], "type": "str"},
            "status": {"v_range": [["6.2.1", "6.2.3"]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "switchcontroller_managedswitch_snmptrapthreshold": {
        "stated": false,
        "options": {
            "trap-high-cpu-threshold": {"v_range": [["6.2.1", "6.2.3"]], "type": "int"},
            "trap-log-full-threshold": {"v_range": [["6.2.1", "6.2.3"]], "type": "int"},
            "trap-low-memory-threshold": {"v_range": [["6.2.1", "6.2.3"]], "type": "int"}
        }
    },
    "switchcontroller_managedswitch_snmpuser": {
        "stated": true,
        "options": {
            "auth-proto": {"v_range": [["6.2.1", "6.2.3"]], "choices": ["md5", "sha"], "type": "str"},
            "auth-pwd": {"v_range": [["6.2.1", "6.2.3"]], "type": "raw"},
            "name": {"v_range": [["6.2.1", "6.2.3"]], "required": true, "type": "str"},
            "priv-proto": {"v_range": [["6.2.1", "6.2.3"]], "choices": ["des", "aes"], "type": "str"},
            "priv-pwd": {"v_range": [["6.2.1", "6.2.3"]], "type": "raw"},
            "queries": {"v_range": [["6.2.1", "6.2.3"]], "choices": ["disable", "enable"], "type": "str"},
            "query-port": {"v_range": [["6.2.1", "6.2.3"]], "type": "int"},
            "security-level": {"v_range": [["6.2.1", "6.2.3"]], "choices": ["no-auth-no-priv", "auth-no-priv", "auth-priv"], "type": "str"}
        }
    },
    "switchcontroller_managedswitch_vlan": {
        "stated": true,
        "options": {"assignment-priority": {"v_range": [["7.4.2", ""]], "type": "int"}, "vlan-name": {"v_range": [["7.4.2", ""]], "type": "str"}}
    },
    "switchcontroller_ptp_profile": {
        "stated": true,
        "options": {
            "description": {"v_range": [["7.4.1", ""]], "type": "str"},
            "domain": {"v_range": [["7.4.1", ""]], "type": "int"},
            "mode": {"v_range": [["7.4.1", ""]], "choices": ["transparent-e2e", "transparent-p2p"], "type": "str"},
            "name": {"v_range": [["7.4.1", ""]], "required": true, "type": "str"},
            "pdelay-req-interval": {"v_range": [["7.4.1", ""]], "choices": ["1sec", "2sec", "4sec", "8sec", "16sec", "32sec"], "type": "str"},
            "ptp-profile": {"v_range": [["7.4.1", ""]], "choices": ["C37.238-2017"], "type": "str"},
            "transport": {"v_range": [["7.4.1", ""]], "choices": ["l2-mcast"], "type": "str"}
        }
    },
    "switchcontroller_qos_dot1pmap": {
        "stated": true,
        "options": {
            "description": {"type": "str"},
            "name": {"required": true, "type": "str"},
            "priority-0": {"choices": ["queue-0", "queue-1", "queue-2", "queue-3", "queue-4", "queue-5", "queue-6", "queue-7"], "type": "str"},
            "priority-1": {"choices": ["queue-0", "queue-1", "queue-2", "queue-3", "queue-4", "queue-5", "queue-6", "queue-7"], "type": "str"},
            "priority-2": {"choices": ["queue-0", "queue-1", "queue-2", "queue-3", "queue-4", "queue-5", "queue-6", "queue-7"], "type": "str"},
            "priority-3": {"choices": ["queue-0", "queue-1", "queue-2", "queue-3", "queue-4", "queue-5", "queue-6", "queue-7"], "type": "str"},
            "priority-4": {"choices": ["queue-0", "queue-1", "queue-2", "queue-3", "queue-4", "queue-5", "queue-6", "queue-7"], "type": "str"},
            "priority-5": {"choices": ["queue-0", "queue-1", "queue-2", "queue-3", "queue-4", "queue-5", "queue-6", "queue-7"], "type": "str"},
            "priority-6": {"choices": ["queue-0", "queue-1", "queue-2", "queue-3", "queue-4", "queue-5", "queue-6", "queue-7"], "type": "str"},
            "priority-7": {"choices": ["queue-0", "queue-1", "queue-2", "queue-3", "queue-4", "queue-5", "queue-6", "queue-7"], "type": "str"},
            "egress-pri-tagging": {"v_range": [["6.2.3", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "switchcontroller_qos_ipdscpmap": {
        "stated": true,
        "options": {
            "description": {"type": "str"},
            "map": {
                "type": "list",
                "options": {
                    "cos-queue": {"type": "int"},
                    "diffserv": {
                        "type": "list",
                        "choices": [
                            "CS0", "CS1", "AF11", "AF12", "AF13", "CS2", "AF21", "AF22", "AF23", "CS3", "AF31", "AF32", "AF33", "CS4", "AF41", "AF42",
                            "AF43", "CS5", "EF", "CS6", "CS7"
                        ],
                        "elements": "str"
                    },
                    "ip-precedence": {
                        "type": "list",
                        "choices": ["network-control", "internetwork-control", "critic-ecp", "flashoverride", "flash", "immediate", "priority", "routine"],
                        "elements": "str"
                    },
                    "name": {"type": "str"},
                    "value": {"type": "str"}
                },
                "elements": "dict"
            },
            "name": {"required": true, "type": "str"}
        }
    },
    "switchcontroller_qos_ipdscpmap_map": {
        "stated": true,
        "options": {
            "cos-queue": {"type": "int"},
            "diffserv": {
                "type": "list",
                "choices": [
                    "CS0", "CS1", "AF11", "AF12", "AF13", "CS2", "AF21", "AF22", "AF23", "CS3", "AF31", "AF32", "AF33", "CS4", "AF41", "AF42", "AF43",
                    "CS5", "EF", "CS6", "CS7"
                ],
                "elements": "str"
            },
            "ip-precedence": {
                "type": "list",
                "choices": ["network-control", "internetwork-control", "critic-ecp", "flashoverride", "flash", "immediate", "priority", "routine"],
                "elements": "str"
            },
            "name": {"required": true, "type": "str"},
            "value": {"type": "str"}
        }
    },
    "switchcontroller_qos_qospolicy": {
        "stated": true,
        "options": {
            "default-cos": {"type": "int"},
            "name": {"required": true, "type": "str"},
            "queue-policy": {"type": "str"},
            "trust-dot1p-map": {"type": "str"},
            "trust-ip-dscp-map": {"type": "str"}
        }
    },
    "switchcontroller_qos_queuepolicy": {
        "stated": true,
        "options": {
            "cos-queue": {
                "type": "list",
                "options": {
                    "description": {"type": "str"},
                    "drop-policy": {"choices": ["taildrop", "weighted-random-early-detection"], "type": "str"},
                    "max-rate": {"type": "int"},
                    "min-rate": {"type": "int"},
                    "name": {"type": "str"},
                    "weight": {"type": "int"},
                    "max-rate-percent": {"v_range": [["6.2.0", ""]], "type": "int"},
                    "min-rate-percent": {"v_range": [["6.2.0", ""]], "type": "int"},
                    "ecn": {"v_range": [["6.4.2", ""]], "choices": ["disable", "enable"], "type": "str"}
                },
                "elements": "dict"
            },
            "name": {"required": true, "type": "str"},
            "schedule": {"choices": ["strict", "round-robin", "weighted"], "type": "str"},
            "rate-by": {"v_range": [["6.2.0", ""]], "choices": ["kbps", "percent"], "type": "str"}
        }
    },
    "switchcontroller_qos_queuepolicy_cosqueue": {
        "stated": true,
        "options": {
            "description": {"type": "str"},
            "drop-policy": {"choices": ["taildrop", "weighted-random-early-detection"], "type": "str"},
            "max-rate": {"type": "int"},
            "min-rate": {"type": "int"},
            "name": {"required": true, "type": "str"},
            "weight": {"type": "int"},
            "max-rate-percent": {"v_range": [["6.2.0", ""]], "type": "int"},
            "min-rate-percent": {"v_range": [["6.2.0", ""]], "type": "int"},
            "ecn": {"v_range": [["6.4.2", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "switchcontroller_securitypolicy_8021x": {
        "stated": true,
        "options": {
            "auth-fail-vlan": {"choices": ["disable", "enable"], "type": "str"},
            "auth-fail-vlan-id": {"type": "str"},
            "auth-fail-vlanid": {"type": "int"},
            "eap-passthru": {"choices": ["disable", "enable"], "type": "str"},
            "guest-auth-delay": {"type": "int"},
            "guest-vlan": {"choices": ["disable", "enable"], "type": "str"},
            "guest-vlan-id": {"type": "str"},
            "guest-vlanid": {"type": "int"},
            "mac-auth-bypass": {"choices": ["disable", "enable"], "type": "str"},
            "name": {"required": true, "type": "str"},
            "open-auth": {"choices": ["disable", "enable"], "type": "str"},
            "policy-type": {"choices": ["802.1X"], "type": "str"},
            "radius-timeout-overwrite": {"choices": ["disable", "enable"], "type": "str"},
            "security-mode": {"choices": ["802.1X", "802.1X-mac-based"], "type": "str"},
            "user-group": {"type": "raw"},
            "framevid-apply": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "eap-auto-untagged-vlans": {"v_range": [["6.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "authserver-timeout-period": {"v_range": [["6.4.3", ""]], "type": "int"},
            "authserver-timeout-vlan": {"v_range": [["6.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "authserver-timeout-vlanid": {"v_range": [["6.4.3", ""]], "type": "str"},
            "authserver-timeout-tagged": {"v_range": [["7.4.3", ""]], "choices": ["static", "disable", "lldp-voice"], "type": "str"},
            "authserver-timeout-tagged-vlanid": {"v_range": [["7.4.3", ""]], "type": "raw"},
            "dacl": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "auth-order": {"v_range": [["7.6.0", ""]], "choices": ["dot1x-mab", "mab-dot1x", "mab"], "type": "str"},
            "auth-priority": {"v_range": [["7.6.0", ""]], "choices": ["dot1x-mab", "mab-dot1x", "legacy"], "type": "str"}
        }
    },
    "switchcontroller_securitypolicy_captiveportal": {
        "stated": true,
        "options": {
            "name": {"v_range": [["6.0.0", "6.2.1"]], "required": true, "type": "str"},
            "policy-type": {"v_range": [["6.0.0", "6.2.1"]], "choices": ["captive-portal"], "type": "str"},
            "vlan": {"v_range": [["6.0.0", "6.2.1"]], "type": "str"}
        }
    },
    "switchcontroller_switchinterfacetag": {"stated": true, "options": {"name": {"v_range": [["7.2.1", ""]], "required": true, "type": "str"}}},
    "switchcontroller_trafficpolicy": {
        "stated": true,
        "options": {
            "cos-queue": {"v_range": [["7.2.1", ""]], "type": "int"},
            "description": {"v_range": [["7.2.1", ""]], "type": "str"},
            "guaranteed-bandwidth": {"v_range": [["7.2.1", ""]], "type": "int"},
            "guaranteed-burst": {"v_range": [["7.2.1", ""]], "type": "int"},
            "id": {"v_range": [["7.2.1", ""]], "required": true, "type": "int"},
            "maximum-burst": {"v_range": [["7.2.1", ""]], "type": "int"},
            "name": {"v_range": [["7.2.1", ""]], "type": "str"},
            "policer-status": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "type": {"v_range": [["7.2.1", ""]], "choices": ["ingress", "egress"], "type": "str"}
        }
    },
    "switchcontroller_vlanpolicy": {
        "stated": true,
        "options": {
            "allowed-vlans": {"v_range": [["7.2.1", ""]], "type": "raw"},
            "allowed-vlans-all": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "description": {"v_range": [["7.2.1", ""]], "type": "str"},
            "discard-mode": {"v_range": [["7.2.1", ""]], "choices": ["none", "all-untagged", "all-tagged"], "type": "str"},
            "name": {"v_range": [["7.2.1", ""]], "required": true, "type": "str"},
            "untagged-vlans": {"v_range": [["7.2.1", ""]], "type": "raw"},
            "vlan": {"v_range": [["7.2.1", ""]], "type": "str"},
            "fortilink": {"v_range": [["7.4.3", ""]], "type": "raw"}
        }
    },
    "sys_api_sdnconnector": {"stated": true, "options": {"adom": {"type": "str"}, "command": {"type": "str"}, "connector_name": {"type": "str"}}},
    "sys_cloud_orchest": {
        "stated": true,
        "options": {
            "action": {"v_range": [["7.4.0", ""]], "choices": ["deploy", "undeploy", "status"], "type": "str"},
            "adom": {"v_range": [["7.4.0", ""]], "type": "str"},
            "adom_oid": {"v_range": [["7.4.0", ""]], "type": "int"},
            "name": {"v_range": [["7.4.0", ""]], "type": "str"}
        }
    },
    "sys_generate_wsdl": {"stated": true, "options": {"endpoint": {"type": "str"}, "target": {"type": "str"}}},
    "sys_hitcount": {
        "stated": true,
        "options": {
            "adom": {"v_range": [["6.4.8", "6.4.14"], ["7.0.3", ""]], "type": "str"},
            "adom_oid": {"v_range": [["6.4.8", "6.4.14"], ["7.0.3", ""]], "type": "int"},
            "pkg": {"v_range": [["6.4.8", "6.4.14"], ["7.0.3", ""]], "type": "str"},
            "pkg_oid": {"v_range": [["6.4.8", "6.4.14"], ["7.0.3", ""]], "type": "int"}
        }
    },
    "sys_login_challenge": {"stated": true, "options": {"answer": {"type": "str"}, "session": {"type": "str"}}},
    "sys_login_user": {"stated": true, "options": {"passwd": {"no_log": true, "type": "str"}, "user": {"type": "str"}}},
    "sys_logout": {"stated": true, "options": {}},
    "sys_proxy_json": {
        "stated": true,
        "options": {
            "action": {"choices": ["get", "post", "put", "delete"], "type": "str"},
            "payload": {"type": "dict"},
            "resource": {"type": "str"},
            "target": {"type": "list", "elements": "str"},
            "timeout": {"v_range": [["7.0.1", ""]], "type": "int"}
        }
    },
    "sys_reboot": {"stated": true, "options": {"fmgr_message": {"v_range": [["6.2.5", ""]], "type": "str"}}},
    "sys_task_result": {"stated": true, "options": {"taskid": {"v_range": [["6.4.8", "6.4.14"], ["7.0.3", ""]], "type": "int"}}},
    "system_admin_group": {
        "stated": true,
        "options": {"member": {"type": "list", "options": {"name": {"type": "str"}}, "elements": "dict"}, "name": {"required": true, "type": "str"}}
    },
    "system_admin_group_member": {"stated": true, "options": {"name": {"required": true, "type": "str"}}},
    "system_admin_ldap": {
        "stated": true,
        "options": {
            "adom": {"type": "list", "options": {"adom-name": {"type": "str"}}, "elements": "dict"},
            "adom-attr": {"type": "str"},
            "attributes": {"type": "str"},
            "ca-cert": {"type": "str"},
            "cnid": {"type": "str"},
            "connect-timeout": {"v_range": [["6.0.0", "7.4.2"]], "type": "int"},
            "dn": {"type": "str"},
            "filter": {"type": "str"},
            "group": {"type": "str"},
            "memberof-attr": {"type": "str"},
            "name": {"required": true, "type": "str"},
            "password": {"no_log": true, "type": "raw"},
            "port": {"type": "int"},
            "profile-attr": {"type": "str"},
            "secondary-server": {"type": "str"},
            "secure": {"choices": ["disable", "starttls", "ldaps"], "type": "str"},
            "server": {"type": "str"},
            "tertiary-server": {"type": "str"},
            "type": {"choices": ["simple", "anonymous", "regular"], "type": "str"},
            "username": {"type": "str"},
            "adom-access": {"v_range": [["7.0.3", ""]], "choices": ["all", "specify"], "type": "str"}
        }
    },
    "system_admin_ldap_adom": {"stated": true, "options": {"adom-name": {"required": true, "type": "str"}}},
    "system_admin_profile": {
        "stated": true,
        "options": {
            "adom-lock": {"choices": ["none", "read", "read-write"], "type": "str"},
            "adom-policy-packages": {"choices": ["none", "read", "read-write"], "type": "str"},
            "adom-switch": {"choices": ["none", "read", "read-write"], "type": "str"},
            "app-filter": {"choices": ["disable", "enable"], "type": "str"},
            "assignment": {"choices": ["none", "read", "read-write"], "type": "str"},
            "change-password": {"choices": ["disable", "enable"], "type": "str"},
            "config-retrieve": {"choices": ["none", "read", "read-write"], "type": "str"},
            "config-revert": {"choices": ["none", "read", "read-write"], "type": "str"},
            "consistency-check": {"choices": ["none", "read", "read-write"], "type": "str"},
            "datamask": {"choices": ["disable", "enable"], "type": "str"},
            "datamask-custom-fields": {
                "type": "list",
                "options": {
                    "field-category": {"type": "list", "choices": ["log", "fortiview", "alert", "ueba", "all"], "elements": "str"},
                    "field-name": {"type": "str"},
                    "field-status": {"choices": ["disable", "enable"], "type": "str"},
                    "field-type": {"choices": ["string", "ip", "mac", "email", "unknown"], "type": "str"}
                },
                "elements": "dict"
            },
            "datamask-custom-priority": {"choices": ["disable", "enable"], "type": "str"},
            "datamask-fields": {
                "type": "list",
                "choices": ["user", "srcip", "srcname", "srcmac", "dstip", "dstname", "email", "message", "domain"],
                "elements": "str"
            },
            "datamask-key": {"no_log": true, "type": "raw"},
            "deploy-management": {"choices": ["none", "read", "read-write"], "type": "str"},
            "description": {"type": "str"},
            "device-ap": {"choices": ["none", "read", "read-write"], "type": "str"},
            "device-config": {"choices": ["none", "read", "read-write"], "type": "str"},
            "device-forticlient": {"v_range": [["6.0.0", "7.4.2"]], "choices": ["none", "read", "read-write"], "type": "str"},
            "device-fortiswitch": {"choices": ["none", "read", "read-write"], "type": "str"},
            "device-manager": {"choices": ["none", "read", "read-write"], "type": "str"},
            "device-op": {"choices": ["none", "read", "read-write"], "type": "str"},
            "device-policy-package-lock": {"choices": ["none", "read", "read-write"], "type": "str"},
            "device-profile": {"choices": ["none", "read", "read-write"], "type": "str"},
            "device-revision-deletion": {"choices": ["none", "read", "read-write"], "type": "str"},
            "device-wan-link-load-balance": {"choices": ["none", "read", "read-write"], "type": "str"},
            "event-management": {"choices": ["none", "read", "read-write"], "type": "str"},
            "fgd-center-advanced": {"choices": ["none", "read", "read-write"], "type": "str"},
            "fgd-center-fmw-mgmt": {"choices": ["none", "read", "read-write"], "type": "str"},
            "fgd-center-licensing": {"choices": ["none", "read", "read-write"], "type": "str"},
            "fgd_center": {"choices": ["none", "read", "read-write"], "type": "str"},
            "global-policy-packages": {"choices": ["none", "read", "read-write"], "type": "str"},
            "import-policy-packages": {"choices": ["none", "read", "read-write"], "type": "str"},
            "intf-mapping": {"choices": ["none", "read", "read-write"], "type": "str"},
            "ips-filter": {"choices": ["disable", "enable"], "type": "str"},
            "log-viewer": {"choices": ["none", "read", "read-write"], "type": "str"},
            "policy-objects": {"choices": ["none", "read", "read-write"], "type": "str"},
            "profileid": {"required": true, "type": "str"},
            "read-passwd": {"v_range": [["6.0.0", "7.4.2"]], "choices": ["none", "read", "read-write"], "type": "str"},
            "realtime-monitor": {"v_range": [["6.0.0", "7.4.2"]], "choices": ["none", "read", "read-write"], "type": "str"},
            "report-viewer": {"choices": ["none", "read", "read-write"], "type": "str"},
            "scope": {"choices": ["global", "adom"], "type": "str"},
            "set-install-targets": {"choices": ["none", "read", "read-write"], "type": "str"},
            "system-setting": {"choices": ["none", "read", "read-write"], "type": "str"},
            "term-access": {"choices": ["none", "read", "read-write"], "type": "str"},
            "type": {"choices": ["system", "restricted"], "type": "str"},
            "vpn-manager": {"choices": ["none", "read", "read-write"], "type": "str"},
            "web-filter": {"choices": ["disable", "enable"], "type": "str"},
            "datamask-unmasked-time": {"v_range": [["6.2.2", ""]], "type": "int"},
            "super-user-profile": {"v_range": [["6.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "allow-to-install": {"v_range": [["6.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "extension-access": {"v_range": [["6.4.2", ""]], "choices": ["none", "read", "read-write"], "type": "str"},
            "fabric-viewer": {"v_range": [["6.4.6", ""]], "choices": ["none", "read", "read-write"], "type": "str"},
            "run-report": {"v_range": [["7.0.0", ""]], "choices": ["none", "read", "read-write"], "type": "str"},
            "script-access": {"v_range": [["7.0.0", ""]], "choices": ["none", "read", "read-write"], "type": "str"},
            "triage-events": {"v_range": [["7.0.0", ""]], "choices": ["none", "read", "read-write"], "type": "str"},
            "update-incidents": {"v_range": [["7.0.0", ""]], "choices": ["none", "read", "read-write"], "type": "str"},
            "ips-objects": {"v_range": [["7.2.0", ""]], "choices": ["none", "read", "read-write"], "type": "str"},
            "ipv6_trusthost1": {"v_range": [["7.0.3", ""]], "type": "str"},
            "ipv6_trusthost10": {"v_range": [["7.0.3", ""]], "type": "str"},
            "ipv6_trusthost2": {"v_range": [["7.0.3", ""]], "type": "str"},
            "ipv6_trusthost3": {"v_range": [["7.0.3", ""]], "type": "str"},
            "ipv6_trusthost4": {"v_range": [["7.0.3", ""]], "type": "str"},
            "ipv6_trusthost5": {"v_range": [["7.0.3", ""]], "type": "str"},
            "ipv6_trusthost6": {"v_range": [["7.0.3", ""]], "type": "str"},
            "ipv6_trusthost7": {"v_range": [["7.0.3", ""]], "type": "str"},
            "ipv6_trusthost8": {"v_range": [["7.0.3", ""]], "type": "str"},
            "ipv6_trusthost9": {"v_range": [["7.0.3", ""]], "type": "str"},
            "rpc-permit": {"v_range": [["7.0.3", ""]], "choices": ["read-write", "none", "read"], "type": "str"},
            "trusthost1": {"v_range": [["7.0.3", ""]], "type": "str"},
            "trusthost10": {"v_range": [["7.0.3", ""]], "type": "str"},
            "trusthost2": {"v_range": [["7.0.3", ""]], "type": "str"},
            "trusthost3": {"v_range": [["7.0.3", ""]], "type": "str"},
            "trusthost4": {"v_range": [["7.0.3", ""]], "type": "str"},
            "trusthost5": {"v_range": [["7.0.3", ""]], "type": "str"},
            "trusthost6": {"v_range": [["7.0.3", ""]], "type": "str"},
            "trusthost7": {"v_range": [["7.0.3", ""]], "type": "str"},
            "trusthost8": {"v_range": [["7.0.3", ""]], "type": "str"},
            "trusthost9": {"v_range": [["7.0.3", ""]], "type": "str"},
            "ips-baseline-cfg": {"v_range": [["7.0.1", "7.0.2"]], "choices": ["none", "read", "read-write"], "type": "str"},
            "ips-baseline-ovrd": {"v_range": [["7.0.1", "7.0.2"]], "choices": ["disable", "enable"], "type": "str"},
            "device-fortiextender": {"v_range": [["7.0.4", "7.0.12"], ["7.2.1", ""]], "choices": ["none", "read", "read-write"], "type": "str"},
            "ips-lock": {"v_range": [["7.2.2", ""]], "choices": ["none", "read", "read-write"], "type": "str"},
            "fgt-gui-proxy": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "policy-ips-attrs": {"v_range": [["7.4.2", ""]], "choices": ["none", "read", "read-write"], "type": "str"},
            "write-passwd-access": {"v_range": [["7.4.2", ""]], "choices": ["all", "specify-by-user", "specify-by-profile"], "type": "str"},
            "write-passwd-profiles": {
                "v_range": [["7.4.2", ""]],
                "no_log": true,
                "type": "list",
                "options": {"profileid": {"v_range": [["7.4.2", ""]], "type": "str"}},
                "elements": "dict"
            },
            "write-passwd-user-list": {
                "v_range": [["7.4.2", ""]],
                "no_log": true,
                "type": "list",
                "options": {"userid": {"v_range": [["7.4.2", ""]], "type": "str"}},
                "elements": "dict"
            },
            "adom-admin": {"v_range": [["7.6.0", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "system_admin_profile_datamaskcustomfields": {
        "stated": true,
        "options": {
            "field-category": {"type": "list", "choices": ["log", "fortiview", "alert", "ueba", "all"], "elements": "str"},
            "field-name": {"required": true, "type": "str"},
            "field-status": {"choices": ["disable", "enable"], "type": "str"},
            "field-type": {"choices": ["string", "ip", "mac", "email", "unknown"], "type": "str"}
        }
    },
    "system_admin_profile_writepasswdprofiles": {"stated": true, "options": {"profileid": {"v_range": [["7.4.2", ""]], "type": "str"}}},
    "system_admin_profile_writepasswduserlist": {"stated": true, "options": {"userid": {"v_range": [["7.4.2", ""]], "type": "str"}}},
    "system_admin_radius": {
        "stated": true,
        "options": {
            "auth-type": {"choices": ["any", "pap", "chap", "mschap2"], "type": "str"},
            "name": {"required": true, "type": "str"},
            "nas-ip": {"type": "str"},
            "port": {"type": "int"},
            "secondary-secret": {"no_log": true, "type": "raw"},
            "secondary-server": {"type": "str"},
            "secret": {"no_log": true, "type": "raw"},
            "server": {"type": "str"}
        }
    },
    "system_admin_setting": {
        "stated": false,
        "options": {
            "access-banner": {"choices": ["disable", "enable"], "type": "str"},
            "admin-https-redirect": {"choices": ["disable", "enable"], "type": "str"},
            "admin-login-max": {"type": "int"},
            "admin_server_cert": {"type": "str"},
            "allow_register": {"choices": ["disable", "enable"], "type": "str"},
            "auto-update": {"choices": ["disable", "enable"], "type": "str"},
            "banner-message": {"type": "str"},
            "chassis-mgmt": {"choices": ["disable", "enable"], "type": "str"},
            "chassis-update-interval": {"type": "int"},
            "device_sync_status": {"choices": ["disable", "enable"], "type": "str"},
            "gui-theme": {
                "choices": [
                    "blue", "green", "red", "melongene", "spring", "summer", "autumn", "winter", "space", "calla-lily", "binary-tunnel", "diving",
                    "dreamy", "technology", "landscape", "twilight", "canyon", "northern-light", "astronomy", "fish", "penguin", "panda", "polar-bear",
                    "parrot", "cave", "mountain", "zebra", "contrast-dark", "circuit-board", "mars", "blue-sea", "mariner", "jade", "neutrino",
                    "dark-matter", "forest", "cat", "graphite"
                ],
                "type": "str"
            },
            "http_port": {"type": "int"},
            "https_port": {"type": "int"},
            "idle_timeout": {"type": "int"},
            "install-ifpolicy-only": {"choices": ["disable", "enable"], "type": "str"},
            "mgmt-addr": {"type": "str"},
            "mgmt-fqdn": {"type": "str"},
            "objects-force-deletion": {"choices": ["disable", "enable"], "type": "str"},
            "offline_mode": {"choices": ["disable", "enable"], "type": "str"},
            "register_passwd": {"no_log": true, "type": "raw"},
            "sdwan-monitor-history": {"choices": ["disable", "enable"], "type": "str"},
            "shell-access": {"v_range": [["6.0.0", "7.4.3"]], "choices": ["disable", "enable"], "type": "str"},
            "shell-password": {"v_range": [["6.0.0", "7.4.3"]], "no_log": true, "type": "raw"},
            "show-add-multiple": {"choices": ["disable", "enable"], "type": "str"},
            "show-adom-devman": {"choices": ["disable", "enable"], "type": "str"},
            "show-checkbox-in-table": {"choices": ["disable", "enable"], "type": "str"},
            "show-device-import-export": {"choices": ["disable", "enable"], "type": "str"},
            "show-hostname": {"choices": ["disable", "enable"], "type": "str"},
            "show_automatic_script": {"choices": ["disable", "enable"], "type": "str"},
            "show_grouping_script": {"choices": ["disable", "enable"], "type": "str"},
            "show_schedule_script": {"choices": ["disable", "enable"], "type": "str"},
            "show_tcl_script": {"choices": ["disable", "enable"], "type": "str"},
            "unreg_dev_opt": {"choices": ["add_no_service", "ignore", "add_allow_service"], "type": "str"},
            "webadmin_language": {
                "choices": ["auto_detect", "english", "simplified_chinese", "traditional_chinese", "japanese", "korean", "spanish", "french"],
                "type": "str"
            },
            "show-fct-manager": {"v_range": [["6.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "sdwan-skip-unmapped-input-device": {"v_range": [["6.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "auth-addr": {"v_range": [["7.0.0", ""]], "type": "str"},
            "auth-port": {"v_range": [["7.0.0", ""]], "type": "int"},
            "idle_timeout_api": {"v_range": [["6.4.6", ""]], "type": "int"},
            "idle_timeout_gui": {"v_range": [["6.4.6", ""]], "type": "int"},
            "central-ftgd-local-cat-id": {"v_range": [["6.4.6", "6.4.14"], ["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "idle_timeout_sso": {"v_range": [["7.0.3", ""]], "type": "int"},
            "preferred-fgfm-intf": {"v_range": [["7.0.2", ""]], "type": "str"},
            "traffic-shaping-history": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "fsw-ignore-platform-check": {"v_range": [["7.0.7", "7.0.12"], ["7.2.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "rtm-max-monitor-by-days": {"v_range": [["7.2.2", ""]], "type": "int"},
            "rtm-temp-file-limit": {"v_range": [["7.2.2", ""]], "type": "int"},
            "firmware-upgrade-check": {"v_range": [["7.2.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "fgt-gui-proxy": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "fgt-gui-proxy-port": {"v_range": [["7.4.2", ""]], "type": "int"}
        }
    },
    "system_admin_tacacs": {
        "stated": true,
        "options": {
            "authen-type": {"choices": ["auto", "ascii", "pap", "chap", "mschap"], "type": "str"},
            "authorization": {"choices": ["disable", "enable"], "type": "str"},
            "key": {"no_log": true, "type": "raw"},
            "name": {"required": true, "type": "str"},
            "port": {"type": "int"},
            "secondary-key": {"no_log": true, "type": "raw"},
            "secondary-server": {"type": "str"},
            "server": {"type": "str"},
            "tertiary-key": {"no_log": true, "type": "raw"},
            "tertiary-server": {"type": "str"}
        }
    },
    "system_admin_user": {
        "stated": true,
        "options": {
            "adom": {"type": "list", "options": {"adom-name": {"type": "str"}}, "elements": "dict"},
            "adom-exclude": {
                "v_range": [["6.0.0", "7.0.2"]],
                "type": "list",
                "options": {"adom-name": {"v_range": [["6.0.0", "7.0.2"]], "type": "str"}},
                "elements": "dict"
            },
            "app-filter": {"type": "list", "options": {"app-filter-name": {"type": "str"}}, "elements": "dict"},
            "avatar": {"type": "str"},
            "ca": {"type": "str"},
            "change-password": {"choices": ["disable", "enable"], "type": "str"},
            "dashboard": {
                "type": "list",
                "options": {
                    "column": {"type": "int"},
                    "diskio-content-type": {"choices": ["util", "iops", "blks"], "type": "str"},
                    "diskio-period": {"choices": ["1hour", "8hour", "24hour"], "type": "str"},
                    "log-rate-period": {"choices": ["2min", "1hour", "6hours"], "type": "str"},
                    "log-rate-topn": {"choices": ["1", "2", "3", "4", "5"], "type": "str"},
                    "log-rate-type": {"choices": ["log", "device"], "type": "str"},
                    "moduleid": {"type": "int"},
                    "name": {"type": "str"},
                    "num-entries": {"type": "int"},
                    "refresh-interval": {"type": "int"},
                    "res-cpu-display": {"choices": ["average", "each"], "type": "str"},
                    "res-period": {"choices": ["10min", "hour", "day"], "type": "str"},
                    "res-view-type": {"choices": ["real-time", "history"], "type": "str"},
                    "status": {"choices": ["close", "open"], "type": "str"},
                    "tabid": {"type": "int"},
                    "time-period": {"choices": ["1hour", "8hour", "24hour"], "type": "str"},
                    "widget-type": {
                        "choices": [
                            "top-lograte", "sysres", "sysinfo", "licinfo", "jsconsole", "sysop", "alert", "statistics", "rpteng", "raid", "logrecv",
                            "devsummary", "logdb-perf", "logdb-lag", "disk-io", "log-rcvd-fwd"
                        ],
                        "type": "str"
                    }
                },
                "elements": "dict"
            },
            "dashboard-tabs": {"type": "list", "options": {"name": {"type": "str"}, "tabid": {"type": "int"}}, "elements": "dict"},
            "description": {"type": "str"},
            "dev-group": {"type": "str"},
            "email-address": {"type": "str"},
            "ext-auth-accprofile-override": {"choices": ["disable", "enable"], "type": "str"},
            "ext-auth-adom-override": {"choices": ["disable", "enable"], "type": "str"},
            "ext-auth-group-match": {"type": "str"},
            "first-name": {"type": "str"},
            "force-password-change": {"choices": ["disable", "enable"], "type": "str"},
            "group": {"type": "str"},
            "hidden": {"type": "int"},
            "ips-filter": {"type": "list", "options": {"ips-filter-name": {"type": "str"}}, "elements": "dict"},
            "ipv6_trusthost1": {"type": "str"},
            "ipv6_trusthost10": {"type": "str"},
            "ipv6_trusthost2": {"type": "str"},
            "ipv6_trusthost3": {"type": "str"},
            "ipv6_trusthost4": {"type": "str"},
            "ipv6_trusthost5": {"type": "str"},
            "ipv6_trusthost6": {"type": "str"},
            "ipv6_trusthost7": {"type": "str"},
            "ipv6_trusthost8": {"type": "str"},
            "ipv6_trusthost9": {"type": "str"},
            "last-name": {"type": "str"},
            "ldap-server": {"type": "str"},
            "meta-data": {
                "type": "list",
                "options": {
                    "fieldlength": {"type": "int"},
                    "fieldname": {"type": "str"},
                    "fieldvalue": {"type": "str"},
                    "importance": {"choices": ["optional", "required"], "type": "str"},
                    "status": {"choices": ["disabled", "enabled"], "type": "str"}
                },
                "elements": "dict"
            },
            "mobile-number": {"type": "str"},
            "pager-number": {"type": "str"},
            "password": {"no_log": true, "type": "raw"},
            "password-expire": {"no_log": true, "type": "raw"},
            "phone-number": {"type": "str"},
            "policy-package": {"type": "list", "options": {"policy-package-name": {"type": "str"}}, "elements": "dict"},
            "profileid": {"type": "str"},
            "radius_server": {"type": "str"},
            "restrict-access": {"v_range": [["6.0.0", "6.2.3"], ["6.4.0", "6.4.0"]], "choices": ["disable", "enable"], "type": "str"},
            "restrict-dev-vdom": {
                "v_range": [["6.0.0", "6.2.3"], ["6.4.0", "6.4.0"]],
                "type": "list",
                "options": {"dev-vdom": {"v_range": [["6.0.0", "6.2.3"], ["6.4.0", "6.4.0"]], "type": "str"}},
                "elements": "dict"
            },
            "rpc-permit": {"choices": ["read-write", "none", "read", "from-profile"], "type": "str"},
            "ssh-public-key1": {"no_log": true, "type": "raw"},
            "ssh-public-key2": {"no_log": true, "type": "raw"},
            "ssh-public-key3": {"no_log": true, "type": "raw"},
            "subject": {"type": "str"},
            "tacacs-plus-server": {"type": "str"},
            "trusthost1": {"type": "str"},
            "trusthost10": {"type": "str"},
            "trusthost2": {"type": "str"},
            "trusthost3": {"type": "str"},
            "trusthost4": {"type": "str"},
            "trusthost5": {"type": "str"},
            "trusthost6": {"type": "str"},
            "trusthost7": {"type": "str"},
            "trusthost8": {"type": "str"},
            "trusthost9": {"type": "str"},
            "two-factor-auth": {"choices": ["disable", "enable", "password", "ftc-ftm", "ftc-email", "ftc-sms"], "type": "str"},
            "user_type": {"choices": ["local", "radius", "ldap", "tacacs-plus", "pki-auth", "group", "sso", "api"], "type": "str"},
            "userid": {"required": true, "type": "str"},
            "web-filter": {"type": "list", "options": {"web-filter-name": {"type": "str"}}, "elements": "dict"},
            "wildcard": {"choices": ["disable", "enable"], "type": "str"},
            "login-max": {"v_range": [["6.4.6", ""]], "type": "int"},
            "use-global-theme": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "user-theme": {
                "v_range": [["7.0.0", ""]],
                "choices": [
                    "blue", "green", "red", "melongene", "spring", "summer", "autumn", "winter", "circuit-board", "calla-lily", "binary-tunnel", "mars",
                    "blue-sea", "technology", "landscape", "twilight", "canyon", "northern-light", "astronomy", "fish", "penguin", "mountain", "panda",
                    "parrot", "cave", "zebra", "contrast-dark", "mariner", "jade", "neutrino", "dark-matter", "forest", "cat", "graphite"
                ],
                "type": "str"
            },
            "adom-access": {"v_range": [["7.0.3", ""]], "choices": ["all", "specify", "exclude", "per-adom-profile"], "type": "str"},
            "fingerprint": {"v_range": [["6.4.8", "6.4.14"], ["7.0.4", ""]], "type": "str"},
            "th-from-profile": {"v_range": [["7.0.3", ""]], "type": "int"},
            "th6-from-profile": {"v_range": [["7.0.3", ""]], "type": "int"},
            "cors-allow-origin": {"v_range": [["7.2.2", ""]], "type": "str"},
            "fortiai": {"v_range": [["7.6.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "policy-block": {
                "v_range": [["7.6.0", ""]],
                "type": "list",
                "options": {"policy-block-name": {"v_range": [["7.6.0", ""]], "type": "str"}},
                "elements": "dict"
            }
        }
    },
    "system_admin_user_adom": {"stated": true, "options": {"adom-name": {"required": true, "type": "str"}}},
    "system_admin_user_adomexclude": {"stated": true, "options": {"adom-name": {"v_range": [["6.0.0", "7.0.2"]], "required": true, "type": "str"}}},
    "system_admin_user_appfilter": {"stated": true, "options": {"app-filter-name": {"required": true, "type": "str"}}},
    "system_admin_user_dashboard": {
        "stated": true,
        "options": {
            "column": {"type": "int"},
            "diskio-content-type": {"choices": ["util", "iops", "blks"], "type": "str"},
            "diskio-period": {"choices": ["1hour", "8hour", "24hour"], "type": "str"},
            "log-rate-period": {"choices": ["2min", "1hour", "6hours"], "type": "str"},
            "log-rate-topn": {"choices": ["1", "2", "3", "4", "5"], "type": "str"},
            "log-rate-type": {"choices": ["log", "device"], "type": "str"},
            "moduleid": {"required": true, "type": "int"},
            "name": {"type": "str"},
            "num-entries": {"type": "int"},
            "refresh-interval": {"type": "int"},
            "res-cpu-display": {"choices": ["average", "each"], "type": "str"},
            "res-period": {"choices": ["10min", "hour", "day"], "type": "str"},
            "res-view-type": {"choices": ["real-time", "history"], "type": "str"},
            "status": {"choices": ["close", "open"], "type": "str"},
            "tabid": {"type": "int"},
            "time-period": {"choices": ["1hour", "8hour", "24hour"], "type": "str"},
            "widget-type": {
                "choices": [
                    "top-lograte", "sysres", "sysinfo", "licinfo", "jsconsole", "sysop", "alert", "statistics", "rpteng", "raid", "logrecv",
                    "devsummary", "logdb-perf", "logdb-lag", "disk-io", "log-rcvd-fwd"
                ],
                "type": "str"
            }
        }
    },
    "system_admin_user_dashboardtabs": {"stated": true, "options": {"name": {"required": true, "type": "str"}, "tabid": {"type": "int"}}},
    "system_admin_user_ipsfilter": {"stated": true, "options": {"ips-filter-name": {"required": true, "type": "str"}}},
    "system_admin_user_metadata": {
        "stated": true,
        "options": {
            "fieldlength": {"type": "int"},
            "fieldname": {"required": true, "type": "str"},
            "fieldvalue": {"type": "str"},
            "importance": {"choices": ["optional", "required"], "type": "str"},
            "status": {"choices": ["disabled", "enabled"], "type": "str"}
        }
    },
    "system_admin_user_policyblock": {"stated": true, "options": {"policy-block-name": {"v_range": [["7.6.0", ""]], "type": "str"}}},
    "system_admin_user_policypackage": {"stated": true, "options": {"policy-package-name": {"required": true, "type": "str"}}},
    "system_admin_user_restrictdevvdom": {
        "stated": true,
        "options": {"dev-vdom": {"v_range": [["6.0.0", "6.2.3"], ["6.4.0", "6.4.0"]], "required": true, "type": "str"}}
    },
    "system_admin_user_webfilter": {"stated": true, "options": {"web-filter-name": {"required": true, "type": "str"}}},
    "system_alertconsole": {
        "stated": false,
        "options": {
            "period": {"choices": ["1", "2", "3", "4", "5", "6", "7"], "type": "str"},
            "severity-level": {
                "type": "list",
                "choices": ["debug", "information", "notify", "warning", "error", "critical", "alert", "emergency"],
                "elements": "str"
            }
        }
    },
    "system_alertemail": {
        "stated": false,
        "options": {
            "authentication": {"choices": ["disable", "enable"], "type": "str"},
            "fromaddress": {"type": "str"},
            "fromname": {"type": "str"},
            "smtppassword": {"no_log": true, "type": "raw"},
            "smtpport": {"type": "int"},
            "smtpserver": {"type": "str"},
            "smtpuser": {"type": "str"}
        }
    },
    "system_alertevent": {
        "stated": true,
        "options": {
            "alert-destination": {
                "type": "list",
                "options": {
                    "from": {"type": "str"},
                    "smtp-name": {"type": "str"},
                    "snmp-name": {"type": "str"},
                    "syslog-name": {"type": "str"},
                    "to": {"type": "str"},
                    "type": {"choices": ["mail", "snmp", "syslog"], "type": "str"}
                },
                "elements": "dict"
            },
            "enable-generic-text": {"type": "list", "choices": ["enable", "disable"], "elements": "str"},
            "enable-severity-filter": {"type": "list", "choices": ["enable", "disable"], "elements": "str"},
            "event-time-period": {"choices": ["0.5", "1", "3", "6", "12", "24", "72", "168"], "type": "str"},
            "generic-text": {"type": "str"},
            "name": {"required": true, "type": "str"},
            "num-events": {"choices": ["1", "5", "10", "50", "100"], "type": "str"},
            "severity-filter": {"choices": ["high", "medium-high", "medium", "medium-low", "low"], "type": "str"},
            "severity-level-comp": {"type": "list", "choices": [">=", "=", "<="], "elements": "str"},
            "severity-level-logs": {
                "type": "list",
                "choices": ["no-check", "information", "notify", "warning", "error", "critical", "alert", "emergency"],
                "elements": "str"
            }
        }
    },
    "system_alertevent_alertdestination": {
        "stated": true,
        "options": {
            "from": {"type": "str"},
            "smtp-name": {"type": "str"},
            "snmp-name": {"type": "str"},
            "syslog-name": {"type": "str"},
            "to": {"type": "str"},
            "type": {"choices": ["mail", "snmp", "syslog"], "type": "str"}
        }
    },
    "system_autodelete": {
        "stated": false,
        "options": {
            "dlp-files-auto-deletion": {
                "type": "dict",
                "options": {
                    "retention": {"choices": ["days", "weeks", "months"], "type": "str"},
                    "runat": {"type": "int"},
                    "status": {"choices": ["disable", "enable"], "type": "str"},
                    "value": {"type": "int"}
                }
            },
            "log-auto-deletion": {
                "type": "dict",
                "options": {
                    "retention": {"choices": ["days", "weeks", "months"], "type": "str"},
                    "runat": {"type": "int"},
                    "status": {"choices": ["disable", "enable"], "type": "str"},
                    "value": {"type": "int"}
                }
            },
            "quarantine-files-auto-deletion": {
                "type": "dict",
                "options": {
                    "retention": {"choices": ["days", "weeks", "months"], "type": "str"},
                    "runat": {"type": "int"},
                    "status": {"choices": ["disable", "enable"], "type": "str"},
                    "value": {"type": "int"}
                }
            },
            "report-auto-deletion": {
                "type": "dict",
                "options": {
                    "retention": {"choices": ["days", "weeks", "months"], "type": "str"},
                    "runat": {"type": "int"},
                    "status": {"choices": ["disable", "enable"], "type": "str"},
                    "value": {"type": "int"}
                }
            },
            "status-fake": {"type": "int"}
        }
    },
    "system_autodelete_dlpfilesautodeletion": {
        "stated": false,
        "options": {
            "retention": {"choices": ["days", "weeks", "months"], "type": "str"},
            "runat": {"type": "int"},
            "status": {"choices": ["disable", "enable"], "type": "str"},
            "value": {"type": "int"}
        }
    },
    "system_autodelete_logautodeletion": {
        "stated": false,
        "options": {
            "retention": {"choices": ["days", "weeks", "months"], "type": "str"},
            "runat": {"type": "int"},
            "status": {"choices": ["disable", "enable"], "type": "str"},
            "value": {"type": "int"}
        }
    },
    "system_autodelete_quarantinefilesautodeletion": {
        "stated": false,
        "options": {
            "retention": {"choices": ["days", "weeks", "months"], "type": "str"},
            "runat": {"type": "int"},
            "status": {"choices": ["disable", "enable"], "type": "str"},
            "value": {"type": "int"}
        }
    },
    "system_autodelete_reportautodeletion": {
        "stated": false,
        "options": {
            "retention": {"choices": ["days", "weeks", "months"], "type": "str"},
            "runat": {"type": "int"},
            "status": {"choices": ["disable", "enable"], "type": "str"},
            "value": {"type": "int"}
        }
    },
    "system_backup_allsettings": {
        "stated": false,
        "options": {
            "cert": {"type": "str"},
            "crptpasswd": {"no_log": true, "type": "raw"},
            "directory": {"type": "str"},
            "passwd": {"no_log": true, "type": "raw"},
            "protocol": {"choices": ["sftp", "ftp", "scp"], "type": "str"},
            "server": {"type": "str"},
            "status": {"choices": ["disable", "enable"], "type": "str"},
            "time": {"type": "str"},
            "user": {"type": "str"},
            "week_days": {"type": "list", "choices": ["monday", "tuesday", "wednesday", "thursday", "friday", "saturday", "sunday"], "elements": "str"}
        }
    },
    "system_certificate_ca": {"stated": true, "options": {"ca": {"type": "raw"}, "comment": {"type": "str"}, "name": {"required": true, "type": "str"}}},
    "system_certificate_crl": {
        "stated": true,
        "options": {
            "comment": {"type": "str"},
            "crl": {"type": "raw"},
            "name": {"required": true, "type": "str"},
            "http-url": {"v_range": [["6.2.2", ""]], "type": "str"},
            "update-interval": {"v_range": [["6.2.2", ""]], "type": "int"}
        }
    },
    "system_certificate_local": {
        "stated": true,
        "options": {
            "certificate": {"type": "raw"},
            "comment": {"type": "str"},
            "csr": {"type": "raw"},
            "name": {"required": true, "type": "str"},
            "password": {"no_log": true, "type": "raw"},
            "private-key": {"no_log": true, "type": "raw"}
        }
    },
    "system_certificate_oftp": {
        "stated": false,
        "options": {
            "certificate": {"type": "raw"},
            "comment": {"type": "str"},
            "custom": {"v_range": [["6.0.0", "6.2.3"], ["6.4.0", "6.4.0"]], "choices": ["disable", "enable"], "type": "str"},
            "password": {"no_log": true, "type": "raw"},
            "private-key": {"no_log": true, "type": "raw"},
            "local": {"v_range": [["6.2.5", "6.2.12"], ["6.4.1", ""]], "type": "str"},
            "mode": {"v_range": [["6.2.5", "6.2.12"], ["6.4.1", ""]], "choices": ["default", "custom", "local"], "type": "str"}
        }
    },
    "system_certificate_remote": {
        "stated": true,
        "options": {"cert": {"type": "raw"}, "comment": {"type": "str"}, "name": {"required": true, "type": "str"}}
    },
    "system_certificate_ssh": {
        "stated": true,
        "options": {
            "certificate": {"type": "raw"},
            "comment": {"type": "str"},
            "name": {"required": true, "type": "str"},
            "private-key": {"no_log": true, "type": "raw"}
        }
    },
    "system_connector": {
        "stated": false,
        "options": {
            "fsso-refresh-interval": {"type": "int"},
            "fsso-sess-timeout": {"type": "int"},
            "px-refresh-interval": {"v_range": [["6.0.0", "7.0.1"]], "type": "int"},
            "px-svr-timeout": {"type": "int"},
            "conn-refresh-interval": {"v_range": [["7.0.2", ""]], "type": "int"},
            "cloud-orchest-refresh-interval": {"v_range": [["7.4.0", ""]], "type": "int"},
            "faznotify-msg-queue-max": {"v_range": [["7.4.2", ""]], "type": "int"},
            "faznotify-msg-timeout": {"v_range": [["7.4.2", ""]], "type": "int"}
        }
    },
    "system_csf": {
        "stated": false,
        "options": {
            "accept-auth-by-cert": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "authorization-request-type": {"v_range": [["7.4.1", ""]], "choices": ["certificate", "serial"], "type": "str"},
            "certificate": {"v_range": [["7.4.1", ""]], "type": "str"},
            "configuration-sync": {"v_range": [["7.4.1", "7.4.3"]], "choices": ["default", "local"], "type": "str"},
            "downstream-access": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "downstream-accprofile": {"v_range": [["7.4.1", ""]], "type": "str"},
            "fabric-connector": {
                "v_range": [["7.4.1", ""]],
                "type": "list",
                "options": {
                    "accprofile": {"v_range": [["7.4.1", ""]], "type": "str"},
                    "configuration-write-access": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "serial": {"v_range": [["7.4.1", ""]], "type": "str"}
                },
                "elements": "dict"
            },
            "fabric-object-unification": {"v_range": [["7.4.1", "7.4.3"]], "choices": ["local", "default"], "type": "str"},
            "fabric-workers": {"v_range": [["7.4.1", ""]], "type": "int"},
            "file-mgmt": {"v_range": [["7.4.1", "7.4.3"]], "choices": ["disable", "enable"], "type": "str"},
            "file-quota": {"v_range": [["7.4.1", "7.4.3"]], "type": "int"},
            "file-quota-warning": {"v_range": [["7.4.1", "7.4.3"]], "type": "int"},
            "fixed-key": {"v_range": [["7.4.1", ""]], "no_log": true, "type": "list", "elements": "str"},
            "forticloud-account-enforcement": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "group-name": {"v_range": [["7.4.1", ""]], "type": "str"},
            "group-password": {"v_range": [["7.4.1", ""]], "no_log": true, "type": "list", "elements": "str"},
            "log-unification": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "saml-configuration-sync": {"v_range": [["7.4.1", "7.4.3"]], "choices": ["local", "default"], "type": "str"},
            "status": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "trusted-list": {
                "v_range": [["7.4.1", ""]],
                "type": "list",
                "options": {
                    "action": {"v_range": [["7.4.1", ""]], "choices": ["accept", "deny"], "type": "str"},
                    "authorization-type": {"v_range": [["7.4.1", ""]], "choices": ["serial", "certificate"], "type": "str"},
                    "certificate": {"v_range": [["7.4.1", ""]], "type": "str"},
                    "downstream-authorization": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "ha-members": {"v_range": [["7.4.1", ""]], "type": "str"},
                    "index": {"v_range": [["7.4.1", ""]], "type": "int"},
                    "name": {"v_range": [["7.4.1", ""]], "type": "str"},
                    "serial": {"v_range": [["7.4.1", ""]], "type": "str"}
                },
                "elements": "dict"
            },
            "upstream": {"v_range": [["7.4.1", ""]], "type": "str"},
            "upstream-port": {"v_range": [["7.4.1", ""]], "type": "int"},
            "upstream-confirm": {"v_range": [["7.6.0", ""]], "choices": ["discover", "confirm"], "type": "str"}
        }
    },
    "system_csf_fabricconnector": {
        "stated": true,
        "options": {
            "accprofile": {"v_range": [["7.4.1", ""]], "type": "str"},
            "configuration-write-access": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "serial": {"v_range": [["7.4.1", ""]], "type": "str"}
        }
    },
    "system_csf_trustedlist": {
        "stated": true,
        "options": {
            "action": {"v_range": [["7.4.1", ""]], "choices": ["accept", "deny"], "type": "str"},
            "authorization-type": {"v_range": [["7.4.1", ""]], "choices": ["serial", "certificate"], "type": "str"},
            "certificate": {"v_range": [["7.4.1", ""]], "type": "str"},
            "downstream-authorization": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ha-members": {"v_range": [["7.4.1", ""]], "type": "str"},
            "index": {"v_range": [["7.4.1", ""]], "type": "int"},
            "name": {"v_range": [["7.4.1", ""]], "required": true, "type": "str"},
            "serial": {"v_range": [["7.4.1", ""]], "type": "str"}
        }
    },
    "system_customlanguage": {
        "stated": true,
        "options": {"comments": {"type": "str"}, "filename": {"type": "str"}, "name": {"required": true, "type": "str"}}
    },
    "system_dhcp_server": {
        "stated": true,
        "options": {
            "auto-configuration": {"choices": ["disable", "enable"], "type": "str"},
            "conflicted-ip-timeout": {"type": "int"},
            "ddns-auth": {"choices": ["disable", "tsig"], "type": "str"},
            "ddns-key": {"no_log": true, "type": "raw"},
            "ddns-keyname": {"no_log": true, "type": "str"},
            "ddns-server-ip": {"type": "str"},
            "ddns-ttl": {"type": "int"},
            "ddns-update": {"choices": ["disable", "enable"], "type": "str"},
            "ddns-update-override": {"choices": ["disable", "enable"], "type": "str"},
            "ddns-zone": {"type": "str"},
            "default-gateway": {"type": "str"},
            "dns-server1": {"type": "str"},
            "dns-server2": {"type": "str"},
            "dns-server3": {"type": "str"},
            "dns-service": {"choices": ["default", "specify", "local"], "type": "str"},
            "domain": {"type": "str"},
            "exclude-range": {
                "type": "list",
                "options": {
                    "end-ip": {"type": "str"},
                    "id": {"type": "int"},
                    "start-ip": {"type": "str"},
                    "vci-match": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "vci-string": {"v_range": [["7.2.1", ""]], "type": "raw"},
                    "lease-time": {"v_range": [["7.2.2", ""]], "type": "int"},
                    "uci-match": {"v_range": [["7.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "uci-string": {"v_range": [["7.2.2", ""]], "type": "raw"}
                },
                "elements": "dict"
            },
            "filename": {"type": "str"},
            "forticlient-on-net-status": {"choices": ["disable", "enable"], "type": "str"},
            "id": {"required": true, "type": "int"},
            "interface": {"type": "str"},
            "ip-mode": {"choices": ["range", "usrgrp"], "type": "str"},
            "ip-range": {
                "type": "list",
                "options": {
                    "end-ip": {"type": "str"},
                    "id": {"type": "int"},
                    "start-ip": {"type": "str"},
                    "vci-match": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "vci-string": {"v_range": [["7.2.1", ""]], "type": "raw"},
                    "lease-time": {"v_range": [["7.2.2", ""]], "type": "int"},
                    "uci-match": {"v_range": [["7.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "uci-string": {"v_range": [["7.2.2", ""]], "type": "raw"}
                },
                "elements": "dict"
            },
            "ipsec-lease-hold": {"type": "int"},
            "lease-time": {"type": "int"},
            "mac-acl-default-action": {"choices": ["assign", "block"], "type": "str"},
            "netmask": {"type": "str"},
            "next-server": {"type": "str"},
            "ntp-server1": {"type": "str"},
            "ntp-server2": {"type": "str"},
            "ntp-server3": {"type": "str"},
            "ntp-service": {"choices": ["default", "specify", "local"], "type": "str"},
            "options": {
                "type": "list",
                "options": {
                    "code": {"type": "int"},
                    "id": {"type": "int"},
                    "ip": {"type": "raw"},
                    "type": {"choices": ["hex", "string", "ip", "fqdn"], "type": "str"},
                    "value": {"type": "str"},
                    "vci-match": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "vci-string": {"v_range": [["7.2.1", ""]], "type": "raw"},
                    "uci-match": {"v_range": [["7.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "uci-string": {"v_range": [["7.2.2", ""]], "type": "raw"}
                },
                "elements": "dict"
            },
            "reserved-address": {
                "type": "list",
                "options": {
                    "action": {"choices": ["assign", "block", "reserved"], "type": "str"},
                    "description": {"type": "str"},
                    "id": {"type": "int"},
                    "ip": {"type": "str"},
                    "mac": {"type": "str"},
                    "circuit-id": {"v_range": [["6.2.0", ""]], "type": "str"},
                    "circuit-id-type": {"v_range": [["6.2.0", ""]], "choices": ["hex", "string"], "type": "str"},
                    "remote-id": {"v_range": [["6.2.0", ""]], "type": "str"},
                    "remote-id-type": {"v_range": [["6.2.0", ""]], "choices": ["hex", "string"], "type": "str"},
                    "type": {"v_range": [["6.2.0", ""]], "choices": ["mac", "option82"], "type": "str"}
                },
                "elements": "dict"
            },
            "server-type": {"choices": ["regular", "ipsec"], "type": "str"},
            "status": {"choices": ["disable", "enable"], "type": "str"},
            "tftp-server": {"type": "raw"},
            "timezone": {
                "choices": [
                    "00", "01", "02", "03", "04", "05", "06", "07", "08", "09", "10", "11", "12", "13", "14", "15", "16", "17", "18", "19", "20", "21",
                    "22", "23", "24", "25", "26", "27", "28", "29", "30", "31", "32", "33", "34", "35", "36", "37", "38", "39", "40", "41", "42", "43",
                    "44", "45", "46", "47", "48", "49", "50", "51", "52", "53", "54", "55", "56", "57", "58", "59", "60", "61", "62", "63", "64", "65",
                    "66", "67", "68", "69", "70", "71", "72", "73", "74", "75", "76", "77", "78", "79", "80", "81", "82", "83", "84", "85", "86", "87"
                ],
                "type": "str"
            },
            "timezone-option": {"choices": ["disable", "default", "specify"], "type": "str"},
            "vci-match": {"choices": ["disable", "enable"], "type": "str"},
            "vci-string": {"type": "raw"},
            "wifi-ac1": {"type": "str"},
            "wifi-ac2": {"type": "str"},
            "wifi-ac3": {"type": "str"},
            "wins-server1": {"type": "str"},
            "wins-server2": {"type": "str"},
            "dns-server4": {"v_range": [["6.2.2", ""]], "type": "str"},
            "wifi-ac-service": {"v_range": [["6.2.2", ""]], "choices": ["specify", "local"], "type": "str"},
            "auto-managed-status": {"v_range": [["6.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "dhcp-settings-from-fortiipam": {"v_range": [["6.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "relay-agent": {"v_range": [["7.4.0", ""]], "type": "str"},
            "shared-subnet": {"v_range": [["7.4.0", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "system_dhcp_server_excluderange": {
        "stated": true,
        "options": {
            "end-ip": {"type": "str"},
            "id": {"required": true, "type": "int"},
            "start-ip": {"type": "str"},
            "vci-match": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "vci-string": {"v_range": [["7.2.1", ""]], "type": "raw"},
            "lease-time": {"v_range": [["7.2.2", ""]], "type": "int"},
            "uci-match": {"v_range": [["7.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "uci-string": {"v_range": [["7.2.2", ""]], "type": "raw"}
        }
    },
    "system_dhcp_server_iprange": {
        "stated": true,
        "options": {
            "end-ip": {"type": "str"},
            "id": {"required": true, "type": "int"},
            "start-ip": {"type": "str"},
            "vci-match": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "vci-string": {"v_range": [["7.2.1", ""]], "type": "raw"},
            "lease-time": {"v_range": [["7.2.2", ""]], "type": "int"},
            "uci-match": {"v_range": [["7.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "uci-string": {"v_range": [["7.2.2", ""]], "type": "raw"}
        }
    },
    "system_dhcp_server_options": {
        "stated": true,
        "options": {
            "code": {"type": "int"},
            "id": {"required": true, "type": "int"},
            "ip": {"type": "raw"},
            "type": {"choices": ["hex", "string", "ip", "fqdn"], "type": "str"},
            "value": {"type": "str"},
            "vci-match": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "vci-string": {"v_range": [["7.2.1", ""]], "type": "raw"},
            "uci-match": {"v_range": [["7.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "uci-string": {"v_range": [["7.2.2", ""]], "type": "raw"}
        }
    },
    "system_dhcp_server_reservedaddress": {
        "stated": true,
        "options": {
            "action": {"choices": ["assign", "block", "reserved"], "type": "str"},
            "description": {"type": "str"},
            "id": {"required": true, "type": "int"},
            "ip": {"type": "str"},
            "mac": {"type": "str"},
            "circuit-id": {"v_range": [["6.2.0", ""]], "type": "str"},
            "circuit-id-type": {"v_range": [["6.2.0", ""]], "choices": ["hex", "string"], "type": "str"},
            "remote-id": {"v_range": [["6.2.0", ""]], "type": "str"},
            "remote-id-type": {"v_range": [["6.2.0", ""]], "choices": ["hex", "string"], "type": "str"},
            "type": {"v_range": [["6.2.0", ""]], "choices": ["mac", "option82"], "type": "str"}
        }
    },
    "system_dm": {
        "stated": false,
        "options": {
            "concurrent-install-image-limit": {"type": "int"},
            "concurrent-install-limit": {"type": "int"},
            "concurrent-install-script-limit": {"type": "int"},
            "discover-timeout": {"type": "int"},
            "dpm-logsize": {"type": "int"},
            "fgfm-sock-timeout": {"type": "int"},
            "fgfm_keepalive_itvl": {"type": "int"},
            "force-remote-diff": {"choices": ["disable", "enable"], "type": "str"},
            "fortiap-refresh-cnt": {"type": "int"},
            "fortiap-refresh-itvl": {"type": "int"},
            "fortiext-refresh-cnt": {"type": "int"},
            "install-image-timeout": {"type": "int"},
            "install-tunnel-retry-itvl": {"type": "int"},
            "max-revs": {"type": "int"},
            "nr-retry": {"type": "int"},
            "retry": {"choices": ["disable", "enable"], "type": "str"},
            "retry-intvl": {"type": "int"},
            "rollback-allow-reboot": {"choices": ["disable", "enable"], "type": "str"},
            "script-logsize": {"type": "int"},
            "skip-scep-check": {"choices": ["disable", "enable"], "type": "str"},
            "skip-tunnel-fcp-req": {"choices": ["disable", "enable"], "type": "str"},
            "verify-install": {"choices": ["disable", "optimal", "enable"], "type": "str"},
            "fgfm-install-refresh-count": {"v_range": [["6.2.5", ""]], "type": "int"},
            "conf-merge-after-script": {"v_range": [["6.2.7", "6.2.12"], ["6.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "log-autoupdate": {
                "v_range": [["6.4.12", "6.4.14"], ["7.0.9", "7.0.12"], ["7.2.4", "7.2.5"], ["7.4.1", ""]],
                "choices": ["disable", "enable"],
                "type": "str"
            },
            "fgfm-auto-retrieve-timeout": {"v_range": [["6.4.13", "6.4.14"], ["7.0.9", "7.0.12"], ["7.2.4", "7.2.5"], ["7.4.1", ""]], "type": "int"},
            "install-fds-timeout": {"v_range": [["7.4.1", ""]], "type": "int"}
        }
    },
    "system_dns": {
        "stated": false,
        "options": {"ip6-primary": {"type": "str"}, "ip6-secondary": {"type": "str"}, "primary": {"type": "str"}, "secondary": {"type": "str"}}
    },
    "system_docker": {
        "stated": false,
        "options": {
            "fortiportal": {"v_range": [["6.4.0", "7.2.4"], ["7.4.0", "7.4.0"]], "choices": ["disable", "enable"], "type": "str"},
            "fortiwlm": {"v_range": [["6.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "sdwancontroller": {"v_range": [["6.4.0", "7.0.12"]], "choices": ["disable", "enable"], "type": "str"},
            "status": {"v_range": [["6.4.0", ""]], "choices": ["disable", "enable", "qa", "dev"], "type": "str"},
            "cpu": {"v_range": [["6.4.5", ""]], "type": "int"},
            "default-address-pool_base": {"v_range": [["6.4.3", ""]], "type": "str"},
            "default-address-pool_size": {"v_range": [["6.4.3", ""]], "type": "int"},
            "fortiauthenticator": {"v_range": [["6.4.3", "7.2.0"]], "choices": ["disable", "enable"], "type": "str"},
            "fortisigconverter": {"v_range": [["6.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "mem": {"v_range": [["6.4.5", ""]], "type": "int"},
            "docker-user-login-max": {"v_range": [["6.4.6", ""]], "type": "int"},
            "fortisoar": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "fortiaiops": {"v_range": [["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "policyanalyzer": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "universalconnector": {"v_range": [["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "fsmcollector": {"v_range": [["7.0.1", "7.0.1"]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "system_externalresource": {
        "stated": true,
        "options": {
            "category": {"type": "int"},
            "comments": {"type": "str"},
            "name": {"required": true, "type": "str"},
            "refresh-rate": {"type": "int"},
            "resource": {"type": "str"},
            "status": {"choices": ["disable", "enable"], "type": "str"},
            "type": {"choices": ["category", "address", "domain", "malware", "url", "mac-address", "data"], "type": "str"},
            "password": {"v_range": [["6.2.0", ""]], "no_log": true, "type": "raw"},
            "source-ip": {"v_range": [["6.2.1", ""]], "type": "str"},
            "username": {"v_range": [["6.2.0", ""]], "type": "str"},
            "interface": {"v_range": [["6.4.2", ""]], "type": "str"},
            "interface-select-method": {"v_range": [["6.4.2", ""]], "choices": ["auto", "sdwan", "specify"], "type": "str"},
            "user-agent": {"v_range": [["6.4.2", ""]], "type": "str"},
            "uuid": {"v_range": [["7.0.0", ""]], "type": "str"},
            "server-identity-check": {"v_range": [["7.0.5", "7.0.12"], ["7.2.2", ""]], "choices": ["none", "basic", "full"], "type": "str"},
            "update-method": {"v_range": [["7.2.1", ""]], "choices": ["feed", "push"], "type": "str"}
        }
    },
    "system_fips": {
        "stated": false,
        "options": {
            "entropy-token": {"choices": ["enable", "disable", "dynamic"], "type": "str"},
            "re-seed-interval": {"type": "int"},
            "status": {"choices": ["disable", "enable"], "type": "str"}
        }
    },
    "system_fmgcluster": {
        "stated": false,
        "options": {
            "fqdn": {"v_range": [["7.6.0", ""]], "type": "str"},
            "ip": {"v_range": [["7.6.0", ""]], "type": "str"},
            "mode": {"v_range": [["7.6.0", ""]], "choices": ["standalone", "primary", "worker"], "type": "str"},
            "peer": {
                "v_range": [["7.6.0", ""]],
                "type": "list",
                "options": {
                    "addr": {"v_range": [["7.6.0", ""]], "type": "str"},
                    "fqdn": {"v_range": [["7.6.0", ""]], "type": "str"},
                    "name": {"v_range": [["7.6.0", ""]], "type": "str"},
                    "sn": {"v_range": [["7.6.0", ""]], "type": "str"}
                },
                "elements": "dict"
            }
        }
    },
    "system_fmgcluster_peer": {
        "stated": true,
        "options": {
            "addr": {"v_range": [["7.6.0", ""]], "type": "str"},
            "fqdn": {"v_range": [["7.6.0", ""]], "type": "str"},
            "name": {"v_range": [["7.6.0", ""]], "required": true, "type": "str"},
            "sn": {"v_range": [["7.6.0", ""]], "type": "str"}
        }
    },
    "system_fortiguard": {
        "stated": false,
        "options": {
            "antispam-cache": {"choices": ["disable", "enable"], "type": "str"},
            "antispam-cache-mpercent": {"type": "int"},
            "antispam-cache-ttl": {"type": "int"},
            "antispam-expiration": {"type": "int"},
            "antispam-force-off": {"choices": ["disable", "enable"], "type": "str"},
            "antispam-license": {"type": "int"},
            "antispam-timeout": {"type": "int"},
            "auto-join-forticloud": {"choices": ["disable", "enable"], "type": "str"},
            "ddns-server-ip": {"type": "str"},
            "ddns-server-port": {"type": "int"},
            "load-balance-servers": {"type": "int"},
            "outbreak-prevention-cache": {"choices": ["disable", "enable"], "type": "str"},
            "outbreak-prevention-cache-mpercent": {"type": "int"},
            "outbreak-prevention-cache-ttl": {"type": "int"},
            "outbreak-prevention-expiration": {"type": "int"},
            "outbreak-prevention-force-off": {"choices": ["disable", "enable"], "type": "str"},
            "outbreak-prevention-license": {"type": "int"},
            "outbreak-prevention-timeout": {"type": "int"},
            "port": {"choices": ["53", "80", "8888", "443"], "type": "str"},
            "sdns-server-ip": {"type": "raw"},
            "sdns-server-port": {"type": "int"},
            "service-account-id": {"type": "str"},
            "source-ip": {"type": "str"},
            "source-ip6": {"type": "str"},
            "update-server-location": {"choices": ["any", "usa", "automatic", "eu"], "type": "str"},
            "webfilter-cache": {"choices": ["disable", "enable"], "type": "str"},
            "webfilter-cache-ttl": {"type": "int"},
            "webfilter-expiration": {"type": "int"},
            "webfilter-force-off": {"choices": ["disable", "enable"], "type": "str"},
            "webfilter-license": {"type": "int"},
            "webfilter-timeout": {"type": "int"},
            "protocol": {"v_range": [["6.2.0", ""]], "choices": ["udp", "http", "https"], "type": "str"},
            "proxy-password": {"v_range": [["6.2.1", ""]], "no_log": true, "type": "raw"},
            "proxy-server-ip": {"v_range": [["6.2.1", ""]], "type": "str"},
            "proxy-server-port": {"v_range": [["6.2.1", ""]], "type": "int"},
            "proxy-username": {"v_range": [["6.2.1", ""]], "type": "str"},
            "sandbox-region": {"v_range": [["6.2.0", ""]], "type": "str"},
            "avquery-cache-ttl": {"v_range": [["6.2.0", "6.4.14"]], "type": "int"},
            "avquery-timeout": {"v_range": [["6.2.0", "6.4.14"]], "type": "int"},
            "avquery-cache": {"v_range": [["6.2.0", "6.4.14"]], "choices": ["disable", "enable"], "type": "str"},
            "avquery-cache-mpercent": {"v_range": [["6.2.0", "6.4.14"]], "type": "int"},
            "avquery-license": {"v_range": [["6.2.0", "6.4.14"]], "type": "int"},
            "avquery-force-off": {"v_range": [["6.2.0", "6.4.14"]], "choices": ["disable", "enable"], "type": "str"},
            "fortiguard-anycast": {"v_range": [["6.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "fortiguard-anycast-source": {"v_range": [["6.2.2", ""]], "choices": ["fortinet", "aws", "debug"], "type": "str"},
            "interface": {"v_range": [["6.2.5", "6.2.12"], ["6.4.1", ""]], "type": "str"},
            "interface-select-method": {"v_range": [["6.2.5", "6.2.12"], ["6.4.1", ""]], "choices": ["auto", "sdwan", "specify"], "type": "str"},
            "sdns-options": {"v_range": [["6.4.0", ""]], "type": "list", "choices": ["include-question-section"], "elements": "str"},
            "anycast-sdns-server-ip": {"v_range": [["6.4.1", ""]], "type": "str"},
            "anycast-sdns-server-port": {"v_range": [["6.4.1", ""]], "type": "int"},
            "persistent-connection": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "update-build-proxy": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "update-extdb": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "update-ffdb": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "update-uwdb": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "videofilter-expiration": {"v_range": [["7.0.0", ""]], "type": "int"},
            "videofilter-license": {"v_range": [["7.0.0", ""]], "type": "int"},
            "ddns-server-ip6": {"v_range": [["7.0.1", ""]], "type": "str"},
            "vdom": {"v_range": [["7.2.0", ""]], "type": "str"},
            "auto-firmware-upgrade": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "auto-firmware-upgrade-day": {
                "v_range": [["7.2.1", ""]],
                "type": "list",
                "choices": ["sunday", "monday", "tuesday", "wednesday", "thursday", "friday", "saturday"],
                "elements": "str"
            },
            "auto-firmware-upgrade-end-hour": {"v_range": [["7.2.1", ""]], "type": "int"},
            "auto-firmware-upgrade-start-hour": {"v_range": [["7.2.1", ""]], "type": "int"},
            "sandbox-inline-scan": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "auto-firmware-upgrade-delay": {"v_range": [["7.2.4", ""]], "type": "int"},
            "gui-prompt-auto-upgrade": {"v_range": [["7.2.4", "7.2.5"], ["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "FDS-license-expiring-days": {"v_range": [["7.4.0", ""]], "type": "int"},
            "antispam-cache-mpermille": {"v_range": [["7.4.0", ""]], "type": "int"},
            "outbreak-prevention-cache-mpermille": {"v_range": [["7.4.0", ""]], "type": "int"},
            "update-dldb": {"v_range": [["7.4.0", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "system_fortiview_autocache": {
        "stated": false,
        "options": {
            "aggressive-fortiview": {"choices": ["disable", "enable"], "type": "str"},
            "interval": {"type": "int"},
            "status": {"choices": ["disable", "enable"], "type": "str"},
            "incr-fortiview": {"v_range": [["7.2.5", "7.2.5"], ["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "system_fortiview_setting": {
        "stated": false,
        "options": {
            "not-scanned-apps": {"choices": ["exclude", "include"], "type": "str"},
            "resolve-ip": {"choices": ["disable", "enable"], "type": "str"},
            "data-source": {"v_range": [["6.4.8", "6.4.14"], ["7.0.3", ""]], "choices": ["auto", "cache-only", "log-and-cache"], "type": "str"}
        }
    },
    "system_geoipcountry": {"stated": true, "options": {"id": {"required": true, "type": "str"}, "name": {"type": "str"}}},
    "system_geoipoverride": {
        "stated": true,
        "options": {
            "description": {"type": "str"},
            "ip-range": {"type": "list", "options": {"end-ip": {"type": "str"}, "id": {"type": "int"}, "start-ip": {"type": "str"}}, "elements": "dict"},
            "name": {"required": true, "type": "str"},
            "ip6-range": {
                "v_range": [["6.4.0", ""]],
                "type": "list",
                "options": {
                    "end-ip": {"v_range": [["6.4.0", ""]], "type": "str"},
                    "id": {"v_range": [["6.4.0", ""]], "type": "int"},
                    "start-ip": {"v_range": [["6.4.0", ""]], "type": "str"}
                },
                "elements": "dict"
            }
        }
    },
    "system_geoipoverride_ip6range": {
        "stated": true,
        "options": {
            "end-ip": {"v_range": [["6.4.0", ""]], "type": "str"},
            "id": {"v_range": [["6.4.0", ""]], "required": true, "type": "int"},
            "start-ip": {"v_range": [["6.4.0", ""]], "type": "str"}
        }
    },
    "system_geoipoverride_iprange": {
        "stated": true,
        "options": {"end-ip": {"type": "str"}, "id": {"required": true, "type": "int"}, "start-ip": {"type": "str"}}
    },
    "system_global": {
        "stated": false,
        "options": {
            "admin-lockout-duration": {"type": "int"},
            "admin-lockout-threshold": {"type": "int"},
            "adom-mode": {"choices": ["normal", "advanced"], "type": "str"},
            "adom-rev-auto-delete": {"choices": ["disable", "by-revisions", "by-days"], "type": "str"},
            "adom-rev-max-backup-revisions": {"type": "int"},
            "adom-rev-max-days": {"type": "int"},
            "adom-rev-max-revisions": {"type": "int"},
            "adom-select": {"choices": ["disable", "enable"], "type": "str"},
            "adom-status": {"choices": ["disable", "enable"], "type": "str"},
            "clt-cert-req": {"choices": ["disable", "enable", "optional"], "type": "str"},
            "console-output": {"choices": ["standard", "more"], "type": "str"},
            "country-flag": {"choices": ["disable", "enable"], "type": "str"},
            "create-revision": {"choices": ["disable", "enable"], "type": "str"},
            "daylightsavetime": {"choices": ["disable", "enable"], "type": "str"},
            "default-disk-quota": {"v_range": [["6.0.0", "6.2.0"]], "type": "int"},
            "detect-unregistered-log-device": {"choices": ["disable", "enable"], "type": "str"},
            "device-view-mode": {"choices": ["regular", "tree"], "type": "str"},
            "dh-params": {"choices": ["1024", "1536", "2048", "3072", "4096", "6144", "8192"], "type": "str"},
            "disable-module": {"type": "list", "choices": ["fortiview-noc", "none", "fortirecorder", "siem", "soc", "ai"], "elements": "str"},
            "enc-algorithm": {"choices": ["low", "medium", "high", "custom"], "type": "str"},
            "faz-status": {"choices": ["disable", "enable"], "type": "str"},
            "fgfm-local-cert": {"type": "str"},
            "fgfm-ssl-protocol": {"choices": ["sslv3", "tlsv1.0", "tlsv1.1", "tlsv1.2", "tlsv1.3"], "type": "str"},
            "ha-member-auto-grouping": {"choices": ["disable", "enable"], "type": "str"},
            "hitcount_concurrent": {"v_range": [["6.0.0", "6.4.2"]], "type": "int"},
            "hitcount_interval": {"v_range": [["6.0.0", "6.4.2"]], "type": "int"},
            "hostname": {"type": "str"},
            "import-ignore-addr-cmt": {"choices": ["disable", "enable"], "type": "str"},
            "language": {"choices": ["english", "simch", "japanese", "korean", "spanish", "trach"], "type": "str"},
            "latitude": {"type": "str"},
            "ldap-cache-timeout": {"type": "int"},
            "ldapconntimeout": {"type": "int"},
            "lock-preempt": {"choices": ["disable", "enable"], "type": "str"},
            "log-checksum": {"choices": ["none", "md5", "md5-auth"], "type": "str"},
            "log-forward-cache-size": {"type": "int"},
            "longitude": {"type": "str"},
            "max-log-forward": {"type": "int"},
            "max-running-reports": {"type": "int"},
            "oftp-ssl-protocol": {"choices": ["sslv3", "tlsv1.0", "tlsv1.1", "tlsv1.2", "tlsv1.3"], "type": "str"},
            "partial-install": {"choices": ["disable", "enable"], "type": "str"},
            "partial-install-force": {"choices": ["disable", "enable"], "type": "str"},
            "partial-install-rev": {"choices": ["disable", "enable"], "type": "str"},
            "perform-improve-by-ha": {"choices": ["disable", "enable"], "type": "str"},
            "policy-hit-count": {"v_range": [["6.0.0", "6.4.2"]], "choices": ["disable", "enable"], "type": "str"},
            "policy-object-in-dual-pane": {"choices": ["disable", "enable"], "type": "str"},
            "pre-login-banner": {"choices": ["disable", "enable"], "type": "str"},
            "pre-login-banner-message": {"type": "str"},
            "remoteauthtimeout": {"type": "int"},
            "search-all-adoms": {"choices": ["disable", "enable"], "type": "str"},
            "ssl-low-encryption": {"choices": ["disable", "enable"], "type": "str"},
            "ssl-protocol": {"type": "list", "choices": ["tlsv1.2", "tlsv1.1", "tlsv1.0", "sslv3", "tlsv1.3"], "elements": "str"},
            "ssl-static-key-ciphers": {"choices": ["disable", "enable"], "type": "str"},
            "task-list-size": {"type": "int"},
            "tftp": {"choices": ["disable", "enable"], "type": "str"},
            "timezone": {
                "choices": [
                    "00", "01", "02", "03", "04", "05", "06", "07", "08", "09", "10", "11", "12", "13", "14", "15", "16", "17", "18", "19", "20", "21",
                    "22", "23", "24", "25", "26", "27", "28", "29", "30", "31", "32", "33", "34", "35", "36", "37", "38", "39", "40", "41", "42", "43",
                    "44", "45", "46", "47", "48", "49", "50", "51", "52", "53", "54", "55", "56", "57", "58", "59", "60", "61", "62", "63", "64", "65",
                    "66", "67", "68", "69", "70", "71", "72", "73", "74", "75", "76", "77", "78", "79", "80", "81", "82", "83", "84", "85", "86", "87",
                    "88", "89", "90", "91"
                ],
                "type": "str"
            },
            "tunnel-mtu": {"type": "int"},
            "usg": {"choices": ["disable", "enable"], "type": "str"},
            "vdom-mirror": {"choices": ["disable", "enable"], "type": "str"},
            "webservice-proto": {"type": "list", "choices": ["tlsv1.2", "tlsv1.1", "tlsv1.0", "sslv3", "sslv2", "tlsv1.3"], "elements": "str"},
            "workflow-max-sessions": {"type": "int"},
            "workspace-mode": {"choices": ["disabled", "normal", "workflow", "per-adom"], "type": "str"},
            "clone-name-option": {"v_range": [["6.2.2", ""]], "choices": ["default", "keep"], "type": "str"},
            "fgfm-ca-cert": {"v_range": [["6.2.3", ""]], "type": "str"},
            "mc-policy-disabled-adoms": {
                "v_range": [["6.2.3", ""]],
                "type": "list",
                "options": {"adom-name": {"v_range": [["6.2.3", ""]], "type": "str"}},
                "elements": "dict"
            },
            "policy-object-icon": {"v_range": [["6.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "private-data-encryption": {"v_range": [["6.2.5", ""]], "choices": ["disable", "enable"], "type": "str"},
            "per-policy-lock": {"v_range": [["6.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "multiple-steps-upgrade-in-autolink": {"v_range": [["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
            "object-revision-db-max": {"v_range": [["7.0.0", ""]], "type": "int"},
            "object-revision-mandatory-note": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "object-revision-object-max": {"v_range": [["7.0.0", ""]], "type": "int"},
            "object-revision-status": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "normalized-intf-zone-only": {"v_range": [["6.4.7", "6.4.14"], ["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ssl-cipher-suites": {
                "v_range": [["6.4.8", "6.4.14"], ["7.0.2", ""]],
                "type": "list",
                "options": {
                    "cipher": {"v_range": [["6.4.8", "6.4.14"], ["7.0.2", ""]], "type": "str"},
                    "priority": {"v_range": [["6.4.8", "6.4.14"], ["7.0.2", ""]], "type": "int"},
                    "version": {"v_range": [["6.4.8", "6.4.14"], ["7.0.2", ""]], "choices": ["tls1.2-or-below", "tls1.3"], "type": "str"}
                },
                "elements": "dict"
            },
            "gui-curl-timeout": {"v_range": [["6.4.11", "6.4.14"], ["7.0.7", "7.0.12"], ["7.2.2", ""]], "type": "int"},
            "table-entry-blink": {"v_range": [["7.0.4", "7.0.12"], ["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "contentpack-fgt-install": {"v_range": [["7.0.5", "7.0.12"], ["7.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "gui-polling-interval": {"v_range": [["7.0.5", "7.0.12"], ["7.2.1", ""]], "type": "int"},
            "no-copy-permission-check": {"v_range": [["7.0.8", "7.0.12"], ["7.2.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ssh-enc-algo": {
                "v_range": [["7.0.11", "7.0.12"], ["7.2.5", "7.2.5"], ["7.4.2", ""]],
                "type": "list",
                "choices": [
                    "chacha20-poly1305@openssh.com", "aes128-ctr", "aes192-ctr", "aes256-ctr", "arcfour256", "arcfour128", "aes128-cbc", "3des-cbc",
                    "blowfish-cbc", "cast128-cbc", "aes192-cbc", "aes256-cbc", "arcfour", "rijndael-cbc@lysator.liu.se", "aes128-gcm@openssh.com",
                    "aes256-gcm@openssh.com"
                ],
                "elements": "str"
            },
            "ssh-hostkey-algo": {
                "v_range": [["7.0.11", "7.0.12"], ["7.2.5", "7.2.5"], ["7.4.2", ""]],
                "type": "list",
                "choices": ["ssh-rsa", "ecdsa-sha2-nistp521", "rsa-sha2-256", "rsa-sha2-512", "ssh-ed25519"],
                "elements": "str"
            },
            "ssh-kex-algo": {
                "v_range": [["7.0.11", "7.0.12"], ["7.2.5", "7.2.5"], ["7.4.2", ""]],
                "type": "list",
                "choices": [
                    "diffie-hellman-group1-sha1", "diffie-hellman-group14-sha1", "diffie-hellman-group14-sha256", "diffie-hellman-group16-sha512",
                    "diffie-hellman-group18-sha512", "diffie-hellman-group-exchange-sha1", "diffie-hellman-group-exchange-sha256",
                    "curve25519-sha256@libssh.org", "ecdh-sha2-nistp256", "ecdh-sha2-nistp384", "ecdh-sha2-nistp521"
                ],
                "elements": "str"
            },
            "ssh-mac-algo": {
                "v_range": [["7.0.11", "7.0.12"], ["7.2.5", "7.2.5"], ["7.4.2", ""]],
                "type": "list",
                "choices": [
                    "hmac-md5", "hmac-md5-etm@openssh.com", "hmac-md5-96", "hmac-md5-96-etm@openssh.com", "hmac-sha1", "hmac-sha1-etm@openssh.com",
                    "hmac-sha2-256", "hmac-sha2-256-etm@openssh.com", "hmac-sha2-512", "hmac-sha2-512-etm@openssh.com", "hmac-ripemd160",
                    "hmac-ripemd160@openssh.com", "hmac-ripemd160-etm@openssh.com", "umac-64@openssh.com", "umac-128@openssh.com",
                    "umac-64-etm@openssh.com", "umac-128-etm@openssh.com"
                ],
                "elements": "str"
            },
            "ssh-strong-crypto": {"v_range": [["7.0.11", "7.0.12"], ["7.2.5", "7.2.5"], ["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "fgfm-cert-exclusive": {"v_range": [["7.0.12", "7.0.12"], ["7.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "fgfm-deny-unknown": {"v_range": [["7.0.12", "7.0.12"], ["7.2.5", "7.2.5"], ["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "fgfm-peercert-withoutsn": {
                "v_range": [["7.0.12", "7.0.12"], ["7.2.5", "7.2.5"], ["7.4.3", ""]],
                "choices": ["disable", "enable"],
                "type": "str"
            },
            "admin-lockout-method": {"v_range": [["7.2.2", ""]], "choices": ["ip", "user"], "type": "str"},
            "workspace-unlock-after-install": {"v_range": [["7.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "log-checksum-upload": {"v_range": [["7.2.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "apache-mode": {"v_range": [["7.2.4", "7.2.5"], ["7.4.1", ""]], "choices": ["event", "prefork"], "type": "str"},
            "no-vip-value-check": {"v_range": [["7.2.4", "7.2.5"], ["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "fortiservice-port": {"v_range": [["7.4.1", ""]], "type": "int"},
            "management-ip": {"v_range": [["7.4.1", ""]], "type": "str"},
            "management-port": {"v_range": [["7.4.1", ""]], "type": "int"},
            "save-last-hit-in-adomdb": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "api-ip-binding": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "admin-host": {"v_range": [["7.6.0", ""]], "type": "str"},
            "admin-ssh-grace-time": {"v_range": [["7.6.0", ""]], "type": "int"},
            "fabric-storage-pool-quota": {"v_range": [["7.6.0", ""]], "type": "int"},
            "fabric-storage-pool-size": {"v_range": [["7.6.0", ""]], "type": "int"},
            "fcp-cfg-service": {"v_range": [["7.6.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "jsonapi-log": {"v_range": [["7.6.0", ""]], "choices": ["disable", "request", "response", "all"], "type": "str"}
        }
    },
    "system_guiact": {
        "stated": false,
        "options": {
            "backup_all": {"v_range": [["6.0.0", "7.0.11"], ["7.2.0", "7.2.4"], ["7.4.0", "7.4.0"]], "type": "str"},
            "backup_conf": {"v_range": [["6.0.0", "7.0.11"], ["7.2.0", "7.2.4"], ["7.4.0", "7.4.0"]], "type": "str"},
            "eventlog_msg": {"v_range": [["6.0.0", "7.0.11"], ["7.2.0", "7.2.4"], ["7.4.0", "7.4.0"]], "type": "str"},
            "eventlog_path": {"v_range": [["6.0.0", "7.0.11"], ["7.2.0", "7.2.4"], ["7.4.0", "7.4.0"]], "type": "str"},
            "reboot": {"v_range": [["6.0.0", "7.0.11"], ["7.2.0", "7.2.4"], ["7.4.0", "7.4.0"]], "type": "int"},
            "reset2default": {"v_range": [["6.0.0", "7.0.11"], ["7.2.0", "7.2.4"], ["7.4.0", "7.4.0"]], "type": "int"},
            "restore_all": {"v_range": [["6.0.0", "7.0.11"], ["7.2.0", "7.2.4"], ["7.4.0", "7.4.0"]], "type": "str"},
            "restore_conf": {"v_range": [["6.0.0", "7.0.11"], ["7.2.0", "7.2.4"], ["7.4.0", "7.4.0"]], "type": "str"},
            "time": {"v_range": [["6.0.0", "7.0.11"], ["7.2.0", "7.2.4"], ["7.4.0", "7.4.0"]], "type": "str"}
        }
    },
    "system_ha": {
        "stated": false,
        "options": {
            "clusterid": {"type": "int"},
            "file-quota": {"type": "int"},
            "hb-interval": {"type": "int"},
            "hb-lost-threshold": {"type": "int"},
            "mode": {"choices": ["standalone", "master", "slave", "primary", "secondary"], "type": "str"},
            "password": {"no_log": true, "type": "raw"},
            "peer": {
                "type": "list",
                "options": {
                    "id": {"type": "int"},
                    "ip": {"type": "str"},
                    "ip6": {"type": "str"},
                    "serial-number": {"type": "str"},
                    "status": {"choices": ["disable", "enable"], "type": "str"}
                },
                "elements": "dict"
            },
            "local-cert": {"v_range": [["6.2.7", ""]], "type": "str"},
            "failover-mode": {"v_range": [["7.2.0", ""]], "choices": ["manual", "vrrp"], "type": "str"},
            "monitored-interfaces": {
                "v_range": [["7.2.0", ""]],
                "type": "list",
                "options": {"interface-name": {"v_range": [["7.2.0", ""]], "type": "str"}},
                "elements": "dict"
            },
            "monitored-ips": {
                "v_range": [["7.2.0", ""]],
                "type": "list",
                "options": {
                    "id": {"v_range": [["7.2.0", ""]], "type": "int"},
                    "interface": {"v_range": [["7.2.0", ""]], "type": "str"},
                    "ip": {"v_range": [["7.2.0", ""]], "type": "str"}
                },
                "elements": "dict"
            },
            "priority": {"v_range": [["7.2.0", ""]], "type": "int"},
            "unicast": {"v_range": [["7.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "vip": {"v_range": [["7.2.0", ""]], "type": "str"},
            "vrrp-adv-interval": {"v_range": [["7.2.0", ""]], "type": "int"},
            "vrrp-interface": {"v_range": [["7.2.0", ""]], "type": "str"},
            "vip-interface": {"v_range": [["7.2.4", "7.2.5"], ["7.4.1", ""]], "type": "str"}
        }
    },
    "system_ha_monitoredinterfaces": {"stated": true, "options": {"interface-name": {"v_range": [["7.2.0", ""]], "type": "str"}}},
    "system_ha_monitoredips": {
        "stated": true,
        "options": {
            "id": {"v_range": [["7.2.0", ""]], "required": true, "type": "int"},
            "interface": {"v_range": [["7.2.0", ""]], "type": "str"},
            "ip": {"v_range": [["7.2.0", ""]], "type": "str"}
        }
    },
    "system_ha_peer": {
        "stated": true,
        "options": {"id": {"required": true, "type": "int"}, "ip": {"type": "str"}, "ip6": {"type": "str"}, "serial-number": {"type": "str"}}
    },
    "system_hascheduledcheck": {
        "stated": false,
        "options": {
            "status": {"v_range": [["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "time": {"v_range": [["7.0.1", ""]], "type": "str"},
            "week_days": {
                "v_range": [["7.0.1", ""]],
                "type": "list",
                "choices": ["monday", "tuesday", "wednesday", "thursday", "friday", "saturday", "sunday"],
                "elements": "str"
            }
        }
    },
    "system_interface": {
        "stated": true,
        "options": {
            "alias": {"type": "str"},
            "allowaccess": {
                "type": "list",
                "choices": ["ping", "https", "ssh", "snmp", "http", "webservice", "https-logging", "soc-fabric", "fabric"],
                "elements": "str"
            },
            "description": {"type": "str"},
            "ip": {"type": "str"},
            "ipv6": {
                "type": "dict",
                "options": {
                    "ip6-address": {"type": "str"},
                    "ip6-allowaccess": {
                        "type": "list",
                        "choices": ["ping", "https", "ssh", "snmp", "http", "webservice", "https-logging", "fabric"],
                        "elements": "str"
                    },
                    "ip6-autoconf": {"choices": ["disable", "enable"], "type": "str"}
                }
            },
            "mtu": {"type": "int"},
            "name": {"required": true, "type": "str"},
            "serviceaccess": {"type": "list", "choices": ["fgtupdates", "fclupdates", "webfilter-antispam"], "elements": "str"},
            "speed": {
                "choices": [
                    "auto", "10full", "10half", "100full", "100half", "1000full", "10000full", "1g/full", "2.5g/full", "5g/full", "10g/full", "14g/full",
                    "20g/full", "25g/full", "40g/full", "50g/full", "56g/full", "100g/full", "1g/half", "200g/full", "400g/full"
                ],
                "type": "str"
            },
            "status": {"choices": ["down", "up", "disable", "enable"], "type": "str"},
            "rating-service-ip": {"v_range": [["6.2.2", ""]], "type": "str"},
            "update-service-ip": {"v_range": [["6.2.2", ""]], "type": "str"},
            "aggregate": {"v_range": [["7.2.0", ""]], "type": "str"},
            "interface": {"v_range": [["7.2.0", ""]], "type": "str"},
            "lacp-mode": {"v_range": [["7.2.0", ""]], "choices": ["active"], "type": "str"},
            "lacp-speed": {"v_range": [["7.2.0", ""]], "choices": ["slow", "fast"], "type": "str"},
            "link-up-delay": {"v_range": [["7.2.0", ""]], "type": "int"},
            "member": {
                "v_range": [["7.2.0", ""]],
                "type": "list",
                "options": {"interface-name": {"v_range": [["7.2.0", ""]], "type": "str"}},
                "elements": "dict"
            },
            "min-links": {"v_range": [["7.2.0", ""]], "type": "int"},
            "min-links-down": {"v_range": [["7.2.0", ""]], "choices": ["operational", "administrative"], "type": "str"},
            "type": {"v_range": [["7.2.0", ""]], "choices": ["vlan", "physical", "aggregate"], "type": "str"},
            "vlan-protocol": {"v_range": [["7.2.0", ""]], "choices": ["8021q", "8021ad"], "type": "str"},
            "vlanid": {"v_range": [["7.2.0", ""]], "type": "int"},
            "lldp": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "defaultgw": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "dhcp-client-identifier": {"v_range": [["7.4.2", ""]], "type": "str"},
            "dns-server-override": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "mode": {"v_range": [["7.4.2", ""]], "choices": ["static", "dhcp"], "type": "str"},
            "mtu-override": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "system_interface_ipv6": {
        "stated": false,
        "options": {
            "ip6-address": {"type": "str"},
            "ip6-allowaccess": {
                "type": "list",
                "choices": ["ping", "https", "ssh", "snmp", "http", "webservice", "https-logging", "fabric"],
                "elements": "str"
            },
            "ip6-autoconf": {"choices": ["disable", "enable"], "type": "str"}
        }
    },
    "system_interface_member": {"stated": true, "options": {"interface-name": {"v_range": [["7.2.0", ""]], "type": "str"}}},
    "system_localinpolicy": {
        "stated": true,
        "options": {
            "action": {"v_range": [["7.2.0", ""]], "choices": ["drop", "reject", "accept"], "type": "str"},
            "dport": {"v_range": [["7.2.0", ""]], "type": "int"},
            "dst": {"v_range": [["7.2.0", ""]], "type": "str"},
            "id": {"v_range": [["7.2.0", ""]], "required": true, "type": "int"},
            "intf": {"v_range": [["7.2.0", ""]], "type": "str"},
            "protocol": {"v_range": [["7.2.0", ""]], "choices": ["tcp", "udp", "tcp_udp"], "type": "str"},
            "src": {"v_range": [["7.2.0", ""]], "type": "str"}
        }
    },
    "system_localinpolicy6": {
        "stated": true,
        "options": {
            "action": {"v_range": [["7.2.0", ""]], "choices": ["drop", "reject", "accept"], "type": "str"},
            "dport": {"v_range": [["7.2.0", ""]], "type": "int"},
            "dst": {"v_range": [["7.2.0", ""]], "type": "str"},
            "id": {"v_range": [["7.2.0", ""]], "required": true, "type": "int"},
            "intf": {"v_range": [["7.2.0", ""]], "type": "str"},
            "protocol": {"v_range": [["7.2.0", ""]], "choices": ["tcp", "udp", "tcp_udp"], "type": "str"},
            "src": {"v_range": [["7.2.0", ""]], "type": "str"}
        }
    },
    "system_locallog_disk_filter": {
        "stated": false,
        "options": {
            "devcfg": {"choices": ["disable", "enable"], "type": "str"},
            "devops": {"choices": ["disable", "enable"], "type": "str"},
            "diskquota": {"choices": ["disable", "enable"], "type": "str"},
            "dm": {"choices": ["disable", "enable"], "type": "str"},
            "dvm": {"choices": ["disable", "enable"], "type": "str"},
            "ediscovery": {"choices": ["disable", "enable"], "type": "str"},
            "epmgr": {"choices": ["disable", "enable"], "type": "str"},
            "event": {"choices": ["disable", "enable"], "type": "str"},
            "eventmgmt": {"choices": ["disable", "enable"], "type": "str"},
            "faz": {"choices": ["disable", "enable"], "type": "str"},
            "fazha": {"choices": ["disable", "enable"], "type": "str"},
            "fazsys": {"choices": ["disable", "enable"], "type": "str"},
            "fgd": {"choices": ["disable", "enable"], "type": "str"},
            "fgfm": {"choices": ["disable", "enable"], "type": "str"},
            "fips": {"choices": ["disable", "enable"], "type": "str"},
            "fmgws": {"choices": ["disable", "enable"], "type": "str"},
            "fmlmgr": {"choices": ["disable", "enable"], "type": "str"},
            "fmwmgr": {"choices": ["disable", "enable"], "type": "str"},
            "fortiview": {"choices": ["disable", "enable"], "type": "str"},
            "glbcfg": {"choices": ["disable", "enable"], "type": "str"},
            "ha": {"choices": ["disable", "enable"], "type": "str"},
            "hcache": {"choices": ["disable", "enable"], "type": "str"},
            "iolog": {"choices": ["disable", "enable"], "type": "str"},
            "logd": {"choices": ["disable", "enable"], "type": "str"},
            "logdb": {"choices": ["disable", "enable"], "type": "str"},
            "logdev": {"choices": ["disable", "enable"], "type": "str"},
            "logfile": {"choices": ["enable", "disable"], "type": "str"},
            "logging": {"choices": ["disable", "enable"], "type": "str"},
            "lrmgr": {"choices": ["disable", "enable"], "type": "str"},
            "objcfg": {"choices": ["disable", "enable"], "type": "str"},
            "report": {"choices": ["disable", "enable"], "type": "str"},
            "rev": {"choices": ["disable", "enable"], "type": "str"},
            "rtmon": {"choices": ["disable", "enable"], "type": "str"},
            "scfw": {"choices": ["disable", "enable"], "type": "str"},
            "scply": {"choices": ["disable", "enable"], "type": "str"},
            "scrmgr": {"choices": ["disable", "enable"], "type": "str"},
            "scvpn": {"choices": ["disable", "enable"], "type": "str"},
            "system": {"choices": ["disable", "enable"], "type": "str"},
            "webport": {"choices": ["disable", "enable"], "type": "str"},
            "incident": {"v_range": [["6.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "aid": {"v_range": [["6.4.1", "7.2.5"]], "choices": ["disable", "enable"], "type": "str"},
            "docker": {"v_range": [["6.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "controller": {"v_range": [["7.0.9", "7.0.12"], ["7.2.4", "7.2.5"], ["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "system_locallog_disk_setting": {
        "stated": false,
        "options": {
            "diskfull": {"choices": ["overwrite", "nolog"], "type": "str"},
            "log-disk-full-percentage": {"type": "int"},
            "max-log-file-size": {"type": "int"},
            "roll-day": {"type": "list", "choices": ["sunday", "monday", "tuesday", "wednesday", "thursday", "friday", "saturday"], "elements": "str"},
            "roll-schedule": {"choices": ["none", "daily", "weekly"], "type": "str"},
            "roll-time": {"type": "raw"},
            "server-type": {"choices": ["FTP", "SFTP", "SCP"], "type": "str"},
            "severity": {"choices": ["emergency", "alert", "critical", "error", "warning", "notification", "information", "debug"], "type": "str"},
            "status": {"choices": ["disable", "enable"], "type": "str"},
            "upload": {"choices": ["disable", "enable"], "type": "str"},
            "upload-delete-files": {"choices": ["disable", "enable"], "type": "str"},
            "upload-time": {"type": "raw"},
            "uploaddir": {"type": "str"},
            "uploadip": {"type": "str"},
            "uploadpass": {"no_log": true, "type": "raw"},
            "uploadport": {"type": "int"},
            "uploadsched": {"choices": ["disable", "enable"], "type": "str"},
            "uploadtype": {"type": "list", "choices": ["event"], "elements": "str"},
            "uploaduser": {"type": "str"},
            "uploadzip": {"choices": ["disable", "enable"], "type": "str"},
            "log-disk-quota": {"v_range": [["7.0.3", ""]], "type": "int"},
            "max-log-file-num": {"v_range": [["6.4.8", "6.4.14"], ["7.0.2", ""]], "type": "int"}
        }
    },
    "system_locallog_fortianalyzer2_filter": {
        "stated": false,
        "options": {
            "devcfg": {"choices": ["disable", "enable"], "type": "str"},
            "devops": {"choices": ["disable", "enable"], "type": "str"},
            "diskquota": {"choices": ["disable", "enable"], "type": "str"},
            "dm": {"choices": ["disable", "enable"], "type": "str"},
            "dvm": {"choices": ["disable", "enable"], "type": "str"},
            "ediscovery": {"choices": ["disable", "enable"], "type": "str"},
            "epmgr": {"choices": ["disable", "enable"], "type": "str"},
            "event": {"choices": ["disable", "enable"], "type": "str"},
            "eventmgmt": {"choices": ["disable", "enable"], "type": "str"},
            "faz": {"choices": ["disable", "enable"], "type": "str"},
            "fazha": {"choices": ["disable", "enable"], "type": "str"},
            "fazsys": {"choices": ["disable", "enable"], "type": "str"},
            "fgd": {"choices": ["disable", "enable"], "type": "str"},
            "fgfm": {"choices": ["disable", "enable"], "type": "str"},
            "fips": {"choices": ["disable", "enable"], "type": "str"},
            "fmgws": {"choices": ["disable", "enable"], "type": "str"},
            "fmlmgr": {"choices": ["disable", "enable"], "type": "str"},
            "fmwmgr": {"choices": ["disable", "enable"], "type": "str"},
            "fortiview": {"choices": ["disable", "enable"], "type": "str"},
            "glbcfg": {"choices": ["disable", "enable"], "type": "str"},
            "ha": {"choices": ["disable", "enable"], "type": "str"},
            "hcache": {"choices": ["disable", "enable"], "type": "str"},
            "iolog": {"choices": ["disable", "enable"], "type": "str"},
            "logd": {"choices": ["disable", "enable"], "type": "str"},
            "logdb": {"choices": ["disable", "enable"], "type": "str"},
            "logdev": {"choices": ["disable", "enable"], "type": "str"},
            "logfile": {"choices": ["enable", "disable"], "type": "str"},
            "logging": {"choices": ["disable", "enable"], "type": "str"},
            "lrmgr": {"choices": ["disable", "enable"], "type": "str"},
            "objcfg": {"choices": ["disable", "enable"], "type": "str"},
            "report": {"choices": ["disable", "enable"], "type": "str"},
            "rev": {"choices": ["disable", "enable"], "type": "str"},
            "rtmon": {"choices": ["disable", "enable"], "type": "str"},
            "scfw": {"choices": ["disable", "enable"], "type": "str"},
            "scply": {"choices": ["disable", "enable"], "type": "str"},
            "scrmgr": {"choices": ["disable", "enable"], "type": "str"},
            "scvpn": {"choices": ["disable", "enable"], "type": "str"},
            "system": {"choices": ["disable", "enable"], "type": "str"},
            "webport": {"choices": ["disable", "enable"], "type": "str"},
            "incident": {"v_range": [["6.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "aid": {"v_range": [["6.4.1", "7.2.5"]], "choices": ["disable", "enable"], "type": "str"},
            "docker": {"v_range": [["6.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "controller": {"v_range": [["7.0.9", "7.0.12"], ["7.2.4", "7.2.5"], ["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "system_locallog_fortianalyzer2_setting": {
        "stated": false,
        "options": {
            "reliable": {"choices": ["disable", "enable"], "type": "str"},
            "secure-connection": {"choices": ["disable", "enable"], "type": "str"},
            "server-ip": {"v_range": [["6.0.0", "6.2.1"]], "type": "str"},
            "severity": {"choices": ["emergency", "alert", "critical", "error", "warning", "notification", "information", "debug"], "type": "str"},
            "status": {"choices": ["disable", "realtime", "upload"], "type": "str"},
            "upload-time": {"type": "raw"},
            "server": {"v_range": [["6.2.2", ""]], "type": "str"},
            "peer-cert-cn": {"v_range": [["7.0.3", ""]], "type": "str"}
        }
    },
    "system_locallog_fortianalyzer3_filter": {
        "stated": false,
        "options": {
            "devcfg": {"choices": ["disable", "enable"], "type": "str"},
            "devops": {"choices": ["disable", "enable"], "type": "str"},
            "diskquota": {"choices": ["disable", "enable"], "type": "str"},
            "dm": {"choices": ["disable", "enable"], "type": "str"},
            "dvm": {"choices": ["disable", "enable"], "type": "str"},
            "ediscovery": {"choices": ["disable", "enable"], "type": "str"},
            "epmgr": {"choices": ["disable", "enable"], "type": "str"},
            "event": {"choices": ["disable", "enable"], "type": "str"},
            "eventmgmt": {"choices": ["disable", "enable"], "type": "str"},
            "faz": {"choices": ["disable", "enable"], "type": "str"},
            "fazha": {"choices": ["disable", "enable"], "type": "str"},
            "fazsys": {"choices": ["disable", "enable"], "type": "str"},
            "fgd": {"choices": ["disable", "enable"], "type": "str"},
            "fgfm": {"choices": ["disable", "enable"], "type": "str"},
            "fips": {"choices": ["disable", "enable"], "type": "str"},
            "fmgws": {"choices": ["disable", "enable"], "type": "str"},
            "fmlmgr": {"choices": ["disable", "enable"], "type": "str"},
            "fmwmgr": {"choices": ["disable", "enable"], "type": "str"},
            "fortiview": {"choices": ["disable", "enable"], "type": "str"},
            "glbcfg": {"choices": ["disable", "enable"], "type": "str"},
            "ha": {"choices": ["disable", "enable"], "type": "str"},
            "hcache": {"choices": ["disable", "enable"], "type": "str"},
            "iolog": {"choices": ["disable", "enable"], "type": "str"},
            "logd": {"choices": ["disable", "enable"], "type": "str"},
            "logdb": {"choices": ["disable", "enable"], "type": "str"},
            "logdev": {"choices": ["disable", "enable"], "type": "str"},
            "logfile": {"choices": ["enable", "disable"], "type": "str"},
            "logging": {"choices": ["disable", "enable"], "type": "str"},
            "lrmgr": {"choices": ["disable", "enable"], "type": "str"},
            "objcfg": {"choices": ["disable", "enable"], "type": "str"},
            "report": {"choices": ["disable", "enable"], "type": "str"},
            "rev": {"choices": ["disable", "enable"], "type": "str"},
            "rtmon": {"choices": ["disable", "enable"], "type": "str"},
            "scfw": {"choices": ["disable", "enable"], "type": "str"},
            "scply": {"choices": ["disable", "enable"], "type": "str"},
            "scrmgr": {"choices": ["disable", "enable"], "type": "str"},
            "scvpn": {"choices": ["disable", "enable"], "type": "str"},
            "system": {"choices": ["disable", "enable"], "type": "str"},
            "webport": {"choices": ["disable", "enable"], "type": "str"},
            "incident": {"v_range": [["6.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "aid": {"v_range": [["6.4.1", "7.2.5"]], "choices": ["disable", "enable"], "type": "str"},
            "docker": {"v_range": [["6.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "controller": {"v_range": [["7.0.9", "7.0.12"], ["7.2.4", "7.2.5"], ["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "system_locallog_fortianalyzer3_setting": {
        "stated": false,
        "options": {
            "reliable": {"choices": ["disable", "enable"], "type": "str"},
            "secure-connection": {"choices": ["disable", "enable"], "type": "str"},
            "server-ip": {"v_range": [["6.0.0", "6.2.1"]], "type": "str"},
            "severity": {"choices": ["emergency", "alert", "critical", "error", "warning", "notification", "information", "debug"], "type": "str"},
            "status": {"choices": ["disable", "realtime", "upload"], "type": "str"},
            "upload-time": {"type": "raw"},
            "server": {"v_range": [["6.2.2", ""]], "type": "str"},
            "peer-cert-cn": {"v_range": [["7.0.3", ""]], "type": "str"}
        }
    },
    "system_locallog_fortianalyzer_filter": {
        "stated": false,
        "options": {
            "devcfg": {"choices": ["disable", "enable"], "type": "str"},
            "devops": {"choices": ["disable", "enable"], "type": "str"},
            "diskquota": {"choices": ["disable", "enable"], "type": "str"},
            "dm": {"choices": ["disable", "enable"], "type": "str"},
            "dvm": {"choices": ["disable", "enable"], "type": "str"},
            "ediscovery": {"choices": ["disable", "enable"], "type": "str"},
            "epmgr": {"choices": ["disable", "enable"], "type": "str"},
            "event": {"choices": ["disable", "enable"], "type": "str"},
            "eventmgmt": {"choices": ["disable", "enable"], "type": "str"},
            "faz": {"choices": ["disable", "enable"], "type": "str"},
            "fazha": {"choices": ["disable", "enable"], "type": "str"},
            "fazsys": {"choices": ["disable", "enable"], "type": "str"},
            "fgd": {"choices": ["disable", "enable"], "type": "str"},
            "fgfm": {"choices": ["disable", "enable"], "type": "str"},
            "fips": {"choices": ["disable", "enable"], "type": "str"},
            "fmgws": {"choices": ["disable", "enable"], "type": "str"},
            "fmlmgr": {"choices": ["disable", "enable"], "type": "str"},
            "fmwmgr": {"choices": ["disable", "enable"], "type": "str"},
            "fortiview": {"choices": ["disable", "enable"], "type": "str"},
            "glbcfg": {"choices": ["disable", "enable"], "type": "str"},
            "ha": {"choices": ["disable", "enable"], "type": "str"},
            "hcache": {"choices": ["disable", "enable"], "type": "str"},
            "iolog": {"choices": ["disable", "enable"], "type": "str"},
            "logd": {"choices": ["disable", "enable"], "type": "str"},
            "logdb": {"choices": ["disable", "enable"], "type": "str"},
            "logdev": {"choices": ["disable", "enable"], "type": "str"},
            "logfile": {"choices": ["enable", "disable"], "type": "str"},
            "logging": {"choices": ["disable", "enable"], "type": "str"},
            "lrmgr": {"choices": ["disable", "enable"], "type": "str"},
            "objcfg": {"choices": ["disable", "enable"], "type": "str"},
            "report": {"choices": ["disable", "enable"], "type": "str"},
            "rev": {"choices": ["disable", "enable"], "type": "str"},
            "rtmon": {"choices": ["disable", "enable"], "type": "str"},
            "scfw": {"choices": ["disable", "enable"], "type": "str"},
            "scply": {"choices": ["disable", "enable"], "type": "str"},
            "scrmgr": {"choices": ["disable", "enable"], "type": "str"},
            "scvpn": {"choices": ["disable", "enable"], "type": "str"},
            "system": {"choices": ["disable", "enable"], "type": "str"},
            "webport": {"choices": ["disable", "enable"], "type": "str"},
            "incident": {"v_range": [["6.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "aid": {"v_range": [["6.4.1", "7.2.5"]], "choices": ["disable", "enable"], "type": "str"},
            "docker": {"v_range": [["6.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "controller": {"v_range": [["7.0.9", "7.0.12"], ["7.2.4", "7.2.5"], ["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "system_locallog_fortianalyzer_setting": {
        "stated": false,
        "options": {
            "reliable": {"choices": ["disable", "enable"], "type": "str"},
            "secure-connection": {"choices": ["disable", "enable"], "type": "str"},
            "server-ip": {"v_range": [["6.0.0", "6.2.1"]], "type": "str"},
            "severity": {"choices": ["emergency", "alert", "critical", "error", "warning", "notification", "information", "debug"], "type": "str"},
            "status": {"choices": ["disable", "realtime", "upload"], "type": "str"},
            "upload-time": {"type": "raw"},
            "server": {"v_range": [["6.2.2", ""]], "type": "str"},
            "peer-cert-cn": {"v_range": [["7.0.3", ""]], "type": "str"}
        }
    },
    "system_locallog_memory_filter": {
        "stated": false,
        "options": {
            "devcfg": {"choices": ["disable", "enable"], "type": "str"},
            "devops": {"choices": ["disable", "enable"], "type": "str"},
            "diskquota": {"choices": ["disable", "enable"], "type": "str"},
            "dm": {"choices": ["disable", "enable"], "type": "str"},
            "dvm": {"choices": ["disable", "enable"], "type": "str"},
            "ediscovery": {"choices": ["disable", "enable"], "type": "str"},
            "epmgr": {"choices": ["disable", "enable"], "type": "str"},
            "event": {"choices": ["disable", "enable"], "type": "str"},
            "eventmgmt": {"choices": ["disable", "enable"], "type": "str"},
            "faz": {"choices": ["disable", "enable"], "type": "str"},
            "fazha": {"choices": ["disable", "enable"], "type": "str"},
            "fazsys": {"choices": ["disable", "enable"], "type": "str"},
            "fgd": {"choices": ["disable", "enable"], "type": "str"},
            "fgfm": {"choices": ["disable", "enable"], "type": "str"},
            "fips": {"choices": ["disable", "enable"], "type": "str"},
            "fmgws": {"choices": ["disable", "enable"], "type": "str"},
            "fmlmgr": {"choices": ["disable", "enable"], "type": "str"},
            "fmwmgr": {"choices": ["disable", "enable"], "type": "str"},
            "fortiview": {"choices": ["disable", "enable"], "type": "str"},
            "glbcfg": {"choices": ["disable", "enable"], "type": "str"},
            "ha": {"choices": ["disable", "enable"], "type": "str"},
            "hcache": {"choices": ["disable", "enable"], "type": "str"},
            "iolog": {"choices": ["disable", "enable"], "type": "str"},
            "logd": {"choices": ["disable", "enable"], "type": "str"},
            "logdb": {"choices": ["disable", "enable"], "type": "str"},
            "logdev": {"choices": ["disable", "enable"], "type": "str"},
            "logfile": {"choices": ["enable", "disable"], "type": "str"},
            "logging": {"choices": ["disable", "enable"], "type": "str"},
            "lrmgr": {"choices": ["disable", "enable"], "type": "str"},
            "objcfg": {"choices": ["disable", "enable"], "type": "str"},
            "report": {"choices": ["disable", "enable"], "type": "str"},
            "rev": {"choices": ["disable", "enable"], "type": "str"},
            "rtmon": {"choices": ["disable", "enable"], "type": "str"},
            "scfw": {"choices": ["disable", "enable"], "type": "str"},
            "scply": {"choices": ["disable", "enable"], "type": "str"},
            "scrmgr": {"choices": ["disable", "enable"], "type": "str"},
            "scvpn": {"choices": ["disable", "enable"], "type": "str"},
            "system": {"choices": ["disable", "enable"], "type": "str"},
            "webport": {"choices": ["disable", "enable"], "type": "str"},
            "incident": {"v_range": [["6.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "aid": {"v_range": [["6.4.1", "7.2.5"]], "choices": ["disable", "enable"], "type": "str"},
            "docker": {"v_range": [["6.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "controller": {"v_range": [["7.0.9", "7.0.12"], ["7.2.4", "7.2.5"], ["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "system_locallog_memory_setting": {
        "stated": false,
        "options": {
            "diskfull": {"choices": ["overwrite", "nolog"], "type": "str"},
            "severity": {"choices": ["emergency", "alert", "critical", "error", "warning", "notification", "information", "debug"], "type": "str"},
            "status": {"choices": ["disable", "enable"], "type": "str"}
        }
    },
    "system_locallog_setting": {
        "stated": false,
        "options": {
            "log-interval-dev-no-logging": {"type": "int"},
            "log-interval-disk-full": {"type": "int"},
            "log-interval-gbday-exceeded": {"type": "int"},
            "log-daemon-crash": {"v_range": [["7.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "no-log-detection-threshold": {"v_range": [["7.2.4", "7.2.5"], ["7.4.2", ""]], "type": "int"},
            "log-interval-adom-perf-stats": {"v_range": [["7.4.0", ""]], "type": "int"}
        }
    },
    "system_locallog_syslogd2_filter": {
        "stated": false,
        "options": {
            "devcfg": {"choices": ["disable", "enable"], "type": "str"},
            "devops": {"choices": ["disable", "enable"], "type": "str"},
            "diskquota": {"choices": ["disable", "enable"], "type": "str"},
            "dm": {"choices": ["disable", "enable"], "type": "str"},
            "dvm": {"choices": ["disable", "enable"], "type": "str"},
            "ediscovery": {"choices": ["disable", "enable"], "type": "str"},
            "epmgr": {"choices": ["disable", "enable"], "type": "str"},
            "event": {"choices": ["disable", "enable"], "type": "str"},
            "eventmgmt": {"choices": ["disable", "enable"], "type": "str"},
            "faz": {"choices": ["disable", "enable"], "type": "str"},
            "fazha": {"choices": ["disable", "enable"], "type": "str"},
            "fazsys": {"choices": ["disable", "enable"], "type": "str"},
            "fgd": {"choices": ["disable", "enable"], "type": "str"},
            "fgfm": {"choices": ["disable", "enable"], "type": "str"},
            "fips": {"choices": ["disable", "enable"], "type": "str"},
            "fmgws": {"choices": ["disable", "enable"], "type": "str"},
            "fmlmgr": {"choices": ["disable", "enable"], "type": "str"},
            "fmwmgr": {"choices": ["disable", "enable"], "type": "str"},
            "fortiview": {"choices": ["disable", "enable"], "type": "str"},
            "glbcfg": {"choices": ["disable", "enable"], "type": "str"},
            "ha": {"choices": ["disable", "enable"], "type": "str"},
            "hcache": {"choices": ["disable", "enable"], "type": "str"},
            "iolog": {"choices": ["disable", "enable"], "type": "str"},
            "logd": {"choices": ["disable", "enable"], "type": "str"},
            "logdb": {"choices": ["disable", "enable"], "type": "str"},
            "logdev": {"choices": ["disable", "enable"], "type": "str"},
            "logfile": {"choices": ["enable", "disable"], "type": "str"},
            "logging": {"choices": ["disable", "enable"], "type": "str"},
            "lrmgr": {"choices": ["disable", "enable"], "type": "str"},
            "objcfg": {"choices": ["disable", "enable"], "type": "str"},
            "report": {"choices": ["disable", "enable"], "type": "str"},
            "rev": {"choices": ["disable", "enable"], "type": "str"},
            "rtmon": {"choices": ["disable", "enable"], "type": "str"},
            "scfw": {"choices": ["disable", "enable"], "type": "str"},
            "scply": {"choices": ["disable", "enable"], "type": "str"},
            "scrmgr": {"choices": ["disable", "enable"], "type": "str"},
            "scvpn": {"choices": ["disable", "enable"], "type": "str"},
            "system": {"choices": ["disable", "enable"], "type": "str"},
            "webport": {"choices": ["disable", "enable"], "type": "str"},
            "incident": {"v_range": [["6.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "aid": {"v_range": [["6.4.1", "7.2.5"]], "choices": ["disable", "enable"], "type": "str"},
            "docker": {"v_range": [["6.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "controller": {"v_range": [["7.0.9", "7.0.12"], ["7.2.4", "7.2.5"], ["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "system_locallog_syslogd2_setting": {
        "stated": false,
        "options": {
            "csv": {"choices": ["disable", "enable"], "type": "str"},
            "facility": {
                "choices": [
                    "kernel", "user", "ntp", "audit", "alert", "clock", "mail", "daemon", "auth", "syslog", "lpr", "news", "uucp", "cron", "authpriv",
                    "ftp", "local0", "local1", "local2", "local3", "local4", "local5", "local6", "local7"
                ],
                "type": "str"
            },
            "severity": {"choices": ["emergency", "alert", "critical", "error", "warning", "notification", "information", "debug"], "type": "str"},
            "status": {"choices": ["disable", "enable"], "type": "str"},
            "syslog-name": {"type": "str"},
            "cert": {"v_range": [["6.4.6", "6.4.14"], ["7.0.1", "7.4.2"]], "type": "str"},
            "reliable": {"v_range": [["6.4.6", "6.4.14"], ["7.0.1", "7.4.2"]], "choices": ["disable", "enable"], "type": "str"},
            "secure-connection": {"v_range": [["6.4.6", "6.4.14"], ["7.0.1", "7.4.2"]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "system_locallog_syslogd3_filter": {
        "stated": false,
        "options": {
            "devcfg": {"choices": ["disable", "enable"], "type": "str"},
            "devops": {"choices": ["disable", "enable"], "type": "str"},
            "diskquota": {"choices": ["disable", "enable"], "type": "str"},
            "dm": {"choices": ["disable", "enable"], "type": "str"},
            "dvm": {"choices": ["disable", "enable"], "type": "str"},
            "ediscovery": {"choices": ["disable", "enable"], "type": "str"},
            "epmgr": {"choices": ["disable", "enable"], "type": "str"},
            "event": {"choices": ["disable", "enable"], "type": "str"},
            "eventmgmt": {"choices": ["disable", "enable"], "type": "str"},
            "faz": {"choices": ["disable", "enable"], "type": "str"},
            "fazha": {"choices": ["disable", "enable"], "type": "str"},
            "fazsys": {"choices": ["disable", "enable"], "type": "str"},
            "fgd": {"choices": ["disable", "enable"], "type": "str"},
            "fgfm": {"choices": ["disable", "enable"], "type": "str"},
            "fips": {"choices": ["disable", "enable"], "type": "str"},
            "fmgws": {"choices": ["disable", "enable"], "type": "str"},
            "fmlmgr": {"choices": ["disable", "enable"], "type": "str"},
            "fmwmgr": {"choices": ["disable", "enable"], "type": "str"},
            "fortiview": {"choices": ["disable", "enable"], "type": "str"},
            "glbcfg": {"choices": ["disable", "enable"], "type": "str"},
            "ha": {"choices": ["disable", "enable"], "type": "str"},
            "hcache": {"choices": ["disable", "enable"], "type": "str"},
            "iolog": {"choices": ["disable", "enable"], "type": "str"},
            "logd": {"choices": ["disable", "enable"], "type": "str"},
            "logdb": {"choices": ["disable", "enable"], "type": "str"},
            "logdev": {"choices": ["disable", "enable"], "type": "str"},
            "logfile": {"choices": ["enable", "disable"], "type": "str"},
            "logging": {"choices": ["disable", "enable"], "type": "str"},
            "lrmgr": {"choices": ["disable", "enable"], "type": "str"},
            "objcfg": {"choices": ["disable", "enable"], "type": "str"},
            "report": {"choices": ["disable", "enable"], "type": "str"},
            "rev": {"choices": ["disable", "enable"], "type": "str"},
            "rtmon": {"choices": ["disable", "enable"], "type": "str"},
            "scfw": {"choices": ["disable", "enable"], "type": "str"},
            "scply": {"choices": ["disable", "enable"], "type": "str"},
            "scrmgr": {"choices": ["disable", "enable"], "type": "str"},
            "scvpn": {"choices": ["disable", "enable"], "type": "str"},
            "system": {"choices": ["disable", "enable"], "type": "str"},
            "webport": {"choices": ["disable", "enable"], "type": "str"},
            "incident": {"v_range": [["6.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "aid": {"v_range": [["6.4.1", "7.2.5"]], "choices": ["disable", "enable"], "type": "str"},
            "docker": {"v_range": [["6.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "controller": {"v_range": [["7.0.9", "7.0.12"], ["7.2.4", "7.2.5"], ["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "system_locallog_syslogd3_setting": {
        "stated": false,
        "options": {
            "csv": {"choices": ["disable", "enable"], "type": "str"},
            "facility": {
                "choices": [
                    "kernel", "user", "ntp", "audit", "alert", "clock", "mail", "daemon", "auth", "syslog", "lpr", "news", "uucp", "cron", "authpriv",
                    "ftp", "local0", "local1", "local2", "local3", "local4", "local5", "local6", "local7"
                ],
                "type": "str"
            },
            "severity": {"choices": ["emergency", "alert", "critical", "error", "warning", "notification", "information", "debug"], "type": "str"},
            "status": {"choices": ["disable", "enable"], "type": "str"},
            "syslog-name": {"type": "str"},
            "cert": {"v_range": [["6.4.6", "6.4.14"], ["7.0.1", "7.4.2"]], "type": "str"},
            "reliable": {"v_range": [["6.4.6", "6.4.14"], ["7.0.1", "7.4.2"]], "choices": ["disable", "enable"], "type": "str"},
            "secure-connection": {"v_range": [["6.4.6", "6.4.14"], ["7.0.1", "7.4.2"]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "system_locallog_syslogd_filter": {
        "stated": false,
        "options": {
            "devcfg": {"choices": ["disable", "enable"], "type": "str"},
            "devops": {"choices": ["disable", "enable"], "type": "str"},
            "diskquota": {"choices": ["disable", "enable"], "type": "str"},
            "dm": {"choices": ["disable", "enable"], "type": "str"},
            "dvm": {"choices": ["disable", "enable"], "type": "str"},
            "ediscovery": {"choices": ["disable", "enable"], "type": "str"},
            "epmgr": {"choices": ["disable", "enable"], "type": "str"},
            "event": {"choices": ["disable", "enable"], "type": "str"},
            "eventmgmt": {"choices": ["disable", "enable"], "type": "str"},
            "faz": {"choices": ["disable", "enable"], "type": "str"},
            "fazha": {"choices": ["disable", "enable"], "type": "str"},
            "fazsys": {"choices": ["disable", "enable"], "type": "str"},
            "fgd": {"choices": ["disable", "enable"], "type": "str"},
            "fgfm": {"choices": ["disable", "enable"], "type": "str"},
            "fips": {"choices": ["disable", "enable"], "type": "str"},
            "fmgws": {"choices": ["disable", "enable"], "type": "str"},
            "fmlmgr": {"choices": ["disable", "enable"], "type": "str"},
            "fmwmgr": {"choices": ["disable", "enable"], "type": "str"},
            "fortiview": {"choices": ["disable", "enable"], "type": "str"},
            "glbcfg": {"choices": ["disable", "enable"], "type": "str"},
            "ha": {"choices": ["disable", "enable"], "type": "str"},
            "hcache": {"choices": ["disable", "enable"], "type": "str"},
            "iolog": {"choices": ["disable", "enable"], "type": "str"},
            "logd": {"choices": ["disable", "enable"], "type": "str"},
            "logdb": {"choices": ["disable", "enable"], "type": "str"},
            "logdev": {"choices": ["disable", "enable"], "type": "str"},
            "logfile": {"choices": ["enable", "disable"], "type": "str"},
            "logging": {"choices": ["disable", "enable"], "type": "str"},
            "lrmgr": {"choices": ["disable", "enable"], "type": "str"},
            "objcfg": {"choices": ["disable", "enable"], "type": "str"},
            "report": {"choices": ["disable", "enable"], "type": "str"},
            "rev": {"choices": ["disable", "enable"], "type": "str"},
            "rtmon": {"choices": ["disable", "enable"], "type": "str"},
            "scfw": {"choices": ["disable", "enable"], "type": "str"},
            "scply": {"choices": ["disable", "enable"], "type": "str"},
            "scrmgr": {"choices": ["disable", "enable"], "type": "str"},
            "scvpn": {"choices": ["disable", "enable"], "type": "str"},
            "system": {"choices": ["disable", "enable"], "type": "str"},
            "webport": {"choices": ["disable", "enable"], "type": "str"},
            "incident": {"v_range": [["6.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "aid": {"v_range": [["6.4.1", "7.2.5"]], "choices": ["disable", "enable"], "type": "str"},
            "docker": {"v_range": [["6.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "controller": {"v_range": [["7.0.9", "7.0.12"], ["7.2.4", "7.2.5"], ["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "system_locallog_syslogd_setting": {
        "stated": false,
        "options": {
            "csv": {"choices": ["disable", "enable"], "type": "str"},
            "facility": {
                "choices": [
                    "kernel", "user", "ntp", "audit", "alert", "clock", "mail", "daemon", "auth", "syslog", "lpr", "news", "uucp", "cron", "authpriv",
                    "ftp", "local0", "local1", "local2", "local3", "local4", "local5", "local6", "local7"
                ],
                "type": "str"
            },
            "severity": {"choices": ["emergency", "alert", "critical", "error", "warning", "notification", "information", "debug"], "type": "str"},
            "status": {"choices": ["disable", "enable"], "type": "str"},
            "syslog-name": {"type": "str"},
            "cert": {"v_range": [["6.4.6", "6.4.14"], ["7.0.1", "7.4.2"]], "type": "str"},
            "reliable": {"v_range": [["6.4.6", "6.4.14"], ["7.0.1", "7.4.2"]], "choices": ["disable", "enable"], "type": "str"},
            "secure-connection": {"v_range": [["6.4.6", "6.4.14"], ["7.0.1", "7.4.2"]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "system_log_alert": {"stated": false, "options": {"max-alert-count": {"type": "int"}}},
    "system_log_devicedisable": {
        "stated": true,
        "options": {
            "TTL": {"v_range": [["6.4.4", ""]], "type": "str"},
            "device": {"v_range": [["6.4.4", ""]], "type": "str"},
            "id": {"v_range": [["6.4.4", ""]], "required": true, "type": "int"}
        }
    },
    "system_log_fospolicystats": {
        "stated": false,
        "options": {
            "retention-days": {"v_range": [["7.0.2", ""]], "type": "int"},
            "sampling-interval": {"v_range": [["7.0.2", ""]], "type": "int"},
            "status": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "system_log_interfacestats": {
        "stated": false,
        "options": {
            "retention-days": {"v_range": [["6.2.1", ""]], "type": "int"},
            "sampling-interval": {"v_range": [["6.2.1", ""]], "type": "int"},
            "status": {"v_range": [["6.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "billing-report": {"v_range": [["6.4.3", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "system_log_ioc": {
        "stated": false,
        "options": {
            "notification": {"choices": ["disable", "enable"], "type": "str"},
            "notification-throttle": {"type": "int"},
            "rescan-max-runner": {"type": "int"},
            "rescan-run-at": {"type": "int"},
            "rescan-status": {"choices": ["disable", "enable"], "type": "str"},
            "status": {"choices": ["disable", "enable"], "type": "str"}
        }
    },
    "system_log_maildomain": {
        "stated": true,
        "options": {"devices": {"type": "str"}, "domain": {"type": "str"}, "id": {"required": true, "type": "int"}, "vdom": {"type": "str"}}
    },
    "system_log_ratelimit": {
        "stated": false,
        "options": {
            "device": {
                "v_range": [["6.4.8", "7.0.2"]],
                "type": "list",
                "options": {
                    "device": {"v_range": [["6.4.8", "7.0.2"]], "type": "str"},
                    "filter-type": {"v_range": [["6.4.8", "7.0.2"]], "choices": ["devid"], "type": "str"},
                    "id": {"v_range": [["6.4.8", "7.0.2"]], "type": "int"},
                    "ratelimit": {"v_range": [["6.4.8", "7.0.2"]], "type": "int"}
                },
                "elements": "dict"
            },
            "device-ratelimit-default": {"v_range": [["6.4.8", ""]], "type": "int"},
            "mode": {"v_range": [["6.4.8", ""]], "choices": ["disable", "manual"], "type": "str"},
            "system-ratelimit": {"v_range": [["6.4.8", ""]], "type": "int"},
            "ratelimits": {
                "v_range": [["7.0.3", ""]],
                "type": "list",
                "options": {
                    "filter": {"v_range": [["7.0.3", ""]], "type": "str"},
                    "filter-type": {"v_range": [["7.0.3", ""]], "choices": ["devid", "adom"], "type": "str"},
                    "id": {"v_range": [["7.0.3", ""]], "type": "int"},
                    "ratelimit": {"v_range": [["7.0.3", ""]], "type": "int"}
                },
                "elements": "dict"
            }
        }
    },
    "system_log_ratelimit_device": {
        "stated": true,
        "options": {
            "device": {"v_range": [["6.4.8", "7.0.2"]], "type": "str"},
            "filter-type": {"v_range": [["6.4.8", "7.0.2"]], "choices": ["devid"], "type": "str"},
            "id": {"v_range": [["6.4.8", "7.0.2"]], "required": true, "type": "int"},
            "ratelimit": {"v_range": [["6.4.8", "7.0.2"]], "type": "int"}
        }
    },
    "system_log_ratelimit_ratelimits": {
        "stated": true,
        "options": {
            "filter": {"v_range": [["7.0.3", ""]], "type": "str"},
            "filter-type": {"v_range": [["7.0.3", ""]], "choices": ["devid", "adom"], "type": "str"},
            "id": {"v_range": [["7.0.3", ""]], "required": true, "type": "int"},
            "ratelimit": {"v_range": [["7.0.3", ""]], "type": "int"}
        }
    },
    "system_log_settings": {
        "stated": false,
        "options": {
            "FAC-custom-field1": {"type": "str"},
            "FAZ-custom-field1": {"v_range": [["6.0.0", "7.2.0"]], "type": "str"},
            "FCH-custom-field1": {"type": "str"},
            "FCT-custom-field1": {"type": "str"},
            "FDD-custom-field1": {"type": "str"},
            "FGT-custom-field1": {"type": "str"},
            "FMG-custom-field1": {"v_range": [["6.0.0", "7.2.0"]], "type": "str"},
            "FML-custom-field1": {"type": "str"},
            "FPX-custom-field1": {"type": "str"},
            "FSA-custom-field1": {"type": "str"},
            "FWB-custom-field1": {"type": "str"},
            "browse-max-logfiles": {"type": "int"},
            "dns-resolve-dstip": {"choices": ["disable", "enable"], "type": "str"},
            "download-max-logs": {"type": "int"},
            "ha-auto-migrate": {"choices": ["disable", "enable"], "type": "str"},
            "import-max-logfiles": {"type": "int"},
            "log-file-archive-name": {"choices": ["basic", "extended"], "type": "str"},
            "rolling-analyzer": {
                "type": "dict",
                "options": {
                    "days": {"type": "list", "choices": ["sun", "mon", "tue", "wed", "thu", "fri", "sat"], "elements": "str"},
                    "del-files": {"choices": ["disable", "enable"], "type": "str"},
                    "directory": {"type": "str"},
                    "file-size": {"type": "int"},
                    "gzip-format": {"choices": ["disable", "enable"], "type": "str"},
                    "hour": {"type": "int"},
                    "ip": {"v_range": [["6.0.0", "7.0.12"]], "type": "str"},
                    "ip2": {"v_range": [["6.0.0", "7.0.12"]], "type": "str"},
                    "ip3": {"v_range": [["6.0.0", "7.0.12"]], "type": "str"},
                    "log-format": {"choices": ["native", "text", "csv"], "type": "str"},
                    "min": {"type": "int"},
                    "password": {"no_log": true, "type": "raw"},
                    "password2": {"no_log": true, "type": "raw"},
                    "password3": {"no_log": true, "type": "raw"},
                    "server-type": {"choices": ["ftp", "sftp", "scp"], "type": "str"},
                    "upload": {"choices": ["disable", "enable"], "type": "str"},
                    "upload-hour": {"type": "int"},
                    "upload-mode": {"choices": ["backup", "mirror"], "type": "str"},
                    "upload-trigger": {"choices": ["on-roll", "on-schedule"], "type": "str"},
                    "username": {"type": "str"},
                    "username2": {"type": "str"},
                    "username3": {"type": "str"},
                    "when": {"choices": ["none", "daily", "weekly"], "type": "str"},
                    "port": {"v_range": [["6.2.2", ""]], "type": "int"},
                    "port2": {"v_range": [["6.2.2", ""]], "type": "int"},
                    "port3": {"v_range": [["6.2.2", ""]], "type": "int"},
                    "rolling-upgrade-status": {"v_range": [["7.0.3", ""]], "type": "int"},
                    "server": {"v_range": [["7.2.0", ""]], "type": "str"},
                    "server2": {"v_range": [["7.2.0", ""]], "type": "str"},
                    "server3": {"v_range": [["7.2.0", ""]], "type": "str"}
                }
            },
            "rolling-local": {
                "type": "dict",
                "options": {
                    "days": {"type": "list", "choices": ["sun", "mon", "tue", "wed", "thu", "fri", "sat"], "elements": "str"},
                    "del-files": {"choices": ["disable", "enable"], "type": "str"},
                    "directory": {"type": "str"},
                    "file-size": {"type": "int"},
                    "gzip-format": {"choices": ["disable", "enable"], "type": "str"},
                    "hour": {"type": "int"},
                    "ip": {"v_range": [["6.0.0", "7.0.12"]], "type": "str"},
                    "ip2": {"v_range": [["6.0.0", "7.0.12"]], "type": "str"},
                    "ip3": {"v_range": [["6.0.0", "7.0.12"]], "type": "str"},
                    "log-format": {"choices": ["native", "text", "csv"], "type": "str"},
                    "min": {"type": "int"},
                    "password": {"no_log": true, "type": "raw"},
                    "password2": {"no_log": true, "type": "raw"},
                    "password3": {"no_log": true, "type": "raw"},
                    "server-type": {"choices": ["ftp", "sftp", "scp"], "type": "str"},
                    "upload": {"choices": ["disable", "enable"], "type": "str"},
                    "upload-hour": {"type": "int"},
                    "upload-mode": {"choices": ["backup", "mirror"], "type": "str"},
                    "upload-trigger": {"choices": ["on-roll", "on-schedule"], "type": "str"},
                    "username": {"type": "str"},
                    "username2": {"type": "str"},
                    "username3": {"type": "str"},
                    "when": {"choices": ["none", "daily", "weekly"], "type": "str"},
                    "port": {"v_range": [["6.2.2", ""]], "type": "int"},
                    "port2": {"v_range": [["6.2.2", ""]], "type": "int"},
                    "port3": {"v_range": [["6.2.2", ""]], "type": "int"},
                    "rolling-upgrade-status": {"v_range": [["7.0.3", ""]], "type": "int"},
                    "server": {"v_range": [["7.2.0", ""]], "type": "str"},
                    "server2": {"v_range": [["7.2.0", ""]], "type": "str"},
                    "server3": {"v_range": [["7.2.0", ""]], "type": "str"}
                }
            },
            "rolling-regular": {
                "type": "dict",
                "options": {
                    "days": {"type": "list", "choices": ["sun", "mon", "tue", "wed", "thu", "fri", "sat"], "elements": "str"},
                    "del-files": {"choices": ["disable", "enable"], "type": "str"},
                    "directory": {"type": "str"},
                    "file-size": {"type": "int"},
                    "gzip-format": {"choices": ["disable", "enable"], "type": "str"},
                    "hour": {"type": "int"},
                    "ip": {"v_range": [["6.0.0", "7.0.12"]], "type": "str"},
                    "ip2": {"v_range": [["6.0.0", "7.0.12"]], "type": "str"},
                    "ip3": {"v_range": [["6.0.0", "7.0.12"]], "type": "str"},
                    "log-format": {"choices": ["native", "text", "csv"], "type": "str"},
                    "min": {"type": "int"},
                    "password": {"no_log": true, "type": "raw"},
                    "password2": {"no_log": true, "type": "raw"},
                    "password3": {"no_log": true, "type": "raw"},
                    "server-type": {"choices": ["ftp", "sftp", "scp"], "type": "str"},
                    "upload": {"choices": ["disable", "enable"], "type": "str"},
                    "upload-hour": {"type": "int"},
                    "upload-mode": {"choices": ["backup", "mirror"], "type": "str"},
                    "upload-trigger": {"choices": ["on-roll", "on-schedule"], "type": "str"},
                    "username": {"type": "str"},
                    "username2": {"type": "str"},
                    "username3": {"type": "str"},
                    "when": {"choices": ["none", "daily", "weekly"], "type": "str"},
                    "port": {"v_range": [["6.2.2", ""]], "type": "int"},
                    "port2": {"v_range": [["6.2.2", ""]], "type": "int"},
                    "port3": {"v_range": [["6.2.2", ""]], "type": "int"},
                    "rolling-upgrade-status": {"v_range": [["7.0.3", ""]], "type": "int"},
                    "server": {"v_range": [["7.2.0", ""]], "type": "str"},
                    "server2": {"v_range": [["7.2.0", ""]], "type": "str"},
                    "server3": {"v_range": [["7.2.0", ""]], "type": "str"}
                }
            },
            "sync-search-timeout": {"type": "int"},
            "keep-dev-logs": {"v_range": [["6.4.7", "6.4.14"], ["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "device-auto-detect": {"v_range": [["7.0.10", "7.0.12"], ["7.2.4", "7.2.5"], ["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "unencrypted-logging": {"v_range": [["7.0.10", "7.0.12"], ["7.2.4", "7.2.5"], ["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "log-interval-dev-no-logging": {"v_range": [["7.2.5", "7.2.5"], ["7.4.2", ""]], "type": "int"},
            "log-upload-interval-dev-no-logging": {"v_range": [["7.2.5", "7.2.5"], ["7.4.2", ""]], "type": "int"}
        }
    },
    "system_log_settings_rollinganalyzer": {
        "stated": false,
        "options": {
            "days": {"type": "list", "choices": ["sun", "mon", "tue", "wed", "thu", "fri", "sat"], "elements": "str"},
            "del-files": {"choices": ["disable", "enable"], "type": "str"},
            "directory": {"type": "str"},
            "file-size": {"type": "int"},
            "gzip-format": {"choices": ["disable", "enable"], "type": "str"},
            "hour": {"type": "int"},
            "ip": {"v_range": [["6.0.0", "7.0.12"]], "type": "str"},
            "ip2": {"v_range": [["6.0.0", "7.0.12"]], "type": "str"},
            "ip3": {"v_range": [["6.0.0", "7.0.12"]], "type": "str"},
            "log-format": {"choices": ["native", "text", "csv"], "type": "str"},
            "min": {"type": "int"},
            "password": {"no_log": true, "type": "raw"},
            "password2": {"no_log": true, "type": "raw"},
            "password3": {"no_log": true, "type": "raw"},
            "server-type": {"choices": ["ftp", "sftp", "scp"], "type": "str"},
            "upload": {"choices": ["disable", "enable"], "type": "str"},
            "upload-hour": {"type": "int"},
            "upload-mode": {"choices": ["backup", "mirror"], "type": "str"},
            "upload-trigger": {"choices": ["on-roll", "on-schedule"], "type": "str"},
            "username": {"type": "str"},
            "username2": {"type": "str"},
            "username3": {"type": "str"},
            "when": {"choices": ["none", "daily", "weekly"], "type": "str"},
            "port": {"v_range": [["6.2.2", ""]], "type": "int"},
            "port2": {"v_range": [["6.2.2", ""]], "type": "int"},
            "port3": {"v_range": [["6.2.2", ""]], "type": "int"},
            "rolling-upgrade-status": {"v_range": [["7.0.3", ""]], "type": "int"},
            "server": {"v_range": [["7.2.0", ""]], "type": "str"},
            "server2": {"v_range": [["7.2.0", ""]], "type": "str"},
            "server3": {"v_range": [["7.2.0", ""]], "type": "str"}
        }
    },
    "system_log_settings_rollinglocal": {
        "stated": false,
        "options": {
            "days": {"type": "list", "choices": ["sun", "mon", "tue", "wed", "thu", "fri", "sat"], "elements": "str"},
            "del-files": {"choices": ["disable", "enable"], "type": "str"},
            "directory": {"type": "str"},
            "file-size": {"type": "int"},
            "gzip-format": {"choices": ["disable", "enable"], "type": "str"},
            "hour": {"type": "int"},
            "ip": {"v_range": [["6.0.0", "7.0.12"]], "type": "str"},
            "ip2": {"v_range": [["6.0.0", "7.0.12"]], "type": "str"},
            "ip3": {"v_range": [["6.0.0", "7.0.12"]], "type": "str"},
            "log-format": {"choices": ["native", "text", "csv"], "type": "str"},
            "min": {"type": "int"},
            "password": {"no_log": true, "type": "raw"},
            "password2": {"no_log": true, "type": "raw"},
            "password3": {"no_log": true, "type": "raw"},
            "server-type": {"choices": ["ftp", "sftp", "scp"], "type": "str"},
            "upload": {"choices": ["disable", "enable"], "type": "str"},
            "upload-hour": {"type": "int"},
            "upload-mode": {"choices": ["backup", "mirror"], "type": "str"},
            "upload-trigger": {"choices": ["on-roll", "on-schedule"], "type": "str"},
            "username": {"type": "str"},
            "username2": {"type": "str"},
            "username3": {"type": "str"},
            "when": {"choices": ["none", "daily", "weekly"], "type": "str"},
            "port": {"v_range": [["6.2.2", ""]], "type": "int"},
            "port2": {"v_range": [["6.2.2", ""]], "type": "int"},
            "port3": {"v_range": [["6.2.2", ""]], "type": "int"},
            "rolling-upgrade-status": {"v_range": [["7.0.3", ""]], "type": "int"},
            "server": {"v_range": [["7.2.0", ""]], "type": "str"},
            "server2": {"v_range": [["7.2.0", ""]], "type": "str"},
            "server3": {"v_range": [["7.2.0", ""]], "type": "str"}
        }
    },
    "system_log_settings_rollingregular": {
        "stated": false,
        "options": {
            "days": {"type": "list", "choices": ["sun", "mon", "tue", "wed", "thu", "fri", "sat"], "elements": "str"},
            "del-files": {"choices": ["disable", "enable"], "type": "str"},
            "directory": {"type": "str"},
            "file-size": {"type": "int"},
            "gzip-format": {"choices": ["disable", "enable"], "type": "str"},
            "hour": {"type": "int"},
            "ip": {"v_range": [["6.0.0", "7.0.12"]], "type": "str"},
            "ip2": {"v_range": [["6.0.0", "7.0.12"]], "type": "str"},
            "ip3": {"v_range": [["6.0.0", "7.0.12"]], "type": "str"},
            "log-format": {"choices": ["native", "text", "csv"], "type": "str"},
            "min": {"type": "int"},
            "password": {"no_log": true, "type": "raw"},
            "password2": {"no_log": true, "type": "raw"},
            "password3": {"no_log": true, "type": "raw"},
            "server-type": {"choices": ["ftp", "sftp", "scp"], "type": "str"},
            "upload": {"choices": ["disable", "enable"], "type": "str"},
            "upload-hour": {"type": "int"},
            "upload-mode": {"choices": ["backup", "mirror"], "type": "str"},
            "upload-trigger": {"choices": ["on-roll", "on-schedule"], "type": "str"},
            "username": {"type": "str"},
            "username2": {"type": "str"},
            "username3": {"type": "str"},
            "when": {"choices": ["none", "daily", "weekly"], "type": "str"},
            "port": {"v_range": [["6.2.2", ""]], "type": "int"},
            "port2": {"v_range": [["6.2.2", ""]], "type": "int"},
            "port3": {"v_range": [["6.2.2", ""]], "type": "int"},
            "rolling-upgrade-status": {"v_range": [["7.0.3", ""]], "type": "int"},
            "server": {"v_range": [["7.2.0", ""]], "type": "str"},
            "server2": {"v_range": [["7.2.0", ""]], "type": "str"},
            "server3": {"v_range": [["7.2.0", ""]], "type": "str"}
        }
    },
    "system_log_topology": {
        "stated": false,
        "options": {
            "max-depth": {"v_range": [["6.4.7", "6.4.14"], ["7.0.2", ""]], "type": "int"},
            "max-depth-share": {"v_range": [["6.4.7", "6.4.14"], ["7.0.2", ""]], "type": "int"}
        }
    },
    "system_log_ueba": {
        "stated": false,
        "options": {
            "ip-only-ep": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ip-unique-scope": {"v_range": [["7.4.3", ""]], "choices": ["adom", "vdom"], "type": "str"}
        }
    },
    "system_logfetch_clientprofile": {
        "stated": true,
        "options": {
            "client-adom": {"v_range": [["6.0.0", "7.2.1"]], "type": "str"},
            "data-range": {"v_range": [["6.0.0", "7.2.1"]], "choices": ["custom"], "type": "str"},
            "data-range-value": {"v_range": [["6.0.0", "7.2.1"]], "type": "int"},
            "device-filter": {
                "v_range": [["6.0.0", "7.2.1"]],
                "type": "list",
                "options": {
                    "adom": {"v_range": [["6.0.0", "7.2.1"]], "type": "str"},
                    "device": {"v_range": [["6.0.0", "7.2.1"]], "type": "str"},
                    "id": {"v_range": [["6.0.0", "7.2.1"]], "type": "int"},
                    "vdom": {"v_range": [["6.0.0", "7.2.1"]], "type": "str"}
                },
                "elements": "dict"
            },
            "end-time": {"v_range": [["6.0.0", "7.2.1"]], "type": "raw"},
            "id": {"v_range": [["6.0.0", "7.2.1"]], "required": true, "type": "int"},
            "index-fetch-logs": {"v_range": [["6.0.0", "7.2.1"]], "choices": ["disable", "enable"], "type": "str"},
            "log-filter": {
                "v_range": [["6.0.0", "7.2.1"]],
                "type": "list",
                "options": {
                    "field": {"v_range": [["6.0.0", "7.2.1"]], "type": "str"},
                    "id": {"v_range": [["6.0.0", "7.2.1"]], "type": "int"},
                    "oper": {
                        "v_range": [["6.0.0", "7.2.1"]],
                        "choices": ["=", "!=", "<", ">", "<=", ">=", "contain", "not-contain", "match"],
                        "type": "str"
                    },
                    "value": {"v_range": [["6.0.0", "7.2.1"]], "type": "str"}
                },
                "elements": "dict"
            },
            "log-filter-logic": {"v_range": [["6.0.0", "7.2.1"]], "choices": ["and", "or"], "type": "str"},
            "log-filter-status": {"v_range": [["6.0.0", "7.2.1"]], "choices": ["disable", "enable"], "type": "str"},
            "name": {"v_range": [["6.0.0", "7.2.1"]], "type": "str"},
            "password": {"v_range": [["6.0.0", "7.2.1"]], "no_log": true, "type": "raw"},
            "secure-connection": {"v_range": [["6.0.0", "7.2.1"]], "choices": ["disable", "enable"], "type": "str"},
            "server-adom": {"v_range": [["6.0.0", "7.2.1"]], "type": "str"},
            "server-ip": {"v_range": [["6.0.0", "7.2.1"]], "type": "str"},
            "start-time": {"v_range": [["6.0.0", "7.2.1"]], "type": "raw"},
            "sync-adom-config": {"v_range": [["6.0.0", "7.2.1"]], "choices": ["disable", "enable"], "type": "str"},
            "user": {"v_range": [["6.0.0", "7.2.1"]], "type": "str"},
            "peer-cert-cn": {"v_range": [["7.0.3", "7.2.1"]], "type": "str"}
        }
    },
    "system_logfetch_clientprofile_devicefilter": {
        "stated": true,
        "options": {
            "adom": {"v_range": [["6.0.0", "7.2.1"]], "type": "str"},
            "device": {"v_range": [["6.0.0", "7.2.1"]], "type": "str"},
            "id": {"v_range": [["6.0.0", "7.2.1"]], "required": true, "type": "int"},
            "vdom": {"v_range": [["6.0.0", "7.2.1"]], "type": "str"}
        }
    },
    "system_logfetch_clientprofile_logfilter": {
        "stated": true,
        "options": {
            "field": {"v_range": [["6.0.0", "7.2.1"]], "type": "str"},
            "id": {"v_range": [["6.0.0", "7.2.1"]], "required": true, "type": "int"},
            "oper": {"v_range": [["6.0.0", "7.2.1"]], "choices": ["=", "!=", "<", ">", "<=", ">=", "contain", "not-contain", "match"], "type": "str"},
            "value": {"v_range": [["6.0.0", "7.2.1"]], "type": "str"}
        }
    },
    "system_logfetch_serversettings": {
        "stated": false,
        "options": {
            "max-conn-per-session": {"v_range": [["6.0.0", "7.2.1"]], "type": "int"},
            "max-sessions": {"v_range": [["6.0.0", "7.2.1"]], "type": "int"},
            "session-timeout": {"v_range": [["6.0.0", "7.2.1"]], "type": "int"}
        }
    },
    "system_mail": {
        "stated": true,
        "options": {
            "auth": {"choices": ["disable", "enable"], "type": "str"},
            "id": {"required": true, "type": "str"},
            "passwd": {"no_log": true, "type": "raw"},
            "port": {"type": "int"},
            "secure-option": {"choices": ["default", "none", "smtps", "starttls"], "type": "str"},
            "server": {"type": "str"},
            "user": {"type": "str"},
            "auth-type": {"v_range": [["6.4.6", ""]], "choices": ["psk", "certificate"], "type": "str"},
            "local-cert": {"v_range": [["6.4.6", ""]], "type": "str"},
            "from": {"v_range": [["7.0.7", "7.0.12"], ["7.2.2", ""]], "type": "str"}
        }
    },
    "system_mcpolicydisabledadoms": {"stated": true, "options": {"adom-name": {"v_range": [["6.2.3", ""]], "type": "str"}}},
    "system_meta": {
        "stated": true,
        "options": {
            "name": {"required": true, "type": "str"},
            "sys_meta_fields": {
                "type": "list",
                "options": {"fieldlength": {"type": "int"}, "importance": {"choices": ["optional", "required"], "type": "str"}, "name": {"type": "str"}},
                "elements": "dict"
            }
        }
    },
    "system_meta_sysmetafields": {
        "stated": true,
        "options": {
            "fieldlength": {"type": "int"},
            "importance": {"choices": ["optional", "required"], "type": "str"},
            "name": {"required": true, "type": "str"}
        }
    },
    "system_metadata_admins": {
        "stated": true,
        "options": {
            "fieldlength": {"choices": ["20", "50", "255"], "type": "str"},
            "fieldname": {"required": true, "type": "str"},
            "importance": {"choices": ["optional", "required"], "type": "str"},
            "status": {"choices": ["disabled", "enabled"], "type": "str"}
        }
    },
    "system_npu": {
        "stated": false,
        "options": {
            "capwap-offload": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "dedicated-management-affinity": {"v_range": [["7.0.1", ""]], "type": "str"},
            "dedicated-management-cpu": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "fastpath": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "fp-anomaly": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "type": "dict",
                "options": {
                    "esp-minlen-err": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["drop", "trap-to-host"], "type": "str"},
                    "icmp-csum-err": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["drop", "trap-to-host"], "type": "str"},
                    "icmp-minlen-err": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["drop", "trap-to-host"], "type": "str"},
                    "ipv4-csum-err": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["drop", "trap-to-host"], "type": "str"},
                    "ipv4-ihl-err": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["drop", "trap-to-host"], "type": "str"},
                    "ipv4-len-err": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["drop", "trap-to-host"], "type": "str"},
                    "ipv4-opt-err": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["drop", "trap-to-host"], "type": "str"},
                    "ipv4-ttlzero-err": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["drop", "trap-to-host"], "type": "str"},
                    "ipv4-ver-err": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["drop", "trap-to-host"], "type": "str"},
                    "ipv6-exthdr-len-err": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["drop", "trap-to-host"], "type": "str"},
                    "ipv6-exthdr-order-err": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["drop", "trap-to-host"], "type": "str"},
                    "ipv6-ihl-err": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["drop", "trap-to-host"], "type": "str"},
                    "ipv6-plen-zero": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["drop", "trap-to-host"], "type": "str"},
                    "ipv6-ver-err": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["drop", "trap-to-host"], "type": "str"},
                    "tcp-csum-err": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["drop", "trap-to-host"], "type": "str"},
                    "tcp-hlen-err": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["drop", "trap-to-host"], "type": "str"},
                    "tcp-plen-err": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["drop", "trap-to-host"], "type": "str"},
                    "udp-csum-err": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["drop", "trap-to-host"], "type": "str"},
                    "udp-hlen-err": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["drop", "trap-to-host"], "type": "str"},
                    "udp-len-err": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["drop", "trap-to-host"], "type": "str"},
                    "udp-plen-err": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["drop", "trap-to-host"], "type": "str"},
                    "udplite-cover-err": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["drop", "trap-to-host"], "type": "str"},
                    "udplite-csum-err": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["drop", "trap-to-host"], "type": "str"},
                    "unknproto-minlen-err": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["drop", "trap-to-host"], "type": "str"},
                    "tcp-fin-only": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["allow", "drop", "trap-to-host"], "type": "str"},
                    "ipv4-optsecurity": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["allow", "drop", "trap-to-host"], "type": "str"},
                    "ipv6-optralert": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["allow", "drop", "trap-to-host"], "type": "str"},
                    "tcp-syn-fin": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["allow", "drop", "trap-to-host"], "type": "str"},
                    "ipv4-proto-err": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["allow", "drop", "trap-to-host"], "type": "str"},
                    "ipv6-saddr-err": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["allow", "drop", "trap-to-host"], "type": "str"},
                    "icmp-frag": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["allow", "drop", "trap-to-host"], "type": "str"},
                    "ipv4-optssrr": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["allow", "drop", "trap-to-host"], "type": "str"},
                    "ipv6-opthomeaddr": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["allow", "drop", "trap-to-host"], "type": "str"},
                    "udp-land": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["allow", "drop", "trap-to-host"], "type": "str"},
                    "ipv6-optinvld": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["allow", "drop", "trap-to-host"], "type": "str"},
                    "tcp-fin-noack": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["allow", "drop", "trap-to-host"], "type": "str"},
                    "ipv6-proto-err": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["allow", "drop", "trap-to-host"], "type": "str"},
                    "tcp-land": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["allow", "drop", "trap-to-host"], "type": "str"},
                    "ipv4-unknopt": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["allow", "drop", "trap-to-host"], "type": "str"},
                    "ipv4-optstream": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["allow", "drop", "trap-to-host"], "type": "str"},
                    "ipv6-optjumbo": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["allow", "drop", "trap-to-host"], "type": "str"},
                    "icmp-land": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["allow", "drop", "trap-to-host"], "type": "str"},
                    "tcp-winnuke": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["allow", "drop", "trap-to-host"], "type": "str"},
                    "ipv6-daddr-err": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["allow", "drop", "trap-to-host"], "type": "str"},
                    "ipv4-land": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["allow", "drop", "trap-to-host"], "type": "str"},
                    "ipv6-opttunnel": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["allow", "drop", "trap-to-host"], "type": "str"},
                    "tcp-no-flag": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["allow", "drop", "trap-to-host"], "type": "str"},
                    "ipv6-land": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["allow", "drop", "trap-to-host"], "type": "str"},
                    "ipv4-optlsrr": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["allow", "drop", "trap-to-host"], "type": "str"},
                    "ipv4-opttimestamp": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["allow", "drop", "trap-to-host"], "type": "str"},
                    "ipv4-optrr": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["allow", "drop", "trap-to-host"], "type": "str"},
                    "ipv6-optnsap": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["allow", "drop", "trap-to-host"], "type": "str"},
                    "ipv6-unknopt": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["allow", "drop", "trap-to-host"], "type": "str"},
                    "tcp-syn-data": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["allow", "drop", "trap-to-host"], "type": "str"},
                    "ipv6-optendpid": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["allow", "drop", "trap-to-host"], "type": "str"},
                    "gtpu-plen-err": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["drop", "trap-to-host"], "type": "str"},
                    "vxlan-minlen-err": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["drop", "trap-to-host"], "type": "str"},
                    "capwap-minlen-err": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["drop", "trap-to-host"], "type": "str"},
                    "gre-csum-err": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["drop", "trap-to-host"], "type": "str"},
                    "nvgre-minlen-err": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["drop", "trap-to-host"], "type": "str"},
                    "sctp-l4len-err": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["drop", "trap-to-host"], "type": "str"},
                    "tcp-hlenvsl4len-err": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["drop", "trap-to-host"], "type": "str"},
                    "sctp-crc-err": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["drop", "trap-to-host"], "type": "str"},
                    "sctp-clen-err": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["drop", "trap-to-host"], "type": "str"},
                    "uesp-minlen-err": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["drop", "trap-to-host"], "type": "str"},
                    "sctp-csum-err": {"v_range": [["7.2.5", "7.2.5"], ["7.4.3", ""]], "choices": ["allow", "drop", "trap-to-host"], "type": "str"}
                }
            },
            "gtp-enhanced-cpu-range": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["0", "1", "2"], "type": "str"},
            "gtp-enhanced-mode": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "host-shortcut-mode": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["bi-directional", "host-shortcut"], "type": "str"},
            "htx-gtse-quota": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "choices": [
                    "100Mbps", "200Mbps", "300Mbps", "400Mbps", "500Mbps", "600Mbps", "700Mbps", "800Mbps", "900Mbps", "1Gbps", "2Gbps", "4Gbps",
                    "8Gbps", "10Gbps"
                ],
                "type": "str"
            },
            "intf-shaping-offload": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "iph-rsvd-re-cksum": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ipsec-dec-subengine-mask": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "str"},
            "ipsec-enc-subengine-mask": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "str"},
            "ipsec-inbound-cache": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ipsec-mtu-override": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ipsec-over-vlink": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "isf-np-queues": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "type": "dict",
                "options": {
                    "cos0": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "str"},
                    "cos1": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "str"},
                    "cos2": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "str"},
                    "cos3": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "str"},
                    "cos4": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "str"},
                    "cos5": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "str"},
                    "cos6": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "str"},
                    "cos7": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "str"}
                }
            },
            "lag-out-port-select": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "mcast-session-accounting": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "choices": ["disable", "session-based", "tpe-based"],
                "type": "str"
            },
            "np6-cps-optimization-mode": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "per-session-accounting": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "choices": ["enable", "disable", "enable-by-log", "all-enable", "traffic-log-only"],
                "type": "str"
            },
            "port-cpu-map": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "type": "list",
                "options": {
                    "cpu-core": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "str"},
                    "interface": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "str"}
                },
                "elements": "dict"
            },
            "port-npu-map": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "type": "list",
                "options": {
                    "interface": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "str"},
                    "npu-group-index": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"}
                },
                "elements": "dict"
            },
            "priority-protocol": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "type": "dict",
                "options": {
                    "bfd": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "bgp": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "slbc": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"}
                }
            },
            "qos-mode": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["disable", "priority", "round-robin"], "type": "str"},
            "rdp-offload": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "recover-np6-link": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "session-denied-offload": {"v_range": [["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "sse-backpressure": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "strip-clear-text-padding": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "strip-esp-padding": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "sw-eh-hash": {
                "v_range": [["7.0.1", ""]],
                "type": "dict",
                "options": {
                    "computation": {"v_range": [["7.0.1", ""]], "choices": ["xor16", "xor8", "xor4", "crc16"], "type": "str"},
                    "destination-ip-lower-16": {"v_range": [["7.0.1", ""]], "choices": ["include", "exclude"], "type": "str"},
                    "destination-ip-upper-16": {"v_range": [["7.0.1", ""]], "choices": ["include", "exclude"], "type": "str"},
                    "destination-port": {"v_range": [["7.0.1", ""]], "choices": ["include", "exclude"], "type": "str"},
                    "ip-protocol": {"v_range": [["7.0.1", ""]], "choices": ["include", "exclude"], "type": "str"},
                    "netmask-length": {"v_range": [["7.0.1", ""]], "type": "int"},
                    "source-ip-lower-16": {"v_range": [["7.0.1", ""]], "choices": ["include", "exclude"], "type": "str"},
                    "source-ip-upper-16": {"v_range": [["7.0.1", ""]], "choices": ["include", "exclude"], "type": "str"},
                    "source-port": {"v_range": [["7.0.1", ""]], "choices": ["include", "exclude"], "type": "str"}
                }
            },
            "sw-np-bandwidth": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "choices": ["0G", "2G", "4G", "5G", "6G", "7G", "8G", "9G"],
                "type": "str"
            },
            "switch-np-hash": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["src-ip", "dst-ip", "src-dst-ip"], "type": "str"},
            "uesp-offload": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "np-queues": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "type": "dict",
                "options": {
                    "ethernet-type": {
                        "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                        "type": "list",
                        "options": {
                            "name": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "str"},
                            "queue": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"},
                            "type": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"},
                            "weight": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"}
                        },
                        "elements": "dict"
                    },
                    "ip-protocol": {
                        "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                        "type": "list",
                        "options": {
                            "name": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "str"},
                            "protocol": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"},
                            "queue": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"},
                            "weight": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"}
                        },
                        "elements": "dict"
                    },
                    "ip-service": {
                        "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                        "type": "list",
                        "options": {
                            "dport": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"},
                            "name": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "str"},
                            "protocol": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"},
                            "queue": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"},
                            "sport": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"},
                            "weight": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"}
                        },
                        "elements": "dict"
                    },
                    "profile": {
                        "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                        "type": "list",
                        "options": {
                            "cos0": {
                                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                                "type": "str"
                            },
                            "cos1": {
                                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                                "type": "str"
                            },
                            "cos2": {
                                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                                "type": "str"
                            },
                            "cos3": {
                                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                                "type": "str"
                            },
                            "cos4": {
                                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                                "type": "str"
                            },
                            "cos5": {
                                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                                "type": "str"
                            },
                            "cos6": {
                                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                                "type": "str"
                            },
                            "cos7": {
                                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                                "type": "str"
                            },
                            "dscp0": {
                                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                                "type": "str"
                            },
                            "dscp1": {
                                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                                "type": "str"
                            },
                            "dscp10": {
                                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                                "type": "str"
                            },
                            "dscp11": {
                                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                                "type": "str"
                            },
                            "dscp12": {
                                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                                "type": "str"
                            },
                            "dscp13": {
                                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                                "type": "str"
                            },
                            "dscp14": {
                                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                                "type": "str"
                            },
                            "dscp15": {
                                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                                "type": "str"
                            },
                            "dscp16": {
                                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                                "type": "str"
                            },
                            "dscp17": {
                                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                                "type": "str"
                            },
                            "dscp18": {
                                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                                "type": "str"
                            },
                            "dscp19": {
                                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                                "type": "str"
                            },
                            "dscp2": {
                                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                                "type": "str"
                            },
                            "dscp20": {
                                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                                "type": "str"
                            },
                            "dscp21": {
                                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                                "type": "str"
                            },
                            "dscp22": {
                                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                                "type": "str"
                            },
                            "dscp23": {
                                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                                "type": "str"
                            },
                            "dscp24": {
                                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                                "type": "str"
                            },
                            "dscp25": {
                                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                                "type": "str"
                            },
                            "dscp26": {
                                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                                "type": "str"
                            },
                            "dscp27": {
                                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                                "type": "str"
                            },
                            "dscp28": {
                                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                                "type": "str"
                            },
                            "dscp29": {
                                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                                "type": "str"
                            },
                            "dscp3": {
                                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                                "type": "str"
                            },
                            "dscp30": {
                                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                                "type": "str"
                            },
                            "dscp31": {
                                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                                "type": "str"
                            },
                            "dscp32": {
                                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                                "type": "str"
                            },
                            "dscp33": {
                                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                                "type": "str"
                            },
                            "dscp34": {
                                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                                "type": "str"
                            },
                            "dscp35": {
                                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                                "type": "str"
                            },
                            "dscp36": {
                                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                                "type": "str"
                            },
                            "dscp37": {
                                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                                "type": "str"
                            },
                            "dscp38": {
                                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                                "type": "str"
                            },
                            "dscp39": {
                                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                                "type": "str"
                            },
                            "dscp4": {
                                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                                "type": "str"
                            },
                            "dscp40": {
                                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                                "type": "str"
                            },
                            "dscp41": {
                                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                                "type": "str"
                            },
                            "dscp42": {
                                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                                "type": "str"
                            },
                            "dscp43": {
                                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                                "type": "str"
                            },
                            "dscp44": {
                                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                                "type": "str"
                            },
                            "dscp45": {
                                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                                "type": "str"
                            },
                            "dscp46": {
                                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                                "type": "str"
                            },
                            "dscp47": {
                                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                                "type": "str"
                            },
                            "dscp48": {
                                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                                "type": "str"
                            },
                            "dscp49": {
                                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                                "type": "str"
                            },
                            "dscp5": {
                                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                                "type": "str"
                            },
                            "dscp50": {
                                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                                "type": "str"
                            },
                            "dscp51": {
                                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                                "type": "str"
                            },
                            "dscp52": {
                                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                                "type": "str"
                            },
                            "dscp53": {
                                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                                "type": "str"
                            },
                            "dscp54": {
                                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                                "type": "str"
                            },
                            "dscp55": {
                                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                                "type": "str"
                            },
                            "dscp56": {
                                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                                "type": "str"
                            },
                            "dscp57": {
                                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                                "type": "str"
                            },
                            "dscp58": {
                                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                                "type": "str"
                            },
                            "dscp59": {
                                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                                "type": "str"
                            },
                            "dscp6": {
                                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                                "type": "str"
                            },
                            "dscp60": {
                                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                                "type": "str"
                            },
                            "dscp61": {
                                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                                "type": "str"
                            },
                            "dscp62": {
                                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                                "type": "str"
                            },
                            "dscp63": {
                                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                                "type": "str"
                            },
                            "dscp7": {
                                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                                "type": "str"
                            },
                            "dscp8": {
                                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                                "type": "str"
                            },
                            "dscp9": {
                                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                                "type": "str"
                            },
                            "id": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"},
                            "type": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["cos", "dscp"], "type": "str"},
                            "weight": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"}
                        },
                        "elements": "dict"
                    },
                    "scheduler": {
                        "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                        "type": "list",
                        "options": {
                            "mode": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["none", "priority", "round-robin"], "type": "str"},
                            "name": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "str"}
                        },
                        "elements": "dict"
                    }
                }
            },
            "udp-timeout-profile": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "type": "list",
                "options": {
                    "id": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"},
                    "udp-idle": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"}
                },
                "elements": "dict"
            },
            "qtm-buf-mode": {"v_range": [["6.4.8", "6.4.14"], ["7.0.3", ""]], "choices": ["6ch", "4ch"], "type": "str"},
            "default-qos-type": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["policing", "shaping", "policing-enhanced"], "type": "str"},
            "tcp-rst-timeout": {"v_range": [["6.4.7", "6.4.14"], ["7.0.2", ""]], "type": "int"},
            "ipsec-local-uesp-port": {"v_range": [["7.0.3", ""]], "type": "int"},
            "htab-dedi-queue-nr": {"v_range": [["6.4.7", "6.4.14"], ["7.0.2", ""]], "type": "int"},
            "double-level-mcast-offload": {"v_range": [["6.4.7", "6.4.14"], ["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "dse-timeout": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"},
            "ippool-overload-low": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"},
            "pba-eim": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["disallow", "allow"], "type": "str"},
            "policy-offload-level": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "choices": ["disable", "dos-offload", "full-offload"],
                "type": "str"
            },
            "max-session-timeout": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"},
            "port-path-option": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "type": "dict",
                "options": {"ports-using-npu": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "raw"}}
            },
            "vlan-lookup-cache": {"v_range": [["6.4.7", "6.4.14"], ["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "dos-options": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "type": "dict",
                "options": {
                    "npu-dos-meter-mode": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["local", "global"], "type": "str"},
                    "npu-dos-synproxy-mode": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["synack2ack", "pass-synack"], "type": "str"},
                    "npu-dos-tpe-mode": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"}
                }
            },
            "hash-tbl-spread": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "tcp-timeout-profile": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "type": "list",
                "options": {
                    "close-wait": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"},
                    "fin-wait": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"},
                    "id": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"},
                    "syn-sent": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"},
                    "syn-wait": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"},
                    "tcp-idle": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"},
                    "time-wait": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"}
                },
                "elements": "dict"
            },
            "ip-reassembly": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "type": "dict",
                "options": {
                    "max-timeout": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"},
                    "min-timeout": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"},
                    "status": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"}
                }
            },
            "gtp-support": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "htx-icmp-csum-chk": {"v_range": [["6.4.8", "6.4.14"], ["7.0.3", ""]], "choices": ["pass", "drop"], "type": "str"},
            "hpe": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "type": "dict",
                "options": {
                    "all-protocol": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"},
                    "arp-max": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"},
                    "enable-shaper": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "esp-max": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"},
                    "high-priority": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"},
                    "icmp-max": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"},
                    "ip-frag-max": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"},
                    "ip-others-max": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"},
                    "l2-others-max": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"},
                    "pri-type-max": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"},
                    "sctp-max": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"},
                    "tcp-max": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"},
                    "tcpfin-rst-max": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"},
                    "tcpsyn-ack-max": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"},
                    "tcpsyn-max": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"},
                    "udp-max": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"},
                    "enable-queue-shaper": {
                        "v_range": [["7.0.9", "7.0.12"], ["7.2.4", "7.2.5"], ["7.4.2", ""]],
                        "choices": ["disable", "enable"],
                        "type": "str"
                    },
                    "exception-code": {"v_range": [["7.0.9", "7.0.12"], ["7.2.4", "7.2.5"], ["7.4.2", ""]], "type": "int"},
                    "fragment-with-sess": {"v_range": [["7.0.9", "7.0.12"], ["7.2.4", "7.2.5"], ["7.4.2", ""]], "type": "int"},
                    "fragment-without-session": {"v_range": [["7.0.9", "7.0.12"], ["7.2.4", "7.2.5"], ["7.4.2", ""]], "type": "int"},
                    "queue-shaper-max": {"v_range": [["7.0.9", "7.0.12"], ["7.2.4", "7.2.5"], ["7.4.2", ""]], "type": "int"}
                }
            },
            "dsw-dts-profile": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "type": "list",
                "options": {
                    "action": {
                        "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                        "choices": ["wait", "drop", "drop_tmr_0", "drop_tmr_1", "enque", "enque_0", "enque_1"],
                        "type": "str"
                    },
                    "min-limit": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"},
                    "profile-id": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"},
                    "step": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"}
                },
                "elements": "dict"
            },
            "hash-config": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["5-tuple", "src-ip", "src-dst-ip"], "type": "str"},
            "ipsec-ob-np-sel": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["RR", "rr", "Packet", "Hash"], "type": "str"},
            "napi-break-interval": {"v_range": [["6.4.7", "6.4.14"], ["7.0.2", ""]], "type": "int"},
            "background-sse-scan": {
                "v_range": [["6.4.8", "6.4.14"], ["7.0.3", ""]],
                "type": "dict",
                "options": {
                    "scan": {"v_range": [["6.4.8", "6.4.14"], ["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "stats-update-interval": {"v_range": [["6.4.8", "6.4.14"], ["7.0.3", ""]], "type": "int"},
                    "udp-keepalive-interval": {"v_range": [["6.4.8", "6.4.14"], ["7.0.3", ""]], "type": "int"},
                    "scan-stale": {"v_range": [["7.0.12", "7.0.12"], ["7.4.1", ""]], "type": "int"},
                    "scan-vt": {"v_range": [["7.0.12", "7.0.12"], ["7.4.1", ""]], "type": "int"},
                    "stats-qual-access": {"v_range": [["7.0.12", "7.0.12"], ["7.4.1", ""]], "type": "int"},
                    "stats-qual-duration": {"v_range": [["7.0.12", "7.0.12"], ["7.4.1", ""]], "type": "int"},
                    "udp-qual-access": {"v_range": [["7.0.12", "7.0.12"], ["7.4.1", ""]], "type": "int"},
                    "udp-qual-duration": {"v_range": [["7.0.12", "7.0.12"], ["7.4.1", ""]], "type": "int"}
                }
            },
            "inbound-dscp-copy-port": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "raw"},
            "session-acct-interval": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"},
            "htab-msg-queue": {"v_range": [["6.4.7", "6.4.14"], ["7.0.2", ""]], "choices": ["idle", "data", "dedicated"], "type": "str"},
            "dsw-queue-dts-profile": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "type": "list",
                "options": {
                    "iport": {
                        "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                        "choices": [
                            "EIF0", "eif0", "EIF1", "eif1", "EIF2", "eif2", "EIF3", "eif3", "EIF4", "eif4", "EIF5", "eif5", "EIF6", "eif6", "EIF7",
                            "eif7", "HTX0", "htx0", "HTX1", "htx1", "SSE0", "sse0", "SSE1", "sse1", "SSE2", "sse2", "SSE3", "sse3", "RLT", "rlt", "DFR",
                            "dfr", "IPSECI", "ipseci", "IPSECO", "ipseco", "IPTI", "ipti", "IPTO", "ipto", "VEP0", "vep0", "VEP2", "vep2", "VEP4",
                            "vep4", "VEP6", "vep6", "IVS", "ivs", "L2TI1", "l2ti1", "L2TO", "l2to", "L2TI0", "l2ti0", "PLE", "ple", "SPATH", "spath",
                            "QTM", "qtm"
                        ],
                        "type": "str"
                    },
                    "name": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "str"},
                    "oport": {
                        "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                        "choices": [
                            "EIF0", "eif0", "EIF1", "eif1", "EIF2", "eif2", "EIF3", "eif3", "EIF4", "eif4", "EIF5", "eif5", "EIF6", "eif6", "EIF7",
                            "eif7", "HRX", "hrx", "SSE0", "sse0", "SSE1", "sse1", "SSE2", "sse2", "SSE3", "sse3", "RLT", "rlt", "DFR", "dfr", "IPSECI",
                            "ipseci", "IPSECO", "ipseco", "IPTI", "ipti", "IPTO", "ipto", "VEP0", "vep0", "VEP2", "vep2", "VEP4", "vep4", "VEP6", "vep6",
                            "IVS", "ivs", "L2TI1", "l2ti1", "L2TO", "l2to", "L2TI0", "l2ti0", "PLE", "ple", "SYNK", "sync", "NSS", "nss", "TSK", "tsk",
                            "QTM", "qtm", "l2tO"
                        ],
                        "type": "str"
                    },
                    "profile-id": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"},
                    "queue-select": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"}
                },
                "elements": "dict"
            },
            "hw-ha-scan-interval": {"v_range": [["6.4.8", "6.4.14"], ["7.0.3", ""]], "type": "int"},
            "ippool-overload-high": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"},
            "nat46-force-ipv4-packet-forwarding": {"v_range": [["6.4.8", "6.4.14"], ["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "prp-port-out": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "raw"},
            "isf-np-rx-tr-distr": {"v_range": [["6.4.8", "6.4.14"], ["7.0.4", ""]], "choices": ["port-flow", "round-robin", "randomized"], "type": "str"},
            "mcast-session-counting6": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "choices": ["disable", "enable", "session-based", "tpe-based"],
                "type": "str"
            },
            "prp-port-in": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "raw"},
            "rps-mode": {"v_range": [["6.4.8", "6.4.14"], ["7.0.4", ""]], "choices": ["disable", "enable"], "type": "str"},
            "per-policy-accounting": {"v_range": [["6.4.8", "6.4.14"], ["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "mcast-session-counting": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "choices": ["disable", "enable", "session-based", "tpe-based"],
                "type": "str"
            },
            "inbound-dscp-copy": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ipsec-host-dfclr": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", "7.2.1"]], "choices": ["disable", "enable"], "type": "str"},
            "process-icmp-by-host": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", "7.2.1"]], "choices": ["disable", "enable"], "type": "str"},
            "dedicated-tx-npu": {"v_range": [["6.4.7", "6.4.14"]], "choices": ["disable", "enable"], "type": "str"},
            "ull-port-mode": {"v_range": [["6.4.9", "6.4.14"], ["7.0.4", "7.0.12"], ["7.2.1", ""]], "choices": ["10G", "25G"], "type": "str"},
            "sse-ha-scan": {
                "v_range": [["6.4.10", "6.4.14"], ["7.0.4", "7.0.12"], ["7.2.1", ""]],
                "type": "dict",
                "options": {
                    "gap": {"v_range": [["6.4.10", "6.4.14"], ["7.0.4", "7.0.12"], ["7.2.1", ""]], "type": "int"},
                    "max-session-cnt": {"v_range": [["6.4.10", "6.4.14"], ["7.0.4", "7.0.12"], ["7.2.1", ""]], "type": "int"},
                    "min-duration": {"v_range": [["6.4.10", "6.4.14"], ["7.0.4", "7.0.12"], ["7.2.1", ""]], "type": "int"}
                }
            },
            "hash-ipv6-sel": {"v_range": [["7.0.4", "7.0.12"], ["7.2.1", ""]], "type": "int"},
            "ip-fragment-offload": {"v_range": [["7.0.4", "7.0.12"], ["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ple-non-syn-tcp-action": {"v_range": [["7.0.5", "7.0.12"], ["7.2.2", ""]], "choices": ["forward", "drop"], "type": "str"},
            "npu-group-effective-scope": {"v_range": [["7.0.6", "7.0.12"], ["7.2.2", ""]], "type": "int"},
            "ipsec-STS-timeout": {
                "v_range": [["7.0.9", "7.0.12"], ["7.2.4", "7.2.5"], ["7.4.2", ""]],
                "choices": ["1", "2", "3", "4", "5", "6", "7", "8", "9", "10"],
                "type": "str"
            },
            "ipsec-throughput-msg-frequency": {
                "v_range": [["7.0.9", "7.0.12"], ["7.2.4", "7.2.5"], ["7.4.2", ""]],
                "choices": [
                    "disable", "32KB", "64KB", "128KB", "256KB", "512KB", "1MB", "2MB", "4MB", "8MB", "16MB", "32MB", "64MB", "128MB", "256MB", "512MB",
                    "1GB"
                ],
                "type": "str"
            },
            "ipt-STS-timeout": {
                "v_range": [["7.0.9", "7.0.12"], ["7.2.4", "7.2.5"], ["7.4.2", ""]],
                "choices": ["1", "2", "3", "4", "5", "6", "7", "8", "9", "10"],
                "type": "str"
            },
            "ipt-throughput-msg-frequency": {
                "v_range": [["7.0.9", "7.0.12"], ["7.2.4", "7.2.5"], ["7.4.2", ""]],
                "choices": [
                    "disable", "32KB", "64KB", "128KB", "256KB", "512KB", "1MB", "2MB", "4MB", "8MB", "16MB", "32MB", "64MB", "128MB", "256MB", "512MB",
                    "1GB"
                ],
                "type": "str"
            },
            "default-tcp-refresh-dir": {"v_range": [["7.0.12", "7.0.12"], ["7.4.1", ""]], "choices": ["both", "outgoing", "incoming"], "type": "str"},
            "default-udp-refresh-dir": {"v_range": [["7.0.12", "7.0.12"], ["7.4.1", ""]], "choices": ["both", "outgoing", "incoming"], "type": "str"},
            "nss-threads-option": {"v_range": [["7.0.12", "7.0.12"], ["7.4.2", ""]], "choices": ["4t-eif", "4t-noeif", "2t"], "type": "str"},
            "prp-session-clear-mode": {"v_range": [["7.2.2", ""]], "choices": ["blocking", "non-blocking", "do-not-clear"], "type": "str"},
            "shaping-stats": {"v_range": [["7.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "sw-tr-hash": {
                "v_range": [["7.2.4", ""]],
                "type": "dict",
                "options": {
                    "draco15": {"v_range": [["7.2.4", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "tcp-udp-port": {"v_range": [["7.2.4", ""]], "choices": ["include", "exclude"], "type": "str"}
                }
            },
            "pba-port-select-mode": {"v_range": [["7.2.5", "7.2.5"], ["7.4.2", ""]], "choices": ["random", "direct"], "type": "str"},
            "spa-port-select-mode": {"v_range": [["7.2.5", "7.2.5"], ["7.4.2", ""]], "choices": ["random", "direct"], "type": "str"},
            "split-ipsec-engines": {"v_range": [["7.2.5", "7.2.5"], ["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "tunnel-over-vlink": {"v_range": [["7.2.5", "7.2.5"], ["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "max-receive-unit": {"v_range": [["7.4.2", ""]], "type": "int"},
            "npu-tcam": {
                "v_range": [["7.4.2", ""]],
                "type": "list",
                "options": {
                    "data": {
                        "v_range": [["7.4.2", ""]],
                        "type": "dict",
                        "options": {
                            "df": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "dstip": {"v_range": [["7.4.2", ""]], "type": "str"},
                            "dstipv6": {"v_range": [["7.4.2", ""]], "type": "str"},
                            "dstmac": {"v_range": [["7.4.2", ""]], "type": "str"},
                            "dstport": {"v_range": [["7.4.2", ""]], "type": "int"},
                            "ethertype": {"v_range": [["7.4.2", ""]], "type": "str"},
                            "ext-tag": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "frag-off": {"v_range": [["7.4.2", ""]], "type": "int"},
                            "gen-buf-cnt": {"v_range": [["7.4.2", ""]], "type": "int"},
                            "gen-iv": {"v_range": [["7.4.2", ""]], "choices": ["invalid", "valid"], "type": "str"},
                            "gen-l3-flags": {"v_range": [["7.4.2", ""]], "type": "int"},
                            "gen-l4-flags": {"v_range": [["7.4.2", ""]], "type": "int"},
                            "gen-pkt-ctrl": {"v_range": [["7.4.2", ""]], "type": "int"},
                            "gen-pri": {"v_range": [["7.4.2", ""]], "type": "int"},
                            "gen-pri-v": {"v_range": [["7.4.2", ""]], "choices": ["invalid", "valid"], "type": "str"},
                            "gen-tv": {"v_range": [["7.4.2", ""]], "choices": ["invalid", "valid"], "type": "str"},
                            "ihl": {"v_range": [["7.4.2", ""]], "type": "int"},
                            "ip4-id": {"v_range": [["7.4.2", ""]], "type": "int"},
                            "ip6-fl": {"v_range": [["7.4.2", ""]], "type": "int"},
                            "ipver": {"v_range": [["7.4.2", ""]], "type": "int"},
                            "l4-wd10": {"v_range": [["7.4.2", ""]], "type": "int"},
                            "l4-wd11": {"v_range": [["7.4.2", ""]], "type": "int"},
                            "l4-wd8": {"v_range": [["7.4.2", ""]], "type": "int"},
                            "l4-wd9": {"v_range": [["7.4.2", ""]], "type": "int"},
                            "mf": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "protocol": {"v_range": [["7.4.2", ""]], "type": "int"},
                            "slink": {"v_range": [["7.4.2", ""]], "type": "int"},
                            "smac-change": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "sp": {"v_range": [["7.4.2", ""]], "type": "int"},
                            "src-cfi": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "src-prio": {"v_range": [["7.4.2", ""]], "type": "int"},
                            "src-updt": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "srcip": {"v_range": [["7.4.2", ""]], "type": "str"},
                            "srcipv6": {"v_range": [["7.4.2", ""]], "type": "str"},
                            "srcmac": {"v_range": [["7.4.2", ""]], "type": "str"},
                            "srcport": {"v_range": [["7.4.2", ""]], "type": "int"},
                            "svid": {"v_range": [["7.4.2", ""]], "type": "int"},
                            "tcp-ack": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "tcp-cwr": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "tcp-ece": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "tcp-fin": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "tcp-push": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "tcp-rst": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "tcp-syn": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "tcp-urg": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "tgt-cfi": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "tgt-prio": {"v_range": [["7.4.2", ""]], "type": "int"},
                            "tgt-updt": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "tgt-v": {"v_range": [["7.4.2", ""]], "choices": ["invalid", "valid"], "type": "str"},
                            "tos": {"v_range": [["7.4.2", ""]], "type": "int"},
                            "tp": {"v_range": [["7.4.2", ""]], "type": "int"},
                            "ttl": {"v_range": [["7.4.2", ""]], "type": "int"},
                            "tvid": {"v_range": [["7.4.2", ""]], "type": "int"},
                            "vdid": {"v_range": [["7.4.2", ""]], "type": "int"}
                        }
                    },
                    "dbg-dump": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "mask": {
                        "v_range": [["7.4.2", ""]],
                        "type": "dict",
                        "options": {
                            "df": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "dstip": {"v_range": [["7.4.2", ""]], "type": "str"},
                            "dstipv6": {"v_range": [["7.4.2", ""]], "type": "str"},
                            "dstmac": {"v_range": [["7.4.2", ""]], "type": "str"},
                            "dstport": {"v_range": [["7.4.2", ""]], "type": "int"},
                            "ethertype": {"v_range": [["7.4.2", ""]], "type": "str"},
                            "ext-tag": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "frag-off": {"v_range": [["7.4.2", ""]], "type": "int"},
                            "gen-buf-cnt": {"v_range": [["7.4.2", ""]], "type": "int"},
                            "gen-iv": {"v_range": [["7.4.2", ""]], "choices": ["invalid", "valid"], "type": "str"},
                            "gen-l3-flags": {"v_range": [["7.4.2", ""]], "type": "int"},
                            "gen-l4-flags": {"v_range": [["7.4.2", ""]], "type": "int"},
                            "gen-pkt-ctrl": {"v_range": [["7.4.2", ""]], "type": "int"},
                            "gen-pri": {"v_range": [["7.4.2", ""]], "type": "int"},
                            "gen-pri-v": {"v_range": [["7.4.2", ""]], "choices": ["invalid", "valid"], "type": "str"},
                            "gen-tv": {"v_range": [["7.4.2", ""]], "choices": ["invalid", "valid"], "type": "str"},
                            "ihl": {"v_range": [["7.4.2", ""]], "type": "int"},
                            "ip4-id": {"v_range": [["7.4.2", ""]], "type": "int"},
                            "ip6-fl": {"v_range": [["7.4.2", ""]], "type": "int"},
                            "ipver": {"v_range": [["7.4.2", ""]], "type": "int"},
                            "l4-wd10": {"v_range": [["7.4.2", ""]], "type": "int"},
                            "l4-wd11": {"v_range": [["7.4.2", ""]], "type": "int"},
                            "l4-wd8": {"v_range": [["7.4.2", ""]], "type": "int"},
                            "l4-wd9": {"v_range": [["7.4.2", ""]], "type": "int"},
                            "mf": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "protocol": {"v_range": [["7.4.2", ""]], "type": "int"},
                            "slink": {"v_range": [["7.4.2", ""]], "type": "int"},
                            "smac-change": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "sp": {"v_range": [["7.4.2", ""]], "type": "int"},
                            "src-cfi": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "src-prio": {"v_range": [["7.4.2", ""]], "type": "int"},
                            "src-updt": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "srcip": {"v_range": [["7.4.2", ""]], "type": "str"},
                            "srcipv6": {"v_range": [["7.4.2", ""]], "type": "str"},
                            "srcmac": {"v_range": [["7.4.2", ""]], "type": "str"},
                            "srcport": {"v_range": [["7.4.2", ""]], "type": "int"},
                            "svid": {"v_range": [["7.4.2", ""]], "type": "int"},
                            "tcp-ack": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "tcp-cwr": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "tcp-ece": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "tcp-fin": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "tcp-push": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "tcp-rst": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "tcp-syn": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "tcp-urg": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "tgt-cfi": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "tgt-prio": {"v_range": [["7.4.2", ""]], "type": "int"},
                            "tgt-updt": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "tgt-v": {"v_range": [["7.4.2", ""]], "choices": ["invalid", "valid"], "type": "str"},
                            "tos": {"v_range": [["7.4.2", ""]], "type": "int"},
                            "tp": {"v_range": [["7.4.2", ""]], "type": "int"},
                            "ttl": {"v_range": [["7.4.2", ""]], "type": "int"},
                            "tvid": {"v_range": [["7.4.2", ""]], "type": "int"},
                            "vdid": {"v_range": [["7.4.2", ""]], "type": "int"}
                        }
                    },
                    "mir-act": {"v_range": [["7.4.2", ""]], "type": "dict", "options": {"vlif": {"v_range": [["7.4.2", ""]], "type": "int"}}},
                    "name": {"v_range": [["7.4.2", ""]], "type": "str"},
                    "oid": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "pri-act": {
                        "v_range": [["7.4.2", ""]],
                        "type": "dict",
                        "options": {"priority": {"v_range": [["7.4.2", ""]], "type": "int"}, "weight": {"v_range": [["7.4.2", ""]], "type": "int"}}
                    },
                    "sact": {
                        "v_range": [["7.4.2", ""]],
                        "type": "dict",
                        "options": {
                            "act": {"v_range": [["7.4.2", ""]], "type": "int"},
                            "act-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "bmproc": {"v_range": [["7.4.2", ""]], "type": "int"},
                            "bmproc-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "df-lif": {"v_range": [["7.4.2", ""]], "type": "int"},
                            "df-lif-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "dfr": {"v_range": [["7.4.2", ""]], "type": "int"},
                            "dfr-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "dmac-skip": {"v_range": [["7.4.2", ""]], "type": "int"},
                            "dmac-skip-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "dosen": {"v_range": [["7.4.2", ""]], "type": "int"},
                            "dosen-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "espff-proc": {"v_range": [["7.4.2", ""]], "type": "int"},
                            "espff-proc-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "etype-pid": {"v_range": [["7.4.2", ""]], "type": "int"},
                            "etype-pid-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "frag-proc": {"v_range": [["7.4.2", ""]], "type": "int"},
                            "frag-proc-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "fwd": {"v_range": [["7.4.2", ""]], "type": "int"},
                            "fwd-lif": {"v_range": [["7.4.2", ""]], "type": "int"},
                            "fwd-lif-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "fwd-tvid": {"v_range": [["7.4.2", ""]], "type": "int"},
                            "fwd-tvid-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "fwd-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "icpen": {"v_range": [["7.4.2", ""]], "type": "int"},
                            "icpen-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "igmp-mld-snp": {"v_range": [["7.4.2", ""]], "type": "int"},
                            "igmp-mld-snp-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "learn": {"v_range": [["7.4.2", ""]], "type": "int"},
                            "learn-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "m-srh-ctrl": {"v_range": [["7.4.2", ""]], "type": "int"},
                            "m-srh-ctrl-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "mac-id": {"v_range": [["7.4.2", ""]], "type": "int"},
                            "mac-id-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "mss": {"v_range": [["7.4.2", ""]], "type": "int"},
                            "mss-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "pleen": {"v_range": [["7.4.2", ""]], "type": "int"},
                            "pleen-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "prio-pid": {"v_range": [["7.4.2", ""]], "type": "int"},
                            "prio-pid-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "promis": {"v_range": [["7.4.2", ""]], "type": "int"},
                            "promis-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "rfsh": {"v_range": [["7.4.2", ""]], "type": "int"},
                            "rfsh-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "smac-skip": {"v_range": [["7.4.2", ""]], "type": "int"},
                            "smac-skip-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "tp-smchk-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "tp_smchk": {"v_range": [["7.4.2", ""]], "type": "int"},
                            "tpe-id": {"v_range": [["7.4.2", ""]], "type": "int"},
                            "tpe-id-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "vdm": {"v_range": [["7.4.2", ""]], "type": "int"},
                            "vdm-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "vdom-id": {"v_range": [["7.4.2", ""]], "type": "int"},
                            "vdom-id-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "x-mode": {"v_range": [["7.4.2", ""]], "type": "int"},
                            "x-mode-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"}
                        }
                    },
                    "tact": {
                        "v_range": [["7.4.2", ""]],
                        "type": "dict",
                        "options": {
                            "act": {"v_range": [["7.4.2", ""]], "type": "int"},
                            "act-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "fmtuv4-s": {"v_range": [["7.4.2", ""]], "type": "int"},
                            "fmtuv4-s-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "fmtuv6-s": {"v_range": [["7.4.2", ""]], "type": "int"},
                            "fmtuv6-s-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "lnkid": {"v_range": [["7.4.2", ""]], "type": "int"},
                            "lnkid-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "mac-id": {"v_range": [["7.4.2", ""]], "type": "int"},
                            "mac-id-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "mss-t": {"v_range": [["7.4.2", ""]], "type": "int"},
                            "mss-t-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "mtuv4": {"v_range": [["7.4.2", ""]], "type": "int"},
                            "mtuv4-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "mtuv6": {"v_range": [["7.4.2", ""]], "type": "int"},
                            "mtuv6-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "slif-act": {"v_range": [["7.4.2", ""]], "type": "int"},
                            "slif-act-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "sublnkid": {"v_range": [["7.4.2", ""]], "type": "int"},
                            "sublnkid-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "tgtv-act": {"v_range": [["7.4.2", ""]], "type": "int"},
                            "tgtv-act-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "tlif-act": {"v_range": [["7.4.2", ""]], "type": "int"},
                            "tlif-act-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "tpeid": {"v_range": [["7.4.2", ""]], "type": "int"},
                            "tpeid-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "v6fe": {"v_range": [["7.4.2", ""]], "type": "int"},
                            "v6fe-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "vep-en-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "vep-slid": {"v_range": [["7.4.2", ""]], "type": "int"},
                            "vep-slid-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "vep_en": {"v_range": [["7.4.2", ""]], "type": "int"},
                            "xlt-lif": {"v_range": [["7.4.2", ""]], "type": "int"},
                            "xlt-lif-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "xlt-vid": {"v_range": [["7.4.2", ""]], "type": "int"},
                            "xlt-vid-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"}
                        }
                    },
                    "type": {
                        "v_range": [["7.4.2", ""]],
                        "choices": [
                            "L2_src_tc", "L2_tgt_tc", "L2_src_mir", "L2_tgt_mir", "L2_src_act", "L2_tgt_act", "IPv4_src_tc", "IPv4_tgt_tc",
                            "IPv4_src_mir", "IPv4_tgt_mir", "IPv4_src_act", "IPv4_tgt_act", "IPv6_src_tc", "IPv6_tgt_tc", "IPv6_src_mir", "IPv6_tgt_mir",
                            "IPv6_src_act", "IPv6_tgt_act"
                        ],
                        "type": "str"
                    },
                    "vid": {"v_range": [["7.4.2", ""]], "type": "int"}
                },
                "elements": "dict"
            },
            "icmp-rate-ctrl": {
                "v_range": [["7.4.3", ""]],
                "type": "dict",
                "options": {
                    "icmp-v4-bucket-size": {"v_range": [["7.4.3", ""]], "type": "int"},
                    "icmp-v4-rate": {"v_range": [["7.4.3", ""]], "type": "int"},
                    "icmp-v6-bucket-size": {"v_range": [["7.4.3", ""]], "type": "int"},
                    "icmp-v6-rate": {"v_range": [["7.4.3", ""]], "type": "int"}
                }
            },
            "vxlan-offload": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "icmp-error-rate-ctrl": {
                "v_range": [["7.6.0", ""]],
                "type": "dict",
                "options": {
                    "icmpv4-error-bucket-size": {"v_range": [["7.6.0", ""]], "type": "int"},
                    "icmpv4-error-rate": {"v_range": [["7.6.0", ""]], "type": "int"},
                    "icmpv4-error-rate-limit": {"v_range": [["7.6.0", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "icmpv6-error-bucket-size": {"v_range": [["7.6.0", ""]], "type": "int"},
                    "icmpv6-error-rate": {"v_range": [["7.6.0", ""]], "type": "int"},
                    "icmpv6-error-rate-limit": {"v_range": [["7.6.0", ""]], "choices": ["disable", "enable"], "type": "str"}
                }
            },
            "ipv4-session-quota": {"v_range": [["7.6.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ipv4-session-quota-high": {"v_range": [["7.6.0", ""]], "type": "int"},
            "ipv4-session-quota-low": {"v_range": [["7.6.0", ""]], "type": "int"},
            "ipv6-prefix-session-quota": {"v_range": [["7.6.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ipv6-prefix-session-quota-high": {"v_range": [["7.6.0", ""]], "type": "int"},
            "ipv6-prefix-session-quota-low": {"v_range": [["7.6.0", ""]], "type": "int"}
        }
    },
    "system_npu_backgroundssescan": {
        "stated": false,
        "options": {
            "scan": {"v_range": [["6.4.8", "6.4.14"], ["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "stats-update-interval": {"v_range": [["6.4.8", "6.4.14"], ["7.0.3", ""]], "type": "int"},
            "udp-keepalive-interval": {"v_range": [["6.4.8", "6.4.14"], ["7.0.3", ""]], "type": "int"},
            "scan-stale": {"v_range": [["7.0.12", "7.0.12"], ["7.4.1", ""]], "type": "int"},
            "scan-vt": {"v_range": [["7.0.12", "7.0.12"], ["7.4.1", ""]], "type": "int"},
            "stats-qual-access": {"v_range": [["7.0.12", "7.0.12"], ["7.4.1", ""]], "type": "int"},
            "stats-qual-duration": {"v_range": [["7.0.12", "7.0.12"], ["7.4.1", ""]], "type": "int"},
            "udp-qual-access": {"v_range": [["7.0.12", "7.0.12"], ["7.4.1", ""]], "type": "int"},
            "udp-qual-duration": {"v_range": [["7.0.12", "7.0.12"], ["7.4.1", ""]], "type": "int"}
        }
    },
    "system_npu_dosoptions": {
        "stated": false,
        "options": {
            "npu-dos-meter-mode": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["local", "global"], "type": "str"},
            "npu-dos-synproxy-mode": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["synack2ack", "pass-synack"], "type": "str"},
            "npu-dos-tpe-mode": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "system_npu_dswdtsprofile": {
        "stated": true,
        "options": {
            "action": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "choices": ["wait", "drop", "drop_tmr_0", "drop_tmr_1", "enque", "enque_0", "enque_1"],
                "type": "str"
            },
            "min-limit": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"},
            "profile-id": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"},
            "step": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"}
        }
    },
    "system_npu_dswqueuedtsprofile": {
        "stated": true,
        "options": {
            "iport": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "choices": [
                    "EIF0", "eif0", "EIF1", "eif1", "EIF2", "eif2", "EIF3", "eif3", "EIF4", "eif4", "EIF5", "eif5", "EIF6", "eif6", "EIF7", "eif7",
                    "HTX0", "htx0", "HTX1", "htx1", "SSE0", "sse0", "SSE1", "sse1", "SSE2", "sse2", "SSE3", "sse3", "RLT", "rlt", "DFR", "dfr", "IPSECI",
                    "ipseci", "IPSECO", "ipseco", "IPTI", "ipti", "IPTO", "ipto", "VEP0", "vep0", "VEP2", "vep2", "VEP4", "vep4", "VEP6", "vep6", "IVS",
                    "ivs", "L2TI1", "l2ti1", "L2TO", "l2to", "L2TI0", "l2ti0", "PLE", "ple", "SPATH", "spath", "QTM", "qtm"
                ],
                "type": "str"
            },
            "name": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "required": true, "type": "str"},
            "oport": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "choices": [
                    "EIF0", "eif0", "EIF1", "eif1", "EIF2", "eif2", "EIF3", "eif3", "EIF4", "eif4", "EIF5", "eif5", "EIF6", "eif6", "EIF7", "eif7",
                    "HRX", "hrx", "SSE0", "sse0", "SSE1", "sse1", "SSE2", "sse2", "SSE3", "sse3", "RLT", "rlt", "DFR", "dfr", "IPSECI", "ipseci",
                    "IPSECO", "ipseco", "IPTI", "ipti", "IPTO", "ipto", "VEP0", "vep0", "VEP2", "vep2", "VEP4", "vep4", "VEP6", "vep6", "IVS", "ivs",
                    "L2TI1", "l2ti1", "L2TO", "l2to", "L2TI0", "l2ti0", "PLE", "ple", "SYNK", "sync", "NSS", "nss", "TSK", "tsk", "QTM", "qtm", "l2tO"
                ],
                "type": "str"
            },
            "profile-id": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"},
            "queue-select": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"}
        }
    },
    "system_npu_fpanomaly": {
        "stated": false,
        "options": {
            "esp-minlen-err": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["drop", "trap-to-host"], "type": "str"},
            "icmp-csum-err": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["drop", "trap-to-host"], "type": "str"},
            "icmp-minlen-err": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["drop", "trap-to-host"], "type": "str"},
            "ipv4-csum-err": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["drop", "trap-to-host"], "type": "str"},
            "ipv4-ihl-err": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["drop", "trap-to-host"], "type": "str"},
            "ipv4-len-err": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["drop", "trap-to-host"], "type": "str"},
            "ipv4-opt-err": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["drop", "trap-to-host"], "type": "str"},
            "ipv4-ttlzero-err": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["drop", "trap-to-host"], "type": "str"},
            "ipv4-ver-err": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["drop", "trap-to-host"], "type": "str"},
            "ipv6-exthdr-len-err": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["drop", "trap-to-host"], "type": "str"},
            "ipv6-exthdr-order-err": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["drop", "trap-to-host"], "type": "str"},
            "ipv6-ihl-err": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["drop", "trap-to-host"], "type": "str"},
            "ipv6-plen-zero": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["drop", "trap-to-host"], "type": "str"},
            "ipv6-ver-err": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["drop", "trap-to-host"], "type": "str"},
            "tcp-csum-err": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["drop", "trap-to-host"], "type": "str"},
            "tcp-hlen-err": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["drop", "trap-to-host"], "type": "str"},
            "tcp-plen-err": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["drop", "trap-to-host"], "type": "str"},
            "udp-csum-err": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["drop", "trap-to-host"], "type": "str"},
            "udp-hlen-err": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["drop", "trap-to-host"], "type": "str"},
            "udp-len-err": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["drop", "trap-to-host"], "type": "str"},
            "udp-plen-err": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["drop", "trap-to-host"], "type": "str"},
            "udplite-cover-err": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["drop", "trap-to-host"], "type": "str"},
            "udplite-csum-err": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["drop", "trap-to-host"], "type": "str"},
            "unknproto-minlen-err": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["drop", "trap-to-host"], "type": "str"},
            "tcp-fin-only": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["allow", "drop", "trap-to-host"], "type": "str"},
            "ipv4-optsecurity": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["allow", "drop", "trap-to-host"], "type": "str"},
            "ipv6-optralert": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["allow", "drop", "trap-to-host"], "type": "str"},
            "tcp-syn-fin": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["allow", "drop", "trap-to-host"], "type": "str"},
            "ipv4-proto-err": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["allow", "drop", "trap-to-host"], "type": "str"},
            "ipv6-saddr-err": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["allow", "drop", "trap-to-host"], "type": "str"},
            "icmp-frag": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["allow", "drop", "trap-to-host"], "type": "str"},
            "ipv4-optssrr": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["allow", "drop", "trap-to-host"], "type": "str"},
            "ipv6-opthomeaddr": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["allow", "drop", "trap-to-host"], "type": "str"},
            "udp-land": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["allow", "drop", "trap-to-host"], "type": "str"},
            "ipv6-optinvld": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["allow", "drop", "trap-to-host"], "type": "str"},
            "tcp-fin-noack": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["allow", "drop", "trap-to-host"], "type": "str"},
            "ipv6-proto-err": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["allow", "drop", "trap-to-host"], "type": "str"},
            "tcp-land": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["allow", "drop", "trap-to-host"], "type": "str"},
            "ipv4-unknopt": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["allow", "drop", "trap-to-host"], "type": "str"},
            "ipv4-optstream": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["allow", "drop", "trap-to-host"], "type": "str"},
            "ipv6-optjumbo": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["allow", "drop", "trap-to-host"], "type": "str"},
            "icmp-land": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["allow", "drop", "trap-to-host"], "type": "str"},
            "tcp-winnuke": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["allow", "drop", "trap-to-host"], "type": "str"},
            "ipv6-daddr-err": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["allow", "drop", "trap-to-host"], "type": "str"},
            "ipv4-land": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["allow", "drop", "trap-to-host"], "type": "str"},
            "ipv6-opttunnel": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["allow", "drop", "trap-to-host"], "type": "str"},
            "tcp-no-flag": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["allow", "drop", "trap-to-host"], "type": "str"},
            "ipv6-land": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["allow", "drop", "trap-to-host"], "type": "str"},
            "ipv4-optlsrr": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["allow", "drop", "trap-to-host"], "type": "str"},
            "ipv4-opttimestamp": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["allow", "drop", "trap-to-host"], "type": "str"},
            "ipv4-optrr": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["allow", "drop", "trap-to-host"], "type": "str"},
            "ipv6-optnsap": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["allow", "drop", "trap-to-host"], "type": "str"},
            "ipv6-unknopt": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["allow", "drop", "trap-to-host"], "type": "str"},
            "tcp-syn-data": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["allow", "drop", "trap-to-host"], "type": "str"},
            "ipv6-optendpid": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["allow", "drop", "trap-to-host"], "type": "str"},
            "gtpu-plen-err": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["drop", "trap-to-host"], "type": "str"},
            "vxlan-minlen-err": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["drop", "trap-to-host"], "type": "str"},
            "capwap-minlen-err": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["drop", "trap-to-host"], "type": "str"},
            "gre-csum-err": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["drop", "trap-to-host"], "type": "str"},
            "nvgre-minlen-err": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["drop", "trap-to-host"], "type": "str"},
            "sctp-l4len-err": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["drop", "trap-to-host"], "type": "str"},
            "tcp-hlenvsl4len-err": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["drop", "trap-to-host"], "type": "str"},
            "sctp-crc-err": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["drop", "trap-to-host"], "type": "str"},
            "sctp-clen-err": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["drop", "trap-to-host"], "type": "str"},
            "uesp-minlen-err": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["drop", "trap-to-host"], "type": "str"},
            "sctp-csum-err": {"v_range": [["7.2.5", "7.2.5"], ["7.4.3", ""]], "choices": ["allow", "drop", "trap-to-host"], "type": "str"}
        }
    },
    "system_npu_hpe": {
        "stated": false,
        "options": {
            "all-protocol": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"},
            "arp-max": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"},
            "enable-shaper": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "esp-max": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"},
            "high-priority": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"},
            "icmp-max": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"},
            "ip-frag-max": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"},
            "ip-others-max": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"},
            "l2-others-max": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"},
            "pri-type-max": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"},
            "sctp-max": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"},
            "tcp-max": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"},
            "tcpfin-rst-max": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"},
            "tcpsyn-ack-max": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"},
            "tcpsyn-max": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"},
            "udp-max": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"},
            "enable-queue-shaper": {"v_range": [["7.0.9", "7.0.12"], ["7.2.4", "7.2.5"], ["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "exception-code": {"v_range": [["7.0.9", "7.0.12"], ["7.2.4", "7.2.5"], ["7.4.2", ""]], "type": "int"},
            "fragment-with-sess": {"v_range": [["7.0.9", "7.0.12"], ["7.2.4", "7.2.5"], ["7.4.2", ""]], "type": "int"},
            "fragment-without-session": {"v_range": [["7.0.9", "7.0.12"], ["7.2.4", "7.2.5"], ["7.4.2", ""]], "type": "int"},
            "queue-shaper-max": {"v_range": [["7.0.9", "7.0.12"], ["7.2.4", "7.2.5"], ["7.4.2", ""]], "type": "int"}
        }
    },
    "system_npu_icmpratectrl": {
        "stated": false,
        "options": {
            "icmp-v4-bucket-size": {"v_range": [["7.4.3", ""]], "type": "int"},
            "icmp-v4-rate": {"v_range": [["7.4.3", ""]], "type": "int"},
            "icmp-v6-bucket-size": {"v_range": [["7.4.3", ""]], "type": "int"},
            "icmp-v6-rate": {"v_range": [["7.4.3", ""]], "type": "int"}
        }
    },
    "system_npu_ipreassembly": {
        "stated": false,
        "options": {
            "max-timeout": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"},
            "min-timeout": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"},
            "status": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "system_npu_isfnpqueues": {
        "stated": false,
        "options": {
            "cos0": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "str"},
            "cos1": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "str"},
            "cos2": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "str"},
            "cos3": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "str"},
            "cos4": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "str"},
            "cos5": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "str"},
            "cos6": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "str"},
            "cos7": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "str"}
        }
    },
    "system_npu_npqueues": {
        "stated": false,
        "options": {
            "ethernet-type": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "type": "list",
                "options": {
                    "name": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "str"},
                    "queue": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"},
                    "type": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"},
                    "weight": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"}
                },
                "elements": "dict"
            },
            "ip-protocol": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "type": "list",
                "options": {
                    "name": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "str"},
                    "protocol": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"},
                    "queue": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"},
                    "weight": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"}
                },
                "elements": "dict"
            },
            "ip-service": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "type": "list",
                "options": {
                    "dport": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"},
                    "name": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "str"},
                    "protocol": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"},
                    "queue": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"},
                    "sport": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"},
                    "weight": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"}
                },
                "elements": "dict"
            },
            "profile": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "type": "list",
                "options": {
                    "cos0": {
                        "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                        "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                        "type": "str"
                    },
                    "cos1": {
                        "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                        "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                        "type": "str"
                    },
                    "cos2": {
                        "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                        "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                        "type": "str"
                    },
                    "cos3": {
                        "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                        "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                        "type": "str"
                    },
                    "cos4": {
                        "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                        "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                        "type": "str"
                    },
                    "cos5": {
                        "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                        "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                        "type": "str"
                    },
                    "cos6": {
                        "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                        "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                        "type": "str"
                    },
                    "cos7": {
                        "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                        "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                        "type": "str"
                    },
                    "dscp0": {
                        "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                        "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                        "type": "str"
                    },
                    "dscp1": {
                        "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                        "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                        "type": "str"
                    },
                    "dscp10": {
                        "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                        "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                        "type": "str"
                    },
                    "dscp11": {
                        "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                        "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                        "type": "str"
                    },
                    "dscp12": {
                        "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                        "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                        "type": "str"
                    },
                    "dscp13": {
                        "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                        "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                        "type": "str"
                    },
                    "dscp14": {
                        "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                        "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                        "type": "str"
                    },
                    "dscp15": {
                        "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                        "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                        "type": "str"
                    },
                    "dscp16": {
                        "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                        "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                        "type": "str"
                    },
                    "dscp17": {
                        "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                        "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                        "type": "str"
                    },
                    "dscp18": {
                        "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                        "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                        "type": "str"
                    },
                    "dscp19": {
                        "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                        "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                        "type": "str"
                    },
                    "dscp2": {
                        "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                        "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                        "type": "str"
                    },
                    "dscp20": {
                        "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                        "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                        "type": "str"
                    },
                    "dscp21": {
                        "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                        "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                        "type": "str"
                    },
                    "dscp22": {
                        "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                        "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                        "type": "str"
                    },
                    "dscp23": {
                        "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                        "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                        "type": "str"
                    },
                    "dscp24": {
                        "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                        "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                        "type": "str"
                    },
                    "dscp25": {
                        "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                        "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                        "type": "str"
                    },
                    "dscp26": {
                        "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                        "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                        "type": "str"
                    },
                    "dscp27": {
                        "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                        "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                        "type": "str"
                    },
                    "dscp28": {
                        "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                        "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                        "type": "str"
                    },
                    "dscp29": {
                        "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                        "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                        "type": "str"
                    },
                    "dscp3": {
                        "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                        "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                        "type": "str"
                    },
                    "dscp30": {
                        "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                        "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                        "type": "str"
                    },
                    "dscp31": {
                        "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                        "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                        "type": "str"
                    },
                    "dscp32": {
                        "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                        "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                        "type": "str"
                    },
                    "dscp33": {
                        "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                        "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                        "type": "str"
                    },
                    "dscp34": {
                        "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                        "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                        "type": "str"
                    },
                    "dscp35": {
                        "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                        "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                        "type": "str"
                    },
                    "dscp36": {
                        "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                        "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                        "type": "str"
                    },
                    "dscp37": {
                        "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                        "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                        "type": "str"
                    },
                    "dscp38": {
                        "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                        "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                        "type": "str"
                    },
                    "dscp39": {
                        "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                        "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                        "type": "str"
                    },
                    "dscp4": {
                        "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                        "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                        "type": "str"
                    },
                    "dscp40": {
                        "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                        "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                        "type": "str"
                    },
                    "dscp41": {
                        "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                        "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                        "type": "str"
                    },
                    "dscp42": {
                        "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                        "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                        "type": "str"
                    },
                    "dscp43": {
                        "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                        "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                        "type": "str"
                    },
                    "dscp44": {
                        "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                        "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                        "type": "str"
                    },
                    "dscp45": {
                        "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                        "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                        "type": "str"
                    },
                    "dscp46": {
                        "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                        "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                        "type": "str"
                    },
                    "dscp47": {
                        "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                        "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                        "type": "str"
                    },
                    "dscp48": {
                        "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                        "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                        "type": "str"
                    },
                    "dscp49": {
                        "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                        "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                        "type": "str"
                    },
                    "dscp5": {
                        "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                        "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                        "type": "str"
                    },
                    "dscp50": {
                        "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                        "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                        "type": "str"
                    },
                    "dscp51": {
                        "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                        "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                        "type": "str"
                    },
                    "dscp52": {
                        "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                        "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                        "type": "str"
                    },
                    "dscp53": {
                        "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                        "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                        "type": "str"
                    },
                    "dscp54": {
                        "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                        "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                        "type": "str"
                    },
                    "dscp55": {
                        "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                        "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                        "type": "str"
                    },
                    "dscp56": {
                        "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                        "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                        "type": "str"
                    },
                    "dscp57": {
                        "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                        "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                        "type": "str"
                    },
                    "dscp58": {
                        "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                        "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                        "type": "str"
                    },
                    "dscp59": {
                        "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                        "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                        "type": "str"
                    },
                    "dscp6": {
                        "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                        "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                        "type": "str"
                    },
                    "dscp60": {
                        "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                        "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                        "type": "str"
                    },
                    "dscp61": {
                        "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                        "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                        "type": "str"
                    },
                    "dscp62": {
                        "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                        "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                        "type": "str"
                    },
                    "dscp63": {
                        "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                        "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                        "type": "str"
                    },
                    "dscp7": {
                        "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                        "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                        "type": "str"
                    },
                    "dscp8": {
                        "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                        "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                        "type": "str"
                    },
                    "dscp9": {
                        "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                        "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                        "type": "str"
                    },
                    "id": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"},
                    "type": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["cos", "dscp"], "type": "str"},
                    "weight": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"}
                },
                "elements": "dict"
            },
            "scheduler": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "type": "list",
                "options": {
                    "mode": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["none", "priority", "round-robin"], "type": "str"},
                    "name": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "str"}
                },
                "elements": "dict"
            }
        }
    },
    "system_npu_npqueues_ethernettype": {
        "stated": true,
        "options": {
            "name": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "required": true, "type": "str"},
            "queue": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"},
            "type": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"},
            "weight": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"}
        }
    },
    "system_npu_npqueues_ipprotocol": {
        "stated": true,
        "options": {
            "name": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "required": true, "type": "str"},
            "protocol": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"},
            "queue": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"},
            "weight": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"}
        }
    },
    "system_npu_npqueues_ipservice": {
        "stated": true,
        "options": {
            "dport": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"},
            "name": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "required": true, "type": "str"},
            "protocol": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"},
            "queue": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"},
            "sport": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"},
            "weight": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"}
        }
    },
    "system_npu_npqueues_profile": {
        "stated": true,
        "options": {
            "cos0": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                "type": "str"
            },
            "cos1": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                "type": "str"
            },
            "cos2": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                "type": "str"
            },
            "cos3": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                "type": "str"
            },
            "cos4": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                "type": "str"
            },
            "cos5": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                "type": "str"
            },
            "cos6": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                "type": "str"
            },
            "cos7": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                "type": "str"
            },
            "dscp0": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                "type": "str"
            },
            "dscp1": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                "type": "str"
            },
            "dscp10": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                "type": "str"
            },
            "dscp11": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                "type": "str"
            },
            "dscp12": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                "type": "str"
            },
            "dscp13": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                "type": "str"
            },
            "dscp14": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                "type": "str"
            },
            "dscp15": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                "type": "str"
            },
            "dscp16": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                "type": "str"
            },
            "dscp17": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                "type": "str"
            },
            "dscp18": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                "type": "str"
            },
            "dscp19": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                "type": "str"
            },
            "dscp2": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                "type": "str"
            },
            "dscp20": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                "type": "str"
            },
            "dscp21": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                "type": "str"
            },
            "dscp22": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                "type": "str"
            },
            "dscp23": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                "type": "str"
            },
            "dscp24": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                "type": "str"
            },
            "dscp25": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                "type": "str"
            },
            "dscp26": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                "type": "str"
            },
            "dscp27": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                "type": "str"
            },
            "dscp28": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                "type": "str"
            },
            "dscp29": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                "type": "str"
            },
            "dscp3": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                "type": "str"
            },
            "dscp30": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                "type": "str"
            },
            "dscp31": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                "type": "str"
            },
            "dscp32": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                "type": "str"
            },
            "dscp33": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                "type": "str"
            },
            "dscp34": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                "type": "str"
            },
            "dscp35": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                "type": "str"
            },
            "dscp36": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                "type": "str"
            },
            "dscp37": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                "type": "str"
            },
            "dscp38": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                "type": "str"
            },
            "dscp39": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                "type": "str"
            },
            "dscp4": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                "type": "str"
            },
            "dscp40": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                "type": "str"
            },
            "dscp41": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                "type": "str"
            },
            "dscp42": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                "type": "str"
            },
            "dscp43": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                "type": "str"
            },
            "dscp44": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                "type": "str"
            },
            "dscp45": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                "type": "str"
            },
            "dscp46": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                "type": "str"
            },
            "dscp47": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                "type": "str"
            },
            "dscp48": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                "type": "str"
            },
            "dscp49": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                "type": "str"
            },
            "dscp5": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                "type": "str"
            },
            "dscp50": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                "type": "str"
            },
            "dscp51": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                "type": "str"
            },
            "dscp52": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                "type": "str"
            },
            "dscp53": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                "type": "str"
            },
            "dscp54": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                "type": "str"
            },
            "dscp55": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                "type": "str"
            },
            "dscp56": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                "type": "str"
            },
            "dscp57": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                "type": "str"
            },
            "dscp58": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                "type": "str"
            },
            "dscp59": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                "type": "str"
            },
            "dscp6": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                "type": "str"
            },
            "dscp60": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                "type": "str"
            },
            "dscp61": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                "type": "str"
            },
            "dscp62": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                "type": "str"
            },
            "dscp63": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                "type": "str"
            },
            "dscp7": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                "type": "str"
            },
            "dscp8": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                "type": "str"
            },
            "dscp9": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "choices": ["queue0", "queue1", "queue2", "queue3", "queue4", "queue5", "queue6", "queue7"],
                "type": "str"
            },
            "id": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "required": true, "type": "int"},
            "type": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["cos", "dscp"], "type": "str"},
            "weight": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"}
        }
    },
    "system_npu_npqueues_scheduler": {
        "stated": true,
        "options": {
            "mode": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["none", "priority", "round-robin"], "type": "str"},
            "name": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "required": true, "type": "str"}
        }
    },
    "system_npu_nputcam": {
        "stated": true,
        "options": {
            "data": {
                "v_range": [["7.4.2", ""]],
                "type": "dict",
                "options": {
                    "df": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "dstip": {"v_range": [["7.4.2", ""]], "type": "str"},
                    "dstipv6": {"v_range": [["7.4.2", ""]], "type": "str"},
                    "dstmac": {"v_range": [["7.4.2", ""]], "type": "str"},
                    "dstport": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "ethertype": {"v_range": [["7.4.2", ""]], "type": "str"},
                    "ext-tag": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "frag-off": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "gen-buf-cnt": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "gen-iv": {"v_range": [["7.4.2", ""]], "choices": ["invalid", "valid"], "type": "str"},
                    "gen-l3-flags": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "gen-l4-flags": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "gen-pkt-ctrl": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "gen-pri": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "gen-pri-v": {"v_range": [["7.4.2", ""]], "choices": ["invalid", "valid"], "type": "str"},
                    "gen-tv": {"v_range": [["7.4.2", ""]], "choices": ["invalid", "valid"], "type": "str"},
                    "ihl": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "ip4-id": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "ip6-fl": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "ipver": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "l4-wd10": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "l4-wd11": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "l4-wd8": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "l4-wd9": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "mf": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "protocol": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "slink": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "smac-change": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "sp": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "src-cfi": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "src-prio": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "src-updt": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "srcip": {"v_range": [["7.4.2", ""]], "type": "str"},
                    "srcipv6": {"v_range": [["7.4.2", ""]], "type": "str"},
                    "srcmac": {"v_range": [["7.4.2", ""]], "type": "str"},
                    "srcport": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "svid": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "tcp-ack": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "tcp-cwr": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "tcp-ece": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "tcp-fin": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "tcp-push": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "tcp-rst": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "tcp-syn": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "tcp-urg": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "tgt-cfi": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "tgt-prio": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "tgt-updt": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "tgt-v": {"v_range": [["7.4.2", ""]], "choices": ["invalid", "valid"], "type": "str"},
                    "tos": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "tp": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "ttl": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "tvid": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "vdid": {"v_range": [["7.4.2", ""]], "type": "int"}
                }
            },
            "dbg-dump": {"v_range": [["7.4.2", ""]], "type": "int"},
            "mask": {
                "v_range": [["7.4.2", ""]],
                "type": "dict",
                "options": {
                    "df": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "dstip": {"v_range": [["7.4.2", ""]], "type": "str"},
                    "dstipv6": {"v_range": [["7.4.2", ""]], "type": "str"},
                    "dstmac": {"v_range": [["7.4.2", ""]], "type": "str"},
                    "dstport": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "ethertype": {"v_range": [["7.4.2", ""]], "type": "str"},
                    "ext-tag": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "frag-off": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "gen-buf-cnt": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "gen-iv": {"v_range": [["7.4.2", ""]], "choices": ["invalid", "valid"], "type": "str"},
                    "gen-l3-flags": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "gen-l4-flags": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "gen-pkt-ctrl": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "gen-pri": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "gen-pri-v": {"v_range": [["7.4.2", ""]], "choices": ["invalid", "valid"], "type": "str"},
                    "gen-tv": {"v_range": [["7.4.2", ""]], "choices": ["invalid", "valid"], "type": "str"},
                    "ihl": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "ip4-id": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "ip6-fl": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "ipver": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "l4-wd10": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "l4-wd11": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "l4-wd8": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "l4-wd9": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "mf": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "protocol": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "slink": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "smac-change": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "sp": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "src-cfi": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "src-prio": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "src-updt": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "srcip": {"v_range": [["7.4.2", ""]], "type": "str"},
                    "srcipv6": {"v_range": [["7.4.2", ""]], "type": "str"},
                    "srcmac": {"v_range": [["7.4.2", ""]], "type": "str"},
                    "srcport": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "svid": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "tcp-ack": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "tcp-cwr": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "tcp-ece": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "tcp-fin": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "tcp-push": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "tcp-rst": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "tcp-syn": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "tcp-urg": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "tgt-cfi": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "tgt-prio": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "tgt-updt": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "tgt-v": {"v_range": [["7.4.2", ""]], "choices": ["invalid", "valid"], "type": "str"},
                    "tos": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "tp": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "ttl": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "tvid": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "vdid": {"v_range": [["7.4.2", ""]], "type": "int"}
                }
            },
            "mir-act": {"v_range": [["7.4.2", ""]], "type": "dict", "options": {"vlif": {"v_range": [["7.4.2", ""]], "type": "int"}}},
            "name": {"v_range": [["7.4.2", ""]], "required": true, "type": "str"},
            "oid": {"v_range": [["7.4.2", ""]], "type": "int"},
            "pri-act": {
                "v_range": [["7.4.2", ""]],
                "type": "dict",
                "options": {"priority": {"v_range": [["7.4.2", ""]], "type": "int"}, "weight": {"v_range": [["7.4.2", ""]], "type": "int"}}
            },
            "sact": {
                "v_range": [["7.4.2", ""]],
                "type": "dict",
                "options": {
                    "act": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "act-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "bmproc": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "bmproc-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "df-lif": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "df-lif-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "dfr": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "dfr-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "dmac-skip": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "dmac-skip-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "dosen": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "dosen-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "espff-proc": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "espff-proc-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "etype-pid": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "etype-pid-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "frag-proc": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "frag-proc-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "fwd": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "fwd-lif": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "fwd-lif-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "fwd-tvid": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "fwd-tvid-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "fwd-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "icpen": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "icpen-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "igmp-mld-snp": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "igmp-mld-snp-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "learn": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "learn-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "m-srh-ctrl": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "m-srh-ctrl-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "mac-id": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "mac-id-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "mss": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "mss-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "pleen": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "pleen-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "prio-pid": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "prio-pid-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "promis": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "promis-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "rfsh": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "rfsh-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "smac-skip": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "smac-skip-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "tp-smchk-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "tp_smchk": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "tpe-id": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "tpe-id-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "vdm": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "vdm-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "vdom-id": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "vdom-id-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "x-mode": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "x-mode-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"}
                }
            },
            "tact": {
                "v_range": [["7.4.2", ""]],
                "type": "dict",
                "options": {
                    "act": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "act-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "fmtuv4-s": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "fmtuv4-s-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "fmtuv6-s": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "fmtuv6-s-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "lnkid": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "lnkid-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "mac-id": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "mac-id-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "mss-t": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "mss-t-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "mtuv4": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "mtuv4-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "mtuv6": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "mtuv6-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "slif-act": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "slif-act-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "sublnkid": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "sublnkid-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "tgtv-act": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "tgtv-act-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "tlif-act": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "tlif-act-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "tpeid": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "tpeid-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "v6fe": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "v6fe-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "vep-en-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "vep-slid": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "vep-slid-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "vep_en": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "xlt-lif": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "xlt-lif-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "xlt-vid": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "xlt-vid-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"}
                }
            },
            "type": {
                "v_range": [["7.4.2", ""]],
                "choices": [
                    "L2_src_tc", "L2_tgt_tc", "L2_src_mir", "L2_tgt_mir", "L2_src_act", "L2_tgt_act", "IPv4_src_tc", "IPv4_tgt_tc", "IPv4_src_mir",
                    "IPv4_tgt_mir", "IPv4_src_act", "IPv4_tgt_act", "IPv6_src_tc", "IPv6_tgt_tc", "IPv6_src_mir", "IPv6_tgt_mir", "IPv6_src_act",
                    "IPv6_tgt_act"
                ],
                "type": "str"
            },
            "vid": {"v_range": [["7.4.2", ""]], "type": "int"}
        }
    },
    "system_npu_nputcam_data": {
        "stated": false,
        "options": {
            "df": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "dstip": {"v_range": [["7.4.2", ""]], "type": "str"},
            "dstipv6": {"v_range": [["7.4.2", ""]], "type": "str"},
            "dstmac": {"v_range": [["7.4.2", ""]], "type": "str"},
            "dstport": {"v_range": [["7.4.2", ""]], "type": "int"},
            "ethertype": {"v_range": [["7.4.2", ""]], "type": "str"},
            "ext-tag": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "frag-off": {"v_range": [["7.4.2", ""]], "type": "int"},
            "gen-buf-cnt": {"v_range": [["7.4.2", ""]], "type": "int"},
            "gen-iv": {"v_range": [["7.4.2", ""]], "choices": ["invalid", "valid"], "type": "str"},
            "gen-l3-flags": {"v_range": [["7.4.2", ""]], "type": "int"},
            "gen-l4-flags": {"v_range": [["7.4.2", ""]], "type": "int"},
            "gen-pkt-ctrl": {"v_range": [["7.4.2", ""]], "type": "int"},
            "gen-pri": {"v_range": [["7.4.2", ""]], "type": "int"},
            "gen-pri-v": {"v_range": [["7.4.2", ""]], "choices": ["invalid", "valid"], "type": "str"},
            "gen-tv": {"v_range": [["7.4.2", ""]], "choices": ["invalid", "valid"], "type": "str"},
            "ihl": {"v_range": [["7.4.2", ""]], "type": "int"},
            "ip4-id": {"v_range": [["7.4.2", ""]], "type": "int"},
            "ip6-fl": {"v_range": [["7.4.2", ""]], "type": "int"},
            "ipver": {"v_range": [["7.4.2", ""]], "type": "int"},
            "l4-wd10": {"v_range": [["7.4.2", ""]], "type": "int"},
            "l4-wd11": {"v_range": [["7.4.2", ""]], "type": "int"},
            "l4-wd8": {"v_range": [["7.4.2", ""]], "type": "int"},
            "l4-wd9": {"v_range": [["7.4.2", ""]], "type": "int"},
            "mf": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "protocol": {"v_range": [["7.4.2", ""]], "type": "int"},
            "slink": {"v_range": [["7.4.2", ""]], "type": "int"},
            "smac-change": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "sp": {"v_range": [["7.4.2", ""]], "type": "int"},
            "src-cfi": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "src-prio": {"v_range": [["7.4.2", ""]], "type": "int"},
            "src-updt": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "srcip": {"v_range": [["7.4.2", ""]], "type": "str"},
            "srcipv6": {"v_range": [["7.4.2", ""]], "type": "str"},
            "srcmac": {"v_range": [["7.4.2", ""]], "type": "str"},
            "srcport": {"v_range": [["7.4.2", ""]], "type": "int"},
            "svid": {"v_range": [["7.4.2", ""]], "type": "int"},
            "tcp-ack": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "tcp-cwr": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "tcp-ece": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "tcp-fin": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "tcp-push": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "tcp-rst": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "tcp-syn": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "tcp-urg": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "tgt-cfi": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "tgt-prio": {"v_range": [["7.4.2", ""]], "type": "int"},
            "tgt-updt": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "tgt-v": {"v_range": [["7.4.2", ""]], "choices": ["invalid", "valid"], "type": "str"},
            "tos": {"v_range": [["7.4.2", ""]], "type": "int"},
            "tp": {"v_range": [["7.4.2", ""]], "type": "int"},
            "ttl": {"v_range": [["7.4.2", ""]], "type": "int"},
            "tvid": {"v_range": [["7.4.2", ""]], "type": "int"},
            "vdid": {"v_range": [["7.4.2", ""]], "type": "int"}
        }
    },
    "system_npu_nputcam_mask": {
        "stated": false,
        "options": {
            "df": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "dstip": {"v_range": [["7.4.2", ""]], "type": "str"},
            "dstipv6": {"v_range": [["7.4.2", ""]], "type": "str"},
            "dstmac": {"v_range": [["7.4.2", ""]], "type": "str"},
            "dstport": {"v_range": [["7.4.2", ""]], "type": "int"},
            "ethertype": {"v_range": [["7.4.2", ""]], "type": "str"},
            "ext-tag": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "frag-off": {"v_range": [["7.4.2", ""]], "type": "int"},
            "gen-buf-cnt": {"v_range": [["7.4.2", ""]], "type": "int"},
            "gen-iv": {"v_range": [["7.4.2", ""]], "choices": ["invalid", "valid"], "type": "str"},
            "gen-l3-flags": {"v_range": [["7.4.2", ""]], "type": "int"},
            "gen-l4-flags": {"v_range": [["7.4.2", ""]], "type": "int"},
            "gen-pkt-ctrl": {"v_range": [["7.4.2", ""]], "type": "int"},
            "gen-pri": {"v_range": [["7.4.2", ""]], "type": "int"},
            "gen-pri-v": {"v_range": [["7.4.2", ""]], "choices": ["invalid", "valid"], "type": "str"},
            "gen-tv": {"v_range": [["7.4.2", ""]], "choices": ["invalid", "valid"], "type": "str"},
            "ihl": {"v_range": [["7.4.2", ""]], "type": "int"},
            "ip4-id": {"v_range": [["7.4.2", ""]], "type": "int"},
            "ip6-fl": {"v_range": [["7.4.2", ""]], "type": "int"},
            "ipver": {"v_range": [["7.4.2", ""]], "type": "int"},
            "l4-wd10": {"v_range": [["7.4.2", ""]], "type": "int"},
            "l4-wd11": {"v_range": [["7.4.2", ""]], "type": "int"},
            "l4-wd8": {"v_range": [["7.4.2", ""]], "type": "int"},
            "l4-wd9": {"v_range": [["7.4.2", ""]], "type": "int"},
            "mf": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "protocol": {"v_range": [["7.4.2", ""]], "type": "int"},
            "slink": {"v_range": [["7.4.2", ""]], "type": "int"},
            "smac-change": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "sp": {"v_range": [["7.4.2", ""]], "type": "int"},
            "src-cfi": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "src-prio": {"v_range": [["7.4.2", ""]], "type": "int"},
            "src-updt": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "srcip": {"v_range": [["7.4.2", ""]], "type": "str"},
            "srcipv6": {"v_range": [["7.4.2", ""]], "type": "str"},
            "srcmac": {"v_range": [["7.4.2", ""]], "type": "str"},
            "srcport": {"v_range": [["7.4.2", ""]], "type": "int"},
            "svid": {"v_range": [["7.4.2", ""]], "type": "int"},
            "tcp-ack": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "tcp-cwr": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "tcp-ece": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "tcp-fin": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "tcp-push": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "tcp-rst": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "tcp-syn": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "tcp-urg": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "tgt-cfi": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "tgt-prio": {"v_range": [["7.4.2", ""]], "type": "int"},
            "tgt-updt": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "tgt-v": {"v_range": [["7.4.2", ""]], "choices": ["invalid", "valid"], "type": "str"},
            "tos": {"v_range": [["7.4.2", ""]], "type": "int"},
            "tp": {"v_range": [["7.4.2", ""]], "type": "int"},
            "ttl": {"v_range": [["7.4.2", ""]], "type": "int"},
            "tvid": {"v_range": [["7.4.2", ""]], "type": "int"},
            "vdid": {"v_range": [["7.4.2", ""]], "type": "int"}
        }
    },
    "system_npu_nputcam_miract": {"stated": false, "options": {"vlif": {"v_range": [["7.4.2", ""]], "type": "int"}}},
    "system_npu_nputcam_priact": {
        "stated": false,
        "options": {"priority": {"v_range": [["7.4.2", ""]], "type": "int"}, "weight": {"v_range": [["7.4.2", ""]], "type": "int"}}
    },
    "system_npu_nputcam_sact": {
        "stated": false,
        "options": {
            "act": {"v_range": [["7.4.2", ""]], "type": "int"},
            "act-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "bmproc": {"v_range": [["7.4.2", ""]], "type": "int"},
            "bmproc-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "df-lif": {"v_range": [["7.4.2", ""]], "type": "int"},
            "df-lif-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "dfr": {"v_range": [["7.4.2", ""]], "type": "int"},
            "dfr-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "dmac-skip": {"v_range": [["7.4.2", ""]], "type": "int"},
            "dmac-skip-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "dosen": {"v_range": [["7.4.2", ""]], "type": "int"},
            "dosen-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "espff-proc": {"v_range": [["7.4.2", ""]], "type": "int"},
            "espff-proc-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "etype-pid": {"v_range": [["7.4.2", ""]], "type": "int"},
            "etype-pid-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "frag-proc": {"v_range": [["7.4.2", ""]], "type": "int"},
            "frag-proc-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "fwd": {"v_range": [["7.4.2", ""]], "type": "int"},
            "fwd-lif": {"v_range": [["7.4.2", ""]], "type": "int"},
            "fwd-lif-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "fwd-tvid": {"v_range": [["7.4.2", ""]], "type": "int"},
            "fwd-tvid-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "fwd-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "icpen": {"v_range": [["7.4.2", ""]], "type": "int"},
            "icpen-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "igmp-mld-snp": {"v_range": [["7.4.2", ""]], "type": "int"},
            "igmp-mld-snp-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "learn": {"v_range": [["7.4.2", ""]], "type": "int"},
            "learn-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "m-srh-ctrl": {"v_range": [["7.4.2", ""]], "type": "int"},
            "m-srh-ctrl-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "mac-id": {"v_range": [["7.4.2", ""]], "type": "int"},
            "mac-id-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "mss": {"v_range": [["7.4.2", ""]], "type": "int"},
            "mss-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "pleen": {"v_range": [["7.4.2", ""]], "type": "int"},
            "pleen-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "prio-pid": {"v_range": [["7.4.2", ""]], "type": "int"},
            "prio-pid-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "promis": {"v_range": [["7.4.2", ""]], "type": "int"},
            "promis-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "rfsh": {"v_range": [["7.4.2", ""]], "type": "int"},
            "rfsh-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "smac-skip": {"v_range": [["7.4.2", ""]], "type": "int"},
            "smac-skip-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "tp-smchk-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "tp_smchk": {"v_range": [["7.4.2", ""]], "type": "int"},
            "tpe-id": {"v_range": [["7.4.2", ""]], "type": "int"},
            "tpe-id-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "vdm": {"v_range": [["7.4.2", ""]], "type": "int"},
            "vdm-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "vdom-id": {"v_range": [["7.4.2", ""]], "type": "int"},
            "vdom-id-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "x-mode": {"v_range": [["7.4.2", ""]], "type": "int"},
            "x-mode-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "system_npu_nputcam_tact": {
        "stated": false,
        "options": {
            "act": {"v_range": [["7.4.2", ""]], "type": "int"},
            "act-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "fmtuv4-s": {"v_range": [["7.4.2", ""]], "type": "int"},
            "fmtuv4-s-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "fmtuv6-s": {"v_range": [["7.4.2", ""]], "type": "int"},
            "fmtuv6-s-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "lnkid": {"v_range": [["7.4.2", ""]], "type": "int"},
            "lnkid-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "mac-id": {"v_range": [["7.4.2", ""]], "type": "int"},
            "mac-id-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "mss-t": {"v_range": [["7.4.2", ""]], "type": "int"},
            "mss-t-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "mtuv4": {"v_range": [["7.4.2", ""]], "type": "int"},
            "mtuv4-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "mtuv6": {"v_range": [["7.4.2", ""]], "type": "int"},
            "mtuv6-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "slif-act": {"v_range": [["7.4.2", ""]], "type": "int"},
            "slif-act-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "sublnkid": {"v_range": [["7.4.2", ""]], "type": "int"},
            "sublnkid-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "tgtv-act": {"v_range": [["7.4.2", ""]], "type": "int"},
            "tgtv-act-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "tlif-act": {"v_range": [["7.4.2", ""]], "type": "int"},
            "tlif-act-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "tpeid": {"v_range": [["7.4.2", ""]], "type": "int"},
            "tpeid-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "v6fe": {"v_range": [["7.4.2", ""]], "type": "int"},
            "v6fe-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "vep-en-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "vep-slid": {"v_range": [["7.4.2", ""]], "type": "int"},
            "vep-slid-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "vep_en": {"v_range": [["7.4.2", ""]], "type": "int"},
            "xlt-lif": {"v_range": [["7.4.2", ""]], "type": "int"},
            "xlt-lif-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "xlt-vid": {"v_range": [["7.4.2", ""]], "type": "int"},
            "xlt-vid-v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "system_npu_portcpumap": {
        "stated": true,
        "options": {
            "cpu-core": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "str"},
            "interface": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "str"}
        }
    },
    "system_npu_portnpumap": {
        "stated": true,
        "options": {
            "interface": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "str"},
            "npu-group-index": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"}
        }
    },
    "system_npu_portpathoption": {"stated": false, "options": {"ports-using-npu": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "raw"}}},
    "system_npu_priorityprotocol": {
        "stated": false,
        "options": {
            "bfd": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "bgp": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "slbc": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "system_npu_ssehascan": {
        "stated": false,
        "options": {
            "gap": {"v_range": [["6.4.10", "6.4.14"], ["7.0.4", "7.0.12"], ["7.2.1", ""]], "type": "int"},
            "max-session-cnt": {"v_range": [["6.4.10", "6.4.14"], ["7.0.4", "7.0.12"], ["7.2.1", ""]], "type": "int"},
            "min-duration": {"v_range": [["6.4.10", "6.4.14"], ["7.0.4", "7.0.12"], ["7.2.1", ""]], "type": "int"}
        }
    },
    "system_npu_swehhash": {
        "stated": false,
        "options": {
            "computation": {"v_range": [["7.0.1", ""]], "choices": ["xor16", "xor8", "xor4", "crc16"], "type": "str"},
            "destination-ip-lower-16": {"v_range": [["7.0.1", ""]], "choices": ["include", "exclude"], "type": "str"},
            "destination-ip-upper-16": {"v_range": [["7.0.1", ""]], "choices": ["include", "exclude"], "type": "str"},
            "destination-port": {"v_range": [["7.0.1", ""]], "choices": ["include", "exclude"], "type": "str"},
            "ip-protocol": {"v_range": [["7.0.1", ""]], "choices": ["include", "exclude"], "type": "str"},
            "netmask-length": {"v_range": [["7.0.1", ""]], "type": "int"},
            "source-ip-lower-16": {"v_range": [["7.0.1", ""]], "choices": ["include", "exclude"], "type": "str"},
            "source-ip-upper-16": {"v_range": [["7.0.1", ""]], "choices": ["include", "exclude"], "type": "str"},
            "source-port": {"v_range": [["7.0.1", ""]], "choices": ["include", "exclude"], "type": "str"}
        }
    },
    "system_npu_swtrhash": {
        "stated": false,
        "options": {
            "draco15": {"v_range": [["7.2.4", ""]], "choices": ["disable", "enable"], "type": "str"},
            "tcp-udp-port": {"v_range": [["7.2.4", ""]], "choices": ["include", "exclude"], "type": "str"}
        }
    },
    "system_npu_tcptimeoutprofile": {
        "stated": true,
        "options": {
            "close-wait": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"},
            "fin-wait": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"},
            "id": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "required": true, "type": "int"},
            "syn-sent": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"},
            "syn-wait": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"},
            "tcp-idle": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"},
            "time-wait": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"}
        }
    },
    "system_npu_udptimeoutprofile": {
        "stated": true,
        "options": {
            "id": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "required": true, "type": "int"},
            "udp-idle": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"}
        }
    },
    "system_ntp": {
        "stated": false,
        "options": {
            "ntpserver": {
                "type": "list",
                "options": {
                    "authentication": {"choices": ["disable", "enable"], "type": "str"},
                    "id": {"type": "int"},
                    "key": {"no_log": true, "type": "raw"},
                    "key-id": {"no_log": true, "type": "int"},
                    "ntpv3": {"choices": ["disable", "enable"], "type": "str"},
                    "server": {"type": "str"},
                    "maxpoll": {"v_range": [["6.4.8", "6.4.14"], ["7.0.3", ""]], "type": "int"},
                    "minpoll": {"v_range": [["6.4.8", "6.4.14"], ["7.0.3", ""]], "type": "int"}
                },
                "elements": "dict"
            },
            "status": {"choices": ["disable", "enable"], "type": "str"},
            "sync_interval": {"v_range": [["6.0.0", "6.4.7"], ["7.0.0", "7.0.2"]], "type": "int"}
        }
    },
    "system_ntp_ntpserver": {
        "stated": true,
        "options": {
            "authentication": {"choices": ["disable", "enable"], "type": "str"},
            "id": {"required": true, "type": "int"},
            "key": {"no_log": true, "type": "raw"},
            "key-id": {"no_log": true, "type": "int"},
            "ntpv3": {"choices": ["disable", "enable"], "type": "str"},
            "server": {"type": "str"},
            "maxpoll": {"v_range": [["6.4.8", "6.4.14"], ["7.0.3", ""]], "type": "int"},
            "minpoll": {"v_range": [["6.4.8", "6.4.14"], ["7.0.3", ""]], "type": "int"}
        }
    },
    "system_objecttag": {"stated": true, "options": {"name": {"v_range": [["6.2.0", "6.4.14"]], "required": true, "type": "str"}}},
    "system_objecttagging": {
        "stated": true,
        "options": {
            "address": {"choices": ["optional", "mandatory", "disable"], "type": "str"},
            "category": {"required": true, "type": "str"},
            "color": {"type": "int"},
            "device": {"choices": ["optional", "mandatory", "disable"], "type": "str"},
            "interface": {"choices": ["optional", "mandatory", "disable"], "type": "str"},
            "multiple": {"choices": ["disable", "enable"], "type": "str"},
            "tags": {"type": "raw"}
        }
    },
    "system_passwordpolicy": {
        "stated": false,
        "options": {
            "change-4-characters": {"choices": ["disable", "enable"], "type": "str"},
            "expire": {"type": "int"},
            "minimum-length": {"type": "int"},
            "must-contain": {"type": "list", "choices": ["upper-case-letter", "lower-case-letter", "number", "non-alphanumeric"], "elements": "str"},
            "status": {"choices": ["disable", "enable"], "type": "str"},
            "password-history": {"v_range": [["7.6.0", ""]], "no_log": true, "type": "int"}
        }
    },
    "system_replacemsggroup": {
        "stated": true,
        "options": {
            "admin": {
                "type": "list",
                "options": {
                    "buffer": {"type": "str"},
                    "format": {"choices": ["none", "text", "html", "wml"], "type": "str"},
                    "header": {"choices": ["none", "http", "8bit"], "type": "str"},
                    "msg-type": {"type": "str"}
                },
                "elements": "dict"
            },
            "alertmail": {
                "type": "list",
                "options": {
                    "buffer": {"type": "str"},
                    "format": {"choices": ["none", "text", "html", "wml"], "type": "str"},
                    "header": {"choices": ["none", "http", "8bit"], "type": "str"},
                    "msg-type": {"type": "str"},
                    "id": {"v_range": [["6.4.11", "6.4.14"], ["7.0.6", "7.0.12"], ["7.2.3", ""]], "type": "int"}
                },
                "elements": "dict"
            },
            "auth": {
                "type": "list",
                "options": {
                    "buffer": {"type": "str"},
                    "format": {"choices": ["none", "text", "html", "wml"], "type": "str"},
                    "header": {"choices": ["none", "http", "8bit"], "type": "str"},
                    "msg-type": {"type": "str"}
                },
                "elements": "dict"
            },
            "comment": {"type": "str"},
            "custom-message": {
                "type": "list",
                "options": {
                    "buffer": {"type": "str"},
                    "format": {"choices": ["none", "text", "html", "wml"], "type": "str"},
                    "header": {"choices": ["none", "http", "8bit"], "type": "str"},
                    "msg-type": {"type": "str"}
                },
                "elements": "dict"
            },
            "device-detection-portal": {
                "type": "list",
                "options": {
                    "buffer": {"type": "str"},
                    "format": {"choices": ["none", "text", "html", "wml"], "type": "str"},
                    "header": {"choices": ["none", "http", "8bit"], "type": "str"},
                    "msg-type": {"type": "str"}
                },
                "elements": "dict"
            },
            "ec": {
                "v_range": [["6.0.0", "7.2.1"]],
                "type": "list",
                "options": {
                    "buffer": {"v_range": [["6.0.0", "7.2.1"]], "type": "str"},
                    "format": {"v_range": [["6.0.0", "7.2.1"]], "choices": ["none", "text", "html", "wml"], "type": "str"},
                    "header": {"v_range": [["6.0.0", "7.2.1"]], "choices": ["none", "http", "8bit"], "type": "str"},
                    "msg-type": {"v_range": [["6.0.0", "7.2.1"]], "type": "str"}
                },
                "elements": "dict"
            },
            "fortiguard-wf": {
                "type": "list",
                "options": {
                    "buffer": {"type": "str"},
                    "format": {"choices": ["none", "text", "html", "wml"], "type": "str"},
                    "header": {"choices": ["none", "http", "8bit"], "type": "str"},
                    "msg-type": {"type": "str"}
                },
                "elements": "dict"
            },
            "ftp": {
                "type": "list",
                "options": {
                    "buffer": {"type": "str"},
                    "format": {"choices": ["none", "text", "html", "wml"], "type": "str"},
                    "header": {"choices": ["none", "http", "8bit"], "type": "str"},
                    "msg-type": {"type": "str"}
                },
                "elements": "dict"
            },
            "group-type": {"choices": ["default", "utm", "auth", "ec", "captive-portal"], "type": "str"},
            "http": {
                "type": "list",
                "options": {
                    "buffer": {"type": "str"},
                    "format": {"choices": ["none", "text", "html", "wml"], "type": "str"},
                    "header": {"choices": ["none", "http", "8bit"], "type": "str"},
                    "msg-type": {"type": "str"}
                },
                "elements": "dict"
            },
            "icap": {
                "type": "list",
                "options": {
                    "buffer": {"type": "str"},
                    "format": {"choices": ["none", "text", "html", "wml"], "type": "str"},
                    "header": {"choices": ["none", "http", "8bit"], "type": "str"},
                    "msg-type": {"type": "str"}
                },
                "elements": "dict"
            },
            "mail": {
                "type": "list",
                "options": {
                    "buffer": {"type": "str"},
                    "format": {"choices": ["none", "text", "html", "wml"], "type": "str"},
                    "header": {"choices": ["none", "http", "8bit"], "type": "str"},
                    "msg-type": {"type": "str"}
                },
                "elements": "dict"
            },
            "mm1": {
                "type": "list",
                "options": {
                    "add-smil": {"choices": ["disable", "enable"], "type": "str"},
                    "charset": {"choices": ["us-ascii", "utf-8"], "type": "str"},
                    "class": {"choices": ["personal", "advertisement", "information", "automatic", "not-included"], "type": "str"},
                    "format": {"choices": ["none", "text", "html", "wml"], "type": "str"},
                    "from": {"type": "str"},
                    "from-sender": {"choices": ["disable", "enable"], "type": "str"},
                    "header": {"choices": ["none", "http", "8bit"], "type": "str"},
                    "image": {"type": "str"},
                    "fmgr_message": {"type": "str"},
                    "msg-type": {"type": "str"},
                    "priority": {"choices": ["low", "normal", "high", "not-included"], "type": "str"},
                    "rsp-status": {
                        "choices": [
                            "ok", "err-unspecified", "err-srv-denied", "err-msg-fmt-corrupt", "err-snd-addr-unresolv", "err-msg-not-found",
                            "err-net-prob", "err-content-not-accept", "err-unsupp-msg"
                        ],
                        "type": "str"
                    },
                    "rsp-text": {"type": "str"},
                    "sender-visibility": {"choices": ["hide", "show", "not-specified"], "type": "str"},
                    "smil-part": {"type": "str"},
                    "subject": {"type": "str"}
                },
                "elements": "dict"
            },
            "mm3": {
                "type": "list",
                "options": {
                    "add-html": {"choices": ["disable", "enable"], "type": "str"},
                    "charset": {"choices": ["us-ascii", "utf-8"], "type": "str"},
                    "format": {"choices": ["none", "text", "html", "wml"], "type": "str"},
                    "from": {"type": "str"},
                    "from-sender": {"choices": ["disable", "enable"], "type": "str"},
                    "header": {"choices": ["none", "http", "8bit"], "type": "str"},
                    "html-part": {"type": "str"},
                    "image": {"type": "str"},
                    "fmgr_message": {"type": "str"},
                    "msg-type": {"type": "str"},
                    "priority": {"choices": ["low", "normal", "high", "not-included"], "type": "str"},
                    "subject": {"type": "str"}
                },
                "elements": "dict"
            },
            "mm4": {
                "type": "list",
                "options": {
                    "add-smil": {"choices": ["disable", "enable"], "type": "str"},
                    "charset": {"choices": ["us-ascii", "utf-8"], "type": "str"},
                    "class": {"choices": ["personal", "advertisement", "informational", "auto", "not-included"], "type": "str"},
                    "domain": {"type": "str"},
                    "format": {"choices": ["none", "text", "html", "wml"], "type": "str"},
                    "from": {"type": "str"},
                    "from-sender": {"choices": ["disable", "enable"], "type": "str"},
                    "header": {"choices": ["none", "http", "8bit"], "type": "str"},
                    "image": {"type": "str"},
                    "fmgr_message": {"type": "str"},
                    "msg-type": {"type": "str"},
                    "priority": {"choices": ["low", "normal", "high", "not-included"], "type": "str"},
                    "rsp-status": {
                        "choices": [
                            "ok", "err-unspecified", "err-srv-denied", "err-msg-fmt-corrupt", "err-snd-addr-unresolv", "err-net-prob",
                            "err-content-not-accept", "err-unsupp-msg"
                        ],
                        "type": "str"
                    },
                    "smil-part": {"type": "str"},
                    "subject": {"type": "str"}
                },
                "elements": "dict"
            },
            "mm7": {
                "type": "list",
                "options": {
                    "add-smil": {"choices": ["disable", "enable"], "type": "str"},
                    "addr-type": {"choices": ["rfc2822-addr", "number", "short-code"], "type": "str"},
                    "allow-content-adaptation": {"choices": ["disable", "enable"], "type": "str"},
                    "charset": {"choices": ["us-ascii", "utf-8"], "type": "str"},
                    "class": {"choices": ["personal", "advertisement", "informational", "auto", "not-included"], "type": "str"},
                    "format": {"choices": ["none", "text", "html", "wml"], "type": "str"},
                    "from": {"type": "str"},
                    "from-sender": {"choices": ["disable", "enable"], "type": "str"},
                    "header": {"choices": ["none", "http", "8bit"], "type": "str"},
                    "image": {"type": "str"},
                    "fmgr_message": {"type": "str"},
                    "msg-type": {"type": "str"},
                    "priority": {"choices": ["low", "normal", "high", "not-included"], "type": "str"},
                    "rsp-status": {
                        "choices": [
                            "success", "partial-success", "client-err", "oper-restrict", "addr-err", "addr-not-found", "content-refused",
                            "msg-id-not-found", "link-id-not-found", "msg-fmt-corrupt", "app-id-not-found", "repl-app-id-not-found", "srv-err",
                            "not-possible", "msg-rejected", "multiple-addr-not-supp", "app-addr-not-supp", "gen-service-err", "improper-ident",
                            "unsupp-ver", "unsupp-oper", "validation-err", "service-err", "service-unavail", "service-denied", "app-denied"
                        ],
                        "type": "str"
                    },
                    "smil-part": {"type": "str"},
                    "subject": {"type": "str"}
                },
                "elements": "dict"
            },
            "mms": {
                "type": "list",
                "options": {
                    "buffer": {"type": "str"},
                    "charset": {"choices": ["us-ascii", "utf-8"], "type": "str"},
                    "format": {"choices": ["none", "text", "html", "wml"], "type": "str"},
                    "header": {"choices": ["none", "http", "8bit"], "type": "str"},
                    "image": {"type": "str"},
                    "msg-type": {"type": "str"}
                },
                "elements": "dict"
            },
            "nac-quar": {
                "type": "list",
                "options": {
                    "buffer": {"type": "str"},
                    "format": {"choices": ["none", "text", "html", "wml"], "type": "str"},
                    "header": {"choices": ["none", "http", "8bit"], "type": "str"},
                    "msg-type": {"type": "str"},
                    "id": {"v_range": [["6.4.11", "6.4.14"], ["7.0.6", "7.0.12"], ["7.2.3", ""]], "type": "int"}
                },
                "elements": "dict"
            },
            "name": {"required": true, "type": "str"},
            "nntp": {
                "type": "list",
                "options": {
                    "buffer": {"type": "str"},
                    "format": {"choices": ["none", "text", "html", "wml"], "type": "str"},
                    "header": {"choices": ["none", "http", "8bit"], "type": "str"},
                    "msg-type": {"type": "str"}
                },
                "elements": "dict"
            },
            "spam": {
                "type": "list",
                "options": {
                    "buffer": {"type": "str"},
                    "format": {"choices": ["none", "text", "html", "wml"], "type": "str"},
                    "header": {"choices": ["none", "http", "8bit"], "type": "str"},
                    "msg-type": {"type": "str"}
                },
                "elements": "dict"
            },
            "sslvpn": {
                "type": "list",
                "options": {
                    "buffer": {"type": "str"},
                    "format": {"choices": ["none", "text", "html", "wml"], "type": "str"},
                    "header": {"choices": ["none", "http", "8bit"], "type": "str"},
                    "msg-type": {"type": "str"}
                },
                "elements": "dict"
            },
            "traffic-quota": {
                "type": "list",
                "options": {
                    "buffer": {"type": "str"},
                    "format": {"choices": ["none", "text", "html", "wml"], "type": "str"},
                    "header": {"choices": ["none", "http", "8bit"], "type": "str"},
                    "msg-type": {"type": "str"}
                },
                "elements": "dict"
            },
            "utm": {
                "type": "list",
                "options": {
                    "buffer": {"type": "str"},
                    "format": {"choices": ["none", "text", "html", "wml"], "type": "str"},
                    "header": {"choices": ["none", "http", "8bit"], "type": "str"},
                    "msg-type": {"type": "str"}
                },
                "elements": "dict"
            },
            "webproxy": {
                "type": "list",
                "options": {
                    "buffer": {"type": "str"},
                    "format": {"choices": ["none", "text", "html", "wml"], "type": "str"},
                    "header": {"choices": ["none", "http", "8bit"], "type": "str"},
                    "msg-type": {"type": "str"}
                },
                "elements": "dict"
            },
            "automation": {
                "v_range": [["7.0.0", ""]],
                "type": "list",
                "options": {
                    "buffer": {"v_range": [["7.0.0", ""]], "type": "str"},
                    "format": {"v_range": [["7.0.0", ""]], "choices": ["none", "text", "html"], "type": "str"},
                    "header": {"v_range": [["7.0.0", ""]], "choices": ["none", "http", "8bit"], "type": "str"},
                    "msg-type": {"v_range": [["7.0.0", ""]], "type": "str"}
                },
                "elements": "dict"
            }
        }
    },
    "system_replacemsggroup_admin": {
        "stated": true,
        "options": {
            "buffer": {"type": "str"},
            "format": {"choices": ["none", "text", "html", "wml"], "type": "str"},
            "header": {"choices": ["none", "http", "8bit"], "type": "str"},
            "msg-type": {"required": true, "type": "str"}
        }
    },
    "system_replacemsggroup_alertmail": {
        "stated": true,
        "options": {
            "buffer": {"type": "str"},
            "format": {"choices": ["none", "text", "html", "wml"], "type": "str"},
            "header": {"choices": ["none", "http", "8bit"], "type": "str"},
            "msg-type": {"required": true, "type": "str"},
            "id": {"v_range": [["6.4.11", "6.4.14"], ["7.0.6", "7.0.12"], ["7.2.3", ""]], "type": "int"}
        }
    },
    "system_replacemsggroup_auth": {
        "stated": true,
        "options": {
            "buffer": {"type": "str"},
            "format": {"choices": ["none", "text", "html", "wml"], "type": "str"},
            "header": {"choices": ["none", "http", "8bit"], "type": "str"},
            "msg-type": {"required": true, "type": "str"}
        }
    },
    "system_replacemsggroup_automation": {
        "stated": true,
        "options": {
            "buffer": {"v_range": [["7.0.0", ""]], "type": "str"},
            "format": {"v_range": [["7.0.0", ""]], "choices": ["none", "text", "html"], "type": "str"},
            "header": {"v_range": [["7.0.0", ""]], "choices": ["none", "http", "8bit"], "type": "str"},
            "msg-type": {"v_range": [["7.0.0", ""]], "type": "str"}
        }
    },
    "system_replacemsggroup_custommessage": {
        "stated": true,
        "options": {
            "buffer": {"type": "str"},
            "format": {"choices": ["none", "text", "html", "wml"], "type": "str"},
            "header": {"choices": ["none", "http", "8bit"], "type": "str"},
            "msg-type": {"required": true, "type": "str"}
        }
    },
    "system_replacemsggroup_devicedetectionportal": {
        "stated": true,
        "options": {
            "buffer": {"type": "str"},
            "format": {"choices": ["none", "text", "html", "wml"], "type": "str"},
            "header": {"choices": ["none", "http", "8bit"], "type": "str"},
            "msg-type": {"required": true, "type": "str"}
        }
    },
    "system_replacemsggroup_ec": {
        "stated": true,
        "options": {
            "buffer": {"v_range": [["6.0.0", "7.2.1"]], "type": "str"},
            "format": {"v_range": [["6.0.0", "7.2.1"]], "choices": ["none", "text", "html", "wml"], "type": "str"},
            "header": {"v_range": [["6.0.0", "7.2.1"]], "choices": ["none", "http", "8bit"], "type": "str"},
            "msg-type": {"v_range": [["6.0.0", "7.2.1"]], "required": true, "type": "str"}
        }
    },
    "system_replacemsggroup_fortiguardwf": {
        "stated": true,
        "options": {
            "buffer": {"type": "str"},
            "format": {"choices": ["none", "text", "html", "wml"], "type": "str"},
            "header": {"choices": ["none", "http", "8bit"], "type": "str"},
            "msg-type": {"required": true, "type": "str"}
        }
    },
    "system_replacemsggroup_ftp": {
        "stated": true,
        "options": {
            "buffer": {"type": "str"},
            "format": {"choices": ["none", "text", "html", "wml"], "type": "str"},
            "header": {"choices": ["none", "http", "8bit"], "type": "str"},
            "msg-type": {"required": true, "type": "str"}
        }
    },
    "system_replacemsggroup_http": {
        "stated": true,
        "options": {
            "buffer": {"type": "str"},
            "format": {"choices": ["none", "text", "html", "wml"], "type": "str"},
            "header": {"choices": ["none", "http", "8bit"], "type": "str"},
            "msg-type": {"required": true, "type": "str"}
        }
    },
    "system_replacemsggroup_icap": {
        "stated": true,
        "options": {
            "buffer": {"type": "str"},
            "format": {"choices": ["none", "text", "html", "wml"], "type": "str"},
            "header": {"choices": ["none", "http", "8bit"], "type": "str"},
            "msg-type": {"required": true, "type": "str"}
        }
    },
    "system_replacemsggroup_mail": {
        "stated": true,
        "options": {
            "buffer": {"type": "str"},
            "format": {"choices": ["none", "text", "html", "wml"], "type": "str"},
            "header": {"choices": ["none", "http", "8bit"], "type": "str"},
            "msg-type": {"required": true, "type": "str"}
        }
    },
    "system_replacemsggroup_mm1": {
        "stated": true,
        "options": {
            "add-smil": {"choices": ["disable", "enable"], "type": "str"},
            "charset": {"choices": ["us-ascii", "utf-8"], "type": "str"},
            "class": {"choices": ["personal", "advertisement", "information", "automatic", "not-included"], "type": "str"},
            "format": {"choices": ["none", "text", "html", "wml"], "type": "str"},
            "from": {"type": "str"},
            "from-sender": {"choices": ["disable", "enable"], "type": "str"},
            "header": {"choices": ["none", "http", "8bit"], "type": "str"},
            "image": {"type": "str"},
            "fmgr_message": {"type": "str"},
            "msg-type": {"required": true, "type": "str"},
            "priority": {"choices": ["low", "normal", "high", "not-included"], "type": "str"},
            "rsp-status": {
                "choices": [
                    "ok", "err-unspecified", "err-srv-denied", "err-msg-fmt-corrupt", "err-snd-addr-unresolv", "err-msg-not-found", "err-net-prob",
                    "err-content-not-accept", "err-unsupp-msg"
                ],
                "type": "str"
            },
            "rsp-text": {"type": "str"},
            "sender-visibility": {"choices": ["hide", "show", "not-specified"], "type": "str"},
            "smil-part": {"type": "str"},
            "subject": {"type": "str"}
        }
    },
    "system_replacemsggroup_mm3": {
        "stated": true,
        "options": {
            "add-html": {"choices": ["disable", "enable"], "type": "str"},
            "charset": {"choices": ["us-ascii", "utf-8"], "type": "str"},
            "format": {"choices": ["none", "text", "html", "wml"], "type": "str"},
            "from": {"type": "str"},
            "from-sender": {"choices": ["disable", "enable"], "type": "str"},
            "header": {"choices": ["none", "http", "8bit"], "type": "str"},
            "html-part": {"type": "str"},
            "image": {"type": "str"},
            "fmgr_message": {"type": "str"},
            "msg-type": {"required": true, "type": "str"},
            "priority": {"choices": ["low", "normal", "high", "not-included"], "type": "str"},
            "subject": {"type": "str"}
        }
    },
    "system_replacemsggroup_mm4": {
        "stated": true,
        "options": {
            "add-smil": {"choices": ["disable", "enable"], "type": "str"},
            "charset": {"choices": ["us-ascii", "utf-8"], "type": "str"},
            "class": {"choices": ["personal", "advertisement", "informational", "auto", "not-included"], "type": "str"},
            "domain": {"type": "str"},
            "format": {"choices": ["none", "text", "html", "wml"], "type": "str"},
            "from": {"type": "str"},
            "from-sender": {"choices": ["disable", "enable"], "type": "str"},
            "header": {"choices": ["none", "http", "8bit"], "type": "str"},
            "image": {"type": "str"},
            "fmgr_message": {"type": "str"},
            "msg-type": {"required": true, "type": "str"},
            "priority": {"choices": ["low", "normal", "high", "not-included"], "type": "str"},
            "rsp-status": {
                "choices": [
                    "ok", "err-unspecified", "err-srv-denied", "err-msg-fmt-corrupt", "err-snd-addr-unresolv", "err-net-prob", "err-content-not-accept",
                    "err-unsupp-msg"
                ],
                "type": "str"
            },
            "smil-part": {"type": "str"},
            "subject": {"type": "str"}
        }
    },
    "system_replacemsggroup_mm7": {
        "stated": true,
        "options": {
            "add-smil": {"choices": ["disable", "enable"], "type": "str"},
            "addr-type": {"choices": ["rfc2822-addr", "number", "short-code"], "type": "str"},
            "allow-content-adaptation": {"choices": ["disable", "enable"], "type": "str"},
            "charset": {"choices": ["us-ascii", "utf-8"], "type": "str"},
            "class": {"choices": ["personal", "advertisement", "informational", "auto", "not-included"], "type": "str"},
            "format": {"choices": ["none", "text", "html", "wml"], "type": "str"},
            "from": {"type": "str"},
            "from-sender": {"choices": ["disable", "enable"], "type": "str"},
            "header": {"choices": ["none", "http", "8bit"], "type": "str"},
            "image": {"type": "str"},
            "fmgr_message": {"type": "str"},
            "msg-type": {"required": true, "type": "str"},
            "priority": {"choices": ["low", "normal", "high", "not-included"], "type": "str"},
            "rsp-status": {
                "choices": [
                    "success", "partial-success", "client-err", "oper-restrict", "addr-err", "addr-not-found", "content-refused", "msg-id-not-found",
                    "link-id-not-found", "msg-fmt-corrupt", "app-id-not-found", "repl-app-id-not-found", "srv-err", "not-possible", "msg-rejected",
                    "multiple-addr-not-supp", "app-addr-not-supp", "gen-service-err", "improper-ident", "unsupp-ver", "unsupp-oper", "validation-err",
                    "service-err", "service-unavail", "service-denied", "app-denied"
                ],
                "type": "str"
            },
            "smil-part": {"type": "str"},
            "subject": {"type": "str"}
        }
    },
    "system_replacemsggroup_mms": {
        "stated": true,
        "options": {
            "buffer": {"type": "str"},
            "charset": {"choices": ["us-ascii", "utf-8"], "type": "str"},
            "format": {"choices": ["none", "text", "html", "wml"], "type": "str"},
            "header": {"choices": ["none", "http", "8bit"], "type": "str"},
            "image": {"type": "str"},
            "msg-type": {"required": true, "type": "str"}
        }
    },
    "system_replacemsggroup_nacquar": {
        "stated": true,
        "options": {
            "buffer": {"type": "str"},
            "format": {"choices": ["none", "text", "html", "wml"], "type": "str"},
            "header": {"choices": ["none", "http", "8bit"], "type": "str"},
            "msg-type": {"required": true, "type": "str"},
            "id": {"v_range": [["6.4.11", "6.4.14"], ["7.0.6", "7.0.12"], ["7.2.3", ""]], "type": "int"}
        }
    },
    "system_replacemsggroup_nntp": {
        "stated": true,
        "options": {
            "buffer": {"type": "str"},
            "format": {"choices": ["none", "text", "html", "wml"], "type": "str"},
            "header": {"choices": ["none", "http", "8bit"], "type": "str"},
            "msg-type": {"required": true, "type": "str"}
        }
    },
    "system_replacemsggroup_spam": {
        "stated": true,
        "options": {
            "buffer": {"type": "str"},
            "format": {"choices": ["none", "text", "html", "wml"], "type": "str"},
            "header": {"choices": ["none", "http", "8bit"], "type": "str"},
            "msg-type": {"required": true, "type": "str"}
        }
    },
    "system_replacemsggroup_sslvpn": {
        "stated": true,
        "options": {
            "buffer": {"type": "str"},
            "format": {"choices": ["none", "text", "html", "wml"], "type": "str"},
            "header": {"choices": ["none", "http", "8bit"], "type": "str"},
            "msg-type": {"required": true, "type": "str"}
        }
    },
    "system_replacemsggroup_trafficquota": {
        "stated": true,
        "options": {
            "buffer": {"type": "str"},
            "format": {"choices": ["none", "text", "html", "wml"], "type": "str"},
            "header": {"choices": ["none", "http", "8bit"], "type": "str"},
            "msg-type": {"required": true, "type": "str"}
        }
    },
    "system_replacemsggroup_utm": {
        "stated": true,
        "options": {
            "buffer": {"type": "str"},
            "format": {"choices": ["none", "text", "html", "wml"], "type": "str"},
            "header": {"choices": ["none", "http", "8bit"], "type": "str"},
            "msg-type": {"required": true, "type": "str"}
        }
    },
    "system_replacemsggroup_webproxy": {
        "stated": true,
        "options": {
            "buffer": {"type": "str"},
            "format": {"choices": ["none", "text", "html", "wml"], "type": "str"},
            "header": {"choices": ["none", "http", "8bit"], "type": "str"},
            "msg-type": {"required": true, "type": "str"}
        }
    },
    "system_replacemsgimage": {
        "stated": true,
        "options": {
            "image-base64": {"type": "str"},
            "image-type": {"choices": ["gif", "jpg", "tiff", "png"], "type": "str"},
            "name": {"required": true, "type": "str"}
        }
    },
    "system_report_autocache": {
        "stated": false,
        "options": {
            "aggressive-schedule": {"choices": ["disable", "enable"], "type": "str"},
            "order": {"choices": ["oldest-first"], "type": "str"},
            "status": {"choices": ["disable", "enable"], "type": "str"},
            "sche-rpt-only": {"v_range": [["7.0.8", "7.0.12"], ["7.2.3", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "system_report_estbrowsetime": {
        "stated": false,
        "options": {"max-read-time": {"type": "int"}, "status": {"choices": ["disable", "enable"], "type": "str"}}
    },
    "system_report_group": {
        "stated": true,
        "options": {
            "adom": {"type": "str"},
            "case-insensitive": {"choices": ["disable", "enable"], "type": "str"},
            "chart-alternative": {"type": "list", "options": {"chart-name": {"type": "str"}, "chart-replace": {"type": "str"}}, "elements": "dict"},
            "group-by": {
                "type": "list",
                "options": {
                    "var-expression": {"type": "str"},
                    "var-name": {"type": "str"},
                    "var-type": {"choices": ["integer", "string", "enum", "ip"], "type": "str"}
                },
                "elements": "dict"
            },
            "group-id": {"required": true, "type": "int"},
            "report-like": {"type": "str"}
        }
    },
    "system_report_group_chartalternative": {
        "stated": true,
        "options": {"chart-name": {"required": true, "type": "str"}, "chart-replace": {"type": "str"}}
    },
    "system_report_group_groupby": {
        "stated": true,
        "options": {
            "var-expression": {"type": "str"},
            "var-name": {"required": true, "type": "str"},
            "var-type": {"choices": ["integer", "string", "enum", "ip"], "type": "str"}
        }
    },
    "system_report_setting": {
        "stated": false,
        "options": {
            "aggregate-report": {"choices": ["disable", "enable"], "type": "str"},
            "hcache-lossless": {"choices": ["disable", "enable"], "type": "str"},
            "ldap-cache-timeout": {"type": "int"},
            "max-table-rows": {"type": "int"},
            "report-priority": {"choices": ["high", "low", "auto"], "type": "str"},
            "template-auto-install": {"choices": ["default", "english"], "type": "str"},
            "week-start": {"choices": ["sun", "mon"], "type": "str"},
            "capwap-port": {"v_range": [["6.2.2", ""]], "type": "int"},
            "capwap-service": {"v_range": [["6.2.2", ""]], "type": "str"},
            "exclude-capwap": {"v_range": [["6.2.2", ""]], "choices": ["disable", "by-port", "by-service"], "type": "str"},
            "max-rpt-pdf-rows": {"v_range": [["7.0.4", ""]], "type": "int"}
        }
    },
    "system_route": {
        "stated": true,
        "options": {"device": {"type": "str"}, "dst": {"type": "str"}, "gateway": {"type": "str"}, "seq_num": {"required": true, "type": "int"}}
    },
    "system_route6": {
        "stated": true,
        "options": {"device": {"type": "str"}, "dst": {"type": "str"}, "gateway": {"type": "str"}, "prio": {"required": true, "type": "int"}}
    },
    "system_saml": {
        "stated": false,
        "options": {
            "acs-url": {"type": "str"},
            "cert": {"type": "str"},
            "entity-id": {"type": "str"},
            "idp-cert": {"type": "str"},
            "idp-entity-id": {"type": "str"},
            "idp-single-logout-url": {"type": "str"},
            "idp-single-sign-on-url": {"type": "str"},
            "login-auto-redirect": {"choices": ["disable", "enable"], "type": "str"},
            "role": {"choices": ["IDP", "SP", "FAB-SP"], "type": "str"},
            "server-address": {"type": "str"},
            "service-providers": {
                "type": "list",
                "options": {
                    "idp-entity-id": {"type": "str"},
                    "idp-single-logout-url": {"type": "str"},
                    "idp-single-sign-on-url": {"type": "str"},
                    "name": {"type": "str"},
                    "prefix": {"type": "str"},
                    "sp-cert": {"type": "str"},
                    "sp-entity-id": {"type": "str"},
                    "sp-single-logout-url": {"type": "str"},
                    "sp-single-sign-on-url": {"type": "str"},
                    "sp-adom": {"v_range": [["7.2.0", ""]], "type": "str"},
                    "sp-profile": {"v_range": [["7.2.0", ""]], "type": "str"}
                },
                "elements": "dict"
            },
            "sls-url": {"type": "str"},
            "status": {"choices": ["disable", "enable"], "type": "str"},
            "default-profile": {"v_range": [["6.2.5", ""]], "type": "str"},
            "fabric-idp": {
                "v_range": [["6.4.0", ""]],
                "type": "list",
                "options": {
                    "dev-id": {"v_range": [["6.4.0", ""]], "type": "str"},
                    "idp-cert": {"v_range": [["6.4.0", ""]], "type": "str"},
                    "idp-entity-id": {"v_range": [["6.4.0", ""]], "type": "str"},
                    "idp-single-logout-url": {"v_range": [["6.4.0", ""]], "type": "str"},
                    "idp-single-sign-on-url": {"v_range": [["6.4.0", ""]], "type": "str"},
                    "idp-status": {"v_range": [["6.4.0", ""]], "choices": ["disable", "enable"], "type": "str"}
                },
                "elements": "dict"
            },
            "forticloud-sso": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "user-auto-create": {"v_range": [["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "auth-request-signed": {"v_range": [["7.2.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "want-assertions-signed": {"v_range": [["7.2.3", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "system_saml_fabricidp": {
        "stated": true,
        "options": {
            "dev-id": {"v_range": [["6.4.0", ""]], "type": "str"},
            "idp-cert": {"v_range": [["6.4.0", ""]], "type": "str"},
            "idp-entity-id": {"v_range": [["6.4.0", ""]], "type": "str"},
            "idp-single-logout-url": {"v_range": [["6.4.0", ""]], "type": "str"},
            "idp-single-sign-on-url": {"v_range": [["6.4.0", ""]], "type": "str"},
            "idp-status": {"v_range": [["6.4.0", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "system_sdnconnector": {
        "stated": true,
        "options": {
            "_local_cert": {"type": "str"},
            "access-key": {"no_log": true, "type": "str"},
            "azure-region": {"choices": ["global", "china", "germany", "usgov", "local"], "type": "str"},
            "client-id": {"type": "str"},
            "client-secret": {"no_log": true, "type": "raw"},
            "compartment-id": {"type": "str"},
            "external-ip": {"type": "list", "options": {"name": {"type": "str"}}, "elements": "dict"},
            "gcp-project": {"type": "str"},
            "key-passwd": {"no_log": true, "type": "raw"},
            "login-endpoint": {"type": "str"},
            "name": {"required": true, "type": "str"},
            "nic": {
                "type": "list",
                "options": {
                    "ip": {
                        "type": "list",
                        "options": {"name": {"type": "str"}, "public-ip": {"type": "str"}, "resource-group": {"v_range": [["6.2.3", ""]], "type": "str"}},
                        "elements": "dict"
                    },
                    "name": {"type": "str"}
                },
                "elements": "dict"
            },
            "nsx-cert-fingerprint": {"type": "str"},
            "oci-cert": {"type": "str"},
            "oci-fingerprint": {"type": "str"},
            "oci-region": {"choices": ["phoenix", "ashburn", "frankfurt", "london", "toronto"], "type": "str"},
            "password": {"no_log": true, "type": "raw"},
            "private-key": {"no_log": true, "type": "str"},
            "region": {"type": "str"},
            "resource-group": {"type": "str"},
            "resource-url": {"type": "str"},
            "rest-interface": {"choices": ["mgmt", "sync"], "type": "str"},
            "rest-password": {"no_log": true, "type": "raw"},
            "rest-sport": {"type": "int"},
            "rest-ssl": {"choices": ["disable", "enable"], "type": "str"},
            "route": {"type": "list", "options": {"name": {"type": "str"}}, "elements": "dict"},
            "route-table": {
                "type": "list",
                "options": {
                    "name": {"type": "str"},
                    "route": {"type": "list", "options": {"name": {"type": "str"}, "next-hop": {"type": "str"}}, "elements": "dict"},
                    "resource-group": {"v_range": [["6.2.3", ""]], "type": "str"},
                    "subscription-id": {"v_range": [["6.2.6", "6.2.12"], ["6.4.1", ""]], "type": "str"}
                },
                "elements": "dict"
            },
            "secret-key": {"no_log": true, "type": "raw"},
            "server": {"type": "str"},
            "server-port": {"type": "int"},
            "service-account": {"type": "str"},
            "status": {"choices": ["disable", "enable"], "type": "str"},
            "subscription-id": {"type": "str"},
            "tenant-id": {"type": "str"},
            "type": {
                "choices": [
                    "aci", "aws", "nsx", "nuage", "azure", "gcp", "oci", "openstack", "kubernetes", "vmware", "acs", "alicloud", "sepm", "aci-direct",
                    "ibm", "nutanix", "sap"
                ],
                "type": "str"
            },
            "update-interval": {"type": "int"},
            "use-metadata-iam": {"choices": ["disable", "enable"], "type": "str"},
            "user-id": {"type": "str"},
            "username": {"type": "str"},
            "vmx-image-url": {"type": "str"},
            "vmx-service-name": {"type": "str"},
            "vpc-id": {"type": "str"},
            "domain": {"v_range": [["6.2.0", ""]], "type": "str"},
            "ha-status": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "last-update": {"v_range": [["6.2.1", "7.2.0"]], "type": "int"},
            "oci-region-type": {"v_range": [["6.2.1", ""]], "choices": ["commercial", "government"], "type": "str"},
            "secret-token": {"v_range": [["6.2.0", ""]], "no_log": true, "type": "str"},
            "updating": {"v_range": [["6.2.1", "7.2.0"]], "type": "int"},
            "server-ip": {"v_range": [["6.2.0", "6.4.14"]], "type": "str"},
            "group-name": {"v_range": [["6.2.2", ""]], "type": "str"},
            "api-key": {"v_range": [["6.4.1", ""]], "no_log": true, "type": "raw"},
            "compute-generation": {"v_range": [["6.4.1", ""]], "type": "int"},
            "ibm-region": {
                "v_range": [["6.4.2", ""]],
                "choices": [
                    "us-south", "us-east", "germany", "great-britain", "japan", "australia", "dallas", "washington-dc", "london", "frankfurt", "sydney",
                    "tokyo", "osaka", "toronto", "sao-paulo", "dallas-private", "washington-dc-private", "london-private", "frankfurt-private",
                    "sydney-private", "tokyo-private", "osaka-private", "toronto-private", "sao-paulo-private"
                ],
                "type": "str"
            },
            "ibm-region-gen1": {
                "v_range": [["6.4.1", ""]],
                "choices": ["us-south", "us-east", "germany", "great-britain", "japan", "australia"],
                "type": "str"
            },
            "ibm-region-gen2": {"v_range": [["6.4.1", ""]], "choices": ["us-south", "us-east", "great-britain"], "type": "str"},
            "vcenter-password": {"v_range": [["6.4.1", ""]], "no_log": true, "type": "raw"},
            "vcenter-server": {"v_range": [["6.4.1", ""]], "type": "str"},
            "vcenter-username": {"v_range": [["6.4.1", ""]], "type": "str"},
            "server-list": {"v_range": [["6.4.4", ""]], "type": "raw"},
            "external-account-list": {
                "v_range": [["7.0.3", ""]],
                "type": "list",
                "options": {
                    "region-list": {"v_range": [["7.0.3", ""]], "type": "raw"},
                    "role-arn": {"v_range": [["7.0.3", ""]], "type": "str"},
                    "external-id": {"v_range": [["7.0.5", "7.0.12"], ["7.2.1", ""]], "type": "str"}
                },
                "elements": "dict"
            },
            "forwarding-rule": {
                "v_range": [["7.0.2", ""]],
                "type": "list",
                "options": {"rule-name": {"v_range": [["7.0.2", ""]], "type": "str"}, "target": {"v_range": [["7.0.2", ""]], "type": "str"}},
                "elements": "dict"
            },
            "gcp-project-list": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.2", ""]],
                "type": "list",
                "options": {
                    "gcp-zone-list": {"v_range": [["6.4.7", "6.4.14"], ["7.0.2", ""]], "type": "raw"},
                    "id": {"v_range": [["6.4.7", "6.4.14"], ["7.0.2", ""]], "type": "str"}
                },
                "elements": "dict"
            },
            "verify-certificate": {"v_range": [["6.4.8", "6.4.14"], ["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "alt-resource-ip": {"v_range": [["7.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "server-ca-cert": {"v_range": [["7.2.2", ""]], "type": "str"},
            "server-cert": {"v_range": [["7.2.2", ""]], "type": "str"},
            "compartment-list": {
                "v_range": [["7.4.0", ""]],
                "type": "list",
                "options": {"compartment-id": {"v_range": [["7.4.0", ""]], "type": "str"}},
                "elements": "dict"
            },
            "oci-region-list": {
                "v_range": [["7.4.0", ""]],
                "type": "list",
                "options": {"region": {"v_range": [["7.4.0", ""]], "type": "str"}},
                "elements": "dict"
            },
            "proxy": {"v_range": [["7.4.0", ""]], "type": "str"}
        }
    },
    "system_sdnconnector_compartmentlist": {"stated": true, "options": {"compartment-id": {"v_range": [["7.4.0", ""]], "type": "str"}}},
    "system_sdnconnector_externalaccountlist": {
        "stated": true,
        "options": {
            "region-list": {"v_range": [["7.0.3", ""]], "type": "raw"},
            "role-arn": {"v_range": [["7.0.3", ""]], "type": "str"},
            "external-id": {"v_range": [["7.0.5", "7.0.12"], ["7.2.1", ""]], "type": "str"}
        }
    },
    "system_sdnconnector_externalip": {"stated": true, "options": {"name": {"required": true, "type": "str"}}},
    "system_sdnconnector_forwardingrule": {
        "stated": true,
        "options": {"rule-name": {"v_range": [["7.0.2", ""]], "type": "str"}, "target": {"v_range": [["7.0.2", ""]], "type": "str"}}
    },
    "system_sdnconnector_gcpprojectlist": {
        "stated": true,
        "options": {
            "gcp-zone-list": {"v_range": [["6.4.7", "6.4.14"], ["7.0.2", ""]], "type": "raw"},
            "id": {"v_range": [["6.4.7", "6.4.14"], ["7.0.2", ""]], "required": true, "type": "str"}
        }
    },
    "system_sdnconnector_nic": {
        "stated": true,
        "options": {
            "ip": {
                "type": "list",
                "options": {"name": {"type": "str"}, "public-ip": {"type": "str"}, "resource-group": {"v_range": [["6.2.3", ""]], "type": "str"}},
                "elements": "dict"
            },
            "name": {"required": true, "type": "str"}
        }
    },
    "system_sdnconnector_nic_ip": {
        "stated": true,
        "options": {
            "name": {"required": true, "type": "str"},
            "public-ip": {"type": "str"},
            "resource-group": {"v_range": [["6.2.3", ""]], "type": "str"}
        }
    },
    "system_sdnconnector_ociregionlist": {"stated": true, "options": {"region": {"v_range": [["7.4.0", ""]], "type": "str"}}},
    "system_sdnconnector_route": {"stated": true, "options": {"name": {"required": true, "type": "str"}}},
    "system_sdnconnector_routetable": {
        "stated": true,
        "options": {
            "name": {"required": true, "type": "str"},
            "route": {"type": "list", "options": {"name": {"type": "str"}, "next-hop": {"type": "str"}}, "elements": "dict"},
            "resource-group": {"v_range": [["6.2.3", ""]], "type": "str"},
            "subscription-id": {"v_range": [["6.2.6", "6.2.12"], ["6.4.1", ""]], "type": "str"}
        }
    },
    "system_sdnconnector_routetable_route": {"stated": true, "options": {"name": {"required": true, "type": "str"}, "next-hop": {"type": "str"}}},
    "system_sdnproxy": {
        "stated": true,
        "options": {
            "name": {"v_range": [["7.4.1", ""]], "required": true, "type": "str"},
            "password": {"v_range": [["7.4.1", ""]], "no_log": true, "type": "list", "elements": "str"},
            "server": {"v_range": [["7.4.1", ""]], "type": "str"},
            "server-port": {"v_range": [["7.4.1", ""]], "type": "int"},
            "type": {"v_range": [["7.4.1", ""]], "choices": ["general", "fortimanager"], "type": "str"},
            "username": {"v_range": [["7.4.1", ""]], "type": "str"}
        }
    },
    "system_smsserver": {"stated": true, "options": {"mail-server": {"type": "str"}, "name": {"required": true, "type": "str"}}},
    "system_sniffer": {
        "stated": true,
        "options": {
            "host": {"v_range": [["6.2.2", ""]], "type": "str"},
            "id": {"v_range": [["6.2.2", ""]], "required": true, "type": "int"},
            "interface": {"v_range": [["6.2.2", ""]], "type": "str"},
            "ipv6": {"v_range": [["6.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "max-packet-count": {"v_range": [["6.2.2", ""]], "type": "int"},
            "non-ip": {"v_range": [["6.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "port": {"v_range": [["6.2.2", ""]], "type": "str"},
            "protocol": {"v_range": [["6.2.2", ""]], "type": "str"},
            "vlan": {"v_range": [["6.2.2", ""]], "type": "str"}
        }
    },
    "system_snmp_community": {
        "stated": true,
        "options": {
            "events": {
                "type": "list",
                "choices": [
                    "disk_low", "ha_switch", "intf_ip_chg", "sys_reboot", "cpu_high", "mem_low", "log-alert", "log-rate", "log-data-rate", "lic-gbday",
                    "lic-dev-quota", "cpu-high-exclude-nice"
                ],
                "elements": "str"
            },
            "hosts": {"type": "list", "options": {"id": {"type": "int"}, "interface": {"type": "str"}, "ip": {"type": "str"}}, "elements": "dict"},
            "hosts6": {"type": "list", "options": {"id": {"type": "int"}, "interface": {"type": "str"}, "ip": {"type": "str"}}, "elements": "dict"},
            "id": {"required": true, "type": "int"},
            "name": {"type": "str"},
            "query_v1_port": {"type": "int"},
            "query_v1_status": {"choices": ["disable", "enable"], "type": "str"},
            "query_v2c_port": {"type": "int"},
            "query_v2c_status": {"choices": ["disable", "enable"], "type": "str"},
            "status": {"choices": ["disable", "enable"], "type": "str"},
            "trap_v1_rport": {"type": "int"},
            "trap_v1_status": {"choices": ["disable", "enable"], "type": "str"},
            "trap_v2c_rport": {"type": "int"},
            "trap_v2c_status": {"choices": ["disable", "enable"], "type": "str"}
        }
    },
    "system_snmp_community_hosts": {
        "stated": true,
        "options": {"id": {"required": true, "type": "int"}, "interface": {"type": "str"}, "ip": {"type": "str"}}
    },
    "system_snmp_community_hosts6": {
        "stated": true,
        "options": {"id": {"required": true, "type": "int"}, "interface": {"type": "str"}, "ip": {"type": "str"}}
    },
    "system_snmp_sysinfo": {
        "stated": false,
        "options": {
            "contact_info": {"type": "str"},
            "description": {"type": "str"},
            "engine-id": {"type": "str"},
            "location": {"type": "str"},
            "status": {"choices": ["disable", "enable"], "type": "str"},
            "trap-cpu-high-exclude-nice-threshold": {"type": "int"},
            "trap-high-cpu-threshold": {"type": "int"},
            "trap-low-memory-threshold": {"type": "int"}
        }
    },
    "system_snmp_user": {
        "stated": true,
        "options": {
            "auth-proto": {"choices": ["md5", "sha", "sha224", "sha256", "sha384", "sha512"], "type": "str"},
            "auth-pwd": {"type": "raw"},
            "events": {
                "type": "list",
                "choices": [
                    "disk_low", "ha_switch", "intf_ip_chg", "sys_reboot", "cpu_high", "mem_low", "log-alert", "log-rate", "log-data-rate", "lic-gbday",
                    "lic-dev-quota", "cpu-high-exclude-nice"
                ],
                "elements": "str"
            },
            "name": {"required": true, "type": "str"},
            "notify-hosts": {"type": "str"},
            "notify-hosts6": {"type": "str"},
            "priv-proto": {"choices": ["aes", "des", "aes256", "aes256cisco"], "type": "str"},
            "priv-pwd": {"type": "raw"},
            "queries": {"choices": ["disable", "enable"], "type": "str"},
            "query-port": {"type": "int"},
            "security-level": {"choices": ["no-auth-no-priv", "auth-no-priv", "auth-priv"], "type": "str"}
        }
    },
    "system_socfabric": {
        "stated": false,
        "options": {
            "name": {"v_range": [["7.0.0", ""]], "type": "str"},
            "port": {"v_range": [["7.0.0", ""]], "type": "int"},
            "psk": {"v_range": [["7.0.0", "7.2.5"]], "type": "raw"},
            "role": {"v_range": [["7.0.0", ""]], "choices": ["member", "supervisor"], "type": "str"},
            "secure-connection": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "status": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "supervisor": {"v_range": [["7.0.0", ""]], "type": "str"},
            "trusted-list": {
                "v_range": [["7.4.0", ""]],
                "type": "list",
                "options": {"id": {"v_range": [["7.4.0", ""]], "type": "int"}, "serial": {"v_range": [["7.4.0", ""]], "type": "str"}},
                "elements": "dict"
            }
        }
    },
    "system_socfabric_trustedlist": {
        "stated": true,
        "options": {"id": {"v_range": [["7.4.0", ""]], "required": true, "type": "int"}, "serial": {"v_range": [["7.4.0", ""]], "type": "str"}}
    },
    "system_sql": {
        "stated": false,
        "options": {
            "background-rebuild": {"choices": ["disable", "enable"], "type": "str"},
            "custom-index": {
                "type": "list",
                "options": {
                    "case-sensitive": {"choices": ["disable", "enable"], "type": "str"},
                    "device-type": {
                        "choices": [
                            "FortiGate", "FortiManager", "FortiClient", "FortiMail", "FortiWeb", "FortiCache", "FortiSandbox", "FortiDDoS",
                            "FortiAuthenticator", "FortiProxy"
                        ],
                        "type": "str"
                    },
                    "id": {"type": "int"},
                    "index-field": {"type": "str"},
                    "log-type": {
                        "choices": [
                            "none", "app-ctrl", "attack", "content", "dlp", "emailfilter", "event", "generic", "history", "traffic", "virus", "voip",
                            "webfilter", "netscan", "fct-event", "fct-traffic", "fct-netscan", "waf", "gtp", "dns", "ssh", "ssl", "file-filter", "asset",
                            "protocol", "siem", "ztna", "security"
                        ],
                        "type": "str"
                    }
                },
                "elements": "dict"
            },
            "database-name": {"type": "str"},
            "database-type": {"choices": ["mysql", "postgres"], "type": "str"},
            "device-count-high": {"choices": ["disable", "enable"], "type": "str"},
            "event-table-partition-time": {"type": "int"},
            "fct-table-partition-time": {"type": "int"},
            "logtype": {
                "type": "list",
                "choices": [
                    "none", "app-ctrl", "attack", "content", "dlp", "emailfilter", "event", "generic", "history", "traffic", "virus", "voip",
                    "webfilter", "netscan", "fct-event", "fct-traffic", "fct-netscan", "waf", "gtp", "dns", "ssh", "ssl", "file-filter", "asset",
                    "protocol", "siem", "ztna", "security"
                ],
                "elements": "str"
            },
            "password": {"no_log": true, "type": "raw"},
            "prompt-sql-upgrade": {"choices": ["disable", "enable"], "type": "str"},
            "rebuild-event": {"v_range": [["6.0.0", "7.4.0"]], "choices": ["disable", "enable"], "type": "str"},
            "rebuild-event-start-time": {"v_range": [["6.0.0", "7.4.0"]], "type": "raw"},
            "server": {"type": "str"},
            "start-time": {"type": "raw"},
            "status": {"choices": ["disable", "local"], "type": "str"},
            "text-search-index": {"choices": ["disable", "enable"], "type": "str"},
            "traffic-table-partition-time": {"type": "int"},
            "ts-index-field": {"type": "list", "options": {"category": {"type": "str"}, "value": {"type": "str"}}, "elements": "dict"},
            "username": {"type": "str"},
            "utm-table-partition-time": {"type": "int"},
            "custom-skipidx": {
                "v_range": [["6.2.3", ""]],
                "type": "list",
                "options": {
                    "device-type": {
                        "v_range": [["6.2.3", ""]],
                        "choices": ["FortiGate", "FortiManager", "FortiClient", "FortiMail", "FortiWeb", "FortiSandbox", "FortiProxy"],
                        "type": "str"
                    },
                    "id": {"v_range": [["6.2.3", ""]], "type": "int"},
                    "index-field": {"v_range": [["6.2.3", ""]], "type": "str"},
                    "log-type": {
                        "v_range": [["6.2.3", ""]],
                        "choices": [
                            "app-ctrl", "attack", "content", "dlp", "emailfilter", "event", "generic", "history", "traffic", "virus", "voip",
                            "webfilter", "netscan", "fct-event", "fct-traffic", "fct-netscan", "waf", "gtp", "dns", "ssh", "ssl", "file-filter", "asset",
                            "protocol", "siem", "ztna", "security"
                        ],
                        "type": "str"
                    }
                },
                "elements": "dict"
            },
            "compress-table-min-age": {"v_range": [["6.4.3", ""]], "type": "int"}
        }
    },
    "system_sql_customindex": {
        "stated": true,
        "options": {
            "case-sensitive": {"choices": ["disable", "enable"], "type": "str"},
            "device-type": {
                "choices": [
                    "FortiGate", "FortiManager", "FortiClient", "FortiMail", "FortiWeb", "FortiCache", "FortiSandbox", "FortiDDoS", "FortiAuthenticator",
                    "FortiProxy"
                ],
                "type": "str"
            },
            "id": {"required": true, "type": "int"},
            "index-field": {"type": "str"},
            "log-type": {
                "choices": [
                    "none", "app-ctrl", "attack", "content", "dlp", "emailfilter", "event", "generic", "history", "traffic", "virus", "voip",
                    "webfilter", "netscan", "fct-event", "fct-traffic", "fct-netscan", "waf", "gtp", "dns", "ssh", "ssl", "file-filter", "asset",
                    "protocol", "siem", "ztna", "security"
                ],
                "type": "str"
            }
        }
    },
    "system_sql_customskipidx": {
        "stated": true,
        "options": {
            "device-type": {
                "v_range": [["6.2.3", ""]],
                "choices": ["FortiGate", "FortiManager", "FortiClient", "FortiMail", "FortiWeb", "FortiSandbox", "FortiProxy"],
                "type": "str"
            },
            "id": {"v_range": [["6.2.3", ""]], "required": true, "type": "int"},
            "index-field": {"v_range": [["6.2.3", ""]], "type": "str"},
            "log-type": {
                "v_range": [["6.2.3", ""]],
                "choices": [
                    "app-ctrl", "attack", "content", "dlp", "emailfilter", "event", "generic", "history", "traffic", "virus", "voip", "webfilter",
                    "netscan", "fct-event", "fct-traffic", "fct-netscan", "waf", "gtp", "dns", "ssh", "ssl", "file-filter", "asset", "protocol", "siem",
                    "ztna", "security"
                ],
                "type": "str"
            }
        }
    },
    "system_sql_tsindexfield": {"stated": true, "options": {"category": {"type": "str"}, "value": {"type": "str"}}},
    "system_sslciphersuites": {
        "stated": true,
        "options": {
            "cipher": {"v_range": [["6.4.8", "6.4.14"], ["7.0.2", ""]], "type": "str"},
            "priority": {"v_range": [["6.4.8", "6.4.14"], ["7.0.2", ""]], "type": "int"},
            "version": {"v_range": [["6.4.8", "6.4.14"], ["7.0.2", ""]], "choices": ["tls1.2-or-below", "tls1.3"], "type": "str"}
        }
    },
    "system_syslog": {
        "stated": true,
        "options": {
            "ip": {"type": "str"},
            "name": {"required": true, "type": "str"},
            "port": {"type": "int"},
            "local-cert": {"v_range": [["6.4.8", "6.4.14"], ["7.0.4", ""]], "type": "str"},
            "peer-cert-cn": {"v_range": [["6.4.8", "6.4.14"], ["7.0.4", ""]], "type": "str"},
            "reliable": {"v_range": [["6.4.8", "6.4.14"], ["7.0.4", ""]], "choices": ["disable", "enable"], "type": "str"},
            "secure-connection": {"v_range": [["6.4.8", "6.4.14"], ["7.0.4", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "system_virtualwirepair": {
        "stated": true,
        "options": {
            "member": {"type": "list", "elements": "str"},
            "name": {"required": true, "type": "str"},
            "vlan-filter": {"type": "str"},
            "wildcard-vlan": {"choices": ["disable", "enable"], "type": "str"},
            "poweron-bypass": {"v_range": [["6.2.7", "6.2.12"], ["6.4.4", ""]], "choices": ["disable", "enable"], "type": "str"},
            "poweroff-bypass": {"v_range": [["6.2.7", "6.2.12"], ["6.4.4", ""]], "choices": ["disable", "enable"], "type": "str"},
            "outer-vlan-id": {"v_range": [["7.2.3", ""]], "type": "raw"}
        }
    },
    "system_webproxy": {
        "stated": false,
        "options": {
            "address": {"v_range": [["6.4.8", "6.4.14"], ["7.0.3", ""]], "type": "str"},
            "mode": {"v_range": [["6.4.8", "6.4.14"], ["7.0.3", ""]], "choices": ["proxy", "tunnel"], "type": "str"},
            "password": {"v_range": [["6.4.8", "6.4.14"], ["7.0.3", ""]], "no_log": true, "type": "raw"},
            "port": {"v_range": [["6.4.8", "6.4.14"], ["7.0.3", ""]], "type": "int"},
            "status": {"v_range": [["6.4.8", "6.4.14"], ["7.0.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "username": {"v_range": [["6.4.8", "6.4.14"], ["7.0.3", ""]], "type": "str"}
        }
    },
    "system_workflow_approvalmatrix": {
        "stated": true,
        "options": {
            "adom-name": {"required": true, "type": "str"},
            "approver": {"type": "list", "options": {"member": {"type": "str"}, "seq_num": {"type": "int"}}, "elements": "dict"},
            "mail-server": {"type": "str"},
            "notify": {"type": "str"}
        }
    },
    "system_workflow_approvalmatrix_approver": {"stated": true, "options": {"member": {"type": "str"}, "seq_num": {"required": true, "type": "int"}}},
    "template": {
        "stated": true,
        "options": {
            "description": {"type": "str"},
            "modification-time": {"v_range": [["6.0.0", "7.0.3"]], "type": "str"},
            "name": {"required": true, "type": "str"},
            "script": {"type": "str"},
            "provision": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "type": {"v_range": [["7.0.1", ""]], "choices": ["cli", "jinja"], "type": "str"},
            "variables": {"v_range": [["7.0.1", ""]], "type": "raw"},
            "position": {"v_range": [["7.4.2", ""]], "choices": ["post-vdom-copy", "prep-vdom-copy"], "type": "str"},
            "option": {"v_range": [["7.6.0", ""]], "type": "list", "choices": ["sdwan-overlay", "sdwan-manager"], "elements": "str"}
        }
    },
    "templategroup": {
        "stated": true,
        "options": {
            "description": {"type": "str"},
            "member": {"type": "list", "elements": "str"},
            "modification-time": {"v_range": [["6.0.0", "7.0.3"]], "type": "str"},
            "name": {"required": true, "type": "str"},
            "variables": {"v_range": [["7.2.3", ""]], "type": "raw"},
            "option": {"v_range": [["7.6.0", ""]], "type": "list", "choices": ["sdwan-overlay", "sdwan-manager"], "elements": "str"}
        }
    },
    "um_image_upgrade": {
        "stated": true,
        "options": {
            "adom": {"v_range": [["7.2.1", ""]], "type": "str"},
            "create_task": {"v_range": [["7.2.1", ""]], "type": "str"},
            "device": {
                "v_range": [["7.2.1", ""]],
                "type": "list",
                "options": {"name": {"v_range": [["7.2.1", ""]], "type": "str"}, "vdom": {"v_range": [["7.2.1", ""]], "type": "str"}},
                "elements": "dict"
            },
            "flags": {
                "v_range": [["7.2.1", ""]],
                "choices": ["f_boot_alt_partition", "f_skip_retrieve", "f_skip_multi_steps", "f_skip_fortiguard_img", "f_preview"],
                "type": "str"
            },
            "image": {
                "v_range": [["7.2.1", ""]],
                "type": "dict",
                "options": {
                    "build": {"v_range": [["7.2.1", ""]], "type": "str"},
                    "id": {"v_range": [["7.2.1", ""]], "type": "raw"},
                    "model": {"v_range": [["7.2.1", ""]], "type": "str"},
                    "release": {"v_range": [["7.2.1", ""]], "type": "str"}
                }
            },
            "schedule_time": {"v_range": [["7.2.1", ""]], "type": "str"}
        }
    },
    "um_image_upgrade_ext": {
        "stated": true,
        "options": {
            "create_task": {"v_range": [["7.2.1", ""]], "type": "str"},
            "device": {
                "v_range": [["7.2.1", ""]],
                "type": "list",
                "options": {"name": {"v_range": [["7.2.1", ""]], "type": "str"}, "vdom": {"v_range": [["7.2.1", ""]], "type": "str"}},
                "elements": "dict"
            },
            "flags": {
                "v_range": [["7.2.1", ""]],
                "choices": ["f_boot_alt_partition", "f_skip_retrieve", "f_skip_multi_steps", "f_skip_fortiguard_img", "f_preview"],
                "type": "str"
            },
            "image": {"v_range": [["7.2.1", ""]], "type": "str"},
            "schedule_time": {"v_range": [["7.2.1", ""]], "type": "str"}
        }
    },
    "user_adgrp": {
        "stated": true,
        "options": {
            "name": {"type": "str"},
            "server-name": {"type": "str"},
            "id": {"v_range": [["6.2.1", ""]], "required": true, "type": "int"},
            "polling-id": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"},
            "connector-source": {"v_range": [["6.4.0", ""]], "type": "str"}
        }
    },
    "user_certificate": {
        "stated": true,
        "options": {
            "common-name": {"v_range": [["7.0.8", "7.0.12"], ["7.2.3", ""]], "type": "str"},
            "id": {"v_range": [["7.0.8", "7.0.12"], ["7.2.3", ""]], "required": true, "type": "int"},
            "issuer": {"v_range": [["7.0.8", "7.0.12"], ["7.2.3", ""]], "type": "str"},
            "name": {"v_range": [["7.0.8", "7.0.12"], ["7.2.3", ""]], "type": "str"},
            "status": {"v_range": [["7.0.8", "7.0.12"], ["7.2.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "type": {"v_range": [["7.0.8", "7.0.12"], ["7.2.3", ""]], "choices": ["single-certificate", "trusted-issuer"], "type": "str"}
        }
    },
    "user_clearpass": {
        "stated": true,
        "options": {
            "client": {"v_range": [["6.2.1", ""]], "type": "str"},
            "name": {"v_range": [["6.2.1", ""]], "required": true, "type": "str"},
            "password": {"v_range": [["6.2.1", ""]], "no_log": true, "type": "raw"},
            "server": {"v_range": [["6.2.1", ""]], "type": "str"},
            "status": {"v_range": [["6.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "user": {"v_range": [["6.2.1", ""]], "type": "str"}
        }
    },
    "user_connector": {
        "stated": true,
        "options": {
            "name": {"v_range": [["7.0.1", ""]], "required": true, "type": "str"},
            "status": {"v_range": [["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "type": {"v_range": [["7.0.1", ""]], "type": "str"}
        }
    },
    "user_device": {
        "stated": true,
        "options": {
            "alias": {"v_range": [["6.0.0", "7.4.2"]], "required": true, "type": "str"},
            "avatar": {"v_range": [["6.0.0", "7.4.2"]], "type": "str"},
            "category": {
                "v_range": [["6.0.0", "7.4.2"]],
                "choices": ["none", "android-device", "blackberry-device", "fortinet-device", "ios-device", "windows-device", "amazon-device"],
                "type": "str"
            },
            "comment": {"v_range": [["6.0.0", "7.4.2"]], "type": "str"},
            "dynamic_mapping": {
                "v_range": [["6.0.0", "7.4.2"]],
                "type": "list",
                "options": {
                    "_scope": {
                        "v_range": [["6.0.0", "7.4.2"]],
                        "type": "list",
                        "options": {"name": {"v_range": [["6.0.0", "7.4.2"]], "type": "str"}, "vdom": {"v_range": [["6.0.0", "7.4.2"]], "type": "str"}},
                        "elements": "dict"
                    },
                    "avatar": {"v_range": [["6.0.0", "7.4.2"]], "type": "str"},
                    "category": {
                        "v_range": [["6.0.0", "7.4.2"]],
                        "choices": ["none", "android-device", "blackberry-device", "fortinet-device", "ios-device", "windows-device", "amazon-device"],
                        "type": "str"
                    },
                    "comment": {"v_range": [["6.0.0", "7.4.2"]], "type": "str"},
                    "mac": {"v_range": [["6.0.0", "7.4.2"]], "type": "str"},
                    "master-device": {"v_range": [["6.0.0", "7.4.2"]], "type": "str"},
                    "tags": {"v_range": [["6.0.0", "7.4.2"]], "type": "raw"},
                    "type": {
                        "v_range": [["6.0.0", "7.4.2"]],
                        "choices": [
                            "ipad", "iphone", "gaming-console", "blackberry-phone", "blackberry-playbook", "linux-pc", "mac", "windows-pc",
                            "android-phone", "android-tablet", "media-streaming", "windows-phone", "fortinet-device", "ip-phone", "router-nat-device",
                            "other-network-device", "windows-tablet", "printer", "forticam", "fortifone", "unknown"
                        ],
                        "type": "str"
                    },
                    "user": {"v_range": [["6.0.0", "7.4.2"]], "type": "str"},
                    "family": {"v_range": [["6.2.1", "7.4.2"]], "type": "str"},
                    "hardware-vendor": {"v_range": [["6.2.1", "7.4.2"]], "type": "str"},
                    "hardware-version": {"v_range": [["6.2.1", "7.4.2"]], "type": "str"},
                    "os": {"v_range": [["6.2.1", "7.4.2"]], "type": "str"},
                    "software-version": {"v_range": [["6.2.1", "7.4.2"]], "type": "str"}
                },
                "elements": "dict"
            },
            "mac": {"v_range": [["6.0.0", "7.4.2"]], "type": "str"},
            "master-device": {"v_range": [["6.0.0", "7.4.2"]], "type": "str"},
            "tagging": {
                "v_range": [["6.0.0", "7.4.2"]],
                "type": "list",
                "options": {
                    "category": {"v_range": [["6.0.0", "7.4.2"]], "type": "str"},
                    "name": {"v_range": [["6.0.0", "7.4.2"]], "type": "str"},
                    "tags": {"v_range": [["6.0.0", "7.4.2"]], "type": "raw"}
                },
                "elements": "dict"
            },
            "type": {
                "v_range": [["6.0.0", "7.4.2"]],
                "choices": [
                    "ipad", "iphone", "gaming-console", "blackberry-phone", "blackberry-playbook", "linux-pc", "mac", "windows-pc", "android-phone",
                    "android-tablet", "media-streaming", "windows-phone", "fortinet-device", "ip-phone", "router-nat-device", "other-network-device",
                    "windows-tablet", "printer", "forticam", "fortifone", "unknown"
                ],
                "type": "str"
            },
            "user": {"v_range": [["6.0.0", "7.4.2"]], "type": "str"},
            "tags": {"v_range": [["6.2.0", "6.4.14"]], "type": "str"}
        }
    },
    "user_device_dynamicmapping": {
        "stated": true,
        "options": {
            "_scope": {
                "v_range": [["6.0.0", "7.4.2"]],
                "type": "list",
                "options": {"name": {"v_range": [["6.0.0", "7.4.2"]], "type": "str"}, "vdom": {"v_range": [["6.0.0", "7.4.2"]], "type": "str"}},
                "elements": "dict"
            },
            "avatar": {"v_range": [["6.0.0", "7.4.2"]], "type": "str"},
            "category": {
                "v_range": [["6.0.0", "7.4.2"]],
                "choices": ["none", "android-device", "blackberry-device", "fortinet-device", "ios-device", "windows-device", "amazon-device"],
                "type": "str"
            },
            "comment": {"v_range": [["6.0.0", "7.4.2"]], "type": "str"},
            "mac": {"v_range": [["6.0.0", "7.4.2"]], "type": "str"},
            "master-device": {"v_range": [["6.0.0", "7.4.2"]], "type": "str"},
            "tags": {"v_range": [["6.0.0", "7.4.2"]], "type": "raw"},
            "type": {
                "v_range": [["6.0.0", "7.4.2"]],
                "choices": [
                    "ipad", "iphone", "gaming-console", "blackberry-phone", "blackberry-playbook", "linux-pc", "mac", "windows-pc", "android-phone",
                    "android-tablet", "media-streaming", "windows-phone", "fortinet-device", "ip-phone", "router-nat-device", "other-network-device",
                    "windows-tablet", "printer", "forticam", "fortifone", "unknown"
                ],
                "type": "str"
            },
            "user": {"v_range": [["6.0.0", "7.4.2"]], "type": "str"},
            "family": {"v_range": [["6.2.1", "7.4.2"]], "type": "str"},
            "hardware-vendor": {"v_range": [["6.2.1", "7.4.2"]], "type": "str"},
            "hardware-version": {"v_range": [["6.2.1", "7.4.2"]], "type": "str"},
            "os": {"v_range": [["6.2.1", "7.4.2"]], "type": "str"},
            "software-version": {"v_range": [["6.2.1", "7.4.2"]], "type": "str"}
        }
    },
    "user_device_tagging": {
        "stated": true,
        "options": {
            "category": {"v_range": [["6.0.0", "7.4.2"]], "type": "str"},
            "name": {"v_range": [["6.0.0", "7.4.2"]], "required": true, "type": "str"},
            "tags": {"v_range": [["6.0.0", "7.4.2"]], "type": "raw"}
        }
    },
    "user_deviceaccesslist": {
        "stated": true,
        "options": {
            "default-action": {"v_range": [["6.2.2", "7.2.1"]], "choices": ["deny", "accept"], "type": "str"},
            "device-list": {
                "v_range": [["6.2.2", "7.2.1"]],
                "type": "list",
                "options": {
                    "action": {"v_range": [["6.2.2", "7.2.1"]], "choices": ["deny", "accept"], "type": "str"},
                    "device": {"v_range": [["6.2.2", "7.2.1"]], "type": "str"},
                    "id": {"v_range": [["6.2.2", "7.2.1"]], "type": "int"}
                },
                "elements": "dict"
            },
            "name": {"v_range": [["6.2.2", "7.2.1"]], "required": true, "type": "str"}
        }
    },
    "user_deviceaccesslist_devicelist": {
        "stated": true,
        "options": {
            "action": {"v_range": [["6.2.2", "7.2.1"]], "choices": ["deny", "accept"], "type": "str"},
            "device": {"v_range": [["6.2.2", "7.2.1"]], "type": "str"},
            "id": {"v_range": [["6.2.2", "7.2.1"]], "required": true, "type": "int"}
        }
    },
    "user_devicecategory": {
        "stated": true,
        "options": {
            "comment": {"v_range": [["6.0.0", "7.2.1"]], "type": "str"},
            "desc": {"v_range": [["6.0.0", "7.2.1"]], "type": "str"},
            "name": {"v_range": [["6.0.0", "7.2.1"]], "required": true, "type": "str"}
        }
    },
    "user_devicegroup": {
        "stated": true,
        "options": {
            "_if_unmanaged": {"v_range": [["6.0.0", "7.2.1"]], "type": "int"},
            "comment": {"v_range": [["6.0.0", "7.2.1"]], "type": "str"},
            "dynamic_mapping": {
                "v_range": [["6.0.0", "7.2.1"]],
                "type": "list",
                "options": {
                    "_if_unmanaged": {"v_range": [["6.0.0", "7.2.1"]], "type": "int"},
                    "_scope": {
                        "v_range": [["6.0.0", "7.2.1"]],
                        "type": "list",
                        "options": {"name": {"v_range": [["6.0.0", "7.2.1"]], "type": "str"}, "vdom": {"v_range": [["6.0.0", "7.2.1"]], "type": "str"}},
                        "elements": "dict"
                    },
                    "comment": {"v_range": [["6.0.0", "7.2.1"]], "type": "str"},
                    "member": {"v_range": [["6.0.0", "7.2.1"]], "type": "list", "elements": "str"}
                },
                "elements": "dict"
            },
            "member": {"v_range": [["6.0.0", "7.2.1"]], "type": "list", "elements": "str"},
            "name": {"v_range": [["6.0.0", "7.2.1"]], "required": true, "type": "str"},
            "tagging": {
                "v_range": [["6.0.0", "7.2.1"]],
                "type": "list",
                "options": {
                    "category": {"v_range": [["6.0.0", "7.2.1"]], "type": "str"},
                    "name": {"v_range": [["6.0.0", "7.2.1"]], "type": "str"},
                    "tags": {"v_range": [["6.0.0", "7.2.1"]], "type": "raw"}
                },
                "elements": "dict"
            }
        }
    },
    "user_devicegroup_dynamicmapping": {
        "stated": true,
        "options": {
            "_if_unmanaged": {"v_range": [["6.0.0", "7.2.1"]], "type": "int"},
            "_scope": {
                "v_range": [["6.0.0", "7.2.1"]],
                "type": "list",
                "options": {"name": {"v_range": [["6.0.0", "7.2.1"]], "type": "str"}, "vdom": {"v_range": [["6.0.0", "7.2.1"]], "type": "str"}},
                "elements": "dict"
            },
            "comment": {"v_range": [["6.0.0", "7.2.1"]], "type": "str"},
            "member": {"v_range": [["6.0.0", "7.2.1"]], "type": "raw"}
        }
    },
    "user_devicegroup_tagging": {
        "stated": true,
        "options": {
            "category": {"v_range": [["6.0.0", "7.2.1"]], "type": "str"},
            "name": {"v_range": [["6.0.0", "7.2.1"]], "required": true, "type": "str"},
            "tags": {"v_range": [["6.0.0", "7.2.1"]], "type": "raw"}
        }
    },
    "user_domaincontroller": {
        "stated": true,
        "options": {
            "domain-name": {"v_range": [["6.2.1", ""]], "type": "str"},
            "extra-server": {
                "v_range": [["6.2.1", ""]],
                "type": "list",
                "options": {
                    "id": {"v_range": [["6.2.1", ""]], "type": "int"},
                    "ip-address": {"v_range": [["6.2.1", ""]], "type": "str"},
                    "port": {"v_range": [["6.2.1", ""]], "type": "int"},
                    "source-ip-address": {"v_range": [["7.0.0", ""]], "type": "str"},
                    "source-port": {"v_range": [["7.0.0", ""]], "type": "int"}
                },
                "elements": "dict"
            },
            "ip-address": {"v_range": [["6.2.1", ""]], "type": "str"},
            "ldap-server": {"v_range": [["6.2.1", ""]], "type": "raw"},
            "name": {"v_range": [["6.2.1", ""]], "required": true, "type": "str"},
            "port": {"v_range": [["6.2.1", ""]], "type": "int"},
            "ad-mode": {"v_range": [["7.0.0", ""]], "choices": ["none", "ds", "lds"], "type": "str"},
            "adlds-dn": {"v_range": [["7.0.0", ""]], "type": "str"},
            "adlds-ip-address": {"v_range": [["7.0.0", ""]], "type": "str"},
            "adlds-ip6": {"v_range": [["7.0.0", ""]], "type": "str"},
            "adlds-port": {"v_range": [["7.0.0", ""]], "type": "int"},
            "dns-srv-lookup": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "hostname": {"v_range": [["7.0.0", ""]], "type": "str"},
            "interface": {"v_range": [["7.0.0", ""]], "type": "str"},
            "interface-select-method": {"v_range": [["7.0.0", ""]], "choices": ["auto", "sdwan", "specify"], "type": "str"},
            "ip6": {"v_range": [["7.0.0", ""]], "type": "str"},
            "password": {"v_range": [["7.0.0", ""]], "no_log": true, "type": "raw"},
            "replication-port": {"v_range": [["7.0.0", ""]], "type": "int"},
            "source-ip-address": {"v_range": [["7.0.0", ""]], "type": "str"},
            "source-ip6": {"v_range": [["7.0.0", ""]], "type": "str"},
            "source-port": {"v_range": [["7.0.0", ""]], "type": "int"},
            "username": {"v_range": [["7.0.0", ""]], "type": "str"},
            "change-detection": {"v_range": [["7.2.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "change-detection-period": {"v_range": [["7.2.3", ""]], "type": "int"}
        }
    },
    "user_domaincontroller_extraserver": {
        "stated": true,
        "options": {
            "id": {"v_range": [["6.2.1", ""]], "required": true, "type": "int"},
            "ip-address": {"v_range": [["6.2.1", ""]], "type": "str"},
            "port": {"v_range": [["6.2.1", ""]], "type": "int"},
            "source-ip-address": {"v_range": [["7.0.0", ""]], "type": "str"},
            "source-port": {"v_range": [["7.0.0", ""]], "type": "int"}
        }
    },
    "user_exchange": {
        "stated": true,
        "options": {
            "addr-type": {"v_range": [["6.2.0", "7.2.0"]], "choices": ["ipv4", "ipv6"], "type": "str"},
            "auth-level": {
                "v_range": [["6.2.0", ""]],
                "choices": ["low", "medium", "normal", "high", "connect", "call", "packet", "integrity", "privacy"],
                "type": "str"
            },
            "auth-type": {"v_range": [["6.2.1", ""]], "choices": ["spnego", "ntlm", "kerberos"], "type": "str"},
            "connect-protocol": {"v_range": [["6.2.0", ""]], "choices": ["rpc-over-tcp", "rpc-over-http", "rpc-over-https"], "type": "str"},
            "domain-name": {"v_range": [["6.2.0", ""]], "type": "str"},
            "http-auth-type": {"v_range": [["6.2.1", ""]], "choices": ["ntlm", "basic"], "type": "str"},
            "ip": {"v_range": [["6.2.0", ""]], "type": "str"},
            "ip6": {"v_range": [["6.2.0", "7.2.0"]], "type": "str"},
            "kdc-ip": {"v_range": [["6.2.1", ""]], "type": "raw"},
            "name": {"v_range": [["6.2.0", ""]], "required": true, "type": "str"},
            "password": {"v_range": [["6.2.0", ""]], "no_log": true, "type": "raw"},
            "server-name": {"v_range": [["6.2.0", ""]], "type": "str"},
            "ssl-min-proto-version": {
                "v_range": [["6.2.1", ""]],
                "choices": ["default", "TLSv1-1", "TLSv1-2", "SSLv3", "TLSv1", "TLSv1-3"],
                "type": "str"
            },
            "username": {"v_range": [["6.2.0", ""]], "type": "str"},
            "auto-discover-kdc": {"v_range": [["6.4.0", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "user_externalidentityprovider": {
        "stated": true,
        "options": {
            "group-attr-name": {"v_range": [["7.4.3", ""]], "type": "str"},
            "interface": {"v_range": [["7.4.3", ""]], "type": "list", "elements": "str"},
            "interface-select-method": {"v_range": [["7.4.3", ""]], "choices": ["auto", "sdwan", "specify"], "type": "str"},
            "name": {"v_range": [["7.4.3", ""]], "required": true, "type": "str"},
            "port": {"v_range": [["7.4.3", ""]], "type": "int"},
            "server-identity-check": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "source-ip": {"v_range": [["7.4.3", ""]], "type": "str"},
            "timeout": {"v_range": [["7.4.3", ""]], "type": "int"},
            "type": {"v_range": [["7.4.3", ""]], "choices": ["ms-graph"], "type": "str"},
            "url": {"v_range": [["7.4.3", ""]], "type": "str"},
            "user-attr-name": {"v_range": [["7.4.3", ""]], "type": "str"},
            "version": {"v_range": [["7.4.3", ""]], "choices": ["beta", "v1.0"], "type": "str"}
        }
    },
    "user_flexvm": {
        "stated": true,
        "options": {
            "config": {"v_range": [["7.2.1", ""]], "type": "str"},
            "folder": {"v_range": [["7.2.1", ""]], "type": "str"},
            "name": {"v_range": [["7.2.1", ""]], "required": true, "type": "str"},
            "password": {"v_range": [["7.2.1", ""]], "no_log": true, "type": "str"},
            "program": {"v_range": [["7.2.1", ""]], "type": "str"},
            "status": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "user": {"v_range": [["7.2.1", ""]], "type": "str"}
        }
    },
    "user_fortitoken": {
        "stated": true,
        "options": {
            "comments": {"type": "str"},
            "license": {"type": "str"},
            "os-ver": {"v_range": [["6.0.0", "6.2.5"], ["6.4.0", "6.4.1"]], "type": "str"},
            "reg-id": {"v_range": [["6.0.0", "6.2.5"], ["6.4.0", "6.4.1"]], "type": "str"},
            "serial-number": {"required": true, "type": "str"},
            "status": {"choices": ["lock", "active"], "type": "str"}
        }
    },
    "user_fsso": {
        "stated": true,
        "options": {
            "_gui_meta": {"type": "str"},
            "dynamic_mapping": {
                "type": "list",
                "options": {
                    "_gui_meta": {"type": "str"},
                    "_scope": {"type": "list", "options": {"name": {"type": "str"}, "vdom": {"type": "str"}}, "elements": "dict"},
                    "ldap-server": {"type": "str"},
                    "password": {"no_log": true, "type": "raw"},
                    "password2": {"no_log": true, "type": "raw"},
                    "password3": {"no_log": true, "type": "raw"},
                    "password4": {"no_log": true, "type": "raw"},
                    "password5": {"no_log": true, "type": "raw"},
                    "port": {"type": "int"},
                    "port2": {"type": "int"},
                    "port3": {"type": "int"},
                    "port4": {"type": "int"},
                    "port5": {"type": "int"},
                    "server": {"type": "str"},
                    "server2": {"type": "str"},
                    "server3": {"type": "str"},
                    "server4": {"type": "str"},
                    "server5": {"type": "str"},
                    "source-ip": {"type": "str"},
                    "source-ip6": {"type": "str"},
                    "ssl": {"choices": ["disable", "enable"], "type": "str"},
                    "ssl-trusted-cert": {"type": "str"},
                    "type": {"choices": ["default", "fortiems", "fortinac", "fortiems-cloud"], "type": "str"},
                    "user-info-server": {"type": "raw"},
                    "ldap-poll": {"v_range": [["6.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "ldap-poll-filter": {"v_range": [["6.2.1", ""]], "type": "str"},
                    "ldap-poll-interval": {"v_range": [["6.2.1", ""]], "type": "int"},
                    "group-poll-interval": {"v_range": [["6.2.2", ""]], "type": "int"},
                    "interface": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "type": "str"},
                    "interface-select-method": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "choices": ["auto", "sdwan", "specify"], "type": "str"},
                    "logon-timeout": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"},
                    "sni": {"v_range": [["7.2.0", ""]], "type": "str"},
                    "ssl-server-host-ip-check": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"}
                },
                "elements": "dict"
            },
            "ldap-server": {"type": "str"},
            "name": {"required": true, "type": "str"},
            "password": {"no_log": true, "type": "raw"},
            "password2": {"no_log": true, "type": "raw"},
            "password3": {"no_log": true, "type": "raw"},
            "password4": {"no_log": true, "type": "raw"},
            "password5": {"no_log": true, "type": "raw"},
            "port": {"type": "int"},
            "port2": {"type": "int"},
            "port3": {"type": "int"},
            "port4": {"type": "int"},
            "port5": {"type": "int"},
            "server": {"type": "str"},
            "server2": {"type": "str"},
            "server3": {"type": "str"},
            "server4": {"type": "str"},
            "server5": {"type": "str"},
            "source-ip": {"type": "str"},
            "source-ip6": {"type": "str"},
            "ldap-poll": {"v_range": [["6.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ldap-poll-filter": {"v_range": [["6.2.1", ""]], "type": "str"},
            "ldap-poll-interval": {"v_range": [["6.2.1", ""]], "type": "int"},
            "ssl": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ssl-trusted-cert": {"v_range": [["6.2.0", ""]], "type": "str"},
            "type": {"v_range": [["6.2.0", ""]], "choices": ["default", "fortiems", "fortinac", "fortiems-cloud"], "type": "str"},
            "user-info-server": {"v_range": [["6.2.0", ""]], "type": "str"},
            "group-poll-interval": {"v_range": [["6.2.2", ""]], "type": "int"},
            "interface": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "type": "str"},
            "interface-select-method": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "choices": ["auto", "sdwan", "specify"], "type": "str"},
            "logon-timeout": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"},
            "sni": {"v_range": [["7.2.0", ""]], "type": "str"},
            "ssl-server-host-ip-check": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "user_fsso_dynamicmapping": {
        "stated": true,
        "options": {
            "_gui_meta": {"type": "str"},
            "_scope": {"type": "list", "options": {"name": {"type": "str"}, "vdom": {"type": "str"}}, "elements": "dict"},
            "ldap-server": {"type": "str"},
            "password": {"no_log": true, "type": "raw"},
            "password2": {"no_log": true, "type": "raw"},
            "password3": {"no_log": true, "type": "raw"},
            "password4": {"no_log": true, "type": "raw"},
            "password5": {"no_log": true, "type": "raw"},
            "port": {"type": "int"},
            "port2": {"type": "int"},
            "port3": {"type": "int"},
            "port4": {"type": "int"},
            "port5": {"type": "int"},
            "server": {"type": "str"},
            "server2": {"type": "str"},
            "server3": {"type": "str"},
            "server4": {"type": "str"},
            "server5": {"type": "str"},
            "source-ip": {"type": "str"},
            "source-ip6": {"type": "str"},
            "ssl": {"choices": ["disable", "enable"], "type": "str"},
            "ssl-trusted-cert": {"type": "str"},
            "type": {"choices": ["default", "fortiems", "fortinac", "fortiems-cloud"], "type": "str"},
            "user-info-server": {"type": "raw"},
            "ldap-poll": {"v_range": [["6.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ldap-poll-filter": {"v_range": [["6.2.1", ""]], "type": "str"},
            "ldap-poll-interval": {"v_range": [["6.2.1", ""]], "type": "int"},
            "group-poll-interval": {"v_range": [["6.2.2", ""]], "type": "int"},
            "interface": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "type": "str"},
            "interface-select-method": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "choices": ["auto", "sdwan", "specify"], "type": "str"},
            "logon-timeout": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "int"},
            "sni": {"v_range": [["7.2.0", ""]], "type": "str"},
            "ssl-server-host-ip-check": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "user_fssopolling": {
        "stated": true,
        "options": {
            "_gui_meta": {"type": "str"},
            "adgrp": {"type": "list", "options": {"name": {"type": "str"}}, "elements": "dict"},
            "default-domain": {"type": "str"},
            "id": {"required": true, "type": "int"},
            "ldap-server": {"type": "str"},
            "logon-history": {"type": "int"},
            "password": {"no_log": true, "type": "raw"},
            "polling-frequency": {"type": "int"},
            "port": {"type": "int"},
            "server": {"type": "str"},
            "status": {"choices": ["disable", "enable"], "type": "str"},
            "user": {"type": "str"},
            "smb-ntlmv1-auth": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "smbv1": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "user_fssopolling_adgrp": {"stated": true, "options": {"name": {"required": true, "type": "str"}}},
    "user_group": {
        "stated": true,
        "options": {
            "auth-concurrent-override": {"choices": ["disable", "enable"], "type": "str"},
            "auth-concurrent-value": {"type": "int"},
            "authtimeout": {"type": "int"},
            "company": {"choices": ["optional", "mandatory", "disabled"], "type": "str"},
            "email": {"choices": ["disable", "enable"], "type": "str"},
            "expire": {"type": "int"},
            "expire-type": {"choices": ["immediately", "first-successful-login"], "type": "str"},
            "group-type": {"choices": ["firewall", "directory-service", "fsso-service", "guest", "rsso"], "type": "str"},
            "guest": {
                "type": "list",
                "options": {
                    "comment": {"type": "str"},
                    "company": {"type": "str"},
                    "email": {"type": "str"},
                    "expiration": {"type": "str"},
                    "mobile-phone": {"type": "str"},
                    "name": {"type": "str"},
                    "password": {"no_log": true, "type": "raw"},
                    "sponsor": {"type": "str"},
                    "user-id": {"type": "str"},
                    "group": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"},
                    "id": {"v_range": [["6.2.3", ""]], "type": "int"}
                },
                "elements": "dict"
            },
            "http-digest-realm": {"type": "str"},
            "id": {"type": "int"},
            "match": {
                "type": "list",
                "options": {"_gui_meta": {"type": "str"}, "group-name": {"type": "str"}, "id": {"type": "int"}, "server-name": {"type": "str"}},
                "elements": "dict"
            },
            "max-accounts": {"type": "int"},
            "member": {"type": "list", "elements": "str"},
            "mobile-phone": {"choices": ["disable", "enable"], "type": "str"},
            "multiple-guest-add": {"choices": ["disable", "enable"], "type": "str"},
            "name": {"required": true, "type": "str"},
            "password": {"choices": ["auto-generate", "specify", "disable"], "no_log": true, "type": "str"},
            "sms-custom-server": {"type": "str"},
            "sms-server": {"choices": ["fortiguard", "custom"], "type": "str"},
            "sponsor": {"choices": ["optional", "mandatory", "disabled"], "type": "str"},
            "sso-attribute-value": {"type": "str"},
            "user-id": {"choices": ["email", "auto-generate", "specify"], "type": "str"},
            "user-name": {"choices": ["disable", "enable"], "type": "str"},
            "dynamic_mapping": {
                "v_range": [["7.0.2", ""]],
                "type": "list",
                "options": {
                    "_scope": {
                        "v_range": [["7.0.2", ""]],
                        "type": "list",
                        "options": {"name": {"v_range": [["7.0.2", ""]], "type": "str"}, "vdom": {"v_range": [["7.0.2", ""]], "type": "str"}},
                        "elements": "dict"
                    },
                    "auth-concurrent-override": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "auth-concurrent-value": {"v_range": [["7.0.2", ""]], "type": "int"},
                    "authtimeout": {"v_range": [["7.0.2", ""]], "type": "int"},
                    "company": {"v_range": [["7.0.2", ""]], "choices": ["optional", "mandatory", "disabled"], "type": "str"},
                    "email": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "expire": {"v_range": [["7.0.2", ""]], "type": "int"},
                    "expire-type": {"v_range": [["7.0.2", ""]], "choices": ["immediately", "first-successful-login"], "type": "str"},
                    "group-type": {
                        "v_range": [["7.0.2", ""]],
                        "choices": ["firewall", "directory-service", "fsso-service", "guest", "rsso"],
                        "type": "str"
                    },
                    "guest": {
                        "v_range": [["7.0.2", ""]],
                        "type": "list",
                        "options": {
                            "comment": {"v_range": [["7.0.2", ""]], "type": "str"},
                            "company": {"v_range": [["7.0.2", ""]], "type": "str"},
                            "email": {"v_range": [["7.0.2", ""]], "type": "str"},
                            "expiration": {"v_range": [["7.0.2", ""]], "type": "str"},
                            "group": {"v_range": [["7.0.2", ""]], "type": "str"},
                            "id": {"v_range": [["7.0.2", ""]], "type": "int"},
                            "mobile-phone": {"v_range": [["7.0.2", ""]], "type": "str"},
                            "name": {"v_range": [["7.0.2", ""]], "type": "str"},
                            "password": {"v_range": [["7.0.2", ""]], "no_log": true, "type": "raw"},
                            "sponsor": {"v_range": [["7.0.2", ""]], "type": "str"},
                            "user-id": {"v_range": [["7.0.2", ""]], "type": "str"}
                        },
                        "elements": "dict"
                    },
                    "http-digest-realm": {"v_range": [["7.0.2", ""]], "type": "str"},
                    "id": {"v_range": [["7.0.2", ""]], "type": "int"},
                    "ldap-memberof": {"v_range": [["7.0.2", ""]], "type": "str"},
                    "logic-type": {"v_range": [["7.0.3", ""]], "choices": ["or", "and"], "type": "str"},
                    "match": {
                        "v_range": [["7.0.2", ""]],
                        "type": "list",
                        "options": {
                            "_gui_meta": {"v_range": [["7.0.2", ""]], "type": "str"},
                            "group-name": {"v_range": [["7.0.2", ""]], "type": "str"},
                            "id": {"v_range": [["7.0.2", ""]], "type": "int"},
                            "server-name": {"v_range": [["7.0.2", ""]], "type": "str"}
                        },
                        "elements": "dict"
                    },
                    "max-accounts": {"v_range": [["7.0.2", ""]], "type": "int"},
                    "member": {"v_range": [["7.0.2", ""]], "type": "raw"},
                    "mobile-phone": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "multiple-guest-add": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "password": {"v_range": [["7.0.2", ""]], "choices": ["auto-generate", "specify", "disable"], "no_log": true, "type": "str"},
                    "redir-url": {"v_range": [["7.0.2", ""]], "type": "str"},
                    "sms-custom-server": {"v_range": [["7.0.2", ""]], "type": "str"},
                    "sms-server": {"v_range": [["7.0.2", ""]], "choices": ["fortiguard", "custom"], "type": "str"},
                    "sponsor": {"v_range": [["7.0.2", ""]], "choices": ["optional", "mandatory", "disabled"], "type": "str"},
                    "sslvpn-bookmarks-group": {"v_range": [["7.0.2", ""]], "type": "raw"},
                    "sslvpn-cache-cleaner": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "sslvpn-client-check": {
                        "v_range": [["7.0.2", ""]],
                        "type": "list",
                        "choices": ["forticlient", "forticlient-av", "forticlient-fw", "3rdAV", "3rdFW"],
                        "elements": "str"
                    },
                    "sslvpn-ftp": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "sslvpn-http": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "sslvpn-os-check": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "sslvpn-os-check-list": {
                        "v_range": [["7.0.2", ""]],
                        "type": "dict",
                        "options": {
                            "action": {"v_range": [["7.0.2", ""]], "choices": ["allow", "check-up-to-date", "deny"], "type": "str"},
                            "latest-patch-level": {"v_range": [["7.0.2", ""]], "type": "str"},
                            "name": {"v_range": [["7.0.2", ""]], "type": "str"},
                            "tolerance": {"v_range": [["7.0.2", ""]], "type": "int"}
                        }
                    },
                    "sslvpn-portal": {"v_range": [["7.0.2", ""]], "type": "raw"},
                    "sslvpn-portal-heading": {"v_range": [["7.0.2", ""]], "type": "str"},
                    "sslvpn-rdp": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "sslvpn-samba": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "sslvpn-split-tunneling": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "sslvpn-ssh": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "sslvpn-telnet": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "sslvpn-tunnel": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "sslvpn-tunnel-endip": {"v_range": [["7.0.2", ""]], "type": "str"},
                    "sslvpn-tunnel-ip-mode": {"v_range": [["7.0.2", ""]], "choices": ["range", "usrgrp"], "type": "str"},
                    "sslvpn-tunnel-startip": {"v_range": [["7.0.2", ""]], "type": "str"},
                    "sslvpn-virtual-desktop": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "sslvpn-vnc": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "sslvpn-webapp": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "sso-attribute-value": {"v_range": [["7.0.2", ""]], "type": "str"},
                    "user-id": {"v_range": [["7.0.2", ""]], "choices": ["email", "auto-generate", "specify"], "type": "str"},
                    "user-name": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"}
                },
                "elements": "dict"
            }
        }
    },
    "user_group_dynamicmapping": {
        "stated": true,
        "options": {
            "_scope": {
                "v_range": [["7.0.2", ""]],
                "type": "list",
                "options": {"name": {"v_range": [["7.0.2", ""]], "type": "str"}, "vdom": {"v_range": [["7.0.2", ""]], "type": "str"}},
                "elements": "dict"
            },
            "auth-concurrent-override": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "auth-concurrent-value": {"v_range": [["7.0.2", ""]], "type": "int"},
            "authtimeout": {"v_range": [["7.0.2", ""]], "type": "int"},
            "company": {"v_range": [["7.0.2", ""]], "choices": ["optional", "mandatory", "disabled"], "type": "str"},
            "email": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "expire": {"v_range": [["7.0.2", ""]], "type": "int"},
            "expire-type": {"v_range": [["7.0.2", ""]], "choices": ["immediately", "first-successful-login"], "type": "str"},
            "group-type": {"v_range": [["7.0.2", ""]], "choices": ["firewall", "directory-service", "fsso-service", "guest", "rsso"], "type": "str"},
            "guest": {
                "v_range": [["7.0.2", ""]],
                "type": "list",
                "options": {
                    "comment": {"v_range": [["7.0.2", ""]], "type": "str"},
                    "company": {"v_range": [["7.0.2", ""]], "type": "str"},
                    "email": {"v_range": [["7.0.2", ""]], "type": "str"},
                    "expiration": {"v_range": [["7.0.2", ""]], "type": "str"},
                    "group": {"v_range": [["7.0.2", ""]], "type": "str"},
                    "id": {"v_range": [["7.0.2", ""]], "type": "int"},
                    "mobile-phone": {"v_range": [["7.0.2", ""]], "type": "str"},
                    "name": {"v_range": [["7.0.2", ""]], "type": "str"},
                    "password": {"v_range": [["7.0.2", ""]], "no_log": true, "type": "raw"},
                    "sponsor": {"v_range": [["7.0.2", ""]], "type": "str"},
                    "user-id": {"v_range": [["7.0.2", ""]], "type": "str"}
                },
                "elements": "dict"
            },
            "http-digest-realm": {"v_range": [["7.0.2", ""]], "type": "str"},
            "id": {"v_range": [["7.0.2", ""]], "required": true, "type": "int"},
            "ldap-memberof": {"v_range": [["7.0.2", ""]], "type": "str"},
            "logic-type": {"v_range": [["7.0.3", ""]], "choices": ["or", "and"], "type": "str"},
            "match": {
                "v_range": [["7.0.2", ""]],
                "type": "list",
                "options": {
                    "_gui_meta": {"v_range": [["7.0.2", ""]], "type": "str"},
                    "group-name": {"v_range": [["7.0.2", ""]], "type": "str"},
                    "id": {"v_range": [["7.0.2", ""]], "type": "int"},
                    "server-name": {"v_range": [["7.0.2", ""]], "type": "str"}
                },
                "elements": "dict"
            },
            "max-accounts": {"v_range": [["7.0.2", ""]], "type": "int"},
            "member": {"v_range": [["7.0.2", ""]], "type": "raw"},
            "mobile-phone": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "multiple-guest-add": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "password": {"v_range": [["7.0.2", ""]], "choices": ["auto-generate", "specify", "disable"], "no_log": true, "type": "str"},
            "redir-url": {"v_range": [["7.0.2", ""]], "type": "str"},
            "sms-custom-server": {"v_range": [["7.0.2", ""]], "type": "str"},
            "sms-server": {"v_range": [["7.0.2", ""]], "choices": ["fortiguard", "custom"], "type": "str"},
            "sponsor": {"v_range": [["7.0.2", ""]], "choices": ["optional", "mandatory", "disabled"], "type": "str"},
            "sslvpn-bookmarks-group": {"v_range": [["7.0.2", ""]], "type": "raw"},
            "sslvpn-cache-cleaner": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "sslvpn-client-check": {
                "v_range": [["7.0.2", ""]],
                "type": "list",
                "choices": ["forticlient", "forticlient-av", "forticlient-fw", "3rdAV", "3rdFW"],
                "elements": "str"
            },
            "sslvpn-ftp": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "sslvpn-http": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "sslvpn-os-check": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "sslvpn-os-check-list": {
                "v_range": [["7.0.2", ""]],
                "type": "dict",
                "options": {
                    "action": {"v_range": [["7.0.2", ""]], "choices": ["allow", "check-up-to-date", "deny"], "type": "str"},
                    "latest-patch-level": {"v_range": [["7.0.2", ""]], "type": "str"},
                    "name": {"v_range": [["7.0.2", ""]], "type": "str"},
                    "tolerance": {"v_range": [["7.0.2", ""]], "type": "int"}
                }
            },
            "sslvpn-portal": {"v_range": [["7.0.2", ""]], "type": "raw"},
            "sslvpn-portal-heading": {"v_range": [["7.0.2", ""]], "type": "str"},
            "sslvpn-rdp": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "sslvpn-samba": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "sslvpn-split-tunneling": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "sslvpn-ssh": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "sslvpn-telnet": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "sslvpn-tunnel": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "sslvpn-tunnel-endip": {"v_range": [["7.0.2", ""]], "type": "str"},
            "sslvpn-tunnel-ip-mode": {"v_range": [["7.0.2", ""]], "choices": ["range", "usrgrp"], "type": "str"},
            "sslvpn-tunnel-startip": {"v_range": [["7.0.2", ""]], "type": "str"},
            "sslvpn-virtual-desktop": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "sslvpn-vnc": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "sslvpn-webapp": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "sso-attribute-value": {"v_range": [["7.0.2", ""]], "type": "str"},
            "user-id": {"v_range": [["7.0.2", ""]], "choices": ["email", "auto-generate", "specify"], "type": "str"},
            "user-name": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "user_group_dynamicmapping_guest": {
        "stated": true,
        "options": {
            "comment": {"v_range": [["7.0.2", "7.4.0"]], "type": "str"},
            "company": {"v_range": [["7.0.2", "7.4.0"]], "type": "str"},
            "email": {"v_range": [["7.0.2", "7.4.0"]], "type": "str"},
            "expiration": {"v_range": [["7.0.2", "7.4.0"]], "type": "str"},
            "group": {"v_range": [["7.0.2", "7.4.0"]], "type": "str"},
            "id": {"v_range": [["7.0.2", "7.4.0"]], "required": true, "type": "int"},
            "mobile-phone": {"v_range": [["7.0.2", "7.4.0"]], "type": "str"},
            "name": {"v_range": [["7.0.2", "7.4.0"]], "type": "str"},
            "password": {"v_range": [["7.0.2", "7.4.0"]], "no_log": true, "type": "raw"},
            "sponsor": {"v_range": [["7.0.2", "7.4.0"]], "type": "str"},
            "user-id": {"v_range": [["7.0.2", "7.4.0"]], "type": "str"}
        }
    },
    "user_group_dynamicmapping_match": {
        "stated": true,
        "options": {
            "_gui_meta": {"v_range": [["7.0.2", "7.4.0"]], "type": "str"},
            "group-name": {"v_range": [["7.0.2", "7.4.0"]], "type": "str"},
            "id": {"v_range": [["7.0.2", "7.4.0"]], "required": true, "type": "int"},
            "server-name": {"v_range": [["7.0.2", "7.4.0"]], "type": "str"}
        }
    },
    "user_group_dynamicmapping_sslvpnoschecklist": {
        "stated": false,
        "options": {
            "action": {"v_range": [["7.0.2", "7.4.0"]], "choices": ["allow", "check-up-to-date", "deny"], "type": "str"},
            "latest-patch-level": {"v_range": [["7.0.2", "7.4.0"]], "type": "str"},
            "name": {"v_range": [["7.0.2", "7.4.0"]], "type": "str"},
            "tolerance": {"v_range": [["7.0.2", "7.4.0"]], "type": "int"}
        }
    },
    "user_group_guest": {
        "stated": true,
        "options": {
            "comment": {"type": "str"},
            "company": {"type": "str"},
            "email": {"type": "str"},
            "expiration": {"type": "str"},
            "mobile-phone": {"type": "str"},
            "name": {"type": "str"},
            "password": {"no_log": true, "type": "raw"},
            "sponsor": {"type": "str"},
            "user-id": {"required": true, "type": "str"},
            "group": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"},
            "id": {"v_range": [["6.2.3", ""]], "type": "int"}
        }
    },
    "user_group_match": {
        "stated": true,
        "options": {"_gui_meta": {"type": "str"}, "group-name": {"type": "str"}, "id": {"required": true, "type": "int"}, "server-name": {"type": "str"}}
    },
    "user_json": {
        "stated": true,
        "options": {
            "name": {"v_range": [["7.2.1", ""]], "required": true, "type": "str"},
            "status": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "user_krbkeytab": {
        "stated": true,
        "options": {
            "keytab": {"v_range": [["6.2.1", ""]], "no_log": true, "type": "str"},
            "ldap-server": {"v_range": [["6.2.1", ""]], "type": "raw"},
            "name": {"v_range": [["6.2.1", ""]], "required": true, "type": "str"},
            "pac-data": {"v_range": [["6.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "principal": {"v_range": [["6.2.1", ""]], "type": "str"},
            "password": {"v_range": [["6.2.2", "7.2.0"], ["7.2.5", "7.2.5"], ["7.4.2", ""]], "no_log": true, "type": "raw"}
        }
    },
    "user_ldap": {
        "stated": true,
        "options": {
            "account-key-filter": {"no_log": true, "type": "str"},
            "account-key-processing": {"choices": ["same", "strip"], "type": "str"},
            "ca-cert": {"type": "str"},
            "cnid": {"type": "str"},
            "dn": {"type": "str"},
            "dynamic_mapping": {
                "type": "list",
                "options": {
                    "_scope": {"type": "list", "options": {"name": {"type": "str"}, "vdom": {"type": "str"}}, "elements": "dict"},
                    "account-key-filter": {"no_log": true, "type": "str"},
                    "account-key-name": {"no_log": true, "type": "str"},
                    "account-key-processing": {"choices": ["same", "strip"], "type": "str"},
                    "ca-cert": {"type": "str"},
                    "cnid": {"type": "str"},
                    "dn": {"type": "str"},
                    "filter": {"type": "str"},
                    "group": {"type": "str"},
                    "group-filter": {"type": "str"},
                    "group-member-check": {"choices": ["user-attr", "group-object", "posix-group-object"], "type": "str"},
                    "group-object-filter": {"type": "str"},
                    "group-object-search-base": {"type": "str"},
                    "group-search-base": {"type": "str"},
                    "member-attr": {"type": "str"},
                    "obtain-user-info": {"choices": ["disable", "enable"], "type": "str"},
                    "password": {"no_log": true, "type": "raw"},
                    "password-expiry-warning": {"choices": ["disable", "enable"], "type": "str"},
                    "password-renewal": {"choices": ["disable", "enable"], "type": "str"},
                    "port": {"type": "int"},
                    "retrieve-protection-profile": {"type": "str"},
                    "search-type": {"type": "list", "choices": ["nested", "recursive"], "elements": "str"},
                    "secondary-server": {"type": "str"},
                    "secure": {"choices": ["disable", "starttls", "ldaps"], "type": "str"},
                    "server": {"type": "str"},
                    "server-identity-check": {"choices": ["disable", "enable"], "type": "str"},
                    "source-ip": {"type": "str"},
                    "ssl-min-proto-version": {"choices": ["default", "TLSv1", "TLSv1-1", "TLSv1-2", "SSLv3", "TLSv1-3"], "type": "str"},
                    "tertiary-server": {"type": "str"},
                    "type": {"choices": ["simple", "anonymous", "regular"], "type": "str"},
                    "user-info-exchange-server": {"type": "str"},
                    "username": {"type": "str"},
                    "two-factor": {"v_range": [["6.2.2", ""]], "choices": ["disable", "fortitoken-cloud"], "type": "str"},
                    "interface": {"v_range": [["6.2.5", "6.2.12"], ["6.4.1", ""]], "type": "str"},
                    "interface-select-method": {"v_range": [["6.2.5", "6.2.12"], ["6.4.1", ""]], "choices": ["auto", "sdwan", "specify"], "type": "str"},
                    "two-factor-authentication": {"v_range": [["6.2.5", ""]], "choices": ["fortitoken", "email", "sms"], "type": "str"},
                    "two-factor-notification": {"v_range": [["6.2.5", ""]], "choices": ["email", "sms"], "type": "str"},
                    "antiphish": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "password-attr": {"v_range": [["7.0.0", ""]], "no_log": true, "type": "str"},
                    "source-port": {"v_range": [["7.0.0", ""]], "type": "int"},
                    "client-cert": {"v_range": [["7.2.0", ""]], "type": "str"},
                    "client-cert-auth": {"v_range": [["7.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "max-connections": {"v_range": [["7.0.11", "7.0.12"], ["7.2.5", "7.2.5"], ["7.4.1", ""]], "type": "int"},
                    "two-factor-filter": {"v_range": [["7.2.1", ""]], "type": "str"},
                    "account-key-upn-san": {"v_range": [["7.2.2", ""]], "choices": ["othername", "rfc822name", "dnsname"], "type": "str"},
                    "account-key-cert-field": {"v_range": [["7.4.1", ""]], "choices": ["othername", "rfc822name", "dnsname", "cn"], "type": "str"},
                    "status-ttl": {"v_range": [["7.4.3", ""]], "type": "int"},
                    "source-ip-interface": {"v_range": [["7.6.0", ""]], "type": "raw"}
                },
                "elements": "dict"
            },
            "group-filter": {"type": "str"},
            "group-member-check": {"choices": ["user-attr", "group-object", "posix-group-object"], "type": "str"},
            "group-object-filter": {"type": "str"},
            "group-search-base": {"type": "str"},
            "member-attr": {"type": "str"},
            "name": {"required": true, "type": "str"},
            "password": {"no_log": true, "type": "raw"},
            "password-expiry-warning": {"choices": ["disable", "enable"], "type": "str"},
            "password-renewal": {"choices": ["disable", "enable"], "type": "str"},
            "port": {"type": "int"},
            "secondary-server": {"type": "str"},
            "secure": {"choices": ["disable", "starttls", "ldaps"], "type": "str"},
            "server": {"type": "str"},
            "server-identity-check": {"choices": ["disable", "enable"], "type": "str"},
            "source-ip": {"type": "str"},
            "ssl-min-proto-version": {"choices": ["default", "TLSv1", "TLSv1-1", "TLSv1-2", "SSLv3", "TLSv1-3"], "type": "str"},
            "tertiary-server": {"type": "str"},
            "type": {"choices": ["simple", "anonymous", "regular"], "type": "str"},
            "username": {"type": "str"},
            "obtain-user-info": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "search-type": {"v_range": [["6.2.0", ""]], "type": "list", "choices": ["nested", "recursive"], "elements": "str"},
            "user-info-exchange-server": {"v_range": [["6.2.0", ""]], "type": "str"},
            "account-key-name": {"v_range": [["6.2.0", "6.4.14"]], "no_log": true, "type": "str"},
            "group-object-search-base": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"},
            "two-factor": {"v_range": [["6.2.2", ""]], "choices": ["disable", "fortitoken-cloud"], "type": "str"},
            "interface": {"v_range": [["6.2.5", "6.2.12"], ["6.4.1", ""]], "type": "str"},
            "interface-select-method": {"v_range": [["6.2.5", "6.2.12"], ["6.4.1", ""]], "choices": ["auto", "sdwan", "specify"], "type": "str"},
            "two-factor-authentication": {"v_range": [["6.2.5", ""]], "choices": ["fortitoken", "email", "sms"], "type": "str"},
            "two-factor-notification": {"v_range": [["6.2.5", ""]], "choices": ["email", "sms"], "type": "str"},
            "antiphish": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "password-attr": {"v_range": [["7.0.0", ""]], "no_log": true, "type": "str"},
            "source-port": {"v_range": [["7.0.0", ""]], "type": "int"},
            "client-cert": {"v_range": [["7.2.0", ""]], "type": "str"},
            "client-cert-auth": {"v_range": [["7.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "two-factor-filter": {"v_range": [["7.2.1", ""]], "type": "str"},
            "account-key-upn-san": {"v_range": [["7.2.2", ""]], "choices": ["othername", "rfc822name", "dnsname"], "type": "str"},
            "account-key-cert-field": {"v_range": [["7.4.1", ""]], "choices": ["othername", "rfc822name", "dnsname", "cn"], "type": "str"},
            "status-ttl": {"v_range": [["7.4.3", ""]], "type": "int"},
            "source-ip-interface": {"v_range": [["7.6.0", ""]], "type": "raw"}
        }
    },
    "user_ldap_dynamicmapping": {
        "stated": true,
        "options": {
            "_scope": {"type": "list", "options": {"name": {"type": "str"}, "vdom": {"type": "str"}}, "elements": "dict"},
            "account-key-filter": {"no_log": true, "type": "str"},
            "account-key-name": {"no_log": true, "type": "str"},
            "account-key-processing": {"choices": ["same", "strip"], "type": "str"},
            "ca-cert": {"type": "str"},
            "cnid": {"type": "str"},
            "dn": {"type": "str"},
            "filter": {"type": "str"},
            "group": {"type": "str"},
            "group-filter": {"type": "str"},
            "group-member-check": {"choices": ["user-attr", "group-object", "posix-group-object"], "type": "str"},
            "group-object-filter": {"type": "str"},
            "group-object-search-base": {"type": "str"},
            "group-search-base": {"type": "str"},
            "member-attr": {"type": "str"},
            "obtain-user-info": {"choices": ["disable", "enable"], "type": "str"},
            "password": {"no_log": true, "type": "raw"},
            "password-expiry-warning": {"choices": ["disable", "enable"], "type": "str"},
            "password-renewal": {"choices": ["disable", "enable"], "type": "str"},
            "port": {"type": "int"},
            "retrieve-protection-profile": {"type": "str"},
            "search-type": {"type": "list", "choices": ["nested", "recursive"], "elements": "str"},
            "secondary-server": {"type": "str"},
            "secure": {"choices": ["disable", "starttls", "ldaps"], "type": "str"},
            "server": {"type": "str"},
            "server-identity-check": {"choices": ["disable", "enable"], "type": "str"},
            "source-ip": {"type": "str"},
            "ssl-min-proto-version": {"choices": ["default", "TLSv1", "TLSv1-1", "TLSv1-2", "SSLv3", "TLSv1-3"], "type": "str"},
            "tertiary-server": {"type": "str"},
            "type": {"choices": ["simple", "anonymous", "regular"], "type": "str"},
            "user-info-exchange-server": {"type": "str"},
            "username": {"type": "str"},
            "two-factor": {"v_range": [["6.2.2", ""]], "choices": ["disable", "fortitoken-cloud"], "type": "str"},
            "interface": {"v_range": [["6.2.5", "6.2.12"], ["6.4.1", ""]], "type": "str"},
            "interface-select-method": {"v_range": [["6.2.5", "6.2.12"], ["6.4.1", ""]], "choices": ["auto", "sdwan", "specify"], "type": "str"},
            "two-factor-authentication": {"v_range": [["6.2.5", ""]], "choices": ["fortitoken", "email", "sms"], "type": "str"},
            "two-factor-notification": {"v_range": [["6.2.5", ""]], "choices": ["email", "sms"], "type": "str"},
            "antiphish": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "password-attr": {"v_range": [["7.0.0", ""]], "no_log": true, "type": "str"},
            "source-port": {"v_range": [["7.0.0", ""]], "type": "int"},
            "client-cert": {"v_range": [["7.2.0", ""]], "type": "str"},
            "client-cert-auth": {"v_range": [["7.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "max-connections": {"v_range": [["7.0.11", "7.0.12"], ["7.2.5", "7.2.5"], ["7.4.1", ""]], "type": "int"},
            "two-factor-filter": {"v_range": [["7.2.1", ""]], "type": "str"},
            "account-key-upn-san": {"v_range": [["7.2.2", ""]], "choices": ["othername", "rfc822name", "dnsname"], "type": "str"},
            "account-key-cert-field": {"v_range": [["7.4.1", ""]], "choices": ["othername", "rfc822name", "dnsname", "cn"], "type": "str"},
            "status-ttl": {"v_range": [["7.4.3", ""]], "type": "int"},
            "source-ip-interface": {"v_range": [["7.6.0", ""]], "type": "raw"}
        }
    },
    "user_local": {
        "stated": true,
        "options": {
            "auth-concurrent-override": {"choices": ["disable", "enable"], "type": "str"},
            "auth-concurrent-value": {"type": "int"},
            "authtimeout": {"type": "int"},
            "email-to": {"type": "str"},
            "fortitoken": {"no_log": true, "type": "str"},
            "id": {"type": "int"},
            "ldap-server": {"type": "str"},
            "name": {"required": true, "type": "str"},
            "passwd": {"no_log": true, "type": "raw"},
            "passwd-policy": {"no_log": true, "type": "str"},
            "ppk-identity": {"type": "str"},
            "ppk-secret": {"no_log": true, "type": "raw"},
            "radius-server": {"type": "str"},
            "sms-custom-server": {"type": "str"},
            "sms-phone": {"type": "str"},
            "sms-server": {"choices": ["fortiguard", "custom"], "type": "str"},
            "status": {"choices": ["disable", "enable"], "type": "str"},
            "tacacs+-server": {"type": "str"},
            "two-factor": {"choices": ["disable", "fortitoken", "email", "sms", "fortitoken-cloud"], "type": "str"},
            "type": {"choices": ["password", "radius", "tacacs+", "ldap"], "type": "str"},
            "workstation": {"type": "str"},
            "two-factor-authentication": {"v_range": [["6.2.5", ""]], "choices": ["fortitoken", "email", "sms"], "type": "str"},
            "two-factor-notification": {"v_range": [["6.2.5", ""]], "choices": ["email", "sms"], "type": "str"},
            "username-case-sensitivity": {"v_range": [["6.2.5", "6.2.12"], ["6.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "username-case-insensitivity": {"v_range": [["6.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "username-sensitivity": {
                "v_range": [["6.2.9", "6.2.12"], ["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "choices": ["disable", "enable"],
                "type": "str"
            },
            "history0": {"v_range": [["7.4.1", ""]], "type": "raw"},
            "history1": {"v_range": [["7.4.1", ""]], "type": "raw"},
            "qkd-profile": {"v_range": [["7.4.2", ""]], "type": "str"}
        }
    },
    "user_nsx": {
        "stated": true,
        "options": {
            "fmgip": {"v_range": [["6.2.1", ""]], "type": "str"},
            "fmguser": {"v_range": [["6.2.1", ""]], "type": "str"},
            "name": {"v_range": [["6.2.1", ""]], "required": true, "type": "str"},
            "password": {"v_range": [["6.2.1", ""]], "no_log": true, "type": "raw"},
            "server": {"v_range": [["6.2.1", ""]], "type": "str"},
            "status": {"v_range": [["6.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "user": {"v_range": [["6.2.1", ""]], "type": "str"},
            "fmgpasswd": {"v_range": [["6.2.2", ""]], "no_log": true, "type": "raw"},
            "service-id": {"v_range": [["6.2.2", ""]], "type": "raw"},
            "if-allgroup": {"v_range": [["7.0.4", ""]], "choices": ["disable", "enable"], "type": "str"},
            "service": {
                "v_range": [["7.0.4", ""]],
                "type": "list",
                "options": {
                    "id": {"v_range": [["7.0.4", ""]], "type": "str"},
                    "integration": {"v_range": [["7.0.4", ""]], "choices": ["east-west", "north-south"], "type": "str"},
                    "name": {"v_range": [["7.0.4", ""]], "type": "str"},
                    "ref-id": {"v_range": [["7.0.4", ""]], "type": "str"}
                },
                "elements": "dict"
            },
            "service-manager-id": {"v_range": [["7.0.4", ""]], "type": "str"},
            "service-manager-rev": {"v_range": [["7.0.4", "7.0.12"], ["7.2.1", ""]], "type": "int"}
        }
    },
    "user_nsx_service": {
        "stated": true,
        "options": {
            "id": {"v_range": [["7.0.4", ""]], "required": true, "type": "str"},
            "integration": {"v_range": [["7.0.4", ""]], "choices": ["east-west", "north-south"], "type": "str"},
            "name": {"v_range": [["7.0.4", ""]], "type": "str"},
            "ref-id": {"v_range": [["7.0.4", ""]], "type": "str"}
        }
    },
    "user_passwordpolicy": {
        "stated": true,
        "options": {
            "expire-days": {"type": "int"},
            "name": {"required": true, "type": "str"},
            "warn-days": {"type": "int"},
            "expired-password-renewal": {"v_range": [["6.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "expire-status": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "min-change-characters": {"v_range": [["7.4.1", ""]], "type": "int"},
            "min-lower-case-letter": {"v_range": [["7.4.1", ""]], "type": "int"},
            "min-non-alphanumeric": {"v_range": [["7.4.1", ""]], "type": "int"},
            "min-number": {"v_range": [["7.4.1", ""]], "type": "int"},
            "min-upper-case-letter": {"v_range": [["7.4.1", ""]], "type": "int"},
            "minimum-length": {"v_range": [["7.4.1", ""]], "type": "int"},
            "reuse-password": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "reuse-password-limit": {"v_range": [["7.6.0", ""]], "no_log": true, "type": "int"}
        }
    },
    "user_peer": {
        "stated": true,
        "options": {
            "ca": {"type": "str"},
            "cn": {"type": "str"},
            "cn-type": {"choices": ["string", "email", "FQDN", "ipv4", "ipv6"], "type": "str"},
            "ldap-mode": {"choices": ["password", "principal-name"], "type": "str"},
            "ldap-password": {"no_log": true, "type": "raw"},
            "ldap-server": {"type": "str"},
            "ldap-username": {"type": "str"},
            "mandatory-ca-verify": {"choices": ["disable", "enable"], "type": "str"},
            "name": {"required": true, "type": "str"},
            "ocsp-override-server": {"type": "str"},
            "passwd": {"no_log": true, "type": "raw"},
            "subject": {"type": "str"},
            "two-factor": {"choices": ["disable", "enable"], "type": "str"},
            "mfa-mode": {"v_range": [["7.4.1", ""]], "choices": ["none", "password", "subject-identity"], "type": "str"},
            "mfa-password": {"v_range": [["7.4.1", ""]], "no_log": true, "type": "raw"},
            "mfa-server": {"v_range": [["7.4.1", ""]], "type": "str"},
            "mfa-username": {"v_range": [["7.4.1", ""]], "type": "str"}
        }
    },
    "user_peergrp": {"stated": true, "options": {"member": {"type": "raw"}, "name": {"required": true, "type": "str"}}},
    "user_pop3": {
        "stated": true,
        "options": {
            "name": {"required": true, "type": "str"},
            "port": {"type": "int"},
            "secure": {"choices": ["none", "starttls", "pop3s"], "type": "str"},
            "server": {"type": "str"},
            "ssl-min-proto-version": {"choices": ["default", "TLSv1", "TLSv1-1", "TLSv1-2", "SSLv3", "TLSv1-3"], "type": "str"}
        }
    },
    "user_pxgrid": {
        "stated": true,
        "options": {
            "ca-cert": {"type": "str"},
            "client-cert": {"type": "str"},
            "name": {"required": true, "type": "str"},
            "server": {"type": "str"},
            "status": {"choices": ["disable", "enable"], "type": "str"}
        }
    },
    "user_radius": {
        "stated": true,
        "options": {
            "accounting-server": {
                "type": "list",
                "options": {
                    "id": {"type": "int"},
                    "port": {"type": "int"},
                    "secret": {"no_log": true, "type": "raw"},
                    "server": {"type": "str"},
                    "source-ip": {"type": "str"},
                    "status": {"choices": ["disable", "enable"], "type": "str"},
                    "interface": {"v_range": [["6.2.5", "6.2.12"], ["6.4.1", ""]], "type": "str"},
                    "interface-select-method": {"v_range": [["6.2.5", "6.2.12"], ["6.4.1", ""]], "choices": ["auto", "sdwan", "specify"], "type": "str"}
                },
                "elements": "dict"
            },
            "acct-all-servers": {"choices": ["disable", "enable"], "type": "str"},
            "acct-interim-interval": {"type": "int"},
            "all-usergroup": {"choices": ["disable", "enable"], "type": "str"},
            "auth-type": {"choices": ["pap", "chap", "ms_chap", "ms_chap_v2", "auto"], "type": "str"},
            "class": {"type": "raw"},
            "dynamic_mapping": {
                "type": "list",
                "options": {
                    "_scope": {"type": "list", "options": {"name": {"type": "str"}, "vdom": {"type": "str"}}, "elements": "dict"},
                    "acct-all-servers": {"choices": ["disable", "enable"], "type": "str"},
                    "acct-interim-interval": {"type": "int"},
                    "all-usergroup": {"choices": ["disable", "enable"], "type": "str"},
                    "auth-type": {"choices": ["pap", "chap", "ms_chap", "ms_chap_v2", "auto"], "type": "str"},
                    "class": {"type": "raw"},
                    "dp-carrier-endpoint-attribute": {
                        "choices": [
                            "User-Name", "User-Password", "CHAP-Password", "NAS-IP-Address", "NAS-Port", "Service-Type", "Framed-Protocol",
                            "Framed-IP-Address", "Framed-IP-Netmask", "Framed-Routing", "Filter-Id", "Framed-MTU", "Framed-Compression", "Login-IP-Host",
                            "Login-Service", "Login-TCP-Port", "Reply-Message", "Callback-Number", "Callback-Id", "Framed-Route", "Framed-IPX-Network",
                            "State", "Class", "Vendor-Specific", "Session-Timeout", "Idle-Timeout", "Termination-Action", "Called-Station-Id",
                            "Calling-Station-Id", "NAS-Identifier", "Proxy-State", "Login-LAT-Service", "Login-LAT-Node", "Login-LAT-Group",
                            "Framed-AppleTalk-Link", "Framed-AppleTalk-Network", "Framed-AppleTalk-Zone", "Acct-Status-Type", "Acct-Delay-Time",
                            "Acct-Input-Octets", "Acct-Output-Octets", "Acct-Session-Id", "Acct-Authentic", "Acct-Session-Time", "Acct-Input-Packets",
                            "Acct-Output-Packets", "Acct-Terminate-Cause", "Acct-Multi-Session-Id", "Acct-Link-Count", "CHAP-Challenge", "NAS-Port-Type",
                            "Port-Limit", "Login-LAT-Port"
                        ],
                        "type": "str"
                    },
                    "dp-carrier-endpoint-block-attribute": {
                        "choices": [
                            "User-Name", "User-Password", "CHAP-Password", "NAS-IP-Address", "NAS-Port", "Service-Type", "Framed-Protocol",
                            "Framed-IP-Address", "Framed-IP-Netmask", "Framed-Routing", "Filter-Id", "Framed-MTU", "Framed-Compression", "Login-IP-Host",
                            "Login-Service", "Login-TCP-Port", "Reply-Message", "Callback-Number", "Callback-Id", "Framed-Route", "Framed-IPX-Network",
                            "State", "Class", "Vendor-Specific", "Session-Timeout", "Idle-Timeout", "Termination-Action", "Called-Station-Id",
                            "Calling-Station-Id", "NAS-Identifier", "Proxy-State", "Login-LAT-Service", "Login-LAT-Node", "Login-LAT-Group",
                            "Framed-AppleTalk-Link", "Framed-AppleTalk-Network", "Framed-AppleTalk-Zone", "Acct-Status-Type", "Acct-Delay-Time",
                            "Acct-Input-Octets", "Acct-Output-Octets", "Acct-Session-Id", "Acct-Authentic", "Acct-Session-Time", "Acct-Input-Packets",
                            "Acct-Output-Packets", "Acct-Terminate-Cause", "Acct-Multi-Session-Id", "Acct-Link-Count", "CHAP-Challenge", "NAS-Port-Type",
                            "Port-Limit", "Login-LAT-Port"
                        ],
                        "type": "str"
                    },
                    "dp-context-timeout": {"type": "int"},
                    "dp-flush-ip-session": {"choices": ["disable", "enable"], "type": "str"},
                    "dp-hold-time": {"type": "int"},
                    "dp-http-header": {"type": "str"},
                    "dp-http-header-fallback": {"choices": ["ip-header-address", "default-profile"], "type": "str"},
                    "dp-http-header-status": {"choices": ["disable", "enable"], "type": "str"},
                    "dp-http-header-suppress": {"choices": ["disable", "enable"], "type": "str"},
                    "dp-log-dyn_flags": {
                        "type": "list",
                        "choices": [
                            "none", "protocol-error", "profile-missing", "context-missing", "accounting-stop-missed", "accounting-event",
                            "radiusd-other", "endpoint-block"
                        ],
                        "elements": "str"
                    },
                    "dp-log-period": {"type": "int"},
                    "dp-mem-percent": {"type": "int"},
                    "dp-profile-attribute": {
                        "choices": [
                            "User-Name", "User-Password", "CHAP-Password", "NAS-IP-Address", "NAS-Port", "Service-Type", "Framed-Protocol",
                            "Framed-IP-Address", "Framed-IP-Netmask", "Framed-Routing", "Filter-Id", "Framed-MTU", "Framed-Compression", "Login-IP-Host",
                            "Login-Service", "Login-TCP-Port", "Reply-Message", "Callback-Number", "Callback-Id", "Framed-Route", "Framed-IPX-Network",
                            "State", "Class", "Vendor-Specific", "Session-Timeout", "Idle-Timeout", "Termination-Action", "Called-Station-Id",
                            "Calling-Station-Id", "NAS-Identifier", "Proxy-State", "Login-LAT-Service", "Login-LAT-Node", "Login-LAT-Group",
                            "Framed-AppleTalk-Link", "Framed-AppleTalk-Network", "Framed-AppleTalk-Zone", "Acct-Status-Type", "Acct-Delay-Time",
                            "Acct-Input-Octets", "Acct-Output-Octets", "Acct-Session-Id", "Acct-Authentic", "Acct-Session-Time", "Acct-Input-Packets",
                            "Acct-Output-Packets", "Acct-Terminate-Cause", "Acct-Multi-Session-Id", "Acct-Link-Count", "CHAP-Challenge", "NAS-Port-Type",
                            "Port-Limit", "Login-LAT-Port"
                        ],
                        "type": "str"
                    },
                    "dp-profile-attribute-key": {"no_log": true, "type": "str"},
                    "dp-radius-response": {"choices": ["disable", "enable"], "type": "str"},
                    "dp-radius-server-port": {"type": "int"},
                    "dp-secret": {"no_log": true, "type": "raw"},
                    "dp-validate-request-secret": {"choices": ["disable", "enable"], "type": "str"},
                    "dynamic-profile": {"choices": ["disable", "enable"], "type": "str"},
                    "endpoint-translation": {"choices": ["disable", "enable"], "type": "str"},
                    "ep-carrier-endpoint-convert-hex": {"choices": ["disable", "enable"], "type": "str"},
                    "ep-carrier-endpoint-header": {"type": "str"},
                    "ep-carrier-endpoint-header-suppress": {"choices": ["disable", "enable"], "type": "str"},
                    "ep-carrier-endpoint-prefix": {"choices": ["disable", "enable"], "type": "str"},
                    "ep-carrier-endpoint-prefix-range-max": {"type": "int"},
                    "ep-carrier-endpoint-prefix-range-min": {"type": "int"},
                    "ep-carrier-endpoint-prefix-string": {"type": "str"},
                    "ep-carrier-endpoint-source": {"choices": ["http-header", "cookie"], "type": "str"},
                    "ep-ip-header": {"type": "str"},
                    "ep-ip-header-suppress": {"choices": ["disable", "enable"], "type": "str"},
                    "ep-missing-header-fallback": {"choices": ["session-ip", "policy-profile"], "type": "str"},
                    "ep-profile-query-type": {"choices": ["session-ip", "extract-ip", "extract-carrier-endpoint"], "type": "str"},
                    "h3c-compatibility": {"choices": ["disable", "enable"], "type": "str"},
                    "nas-ip": {"type": "str"},
                    "password-encoding": {"choices": ["ISO-8859-1", "auto"], "type": "str"},
                    "password-renewal": {"choices": ["disable", "enable"], "type": "str"},
                    "radius-coa": {"choices": ["disable", "enable"], "type": "str"},
                    "radius-port": {"type": "int"},
                    "rsso": {"choices": ["disable", "enable"], "type": "str"},
                    "rsso-context-timeout": {"type": "int"},
                    "rsso-endpoint-attribute": {
                        "choices": [
                            "User-Name", "User-Password", "CHAP-Password", "NAS-IP-Address", "NAS-Port", "Service-Type", "Framed-Protocol",
                            "Framed-IP-Address", "Framed-IP-Netmask", "Framed-Routing", "Filter-Id", "Framed-MTU", "Framed-Compression", "Login-IP-Host",
                            "Login-Service", "Login-TCP-Port", "Reply-Message", "Callback-Number", "Callback-Id", "Framed-Route", "Framed-IPX-Network",
                            "State", "Class", "Session-Timeout", "Idle-Timeout", "Termination-Action", "Called-Station-Id", "Calling-Station-Id",
                            "NAS-Identifier", "Proxy-State", "Login-LAT-Service", "Login-LAT-Node", "Login-LAT-Group", "Framed-AppleTalk-Link",
                            "Framed-AppleTalk-Network", "Framed-AppleTalk-Zone", "Acct-Status-Type", "Acct-Delay-Time", "Acct-Input-Octets",
                            "Acct-Output-Octets", "Acct-Session-Id", "Acct-Authentic", "Acct-Session-Time", "Acct-Input-Packets", "Acct-Output-Packets",
                            "Acct-Terminate-Cause", "Acct-Multi-Session-Id", "Acct-Link-Count", "CHAP-Challenge", "NAS-Port-Type", "Port-Limit",
                            "Login-LAT-Port"
                        ],
                        "type": "str"
                    },
                    "rsso-endpoint-block-attribute": {
                        "choices": [
                            "User-Name", "User-Password", "CHAP-Password", "NAS-IP-Address", "NAS-Port", "Service-Type", "Framed-Protocol",
                            "Framed-IP-Address", "Framed-IP-Netmask", "Framed-Routing", "Filter-Id", "Framed-MTU", "Framed-Compression", "Login-IP-Host",
                            "Login-Service", "Login-TCP-Port", "Reply-Message", "Callback-Number", "Callback-Id", "Framed-Route", "Framed-IPX-Network",
                            "State", "Class", "Session-Timeout", "Idle-Timeout", "Termination-Action", "Called-Station-Id", "Calling-Station-Id",
                            "NAS-Identifier", "Proxy-State", "Login-LAT-Service", "Login-LAT-Node", "Login-LAT-Group", "Framed-AppleTalk-Link",
                            "Framed-AppleTalk-Network", "Framed-AppleTalk-Zone", "Acct-Status-Type", "Acct-Delay-Time", "Acct-Input-Octets",
                            "Acct-Output-Octets", "Acct-Session-Id", "Acct-Authentic", "Acct-Session-Time", "Acct-Input-Packets", "Acct-Output-Packets",
                            "Acct-Terminate-Cause", "Acct-Multi-Session-Id", "Acct-Link-Count", "CHAP-Challenge", "NAS-Port-Type", "Port-Limit",
                            "Login-LAT-Port"
                        ],
                        "type": "str"
                    },
                    "rsso-ep-one-ip-only": {"choices": ["disable", "enable"], "type": "str"},
                    "rsso-flush-ip-session": {"choices": ["disable", "enable"], "type": "str"},
                    "rsso-log-flags": {
                        "type": "list",
                        "choices": [
                            "none", "protocol-error", "profile-missing", "context-missing", "accounting-stop-missed", "accounting-event",
                            "radiusd-other", "endpoint-block"
                        ],
                        "elements": "str"
                    },
                    "rsso-log-period": {"type": "int"},
                    "rsso-radius-response": {"choices": ["disable", "enable"], "type": "str"},
                    "rsso-radius-server-port": {"type": "int"},
                    "rsso-secret": {"no_log": true, "type": "raw"},
                    "rsso-validate-request-secret": {"choices": ["disable", "enable"], "type": "str"},
                    "secondary-secret": {"no_log": true, "type": "raw"},
                    "secondary-server": {"type": "str"},
                    "secret": {"no_log": true, "type": "raw"},
                    "server": {"type": "str"},
                    "source-ip": {"type": "str"},
                    "sso-attribute": {
                        "choices": [
                            "User-Name", "User-Password", "CHAP-Password", "NAS-IP-Address", "NAS-Port", "Service-Type", "Framed-Protocol",
                            "Framed-IP-Address", "Framed-IP-Netmask", "Framed-Routing", "Filter-Id", "Framed-MTU", "Framed-Compression", "Login-IP-Host",
                            "Login-Service", "Login-TCP-Port", "Reply-Message", "Callback-Number", "Callback-Id", "Framed-Route", "Framed-IPX-Network",
                            "State", "Class", "Session-Timeout", "Idle-Timeout", "Termination-Action", "Called-Station-Id", "Calling-Station-Id",
                            "NAS-Identifier", "Proxy-State", "Login-LAT-Service", "Login-LAT-Node", "Login-LAT-Group", "Framed-AppleTalk-Link",
                            "Framed-AppleTalk-Network", "Framed-AppleTalk-Zone", "Acct-Status-Type", "Acct-Delay-Time", "Acct-Input-Octets",
                            "Acct-Output-Octets", "Acct-Session-Id", "Acct-Authentic", "Acct-Session-Time", "Acct-Input-Packets", "Acct-Output-Packets",
                            "Acct-Terminate-Cause", "Acct-Multi-Session-Id", "Acct-Link-Count", "CHAP-Challenge", "NAS-Port-Type", "Port-Limit",
                            "Login-LAT-Port"
                        ],
                        "type": "str"
                    },
                    "sso-attribute-key": {"no_log": true, "type": "str"},
                    "sso-attribute-value-override": {"choices": ["disable", "enable"], "type": "str"},
                    "tertiary-secret": {"no_log": true, "type": "raw"},
                    "tertiary-server": {"type": "str"},
                    "timeout": {"type": "int"},
                    "use-group-for-profile": {"choices": ["disable", "enable"], "type": "str"},
                    "use-management-vdom": {"choices": ["disable", "enable"], "type": "str"},
                    "username-case-sensitive": {"choices": ["disable", "enable"], "type": "str"},
                    "interface": {"v_range": [["6.2.5", "6.2.12"], ["6.4.1", ""]], "type": "str"},
                    "interface-select-method": {"v_range": [["6.2.5", "6.2.12"], ["6.4.1", ""]], "choices": ["auto", "sdwan", "specify"], "type": "str"},
                    "group-override-attr-type": {"v_range": [["6.4.0", ""]], "choices": ["filter-Id", "class"], "type": "str"},
                    "switch-controller-acct-fast-framedip-detect": {"v_range": [["6.4.0", ""]], "type": "int"},
                    "accounting-server": {
                        "v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]],
                        "type": "list",
                        "options": {
                            "id": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "type": "int"},
                            "interface": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "type": "str"},
                            "interface-select-method": {
                                "v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]],
                                "choices": ["auto", "sdwan", "specify"],
                                "type": "str"
                            },
                            "port": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "type": "int"},
                            "secret": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "no_log": true, "type": "raw"},
                            "server": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "type": "str"},
                            "source-ip": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "type": "str"},
                            "status": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "choices": ["disable", "enable"], "type": "str"}
                        },
                        "elements": "dict"
                    },
                    "switch-controller-service-type": {
                        "v_range": [["6.4.2", ""]],
                        "type": "list",
                        "choices": [
                            "login", "framed", "callback-login", "callback-framed", "outbound", "administrative", "nas-prompt", "authenticate-only",
                            "callback-nas-prompt", "call-check", "callback-administrative"
                        ],
                        "elements": "str"
                    },
                    "delimiter": {"v_range": [["7.2.0", ""]], "choices": ["plus", "comma"], "type": "str"},
                    "mac-case": {"v_range": [["7.2.1", ""]], "choices": ["uppercase", "lowercase"], "type": "str"},
                    "mac-password-delimiter": {"v_range": [["7.2.1", ""]], "choices": ["hyphen", "single-hyphen", "colon", "none"], "type": "str"},
                    "mac-username-delimiter": {"v_range": [["7.2.1", ""]], "choices": ["hyphen", "single-hyphen", "colon", "none"], "type": "str"},
                    "nas-id": {"v_range": [["7.2.2", ""]], "type": "str"},
                    "nas-id-type": {"v_range": [["7.2.2", ""]], "choices": ["legacy", "custom", "hostname"], "type": "str"},
                    "ca-cert": {"v_range": [["7.4.0", ""]], "type": "str"},
                    "client-cert": {"v_range": [["7.4.0", ""]], "type": "str"},
                    "server-identity-check": {"v_range": [["7.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "status-ttl": {"v_range": [["7.4.0", ""]], "type": "int"},
                    "tls-min-proto-version": {
                        "v_range": [["7.4.0", ""]],
                        "choices": ["default", "TLSv1", "TLSv1-1", "TLSv1-2", "SSLv3", "TLSv1-3"],
                        "type": "str"
                    },
                    "transport-protocol": {"v_range": [["7.4.0", ""]], "choices": ["udp", "tcp", "tls"], "type": "str"},
                    "account-key-cert-field": {"v_range": [["7.4.1", ""]], "choices": ["othername", "rfc822name", "dnsname", "cn"], "type": "str"},
                    "account-key-processing": {"v_range": [["7.4.1", ""]], "choices": ["same", "strip"], "type": "str"},
                    "call-station-id-type": {"v_range": [["7.4.1", ""]], "choices": ["legacy", "IP", "MAC"], "type": "str"},
                    "switch-controller-nas-ip-dynamic": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "source-ip-interface": {"v_range": [["7.6.0", ""]], "type": "raw"}
                },
                "elements": "dict"
            },
            "h3c-compatibility": {"choices": ["disable", "enable"], "type": "str"},
            "name": {"required": true, "type": "str"},
            "nas-ip": {"type": "str"},
            "password-encoding": {"choices": ["ISO-8859-1", "auto"], "type": "str"},
            "password-renewal": {"choices": ["disable", "enable"], "type": "str"},
            "radius-coa": {"choices": ["disable", "enable"], "type": "str"},
            "radius-port": {"type": "int"},
            "rsso": {"choices": ["disable", "enable"], "type": "str"},
            "rsso-context-timeout": {"type": "int"},
            "rsso-endpoint-attribute": {
                "choices": [
                    "User-Name", "User-Password", "CHAP-Password", "NAS-IP-Address", "NAS-Port", "Service-Type", "Framed-Protocol", "Framed-IP-Address",
                    "Framed-IP-Netmask", "Framed-Routing", "Filter-Id", "Framed-MTU", "Framed-Compression", "Login-IP-Host", "Login-Service",
                    "Login-TCP-Port", "Reply-Message", "Callback-Number", "Callback-Id", "Framed-Route", "Framed-IPX-Network", "State", "Class",
                    "Session-Timeout", "Idle-Timeout", "Termination-Action", "Called-Station-Id", "Calling-Station-Id", "NAS-Identifier", "Proxy-State",
                    "Login-LAT-Service", "Login-LAT-Node", "Login-LAT-Group", "Framed-AppleTalk-Link", "Framed-AppleTalk-Network",
                    "Framed-AppleTalk-Zone", "Acct-Status-Type", "Acct-Delay-Time", "Acct-Input-Octets", "Acct-Output-Octets", "Acct-Session-Id",
                    "Acct-Authentic", "Acct-Session-Time", "Acct-Input-Packets", "Acct-Output-Packets", "Acct-Terminate-Cause", "Acct-Multi-Session-Id",
                    "Acct-Link-Count", "CHAP-Challenge", "NAS-Port-Type", "Port-Limit", "Login-LAT-Port"
                ],
                "type": "str"
            },
            "rsso-endpoint-block-attribute": {
                "choices": [
                    "User-Name", "User-Password", "CHAP-Password", "NAS-IP-Address", "NAS-Port", "Service-Type", "Framed-Protocol", "Framed-IP-Address",
                    "Framed-IP-Netmask", "Framed-Routing", "Filter-Id", "Framed-MTU", "Framed-Compression", "Login-IP-Host", "Login-Service",
                    "Login-TCP-Port", "Reply-Message", "Callback-Number", "Callback-Id", "Framed-Route", "Framed-IPX-Network", "State", "Class",
                    "Session-Timeout", "Idle-Timeout", "Termination-Action", "Called-Station-Id", "Calling-Station-Id", "NAS-Identifier", "Proxy-State",
                    "Login-LAT-Service", "Login-LAT-Node", "Login-LAT-Group", "Framed-AppleTalk-Link", "Framed-AppleTalk-Network",
                    "Framed-AppleTalk-Zone", "Acct-Status-Type", "Acct-Delay-Time", "Acct-Input-Octets", "Acct-Output-Octets", "Acct-Session-Id",
                    "Acct-Authentic", "Acct-Session-Time", "Acct-Input-Packets", "Acct-Output-Packets", "Acct-Terminate-Cause", "Acct-Multi-Session-Id",
                    "Acct-Link-Count", "CHAP-Challenge", "NAS-Port-Type", "Port-Limit", "Login-LAT-Port"
                ],
                "type": "str"
            },
            "rsso-ep-one-ip-only": {"choices": ["disable", "enable"], "type": "str"},
            "rsso-flush-ip-session": {"choices": ["disable", "enable"], "type": "str"},
            "rsso-log-flags": {
                "type": "list",
                "choices": [
                    "none", "protocol-error", "profile-missing", "context-missing", "accounting-stop-missed", "accounting-event", "radiusd-other",
                    "endpoint-block"
                ],
                "elements": "str"
            },
            "rsso-log-period": {"type": "int"},
            "rsso-radius-response": {"choices": ["disable", "enable"], "type": "str"},
            "rsso-radius-server-port": {"type": "int"},
            "rsso-secret": {"no_log": true, "type": "raw"},
            "rsso-validate-request-secret": {"choices": ["disable", "enable"], "type": "str"},
            "secondary-secret": {"no_log": true, "type": "raw"},
            "secondary-server": {"type": "str"},
            "secret": {"no_log": true, "type": "raw"},
            "server": {"type": "str"},
            "source-ip": {"type": "str"},
            "sso-attribute": {
                "choices": [
                    "User-Name", "User-Password", "CHAP-Password", "NAS-IP-Address", "NAS-Port", "Service-Type", "Framed-Protocol", "Framed-IP-Address",
                    "Framed-IP-Netmask", "Framed-Routing", "Filter-Id", "Framed-MTU", "Framed-Compression", "Login-IP-Host", "Login-Service",
                    "Login-TCP-Port", "Reply-Message", "Callback-Number", "Callback-Id", "Framed-Route", "Framed-IPX-Network", "State", "Class",
                    "Session-Timeout", "Idle-Timeout", "Termination-Action", "Called-Station-Id", "Calling-Station-Id", "NAS-Identifier", "Proxy-State",
                    "Login-LAT-Service", "Login-LAT-Node", "Login-LAT-Group", "Framed-AppleTalk-Link", "Framed-AppleTalk-Network",
                    "Framed-AppleTalk-Zone", "Acct-Status-Type", "Acct-Delay-Time", "Acct-Input-Octets", "Acct-Output-Octets", "Acct-Session-Id",
                    "Acct-Authentic", "Acct-Session-Time", "Acct-Input-Packets", "Acct-Output-Packets", "Acct-Terminate-Cause", "Acct-Multi-Session-Id",
                    "Acct-Link-Count", "CHAP-Challenge", "NAS-Port-Type", "Port-Limit", "Login-LAT-Port"
                ],
                "type": "str"
            },
            "sso-attribute-key": {"no_log": true, "type": "str"},
            "sso-attribute-value-override": {"choices": ["disable", "enable"], "type": "str"},
            "tertiary-secret": {"no_log": true, "type": "raw"},
            "tertiary-server": {"type": "str"},
            "timeout": {"type": "int"},
            "use-management-vdom": {"choices": ["disable", "enable"], "type": "str"},
            "username-case-sensitive": {"choices": ["disable", "enable"], "type": "str"},
            "interface": {"v_range": [["6.2.5", "6.2.12"], ["6.4.1", ""]], "type": "str"},
            "interface-select-method": {"v_range": [["6.2.5", "6.2.12"], ["6.4.1", ""]], "choices": ["auto", "sdwan", "specify"], "type": "str"},
            "group-override-attr-type": {"v_range": [["6.4.0", ""]], "choices": ["filter-Id", "class"], "type": "str"},
            "switch-controller-acct-fast-framedip-detect": {"v_range": [["6.4.0", ""]], "type": "int"},
            "switch-controller-service-type": {
                "v_range": [["6.4.2", ""]],
                "type": "list",
                "choices": [
                    "login", "framed", "callback-login", "callback-framed", "outbound", "administrative", "nas-prompt", "authenticate-only",
                    "callback-nas-prompt", "call-check", "callback-administrative"
                ],
                "elements": "str"
            },
            "delimiter": {"v_range": [["7.2.0", ""]], "choices": ["plus", "comma"], "type": "str"},
            "mac-case": {"v_range": [["7.2.1", ""]], "choices": ["uppercase", "lowercase"], "type": "str"},
            "mac-password-delimiter": {"v_range": [["7.2.1", ""]], "choices": ["hyphen", "single-hyphen", "colon", "none"], "type": "str"},
            "mac-username-delimiter": {"v_range": [["7.2.1", ""]], "choices": ["hyphen", "single-hyphen", "colon", "none"], "type": "str"},
            "nas-id": {"v_range": [["7.2.2", ""]], "type": "str"},
            "nas-id-type": {"v_range": [["7.2.2", ""]], "choices": ["legacy", "custom", "hostname"], "type": "str"},
            "ca-cert": {"v_range": [["7.4.0", ""]], "type": "str"},
            "client-cert": {"v_range": [["7.4.0", ""]], "type": "str"},
            "server-identity-check": {"v_range": [["7.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "status-ttl": {"v_range": [["7.4.0", ""]], "type": "int"},
            "tls-min-proto-version": {
                "v_range": [["7.4.0", ""]],
                "choices": ["default", "TLSv1", "TLSv1-1", "TLSv1-2", "SSLv3", "TLSv1-3"],
                "type": "str"
            },
            "transport-protocol": {"v_range": [["7.4.0", ""]], "choices": ["udp", "tcp", "tls"], "type": "str"},
            "account-key-cert-field": {"v_range": [["7.4.1", ""]], "choices": ["othername", "rfc822name", "dnsname", "cn"], "type": "str"},
            "account-key-processing": {"v_range": [["7.4.1", ""]], "choices": ["same", "strip"], "type": "str"},
            "call-station-id-type": {"v_range": [["7.4.1", ""]], "choices": ["legacy", "IP", "MAC"], "type": "str"},
            "switch-controller-nas-ip-dynamic": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "source-ip-interface": {"v_range": [["7.6.0", ""]], "type": "raw"}
        }
    },
    "user_radius_accountingserver": {
        "stated": true,
        "options": {
            "id": {"required": true, "type": "int"},
            "port": {"type": "int"},
            "secret": {"no_log": true, "type": "raw"},
            "server": {"type": "str"},
            "source-ip": {"type": "str"},
            "status": {"choices": ["disable", "enable"], "type": "str"},
            "interface": {"v_range": [["6.2.5", "6.2.12"], ["6.4.1", ""]], "type": "str"},
            "interface-select-method": {"v_range": [["6.2.5", "6.2.12"], ["6.4.1", ""]], "choices": ["auto", "sdwan", "specify"], "type": "str"}
        }
    },
    "user_radius_dynamicmapping": {
        "stated": true,
        "options": {
            "_scope": {"type": "list", "options": {"name": {"type": "str"}, "vdom": {"type": "str"}}, "elements": "dict"},
            "acct-all-servers": {"choices": ["disable", "enable"], "type": "str"},
            "acct-interim-interval": {"type": "int"},
            "all-usergroup": {"choices": ["disable", "enable"], "type": "str"},
            "auth-type": {"choices": ["pap", "chap", "ms_chap", "ms_chap_v2", "auto"], "type": "str"},
            "class": {"type": "raw"},
            "dp-carrier-endpoint-attribute": {
                "choices": [
                    "User-Name", "User-Password", "CHAP-Password", "NAS-IP-Address", "NAS-Port", "Service-Type", "Framed-Protocol", "Framed-IP-Address",
                    "Framed-IP-Netmask", "Framed-Routing", "Filter-Id", "Framed-MTU", "Framed-Compression", "Login-IP-Host", "Login-Service",
                    "Login-TCP-Port", "Reply-Message", "Callback-Number", "Callback-Id", "Framed-Route", "Framed-IPX-Network", "State", "Class",
                    "Vendor-Specific", "Session-Timeout", "Idle-Timeout", "Termination-Action", "Called-Station-Id", "Calling-Station-Id",
                    "NAS-Identifier", "Proxy-State", "Login-LAT-Service", "Login-LAT-Node", "Login-LAT-Group", "Framed-AppleTalk-Link",
                    "Framed-AppleTalk-Network", "Framed-AppleTalk-Zone", "Acct-Status-Type", "Acct-Delay-Time", "Acct-Input-Octets",
                    "Acct-Output-Octets", "Acct-Session-Id", "Acct-Authentic", "Acct-Session-Time", "Acct-Input-Packets", "Acct-Output-Packets",
                    "Acct-Terminate-Cause", "Acct-Multi-Session-Id", "Acct-Link-Count", "CHAP-Challenge", "NAS-Port-Type", "Port-Limit",
                    "Login-LAT-Port"
                ],
                "type": "str"
            },
            "dp-carrier-endpoint-block-attribute": {
                "choices": [
                    "User-Name", "User-Password", "CHAP-Password", "NAS-IP-Address", "NAS-Port", "Service-Type", "Framed-Protocol", "Framed-IP-Address",
                    "Framed-IP-Netmask", "Framed-Routing", "Filter-Id", "Framed-MTU", "Framed-Compression", "Login-IP-Host", "Login-Service",
                    "Login-TCP-Port", "Reply-Message", "Callback-Number", "Callback-Id", "Framed-Route", "Framed-IPX-Network", "State", "Class",
                    "Vendor-Specific", "Session-Timeout", "Idle-Timeout", "Termination-Action", "Called-Station-Id", "Calling-Station-Id",
                    "NAS-Identifier", "Proxy-State", "Login-LAT-Service", "Login-LAT-Node", "Login-LAT-Group", "Framed-AppleTalk-Link",
                    "Framed-AppleTalk-Network", "Framed-AppleTalk-Zone", "Acct-Status-Type", "Acct-Delay-Time", "Acct-Input-Octets",
                    "Acct-Output-Octets", "Acct-Session-Id", "Acct-Authentic", "Acct-Session-Time", "Acct-Input-Packets", "Acct-Output-Packets",
                    "Acct-Terminate-Cause", "Acct-Multi-Session-Id", "Acct-Link-Count", "CHAP-Challenge", "NAS-Port-Type", "Port-Limit",
                    "Login-LAT-Port"
                ],
                "type": "str"
            },
            "dp-context-timeout": {"type": "int"},
            "dp-flush-ip-session": {"choices": ["disable", "enable"], "type": "str"},
            "dp-hold-time": {"type": "int"},
            "dp-http-header": {"type": "str"},
            "dp-http-header-fallback": {"choices": ["ip-header-address", "default-profile"], "type": "str"},
            "dp-http-header-status": {"choices": ["disable", "enable"], "type": "str"},
            "dp-http-header-suppress": {"choices": ["disable", "enable"], "type": "str"},
            "dp-log-dyn_flags": {
                "type": "list",
                "choices": [
                    "none", "protocol-error", "profile-missing", "context-missing", "accounting-stop-missed", "accounting-event", "radiusd-other",
                    "endpoint-block"
                ],
                "elements": "str"
            },
            "dp-log-period": {"type": "int"},
            "dp-mem-percent": {"type": "int"},
            "dp-profile-attribute": {
                "choices": [
                    "User-Name", "User-Password", "CHAP-Password", "NAS-IP-Address", "NAS-Port", "Service-Type", "Framed-Protocol", "Framed-IP-Address",
                    "Framed-IP-Netmask", "Framed-Routing", "Filter-Id", "Framed-MTU", "Framed-Compression", "Login-IP-Host", "Login-Service",
                    "Login-TCP-Port", "Reply-Message", "Callback-Number", "Callback-Id", "Framed-Route", "Framed-IPX-Network", "State", "Class",
                    "Vendor-Specific", "Session-Timeout", "Idle-Timeout", "Termination-Action", "Called-Station-Id", "Calling-Station-Id",
                    "NAS-Identifier", "Proxy-State", "Login-LAT-Service", "Login-LAT-Node", "Login-LAT-Group", "Framed-AppleTalk-Link",
                    "Framed-AppleTalk-Network", "Framed-AppleTalk-Zone", "Acct-Status-Type", "Acct-Delay-Time", "Acct-Input-Octets",
                    "Acct-Output-Octets", "Acct-Session-Id", "Acct-Authentic", "Acct-Session-Time", "Acct-Input-Packets", "Acct-Output-Packets",
                    "Acct-Terminate-Cause", "Acct-Multi-Session-Id", "Acct-Link-Count", "CHAP-Challenge", "NAS-Port-Type", "Port-Limit",
                    "Login-LAT-Port"
                ],
                "type": "str"
            },
            "dp-profile-attribute-key": {"no_log": true, "type": "str"},
            "dp-radius-response": {"choices": ["disable", "enable"], "type": "str"},
            "dp-radius-server-port": {"type": "int"},
            "dp-secret": {"no_log": true, "type": "raw"},
            "dp-validate-request-secret": {"choices": ["disable", "enable"], "type": "str"},
            "dynamic-profile": {"choices": ["disable", "enable"], "type": "str"},
            "endpoint-translation": {"choices": ["disable", "enable"], "type": "str"},
            "ep-carrier-endpoint-convert-hex": {"choices": ["disable", "enable"], "type": "str"},
            "ep-carrier-endpoint-header": {"type": "str"},
            "ep-carrier-endpoint-header-suppress": {"choices": ["disable", "enable"], "type": "str"},
            "ep-carrier-endpoint-prefix": {"choices": ["disable", "enable"], "type": "str"},
            "ep-carrier-endpoint-prefix-range-max": {"type": "int"},
            "ep-carrier-endpoint-prefix-range-min": {"type": "int"},
            "ep-carrier-endpoint-prefix-string": {"type": "str"},
            "ep-carrier-endpoint-source": {"choices": ["http-header", "cookie"], "type": "str"},
            "ep-ip-header": {"type": "str"},
            "ep-ip-header-suppress": {"choices": ["disable", "enable"], "type": "str"},
            "ep-missing-header-fallback": {"choices": ["session-ip", "policy-profile"], "type": "str"},
            "ep-profile-query-type": {"choices": ["session-ip", "extract-ip", "extract-carrier-endpoint"], "type": "str"},
            "h3c-compatibility": {"choices": ["disable", "enable"], "type": "str"},
            "nas-ip": {"type": "str"},
            "password-encoding": {"choices": ["ISO-8859-1", "auto"], "type": "str"},
            "password-renewal": {"choices": ["disable", "enable"], "type": "str"},
            "radius-coa": {"choices": ["disable", "enable"], "type": "str"},
            "radius-port": {"type": "int"},
            "rsso": {"choices": ["disable", "enable"], "type": "str"},
            "rsso-context-timeout": {"type": "int"},
            "rsso-endpoint-attribute": {
                "choices": [
                    "User-Name", "User-Password", "CHAP-Password", "NAS-IP-Address", "NAS-Port", "Service-Type", "Framed-Protocol", "Framed-IP-Address",
                    "Framed-IP-Netmask", "Framed-Routing", "Filter-Id", "Framed-MTU", "Framed-Compression", "Login-IP-Host", "Login-Service",
                    "Login-TCP-Port", "Reply-Message", "Callback-Number", "Callback-Id", "Framed-Route", "Framed-IPX-Network", "State", "Class",
                    "Session-Timeout", "Idle-Timeout", "Termination-Action", "Called-Station-Id", "Calling-Station-Id", "NAS-Identifier", "Proxy-State",
                    "Login-LAT-Service", "Login-LAT-Node", "Login-LAT-Group", "Framed-AppleTalk-Link", "Framed-AppleTalk-Network",
                    "Framed-AppleTalk-Zone", "Acct-Status-Type", "Acct-Delay-Time", "Acct-Input-Octets", "Acct-Output-Octets", "Acct-Session-Id",
                    "Acct-Authentic", "Acct-Session-Time", "Acct-Input-Packets", "Acct-Output-Packets", "Acct-Terminate-Cause", "Acct-Multi-Session-Id",
                    "Acct-Link-Count", "CHAP-Challenge", "NAS-Port-Type", "Port-Limit", "Login-LAT-Port"
                ],
                "type": "str"
            },
            "rsso-endpoint-block-attribute": {
                "choices": [
                    "User-Name", "User-Password", "CHAP-Password", "NAS-IP-Address", "NAS-Port", "Service-Type", "Framed-Protocol", "Framed-IP-Address",
                    "Framed-IP-Netmask", "Framed-Routing", "Filter-Id", "Framed-MTU", "Framed-Compression", "Login-IP-Host", "Login-Service",
                    "Login-TCP-Port", "Reply-Message", "Callback-Number", "Callback-Id", "Framed-Route", "Framed-IPX-Network", "State", "Class",
                    "Session-Timeout", "Idle-Timeout", "Termination-Action", "Called-Station-Id", "Calling-Station-Id", "NAS-Identifier", "Proxy-State",
                    "Login-LAT-Service", "Login-LAT-Node", "Login-LAT-Group", "Framed-AppleTalk-Link", "Framed-AppleTalk-Network",
                    "Framed-AppleTalk-Zone", "Acct-Status-Type", "Acct-Delay-Time", "Acct-Input-Octets", "Acct-Output-Octets", "Acct-Session-Id",
                    "Acct-Authentic", "Acct-Session-Time", "Acct-Input-Packets", "Acct-Output-Packets", "Acct-Terminate-Cause", "Acct-Multi-Session-Id",
                    "Acct-Link-Count", "CHAP-Challenge", "NAS-Port-Type", "Port-Limit", "Login-LAT-Port"
                ],
                "type": "str"
            },
            "rsso-ep-one-ip-only": {"choices": ["disable", "enable"], "type": "str"},
            "rsso-flush-ip-session": {"choices": ["disable", "enable"], "type": "str"},
            "rsso-log-flags": {
                "type": "list",
                "choices": [
                    "none", "protocol-error", "profile-missing", "context-missing", "accounting-stop-missed", "accounting-event", "radiusd-other",
                    "endpoint-block"
                ],
                "elements": "str"
            },
            "rsso-log-period": {"type": "int"},
            "rsso-radius-response": {"choices": ["disable", "enable"], "type": "str"},
            "rsso-radius-server-port": {"type": "int"},
            "rsso-secret": {"no_log": true, "type": "raw"},
            "rsso-validate-request-secret": {"choices": ["disable", "enable"], "type": "str"},
            "secondary-secret": {"no_log": true, "type": "raw"},
            "secondary-server": {"type": "str"},
            "secret": {"no_log": true, "type": "raw"},
            "server": {"type": "str"},
            "source-ip": {"type": "str"},
            "sso-attribute": {
                "choices": [
                    "User-Name", "User-Password", "CHAP-Password", "NAS-IP-Address", "NAS-Port", "Service-Type", "Framed-Protocol", "Framed-IP-Address",
                    "Framed-IP-Netmask", "Framed-Routing", "Filter-Id", "Framed-MTU", "Framed-Compression", "Login-IP-Host", "Login-Service",
                    "Login-TCP-Port", "Reply-Message", "Callback-Number", "Callback-Id", "Framed-Route", "Framed-IPX-Network", "State", "Class",
                    "Session-Timeout", "Idle-Timeout", "Termination-Action", "Called-Station-Id", "Calling-Station-Id", "NAS-Identifier", "Proxy-State",
                    "Login-LAT-Service", "Login-LAT-Node", "Login-LAT-Group", "Framed-AppleTalk-Link", "Framed-AppleTalk-Network",
                    "Framed-AppleTalk-Zone", "Acct-Status-Type", "Acct-Delay-Time", "Acct-Input-Octets", "Acct-Output-Octets", "Acct-Session-Id",
                    "Acct-Authentic", "Acct-Session-Time", "Acct-Input-Packets", "Acct-Output-Packets", "Acct-Terminate-Cause", "Acct-Multi-Session-Id",
                    "Acct-Link-Count", "CHAP-Challenge", "NAS-Port-Type", "Port-Limit", "Login-LAT-Port"
                ],
                "type": "str"
            },
            "sso-attribute-key": {"no_log": true, "type": "str"},
            "sso-attribute-value-override": {"choices": ["disable", "enable"], "type": "str"},
            "tertiary-secret": {"no_log": true, "type": "raw"},
            "tertiary-server": {"type": "str"},
            "timeout": {"type": "int"},
            "use-group-for-profile": {"choices": ["disable", "enable"], "type": "str"},
            "use-management-vdom": {"choices": ["disable", "enable"], "type": "str"},
            "username-case-sensitive": {"choices": ["disable", "enable"], "type": "str"},
            "interface": {"v_range": [["6.2.5", "6.2.12"], ["6.4.1", ""]], "type": "str"},
            "interface-select-method": {"v_range": [["6.2.5", "6.2.12"], ["6.4.1", ""]], "choices": ["auto", "sdwan", "specify"], "type": "str"},
            "group-override-attr-type": {"v_range": [["6.4.0", ""]], "choices": ["filter-Id", "class"], "type": "str"},
            "switch-controller-acct-fast-framedip-detect": {"v_range": [["6.4.0", ""]], "type": "int"},
            "accounting-server": {
                "v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]],
                "type": "list",
                "options": {
                    "id": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "type": "int"},
                    "interface": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "type": "str"},
                    "interface-select-method": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "choices": ["auto", "sdwan", "specify"], "type": "str"},
                    "port": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "type": "int"},
                    "secret": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "no_log": true, "type": "raw"},
                    "server": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "type": "str"},
                    "source-ip": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "type": "str"},
                    "status": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "choices": ["disable", "enable"], "type": "str"}
                },
                "elements": "dict"
            },
            "switch-controller-service-type": {
                "v_range": [["6.4.2", ""]],
                "type": "list",
                "choices": [
                    "login", "framed", "callback-login", "callback-framed", "outbound", "administrative", "nas-prompt", "authenticate-only",
                    "callback-nas-prompt", "call-check", "callback-administrative"
                ],
                "elements": "str"
            },
            "delimiter": {"v_range": [["7.2.0", ""]], "choices": ["plus", "comma"], "type": "str"},
            "mac-case": {"v_range": [["7.2.1", ""]], "choices": ["uppercase", "lowercase"], "type": "str"},
            "mac-password-delimiter": {"v_range": [["7.2.1", ""]], "choices": ["hyphen", "single-hyphen", "colon", "none"], "type": "str"},
            "mac-username-delimiter": {"v_range": [["7.2.1", ""]], "choices": ["hyphen", "single-hyphen", "colon", "none"], "type": "str"},
            "nas-id": {"v_range": [["7.2.2", ""]], "type": "str"},
            "nas-id-type": {"v_range": [["7.2.2", ""]], "choices": ["legacy", "custom", "hostname"], "type": "str"},
            "ca-cert": {"v_range": [["7.4.0", ""]], "type": "str"},
            "client-cert": {"v_range": [["7.4.0", ""]], "type": "str"},
            "server-identity-check": {"v_range": [["7.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "status-ttl": {"v_range": [["7.4.0", ""]], "type": "int"},
            "tls-min-proto-version": {
                "v_range": [["7.4.0", ""]],
                "choices": ["default", "TLSv1", "TLSv1-1", "TLSv1-2", "SSLv3", "TLSv1-3"],
                "type": "str"
            },
            "transport-protocol": {"v_range": [["7.4.0", ""]], "choices": ["udp", "tcp", "tls"], "type": "str"},
            "account-key-cert-field": {"v_range": [["7.4.1", ""]], "choices": ["othername", "rfc822name", "dnsname", "cn"], "type": "str"},
            "account-key-processing": {"v_range": [["7.4.1", ""]], "choices": ["same", "strip"], "type": "str"},
            "call-station-id-type": {"v_range": [["7.4.1", ""]], "choices": ["legacy", "IP", "MAC"], "type": "str"},
            "switch-controller-nas-ip-dynamic": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "source-ip-interface": {"v_range": [["7.6.0", ""]], "type": "raw"}
        }
    },
    "user_radius_dynamicmapping_accountingserver": {
        "stated": true,
        "options": {
            "id": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", "7.4.0"]], "required": true, "type": "int"},
            "interface": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", "7.4.0"]], "type": "str"},
            "interface-select-method": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", "7.4.0"]], "choices": ["auto", "sdwan", "specify"], "type": "str"},
            "port": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", "7.4.0"]], "type": "int"},
            "secret": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", "7.4.0"]], "no_log": true, "type": "raw"},
            "server": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", "7.4.0"]], "type": "str"},
            "source-ip": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", "7.4.0"]], "type": "str"},
            "status": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", "7.4.0"]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "user_saml": {
        "stated": true,
        "options": {
            "cert": {"v_range": [["6.4.0", ""]], "type": "str"},
            "entity-id": {"v_range": [["6.4.0", ""]], "type": "str"},
            "group-name": {"v_range": [["6.4.0", ""]], "type": "str"},
            "idp-cert": {"v_range": [["6.4.0", ""]], "type": "str"},
            "idp-entity-id": {"v_range": [["6.4.0", ""]], "type": "str"},
            "idp-single-logout-url": {"v_range": [["6.4.0", ""]], "type": "str"},
            "idp-single-sign-on-url": {"v_range": [["6.4.0", ""]], "type": "str"},
            "name": {"v_range": [["6.4.0", ""]], "required": true, "type": "str"},
            "single-logout-url": {"v_range": [["6.4.0", ""]], "type": "str"},
            "single-sign-on-url": {"v_range": [["6.4.0", ""]], "type": "str"},
            "user-name": {"v_range": [["6.4.0", ""]], "type": "str"},
            "adfs-claim": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "digest-method": {"v_range": [["7.0.0", ""]], "choices": ["sha1", "sha256"], "type": "str"},
            "group-claim-type": {
                "v_range": [["7.0.0", ""]],
                "choices": [
                    "email", "given-name", "name", "upn", "common-name", "email-adfs-1x", "group", "upn-adfs-1x", "role", "sur-name", "ppid",
                    "name-identifier", "authentication-method", "deny-only-group-sid", "deny-only-primary-sid", "deny-only-primary-group-sid",
                    "group-sid", "primary-group-sid", "primary-sid", "windows-account-name"
                ],
                "type": "str"
            },
            "limit-relaystate": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "user-claim-type": {
                "v_range": [["7.0.0", ""]],
                "choices": [
                    "email", "given-name", "name", "upn", "common-name", "email-adfs-1x", "group", "upn-adfs-1x", "role", "sur-name", "ppid",
                    "name-identifier", "authentication-method", "deny-only-group-sid", "deny-only-primary-sid", "deny-only-primary-group-sid",
                    "group-sid", "primary-group-sid", "primary-sid", "windows-account-name"
                ],
                "type": "str"
            },
            "clock-tolerance": {"v_range": [["7.0.3", ""]], "type": "int"},
            "dynamic_mapping": {
                "v_range": [["7.0.5", "7.0.12"], ["7.2.1", ""]],
                "type": "list",
                "options": {
                    "_scope": {
                        "v_range": [["7.0.5", "7.0.12"], ["7.2.1", ""]],
                        "type": "list",
                        "options": {
                            "name": {"v_range": [["7.0.5", "7.0.12"], ["7.2.1", ""]], "type": "str"},
                            "vdom": {"v_range": [["7.0.5", "7.0.12"], ["7.2.1", ""]], "type": "str"}
                        },
                        "elements": "dict"
                    },
                    "adfs-claim": {"v_range": [["7.0.5", "7.0.12"], ["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "cert": {"v_range": [["7.0.5", "7.0.12"], ["7.2.1", ""]], "type": "str"},
                    "clock-tolerance": {"v_range": [["7.0.5", "7.0.12"], ["7.2.1", ""]], "type": "int"},
                    "digest-method": {"v_range": [["7.0.5", "7.0.12"], ["7.2.1", ""]], "choices": ["sha1", "sha256"], "type": "str"},
                    "entity-id": {"v_range": [["7.0.5", "7.0.12"], ["7.2.1", ""]], "type": "str"},
                    "group-claim-type": {
                        "v_range": [["7.0.5", "7.0.12"], ["7.2.1", ""]],
                        "choices": [
                            "email", "given-name", "name", "upn", "common-name", "email-adfs-1x", "group", "upn-adfs-1x", "role", "sur-name", "ppid",
                            "name-identifier", "authentication-method", "deny-only-group-sid", "deny-only-primary-sid", "deny-only-primary-group-sid",
                            "group-sid", "primary-group-sid", "primary-sid", "windows-account-name"
                        ],
                        "type": "str"
                    },
                    "group-name": {"v_range": [["7.0.5", "7.0.12"], ["7.2.1", ""]], "type": "str"},
                    "idp-cert": {"v_range": [["7.0.5", "7.0.12"], ["7.2.1", ""]], "type": "str"},
                    "idp-entity-id": {"v_range": [["7.0.5", "7.0.12"], ["7.2.1", ""]], "type": "str"},
                    "idp-single-logout-url": {"v_range": [["7.0.5", "7.0.12"], ["7.2.1", ""]], "type": "str"},
                    "idp-single-sign-on-url": {"v_range": [["7.0.5", "7.0.12"], ["7.2.1", ""]], "type": "str"},
                    "limit-relaystate": {"v_range": [["7.0.5", "7.0.12"], ["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "single-logout-url": {"v_range": [["7.0.5", "7.0.12"], ["7.2.1", ""]], "type": "str"},
                    "single-sign-on-url": {"v_range": [["7.0.5", "7.0.12"], ["7.2.1", ""]], "type": "str"},
                    "user-claim-type": {
                        "v_range": [["7.0.5", "7.0.12"], ["7.2.1", ""]],
                        "choices": [
                            "email", "given-name", "name", "upn", "common-name", "email-adfs-1x", "group", "upn-adfs-1x", "role", "sur-name", "ppid",
                            "name-identifier", "authentication-method", "deny-only-group-sid", "deny-only-primary-sid", "deny-only-primary-group-sid",
                            "group-sid", "primary-group-sid", "primary-sid", "windows-account-name"
                        ],
                        "type": "str"
                    },
                    "user-name": {"v_range": [["7.0.5", "7.0.12"], ["7.2.1", ""]], "type": "str"},
                    "auth-url": {"v_range": [["7.2.1", ""]], "type": "str"},
                    "reauth": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "scim-client": {"v_range": [["7.6.0", ""]], "type": "raw"}
                },
                "elements": "dict"
            },
            "auth-url": {"v_range": [["7.2.1", ""]], "type": "str"},
            "reauth": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "scim-client": {"v_range": [["7.6.0", ""]], "type": "raw"}
        }
    },
    "user_saml_dynamicmapping": {
        "stated": true,
        "options": {
            "_scope": {
                "v_range": [["7.0.5", "7.0.12"], ["7.2.1", ""]],
                "type": "list",
                "options": {
                    "name": {"v_range": [["7.0.5", "7.0.12"], ["7.2.1", ""]], "type": "str"},
                    "vdom": {"v_range": [["7.0.5", "7.0.12"], ["7.2.1", ""]], "type": "str"}
                },
                "elements": "dict"
            },
            "adfs-claim": {"v_range": [["7.0.5", "7.0.12"], ["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "cert": {"v_range": [["7.0.5", "7.0.12"], ["7.2.1", ""]], "type": "str"},
            "clock-tolerance": {"v_range": [["7.0.5", "7.0.12"], ["7.2.1", ""]], "type": "int"},
            "digest-method": {"v_range": [["7.0.5", "7.0.12"], ["7.2.1", ""]], "choices": ["sha1", "sha256"], "type": "str"},
            "entity-id": {"v_range": [["7.0.5", "7.0.12"], ["7.2.1", ""]], "type": "str"},
            "group-claim-type": {
                "v_range": [["7.0.5", "7.0.12"], ["7.2.1", ""]],
                "choices": [
                    "email", "given-name", "name", "upn", "common-name", "email-adfs-1x", "group", "upn-adfs-1x", "role", "sur-name", "ppid",
                    "name-identifier", "authentication-method", "deny-only-group-sid", "deny-only-primary-sid", "deny-only-primary-group-sid",
                    "group-sid", "primary-group-sid", "primary-sid", "windows-account-name"
                ],
                "type": "str"
            },
            "group-name": {"v_range": [["7.0.5", "7.0.12"], ["7.2.1", ""]], "type": "str"},
            "idp-cert": {"v_range": [["7.0.5", "7.0.12"], ["7.2.1", ""]], "type": "str"},
            "idp-entity-id": {"v_range": [["7.0.5", "7.0.12"], ["7.2.1", ""]], "type": "str"},
            "idp-single-logout-url": {"v_range": [["7.0.5", "7.0.12"], ["7.2.1", ""]], "type": "str"},
            "idp-single-sign-on-url": {"v_range": [["7.0.5", "7.0.12"], ["7.2.1", ""]], "type": "str"},
            "limit-relaystate": {"v_range": [["7.0.5", "7.0.12"], ["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "single-logout-url": {"v_range": [["7.0.5", "7.0.12"], ["7.2.1", ""]], "type": "str"},
            "single-sign-on-url": {"v_range": [["7.0.5", "7.0.12"], ["7.2.1", ""]], "type": "str"},
            "user-claim-type": {
                "v_range": [["7.0.5", "7.0.12"], ["7.2.1", ""]],
                "choices": [
                    "email", "given-name", "name", "upn", "common-name", "email-adfs-1x", "group", "upn-adfs-1x", "role", "sur-name", "ppid",
                    "name-identifier", "authentication-method", "deny-only-group-sid", "deny-only-primary-sid", "deny-only-primary-group-sid",
                    "group-sid", "primary-group-sid", "primary-sid", "windows-account-name"
                ],
                "type": "str"
            },
            "user-name": {"v_range": [["7.0.5", "7.0.12"], ["7.2.1", ""]], "type": "str"},
            "auth-url": {"v_range": [["7.2.1", ""]], "type": "str"},
            "reauth": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "scim-client": {"v_range": [["7.6.0", ""]], "type": "raw"}
        }
    },
    "user_securityexemptlist": {
        "stated": true,
        "options": {
            "description": {"type": "str"},
            "name": {"required": true, "type": "str"},
            "rule": {
                "type": "list",
                "options": {
                    "devices": {"v_range": [["6.0.0", "7.2.1"]], "type": "raw"},
                    "dstaddr": {"type": "raw"},
                    "id": {"type": "int"},
                    "service": {"type": "raw"},
                    "srcaddr": {"type": "raw"}
                },
                "elements": "dict"
            }
        }
    },
    "user_securityexemptlist_rule": {
        "stated": true,
        "options": {
            "devices": {"v_range": [["6.0.0", "7.2.1"]], "type": "raw"},
            "dstaddr": {"type": "raw"},
            "id": {"required": true, "type": "int"},
            "service": {"type": "raw"},
            "srcaddr": {"type": "raw"}
        }
    },
    "user_tacacs": {
        "stated": true,
        "options": {
            "authen-type": {"choices": ["auto", "ascii", "pap", "chap", "mschap"], "type": "str"},
            "authorization": {"choices": ["disable", "enable"], "type": "str"},
            "dynamic_mapping": {
                "type": "list",
                "options": {
                    "_scope": {"type": "list", "options": {"name": {"type": "str"}, "vdom": {"type": "str"}}, "elements": "dict"},
                    "authen-type": {"choices": ["auto", "ascii", "pap", "chap", "mschap"], "type": "str"},
                    "authorization": {"choices": ["disable", "enable"], "type": "str"},
                    "key": {"no_log": true, "type": "raw"},
                    "port": {"type": "int"},
                    "secondary-key": {"no_log": true, "type": "raw"},
                    "secondary-server": {"type": "str"},
                    "server": {"type": "str"},
                    "source-ip": {"type": "str"},
                    "tertiary-key": {"no_log": true, "type": "raw"},
                    "tertiary-server": {"type": "str"},
                    "interface": {"v_range": [["6.2.5", "6.2.12"], ["6.4.1", ""]], "type": "str"},
                    "interface-select-method": {"v_range": [["6.2.5", "6.2.12"], ["6.4.1", ""]], "choices": ["auto", "sdwan", "specify"], "type": "str"},
                    "status-ttl": {"v_range": [["7.4.3", ""]], "type": "int"}
                },
                "elements": "dict"
            },
            "key": {"no_log": true, "type": "raw"},
            "name": {"required": true, "type": "str"},
            "port": {"type": "int"},
            "secondary-key": {"no_log": true, "type": "raw"},
            "secondary-server": {"type": "str"},
            "server": {"type": "str"},
            "source-ip": {"type": "str"},
            "tertiary-key": {"no_log": true, "type": "raw"},
            "tertiary-server": {"type": "str"},
            "interface": {"v_range": [["6.2.5", "6.2.12"], ["6.4.1", ""]], "type": "str"},
            "interface-select-method": {"v_range": [["6.2.5", "6.2.12"], ["6.4.1", ""]], "choices": ["auto", "sdwan", "specify"], "type": "str"},
            "status-ttl": {"v_range": [["7.4.3", ""]], "type": "int"}
        }
    },
    "user_tacacs_dynamicmapping": {
        "stated": true,
        "options": {
            "_scope": {"type": "list", "options": {"name": {"type": "str"}, "vdom": {"type": "str"}}, "elements": "dict"},
            "authen-type": {"choices": ["auto", "ascii", "pap", "chap", "mschap"], "type": "str"},
            "authorization": {"choices": ["disable", "enable"], "type": "str"},
            "key": {"no_log": true, "type": "raw"},
            "port": {"type": "int"},
            "secondary-key": {"no_log": true, "type": "raw"},
            "secondary-server": {"type": "str"},
            "server": {"type": "str"},
            "source-ip": {"type": "str"},
            "tertiary-key": {"no_log": true, "type": "raw"},
            "tertiary-server": {"type": "str"},
            "interface": {"v_range": [["6.2.5", "6.2.12"], ["6.4.1", ""]], "type": "str"},
            "interface-select-method": {"v_range": [["6.2.5", "6.2.12"], ["6.4.1", ""]], "choices": ["auto", "sdwan", "specify"], "type": "str"},
            "status-ttl": {"v_range": [["7.4.3", ""]], "type": "int"}
        }
    },
    "user_vcenter": {
        "stated": true,
        "options": {
            "name": {"v_range": [["6.4.0", ""]], "required": true, "type": "str"},
            "password": {"v_range": [["6.4.0", ""]], "no_log": true, "type": "raw"},
            "rule": {
                "v_range": [["6.4.0", ""]],
                "type": "list",
                "options": {"name": {"v_range": [["6.4.0", ""]], "type": "str"}, "rule": {"v_range": [["6.4.0", ""]], "type": "str"}},
                "elements": "dict"
            },
            "server": {"v_range": [["6.4.0", ""]], "type": "str"},
            "status": {"v_range": [["6.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "upd_interval": {"v_range": [["6.4.0", ""]], "type": "int"},
            "user": {"v_range": [["6.4.0", ""]], "type": "str"}
        }
    },
    "user_vcenter_rule": {
        "stated": true,
        "options": {"name": {"v_range": [["6.4.0", ""]], "required": true, "type": "str"}, "rule": {"v_range": [["6.4.0", ""]], "type": "str"}}
    },
    "utmprofile": {
        "stated": true,
        "options": {
            "antivirus-profile": {"v_range": [["6.2.2", ""]], "type": "str"},
            "application-list": {"v_range": [["6.2.2", ""]], "type": "str"},
            "comment": {"v_range": [["6.2.2", ""]], "type": "str"},
            "ips-sensor": {"v_range": [["6.2.2", ""]], "type": "str"},
            "name": {"v_range": [["6.2.2", ""]], "required": true, "type": "str"},
            "scan-botnet-connections": {"v_range": [["6.2.2", ""]], "choices": ["disable", "block", "monitor"], "type": "str"},
            "utm-log": {"v_range": [["6.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "webfilter-profile": {"v_range": [["6.2.2", ""]], "type": "str"}
        }
    },
    "vap": {
        "stated": true,
        "options": {
            "_centmgmt": {"choices": ["disable", "enable"], "type": "str"},
            "_dhcp_svr_id": {"type": "str"},
            "_intf_allowaccess": {
                "type": "list",
                "choices": [
                    "https", "ping", "ssh", "snmp", "http", "telnet", "fgfm", "auto-ipsec", "radius-acct", "probe-response", "capwap", "dnp", "ftm",
                    "fabric", "speed-test"
                ],
                "elements": "str"
            },
            "_intf_device-identification": {"choices": ["disable", "enable"], "type": "str"},
            "_intf_device-netscan": {"choices": ["disable", "enable"], "type": "str"},
            "_intf_dhcp-relay-ip": {"type": "raw"},
            "_intf_dhcp-relay-service": {"choices": ["disable", "enable"], "type": "str"},
            "_intf_dhcp-relay-type": {"choices": ["regular", "ipsec"], "type": "str"},
            "_intf_dhcp6-relay-ip": {"type": "str"},
            "_intf_dhcp6-relay-service": {"choices": ["disable", "enable"], "type": "str"},
            "_intf_dhcp6-relay-type": {"choices": ["regular"], "type": "str"},
            "_intf_ip": {"type": "str"},
            "_intf_ip6-address": {"type": "str"},
            "_intf_ip6-allowaccess": {
                "type": "list",
                "choices": ["https", "ping", "ssh", "snmp", "http", "telnet", "any", "fgfm", "capwap"],
                "elements": "str"
            },
            "_intf_listen-forticlient-connection": {"choices": ["disable", "enable"], "type": "str"},
            "acct-interim-interval": {"type": "int"},
            "alias": {"type": "str"},
            "auth": {"choices": ["PSK", "psk", "RADIUS", "radius", "usergroup"], "type": "str"},
            "broadcast-ssid": {"choices": ["disable", "enable"], "type": "str"},
            "broadcast-suppression": {
                "type": "list",
                "choices": [
                    "dhcp", "arp", "dhcp2", "arp2", "netbios-ns", "netbios-ds", "arp3", "dhcp-up", "dhcp-down", "arp-known", "arp-unknown", "arp-reply",
                    "ipv6", "dhcp-starvation", "arp-poison", "all-other-mc", "all-other-bc", "arp-proxy", "dhcp-ucast"
                ],
                "elements": "str"
            },
            "captive-portal-ac-name": {"type": "str"},
            "captive-portal-macauth-radius-secret": {"no_log": true, "type": "raw"},
            "captive-portal-macauth-radius-server": {"type": "str"},
            "captive-portal-radius-secret": {"no_log": true, "type": "raw"},
            "captive-portal-radius-server": {"type": "str"},
            "captive-portal-session-timeout-interval": {"type": "int"},
            "dhcp-lease-time": {"type": "int"},
            "dhcp-option82-circuit-id-insertion": {"choices": ["disable", "style-1", "style-2", "style-3"], "type": "str"},
            "dhcp-option82-insertion": {"choices": ["disable", "enable"], "type": "str"},
            "dhcp-option82-remote-id-insertion": {"choices": ["disable", "style-1"], "type": "str"},
            "dynamic-vlan": {"choices": ["disable", "enable"], "type": "str"},
            "dynamic_mapping": {
                "type": "list",
                "options": {
                    "_centmgmt": {"choices": ["disable", "enable"], "type": "str"},
                    "_dhcp_svr_id": {"type": "str"},
                    "_intf_allowaccess": {
                        "type": "list",
                        "choices": [
                            "https", "ping", "ssh", "snmp", "http", "telnet", "fgfm", "auto-ipsec", "radius-acct", "probe-response", "capwap", "dnp",
                            "ftm", "fabric", "speed-test"
                        ],
                        "elements": "str"
                    },
                    "_intf_device-identification": {"choices": ["disable", "enable"], "type": "str"},
                    "_intf_device-netscan": {"choices": ["disable", "enable"], "type": "str"},
                    "_intf_dhcp-relay-ip": {"type": "raw"},
                    "_intf_dhcp-relay-service": {"choices": ["disable", "enable"], "type": "str"},
                    "_intf_dhcp-relay-type": {"choices": ["regular", "ipsec"], "type": "str"},
                    "_intf_dhcp6-relay-ip": {"type": "str"},
                    "_intf_dhcp6-relay-service": {"choices": ["disable", "enable"], "type": "str"},
                    "_intf_dhcp6-relay-type": {"choices": ["regular"], "type": "str"},
                    "_intf_ip": {"type": "str"},
                    "_intf_ip6-address": {"type": "str"},
                    "_intf_ip6-allowaccess": {
                        "type": "list",
                        "choices": ["https", "ping", "ssh", "snmp", "http", "telnet", "any", "fgfm", "capwap"],
                        "elements": "str"
                    },
                    "_intf_listen-forticlient-connection": {"choices": ["disable", "enable"], "type": "str"},
                    "_scope": {"type": "list", "options": {"name": {"type": "str"}, "vdom": {"type": "str"}}, "elements": "dict"},
                    "acct-interim-interval": {"type": "int"},
                    "address-group": {"type": "str"},
                    "alias": {"type": "str"},
                    "atf-weight": {"type": "int"},
                    "auth": {"choices": ["PSK", "psk", "RADIUS", "radius", "usergroup"], "type": "str"},
                    "broadcast-ssid": {"choices": ["disable", "enable"], "type": "str"},
                    "broadcast-suppression": {
                        "type": "list",
                        "choices": [
                            "dhcp", "arp", "dhcp2", "arp2", "netbios-ns", "netbios-ds", "arp3", "dhcp-up", "dhcp-down", "arp-known", "arp-unknown",
                            "arp-reply", "ipv6", "dhcp-starvation", "arp-poison", "all-other-mc", "all-other-bc", "arp-proxy", "dhcp-ucast"
                        ],
                        "elements": "str"
                    },
                    "captive-portal-ac-name": {"type": "str"},
                    "captive-portal-macauth-radius-secret": {"no_log": true, "type": "raw"},
                    "captive-portal-macauth-radius-server": {"type": "str"},
                    "captive-portal-radius-secret": {"no_log": true, "type": "raw"},
                    "captive-portal-radius-server": {"type": "str"},
                    "captive-portal-session-timeout-interval": {"type": "int"},
                    "client-count": {"type": "int"},
                    "dhcp-lease-time": {"type": "int"},
                    "dhcp-option82-circuit-id-insertion": {"choices": ["disable", "style-1", "style-2", "style-3"], "type": "str"},
                    "dhcp-option82-insertion": {"choices": ["disable", "enable"], "type": "str"},
                    "dhcp-option82-remote-id-insertion": {"choices": ["disable", "style-1"], "type": "str"},
                    "dynamic-vlan": {"choices": ["disable", "enable"], "type": "str"},
                    "eap-reauth": {"choices": ["disable", "enable"], "type": "str"},
                    "eap-reauth-intv": {"type": "int"},
                    "eapol-key-retries": {"choices": ["disable", "enable"], "type": "str"},
                    "encrypt": {"choices": ["TKIP", "AES", "TKIP-AES"], "type": "str"},
                    "external-fast-roaming": {"choices": ["disable", "enable"], "type": "str"},
                    "external-logout": {"type": "str"},
                    "external-web": {"type": "str"},
                    "fast-bss-transition": {"choices": ["disable", "enable"], "type": "str"},
                    "fast-roaming": {"choices": ["disable", "enable"], "type": "str"},
                    "ft-mobility-domain": {"type": "int"},
                    "ft-over-ds": {"choices": ["disable", "enable"], "type": "str"},
                    "ft-r0-key-lifetime": {"no_log": true, "type": "int"},
                    "gtk-rekey": {"choices": ["disable", "enable"], "type": "str"},
                    "gtk-rekey-intv": {"no_log": true, "type": "int"},
                    "hotspot20-profile": {"type": "str"},
                    "intra-vap-privacy": {"choices": ["disable", "enable"], "type": "str"},
                    "ip": {"type": "str"},
                    "key": {"no_log": true, "type": "raw"},
                    "keyindex": {"no_log": true, "type": "int"},
                    "ldpc": {"choices": ["disable", "tx", "rx", "rxtx"], "type": "str"},
                    "local-authentication": {"choices": ["disable", "enable"], "type": "str"},
                    "local-bridging": {"choices": ["disable", "enable"], "type": "str"},
                    "local-lan": {"choices": ["deny", "allow"], "type": "str"},
                    "local-standalone": {"choices": ["disable", "enable"], "type": "str"},
                    "local-standalone-nat": {"choices": ["disable", "enable"], "type": "str"},
                    "local-switching": {"choices": ["disable", "enable"], "type": "str"},
                    "mac-auth-bypass": {"choices": ["disable", "enable"], "type": "str"},
                    "mac-filter": {"choices": ["disable", "enable"], "type": "str"},
                    "mac-filter-policy-other": {"choices": ["deny", "allow"], "type": "str"},
                    "max-clients": {"type": "int"},
                    "max-clients-ap": {"type": "int"},
                    "me-disable-thresh": {"type": "int"},
                    "mesh-backhaul": {"choices": ["disable", "enable"], "type": "str"},
                    "mpsk": {"choices": ["disable", "enable"], "type": "str"},
                    "mpsk-concurrent-clients": {"type": "int"},
                    "multicast-enhance": {"choices": ["disable", "enable"], "type": "str"},
                    "multicast-rate": {"choices": ["0", "6000", "12000", "24000"], "type": "str"},
                    "okc": {"choices": ["disable", "enable"], "type": "str"},
                    "owe-groups": {"type": "list", "choices": ["19", "20", "21"], "elements": "str"},
                    "owe-transition": {"choices": ["disable", "enable"], "type": "str"},
                    "owe-transition-ssid": {"type": "str"},
                    "passphrase": {"no_log": true, "type": "raw"},
                    "pmf": {"choices": ["disable", "enable", "optional"], "type": "str"},
                    "pmf-assoc-comeback-timeout": {"type": "int"},
                    "pmf-sa-query-retry-timeout": {"type": "int"},
                    "portal-message-override-group": {"type": "str"},
                    "portal-type": {
                        "choices": [
                            "auth", "auth+disclaimer", "disclaimer", "email-collect", "cmcc", "cmcc-macauth", "auth-mac", "external-auth",
                            "external-macauth"
                        ],
                        "type": "str"
                    },
                    "probe-resp-suppression": {"choices": ["disable", "enable"], "type": "str"},
                    "probe-resp-threshold": {"type": "str"},
                    "ptk-rekey": {"choices": ["disable", "enable"], "type": "str"},
                    "ptk-rekey-intv": {"no_log": true, "type": "int"},
                    "qos-profile": {"type": "str"},
                    "quarantine": {"choices": ["disable", "enable"], "type": "str"},
                    "radio-2g-threshold": {"type": "str"},
                    "radio-5g-threshold": {"type": "str"},
                    "radio-sensitivity": {"choices": ["disable", "enable"], "type": "str"},
                    "radius-mac-auth": {"choices": ["disable", "enable"], "type": "str"},
                    "radius-mac-auth-server": {"type": "str"},
                    "radius-mac-auth-usergroups": {"type": "raw"},
                    "radius-server": {"type": "str"},
                    "rates-11a": {
                        "type": "list",
                        "choices": [
                            "1", "1-basic", "2", "2-basic", "5.5", "5.5-basic", "6", "6-basic", "9", "9-basic", "12", "12-basic", "18", "18-basic", "24",
                            "24-basic", "36", "36-basic", "48", "48-basic", "54", "54-basic", "11", "11-basic"
                        ],
                        "elements": "str"
                    },
                    "rates-11ac-ss12": {
                        "type": "list",
                        "choices": [
                            "mcs0/1", "mcs1/1", "mcs2/1", "mcs3/1", "mcs4/1", "mcs5/1", "mcs6/1", "mcs7/1", "mcs8/1", "mcs9/1", "mcs0/2", "mcs1/2",
                            "mcs2/2", "mcs3/2", "mcs4/2", "mcs5/2", "mcs6/2", "mcs7/2", "mcs8/2", "mcs9/2", "mcs10/1", "mcs11/1", "mcs10/2", "mcs11/2"
                        ],
                        "elements": "str"
                    },
                    "rates-11ac-ss34": {
                        "type": "list",
                        "choices": [
                            "mcs0/3", "mcs1/3", "mcs2/3", "mcs3/3", "mcs4/3", "mcs5/3", "mcs6/3", "mcs7/3", "mcs8/3", "mcs9/3", "mcs0/4", "mcs1/4",
                            "mcs2/4", "mcs3/4", "mcs4/4", "mcs5/4", "mcs6/4", "mcs7/4", "mcs8/4", "mcs9/4", "mcs10/3", "mcs11/3", "mcs10/4", "mcs11/4"
                        ],
                        "elements": "str"
                    },
                    "rates-11bg": {
                        "type": "list",
                        "choices": [
                            "1", "1-basic", "2", "2-basic", "5.5", "5.5-basic", "6", "6-basic", "9", "9-basic", "12", "12-basic", "18", "18-basic", "24",
                            "24-basic", "36", "36-basic", "48", "48-basic", "54", "54-basic", "11", "11-basic"
                        ],
                        "elements": "str"
                    },
                    "rates-11n-ss12": {
                        "type": "list",
                        "choices": [
                            "mcs0/1", "mcs1/1", "mcs2/1", "mcs3/1", "mcs4/1", "mcs5/1", "mcs6/1", "mcs7/1", "mcs8/2", "mcs9/2", "mcs10/2", "mcs11/2",
                            "mcs12/2", "mcs13/2", "mcs14/2", "mcs15/2"
                        ],
                        "elements": "str"
                    },
                    "rates-11n-ss34": {
                        "type": "list",
                        "choices": [
                            "mcs16/3", "mcs17/3", "mcs18/3", "mcs19/3", "mcs20/3", "mcs21/3", "mcs22/3", "mcs23/3", "mcs24/4", "mcs25/4", "mcs26/4",
                            "mcs27/4", "mcs28/4", "mcs29/4", "mcs30/4", "mcs31/4"
                        ],
                        "elements": "str"
                    },
                    "sae-groups": {
                        "type": "list",
                        "choices": ["1", "2", "5", "14", "15", "16", "17", "18", "19", "20", "21", "27", "28", "29", "30", "31"],
                        "elements": "str"
                    },
                    "sae-password": {"no_log": true, "type": "raw"},
                    "schedule": {"type": "raw"},
                    "security": {
                        "choices": [
                            "None", "WEP64", "wep64", "WEP128", "wep128", "WPA_PSK", "WPA_RADIUS", "WPA", "WPA2", "WPA2_AUTO", "open", "wpa-personal",
                            "wpa-enterprise", "captive-portal", "wpa-only-personal", "wpa-only-enterprise", "wpa2-only-personal", "wpa2-only-enterprise",
                            "wpa-personal+captive-portal", "wpa-only-personal+captive-portal", "wpa2-only-personal+captive-portal", "osen",
                            "wpa3-enterprise", "sae", "sae-transition", "owe", "wpa3-sae", "wpa3-sae-transition", "wpa3-only-enterprise",
                            "wpa3-enterprise-transition"
                        ],
                        "type": "str"
                    },
                    "security-exempt-list": {"type": "str"},
                    "security-obsolete-option": {"choices": ["disable", "enable"], "type": "str"},
                    "security-redirect-url": {"type": "str"},
                    "selected-usergroups": {"type": "raw"},
                    "split-tunneling": {"choices": ["disable", "enable"], "type": "str"},
                    "ssid": {"type": "str"},
                    "tkip-counter-measure": {"choices": ["disable", "enable"], "type": "str"},
                    "usergroup": {"type": "raw"},
                    "utm-profile": {"type": "str"},
                    "vdom": {"type": "raw"},
                    "vlan-auto": {"choices": ["disable", "enable"], "type": "str"},
                    "vlan-pooling": {"choices": ["wtp-group", "round-robin", "hash", "disable"], "type": "str"},
                    "vlanid": {"type": "int"},
                    "voice-enterprise": {"choices": ["disable", "enable"], "type": "str"},
                    "mu-mimo": {"v_range": [["6.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "_intf_device-access-list": {"v_range": [["6.2.2", ""]], "type": "str"},
                    "external-web-format": {
                        "v_range": [["6.2.2", ""]],
                        "choices": ["auto-detect", "no-query-string", "partial-query-string"],
                        "type": "str"
                    },
                    "high-efficiency": {"v_range": [["6.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "primary-wag-profile": {"v_range": [["6.2.2", ""]], "type": "str"},
                    "secondary-wag-profile": {"v_range": [["6.2.2", ""]], "type": "str"},
                    "target-wake-time": {"v_range": [["6.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "tunnel-echo-interval": {"v_range": [["6.2.2", ""]], "type": "int"},
                    "tunnel-fallback-interval": {"v_range": [["6.2.2", ""]], "type": "int"},
                    "access-control-list": {"v_range": [["6.4.0", ""]], "type": "str"},
                    "captive-portal-auth-timeout": {"v_range": [["6.4.0", ""]], "type": "int"},
                    "ipv6-rules": {
                        "v_range": [["6.4.0", ""]],
                        "type": "list",
                        "choices": [
                            "drop-icmp6ra", "drop-icmp6rs", "drop-llmnr6", "drop-icmp6mld2", "drop-dhcp6s", "drop-dhcp6c", "ndp-proxy", "drop-ns-dad",
                            "drop-ns-nondad"
                        ],
                        "elements": "str"
                    },
                    "sticky-client-remove": {"v_range": [["6.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "sticky-client-threshold-2g": {"v_range": [["6.4.0", ""]], "type": "str"},
                    "sticky-client-threshold-5g": {"v_range": [["6.4.0", ""]], "type": "str"},
                    "bss-color-partial": {"v_range": [["6.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "dhcp-option43-insertion": {"v_range": [["6.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "mpsk-profile": {"v_range": [["6.4.2", ""]], "type": "str"},
                    "igmp-snooping": {"v_range": [["6.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "port-macauth": {"v_range": [["6.2.8", "6.2.12"], ["6.4.3", ""]], "choices": ["disable", "radius", "address-group"], "type": "str"},
                    "port-macauth-reauth-timeout": {"v_range": [["6.2.8", "6.2.12"], ["6.4.3", ""]], "type": "int"},
                    "port-macauth-timeout": {"v_range": [["6.2.8", "6.2.12"], ["6.4.3", ""]], "type": "int"},
                    "additional-akms": {"v_range": [["7.0.0", ""]], "type": "list", "choices": ["akm6", "akm24"], "elements": "str"},
                    "bstm-disassociation-imminent": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "bstm-load-balancing-disassoc-timer": {"v_range": [["7.0.0", ""]], "type": "int"},
                    "bstm-rssi-disassoc-timer": {"v_range": [["7.0.0", ""]], "type": "int"},
                    "dhcp-address-enforcement": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "gas-comeback-delay": {"v_range": [["7.0.0", ""]], "type": "int"},
                    "gas-fragmentation-limit": {"v_range": [["7.0.0", ""]], "type": "int"},
                    "mac-called-station-delimiter": {"v_range": [["7.0.0", ""]], "choices": ["hyphen", "single-hyphen", "colon", "none"], "type": "str"},
                    "mac-calling-station-delimiter": {"v_range": [["7.0.0", ""]], "choices": ["hyphen", "single-hyphen", "colon", "none"], "type": "str"},
                    "mac-case": {"v_range": [["7.0.0", ""]], "choices": ["uppercase", "lowercase"], "type": "str"},
                    "mac-password-delimiter": {"v_range": [["7.0.0", ""]], "choices": ["hyphen", "single-hyphen", "colon", "none"], "type": "str"},
                    "mac-username-delimiter": {"v_range": [["7.0.0", ""]], "choices": ["hyphen", "single-hyphen", "colon", "none"], "type": "str"},
                    "mbo": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "mbo-cell-data-conn-pref": {"v_range": [["7.0.0", ""]], "choices": ["excluded", "prefer-not", "prefer-use"], "type": "str"},
                    "nac": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "nac-profile": {"v_range": [["7.0.0", ""]], "type": "str"},
                    "neighbor-report-dual-band": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "address-group-policy": {"v_range": [["7.2.0", ""]], "choices": ["disable", "allow", "deny"], "type": "str"},
                    "antivirus-profile": {"v_range": [["7.0.1", ""]], "type": "str"},
                    "application-detection-engine": {"v_range": [["7.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "application-list": {"v_range": [["7.0.1", ""]], "type": "str"},
                    "application-report-intv": {"v_range": [["7.2.0", ""]], "type": "int"},
                    "auth-cert": {"v_range": [["7.0.3", ""]], "type": "str"},
                    "auth-portal-addr": {"v_range": [["7.0.3", ""]], "type": "str"},
                    "beacon-advertising": {"v_range": [["7.0.2", ""]], "type": "list", "choices": ["name", "model", "serial-number"], "elements": "str"},
                    "ips-sensor": {"v_range": [["7.0.1", ""]], "type": "str"},
                    "l3-roaming": {"v_range": [["7.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "local-standalone-dns": {"v_range": [["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "local-standalone-dns-ip": {"v_range": [["7.0.1", ""]], "type": "raw"},
                    "osen": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "radius-mac-mpsk-auth": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "radius-mac-mpsk-timeout": {"v_range": [["7.0.2", ""]], "type": "int"},
                    "rates-11ax-ss12": {
                        "v_range": [["7.0.2", ""]],
                        "type": "list",
                        "choices": [
                            "mcs0/1", "mcs1/1", "mcs2/1", "mcs3/1", "mcs4/1", "mcs5/1", "mcs6/1", "mcs7/1", "mcs8/1", "mcs9/1", "mcs10/1", "mcs11/1",
                            "mcs0/2", "mcs1/2", "mcs2/2", "mcs3/2", "mcs4/2", "mcs5/2", "mcs6/2", "mcs7/2", "mcs8/2", "mcs9/2", "mcs10/2", "mcs11/2"
                        ],
                        "elements": "str"
                    },
                    "rates-11ax-ss34": {
                        "v_range": [["7.0.2", ""]],
                        "type": "list",
                        "choices": [
                            "mcs0/3", "mcs1/3", "mcs2/3", "mcs3/3", "mcs4/3", "mcs5/3", "mcs6/3", "mcs7/3", "mcs8/3", "mcs9/3", "mcs10/3", "mcs11/3",
                            "mcs0/4", "mcs1/4", "mcs2/4", "mcs3/4", "mcs4/4", "mcs5/4", "mcs6/4", "mcs7/4", "mcs8/4", "mcs9/4", "mcs10/4", "mcs11/4"
                        ],
                        "elements": "str"
                    },
                    "scan-botnet-connections": {"v_range": [["7.0.1", ""]], "choices": ["disable", "block", "monitor"], "type": "str"},
                    "utm-log": {"v_range": [["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "utm-status": {"v_range": [["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "webfilter-profile": {"v_range": [["7.0.1", ""]], "type": "str"},
                    "sae-h2e-only": {"v_range": [["7.0.5", "7.0.12"], ["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "sae-pk": {"v_range": [["7.0.5", "7.0.12"], ["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "sae-private-key": {"v_range": [["7.0.5", "7.0.12"], ["7.2.1", ""]], "no_log": true, "type": "str"},
                    "sticky-client-threshold-6g": {"v_range": [["7.0.5", "7.0.12"], ["7.2.1", ""]], "type": "str"},
                    "application-dscp-marking": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "l3-roaming-mode": {"v_range": [["7.2.1", ""]], "choices": ["direct", "indirect"], "type": "str"},
                    "rates-11ac-mcs-map": {"v_range": [["7.2.1", ""]], "type": "str"},
                    "rates-11ax-mcs-map": {"v_range": [["7.2.1", ""]], "type": "str"},
                    "captive-portal-fw-accounting": {"v_range": [["7.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "radius-mac-auth-block-interval": {"v_range": [["7.2.2", ""]], "type": "int"},
                    "_is_factory_setting": {"v_range": [["7.4.0", ""]], "choices": ["disable", "enable", "ext"], "type": "str"},
                    "80211k": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "80211v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "roaming-acct-interim-update": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "sae-hnp-only": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "akm24-only": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "beacon-protection": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "captive-portal": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "nas-filter-rule": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "rates-11be-mcs-map": {"v_range": [["7.4.3", ""]], "type": "str"},
                    "rates-11be-mcs-map-160": {"v_range": [["7.4.3", ""]], "type": "str"},
                    "rates-11be-mcs-map-320": {"v_range": [["7.4.3", ""]], "type": "str"},
                    "_intf_ip-managed-by-fortiipam": {"v_range": [["7.6.0", ""]], "choices": ["disable", "enable", "inherit-global"], "type": "str"},
                    "_intf_managed-subnetwork-size": {
                        "v_range": [["7.6.0", ""]],
                        "choices": ["32", "64", "128", "256", "512", "1024", "2048", "4096", "8192", "16384", "32768", "65536"],
                        "type": "str"
                    },
                    "domain-name-stripping": {"v_range": [["7.6.0", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "local-lan-partition": {"v_range": [["7.6.0", ""]], "choices": ["disable", "enable"], "type": "str"}
                },
                "elements": "dict"
            },
            "eap-reauth": {"choices": ["disable", "enable"], "type": "str"},
            "eap-reauth-intv": {"type": "int"},
            "eapol-key-retries": {"choices": ["disable", "enable"], "type": "str"},
            "encrypt": {"choices": ["TKIP", "AES", "TKIP-AES"], "type": "str"},
            "external-fast-roaming": {"choices": ["disable", "enable"], "type": "str"},
            "external-logout": {"type": "str"},
            "external-web": {"type": "str"},
            "fast-bss-transition": {"choices": ["disable", "enable"], "type": "str"},
            "fast-roaming": {"choices": ["disable", "enable"], "type": "str"},
            "ft-mobility-domain": {"type": "int"},
            "ft-over-ds": {"choices": ["disable", "enable"], "type": "str"},
            "ft-r0-key-lifetime": {"no_log": true, "type": "int"},
            "gtk-rekey": {"choices": ["disable", "enable"], "type": "str"},
            "gtk-rekey-intv": {"no_log": true, "type": "int"},
            "hotspot20-profile": {"type": "str"},
            "intra-vap-privacy": {"choices": ["disable", "enable"], "type": "str"},
            "ip": {"type": "str"},
            "key": {"no_log": true, "type": "raw"},
            "keyindex": {"no_log": true, "type": "int"},
            "ldpc": {"choices": ["disable", "tx", "rx", "rxtx"], "type": "str"},
            "local-authentication": {"choices": ["disable", "enable"], "type": "str"},
            "local-bridging": {"choices": ["disable", "enable"], "type": "str"},
            "local-lan": {"choices": ["deny", "allow"], "type": "str"},
            "local-standalone": {"choices": ["disable", "enable"], "type": "str"},
            "local-standalone-nat": {"choices": ["disable", "enable"], "type": "str"},
            "mac-auth-bypass": {"choices": ["disable", "enable"], "type": "str"},
            "mac-filter": {"choices": ["disable", "enable"], "type": "str"},
            "mac-filter-list": {
                "type": "list",
                "options": {"id": {"type": "int"}, "mac": {"type": "str"}, "mac-filter-policy": {"choices": ["deny", "allow"], "type": "str"}},
                "elements": "dict"
            },
            "mac-filter-policy-other": {"choices": ["deny", "allow"], "type": "str"},
            "max-clients": {"type": "int"},
            "max-clients-ap": {"type": "int"},
            "me-disable-thresh": {"type": "int"},
            "mesh-backhaul": {"choices": ["disable", "enable"], "type": "str"},
            "mpsk": {"choices": ["disable", "enable"], "type": "str"},
            "mpsk-concurrent-clients": {"type": "int"},
            "mpsk-key": {
                "no_log": true,
                "type": "list",
                "options": {
                    "comment": {"type": "str"},
                    "concurrent-clients": {"type": "str"},
                    "key-name": {"no_log": true, "type": "str"},
                    "passphrase": {"no_log": true, "type": "raw"},
                    "mpsk-schedules": {"v_range": [["6.2.2", ""]], "type": "raw"}
                },
                "elements": "dict"
            },
            "multicast-enhance": {"choices": ["disable", "enable"], "type": "str"},
            "multicast-rate": {"choices": ["0", "6000", "12000", "24000"], "type": "str"},
            "name": {"required": true, "type": "str"},
            "okc": {"choices": ["disable", "enable"], "type": "str"},
            "passphrase": {"no_log": true, "type": "raw"},
            "pmf": {"choices": ["disable", "enable", "optional"], "type": "str"},
            "pmf-assoc-comeback-timeout": {"type": "int"},
            "pmf-sa-query-retry-timeout": {"type": "int"},
            "portal-message-override-group": {"type": "str"},
            "portal-type": {
                "choices": [
                    "auth", "auth+disclaimer", "disclaimer", "email-collect", "cmcc", "cmcc-macauth", "auth-mac", "external-auth", "external-macauth"
                ],
                "type": "str"
            },
            "probe-resp-suppression": {"choices": ["disable", "enable"], "type": "str"},
            "probe-resp-threshold": {"type": "str"},
            "ptk-rekey": {"choices": ["disable", "enable"], "type": "str"},
            "ptk-rekey-intv": {"no_log": true, "type": "int"},
            "qos-profile": {"type": "str"},
            "quarantine": {"choices": ["disable", "enable"], "type": "str"},
            "radio-2g-threshold": {"type": "str"},
            "radio-5g-threshold": {"type": "str"},
            "radio-sensitivity": {"choices": ["disable", "enable"], "type": "str"},
            "radius-mac-auth": {"choices": ["disable", "enable"], "type": "str"},
            "radius-mac-auth-server": {"type": "str"},
            "radius-mac-auth-usergroups": {"type": "raw"},
            "radius-server": {"type": "str"},
            "rates-11a": {
                "type": "list",
                "choices": [
                    "1", "1-basic", "2", "2-basic", "5.5", "5.5-basic", "6", "6-basic", "9", "9-basic", "12", "12-basic", "18", "18-basic", "24",
                    "24-basic", "36", "36-basic", "48", "48-basic", "54", "54-basic", "11", "11-basic"
                ],
                "elements": "str"
            },
            "rates-11ac-ss12": {
                "type": "list",
                "choices": [
                    "mcs0/1", "mcs1/1", "mcs2/1", "mcs3/1", "mcs4/1", "mcs5/1", "mcs6/1", "mcs7/1", "mcs8/1", "mcs9/1", "mcs0/2", "mcs1/2", "mcs2/2",
                    "mcs3/2", "mcs4/2", "mcs5/2", "mcs6/2", "mcs7/2", "mcs8/2", "mcs9/2", "mcs10/1", "mcs11/1", "mcs10/2", "mcs11/2"
                ],
                "elements": "str"
            },
            "rates-11ac-ss34": {
                "type": "list",
                "choices": [
                    "mcs0/3", "mcs1/3", "mcs2/3", "mcs3/3", "mcs4/3", "mcs5/3", "mcs6/3", "mcs7/3", "mcs8/3", "mcs9/3", "mcs0/4", "mcs1/4", "mcs2/4",
                    "mcs3/4", "mcs4/4", "mcs5/4", "mcs6/4", "mcs7/4", "mcs8/4", "mcs9/4", "mcs10/3", "mcs11/3", "mcs10/4", "mcs11/4"
                ],
                "elements": "str"
            },
            "rates-11bg": {
                "type": "list",
                "choices": [
                    "1", "1-basic", "2", "2-basic", "5.5", "5.5-basic", "6", "6-basic", "9", "9-basic", "12", "12-basic", "18", "18-basic", "24",
                    "24-basic", "36", "36-basic", "48", "48-basic", "54", "54-basic", "11", "11-basic"
                ],
                "elements": "str"
            },
            "rates-11n-ss12": {
                "type": "list",
                "choices": [
                    "mcs0/1", "mcs1/1", "mcs2/1", "mcs3/1", "mcs4/1", "mcs5/1", "mcs6/1", "mcs7/1", "mcs8/2", "mcs9/2", "mcs10/2", "mcs11/2", "mcs12/2",
                    "mcs13/2", "mcs14/2", "mcs15/2"
                ],
                "elements": "str"
            },
            "rates-11n-ss34": {
                "type": "list",
                "choices": [
                    "mcs16/3", "mcs17/3", "mcs18/3", "mcs19/3", "mcs20/3", "mcs21/3", "mcs22/3", "mcs23/3", "mcs24/4", "mcs25/4", "mcs26/4", "mcs27/4",
                    "mcs28/4", "mcs29/4", "mcs30/4", "mcs31/4"
                ],
                "elements": "str"
            },
            "schedule": {"type": "raw"},
            "security": {
                "choices": [
                    "None", "WEP64", "wep64", "WEP128", "wep128", "WPA_PSK", "WPA_RADIUS", "WPA", "WPA2", "WPA2_AUTO", "open", "wpa-personal",
                    "wpa-enterprise", "captive-portal", "wpa-only-personal", "wpa-only-enterprise", "wpa2-only-personal", "wpa2-only-enterprise",
                    "wpa-personal+captive-portal", "wpa-only-personal+captive-portal", "wpa2-only-personal+captive-portal", "osen", "wpa3-enterprise",
                    "sae", "sae-transition", "owe", "wpa3-sae", "wpa3-sae-transition", "wpa3-only-enterprise", "wpa3-enterprise-transition"
                ],
                "type": "str"
            },
            "security-exempt-list": {"type": "str"},
            "security-obsolete-option": {"choices": ["disable", "enable"], "type": "str"},
            "security-redirect-url": {"type": "str"},
            "selected-usergroups": {"type": "raw"},
            "split-tunneling": {"choices": ["disable", "enable"], "type": "str"},
            "ssid": {"type": "str"},
            "tkip-counter-measure": {"choices": ["disable", "enable"], "type": "str"},
            "usergroup": {"type": "raw"},
            "utm-profile": {"type": "str"},
            "vdom": {"v_range": [["6.0.0", "7.2.1"]], "type": "str"},
            "vlan-auto": {"choices": ["disable", "enable"], "type": "str"},
            "vlan-pool": {
                "type": "list",
                "options": {
                    "_wtp-group": {"type": "str"},
                    "id": {"type": "int"},
                    "wtp-group": {"v_range": [["6.0.0", "6.2.0"], ["7.4.3", ""]], "type": "str"}
                },
                "elements": "dict"
            },
            "vlan-pooling": {"choices": ["wtp-group", "round-robin", "hash", "disable"], "type": "str"},
            "vlanid": {"type": "int"},
            "voice-enterprise": {"choices": ["disable", "enable"], "type": "str"},
            "address-group": {"v_range": [["6.2.0", ""]], "type": "str"},
            "atf-weight": {"v_range": [["6.2.0", ""]], "type": "int"},
            "mu-mimo": {"v_range": [["6.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "owe-groups": {"v_range": [["6.2.0", ""]], "type": "list", "choices": ["19", "20", "21"], "elements": "str"},
            "owe-transition": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "owe-transition-ssid": {"v_range": [["6.2.0", ""]], "type": "str"},
            "sae-groups": {
                "v_range": [["6.2.0", ""]],
                "type": "list",
                "choices": ["1", "2", "5", "14", "15", "16", "17", "18", "19", "20", "21", "27", "28", "29", "30", "31"],
                "elements": "str"
            },
            "sae-password": {"v_range": [["6.2.0", ""]], "no_log": true, "type": "raw"},
            "_intf_device-access-list": {"v_range": [["6.2.2", ""]], "type": "str"},
            "external-web-format": {"v_range": [["6.2.2", ""]], "choices": ["auto-detect", "no-query-string", "partial-query-string"], "type": "str"},
            "high-efficiency": {"v_range": [["6.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "primary-wag-profile": {"v_range": [["6.2.2", ""]], "type": "str"},
            "secondary-wag-profile": {"v_range": [["6.2.2", ""]], "type": "str"},
            "target-wake-time": {"v_range": [["6.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "tunnel-echo-interval": {"v_range": [["6.2.2", ""]], "type": "int"},
            "tunnel-fallback-interval": {"v_range": [["6.2.2", ""]], "type": "int"},
            "access-control-list": {"v_range": [["6.4.0", ""]], "type": "str"},
            "captive-portal-auth-timeout": {"v_range": [["6.4.0", ""]], "type": "int"},
            "ipv6-rules": {
                "v_range": [["6.4.0", ""]],
                "type": "list",
                "choices": [
                    "drop-icmp6ra", "drop-icmp6rs", "drop-llmnr6", "drop-icmp6mld2", "drop-dhcp6s", "drop-dhcp6c", "ndp-proxy", "drop-ns-dad",
                    "drop-ns-nondad"
                ],
                "elements": "str"
            },
            "sticky-client-remove": {"v_range": [["6.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "sticky-client-threshold-2g": {"v_range": [["6.4.0", ""]], "type": "str"},
            "sticky-client-threshold-5g": {"v_range": [["6.4.0", ""]], "type": "str"},
            "bss-color-partial": {"v_range": [["6.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "dhcp-option43-insertion": {"v_range": [["6.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "mpsk-profile": {"v_range": [["6.4.2", ""]], "type": "str"},
            "igmp-snooping": {"v_range": [["6.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "port-macauth": {"v_range": [["6.2.8", "6.2.12"], ["6.4.3", ""]], "choices": ["disable", "radius", "address-group"], "type": "str"},
            "port-macauth-reauth-timeout": {"v_range": [["6.2.8", "6.2.12"], ["6.4.3", ""]], "type": "int"},
            "port-macauth-timeout": {"v_range": [["6.2.8", "6.2.12"], ["6.4.3", ""]], "type": "int"},
            "portal-message-overrides": {
                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                "type": "dict",
                "options": {
                    "auth-disclaimer-page": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "auth-login-failed-page": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "auth-login-page": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "auth-reject-page": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"}
                }
            },
            "additional-akms": {"v_range": [["7.0.0", ""]], "type": "list", "choices": ["akm6", "akm24"], "elements": "str"},
            "bstm-disassociation-imminent": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "bstm-load-balancing-disassoc-timer": {"v_range": [["7.0.0", ""]], "type": "int"},
            "bstm-rssi-disassoc-timer": {"v_range": [["7.0.0", ""]], "type": "int"},
            "dhcp-address-enforcement": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "gas-comeback-delay": {"v_range": [["7.0.0", ""]], "type": "int"},
            "gas-fragmentation-limit": {"v_range": [["7.0.0", ""]], "type": "int"},
            "mac-called-station-delimiter": {"v_range": [["7.0.0", ""]], "choices": ["hyphen", "single-hyphen", "colon", "none"], "type": "str"},
            "mac-calling-station-delimiter": {"v_range": [["7.0.0", ""]], "choices": ["hyphen", "single-hyphen", "colon", "none"], "type": "str"},
            "mac-case": {"v_range": [["7.0.0", ""]], "choices": ["uppercase", "lowercase"], "type": "str"},
            "mac-password-delimiter": {"v_range": [["7.0.0", ""]], "choices": ["hyphen", "single-hyphen", "colon", "none"], "type": "str"},
            "mac-username-delimiter": {"v_range": [["7.0.0", ""]], "choices": ["hyphen", "single-hyphen", "colon", "none"], "type": "str"},
            "mbo": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "mbo-cell-data-conn-pref": {"v_range": [["7.0.0", ""]], "choices": ["excluded", "prefer-not", "prefer-use"], "type": "str"},
            "nac": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "nac-profile": {"v_range": [["7.0.0", ""]], "type": "str"},
            "neighbor-report-dual-band": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "address-group-policy": {"v_range": [["7.2.0", ""]], "choices": ["disable", "allow", "deny"], "type": "str"},
            "antivirus-profile": {"v_range": [["7.0.1", ""]], "type": "str"},
            "application-detection-engine": {"v_range": [["7.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "application-list": {"v_range": [["7.0.1", ""]], "type": "str"},
            "application-report-intv": {"v_range": [["7.2.0", ""]], "type": "int"},
            "auth-cert": {"v_range": [["7.0.3", ""]], "type": "str"},
            "auth-portal-addr": {"v_range": [["7.0.3", ""]], "type": "str"},
            "beacon-advertising": {"v_range": [["7.0.2", ""]], "type": "list", "choices": ["name", "model", "serial-number"], "elements": "str"},
            "ips-sensor": {"v_range": [["7.0.1", ""]], "type": "str"},
            "l3-roaming": {"v_range": [["7.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "local-standalone-dns": {"v_range": [["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "local-standalone-dns-ip": {"v_range": [["7.0.1", ""]], "type": "raw"},
            "osen": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "radius-mac-mpsk-auth": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "radius-mac-mpsk-timeout": {"v_range": [["7.0.2", ""]], "type": "int"},
            "rates-11ax-ss12": {
                "v_range": [["7.0.2", ""]],
                "type": "list",
                "choices": [
                    "mcs0/1", "mcs1/1", "mcs2/1", "mcs3/1", "mcs4/1", "mcs5/1", "mcs6/1", "mcs7/1", "mcs8/1", "mcs9/1", "mcs10/1", "mcs11/1", "mcs0/2",
                    "mcs1/2", "mcs2/2", "mcs3/2", "mcs4/2", "mcs5/2", "mcs6/2", "mcs7/2", "mcs8/2", "mcs9/2", "mcs10/2", "mcs11/2"
                ],
                "elements": "str"
            },
            "rates-11ax-ss34": {
                "v_range": [["7.0.2", ""]],
                "type": "list",
                "choices": [
                    "mcs0/3", "mcs1/3", "mcs2/3", "mcs3/3", "mcs4/3", "mcs5/3", "mcs6/3", "mcs7/3", "mcs8/3", "mcs9/3", "mcs10/3", "mcs11/3", "mcs0/4",
                    "mcs1/4", "mcs2/4", "mcs3/4", "mcs4/4", "mcs5/4", "mcs6/4", "mcs7/4", "mcs8/4", "mcs9/4", "mcs10/4", "mcs11/4"
                ],
                "elements": "str"
            },
            "scan-botnet-connections": {"v_range": [["7.0.1", ""]], "choices": ["disable", "block", "monitor"], "type": "str"},
            "utm-log": {"v_range": [["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "utm-status": {"v_range": [["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "vlan-name": {
                "v_range": [["7.0.3", ""]],
                "type": "list",
                "options": {"name": {"v_range": [["7.0.3", ""]], "type": "str"}, "vlan-id": {"v_range": [["7.0.3", ""]], "type": "int"}},
                "elements": "dict"
            },
            "webfilter-profile": {"v_range": [["7.0.1", ""]], "type": "str"},
            "sae-h2e-only": {"v_range": [["7.0.5", "7.0.12"], ["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "sae-pk": {"v_range": [["7.0.5", "7.0.12"], ["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "sae-private-key": {"v_range": [["7.0.5", "7.0.12"], ["7.2.1", ""]], "no_log": true, "type": "str"},
            "sticky-client-threshold-6g": {"v_range": [["7.0.5", "7.0.12"], ["7.2.1", ""]], "type": "str"},
            "application-dscp-marking": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "l3-roaming-mode": {"v_range": [["7.2.1", ""]], "choices": ["direct", "indirect"], "type": "str"},
            "rates-11ac-mcs-map": {"v_range": [["7.2.1", ""]], "type": "str"},
            "rates-11ax-mcs-map": {"v_range": [["7.2.1", ""]], "type": "str"},
            "captive-portal-fw-accounting": {"v_range": [["7.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "radius-mac-auth-block-interval": {"v_range": [["7.2.2", ""]], "type": "int"},
            "_is_factory_setting": {"v_range": [["7.4.0", ""]], "choices": ["disable", "enable", "ext"], "type": "str"},
            "80211k": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "80211v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "roaming-acct-interim-update": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "sae-hnp-only": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "akm24-only": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "beacon-protection": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "captive-portal": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "nas-filter-rule": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "rates-11be-mcs-map": {"v_range": [["7.4.3", ""]], "type": "str"},
            "rates-11be-mcs-map-160": {"v_range": [["7.4.3", ""]], "type": "str"},
            "rates-11be-mcs-map-320": {"v_range": [["7.4.3", ""]], "type": "str"},
            "_intf_ip-managed-by-fortiipam": {"v_range": [["7.6.0", ""]], "choices": ["disable", "enable", "inherit-global"], "type": "str"},
            "_intf_managed-subnetwork-size": {
                "v_range": [["7.6.0", ""]],
                "choices": ["32", "64", "128", "256", "512", "1024", "2048", "4096", "8192", "16384", "32768", "65536"],
                "type": "str"
            },
            "domain-name-stripping": {"v_range": [["7.6.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "local-lan-partition": {"v_range": [["7.6.0", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "vap_dynamicmapping": {
        "stated": true,
        "options": {
            "_centmgmt": {"choices": ["disable", "enable"], "type": "str"},
            "_dhcp_svr_id": {"type": "str"},
            "_intf_allowaccess": {
                "type": "list",
                "choices": [
                    "https", "ping", "ssh", "snmp", "http", "telnet", "fgfm", "auto-ipsec", "radius-acct", "probe-response", "capwap", "dnp", "ftm",
                    "fabric", "speed-test"
                ],
                "elements": "str"
            },
            "_intf_device-identification": {"choices": ["disable", "enable"], "type": "str"},
            "_intf_device-netscan": {"choices": ["disable", "enable"], "type": "str"},
            "_intf_dhcp-relay-ip": {"type": "raw"},
            "_intf_dhcp-relay-service": {"choices": ["disable", "enable"], "type": "str"},
            "_intf_dhcp-relay-type": {"choices": ["regular", "ipsec"], "type": "str"},
            "_intf_dhcp6-relay-ip": {"type": "str"},
            "_intf_dhcp6-relay-service": {"choices": ["disable", "enable"], "type": "str"},
            "_intf_dhcp6-relay-type": {"choices": ["regular"], "type": "str"},
            "_intf_ip": {"type": "str"},
            "_intf_ip6-address": {"type": "str"},
            "_intf_ip6-allowaccess": {
                "type": "list",
                "choices": ["https", "ping", "ssh", "snmp", "http", "telnet", "any", "fgfm", "capwap"],
                "elements": "str"
            },
            "_intf_listen-forticlient-connection": {"choices": ["disable", "enable"], "type": "str"},
            "_scope": {"type": "list", "options": {"name": {"type": "str"}, "vdom": {"type": "str"}}, "elements": "dict"},
            "acct-interim-interval": {"type": "int"},
            "address-group": {"type": "str"},
            "alias": {"type": "str"},
            "atf-weight": {"type": "int"},
            "auth": {"choices": ["PSK", "psk", "RADIUS", "radius", "usergroup"], "type": "str"},
            "broadcast-ssid": {"choices": ["disable", "enable"], "type": "str"},
            "broadcast-suppression": {
                "type": "list",
                "choices": [
                    "dhcp", "arp", "dhcp2", "arp2", "netbios-ns", "netbios-ds", "arp3", "dhcp-up", "dhcp-down", "arp-known", "arp-unknown", "arp-reply",
                    "ipv6", "dhcp-starvation", "arp-poison", "all-other-mc", "all-other-bc", "arp-proxy", "dhcp-ucast"
                ],
                "elements": "str"
            },
            "captive-portal-ac-name": {"type": "str"},
            "captive-portal-macauth-radius-secret": {"no_log": true, "type": "raw"},
            "captive-portal-macauth-radius-server": {"type": "str"},
            "captive-portal-radius-secret": {"no_log": true, "type": "raw"},
            "captive-portal-radius-server": {"type": "str"},
            "captive-portal-session-timeout-interval": {"type": "int"},
            "client-count": {"type": "int"},
            "dhcp-lease-time": {"type": "int"},
            "dhcp-option82-circuit-id-insertion": {"choices": ["disable", "style-1", "style-2", "style-3"], "type": "str"},
            "dhcp-option82-insertion": {"choices": ["disable", "enable"], "type": "str"},
            "dhcp-option82-remote-id-insertion": {"choices": ["disable", "style-1"], "type": "str"},
            "dynamic-vlan": {"choices": ["disable", "enable"], "type": "str"},
            "eap-reauth": {"choices": ["disable", "enable"], "type": "str"},
            "eap-reauth-intv": {"type": "int"},
            "eapol-key-retries": {"choices": ["disable", "enable"], "type": "str"},
            "encrypt": {"choices": ["TKIP", "AES", "TKIP-AES"], "type": "str"},
            "external-fast-roaming": {"choices": ["disable", "enable"], "type": "str"},
            "external-logout": {"type": "str"},
            "external-web": {"type": "str"},
            "fast-bss-transition": {"choices": ["disable", "enable"], "type": "str"},
            "fast-roaming": {"choices": ["disable", "enable"], "type": "str"},
            "ft-mobility-domain": {"type": "int"},
            "ft-over-ds": {"choices": ["disable", "enable"], "type": "str"},
            "ft-r0-key-lifetime": {"no_log": true, "type": "int"},
            "gtk-rekey": {"choices": ["disable", "enable"], "type": "str"},
            "gtk-rekey-intv": {"no_log": true, "type": "int"},
            "hotspot20-profile": {"type": "str"},
            "intra-vap-privacy": {"choices": ["disable", "enable"], "type": "str"},
            "ip": {"type": "str"},
            "key": {"no_log": true, "type": "raw"},
            "keyindex": {"no_log": true, "type": "int"},
            "ldpc": {"choices": ["disable", "tx", "rx", "rxtx"], "type": "str"},
            "local-authentication": {"choices": ["disable", "enable"], "type": "str"},
            "local-bridging": {"choices": ["disable", "enable"], "type": "str"},
            "local-lan": {"choices": ["deny", "allow"], "type": "str"},
            "local-standalone": {"choices": ["disable", "enable"], "type": "str"},
            "local-standalone-nat": {"choices": ["disable", "enable"], "type": "str"},
            "local-switching": {"choices": ["disable", "enable"], "type": "str"},
            "mac-auth-bypass": {"choices": ["disable", "enable"], "type": "str"},
            "mac-filter": {"choices": ["disable", "enable"], "type": "str"},
            "mac-filter-policy-other": {"choices": ["deny", "allow"], "type": "str"},
            "max-clients": {"type": "int"},
            "max-clients-ap": {"type": "int"},
            "me-disable-thresh": {"type": "int"},
            "mesh-backhaul": {"choices": ["disable", "enable"], "type": "str"},
            "mpsk": {"choices": ["disable", "enable"], "type": "str"},
            "mpsk-concurrent-clients": {"type": "int"},
            "multicast-enhance": {"choices": ["disable", "enable"], "type": "str"},
            "multicast-rate": {"choices": ["0", "6000", "12000", "24000"], "type": "str"},
            "okc": {"choices": ["disable", "enable"], "type": "str"},
            "owe-groups": {"type": "list", "choices": ["19", "20", "21"], "elements": "str"},
            "owe-transition": {"choices": ["disable", "enable"], "type": "str"},
            "owe-transition-ssid": {"type": "str"},
            "passphrase": {"no_log": true, "type": "raw"},
            "pmf": {"choices": ["disable", "enable", "optional"], "type": "str"},
            "pmf-assoc-comeback-timeout": {"type": "int"},
            "pmf-sa-query-retry-timeout": {"type": "int"},
            "portal-message-override-group": {"type": "str"},
            "portal-type": {
                "choices": [
                    "auth", "auth+disclaimer", "disclaimer", "email-collect", "cmcc", "cmcc-macauth", "auth-mac", "external-auth", "external-macauth"
                ],
                "type": "str"
            },
            "probe-resp-suppression": {"choices": ["disable", "enable"], "type": "str"},
            "probe-resp-threshold": {"type": "str"},
            "ptk-rekey": {"choices": ["disable", "enable"], "type": "str"},
            "ptk-rekey-intv": {"no_log": true, "type": "int"},
            "qos-profile": {"type": "str"},
            "quarantine": {"choices": ["disable", "enable"], "type": "str"},
            "radio-2g-threshold": {"type": "str"},
            "radio-5g-threshold": {"type": "str"},
            "radio-sensitivity": {"choices": ["disable", "enable"], "type": "str"},
            "radius-mac-auth": {"choices": ["disable", "enable"], "type": "str"},
            "radius-mac-auth-server": {"type": "str"},
            "radius-mac-auth-usergroups": {"type": "raw"},
            "radius-server": {"type": "str"},
            "rates-11a": {
                "type": "list",
                "choices": [
                    "1", "1-basic", "2", "2-basic", "5.5", "5.5-basic", "6", "6-basic", "9", "9-basic", "12", "12-basic", "18", "18-basic", "24",
                    "24-basic", "36", "36-basic", "48", "48-basic", "54", "54-basic", "11", "11-basic"
                ],
                "elements": "str"
            },
            "rates-11ac-ss12": {
                "type": "list",
                "choices": [
                    "mcs0/1", "mcs1/1", "mcs2/1", "mcs3/1", "mcs4/1", "mcs5/1", "mcs6/1", "mcs7/1", "mcs8/1", "mcs9/1", "mcs0/2", "mcs1/2", "mcs2/2",
                    "mcs3/2", "mcs4/2", "mcs5/2", "mcs6/2", "mcs7/2", "mcs8/2", "mcs9/2", "mcs10/1", "mcs11/1", "mcs10/2", "mcs11/2"
                ],
                "elements": "str"
            },
            "rates-11ac-ss34": {
                "type": "list",
                "choices": [
                    "mcs0/3", "mcs1/3", "mcs2/3", "mcs3/3", "mcs4/3", "mcs5/3", "mcs6/3", "mcs7/3", "mcs8/3", "mcs9/3", "mcs0/4", "mcs1/4", "mcs2/4",
                    "mcs3/4", "mcs4/4", "mcs5/4", "mcs6/4", "mcs7/4", "mcs8/4", "mcs9/4", "mcs10/3", "mcs11/3", "mcs10/4", "mcs11/4"
                ],
                "elements": "str"
            },
            "rates-11bg": {
                "type": "list",
                "choices": [
                    "1", "1-basic", "2", "2-basic", "5.5", "5.5-basic", "6", "6-basic", "9", "9-basic", "12", "12-basic", "18", "18-basic", "24",
                    "24-basic", "36", "36-basic", "48", "48-basic", "54", "54-basic", "11", "11-basic"
                ],
                "elements": "str"
            },
            "rates-11n-ss12": {
                "type": "list",
                "choices": [
                    "mcs0/1", "mcs1/1", "mcs2/1", "mcs3/1", "mcs4/1", "mcs5/1", "mcs6/1", "mcs7/1", "mcs8/2", "mcs9/2", "mcs10/2", "mcs11/2", "mcs12/2",
                    "mcs13/2", "mcs14/2", "mcs15/2"
                ],
                "elements": "str"
            },
            "rates-11n-ss34": {
                "type": "list",
                "choices": [
                    "mcs16/3", "mcs17/3", "mcs18/3", "mcs19/3", "mcs20/3", "mcs21/3", "mcs22/3", "mcs23/3", "mcs24/4", "mcs25/4", "mcs26/4", "mcs27/4",
                    "mcs28/4", "mcs29/4", "mcs30/4", "mcs31/4"
                ],
                "elements": "str"
            },
            "sae-groups": {
                "type": "list",
                "choices": ["1", "2", "5", "14", "15", "16", "17", "18", "19", "20", "21", "27", "28", "29", "30", "31"],
                "elements": "str"
            },
            "sae-password": {"no_log": true, "type": "raw"},
            "schedule": {"type": "raw"},
            "security": {
                "choices": [
                    "None", "WEP64", "wep64", "WEP128", "wep128", "WPA_PSK", "WPA_RADIUS", "WPA", "WPA2", "WPA2_AUTO", "open", "wpa-personal",
                    "wpa-enterprise", "captive-portal", "wpa-only-personal", "wpa-only-enterprise", "wpa2-only-personal", "wpa2-only-enterprise",
                    "wpa-personal+captive-portal", "wpa-only-personal+captive-portal", "wpa2-only-personal+captive-portal", "osen", "wpa3-enterprise",
                    "sae", "sae-transition", "owe", "wpa3-sae", "wpa3-sae-transition", "wpa3-only-enterprise", "wpa3-enterprise-transition"
                ],
                "type": "str"
            },
            "security-exempt-list": {"type": "str"},
            "security-obsolete-option": {"choices": ["disable", "enable"], "type": "str"},
            "security-redirect-url": {"type": "str"},
            "selected-usergroups": {"type": "raw"},
            "split-tunneling": {"choices": ["disable", "enable"], "type": "str"},
            "ssid": {"type": "str"},
            "tkip-counter-measure": {"choices": ["disable", "enable"], "type": "str"},
            "usergroup": {"type": "raw"},
            "utm-profile": {"type": "str"},
            "vdom": {"type": "raw"},
            "vlan-auto": {"choices": ["disable", "enable"], "type": "str"},
            "vlan-pooling": {"choices": ["wtp-group", "round-robin", "hash", "disable"], "type": "str"},
            "vlanid": {"type": "int"},
            "voice-enterprise": {"choices": ["disable", "enable"], "type": "str"},
            "mu-mimo": {"v_range": [["6.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "_intf_device-access-list": {"v_range": [["6.2.2", ""]], "type": "str"},
            "external-web-format": {"v_range": [["6.2.2", ""]], "choices": ["auto-detect", "no-query-string", "partial-query-string"], "type": "str"},
            "high-efficiency": {"v_range": [["6.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "primary-wag-profile": {"v_range": [["6.2.2", ""]], "type": "str"},
            "secondary-wag-profile": {"v_range": [["6.2.2", ""]], "type": "str"},
            "target-wake-time": {"v_range": [["6.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "tunnel-echo-interval": {"v_range": [["6.2.2", ""]], "type": "int"},
            "tunnel-fallback-interval": {"v_range": [["6.2.2", ""]], "type": "int"},
            "access-control-list": {"v_range": [["6.4.0", ""]], "type": "str"},
            "captive-portal-auth-timeout": {"v_range": [["6.4.0", ""]], "type": "int"},
            "ipv6-rules": {
                "v_range": [["6.4.0", ""]],
                "type": "list",
                "choices": [
                    "drop-icmp6ra", "drop-icmp6rs", "drop-llmnr6", "drop-icmp6mld2", "drop-dhcp6s", "drop-dhcp6c", "ndp-proxy", "drop-ns-dad",
                    "drop-ns-nondad"
                ],
                "elements": "str"
            },
            "sticky-client-remove": {"v_range": [["6.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "sticky-client-threshold-2g": {"v_range": [["6.4.0", ""]], "type": "str"},
            "sticky-client-threshold-5g": {"v_range": [["6.4.0", ""]], "type": "str"},
            "bss-color-partial": {"v_range": [["6.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "dhcp-option43-insertion": {"v_range": [["6.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "mpsk-profile": {"v_range": [["6.4.2", ""]], "type": "str"},
            "igmp-snooping": {"v_range": [["6.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "port-macauth": {"v_range": [["6.2.8", "6.2.12"], ["6.4.3", ""]], "choices": ["disable", "radius", "address-group"], "type": "str"},
            "port-macauth-reauth-timeout": {"v_range": [["6.2.8", "6.2.12"], ["6.4.3", ""]], "type": "int"},
            "port-macauth-timeout": {"v_range": [["6.2.8", "6.2.12"], ["6.4.3", ""]], "type": "int"},
            "additional-akms": {"v_range": [["7.0.0", ""]], "type": "list", "choices": ["akm6", "akm24"], "elements": "str"},
            "bstm-disassociation-imminent": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "bstm-load-balancing-disassoc-timer": {"v_range": [["7.0.0", ""]], "type": "int"},
            "bstm-rssi-disassoc-timer": {"v_range": [["7.0.0", ""]], "type": "int"},
            "dhcp-address-enforcement": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "gas-comeback-delay": {"v_range": [["7.0.0", ""]], "type": "int"},
            "gas-fragmentation-limit": {"v_range": [["7.0.0", ""]], "type": "int"},
            "mac-called-station-delimiter": {"v_range": [["7.0.0", ""]], "choices": ["hyphen", "single-hyphen", "colon", "none"], "type": "str"},
            "mac-calling-station-delimiter": {"v_range": [["7.0.0", ""]], "choices": ["hyphen", "single-hyphen", "colon", "none"], "type": "str"},
            "mac-case": {"v_range": [["7.0.0", ""]], "choices": ["uppercase", "lowercase"], "type": "str"},
            "mac-password-delimiter": {"v_range": [["7.0.0", ""]], "choices": ["hyphen", "single-hyphen", "colon", "none"], "type": "str"},
            "mac-username-delimiter": {"v_range": [["7.0.0", ""]], "choices": ["hyphen", "single-hyphen", "colon", "none"], "type": "str"},
            "mbo": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "mbo-cell-data-conn-pref": {"v_range": [["7.0.0", ""]], "choices": ["excluded", "prefer-not", "prefer-use"], "type": "str"},
            "nac": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "nac-profile": {"v_range": [["7.0.0", ""]], "type": "str"},
            "neighbor-report-dual-band": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "address-group-policy": {"v_range": [["7.2.0", ""]], "choices": ["disable", "allow", "deny"], "type": "str"},
            "antivirus-profile": {"v_range": [["7.0.1", ""]], "type": "str"},
            "application-detection-engine": {"v_range": [["7.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "application-list": {"v_range": [["7.0.1", ""]], "type": "str"},
            "application-report-intv": {"v_range": [["7.2.0", ""]], "type": "int"},
            "auth-cert": {"v_range": [["7.0.3", ""]], "type": "str"},
            "auth-portal-addr": {"v_range": [["7.0.3", ""]], "type": "str"},
            "beacon-advertising": {"v_range": [["7.0.2", ""]], "type": "list", "choices": ["name", "model", "serial-number"], "elements": "str"},
            "ips-sensor": {"v_range": [["7.0.1", ""]], "type": "str"},
            "l3-roaming": {"v_range": [["7.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "local-standalone-dns": {"v_range": [["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "local-standalone-dns-ip": {"v_range": [["7.0.1", ""]], "type": "raw"},
            "osen": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "radius-mac-mpsk-auth": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "radius-mac-mpsk-timeout": {"v_range": [["7.0.2", ""]], "type": "int"},
            "rates-11ax-ss12": {
                "v_range": [["7.0.2", ""]],
                "type": "list",
                "choices": [
                    "mcs0/1", "mcs1/1", "mcs2/1", "mcs3/1", "mcs4/1", "mcs5/1", "mcs6/1", "mcs7/1", "mcs8/1", "mcs9/1", "mcs10/1", "mcs11/1", "mcs0/2",
                    "mcs1/2", "mcs2/2", "mcs3/2", "mcs4/2", "mcs5/2", "mcs6/2", "mcs7/2", "mcs8/2", "mcs9/2", "mcs10/2", "mcs11/2"
                ],
                "elements": "str"
            },
            "rates-11ax-ss34": {
                "v_range": [["7.0.2", ""]],
                "type": "list",
                "choices": [
                    "mcs0/3", "mcs1/3", "mcs2/3", "mcs3/3", "mcs4/3", "mcs5/3", "mcs6/3", "mcs7/3", "mcs8/3", "mcs9/3", "mcs10/3", "mcs11/3", "mcs0/4",
                    "mcs1/4", "mcs2/4", "mcs3/4", "mcs4/4", "mcs5/4", "mcs6/4", "mcs7/4", "mcs8/4", "mcs9/4", "mcs10/4", "mcs11/4"
                ],
                "elements": "str"
            },
            "scan-botnet-connections": {"v_range": [["7.0.1", ""]], "choices": ["disable", "block", "monitor"], "type": "str"},
            "utm-log": {"v_range": [["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "utm-status": {"v_range": [["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "webfilter-profile": {"v_range": [["7.0.1", ""]], "type": "str"},
            "sae-h2e-only": {"v_range": [["7.0.5", "7.0.12"], ["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "sae-pk": {"v_range": [["7.0.5", "7.0.12"], ["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "sae-private-key": {"v_range": [["7.0.5", "7.0.12"], ["7.2.1", ""]], "no_log": true, "type": "str"},
            "sticky-client-threshold-6g": {"v_range": [["7.0.5", "7.0.12"], ["7.2.1", ""]], "type": "str"},
            "application-dscp-marking": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "l3-roaming-mode": {"v_range": [["7.2.1", ""]], "choices": ["direct", "indirect"], "type": "str"},
            "rates-11ac-mcs-map": {"v_range": [["7.2.1", ""]], "type": "str"},
            "rates-11ax-mcs-map": {"v_range": [["7.2.1", ""]], "type": "str"},
            "captive-portal-fw-accounting": {"v_range": [["7.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "radius-mac-auth-block-interval": {"v_range": [["7.2.2", ""]], "type": "int"},
            "_is_factory_setting": {"v_range": [["7.4.0", ""]], "choices": ["disable", "enable", "ext"], "type": "str"},
            "80211k": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "80211v": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "roaming-acct-interim-update": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "sae-hnp-only": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "akm24-only": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "beacon-protection": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "captive-portal": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "nas-filter-rule": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "rates-11be-mcs-map": {"v_range": [["7.4.3", ""]], "type": "str"},
            "rates-11be-mcs-map-160": {"v_range": [["7.4.3", ""]], "type": "str"},
            "rates-11be-mcs-map-320": {"v_range": [["7.4.3", ""]], "type": "str"},
            "_intf_ip-managed-by-fortiipam": {"v_range": [["7.6.0", ""]], "choices": ["disable", "enable", "inherit-global"], "type": "str"},
            "_intf_managed-subnetwork-size": {
                "v_range": [["7.6.0", ""]],
                "choices": ["32", "64", "128", "256", "512", "1024", "2048", "4096", "8192", "16384", "32768", "65536"],
                "type": "str"
            },
            "domain-name-stripping": {"v_range": [["7.6.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "local-lan-partition": {"v_range": [["7.6.0", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "vap_macfilterlist": {
        "stated": true,
        "options": {"id": {"required": true, "type": "int"}, "mac": {"type": "str"}, "mac-filter-policy": {"choices": ["deny", "allow"], "type": "str"}}
    },
    "vap_mpskkey": {
        "stated": true,
        "options": {
            "comment": {"type": "str"},
            "concurrent-clients": {"type": "str"},
            "key-name": {"required": true, "no_log": true, "type": "str"},
            "passphrase": {"no_log": true, "type": "raw"},
            "mpsk-schedules": {"v_range": [["6.2.2", ""]], "type": "raw"}
        }
    },
    "vap_portalmessageoverrides": {
        "stated": false,
        "options": {
            "auth-disclaimer-page": {"type": "str"},
            "auth-login-failed-page": {"type": "str"},
            "auth-login-page": {"type": "str"},
            "auth-reject-page": {"type": "str"}
        }
    },
    "vap_vlanname": {
        "stated": true,
        "options": {"name": {"v_range": [["7.0.3", ""]], "required": true, "type": "str"}, "vlan-id": {"v_range": [["7.0.3", ""]], "type": "int"}}
    },
    "vap_vlanpool": {
        "stated": true,
        "options": {
            "_wtp-group": {"type": "str"},
            "id": {"required": true, "type": "int"},
            "wtp-group": {"v_range": [["6.0.0", "6.2.0"], ["7.4.3", ""]], "type": "str"}
        }
    },
    "vapgroup": {"stated": true, "options": {"comment": {"type": "str"}, "name": {"required": true, "type": "str"}, "vaps": {"type": "raw"}}},
    "videofilter_keyword": {
        "stated": true,
        "options": {
            "comment": {"v_range": [["7.4.2", ""]], "type": "str"},
            "id": {"v_range": [["7.4.2", ""]], "required": true, "type": "int"},
            "match": {"v_range": [["7.4.2", ""]], "choices": ["or", "and"], "type": "str"},
            "name": {"v_range": [["7.4.2", ""]], "type": "str"},
            "word": {
                "v_range": [["7.4.2", ""]],
                "type": "list",
                "options": {
                    "comment": {"v_range": [["7.4.2", ""]], "type": "str"},
                    "name": {"v_range": [["7.4.2", ""]], "type": "str"},
                    "pattern-type": {"v_range": [["7.4.2", ""]], "choices": ["wildcard", "regex"], "type": "str"},
                    "status": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"}
                },
                "elements": "dict"
            }
        }
    },
    "videofilter_keyword_word": {
        "stated": true,
        "options": {
            "comment": {"v_range": [["7.4.2", ""]], "type": "str"},
            "name": {"v_range": [["7.4.2", ""]], "required": true, "type": "str"},
            "pattern-type": {"v_range": [["7.4.2", ""]], "choices": ["wildcard", "regex"], "type": "str"},
            "status": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "videofilter_profile": {
        "stated": true,
        "options": {
            "comment": {"v_range": [["7.0.0", ""]], "type": "str"},
            "dailymotion": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "fortiguard-category": {
                "v_range": [["7.0.0", ""]],
                "type": "dict",
                "options": {
                    "filters": {
                        "v_range": [["7.0.0", ""]],
                        "type": "list",
                        "options": {
                            "action": {"v_range": [["7.0.0", ""]], "choices": ["block", "bypass", "monitor", "allow"], "type": "str"},
                            "category-id": {"v_range": [["7.0.0", ""]], "type": "int"},
                            "id": {"v_range": [["7.0.0", ""]], "type": "int"},
                            "log": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"}
                        },
                        "elements": "dict"
                    }
                }
            },
            "name": {"v_range": [["7.0.0", ""]], "required": true, "type": "str"},
            "vimeo": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "vimeo-restrict": {"v_range": [["7.0.0", ""]], "type": "str"},
            "youtube": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "youtube-channel-filter": {"v_range": [["7.0.0", ""]], "type": "str"},
            "youtube-restrict": {"v_range": [["7.0.0", ""]], "choices": ["strict", "none", "moderate"], "type": "str"},
            "replacemsg-group": {"v_range": [["7.0.1", ""]], "type": "str"},
            "default-action": {"v_range": [["7.2.3", ""]], "choices": ["block", "monitor", "allow"], "type": "str"},
            "log": {"v_range": [["7.2.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "filters": {
                "v_range": [["7.4.2", ""]],
                "type": "list",
                "options": {
                    "action": {"v_range": [["7.4.2", ""]], "choices": ["block", "monitor", "allow"], "type": "str"},
                    "category": {"v_range": [["7.4.2", ""]], "type": "str"},
                    "channel": {"v_range": [["7.4.2", ""]], "type": "str"},
                    "comment": {"v_range": [["7.4.2", ""]], "type": "str"},
                    "id": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "keyword": {"v_range": [["7.4.2", ""]], "no_log": true, "type": "str"},
                    "log": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "type": {"v_range": [["7.4.2", ""]], "choices": ["category", "channel", "title", "description"], "type": "str"}
                },
                "elements": "dict"
            }
        }
    },
    "videofilter_profile_filters": {
        "stated": true,
        "options": {
            "action": {"v_range": [["7.4.2", ""]], "choices": ["block", "monitor", "allow"], "type": "str"},
            "category": {"v_range": [["7.4.2", ""]], "type": "str"},
            "channel": {"v_range": [["7.4.2", ""]], "type": "str"},
            "comment": {"v_range": [["7.4.2", ""]], "type": "str"},
            "id": {"v_range": [["7.4.2", ""]], "required": true, "type": "int"},
            "keyword": {"v_range": [["7.4.2", ""]], "no_log": true, "type": "str"},
            "log": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "type": {"v_range": [["7.4.2", ""]], "choices": ["category", "channel", "title", "description"], "type": "str"}
        }
    },
    "videofilter_profile_fortiguardcategory": {
        "stated": false,
        "options": {
            "filters": {
                "v_range": [["7.0.0", ""]],
                "type": "list",
                "options": {
                    "action": {"v_range": [["7.0.0", ""]], "choices": ["block", "bypass", "monitor", "allow"], "type": "str"},
                    "category-id": {"v_range": [["7.0.0", ""]], "type": "int"},
                    "id": {"v_range": [["7.0.0", ""]], "type": "int"},
                    "log": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"}
                },
                "elements": "dict"
            }
        }
    },
    "videofilter_profile_fortiguardcategory_filters": {
        "stated": true,
        "options": {
            "action": {"v_range": [["7.0.0", ""]], "choices": ["block", "bypass", "monitor", "allow"], "type": "str"},
            "category-id": {"v_range": [["7.0.0", ""]], "type": "int"},
            "id": {"v_range": [["7.0.0", ""]], "required": true, "type": "int"},
            "log": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "videofilter_youtubechannelfilter": {
        "stated": true,
        "options": {
            "comment": {"v_range": [["7.0.0", ""]], "type": "str"},
            "entries": {
                "v_range": [["7.0.0", ""]],
                "type": "list",
                "options": {
                    "action": {"v_range": [["7.0.0", ""]], "choices": ["block", "bypass", "monitor", "allow"], "type": "str"},
                    "channel-id": {"v_range": [["7.0.0", ""]], "type": "str"},
                    "comment": {"v_range": [["7.0.0", ""]], "type": "str"},
                    "id": {"v_range": [["7.0.0", ""]], "type": "int"}
                },
                "elements": "dict"
            },
            "id": {"v_range": [["7.0.0", ""]], "required": true, "type": "int"},
            "name": {"v_range": [["7.0.0", ""]], "type": "str"},
            "default-action": {"v_range": [["7.0.1", ""]], "choices": ["monitor", "block", "allow"], "type": "str"},
            "log": {"v_range": [["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "override-category": {"v_range": [["7.0.4", "7.0.12"], ["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "videofilter_youtubechannelfilter_entries": {
        "stated": true,
        "options": {
            "action": {"v_range": [["7.0.0", ""]], "choices": ["block", "bypass", "monitor", "allow"], "type": "str"},
            "channel-id": {"v_range": [["7.0.0", ""]], "type": "str"},
            "comment": {"v_range": [["7.0.0", ""]], "type": "str"},
            "id": {"v_range": [["7.0.0", ""]], "required": true, "type": "int"}
        }
    },
    "videofilter_youtubekey": {
        "stated": true,
        "options": {
            "id": {"v_range": [["7.4.2", ""]], "required": true, "type": "int"},
            "key": {"v_range": [["7.4.2", ""]], "no_log": true, "type": "str"},
            "status": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "virtualpatch_profile": {
        "stated": true,
        "options": {
            "action": {"v_range": [["7.4.1", ""]], "choices": ["pass", "block"], "type": "str"},
            "comment": {"v_range": [["7.4.1", ""]], "type": "str"},
            "exemption": {
                "v_range": [["7.4.1", ""]],
                "type": "list",
                "options": {
                    "device": {"v_range": [["7.4.1", ""]], "type": "list", "elements": "str"},
                    "id": {"v_range": [["7.4.1", ""]], "type": "int"},
                    "rule": {"v_range": [["7.4.1", ""]], "type": "list", "elements": "int"},
                    "status": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"}
                },
                "elements": "dict"
            },
            "log": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "name": {"v_range": [["7.4.1", ""]], "required": true, "type": "str"},
            "severity": {"v_range": [["7.4.1", ""]], "type": "list", "choices": ["low", "medium", "high", "critical", "info"], "elements": "str"}
        }
    },
    "virtualpatch_profile_exemption": {
        "stated": true,
        "options": {
            "device": {"v_range": [["7.4.1", ""]], "type": "list", "elements": "str"},
            "id": {"v_range": [["7.4.1", ""]], "required": true, "type": "int"},
            "rule": {"v_range": [["7.4.1", ""]], "type": "list", "elements": "int"},
            "status": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "voip_profile": {
        "stated": true,
        "options": {
            "comment": {"type": "str"},
            "name": {"required": true, "type": "str"},
            "sccp": {
                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                "type": "dict",
                "options": {
                    "block-mcast": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "log-call-summary": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "log-violations": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "max-calls": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "status": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "verify-header": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"}
                }
            },
            "sip": {
                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                "type": "dict",
                "options": {
                    "ack-rate": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "block-ack": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "block-bye": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "block-cancel": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "block-geo-red-options": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "block-info": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "block-invite": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "block-long-lines": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "block-message": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "block-notify": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "block-options": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "block-prack": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "block-publish": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "block-refer": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "block-register": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "block-subscribe": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "block-unknown": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "block-update": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "bye-rate": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "call-keepalive": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "cancel-rate": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "contact-fixup": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "hnt-restrict-source-ip": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "hosted-nat-traversal": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "info-rate": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "invite-rate": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "ips-rtp": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "log-call-summary": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "log-violations": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "malformed-header-allow": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["pass", "discard", "respond"], "type": "str"},
                    "malformed-header-call-id": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "choices": ["pass", "discard", "respond"],
                        "type": "str"
                    },
                    "malformed-header-contact": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "choices": ["pass", "discard", "respond"],
                        "type": "str"
                    },
                    "malformed-header-content-length": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "choices": ["pass", "discard", "respond"],
                        "type": "str"
                    },
                    "malformed-header-content-type": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "choices": ["pass", "discard", "respond"],
                        "type": "str"
                    },
                    "malformed-header-cseq": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["pass", "discard", "respond"], "type": "str"},
                    "malformed-header-expires": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "choices": ["pass", "discard", "respond"],
                        "type": "str"
                    },
                    "malformed-header-from": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["pass", "discard", "respond"], "type": "str"},
                    "malformed-header-max-forwards": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "choices": ["pass", "discard", "respond"],
                        "type": "str"
                    },
                    "malformed-header-p-asserted-identity": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "choices": ["pass", "discard", "respond"],
                        "type": "str"
                    },
                    "malformed-header-rack": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["pass", "discard", "respond"], "type": "str"},
                    "malformed-header-record-route": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "choices": ["pass", "discard", "respond"],
                        "type": "str"
                    },
                    "malformed-header-route": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["pass", "discard", "respond"], "type": "str"},
                    "malformed-header-rseq": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["pass", "discard", "respond"], "type": "str"},
                    "malformed-header-sdp-a": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["pass", "discard", "respond"], "type": "str"},
                    "malformed-header-sdp-b": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["pass", "discard", "respond"], "type": "str"},
                    "malformed-header-sdp-c": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["pass", "discard", "respond"], "type": "str"},
                    "malformed-header-sdp-i": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["pass", "discard", "respond"], "type": "str"},
                    "malformed-header-sdp-k": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["pass", "discard", "respond"], "type": "str"},
                    "malformed-header-sdp-m": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["pass", "discard", "respond"], "type": "str"},
                    "malformed-header-sdp-o": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["pass", "discard", "respond"], "type": "str"},
                    "malformed-header-sdp-r": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["pass", "discard", "respond"], "type": "str"},
                    "malformed-header-sdp-s": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["pass", "discard", "respond"], "type": "str"},
                    "malformed-header-sdp-t": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["pass", "discard", "respond"], "type": "str"},
                    "malformed-header-sdp-v": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["pass", "discard", "respond"], "type": "str"},
                    "malformed-header-sdp-z": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["pass", "discard", "respond"], "type": "str"},
                    "malformed-header-to": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["pass", "discard", "respond"], "type": "str"},
                    "malformed-header-via": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["pass", "discard", "respond"], "type": "str"},
                    "malformed-request-line": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["pass", "discard", "respond"], "type": "str"},
                    "max-body-length": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "max-dialogs": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "max-idle-dialogs": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "max-line-length": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "message-rate": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "nat-port-range": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "nat-trace": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "no-sdp-fixup": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "notify-rate": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "open-contact-pinhole": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "open-record-route-pinhole": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "open-register-pinhole": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "open-via-pinhole": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "options-rate": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "prack-rate": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "preserve-override": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "provisional-invite-expiry-time": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "publish-rate": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "refer-rate": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "register-contact-trace": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "register-rate": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "rfc2543-branch": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "rtp": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "ssl-algorithm": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["high", "medium", "low"], "type": "str"},
                    "ssl-auth-client": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "ssl-auth-server": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "ssl-client-certificate": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "ssl-client-renegotiation": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["allow", "deny", "secure"], "type": "str"},
                    "ssl-max-version": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "choices": ["ssl-3.0", "tls-1.0", "tls-1.1", "tls-1.2", "tls-1.3"],
                        "type": "str"
                    },
                    "ssl-min-version": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "choices": ["ssl-3.0", "tls-1.0", "tls-1.1", "tls-1.2", "tls-1.3"],
                        "type": "str"
                    },
                    "ssl-mode": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["off", "full"], "type": "str"},
                    "ssl-pfs": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["require", "deny", "allow"], "type": "str"},
                    "ssl-send-empty-frags": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "ssl-server-certificate": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "status": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "strict-register": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "subscribe-rate": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "unknown-header": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["pass", "discard", "respond"], "type": "str"},
                    "update-rate": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "ack-rate-track": {"v_range": [["7.0.0", ""]], "choices": ["none", "src-ip", "dest-ip"], "type": "str"},
                    "bye-rate-track": {"v_range": [["7.0.0", ""]], "choices": ["none", "src-ip", "dest-ip"], "type": "str"},
                    "cancel-rate-track": {"v_range": [["7.0.0", ""]], "choices": ["none", "src-ip", "dest-ip"], "type": "str"},
                    "info-rate-track": {"v_range": [["7.0.0", ""]], "choices": ["none", "src-ip", "dest-ip"], "type": "str"},
                    "invite-rate-track": {"v_range": [["7.0.0", ""]], "choices": ["none", "src-ip", "dest-ip"], "type": "str"},
                    "malformed-header-no-proxy-require": {"v_range": [["7.0.0", ""]], "choices": ["pass", "discard", "respond"], "type": "str"},
                    "malformed-header-no-require": {"v_range": [["7.0.0", ""]], "choices": ["pass", "discard", "respond"], "type": "str"},
                    "message-rate-track": {"v_range": [["7.0.0", ""]], "choices": ["none", "src-ip", "dest-ip"], "type": "str"},
                    "notify-rate-track": {"v_range": [["7.0.0", ""]], "choices": ["none", "src-ip", "dest-ip"], "type": "str"},
                    "options-rate-track": {"v_range": [["7.0.0", ""]], "choices": ["none", "src-ip", "dest-ip"], "type": "str"},
                    "prack-rate-track": {"v_range": [["7.0.0", ""]], "choices": ["none", "src-ip", "dest-ip"], "type": "str"},
                    "publish-rate-track": {"v_range": [["7.0.0", ""]], "choices": ["none", "src-ip", "dest-ip"], "type": "str"},
                    "refer-rate-track": {"v_range": [["7.0.0", ""]], "choices": ["none", "src-ip", "dest-ip"], "type": "str"},
                    "register-rate-track": {"v_range": [["7.0.0", ""]], "choices": ["none", "src-ip", "dest-ip"], "type": "str"},
                    "subscribe-rate-track": {"v_range": [["7.0.0", ""]], "choices": ["none", "src-ip", "dest-ip"], "type": "str"},
                    "update-rate-track": {"v_range": [["7.0.0", ""]], "choices": ["none", "src-ip", "dest-ip"], "type": "str"},
                    "call-id-regex": {"v_range": [["7.2.3", ""]], "type": "str"},
                    "content-type-regex": {"v_range": [["7.2.3", ""]], "type": "str"}
                }
            },
            "feature-set": {"v_range": [["7.0.0", ""]], "choices": ["flow", "proxy", "ips", "voipd"], "type": "str"},
            "msrp": {
                "v_range": [["7.0.2", ""]],
                "type": "dict",
                "options": {
                    "log-violations": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "max-msg-size": {"v_range": [["7.0.2", ""]], "type": "int"},
                    "max-msg-size-action": {"v_range": [["7.0.2", ""]], "choices": ["pass", "block", "reset", "monitor"], "type": "str"},
                    "status": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"}
                }
            }
        }
    },
    "voip_profile_msrp": {
        "stated": false,
        "options": {
            "log-violations": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "max-msg-size": {"v_range": [["7.0.2", ""]], "type": "int"},
            "max-msg-size-action": {"v_range": [["7.0.2", ""]], "choices": ["pass", "block", "reset", "monitor"], "type": "str"},
            "status": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "voip_profile_sccp": {
        "stated": false,
        "options": {
            "block-mcast": {"choices": ["disable", "enable"], "type": "str"},
            "log-call-summary": {"choices": ["disable", "enable"], "type": "str"},
            "log-violations": {"choices": ["disable", "enable"], "type": "str"},
            "max-calls": {"type": "int"},
            "status": {"choices": ["disable", "enable"], "type": "str"},
            "verify-header": {"choices": ["disable", "enable"], "type": "str"}
        }
    },
    "voip_profile_sip": {
        "stated": false,
        "options": {
            "ack-rate": {"type": "int"},
            "block-ack": {"choices": ["disable", "enable"], "type": "str"},
            "block-bye": {"choices": ["disable", "enable"], "type": "str"},
            "block-cancel": {"choices": ["disable", "enable"], "type": "str"},
            "block-geo-red-options": {"choices": ["disable", "enable"], "type": "str"},
            "block-info": {"choices": ["disable", "enable"], "type": "str"},
            "block-invite": {"choices": ["disable", "enable"], "type": "str"},
            "block-long-lines": {"choices": ["disable", "enable"], "type": "str"},
            "block-message": {"choices": ["disable", "enable"], "type": "str"},
            "block-notify": {"choices": ["disable", "enable"], "type": "str"},
            "block-options": {"choices": ["disable", "enable"], "type": "str"},
            "block-prack": {"choices": ["disable", "enable"], "type": "str"},
            "block-publish": {"choices": ["disable", "enable"], "type": "str"},
            "block-refer": {"choices": ["disable", "enable"], "type": "str"},
            "block-register": {"choices": ["disable", "enable"], "type": "str"},
            "block-subscribe": {"choices": ["disable", "enable"], "type": "str"},
            "block-unknown": {"choices": ["disable", "enable"], "type": "str"},
            "block-update": {"choices": ["disable", "enable"], "type": "str"},
            "bye-rate": {"type": "int"},
            "call-keepalive": {"type": "int"},
            "cancel-rate": {"type": "int"},
            "contact-fixup": {"choices": ["disable", "enable"], "type": "str"},
            "hnt-restrict-source-ip": {"choices": ["disable", "enable"], "type": "str"},
            "hosted-nat-traversal": {"choices": ["disable", "enable"], "type": "str"},
            "info-rate": {"type": "int"},
            "invite-rate": {"type": "int"},
            "ips-rtp": {"choices": ["disable", "enable"], "type": "str"},
            "log-call-summary": {"choices": ["disable", "enable"], "type": "str"},
            "log-violations": {"choices": ["disable", "enable"], "type": "str"},
            "malformed-header-allow": {"choices": ["pass", "discard", "respond"], "type": "str"},
            "malformed-header-call-id": {"choices": ["pass", "discard", "respond"], "type": "str"},
            "malformed-header-contact": {"choices": ["pass", "discard", "respond"], "type": "str"},
            "malformed-header-content-length": {"choices": ["pass", "discard", "respond"], "type": "str"},
            "malformed-header-content-type": {"choices": ["pass", "discard", "respond"], "type": "str"},
            "malformed-header-cseq": {"choices": ["pass", "discard", "respond"], "type": "str"},
            "malformed-header-expires": {"choices": ["pass", "discard", "respond"], "type": "str"},
            "malformed-header-from": {"choices": ["pass", "discard", "respond"], "type": "str"},
            "malformed-header-max-forwards": {"choices": ["pass", "discard", "respond"], "type": "str"},
            "malformed-header-p-asserted-identity": {"choices": ["pass", "discard", "respond"], "type": "str"},
            "malformed-header-rack": {"choices": ["pass", "discard", "respond"], "type": "str"},
            "malformed-header-record-route": {"choices": ["pass", "discard", "respond"], "type": "str"},
            "malformed-header-route": {"choices": ["pass", "discard", "respond"], "type": "str"},
            "malformed-header-rseq": {"choices": ["pass", "discard", "respond"], "type": "str"},
            "malformed-header-sdp-a": {"choices": ["pass", "discard", "respond"], "type": "str"},
            "malformed-header-sdp-b": {"choices": ["pass", "discard", "respond"], "type": "str"},
            "malformed-header-sdp-c": {"choices": ["pass", "discard", "respond"], "type": "str"},
            "malformed-header-sdp-i": {"choices": ["pass", "discard", "respond"], "type": "str"},
            "malformed-header-sdp-k": {"choices": ["pass", "discard", "respond"], "type": "str"},
            "malformed-header-sdp-m": {"choices": ["pass", "discard", "respond"], "type": "str"},
            "malformed-header-sdp-o": {"choices": ["pass", "discard", "respond"], "type": "str"},
            "malformed-header-sdp-r": {"choices": ["pass", "discard", "respond"], "type": "str"},
            "malformed-header-sdp-s": {"choices": ["pass", "discard", "respond"], "type": "str"},
            "malformed-header-sdp-t": {"choices": ["pass", "discard", "respond"], "type": "str"},
            "malformed-header-sdp-v": {"choices": ["pass", "discard", "respond"], "type": "str"},
            "malformed-header-sdp-z": {"choices": ["pass", "discard", "respond"], "type": "str"},
            "malformed-header-to": {"choices": ["pass", "discard", "respond"], "type": "str"},
            "malformed-header-via": {"choices": ["pass", "discard", "respond"], "type": "str"},
            "malformed-request-line": {"choices": ["pass", "discard", "respond"], "type": "str"},
            "max-body-length": {"type": "int"},
            "max-dialogs": {"type": "int"},
            "max-idle-dialogs": {"type": "int"},
            "max-line-length": {"type": "int"},
            "message-rate": {"type": "int"},
            "nat-trace": {"choices": ["disable", "enable"], "type": "str"},
            "no-sdp-fixup": {"choices": ["disable", "enable"], "type": "str"},
            "notify-rate": {"type": "int"},
            "open-contact-pinhole": {"choices": ["disable", "enable"], "type": "str"},
            "open-record-route-pinhole": {"choices": ["disable", "enable"], "type": "str"},
            "open-register-pinhole": {"choices": ["disable", "enable"], "type": "str"},
            "open-via-pinhole": {"choices": ["disable", "enable"], "type": "str"},
            "options-rate": {"type": "int"},
            "prack-rate": {"type": "int"},
            "preserve-override": {"choices": ["disable", "enable"], "type": "str"},
            "provisional-invite-expiry-time": {"type": "int"},
            "publish-rate": {"type": "int"},
            "refer-rate": {"type": "int"},
            "register-contact-trace": {"choices": ["disable", "enable"], "type": "str"},
            "register-rate": {"type": "int"},
            "rfc2543-branch": {"choices": ["disable", "enable"], "type": "str"},
            "rtp": {"choices": ["disable", "enable"], "type": "str"},
            "ssl-algorithm": {"choices": ["high", "medium", "low"], "type": "str"},
            "ssl-auth-client": {"type": "str"},
            "ssl-auth-server": {"type": "str"},
            "ssl-client-certificate": {"type": "str"},
            "ssl-client-renegotiation": {"choices": ["allow", "deny", "secure"], "type": "str"},
            "ssl-max-version": {"choices": ["ssl-3.0", "tls-1.0", "tls-1.1", "tls-1.2", "tls-1.3"], "type": "str"},
            "ssl-min-version": {"choices": ["ssl-3.0", "tls-1.0", "tls-1.1", "tls-1.2", "tls-1.3"], "type": "str"},
            "ssl-mode": {"choices": ["off", "full"], "type": "str"},
            "ssl-pfs": {"choices": ["require", "deny", "allow"], "type": "str"},
            "ssl-send-empty-frags": {"choices": ["disable", "enable"], "type": "str"},
            "ssl-server-certificate": {"type": "str"},
            "status": {"choices": ["disable", "enable"], "type": "str"},
            "strict-register": {"choices": ["disable", "enable"], "type": "str"},
            "subscribe-rate": {"type": "int"},
            "unknown-header": {"choices": ["pass", "discard", "respond"], "type": "str"},
            "update-rate": {"type": "int"},
            "nat-port-range": {"v_range": [["6.2.0", ""]], "type": "str"},
            "ack-rate-track": {"v_range": [["7.0.0", ""]], "choices": ["none", "src-ip", "dest-ip"], "type": "str"},
            "bye-rate-track": {"v_range": [["7.0.0", ""]], "choices": ["none", "src-ip", "dest-ip"], "type": "str"},
            "cancel-rate-track": {"v_range": [["7.0.0", ""]], "choices": ["none", "src-ip", "dest-ip"], "type": "str"},
            "info-rate-track": {"v_range": [["7.0.0", ""]], "choices": ["none", "src-ip", "dest-ip"], "type": "str"},
            "invite-rate-track": {"v_range": [["7.0.0", ""]], "choices": ["none", "src-ip", "dest-ip"], "type": "str"},
            "malformed-header-no-proxy-require": {"v_range": [["7.0.0", ""]], "choices": ["pass", "discard", "respond"], "type": "str"},
            "malformed-header-no-require": {"v_range": [["7.0.0", ""]], "choices": ["pass", "discard", "respond"], "type": "str"},
            "message-rate-track": {"v_range": [["7.0.0", ""]], "choices": ["none", "src-ip", "dest-ip"], "type": "str"},
            "notify-rate-track": {"v_range": [["7.0.0", ""]], "choices": ["none", "src-ip", "dest-ip"], "type": "str"},
            "options-rate-track": {"v_range": [["7.0.0", ""]], "choices": ["none", "src-ip", "dest-ip"], "type": "str"},
            "prack-rate-track": {"v_range": [["7.0.0", ""]], "choices": ["none", "src-ip", "dest-ip"], "type": "str"},
            "publish-rate-track": {"v_range": [["7.0.0", ""]], "choices": ["none", "src-ip", "dest-ip"], "type": "str"},
            "refer-rate-track": {"v_range": [["7.0.0", ""]], "choices": ["none", "src-ip", "dest-ip"], "type": "str"},
            "register-rate-track": {"v_range": [["7.0.0", ""]], "choices": ["none", "src-ip", "dest-ip"], "type": "str"},
            "subscribe-rate-track": {"v_range": [["7.0.0", ""]], "choices": ["none", "src-ip", "dest-ip"], "type": "str"},
            "update-rate-track": {"v_range": [["7.0.0", ""]], "choices": ["none", "src-ip", "dest-ip"], "type": "str"},
            "call-id-regex": {"v_range": [["7.2.3", ""]], "type": "str"},
            "content-type-regex": {"v_range": [["7.2.3", ""]], "type": "str"}
        }
    },
    "vpn_certificate_ca": {
        "stated": true,
        "options": {
            "_private_key": {"no_log": true, "type": "str"},
            "auto-update-days": {"type": "int"},
            "auto-update-days-warning": {"type": "int"},
            "ca": {"type": "str"},
            "last-updated": {"type": "int"},
            "name": {"required": true, "type": "str"},
            "range": {"choices": ["global", "vdom"], "type": "str"},
            "scep-url": {"type": "str"},
            "source": {"choices": ["factory", "user", "bundle", "fortiguard"], "type": "str"},
            "source-ip": {"type": "str"},
            "trusted": {"v_range": [["6.0.0", "7.2.1"]], "choices": ["disable", "enable"], "type": "str"},
            "ssl-inspection-trusted": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ca-identifier": {"v_range": [["7.0.2", ""]], "type": "str"},
            "obsolete": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "est-url": {"v_range": [["7.4.1", ""]], "type": "str"},
            "fabric-ca": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "non-fabric-name": {"v_range": [["7.4.3", ""]], "type": "str"}
        }
    },
    "vpn_certificate_ocspserver": {
        "stated": true,
        "options": {
            "cert": {"type": "str"},
            "name": {"required": true, "type": "str"},
            "secondary-cert": {"type": "str"},
            "secondary-url": {"type": "str"},
            "source-ip": {"type": "str"},
            "unavail-action": {"choices": ["revoke", "ignore"], "type": "str"},
            "url": {"type": "str"}
        }
    },
    "vpn_certificate_remote": {
        "stated": true,
        "options": {
            "name": {"required": true, "type": "str"},
            "range": {"choices": ["global", "vdom"], "type": "str"},
            "remote": {"type": "str"},
            "source": {"choices": ["factory", "user", "bundle", "fortiguard"], "type": "str"}
        }
    },
    "vpn_ipsec_fec": {
        "stated": true,
        "options": {
            "mappings": {
                "v_range": [["7.2.0", ""]],
                "type": "list",
                "options": {
                    "bandwidth-bi-threshold": {"v_range": [["7.2.0", ""]], "type": "int"},
                    "bandwidth-down-threshold": {"v_range": [["7.2.0", ""]], "type": "int"},
                    "bandwidth-up-threshold": {"v_range": [["7.2.0", ""]], "type": "int"},
                    "base": {"v_range": [["7.2.0", ""]], "type": "int"},
                    "latency-threshold": {"v_range": [["7.2.0", ""]], "type": "int"},
                    "packet-loss-threshold": {"v_range": [["7.2.0", ""]], "type": "int"},
                    "redundant": {"v_range": [["7.2.0", ""]], "type": "int"},
                    "seqno": {"v_range": [["7.2.0", ""]], "type": "int"}
                },
                "elements": "dict"
            },
            "name": {"v_range": [["7.2.0", ""]], "required": true, "type": "str"}
        }
    },
    "vpn_ipsec_fec_mappings": {
        "stated": true,
        "options": {
            "bandwidth-bi-threshold": {"v_range": [["7.2.0", ""]], "type": "int"},
            "bandwidth-down-threshold": {"v_range": [["7.2.0", ""]], "type": "int"},
            "bandwidth-up-threshold": {"v_range": [["7.2.0", ""]], "type": "int"},
            "base": {"v_range": [["7.2.0", ""]], "type": "int"},
            "latency-threshold": {"v_range": [["7.2.0", ""]], "type": "int"},
            "packet-loss-threshold": {"v_range": [["7.2.0", ""]], "type": "int"},
            "redundant": {"v_range": [["7.2.0", ""]], "type": "int"},
            "seqno": {"v_range": [["7.2.0", ""]], "type": "int"}
        }
    },
    "vpn_ssl_settings": {
        "stated": false,
        "options": {
            "algorithm": {
                "v_range": [["6.2.6", "6.2.12"], ["6.4.2", "7.2.1"], ["7.4.3", ""]],
                "choices": ["default", "high", "low", "medium"],
                "type": "str"
            },
            "auth-session-check-source-ip": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "auth-timeout": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "type": "int"},
            "authentication-rule": {
                "v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]],
                "type": "list",
                "options": {
                    "auth": {
                        "v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]],
                        "choices": ["any", "local", "radius", "ldap", "tacacs+", "peer"],
                        "type": "str"
                    },
                    "cipher": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "choices": ["any", "high", "medium"], "type": "str"},
                    "client-cert": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "groups": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "type": "raw"},
                    "id": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "type": "int"},
                    "portal": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "type": "str"},
                    "realm": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "type": "str"},
                    "source-address": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "type": "raw"},
                    "source-address-negate": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "source-address6": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "type": "raw"},
                    "source-address6-negate": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "source-interface": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "type": "raw"},
                    "user-peer": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "type": "str"},
                    "users": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "type": "raw"}
                },
                "elements": "dict"
            },
            "auto-tunnel-static-route": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "banned-cipher": {
                "v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]],
                "type": "list",
                "choices": [
                    "RSA", "DH", "DHE", "ECDH", "ECDHE", "DSS", "ECDSA", "AES", "AESGCM", "CAMELLIA", "3DES", "SHA1", "SHA256", "SHA384", "STATIC",
                    "CHACHA20", "ARIA", "AESCCM"
                ],
                "elements": "str"
            },
            "check-referer": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "default-portal": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "type": "str"},
            "deflate-compression-level": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "type": "int"},
            "deflate-min-data-size": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "type": "int"},
            "dns-server1": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "type": "str"},
            "dns-server2": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "type": "str"},
            "dns-suffix": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "type": "str"},
            "dtls-hello-timeout": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "type": "int"},
            "dtls-max-proto-ver": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "choices": ["dtls1-0", "dtls1-2"], "type": "str"},
            "dtls-min-proto-ver": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "choices": ["dtls1-0", "dtls1-2"], "type": "str"},
            "dtls-tunnel": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "encode-2f-sequence": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "encrypt-and-store-password": {"v_range": [["6.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "force-two-factor-auth": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "header-x-forwarded-for": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "choices": ["pass", "add", "remove"], "type": "str"},
            "hsts-include-subdomains": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "http-compression": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "http-only-cookie": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "http-request-body-timeout": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "type": "int"},
            "http-request-header-timeout": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "type": "int"},
            "https-redirect": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "idle-timeout": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "type": "int"},
            "ipv6-dns-server1": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "type": "str"},
            "ipv6-dns-server2": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "type": "str"},
            "ipv6-wins-server1": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "type": "str"},
            "ipv6-wins-server2": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "type": "str"},
            "login-attempt-limit": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "type": "int"},
            "login-block-time": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "type": "int"},
            "login-timeout": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "type": "int"},
            "port": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "type": "int"},
            "port-precedence": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "reqclientcert": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "route-source-interface": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", "7.2.0"]], "choices": ["disable", "enable"], "type": "str"},
            "servercert": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "type": "str"},
            "source-address": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "type": "raw"},
            "source-address-negate": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "source-address6": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "type": "raw"},
            "source-address6-negate": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "source-interface": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "type": "raw"},
            "ssl-client-renegotiation": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ssl-insert-empty-fragment": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ssl-max-proto-ver": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "choices": ["tls1-0", "tls1-1", "tls1-2", "tls1-3"], "type": "str"},
            "ssl-min-proto-ver": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "choices": ["tls1-0", "tls1-1", "tls1-2", "tls1-3"], "type": "str"},
            "tlsv1-0": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", "7.2.1"]], "choices": ["disable", "enable"], "type": "str"},
            "tlsv1-1": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", "7.2.1"]], "choices": ["disable", "enable"], "type": "str"},
            "tlsv1-2": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", "7.2.1"]], "choices": ["disable", "enable"], "type": "str"},
            "tlsv1-3": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", "7.2.0"]], "choices": ["disable", "enable"], "type": "str"},
            "transform-backward-slashes": {"v_range": [["6.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "tunnel-connect-without-reauth": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "tunnel-ip-pools": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "type": "raw"},
            "tunnel-ipv6-pools": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "type": "raw"},
            "tunnel-user-session-timeout": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "type": "int"},
            "unsafe-legacy-renegotiation": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "url-obscuration": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "user-peer": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "type": "str"},
            "wins-server1": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "type": "str"},
            "wins-server2": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "type": "str"},
            "x-content-type-options": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "sslv3": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", "6.4.14"]], "choices": ["disable", "enable"], "type": "str"},
            "ssl-big-buffer": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", "6.4.14"]], "choices": ["disable", "enable"], "type": "str"},
            "client-sigalgs": {"v_range": [["6.4.4", ""]], "choices": ["no-rsa-pss", "all"], "type": "str"},
            "ciphersuite": {
                "v_range": [["6.4.8", ""]],
                "type": "list",
                "choices": [
                    "TLS-AES-128-GCM-SHA256", "TLS-AES-256-GCM-SHA384", "TLS-CHACHA20-POLY1305-SHA256", "TLS-AES-128-CCM-SHA256",
                    "TLS-AES-128-CCM-8-SHA256"
                ],
                "elements": "str"
            },
            "dual-stack-mode": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "tunnel-addr-assigned-method": {"v_range": [["7.0.0", ""]], "choices": ["first-available", "round-robin"], "type": "str"},
            "browser-language-detection": {"v_range": [["7.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "saml-redirect-port": {"v_range": [["7.0.1", ""]], "type": "int"},
            "status": {"v_range": [["6.4.8", "6.4.14"], ["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "web-mode-snat": {"v_range": [["7.0.4", "7.2.3"], ["7.4.0", "7.4.1"], ["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ztna-trusted-client": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "dtls-heartbeat-fail-count": {"v_range": [["7.4.0", ""]], "type": "int"},
            "dtls-heartbeat-idle-timeout": {"v_range": [["7.4.0", ""]], "type": "int"},
            "dtls-heartbeat-interval": {"v_range": [["7.4.0", ""]], "type": "int"},
            "server-hostname": {"v_range": [["7.4.0", ""]], "type": "str"}
        }
    },
    "vpn_ssl_settings_authenticationrule": {
        "stated": true,
        "options": {
            "auth": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "choices": ["any", "local", "radius", "ldap", "tacacs+", "peer"], "type": "str"},
            "cipher": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "choices": ["any", "high", "medium"], "type": "str"},
            "client-cert": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "groups": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "type": "raw"},
            "id": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "required": true, "type": "int"},
            "portal": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "type": "str"},
            "realm": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "type": "str"},
            "source-address": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "type": "raw"},
            "source-address-negate": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "source-address6": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "type": "raw"},
            "source-address6-negate": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "source-interface": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "type": "raw"},
            "user-peer": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "type": "str"},
            "users": {"v_range": [["6.2.6", "6.2.12"], ["6.4.2", ""]], "type": "raw"}
        }
    },
    "vpnmgr_node": {
        "stated": true,
        "options": {
            "add-route": {"choices": ["disable", "enable"], "type": "str"},
            "assign-ip": {"choices": ["disable", "enable"], "type": "str"},
            "assign-ip-from": {"choices": ["range", "usrgrp", "dhcp", "name"], "type": "str"},
            "authpasswd": {"no_log": true, "type": "raw"},
            "authusr": {"type": "str"},
            "authusrgrp": {"type": "str"},
            "auto-configuration": {"choices": ["disable", "enable"], "type": "str"},
            "automatic_routing": {"choices": ["disable", "enable"], "type": "str"},
            "banner": {"type": "str"},
            "default-gateway": {"type": "str"},
            "dhcp-server": {"choices": ["disable", "enable"], "type": "str"},
            "dns-mode": {"choices": ["auto", "manual"], "type": "str"},
            "dns-service": {"choices": ["default", "specify", "local"], "type": "str"},
            "domain": {"type": "str"},
            "extgw": {"type": "str"},
            "extgw_hubip": {"type": "str"},
            "extgw_p2_per_net": {"choices": ["disable", "enable"], "type": "str"},
            "extgwip": {"type": "str"},
            "hub_iface": {"type": "raw"},
            "id": {"required": true, "type": "int"},
            "iface": {"type": "raw"},
            "ip-range": {"type": "list", "options": {"end-ip": {"type": "str"}, "id": {"type": "int"}, "start-ip": {"type": "str"}}, "elements": "dict"},
            "ipsec-lease-hold": {"type": "int"},
            "ipv4-dns-server1": {"type": "str"},
            "ipv4-dns-server2": {"type": "str"},
            "ipv4-dns-server3": {"type": "str"},
            "ipv4-end-ip": {"type": "str"},
            "ipv4-exclude-range": {
                "type": "list",
                "options": {"end-ip": {"type": "str"}, "id": {"type": "int"}, "start-ip": {"type": "str"}},
                "elements": "dict"
            },
            "ipv4-netmask": {"type": "str"},
            "ipv4-split-include": {"type": "str"},
            "ipv4-start-ip": {"type": "str"},
            "ipv4-wins-server1": {"type": "str"},
            "ipv4-wins-server2": {"type": "str"},
            "local-gw": {"type": "str"},
            "localid": {"type": "str"},
            "mode-cfg": {"choices": ["disable", "enable"], "type": "str"},
            "mode-cfg-ip-version": {"choices": ["4", "6"], "type": "str"},
            "net-device": {"choices": ["disable", "enable"], "type": "str"},
            "peer": {"type": "raw"},
            "peergrp": {"type": "str"},
            "peerid": {"type": "str"},
            "peertype": {"choices": ["any", "one", "dialup", "peer", "peergrp"], "type": "str"},
            "protected_subnet": {"type": "list", "options": {"addr": {"type": "raw"}, "seq": {"type": "int"}}, "elements": "dict"},
            "public-ip": {"type": "str"},
            "role": {"choices": ["hub", "spoke"], "type": "str"},
            "route-overlap": {"choices": ["use-old", "use-new", "allow"], "type": "str"},
            "spoke-zone": {"type": "raw"},
            "summary_addr": {
                "type": "list",
                "options": {"addr": {"type": "str"}, "priority": {"type": "int"}, "seq": {"type": "int"}},
                "elements": "dict"
            },
            "tunnel-search": {"choices": ["selectors", "nexthop"], "type": "str"},
            "unity-support": {"choices": ["disable", "enable"], "type": "str"},
            "usrgrp": {"type": "str"},
            "vpn-interface-priority": {"type": "int"},
            "vpn-zone": {"type": "raw"},
            "vpntable": {"type": "raw"},
            "xauthtype": {"choices": ["disable", "client", "pap", "chap", "auto"], "type": "str"},
            "exchange-interface-ip": {"v_range": [["6.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "hub-public-ip": {"v_range": [["6.2.2", ""]], "type": "str"},
            "ipv4-split-exclude": {"v_range": [["6.4.6", ""]], "type": "str"},
            "scope member": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "type": "list",
                "options": {
                    "name": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "str"},
                    "vdom": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "type": "str"}
                },
                "elements": "dict"
            },
            "dhcp-ra-giaddr": {"v_range": [["6.4.8", "6.4.14"], ["7.0.4", ""]], "type": "str"},
            "encapsulation": {"v_range": [["7.0.2", ""]], "choices": ["tunnel-mode", "transport-mode"], "type": "str"},
            "ipv4-name": {"v_range": [["6.4.8", "6.4.14"], ["7.0.4", ""]], "type": "str"},
            "l2tp": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "auto-discovery-receiver": {"v_range": [["7.0.8", "7.0.12"], ["7.2.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "auto-discovery-sender": {"v_range": [["7.0.8", "7.0.12"], ["7.2.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "network-id": {"v_range": [["7.0.8", "7.0.12"], ["7.2.3", ""]], "type": "int"},
            "network-overlay": {"v_range": [["7.0.8", "7.0.12"], ["7.2.3", ""]], "choices": ["enable", "disable"], "type": "str"},
            "protocol": {"v_range": [["7.2.5", "7.2.5"], ["7.4.1", ""]], "type": "int"}
        }
    },
    "vpnmgr_node_iprange": {"stated": true, "options": {"end-ip": {"type": "str"}, "id": {"required": true, "type": "int"}, "start-ip": {"type": "str"}}},
    "vpnmgr_node_ipv4excluderange": {
        "stated": true,
        "options": {"end-ip": {"type": "str"}, "id": {"required": true, "type": "int"}, "start-ip": {"type": "str"}}
    },
    "vpnmgr_node_protectedsubnet": {"stated": true, "options": {"addr": {"type": "raw"}, "seq": {"required": true, "type": "int"}}},
    "vpnmgr_node_summaryaddr": {
        "stated": true,
        "options": {"addr": {"type": "str"}, "priority": {"type": "int"}, "seq": {"required": true, "type": "int"}}
    },
    "vpnmgr_vpntable": {
        "stated": true,
        "options": {
            "authmethod": {"choices": ["psk", "rsa-signature", "signature"], "type": "str"},
            "auto-zone-policy": {"choices": ["disable", "enable"], "type": "str"},
            "certificate": {"type": "raw"},
            "description": {"type": "str"},
            "dpd": {"choices": ["disable", "enable", "on-idle", "on-demand"], "type": "str"},
            "dpd-retrycount": {"type": "int"},
            "dpd-retryinterval": {"type": "raw"},
            "fcc-enforcement": {"choices": ["disable", "enable"], "type": "str"},
            "hub2spoke-zone": {"type": "raw"},
            "ike-version": {"choices": ["1", "2"], "type": "str"},
            "ike1dhgroup": {
                "type": "list",
                "choices": ["1", "2", "5", "14", "15", "16", "17", "18", "19", "20", "21", "27", "28", "29", "30", "31", "32"],
                "elements": "str"
            },
            "ike1dpd": {"choices": ["disable", "enable"], "type": "str"},
            "ike1keylifesec": {"no_log": true, "type": "int"},
            "ike1localid": {"type": "str"},
            "ike1mode": {"choices": ["main", "aggressive"], "type": "str"},
            "ike1natkeepalive": {"type": "int"},
            "ike1nattraversal": {"choices": ["disable", "enable", "forced"], "type": "str"},
            "ike1proposal": {
                "choices": [
                    "des-md5", "des-sha1", "3des-md5", "3des-sha1", "aes128-md5", "aes128-sha1", "aes192-md5", "aes192-sha1", "aes256-md5",
                    "aes256-sha1", "des-sha256", "3des-sha256", "aes128-sha256", "aes192-sha256", "aes256-sha256", "des-sha384", "des-sha512",
                    "3des-sha384", "3des-sha512", "aes128-sha384", "aes128-sha512", "aes192-sha384", "aes192-sha512", "aes256-sha384", "aes256-sha512",
                    "aria128-md5", "aria128-sha1", "aria128-sha256", "aria128-sha384", "aria128-sha512", "aria192-md5", "aria192-sha1", "aria192-sha256",
                    "aria192-sha384", "aria192-sha512", "aria256-md5", "aria256-sha1", "aria256-sha256", "aria256-sha384", "aria256-sha512", "seed-md5",
                    "seed-sha1", "seed-sha256", "seed-sha384", "seed-sha512", "aes128gcm-prfsha1", "aes128gcm-prfsha256", "aes128gcm-prfsha384",
                    "aes128gcm-prfsha512", "aes256gcm-prfsha1", "aes256gcm-prfsha256", "aes256gcm-prfsha384", "aes256gcm-prfsha512",
                    "chacha20poly1305-prfsha1", "chacha20poly1305-prfsha256", "chacha20poly1305-prfsha384", "chacha20poly1305-prfsha512"
                ],
                "type": "str"
            },
            "ike2autonego": {"choices": ["disable", "enable"], "type": "str"},
            "ike2dhgroup": {
                "type": "list",
                "choices": ["1", "2", "5", "14", "15", "16", "17", "18", "19", "20", "21", "27", "28", "29", "30", "31", "32"],
                "elements": "str"
            },
            "ike2keepalive": {"choices": ["disable", "enable"], "type": "str"},
            "ike2keylifekbs": {"no_log": true, "type": "int"},
            "ike2keylifesec": {"no_log": true, "type": "int"},
            "ike2keylifetype": {"choices": ["seconds", "kbs", "both"], "type": "str"},
            "ike2proposal": {
                "choices": [
                    "null-md5", "null-sha1", "des-null", "3des-null", "des-md5", "des-sha1", "3des-md5", "3des-sha1", "aes128-md5", "aes128-sha1",
                    "aes192-md5", "aes192-sha1", "aes256-md5", "aes256-sha1", "aes128-null", "aes192-null", "aes256-null", "null-sha256", "des-sha256",
                    "3des-sha256", "aes128-sha256", "aes192-sha256", "aes256-sha256", "des-sha384", "des-sha512", "3des-sha384", "3des-sha512",
                    "aes128-sha384", "aes128-sha512", "aes192-sha384", "aes192-sha512", "aes256-sha384", "aes256-sha512", "null-sha384", "null-sha512",
                    "aria128-null", "aria128-md5", "aria128-sha1", "aria128-sha256", "aria128-sha384", "aria128-sha512", "aria192-null", "aria192-md5",
                    "aria192-sha1", "aria192-sha256", "aria192-sha384", "aria192-sha512", "aria256-null", "aria256-md5", "aria256-sha1",
                    "aria256-sha256", "aria256-sha384", "aria256-sha512", "seed-null", "seed-md5", "seed-sha1", "seed-sha256", "seed-sha384",
                    "seed-sha512", "aes128gcm", "aes256gcm", "chacha20poly1305"
                ],
                "type": "str"
            },
            "inter-vdom": {"choices": ["disable", "enable"], "type": "str"},
            "intf-mode": {"choices": ["off", "on"], "type": "str"},
            "localid-type": {"choices": ["auto", "fqdn", "user-fqdn", "keyid", "address", "asn1dn"], "type": "str"},
            "name": {"required": true, "type": "str"},
            "negotiate-timeout": {"type": "int"},
            "npu-offload": {"choices": ["disable", "enable"], "type": "str"},
            "pfs": {"choices": ["disable", "enable"], "type": "str"},
            "psk-auto-generate": {"choices": ["disable", "enable"], "type": "str"},
            "psksecret": {"no_log": true, "type": "raw"},
            "replay": {"choices": ["disable", "enable"], "type": "str"},
            "rsa-certificate": {"type": "str"},
            "spoke2hub-zone": {"type": "raw"},
            "topology": {"choices": ["meshed", "star", "dialup"], "type": "str"},
            "vpn-zone": {"type": "raw"},
            "network-id": {"v_range": [["6.2.5", ""]], "type": "int"},
            "network-overlay": {"v_range": [["6.2.5", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "vpnsslweb_hostchecksoftware": {
        "stated": true,
        "options": {
            "check-item-list": {
                "type": "list",
                "options": {
                    "action": {"choices": ["deny", "require"], "type": "str"},
                    "id": {"type": "int"},
                    "md5s": {"type": "raw"},
                    "target": {"type": "str"},
                    "type": {"choices": ["file", "registry", "process"], "type": "str"},
                    "version": {"type": "str"}
                },
                "elements": "dict"
            },
            "guid": {"type": "str"},
            "name": {"required": true, "type": "str"},
            "os-type": {"choices": ["macos", "windows"], "type": "str"},
            "type": {"choices": ["av", "fw"], "type": "str"},
            "version": {"type": "str"}
        }
    },
    "vpnsslweb_hostchecksoftware_checkitemlist": {
        "stated": true,
        "options": {
            "action": {"choices": ["deny", "require"], "type": "str"},
            "id": {"required": true, "type": "int"},
            "md5s": {"type": "raw"},
            "target": {"type": "str"},
            "type": {"choices": ["file", "registry", "process"], "type": "str"},
            "version": {"type": "str"}
        }
    },
    "vpnsslweb_portal": {
        "stated": true,
        "options": {
            "allow-user-access": {
                "type": "list",
                "choices": ["web", "ftp", "telnet", "smb", "vnc", "rdp", "ssh", "ping", "citrix", "portforward", "sftp"],
                "elements": "str"
            },
            "auto-connect": {"choices": ["disable", "enable"], "type": "str"},
            "bookmark-group": {
                "type": "list",
                "options": {
                    "bookmarks": {
                        "type": "list",
                        "options": {
                            "additional-params": {"type": "str"},
                            "apptype": {
                                "choices": ["web", "telnet", "ssh", "ftp", "smb", "vnc", "rdp", "citrix", "rdpnative", "portforward", "sftp"],
                                "type": "str"
                            },
                            "description": {"type": "str"},
                            "folder": {"type": "str"},
                            "form-data": {"type": "list", "options": {"name": {"type": "str"}, "value": {"type": "str"}}, "elements": "dict"},
                            "host": {"type": "str"},
                            "listening-port": {"type": "int"},
                            "load-balancing-info": {"type": "str"},
                            "logon-password": {"no_log": true, "type": "raw"},
                            "logon-user": {"type": "str"},
                            "name": {"type": "str"},
                            "port": {"type": "int"},
                            "preconnection-blob": {"type": "str"},
                            "preconnection-id": {"type": "int"},
                            "remote-port": {"type": "int"},
                            "security": {"choices": ["rdp", "nla", "tls", "any"], "type": "str"},
                            "server-layout": {
                                "choices": [
                                    "en-us-qwerty", "de-de-qwertz", "fr-fr-azerty", "it-it-qwerty", "sv-se-qwerty", "failsafe", "en-gb-qwerty",
                                    "es-es-qwerty", "fr-ch-qwertz", "ja-jp-qwerty", "pt-br-qwerty", "tr-tr-qwerty", "fr-ca-qwerty"
                                ],
                                "type": "str"
                            },
                            "show-status-window": {"choices": ["disable", "enable"], "type": "str"},
                            "sso": {"choices": ["disable", "static", "auto"], "type": "str"},
                            "sso-credential": {"choices": ["sslvpn-login", "alternative"], "type": "str"},
                            "sso-credential-sent-once": {"choices": ["disable", "enable"], "type": "str"},
                            "sso-password": {"no_log": true, "type": "raw"},
                            "sso-username": {"type": "str"},
                            "url": {"type": "str"},
                            "domain": {"v_range": [["6.4.2", ""]], "type": "str"},
                            "color-depth": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["8", "16", "32"], "type": "str"},
                            "height": {"v_range": [["7.0.3", ""]], "type": "int"},
                            "keyboard-layout": {
                                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                                "choices": [
                                    "ar", "da", "de", "de-ch", "en-gb", "en-uk", "en-us", "es", "fi", "fr", "fr-be", "fr-ca", "fr-ch", "hr", "hu", "it",
                                    "ja", "lt", "lv", "mk", "no", "pl", "pt", "pt-br", "ru", "sl", "sv", "tk", "tr", "fr-ca-m", "wg", "ar-101", "ar-102",
                                    "ar-102-azerty", "can-mul", "cz", "cz-qwerty", "cz-pr", "nl", "de-ibm", "en-uk-ext", "en-us-dvorak", "es-var",
                                    "fi-sami", "hu-101", "it-142", "ko", "lt-ibm", "lt-std", "lav-std", "lav-leg", "mk-std", "no-sami", "pol-214",
                                    "pol-pr", "pt-br-abnt2", "ru-mne", "ru-t", "sv-sami", "tuk", "tur-f", "tur-q", "zh-sym-sg-us", "zh-sym-us",
                                    "zh-tr-hk", "zh-tr-mo", "zh-tr-us", "fr-apple", "la-am", "ja-106"
                                ],
                                "type": "str"
                            },
                            "restricted-admin": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "send-preconnection-id": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "width": {"v_range": [["7.0.3", ""]], "type": "int"},
                            "vnc-keyboard-layout": {
                                "v_range": [["7.2.2", ""]],
                                "choices": [
                                    "da", "de", "de-ch", "en-uk", "es", "fi", "fr", "fr-be", "it", "no", "pt", "sv", "nl", "en-uk-ext", "it-142",
                                    "pt-br-abnt2", "default", "fr-ca-mul", "gd", "us-intl"
                                ],
                                "type": "str"
                            }
                        },
                        "elements": "dict"
                    },
                    "name": {"type": "str"}
                },
                "elements": "dict"
            },
            "custom-lang": {"type": "str"},
            "customize-forticlient-download-url": {"choices": ["disable", "enable"], "type": "str"},
            "display-bookmark": {"choices": ["disable", "enable"], "type": "str"},
            "display-connection-tools": {"choices": ["disable", "enable"], "type": "str"},
            "display-history": {"choices": ["disable", "enable"], "type": "str"},
            "display-status": {"choices": ["disable", "enable"], "type": "str"},
            "dns-server1": {"type": "str"},
            "dns-server2": {"type": "str"},
            "dns-suffix": {"type": "str"},
            "exclusive-routing": {"choices": ["disable", "enable"], "type": "str"},
            "forticlient-download": {"choices": ["disable", "enable"], "type": "str"},
            "forticlient-download-method": {"choices": ["direct", "ssl-vpn"], "type": "str"},
            "heading": {"type": "str"},
            "hide-sso-credential": {"choices": ["disable", "enable"], "type": "str"},
            "host-check": {"choices": ["none", "av", "fw", "av-fw", "custom"], "type": "str"},
            "host-check-interval": {"type": "int"},
            "host-check-policy": {"type": "raw"},
            "ip-mode": {"choices": ["range", "user-group", "dhcp", "no-ip"], "type": "str"},
            "ip-pools": {"type": "raw"},
            "ipv6-dns-server1": {"type": "str"},
            "ipv6-dns-server2": {"type": "str"},
            "ipv6-exclusive-routing": {"choices": ["disable", "enable"], "type": "str"},
            "ipv6-pools": {"type": "raw"},
            "ipv6-service-restriction": {"choices": ["disable", "enable"], "type": "str"},
            "ipv6-split-tunneling": {"choices": ["disable", "enable"], "type": "str"},
            "ipv6-split-tunneling-routing-address": {"type": "raw"},
            "ipv6-tunnel-mode": {"choices": ["disable", "enable"], "type": "str"},
            "ipv6-wins-server1": {"type": "str"},
            "ipv6-wins-server2": {"type": "str"},
            "keep-alive": {"choices": ["disable", "enable"], "type": "str"},
            "limit-user-logins": {"choices": ["disable", "enable"], "type": "str"},
            "mac-addr-action": {"choices": ["deny", "allow"], "type": "str"},
            "mac-addr-check": {"choices": ["disable", "enable"], "type": "str"},
            "mac-addr-check-rule": {
                "type": "list",
                "options": {"mac-addr-list": {"type": "raw"}, "mac-addr-mask": {"type": "int"}, "name": {"type": "str"}},
                "elements": "dict"
            },
            "macos-forticlient-download-url": {"type": "str"},
            "name": {"required": true, "type": "str"},
            "os-check": {"choices": ["disable", "enable"], "type": "str"},
            "redir-url": {"type": "str"},
            "save-password": {"choices": ["disable", "enable"], "type": "str"},
            "service-restriction": {"choices": ["disable", "enable"], "type": "str"},
            "skip-check-for-unsupported-browser": {"v_range": [["6.0.0", "7.2.1"]], "choices": ["disable", "enable"], "type": "str"},
            "skip-check-for-unsupported-os": {"choices": ["disable", "enable"], "type": "str"},
            "smb-ntlmv1-auth": {"choices": ["disable", "enable"], "type": "str"},
            "smbv1": {"v_range": [["6.0.0", "7.2.1"]], "choices": ["disable", "enable"], "type": "str"},
            "split-dns": {
                "type": "list",
                "options": {
                    "dns-server1": {"type": "str"},
                    "dns-server2": {"type": "str"},
                    "domains": {"type": "str"},
                    "id": {"type": "int"},
                    "ipv6-dns-server1": {"type": "str"},
                    "ipv6-dns-server2": {"type": "str"}
                },
                "elements": "dict"
            },
            "split-tunneling": {"choices": ["disable", "enable"], "type": "str"},
            "split-tunneling-routing-address": {"type": "raw"},
            "theme": {
                "choices": [
                    "gray", "blue", "orange", "crimson", "steelblue", "darkgrey", "green", "melongene", "red", "mariner", "neutrino", "jade", "graphite",
                    "dark-matter", "onyx", "eclipse", "jet-stream", "security-fabric"
                ],
                "type": "str"
            },
            "tunnel-mode": {"choices": ["disable", "enable"], "type": "str"},
            "user-bookmark": {"choices": ["disable", "enable"], "type": "str"},
            "user-group-bookmark": {"choices": ["disable", "enable"], "type": "str"},
            "web-mode": {"choices": ["disable", "enable"], "type": "str"},
            "windows-forticlient-download-url": {"type": "str"},
            "wins-server1": {"type": "str"},
            "wins-server2": {"type": "str"},
            "skip-check-for-browser": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "smb-max-version": {"v_range": [["6.2.0", ""]], "choices": ["smbv1", "smbv2", "smbv3"], "type": "str"},
            "smb-min-version": {"v_range": [["6.2.0", ""]], "choices": ["smbv1", "smbv2", "smbv3"], "type": "str"},
            "virtual-desktop-logout-when-browser-close": {"v_range": [["6.2.0", "6.2.12"]], "choices": ["disable", "enable"], "type": "str"},
            "virtual-desktop-clipboard-share": {"v_range": [["6.2.0", "6.2.12"]], "choices": ["disable", "enable"], "type": "str"},
            "virtual-desktop-desktop-switch": {"v_range": [["6.2.0", "6.2.12"]], "choices": ["disable", "enable"], "type": "str"},
            "virtual-desktop": {"v_range": [["6.2.0", "6.2.12"]], "choices": ["disable", "enable"], "type": "str"},
            "virtual-desktop-network-share-access": {"v_range": [["6.2.0", "6.2.12"]], "choices": ["disable", "enable"], "type": "str"},
            "virtual-desktop-printing": {"v_range": [["6.2.0", "6.2.12"]], "choices": ["disable", "enable"], "type": "str"},
            "virtual-desktop-app-list": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"},
            "virtual-desktop-removable-media-access": {"v_range": [["6.2.0", "6.2.12"]], "choices": ["disable", "enable"], "type": "str"},
            "transform-backward-slashes": {"v_range": [["6.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ipv6-split-tunneling-routing-negate": {"v_range": [["6.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "split-tunneling-routing-negate": {"v_range": [["6.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "os-check-list": {
                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                "type": "dict",
                "options": {
                    "action": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["allow", "check-up-to-date", "deny"], "type": "str"},
                    "latest-patch-level": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "name": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "tolerance": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "minor-version": {"v_range": [["7.6.0", ""]], "type": "int"}
                }
            },
            "use-sdwan": {"v_range": [["6.2.7", "6.2.12"], ["6.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "prefer-ipv6-dns": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "rewrite-ip-uri-ui": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "clipboard": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "default-window-height": {"v_range": [["7.0.4", ""]], "type": "int"},
            "default-window-width": {"v_range": [["7.0.4", ""]], "type": "int"},
            "dhcp-ip-overlap": {"v_range": [["7.0.4", "7.0.12"], ["7.2.1", ""]], "choices": ["use-old", "use-new"], "type": "str"},
            "client-src-range": {"v_range": [["7.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "dhcp-ra-giaddr": {"v_range": [["7.2.2", ""]], "type": "str"},
            "dhcp6-ra-linkaddr": {"v_range": [["7.2.2", ""]], "type": "str"},
            "landing-page": {
                "v_range": [["7.4.0", ""]],
                "type": "dict",
                "options": {
                    "form-data": {
                        "v_range": [["7.4.0", ""]],
                        "type": "list",
                        "options": {"name": {"v_range": [["7.4.0", ""]], "type": "str"}, "value": {"v_range": [["7.4.0", ""]], "type": "str"}},
                        "elements": "dict"
                    },
                    "logout-url": {"v_range": [["7.4.0", "7.4.1"]], "type": "str"},
                    "sso": {"v_range": [["7.4.0", ""]], "choices": ["disable", "static", "auto"], "type": "str"},
                    "sso-credential": {"v_range": [["7.4.0", ""]], "choices": ["sslvpn-login", "alternative"], "type": "str"},
                    "sso-password": {"v_range": [["7.4.0", ""]], "no_log": true, "type": "raw"},
                    "sso-username": {"v_range": [["7.4.0", ""]], "type": "str"},
                    "url": {"v_range": [["7.4.0", ""]], "type": "str"}
                }
            },
            "landing-page-mode": {"v_range": [["7.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "default-protocol": {"v_range": [["7.4.1", ""]], "choices": ["web", "ftp", "telnet", "smb", "vnc", "rdp", "ssh", "sftp"], "type": "str"},
            "focus-bookmark": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "dhcp-reservation": {"v_range": [["7.6.0", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "vpnsslweb_portal_bookmarkgroup": {
        "stated": true,
        "options": {
            "bookmarks": {
                "type": "list",
                "options": {
                    "additional-params": {"type": "str"},
                    "apptype": {
                        "choices": ["web", "telnet", "ssh", "ftp", "smb", "vnc", "rdp", "citrix", "rdpnative", "portforward", "sftp"],
                        "type": "str"
                    },
                    "description": {"type": "str"},
                    "folder": {"type": "str"},
                    "form-data": {"type": "list", "options": {"name": {"type": "str"}, "value": {"type": "str"}}, "elements": "dict"},
                    "host": {"type": "str"},
                    "listening-port": {"type": "int"},
                    "load-balancing-info": {"type": "str"},
                    "logon-password": {"no_log": true, "type": "raw"},
                    "logon-user": {"type": "str"},
                    "name": {"type": "str"},
                    "port": {"type": "int"},
                    "preconnection-blob": {"type": "str"},
                    "preconnection-id": {"type": "int"},
                    "remote-port": {"type": "int"},
                    "security": {"choices": ["rdp", "nla", "tls", "any"], "type": "str"},
                    "server-layout": {
                        "choices": [
                            "en-us-qwerty", "de-de-qwertz", "fr-fr-azerty", "it-it-qwerty", "sv-se-qwerty", "failsafe", "en-gb-qwerty", "es-es-qwerty",
                            "fr-ch-qwertz", "ja-jp-qwerty", "pt-br-qwerty", "tr-tr-qwerty", "fr-ca-qwerty"
                        ],
                        "type": "str"
                    },
                    "show-status-window": {"choices": ["disable", "enable"], "type": "str"},
                    "sso": {"choices": ["disable", "static", "auto"], "type": "str"},
                    "sso-credential": {"choices": ["sslvpn-login", "alternative"], "type": "str"},
                    "sso-credential-sent-once": {"choices": ["disable", "enable"], "type": "str"},
                    "sso-password": {"no_log": true, "type": "raw"},
                    "sso-username": {"type": "str"},
                    "url": {"type": "str"},
                    "domain": {"v_range": [["6.4.2", ""]], "type": "str"},
                    "color-depth": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["8", "16", "32"], "type": "str"},
                    "height": {"v_range": [["7.0.3", ""]], "type": "int"},
                    "keyboard-layout": {
                        "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                        "choices": [
                            "ar", "da", "de", "de-ch", "en-gb", "en-uk", "en-us", "es", "fi", "fr", "fr-be", "fr-ca", "fr-ch", "hr", "hu", "it", "ja",
                            "lt", "lv", "mk", "no", "pl", "pt", "pt-br", "ru", "sl", "sv", "tk", "tr", "fr-ca-m", "wg", "ar-101", "ar-102",
                            "ar-102-azerty", "can-mul", "cz", "cz-qwerty", "cz-pr", "nl", "de-ibm", "en-uk-ext", "en-us-dvorak", "es-var", "fi-sami",
                            "hu-101", "it-142", "ko", "lt-ibm", "lt-std", "lav-std", "lav-leg", "mk-std", "no-sami", "pol-214", "pol-pr", "pt-br-abnt2",
                            "ru-mne", "ru-t", "sv-sami", "tuk", "tur-f", "tur-q", "zh-sym-sg-us", "zh-sym-us", "zh-tr-hk", "zh-tr-mo", "zh-tr-us",
                            "fr-apple", "la-am", "ja-106"
                        ],
                        "type": "str"
                    },
                    "restricted-admin": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "send-preconnection-id": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "width": {"v_range": [["7.0.3", ""]], "type": "int"},
                    "vnc-keyboard-layout": {
                        "v_range": [["7.2.2", ""]],
                        "choices": [
                            "da", "de", "de-ch", "en-uk", "es", "fi", "fr", "fr-be", "it", "no", "pt", "sv", "nl", "en-uk-ext", "it-142", "pt-br-abnt2",
                            "default", "fr-ca-mul", "gd", "us-intl"
                        ],
                        "type": "str"
                    }
                },
                "elements": "dict"
            },
            "name": {"required": true, "type": "str"}
        }
    },
    "vpnsslweb_portal_bookmarkgroup_bookmarks": {
        "stated": true,
        "options": {
            "additional-params": {"type": "str"},
            "apptype": {"choices": ["web", "telnet", "ssh", "ftp", "smb", "vnc", "rdp", "citrix", "rdpnative", "portforward", "sftp"], "type": "str"},
            "description": {"type": "str"},
            "folder": {"type": "str"},
            "form-data": {"type": "list", "options": {"name": {"type": "str"}, "value": {"type": "str"}}, "elements": "dict"},
            "host": {"type": "str"},
            "listening-port": {"type": "int"},
            "load-balancing-info": {"type": "str"},
            "logon-password": {"no_log": true, "type": "raw"},
            "logon-user": {"type": "str"},
            "name": {"required": true, "type": "str"},
            "port": {"type": "int"},
            "preconnection-blob": {"type": "str"},
            "preconnection-id": {"type": "int"},
            "remote-port": {"type": "int"},
            "security": {"choices": ["rdp", "nla", "tls", "any"], "type": "str"},
            "server-layout": {
                "choices": [
                    "en-us-qwerty", "de-de-qwertz", "fr-fr-azerty", "it-it-qwerty", "sv-se-qwerty", "failsafe", "en-gb-qwerty", "es-es-qwerty",
                    "fr-ch-qwertz", "ja-jp-qwerty", "pt-br-qwerty", "tr-tr-qwerty", "fr-ca-qwerty"
                ],
                "type": "str"
            },
            "show-status-window": {"choices": ["disable", "enable"], "type": "str"},
            "sso": {"choices": ["disable", "static", "auto"], "type": "str"},
            "sso-credential": {"choices": ["sslvpn-login", "alternative"], "type": "str"},
            "sso-credential-sent-once": {"choices": ["disable", "enable"], "type": "str"},
            "sso-password": {"no_log": true, "type": "raw"},
            "sso-username": {"type": "str"},
            "url": {"type": "str"},
            "domain": {"v_range": [["6.4.2", ""]], "type": "str"},
            "color-depth": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["8", "16", "32"], "type": "str"},
            "height": {"v_range": [["7.0.3", ""]], "type": "int"},
            "keyboard-layout": {
                "v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]],
                "choices": [
                    "ar", "da", "de", "de-ch", "en-gb", "en-uk", "en-us", "es", "fi", "fr", "fr-be", "fr-ca", "fr-ch", "hr", "hu", "it", "ja", "lt",
                    "lv", "mk", "no", "pl", "pt", "pt-br", "ru", "sl", "sv", "tk", "tr", "fr-ca-m", "wg", "ar-101", "ar-102", "ar-102-azerty", "can-mul",
                    "cz", "cz-qwerty", "cz-pr", "nl", "de-ibm", "en-uk-ext", "en-us-dvorak", "es-var", "fi-sami", "hu-101", "it-142", "ko", "lt-ibm",
                    "lt-std", "lav-std", "lav-leg", "mk-std", "no-sami", "pol-214", "pol-pr", "pt-br-abnt2", "ru-mne", "ru-t", "sv-sami", "tuk", "tur-f",
                    "tur-q", "zh-sym-sg-us", "zh-sym-us", "zh-tr-hk", "zh-tr-mo", "zh-tr-us", "fr-apple", "la-am", "ja-106"
                ],
                "type": "str"
            },
            "restricted-admin": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "send-preconnection-id": {"v_range": [["6.4.7", "6.4.14"], ["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "width": {"v_range": [["7.0.3", ""]], "type": "int"},
            "vnc-keyboard-layout": {
                "v_range": [["7.2.2", ""]],
                "choices": [
                    "da", "de", "de-ch", "en-uk", "es", "fi", "fr", "fr-be", "it", "no", "pt", "sv", "nl", "en-uk-ext", "it-142", "pt-br-abnt2",
                    "default", "fr-ca-mul", "gd", "us-intl"
                ],
                "type": "str"
            }
        }
    },
    "vpnsslweb_portal_bookmarkgroup_bookmarks_formdata": {
        "stated": true,
        "options": {"name": {"required": true, "type": "str"}, "value": {"type": "str"}}
    },
    "vpnsslweb_portal_landingpage": {
        "stated": false,
        "options": {
            "form-data": {
                "v_range": [["7.4.0", ""]],
                "type": "list",
                "options": {"name": {"v_range": [["7.4.0", ""]], "type": "str"}, "value": {"v_range": [["7.4.0", ""]], "type": "str"}},
                "elements": "dict"
            },
            "logout-url": {"v_range": [["7.4.0", "7.4.1"]], "type": "str"},
            "sso": {"v_range": [["7.4.0", ""]], "choices": ["disable", "static", "auto"], "type": "str"},
            "sso-credential": {"v_range": [["7.4.0", ""]], "choices": ["sslvpn-login", "alternative"], "type": "str"},
            "sso-password": {"v_range": [["7.4.0", ""]], "no_log": true, "type": "raw"},
            "sso-username": {"v_range": [["7.4.0", ""]], "type": "str"},
            "url": {"v_range": [["7.4.0", ""]], "type": "str"}
        }
    },
    "vpnsslweb_portal_landingpage_formdata": {
        "stated": true,
        "options": {"name": {"v_range": [["7.4.0", ""]], "required": true, "type": "str"}, "value": {"v_range": [["7.4.0", ""]], "type": "str"}}
    },
    "vpnsslweb_portal_macaddrcheckrule": {
        "stated": true,
        "options": {"mac-addr-list": {"type": "raw"}, "mac-addr-mask": {"type": "int"}, "name": {"required": true, "type": "str"}}
    },
    "vpnsslweb_portal_oschecklist": {
        "stated": false,
        "options": {
            "action": {"choices": ["allow", "check-up-to-date", "deny"], "type": "str"},
            "latest-patch-level": {"type": "str"},
            "name": {"type": "str"},
            "tolerance": {"type": "int"},
            "minor-version": {"v_range": [["7.6.0", ""]], "type": "int"}
        }
    },
    "vpnsslweb_portal_splitdns": {
        "stated": true,
        "options": {
            "dns-server1": {"type": "str"},
            "dns-server2": {"type": "str"},
            "domains": {"type": "str"},
            "id": {"required": true, "type": "int"},
            "ipv6-dns-server1": {"type": "str"},
            "ipv6-dns-server2": {"type": "str"}
        }
    },
    "vpnsslweb_realm": {
        "stated": true,
        "options": {
            "login-page": {"type": "str"},
            "max-concurrent-user": {"type": "int"},
            "url-path": {"type": "str"},
            "virtual-host": {"type": "str"},
            "nas-ip": {"v_range": [["6.4.0", ""]], "type": "str"},
            "radius-server": {"v_range": [["6.4.0", ""]], "type": "str"},
            "radius-port": {"v_range": [["6.4.2", ""]], "type": "int"},
            "virtual-host-only": {"v_range": [["6.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "virtual-host-server-cert": {"v_range": [["7.0.2", ""]], "type": "str"}
        }
    },
    "vpnsslweb_virtualdesktopapplist": {
        "stated": true,
        "options": {
            "action": {"v_range": [["6.2.0", "6.2.12"]], "choices": ["block", "allow"], "type": "str"},
            "apps": {
                "v_range": [["6.2.0", "6.2.12"]],
                "type": "list",
                "options": {"md5s": {"v_range": [["6.2.0", "6.2.12"]], "type": "raw"}, "name": {"v_range": [["6.2.0", "6.2.12"]], "type": "str"}},
                "elements": "dict"
            },
            "name": {"v_range": [["6.2.0", "6.2.12"]], "required": true, "type": "str"}
        }
    },
    "vpnsslweb_virtualdesktopapplist_apps": {
        "stated": true,
        "options": {
            "md5s": {"v_range": [["6.2.0", "6.2.12"]], "type": "raw"},
            "name": {"v_range": [["6.2.0", "6.2.12"]], "required": true, "type": "str"}
        }
    },
    "waf_mainclass": {"stated": true, "options": {"id": {"required": true, "type": "int"}, "name": {"type": "str"}}},
    "waf_profile": {
        "stated": true,
        "options": {
            "comment": {"type": "str"},
            "extended-log": {"choices": ["disable", "enable"], "type": "str"},
            "external": {"choices": ["disable", "enable"], "type": "str"},
            "name": {"required": true, "type": "str"},
            "url-access": {
                "type": "list",
                "options": {
                    "access-pattern": {
                        "type": "list",
                        "options": {
                            "id": {"type": "int"},
                            "negate": {"choices": ["disable", "enable"], "type": "str"},
                            "pattern": {"type": "str"},
                            "regex": {"choices": ["disable", "enable"], "type": "str"},
                            "srcaddr": {"type": "str"}
                        },
                        "elements": "dict"
                    },
                    "action": {"choices": ["bypass", "permit", "block"], "type": "str"},
                    "address": {"type": "str"},
                    "id": {"type": "int"},
                    "log": {"choices": ["disable", "enable"], "type": "str"},
                    "severity": {"choices": ["low", "medium", "high"], "type": "str"}
                },
                "elements": "dict"
            },
            "address-list": {
                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                "type": "dict",
                "options": {
                    "blocked-address": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "raw"},
                    "blocked-log": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "severity": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["low", "medium", "high"], "type": "str"},
                    "status": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "trusted-address": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "raw"}
                }
            },
            "constraint": {
                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                "type": "dict",
                "options": {
                    "content-length": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "type": "dict",
                        "options": {
                            "action": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["allow", "block"], "type": "str"},
                            "length": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                            "log": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "severity": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["low", "medium", "high"], "type": "str"},
                            "status": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"}
                        }
                    },
                    "exception": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "type": "list",
                        "options": {
                            "address": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                            "content-length": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "header-length": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "hostname": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "id": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                            "line-length": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "malformed": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "max-cookie": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "max-header-line": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "max-range-segment": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "max-url-param": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "method": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "param-length": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "pattern": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                            "regex": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "url-param-length": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "version": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"}
                        },
                        "elements": "dict"
                    },
                    "header-length": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "type": "dict",
                        "options": {
                            "action": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["allow", "block"], "type": "str"},
                            "length": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                            "log": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "severity": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["low", "medium", "high"], "type": "str"},
                            "status": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"}
                        }
                    },
                    "hostname": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "type": "dict",
                        "options": {
                            "action": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["allow", "block"], "type": "str"},
                            "log": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "severity": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["low", "medium", "high"], "type": "str"},
                            "status": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"}
                        }
                    },
                    "line-length": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "type": "dict",
                        "options": {
                            "action": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["allow", "block"], "type": "str"},
                            "length": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                            "log": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "severity": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["low", "medium", "high"], "type": "str"},
                            "status": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"}
                        }
                    },
                    "malformed": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "type": "dict",
                        "options": {
                            "action": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["allow", "block"], "type": "str"},
                            "log": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "severity": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["low", "medium", "high"], "type": "str"},
                            "status": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"}
                        }
                    },
                    "max-cookie": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "type": "dict",
                        "options": {
                            "action": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["allow", "block"], "type": "str"},
                            "log": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "max-cookie": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                            "severity": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["low", "medium", "high"], "type": "str"},
                            "status": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"}
                        }
                    },
                    "max-header-line": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "type": "dict",
                        "options": {
                            "action": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["allow", "block"], "type": "str"},
                            "log": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "max-header-line": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                            "severity": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["low", "medium", "high"], "type": "str"},
                            "status": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"}
                        }
                    },
                    "max-range-segment": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "type": "dict",
                        "options": {
                            "action": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["allow", "block"], "type": "str"},
                            "log": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "max-range-segment": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                            "severity": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["low", "medium", "high"], "type": "str"},
                            "status": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"}
                        }
                    },
                    "max-url-param": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "type": "dict",
                        "options": {
                            "action": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["allow", "block"], "type": "str"},
                            "log": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "max-url-param": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                            "severity": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["low", "medium", "high"], "type": "str"},
                            "status": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"}
                        }
                    },
                    "method": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "type": "dict",
                        "options": {
                            "action": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["allow", "block"], "type": "str"},
                            "log": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "severity": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["low", "medium", "high"], "type": "str"},
                            "status": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"}
                        }
                    },
                    "param-length": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "type": "dict",
                        "options": {
                            "action": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["allow", "block"], "type": "str"},
                            "length": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                            "log": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "severity": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["low", "medium", "high"], "type": "str"},
                            "status": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"}
                        }
                    },
                    "url-param-length": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "type": "dict",
                        "options": {
                            "action": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["allow", "block"], "type": "str"},
                            "length": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                            "log": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "severity": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["low", "medium", "high"], "type": "str"},
                            "status": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"}
                        }
                    },
                    "version": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "type": "dict",
                        "options": {
                            "action": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["allow", "block"], "type": "str"},
                            "log": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "severity": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["low", "medium", "high"], "type": "str"},
                            "status": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"}
                        }
                    }
                }
            },
            "method": {
                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                "type": "dict",
                "options": {
                    "default-allowed-methods": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "type": "list",
                        "choices": ["delete", "get", "head", "options", "post", "put", "trace", "others", "connect"],
                        "elements": "str"
                    },
                    "log": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "method-policy": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "type": "list",
                        "options": {
                            "address": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                            "allowed-methods": {
                                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                                "type": "list",
                                "choices": ["delete", "get", "head", "options", "post", "put", "trace", "others", "connect"],
                                "elements": "str"
                            },
                            "id": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                            "pattern": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                            "regex": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"}
                        },
                        "elements": "dict"
                    },
                    "severity": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["low", "medium", "high"], "type": "str"},
                    "status": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"}
                }
            },
            "signature": {
                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                "type": "dict",
                "options": {
                    "credit-card-detection-threshold": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "custom-signature": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "type": "list",
                        "options": {
                            "action": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["allow", "block", "erase"], "type": "str"},
                            "case-sensitivity": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "direction": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["request", "response"], "type": "str"},
                            "log": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "name": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                            "pattern": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                            "severity": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["low", "medium", "high"], "type": "str"},
                            "status": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "target": {
                                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                                "type": "list",
                                "choices": [
                                    "arg", "arg-name", "req-body", "req-cookie", "req-cookie-name", "req-filename", "req-header", "req-header-name",
                                    "req-raw-uri", "req-uri", "resp-body", "resp-hdr", "resp-status"
                                ],
                                "elements": "str"
                            }
                        },
                        "elements": "dict"
                    },
                    "disabled-signature": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "raw"},
                    "disabled-sub-class": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "raw"},
                    "main-class": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "type": "dict",
                        "options": {
                            "action": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["allow", "block", "erase"], "type": "str"},
                            "id": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                            "log": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "severity": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["low", "medium", "high"], "type": "str"},
                            "status": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"}
                        }
                    }
                }
            }
        }
    },
    "waf_profile_addresslist": {
        "stated": false,
        "options": {
            "blocked-address": {"type": "raw"},
            "blocked-log": {"choices": ["disable", "enable"], "type": "str"},
            "severity": {"choices": ["low", "medium", "high"], "type": "str"},
            "status": {"choices": ["disable", "enable"], "type": "str"},
            "trusted-address": {"type": "raw"}
        }
    },
    "waf_profile_constraint": {
        "stated": false,
        "options": {
            "content-length": {
                "type": "dict",
                "options": {
                    "action": {"choices": ["allow", "block"], "type": "str"},
                    "length": {"type": "int"},
                    "log": {"choices": ["disable", "enable"], "type": "str"},
                    "severity": {"choices": ["low", "medium", "high"], "type": "str"},
                    "status": {"choices": ["disable", "enable"], "type": "str"}
                }
            },
            "exception": {
                "type": "list",
                "options": {
                    "address": {"type": "str"},
                    "content-length": {"choices": ["disable", "enable"], "type": "str"},
                    "header-length": {"choices": ["disable", "enable"], "type": "str"},
                    "hostname": {"choices": ["disable", "enable"], "type": "str"},
                    "id": {"type": "int"},
                    "line-length": {"choices": ["disable", "enable"], "type": "str"},
                    "malformed": {"choices": ["disable", "enable"], "type": "str"},
                    "max-cookie": {"choices": ["disable", "enable"], "type": "str"},
                    "max-header-line": {"choices": ["disable", "enable"], "type": "str"},
                    "max-range-segment": {"choices": ["disable", "enable"], "type": "str"},
                    "max-url-param": {"choices": ["disable", "enable"], "type": "str"},
                    "method": {"choices": ["disable", "enable"], "type": "str"},
                    "param-length": {"choices": ["disable", "enable"], "type": "str"},
                    "pattern": {"type": "str"},
                    "regex": {"choices": ["disable", "enable"], "type": "str"},
                    "url-param-length": {"choices": ["disable", "enable"], "type": "str"},
                    "version": {"choices": ["disable", "enable"], "type": "str"}
                },
                "elements": "dict"
            },
            "header-length": {
                "type": "dict",
                "options": {
                    "action": {"choices": ["allow", "block"], "type": "str"},
                    "length": {"type": "int"},
                    "log": {"choices": ["disable", "enable"], "type": "str"},
                    "severity": {"choices": ["low", "medium", "high"], "type": "str"},
                    "status": {"choices": ["disable", "enable"], "type": "str"}
                }
            },
            "hostname": {
                "type": "dict",
                "options": {
                    "action": {"choices": ["allow", "block"], "type": "str"},
                    "log": {"choices": ["disable", "enable"], "type": "str"},
                    "severity": {"choices": ["low", "medium", "high"], "type": "str"},
                    "status": {"choices": ["disable", "enable"], "type": "str"}
                }
            },
            "line-length": {
                "type": "dict",
                "options": {
                    "action": {"choices": ["allow", "block"], "type": "str"},
                    "length": {"type": "int"},
                    "log": {"choices": ["disable", "enable"], "type": "str"},
                    "severity": {"choices": ["low", "medium", "high"], "type": "str"},
                    "status": {"choices": ["disable", "enable"], "type": "str"}
                }
            },
            "malformed": {
                "type": "dict",
                "options": {
                    "action": {"choices": ["allow", "block"], "type": "str"},
                    "log": {"choices": ["disable", "enable"], "type": "str"},
                    "severity": {"choices": ["low", "medium", "high"], "type": "str"},
                    "status": {"choices": ["disable", "enable"], "type": "str"}
                }
            },
            "max-cookie": {
                "type": "dict",
                "options": {
                    "action": {"choices": ["allow", "block"], "type": "str"},
                    "log": {"choices": ["disable", "enable"], "type": "str"},
                    "max-cookie": {"type": "int"},
                    "severity": {"choices": ["low", "medium", "high"], "type": "str"},
                    "status": {"choices": ["disable", "enable"], "type": "str"}
                }
            },
            "max-header-line": {
                "type": "dict",
                "options": {
                    "action": {"choices": ["allow", "block"], "type": "str"},
                    "log": {"choices": ["disable", "enable"], "type": "str"},
                    "max-header-line": {"type": "int"},
                    "severity": {"choices": ["low", "medium", "high"], "type": "str"},
                    "status": {"choices": ["disable", "enable"], "type": "str"}
                }
            },
            "max-range-segment": {
                "type": "dict",
                "options": {
                    "action": {"choices": ["allow", "block"], "type": "str"},
                    "log": {"choices": ["disable", "enable"], "type": "str"},
                    "max-range-segment": {"type": "int"},
                    "severity": {"choices": ["low", "medium", "high"], "type": "str"},
                    "status": {"choices": ["disable", "enable"], "type": "str"}
                }
            },
            "max-url-param": {
                "type": "dict",
                "options": {
                    "action": {"choices": ["allow", "block"], "type": "str"},
                    "log": {"choices": ["disable", "enable"], "type": "str"},
                    "max-url-param": {"type": "int"},
                    "severity": {"choices": ["low", "medium", "high"], "type": "str"},
                    "status": {"choices": ["disable", "enable"], "type": "str"}
                }
            },
            "method": {
                "type": "dict",
                "options": {
                    "action": {"choices": ["allow", "block"], "type": "str"},
                    "log": {"choices": ["disable", "enable"], "type": "str"},
                    "severity": {"choices": ["low", "medium", "high"], "type": "str"},
                    "status": {"choices": ["disable", "enable"], "type": "str"}
                }
            },
            "param-length": {
                "type": "dict",
                "options": {
                    "action": {"choices": ["allow", "block"], "type": "str"},
                    "length": {"type": "int"},
                    "log": {"choices": ["disable", "enable"], "type": "str"},
                    "severity": {"choices": ["low", "medium", "high"], "type": "str"},
                    "status": {"choices": ["disable", "enable"], "type": "str"}
                }
            },
            "url-param-length": {
                "type": "dict",
                "options": {
                    "action": {"choices": ["allow", "block"], "type": "str"},
                    "length": {"type": "int"},
                    "log": {"choices": ["disable", "enable"], "type": "str"},
                    "severity": {"choices": ["low", "medium", "high"], "type": "str"},
                    "status": {"choices": ["disable", "enable"], "type": "str"}
                }
            },
            "version": {
                "type": "dict",
                "options": {
                    "action": {"choices": ["allow", "block"], "type": "str"},
                    "log": {"choices": ["disable", "enable"], "type": "str"},
                    "severity": {"choices": ["low", "medium", "high"], "type": "str"},
                    "status": {"choices": ["disable", "enable"], "type": "str"}
                }
            }
        }
    },
    "waf_profile_constraint_contentlength": {
        "stated": false,
        "options": {
            "action": {"choices": ["allow", "block"], "type": "str"},
            "length": {"type": "int"},
            "log": {"choices": ["disable", "enable"], "type": "str"},
            "severity": {"choices": ["low", "medium", "high"], "type": "str"},
            "status": {"choices": ["disable", "enable"], "type": "str"}
        }
    },
    "waf_profile_constraint_exception": {
        "stated": true,
        "options": {
            "address": {"type": "str"},
            "content-length": {"choices": ["disable", "enable"], "type": "str"},
            "header-length": {"choices": ["disable", "enable"], "type": "str"},
            "hostname": {"choices": ["disable", "enable"], "type": "str"},
            "id": {"required": true, "type": "int"},
            "line-length": {"choices": ["disable", "enable"], "type": "str"},
            "malformed": {"choices": ["disable", "enable"], "type": "str"},
            "max-cookie": {"choices": ["disable", "enable"], "type": "str"},
            "max-header-line": {"choices": ["disable", "enable"], "type": "str"},
            "max-range-segment": {"choices": ["disable", "enable"], "type": "str"},
            "max-url-param": {"choices": ["disable", "enable"], "type": "str"},
            "method": {"choices": ["disable", "enable"], "type": "str"},
            "param-length": {"choices": ["disable", "enable"], "type": "str"},
            "pattern": {"type": "str"},
            "regex": {"choices": ["disable", "enable"], "type": "str"},
            "url-param-length": {"choices": ["disable", "enable"], "type": "str"},
            "version": {"choices": ["disable", "enable"], "type": "str"}
        }
    },
    "waf_profile_constraint_headerlength": {
        "stated": false,
        "options": {
            "action": {"choices": ["allow", "block"], "type": "str"},
            "length": {"type": "int"},
            "log": {"choices": ["disable", "enable"], "type": "str"},
            "severity": {"choices": ["low", "medium", "high"], "type": "str"},
            "status": {"choices": ["disable", "enable"], "type": "str"}
        }
    },
    "waf_profile_constraint_hostname": {
        "stated": false,
        "options": {
            "action": {"choices": ["allow", "block"], "type": "str"},
            "log": {"choices": ["disable", "enable"], "type": "str"},
            "severity": {"choices": ["low", "medium", "high"], "type": "str"},
            "status": {"choices": ["disable", "enable"], "type": "str"}
        }
    },
    "waf_profile_constraint_linelength": {
        "stated": false,
        "options": {
            "action": {"choices": ["allow", "block"], "type": "str"},
            "length": {"type": "int"},
            "log": {"choices": ["disable", "enable"], "type": "str"},
            "severity": {"choices": ["low", "medium", "high"], "type": "str"},
            "status": {"choices": ["disable", "enable"], "type": "str"}
        }
    },
    "waf_profile_constraint_malformed": {
        "stated": false,
        "options": {
            "action": {"choices": ["allow", "block"], "type": "str"},
            "log": {"choices": ["disable", "enable"], "type": "str"},
            "severity": {"choices": ["low", "medium", "high"], "type": "str"},
            "status": {"choices": ["disable", "enable"], "type": "str"}
        }
    },
    "waf_profile_constraint_maxcookie": {
        "stated": false,
        "options": {
            "action": {"choices": ["allow", "block"], "type": "str"},
            "log": {"choices": ["disable", "enable"], "type": "str"},
            "max-cookie": {"type": "int"},
            "severity": {"choices": ["low", "medium", "high"], "type": "str"},
            "status": {"choices": ["disable", "enable"], "type": "str"}
        }
    },
    "waf_profile_constraint_maxheaderline": {
        "stated": false,
        "options": {
            "action": {"choices": ["allow", "block"], "type": "str"},
            "log": {"choices": ["disable", "enable"], "type": "str"},
            "max-header-line": {"type": "int"},
            "severity": {"choices": ["low", "medium", "high"], "type": "str"},
            "status": {"choices": ["disable", "enable"], "type": "str"}
        }
    },
    "waf_profile_constraint_maxrangesegment": {
        "stated": false,
        "options": {
            "action": {"choices": ["allow", "block"], "type": "str"},
            "log": {"choices": ["disable", "enable"], "type": "str"},
            "max-range-segment": {"type": "int"},
            "severity": {"choices": ["low", "medium", "high"], "type": "str"},
            "status": {"choices": ["disable", "enable"], "type": "str"}
        }
    },
    "waf_profile_constraint_maxurlparam": {
        "stated": false,
        "options": {
            "action": {"choices": ["allow", "block"], "type": "str"},
            "log": {"choices": ["disable", "enable"], "type": "str"},
            "max-url-param": {"type": "int"},
            "severity": {"choices": ["low", "medium", "high"], "type": "str"},
            "status": {"choices": ["disable", "enable"], "type": "str"}
        }
    },
    "waf_profile_constraint_method": {
        "stated": false,
        "options": {
            "action": {"choices": ["allow", "block"], "type": "str"},
            "log": {"choices": ["disable", "enable"], "type": "str"},
            "severity": {"choices": ["low", "medium", "high"], "type": "str"},
            "status": {"choices": ["disable", "enable"], "type": "str"}
        }
    },
    "waf_profile_constraint_paramlength": {
        "stated": false,
        "options": {
            "action": {"choices": ["allow", "block"], "type": "str"},
            "length": {"type": "int"},
            "log": {"choices": ["disable", "enable"], "type": "str"},
            "severity": {"choices": ["low", "medium", "high"], "type": "str"},
            "status": {"choices": ["disable", "enable"], "type": "str"}
        }
    },
    "waf_profile_constraint_urlparamlength": {
        "stated": false,
        "options": {
            "action": {"choices": ["allow", "block"], "type": "str"},
            "length": {"type": "int"},
            "log": {"choices": ["disable", "enable"], "type": "str"},
            "severity": {"choices": ["low", "medium", "high"], "type": "str"},
            "status": {"choices": ["disable", "enable"], "type": "str"}
        }
    },
    "waf_profile_constraint_version": {
        "stated": false,
        "options": {
            "action": {"choices": ["allow", "block"], "type": "str"},
            "log": {"choices": ["disable", "enable"], "type": "str"},
            "severity": {"choices": ["low", "medium", "high"], "type": "str"},
            "status": {"choices": ["disable", "enable"], "type": "str"}
        }
    },
    "waf_profile_method": {
        "stated": false,
        "options": {
            "default-allowed-methods": {
                "type": "list",
                "choices": ["delete", "get", "head", "options", "post", "put", "trace", "others", "connect"],
                "elements": "str"
            },
            "log": {"choices": ["disable", "enable"], "type": "str"},
            "method-policy": {
                "type": "list",
                "options": {
                    "address": {"type": "str"},
                    "allowed-methods": {
                        "type": "list",
                        "choices": ["delete", "get", "head", "options", "post", "put", "trace", "others", "connect"],
                        "elements": "str"
                    },
                    "id": {"type": "int"},
                    "pattern": {"type": "str"},
                    "regex": {"choices": ["disable", "enable"], "type": "str"}
                },
                "elements": "dict"
            },
            "severity": {"choices": ["low", "medium", "high"], "type": "str"},
            "status": {"choices": ["disable", "enable"], "type": "str"}
        }
    },
    "waf_profile_method_methodpolicy": {
        "stated": true,
        "options": {
            "address": {"type": "str"},
            "allowed-methods": {
                "type": "list",
                "choices": ["delete", "get", "head", "options", "post", "put", "trace", "others", "connect"],
                "elements": "str"
            },
            "id": {"required": true, "type": "int"},
            "pattern": {"type": "str"},
            "regex": {"choices": ["disable", "enable"], "type": "str"}
        }
    },
    "waf_profile_signature": {
        "stated": false,
        "options": {
            "credit-card-detection-threshold": {"type": "int"},
            "custom-signature": {
                "type": "list",
                "options": {
                    "action": {"choices": ["allow", "block", "erase"], "type": "str"},
                    "case-sensitivity": {"choices": ["disable", "enable"], "type": "str"},
                    "direction": {"choices": ["request", "response"], "type": "str"},
                    "log": {"choices": ["disable", "enable"], "type": "str"},
                    "name": {"type": "str"},
                    "pattern": {"type": "str"},
                    "severity": {"choices": ["low", "medium", "high"], "type": "str"},
                    "status": {"choices": ["disable", "enable"], "type": "str"},
                    "target": {
                        "type": "list",
                        "choices": [
                            "arg", "arg-name", "req-body", "req-cookie", "req-cookie-name", "req-filename", "req-header", "req-header-name",
                            "req-raw-uri", "req-uri", "resp-body", "resp-hdr", "resp-status"
                        ],
                        "elements": "str"
                    }
                },
                "elements": "dict"
            },
            "disabled-signature": {"type": "raw"},
            "disabled-sub-class": {"type": "raw"},
            "main-class": {
                "type": "dict",
                "options": {
                    "action": {"choices": ["allow", "block", "erase"], "type": "str"},
                    "id": {"type": "int"},
                    "log": {"choices": ["disable", "enable"], "type": "str"},
                    "severity": {"choices": ["low", "medium", "high"], "type": "str"},
                    "status": {"choices": ["disable", "enable"], "type": "str"}
                }
            }
        }
    },
    "waf_profile_signature_customsignature": {
        "stated": true,
        "options": {
            "action": {"choices": ["allow", "block", "erase"], "type": "str"},
            "case-sensitivity": {"choices": ["disable", "enable"], "type": "str"},
            "direction": {"choices": ["request", "response"], "type": "str"},
            "log": {"choices": ["disable", "enable"], "type": "str"},
            "name": {"required": true, "type": "str"},
            "pattern": {"type": "str"},
            "severity": {"choices": ["low", "medium", "high"], "type": "str"},
            "status": {"choices": ["disable", "enable"], "type": "str"},
            "target": {
                "type": "list",
                "choices": [
                    "arg", "arg-name", "req-body", "req-cookie", "req-cookie-name", "req-filename", "req-header", "req-header-name", "req-raw-uri",
                    "req-uri", "resp-body", "resp-hdr", "resp-status"
                ],
                "elements": "str"
            }
        }
    },
    "waf_profile_signature_mainclass": {
        "stated": false,
        "options": {
            "action": {"choices": ["allow", "block", "erase"], "type": "str"},
            "id": {"type": "int"},
            "log": {"choices": ["disable", "enable"], "type": "str"},
            "severity": {"choices": ["low", "medium", "high"], "type": "str"},
            "status": {"choices": ["disable", "enable"], "type": "str"}
        }
    },
    "waf_profile_urlaccess": {
        "stated": true,
        "options": {
            "access-pattern": {
                "type": "list",
                "options": {
                    "id": {"type": "int"},
                    "negate": {"choices": ["disable", "enable"], "type": "str"},
                    "pattern": {"type": "str"},
                    "regex": {"choices": ["disable", "enable"], "type": "str"},
                    "srcaddr": {"type": "str"}
                },
                "elements": "dict"
            },
            "action": {"choices": ["bypass", "permit", "block"], "type": "str"},
            "address": {"type": "str"},
            "id": {"required": true, "type": "int"},
            "log": {"choices": ["disable", "enable"], "type": "str"},
            "severity": {"choices": ["low", "medium", "high"], "type": "str"}
        }
    },
    "waf_profile_urlaccess_accesspattern": {
        "stated": true,
        "options": {
            "id": {"required": true, "type": "int"},
            "negate": {"choices": ["disable", "enable"], "type": "str"},
            "pattern": {"type": "str"},
            "regex": {"choices": ["disable", "enable"], "type": "str"},
            "srcaddr": {"type": "str"}
        }
    },
    "waf_signature": {"stated": true, "options": {"desc": {"type": "str"}, "id": {"required": true, "type": "int"}}},
    "waf_subclass": {"stated": true, "options": {"id": {"required": true, "type": "int"}, "name": {"type": "str"}}},
    "wagprofile": {
        "stated": true,
        "options": {
            "comment": {"v_range": [["6.2.3", ""]], "type": "str"},
            "dhcp-ip-addr": {"v_range": [["6.2.3", ""]], "type": "str"},
            "name": {"v_range": [["6.2.3", ""]], "required": true, "type": "str"},
            "ping-interval": {"v_range": [["6.2.3", ""]], "type": "int"},
            "ping-number": {"v_range": [["6.2.3", ""]], "type": "int"},
            "return-packet-timeout": {"v_range": [["6.2.3", ""]], "type": "int"},
            "tunnel-type": {"v_range": [["6.2.3", ""]], "choices": ["gre", "l2tpv3"], "type": "str"},
            "wag-ip": {"v_range": [["6.2.3", ""]], "type": "str"},
            "wag-port": {"v_range": [["6.2.3", ""]], "type": "int"}
        }
    },
    "wanopt_authgroup": {
        "stated": true,
        "options": {
            "auth-method": {"choices": ["cert", "psk"], "type": "str"},
            "cert": {"type": "str"},
            "name": {"required": true, "type": "str"},
            "peer": {"type": "str"},
            "peer-accept": {"choices": ["any", "defined", "one"], "type": "str"},
            "psk": {"type": "raw"}
        }
    },
    "wanopt_peer": {"stated": true, "options": {"ip": {"type": "str"}, "peer-host-id": {"required": true, "type": "str"}}},
    "wanopt_profile": {
        "stated": true,
        "options": {
            "auth-group": {"type": "str"},
            "comments": {"type": "str"},
            "name": {"required": true, "type": "str"},
            "transparent": {"choices": ["disable", "enable"], "type": "str"},
            "cifs": {
                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                "type": "dict",
                "options": {
                    "byte-caching": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "log-traffic": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "prefer-chunking": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["dynamic", "fix"], "type": "str"},
                    "protocol-opt": {"v_range": [["6.4.5", ""]], "choices": ["protocol", "tcp"], "type": "str"},
                    "secure-tunnel": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "status": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "tunnel-sharing": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["private", "shared", "express-shared"], "type": "str"},
                    "port": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "raw"}
                }
            },
            "ftp": {
                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                "type": "dict",
                "options": {
                    "byte-caching": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "log-traffic": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "prefer-chunking": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["dynamic", "fix"], "type": "str"},
                    "protocol-opt": {"v_range": [["6.4.5", ""]], "choices": ["protocol", "tcp"], "type": "str"},
                    "secure-tunnel": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "ssl": {"v_range": [["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "status": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "tunnel-sharing": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["private", "shared", "express-shared"], "type": "str"},
                    "port": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "raw"}
                }
            },
            "http": {
                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                "type": "dict",
                "options": {
                    "byte-caching": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "log-traffic": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "prefer-chunking": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["dynamic", "fix"], "type": "str"},
                    "protocol-opt": {"v_range": [["6.4.5", ""]], "choices": ["protocol", "tcp"], "type": "str"},
                    "secure-tunnel": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "ssl": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "status": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "tunnel-sharing": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["private", "shared", "express-shared"], "type": "str"},
                    "tunnel-non-http": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "unknown-http-version": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "choices": ["best-effort", "reject", "tunnel"],
                        "type": "str"
                    },
                    "port": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "raw"},
                    "ssl-port": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "raw"}
                }
            },
            "mapi": {
                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                "type": "dict",
                "options": {
                    "byte-caching": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "log-traffic": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "secure-tunnel": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "status": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "tunnel-sharing": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["private", "shared", "express-shared"], "type": "str"},
                    "port": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "raw"}
                }
            },
            "tcp": {
                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                "type": "dict",
                "options": {
                    "byte-caching": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "byte-caching-opt": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["mem-only", "mem-disk"], "type": "str"},
                    "log-traffic": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "port": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "secure-tunnel": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "ssl": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "ssl-port": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "raw"},
                    "status": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "tunnel-sharing": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["private", "shared", "express-shared"], "type": "str"}
                }
            }
        }
    },
    "wanopt_profile_cifs": {
        "stated": false,
        "options": {
            "byte-caching": {"choices": ["disable", "enable"], "type": "str"},
            "log-traffic": {"choices": ["disable", "enable"], "type": "str"},
            "port": {"type": "raw"},
            "prefer-chunking": {"choices": ["dynamic", "fix"], "type": "str"},
            "secure-tunnel": {"choices": ["disable", "enable"], "type": "str"},
            "status": {"choices": ["disable", "enable"], "type": "str"},
            "tunnel-sharing": {"choices": ["private", "shared", "express-shared"], "type": "str"},
            "protocol-opt": {"v_range": [["6.4.0", ""]], "choices": ["protocol", "tcp"], "type": "str"}
        }
    },
    "wanopt_profile_ftp": {
        "stated": false,
        "options": {
            "byte-caching": {"choices": ["disable", "enable"], "type": "str"},
            "log-traffic": {"choices": ["disable", "enable"], "type": "str"},
            "port": {"type": "raw"},
            "prefer-chunking": {"choices": ["dynamic", "fix"], "type": "str"},
            "secure-tunnel": {"choices": ["disable", "enable"], "type": "str"},
            "status": {"choices": ["disable", "enable"], "type": "str"},
            "tunnel-sharing": {"choices": ["private", "shared", "express-shared"], "type": "str"},
            "protocol-opt": {"v_range": [["6.4.0", ""]], "choices": ["protocol", "tcp"], "type": "str"},
            "ssl": {"v_range": [["6.4.0", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "wanopt_profile_http": {
        "stated": false,
        "options": {
            "byte-caching": {"choices": ["disable", "enable"], "type": "str"},
            "log-traffic": {"choices": ["disable", "enable"], "type": "str"},
            "port": {"type": "raw"},
            "prefer-chunking": {"choices": ["dynamic", "fix"], "type": "str"},
            "secure-tunnel": {"choices": ["disable", "enable"], "type": "str"},
            "ssl": {"choices": ["disable", "enable"], "type": "str"},
            "ssl-port": {"type": "raw"},
            "status": {"choices": ["disable", "enable"], "type": "str"},
            "tunnel-non-http": {"choices": ["disable", "enable"], "type": "str"},
            "tunnel-sharing": {"choices": ["private", "shared", "express-shared"], "type": "str"},
            "unknown-http-version": {"choices": ["best-effort", "reject", "tunnel"], "type": "str"},
            "protocol-opt": {"v_range": [["6.4.0", ""]], "choices": ["protocol", "tcp"], "type": "str"}
        }
    },
    "wanopt_profile_mapi": {
        "stated": false,
        "options": {
            "byte-caching": {"choices": ["disable", "enable"], "type": "str"},
            "log-traffic": {"choices": ["disable", "enable"], "type": "str"},
            "port": {"type": "raw"},
            "secure-tunnel": {"choices": ["disable", "enable"], "type": "str"},
            "status": {"choices": ["disable", "enable"], "type": "str"},
            "tunnel-sharing": {"choices": ["private", "shared", "express-shared"], "type": "str"}
        }
    },
    "wanopt_profile_tcp": {
        "stated": false,
        "options": {
            "byte-caching": {"choices": ["disable", "enable"], "type": "str"},
            "byte-caching-opt": {"choices": ["mem-only", "mem-disk"], "type": "str"},
            "log-traffic": {"choices": ["disable", "enable"], "type": "str"},
            "port": {"type": "str"},
            "secure-tunnel": {"choices": ["disable", "enable"], "type": "str"},
            "ssl": {"choices": ["disable", "enable"], "type": "str"},
            "ssl-port": {"type": "raw"},
            "status": {"choices": ["disable", "enable"], "type": "str"},
            "tunnel-sharing": {"choices": ["private", "shared", "express-shared"], "type": "str"}
        }
    },
    "wanprof_system_sdwan": {
        "stated": false,
        "options": {
            "duplication": {
                "v_range": [["6.4.2", ""]],
                "type": "list",
                "options": {
                    "dstaddr": {"v_range": [["6.4.2", ""]], "type": "raw"},
                    "dstaddr6": {"v_range": [["6.4.2", ""]], "type": "raw"},
                    "dstintf": {"v_range": [["6.4.2", ""]], "type": "raw"},
                    "id": {"v_range": [["6.4.2", ""]], "type": "int"},
                    "packet-de-duplication": {"v_range": [["6.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "packet-duplication": {"v_range": [["6.4.2", ""]], "choices": ["disable", "force", "on-demand"], "type": "str"},
                    "service": {"v_range": [["6.4.2", ""]], "type": "raw"},
                    "srcaddr": {"v_range": [["6.4.2", ""]], "type": "raw"},
                    "srcaddr6": {"v_range": [["6.4.2", ""]], "type": "raw"},
                    "srcintf": {"v_range": [["6.4.2", ""]], "type": "raw"},
                    "service-id": {"v_range": [["6.4.3", ""]], "type": "raw"},
                    "sla-match-service": {"v_range": [["7.2.0", ""]], "choices": ["disable", "enable"], "type": "str"}
                },
                "elements": "dict"
            },
            "duplication-max-num": {"v_range": [["6.4.2", ""]], "type": "int"},
            "fail-detect": {"v_range": [["6.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "health-check": {
                "v_range": [["6.4.1", ""]],
                "type": "list",
                "options": {
                    "_dynamic-server": {"v_range": [["6.4.1", "6.4.14"]], "type": "str"},
                    "addr-mode": {"v_range": [["6.4.1", ""]], "choices": ["ipv4", "ipv6"], "type": "str"},
                    "diffservcode": {"v_range": [["6.4.1", ""]], "type": "str"},
                    "dns-match-ip": {"v_range": [["6.4.2", ""]], "type": "str"},
                    "dns-request-domain": {"v_range": [["6.4.1", ""]], "type": "str"},
                    "failtime": {"v_range": [["6.4.1", ""]], "type": "int"},
                    "ftp-file": {"v_range": [["6.4.2", ""]], "type": "str"},
                    "ftp-mode": {"v_range": [["6.4.2", ""]], "choices": ["passive", "port"], "type": "str"},
                    "ha-priority": {"v_range": [["6.4.1", ""]], "type": "int"},
                    "http-agent": {"v_range": [["6.4.1", ""]], "type": "str"},
                    "http-get": {"v_range": [["6.4.1", ""]], "type": "str"},
                    "http-match": {"v_range": [["6.4.1", ""]], "type": "str"},
                    "interval": {"v_range": [["6.4.1", ""]], "type": "int"},
                    "members": {"v_range": [["6.4.1", ""]], "type": "raw"},
                    "name": {"v_range": [["6.4.1", ""]], "type": "str"},
                    "packet-size": {"v_range": [["6.4.1", ""]], "type": "int"},
                    "password": {"v_range": [["6.4.1", ""]], "no_log": true, "type": "raw"},
                    "port": {"v_range": [["6.4.1", ""]], "type": "int"},
                    "probe-count": {"v_range": [["6.4.1", ""]], "type": "int"},
                    "probe-packets": {"v_range": [["6.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "probe-timeout": {"v_range": [["6.4.1", ""]], "type": "int"},
                    "protocol": {
                        "v_range": [["6.4.1", ""]],
                        "choices": ["ping", "tcp-echo", "udp-echo", "http", "twamp", "ping6", "dns", "tcp-connect", "ftp", "https"],
                        "type": "str"
                    },
                    "quality-measured-method": {"v_range": [["6.4.2", ""]], "choices": ["half-close", "half-open"], "type": "str"},
                    "recoverytime": {"v_range": [["6.4.1", ""]], "type": "int"},
                    "security-mode": {"v_range": [["6.4.1", ""]], "choices": ["none", "authentication"], "type": "str"},
                    "server": {"v_range": [["6.4.1", ""]], "type": "raw"},
                    "sla": {
                        "v_range": [["6.4.1", ""]],
                        "type": "list",
                        "options": {
                            "id": {"v_range": [["6.4.1", ""]], "type": "int"},
                            "jitter-threshold": {"v_range": [["6.4.1", ""]], "type": "int"},
                            "latency-threshold": {"v_range": [["6.4.1", ""]], "type": "int"},
                            "link-cost-factor": {
                                "v_range": [["6.4.1", ""]],
                                "type": "list",
                                "choices": ["latency", "jitter", "packet-loss", "mos", "remote"],
                                "elements": "str"
                            },
                            "packetloss-threshold": {"v_range": [["6.4.1", ""]], "type": "int"},
                            "mos-threshold": {"v_range": [["7.2.0", ""]], "type": "str"},
                            "priority-in-sla": {"v_range": [["7.2.1", ""]], "type": "int"},
                            "priority-out-sla": {"v_range": [["7.2.1", ""]], "type": "int"}
                        },
                        "elements": "dict"
                    },
                    "sla-fail-log-period": {"v_range": [["6.4.1", ""]], "type": "int"},
                    "sla-pass-log-period": {"v_range": [["6.4.1", ""]], "no_log": true, "type": "int"},
                    "system-dns": {"v_range": [["6.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "threshold-alert-jitter": {"v_range": [["6.4.1", ""]], "type": "int"},
                    "threshold-alert-latency": {"v_range": [["6.4.1", ""]], "type": "int"},
                    "threshold-alert-packetloss": {"v_range": [["6.4.1", ""]], "type": "int"},
                    "threshold-warning-jitter": {"v_range": [["6.4.1", ""]], "type": "int"},
                    "threshold-warning-latency": {"v_range": [["6.4.1", ""]], "type": "int"},
                    "threshold-warning-packetloss": {"v_range": [["6.4.1", ""]], "type": "int"},
                    "update-cascade-interface": {"v_range": [["6.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "update-static-route": {"v_range": [["6.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "user": {"v_range": [["6.4.2", ""]], "type": "str"},
                    "detect-mode": {
                        "v_range": [["7.0.0", ""]],
                        "choices": ["active", "passive", "prefer-passive", "remote", "agent-based"],
                        "type": "str"
                    },
                    "mos-codec": {"v_range": [["7.2.0", ""]], "choices": ["g711", "g722", "g729"], "type": "str"},
                    "source": {"v_range": [["7.2.0", ""]], "type": "str"},
                    "vrf": {"v_range": [["7.2.0", ""]], "type": "int"},
                    "embed-measured-health": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "sla-id-redistribute": {"v_range": [["7.2.1", ""]], "type": "int"},
                    "class-id": {"v_range": [["7.4.0", ""]], "type": "str"},
                    "source6": {"v_range": [["7.4.0", ""]], "type": "str"}
                },
                "elements": "dict"
            },
            "load-balance-mode": {
                "v_range": [["6.4.1", ""]],
                "choices": ["source-ip-based", "weight-based", "usage-based", "source-dest-ip-based", "measured-volume-based"],
                "type": "str"
            },
            "members": {
                "v_range": [["6.4.1", ""]],
                "type": "list",
                "options": {
                    "_dynamic-member": {"v_range": [["6.4.1", "6.4.14"]], "type": "str"},
                    "comment": {"v_range": [["6.4.1", ""]], "type": "str"},
                    "cost": {"v_range": [["6.4.1", ""]], "type": "int"},
                    "gateway": {"v_range": [["6.4.1", ""]], "type": "str"},
                    "gateway6": {"v_range": [["6.4.1", ""]], "type": "str"},
                    "ingress-spillover-threshold": {"v_range": [["6.4.1", ""]], "type": "int"},
                    "interface": {"v_range": [["6.4.1", ""]], "type": "str"},
                    "priority": {"v_range": [["6.4.1", ""]], "type": "int"},
                    "seq-num": {"v_range": [["6.4.1", ""]], "type": "int"},
                    "source": {"v_range": [["6.4.1", ""]], "type": "str"},
                    "source6": {"v_range": [["6.4.1", ""]], "type": "str"},
                    "spillover-threshold": {"v_range": [["6.4.1", ""]], "type": "int"},
                    "status": {"v_range": [["6.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "volume-ratio": {"v_range": [["6.4.1", ""]], "type": "int"},
                    "weight": {"v_range": [["6.4.1", ""]], "type": "int"},
                    "zone": {"v_range": [["6.4.1", ""]], "type": "str"},
                    "priority6": {"v_range": [["7.0.0", ""]], "type": "int"},
                    "preferred-source": {"v_range": [["7.4.0", ""]], "type": "str"},
                    "transport-group": {"v_range": [["7.4.2", ""]], "type": "int"},
                    "priority-in-sla": {"v_range": [["7.6.0", ""]], "type": "int"},
                    "priority-out-sla": {"v_range": [["7.6.0", ""]], "type": "int"}
                },
                "elements": "dict"
            },
            "neighbor": {
                "v_range": [["6.4.1", ""]],
                "type": "list",
                "options": {
                    "health-check": {"v_range": [["6.4.1", ""]], "type": "str"},
                    "ip": {"v_range": [["6.4.1", ""]], "type": "str"},
                    "member": {"v_range": [["6.4.1", ""]], "type": "raw"},
                    "role": {"v_range": [["6.4.1", ""]], "choices": ["primary", "secondary", "standalone"], "type": "str"},
                    "sla-id": {"v_range": [["6.4.1", ""]], "type": "int"},
                    "minimum-sla-meet-members": {"v_range": [["7.2.0", ""]], "type": "int"},
                    "mode": {"v_range": [["7.0.1", ""]], "choices": ["sla", "speedtest"], "type": "str"},
                    "service-id": {"v_range": [["7.4.1", ""]], "type": "str"}
                },
                "elements": "dict"
            },
            "neighbor-hold-boot-time": {"v_range": [["6.4.1", ""]], "type": "int"},
            "neighbor-hold-down": {"v_range": [["6.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "neighbor-hold-down-time": {"v_range": [["6.4.1", ""]], "type": "int"},
            "service": {
                "v_range": [["6.4.1", ""]],
                "type": "list",
                "options": {
                    "addr-mode": {"v_range": [["6.4.1", ""]], "choices": ["ipv4", "ipv6"], "type": "str"},
                    "bandwidth-weight": {"v_range": [["6.4.1", ""]], "type": "int"},
                    "default": {"v_range": [["6.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "dscp-forward": {"v_range": [["6.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "dscp-forward-tag": {"v_range": [["6.4.1", ""]], "type": "str"},
                    "dscp-reverse": {"v_range": [["6.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "dscp-reverse-tag": {"v_range": [["6.4.1", ""]], "type": "str"},
                    "dst": {"v_range": [["6.4.1", ""]], "type": "raw"},
                    "dst-negate": {"v_range": [["6.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "dst6": {"v_range": [["6.4.1", ""]], "type": "raw"},
                    "end-port": {"v_range": [["6.4.1", ""]], "type": "int"},
                    "gateway": {"v_range": [["6.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "groups": {"v_range": [["6.4.1", ""]], "type": "raw"},
                    "hash-mode": {
                        "v_range": [["6.4.2", ""]],
                        "choices": ["round-robin", "source-ip-based", "source-dest-ip-based", "inbandwidth", "outbandwidth", "bibandwidth"],
                        "type": "str"
                    },
                    "health-check": {"v_range": [["6.4.1", ""]], "type": "raw"},
                    "hold-down-time": {"v_range": [["6.4.1", ""]], "type": "int"},
                    "id": {"v_range": [["6.4.1", ""]], "type": "int"},
                    "input-device": {"v_range": [["6.4.1", ""]], "type": "raw"},
                    "input-device-negate": {"v_range": [["6.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "internet-service": {"v_range": [["6.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "internet-service-app-ctrl": {"v_range": [["6.4.1", ""]], "type": "raw"},
                    "internet-service-app-ctrl-group": {"v_range": [["6.4.1", ""]], "type": "raw"},
                    "internet-service-custom": {"v_range": [["6.4.1", ""]], "type": "raw"},
                    "internet-service-custom-group": {"v_range": [["6.4.1", ""]], "type": "raw"},
                    "internet-service-group": {"v_range": [["6.4.1", ""]], "type": "raw"},
                    "internet-service-name": {"v_range": [["6.4.1", ""]], "type": "raw"},
                    "jitter-weight": {"v_range": [["6.4.1", ""]], "type": "int"},
                    "latency-weight": {"v_range": [["6.4.1", ""]], "type": "int"},
                    "link-cost-factor": {
                        "v_range": [["6.4.1", ""]],
                        "choices": ["latency", "jitter", "packet-loss", "inbandwidth", "outbandwidth", "bibandwidth", "custom-profile-1"],
                        "type": "str"
                    },
                    "link-cost-threshold": {"v_range": [["6.4.1", ""]], "type": "int"},
                    "minimum-sla-meet-members": {"v_range": [["6.4.2", ""]], "type": "int"},
                    "mode": {"v_range": [["6.4.1", ""]], "choices": ["auto", "manual", "priority", "sla", "load-balance"], "type": "str"},
                    "name": {"v_range": [["6.4.1", ""]], "type": "str"},
                    "packet-loss-weight": {"v_range": [["6.4.1", ""]], "type": "int"},
                    "priority-members": {"v_range": [["6.4.1", ""]], "type": "raw"},
                    "protocol": {"v_range": [["6.4.1", ""]], "type": "int"},
                    "quality-link": {"v_range": [["6.4.1", ""]], "type": "int"},
                    "role": {"v_range": [["6.4.1", ""]], "choices": ["primary", "secondary", "standalone"], "type": "str"},
                    "route-tag": {"v_range": [["6.4.1", ""]], "type": "int"},
                    "sla": {
                        "v_range": [["6.4.1", ""]],
                        "type": "list",
                        "options": {"health-check": {"v_range": [["6.4.1", ""]], "type": "str"}, "id": {"v_range": [["6.4.1", ""]], "type": "int"}},
                        "elements": "dict"
                    },
                    "sla-compare-method": {"v_range": [["6.4.1", ""]], "choices": ["order", "number"], "type": "str"},
                    "src": {"v_range": [["6.4.1", ""]], "type": "raw"},
                    "src-negate": {"v_range": [["6.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "src6": {"v_range": [["6.4.1", ""]], "type": "raw"},
                    "standalone-action": {"v_range": [["6.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "start-port": {"v_range": [["6.4.1", ""]], "type": "int"},
                    "status": {"v_range": [["6.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "tos": {"v_range": [["6.4.1", ""]], "type": "str"},
                    "tos-mask": {"v_range": [["6.4.1", ""]], "type": "str"},
                    "users": {"v_range": [["6.4.1", ""]], "type": "raw"},
                    "tie-break": {"v_range": [["6.4.3", ""]], "choices": ["zone", "cfg-order", "fib-best-match", "input-device"], "type": "str"},
                    "use-shortcut-sla": {"v_range": [["6.4.4", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "input-zone": {"v_range": [["7.2.0", ""]], "type": "raw"},
                    "internet-service-app-ctrl-category": {"v_range": [["7.2.0", ""]], "type": "raw"},
                    "passive-measurement": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "priority-zone": {"v_range": [["7.0.1", ""]], "type": "raw"},
                    "agent-exclusive": {"v_range": [["7.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "shortcut": {"v_range": [["7.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "shortcut-stickiness": {"v_range": [["7.4.0", "7.4.1"]], "choices": ["disable", "enable"], "type": "str"},
                    "end-src-port": {"v_range": [["7.4.1", ""]], "type": "int"},
                    "load-balance": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "sla-stickiness": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "start-src-port": {"v_range": [["7.4.1", ""]], "type": "int"},
                    "zone-mode": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "shortcut-priority": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable", "auto"], "type": "str"},
                    "comment": {"v_range": [["7.6.0", ""]], "type": "str"}
                },
                "elements": "dict"
            },
            "status": {"v_range": [["6.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "zone": {
                "v_range": [["6.4.1", ""]],
                "type": "list",
                "options": {
                    "name": {"v_range": [["6.4.1", ""]], "type": "str"},
                    "service-sla-tie-break": {"v_range": [["6.4.3", ""]], "choices": ["cfg-order", "fib-best-match", "input-device"], "type": "str"},
                    "minimum-sla-meet-members": {"v_range": [["7.4.1", ""]], "type": "int"},
                    "advpn-health-check": {"v_range": [["7.4.2", ""]], "type": "str"},
                    "advpn-select": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"}
                },
                "elements": "dict"
            },
            "speedtest-bypass-routing": {"v_range": [["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "fail-alert-interfaces": {"v_range": [["7.2.3", ""]], "type": "raw"},
            "app-perf-log-period": {"v_range": [["7.4.0", ""]], "type": "int"},
            "health-check-fortiguard": {
                "v_range": [["7.6.0", ""]],
                "type": "list",
                "options": {
                    "addr-mode": {"v_range": [["7.6.0", ""]], "choices": ["ipv4", "ipv6"], "type": "str"},
                    "class-id": {"v_range": [["7.6.0", ""]], "type": "raw"},
                    "detect-mode": {
                        "v_range": [["7.6.0", ""]],
                        "choices": ["active", "passive", "prefer-passive", "remote", "agent-based"],
                        "type": "str"
                    },
                    "diffservcode": {"v_range": [["7.6.0", ""]], "type": "str"},
                    "dns-match-ip": {"v_range": [["7.6.0", ""]], "type": "str"},
                    "dns-request-domain": {"v_range": [["7.6.0", ""]], "type": "str"},
                    "embed-measured-health": {"v_range": [["7.6.0", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "failtime": {"v_range": [["7.6.0", ""]], "type": "int"},
                    "ftp-file": {"v_range": [["7.6.0", ""]], "type": "str"},
                    "ftp-mode": {"v_range": [["7.6.0", ""]], "choices": ["passive", "port"], "type": "str"},
                    "ha-priority": {"v_range": [["7.6.0", ""]], "type": "int"},
                    "http-agent": {"v_range": [["7.6.0", ""]], "type": "str"},
                    "http-get": {"v_range": [["7.6.0", ""]], "type": "str"},
                    "http-match": {"v_range": [["7.6.0", ""]], "type": "str"},
                    "interval": {"v_range": [["7.6.0", ""]], "type": "int"},
                    "members": {"v_range": [["7.6.0", ""]], "type": "raw"},
                    "mos-codec": {"v_range": [["7.6.0", ""]], "choices": ["g711", "g722", "g729"], "type": "str"},
                    "packet-size": {"v_range": [["7.6.0", ""]], "type": "int"},
                    "password": {"v_range": [["7.6.0", ""]], "no_log": true, "type": "raw"},
                    "port": {"v_range": [["7.6.0", ""]], "type": "int"},
                    "probe-count": {"v_range": [["7.6.0", ""]], "type": "int"},
                    "probe-packets": {"v_range": [["7.6.0", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "probe-timeout": {"v_range": [["7.6.0", ""]], "type": "int"},
                    "protocol": {
                        "v_range": [["7.6.0", ""]],
                        "choices": ["ping", "tcp-echo", "udp-echo", "http", "twamp", "dns", "tcp-connect", "ftp", "https"],
                        "type": "str"
                    },
                    "quality-measured-method": {"v_range": [["7.6.0", ""]], "choices": ["half-close", "half-open"], "type": "str"},
                    "recoverytime": {"v_range": [["7.6.0", ""]], "type": "int"},
                    "security-mode": {"v_range": [["7.6.0", ""]], "choices": ["none", "authentication"], "type": "str"},
                    "server": {"v_range": [["7.6.0", ""]], "type": "raw"},
                    "sla": {
                        "v_range": [["7.6.0", ""]],
                        "type": "list",
                        "options": {
                            "id": {"v_range": [["7.6.0", ""]], "type": "int"},
                            "jitter-threshold": {"v_range": [["7.6.0", ""]], "type": "int"},
                            "latency-threshold": {"v_range": [["7.6.0", ""]], "type": "int"},
                            "link-cost-factor": {
                                "v_range": [["7.6.0", ""]],
                                "type": "list",
                                "choices": ["latency", "jitter", "packet-loss", "mos", "remote"],
                                "elements": "str"
                            },
                            "mos-threshold": {"v_range": [["7.6.0", ""]], "type": "str"},
                            "packetloss-threshold": {"v_range": [["7.6.0", ""]], "type": "int"},
                            "priority-in-sla": {"v_range": [["7.6.0", ""]], "type": "int"},
                            "priority-out-sla": {"v_range": [["7.6.0", ""]], "type": "int"}
                        },
                        "elements": "dict"
                    },
                    "sla-fail-log-period": {"v_range": [["7.6.0", ""]], "type": "int"},
                    "sla-id-redistribute": {"v_range": [["7.6.0", ""]], "type": "int"},
                    "sla-pass-log-period": {"v_range": [["7.6.0", ""]], "no_log": true, "type": "int"},
                    "source": {"v_range": [["7.6.0", ""]], "type": "str"},
                    "source6": {"v_range": [["7.6.0", ""]], "type": "str"},
                    "system-dns": {"v_range": [["7.6.0", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "target-name": {"v_range": [["7.6.0", ""]], "type": "str"},
                    "threshold-alert-jitter": {"v_range": [["7.6.0", ""]], "type": "int"},
                    "threshold-alert-latency": {"v_range": [["7.6.0", ""]], "type": "int"},
                    "threshold-alert-packetloss": {"v_range": [["7.6.0", ""]], "type": "int"},
                    "threshold-warning-jitter": {"v_range": [["7.6.0", ""]], "type": "int"},
                    "threshold-warning-latency": {"v_range": [["7.6.0", ""]], "type": "int"},
                    "threshold-warning-packetloss": {"v_range": [["7.6.0", ""]], "type": "int"},
                    "update-cascade-interface": {"v_range": [["7.6.0", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "update-static-route": {"v_range": [["7.6.0", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "user": {"v_range": [["7.6.0", ""]], "type": "str"},
                    "vrf": {"v_range": [["7.6.0", ""]], "type": "int"}
                },
                "elements": "dict"
            },
            "option": {"v_range": [["7.6.0", ""]], "type": "list", "choices": ["sdwan-overlay", "sdwan-manager"], "elements": "str"}
        }
    },
    "wanprof_system_sdwan_duplication": {
        "stated": true,
        "options": {
            "dstaddr": {"v_range": [["6.4.2", ""]], "type": "raw"},
            "dstaddr6": {"v_range": [["6.4.2", ""]], "type": "raw"},
            "dstintf": {"v_range": [["6.4.2", ""]], "type": "raw"},
            "id": {"v_range": [["6.4.2", ""]], "required": true, "type": "int"},
            "packet-de-duplication": {"v_range": [["6.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "packet-duplication": {"v_range": [["6.4.2", ""]], "choices": ["disable", "force", "on-demand"], "type": "str"},
            "service": {"v_range": [["6.4.2", ""]], "type": "raw"},
            "srcaddr": {"v_range": [["6.4.2", ""]], "type": "raw"},
            "srcaddr6": {"v_range": [["6.4.2", ""]], "type": "raw"},
            "srcintf": {"v_range": [["6.4.2", ""]], "type": "raw"},
            "service-id": {"v_range": [["6.4.3", ""]], "type": "raw"},
            "sla-match-service": {"v_range": [["7.2.0", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "wanprof_system_sdwan_healthcheck": {
        "stated": true,
        "options": {
            "_dynamic-server": {"v_range": [["6.4.1", "6.4.14"]], "type": "str"},
            "addr-mode": {"v_range": [["6.4.1", ""]], "choices": ["ipv4", "ipv6"], "type": "str"},
            "diffservcode": {"v_range": [["6.4.1", ""]], "type": "str"},
            "dns-match-ip": {"v_range": [["6.4.2", ""]], "type": "str"},
            "dns-request-domain": {"v_range": [["6.4.1", ""]], "type": "str"},
            "failtime": {"v_range": [["6.4.1", ""]], "type": "int"},
            "ftp-file": {"v_range": [["6.4.2", ""]], "type": "str"},
            "ftp-mode": {"v_range": [["6.4.2", ""]], "choices": ["passive", "port"], "type": "str"},
            "ha-priority": {"v_range": [["6.4.1", ""]], "type": "int"},
            "http-agent": {"v_range": [["6.4.1", ""]], "type": "str"},
            "http-get": {"v_range": [["6.4.1", ""]], "type": "str"},
            "http-match": {"v_range": [["6.4.1", ""]], "type": "str"},
            "interval": {"v_range": [["6.4.1", ""]], "type": "int"},
            "members": {"v_range": [["6.4.1", ""]], "type": "raw"},
            "name": {"v_range": [["6.4.1", ""]], "required": true, "type": "str"},
            "packet-size": {"v_range": [["6.4.1", ""]], "type": "int"},
            "password": {"v_range": [["6.4.1", ""]], "no_log": true, "type": "raw"},
            "port": {"v_range": [["6.4.1", ""]], "type": "int"},
            "probe-count": {"v_range": [["6.4.1", ""]], "type": "int"},
            "probe-packets": {"v_range": [["6.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "probe-timeout": {"v_range": [["6.4.1", ""]], "type": "int"},
            "protocol": {
                "v_range": [["6.4.1", ""]],
                "choices": ["ping", "tcp-echo", "udp-echo", "http", "twamp", "ping6", "dns", "tcp-connect", "ftp", "https"],
                "type": "str"
            },
            "quality-measured-method": {"v_range": [["6.4.2", ""]], "choices": ["half-close", "half-open"], "type": "str"},
            "recoverytime": {"v_range": [["6.4.1", ""]], "type": "int"},
            "security-mode": {"v_range": [["6.4.1", ""]], "choices": ["none", "authentication"], "type": "str"},
            "server": {"v_range": [["6.4.1", ""]], "type": "raw"},
            "sla": {
                "v_range": [["6.4.1", ""]],
                "type": "list",
                "options": {
                    "id": {"v_range": [["6.4.1", ""]], "type": "int"},
                    "jitter-threshold": {"v_range": [["6.4.1", ""]], "type": "int"},
                    "latency-threshold": {"v_range": [["6.4.1", ""]], "type": "int"},
                    "link-cost-factor": {
                        "v_range": [["6.4.1", ""]],
                        "type": "list",
                        "choices": ["latency", "jitter", "packet-loss", "mos", "remote"],
                        "elements": "str"
                    },
                    "packetloss-threshold": {"v_range": [["6.4.1", ""]], "type": "int"},
                    "mos-threshold": {"v_range": [["7.2.0", ""]], "type": "str"},
                    "priority-in-sla": {"v_range": [["7.2.1", ""]], "type": "int"},
                    "priority-out-sla": {"v_range": [["7.2.1", ""]], "type": "int"}
                },
                "elements": "dict"
            },
            "sla-fail-log-period": {"v_range": [["6.4.1", ""]], "type": "int"},
            "sla-pass-log-period": {"v_range": [["6.4.1", ""]], "no_log": true, "type": "int"},
            "system-dns": {"v_range": [["6.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "threshold-alert-jitter": {"v_range": [["6.4.1", ""]], "type": "int"},
            "threshold-alert-latency": {"v_range": [["6.4.1", ""]], "type": "int"},
            "threshold-alert-packetloss": {"v_range": [["6.4.1", ""]], "type": "int"},
            "threshold-warning-jitter": {"v_range": [["6.4.1", ""]], "type": "int"},
            "threshold-warning-latency": {"v_range": [["6.4.1", ""]], "type": "int"},
            "threshold-warning-packetloss": {"v_range": [["6.4.1", ""]], "type": "int"},
            "update-cascade-interface": {"v_range": [["6.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "update-static-route": {"v_range": [["6.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "user": {"v_range": [["6.4.2", ""]], "type": "str"},
            "detect-mode": {"v_range": [["7.0.0", ""]], "choices": ["active", "passive", "prefer-passive", "remote", "agent-based"], "type": "str"},
            "mos-codec": {"v_range": [["7.2.0", ""]], "choices": ["g711", "g722", "g729"], "type": "str"},
            "source": {"v_range": [["7.2.0", ""]], "type": "str"},
            "vrf": {"v_range": [["7.2.0", ""]], "type": "int"},
            "embed-measured-health": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "sla-id-redistribute": {"v_range": [["7.2.1", ""]], "type": "int"},
            "class-id": {"v_range": [["7.4.0", ""]], "type": "str"},
            "source6": {"v_range": [["7.4.0", ""]], "type": "str"}
        }
    },
    "wanprof_system_sdwan_healthcheck_sla": {
        "stated": true,
        "options": {
            "id": {"v_range": [["6.4.1", ""]], "required": true, "type": "int"},
            "jitter-threshold": {"v_range": [["6.4.1", ""]], "type": "int"},
            "latency-threshold": {"v_range": [["6.4.1", ""]], "type": "int"},
            "link-cost-factor": {
                "v_range": [["6.4.1", ""]],
                "type": "list",
                "choices": ["latency", "jitter", "packet-loss", "mos", "remote"],
                "elements": "str"
            },
            "packetloss-threshold": {"v_range": [["6.4.1", ""]], "type": "int"},
            "mos-threshold": {"v_range": [["7.2.0", ""]], "type": "str"},
            "priority-in-sla": {"v_range": [["7.2.1", ""]], "type": "int"},
            "priority-out-sla": {"v_range": [["7.2.1", ""]], "type": "int"}
        }
    },
    "wanprof_system_sdwan_members": {
        "stated": true,
        "options": {
            "_dynamic-member": {"v_range": [["6.4.1", "6.4.14"]], "type": "str"},
            "comment": {"v_range": [["6.4.1", ""]], "type": "str"},
            "cost": {"v_range": [["6.4.1", ""]], "type": "int"},
            "gateway": {"v_range": [["6.4.1", ""]], "type": "str"},
            "gateway6": {"v_range": [["6.4.1", ""]], "type": "str"},
            "ingress-spillover-threshold": {"v_range": [["6.4.1", ""]], "type": "int"},
            "interface": {"v_range": [["6.4.1", ""]], "type": "str"},
            "priority": {"v_range": [["6.4.1", ""]], "type": "int"},
            "seq-num": {"v_range": [["6.4.1", ""]], "required": true, "type": "int"},
            "source": {"v_range": [["6.4.1", ""]], "type": "str"},
            "source6": {"v_range": [["6.4.1", ""]], "type": "str"},
            "spillover-threshold": {"v_range": [["6.4.1", ""]], "type": "int"},
            "status": {"v_range": [["6.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "volume-ratio": {"v_range": [["6.4.1", ""]], "type": "int"},
            "weight": {"v_range": [["6.4.1", ""]], "type": "int"},
            "zone": {"v_range": [["6.4.1", ""]], "type": "str"},
            "priority6": {"v_range": [["7.0.0", ""]], "type": "int"},
            "preferred-source": {"v_range": [["7.4.0", ""]], "type": "str"},
            "transport-group": {"v_range": [["7.4.2", ""]], "type": "int"},
            "priority-in-sla": {"v_range": [["7.6.0", ""]], "type": "int"},
            "priority-out-sla": {"v_range": [["7.6.0", ""]], "type": "int"}
        }
    },
    "wanprof_system_sdwan_neighbor": {
        "stated": true,
        "options": {
            "health-check": {"v_range": [["6.4.1", ""]], "type": "str"},
            "ip": {"v_range": [["6.4.1", ""]], "type": "str"},
            "member": {"v_range": [["6.4.1", ""]], "type": "raw"},
            "role": {"v_range": [["6.4.1", ""]], "choices": ["primary", "secondary", "standalone"], "type": "str"},
            "sla-id": {"v_range": [["6.4.1", ""]], "type": "int"},
            "minimum-sla-meet-members": {"v_range": [["7.2.0", ""]], "type": "int"},
            "mode": {"v_range": [["7.0.1", ""]], "choices": ["sla", "speedtest"], "type": "str"},
            "service-id": {"v_range": [["7.4.1", ""]], "type": "str"}
        }
    },
    "wanprof_system_sdwan_service": {
        "stated": true,
        "options": {
            "addr-mode": {"v_range": [["6.4.1", ""]], "choices": ["ipv4", "ipv6"], "type": "str"},
            "bandwidth-weight": {"v_range": [["6.4.1", ""]], "type": "int"},
            "default": {"v_range": [["6.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "dscp-forward": {"v_range": [["6.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "dscp-forward-tag": {"v_range": [["6.4.1", ""]], "type": "str"},
            "dscp-reverse": {"v_range": [["6.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "dscp-reverse-tag": {"v_range": [["6.4.1", ""]], "type": "str"},
            "dst": {"v_range": [["6.4.1", ""]], "type": "raw"},
            "dst-negate": {"v_range": [["6.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "dst6": {"v_range": [["6.4.1", ""]], "type": "raw"},
            "end-port": {"v_range": [["6.4.1", ""]], "type": "int"},
            "gateway": {"v_range": [["6.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "groups": {"v_range": [["6.4.1", ""]], "type": "raw"},
            "hash-mode": {
                "v_range": [["6.4.2", ""]],
                "choices": ["round-robin", "source-ip-based", "source-dest-ip-based", "inbandwidth", "outbandwidth", "bibandwidth"],
                "type": "str"
            },
            "health-check": {"v_range": [["6.4.1", ""]], "type": "raw"},
            "hold-down-time": {"v_range": [["6.4.1", ""]], "type": "int"},
            "id": {"v_range": [["6.4.1", ""]], "required": true, "type": "int"},
            "input-device": {"v_range": [["6.4.1", ""]], "type": "raw"},
            "input-device-negate": {"v_range": [["6.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "internet-service": {"v_range": [["6.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "internet-service-app-ctrl": {"v_range": [["6.4.1", ""]], "type": "raw"},
            "internet-service-app-ctrl-group": {"v_range": [["6.4.1", ""]], "type": "raw"},
            "internet-service-custom": {"v_range": [["6.4.1", ""]], "type": "raw"},
            "internet-service-custom-group": {"v_range": [["6.4.1", ""]], "type": "raw"},
            "internet-service-group": {"v_range": [["6.4.1", ""]], "type": "raw"},
            "internet-service-name": {"v_range": [["6.4.1", ""]], "type": "raw"},
            "jitter-weight": {"v_range": [["6.4.1", ""]], "type": "int"},
            "latency-weight": {"v_range": [["6.4.1", ""]], "type": "int"},
            "link-cost-factor": {
                "v_range": [["6.4.1", ""]],
                "choices": ["latency", "jitter", "packet-loss", "inbandwidth", "outbandwidth", "bibandwidth", "custom-profile-1"],
                "type": "str"
            },
            "link-cost-threshold": {"v_range": [["6.4.1", ""]], "type": "int"},
            "minimum-sla-meet-members": {"v_range": [["6.4.2", ""]], "type": "int"},
            "mode": {"v_range": [["6.4.1", ""]], "choices": ["auto", "manual", "priority", "sla", "load-balance"], "type": "str"},
            "name": {"v_range": [["6.4.1", ""]], "type": "str"},
            "packet-loss-weight": {"v_range": [["6.4.1", ""]], "type": "int"},
            "priority-members": {"v_range": [["6.4.1", ""]], "type": "raw"},
            "protocol": {"v_range": [["6.4.1", ""]], "type": "int"},
            "quality-link": {"v_range": [["6.4.1", ""]], "type": "int"},
            "role": {"v_range": [["6.4.1", ""]], "choices": ["primary", "secondary", "standalone"], "type": "str"},
            "route-tag": {"v_range": [["6.4.1", ""]], "type": "int"},
            "sla": {
                "v_range": [["6.4.1", ""]],
                "type": "list",
                "options": {"health-check": {"v_range": [["6.4.1", ""]], "type": "str"}, "id": {"v_range": [["6.4.1", ""]], "type": "int"}},
                "elements": "dict"
            },
            "sla-compare-method": {"v_range": [["6.4.1", ""]], "choices": ["order", "number"], "type": "str"},
            "src": {"v_range": [["6.4.1", ""]], "type": "raw"},
            "src-negate": {"v_range": [["6.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "src6": {"v_range": [["6.4.1", ""]], "type": "raw"},
            "standalone-action": {"v_range": [["6.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "start-port": {"v_range": [["6.4.1", ""]], "type": "int"},
            "status": {"v_range": [["6.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "tos": {"v_range": [["6.4.1", ""]], "type": "str"},
            "tos-mask": {"v_range": [["6.4.1", ""]], "type": "str"},
            "users": {"v_range": [["6.4.1", ""]], "type": "raw"},
            "tie-break": {"v_range": [["6.4.3", ""]], "choices": ["zone", "cfg-order", "fib-best-match", "input-device"], "type": "str"},
            "use-shortcut-sla": {"v_range": [["6.4.4", ""]], "choices": ["disable", "enable"], "type": "str"},
            "input-zone": {"v_range": [["7.2.0", ""]], "type": "raw"},
            "internet-service-app-ctrl-category": {"v_range": [["7.2.0", ""]], "type": "raw"},
            "passive-measurement": {"v_range": [["7.0.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "priority-zone": {"v_range": [["7.0.1", ""]], "type": "raw"},
            "agent-exclusive": {"v_range": [["7.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "shortcut": {"v_range": [["7.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "shortcut-stickiness": {"v_range": [["7.4.0", "7.4.1"]], "choices": ["disable", "enable"], "type": "str"},
            "end-src-port": {"v_range": [["7.4.1", ""]], "type": "int"},
            "load-balance": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "sla-stickiness": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "start-src-port": {"v_range": [["7.4.1", ""]], "type": "int"},
            "zone-mode": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "shortcut-priority": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable", "auto"], "type": "str"},
            "comment": {"v_range": [["7.6.0", ""]], "type": "str"}
        }
    },
    "wanprof_system_sdwan_service_sla": {
        "stated": true,
        "options": {"health-check": {"v_range": [["6.4.1", ""]], "type": "str"}, "id": {"v_range": [["6.4.1", ""]], "required": true, "type": "int"}}
    },
    "wanprof_system_sdwan_zone": {
        "stated": true,
        "options": {
            "name": {"v_range": [["6.4.1", ""]], "required": true, "type": "str"},
            "service-sla-tie-break": {"v_range": [["6.4.3", ""]], "choices": ["cfg-order", "fib-best-match", "input-device"], "type": "str"},
            "minimum-sla-meet-members": {"v_range": [["7.4.1", ""]], "type": "int"},
            "advpn-health-check": {"v_range": [["7.4.2", ""]], "type": "str"},
            "advpn-select": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "wanprof_system_virtualwanlink": {
        "stated": false,
        "options": {
            "fail-detect": {"choices": ["disable", "enable"], "type": "str"},
            "health-check": {
                "type": "list",
                "options": {
                    "_dynamic-server": {"v_range": [["6.0.0", "6.4.14"]], "type": "str"},
                    "addr-mode": {"choices": ["ipv4", "ipv6"], "type": "str"},
                    "failtime": {"type": "int"},
                    "http-agent": {"type": "str"},
                    "http-get": {"type": "str"},
                    "http-match": {"type": "str"},
                    "interval": {"type": "int"},
                    "members": {"type": "raw"},
                    "name": {"type": "str"},
                    "packet-size": {"type": "int"},
                    "password": {"no_log": true, "type": "raw"},
                    "port": {"type": "int"},
                    "protocol": {"choices": ["ping", "tcp-echo", "udp-echo", "http", "twamp", "ping6", "dns"], "type": "str"},
                    "recoverytime": {"type": "int"},
                    "security-mode": {"choices": ["none", "authentication"], "type": "str"},
                    "server": {"type": "raw"},
                    "sla": {
                        "type": "list",
                        "options": {
                            "id": {"type": "int"},
                            "jitter-threshold": {"type": "int"},
                            "latency-threshold": {"type": "int"},
                            "link-cost-factor": {"type": "list", "choices": ["latency", "jitter", "packet-loss"], "elements": "str"},
                            "packetloss-threshold": {"type": "int"}
                        },
                        "elements": "dict"
                    },
                    "threshold-alert-jitter": {"type": "int"},
                    "threshold-alert-latency": {"type": "int"},
                    "threshold-alert-packetloss": {"type": "int"},
                    "threshold-warning-jitter": {"type": "int"},
                    "threshold-warning-latency": {"type": "int"},
                    "threshold-warning-packetloss": {"type": "int"},
                    "update-cascade-interface": {"choices": ["disable", "enable"], "type": "str"},
                    "update-static-route": {"choices": ["disable", "enable"], "type": "str"},
                    "internet-service-id": {"v_range": [["6.2.0", "7.2.0"]], "type": "str"},
                    "probe-packets": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "sla-fail-log-period": {"v_range": [["6.2.0", ""]], "type": "int"},
                    "sla-pass-log-period": {"v_range": [["6.2.0", ""]], "no_log": true, "type": "int"},
                    "timeout": {"v_range": [["6.2.0", "6.4.14"]], "type": "int"},
                    "ha-priority": {"v_range": [["6.2.2", ""]], "type": "int"},
                    "diffservcode": {"v_range": [["6.2.5", ""]], "type": "str"},
                    "probe-timeout": {"v_range": [["6.2.5", ""]], "type": "int"},
                    "dns-request-domain": {"v_range": [["6.4.0", "6.4.0"]], "type": "str"},
                    "probe-count": {"v_range": [["6.4.0", "6.4.0"]], "type": "int"},
                    "system-dns": {"v_range": [["6.4.0", "6.4.0"]], "choices": ["disable", "enable"], "type": "str"}
                },
                "elements": "dict"
            },
            "load-balance-mode": {
                "choices": ["source-ip-based", "weight-based", "usage-based", "source-dest-ip-based", "measured-volume-based"],
                "type": "str"
            },
            "members": {
                "type": "list",
                "options": {
                    "_dynamic-member": {"v_range": [["6.0.0", "6.4.14"]], "type": "str"},
                    "comment": {"type": "str"},
                    "gateway": {"type": "str"},
                    "gateway6": {"type": "str"},
                    "ingress-spillover-threshold": {"type": "int"},
                    "interface": {"type": "str"},
                    "priority": {"type": "int"},
                    "seq-num": {"type": "int"},
                    "source": {"type": "str"},
                    "source6": {"type": "str"},
                    "spillover-threshold": {"type": "int"},
                    "status": {"choices": ["disable", "enable"], "type": "str"},
                    "volume-ratio": {"type": "int"},
                    "weight": {"type": "int"},
                    "cost": {"v_range": [["6.2.0", ""]], "type": "int"}
                },
                "elements": "dict"
            },
            "service": {
                "type": "list",
                "options": {
                    "addr-mode": {"choices": ["ipv4", "ipv6"], "type": "str"},
                    "bandwidth-weight": {"type": "int"},
                    "default": {"choices": ["disable", "enable"], "type": "str"},
                    "dscp-forward": {"choices": ["disable", "enable"], "type": "str"},
                    "dscp-forward-tag": {"type": "str"},
                    "dscp-reverse": {"choices": ["disable", "enable"], "type": "str"},
                    "dscp-reverse-tag": {"type": "str"},
                    "dst": {"type": "raw"},
                    "dst-negate": {"choices": ["disable", "enable"], "type": "str"},
                    "dst6": {"type": "raw"},
                    "end-port": {"type": "int"},
                    "gateway": {"choices": ["disable", "enable"], "type": "str"},
                    "groups": {"type": "raw"},
                    "health-check": {"type": "str"},
                    "hold-down-time": {"type": "int"},
                    "id": {"type": "int"},
                    "internet-service": {"choices": ["disable", "enable"], "type": "str"},
                    "internet-service-ctrl": {"v_range": [["6.0.0", "7.2.1"]], "type": "raw"},
                    "internet-service-ctrl-group": {"v_range": [["6.0.0", "7.2.1"]], "type": "raw"},
                    "internet-service-custom": {"type": "raw"},
                    "internet-service-custom-group": {"type": "raw"},
                    "internet-service-group": {"type": "raw"},
                    "internet-service-id": {"type": "raw"},
                    "jitter-weight": {"type": "int"},
                    "latency-weight": {"type": "int"},
                    "link-cost-factor": {
                        "choices": ["latency", "jitter", "packet-loss", "inbandwidth", "outbandwidth", "bibandwidth", "custom-profile-1"],
                        "type": "str"
                    },
                    "link-cost-threshold": {"type": "int"},
                    "member": {"v_range": [["6.0.0", "7.2.1"]], "type": "str"},
                    "mode": {"choices": ["auto", "manual", "priority", "sla", "load-balance"], "type": "str"},
                    "name": {"type": "str"},
                    "packet-loss-weight": {"type": "int"},
                    "priority-members": {"type": "raw"},
                    "protocol": {"type": "int"},
                    "quality-link": {"type": "int"},
                    "route-tag": {"type": "int"},
                    "sla": {"type": "list", "options": {"health-check": {"type": "str"}, "id": {"type": "int"}}, "elements": "dict"},
                    "src": {"type": "raw"},
                    "src-negate": {"choices": ["disable", "enable"], "type": "str"},
                    "src6": {"type": "raw"},
                    "start-port": {"type": "int"},
                    "status": {"choices": ["disable", "enable"], "type": "str"},
                    "tos": {"type": "str"},
                    "tos-mask": {"type": "str"},
                    "users": {"type": "raw"},
                    "internet-service-app-ctrl": {"v_range": [["6.2.0", ""]], "type": "raw"},
                    "internet-service-app-ctrl-group": {"v_range": [["6.2.0", ""]], "type": "raw"},
                    "role": {"v_range": [["6.2.1", ""]], "choices": ["primary", "secondary", "standalone"], "type": "str"},
                    "sla-compare-method": {"v_range": [["6.2.1", ""]], "choices": ["order", "number"], "type": "str"},
                    "standalone-action": {"v_range": [["6.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "input-device": {"v_range": [["6.2.2", ""]], "type": "raw"},
                    "internet-service-name": {"v_range": [["6.4.0", "6.4.0"]], "type": "str"},
                    "input-device-negate": {"v_range": [["6.4.1", ""]], "choices": ["disable", "enable"], "type": "str"}
                },
                "elements": "dict"
            },
            "status": {"choices": ["disable", "enable"], "type": "str"},
            "neighbor": {
                "v_range": [["6.2.1", ""]],
                "type": "list",
                "options": {
                    "health-check": {"v_range": [["6.2.1", ""]], "type": "str"},
                    "ip": {"v_range": [["6.2.1", ""]], "type": "str"},
                    "member": {"v_range": [["6.2.1", ""]], "type": "str"},
                    "role": {"v_range": [["6.2.1", ""]], "choices": ["primary", "secondary", "standalone"], "type": "str"},
                    "sla-id": {"v_range": [["6.2.1", ""]], "type": "int"}
                },
                "elements": "dict"
            },
            "neighbor-hold-boot-time": {"v_range": [["6.2.1", ""]], "type": "int"},
            "neighbor-hold-down": {"v_range": [["6.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "neighbor-hold-down-time": {"v_range": [["6.2.1", ""]], "type": "int"},
            "fail-alert-interfaces": {"v_range": [["7.2.3", ""]], "type": "raw"}
        }
    },
    "wanprof_system_virtualwanlink_healthcheck": {
        "stated": true,
        "options": {
            "_dynamic-server": {"v_range": [["6.0.0", "6.4.14"]], "type": "str"},
            "addr-mode": {"choices": ["ipv4", "ipv6"], "type": "str"},
            "failtime": {"type": "int"},
            "http-agent": {"type": "str"},
            "http-get": {"type": "str"},
            "http-match": {"type": "str"},
            "interval": {"type": "int"},
            "members": {"type": "raw"},
            "name": {"required": true, "type": "str"},
            "packet-size": {"type": "int"},
            "password": {"no_log": true, "type": "raw"},
            "port": {"type": "int"},
            "protocol": {"choices": ["ping", "tcp-echo", "udp-echo", "http", "twamp", "ping6", "dns"], "type": "str"},
            "recoverytime": {"type": "int"},
            "security-mode": {"choices": ["none", "authentication"], "type": "str"},
            "server": {"type": "raw"},
            "sla": {
                "type": "list",
                "options": {
                    "id": {"type": "int"},
                    "jitter-threshold": {"type": "int"},
                    "latency-threshold": {"type": "int"},
                    "link-cost-factor": {"type": "list", "choices": ["latency", "jitter", "packet-loss"], "elements": "str"},
                    "packetloss-threshold": {"type": "int"}
                },
                "elements": "dict"
            },
            "threshold-alert-jitter": {"type": "int"},
            "threshold-alert-latency": {"type": "int"},
            "threshold-alert-packetloss": {"type": "int"},
            "threshold-warning-jitter": {"type": "int"},
            "threshold-warning-latency": {"type": "int"},
            "threshold-warning-packetloss": {"type": "int"},
            "update-cascade-interface": {"choices": ["disable", "enable"], "type": "str"},
            "update-static-route": {"choices": ["disable", "enable"], "type": "str"},
            "internet-service-id": {"v_range": [["6.2.0", "7.2.0"]], "type": "str"},
            "probe-packets": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "sla-fail-log-period": {"v_range": [["6.2.0", ""]], "type": "int"},
            "sla-pass-log-period": {"v_range": [["6.2.0", ""]], "no_log": true, "type": "int"},
            "timeout": {"v_range": [["6.2.0", "6.4.14"]], "type": "int"},
            "ha-priority": {"v_range": [["6.2.2", ""]], "type": "int"},
            "diffservcode": {"v_range": [["6.2.5", ""]], "type": "str"},
            "probe-timeout": {"v_range": [["6.2.5", ""]], "type": "int"},
            "dns-request-domain": {"v_range": [["6.4.0", "6.4.0"]], "type": "str"},
            "probe-count": {"v_range": [["6.4.0", "6.4.0"]], "type": "int"},
            "system-dns": {"v_range": [["6.4.0", "6.4.0"]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "wanprof_system_virtualwanlink_healthcheck_sla": {
        "stated": true,
        "options": {
            "id": {"required": true, "type": "int"},
            "jitter-threshold": {"type": "int"},
            "latency-threshold": {"type": "int"},
            "link-cost-factor": {"type": "list", "choices": ["latency", "jitter", "packet-loss"], "elements": "str"},
            "packetloss-threshold": {"type": "int"}
        }
    },
    "wanprof_system_virtualwanlink_members": {
        "stated": true,
        "options": {
            "_dynamic-member": {"v_range": [["6.0.0", "6.4.14"]], "type": "str"},
            "comment": {"type": "str"},
            "gateway": {"type": "str"},
            "gateway6": {"type": "str"},
            "ingress-spillover-threshold": {"type": "int"},
            "interface": {"type": "str"},
            "priority": {"type": "int"},
            "seq-num": {"required": true, "type": "int"},
            "source": {"type": "str"},
            "source6": {"type": "str"},
            "spillover-threshold": {"type": "int"},
            "status": {"choices": ["disable", "enable"], "type": "str"},
            "volume-ratio": {"type": "int"},
            "weight": {"type": "int"},
            "cost": {"v_range": [["6.2.0", ""]], "type": "int"}
        }
    },
    "wanprof_system_virtualwanlink_neighbor": {
        "stated": true,
        "options": {
            "health-check": {"v_range": [["6.2.1", ""]], "type": "str"},
            "ip": {"v_range": [["6.2.1", ""]], "type": "str"},
            "member": {"v_range": [["6.2.1", ""]], "type": "str"},
            "role": {"v_range": [["6.2.1", ""]], "choices": ["primary", "secondary", "standalone"], "type": "str"},
            "sla-id": {"v_range": [["6.2.1", ""]], "type": "int"}
        }
    },
    "wanprof_system_virtualwanlink_service": {
        "stated": true,
        "options": {
            "addr-mode": {"choices": ["ipv4", "ipv6"], "type": "str"},
            "bandwidth-weight": {"type": "int"},
            "default": {"choices": ["disable", "enable"], "type": "str"},
            "dscp-forward": {"choices": ["disable", "enable"], "type": "str"},
            "dscp-forward-tag": {"type": "str"},
            "dscp-reverse": {"choices": ["disable", "enable"], "type": "str"},
            "dscp-reverse-tag": {"type": "str"},
            "dst": {"type": "raw"},
            "dst-negate": {"choices": ["disable", "enable"], "type": "str"},
            "dst6": {"type": "raw"},
            "end-port": {"type": "int"},
            "gateway": {"choices": ["disable", "enable"], "type": "str"},
            "groups": {"type": "raw"},
            "health-check": {"type": "str"},
            "hold-down-time": {"type": "int"},
            "id": {"required": true, "type": "int"},
            "internet-service": {"choices": ["disable", "enable"], "type": "str"},
            "internet-service-ctrl": {"v_range": [["6.0.0", "7.2.1"]], "type": "raw"},
            "internet-service-ctrl-group": {"v_range": [["6.0.0", "7.2.1"]], "type": "raw"},
            "internet-service-custom": {"type": "raw"},
            "internet-service-custom-group": {"type": "raw"},
            "internet-service-group": {"type": "raw"},
            "internet-service-id": {"type": "raw"},
            "jitter-weight": {"type": "int"},
            "latency-weight": {"type": "int"},
            "link-cost-factor": {
                "choices": ["latency", "jitter", "packet-loss", "inbandwidth", "outbandwidth", "bibandwidth", "custom-profile-1"],
                "type": "str"
            },
            "link-cost-threshold": {"type": "int"},
            "member": {"v_range": [["6.0.0", "7.2.1"]], "type": "str"},
            "mode": {"choices": ["auto", "manual", "priority", "sla", "load-balance"], "type": "str"},
            "name": {"type": "str"},
            "packet-loss-weight": {"type": "int"},
            "priority-members": {"type": "raw"},
            "protocol": {"type": "int"},
            "quality-link": {"type": "int"},
            "route-tag": {"type": "int"},
            "sla": {"type": "list", "options": {"health-check": {"type": "str"}, "id": {"type": "int"}}, "elements": "dict"},
            "src": {"type": "raw"},
            "src-negate": {"choices": ["disable", "enable"], "type": "str"},
            "src6": {"type": "raw"},
            "start-port": {"type": "int"},
            "status": {"choices": ["disable", "enable"], "type": "str"},
            "tos": {"type": "str"},
            "tos-mask": {"type": "str"},
            "users": {"type": "raw"},
            "internet-service-app-ctrl": {"v_range": [["6.2.0", ""]], "type": "raw"},
            "internet-service-app-ctrl-group": {"v_range": [["6.2.0", ""]], "type": "raw"},
            "role": {"v_range": [["6.2.1", ""]], "choices": ["primary", "secondary", "standalone"], "type": "str"},
            "sla-compare-method": {"v_range": [["6.2.1", ""]], "choices": ["order", "number"], "type": "str"},
            "standalone-action": {"v_range": [["6.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "input-device": {"v_range": [["6.2.2", ""]], "type": "raw"},
            "internet-service-name": {"v_range": [["6.4.0", "6.4.0"]], "type": "str"},
            "input-device-negate": {"v_range": [["6.4.1", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "wanprof_system_virtualwanlink_service_sla": {"stated": true, "options": {"health-check": {"type": "str"}, "id": {"required": true, "type": "int"}}},
    "webfilter_categories": {"stated": true, "options": {"id": {"required": true, "type": "str"}}},
    "webfilter_content": {
        "stated": true,
        "options": {
            "comment": {"type": "str"},
            "entries": {
                "type": "list",
                "options": {
                    "action": {"choices": ["exempt", "block"], "type": "str"},
                    "lang": {"choices": ["western", "simch", "trach", "japanese", "korean", "french", "thai", "spanish", "cyrillic"], "type": "str"},
                    "name": {"type": "str"},
                    "pattern-type": {"choices": ["wildcard", "regexp"], "type": "str"},
                    "score": {"type": "int"},
                    "status": {"choices": ["disable", "enable"], "type": "str"}
                },
                "elements": "dict"
            },
            "id": {"required": true, "type": "int"},
            "name": {"type": "str"}
        }
    },
    "webfilter_content_entries": {
        "stated": true,
        "options": {
            "action": {"choices": ["exempt", "block"], "type": "str"},
            "lang": {"choices": ["western", "simch", "trach", "japanese", "korean", "french", "thai", "spanish", "cyrillic"], "type": "str"},
            "name": {"required": true, "type": "str"},
            "pattern-type": {"choices": ["wildcard", "regexp"], "type": "str"},
            "score": {"type": "int"},
            "status": {"choices": ["disable", "enable"], "type": "str"}
        }
    },
    "webfilter_contentheader": {
        "stated": true,
        "options": {
            "comment": {"type": "str"},
            "entries": {
                "type": "list",
                "options": {"action": {"choices": ["exempt", "block", "allow"], "type": "str"}, "category": {"type": "raw"}, "pattern": {"type": "str"}},
                "elements": "dict"
            },
            "id": {"required": true, "type": "int"},
            "name": {"type": "str"}
        }
    },
    "webfilter_contentheader_entries": {
        "stated": true,
        "options": {"action": {"choices": ["exempt", "block", "allow"], "type": "str"}, "category": {"type": "raw"}, "pattern": {"type": "str"}}
    },
    "webfilter_ftgdlocalcat": {
        "stated": true,
        "options": {"desc": {"type": "str"}, "id": {"required": true, "type": "int"}, "status": {"choices": ["disable", "enable"], "type": "str"}}
    },
    "webfilter_ftgdlocalrating": {
        "stated": true,
        "options": {
            "rating": {"required": true, "type": "raw"},
            "status": {"choices": ["disable", "enable"], "type": "str"},
            "url": {"type": "str"},
            "comment": {"v_range": [["6.4.3", ""]], "type": "str"}
        }
    },
    "webfilter_profile": {
        "stated": true,
        "options": {
            "comment": {"type": "str"},
            "extended-log": {"choices": ["disable", "enable"], "type": "str"},
            "https-replacemsg": {"choices": ["disable", "enable"], "type": "str"},
            "inspection-mode": {"v_range": [["6.0.0", "7.2.1"]], "choices": ["proxy", "flow-based", "dns"], "type": "str"},
            "log-all-url": {"choices": ["disable", "enable"], "type": "str"},
            "name": {"required": true, "type": "str"},
            "options": {
                "type": "list",
                "choices": [
                    "block-invalid-url", "jscript", "js", "vbs", "unknown", "wf-referer", "https-scan", "intrinsic", "wf-cookie", "per-user-bwl",
                    "activexfilter", "cookiefilter", "https-url-scan", "javafilter", "rangeblock", "contenttype-check", "per-user-bal"
                ],
                "elements": "str"
            },
            "ovrd-perm": {
                "type": "list",
                "choices": ["bannedword-override", "urlfilter-override", "fortiguard-wf-override", "contenttype-check-override"],
                "elements": "str"
            },
            "post-action": {"choices": ["normal", "comfort", "block"], "type": "str"},
            "replacemsg-group": {"type": "str"},
            "web-content-log": {"choices": ["disable", "enable"], "type": "str"},
            "web-extended-all-action-log": {"choices": ["disable", "enable"], "type": "str"},
            "web-filter-activex-log": {"choices": ["disable", "enable"], "type": "str"},
            "web-filter-applet-log": {"choices": ["disable", "enable"], "type": "str"},
            "web-filter-command-block-log": {"choices": ["disable", "enable"], "type": "str"},
            "web-filter-cookie-log": {"choices": ["disable", "enable"], "type": "str"},
            "web-filter-cookie-removal-log": {"choices": ["disable", "enable"], "type": "str"},
            "web-filter-js-log": {"choices": ["disable", "enable"], "type": "str"},
            "web-filter-jscript-log": {"choices": ["disable", "enable"], "type": "str"},
            "web-filter-referer-log": {"choices": ["disable", "enable"], "type": "str"},
            "web-filter-unknown-log": {"choices": ["disable", "enable"], "type": "str"},
            "web-filter-vbs-log": {"choices": ["disable", "enable"], "type": "str"},
            "web-ftgd-err-log": {"choices": ["disable", "enable"], "type": "str"},
            "web-ftgd-quota-usage": {"choices": ["disable", "enable"], "type": "str"},
            "web-invalid-domain-log": {"choices": ["disable", "enable"], "type": "str"},
            "web-url-log": {"choices": ["disable", "enable"], "type": "str"},
            "wisp": {"choices": ["disable", "enable"], "type": "str"},
            "wisp-algorithm": {"choices": ["auto-learning", "primary-secondary", "round-robin"], "type": "str"},
            "wisp-servers": {"type": "raw"},
            "youtube-channel-filter": {
                "type": "list",
                "options": {"channel-id": {"type": "str"}, "comment": {"type": "str"}, "id": {"type": "int"}},
                "elements": "dict"
            },
            "youtube-channel-status": {"choices": ["disable", "blacklist", "whitelist"], "type": "str"},
            "feature-set": {"v_range": [["6.4.0", ""]], "choices": ["proxy", "flow"], "type": "str"},
            "web-antiphishing-log": {"v_range": [["6.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "antiphish": {
                "v_range": [["6.4.5", ""]],
                "type": "dict",
                "options": {
                    "check-basic-auth": {"v_range": [["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "check-uri": {"v_range": [["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "check-username-only": {"v_range": [["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "custom-patterns": {
                        "v_range": [["6.4.5", ""]],
                        "type": "list",
                        "options": {
                            "category": {"v_range": [["6.4.5", ""]], "choices": ["username", "password"], "type": "str"},
                            "pattern": {"v_range": [["6.4.5", ""]], "type": "str"},
                            "type": {"v_range": [["7.0.0", ""]], "choices": ["regex", "literal"], "type": "str"}
                        },
                        "elements": "dict"
                    },
                    "default-action": {"v_range": [["6.4.5", ""]], "choices": ["log", "block", "exempt"], "type": "str"},
                    "domain-controller": {"v_range": [["6.4.5", ""]], "type": "str"},
                    "inspection-entries": {
                        "v_range": [["6.4.5", ""]],
                        "type": "list",
                        "options": {
                            "action": {"v_range": [["6.4.5", ""]], "choices": ["log", "block", "exempt"], "type": "str"},
                            "fortiguard-category": {"v_range": [["6.4.5", ""]], "type": "raw"},
                            "name": {"v_range": [["6.4.5", ""]], "type": "str"}
                        },
                        "elements": "dict"
                    },
                    "max-body-len": {"v_range": [["6.4.5", ""]], "type": "int"},
                    "status": {"v_range": [["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "authentication": {"v_range": [["7.0.0", ""]], "choices": ["domain-controller", "ldap"], "type": "str"},
                    "ldap": {"v_range": [["7.0.0", ""]], "type": "str"}
                }
            },
            "ftgd-wf": {
                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                "type": "dict",
                "options": {
                    "exempt-quota": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "raw"},
                    "filters": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "type": "list",
                        "options": {
                            "action": {
                                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                                "choices": ["block", "monitor", "warning", "authenticate"],
                                "type": "str"
                            },
                            "auth-usr-grp": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "raw"},
                            "category": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                            "id": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                            "log": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                            "override-replacemsg": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                            "warn-duration": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                            "warning-duration-type": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["session", "timeout"], "type": "str"},
                            "warning-prompt": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["per-domain", "per-category"], "type": "str"}
                        },
                        "elements": "dict"
                    },
                    "max-quota-timeout": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "options": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "type": "list",
                        "choices": [
                            "error-allow", "http-err-detail", "rate-image-urls", "strict-blocking", "rate-server-ip", "redir-block",
                            "connect-request-bypass", "log-all-url", "ftgd-disable"
                        ],
                        "elements": "str"
                    },
                    "ovrd": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "raw"},
                    "quota": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "type": "list",
                        "options": {
                            "category": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "raw"},
                            "duration": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                            "id": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                            "override-replacemsg": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                            "type": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["time", "traffic"], "type": "str"},
                            "unit": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["B", "KB", "MB", "GB"], "type": "str"},
                            "value": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"}
                        },
                        "elements": "dict"
                    },
                    "rate-crl-urls": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "rate-css-urls": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "rate-image-urls": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "rate-javascript-urls": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "category-override": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", "6.4.14"]], "type": "str"}
                }
            },
            "override": {
                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                "type": "dict",
                "options": {
                    "ovrd-cookie": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["deny", "allow"], "type": "str"},
                    "ovrd-dur": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "ovrd-dur-mode": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["constant", "ask"], "type": "str"},
                    "ovrd-scope": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "choices": ["user", "user-group", "ip", "ask", "browser"],
                        "type": "str"
                    },
                    "ovrd-user-group": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "raw"},
                    "profile": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "raw"},
                    "profile-attribute": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "choices": [
                            "User-Name", "User-Password", "CHAP-Password", "NAS-IP-Address", "NAS-Port", "Service-Type", "Framed-Protocol",
                            "Framed-IP-Address", "Framed-IP-Netmask", "Framed-Routing", "Filter-Id", "Framed-MTU", "Framed-Compression", "Login-IP-Host",
                            "Login-Service", "Login-TCP-Port", "Reply-Message", "Callback-Number", "Callback-Id", "Framed-Route", "Framed-IPX-Network",
                            "State", "Class", "Vendor-Specific", "Session-Timeout", "Idle-Timeout", "Termination-Action", "Called-Station-Id",
                            "Calling-Station-Id", "NAS-Identifier", "Proxy-State", "Login-LAT-Service", "Login-LAT-Node", "Login-LAT-Group",
                            "Framed-AppleTalk-Link", "Framed-AppleTalk-Network", "Framed-AppleTalk-Zone", "Acct-Status-Type", "Acct-Delay-Time",
                            "Acct-Input-Octets", "Acct-Output-Octets", "Acct-Session-Id", "Acct-Authentic", "Acct-Session-Time", "Acct-Input-Packets",
                            "Acct-Output-Packets", "Acct-Terminate-Cause", "Acct-Multi-Session-Id", "Acct-Link-Count", "CHAP-Challenge", "NAS-Port-Type",
                            "Port-Limit", "Login-LAT-Port"
                        ],
                        "type": "str"
                    },
                    "profile-type": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["list", "radius"], "type": "str"}
                }
            },
            "url-extraction": {
                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                "type": "dict",
                "options": {
                    "redirect-header": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "redirect-no-content": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "redirect-url": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "server-fqdn": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "status": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"}
                }
            },
            "web": {
                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                "type": "dict",
                "options": {
                    "blacklist": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "bword-table": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "bword-threshold": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "content-header-list": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "keyword-match": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "no_log": true, "type": "raw"},
                    "log-search": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "safe-search": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "type": "list",
                        "choices": ["google", "yahoo", "bing", "url", "header"],
                        "elements": "str"
                    },
                    "urlfilter-table": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "whitelist": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "type": "list",
                        "choices": [
                            "exempt-av", "exempt-webcontent", "exempt-activex-java-cookie", "exempt-dlp", "exempt-rangeblock", "extended-log-others"
                        ],
                        "elements": "str"
                    },
                    "youtube-restrict": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["strict", "none", "moderate"], "type": "str"},
                    "allowlist": {
                        "v_range": [["7.0.0", ""]],
                        "type": "list",
                        "choices": [
                            "exempt-av", "exempt-webcontent", "exempt-activex-java-cookie", "exempt-dlp", "exempt-rangeblock", "extended-log-others"
                        ],
                        "elements": "str"
                    },
                    "blocklist": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "vimeo-restrict": {"v_range": [["7.0.1", ""]], "type": "str"}
                }
            },
            "file-filter": {
                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                "type": "dict",
                "options": {
                    "entries": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "type": "list",
                        "options": {
                            "action": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["log", "block"], "type": "str"},
                            "comment": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                            "direction": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["any", "incoming", "outgoing"], "type": "str"},
                            "encryption": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", "7.2.0"]], "choices": ["any", "yes"], "type": "str"},
                            "file-type": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "raw"},
                            "filter": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                            "password-protected": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["any", "yes"], "type": "str"},
                            "protocol": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "list", "choices": ["http", "ftp"], "elements": "str"}
                        },
                        "elements": "dict"
                    },
                    "log": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "scan-archive-contents": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "status": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"}
                }
            },
            "web-flow-log-encoding": {"v_range": [["7.4.2", ""]], "choices": ["utf-8", "punycode"], "type": "str"}
        }
    },
    "webfilter_profile_antiphish": {
        "stated": false,
        "options": {
            "check-basic-auth": {"v_range": [["6.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "check-uri": {"v_range": [["6.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "custom-patterns": {
                "v_range": [["6.4.0", ""]],
                "type": "list",
                "options": {
                    "category": {"v_range": [["6.4.0", ""]], "choices": ["username", "password"], "type": "str"},
                    "pattern": {"v_range": [["6.4.0", ""]], "type": "str"},
                    "type": {"v_range": [["7.0.0", ""]], "choices": ["regex", "literal"], "type": "str"}
                },
                "elements": "dict"
            },
            "default-action": {"v_range": [["6.4.0", ""]], "choices": ["log", "block", "exempt"], "type": "str"},
            "domain-controller": {"v_range": [["6.4.0", ""]], "type": "str"},
            "inspection-entries": {
                "v_range": [["6.4.0", ""]],
                "type": "list",
                "options": {
                    "action": {"v_range": [["6.4.0", ""]], "choices": ["log", "block", "exempt"], "type": "str"},
                    "fortiguard-category": {"v_range": [["6.4.0", ""]], "type": "raw"},
                    "name": {"v_range": [["6.4.0", ""]], "type": "str"}
                },
                "elements": "dict"
            },
            "max-body-len": {"v_range": [["6.4.0", ""]], "type": "int"},
            "status": {"v_range": [["6.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "check-username-only": {"v_range": [["6.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "authentication": {"v_range": [["7.0.0", ""]], "choices": ["domain-controller", "ldap"], "type": "str"},
            "ldap": {"v_range": [["7.0.0", ""]], "type": "str"}
        }
    },
    "webfilter_profile_antiphish_custompatterns": {
        "stated": true,
        "options": {
            "category": {"v_range": [["6.4.0", ""]], "choices": ["username", "password"], "type": "str"},
            "pattern": {"v_range": [["6.4.0", ""]], "type": "str"},
            "type": {"v_range": [["7.0.0", ""]], "choices": ["regex", "literal"], "type": "str"}
        }
    },
    "webfilter_profile_antiphish_inspectionentries": {
        "stated": true,
        "options": {
            "action": {"v_range": [["6.4.0", ""]], "choices": ["log", "block", "exempt"], "type": "str"},
            "fortiguard-category": {"v_range": [["6.4.0", ""]], "type": "raw"},
            "name": {"v_range": [["6.4.0", ""]], "required": true, "type": "str"}
        }
    },
    "webfilter_profile_filefilter": {
        "stated": false,
        "options": {
            "entries": {
                "v_range": [["6.2.0", ""]],
                "type": "list",
                "options": {
                    "action": {"v_range": [["6.2.0", ""]], "choices": ["log", "block"], "type": "str"},
                    "comment": {"v_range": [["6.2.0", ""]], "type": "str"},
                    "direction": {"v_range": [["6.2.0", ""]], "choices": ["any", "incoming", "outgoing"], "type": "str"},
                    "encryption": {"v_range": [["6.2.0", "7.2.0"]], "choices": ["any", "yes"], "type": "str"},
                    "file-type": {"v_range": [["6.2.0", ""]], "type": "raw"},
                    "filter": {"v_range": [["6.2.0", ""]], "type": "str"},
                    "password-protected": {"v_range": [["6.2.1", ""]], "choices": ["any", "yes"], "type": "str"},
                    "protocol": {"v_range": [["6.2.0", ""]], "type": "list", "choices": ["http", "ftp"], "elements": "str"}
                },
                "elements": "dict"
            },
            "log": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "scan-archive-contents": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "status": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "webfilter_profile_filefilter_entries": {
        "stated": true,
        "options": {
            "action": {"v_range": [["6.2.0", ""]], "choices": ["log", "block"], "type": "str"},
            "comment": {"v_range": [["6.2.0", ""]], "type": "str"},
            "direction": {"v_range": [["6.2.0", ""]], "choices": ["any", "incoming", "outgoing"], "type": "str"},
            "encryption": {"v_range": [["6.2.0", "7.2.0"]], "choices": ["any", "yes"], "type": "str"},
            "file-type": {"v_range": [["6.2.0", ""]], "type": "raw"},
            "filter": {"v_range": [["6.2.0", ""]], "type": "str"},
            "password-protected": {"v_range": [["6.2.1", ""]], "choices": ["any", "yes"], "type": "str"},
            "protocol": {"v_range": [["6.2.0", ""]], "type": "list", "choices": ["http", "ftp"], "elements": "str"}
        }
    },
    "webfilter_profile_ftgdwf": {
        "stated": false,
        "options": {
            "exempt-quota": {"type": "raw"},
            "filters": {
                "type": "list",
                "options": {
                    "action": {"choices": ["block", "monitor", "warning", "authenticate"], "type": "str"},
                    "auth-usr-grp": {"type": "raw"},
                    "category": {"type": "str"},
                    "id": {"type": "int"},
                    "log": {"choices": ["disable", "enable"], "type": "str"},
                    "override-replacemsg": {"type": "str"},
                    "warn-duration": {"type": "str"},
                    "warning-duration-type": {"choices": ["session", "timeout"], "type": "str"},
                    "warning-prompt": {"choices": ["per-domain", "per-category"], "type": "str"}
                },
                "elements": "dict"
            },
            "max-quota-timeout": {"type": "int"},
            "options": {
                "type": "list",
                "choices": [
                    "error-allow", "http-err-detail", "rate-image-urls", "strict-blocking", "rate-server-ip", "redir-block", "connect-request-bypass",
                    "log-all-url", "ftgd-disable"
                ],
                "elements": "str"
            },
            "ovrd": {"type": "raw"},
            "quota": {
                "type": "list",
                "options": {
                    "category": {"type": "raw"},
                    "duration": {"type": "str"},
                    "id": {"type": "int"},
                    "override-replacemsg": {"type": "str"},
                    "type": {"choices": ["time", "traffic"], "type": "str"},
                    "unit": {"choices": ["B", "KB", "MB", "GB"], "type": "str"},
                    "value": {"type": "int"}
                },
                "elements": "dict"
            },
            "rate-crl-urls": {"choices": ["disable", "enable"], "type": "str"},
            "rate-css-urls": {"choices": ["disable", "enable"], "type": "str"},
            "rate-image-urls": {"choices": ["disable", "enable"], "type": "str"},
            "rate-javascript-urls": {"choices": ["disable", "enable"], "type": "str"},
            "category-override": {"v_range": [["6.2.0", "6.4.14"]], "type": "str"}
        }
    },
    "webfilter_profile_ftgdwf_filters": {
        "stated": true,
        "options": {
            "action": {"choices": ["block", "monitor", "warning", "authenticate"], "type": "str"},
            "auth-usr-grp": {"type": "raw"},
            "category": {"type": "str"},
            "id": {"required": true, "type": "int"},
            "log": {"choices": ["disable", "enable"], "type": "str"},
            "override-replacemsg": {"type": "str"},
            "warn-duration": {"type": "str"},
            "warning-duration-type": {"choices": ["session", "timeout"], "type": "str"},
            "warning-prompt": {"choices": ["per-domain", "per-category"], "type": "str"}
        }
    },
    "webfilter_profile_ftgdwf_quota": {
        "stated": true,
        "options": {
            "category": {"type": "raw"},
            "duration": {"type": "str"},
            "id": {"required": true, "type": "int"},
            "override-replacemsg": {"type": "str"},
            "type": {"choices": ["time", "traffic"], "type": "str"},
            "unit": {"choices": ["B", "KB", "MB", "GB"], "type": "str"},
            "value": {"type": "int"}
        }
    },
    "webfilter_profile_override": {
        "stated": false,
        "options": {
            "ovrd-cookie": {"choices": ["deny", "allow"], "type": "str"},
            "ovrd-dur": {"type": "str"},
            "ovrd-dur-mode": {"choices": ["constant", "ask"], "type": "str"},
            "ovrd-scope": {"choices": ["user", "user-group", "ip", "ask", "browser"], "type": "str"},
            "ovrd-user-group": {"type": "raw"},
            "profile": {"type": "raw"},
            "profile-attribute": {
                "choices": [
                    "User-Name", "User-Password", "CHAP-Password", "NAS-IP-Address", "NAS-Port", "Service-Type", "Framed-Protocol", "Framed-IP-Address",
                    "Framed-IP-Netmask", "Framed-Routing", "Filter-Id", "Framed-MTU", "Framed-Compression", "Login-IP-Host", "Login-Service",
                    "Login-TCP-Port", "Reply-Message", "Callback-Number", "Callback-Id", "Framed-Route", "Framed-IPX-Network", "State", "Class",
                    "Vendor-Specific", "Session-Timeout", "Idle-Timeout", "Termination-Action", "Called-Station-Id", "Calling-Station-Id",
                    "NAS-Identifier", "Proxy-State", "Login-LAT-Service", "Login-LAT-Node", "Login-LAT-Group", "Framed-AppleTalk-Link",
                    "Framed-AppleTalk-Network", "Framed-AppleTalk-Zone", "Acct-Status-Type", "Acct-Delay-Time", "Acct-Input-Octets",
                    "Acct-Output-Octets", "Acct-Session-Id", "Acct-Authentic", "Acct-Session-Time", "Acct-Input-Packets", "Acct-Output-Packets",
                    "Acct-Terminate-Cause", "Acct-Multi-Session-Id", "Acct-Link-Count", "CHAP-Challenge", "NAS-Port-Type", "Port-Limit",
                    "Login-LAT-Port"
                ],
                "type": "str"
            },
            "profile-type": {"choices": ["list", "radius"], "type": "str"}
        }
    },
    "webfilter_profile_urlextraction": {
        "stated": false,
        "options": {
            "redirect-header": {"type": "str"},
            "redirect-no-content": {"choices": ["disable", "enable"], "type": "str"},
            "redirect-url": {"type": "str"},
            "server-fqdn": {"type": "str"},
            "status": {"choices": ["disable", "enable"], "type": "str"}
        }
    },
    "webfilter_profile_web": {
        "stated": false,
        "options": {
            "blacklist": {"choices": ["disable", "enable"], "type": "str"},
            "bword-table": {"type": "str"},
            "bword-threshold": {"type": "int"},
            "content-header-list": {"type": "str"},
            "keyword-match": {"no_log": true, "type": "raw"},
            "log-search": {"choices": ["disable", "enable"], "type": "str"},
            "safe-search": {"type": "list", "choices": ["google", "yahoo", "bing", "url", "header"], "elements": "str"},
            "urlfilter-table": {"type": "str"},
            "whitelist": {
                "type": "list",
                "choices": ["exempt-av", "exempt-webcontent", "exempt-activex-java-cookie", "exempt-dlp", "exempt-rangeblock", "extended-log-others"],
                "elements": "str"
            },
            "youtube-restrict": {"choices": ["strict", "none", "moderate"], "type": "str"},
            "allowlist": {
                "v_range": [["7.0.0", ""]],
                "type": "list",
                "choices": ["exempt-av", "exempt-webcontent", "exempt-activex-java-cookie", "exempt-dlp", "exempt-rangeblock", "extended-log-others"],
                "elements": "str"
            },
            "blocklist": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "vimeo-restrict": {"v_range": [["7.0.1", ""]], "type": "str"}
        }
    },
    "webfilter_profile_youtubechannelfilter": {
        "stated": true,
        "options": {"channel-id": {"type": "str"}, "comment": {"type": "str"}, "id": {"required": true, "type": "int"}}
    },
    "webfilter_urlfilter": {
        "stated": true,
        "options": {
            "comment": {"type": "str"},
            "entries": {
                "type": "list",
                "options": {
                    "action": {"choices": ["exempt", "block", "allow", "monitor", "pass"], "type": "str"},
                    "dns-address-family": {"choices": ["ipv4", "ipv6", "both"], "type": "str"},
                    "exempt": {
                        "type": "list",
                        "choices": [
                            "av", "web-content", "activex-java-cookie", "dlp", "fortiguard", "all", "filepattern", "pass", "range-block", "antiphish"
                        ],
                        "elements": "str"
                    },
                    "id": {"type": "int"},
                    "referrer-host": {"type": "str"},
                    "status": {"choices": ["disable", "enable"], "type": "str"},
                    "type": {"choices": ["simple", "regex", "wildcard"], "type": "str"},
                    "url": {"type": "str"},
                    "web-proxy-profile": {"type": "str"},
                    "antiphish-action": {"v_range": [["6.4.0", ""]], "choices": ["block", "log"], "type": "str"}
                },
                "elements": "dict"
            },
            "id": {"required": true, "type": "int"},
            "ip-addr-block": {"choices": ["disable", "enable"], "type": "str"},
            "name": {"type": "str"},
            "one-arm-ips-urlfilter": {"choices": ["disable", "enable"], "type": "str"},
            "ip4-mapped-ip6": {"v_range": [["7.2.2", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "webfilter_urlfilter_entries": {
        "stated": true,
        "options": {
            "action": {"choices": ["exempt", "block", "allow", "monitor", "pass"], "type": "str"},
            "dns-address-family": {"choices": ["ipv4", "ipv6", "both"], "type": "str"},
            "exempt": {
                "type": "list",
                "choices": ["av", "web-content", "activex-java-cookie", "dlp", "fortiguard", "all", "filepattern", "pass", "range-block", "antiphish"],
                "elements": "str"
            },
            "id": {"required": true, "type": "int"},
            "referrer-host": {"type": "str"},
            "status": {"choices": ["disable", "enable"], "type": "str"},
            "type": {"choices": ["simple", "regex", "wildcard"], "type": "str"},
            "url": {"type": "str"},
            "web-proxy-profile": {"type": "str"},
            "antiphish-action": {"v_range": [["6.4.0", ""]], "choices": ["block", "log"], "type": "str"}
        }
    },
    "webproxy_forwardserver": {
        "stated": true,
        "options": {
            "addr-type": {"choices": ["fqdn", "ip", "ipv6"], "type": "str"},
            "comment": {"type": "str"},
            "fqdn": {"type": "str"},
            "healthcheck": {"choices": ["disable", "enable"], "type": "str"},
            "ip": {"type": "str"},
            "monitor": {"type": "str"},
            "name": {"required": true, "type": "str"},
            "port": {"type": "int"},
            "server-down-option": {"choices": ["block", "pass"], "type": "str"},
            "password": {"v_range": [["6.4.0", ""]], "no_log": true, "type": "raw"},
            "username": {"v_range": [["6.4.0", ""]], "type": "str"},
            "ipv6": {"v_range": [["7.4.1", ""]], "type": "str"},
            "masquerade": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "webproxy_forwardservergroup": {
        "stated": true,
        "options": {
            "affinity": {"choices": ["disable", "enable"], "type": "str"},
            "group-down-option": {"choices": ["block", "pass"], "type": "str"},
            "ldb-method": {"choices": ["weighted", "least-session", "active-passive"], "type": "str"},
            "name": {"required": true, "type": "str"},
            "server-list": {"type": "list", "options": {"name": {"type": "str"}, "weight": {"type": "int"}}, "elements": "dict"}
        }
    },
    "webproxy_forwardservergroup_serverlist": {"stated": true, "options": {"name": {"required": true, "type": "str"}, "weight": {"type": "int"}}},
    "webproxy_profile": {
        "stated": true,
        "options": {
            "header-client-ip": {"choices": ["pass", "add", "remove"], "type": "str"},
            "header-front-end-https": {"choices": ["pass", "add", "remove"], "type": "str"},
            "header-via-request": {"choices": ["pass", "add", "remove"], "type": "str"},
            "header-via-response": {"choices": ["pass", "add", "remove"], "type": "str"},
            "header-x-authenticated-groups": {"choices": ["pass", "add", "remove"], "type": "str"},
            "header-x-authenticated-user": {"choices": ["pass", "add", "remove"], "type": "str"},
            "header-x-forwarded-for": {"choices": ["pass", "add", "remove"], "type": "str"},
            "headers": {
                "type": "list",
                "options": {
                    "action": {
                        "choices": [
                            "add-to-request", "add-to-response", "remove-from-request", "remove-from-response", "monitor-request", "monitor-response"
                        ],
                        "type": "str"
                    },
                    "content": {"type": "str"},
                    "id": {"type": "int"},
                    "name": {"type": "str"},
                    "add-option": {"v_range": [["6.2.0", ""]], "choices": ["append", "new-on-not-found", "new"], "type": "str"},
                    "base64-encoding": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "dstaddr": {"v_range": [["6.2.0", ""]], "type": "raw"},
                    "dstaddr6": {"v_range": [["6.2.0", ""]], "type": "raw"},
                    "protocol": {"v_range": [["6.2.0", ""]], "type": "list", "choices": ["https", "http"], "elements": "str"}
                },
                "elements": "dict"
            },
            "log-header-change": {"choices": ["disable", "enable"], "type": "str"},
            "name": {"required": true, "type": "str"},
            "strip-encoding": {"choices": ["disable", "enable"], "type": "str"},
            "header-x-forwarded-client-cert": {"v_range": [["7.0.1", ""]], "choices": ["pass", "add", "remove"], "type": "str"}
        }
    },
    "webproxy_profile_headers": {
        "stated": true,
        "options": {
            "action": {
                "choices": ["add-to-request", "add-to-response", "remove-from-request", "remove-from-response", "monitor-request", "monitor-response"],
                "type": "str"
            },
            "content": {"type": "str"},
            "id": {"required": true, "type": "int"},
            "name": {"type": "str"},
            "add-option": {"v_range": [["6.2.0", ""]], "choices": ["append", "new-on-not-found", "new"], "type": "str"},
            "base64-encoding": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "dstaddr": {"v_range": [["6.2.0", ""]], "type": "raw"},
            "dstaddr6": {"v_range": [["6.2.0", ""]], "type": "raw"},
            "protocol": {"v_range": [["6.2.0", ""]], "type": "list", "choices": ["https", "http"], "elements": "str"}
        }
    },
    "webproxy_wisp": {
        "stated": true,
        "options": {
            "comment": {"type": "str"},
            "max-connections": {"type": "int"},
            "name": {"required": true, "type": "str"},
            "outgoing-ip": {"type": "str"},
            "server-ip": {"type": "str"},
            "server-port": {"type": "int"},
            "timeout": {"type": "int"}
        }
    },
    "widsprofile": {
        "stated": true,
        "options": {
            "ap-auto-suppress": {"choices": ["disable", "enable"], "type": "str"},
            "ap-bgscan-disable-day": {
                "v_range": [["6.0.0", "7.2.1"]],
                "type": "list",
                "choices": ["sunday", "monday", "tuesday", "wednesday", "thursday", "friday", "saturday"],
                "elements": "str"
            },
            "ap-bgscan-disable-end": {"v_range": [["6.0.0", "7.2.1"]], "type": "str"},
            "ap-bgscan-disable-start": {"v_range": [["6.0.0", "7.2.1"]], "type": "str"},
            "ap-bgscan-duration": {"type": "int"},
            "ap-bgscan-idle": {"type": "int"},
            "ap-bgscan-intv": {"type": "int"},
            "ap-bgscan-period": {"type": "int"},
            "ap-bgscan-report-intv": {"type": "int"},
            "ap-fgscan-report-intv": {"type": "int"},
            "ap-scan": {"choices": ["disable", "enable"], "type": "str"},
            "ap-scan-passive": {"choices": ["disable", "enable"], "type": "str"},
            "asleap-attack": {"choices": ["disable", "enable"], "type": "str"},
            "assoc-flood-thresh": {"type": "int"},
            "assoc-flood-time": {"type": "int"},
            "assoc-frame-flood": {"choices": ["disable", "enable"], "type": "str"},
            "auth-flood-thresh": {"type": "int"},
            "auth-flood-time": {"type": "int"},
            "auth-frame-flood": {"choices": ["disable", "enable"], "type": "str"},
            "comment": {"type": "str"},
            "deauth-broadcast": {"choices": ["disable", "enable"], "type": "str"},
            "deauth-unknown-src-thresh": {"type": "int"},
            "eapol-fail-flood": {"choices": ["disable", "enable"], "type": "str"},
            "eapol-fail-intv": {"type": "int"},
            "eapol-fail-thresh": {"type": "int"},
            "eapol-logoff-flood": {"choices": ["disable", "enable"], "type": "str"},
            "eapol-logoff-intv": {"type": "int"},
            "eapol-logoff-thresh": {"type": "int"},
            "eapol-pre-fail-flood": {"choices": ["disable", "enable"], "type": "str"},
            "eapol-pre-fail-intv": {"type": "int"},
            "eapol-pre-fail-thresh": {"type": "int"},
            "eapol-pre-succ-flood": {"choices": ["disable", "enable"], "type": "str"},
            "eapol-pre-succ-intv": {"type": "int"},
            "eapol-pre-succ-thresh": {"type": "int"},
            "eapol-start-flood": {"choices": ["disable", "enable"], "type": "str"},
            "eapol-start-intv": {"type": "int"},
            "eapol-start-thresh": {"type": "int"},
            "eapol-succ-flood": {"choices": ["disable", "enable"], "type": "str"},
            "eapol-succ-intv": {"type": "int"},
            "eapol-succ-thresh": {"type": "int"},
            "invalid-mac-oui": {"choices": ["disable", "enable"], "type": "str"},
            "long-duration-attack": {"choices": ["disable", "enable"], "type": "str"},
            "long-duration-thresh": {"type": "int"},
            "name": {"required": true, "type": "str"},
            "null-ssid-probe-resp": {"choices": ["disable", "enable"], "type": "str"},
            "sensor-mode": {"choices": ["disable", "foreign", "both"], "type": "str"},
            "spoofed-deauth": {"choices": ["disable", "enable"], "type": "str"},
            "weak-wep-iv": {"choices": ["disable", "enable"], "type": "str"},
            "wireless-bridge": {"choices": ["disable", "enable"], "type": "str"},
            "ap-bgscan-disable-schedules": {"v_range": [["6.2.1", ""]], "type": "raw"},
            "rogue-scan": {"v_range": [["6.2.0", "6.2.12"]], "choices": ["disable", "enable"], "type": "str"},
            "ap-scan-threshold": {"v_range": [["6.2.3", ""]], "type": "str"},
            "ap-scan-channel-list-2G-5G": {"v_range": [["7.4.1", ""]], "type": "raw"},
            "ap-scan-channel-list-6G": {"v_range": [["7.4.1", ""]], "type": "raw"}
        }
    },
    "wireless_accesscontrollist": {
        "stated": true,
        "options": {
            "comment": {"v_range": [["7.2.1", ""]], "type": "str"},
            "layer3-ipv4-rules": {
                "v_range": [["7.2.1", ""]],
                "type": "list",
                "options": {
                    "action": {"v_range": [["7.2.1", ""]], "choices": ["allow", "deny"], "type": "str"},
                    "comment": {"v_range": [["7.2.1", ""]], "type": "str"},
                    "dstaddr": {"v_range": [["7.2.1", ""]], "type": "str"},
                    "dstport": {"v_range": [["7.2.1", ""]], "type": "int"},
                    "protocol": {"v_range": [["7.2.1", ""]], "type": "int"},
                    "rule-id": {"v_range": [["7.2.1", ""]], "type": "int"},
                    "srcaddr": {"v_range": [["7.2.1", ""]], "type": "str"},
                    "srcport": {"v_range": [["7.2.1", ""]], "type": "int"}
                },
                "elements": "dict"
            },
            "layer3-ipv6-rules": {
                "v_range": [["7.2.1", ""]],
                "type": "list",
                "options": {
                    "action": {"v_range": [["7.2.1", ""]], "choices": ["allow", "deny"], "type": "str"},
                    "comment": {"v_range": [["7.2.1", ""]], "type": "str"},
                    "dstaddr": {"v_range": [["7.2.1", ""]], "type": "str"},
                    "dstport": {"v_range": [["7.2.1", ""]], "type": "int"},
                    "protocol": {"v_range": [["7.2.1", ""]], "type": "int"},
                    "rule-id": {"v_range": [["7.2.1", ""]], "type": "int"},
                    "srcaddr": {"v_range": [["7.2.1", ""]], "type": "str"},
                    "srcport": {"v_range": [["7.2.1", ""]], "type": "int"}
                },
                "elements": "dict"
            },
            "name": {"v_range": [["7.2.1", ""]], "required": true, "type": "str"}
        }
    },
    "wireless_accesscontrollist_layer3ipv4rules": {
        "stated": true,
        "options": {
            "action": {"v_range": [["7.2.1", ""]], "choices": ["allow", "deny"], "type": "str"},
            "comment": {"v_range": [["7.2.1", ""]], "type": "str"},
            "dstaddr": {"v_range": [["7.2.1", ""]], "type": "str"},
            "dstport": {"v_range": [["7.2.1", ""]], "type": "int"},
            "protocol": {"v_range": [["7.2.1", ""]], "type": "int"},
            "rule-id": {"v_range": [["7.2.1", ""]], "type": "int"},
            "srcaddr": {"v_range": [["7.2.1", ""]], "type": "str"},
            "srcport": {"v_range": [["7.2.1", ""]], "type": "int"}
        }
    },
    "wireless_accesscontrollist_layer3ipv6rules": {
        "stated": true,
        "options": {
            "action": {"v_range": [["7.2.1", ""]], "choices": ["allow", "deny"], "type": "str"},
            "comment": {"v_range": [["7.2.1", ""]], "type": "str"},
            "dstaddr": {"v_range": [["7.2.1", ""]], "type": "str"},
            "dstport": {"v_range": [["7.2.1", ""]], "type": "int"},
            "protocol": {"v_range": [["7.2.1", ""]], "type": "int"},
            "rule-id": {"v_range": [["7.2.1", ""]], "type": "int"},
            "srcaddr": {"v_range": [["7.2.1", ""]], "type": "str"},
            "srcport": {"v_range": [["7.2.1", ""]], "type": "int"}
        }
    },
    "wireless_address": {
        "stated": true,
        "options": {
            "id": {"v_range": [["7.0.1", ""]], "required": true, "type": "str"},
            "mac": {"v_range": [["7.0.1", ""]], "type": "str"},
            "policy": {"v_range": [["7.0.1", ""]], "choices": ["deny", "allow"], "type": "str"}
        }
    },
    "wireless_addrgrp": {
        "stated": true,
        "options": {
            "addresses": {"v_range": [["7.0.1", ""]], "type": "raw"},
            "default-policy": {"v_range": [["7.0.1", ""]], "choices": ["deny", "allow"], "type": "str"},
            "id": {"v_range": [["7.0.1", ""]], "required": true, "type": "str"}
        }
    },
    "wireless_ssidpolicy": {
        "stated": true,
        "options": {
            "description": {"v_range": [["7.2.1", ""]], "type": "str"},
            "name": {"v_range": [["7.2.1", ""]], "required": true, "type": "str"},
            "vlan": {"v_range": [["7.2.1", ""]], "type": "str"}
        }
    },
    "wireless_syslogprofile": {
        "stated": true,
        "options": {
            "comment": {"v_range": [["7.2.1", ""]], "type": "str"},
            "log-level": {
                "v_range": [["7.2.1", ""]],
                "choices": ["emergency", "alert", "critical", "error", "warning", "notification", "information", "debugging"],
                "type": "str"
            },
            "name": {"v_range": [["7.2.1", ""]], "required": true, "type": "str"},
            "server-addr-type": {"v_range": [["7.2.1", ""]], "choices": ["fqdn", "ip"], "type": "str"},
            "server-fqdn": {"v_range": [["7.2.1", ""]], "type": "str"},
            "server-ip": {"v_range": [["7.2.1", ""]], "type": "str"},
            "server-port": {"v_range": [["7.2.1", ""]], "type": "int"},
            "server-status": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "wtpprofile": {
        "stated": true,
        "options": {
            "allowaccess": {"type": "list", "choices": ["https", "ssh", "snmp", "http", "telnet"], "elements": "str"},
            "ap-country": {
                "choices": [
                    "AL", "DZ", "AR", "AM", "AU", "AT", "AZ", "BH", "BD", "BY", "BE", "BZ", "BO", "BA", "BR", "BN", "BG", "CA", "CL", "CN", "CO", "CR",
                    "HR", "CY", "CZ", "DK", "DO", "EC", "EG", "SV", "EE", "FI", "FR", "GE", "DE", "GR", "GT", "HN", "HK", "HU", "IS", "IN", "ID", "IR",
                    "IE", "IL", "IT", "JM", "JP", "JO", "KZ", "KE", "KP", "KR", "KW", "LV", "LB", "LI", "LT", "LU", "MO", "MK", "MY", "MT", "MX", "MC",
                    "MA", "NP", "NL", "AN", "NZ", "NO", "OM", "PK", "PA", "PG", "PE", "PH", "PL", "PT", "PR", "QA", "RO", "RU", "SA", "SG", "SK", "SI",
                    "ZA", "ES", "LK", "SE", "CH", "SY", "TW", "TH", "TT", "TN", "TR", "AE", "UA", "GB", "US", "PS", "UY", "UZ", "VE", "VN", "YE", "ZW",
                    "NA", "KH", "TZ", "SD", "AO", "RW", "MZ", "RS", "ME", "BB", "GD", "GL", "GU", "PY", "HT", "AW", "MM", "ZB", "CF", "BS", "VC", "MV",
                    "SN", "CI", "GH", "MW", "UG", "BF", "KY", "TC", "TM", "VU", "FM", "GY", "KN", "LC", "CX", "AF", "CM", "ML", "BJ", "MG", "TD", "BW",
                    "LY", "LS", "MU", "SL", "NE", "TG", "RE", "MD", "BM", "VI", "PM", "MF", "IM", "FO", "GI", "LA", "WF", "MH", "BT", "PF", "NI", "GF",
                    "AS", "MP", "PW", "GP", "ET", "SR", "DM", "MQ", "YT", "BL", "ZM", "CG", "CD", "MR", "IQ", "FJ", "--", "MN", "NG", "GA", "GM", "SO",
                    "SZ", "LR", "DJ"
                ],
                "type": "str"
            },
            "ble-profile": {"type": "str"},
            "comment": {"type": "str"},
            "control-message-offload": {
                "type": "list",
                "choices": [
                    "ebp-frame", "aeroscout-tag", "ap-list", "sta-list", "sta-cap-list", "stats", "aeroscout-mu", "sta-health", "spectral-analysis"
                ],
                "elements": "str"
            },
            "deny-mac-list": {"type": "list", "options": {"id": {"type": "int"}, "mac": {"type": "str"}}, "elements": "dict"},
            "dtls-in-kernel": {"choices": ["disable", "enable"], "type": "str"},
            "dtls-policy": {"type": "list", "choices": ["clear-text", "dtls-enabled", "ipsec-vpn", "ipsec-sn-vpn"], "elements": "str"},
            "energy-efficient-ethernet": {"choices": ["disable", "enable"], "type": "str"},
            "ext-info-enable": {"choices": ["disable", "enable"], "type": "str"},
            "handoff-roaming": {"choices": ["disable", "enable"], "type": "str"},
            "handoff-rssi": {"type": "int"},
            "handoff-sta-thresh": {"type": "int"},
            "ip-fragment-preventing": {"type": "list", "choices": ["tcp-mss-adjust", "icmp-unreachable"], "elements": "str"},
            "led-schedules": {"type": "raw"},
            "led-state": {"choices": ["disable", "enable"], "type": "str"},
            "lldp": {"choices": ["disable", "enable"], "type": "str"},
            "login-passwd": {"no_log": true, "type": "raw"},
            "login-passwd-change": {"choices": ["no", "yes", "default"], "type": "str"},
            "max-clients": {"type": "int"},
            "name": {"required": true, "type": "str"},
            "poe-mode": {"choices": ["auto", "8023af", "8023at", "power-adapter", "full", "high", "low"], "type": "str"},
            "split-tunneling-acl": {"type": "list", "options": {"dest-ip": {"type": "str"}, "id": {"type": "int"}}, "elements": "dict"},
            "split-tunneling-acl-local-ap-subnet": {"choices": ["disable", "enable"], "type": "str"},
            "split-tunneling-acl-path": {"choices": ["tunnel", "local"], "type": "str"},
            "tun-mtu-downlink": {"type": "int"},
            "tun-mtu-uplink": {"type": "int"},
            "wan-port-mode": {"choices": ["wan-lan", "wan-only"], "type": "str"},
            "snmp": {"v_range": [["6.2.0", "7.2.0"]], "choices": ["disable", "enable"], "type": "str"},
            "ap-handoff": {"v_range": [["6.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "apcfg-profile": {"v_range": [["6.4.0", ""]], "type": "str"},
            "frequency-handoff": {"v_range": [["6.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "lan": {
                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                "type": "dict",
                "options": {
                    "port-esl-mode": {"v_range": [["6.4.5", ""]], "choices": ["offline", "bridge-to-wan", "bridge-to-ssid", "nat-to-wan"], "type": "str"},
                    "port-esl-ssid": {"v_range": [["6.4.5", ""]], "type": "str"},
                    "port-mode": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "choices": ["offline", "bridge-to-wan", "bridge-to-ssid", "nat-to-wan"],
                        "type": "str"
                    },
                    "port-ssid": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "port1-mode": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "choices": ["offline", "bridge-to-wan", "bridge-to-ssid", "nat-to-wan"],
                        "type": "str"
                    },
                    "port1-ssid": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "port2-mode": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "choices": ["offline", "bridge-to-wan", "bridge-to-ssid", "nat-to-wan"],
                        "type": "str"
                    },
                    "port2-ssid": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "port3-mode": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "choices": ["offline", "bridge-to-wan", "bridge-to-ssid", "nat-to-wan"],
                        "type": "str"
                    },
                    "port3-ssid": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "port4-mode": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "choices": ["offline", "bridge-to-wan", "bridge-to-ssid", "nat-to-wan"],
                        "type": "str"
                    },
                    "port4-ssid": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "port5-mode": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "choices": ["offline", "bridge-to-wan", "bridge-to-ssid", "nat-to-wan"],
                        "type": "str"
                    },
                    "port5-ssid": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "port6-mode": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "choices": ["offline", "bridge-to-wan", "bridge-to-ssid", "nat-to-wan"],
                        "type": "str"
                    },
                    "port6-ssid": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "port7-mode": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "choices": ["offline", "bridge-to-wan", "bridge-to-ssid", "nat-to-wan"],
                        "type": "str"
                    },
                    "port7-ssid": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "port8-mode": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "choices": ["offline", "bridge-to-wan", "bridge-to-ssid", "nat-to-wan"],
                        "type": "str"
                    },
                    "port8-ssid": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"}
                }
            },
            "lbs": {
                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                "type": "dict",
                "options": {
                    "aeroscout": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "aeroscout-ap-mac": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["bssid", "board-mac"], "type": "str"},
                    "aeroscout-mmu-report": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "aeroscout-mu": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "aeroscout-mu-factor": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "aeroscout-mu-timeout": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "aeroscout-server-ip": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "aeroscout-server-port": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "ekahau-blink-mode": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "ekahau-tag": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "erc-server-ip": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "erc-server-port": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "fortipresence": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "choices": ["disable", "enable", "enable2", "foreign", "both"],
                        "type": "str"
                    },
                    "fortipresence-ble": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "fortipresence-frequency": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "fortipresence-port": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "fortipresence-project": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "fortipresence-rogue": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "fortipresence-secret": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "no_log": true, "type": "raw"},
                    "fortipresence-server": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "fortipresence-unassoc": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "station-locate": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "fortipresence-server-addr-type": {"v_range": [["7.0.2", ""]], "choices": ["fqdn", "ipv4"], "type": "str"},
                    "fortipresence-server-fqdn": {"v_range": [["7.0.2", ""]], "type": "str"},
                    "polestar": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "polestar-accumulation-interval": {"v_range": [["7.4.1", ""]], "type": "int"},
                    "polestar-asset-addrgrp-list": {"v_range": [["7.4.1", ""]], "type": "str"},
                    "polestar-asset-uuid-list1": {"v_range": [["7.4.1", ""]], "type": "str"},
                    "polestar-asset-uuid-list2": {"v_range": [["7.4.1", ""]], "type": "str"},
                    "polestar-asset-uuid-list3": {"v_range": [["7.4.1", ""]], "type": "str"},
                    "polestar-asset-uuid-list4": {"v_range": [["7.4.1", ""]], "type": "str"},
                    "polestar-protocol": {"v_range": [["7.4.1", ""]], "choices": ["WSS"], "type": "str"},
                    "polestar-reporting-interval": {"v_range": [["7.4.1", ""]], "type": "int"},
                    "polestar-server-fqdn": {"v_range": [["7.4.1", ""]], "type": "str"},
                    "polestar-server-path": {"v_range": [["7.4.1", ""]], "type": "str"},
                    "polestar-server-port": {"v_range": [["7.4.1", ""]], "type": "int"},
                    "polestar-server-token": {"v_range": [["7.4.1", ""]], "no_log": true, "type": "str"}
                }
            },
            "platform": {
                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                "type": "dict",
                "options": {
                    "ddscan": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "mode": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["dual-5G", "single-5G"], "type": "str"},
                    "type": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "choices": [
                            "30B-50B", "60B", "80CM-81CM", "220A", "220B", "210B", "60C", "222B", "112B", "320B", "11C", "14C", "223B", "28C", "320C",
                            "221C", "25D", "222C", "224D", "214B", "21D", "24D", "112D", "223C", "321C", "C220C", "C225C", "S321C", "S323C", "FWF",
                            "S311C", "S313C", "AP-11N", "S322C", "S321CR", "S322CR", "S323CR", "S421E", "S422E", "S423E", "421E", "423E", "C221E",
                            "C226E", "C23JD", "C24JE", "C21D", "U421E", "U423E", "221E", "222E", "223E", "S221E", "S223E", "U221EV", "U223EV", "U321EV",
                            "U323EV", "224E", "U422EV", "U24JEV", "321E", "U431F", "U433F", "231E", "431F", "433F", "231F", "432F", "234F", "23JF",
                            "U231F", "831F", "U234F", "U432F", "431FL", "432FR", "433FL", "231FL", "231G", "233G", "431G", "433G", "U231G", "U441G",
                            "234G", "432G", "441K", "443K", "241K", "243K"
                        ],
                        "type": "str"
                    },
                    "_local_platform_str": {"v_range": [["6.2.8", "6.2.12"], ["6.4.6", ""]], "type": "str"}
                }
            },
            "radio-1": {
                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                "type": "dict",
                "options": {
                    "airtime-fairness": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "amsdu": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "ap-sniffer-addr": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "ap-sniffer-bufsize": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "ap-sniffer-chan": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "ap-sniffer-ctl": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "ap-sniffer-data": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "ap-sniffer-mgmt-beacon": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "ap-sniffer-mgmt-other": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "ap-sniffer-mgmt-probe": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "auto-power-high": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "auto-power-level": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "auto-power-low": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "auto-power-target": {"v_range": [["6.4.5", ""]], "type": "str"},
                    "band": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "choices": [
                            "802.11b", "802.11a", "802.11g", "802.11n", "802.11ac", "802.11n-5G", "802.11ax-5G", "802.11ax", "802.11ac-2G",
                            "802.11g-only", "802.11n-only", "802.11n,g-only", "802.11ac-only", "802.11ac,n-only", "802.11n-5G-only", "802.11ax-5G-only",
                            "802.11ax,ac-only", "802.11ax,ac,n-only", "802.11ax-only", "802.11ax,n-only", "802.11ax,n,g-only", "802.11ax-6G",
                            "802.11n-2G", "802.11ac-5G", "802.11ax-2G", "802.11be-2G", "802.11be-5G", "802.11be-6G"
                        ],
                        "type": "str"
                    },
                    "band-5g-type": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["5g-full", "5g-high", "5g-low"], "type": "str"},
                    "bandwidth-admission-control": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "bandwidth-capacity": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "beacon-interval": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "bss-color": {"v_range": [["6.4.5", ""]], "type": "int"},
                    "call-admission-control": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "call-capacity": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "channel": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "raw"},
                    "channel-bonding": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "choices": ["disable", "enable", "80MHz", "40MHz", "20MHz", "160MHz", "320MHz", "240MHz"],
                        "type": "str"
                    },
                    "channel-utilization": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "coexistence": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "darrp": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "drma": {"v_range": [["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "drma-sensitivity": {"v_range": [["6.4.5", ""]], "choices": ["low", "medium", "high"], "type": "str"},
                    "dtim": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "frag-threshold": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "max-clients": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "max-distance": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "mode": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disabled", "ap", "monitor", "sniffer", "sam"], "type": "str"},
                    "power-level": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "powersave-optimize": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "type": "list",
                        "choices": ["tim", "ac-vo", "no-obss-scan", "no-11b-rate", "client-rate-follow"],
                        "elements": "str"
                    },
                    "protection-mode": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["rtscts", "ctsonly", "disable"], "type": "str"},
                    "radio-id": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "rts-threshold": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "short-guard-interval": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "spectrum-analysis": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable", "scan-only"], "type": "str"},
                    "transmit-optimize": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "type": "list",
                        "choices": ["disable", "power-save", "aggr-limit", "retry-limit", "send-bar"],
                        "elements": "str"
                    },
                    "vap-all": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "choices": ["disable", "enable", "tunnel", "bridge", "manual"],
                        "type": "str"
                    },
                    "vap1": {"v_range": [["6.4.5", ""]], "type": "str"},
                    "vap2": {"v_range": [["6.4.5", ""]], "type": "str"},
                    "vap3": {"v_range": [["6.4.5", ""]], "type": "str"},
                    "vap4": {"v_range": [["6.4.5", ""]], "type": "str"},
                    "vap5": {"v_range": [["6.4.5", ""]], "type": "str"},
                    "vap6": {"v_range": [["6.4.5", ""]], "type": "str"},
                    "vap7": {"v_range": [["6.4.5", ""]], "type": "str"},
                    "vap8": {"v_range": [["6.4.5", ""]], "type": "str"},
                    "vaps": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "raw"},
                    "wids-profile": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "zero-wait-dfs": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "frequency-handoff": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "ap-handoff": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "iperf-protocol": {"v_range": [["7.0.0", ""]], "choices": ["udp", "tcp"], "type": "str"},
                    "iperf-server-port": {"v_range": [["7.0.0", ""]], "type": "int"},
                    "power-mode": {"v_range": [["7.0.0", ""]], "choices": ["dBm", "percentage"], "type": "str"},
                    "power-value": {"v_range": [["7.0.0", ""]], "type": "int"},
                    "sam-bssid": {"v_range": [["7.0.0", ""]], "type": "str"},
                    "sam-captive-portal": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "sam-password": {"v_range": [["7.0.0", ""]], "no_log": true, "type": "raw"},
                    "sam-report-intv": {"v_range": [["7.0.0", ""]], "type": "int"},
                    "sam-security-type": {
                        "v_range": [["7.0.0", ""]],
                        "choices": ["open", "wpa-personal", "wpa-enterprise", "owe", "wpa3-sae"],
                        "type": "str"
                    },
                    "sam-server": {"v_range": [["7.0.0", ""]], "type": "str"},
                    "sam-ssid": {"v_range": [["7.0.0", ""]], "type": "str"},
                    "sam-test": {"v_range": [["7.0.0", ""]], "choices": ["ping", "iperf"], "type": "str"},
                    "sam-username": {"v_range": [["7.0.0", ""]], "type": "str"},
                    "arrp-profile": {"v_range": [["7.0.3", ""]], "type": "str"},
                    "bss-color-mode": {"v_range": [["7.0.2", ""]], "choices": ["auto", "static"], "type": "str"},
                    "sam-cwp-failure-string": {"v_range": [["7.0.1", ""]], "type": "str"},
                    "sam-cwp-match-string": {"v_range": [["7.0.1", ""]], "type": "str"},
                    "sam-cwp-password": {"v_range": [["7.0.1", ""]], "no_log": true, "type": "raw"},
                    "sam-cwp-success-string": {"v_range": [["7.0.1", ""]], "type": "str"},
                    "sam-cwp-test-url": {"v_range": [["7.0.1", ""]], "type": "str"},
                    "sam-cwp-username": {"v_range": [["7.0.1", ""]], "type": "str"},
                    "sam-server-fqdn": {"v_range": [["7.0.1", ""]], "type": "str"},
                    "sam-server-ip": {"v_range": [["7.0.1", ""]], "type": "str"},
                    "sam-server-type": {"v_range": [["7.0.1", ""]], "choices": ["ip", "fqdn"], "type": "str"},
                    "80211d": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "optional-antenna": {
                        "v_range": [["7.2.3", ""]],
                        "choices": [
                            "none", "FANT-04ABGN-0606-O-N", "FANT-04ABGN-1414-P-N", "FANT-04ABGN-8065-P-N", "FANT-04ABGN-0606-O-R",
                            "FANT-04ABGN-0606-P-R", "FANT-10ACAX-1213-D-N", "FANT-08ABGN-1213-D-R", "custom"
                        ],
                        "type": "str"
                    },
                    "mimo-mode": {"v_range": [["7.4.1", ""]], "choices": ["default", "1x1", "2x2", "3x3", "4x4", "8x8"], "type": "str"},
                    "optional-antenna-gain": {"v_range": [["7.4.2", ""]], "type": "str"},
                    "sam-ca-certificate": {"v_range": [["7.4.2", ""]], "type": "str"},
                    "sam-client-certificate": {"v_range": [["7.4.2", ""]], "type": "str"},
                    "sam-eap-method": {"v_range": [["7.4.2", ""]], "choices": ["tls", "peap", "both"], "type": "str"},
                    "sam-private-key": {"v_range": [["7.4.2", ""]], "no_log": true, "type": "str"},
                    "sam-private-key-password": {"v_range": [["7.4.2", ""]], "no_log": true, "type": "raw"},
                    "channel-bonding-ext": {"v_range": [["7.4.3", ""]], "choices": ["320MHz-1", "320MHz-2"], "type": "str"},
                    "80211mc": {"v_range": [["7.6.0", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "ap-sniffer-chan-width": {
                        "v_range": [["7.6.0", ""]],
                        "choices": ["320MHz", "240MHz", "160MHz", "80MHz", "40MHz", "20MHz"],
                        "type": "str"
                    }
                }
            },
            "radio-2": {
                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                "type": "dict",
                "options": {
                    "airtime-fairness": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "amsdu": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "ap-sniffer-addr": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "ap-sniffer-bufsize": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "ap-sniffer-chan": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "ap-sniffer-ctl": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "ap-sniffer-data": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "ap-sniffer-mgmt-beacon": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "ap-sniffer-mgmt-other": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "ap-sniffer-mgmt-probe": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "auto-power-high": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "auto-power-level": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "auto-power-low": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "auto-power-target": {"v_range": [["6.4.5", ""]], "type": "str"},
                    "band": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "choices": [
                            "802.11b", "802.11a", "802.11g", "802.11n", "802.11ac", "802.11n-5G", "802.11ax-5G", "802.11ax", "802.11ac-2G",
                            "802.11g-only", "802.11n-only", "802.11n,g-only", "802.11ac-only", "802.11ac,n-only", "802.11n-5G-only", "802.11ax-5G-only",
                            "802.11ax,ac-only", "802.11ax,ac,n-only", "802.11ax-only", "802.11ax,n-only", "802.11ax,n,g-only", "802.11ax-6G",
                            "802.11n-2G", "802.11ac-5G", "802.11ax-2G", "802.11be-2G", "802.11be-5G", "802.11be-6G"
                        ],
                        "type": "str"
                    },
                    "band-5g-type": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["5g-full", "5g-high", "5g-low"], "type": "str"},
                    "bandwidth-admission-control": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "bandwidth-capacity": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "beacon-interval": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "bss-color": {"v_range": [["6.4.5", ""]], "type": "int"},
                    "call-admission-control": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "call-capacity": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "channel": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "raw"},
                    "channel-bonding": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "choices": ["disable", "enable", "80MHz", "40MHz", "20MHz", "160MHz", "320MHz", "240MHz"],
                        "type": "str"
                    },
                    "channel-utilization": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "coexistence": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "darrp": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "drma": {"v_range": [["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "drma-sensitivity": {"v_range": [["6.4.5", ""]], "choices": ["low", "medium", "high"], "type": "str"},
                    "dtim": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "frag-threshold": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "max-clients": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "max-distance": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "mode": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disabled", "ap", "monitor", "sniffer", "sam"], "type": "str"},
                    "power-level": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "powersave-optimize": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "type": "list",
                        "choices": ["tim", "ac-vo", "no-obss-scan", "no-11b-rate", "client-rate-follow"],
                        "elements": "str"
                    },
                    "protection-mode": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["rtscts", "ctsonly", "disable"], "type": "str"},
                    "radio-id": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "rts-threshold": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "short-guard-interval": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "spectrum-analysis": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable", "scan-only"], "type": "str"},
                    "transmit-optimize": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "type": "list",
                        "choices": ["disable", "power-save", "aggr-limit", "retry-limit", "send-bar"],
                        "elements": "str"
                    },
                    "vap-all": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "choices": ["disable", "enable", "tunnel", "bridge", "manual"],
                        "type": "str"
                    },
                    "vap1": {"v_range": [["6.4.5", ""]], "type": "str"},
                    "vap2": {"v_range": [["6.4.5", ""]], "type": "str"},
                    "vap3": {"v_range": [["6.4.5", ""]], "type": "str"},
                    "vap4": {"v_range": [["6.4.5", ""]], "type": "str"},
                    "vap5": {"v_range": [["6.4.5", ""]], "type": "str"},
                    "vap6": {"v_range": [["6.4.5", ""]], "type": "str"},
                    "vap7": {"v_range": [["6.4.5", ""]], "type": "str"},
                    "vap8": {"v_range": [["6.4.5", ""]], "type": "str"},
                    "vaps": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "raw"},
                    "wids-profile": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "zero-wait-dfs": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "frequency-handoff": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "ap-handoff": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "iperf-protocol": {"v_range": [["7.0.0", ""]], "choices": ["udp", "tcp"], "type": "str"},
                    "iperf-server-port": {"v_range": [["7.0.0", ""]], "type": "int"},
                    "power-mode": {"v_range": [["7.0.0", ""]], "choices": ["dBm", "percentage"], "type": "str"},
                    "power-value": {"v_range": [["7.0.0", ""]], "type": "int"},
                    "sam-bssid": {"v_range": [["7.0.0", ""]], "type": "str"},
                    "sam-captive-portal": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "sam-password": {"v_range": [["7.0.0", ""]], "no_log": true, "type": "raw"},
                    "sam-report-intv": {"v_range": [["7.0.0", ""]], "type": "int"},
                    "sam-security-type": {
                        "v_range": [["7.0.0", ""]],
                        "choices": ["open", "wpa-personal", "wpa-enterprise", "owe", "wpa3-sae"],
                        "type": "str"
                    },
                    "sam-server": {"v_range": [["7.0.0", ""]], "type": "str"},
                    "sam-ssid": {"v_range": [["7.0.0", ""]], "type": "str"},
                    "sam-test": {"v_range": [["7.0.0", ""]], "choices": ["ping", "iperf"], "type": "str"},
                    "sam-username": {"v_range": [["7.0.0", ""]], "type": "str"},
                    "arrp-profile": {"v_range": [["7.0.3", ""]], "type": "str"},
                    "bss-color-mode": {"v_range": [["7.0.2", ""]], "choices": ["auto", "static"], "type": "str"},
                    "sam-cwp-failure-string": {"v_range": [["7.0.1", ""]], "type": "str"},
                    "sam-cwp-match-string": {"v_range": [["7.0.1", ""]], "type": "str"},
                    "sam-cwp-password": {"v_range": [["7.0.1", ""]], "no_log": true, "type": "raw"},
                    "sam-cwp-success-string": {"v_range": [["7.0.1", ""]], "type": "str"},
                    "sam-cwp-test-url": {"v_range": [["7.0.1", ""]], "type": "str"},
                    "sam-cwp-username": {"v_range": [["7.0.1", ""]], "type": "str"},
                    "sam-server-fqdn": {"v_range": [["7.0.1", ""]], "type": "str"},
                    "sam-server-ip": {"v_range": [["7.0.1", ""]], "type": "str"},
                    "sam-server-type": {"v_range": [["7.0.1", ""]], "choices": ["ip", "fqdn"], "type": "str"},
                    "80211d": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "optional-antenna": {
                        "v_range": [["7.2.3", ""]],
                        "choices": [
                            "none", "FANT-04ABGN-0606-O-N", "FANT-04ABGN-1414-P-N", "FANT-04ABGN-8065-P-N", "FANT-04ABGN-0606-O-R",
                            "FANT-04ABGN-0606-P-R", "FANT-10ACAX-1213-D-N", "FANT-08ABGN-1213-D-R", "custom"
                        ],
                        "type": "str"
                    },
                    "mimo-mode": {"v_range": [["7.4.1", ""]], "choices": ["default", "1x1", "2x2", "3x3", "4x4", "8x8"], "type": "str"},
                    "optional-antenna-gain": {"v_range": [["7.4.2", ""]], "type": "str"},
                    "sam-ca-certificate": {"v_range": [["7.4.2", ""]], "type": "str"},
                    "sam-client-certificate": {"v_range": [["7.4.2", ""]], "type": "str"},
                    "sam-eap-method": {"v_range": [["7.4.2", ""]], "choices": ["tls", "peap", "both"], "type": "str"},
                    "sam-private-key": {"v_range": [["7.4.2", ""]], "no_log": true, "type": "str"},
                    "sam-private-key-password": {"v_range": [["7.4.2", ""]], "no_log": true, "type": "raw"},
                    "channel-bonding-ext": {"v_range": [["7.4.3", ""]], "choices": ["320MHz-1", "320MHz-2"], "type": "str"},
                    "80211mc": {"v_range": [["7.6.0", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "ap-sniffer-chan-width": {
                        "v_range": [["7.6.0", ""]],
                        "choices": ["320MHz", "240MHz", "160MHz", "80MHz", "40MHz", "20MHz"],
                        "type": "str"
                    }
                }
            },
            "radio-3": {
                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                "type": "dict",
                "options": {
                    "airtime-fairness": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "amsdu": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "ap-sniffer-addr": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "ap-sniffer-bufsize": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "ap-sniffer-chan": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "ap-sniffer-ctl": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "ap-sniffer-data": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "ap-sniffer-mgmt-beacon": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "ap-sniffer-mgmt-other": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "ap-sniffer-mgmt-probe": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "auto-power-high": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "auto-power-level": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "auto-power-low": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "auto-power-target": {"v_range": [["6.4.5", ""]], "type": "str"},
                    "band": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "choices": [
                            "802.11b", "802.11a", "802.11g", "802.11n", "802.11ac", "802.11n-5G", "802.11ax-5G", "802.11ax", "802.11ac-2G",
                            "802.11g-only", "802.11n-only", "802.11n,g-only", "802.11ac-only", "802.11ac,n-only", "802.11n-5G-only", "802.11ax-5G-only",
                            "802.11ax,ac-only", "802.11ax,ac,n-only", "802.11ax-only", "802.11ax,n-only", "802.11ax,n,g-only", "802.11ax-6G",
                            "802.11n-2G", "802.11ac-5G", "802.11ax-2G", "802.11be-2G", "802.11be-5G", "802.11be-6G"
                        ],
                        "type": "str"
                    },
                    "band-5g-type": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["5g-full", "5g-high", "5g-low"], "type": "str"},
                    "bandwidth-admission-control": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "bandwidth-capacity": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "beacon-interval": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "bss-color": {"v_range": [["6.4.5", ""]], "type": "int"},
                    "call-admission-control": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "call-capacity": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "channel": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "raw"},
                    "channel-bonding": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "choices": ["80MHz", "40MHz", "20MHz", "160MHz", "320MHz", "240MHz"],
                        "type": "str"
                    },
                    "channel-utilization": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "coexistence": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "darrp": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "drma": {"v_range": [["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "drma-sensitivity": {"v_range": [["6.4.5", ""]], "choices": ["low", "medium", "high"], "type": "str"},
                    "dtim": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "frag-threshold": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "max-clients": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "max-distance": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "mode": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disabled", "ap", "monitor", "sniffer", "sam"], "type": "str"},
                    "power-level": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "powersave-optimize": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "type": "list",
                        "choices": ["tim", "ac-vo", "no-obss-scan", "no-11b-rate", "client-rate-follow"],
                        "elements": "str"
                    },
                    "protection-mode": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["rtscts", "ctsonly", "disable"], "type": "str"},
                    "radio-id": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "rts-threshold": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "short-guard-interval": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "spectrum-analysis": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable", "scan-only"], "type": "str"},
                    "transmit-optimize": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "type": "list",
                        "choices": ["disable", "power-save", "aggr-limit", "retry-limit", "send-bar"],
                        "elements": "str"
                    },
                    "vap-all": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "choices": ["disable", "enable", "tunnel", "bridge", "manual"],
                        "type": "str"
                    },
                    "vap1": {"v_range": [["6.4.5", ""]], "type": "str"},
                    "vap2": {"v_range": [["6.4.5", ""]], "type": "str"},
                    "vap3": {"v_range": [["6.4.5", ""]], "type": "str"},
                    "vap4": {"v_range": [["6.4.5", ""]], "type": "str"},
                    "vap5": {"v_range": [["6.4.5", ""]], "type": "str"},
                    "vap6": {"v_range": [["6.4.5", ""]], "type": "str"},
                    "vap7": {"v_range": [["6.4.5", ""]], "type": "str"},
                    "vap8": {"v_range": [["6.4.5", ""]], "type": "str"},
                    "vaps": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "raw"},
                    "wids-profile": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "zero-wait-dfs": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "frequency-handoff": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "ap-handoff": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "iperf-protocol": {"v_range": [["7.0.0", ""]], "choices": ["udp", "tcp"], "type": "str"},
                    "iperf-server-port": {"v_range": [["7.0.0", ""]], "type": "int"},
                    "power-mode": {"v_range": [["7.0.0", ""]], "choices": ["dBm", "percentage"], "type": "str"},
                    "power-value": {"v_range": [["7.0.0", ""]], "type": "int"},
                    "sam-bssid": {"v_range": [["7.0.0", ""]], "type": "str"},
                    "sam-captive-portal": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "sam-password": {"v_range": [["7.0.0", ""]], "no_log": true, "type": "raw"},
                    "sam-report-intv": {"v_range": [["7.0.0", ""]], "type": "int"},
                    "sam-security-type": {
                        "v_range": [["7.0.0", ""]],
                        "choices": ["open", "wpa-personal", "wpa-enterprise", "owe", "wpa3-sae"],
                        "type": "str"
                    },
                    "sam-server": {"v_range": [["7.0.0", ""]], "type": "str"},
                    "sam-ssid": {"v_range": [["7.0.0", ""]], "type": "str"},
                    "sam-test": {"v_range": [["7.0.0", ""]], "choices": ["ping", "iperf"], "type": "str"},
                    "sam-username": {"v_range": [["7.0.0", ""]], "type": "str"},
                    "arrp-profile": {"v_range": [["7.0.3", ""]], "type": "str"},
                    "bss-color-mode": {"v_range": [["7.0.2", ""]], "choices": ["auto", "static"], "type": "str"},
                    "sam-cwp-failure-string": {"v_range": [["7.0.1", ""]], "type": "str"},
                    "sam-cwp-match-string": {"v_range": [["7.0.1", ""]], "type": "str"},
                    "sam-cwp-password": {"v_range": [["7.0.1", ""]], "no_log": true, "type": "raw"},
                    "sam-cwp-success-string": {"v_range": [["7.0.1", ""]], "type": "str"},
                    "sam-cwp-test-url": {"v_range": [["7.0.1", ""]], "type": "str"},
                    "sam-cwp-username": {"v_range": [["7.0.1", ""]], "type": "str"},
                    "sam-server-fqdn": {"v_range": [["7.0.1", ""]], "type": "str"},
                    "sam-server-ip": {"v_range": [["7.0.1", ""]], "type": "str"},
                    "sam-server-type": {"v_range": [["7.0.1", ""]], "choices": ["ip", "fqdn"], "type": "str"},
                    "80211d": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "optional-antenna": {
                        "v_range": [["7.2.3", ""]],
                        "choices": [
                            "none", "FANT-04ABGN-0606-O-N", "FANT-04ABGN-1414-P-N", "FANT-04ABGN-8065-P-N", "FANT-04ABGN-0606-O-R",
                            "FANT-04ABGN-0606-P-R", "FANT-10ACAX-1213-D-N", "FANT-08ABGN-1213-D-R", "custom"
                        ],
                        "type": "str"
                    },
                    "mimo-mode": {"v_range": [["7.4.1", ""]], "choices": ["default", "1x1", "2x2", "3x3", "4x4", "8x8"], "type": "str"},
                    "optional-antenna-gain": {"v_range": [["7.4.2", ""]], "type": "str"},
                    "sam-ca-certificate": {"v_range": [["7.4.2", ""]], "type": "str"},
                    "sam-client-certificate": {"v_range": [["7.4.2", ""]], "type": "str"},
                    "sam-eap-method": {"v_range": [["7.4.2", ""]], "choices": ["tls", "peap", "both"], "type": "str"},
                    "sam-private-key": {"v_range": [["7.4.2", ""]], "no_log": true, "type": "str"},
                    "sam-private-key-password": {"v_range": [["7.4.2", ""]], "no_log": true, "type": "raw"},
                    "channel-bonding-ext": {"v_range": [["7.4.3", ""]], "choices": ["320MHz-1", "320MHz-2"], "type": "str"},
                    "80211mc": {"v_range": [["7.6.0", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "ap-sniffer-chan-width": {
                        "v_range": [["7.6.0", ""]],
                        "choices": ["320MHz", "240MHz", "160MHz", "80MHz", "40MHz", "20MHz"],
                        "type": "str"
                    }
                }
            },
            "radio-4": {
                "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                "type": "dict",
                "options": {
                    "airtime-fairness": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "amsdu": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "ap-sniffer-addr": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "ap-sniffer-bufsize": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "ap-sniffer-chan": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "ap-sniffer-ctl": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "ap-sniffer-data": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "ap-sniffer-mgmt-beacon": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "ap-sniffer-mgmt-other": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "ap-sniffer-mgmt-probe": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "auto-power-high": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "auto-power-level": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "auto-power-low": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "auto-power-target": {"v_range": [["6.4.5", ""]], "type": "str"},
                    "band": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "choices": [
                            "802.11b", "802.11a", "802.11g", "802.11n", "802.11ac", "802.11n-5G", "802.11ax-5G", "802.11ax", "802.11ac-2G",
                            "802.11g-only", "802.11n-only", "802.11n,g-only", "802.11ac-only", "802.11ac,n-only", "802.11n-5G-only", "802.11ax-5G-only",
                            "802.11ax,ac-only", "802.11ax,ac,n-only", "802.11ax-only", "802.11ax,n-only", "802.11ax,n,g-only", "802.11ax-6G",
                            "802.11n-2G", "802.11ac-5G", "802.11ax-2G", "802.11be-2G", "802.11be-5G", "802.11be-6G"
                        ],
                        "type": "str"
                    },
                    "band-5g-type": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["5g-full", "5g-high", "5g-low"], "type": "str"},
                    "bandwidth-admission-control": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "bandwidth-capacity": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "beacon-interval": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "bss-color": {"v_range": [["6.4.5", ""]], "type": "int"},
                    "call-admission-control": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "call-capacity": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "channel": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "raw"},
                    "channel-bonding": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "choices": ["80MHz", "40MHz", "20MHz", "160MHz", "320MHz", "240MHz"],
                        "type": "str"
                    },
                    "channel-utilization": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "coexistence": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "darrp": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "drma": {"v_range": [["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "drma-sensitivity": {"v_range": [["6.4.5", ""]], "choices": ["low", "medium", "high"], "type": "str"},
                    "dtim": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "frag-threshold": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "max-clients": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "max-distance": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "mode": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["ap", "monitor", "sniffer", "disabled", "sam"], "type": "str"},
                    "power-level": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "powersave-optimize": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "type": "list",
                        "choices": ["tim", "ac-vo", "no-obss-scan", "no-11b-rate", "client-rate-follow"],
                        "elements": "str"
                    },
                    "protection-mode": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["rtscts", "ctsonly", "disable"], "type": "str"},
                    "radio-id": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "rts-threshold": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "int"},
                    "short-guard-interval": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "spectrum-analysis": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable", "scan-only"], "type": "str"},
                    "transmit-optimize": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "type": "list",
                        "choices": ["disable", "power-save", "aggr-limit", "retry-limit", "send-bar"],
                        "elements": "str"
                    },
                    "vap-all": {
                        "v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]],
                        "choices": ["disable", "enable", "tunnel", "bridge", "manual"],
                        "type": "str"
                    },
                    "vap1": {"v_range": [["6.4.5", ""]], "type": "str"},
                    "vap2": {"v_range": [["6.4.5", ""]], "type": "str"},
                    "vap3": {"v_range": [["6.4.5", ""]], "type": "str"},
                    "vap4": {"v_range": [["6.4.5", ""]], "type": "str"},
                    "vap5": {"v_range": [["6.4.5", ""]], "type": "str"},
                    "vap6": {"v_range": [["6.4.5", ""]], "type": "str"},
                    "vap7": {"v_range": [["6.4.5", ""]], "type": "str"},
                    "vap8": {"v_range": [["6.4.5", ""]], "type": "str"},
                    "vaps": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "raw"},
                    "wids-profile": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "type": "str"},
                    "zero-wait-dfs": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "frequency-handoff": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "ap-handoff": {"v_range": [["6.2.8", "6.2.12"], ["6.4.5", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "iperf-protocol": {"v_range": [["7.0.0", ""]], "choices": ["udp", "tcp"], "type": "str"},
                    "iperf-server-port": {"v_range": [["7.0.0", ""]], "type": "int"},
                    "power-mode": {"v_range": [["7.0.0", ""]], "choices": ["dBm", "percentage"], "type": "str"},
                    "power-value": {"v_range": [["7.0.0", ""]], "type": "int"},
                    "sam-bssid": {"v_range": [["7.0.0", ""]], "type": "str"},
                    "sam-captive-portal": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "sam-password": {"v_range": [["7.0.0", ""]], "no_log": true, "type": "raw"},
                    "sam-report-intv": {"v_range": [["7.0.0", ""]], "type": "int"},
                    "sam-security-type": {
                        "v_range": [["7.0.0", ""]],
                        "choices": ["open", "wpa-personal", "wpa-enterprise", "owe", "wpa3-sae"],
                        "type": "str"
                    },
                    "sam-server": {"v_range": [["7.0.0", ""]], "type": "str"},
                    "sam-ssid": {"v_range": [["7.0.0", ""]], "type": "str"},
                    "sam-test": {"v_range": [["7.0.0", ""]], "choices": ["ping", "iperf"], "type": "str"},
                    "sam-username": {"v_range": [["7.0.0", ""]], "type": "str"},
                    "arrp-profile": {"v_range": [["7.0.3", ""]], "type": "str"},
                    "bss-color-mode": {"v_range": [["7.0.2", ""]], "choices": ["auto", "static"], "type": "str"},
                    "sam-cwp-failure-string": {"v_range": [["7.0.1", ""]], "type": "str"},
                    "sam-cwp-match-string": {"v_range": [["7.0.1", ""]], "type": "str"},
                    "sam-cwp-password": {"v_range": [["7.0.1", ""]], "no_log": true, "type": "raw"},
                    "sam-cwp-success-string": {"v_range": [["7.0.1", ""]], "type": "str"},
                    "sam-cwp-test-url": {"v_range": [["7.0.1", ""]], "type": "str"},
                    "sam-cwp-username": {"v_range": [["7.0.1", ""]], "type": "str"},
                    "sam-server-fqdn": {"v_range": [["7.0.1", ""]], "type": "str"},
                    "sam-server-ip": {"v_range": [["7.0.1", ""]], "type": "str"},
                    "sam-server-type": {"v_range": [["7.0.1", ""]], "choices": ["ip", "fqdn"], "type": "str"},
                    "80211d": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "optional-antenna": {
                        "v_range": [["7.2.3", ""]],
                        "choices": [
                            "none", "FANT-04ABGN-0606-O-N", "FANT-04ABGN-1414-P-N", "FANT-04ABGN-8065-P-N", "FANT-04ABGN-0606-O-R",
                            "FANT-04ABGN-0606-P-R", "FANT-10ACAX-1213-D-N", "FANT-08ABGN-1213-D-R", "custom"
                        ],
                        "type": "str"
                    },
                    "mimo-mode": {"v_range": [["7.4.1", ""]], "choices": ["default", "1x1", "2x2", "3x3", "4x4", "8x8"], "type": "str"},
                    "optional-antenna-gain": {"v_range": [["7.4.2", ""]], "type": "str"},
                    "sam-ca-certificate": {"v_range": [["7.4.2", ""]], "type": "str"},
                    "sam-client-certificate": {"v_range": [["7.4.2", ""]], "type": "str"},
                    "sam-eap-method": {"v_range": [["7.4.2", ""]], "choices": ["tls", "peap", "both"], "type": "str"},
                    "sam-private-key": {"v_range": [["7.4.2", ""]], "no_log": true, "type": "str"},
                    "sam-private-key-password": {"v_range": [["7.4.2", ""]], "no_log": true, "type": "raw"},
                    "channel-bonding-ext": {"v_range": [["7.4.3", ""]], "choices": ["320MHz-1", "320MHz-2"], "type": "str"},
                    "80211mc": {"v_range": [["7.6.0", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "ap-sniffer-chan-width": {
                        "v_range": [["7.6.0", ""]],
                        "choices": ["320MHz", "240MHz", "160MHz", "80MHz", "40MHz", "20MHz"],
                        "type": "str"
                    }
                }
            },
            "console-login": {"v_range": [["6.2.9", "6.2.12"], ["6.4.8", "6.4.14"], ["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "esl-ses-dongle": {
                "v_range": [["7.0.1", ""]],
                "type": "dict",
                "options": {
                    "apc-addr-type": {"v_range": [["7.0.1", ""]], "choices": ["fqdn", "ip"], "type": "str"},
                    "apc-fqdn": {"v_range": [["7.0.1", ""]], "type": "str"},
                    "apc-ip": {"v_range": [["7.0.1", ""]], "type": "str"},
                    "apc-port": {"v_range": [["7.0.1", ""]], "type": "int"},
                    "coex-level": {"v_range": [["7.0.1", ""]], "choices": ["none"], "type": "str"},
                    "compliance-level": {"v_range": [["7.0.1", ""]], "choices": ["compliance-level-2"], "type": "str"},
                    "esl-channel": {
                        "v_range": [["7.0.1", ""]],
                        "choices": ["0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "10", "127", "-1"],
                        "type": "str"
                    },
                    "output-power": {"v_range": [["7.0.1", ""]], "choices": ["a", "b", "c", "d", "e", "f", "g", "h"], "type": "str"},
                    "scd-enable": {"v_range": [["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "tls-cert-verification": {"v_range": [["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
                    "tls-fqdn-verification": {"v_range": [["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"}
                }
            },
            "indoor-outdoor-deployment": {"v_range": [["7.0.1", ""]], "choices": ["platform-determined", "outdoor", "indoor"], "type": "str"},
            "syslog-profile": {"v_range": [["7.0.2", ""]], "type": "str"},
            "wan-port-auth": {"v_range": [["7.0.2", ""]], "choices": ["none", "802.1x"], "type": "str"},
            "wan-port-auth-methods": {"v_range": [["7.0.2", ""]], "choices": ["all", "EAP-FAST", "EAP-TLS", "EAP-PEAP"], "type": "str"},
            "wan-port-auth-password": {"v_range": [["7.0.2", ""]], "no_log": true, "type": "raw"},
            "wan-port-auth-usrname": {"v_range": [["7.0.2", ""]], "type": "str"},
            "_is_factory_setting": {"v_range": [["7.4.0", ""]], "choices": ["disable", "enable", "ext"], "type": "str"},
            "unii-4-5ghz-band": {"v_range": [["7.4.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "bonjour-profile": {"v_range": [["7.4.2", ""]], "type": "str"},
            "wan-port-auth-macsec": {"v_range": [["7.4.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "usb-port": {"v_range": [["7.4.3", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "wtpprofile_denymaclist": {"stated": true, "options": {"id": {"required": true, "type": "int"}, "mac": {"type": "str"}}},
    "wtpprofile_eslsesdongle": {
        "stated": false,
        "options": {
            "apc-addr-type": {"v_range": [["7.0.1", ""]], "choices": ["fqdn", "ip"], "type": "str"},
            "apc-fqdn": {"v_range": [["7.0.1", ""]], "type": "str"},
            "apc-ip": {"v_range": [["7.0.1", ""]], "type": "str"},
            "apc-port": {"v_range": [["7.0.1", ""]], "type": "int"},
            "coex-level": {"v_range": [["7.0.1", ""]], "choices": ["none"], "type": "str"},
            "compliance-level": {"v_range": [["7.0.1", ""]], "choices": ["compliance-level-2"], "type": "str"},
            "esl-channel": {"v_range": [["7.0.1", ""]], "choices": ["0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "10", "127", "-1"], "type": "str"},
            "output-power": {"v_range": [["7.0.1", ""]], "choices": ["a", "b", "c", "d", "e", "f", "g", "h"], "type": "str"},
            "scd-enable": {"v_range": [["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "tls-cert-verification": {"v_range": [["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "tls-fqdn-verification": {"v_range": [["7.0.1", ""]], "choices": ["disable", "enable"], "type": "str"}
        }
    },
    "wtpprofile_lan": {
        "stated": false,
        "options": {
            "port-mode": {"choices": ["offline", "bridge-to-wan", "bridge-to-ssid", "nat-to-wan"], "type": "str"},
            "port-ssid": {"type": "str"},
            "port1-mode": {"choices": ["offline", "bridge-to-wan", "bridge-to-ssid", "nat-to-wan"], "type": "str"},
            "port1-ssid": {"type": "str"},
            "port2-mode": {"choices": ["offline", "bridge-to-wan", "bridge-to-ssid", "nat-to-wan"], "type": "str"},
            "port2-ssid": {"type": "str"},
            "port3-mode": {"choices": ["offline", "bridge-to-wan", "bridge-to-ssid", "nat-to-wan"], "type": "str"},
            "port3-ssid": {"type": "str"},
            "port4-mode": {"choices": ["offline", "bridge-to-wan", "bridge-to-ssid", "nat-to-wan"], "type": "str"},
            "port4-ssid": {"type": "str"},
            "port5-mode": {"choices": ["offline", "bridge-to-wan", "bridge-to-ssid", "nat-to-wan"], "type": "str"},
            "port5-ssid": {"type": "str"},
            "port6-mode": {"choices": ["offline", "bridge-to-wan", "bridge-to-ssid", "nat-to-wan"], "type": "str"},
            "port6-ssid": {"type": "str"},
            "port7-mode": {"choices": ["offline", "bridge-to-wan", "bridge-to-ssid", "nat-to-wan"], "type": "str"},
            "port7-ssid": {"type": "str"},
            "port8-mode": {"choices": ["offline", "bridge-to-wan", "bridge-to-ssid", "nat-to-wan"], "type": "str"},
            "port8-ssid": {"type": "str"},
            "port-esl-mode": {"v_range": [["6.4.2", ""]], "choices": ["offline", "bridge-to-wan", "bridge-to-ssid", "nat-to-wan"], "type": "str"},
            "port-esl-ssid": {"v_range": [["6.4.2", ""]], "type": "str"}
        }
    },
    "wtpprofile_lbs": {
        "stated": false,
        "options": {
            "aeroscout": {"choices": ["disable", "enable"], "type": "str"},
            "aeroscout-ap-mac": {"choices": ["bssid", "board-mac"], "type": "str"},
            "aeroscout-mmu-report": {"choices": ["disable", "enable"], "type": "str"},
            "aeroscout-mu": {"choices": ["disable", "enable"], "type": "str"},
            "aeroscout-mu-factor": {"type": "int"},
            "aeroscout-mu-timeout": {"type": "int"},
            "aeroscout-server-ip": {"type": "str"},
            "aeroscout-server-port": {"type": "int"},
            "ekahau-blink-mode": {"choices": ["disable", "enable"], "type": "str"},
            "ekahau-tag": {"type": "str"},
            "erc-server-ip": {"type": "str"},
            "erc-server-port": {"type": "int"},
            "fortipresence": {"choices": ["disable", "enable", "enable2", "foreign", "both"], "type": "str"},
            "fortipresence-frequency": {"type": "int"},
            "fortipresence-port": {"type": "int"},
            "fortipresence-project": {"type": "str"},
            "fortipresence-rogue": {"choices": ["disable", "enable"], "type": "str"},
            "fortipresence-secret": {"no_log": true, "type": "raw"},
            "fortipresence-server": {"type": "str"},
            "fortipresence-unassoc": {"choices": ["disable", "enable"], "type": "str"},
            "station-locate": {"choices": ["disable", "enable"], "type": "str"},
            "fortipresence-ble": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "fortipresence-server-addr-type": {"v_range": [["7.0.2", ""]], "choices": ["fqdn", "ipv4"], "type": "str"},
            "fortipresence-server-fqdn": {"v_range": [["7.0.2", ""]], "type": "str"},
            "polestar": {"v_range": [["7.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "polestar-accumulation-interval": {"v_range": [["7.4.1", ""]], "type": "int"},
            "polestar-asset-addrgrp-list": {"v_range": [["7.4.1", ""]], "type": "str"},
            "polestar-asset-uuid-list1": {"v_range": [["7.4.1", ""]], "type": "str"},
            "polestar-asset-uuid-list2": {"v_range": [["7.4.1", ""]], "type": "str"},
            "polestar-asset-uuid-list3": {"v_range": [["7.4.1", ""]], "type": "str"},
            "polestar-asset-uuid-list4": {"v_range": [["7.4.1", ""]], "type": "str"},
            "polestar-protocol": {"v_range": [["7.4.1", ""]], "choices": ["WSS"], "type": "str"},
            "polestar-reporting-interval": {"v_range": [["7.4.1", ""]], "type": "int"},
            "polestar-server-fqdn": {"v_range": [["7.4.1", ""]], "type": "str"},
            "polestar-server-path": {"v_range": [["7.4.1", ""]], "type": "str"},
            "polestar-server-port": {"v_range": [["7.4.1", ""]], "type": "int"},
            "polestar-server-token": {"v_range": [["7.4.1", ""]], "no_log": true, "type": "str"}
        }
    },
    "wtpprofile_platform": {
        "stated": false,
        "options": {
            "type": {
                "choices": [
                    "30B-50B", "60B", "80CM-81CM", "220A", "220B", "210B", "60C", "222B", "112B", "320B", "11C", "14C", "223B", "28C", "320C", "221C",
                    "25D", "222C", "224D", "214B", "21D", "24D", "112D", "223C", "321C", "C220C", "C225C", "S321C", "S323C", "FWF", "S311C", "S313C",
                    "AP-11N", "S322C", "S321CR", "S322CR", "S323CR", "S421E", "S422E", "S423E", "421E", "423E", "C221E", "C226E", "C23JD", "C24JE",
                    "C21D", "U421E", "U423E", "221E", "222E", "223E", "S221E", "S223E", "U221EV", "U223EV", "U321EV", "U323EV", "224E", "U422EV",
                    "U24JEV", "321E", "U431F", "U433F", "231E", "431F", "433F", "231F", "432F", "234F", "23JF", "U231F", "831F", "U234F", "U432F",
                    "431FL", "432FR", "433FL", "231FL", "231G", "233G", "431G", "433G", "U231G", "U441G", "234G", "432G", "441K", "443K", "241K", "243K"
                ],
                "type": "str"
            },
            "mode": {"v_range": [["6.2.2", ""]], "choices": ["dual-5G", "single-5G"], "type": "str"},
            "ddscan": {"v_range": [["6.2.5", ""]], "choices": ["disable", "enable"], "type": "str"},
            "_local_platform_str": {"v_range": [["6.2.8", "6.2.12"], ["6.4.6", ""]], "type": "str"}
        }
    },
    "wtpprofile_radio1": {
        "stated": false,
        "options": {
            "amsdu": {"choices": ["disable", "enable"], "type": "str"},
            "ap-handoff": {"choices": ["disable", "enable"], "type": "str"},
            "ap-sniffer-addr": {"type": "str"},
            "ap-sniffer-bufsize": {"type": "int"},
            "ap-sniffer-chan": {"type": "int"},
            "ap-sniffer-ctl": {"choices": ["disable", "enable"], "type": "str"},
            "ap-sniffer-data": {"choices": ["disable", "enable"], "type": "str"},
            "ap-sniffer-mgmt-beacon": {"choices": ["disable", "enable"], "type": "str"},
            "ap-sniffer-mgmt-other": {"choices": ["disable", "enable"], "type": "str"},
            "ap-sniffer-mgmt-probe": {"choices": ["disable", "enable"], "type": "str"},
            "auto-power-high": {"type": "int"},
            "auto-power-level": {"choices": ["disable", "enable"], "type": "str"},
            "auto-power-low": {"type": "int"},
            "band": {
                "choices": [
                    "802.11b", "802.11a", "802.11g", "802.11n", "802.11ac", "802.11n-5G", "802.11g-only", "802.11n-only", "802.11n,g-only",
                    "802.11ac-only", "802.11ac,n-only", "802.11n-5G-only", "802.11ax-5G", "802.11ax", "802.11ax-5G-only", "802.11ax,ac-only",
                    "802.11ax,ac,n-only", "802.11ax-only", "802.11ax,n-only", "802.11ax,n,g-only", "802.11ac-2G", "802.11ax-6G", "802.11n-2G",
                    "802.11ac-5G", "802.11ax-2G", "802.11be-2G", "802.11be-5G", "802.11be-6G"
                ],
                "type": "str"
            },
            "bandwidth-admission-control": {"choices": ["disable", "enable"], "type": "str"},
            "bandwidth-capacity": {"type": "int"},
            "beacon-interval": {"type": "int"},
            "call-admission-control": {"choices": ["disable", "enable"], "type": "str"},
            "call-capacity": {"type": "int"},
            "channel": {"type": "raw"},
            "channel-bonding": {"choices": ["disable", "enable", "80MHz", "40MHz", "20MHz", "160MHz", "320MHz", "240MHz"], "type": "str"},
            "channel-utilization": {"choices": ["disable", "enable"], "type": "str"},
            "coexistence": {"choices": ["disable", "enable"], "type": "str"},
            "darrp": {"choices": ["disable", "enable"], "type": "str"},
            "dtim": {"type": "int"},
            "frag-threshold": {"type": "int"},
            "frequency-handoff": {"choices": ["disable", "enable"], "type": "str"},
            "max-clients": {"type": "int"},
            "max-distance": {"type": "int"},
            "mode": {"choices": ["disabled", "ap", "monitor", "sniffer", "sam"], "type": "str"},
            "power-level": {"type": "int"},
            "powersave-optimize": {"type": "list", "choices": ["tim", "ac-vo", "no-obss-scan", "no-11b-rate", "client-rate-follow"], "elements": "str"},
            "protection-mode": {"choices": ["rtscts", "ctsonly", "disable"], "type": "str"},
            "radio-id": {"type": "int"},
            "rts-threshold": {"type": "int"},
            "short-guard-interval": {"choices": ["disable", "enable"], "type": "str"},
            "spectrum-analysis": {"choices": ["disable", "enable", "scan-only"], "type": "str"},
            "transmit-optimize": {"type": "list", "choices": ["disable", "power-save", "aggr-limit", "retry-limit", "send-bar"], "elements": "str"},
            "vap-all": {"choices": ["disable", "enable", "tunnel", "bridge", "manual"], "type": "str"},
            "vaps": {"type": "raw"},
            "wids-profile": {"type": "str"},
            "airtime-fairness": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "band-5g-type": {"v_range": [["6.2.5", ""]], "choices": ["5g-full", "5g-high", "5g-low"], "type": "str"},
            "zero-wait-dfs": {"v_range": [["6.2.5", ""]], "choices": ["disable", "enable"], "type": "str"},
            "vap1": {"v_range": [["6.4.0", ""]], "type": "str"},
            "vap2": {"v_range": [["6.4.0", ""]], "type": "str"},
            "vap3": {"v_range": [["6.4.0", ""]], "type": "str"},
            "vap4": {"v_range": [["6.4.0", ""]], "type": "str"},
            "vap5": {"v_range": [["6.4.0", ""]], "type": "str"},
            "vap6": {"v_range": [["6.4.0", ""]], "type": "str"},
            "vap7": {"v_range": [["6.4.0", ""]], "type": "str"},
            "vap8": {"v_range": [["6.4.0", ""]], "type": "str"},
            "bss-color": {"v_range": [["6.4.2", ""]], "type": "int"},
            "auto-power-target": {"v_range": [["6.4.3", ""]], "type": "str"},
            "drma": {"v_range": [["6.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "drma-sensitivity": {"v_range": [["6.4.3", ""]], "choices": ["low", "medium", "high"], "type": "str"},
            "iperf-protocol": {"v_range": [["7.0.0", ""]], "choices": ["udp", "tcp"], "type": "str"},
            "iperf-server-port": {"v_range": [["7.0.0", ""]], "type": "int"},
            "power-mode": {"v_range": [["7.0.0", ""]], "choices": ["dBm", "percentage"], "type": "str"},
            "power-value": {"v_range": [["7.0.0", ""]], "type": "int"},
            "sam-bssid": {"v_range": [["7.0.0", ""]], "type": "str"},
            "sam-captive-portal": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "sam-password": {"v_range": [["7.0.0", ""]], "no_log": true, "type": "raw"},
            "sam-report-intv": {"v_range": [["7.0.0", ""]], "type": "int"},
            "sam-security-type": {"v_range": [["7.0.0", ""]], "choices": ["open", "wpa-personal", "wpa-enterprise", "owe", "wpa3-sae"], "type": "str"},
            "sam-server": {"v_range": [["7.0.0", ""]], "type": "str"},
            "sam-ssid": {"v_range": [["7.0.0", ""]], "type": "str"},
            "sam-test": {"v_range": [["7.0.0", ""]], "choices": ["ping", "iperf"], "type": "str"},
            "sam-username": {"v_range": [["7.0.0", ""]], "type": "str"},
            "arrp-profile": {"v_range": [["7.0.3", ""]], "type": "str"},
            "bss-color-mode": {"v_range": [["7.0.2", ""]], "choices": ["auto", "static"], "type": "str"},
            "sam-cwp-failure-string": {"v_range": [["7.0.1", ""]], "type": "str"},
            "sam-cwp-match-string": {"v_range": [["7.0.1", ""]], "type": "str"},
            "sam-cwp-password": {"v_range": [["7.0.1", ""]], "no_log": true, "type": "raw"},
            "sam-cwp-success-string": {"v_range": [["7.0.1", ""]], "type": "str"},
            "sam-cwp-test-url": {"v_range": [["7.0.1", ""]], "type": "str"},
            "sam-cwp-username": {"v_range": [["7.0.1", ""]], "type": "str"},
            "sam-server-fqdn": {"v_range": [["7.0.1", ""]], "type": "str"},
            "sam-server-ip": {"v_range": [["7.0.1", ""]], "type": "str"},
            "sam-server-type": {"v_range": [["7.0.1", ""]], "choices": ["ip", "fqdn"], "type": "str"},
            "80211d": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "optional-antenna": {
                "v_range": [["7.2.3", ""]],
                "choices": [
                    "none", "FANT-04ABGN-0606-O-N", "FANT-04ABGN-1414-P-N", "FANT-04ABGN-8065-P-N", "FANT-04ABGN-0606-O-R", "FANT-04ABGN-0606-P-R",
                    "FANT-10ACAX-1213-D-N", "FANT-08ABGN-1213-D-R", "custom"
                ],
                "type": "str"
            },
            "mimo-mode": {"v_range": [["7.4.1", ""]], "choices": ["default", "1x1", "2x2", "3x3", "4x4", "8x8"], "type": "str"},
            "optional-antenna-gain": {"v_range": [["7.4.2", ""]], "type": "str"},
            "sam-ca-certificate": {"v_range": [["7.4.2", ""]], "type": "str"},
            "sam-client-certificate": {"v_range": [["7.4.2", ""]], "type": "str"},
            "sam-eap-method": {"v_range": [["7.4.2", ""]], "choices": ["tls", "peap", "both"], "type": "str"},
            "sam-private-key": {"v_range": [["7.4.2", ""]], "no_log": true, "type": "str"},
            "sam-private-key-password": {"v_range": [["7.4.2", ""]], "no_log": true, "type": "raw"},
            "channel-bonding-ext": {"v_range": [["7.4.3", ""]], "choices": ["320MHz-1", "320MHz-2"], "type": "str"},
            "80211mc": {"v_range": [["7.6.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ap-sniffer-chan-width": {"v_range": [["7.6.0", ""]], "choices": ["320MHz", "240MHz", "160MHz", "80MHz", "40MHz", "20MHz"], "type": "str"}
        }
    },
    "wtpprofile_radio2": {
        "stated": false,
        "options": {
            "amsdu": {"choices": ["disable", "enable"], "type": "str"},
            "ap-handoff": {"choices": ["disable", "enable"], "type": "str"},
            "ap-sniffer-addr": {"type": "str"},
            "ap-sniffer-bufsize": {"type": "int"},
            "ap-sniffer-chan": {"type": "int"},
            "ap-sniffer-ctl": {"choices": ["disable", "enable"], "type": "str"},
            "ap-sniffer-data": {"choices": ["disable", "enable"], "type": "str"},
            "ap-sniffer-mgmt-beacon": {"choices": ["disable", "enable"], "type": "str"},
            "ap-sniffer-mgmt-other": {"choices": ["disable", "enable"], "type": "str"},
            "ap-sniffer-mgmt-probe": {"choices": ["disable", "enable"], "type": "str"},
            "auto-power-high": {"type": "int"},
            "auto-power-level": {"choices": ["disable", "enable"], "type": "str"},
            "auto-power-low": {"type": "int"},
            "band": {
                "choices": [
                    "802.11b", "802.11a", "802.11g", "802.11n", "802.11ac", "802.11n-5G", "802.11g-only", "802.11n-only", "802.11n,g-only",
                    "802.11ac-only", "802.11ac,n-only", "802.11n-5G-only", "802.11ax-5G", "802.11ax", "802.11ax-5G-only", "802.11ax,ac-only",
                    "802.11ax,ac,n-only", "802.11ax-only", "802.11ax,n-only", "802.11ax,n,g-only", "802.11ac-2G", "802.11ax-6G", "802.11n-2G",
                    "802.11ac-5G", "802.11ax-2G", "802.11be-2G", "802.11be-5G", "802.11be-6G"
                ],
                "type": "str"
            },
            "bandwidth-admission-control": {"choices": ["disable", "enable"], "type": "str"},
            "bandwidth-capacity": {"type": "int"},
            "beacon-interval": {"type": "int"},
            "call-admission-control": {"choices": ["disable", "enable"], "type": "str"},
            "call-capacity": {"type": "int"},
            "channel": {"type": "raw"},
            "channel-bonding": {"choices": ["disable", "enable", "80MHz", "40MHz", "20MHz", "160MHz", "320MHz", "240MHz"], "type": "str"},
            "channel-utilization": {"choices": ["disable", "enable"], "type": "str"},
            "coexistence": {"choices": ["disable", "enable"], "type": "str"},
            "darrp": {"choices": ["disable", "enable"], "type": "str"},
            "dtim": {"type": "int"},
            "frag-threshold": {"type": "int"},
            "frequency-handoff": {"choices": ["disable", "enable"], "type": "str"},
            "max-clients": {"type": "int"},
            "max-distance": {"type": "int"},
            "mode": {"choices": ["disabled", "ap", "monitor", "sniffer", "sam"], "type": "str"},
            "power-level": {"type": "int"},
            "powersave-optimize": {"type": "list", "choices": ["tim", "ac-vo", "no-obss-scan", "no-11b-rate", "client-rate-follow"], "elements": "str"},
            "protection-mode": {"choices": ["rtscts", "ctsonly", "disable"], "type": "str"},
            "radio-id": {"type": "int"},
            "rts-threshold": {"type": "int"},
            "short-guard-interval": {"choices": ["disable", "enable"], "type": "str"},
            "spectrum-analysis": {"choices": ["disable", "enable", "scan-only"], "type": "str"},
            "transmit-optimize": {"type": "list", "choices": ["disable", "power-save", "aggr-limit", "retry-limit", "send-bar"], "elements": "str"},
            "vap-all": {"choices": ["disable", "enable", "tunnel", "bridge", "manual"], "type": "str"},
            "vaps": {"type": "raw"},
            "wids-profile": {"type": "str"},
            "airtime-fairness": {"v_range": [["6.2.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "band-5g-type": {"v_range": [["6.2.5", ""]], "choices": ["5g-full", "5g-high", "5g-low"], "type": "str"},
            "zero-wait-dfs": {"v_range": [["6.2.5", ""]], "choices": ["disable", "enable"], "type": "str"},
            "vap1": {"v_range": [["6.4.0", ""]], "type": "str"},
            "vap2": {"v_range": [["6.4.0", ""]], "type": "str"},
            "vap3": {"v_range": [["6.4.0", ""]], "type": "str"},
            "vap4": {"v_range": [["6.4.0", ""]], "type": "str"},
            "vap5": {"v_range": [["6.4.0", ""]], "type": "str"},
            "vap6": {"v_range": [["6.4.0", ""]], "type": "str"},
            "vap7": {"v_range": [["6.4.0", ""]], "type": "str"},
            "vap8": {"v_range": [["6.4.0", ""]], "type": "str"},
            "bss-color": {"v_range": [["6.4.2", ""]], "type": "int"},
            "auto-power-target": {"v_range": [["6.4.3", ""]], "type": "str"},
            "drma": {"v_range": [["6.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "drma-sensitivity": {"v_range": [["6.4.3", ""]], "choices": ["low", "medium", "high"], "type": "str"},
            "iperf-protocol": {"v_range": [["7.0.0", ""]], "choices": ["udp", "tcp"], "type": "str"},
            "iperf-server-port": {"v_range": [["7.0.0", ""]], "type": "int"},
            "power-mode": {"v_range": [["7.0.0", ""]], "choices": ["dBm", "percentage"], "type": "str"},
            "power-value": {"v_range": [["7.0.0", ""]], "type": "int"},
            "sam-bssid": {"v_range": [["7.0.0", ""]], "type": "str"},
            "sam-captive-portal": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "sam-password": {"v_range": [["7.0.0", ""]], "no_log": true, "type": "raw"},
            "sam-report-intv": {"v_range": [["7.0.0", ""]], "type": "int"},
            "sam-security-type": {"v_range": [["7.0.0", ""]], "choices": ["open", "wpa-personal", "wpa-enterprise", "owe", "wpa3-sae"], "type": "str"},
            "sam-server": {"v_range": [["7.0.0", ""]], "type": "str"},
            "sam-ssid": {"v_range": [["7.0.0", ""]], "type": "str"},
            "sam-test": {"v_range": [["7.0.0", ""]], "choices": ["ping", "iperf"], "type": "str"},
            "sam-username": {"v_range": [["7.0.0", ""]], "type": "str"},
            "arrp-profile": {"v_range": [["7.0.3", ""]], "type": "str"},
            "bss-color-mode": {"v_range": [["7.0.2", ""]], "choices": ["auto", "static"], "type": "str"},
            "sam-cwp-failure-string": {"v_range": [["7.0.1", ""]], "type": "str"},
            "sam-cwp-match-string": {"v_range": [["7.0.1", ""]], "type": "str"},
            "sam-cwp-password": {"v_range": [["7.0.1", ""]], "no_log": true, "type": "raw"},
            "sam-cwp-success-string": {"v_range": [["7.0.1", ""]], "type": "str"},
            "sam-cwp-test-url": {"v_range": [["7.0.1", ""]], "type": "str"},
            "sam-cwp-username": {"v_range": [["7.0.1", ""]], "type": "str"},
            "sam-server-fqdn": {"v_range": [["7.0.1", ""]], "type": "str"},
            "sam-server-ip": {"v_range": [["7.0.1", ""]], "type": "str"},
            "sam-server-type": {"v_range": [["7.0.1", ""]], "choices": ["ip", "fqdn"], "type": "str"},
            "80211d": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "optional-antenna": {
                "v_range": [["7.2.3", ""]],
                "choices": [
                    "none", "FANT-04ABGN-0606-O-N", "FANT-04ABGN-1414-P-N", "FANT-04ABGN-8065-P-N", "FANT-04ABGN-0606-O-R", "FANT-04ABGN-0606-P-R",
                    "FANT-10ACAX-1213-D-N", "FANT-08ABGN-1213-D-R", "custom"
                ],
                "type": "str"
            },
            "mimo-mode": {"v_range": [["7.4.1", ""]], "choices": ["default", "1x1", "2x2", "3x3", "4x4", "8x8"], "type": "str"},
            "optional-antenna-gain": {"v_range": [["7.4.2", ""]], "type": "str"},
            "sam-ca-certificate": {"v_range": [["7.4.2", ""]], "type": "str"},
            "sam-client-certificate": {"v_range": [["7.4.2", ""]], "type": "str"},
            "sam-eap-method": {"v_range": [["7.4.2", ""]], "choices": ["tls", "peap", "both"], "type": "str"},
            "sam-private-key": {"v_range": [["7.4.2", ""]], "no_log": true, "type": "str"},
            "sam-private-key-password": {"v_range": [["7.4.2", ""]], "no_log": true, "type": "raw"},
            "channel-bonding-ext": {"v_range": [["7.4.3", ""]], "choices": ["320MHz-1", "320MHz-2"], "type": "str"},
            "80211mc": {"v_range": [["7.6.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ap-sniffer-chan-width": {"v_range": [["7.6.0", ""]], "choices": ["320MHz", "240MHz", "160MHz", "80MHz", "40MHz", "20MHz"], "type": "str"}
        }
    },
    "wtpprofile_radio3": {
        "stated": false,
        "options": {
            "airtime-fairness": {"v_range": [["6.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "amsdu": {"v_range": [["6.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ap-handoff": {"v_range": [["6.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ap-sniffer-addr": {"v_range": [["6.2.2", ""]], "type": "str"},
            "ap-sniffer-bufsize": {"v_range": [["6.2.2", ""]], "type": "int"},
            "ap-sniffer-chan": {"v_range": [["6.2.2", ""]], "type": "int"},
            "ap-sniffer-ctl": {"v_range": [["6.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ap-sniffer-data": {"v_range": [["6.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ap-sniffer-mgmt-beacon": {"v_range": [["6.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ap-sniffer-mgmt-other": {"v_range": [["6.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ap-sniffer-mgmt-probe": {"v_range": [["6.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "auto-power-high": {"v_range": [["6.2.2", ""]], "type": "int"},
            "auto-power-level": {"v_range": [["6.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "auto-power-low": {"v_range": [["6.2.2", ""]], "type": "int"},
            "band": {
                "v_range": [["6.2.2", ""]],
                "choices": [
                    "802.11b", "802.11a", "802.11g", "802.11n", "802.11ac", "802.11n-5G", "802.11ax-5G", "802.11ax", "802.11g-only", "802.11n-only",
                    "802.11n,g-only", "802.11ac-only", "802.11ac,n-only", "802.11n-5G-only", "802.11ax-5G-only", "802.11ax,ac-only",
                    "802.11ax,ac,n-only", "802.11ax-only", "802.11ax,n-only", "802.11ax,n,g-only", "802.11ac-2G", "802.11ax-6G", "802.11n-2G",
                    "802.11ac-5G", "802.11ax-2G", "802.11be-2G", "802.11be-5G", "802.11be-6G"
                ],
                "type": "str"
            },
            "bandwidth-admission-control": {"v_range": [["6.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "bandwidth-capacity": {"v_range": [["6.2.2", ""]], "type": "int"},
            "beacon-interval": {"v_range": [["6.2.2", ""]], "type": "int"},
            "call-admission-control": {"v_range": [["6.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "call-capacity": {"v_range": [["6.2.2", ""]], "type": "int"},
            "channel": {"v_range": [["6.2.2", ""]], "type": "raw"},
            "channel-bonding": {"v_range": [["6.2.2", ""]], "choices": ["80MHz", "40MHz", "20MHz", "160MHz", "320MHz", "240MHz"], "type": "str"},
            "channel-utilization": {"v_range": [["6.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "coexistence": {"v_range": [["6.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "darrp": {"v_range": [["6.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "dtim": {"v_range": [["6.2.2", ""]], "type": "int"},
            "frag-threshold": {"v_range": [["6.2.2", ""]], "type": "int"},
            "frequency-handoff": {"v_range": [["6.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "max-clients": {"v_range": [["6.2.2", ""]], "type": "int"},
            "max-distance": {"v_range": [["6.2.2", ""]], "type": "int"},
            "mode": {"v_range": [["6.2.2", ""]], "choices": ["disabled", "ap", "monitor", "sniffer", "sam"], "type": "str"},
            "power-level": {"v_range": [["6.2.2", ""]], "type": "int"},
            "powersave-optimize": {
                "v_range": [["6.2.2", ""]],
                "type": "list",
                "choices": ["tim", "ac-vo", "no-obss-scan", "no-11b-rate", "client-rate-follow"],
                "elements": "str"
            },
            "protection-mode": {"v_range": [["6.2.2", ""]], "choices": ["rtscts", "ctsonly", "disable"], "type": "str"},
            "radio-id": {"v_range": [["6.2.2", ""]], "type": "int"},
            "rts-threshold": {"v_range": [["6.2.2", ""]], "type": "int"},
            "short-guard-interval": {"v_range": [["6.2.2", ""]], "choices": ["disable", "enable"], "type": "str"},
            "spectrum-analysis": {"v_range": [["6.2.2", ""]], "choices": ["disable", "enable", "scan-only"], "type": "str"},
            "transmit-optimize": {
                "v_range": [["6.2.2", ""]],
                "type": "list",
                "choices": ["disable", "power-save", "aggr-limit", "retry-limit", "send-bar"],
                "elements": "str"
            },
            "vap-all": {"v_range": [["6.2.2", ""]], "choices": ["disable", "enable", "tunnel", "bridge", "manual"], "type": "str"},
            "vaps": {"v_range": [["6.2.2", ""]], "type": "raw"},
            "wids-profile": {"v_range": [["6.2.2", ""]], "type": "str"},
            "band-5g-type": {"v_range": [["6.2.5", ""]], "choices": ["5g-full", "5g-high", "5g-low"], "type": "str"},
            "zero-wait-dfs": {"v_range": [["6.2.5", ""]], "choices": ["disable", "enable"], "type": "str"},
            "vap1": {"v_range": [["6.4.0", ""]], "type": "str"},
            "vap2": {"v_range": [["6.4.0", ""]], "type": "str"},
            "vap3": {"v_range": [["6.4.0", ""]], "type": "str"},
            "vap4": {"v_range": [["6.4.0", ""]], "type": "str"},
            "vap5": {"v_range": [["6.4.0", ""]], "type": "str"},
            "vap6": {"v_range": [["6.4.0", ""]], "type": "str"},
            "vap7": {"v_range": [["6.4.0", ""]], "type": "str"},
            "vap8": {"v_range": [["6.4.0", ""]], "type": "str"},
            "bss-color": {"v_range": [["6.4.2", ""]], "type": "int"},
            "auto-power-target": {"v_range": [["6.4.3", ""]], "type": "str"},
            "drma": {"v_range": [["6.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "drma-sensitivity": {"v_range": [["6.4.3", ""]], "choices": ["low", "medium", "high"], "type": "str"},
            "iperf-protocol": {"v_range": [["7.0.0", ""]], "choices": ["udp", "tcp"], "type": "str"},
            "iperf-server-port": {"v_range": [["7.0.0", ""]], "type": "int"},
            "power-mode": {"v_range": [["7.0.0", ""]], "choices": ["dBm", "percentage"], "type": "str"},
            "power-value": {"v_range": [["7.0.0", ""]], "type": "int"},
            "sam-bssid": {"v_range": [["7.0.0", ""]], "type": "str"},
            "sam-captive-portal": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "sam-password": {"v_range": [["7.0.0", ""]], "no_log": true, "type": "raw"},
            "sam-report-intv": {"v_range": [["7.0.0", ""]], "type": "int"},
            "sam-security-type": {"v_range": [["7.0.0", ""]], "choices": ["open", "wpa-personal", "wpa-enterprise", "owe", "wpa3-sae"], "type": "str"},
            "sam-server": {"v_range": [["7.0.0", ""]], "type": "str"},
            "sam-ssid": {"v_range": [["7.0.0", ""]], "type": "str"},
            "sam-test": {"v_range": [["7.0.0", ""]], "choices": ["ping", "iperf"], "type": "str"},
            "sam-username": {"v_range": [["7.0.0", ""]], "type": "str"},
            "arrp-profile": {"v_range": [["7.0.3", ""]], "type": "str"},
            "bss-color-mode": {"v_range": [["7.0.2", ""]], "choices": ["auto", "static"], "type": "str"},
            "sam-cwp-failure-string": {"v_range": [["7.0.1", ""]], "type": "str"},
            "sam-cwp-match-string": {"v_range": [["7.0.1", ""]], "type": "str"},
            "sam-cwp-password": {"v_range": [["7.0.1", ""]], "no_log": true, "type": "raw"},
            "sam-cwp-success-string": {"v_range": [["7.0.1", ""]], "type": "str"},
            "sam-cwp-test-url": {"v_range": [["7.0.1", ""]], "type": "str"},
            "sam-cwp-username": {"v_range": [["7.0.1", ""]], "type": "str"},
            "sam-server-fqdn": {"v_range": [["7.0.1", ""]], "type": "str"},
            "sam-server-ip": {"v_range": [["7.0.1", ""]], "type": "str"},
            "sam-server-type": {"v_range": [["7.0.1", ""]], "choices": ["ip", "fqdn"], "type": "str"},
            "80211d": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "optional-antenna": {
                "v_range": [["7.2.3", ""]],
                "choices": [
                    "none", "FANT-04ABGN-0606-O-N", "FANT-04ABGN-1414-P-N", "FANT-04ABGN-8065-P-N", "FANT-04ABGN-0606-O-R", "FANT-04ABGN-0606-P-R",
                    "FANT-10ACAX-1213-D-N", "FANT-08ABGN-1213-D-R", "custom"
                ],
                "type": "str"
            },
            "mimo-mode": {"v_range": [["7.4.1", ""]], "choices": ["default", "1x1", "2x2", "3x3", "4x4", "8x8"], "type": "str"},
            "optional-antenna-gain": {"v_range": [["7.4.2", ""]], "type": "str"},
            "sam-ca-certificate": {"v_range": [["7.4.2", ""]], "type": "str"},
            "sam-client-certificate": {"v_range": [["7.4.2", ""]], "type": "str"},
            "sam-eap-method": {"v_range": [["7.4.2", ""]], "choices": ["tls", "peap", "both"], "type": "str"},
            "sam-private-key": {"v_range": [["7.4.2", ""]], "no_log": true, "type": "str"},
            "sam-private-key-password": {"v_range": [["7.4.2", ""]], "no_log": true, "type": "raw"},
            "channel-bonding-ext": {"v_range": [["7.4.3", ""]], "choices": ["320MHz-1", "320MHz-2"], "type": "str"},
            "80211mc": {"v_range": [["7.6.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ap-sniffer-chan-width": {"v_range": [["7.6.0", ""]], "choices": ["320MHz", "240MHz", "160MHz", "80MHz", "40MHz", "20MHz"], "type": "str"}
        }
    },
    "wtpprofile_radio4": {
        "stated": false,
        "options": {
            "airtime-fairness": {"v_range": [["6.2.5", ""]], "choices": ["disable", "enable"], "type": "str"},
            "amsdu": {"v_range": [["6.2.5", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ap-handoff": {"v_range": [["6.2.5", "6.2.12"], ["6.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ap-sniffer-addr": {"v_range": [["6.2.5", ""]], "type": "str"},
            "ap-sniffer-bufsize": {"v_range": [["6.2.5", ""]], "type": "int"},
            "ap-sniffer-chan": {"v_range": [["6.2.5", ""]], "type": "int"},
            "ap-sniffer-ctl": {"v_range": [["6.2.5", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ap-sniffer-data": {"v_range": [["6.2.5", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ap-sniffer-mgmt-beacon": {"v_range": [["6.2.5", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ap-sniffer-mgmt-other": {"v_range": [["6.2.5", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ap-sniffer-mgmt-probe": {"v_range": [["6.2.5", ""]], "choices": ["disable", "enable"], "type": "str"},
            "auto-power-high": {"v_range": [["6.2.5", ""]], "type": "int"},
            "auto-power-level": {"v_range": [["6.2.5", ""]], "choices": ["disable", "enable"], "type": "str"},
            "auto-power-low": {"v_range": [["6.2.5", ""]], "type": "int"},
            "band": {
                "v_range": [["6.2.5", ""]],
                "choices": [
                    "802.11b", "802.11a", "802.11g", "802.11n", "802.11ac", "802.11n-5G", "802.11ax-5G", "802.11ax", "802.11g-only", "802.11n-only",
                    "802.11n,g-only", "802.11ac-only", "802.11ac,n-only", "802.11n-5G-only", "802.11ax-5G-only", "802.11ax,ac-only",
                    "802.11ax,ac,n-only", "802.11ax-only", "802.11ax,n-only", "802.11ax,n,g-only", "802.11ac-2G", "802.11ax-6G", "802.11n-2G",
                    "802.11ac-5G", "802.11ax-2G", "802.11be-2G", "802.11be-5G", "802.11be-6G"
                ],
                "type": "str"
            },
            "band-5g-type": {"v_range": [["6.2.5", ""]], "choices": ["5g-full", "5g-high", "5g-low"], "type": "str"},
            "bandwidth-admission-control": {"v_range": [["6.2.5", ""]], "choices": ["disable", "enable"], "type": "str"},
            "bandwidth-capacity": {"v_range": [["6.2.5", ""]], "type": "int"},
            "beacon-interval": {"v_range": [["6.2.5", ""]], "type": "int"},
            "call-admission-control": {"v_range": [["6.2.5", ""]], "choices": ["disable", "enable"], "type": "str"},
            "call-capacity": {"v_range": [["6.2.5", ""]], "type": "int"},
            "channel": {"v_range": [["6.2.5", ""]], "type": "raw"},
            "channel-bonding": {"v_range": [["6.2.5", ""]], "choices": ["80MHz", "40MHz", "20MHz", "160MHz", "320MHz", "240MHz"], "type": "str"},
            "channel-utilization": {"v_range": [["6.2.5", ""]], "choices": ["disable", "enable"], "type": "str"},
            "coexistence": {"v_range": [["6.2.5", ""]], "choices": ["disable", "enable"], "type": "str"},
            "darrp": {"v_range": [["6.2.5", ""]], "choices": ["disable", "enable"], "type": "str"},
            "dtim": {"v_range": [["6.2.5", ""]], "type": "int"},
            "frag-threshold": {"v_range": [["6.2.5", ""]], "type": "int"},
            "frequency-handoff": {"v_range": [["6.2.5", "6.2.12"], ["6.4.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "max-clients": {"v_range": [["6.2.5", ""]], "type": "int"},
            "max-distance": {"v_range": [["6.2.5", ""]], "type": "int"},
            "mode": {"v_range": [["6.2.5", ""]], "choices": ["ap", "monitor", "sniffer", "disabled", "sam"], "type": "str"},
            "power-level": {"v_range": [["6.2.5", ""]], "type": "int"},
            "powersave-optimize": {
                "v_range": [["6.2.5", ""]],
                "type": "list",
                "choices": ["tim", "ac-vo", "no-obss-scan", "no-11b-rate", "client-rate-follow"],
                "elements": "str"
            },
            "protection-mode": {"v_range": [["6.2.5", ""]], "choices": ["rtscts", "ctsonly", "disable"], "type": "str"},
            "radio-id": {"v_range": [["6.2.5", ""]], "type": "int"},
            "rts-threshold": {"v_range": [["6.2.5", ""]], "type": "int"},
            "short-guard-interval": {"v_range": [["6.2.5", ""]], "choices": ["disable", "enable"], "type": "str"},
            "spectrum-analysis": {"v_range": [["6.2.5", ""]], "choices": ["disable", "enable", "scan-only"], "type": "str"},
            "transmit-optimize": {
                "v_range": [["6.2.5", ""]],
                "type": "list",
                "choices": ["disable", "power-save", "aggr-limit", "retry-limit", "send-bar"],
                "elements": "str"
            },
            "vap-all": {"v_range": [["6.2.5", ""]], "choices": ["disable", "enable", "tunnel", "bridge", "manual"], "type": "str"},
            "vaps": {"v_range": [["6.2.5", ""]], "type": "raw"},
            "wids-profile": {"v_range": [["6.2.5", ""]], "type": "str"},
            "zero-wait-dfs": {"v_range": [["6.2.5", ""]], "choices": ["disable", "enable"], "type": "str"},
            "vap1": {"v_range": [["6.4.0", ""]], "type": "str"},
            "vap2": {"v_range": [["6.4.0", ""]], "type": "str"},
            "vap3": {"v_range": [["6.4.0", ""]], "type": "str"},
            "vap4": {"v_range": [["6.4.0", ""]], "type": "str"},
            "vap5": {"v_range": [["6.4.0", ""]], "type": "str"},
            "vap6": {"v_range": [["6.4.0", ""]], "type": "str"},
            "vap7": {"v_range": [["6.4.0", ""]], "type": "str"},
            "vap8": {"v_range": [["6.4.0", ""]], "type": "str"},
            "bss-color": {"v_range": [["6.4.2", ""]], "type": "int"},
            "auto-power-target": {"v_range": [["6.4.3", ""]], "type": "str"},
            "drma": {"v_range": [["6.4.3", ""]], "choices": ["disable", "enable"], "type": "str"},
            "drma-sensitivity": {"v_range": [["6.4.3", ""]], "choices": ["low", "medium", "high"], "type": "str"},
            "iperf-protocol": {"v_range": [["7.0.0", ""]], "choices": ["udp", "tcp"], "type": "str"},
            "iperf-server-port": {"v_range": [["7.0.0", ""]], "type": "int"},
            "power-mode": {"v_range": [["7.0.0", ""]], "choices": ["dBm", "percentage"], "type": "str"},
            "power-value": {"v_range": [["7.0.0", ""]], "type": "int"},
            "sam-bssid": {"v_range": [["7.0.0", ""]], "type": "str"},
            "sam-captive-portal": {"v_range": [["7.0.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "sam-password": {"v_range": [["7.0.0", ""]], "no_log": true, "type": "raw"},
            "sam-report-intv": {"v_range": [["7.0.0", ""]], "type": "int"},
            "sam-security-type": {"v_range": [["7.0.0", ""]], "choices": ["open", "wpa-personal", "wpa-enterprise", "owe", "wpa3-sae"], "type": "str"},
            "sam-server": {"v_range": [["7.0.0", ""]], "type": "str"},
            "sam-ssid": {"v_range": [["7.0.0", ""]], "type": "str"},
            "sam-test": {"v_range": [["7.0.0", ""]], "choices": ["ping", "iperf"], "type": "str"},
            "sam-username": {"v_range": [["7.0.0", ""]], "type": "str"},
            "arrp-profile": {"v_range": [["7.0.3", ""]], "type": "str"},
            "bss-color-mode": {"v_range": [["7.0.2", ""]], "choices": ["auto", "static"], "type": "str"},
            "sam-cwp-failure-string": {"v_range": [["7.0.1", ""]], "type": "str"},
            "sam-cwp-match-string": {"v_range": [["7.0.1", ""]], "type": "str"},
            "sam-cwp-password": {"v_range": [["7.0.1", ""]], "no_log": true, "type": "raw"},
            "sam-cwp-success-string": {"v_range": [["7.0.1", ""]], "type": "str"},
            "sam-cwp-test-url": {"v_range": [["7.0.1", ""]], "type": "str"},
            "sam-cwp-username": {"v_range": [["7.0.1", ""]], "type": "str"},
            "sam-server-fqdn": {"v_range": [["7.0.1", ""]], "type": "str"},
            "sam-server-ip": {"v_range": [["7.0.1", ""]], "type": "str"},
            "sam-server-type": {"v_range": [["7.0.1", ""]], "choices": ["ip", "fqdn"], "type": "str"},
            "80211d": {"v_range": [["7.2.1", ""]], "choices": ["disable", "enable"], "type": "str"},
            "optional-antenna": {
                "v_range": [["7.2.3", ""]],
                "choices": [
                    "none", "FANT-04ABGN-0606-O-N", "FANT-04ABGN-1414-P-N", "FANT-04ABGN-8065-P-N", "FANT-04ABGN-0606-O-R", "FANT-04ABGN-0606-P-R",
                    "FANT-10ACAX-1213-D-N", "FANT-08ABGN-1213-D-R", "custom"
                ],
                "type": "str"
            },
            "mimo-mode": {"v_range": [["7.4.1", ""]], "choices": ["default", "1x1", "2x2", "3x3", "4x4", "8x8"], "type": "str"},
            "optional-antenna-gain": {"v_range": [["7.4.2", ""]], "type": "str"},
            "sam-ca-certificate": {"v_range": [["7.4.2", ""]], "type": "str"},
            "sam-client-certificate": {"v_range": [["7.4.2", ""]], "type": "str"},
            "sam-eap-method": {"v_range": [["7.4.2", ""]], "choices": ["tls", "peap", "both"], "type": "str"},
            "sam-private-key": {"v_range": [["7.4.2", ""]], "no_log": true, "type": "str"},
            "sam-private-key-password": {"v_range": [["7.4.2", ""]], "no_log": true, "type": "raw"},
            "channel-bonding-ext": {"v_range": [["7.4.3", ""]], "choices": ["320MHz-1", "320MHz-2"], "type": "str"},
            "80211mc": {"v_range": [["7.6.0", ""]], "choices": ["disable", "enable"], "type": "str"},
            "ap-sniffer-chan-width": {"v_range": [["7.6.0", ""]], "choices": ["320MHz", "240MHz", "160MHz", "80MHz", "40MHz", "20MHz"], "type": "str"}
        }
    },
    "wtpprofile_splittunnelingacl": {"stated": true, "options": {"dest-ip": {"type": "str"}, "id": {"required": true, "type": "int"}}}
}

'''

schemas = json.loads(SCHEMA_DATA)
